=== modified file 'INSTALL'
--- INSTALL	2010-01-29 10:36:23 +0000
+++ INSTALL	2010-06-02 05:03:31 +0000
@@ -15,10 +15,10 @@
 
 bzr can optionally use compiled versions of some parts of the code
 for increased speed. When installing bzr you need the ability to
-build C extensions. Some Linux distributions package the necessary
+build C extensions. Some GNU/Linux distributions package the necessary
 headers separately from the main Python package. This package is
 probably named something like python-dev or python-devel. FreeBSD,
-Windows, source-based Linux distributions, and possibly other operating
+Windows, source-based GNU/Linux distributions, and possibly other operating
 systems, have the required files installed by default.
 
 If you are installing bzr from a bzr branch rather than a release tarball,

=== modified file 'Makefile'
--- Makefile	2010-04-06 06:59:03 +0000
+++ Makefile	2010-08-30 21:23:49 +0000
@@ -39,8 +39,14 @@
 check: docs check-nodocs
 
 check-nodocs: extensions
+	set -e
 	# Generate a stream for PQM to watch.
+	-$(RM) -f selftest.log
 	$(PYTHON) -Werror -O ./bzr selftest --subunit $(tests) | tee selftest.log
+	# An empty log file should catch errors in the $(PYTHON)
+	# command above (the '|' swallow any errors since 'make'
+	# sees the 'tee' exit code for the whole line
+	if [ ! -s selftest.log ] ; then exit 1 ; fi
 	# Check that there were no errors reported.
 	subunit-stats < selftest.log
 

=== modified file 'NEWS'
--- NEWS	2010-05-06 07:48:22 +0000
+++ NEWS	2010-10-07 12:45:51 +0000
@@ -5,14 +5,1267 @@
 .. contents:: List of Releases
    :depth: 1
 
+bzr 2.3b2
+#########
+
+:2.3.b2: NOT RELEASED YET
+
+Compatibility Breaks
+********************
+
+* The ``bzr tags`` command sorts tag names using a natural sort by
+  default (so tag2 sorts before tag10).  The previous default was
+  strictly "asciibetical".  That behavior is still available as ``bzr tags
+  --sort=alpha``. (Neil Martinsen-Burrell, #640760)
+
+New Features
+************
+
+* Add ``mainline`` revision specifier, which selects the revision that
+  merged a specified revision into the mainline.  (Aaron Bentley)
+
+* Add ``annotate`` revision specifier, which selects the revision that
+  introduced a specified line of a file.  (Aaron Bentley)
+
+* ``bzr status`` now displays a summary of existing shelves after
+  the other status information. This is done using a ``post_status``
+  hook.
+  (Parth Malwankar, #403687)
+
+* GNU lsh is now a supported lsh client; just set BZR_SSH to 'lsh'.
+  Also, bzr will recognize if the 'ssh' comand is a symlink to lsh.
+  (Matthew Gordon, #374700)
+
+Bug Fixes
+*********
+
+* ``bzr break-lock --force`` breaks the lock without prompting.  (Before
+  using this, make sure the process holding the lock really is dead.)
+  (Martin Pool, #397315)
+
+* Don't force openssh to use protocol=2, since that is now the default.
+  (Neil Martinsen-Burrell, #561061)
+
+* Print junk rather than throwing a UnicodeDecodeError from ``bzr version-info``
+  when using the rio format (with non-ascii information) on a non-utf-8 terminal.
+  (Andrej A Antonov, #518609)
+
+* Skip tests that needs a bzr source tree when there isn't one. This is
+  needed to succesfully run the test suite for installed versions.
+  (Vincent Ladeuil, #644855).
+
+* Skip the tests that requires respecting the chmod bits when running as root.
+  (Vincent Ladeuil, #646133)
+
+* ``unshelve --preview`` now can show diff in a non-ascii encoding.
+  (Andrej A Antonov, #518916)
+
+Improvements
+************
+
+* ``bzr remove`` now takes a ``--no-backup`` option for when you don't want it
+  to backup anything, just delete it. This option used to be called ``--force``
+  which is now deprecated.
+  (Marius Kruger, #400554)
+
+Documentation
+*************
+
+* Provide more detailed help on the Launchpad plugin at "bzr help
+  plugins/launchpad".  (Neil Martinsen-Burrell, #589379)
+
+API Changes
+***********
+
+Internals
+*********
+
+* Small change to GroupCompressBlock to work more in terms of 'chunks'
+  rather than 'content' for its compressed storage. (John Arbash Meinel)
+
+* When running ``bzr selftest --subunit`` the subunit stream will no
+  longer include the "log" information for tests which are considered to
+  be 'successes' (success, xfail, skip, etc) (John Arbash Meinel)
+
+Testing
+*******
+
+* Fix tests that failed when run under ``LANG=C``.
+  (Andrew Bennetts, #632387)
+
+* Use tests.TestCaseInTempDir for tests that requires disk resources.
+  (Vincent Ladeuil, #650001)
+
+bzr 2.3b1
+#########
+
+:2.3b1: 2010-09-20
+
+This is the first beta of the 2.3 series, leading up to a 2.3.0
+release in January or February.  Beta releases are suitable for everyday use
+but may cause some incompatibilities with plugins.  Some plugins may need
+small updates to work with 2.3b1.
+
+2.3b1 includes some performance improvements in both speed and memory
+consumption, some preliminary support for generating a texinfo version of
+the doc and better support for launchpad.  Many changes were made to make
+our test suite more robust as well as numerous documentation fixes.  It
+improves the common infrastructure for dealing with colocated named
+branches and foreign branches.  We plan to continue with these themes
+through the 2.3 series.
+
+It also includes bug fixes for 2.0.6, 2.1.3 and 2.2.1 and over 40 fixes of
+its own.
+
+
+Compatibility Breaks
+********************
+
+* BzrError subclasses no longer support the name "message" to be used
+  as an argument for __init__ or in _fmt format specification as this
+  breaks in some Python versions. errors.LockError.__init__ argument
+  is now named "msg" instead of earlier "message".
+  (Parth Malwankar, #603461)
+
+* Additional merges after an unrelated branch has been merged with its
+  history no longer crash when deleted files are involved.
+  (Vincent Ladeuil, John Arbash Meinel, #375898)
+
+* ``bzr add SYMLINK/FILE`` now works properly when the symlink points to a
+  previously-unversioned directory within the tree: the directory is
+  marked versioned too.
+  (Martin Pool, #192859)
+
+* ``bzr commit SYMLINK`` now works, rather than trying to commit the
+  target of the symlink.
+  (Martin Pool, John Arbash Meinel, #128562)
+
+* ``bzr upgrade`` now creates the ``backup.bzr`` directory with the same
+  permissions as ``.bzr`` directory on a POSIX OS.
+  (Parth Malwankar, #262450)
+
+* ``bzrlib.transform.TreeTransformBase.final_kind``,
+  ``bzrlib.transform.TreeTransform.tree_kind`` and
+  ``bzrlib.transform.TransformPreview.tree_kind`` now return None instead
+  of raising NoSuchFile.  (Vincent Ladeuil)
+
+* CommitBuilder now uses the committer instead of _config.username to generate
+  the revision-id.  (Aaron Bentley, #614404)
+
+* Configuration files in ``${BZR_HOME}`` are now written in an atomic
+  way which should help avoid problems with concurrent writers.
+  (Vincent Ladeuil, #525571)
+
+* `ControlDirFormat` and  `ControlDir` have been split out of `BzrDirFormat`
+  and `BzrDir`, respectively. `ControlDirFormat`
+  and `ControlDir` should be used as the base classes for new non-.bzr
+  implementations.
+
+  `BzrDirFormat.register_control_format` has been renamed to
+  `ControlDirFormat.register_format`.
+
+  `BzrDirFormat.register_server_control_format` has been removed.
+
+  Probing for control directories is now done by separate objects derived
+  from `bzrlib.controldir.Prober` and registered using
+  `bzrlib.controldir.ControlDirFormat.register_prober` or
+  `bzrlib.controldir.ControlDirFormat.register_server_prober`.
+  `BzrDirFormat.probe_transport` has been moved onto `Prober`.
+
+  `BzrDirFormat.register_format` has been renamed to
+  `BzrProber.register_bzrdir_format`.
+
+  `bzrlib.bzrdir.network_format_registry` has been moved to
+  `bzrlib.controldir`.
+
+  (Jelmer Vernooij)
+
+* Cope with Microsoft FTP server that returns reply '250 Directory
+  created' when mkdir succeeds.  (Martin Pool, #224373)
+
+* Don't traceback when a lockdir's ``held/info`` file is corrupt (e.g.
+  contains only NUL bytes).  Instead warn the user, and allow ``bzr
+  break-lock`` to remove it.  (Andrew Bennetts, #619872)
+  
+* `decode` parameter to get() method in FtpTransport and GioTransport classes
+  is deprecated. (Alexander Belchenko)
+
+* `FileInWrongBranch` is deprecated in favour of `PathNotChild` and no
+  longer raised.
+  (Martin Pool)
+
+* Fix ``AttributeError on parent.children`` when adding a file under a
+  directory that was a symlink in the previous commit.
+  (Martin Pool, #192859)
+
+* Fix ``AttributeError: 'NoneType' object has no attribute 'close'`` in
+  ``_close_ssh_proc`` when using ``bzr+ssh://``.  This was causing
+  connections to pre-1.6 bzr+ssh servers to fail, and causing warnings on
+  stderr in some other circumstances.  (Andrew Bennetts, #633745)
+
+* Only call ``setlocale`` in the bzr startup script on posix systems. This
+  avoids an issue with the newer windows C runtimes used by Python 2.6 and
+  later which can mangle bytestrings printed to the console.
+  (Martin [gz], #631350)
+
+* Prevent ``CHKMap.apply_delta`` from generating non-canonical CHK maps,
+  which can result in "missing referenced chk root keys" errors when
+  fetching from repositories with affected revisions.
+  (Andrew Bennetts, #522637)
+
+* Raise ValueError instead of a string exception.
+  (John Arbash Meinel, #586926)
+
+* Reduce peak memory by one copy of compressed text.
+  (John Arbash Meinel, #566940)
+
+* Repositories accessed via a smart server now reject being stacked on a
+  repository in an incompatible format, as is the case when accessing them
+  via other methods.  This was causing fetches from those repositories via
+  a smart server (e.g. using ``bzr branch``) to receive invalid data.
+  (Andrew Bennetts, #562380)
+
+* Selftest with versions of subunit that support ``stopTestRun`` will no longer
+  error. This error was caused by 2.0 not being updated when upstream
+  python merged the end of run patch, which chose ``stopTestRun`` rather than
+  ``done``. (Robert Collins, #571437)
+
+* Stop ``AttributeError: 'module' object has no attribute 'ElementTree'``
+  being thrown from ``xml_serializer`` on certain cElementTree setups.
+  (Martin [gz], #254278)
+
+* The old ``bzr selftest --benchmark`` option has been removed.
+  <https://launchpad.net/bzr-usertest> is an actively-maintained
+  macrobenchmark suite.
+  (Martin Pool)
+
+* `tree_files` and `internal_tree_files` are now deprecated in favor of
+  `WorkingTree.open_containing_paths`.
+  (Martin Pool)
+
+* Upgrading or fetching from a non-rich-root repository to a rich-root
+  repository (e.g. from pack-0.92 to 2a) no longer fails with
+  ``'Inter1and2Helper' object has no attribute 'source_repo'``.  This was
+  a regression from Bazaar 2.1.  (Andrew Bennetts, #636930)
+  
+* When passing a file to ``UTF8DirReader`` make sure to close the current
+  directory file handle after the chdir fails. Otherwise when passing many
+  filenames into a command line ``bzr status`` we would leak descriptors.
+  (John Arbash Meinel, #583486)
+
+New Features
+************
+
+* Added ``pre_status`` and ``post_status`` hooks. This allows plugins
+  to register custom handlers which will be invoked before/after the
+  standard status output is displayed. (Parth Malwankar)
+
+* ``bzr break-lock --config [location]`` can now break config files
+  locks. (Vincent Ladeuil, #525571)
+
+* ``bzrlib.config.LockableConfig`` is a base class for config files that
+  needs to be protected against multiple writers. All methods that
+  change a configuration variable value must be decorated with
+  @needs_write_lock (set_option() for example).
+  (Vincent Ladeuil,  #525571)
+
+* The ``lp:`` prefix will now use your known username (from
+  ``bzr launchpad-login``) to expand ``~`` to your username.  For example:
+  ``bzr launchpad-login user && bzr push lp:~/project/branch`` will now
+  push to ``lp:~user/project/branch``.  (John Arbash Meinel)
+
+* New development format ``development8-subtree`` which is similar to the 
+  ``2a`` format and adds subtree support. (Jelmer Vernooij)
+
+* The ``pull`` and ``update`` commands now take a ``-show-base``
+  option that, in the case of conflicts, shows the base revision text.
+  (Rory Yorke,  #202374)
+
+Bug Fixes
+*********
+
+* Fix traceback with python-2.7's xmlrpclib
+  (Toshio Kuratomi, #612096)
+
+* Allow using both --using and --diff-options. 
+  (Matthäus G. Chajdas, #234708)
+
+* Allow using non-integer bug ID with generic bug trackers.
+  (Alexandre Garnier, #440472)
+
+* ``bzr add SYMLINK/FILE`` now works properly when the symlink points to a
+  previously-unversioned directory within the tree: the directory is
+  marked versioned too.  
+  (Martin Pool, #192859)
+
+* ``bzr ignore PATTERNS`` exits with error if a bad pattern is supplied.
+  ``InvalidPattern`` exception error message now shows faulting
+  regular expression.
+  (Parth Malwankar #300062)
+
+* ``clean-tree`` issues a warning if it is unable to delete a file
+  due to ``errno.EACCES`` instead of exiting with an error on Windows.
+  (Parth Malwankar, #430785)
+
+* CommitBuilder now uses the committer instead of _config.username to generate
+  the revision-id.  (Aaron Bentley, #614404)
+
+* Configuration files in ``${BZR_HOME}`` are now protected against
+  concurrent writers by using a lock. (Vincent Ladeuil, #525571)
+
+* Cope with Microsoft FTP Server and VSFTPd that return reply '250
+  Directory created' when mkdir succeeds.  (Martin Pool, #224373)
+
+* Decrease peak memory during ``bzr send``. The old code was caching all
+  text content and all inventory strings for all revisions before
+  computing the diffs. Now we only cache as long as there is a child that
+  will need them. Sending 2000 bzr revisions drops from 1.2GB peak to
+  256MB peak. (John Arbash Meinel, #614576)
+
+* Don't print internal object name when print an invalid revision spec
+  error.  (Neil Martinsen-Burrell, #598701)
+
+* Don't traceback when a lockdir's ``held/info`` file is corrupt (e.g.
+  contains only NUL bytes).  Instead warn the user, and allow ``bzr
+  break-lock`` to remove it.  (Andrew Bennetts, #619872)
+  
+* ``EPIPE`` can be raised during test server shutdown. This happened on
+  gentoo only so far. (Vincent Ladeuil, #627277)
+
+* Errors occurring during http(s) test server starts should now be
+  handled cleanly. (Vincent Ladeuil, #392402)
+
+* Fix ``AttributeError on parent.children`` when adding a file under a 
+  directory that was a symlink in the previous commit.
+  (Martin Pool, #192859)
+
+* Fix ``AttributeError: 'NoneType' object has no attribute 'close'`` in
+  ``_close_ssh_proc`` when using ``bzr+ssh://``.  This was causing
+  connections to pre-1.6 bzr+ssh servers to fail, and causing warnings on
+  stderr in some other circumstances.  (Andrew Bennetts, #633745)
+
+* Fix signature of RemoteBzrDir.create_workingtree to match that of its
+  superclass.  (Neil Martinsen-Burrell, Martin Pool, #524627)
+
+* Fix spurious paramiko warning on hardy by ensuring that ``selftest``
+  properly remove its warning filter. (Vincent Ladeuil, #625686)
+
+* ``HTTP/1.1`` test servers now set a ``Content-Length`` header to comply
+  with pedantic ``HTTP/1.1`` clients. (Vincent Ladeuil, #568421)
+
+* Most of the leaked threads during selftest are now fixed, allowing the
+  full test suite to pass on gentoo.
+  (Vincent Ladeuil, #392127)
+
+* Only call ``setlocale`` in the bzr startup script on posix systems. This
+  avoids an issue with the newer windows C runtimes used by Python 2.6 and
+  later which can mangle bytestrings printed to the console.
+  (Martin [gz], #631350)
+
+* `PathNotChild` should not give a traceback.
+  (Martin Pool, #98735)
+
+* ``PQM`` will no longer ignore syntax errors in submissions.
+  (Vincent Ladeuil, #626667)
+
+* Prevent ``CHKMap.apply_delta`` from generating non-canonical CHK maps,
+  which can result in "missing referenced chk root keys" errors when
+  fetching from repositories with affected revisions.
+  (Andrew Bennetts, #522637)
+
+* strace test-helper tests cope with the new Ubuntu policy of not allowing
+  users to attach to their own processes by default.
+  (Martin Pool, #626679)
+
+* Test classes like ``TestCase``, ``TestLoader``, and ``TestSuite`` should
+  be available from ``bzrlib.tests.*``. They used to be, but were
+  accidentally removed. (John Arbash Meinel, #627438)
+
+* Treat all IO, OS, and socket errors consistently when establishing
+  SSH/SFTP connections via a subprocess.  (Andrew Bennetts)
+
+* ``Transport.stat`` on a symlink, including a transport pointing directly
+  to a symlink, now returns information about the symlink.
+  (Martin Pool)
+
+* Upgrading or fetching from a non-rich-root repository to a rich-root
+  repository (e.g. from pack-0.92 to 2a) no longer fails with
+  ``'Inter1and2Helper' object has no attribute 'source_repo'``.
+  (Andrew Bennetts, #636930)
+  
+* Wait for the SSH server to actually finish, rather than just waiting for
+  it to negotiate the key exchange. (John Arbash Meinel, #626876)
+
+Improvements
+************
+
+* ``bzr remove`` now just backs up changed files instead of exiting,
+  forcing you to choose to either keep or delete them. Bazaar will now delete
+  the files if they can easily be recovered using revert, otherwise they
+  will be backed up (adding an extention of the form .~#~).
+  (Marius Kruger, #400554)
+
+* ``bzr revert`` and ``bzr status`` are up to 15% faster on large trees
+  with many changes by not repeatedly building a list of all file-ids.
+  (Andrew Bennetts)
+
+* Decrease memory consumption when many chk index pages are loaded. (Such
+  as during ``bzr co`` or ``bzr ls -R`` of a large tree.) Often we need to
+  read many chk pages because the individual chk map nodes will be spread
+  randomly. Peak memory for 'bzr ls -R' on a large tree dropped from 396MB
+  down to 247MB, expect even more significant savings on 64-bit platforms.
+  (John Arbash Meinel)
+
+* ``DirState`` internals use a little bit less memory. For bzr.dev it
+  drops the memory from 1MB down to about 800kB. And replaces a few
+  thousand tuples and sets with StaticTuple.  (John Arbash Meinel)
+
+* Inventory entries now consume less memory (on 32-bit Ubuntu file entries
+  have dropped from 68 bytes to 40, and directory entries from 120 bytes
+  to 48).  (Andrew Bennetts)
+
+* When building new working trees, default to reading from the repository
+  rather than the source tree unless explicitly requested. (via
+  ``--files-from`` and ``--hardlink`` for ``bzr branch`` and
+  ``bzr checkout``. Generally, 2a format repositories extract
+  content faster than seeking and reading content from another tree,
+  especially in cold-cache situations. (John Arbash Meinel, #607298)
+
+* Add ``__pycache__`` to the default ``ignores`` file. Future releases of
+  Python will use this directory to store bytecodes.
+  (Andrea Corbellini, #626687)
+
+Documentation
+*************
+
+* Added a builder/writer sphinx extension that can generate texinfo files. The
+  generated files are syntactically correct but the info navigation nodes
+  needs more work. (Vincent Ladeuil, #219334)
+
+* First tests defined for sphinx, including a new bzrlib.tests.features.sphinx
+  to make the tests conditional.
+  (Vincent Ladeuil)
+
+* Fix a lot of references in the docs to the old http://bazaar-vcs.org to
+  the new http://bazaar.canonical.com or http://wiki.bazaar.canonical.com
+  (John Arbash Meinel, #617503)
+
+* Suggest ``bzr revert`` for restoring locally deleted files in help text
+  for ``bzr update``.  (John C Barstow, #191466)
+
+API Changes
+***********
+
+* Configuration files should now use the ``from_string`` constructor rather
+  than the ``file`` parameter of the ``_get_parser`` method. The later has
+  been deprecated. ``from_string`` also accept a ``save=True`` parameter to
+  have the configuration file immediately written to disk. 
+  (Vincent Ladeuil)
+
+* Deprecate treating a `PushResult` and `PullResult` as an integer for the
+  relative change in revno.
+  (Martin Pool)
+  `
+* ``IniBaseConfig`` objects should now use the ``from_string`` constructor
+  the rather than the ``file`` parameter of the ``_get_parser`` method. The
+  later has been deprecated. (Vincent Ladeuil)
+
+* InventoryEntry instances now raise AttributeError if you try to assign
+  to attributes that are irrelevant to that kind of entry.  e.g. setting
+  ``symlink_target`` on an InventoryFile will fail.  It is still okay to
+  read those attributes on any kind of InventoryEntry.  The complete list
+  of affected attributes is: ``executable``, ``text_id``, ``text_sha1``,
+  ``text_size`` (only valid for kind == file); ``symlink_target`` (only
+  valid for kind == link); and ``reference_revision`` (only valid for kind
+  == tree-reference).  (Andrew Bennetts)
+
+* InventoryEntry objects no longer have ``_put_in_tar`` or
+  ``_put_on_disk`` methods.  (Andrew Bennetts)
+
+* The ``get_filename`` parameter in the ``config.IniBaseConfig``
+  constructor has been deprecated, use the ``file_name`` parameter instead.
+  (Vincent Ladeuil)
+
+* ``WorkingTree`` methods ``pull``, ``update``, and ``_update_tree``
+  now have an optional argument, ``show_base``, which is by default
+  False.  This is flag is ultimately passed to ``merge.merge_inner``
+  in each case. (Rory Yorke, #202374)
+
+Internals
+*********
+
+* Remove used and broken code path in ``BranchInitHookParams.__repr__``.
+  (Andrew Bennetts)
+
+Testing
+*******
+
+* ``build_tree_contents`` can create symlinks.
+  (Martin Pool, John Arbash Meinel)
+
+* Catch socket errors to avoid
+  bt.test_sftp_transport.SSHVendorBadConnection.test_bad_connection_ssh
+  random failures. (Vincent Ladeuil, #601804)
+
+* HTTP test servers will leak less threads (and sockets) and will not hang on
+  AIX anymore. (Vincent Ladeuil, #405745)
+
+* On platforms that don't support forking give a nice error message saying so
+  when ``bzr selftest --parallel=fork`` is used. (Martin [gz], #528730)
+
+* Rearrange thread leak detection code to eliminate global state and make it
+  possible to extend the reporting. (Martin [gz], #633462)
+
+* Suppress the "maximum recursion depth exceeded in __subclasscheck__"
+  warning on stderr emitted during ``test_dict_deepnested`` in
+  ``bzrlib/tests/test__bencode.py``.  (Andrew Bennetts)
+
+* The test suite now simply holds log files in memory, rather than writing them
+  out to disk and then reading them back in and deleting them.
+  (Andrew Bennetts)
+
+* The way ``bzr selftest --parallel`` generates N partitions of tests to
+  run in parallel has changed.  Instead of splitting the list of tests at
+  N-1 points, it distributes the tests one-by-one into the partitions in a
+  round robin fashion.  This reduces the total time to run the tests in
+  parallel because a series of slow tests in the test suite will be
+  distributed evenly among the parallel test suites, rather than slowing
+  down just one suite.  (Andrew Bennetts)
+
+* Tracebacks from a parameterized test are no longer reported against every
+  parameterization of that test.  This was done by adding a hack to
+  ``bzrlib.tests.clone_test`` so that it no longer causes
+  testtools.TestCase instances to share a details dict.
+  (Andrew Bennetts, #625574)
+
+
+bzr 2.2.2
+#########
+
+:2.2.2: NOT RELEASED YET
+
+Compatibility Breaks
+********************
+
+New Features
+************
+
+Bug Fixes
+*********
+
+* Skip tests that needs a bzr source tree when there isn't one. This is
+  needed to succesfully run the test suite for installed versions.
+  (Vincent Ladeuil, #644855).
+
+* Skip the tests that requires respecting the chmod bits when running as root.
+  (Vincent Ladeuil, #646133)
+
+Improvements
+************
+
+Documentation
+*************
+
+API Changes
+***********
+
+Internals
+*********
+
+Testing
+*******
+
+* Fix tests that failed when run under ``LANG=C``.
+  (Andrew Bennetts, #632387)
+
+
+bzr 2.2.1
+#########
+
+:2.2.1: 2010-09-17
+
+This is a bugfix release which also includes bugfixes from 2.0.6 and
+2.1.3. None are critical, but upgrading is recommended for all users on
+earlier 2.2 releases.
+
+Bug Fixes
+*********
+
+* Additional merges after an unrelated branch has been merged with its
+  history no longer crash when deleted files are involved.
+  (Vincent Ladeuil, John Arbash Meinel, #375898)
+
+* ``bzr add SYMLINK/FILE`` now works properly when the symlink points to a
+  previously-unversioned directory within the tree: the directory is
+  marked versioned too.
+  (Martin Pool, #192859)
+
+* ``bzr commit SYMLINK`` now works, rather than trying to commit the
+  target of the symlink.
+  (Martin Pool, John Arbash Meinel, #128562)
+
+* ``bzr upgrade`` now creates the ``backup.bzr`` directory with the same
+  permissions as ``.bzr`` directory on a POSIX OS.
+  (Parth Malwankar, #262450)
+
+* CommitBuilder now uses the committer instead of _config.username to generate
+  the revision-id.  (Aaron Bentley, #614404)
+
+* Configuration files in ``${BZR_HOME}`` are now written in an atomic
+  way which should help avoid problems with concurrent writers.
+  (Vincent Ladeuil, #525571)
+
+* Cope with Microsoft FTP server that returns reply '250 Directory
+  created' when mkdir succeeds.  (Martin Pool, #224373)
+
+* Don't traceback trying to unversion children files of an already
+  unversioned directory.  (Vincent Ladeuil, #494221)
+
+* Don't traceback when a lockdir's ``held/info`` file is corrupt (e.g.
+  contains only NUL bytes).  Instead warn the user, and allow ``bzr
+  break-lock`` to remove it.  (Andrew Bennetts, #619872)
+  
+* Fix ``AttributeError on parent.children`` when adding a file under a
+  directory that was a symlink in the previous commit.
+  (Martin Pool, #192859)
+
+* Fix ``AttributeError: 'NoneType' object has no attribute 'close'`` in
+  ``_close_ssh_proc`` when using ``bzr+ssh://``.  This was causing
+  connections to pre-1.6 bzr+ssh servers to fail, and causing warnings on
+  stderr in some other circumstances.  (Andrew Bennetts, #633745)
+
+* Only call ``setlocale`` in the bzr startup script on posix systems. This
+  avoids an issue with the newer windows C runtimes used by Python 2.6 and
+  later which can mangle bytestrings printed to the console.
+  (Martin [gz], #631350)
+
+* Prevent ``CHKMap.apply_delta`` from generating non-canonical CHK maps,
+  which can result in "missing referenced chk root keys" errors when
+  fetching from repositories with affected revisions.
+  (Andrew Bennetts, #522637)
+
+* Raise ValueError instead of a string exception.
+  (John Arbash Meinel, #586926)
+
+* Reduce peak memory by one copy of compressed text.
+  (John Arbash Meinel, #566940)
+
+* Repositories accessed via a smart server now reject being stacked on a
+  repository in an incompatible format, as is the case when accessing them
+  via other methods.  This was causing fetches from those repositories via
+  a smart server (e.g. using ``bzr branch``) to receive invalid data.
+  (Andrew Bennetts, #562380)
+
+* Selftest with versions of subunit that support ``stopTestRun`` will no longer
+  error. This error was caused by 2.0 not being updated when upstream
+  python merged the end of run patch, which chose ``stopTestRun`` rather than
+  ``done``. (Robert Collins, #571437)
+
+* Stop ``AttributeError: 'module' object has no attribute 'ElementTree'``
+  being thrown from ``xml_serializer`` on certain cElementTree setups.
+  (Martin [gz], #254278)
+
+* Upgrading or fetching from a non-rich-root repository to a rich-root
+  repository (e.g. from pack-0.92 to 2a) no longer fails with
+  ``'Inter1and2Helper' object has no attribute 'source_repo'``.  This was
+  a regression from Bazaar 2.1.  (Andrew Bennetts, #636930)
+  
+* When passing a file to ``UTF8DirReader`` make sure to close the current
+  directory file handle after the chdir fails. Otherwise when passing many
+  filenames into a command line ``bzr status`` we would leak descriptors.
+  (John Arbash Meinel, #583486)
+
+Documentation
+*************
+
+* Fix a lot of references in the docs to the old http://bazaar-vcs.org to
+  the new http://bazaar.canonical.com or http://wiki.bazaar.canonical.com
+  (John Arbash Meinel, #617503)
+
+Internals
+*********
+
+* Remove used and broken code path in ``BranchInitHookParams.__repr__``.
+  (Andrew Bennetts)
+
+Testing
+*******
+
+* ``build_tree_contents`` can create symlinks.
+  (Martin Pool, John Arbash Meinel)
+
+* Tracebacks from a parameterized test are no longer reported against every
+  parameterization of that test.  This was done by adding a hack to
+  ``bzrlib.tests.clone_test`` so that it no longer causes
+  testtools.TestCase instances to share a details dict.
+  (Andrew Bennetts, #625574)
+
+
+bzr 2.1.4
+#########
+
+:2.1.4: NOT RELEASED YET
+
+Compatibility Breaks
+********************
+
+New Features
+************
+
+Bug Fixes
+*********
+
+* Skip tests that needs a bzr source tree when there isn't one. This is
+  needed to succesfully run the test suite for installed versions.
+  (Vincent Ladeuil, #644855).
+
+* Skip the tests that requires respecting the chmod bits when running as root.
+  (Vincent Ladeuil, #646133)
+
+Improvements
+************
+
+Documentation
+*************
+
+API Changes
+***********
+
+Internals
+*********
+
+Testing
+*******
+
+
+bzr 2.1.3
+#########
+
+:Codename: Do run run
+:2.1.3: 2010-09-17
+
+The third release in our 2.1 series addresses several user-inconvenience bugs
+(and includes the fixes done in 2.0.6).  None are critical, but upgrading is
+recommended for all users on earlier 2.1 releases.
+
+Bug Fixes
+*********
+
+* Additional merges after an unrelated branch has been merged with its
+  history no longer crash when deleted files are involved.
+  (Vincent Ladeuil, John Arbash Meinel, #375898)
+
+* ``bzr add SYMLINK/FILE`` now works properly when the symlink points to a
+  previously-unversioned directory within the tree: the directory is
+  marked versioned too.  
+  (Martin Pool, #192859)
+
+* ``bzr commit SYMLINK`` now works, rather than trying to commit the
+  target of the symlink.
+  (Martin Pool, John Arbash Meinel, #128562)
+
+* ``bzr upgrade`` now creates the ``backup.bzr`` directory with the same
+  permissions as ``.bzr`` directory on a POSIX OS.
+  (Parth Malwankar, #262450)
+
+* Configuration files in ``${BZR_HOME}`` are now written in an atomic
+  way which should help avoid problems with concurrent writers.
+  (Vincent Ladeuil, #525571)
+
+* Don't traceback trying to unversion children files of an already
+  unversioned directory.  (Vincent Ladeuil, #494221)
+
+* Don't traceback when a lockdir's ``held/info`` file is corrupt (e.g.
+  contains only NUL bytes).  Instead warn the user, and allow ``bzr
+  break-lock`` to remove it.  (Andrew Bennetts, #619872)
+  
+* Fix ``AttributeError on parent.children`` when adding a file under a 
+  directory that was a symlink in the previous commit.
+  (Martin Pool, #192859)
+
+* Prevent ``CHKMap.apply_delta`` from generating non-canonical CHK maps,
+  which can result in "missing referenced chk root keys" errors when
+  fetching from repositories with affected revisions.
+  (Andrew Bennetts, #522637)
+
+* Raise ValueError instead of a string exception.
+  (John Arbash Meinel, #586926)
+
+* Reduce peak memory by one copy of compressed text.
+  (John Arbash Meinel, #566940)
+
+* Repositories accessed via a smart server now reject being stacked on a
+  repository in an incompatible format, as is the case when accessing them
+  via other methods.  This was causing fetches from those repositories via
+  a smart server (e.g. using ``bzr branch``) to receive invalid data.
+  (Andrew Bennetts, #562380)
+
+* Selftest with versions of subunit that support ``stopTestRun`` will no longer
+  error. This error was caused by 2.0 not being updated when upstream
+  python merged the end of run patch, which chose ``stopTestRun`` rather than
+  ``done``. (Robert Collins, #571437)
+
+* Stop ``AttributeError: 'module' object has no attribute 'ElementTree'``
+  being thrown from ``xml_serializer`` on certain cElementTree setups.
+  (Martin [gz], #254278)
+
+* When passing a file to ``UTF8DirReader`` make sure to close the current
+  directory file handle after the chdir fails. Otherwise when passing many
+  filenames into a command line ``bzr status`` we would leak descriptors.
+  (John Arbash Meinel, #583486)
+
+Testing
+*******
+
+* ``build_tree_contents`` can create symlinks.
+  (Martin Pool, John Arbash Meinel)
+
+
+bzr 2.0.6
+#########
+
+:2.0.6: 2010-09-17
+
+The sixth release in our 2.0 series addresses several user-inconvenience
+bugs.  None are critical, but upgrading is recommended for all users on
+earlier 2.0 releases.
+
+Bug Fixes
+*********
+
+* Additional merges after an unrelated branch has been merged with its
+  history no longer crash when deleted files are involved.
+  (Vincent Ladeuil, John Arbash Meinel, #375898)
+
+* ``bzr add SYMLINK/FILE`` now works properly when the symlink points to a
+  previously-unversioned directory within the tree: the directory is
+  marked versioned too.  
+  (Martin Pool, #192859)
+
+* ``bzr commit SYMLINK`` now works, rather than trying to commit the
+  target of the symlink.
+  (Martin Pool, John Arbash Meinel, #128562)
+
+* ``bzr revert`` now only takes write lock on working tree, instead of on 
+  both working tree and branch.
+  (Danny van Heumen, #498409)
+
+* ``bzr upgrade`` now creates the ``backup.bzr`` directory with the same
+  permissions as ``.bzr`` directory on a POSIX OS.
+  (Parth Malwankar, #262450)
+
+* Don't traceback trying to unversion children files of an already
+  unversioned directory.  (Vincent Ladeuil, #494221)
+
+* Don't traceback when a lockdir's ``held/info`` file is corrupt (e.g.
+  contains only NUL bytes).  Instead warn the user, and allow ``bzr
+  break-lock`` to remove it.  (Andrew Bennetts, #619872)
+  
+* Fix ``AttributeError on parent.children`` when adding a file under a 
+  directory that was a symlink in the previous commit.
+  (Martin Pool, #192859)
+
+* Prevent ``CHKMap.apply_delta`` from generating non-canonical CHK maps,
+  which can result in "missing referenced chk root keys" errors when
+  fetching from repositories with affected revisions.
+  (Andrew Bennetts, #522637)
+
+* Raise ValueError instead of a string exception.
+  (John Arbash Meinel, #586926)
+
+* Reduce peak memory by one copy of compressed text.
+  (John Arbash Meinel, #566940)
+
+* Repositories accessed via a smart server now reject being stacked on a
+  repository in an incompatible format, as is the case when accessing them
+  via other methods.  This was causing fetches from those repositories via
+  a smart server (e.g. using ``bzr branch``) to receive invalid data.
+  (Andrew Bennetts, #562380)
+
+* Selftest with versions of subunit that support ``stopTestRun`` will no longer
+  error. This error was caused by 2.0 not being updated when upstream
+  python merged the end of run patch, which chose ``stopTestRun`` rather than
+  ``done``. (Robert Collins, #571437)
+
+* When passing a file to ``UTF8DirReader`` make sure to close the current
+  directory file handle after the chdir fails. Otherwise when passing many
+  filenames into a command line ``bzr status`` we would leak descriptors.
+  (John Arbash Meinel, #583486)
+
+
+Testing
+*******
+
+* ``build_tree_contents`` can create symlinks.
+  (Martin Pool, John Arbash Meinel)
+
+
+bzr 2.2
+#######
+
+:Codename: La Hulpe
+:2.2: 2010-08-06
+
+This release marks the start of another long-term-stable series. From
+here, we will only make bugfix releases on the 2.2 series (2.2.1, etc),
+while 2.3 will become our new development series. The 2.0 and 2.1 series
+will also continue to get bugfixes. (Currently 2.0 is planned to be
+supported for another 6 months.)
+
+This is primarily a bugfix and polish release over the 2.1 series, with
+a large number of bugs fixed (>120), and some performance improvements.
+
+There are some compatibility changes in this release.  For users of bzrlib
+as a library, we now request that they call ``bzrlib.initialize`` and use
+the returned context manager appropriately. For commandline users we no
+longer guess user identity for ``bzr commit``, users must specify their
+identity using ``bzr whoami`` (you don't need to specify your identity for
+readonly operations).
+
+Users are encouraged to upgrade from the other stable series.
+
+Compatibility Breaks
+********************
+
+* BzrError subclasses no longer support the name "message" to be used
+  as an argument for __init__ or in _fmt format specification as this
+  breaks in some Python versions. errors.LockError.__init__ argument
+  is now named "msg" instead of earlier "message".
+  (Parth Malwankar, #603461)
+
+* The old ``bzr selftest --benchmark`` option has been removed.
+  <https://launchpad.net/bzr-usertest> is an actively-maintained
+  macrobenchmark suite.
+  (Martin Pool)
+
+Bug Fixes
+*********
+
+* ``bzr ignore PATTERNS`` exits with error if a bad pattern is supplied.
+  ``InvalidPattern`` exception error message now shows faulting
+  regular expression.
+  (Parth Malwankar #300062)
+
+* Configuration files in ``${BZR_HOME}`` are now written in an atomic
+  way which should help avoid problems with concurrent writers.
+  (Vincent Ladeuil, #525571)
+
+* Don't traceback trying to unversion children files of an already
+  unversioned directory.  (Vincent Ladeuil, #494221)
+
+* ``HTTP/1.1`` test servers now set a ``Content-Length`` header to comply
+  with pedantic ``HTTP/1.1`` clients. (Vincent Ladeuil, #568421)
+
+* Progress bars prefer to truncate the text message rather than the
+  counters.  The spinner is shown between the network transfer indicator
+  and the progress message.  Progress bars are correctly cleared off when 
+  they finish.  (Martin Pool, #611127)
+
+* Recursive binding for checkouts is now detected by bzr. A clear error
+  message is shown to the user. (Parth Malwankar, #405192)
+
+Improvements
+************
+
+* Add ``bzrlib.merge.MergeIntoMerger``, which can merge part or all of a
+  tree, and works with unrelated branches.  (Andrew Bennetts)
+
+* Add py2exe windows target ``bzrw.exe``. This allow for starting a Bazaar
+  GUI with out have a console open in the background.
+  (Gary van der Merwe, #433781)
+
+Documentation
+*************
+
+* ``bzr help patterns`` now explains case insensitive patterns and
+  points to Python regular expression documentation.
+  (Parth Malwankar, #594386)
+
+API Changes
+***********
+
+* Delete ``ProgressTask.note``, which was deprecated in 2.1.
+
+Testing
+*******
+
+* Unit test added to ensure that "message" is not uses as a format variable
+  name in BzrError subclasses as this conflicts with some Python versions.
+  (Parth Malwankar, #603461)
+
+bzr 2.2b4
+#########
+
+:Codename: Monkey Magic
+:2.2b4: 2010-07-10
+
+
+This fourth and final beta in the 2.2 series now stabilizes the internal
+APIs. Plugin authors are recommended to ensure their releases are
+compatible, so that 2.2rc1 can be a true release candidate, containing
+stable and compatible plugin versions. 
+
+For users of bzrlib as a library, one of the primary changes is to request
+that they call ``bzrlib.initialize`` and use the returned context manager
+appropriately.
+
+Better interaction with ``bzr-loom`` to make sure branching from a loom
+even over a smart server still yields a local loom. Not to mention lots of
+bugfixes over 2.2b3.
+
+Compatibility Breaks
+********************
+
+* bzrlib library users now need to call ``__enter__`` and ``__exit__`` on
+  the result of ``bzrlib.initialize``. This change was made when fixing
+  the bad habit recent bzr versions have had of leaving progress bars 
+  behind on the screen. That required calling another function before
+  exiting the program, and it made sense to provide a full context
+  manager at the same time. (Robert Collins)
+
+* The ``bzr`` front end now requires a ``bzrlib.ui.ui_factory`` which is a
+  context manager in the Python 2.5 and above sense. The bzrlib base class
+  is such a manager, but third party UI factories which do not derive from
+  ``bzrlib.ui.UIFactory`` will be incompatible with the command line front
+  end.
+
+* URLs like ``foo:bar/baz`` are now always parsed as a URL with scheme "foo"
+  and path "bar/baz", even if bzr does not recognize "foo" as a known URL
+  scheme.  Previously these URLs would be treated as local paths.
+  (Gordon Tyler)
+
+
+New Features
+************
+
+* Support ``--directory`` option for a number of additional commands:
+  conflicts, merge-directive, missing, resolve, shelve, switch,
+  unshelve, whoami. (Martin von Gagern, #527878)
+
+Bug Fixes
+*********
+
+* ``bzr branch`` to a new repository with a default stacking policy no
+  longer transfers the full history unnecessarily.
+  (Andrew Bennetts, #597942)
+
+* ``bzr init`` does not recursively scan directory contents anymore
+  leading to faster init for directories with existing content.
+  (Martin [gz], Parth Malwankar, #501307)
+
+* ``bzr log --exclude-common-ancestry`` is now taken into account for
+  linear ancetries. (Vincent Ladeuil, #575631)
+
+* ``bzr log -r branch:REMOTE`` can now properly log the remote branch,
+  rather than trying to fetch the data locally and failing because of a
+  readonly error. (Martin von Gagern, #149270)
+
+* ``bzr pull`` now works when a lp: URL is explicitly defined as the parent
+  or pull location in locations.conf or branch.conf.
+  (Gordon Tyler, #534787)
+
+* ``bzr reconfigure --unstacked`` now works with branches accessed via a
+  smart server. (Andrew Bennetts, #551525)
+
+* ``BzrDir.find_branches`` should ignore branches with missing repositories.
+  (Marius Kruger, Robert Collins)
+
+* ``BzrDir.find_bzrdirs`` should ignore dirs that raises PermissionDenied.
+  (Marius Kruger, Robert Collins)
+
+* Ensure that wrong path specifications in ``BZR_PLUGINS_AT`` display
+  proper error messages. (Vincent Ladeuil, #591215)
+
+* Explicitly removing ``--profile-imports`` option from parsed command-line
+  arguments on Windows, because bzr script does the same.
+  (Alexander Belchenko, #588277)
+
+* Fetching was slightly confused about the best code to use and was
+  using a new code path for all branches, resulting in more lookups than
+  necessary on old branches. (Robert Collins, #593515)
+
+* Final fix for 'no help for command' issue. We now show a clean message
+  when a command has no help, document how to set help more clearly, and
+  test that all commands available to the test suite have help.
+  (Robert Collins, #177500)
+
+* Invalid patterns supplied to ``Globster`` or ``lazy_regex`` now raise
+  ``InvalidPattern`` exception showing clear error message to the user.
+  (Parth Malwankar #300062)
+
+* Progress output is cleaned up when exiting.  (Aaron Bentley)
+
+* Raise ValueError instead of a string exception.
+  (John Arbash Meinel, #586926)
+
+* Relative imports in plugins are now handled correctly when using
+  BZR_PLUGINS_AT. (Vincent Ladeuil, #588959)
+
+* ``ScriptRunner`` now strips off leading indentation from test scripts,
+  which previously caused "SyntaxError: No command for line".
+  (Martin Pool)
+
+* Show unicode filenames in diff headers using terminal encoding. 
+  (Alexander Belchenko, Bug #382699)
+  NOTE for Windows users: If user need to save diff to file then user need to
+  change encoding of the terminal to ANSI encoding with command ``chcp XXX``
+  (e.g. ``chcp 1251`` for Russian Windows).
+
+* URL displayed for use with ``break-lock`` when smart server sees lock
+  contention are now valid. Default timeout for lock contention retry is
+  now 30 seconds instead of 300 seconds.
+  (Parth Malwankar, #250451)
+
+* ``walkdirs`` now raises a useful message when the filenames are not using
+  the filesystem encoding. (Eric Moritz, #488519)
+
+* Enable debugging of bzr on windows with pdb and other tools. This was 
+  broken because we call GetCommandLineW on windows. The fix adjusts the 
+  command line we get to be the same length as sys.argv.
+  (Jason Spashett, Alexander Belchenko, #587868)
+
+Improvements
+************
+
+* Bazaar now reads data from SSH connections more efficiently on platforms
+  that provide the ``socketpair`` function, and when using paramiko.
+  (Andrew Bennetts, #590637)
+
+* ``Branch.copy_content_into`` is now a convenience method dispatching to
+  a ``InterBranch`` multi-method. This permits ``bzr-loom`` and other
+  plugins to intercept this even when a ``RemoteBranch`` proxy is in use.
+  (Robert Collins, #201613)
+
+* ``Branch`` formats can now be loaded lazily by registering a
+  ``MetaDirBranchFormatFactory`` rather than an actual format. This will
+  cause the named format class to be loaded only when an enumeration of
+  formats is needed or when the format string for the object is
+  encountered. (Robert Collins, Jelmer Vernooij)
+
+* The encoding that bzr uses to output things other than file content can
+  now be overridden via the output_encoding configuration option.
+  (Martin Pool, #340394)
+
+* Use lazy imports in ``bzrlib/merge.py`` so that plugins like ``news_merge``
+  do not cause modules to be loaded unnecessarily just because the plugin
+  registers a merge hook.  This improves ``bzr rocks`` time by about 25%
+  in a default installation (with just the core plugins).
+  (Andrew Bennetts)
+
+Documentation
+*************
+
+* Added ``regression`` tag to our tags list. (Robert Collins)
+
+* Improved our release checklist to have a bit less churn and leave things
+  ready-to-go for the next action (including other people doing
+  development). (Robert Collins)
+
+* Remove obsolete discussion of PQM in documentation about how to
+  contribute to Bazaar.  (Martin Pool, #588444)
+
+API Changes
+***********
+
+* ``bzrlib.branch.InterBranch._get_branch_formats_to_test`` now returns
+  an iterable of format pairs, rather than just a single pair, permitting
+  InterBranch objects that work with multiple permutations to be
+  comprehensively tested. (Robert Collins)
+
+* ``bzrlib.lsprof.profile`` will no longer silently generate bad threaded
+  profiles when concurrent profile requests are made. Instead the profile
+  requests will be serialised. Reentrant requests will now deadlock.
+  (Robert Collins)
+
+* ``bzrlib.knit.KnitSequenceMatcher``, which has been deprecated since
+  2007, has been deleted.  Use ``PatienceSequenceMatcher`` from
+  ``bzrlib.patiencediff`` instead. (Andrew Bennetts)
+
+* ``bzrlib.re_compile_checked`` is now deprecated. Caller should handle
+  ``bzrlib.errors.InvalidPattern`` exception thrown by ``re.match`` in
+  case the default error message not suitable for the use case.
+  (Parth Malwankar)
+
+* ``bzrlib.tests.blackbox.ExternalBase`` is deprecated.  It provided only
+  one method ``check_output``, and we now recommend checking command
+  output using ``run_script``. (Martin Pool)
+
+* ``bzrlib.transport.ssh.SSHVendor.connect_ssh`` now returns an object
+  that implements the interface of ``bzrlib.transport.ssh.SSHConnection``.
+  Third-party implementations of ``SSHVendor`` may need to be updated
+  accordingly.  Similarly, any code using ``SSHConnection`` directly will
+  need to be updated.  (Andrew Bennetts)
+
+* The constructor of ``bzrilb.smart.medium.SmartSSHClientMedium`` has
+  changed to take an ``SSHParams`` instance (replacing many individual
+  values).  (Andrew Bennetts)
+
+Internals
+*********
+
+* ``bzrlib.osutils.get_terminal_encoding`` will now only mutter its
+  selection when explicitly requested; this avoids many duplicate calls
+  being logged when helpers, wrappers and older code that manually calls
+  it are executed it is now logged deliberately by the ui setup code.
+  (Robert Collins)
+
+* Improved ``bzrlib.urlutils`` to handle lp:foo/bar URLs. (Gordon Tyler)
+
+* ``bzrlib._c_static_tuple.StaticTuple`` now implements ``__sizeof__``, so
+  that ``sys.getsizeof`` and other memory analysis tools will report more
+  accurate results. (Andrew Bennetts)
+
+* The symbol_versioning module can now cleanup after itself -
+  ``suppress_deprecation_warnings`` now returns a cleanup function.
+  (Robert Collins)
+
+Testing
+*******
+
+* Add ``bzrlib.tests.fixtures`` to hold code for setting up objects
+  to test.  (Martin Pool)
+
+* ``test_import_tariff`` now respects BZR_PLUGINS_AT and BZR_PLUGINS_DISABLE.
+  (Vincent Ladeuil, #595587)
+
 bzr 2.2b3
 #########
 
-:2.2b3: NOT RELEASED YET
+:2.2b3: 2010-05-28
+
+This third beta in the 2.2 series brings with it all the goodness of 2.1.2
+and 2.0.6 (though it preceeds 2.0.6 slightly). Of particular note for
+users are compatibility fixes with bzr 1.5 and below servers, a hopeful
+end to the EINTR errors caused by SIGWINCH interactions, a shiny new
+bash completion script and bzr will no longer guess at identity details -
+it was too unreliable in reality. Use ``bzr whoami`` on every new install.
+For developers we have some API changes which may impact plugins as well
+as a bunch of our regular improvements to internal clarity and test
+support.
 
 Compatibility Breaks
 ********************
 
+* An API break has been made to the lock_write method of ``Branch`` and
+  ``Repository`` objects; they now return ``branch.BranchWriteLockResult``
+  and ``repository.RepositoryWriteLockResult`` objects. This makes
+  changing the API in future easier and permits some cleaner calling code.
+  The lock_read method has also changed from having no defined return
+  value to returning ``LogicalLockResult`` objects.
+  (Robert Collins)
+
+* ``bzr`` does not try to guess the username as ``username@hostname``
+  and requires it to be explictly set. This can be set using ``bzr
+  whoami``. (Parth Malwankar, #549310)
+
+* ``bzrlib.commands.Command`` will now raise ValueError during
+  construction if there is no __doc__ set. (Note, this will be reverted in
+  2.2b4) (Robert Collins)
+
+* The source tree no longer contains a contrib/zsh/_bzr completion
+  script. The new file contrib/zsh/README suggests alternatives.
+  (Martin von Gagern, #560030)
+
 New Features
 ************
 
@@ -30,25 +1283,76 @@
   pack operation.
   (Parth Malwankar, #304320)
 
+* New command line option ``--authors`` to ``bzr log`` allows users to
+  select which of the apparent authors and committer should be
+  included in the log. Defaults depend on format. (Martin von Gagern, #513322)
+
+* Support ``--directory`` option for a number of additional commands:
+  added, annotate, bind, cat, cat-revision, clean-tree, deleted,
+  export, ignore, ignored, lookup-revision, ls, modified, nick,
+  re-sign, unbind, unknowns.
+  (Martin von Gagern, #527878)
+
+* The bash_completion plugin from the bzr-bash-completion project has
+  been merged into the tree. It provides a bash-completion command and
+  replaces the outdated ``contrib/bash/bzr`` script with a version
+  using the plugin. (Martin von Gagern, #560030)
+
+* A new transport based on GIO (the gnome i/o library) provides access to
+  samba shares, webdav using gio+smb and gio+dav. It is also possible to
+  use gio for some already existing transport methods as gio+file,
+  gio+sftp, gio+ftp. 
+  (Mattias Eriksson)
+
 Bug Fixes
 *********
 
+* Alias information shown by ``bzr help`` is now accurate. This
+  was showing an internal object name for some plugin aliases.
+  (Parth Malwankar, #584650)
+
 * ``.bazaar``, ``.bazaar/bazaar.conf`` and ``.bzr.log`` inherit user and
   group ownership from the containing directory. This allow bzr to work
   better with sudo.
   (Martin <gzlist@googlemail.com>, Parth Malwankar, #376388)
 
+* ``bzr clean-tree`` should not delete nested bzrdirs. Required for proper
+  support of bzr-externals and scmproj plugins.
+  (Alexander Belchenko, bug #572098)
+
+* ``bzr ignore`` will no longer add duplicate patterns to .bzrignore.
+  (Gordon Tyler, #572092)
+
 * ``bzr log --exclude-common-ancestry -r X..Y`` displays the revisions that
   are part of Y ancestry but not part of X ancestry (aka the graph
   difference).
   (Vincent Ladeuil, #320119)
 
+* ``bzr lp-propose`` which was switched to use production Launchpad API
+  servers a few commits ago has been reverted to use edge: there is a
+  problem with using production which isn't trivially obvious, so we've
+  filed a bug to track it, and until thats fixed will be using edge.
+  (Robert Collins, #583667)
+
+* ``bzr rm`` should not refuse to delete directories which contained a file
+  which has been moved elsewhere in the tree after the previous commit.
+  (Marius Kruger, Daniel Watkins, #129880)
+
 * ``bzr selftest --parallel=fork`` wait for its children avoiding zombies.
   (Vincent Ladeuil, #566670)
 
 * ``bzr selftest`` should not use ui.note() since it's not unicode safe.
   (Vincent Ladeuil, #563997)
 
+* CommitBuilder refuses to create revisions whose trees have no root.
+  (Aaron Bentley)
+
+* Do not register a SIGWINCH signal handler, instead just poll for the
+  terminal width as needed.  This avoids the "Interrupted System Call"
+  problems that occur on POSIX with all currently released versions of
+  Python.
+  (Andrew Bennetts, #583941)
+
 * Don't mention --no-strict when we just issue the warning about unclean trees.
   (Vincent Ladeuil, #401599)
 
@@ -56,13 +1360,50 @@
   versions before 1.6.
   (Andrew Bennetts, #528041)
 
+* Improved progress bar for fetch (2a format only). Bazaar now shows an
+  estimate of the number of records to be fetched vs actually fetched.
+  (Parth Malwankar, #374740, #538868)
+
 * Reduce peak memory by one copy of compressed text.
   (John Arbash Meinel, #566940)
 
+* ``RemoteBranch.lock_write`` raises ``ReadOnlyError`` if called during a
+  read lock, rather than causing an ``AttributeError``.
+  (Andrew Bennetts, Данило Шеган, #582781)
+
+* Selftest was failing with testtools 0.9.3, which caused an
+  AssertionError raised from a cleanUp to be reported as a Failure, not an
+  Error, breaking on of our test hygiene tests.
+  (Robert Collins, Vincent Ladeuil).
+
+* ``set_user_option`` with a dict on remote branches no longer fails with
+  an AttributeError.  There is a new ``Branch.set_config_option_dict`` RPC
+  to support this efficiently.
+  (Andrew Bennetts, #430382)
+  
 * Show the filenames when a file rename fails so that the error will be
   more comprehensible.
   (Martin Pool, #491763)
 
+* Support Pyrex 0.9.9, required changing how we handle exceptions in Pyrex.
+  (John Arbash Meinel, #582656)
+
+* Unicode characters in aliases are now handled correctly and do not cause
+  UnicodeEncodeError exception. (Parth Malwankar, #529930)
+
+* Unicode commit messages that are the same as a file name no longer cause
+  UnicodeEncodeError. ``ui.text.show_warning`` now handles unicode
+  messages.
+  (Parth Malwankar, #563646)
+
+* Using bzr with `lp:` urls behind an http proxy should work.
+  (Robert Collins, #558343)
+
+* When passing a file to ``UTF8DirReader`` make sure to close the current
+  directory file handle after the chdir fails. Otherwise when passing many
+  filenames into a command line ``bzr status`` we would leak descriptors.
+  (John Arbash Meinel, #583486)
+
 Improvements
 ************
 
@@ -74,12 +1415,19 @@
   (case-sensitive) as false.
   (Brian de Alwis, Vincent Ladeuil)
 
+* ``bzr ls`` now supports short options for existing long options.
+  ``-k/--kind``, ``-i/--ignored``, ``-u/--unknown`` and ``-0/--null``.
+  (Parth Malwankar, #181124)
+
 * ``Config.get_user_option_as_bool`` will now warn if a value cannot
   be interpreted as a boolean.
   (Vincent Ladeuil)
 
-Documentation
-*************
+* The all-in-one Windows installer will now be built with docstrings stripped
+  from the library zip, reducing the size and slightly improving cold startup
+  time. Bundled plugins are unchanged for the moment, but if adding other new
+  plugins to an all-in-one installation, ensure they are compiled and
+  installed with -O1 or help may not work. (Martin [gz])
 
 API Changes
 ***********
@@ -96,9 +1444,32 @@
   implementations.
   (Martin Pool)
 
+* Lock methods on ``Tree``, ``Branch`` and ``Repository`` are now
+  expected to return an object which can be used to unlock them. This reduces
+  duplicate code when using cleanups. The previous 'tokens's returned by
+  ``Branch.lock_write`` and ``Repository.lock_write`` are now attributes
+  on the result of the lock_write. ``repository.RepositoryWriteLockResult``
+  and ``branch.BranchWriteLockResult`` document this. (Robert Collins)
+
+* ``Repository.refresh_data`` may now be called in a write group on
+  pack-based repositories.  Older repositories will still raise an error
+  in this case.  Subclasses of ``Repository`` can still override
+  ``Repository._refresh_data``, but are now responsible for raising
+  ``bzrlib.repository.IsInWriteGroupError`` if they do not support
+  ``refresh_data`` during a write group.
+  (Andrew Bennetts, #574236)
+
 Internals
 *********
 
+* ``chk_map._bytes_to_text_key`` is now an optimized function to extract
+  the (file-id, revision-id) key from a CHKInventory entry. This can
+  potentially shave 5-10% time off during a large fetch. Related to bug
+  #562666. (John Arbash Meinel)
+
+* ``log._get_info_for_log_files`` now takes an add_cleanup callable.
+  (Robert Collins)
+
 * ``_remember_remote_is_before`` no longer raises AssertionError when
   suboptimal network behaviour is noticed; instead it just mutters to the
   log file (and warns the user if they have set the ``hpss`` debug flag).
@@ -111,6 +1482,10 @@
   stripped, the prefix ``__doc__ =`` should now be used.
   (Martin <gzlist@googlemail.com>)
 
+* No longer require zlib headers to build extensions, and remove the need
+  for seperate copy of zlib library on windows.
+  (John Arbash Meinel, Martin <gzlist@googlemail.com>, #566923)
+
 Testing
 *******
 
@@ -125,6 +1500,67 @@
   failures on Lucid, FreeBSD and gentoo.  
   (Vincent Ladeuil, #528436)
 
+* New class ``ExecutableFeature`` for checking the availability of
+  executables on the ``PATH``. Migrated from bash_completion plugin.
+  (Martin von Gagern)
+
+bzr 2.1.2
+#########
+
+:2.1.2: 2010-05-28
+
+This release fixes two critical networking issues with older servers and
+with interrupted system call errors when pushing or pulling.  We recommend
+upgrading to anyone running a 2.1.x version of bzr.
+
+Bug Fixes
+*********
+
+* ``bzr clean-tree`` should not delete nested bzrdirs. Required for proper
+  support of bzr-externals and scmproj plugins.
+  (Alexander Belchenko, bug #572098)
+
+* ``bzr switch`` does not die if a ConfigurableFileMerger is used.
+  (Aaron Bentley, #559436)
+
+* Do not register a SIGWINCH signal handler, instead just poll for the
+  terminal width as needed.  This avoids the "Interrupted System Call"
+  problems that occur on POSIX with all currently released versions of
+  Python.
+  (Andrew Bennetts, #583941)
+
+* Fixed ``AssertionError`` when accessing smart servers running Bazaar
+  versions before 1.6.
+  (Andrew Bennetts, #528041)
+
+* Reset ``siginterrupt`` flag to False every time we handle a signal
+  installed with ``set_signal_handler(..., restart_syscall=True)`` (from
+  ``bzrlib.osutils``.  Reduces the likelihood of "Interrupted System Call"
+  errors compared to registering ``signal.signal`` directly.
+  (Andrew Bennetts)
+
+* Reduce peak memory by one copy of compressed text.
+  (John Arbash Meinel, #566940)
+
+* Support Pyrex 0.9.9, required changing how we handle exceptions in Pyrex.
+  (John Arbash Meinel, #582656)
+
+* When passing a file to ``UTF8DirReader`` make sure to close the current
+  directory file handle after the chdir fails. Otherwise when passing many
+  filenames into a command line ``bzr status`` we would leak descriptors.
+  (John Arbash Meinel, #583486)
+
+Internals
+*********
+
+* ``_remember_remote_is_before`` no longer raises AssertionError when
+  suboptimal network behaviour is noticed; instead it just mutters to the
+  log file (and warns the user if they have set the ``hpss`` debug flag).
+  This was causing unnecessary aborts for performance bugs that are minor
+  at worst.
+  (Andrew Bennetts, #528041)
+
+
 bzr 2.2b2
 #########
 
@@ -184,7 +1620,7 @@
 * Reset ``siginterrupt`` flag to False every time we handle a signal
   installed with ``set_signal_handler(..., restart_syscall=True)`` (from
   ``bzrlib.osutils``.  Reduces the likelihood of "Interrupted System Call"
-  errors after two window resizes.
+  errors compared to registering ``signal.signal`` directly.
   (Andrew Bennetts)
 
 * When invoked with a range revision, ``bzr log`` doesn't show revisions
@@ -193,7 +1629,7 @@
   (Vincent Ladeuil, #474807)
 
 * Properly handle ``param_name`` attribute for ``ListOption``.
-  (Martin von Gagern, 387117)
+  (Martin von Gagern, #387117)
 
 Improvements
 ************
@@ -232,6 +1668,9 @@
 * ``bzrlib.diff.get_trees_and_branches_to_diff`` is deprecated.  Use
   ``get_trees_and_branches_to_diff_locked`` instead.
   (Andrew Bennetts)
+
+* ``TreeTransform.commit`` supports the full set of commit parameters, and
+  auto-determines branch nick if not supplied.  (Aaron Bentley)
   
 Internals
 *********
@@ -242,10 +1681,28 @@
   Command is now transient and only exists for the duration of ``run()``.
   (Robert Collins)
 
-bzr 2.2.0b1
-###########
-
-:2.2.0b1: 2010-04-01
+bzr 2.2b1
+#########
+
+:2.2b1: 2010-04-01
+
+This is the first beta of the 2.2 series, leading up to a 2.2.0
+release in July or August.  Beta releases are suitable for everyday use
+but may cause some incompatibilities with plugins.  Some plugins may need
+small updates to work with 2.2b1.
+
+2.2b1 includes some changes to make merge conflicts easier to understand
+and resolve.  It also removes some old unnecessary code, and loads
+somewhat less code at startup.  It starts adding a common infrastructure
+for dealing with colocated named branches, which can be implemented in
+various ways in either bzr native or foreign formats.   On Ubuntu and
+other platforms with the apport bug-reporting library, there's an easier
+path to report problems with bzr.  We plan to continue with these themes
+through the 2.2 series.
+
+Over thirty bugs have been fixed, including in the log command, exporting
+to tarballs, restarting interrupted system calls, portability of compiled
+extensions, making backups during upgrade, and locking on ftp.
 
 Compatibility Breaks
 ********************
@@ -291,8 +1748,8 @@
   without specific permission from the user.  (Martin Pool, #515052)
 
 * Parsing of command lines, for example in ``diff --using``, no longer
-  treats backslash as an escape character on Windows.   (Gordon Tyler,
-  #392248)
+  treats backslash as an escape character on Windows.
+  (Gordon Tyler, #392428)
 
 * Plugins can be disabled by defining ``BZR_DISABLE_PLUGINS`` as
   a list of plugin names separated by ':' (';' on windows).
@@ -527,38 +1984,6 @@
   happens, and another warning will be written if the log file could not
   be closed after retrying 100 times.  (Andrew Bennetts, #531746)
 
-bzr 2.1.2
-#########
-
-:2.1.2: NOT RELEASED YET
-
-Bug Fixes
-*********
-
-* ``bzr switch`` does not die if a ConfigurableFileMerger is used.
-  (Aaron Bentley, #559436)
-
-* Fixed ``AssertionError`` when accessing smart servers running Bazaar
-  versions before 1.6.
-  (Andrew Bennetts, #528041)
-
-* Reset ``siginterrupt`` flag to False every time we handle a signal
-  installed with ``set_signal_handler(..., restart_syscall=True)`` (from
-  ``bzrlib.osutils``.  Reduces the likelihood of "Interrupted System Call"
-  errors after two window resizes.
-  (Andrew Bennetts)
-
-Internals
-*********
-
-* ``_remember_remote_is_before`` no longer raises AssertionError when
-  suboptimal network behaviour is noticed; instead it just mutters to the
-  log file (and warns the user if they have set the ``hpss`` debug flag).
-  This was causing unnecessary aborts for performance bugs that are minor
-  at worst.
-  (Andrew Bennetts, #528041)
-
-
 bzr 2.1.1
 #########
 
@@ -624,6 +2049,65 @@
   (Andrew Bennetts, #496813)
 
 
+bzr 2.0.5
+#########
+
+:2.0.5: 2010-03-23
+
+This fifth release in our 2.0 series addresses several user-inconvenience
+bugs.  None are critical, but upgrading is recommended for all users on
+earlier 2.0 releases.
+
+Bug Fixes
+*********
+
+* Avoid ``malloc(0)`` in ``patiencediff``, which is non-portable.
+  (Martin Pool, #331095)
+
+* Concurrent autopacking is more resilient to already-renamed pack files.
+  If we find that a file we are about to obsolete is already obsoleted, we
+  do not try to rename it, and we leave the file in ``obsolete_packs``.
+  The code is also fault tolerant if a file goes missing, assuming that
+  another process already removed the file.
+  (John Arbash Meinel, Gareth White, #507557)
+
+* Cope with the lockdir ``held/info`` file being empty, which seems to
+  happen fairly often if the process is suddenly interrupted while taking
+  a lock.
+  (Martin Pool, #185103)
+
+* Give the warning about potentially slow cross-format fetches much
+  earlier on in the fetch operation.  Don't show this message during
+  upgrades, and show the correct format indication for remote
+  repositories.
+  (Martin Pool, #456077, #515356, #513157)
+
+* Handle renames correctly when there are files or directories that 
+  differ only in case.  (Chris Jones, Martin Pool, #368931)
+
+* If ``bzr push --create-prefix`` triggers an unexpected ``NoSuchFile``
+  error, report that error rather than failing with an unhelpful
+  ``UnboundLocalError``.
+  (Andrew Bennetts, #423563)
+
+* Running ``bzr`` command without any arguments now shows bzr
+  version number along with rest of the help text.
+  (Parth Malwankar, #369501)
+
+* Use osutils.O_NOINHERIT for some files on win32 to avoid PermissionDenied
+  errors.
+  (Inada Naoki, #524560)
+
+Documentation
+*************
+
+* Added ``location-alias`` help topic.
+  (Andrew Bennetts, #337834)
+
+* Fixed CHM generation by moving the NEWS section template into
+  a separate file. (Ian Clatworthy, #524184)
+
+
 bzr 2.1.0
 #########
 
@@ -977,91 +2461,6 @@
   tests that 'failed' - they're all just failures.
   (Martin Pool)
 
-bzr 2.0.6
-#########
-
-:2.0.6: NOT RELEASED YET
-
-Bug Fixes
-*********
-
-* Additional merges after an unrelated branch has been merged with its
-  history no longer crash when deleted files are involved.
-  (Vincent Ladeuil, John Arbash Meinel, #375898)
-
-* ``bzr revert`` now only takes write lock on working tree, instead of on 
-  both working tree and branch.
-  (Danny van Heumen, #498409)
-
-* ``bzr upgrade`` now creates the ``backup.bzr`` directory with the same
-  permissions as ``.bzr`` directory on a POSIX OS.
-  (Parth Malwankar, #262450)
-
-* Repositories accessed via a smart server now reject being stacked on a
-  repository in an incompatible format, as is the case when accessing them
-  via other methods.  This was causing fetches from those repositories via
-  a smart server (e.g. using ``bzr branch``) to receive invalid data.
-  (Andrew Bennetts, #562380)
-
-bzr 2.0.5
-#########
-
-:2.0.5: 2010-03-23
-
-This fifth release in our 2.0 series addresses several user-inconvenience
-bugs.  None are critical, but upgrading is recommended for all users on
-earlier 2.0 releases.
-
-Bug Fixes
-*********
-
-* Avoid ``malloc(0)`` in ``patiencediff``, which is non-portable.
-  (Martin Pool, #331095)
-
-* Concurrent autopacking is more resilient to already-renamed pack files.
-  If we find that a file we are about to obsolete is already obsoleted, we
-  do not try to rename it, and we leave the file in ``obsolete_packs``.
-  The code is also fault tolerant if a file goes missing, assuming that
-  another process already removed the file.
-  (John Arbash Meinel, Gareth White, #507557)
-
-* Cope with the lockdir ``held/info`` file being empty, which seems to
-  happen fairly often if the process is suddenly interrupted while taking
-  a lock.
-  (Martin Pool, #185103)
-
-* Give the warning about potentially slow cross-format fetches much
-  earlier on in the fetch operation.  Don't show this message during
-  upgrades, and show the correct format indication for remote
-  repositories.
-  (Martin Pool, #456077, #515356, #513157)
-
-* Handle renames correctly when there are files or directories that 
-  differ only in case.  (Chris Jones, Martin Pool, #368931)
-
-* If ``bzr push --create-prefix`` triggers an unexpected ``NoSuchFile``
-  error, report that error rather than failing with an unhelpful
-  ``UnboundLocalError``.
-  (Andrew Bennetts, #423563)
-
-* Running ``bzr`` command without any arguments now shows bzr
-  version number along with rest of the help text.
-  (Parth Malwankar, #369501)
-
-* Use osutils.O_NOINHERIT for some files on win32 to avoid PermissionDenied
-  errors.
-  (Inada Naoki, #524560)
-
-Documentation
-*************
-
-* Added ``location-alias`` help topic.
-  (Andrew Bennetts, #337834)
-
-* Fixed CHM generation by moving the NEWS section template into
-  a separate file. (Ian Clatworthy, #524184)
-
-
 bzr 2.0.4
 #########
 
@@ -3702,8 +5101,8 @@
   can have a large effect on ``bzr checkout`` times. (John Arbash Meinel)
 
 * selftest now supports a --parallel option, with values of 'fork' or
-  'subprocess' to run the test suite in parallel. Currently only linux
-  machine work, other platforms need patches submitted. (Robert Collins,
+  'subprocess' to run the test suite in parallel. Currently only Linux
+  machines work, other platforms need patches submitted. (Robert Collins,
   Vincent Ladeuil)
 
 * ``tests.run_suite`` has a new parameter ``suite_decorators``, a list of 
@@ -5299,7 +6698,7 @@
   when the path to the root of the tree has been given. Users of
   the internal ``show_tree_status`` function should be aware that
   the show_pending flag is now authoritative for showing pending
-  merges, as it was originally. (Robert Collins, #225204)
+  merges, as it was originally. (Robert Collins, #255204)
 
 * Set valid default _param_name for Option so that ListOption can embed
   '-' in names. (Vincent Ladeuil, #263249)
@@ -6457,7 +7856,7 @@
 
 * bzr main script cannot be imported (Benjamin Peterson)
 
-* On Linux bzr additionally looks for plugins in arch-independent site
+* On GNU/Linux bzr additionally looks for plugins in arch-independent site
   directory. (Toshio Kuratomi)
 
 * The ``set_rh`` branch hook is now deprecated. Please migrate
@@ -6800,7 +8199,7 @@
 
 * BZR_LOG environment variable controls location of .bzr.log trace file.
   User can suppress writing messages to .bzr.log by using '/dev/null'
-  filename (on Linux) or 'NUL' (on Windows). If BZR_LOG variable
+  filename (on Unix) or 'NUL' (on Windows). If BZR_LOG variable
   is not defined but BZR_HOME is defined then default location
   for .bzr.log trace file is ``$BZR_HOME/.bzr.log``.
   (Alexander Belchenko, #106117)
@@ -9215,7 +10614,7 @@
   hyphen. (``'abc'`` versus ``'abc-2'``). The WT4._iter_changes
   iterator was using direct comparison and ``'abc/a'`` sorts after
   ``'abc-2'``, but ``('abc', 'a')`` sorts before ``('abc-2',)``.
-  (John Arbash Meinel, #111227)
+  (John Arbash Meinel, #111127)
 
 * Handle when someone renames a file on disk without telling bzr.
   Previously we would report the first file as missing, but not show
@@ -10499,11 +11898,11 @@
 * ``Branch.bind(other_branch)`` no longer takes a write lock on the
   other branch, and will not push or pull between the two branches.
   API users will need to perform a push or pull or update operation if they
-  require branch synchronisation to take place. (Robert Collins, #47344)
+  require branch synchronisation to take place. (Robert Collins, #43744)
 
 * When creating a tarball or zipfile export, export unicode names as utf-8
   paths. This may not work perfectly on all platforms, but has the best
-  chance of working in the common case. (John Arbash Meinel, #56816)
+  chance of working in the common case. (John Arbash Meinel, #56815)
 
 * When committing, only files that exist in working tree or basis tree
   may be specified (Aaron Bentley, #50793)
@@ -12543,4 +13942,4 @@
 
 
 ..
-   vim: tw=74 ft=rst ff=unix encoding=utf-8
+   vim: tw=74 ft=rst ff=unix

=== modified file 'bzr'
--- bzr	2010-04-23 08:51:52 +0000
+++ bzr	2010-09-27 19:31:45 +0000
@@ -23,7 +23,7 @@
 import warnings
 
 # update this on each release
-_script_version = (2, 2, 0)
+_script_version = (2, 3, 0)
 
 try:
     version_info = sys.version_info
@@ -52,7 +52,6 @@
 
 profiling = False
 if '--profile-imports' in sys.argv:
-    sys.argv.remove('--profile-imports')
     import profile_imports
     profile_imports.install()
     profiling = True
@@ -76,6 +75,16 @@
 else:
     import locale
 
+if os.name == "posix":
+    try:
+        locale.setlocale(locale.LC_ALL, '')
+    except locale.Error, e:
+        sys.stderr.write('bzr: warning: %s\n'
+            '  bzr could not set the application locale.\n'
+            '  Although this should be no problem for bzr itself, it might\n'
+            '  cause problems with some plugins. To investigate the issue,\n'
+            '  look at the output of the locale(1p) tool.\n' % e)
+
 
 # The python2.6 release includes some libraries that have deprecation warnings
 # against the interpreter - see https://bugs.launchpad.net/bzr/+bug/387139
@@ -87,17 +96,6 @@
     )
 
 
-try:
-    locale.setlocale(locale.LC_ALL, '')
-except locale.Error, e:
-    sys.stderr.write('bzr: warning: %s\n'
-                     '  bzr could not set the application locale.\n'
-                     '  Although this should be no problem for bzr itself,\n'
-                     '  it might cause problems with some plugins.\n'
-                     '  To investigate the issue, look at the output\n'
-                     '  of the locale(1p) tool available on POSIX systems.\n'
-                     % e)
-
 # instruct bzrlib/__init__.py to install lazy_regex
 sys._bzr_lazy_regex = True
 try:
@@ -135,11 +133,14 @@
 
 
 if __name__ == '__main__':
-    bzrlib.initialize()
-    exit_val = bzrlib.commands.main()
-
-    if profiling:
-        profile_imports.log_stack_info(sys.stderr)
+    library_state = bzrlib.initialize()
+    library_state.__enter__()
+    try:
+        exit_val = bzrlib.commands.main()
+        if profiling:
+            profile_imports.log_stack_info(sys.stderr)
+    finally:
+        library_state.__exit__(None, None, None)
 
     # By this point we really have completed everything we want to do, and
     # there's no point doing any additional cleanup.  Abruptly exiting here

=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2010-04-22 10:20:40 +0000
+++ bzrlib/__init__.py	2010-10-07 14:34:39 +0000
@@ -14,7 +14,22 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-"""bzr library"""
+"""All of bzr.
+
+Developer documentation is available at
+http://doc.bazaar.canonical.com/bzr.dev/developers/
+
+The project website is at http://bazaar.canonical.com/
+
+Some particularly interesting things in bzrlib are:
+
+ * bzrlib.initialize -- setup the library for use
+ * bzrlib.plugin.load_plugins -- load all installed plugins
+ * bzrlib.branch.Branch.open -- open a branch
+ * bzrlib.workingtree.WorkingTree.open -- open a working tree
+
+We hope you enjoy this library.
+"""
 
 import time
 
@@ -23,13 +38,6 @@
 _start_time = time.time()
 
 import sys
-if getattr(sys, '_bzr_lazy_regex', False):
-    # The 'bzr' executable sets _bzr_lazy_regex.  We install the lazy regex
-    # hack as soon as possible so that as much of the standard library can
-    # benefit, including the 'string' module.
-    del sys._bzr_lazy_regex
-    import bzrlib.lazy_regex
-    bzrlib.lazy_regex.install_lazy_compile()
 
 
 IGNORE_FILENAME = ".bzrignore"
@@ -44,10 +52,10 @@
 # Python version 2.0 is (2, 0, 0, 'final', 0)."  Additionally we use a
 # releaselevel of 'dev' for unreleased under-development code.
 
-version_info = (2, 2, 0, 'dev', 1)
+version_info = (2, 3, 0, 'dev', 2)
 
 # API compatibility version
-api_minimum_version = (2, 2, 0)
+api_minimum_version = (2, 3, 0)
 
 
 def _format_version_tuple(version_info):
@@ -111,18 +119,37 @@
     return main_version + sub_string
 
 
+# lazy_regex import must be done after _format_version_tuple definition
+# to avoid "no attribute '_format_version_tuple'" error when using
+# deprecated_function in the lazy_regex module.
+if getattr(sys, '_bzr_lazy_regex', False):
+    # The 'bzr' executable sets _bzr_lazy_regex.  We install the lazy regex
+    # hack as soon as possible so that as much of the standard library can
+    # benefit, including the 'string' module.
+    del sys._bzr_lazy_regex
+    import bzrlib.lazy_regex
+    bzrlib.lazy_regex.install_lazy_compile()
+
+
 __version__ = _format_version_tuple(version_info)
 version_string = __version__
 
-
-def test_suite():
-    import tests
-    return tests.test_suite()
-
-
-def initialize(
-    setup_ui=True,
-    stdin=None, stdout=None, stderr=None):
+# bzr has various bits of global state that are slowly being eliminated.
+# This variable is intended to permit any new state-like things to be attached
+# to a library_state.BzrLibraryState object rather than getting new global
+# variables that need to be hunted down. Accessing the current BzrLibraryState
+# through this variable is not encouraged: it is better to pass it around as
+# part of the context of an operation than to look it up directly, but when
+# that is too hard, it is better to use this variable than to make a branch new
+# global variable.
+# If using this variable by looking it up (because it can't be easily obtained)
+# it is important to store the reference you get, rather than looking it up
+# repeatedly; that way your code will behave properly in the bzrlib test suite
+# and from programs that do use multiple library contexts.
+global_state = None
+
+
+def initialize(setup_ui=True, stdin=None, stdout=None, stderr=None):
     """Set up everything needed for normal use of bzrlib.
 
     Most applications that embed bzrlib, including bzr itself, should call
@@ -131,39 +158,29 @@
     More options may be added in future so callers should use named arguments.
 
     :param setup_ui: If true (default) use a terminal UI; otherwise 
-        something else must be put into `bzrlib.ui.ui_factory`.
+        some other ui_factory must be assigned to `bzrlib.ui.ui_factory` by
+        the caller.
     :param stdin, stdout, stderr: If provided, use these for terminal IO;
         otherwise use the files in `sys`.
+    :return: A context manager for the use of bzrlib. The __enter__ method of
+        this context needs to be called before it takes effect, and the __exit__
+        should be called by the caller before exiting their process or
+        otherwise stopping use of bzrlib. Advanced callers can use
+        BzrLibraryState directly.
     """
-    # TODO: mention this in a guide to embedding bzrlib
-    #
-    # NB: This function tweaks so much global state it's hard to test it in
-    # isolation within the same interpreter.  It's not reached on normal
-    # in-process run_bzr calls.  If it's broken, we expect that
-    # TestRunBzrSubprocess may fail.
-    
-    import atexit
-    import bzrlib.trace
-
-    bzrlib.trace.enable_default_logging()
-    atexit.register(bzrlib.trace._flush_stdout_stderr)
-    atexit.register(bzrlib.trace._flush_trace)
-
-    import bzrlib.ui
-    if stdin is None:
-        stdin = sys.stdin
-    if stdout is None:
-        stdout = sys.stdout
-    if stderr is None:
-        stderr = sys.stderr
-
+    from bzrlib import library_state, trace
     if setup_ui:
-        bzrlib.ui.ui_factory = bzrlib.ui.make_ui_for_terminal(
-            stdin, stdout, stderr)
-
-    if bzrlib.version_info[3] == 'final':
-        from bzrlib.symbol_versioning import suppress_deprecation_warnings
-        suppress_deprecation_warnings(override=True)
-
-    import bzrlib.osutils
-    atexit.register(osutils.report_extension_load_failures)
+        import bzrlib.ui
+        stdin = stdin or sys.stdin
+        stdout = stdout or sys.stdout
+        stderr = stderr or sys.stderr
+        ui_factory = bzrlib.ui.make_ui_for_terminal(stdin, stdout, stderr)
+    else:
+        ui_factory = None
+    tracer = trace.DefaultConfig()
+    return library_state.BzrLibraryState(ui=ui_factory, trace=tracer)
+
+
+def test_suite():
+    import tests
+    return tests.test_suite()

=== modified file 'bzrlib/_annotator_py.py'
--- bzrlib/_annotator_py.py	2009-07-08 23:10:47 +0000
+++ bzrlib/_annotator_py.py	2010-06-17 09:23:19 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009 Canonical Ltd
+# Copyright (C) 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,13 +18,15 @@
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
-from bzrlib import annotate # Must be lazy to avoid circular importing
+from bzrlib import (
+    annotate, # Must be lazy to avoid circular importing
+    graph as _mod_graph,
+    patiencediff,
+    )
 """)
 from bzrlib import (
     errors,
-    graph as _mod_graph,
     osutils,
-    patiencediff,
     ui,
     )
 

=== modified file 'bzrlib/_btree_serializer_pyx.pyx'
--- bzrlib/_btree_serializer_pyx.pyx	2010-02-17 17:11:16 +0000
+++ bzrlib/_btree_serializer_pyx.pyx	2010-08-24 19:21:32 +0000
@@ -33,6 +33,7 @@
     char *PyString_AsString(object p) except NULL
     object PyString_FromStringAndSize(char *, Py_ssize_t)
     PyObject *PyString_FromStringAndSize_ptr "PyString_FromStringAndSize" (char *, Py_ssize_t)
+    object PyString_FromFormat(char *, ...)
     int PyString_CheckExact(object s)
     int PyString_CheckExact_ptr "PyString_CheckExact" (PyObject *)
     Py_ssize_t PyString_Size(object p)
@@ -49,19 +50,31 @@
     PyObject *PyTuple_GET_ITEM_ptr_object "PyTuple_GET_ITEM" (object tpl, int index)
     void Py_INCREF(object)
     void Py_DECREF_ptr "Py_DECREF" (PyObject *)
+    void *PyMem_Malloc(size_t nbytes)
+    void PyMem_Free(void *)
+    void memset(void *, int, size_t)
 
 cdef extern from "string.h":
     void *memcpy(void *dest, void *src, size_t n)
     void *memchr(void *s, int c, size_t n)
+    int memcmp(void *s1, void *s2, size_t n)
     # GNU extension
     # void *memrchr(void *s, int c, size_t n)
     int strncmp(char *s1, char *s2, size_t n)
+    unsigned long strtoul(char *s1, char **out, int base)
+    long long strtoll(char *s1, char **out, int base)
+
 
 # It seems we need to import the definitions so that the pyrex compiler has
 # local names to access them.
 from _static_tuple_c cimport StaticTuple, \
     import_static_tuple_c, StaticTuple_New, \
-    StaticTuple_Intern, StaticTuple_SET_ITEM, StaticTuple_CheckExact
+    StaticTuple_Intern, StaticTuple_SET_ITEM, StaticTuple_CheckExact, \
+    StaticTuple_GET_SIZE, StaticTuple_GET_ITEM
+# This tells the test infrastructure that StaticTuple is a class, so we don't
+# have to worry about exception checking.
+## extern cdef class StaticTuple
+import sys
 
 
 # TODO: Find some way to import this from _dirstate_helpers
@@ -103,7 +116,6 @@
     Py_DECREF_ptr(py_str)
     return result
 
-from bzrlib import _static_tuple_c
 # This sets up the StaticTuple C_API functionality
 import_static_tuple_c()
 
@@ -315,6 +327,537 @@
     return parser.parse()
 
 
+# TODO: We can go from 8 byte offset + 4 byte length to a simple lookup,
+#       because the block_offset + length is likely to be repeated. However,
+#       the big win there is to cache across pages, and not just one page
+#       Though if we did cache in a page, we could certainly use a short int.
+#       And this goes from 40 bytes to 30 bytes.
+#       One slightly ugly option would be to cache block offsets in a global.
+#       However, that leads to thread-safety issues, etc.
+ctypedef struct gc_chk_sha1_record:
+    long long block_offset
+    unsigned int block_length
+    unsigned int record_start
+    unsigned int record_end
+    char sha1[20]
+
+
+cdef int _unhexbuf[256]
+cdef char *_hexbuf
+_hexbuf = '0123456789abcdef'
+
+cdef _populate_unhexbuf():
+    cdef int i
+    for i from 0 <= i < 256:
+        _unhexbuf[i] = -1
+    for i from 0 <= i < 10: # 0123456789 => map to the raw number
+        _unhexbuf[(i + c'0')] = i
+    for i from 10 <= i < 16: # abcdef => 10, 11, 12, 13, 14, 15, 16
+        _unhexbuf[(i - 10 + c'a')] = i
+    for i from 10 <= i < 16: # ABCDEF => 10, 11, 12, 13, 14, 15, 16
+        _unhexbuf[(i - 10 + c'A')] = i
+_populate_unhexbuf()
+
+
+cdef int _unhexlify_sha1(char *as_hex, char *as_bin): # cannot_raise
+    """Take the hex sha1 in as_hex and make it binary in as_bin
+    
+    Same as binascii.unhexlify, but working on C strings, not Python objects.
+    """
+    cdef int top
+    cdef int bot
+    cdef int i, j
+    cdef char *cur
+    
+    # binascii does this using isupper() and tolower() and ?: syntax. I'm
+    # guessing a simple lookup array should be faster.
+    j = 0
+    for i from 0 <= i < 20:
+        top = _unhexbuf[<unsigned char>(as_hex[j])]
+        j = j + 1
+        bot = _unhexbuf[<unsigned char>(as_hex[j])]
+        j = j + 1
+        if top == -1 or bot == -1:
+            return 0
+        as_bin[i] = <unsigned char>((top << 4) + bot);
+    return 1
+
+
+def _py_unhexlify(as_hex):
+    """For the test infrastructure, just thunks to _unhexlify_sha1"""
+    if len(as_hex) != 40 or not PyString_CheckExact(as_hex):
+        raise ValueError('not a 40-byte hex digest')
+    as_bin = PyString_FromStringAndSize(NULL, 20)
+    if _unhexlify_sha1(PyString_AS_STRING(as_hex), PyString_AS_STRING(as_bin)):
+        return as_bin
+    return None
+
+
+cdef void _hexlify_sha1(char *as_bin, char *as_hex): # cannot_raise
+    cdef int i, j
+    cdef char c
+
+    j = 0
+    for i from 0 <= i < 20:
+        c = as_bin[i]
+        as_hex[j] = _hexbuf[(c>>4)&0xf]
+        j = j + 1
+        as_hex[j] = _hexbuf[(c)&0xf]
+        j = j + 1
+
+
+def _py_hexlify(as_bin):
+    """For test infrastructure, thunk to _hexlify_sha1"""
+    if len(as_bin) != 20 or not PyString_CheckExact(as_bin):
+        raise ValueError('not a 20-byte binary digest')
+    as_hex = PyString_FromStringAndSize(NULL, 40)
+    _hexlify_sha1(PyString_AS_STRING(as_bin), PyString_AS_STRING(as_hex))
+    return as_hex
+
+
+cdef int _key_to_sha1(key, char *sha1): # cannot_raise
+    """Map a key into its sha1 content.
+
+    :param key: A tuple of style ('sha1:abcd...',)
+    :param sha1: A char buffer of 20 bytes
+    :return: 1 if this could be converted, 0 otherwise
+    """
+    cdef char *c_val
+    cdef PyObject *p_val
+
+    if StaticTuple_CheckExact(key) and StaticTuple_GET_SIZE(key) == 1:
+        p_val = <PyObject *>StaticTuple_GET_ITEM(key, 0)
+    elif (PyTuple_CheckExact(key) and PyTuple_GET_SIZE(key) == 1):
+        p_val = PyTuple_GET_ITEM_ptr_object(key, 0)
+    else:
+        # Not a tuple or a StaticTuple
+        return 0
+    if (PyString_CheckExact_ptr(p_val) and PyString_GET_SIZE_ptr(p_val) == 45):
+        c_val = PyString_AS_STRING_ptr(p_val)
+    else:
+        return 0
+    if strncmp(c_val, 'sha1:', 5) != 0:
+        return 0
+    if not _unhexlify_sha1(c_val + 5, sha1):
+        return 0
+    return 1
+
+
+def _py_key_to_sha1(key):
+    """Map a key to a simple sha1 string.
+
+    This is a testing thunk to the C function.
+    """
+    as_bin_sha = PyString_FromStringAndSize(NULL, 20)
+    if _key_to_sha1(key, PyString_AS_STRING(as_bin_sha)):
+        return as_bin_sha
+    return None
+
+
+cdef StaticTuple _sha1_to_key(char *sha1):
+    """Compute a ('sha1:abcd',) key for a given sha1."""
+    cdef StaticTuple key
+    cdef object hexxed
+    cdef char *c_buf
+    hexxed = PyString_FromStringAndSize(NULL, 45)
+    c_buf = PyString_AS_STRING(hexxed)
+    memcpy(c_buf, 'sha1:', 5)
+    _hexlify_sha1(sha1, c_buf+5)
+    key = StaticTuple_New(1)
+    Py_INCREF(hexxed)
+    StaticTuple_SET_ITEM(key, 0, hexxed)
+    # This is a bit expensive. To parse 120 keys takes 48us, to return them all
+    # can be done in 66.6us (so 18.6us to build them all).
+    # Adding simple hash() here brings it to 76.6us (so computing the hash
+    # value of 120keys is 10us), Intern is 86.9us (another 10us to look and add
+    # them to the intern structure.)
+    # However, since we only intern keys that are in active use, it is probably
+    # a win. Since they would have been read from elsewhere anyway.
+    # We *could* hang the PyObject form off of the gc_chk_sha1_record for ones
+    # that we have deserialized. Something to think about, at least.
+    key = StaticTuple_Intern(key)
+    return key
+
+
+def _py_sha1_to_key(sha1_bin):
+    """Test thunk to check the sha1 mapping."""
+    if not PyString_CheckExact(sha1_bin) or PyString_GET_SIZE(sha1_bin) != 20:
+        raise ValueError('sha1_bin must be a str of exactly 20 bytes')
+    return _sha1_to_key(PyString_AS_STRING(sha1_bin))
+
+
+cdef unsigned int _sha1_to_uint(char *sha1): # cannot_raise
+    cdef unsigned int val
+    # Must be in MSB, because that is how the content is sorted
+    val = (((<unsigned int>(sha1[0]) & 0xff) << 24)
+           | ((<unsigned int>(sha1[1]) & 0xff) << 16)
+           | ((<unsigned int>(sha1[2]) & 0xff) << 8)
+           | ((<unsigned int>(sha1[3]) & 0xff) << 0))
+    return val
+
+
+cdef _format_record_py24(gc_chk_sha1_record *record):
+    """Python2.4 PyString_FromFormat doesn't have %u.
+
+    It only has %d and %ld. We would really like to even have %llu, which
+    is only in python2.7. So we go back into casting to regular objects.
+    """
+    return "%s %s %s %s" % (record.block_offset, record.block_length,
+                            record.record_start, record.record_end)
+
+
+cdef _format_record(gc_chk_sha1_record *record):
+    # This is inefficient to go from a logical state back to a
+    # string, but it makes things work a bit better internally for now.
+    if record.block_offset >= 0xFFFFFFFF:
+        # %llu is what we really want, but unfortunately it was only added
+        # in python 2.7... :(
+        block_offset_str = str(record.block_offset)
+        value = PyString_FromFormat('%s %lu %lu %lu',
+                                PyString_AS_STRING(block_offset_str),
+                                record.block_length,
+                                record.record_start, record.record_end)
+    else:
+        value = PyString_FromFormat('%lu %lu %lu %lu',
+                                    <unsigned long>record.block_offset,
+                                    record.block_length,
+                                    record.record_start, record.record_end)
+    return value
+
+ctypedef object (*formatproc)(gc_chk_sha1_record *)
+cdef formatproc _record_formatter
+_record_formatter = _format_record
+if sys.version_info[:2] == (2, 4):
+    _record_formatter = _format_record_py24
+
+
+cdef class GCCHKSHA1LeafNode:
+    """Track all the entries for a given leaf node."""
+
+    cdef gc_chk_sha1_record *records
+    cdef public object last_key
+    cdef gc_chk_sha1_record *last_record
+    cdef public int num_records
+    # This is the number of bits to shift to get to the interesting byte. A
+    # value of 24 means that the very first byte changes across all keys.
+    # Anything else means that there is a common prefix of bits that we can
+    # ignore. 0 means that at least the first 3 bytes are identical, though
+    # that is going to be very rare
+    cdef public unsigned char common_shift
+    # This maps an interesting byte to the first record that matches.
+    # Equivalent to bisect.bisect_left(self.records, sha1), though only taking
+    # into account that one byte.
+    cdef unsigned char offsets[257]
+
+    def __sizeof__(self):
+        # :( Why doesn't Pyrex let me do a simple sizeof(GCCHKSHA1LeafNode)
+        # like Cython? Explicitly enumerating everything here seems to leave my
+        # size off by 2 (286 bytes vs 288 bytes actual). I'm guessing it is an
+        # alignment/padding issue. Oh well- at least we scale properly with
+        # num_records and are very close to correct, which is what I care
+        # about.
+        # If we ever decide to require cython:
+        # return (sizeof(GCCHKSHA1LeafNode)
+        #     + sizeof(gc_chk_sha1_record)*self.num_records)
+        return (sizeof(PyObject) + sizeof(void*) + sizeof(int)
+            + sizeof(gc_chk_sha1_record*) + sizeof(PyObject *)
+            + sizeof(gc_chk_sha1_record*) + sizeof(char)
+            + sizeof(unsigned char)*257
+            + sizeof(gc_chk_sha1_record)*self.num_records)
+
+    def __dealloc__(self):
+        if self.records != NULL:
+            PyMem_Free(self.records)
+            self.records = NULL
+
+    def __init__(self, bytes):
+        self._parse_bytes(bytes)
+        self.last_key = None
+        self.last_record = NULL
+
+    property min_key:
+        def __get__(self):
+            if self.num_records > 0:
+                return _sha1_to_key(self.records[0].sha1)
+            return None
+
+    property max_key:
+        def __get__(self):
+            if self.num_records > 0:
+                return _sha1_to_key(self.records[self.num_records-1].sha1)
+            return None
+
+    cdef StaticTuple _record_to_value_and_refs(self,
+                                               gc_chk_sha1_record *record):
+        """Extract the refs and value part of this record."""
+        cdef StaticTuple value_and_refs
+        cdef StaticTuple empty
+        value_and_refs = StaticTuple_New(2)
+        value = _record_formatter(record)
+        Py_INCREF(value)
+        StaticTuple_SET_ITEM(value_and_refs, 0, value)
+        # Always empty refs
+        empty = StaticTuple_New(0)
+        Py_INCREF(empty)
+        StaticTuple_SET_ITEM(value_and_refs, 1, empty)
+        return value_and_refs
+
+    cdef StaticTuple _record_to_item(self, gc_chk_sha1_record *record):
+        """Turn a given record back into a fully fledged item.
+        """
+        cdef StaticTuple item
+        cdef StaticTuple key
+        cdef StaticTuple value_and_refs
+        cdef object value
+        key = _sha1_to_key(record.sha1)
+        item = StaticTuple_New(2)
+        Py_INCREF(key)
+        StaticTuple_SET_ITEM(item, 0, key)
+        value_and_refs = self._record_to_value_and_refs(record)
+        Py_INCREF(value_and_refs)
+        StaticTuple_SET_ITEM(item, 1, value_and_refs)
+        return item
+
+    cdef gc_chk_sha1_record* _lookup_record(self, char *sha1) except? NULL:
+        """Find a gc_chk_sha1_record that matches the sha1 supplied."""
+        cdef int lo, hi, mid, the_cmp
+        cdef int offset
+
+        # TODO: We can speed up misses by comparing this sha1 to the common
+        #       bits, and seeing if the common prefix matches, if not, we don't
+        #       need to search for anything because it cannot match
+        # Use the offset array to find the closest fit for this entry
+        # follow that up with bisecting, since multiple keys can be in one
+        # spot
+        # Bisecting dropped us from 7000 comparisons to 582 (4.8/key), using
+        # the offset array dropped us from 23us to 20us and 156 comparisions
+        # (1.3/key)
+        offset = self._offset_for_sha1(sha1)
+        lo = self.offsets[offset]
+        hi = self.offsets[offset+1]
+        if hi == 255:
+            # if hi == 255 that means we potentially ran off the end of the
+            # list, so push it up to num_records
+            # note that if 'lo' == 255, that is ok, because we can start
+            # searching from that part of the list.
+            hi = self.num_records
+        local_n_cmp = 0
+        while lo < hi:
+            mid = (lo + hi) / 2
+            the_cmp = memcmp(self.records[mid].sha1, sha1, 20)
+            if the_cmp == 0:
+                return &self.records[mid]
+            elif the_cmp < 0:
+                lo = mid + 1
+            else:
+                hi = mid
+        return NULL
+
+    def __contains__(self, key):
+        cdef char sha1[20]
+        cdef gc_chk_sha1_record *record
+        if _key_to_sha1(key, sha1):
+            # If it isn't a sha1 key, then it won't be in this leaf node
+            record = self._lookup_record(sha1)
+            if record != NULL:
+                self.last_key = key
+                self.last_record = record
+                return True
+        return False
+
+    def __getitem__(self, key):
+        cdef char sha1[20]
+        cdef gc_chk_sha1_record *record
+        record = NULL
+        if self.last_record != NULL and key is self.last_key:
+            record = self.last_record
+        elif _key_to_sha1(key, sha1):
+            record = self._lookup_record(sha1)
+        if record == NULL:
+            raise KeyError('key %r is not present' % (key,))
+        return self._record_to_value_and_refs(record)
+
+    def __len__(self):
+        return self.num_records
+
+    def all_keys(self):
+        cdef int i
+        result = []
+        for i from 0 <= i < self.num_records:
+            PyList_Append(result, _sha1_to_key(self.records[i].sha1))
+        return result
+
+    def all_items(self):
+        cdef int i
+        result = []
+        for i from 0 <= i < self.num_records:
+            item = self._record_to_item(&self.records[i])
+            PyList_Append(result, item)
+        return result
+
+    cdef int _count_records(self, char *c_content, char *c_end): # cannot_raise
+        """Count how many records are in this section."""
+        cdef char *c_cur
+        cdef int num_records
+
+        c_cur = c_content
+        num_records = 0
+        while c_cur != NULL and c_cur < c_end:
+            c_cur = <char *>memchr(c_cur, c'\n', c_end - c_cur);
+            if c_cur == NULL:
+                break
+            c_cur = c_cur + 1
+            num_records = num_records + 1
+        return num_records
+
+    cdef _parse_bytes(self, bytes):
+        """Parse the string 'bytes' into content."""
+        cdef char *c_bytes
+        cdef char *c_cur
+        cdef char *c_end
+        cdef Py_ssize_t n_bytes
+        cdef int num_records
+        cdef int entry
+        cdef gc_chk_sha1_record *cur_record
+
+        if not PyString_CheckExact(bytes):
+            raise TypeError('We only support parsing plain 8-bit strings.')
+        # Pass 1, count how many records there will be
+        n_bytes = PyString_GET_SIZE(bytes)
+        c_bytes = PyString_AS_STRING(bytes)
+        c_end = c_bytes + n_bytes
+        if strncmp(c_bytes, 'type=leaf\n', 10):
+            raise ValueError("bytes did not start with 'type=leaf\\n': %r"
+                             % (bytes[:10],))
+        c_cur = c_bytes + 10
+        num_records = self._count_records(c_cur, c_end)
+        # Now allocate the memory for these items, and go to town
+        self.records = <gc_chk_sha1_record*>PyMem_Malloc(num_records *
+            (sizeof(unsigned short) + sizeof(gc_chk_sha1_record)))
+        self.num_records = num_records
+        cur_record = self.records
+        entry = 0
+        while c_cur != NULL and c_cur < c_end and entry < num_records:
+            c_cur = self._parse_one_entry(c_cur, c_end, cur_record)
+            cur_record = cur_record + 1
+            entry = entry + 1
+        if (entry != self.num_records
+            or c_cur != c_end
+            or cur_record != self.records + self.num_records):
+            raise ValueError('Something went wrong while parsing.')
+        # Pass 3: build the offset map
+        self._compute_common()
+
+    cdef char *_parse_one_entry(self, char *c_cur, char *c_end,
+                                gc_chk_sha1_record *cur_record) except NULL:
+        """Read a single sha record from the bytes.
+        :param c_cur: The pointer to the start of bytes
+        :param cur_record: 
+        """
+        cdef char *c_next
+        if strncmp(c_cur, 'sha1:', 5):
+            raise ValueError('line did not start with sha1: %r'
+                % (safe_string_from_size(c_cur, 10),))
+        c_cur = c_cur + 5
+        c_next = <char *>memchr(c_cur, c'\0', c_end - c_cur)
+        if c_next == NULL or (c_next - c_cur != 40):
+            raise ValueError('Line did not contain 40 hex bytes')
+        if not _unhexlify_sha1(c_cur, cur_record.sha1):
+            raise ValueError('We failed to unhexlify')
+        c_cur = c_next + 1
+        if c_cur[0] != c'\0':
+            raise ValueError('only 1 null, not 2 as expected')
+        c_cur = c_cur + 1
+        cur_record.block_offset = strtoll(c_cur, &c_next, 10)
+        if c_cur == c_next or c_next[0] != c' ':
+            raise ValueError('Failed to parse block offset')
+        c_cur = c_next + 1
+        cur_record.block_length = strtoul(c_cur, &c_next, 10)
+        if c_cur == c_next or c_next[0] != c' ':
+            raise ValueError('Failed to parse block length')
+        c_cur = c_next + 1
+        cur_record.record_start = strtoul(c_cur, &c_next, 10)
+        if c_cur == c_next or c_next[0] != c' ':
+            raise ValueError('Failed to parse block length')
+        c_cur = c_next + 1
+        cur_record.record_end = strtoul(c_cur, &c_next, 10)
+        if c_cur == c_next or c_next[0] != c'\n':
+            raise ValueError('Failed to parse record end')
+        c_cur = c_next + 1
+        return c_cur
+
+    cdef int _offset_for_sha1(self, char *sha1) except -1:
+        """Find the first interesting 8-bits of this sha1."""
+        cdef int this_offset
+        cdef unsigned int as_uint
+        as_uint = _sha1_to_uint(sha1)
+        this_offset = (as_uint >> self.common_shift) & 0xFF
+        return this_offset
+
+    def _get_offset_for_sha1(self, sha1):
+        return self._offset_for_sha1(PyString_AS_STRING(sha1))
+
+    cdef _compute_common(self):
+        cdef unsigned int first
+        cdef unsigned int this
+        cdef unsigned int common_mask
+        cdef unsigned char common_shift
+        cdef int i
+        cdef int offset, this_offset
+        cdef int max_offset
+        # The idea with the offset map is that we should be able to quickly
+        # jump to the key that matches a gives sha1. We know that the keys are
+        # in sorted order, and we know that a lot of the prefix is going to be
+        # the same across them.
+        # By XORing the records together, we can determine what bits are set in
+        # all of them
+        if self.num_records < 2:
+            # Everything is in common if you have 0 or 1 leaves
+            # So we'll always just shift to the first byte
+            self.common_shift = 24
+        else:
+            common_mask = 0xFFFFFFFF
+            first = _sha1_to_uint(self.records[0].sha1)
+            for i from 0 < i < self.num_records:
+                this = _sha1_to_uint(self.records[i].sha1)
+                common_mask = (~(first ^ this)) & common_mask
+            common_shift = 24
+            while common_mask & 0x80000000 and common_shift > 0:
+                common_mask = common_mask << 1
+                common_shift = common_shift - 1
+            self.common_shift = common_shift
+        offset = 0
+        max_offset = self.num_records
+        # We cap this loop at 254 records. All the other offsets just get
+        # filled with 0xff as the singleton saying 'too many'.
+        # It means that if we have >255 records we have to bisect the second
+        # half of the list, but this is going to be very rare in practice.
+        if max_offset > 255:
+            max_offset = 255
+        for i from 0 <= i < max_offset:
+            this_offset = self._offset_for_sha1(self.records[i].sha1)
+            while offset <= this_offset:
+                self.offsets[offset] = i
+                offset = offset + 1
+        while offset < 257:
+            self.offsets[offset] = max_offset
+            offset = offset + 1
+
+    def _get_offsets(self):
+        cdef int i
+        result = []
+        for i from 0 <= i < 257:
+            PyList_Append(result, self.offsets[i])
+        return result
+
+
+def _parse_into_chk(bytes, key_length, ref_list_length):
+    """Parse into a format optimized for chk records."""
+    assert key_length == 1
+    assert ref_list_length == 0
+    return GCCHKSHA1LeafNode(bytes)
+
+
 def _flatten_node(node, reference_lists):
     """Convert a node into the serialized form.
 

=== modified file 'bzrlib/_chk_map_py.py'
--- bzrlib/_chk_map_py.py	2010-02-17 17:11:16 +0000
+++ bzrlib/_chk_map_py.py	2010-05-11 10:45:26 +0000
@@ -157,3 +157,11 @@
     result._node_width = len(prefix)
     result._search_prefix = common_prefix
     return result
+
+
+def _bytes_to_text_key(bytes):
+    """Take a CHKInventory value string and return a (file_id, rev_id) tuple"""
+    sections = bytes.split('\n')
+    kind, file_id = sections[0].split(': ')
+    return (intern(file_id), intern(sections[3]))
+

=== modified file 'bzrlib/_chk_map_pyx.pyx'
--- bzrlib/_chk_map_pyx.pyx	2010-02-17 17:11:16 +0000
+++ bzrlib/_chk_map_pyx.pyx	2010-05-16 15:18:43 +0000
@@ -35,40 +35,26 @@
     Py_ssize_t PyTuple_GET_SIZE(object t)
     int PyString_CheckExact(object)
     char *PyString_AS_STRING(object s)
+    PyObject *PyString_FromStringAndSize_ptr "PyString_FromStringAndSize" (char *, Py_ssize_t)
     Py_ssize_t PyString_GET_SIZE(object)
+    void PyString_InternInPlace(PyObject **)
+    long PyInt_AS_LONG(object)
 
     int PyDict_SetItem(object d, object k, object v) except -1
 
-    object PyTuple_New(Py_ssize_t count)
-    void PyTuple_SET_ITEM(object t, Py_ssize_t offset, object)
-
     void Py_INCREF(object)
+    void Py_DECREF_ptr "Py_DECREF" (PyObject *)
 
-    PyObject * PyTuple_GET_ITEM_ptr "PyTuple_GET_ITEM" (object t,
-                                                        Py_ssize_t offset)
-    int PyString_CheckExact_ptr "PyString_CheckExact" (PyObject *p)
-    Py_ssize_t PyString_GET_SIZE_ptr "PyString_GET_SIZE" (PyObject *s)
-    char *PyString_AS_STRING_ptr "PyString_AS_STRING" (PyObject *s)
     object PyString_FromStringAndSize(char*, Py_ssize_t)
 
 # cimport all of the definitions we will need to access
 from _static_tuple_c cimport StaticTuple,\
     import_static_tuple_c, StaticTuple_New, \
-    StaticTuple_Intern, StaticTuple_SET_ITEM, StaticTuple_CheckExact
-
-cdef extern from "_static_tuple_c.h":
-    # Defined explicitly rather than cimport-ing. Trying to use cimport, the
-    # type for PyObject is a different class that happens to have the same
-    # name...
-    PyObject * StaticTuple_GET_ITEM_ptr "StaticTuple_GET_ITEM" (StaticTuple,
-                                                                Py_ssize_t)
-
-cdef extern from "zlib.h":
-    ctypedef unsigned long uLong
-    ctypedef unsigned int uInt
-    ctypedef unsigned char Bytef
-
-    uLong crc32(uLong crc, Bytef *buf, uInt len)
+    StaticTuple_Intern, StaticTuple_SET_ITEM, StaticTuple_CheckExact, \
+    StaticTuple_GET_SIZE
+
+cdef object crc32
+from zlib import crc32
 
 
 # Set up the StaticTuple C_API functionality
@@ -96,20 +82,30 @@
     return NULL
 
 
+cdef object safe_interned_string_from_size(char *s, Py_ssize_t size):
+    cdef PyObject *py_str
+    if size < 0:
+        raise AssertionError(
+            'tried to create a string with an invalid size: %d @0x%x'
+            % (size, <int>s))
+    py_str = PyString_FromStringAndSize_ptr(s, size)
+    PyString_InternInPlace(&py_str)
+    result = <object>py_str
+    # Casting a PyObject* to an <object> triggers an INCREF from Pyrex, so we
+    # DECREF it to avoid geting immortal strings
+    Py_DECREF_ptr(py_str)
+    return result
+
+
 def _search_key_16(key):
     """See chk_map._search_key_16."""
     cdef Py_ssize_t num_bits
     cdef Py_ssize_t i, j
     cdef Py_ssize_t num_out_bytes
-    cdef Bytef *c_bit
-    cdef uLong c_len
-    cdef uInt crc_val
+    cdef unsigned long crc_val
     cdef Py_ssize_t out_off
     cdef char *c_out
-    cdef PyObject *bit
 
-    if not StaticTuple_CheckExact(key):
-        raise TypeError('key %r is not a StaticTuple' % (key,))
     num_bits = len(key)
     # 4 bytes per crc32, and another 1 byte between bits
     num_out_bytes = (9 * num_bits) - 1
@@ -119,15 +115,7 @@
         if i > 0:
             c_out[0] = c'\x00'
             c_out = c_out + 1
-        # We use the _ptr variant, because GET_ITEM returns a borrowed
-        # reference, and Pyrex assumes that returned 'object' are a new
-        # reference
-        bit = StaticTuple_GET_ITEM_ptr(key, i)
-        if not PyString_CheckExact_ptr(bit):
-            raise TypeError('Bit %d of %r is not a string' % (i, key))
-        c_bit = <Bytef *>PyString_AS_STRING_ptr(bit)
-        c_len = PyString_GET_SIZE_ptr(bit)
-        crc_val = crc32(0, c_bit, c_len)
+        crc_val = PyInt_AS_LONG(crc32(key[i]))
         # Hex(val) order
         sprintf(c_out, '%08X', crc_val)
         c_out = c_out + 8
@@ -139,15 +127,10 @@
     cdef Py_ssize_t num_bits
     cdef Py_ssize_t i, j
     cdef Py_ssize_t num_out_bytes
-    cdef Bytef *c_bit
-    cdef uLong c_len
-    cdef uInt crc_val
+    cdef unsigned long crc_val
     cdef Py_ssize_t out_off
     cdef char *c_out
-    cdef PyObject *bit
 
-    if not StaticTuple_CheckExact(key):
-        raise TypeError('key %r is not a StaticTuple' % (key,))
     num_bits = len(key)
     # 4 bytes per crc32, and another 1 byte between bits
     num_out_bytes = (5 * num_bits) - 1
@@ -157,13 +140,7 @@
         if i > 0:
             c_out[0] = c'\x00'
             c_out = c_out + 1
-        bit = StaticTuple_GET_ITEM_ptr(key, i)
-        if not PyString_CheckExact_ptr(bit):
-            raise TypeError('Bit %d of %r is not a string: %r'
-                            % (i, key, <object>bit))
-        c_bit = <Bytef *>PyString_AS_STRING_ptr(bit)
-        c_len = PyString_GET_SIZE_ptr(bit)
-        crc_val = crc32(0, c_bit, c_len)
+        crc_val = PyInt_AS_LONG(crc32(key[i]))
         # MSB order
         c_out[0] = (crc_val >> 24) & 0xFF
         c_out[1] = (crc_val >> 16) & 0xFF
@@ -198,6 +175,16 @@
     return value
 
 
+cdef _import_globals():
+    """Set the global attributes. Done lazy to avoid recursive import loops."""
+    global _LeafNode, _InternalNode, _unknown
+
+    from bzrlib import chk_map
+    _LeafNode = chk_map.LeafNode
+    _InternalNode = chk_map.InternalNode
+    _unknown = chk_map._unknown
+
+
 def _deserialise_leaf_node(bytes, key, search_key_func=None):
     """Deserialise bytes, with key key, into a LeafNode.
 
@@ -215,10 +202,7 @@
     cdef StaticTuple entry_bits
 
     if _LeafNode is None:
-        from bzrlib import chk_map
-        _LeafNode = chk_map.LeafNode
-        _InternalNode = chk_map.InternalNode
-        _unknown = chk_map._unknown
+        _import_globals()
 
     result = _LeafNode(search_key_func=search_key_func)
     # Splitlines can split on '\r' so don't use it, split('\n') adds an
@@ -322,7 +306,7 @@
                                                next_null - entry_start)
             Py_INCREF(entry)
             StaticTuple_SET_ITEM(entry_bits, i, entry)
-        if len(entry_bits) != width:
+        if StaticTuple_GET_SIZE(entry_bits) != width:
             raise AssertionError(
                 'Incorrect number of elements (%d vs %d)'
                 % (len(entry_bits)+1, width + 1))
@@ -358,10 +342,7 @@
     cdef char *prefix, *line_prefix, *next_null, *c_item_prefix
 
     if _InternalNode is None:
-        from bzrlib import chk_map
-        _LeafNode = chk_map.LeafNode
-        _InternalNode = chk_map.InternalNode
-        _unknown = chk_map._unknown
+        _import_globals()
     result = _InternalNode(search_key_func=search_key_func)
 
     if not StaticTuple_CheckExact(key):
@@ -422,3 +403,51 @@
     result._node_width = len(item_prefix)
     result._search_prefix = PyString_FromStringAndSize(prefix, prefix_length)
     return result
+
+
+def _bytes_to_text_key(bytes):
+    """Take a CHKInventory value string and return a (file_id, rev_id) tuple"""
+    cdef StaticTuple key
+    cdef char *byte_str, *cur_end, *file_id_str, *byte_end
+    cdef char *revision_str
+    cdef Py_ssize_t byte_size, pos, file_id_len
+
+    if not PyString_CheckExact(bytes):
+        raise TypeError('bytes must be a string')
+    byte_str = PyString_AS_STRING(bytes)
+    byte_size = PyString_GET_SIZE(bytes)
+    byte_end = byte_str + byte_size
+    cur_end = <char*>memchr(byte_str, c':', byte_size)
+    if cur_end == NULL:
+        raise ValueError('No kind section found.')
+    if cur_end[1] != c' ':
+        raise ValueError('Kind section should end with ": "')
+    file_id_str = cur_end + 2
+    # file_id is now the data up until the next newline
+    cur_end = <char*>memchr(file_id_str, c'\n', byte_end - file_id_str)
+    if cur_end == NULL:
+        raise ValueError('no newline after file-id')
+    file_id = safe_interned_string_from_size(file_id_str,
+                                             cur_end - file_id_str)
+    # this is the end of the parent_str
+    cur_end = <char*>memchr(cur_end + 1, c'\n', byte_end - cur_end - 1)
+    if cur_end == NULL:
+        raise ValueError('no newline after parent_str')
+    # end of the name str
+    cur_end = <char*>memchr(cur_end + 1, c'\n', byte_end - cur_end - 1)
+    if cur_end == NULL:
+        raise ValueError('no newline after name str')
+    # the next section is the revision info
+    revision_str = cur_end + 1
+    cur_end = <char*>memchr(cur_end + 1, c'\n', byte_end - cur_end - 1)
+    if cur_end == NULL:
+        # This is probably a dir: entry, which has revision as the last item
+        cur_end = byte_end
+    revision = safe_interned_string_from_size(revision_str,
+        cur_end - revision_str)
+    key = StaticTuple_New(2)
+    Py_INCREF(file_id)
+    StaticTuple_SET_ITEM(key, 0, file_id) 
+    Py_INCREF(revision)
+    StaticTuple_SET_ITEM(key, 1, revision) 
+    return StaticTuple_Intern(key)

=== modified file 'bzrlib/_dirstate_helpers_pyx.pyx'
--- bzrlib/_dirstate_helpers_pyx.pyx	2010-02-23 07:43:11 +0000
+++ bzrlib/_dirstate_helpers_pyx.pyx	2010-08-27 18:02:22 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007, 2008, 2010 Canonical Ltd
+# Copyright (C) 2007-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -118,6 +118,11 @@
     # ??? memrchr is a GNU extension :(
     # void *memrchr(void *s, int c, size_t len)
 
+# cimport all of the definitions we will need to access
+from _static_tuple_c cimport import_static_tuple_c, StaticTuple, \
+    StaticTuple_New, StaticTuple_SET_ITEM
+
+import_static_tuple_c()
 
 cdef void* _my_memrchr(void *s, int c, size_t n): # cannot_raise
     # memrchr seems to be a GNU extension, so we have to implement it ourselves
@@ -610,7 +615,8 @@
         :param new_block: This is to let the caller know that it needs to
             create a new directory block to store the next entry.
         """
-        cdef object path_name_file_id_key
+        cdef StaticTuple path_name_file_id_key
+        cdef StaticTuple tmp
         cdef char *entry_size_cstr
         cdef unsigned long int entry_size
         cdef char* executable_cstr
@@ -650,10 +656,20 @@
         # Build up the key that will be used.
         # By using <object>(void *) Pyrex will automatically handle the
         # Py_INCREF that we need.
-        path_name_file_id_key = (<object>p_current_dirname[0],
-                                 self.get_next_str(),
-                                 self.get_next_str(),
-                                )
+        cur_dirname = <object>p_current_dirname[0]
+        # Use StaticTuple_New to pre-allocate, rather than creating a regular
+        # tuple and passing it to the StaticTuple constructor.
+        # path_name_file_id_key = StaticTuple(<object>p_current_dirname[0],
+        #                          self.get_next_str(),
+        #                          self.get_next_str(),
+        #                         )
+        tmp = StaticTuple_New(3)
+        Py_INCREF(cur_dirname); StaticTuple_SET_ITEM(tmp, 0, cur_dirname)
+        cur_basename = self.get_next_str()
+        cur_file_id = self.get_next_str()
+        Py_INCREF(cur_basename); StaticTuple_SET_ITEM(tmp, 1, cur_basename)
+        Py_INCREF(cur_file_id); StaticTuple_SET_ITEM(tmp, 2, cur_file_id)
+        path_name_file_id_key = tmp
 
         # Parse all of the per-tree information. current has the information in
         # the same location as parent trees. The only difference is that 'info'
@@ -677,7 +693,20 @@
             executable_cstr = self.get_next(&cur_size)
             is_executable = (executable_cstr[0] == c'y')
             info = self.get_next_str()
-            PyList_Append(trees, (
+            # TODO: If we want to use StaticTuple_New here we need to be pretty
+            #       careful. We are relying on a bit of Pyrex
+            #       automatic-conversion from 'int' to PyInt, and that doesn't
+            #       play well with the StaticTuple_SET_ITEM macro.
+            #       Timing doesn't (yet) show a worthwile improvement in speed
+            #       versus complexity and maintainability.
+            # tmp = StaticTuple_New(5)
+            # Py_INCREF(minikind); StaticTuple_SET_ITEM(tmp, 0, minikind)
+            # Py_INCREF(fingerprint); StaticTuple_SET_ITEM(tmp, 1, fingerprint)
+            # Py_INCREF(entry_size); StaticTuple_SET_ITEM(tmp, 2, entry_size)
+            # Py_INCREF(is_executable); StaticTuple_SET_ITEM(tmp, 3, is_executable)
+            # Py_INCREF(info); StaticTuple_SET_ITEM(tmp, 4, info)
+            # PyList_Append(trees, tmp)
+            PyList_Append(trees, StaticTuple(
                 minikind,     # minikind
                 fingerprint,  # fingerprint
                 entry_size,   # size
@@ -1219,7 +1248,7 @@
             else:
                 try:
                     source_parent_id = self.old_dirname_to_file_id[old_dirname]
-                except KeyError:
+                except KeyError, _:
                     source_parent_entry = self.state._get_entry(self.source_index,
                                                            path_utf8=old_dirname)
                     source_parent_id = source_parent_entry[0][2]
@@ -1236,7 +1265,7 @@
             else:
                 try:
                     target_parent_id = self.new_dirname_to_file_id[new_dirname]
-                except KeyError:
+                except KeyError, _:
                     # TODO: We don't always need to do the lookup, because the
                     #       parent entry will be the same as the source entry.
                     target_parent_entry = self.state._get_entry(self.target_index,
@@ -1478,7 +1507,7 @@
             # interface doesn't require it.
             try:
                 self.current_root = self.search_specific_files.pop()
-            except KeyError:
+            except KeyError, _:
                 raise StopIteration()
             self.searched_specific_files.add(self.current_root)
             # process the entries for this containing directory: the rest will be
@@ -1567,7 +1596,7 @@
                         #            and e.winerror == ERROR_DIRECTORY
                         try:
                             e_winerror = e.winerror
-                        except AttributeError:
+                        except AttributeError, _:
                             e_winerror = None
                         win_errors = (ERROR_DIRECTORY, ERROR_PATH_NOT_FOUND)
                         if (e.errno in win_errors or e_winerror in win_errors):
@@ -1656,7 +1685,7 @@
                     try:
                         self.current_dir_info = self.dir_iterator.next()
                         self.current_dir_list = self.current_dir_info[1]
-                    except StopIteration:
+                    except StopIteration, _:
                         self.current_dir_info = None
                 else: #(dircmp > 0)
                     # We have a dirblock entry for this location, but there
@@ -1803,7 +1832,7 @@
                                 and stat.S_IEXEC & current_path_info[3].st_mode)
                             try:
                                 relpath_unicode = self.utf8_decode(current_path_info[0])[0]
-                            except UnicodeDecodeError:
+                            except UnicodeDecodeError, _:
                                 raise errors.BadFilenameEncoding(
                                     current_path_info[0], osutils._fs_enc)
                             if changed is not None:
@@ -1851,7 +1880,7 @@
                 try:
                     self.current_dir_info = self.dir_iterator.next()
                     self.current_dir_list = self.current_dir_info[1]
-                except StopIteration:
+                except StopIteration, _:
                     self.current_dir_info = None
 
     cdef object _next_consistent_entries(self):

=== modified file 'bzrlib/_groupcompress_pyx.pyx'
--- bzrlib/_groupcompress_pyx.pyx	2010-02-17 17:11:16 +0000
+++ bzrlib/_groupcompress_pyx.pyx	2010-08-02 17:08:29 +0000
@@ -22,6 +22,8 @@
 
 
 cdef extern from "Python.h":
+    ctypedef struct PyObject:
+        pass
     ctypedef int Py_ssize_t # Required for older pyrex versions
     int PyString_CheckExact(object)
     char * PyString_AS_STRING(object)
@@ -53,6 +55,7 @@
              unsigned long *delta_size, unsigned long max_delta_size) nogil
     unsigned long get_delta_hdr_size(unsigned char **datap,
                                      unsigned char *top) nogil
+    unsigned long sizeof_delta_index(delta_index *index)
     Py_ssize_t DELTA_SIZE_MIN
 
 
@@ -91,8 +94,8 @@
     cdef readonly object _sources
     cdef source_info *_source_infos
     cdef delta_index *_index
+    cdef public unsigned long _source_offset
     cdef readonly unsigned int _max_num_sources
-    cdef public unsigned long _source_offset
 
     def __init__(self, source=None):
         self._sources = []
@@ -105,6 +108,22 @@
         if source is not None:
             self.add_source(source, 0)
 
+    def __sizeof__(self):
+        # We want to track the _source_infos allocations, but the referenced
+        # void* are actually tracked in _sources itself.
+        # XXX: Cython is capable of doing sizeof(class) and returning the size
+        #      of the underlying struct. Pyrex (<= 0.9.9) refuses, so we need
+        #      to do it manually. *sigh* Note that we might get it wrong
+        #      because of alignment issues.
+        cdef Py_ssize_t size
+        # PyObject start, vtable *, 3 object pointers, 2 C ints
+        size = ((sizeof(PyObject) + sizeof(void*) + 3*sizeof(PyObject*)
+                 + sizeof(unsigned long)
+                 + sizeof(unsigned int))
+                + (sizeof(source_info) * self._max_num_sources)
+                + sizeof_delta_index(self._index))
+        return size
+
     def __repr__(self):
         return '%s(%d, %d)' % (self.__class__.__name__,
             len(self._sources), self._source_offset)

=== modified file 'bzrlib/_readdir_pyx.pyx'
--- bzrlib/_readdir_pyx.pyx	2010-02-17 17:11:16 +0000
+++ bzrlib/_readdir_pyx.pyx	2010-05-21 13:36:51 +0000
@@ -106,15 +106,22 @@
     int closedir(DIR * dir)
     dirent *readdir(DIR *dir)
 
+cdef object _directory
 _directory = 'directory'
+cdef object _chardev
 _chardev = 'chardev'
+cdef object _block
 _block = 'block'
+cdef object _file
 _file = 'file'
+cdef object _fifo
 _fifo = 'fifo'
+cdef object _symlink
 _symlink = 'symlink'
+cdef object _socket
 _socket = 'socket'
+cdef object _unknown
 _unknown = 'unknown'
-_missing = 'missing'
 
 # add a typedef struct dirent dirent to workaround pyrex
 cdef extern from 'readdir.h':
@@ -161,25 +168,12 @@
 
 from bzrlib import osutils
 
+cdef object _safe_utf8
+_safe_utf8 = osutils.safe_utf8
 
 cdef class UTF8DirReader:
     """A dir reader for utf8 file systems."""
 
-    cdef readonly object _safe_utf8
-    cdef _directory, _chardev, _block, _file, _fifo, _symlink
-    cdef _socket, _unknown
-
-    def __init__(self):
-        self._safe_utf8 = osutils.safe_utf8
-        self._directory = _directory
-        self._chardev = _chardev
-        self._block = _block
-        self._file = _file
-        self._fifo = _fifo
-        self._symlink = _symlink
-        self._socket = _socket
-        self._unknown = _unknown
-
     def kind_from_mode(self, int mode):
         """Get the kind of a path from a mode status."""
         return self._kind_from_mode(mode)
@@ -187,25 +181,24 @@
     cdef _kind_from_mode(self, int mode):
         # Files and directories are the most common - check them first.
         if S_ISREG(mode):
-            return self._file
+            return _file
         if S_ISDIR(mode):
-            return self._directory
+            return _directory
         if S_ISCHR(mode):
-            return self._chardev
+            return _chardev
         if S_ISBLK(mode):
-            return self._block
+            return _block
         if S_ISLNK(mode):
-            return self._symlink
+            return _symlink
         if S_ISFIFO(mode):
-            return self._fifo
+            return _fifo
         if S_ISSOCK(mode):
-            return self._socket
-        return self._unknown
+            return _socket
+        return _unknown
 
     def top_prefix_to_starting_dir(self, top, prefix=""):
         """See DirReader.top_prefix_to_starting_dir."""
-        return (self._safe_utf8(prefix), None, None, None,
-            self._safe_utf8(top))
+        return (_safe_utf8(prefix), None, None, None, _safe_utf8(top))
 
     def read_dir(self, prefix, top):
         """Read a single directory from a utf8 file system.
@@ -307,6 +300,9 @@
         if orig_dir_fd == -1:
             raise_os_error(errno, "open: ", ".")
         if -1 == chdir(path):
+            # Ignore the return value, because we are already raising an
+            # exception
+            close(orig_dir_fd)
             raise_os_error(errno, "chdir: ", path)
     else:
         orig_dir_fd = -1

=== modified file 'bzrlib/_simple_set_pyx.pyx'
--- bzrlib/_simple_set_pyx.pyx	2010-02-17 17:11:16 +0000
+++ bzrlib/_simple_set_pyx.pyx	2010-08-02 17:16:12 +0000
@@ -115,6 +115,20 @@
             raise MemoryError()
         memset(self._table, 0, n_bytes)
 
+    def __sizeof__(self):
+        # Note: Pyrex doesn't allow sizeof(class) so we re-implement it here.
+        # Bits are:
+        #   1: PyObject
+        #   2: vtable *
+        #   3: 3 Py_ssize_t
+        #   4: PyObject**
+        # Note that we might get alignment, etc, wrong, but at least this is
+        # better than no estimate at all
+        # return sizeof(SimpleSet) + (self._mask + 1) * (sizeof(PyObject*))
+        return (sizeof(PyObject) + sizeof(void*)
+                + 3*sizeof(Py_ssize_t) + sizeof(PyObject**)
+                + (self._mask + 1) * sizeof(PyObject*))
+
     def __dealloc__(self):
         if self._table != NULL:
             PyMem_Free(self._table)

=== modified file 'bzrlib/_static_tuple_c.c'
--- bzrlib/_static_tuple_c.c	2010-02-23 07:43:11 +0000
+++ bzrlib/_static_tuple_c.c	2010-06-27 05:25:03 +0000
@@ -703,6 +703,18 @@
     return 0;
 }
 
+
+static PyObject *
+StaticTuple_sizeof(StaticTuple *self)
+{
+	Py_ssize_t res;
+
+	res = _PyObject_SIZE(&StaticTuple_Type) + (int)self->size * sizeof(void*);
+	return PyInt_FromSsize_t(res);
+}
+
+
+
 static char StaticTuple_doc[] =
     "C implementation of a StaticTuple structure."
     "\n This is used as StaticTuple(item1, item2, item3)"
@@ -722,6 +734,7 @@
      "Create a StaticTuple from a given sequence. This functions"
      " the same as the tuple() constructor."},
     {"__reduce__", (PyCFunction)StaticTuple_reduce, METH_NOARGS, StaticTuple_reduce_doc},
+    {"__sizeof__",  (PyCFunction)StaticTuple_sizeof, METH_NOARGS}, 
     {NULL, NULL} /* sentinel */
 };
 

=== modified file 'bzrlib/_static_tuple_c.h'
--- bzrlib/_static_tuple_c.h	2009-10-13 18:00:16 +0000
+++ bzrlib/_static_tuple_c.h	2010-05-11 14:13:31 +0000
@@ -1,4 +1,4 @@
-/* Copyright (C) 2009 Canonical Ltd
+/* Copyright (C) 2009, 2010 Canonical Ltd
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -67,6 +67,7 @@
 #define StaticTuple_SET_ITEM(key, offset, val) \
     ((((StaticTuple*)(key))->items[(offset)]) = ((PyObject *)(val)))
 #define StaticTuple_GET_ITEM(key, offset) (((StaticTuple*)key)->items[offset])
+#define StaticTuple_GET_SIZE(key) (((StaticTuple*)key)->size)
 
 
 #ifdef STATIC_TUPLE_MODULE

=== modified file 'bzrlib/_static_tuple_c.pxd'
--- bzrlib/_static_tuple_c.pxd	2010-02-17 17:11:16 +0000
+++ bzrlib/_static_tuple_c.pxd	2010-08-04 14:28:10 +0000
@@ -33,6 +33,7 @@
     int import_static_tuple_c() except -1
     StaticTuple StaticTuple_New(Py_ssize_t)
     StaticTuple StaticTuple_Intern(StaticTuple)
+    StaticTuple StaticTuple_FromSequence(object)
 
     # Steals a reference and val must be a valid type, no checking is done
     void StaticTuple_SET_ITEM(StaticTuple key, Py_ssize_t offset, object val)
@@ -42,3 +43,4 @@
     # not an 'object' return value.
     void *StaticTuple_GET_ITEM(StaticTuple key, Py_ssize_t offset)
     int StaticTuple_CheckExact(object)
+    Py_ssize_t StaticTuple_GET_SIZE(StaticTuple key)

=== modified file 'bzrlib/add.py'
--- bzrlib/add.py	2010-04-30 11:03:59 +0000
+++ bzrlib/add.py	2010-06-23 08:19:28 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,9 +18,7 @@
 
 import sys
 
-import bzrlib.bzrdir
 import bzrlib.osutils
-from bzrlib.symbol_versioning import *
 
 
 class AddAction(object):
@@ -95,14 +93,3 @@
         full_base_path = bzrlib.osutils.pathjoin(self.base_path, path.raw_path)
         # This may return None, but it is our last attempt
         return self.base_tree.path2id(full_base_path), full_base_path
-
-
-# TODO: jam 20050105 These could be used for compatibility
-#       however, they bind against the current stdout, not the
-#       one which exists at the time they are called, so they
-#       don't work for the test suite.
-# deprecated
-add_action_add = AddAction()
-add_action_null = add_action_add
-add_action_add_and_print = AddAction(should_print=True)
-add_action_print = add_action_add_and_print

=== modified file 'bzrlib/annotate.py'
--- bzrlib/annotate.py	2009-09-14 01:48:28 +0000
+++ bzrlib/annotate.py	2010-06-17 09:23:19 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,11 +28,16 @@
 import sys
 import time
 
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+from bzrlib import (
+    patiencediff,
+    tsort,
+    )
+""")
 from bzrlib import (
     errors,
     osutils,
-    patiencediff,
-    tsort,
     )
 from bzrlib.config import extract_email_address
 from bzrlib.repository import _strip_NULL_ghosts
@@ -312,8 +317,7 @@
 
 
 def _get_matching_blocks(old, new):
-    matcher = patiencediff.PatienceSequenceMatcher(None,
-        old, new)
+    matcher = patiencediff.PatienceSequenceMatcher(None, old, new)
     return matcher.get_matching_blocks()
 
 

=== removed directory 'bzrlib/benchmarks'
=== removed file 'bzrlib/benchmarks/__init__.py'
--- bzrlib/benchmarks/__init__.py	2009-06-10 03:56:49 +0000
+++ bzrlib/benchmarks/__init__.py	1970-01-01 00:00:00 +0000
@@ -1,206 +0,0 @@
-# Copyright (C) 2006 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Benchmark test suite for bzr."""
-
-from bzrlib import (
-    bzrdir,
-    )
-from bzrlib import plugin as _mod_plugin
-import bzrlib.branch
-from bzrlib.tests.TestUtil import TestLoader
-from bzrlib.tests.blackbox import ExternalBase
-
-
-class Benchmark(ExternalBase):
-    """A Test class which provides helpers for writing benchmark tests."""
-
-    def make_kernel_like_tree(self, url=None, root='.',
-                              link_working=False):
-        """Setup a temporary tree roughly like a kernel tree.
-
-        :param url: Creat the kernel like tree as a lightweight checkout
-            of a new branch created at url.
-        :param root: Path where the tree will be created.
-        :param link_working: instead of creating a new copy of all files
-            just hardlink the working tree. Tests must request this, because
-            they must break links if they want to change the files
-        :return: A newly created tree.
-        """
-        from bzrlib.benchmarks.tree_creator.kernel_like import (
-            KernelLikeTreeCreator,
-            )
-        creator = KernelLikeTreeCreator(self, link_working=link_working,
-                                        url=url)
-        return creator.create(root=root)
-
-    def make_kernel_like_added_tree(self, root='.',
-                                    link_working=True,
-                                    hot_cache=True):
-        """Make a kernel like tree, with all files added
-
-        :param root: Where to create the files
-        :param link_working: Instead of copying all of the working tree
-            files, just hardlink them to the cached files. Tests can unlink
-            files that they will change.
-        :param hot_cache: Run through the newly created tree and make sure
-            the stat-cache is correct. The old way of creating a freshly
-            added tree always had a hot cache.
-        """
-        from bzrlib.benchmarks.tree_creator.kernel_like import (
-            KernelLikeAddedTreeCreator,
-            )
-        creator = KernelLikeAddedTreeCreator(self, link_working=link_working,
-                                             hot_cache=hot_cache)
-        return creator.create(root=root)
-
-    def make_kernel_like_committed_tree(self, root='.',
-                                    link_working=True,
-                                    link_bzr=False,
-                                    hot_cache=True):
-        """Make a kernel like tree, with all files added and committed
-
-        :param root: Where to create the files
-        :param link_working: Instead of copying all of the working tree
-            files, just hardlink them to the cached files. Tests can unlink
-            files that they will change.
-        :param link_bzr: Hardlink the .bzr directory. For readonly
-            operations this is safe, and shaves off a lot of setup time
-        """
-        from bzrlib.benchmarks.tree_creator.kernel_like import (
-            KernelLikeCommittedTreeCreator,
-            )
-        creator = KernelLikeCommittedTreeCreator(self,
-                                                 link_working=link_working,
-                                                 link_bzr=link_bzr,
-                                                 hot_cache=hot_cache)
-        return creator.create(root=root)
-
-    def make_kernel_like_inventory(self):
-        """Create an inventory with the properties of a kernel-like tree
-
-        This should be equivalent to a committed kernel like tree, not
-        just a working tree.
-        """
-        from bzrlib.benchmarks.tree_creator.kernel_like import (
-            KernelLikeInventoryCreator,
-            )
-        creator = KernelLikeInventoryCreator(self)
-        return creator.create()
-
-    def make_many_commit_tree(self, directory_name='.',
-                              hardlink=False):
-        """Create a tree with many commits.
-
-        No file changes are included. Not hardlinking the working tree,
-        because there are no working tree files.
-        """
-        from bzrlib.benchmarks.tree_creator.simple_many_commit import (
-            SimpleManyCommitTreeCreator,
-            )
-        creator = SimpleManyCommitTreeCreator(self, link_bzr=hardlink)
-        return creator.create(root=directory_name)
-
-    def make_heavily_merged_tree(self, directory_name='.',
-                                 hardlink=False):
-        """Create a tree in which almost every commit is a merge.
-
-        No file changes are included.  This produces two trees,
-        one of which is returned.  Except for the first commit, every
-        commit in its revision-history is a merge another commit in the other
-        tree.  Not hardlinking the working tree, because there are no working
-        tree files.
-        """
-        from bzrlib.benchmarks.tree_creator.heavily_merged import (
-            HeavilyMergedTreeCreator,
-            )
-        creator = HeavilyMergedTreeCreator(self, link_bzr=hardlink)
-        return creator.create(root=directory_name)
-
-    def create_with_commits(self, num_files, num_commits, directory_name='.',
-                            hardlink=False):
-        """Create a tree with many files and many commits. Every commit changes
-        exactly one file.
-
-        :param num_files: number of files to be created
-        :param num_commits: number of commits in the newly created tree
-        """
-        from bzrlib.benchmarks.tree_creator.many_commit import (
-            ManyCommitTreeCreator,
-            )
-        creator = ManyCommitTreeCreator(self, link_bzr=hardlink,
-                                        num_files=num_files,
-                                        num_commits=num_commits)
-        tree = creator.create(root=directory_name)
-        files = ["%s/%s" % (directory_name, fn) for fn in creator.files]
-        return tree, files
-
-    def commit_some_revisions(self, tree, files, num_commits,
-                              changes_per_commit):
-        """Commit a specified number of revisions to some files in a tree,
-        makeing a specified number of changes per commit.
-
-        :param tree: The tree in which the changes happen.
-        :param files: The list of files where changes should occur.
-        :param num_commits: The number of commits
-        :param changes_per_commit: The number of files that are touched in
-            each commit.
-        """
-        for j in range(num_commits):
-            for i in range(changes_per_commit):
-                fn = files[(i + j) % changes_per_commit]
-                content = range(i) + [i, i, i, '']
-                f = open(fn, "w")
-                try:
-                    f.write("\n".join([str(k) for k in content]))
-                finally:
-                    f.close()
-            tree.commit("new revision")
-
-
-def test_suite():
-    """Build and return a TestSuite which contains benchmark tests only."""
-    testmod_names = [ \
-                   'bzrlib.benchmarks.bench_add',
-                   'bzrlib.benchmarks.bench_bench',
-                   'bzrlib.benchmarks.bench_bundle',
-                   'bzrlib.benchmarks.bench_cache_utf8',
-                   'bzrlib.benchmarks.bench_checkout',
-                   'bzrlib.benchmarks.bench_commit',
-                   'bzrlib.benchmarks.bench_dirstate',
-                   'bzrlib.benchmarks.bench_info',
-                   'bzrlib.benchmarks.bench_inventory',
-                   'bzrlib.benchmarks.bench_knit',
-                   'bzrlib.benchmarks.bench_log',
-                   'bzrlib.benchmarks.bench_pack',
-                   'bzrlib.benchmarks.bench_osutils',
-                   'bzrlib.benchmarks.bench_rocks',
-                   'bzrlib.benchmarks.bench_startup',
-                   'bzrlib.benchmarks.bench_status',
-                   'bzrlib.benchmarks.bench_tags',
-                   'bzrlib.benchmarks.bench_transform',
-                   'bzrlib.benchmarks.bench_workingtree',
-                   'bzrlib.benchmarks.bench_sftp',
-                   'bzrlib.benchmarks.bench_xml',
-                   ]
-    suite = TestLoader().loadTestsFromModuleNames(testmod_names)
-
-    # Load any benchmarks from plugins
-    for name, plugin in _mod_plugin.plugins().items():
-        if getattr(plugin.module, 'bench_suite', None) is not None:
-            suite.addTest(plugin.module.bench_suite())
-
-    return suite

=== removed file 'bzrlib/benchmarks/bench_add.py'
--- bzrlib/benchmarks/bench_add.py	2009-03-23 14:59:43 +0000
+++ bzrlib/benchmarks/bench_add.py	1970-01-01 00:00:00 +0000
@@ -1,32 +0,0 @@
-# Copyright (C) 2006 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Tests for bzr add performance."""
-
-
-from bzrlib.benchmarks import Benchmark
-
-
-class AddBenchmark(Benchmark):
-    """Benchmarks for 'bzr add'"""
-
-    def test_one_add_kernel_like_tree(self):
-        """Adding a kernel sized tree should be bearable (<5secs) fast."""
-        self.make_kernel_like_tree(link_working=True)
-        # on roberts machine: this originally took:  25936ms/32244ms
-        # after making smart_add use the parent_ie:   5033ms/ 9368ms
-        # plain os.walk takes 213ms on this tree
-        self.time(self.run_bzr, 'add')

=== removed file 'bzrlib/benchmarks/bench_bench.py'
--- bzrlib/benchmarks/bench_bench.py	2009-03-23 14:59:43 +0000
+++ bzrlib/benchmarks/bench_bench.py	1970-01-01 00:00:00 +0000
@@ -1,101 +0,0 @@
-# Copyright (C) 2006 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Tests for bzr benchmark utilities performance."""
-
-from bzrlib import (
-    osutils,
-    )
-from bzrlib.benchmarks import Benchmark
-from bzrlib.benchmarks.tree_creator.kernel_like import (
-    KernelLikeTreeCreator,
-    KernelLikeAddedTreeCreator,
-    KernelLikeCommittedTreeCreator,
-    )
-from bzrlib.tests import TestSkipped
-
-
-class MakeKernelLikeTreeBenchmark(Benchmark):
-    """Benchmark creating benchmark trees."""
-
-    def test_make_kernel_like_tree(self):
-        """Making a kernel sized tree should be ~ 5seconds on modern disk."""
-        # on roberts machine: this originally took:  7372ms/ 7479ms
-        # with the LocalTransport._abspath call:     3730ms/ 3778ms
-        # with AtomicFile tuning:                    2888ms/ 2926ms
-        # switching to transport.append_bytes:       1468ms/ 2849ms
-        self.time(self.make_kernel_like_tree)
-
-    def test_02_make_kernel_like_tree(self):
-        """Hardlinking a kernel-like working tree should be ~1s"""
-        # make sure kernel_like_tree is cached
-        creator = KernelLikeTreeCreator(self, link_working=True)
-        if not creator.is_caching_enabled():
-            raise TestSkipped('caching is disabled')
-        creator.ensure_cached()
-        self.time(creator.create, root='bar')
-
-    def test_03_make_kernel_like_added_tree(self):
-        """Time the first creation of a kernel like added tree"""
-        creator = KernelLikeAddedTreeCreator(self)
-        creator.disable_cache()
-        self.time(creator.create, root='foo')
-
-    def test_04_make_kernel_like_added_tree(self):
-        """Time the second creation of a kernel like added tree
-        (this should be a clone)
-        """
-        # make sure kernel_like_added_tree is cached
-        creator = KernelLikeAddedTreeCreator(self, link_working=True)
-        if not creator.is_caching_enabled():
-            # Caching is disabled, this test is meaningless
-            raise TestSkipped('caching is disabled')
-        creator.ensure_cached()
-        self.time(creator.create, root='bar')
-
-    def test_05_make_kernel_like_committed_tree(self):
-        """Time the first creation of a committed kernel like tree"""
-        creator = KernelLikeCommittedTreeCreator(self)
-        creator.disable_cache()
-        self.time(creator.create, root='foo')
-
-    def test_06_make_kernel_like_committed_tree(self):
-        """Time the second creation of a committed kernel like tree
-        (this should be a clone)
-        """
-        creator = KernelLikeCommittedTreeCreator(self,
-                                                 link_working=True,
-                                                 link_bzr=False)
-        if not creator.is_caching_enabled():
-            # Caching is disabled, this test is meaningless
-            raise TestSkipped('caching is disabled')
-        creator.ensure_cached()
-        self.time(creator.create, root='bar')
-
-    def test_07_make_kernel_like_committed_tree_hardlink(self):
-        """Time the creation of a committed kernel like tree
-        (this should also hardlink the .bzr/ directory)
-        """
-        creator = KernelLikeCommittedTreeCreator(self,
-                                                 link_working=True,
-                                                 link_bzr=True)
-        if not creator.is_caching_enabled():
-            # Caching is disabled, this test is meaningless
-            raise TestSkipped('caching is disabled')
-        creator.ensure_cached()
-        self.time(creator.create, root='bar')
-
-

=== removed file 'bzrlib/benchmarks/bench_bundle.py'
--- bzrlib/benchmarks/bench_bundle.py	2009-03-23 14:59:43 +0000
+++ bzrlib/benchmarks/bench_bundle.py	1970-01-01 00:00:00 +0000
@@ -1,207 +0,0 @@
-# Copyright (C) 2006 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Tests for bzr bundle performance."""
-
-from cStringIO import StringIO
-import os
-import shutil
-
-from bzrlib import bzrdir
-from bzrlib.benchmarks import Benchmark
-from bzrlib.branch import Branch
-from bzrlib.bundle.apply_bundle import install_bundle
-from bzrlib.bundle.serializer import read_bundle, write_bundle
-from bzrlib.revision import NULL_REVISION
-from bzrlib.revisionspec import RevisionSpec
-from bzrlib.workingtree import WorkingTree
-
-
-class BundleBenchmark(Benchmark):
-    """Benchmarks for bzr bundle performance and bzr merge with a bundle."""
-
-    def test_create_bundle_known_kernel_like_tree(self):
-        """Create a bundle for a kernel sized tree with no ignored, unknowns,
-        or added and one commit.
-        """
-        self.make_kernel_like_committed_tree()
-        self.time(self.run_bzr, ['bundle', '--revision', '..-1'])
-
-    def test_create_bundle_many_commit_tree (self):
-        """Create a bundle for a tree with many commits but no changes."""
-        self.make_many_commit_tree()
-        self.time(self.run_bzr, ['bundle', '--revision', '..-1'])
-
-    def test_create_bundle_heavily_merged_tree(self):
-        """Create a bundle for a heavily merged tree."""
-        self.make_heavily_merged_tree()
-        self.time(self.run_bzr, ['bundle', '--revision', '..-1'])
-
-    def test_apply_bundle_known_kernel_like_tree(self):
-        """Create a bundle for a kernel sized tree with no ignored, unknowns,
-        or added and one commit.
-        """
-        tree = self.make_kernel_like_committed_tree('tree')
-
-        f = open('bundle', 'wb')
-        try:
-            write_bundle(tree.branch.repository, tree.last_revision(),
-                         NULL_REVISION, f)
-        finally:
-            f.close()
-
-        tree2 = self.make_branch_and_tree('branch_a')
-        os.chdir('branch_a')
-        self.time(self.run_bzr, ['merge', '../bundle'])
-
-
-class BundleLibraryLevelWriteBenchmark(Benchmark):
-    """ Benchmarks for the write_bundle library function. """
-
-    def _time_read_write(self):
-        branch, relpath = Branch.open_containing("a")
-        revision_history = branch.revision_history()
-        bundle_text = StringIO()
-        self.time(write_bundle, branch.repository, revision_history[-1],
-                  NULL_REVISION, bundle_text)
-        bundle_text.seek(0)
-        target_tree = self.make_branch_and_tree('b')
-        bundle = self.time(read_bundle, bundle_text)
-        self.time(install_bundle, target_tree.branch.repository, bundle)
-
-    def test_few_files_small_tree_1_revision(self):
-        os.mkdir("a")
-        tree, files = self.create_with_commits(5, 1, directory_name="a")
-        self.commit_some_revisions(tree, files[:5], 1, 1)
-        self._time_read_write()
-
-    def test_few_files_small_tree_100_revision(self):
-        os.mkdir("a")
-        tree, files = self.create_with_commits(5, 1, directory_name="a")
-        self.commit_some_revisions(tree, files[:5], 100, 1)
-        self._time_read_write()
-
-    def test_few_files_moderate_tree_1_revision(self):
-        os.mkdir("a")
-        tree, files = self.create_with_commits(100, 1, directory_name="a")
-        self.commit_some_revisions(tree, files[:5], 1, 1)
-        self._time_read_write()
-
-    def test_few_files_moderate_tree_100_revision(self):
-        os.mkdir("a")
-        tree, files = self.create_with_commits(100, 1, directory_name="a")
-        self.commit_some_revisions(tree, files[:5], 100, 1)
-        self._time_read_write()
-
-    def test_some_files_moderate_tree_1_revision(self):
-        os.mkdir("a")
-        tree, files = self.create_with_commits(100, 1, directory_name="a")
-        self.commit_some_revisions(tree, files[:100], 1, 1)
-        self._time_read_write()
-
-    def test_few_files_big_tree_1_revision(self):
-        os.mkdir("a")
-        tree, files = self.create_with_commits(1000, 1, directory_name="a")
-        self.commit_some_revisions(tree, files[:5], 1, 1)
-        self._time_read_write()
-
-    def test_some_files_big_tree_1_revision(self):
-        os.mkdir("a")
-        tree, files = self.create_with_commits(1000, 1, directory_name="a")
-        self.commit_some_revisions(tree, files[:100], 1, 1)
-        self._time_read_write()
-
-
-class BundleLibraryLevelInstallBenchmark(Benchmark):
-    """ Benchmarks for the install_bundle library function. """
-
-    def _time_read_write(self):
-        branch, relpath = Branch.open_containing("a")
-        revision_history = branch.revision_history()
-        bundle_text = StringIO()
-        write_bundle(branch.repository, revision_history[-1],
-                     NULL_REVISION, bundle_text)
-        bundle_text.seek(0)
-        target_tree = self.make_branch_and_tree('b')
-        bundle = self.time(read_bundle, bundle_text)
-        self.time(install_bundle, target_tree.branch.repository, bundle)
-
-    def test_few_files_small_tree_1_revision(self):
-        os.mkdir("a")
-        tree, files = self.create_with_commits(5, 1, directory_name="a")
-        self.commit_some_revisions(tree, files[:5], 1, 1)
-        self._time_read_write()
-
-    def test_few_files_small_tree_100_revision(self):
-        os.mkdir("a")
-        tree, files = self.create_with_commits(5, 1, directory_name="a")
-        self.commit_some_revisions(tree, files[:5], 100, 1)
-        self._time_read_write()
-
-    def test_few_files_moderate_tree_1_revision(self):
-        os.mkdir("a")
-        tree, files = self.create_with_commits(100, 1, directory_name="a")
-        self.commit_some_revisions(tree, files[:5], 1, 1)
-        self._time_read_write()
-
-    def test_few_files_moderate_tree_100_revision(self):
-        os.mkdir("a")
-        tree, files = self.create_with_commits(100, 1, directory_name="a")
-        self.commit_some_revisions(tree, files[:5], 100, 1)
-        self._time_read_write()
-
-    def test_some_files_moderate_tree_1_revision(self):
-        os.mkdir("a")
-        tree, files = self.create_with_commits(100, 1, directory_name="a")
-        self.commit_some_revisions(tree, files[:100], 1, 1)
-        self._time_read_write()
-
-    def test_few_files_big_tree_1_revision(self):
-        os.mkdir("a")
-        tree, files = self.create_with_commits(1000, 1, directory_name="a")
-        self.commit_some_revisions(tree, files[:5], 1, 1)
-        self._time_read_write()
-
-    def test_some_files_big_tree_1_revision(self):
-        os.mkdir("a")
-        tree, files = self.create_with_commits(1000, 1, directory_name="a")
-        self.commit_some_revisions(tree, files[:100], 1, 1)
-        self._time_read_write()
-
-
-if __name__ == '__main__':
-    # USE the following if you want to regenerate the above test functions
-    for treesize, treesize_h in [(5, "small"), (100, "moderate"),
-                                 (1000, "big")]:
-        for bundlefiles, bundlefiles_h in [(5, "few"), (100, "some")]:
-            if bundlefiles > treesize:
-                continue
-            for num_revisions in [1, 100]:
-                if (num_revisions >= 100 and
-                        (bundlefiles >= 100 or treesize >= 1000)):
-                    # Skip the 100x100x? tests.
-                    # And the 100x?x1000
-                    continue
-                code = """\
-    def test_%s_files_%s_tree_%s_revision(self):
-        os.mkdir("a")
-        tree, files = self.create_with_commits(%s, 1, directory_name="a")
-        self.commit_some_revisions(tree, files[:%s], %s, 1)
-        self._time_read_write()
-""" % (bundlefiles_h, treesize_h, num_revisions,
-       treesize, bundlefiles, num_revisions)
-                print code
-

=== removed file 'bzrlib/benchmarks/bench_cache_utf8.py'
--- bzrlib/benchmarks/bench_cache_utf8.py	2009-03-23 14:59:43 +0000
+++ bzrlib/benchmarks/bench_cache_utf8.py	1970-01-01 00:00:00 +0000
@@ -1,226 +0,0 @@
-# Copyright (C) 2006 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-
-"""Tests for encoding performance."""
-
-from bzrlib import (
-    cache_utf8,
-    osutils,
-    )
-
-from bzrlib.benchmarks import Benchmark
-
-
-_normal_revision_id = (u'john@arbash-meinel.com-20060801200018'
-                       u'-cafa6272d9b8cac4')
-_unicode_revision_id = (u'\u062c\u0648\u062c\u0648@\xe5rbash-meinel.com-'
-                        u'\xb5\xb5\xb5-20060801200018-cafa6272d9b8cac4')
-
-_normal_revision_id_utf8 = _normal_revision_id.encode('utf-8')
-_unicode_revision_id_utf8 = _unicode_revision_id.encode('utf-8')
-
-
-class EncodingBenchmark(Benchmark):
-    """Benchmark the time to encode strings."""
-
-    def setUp(self):
-        super(EncodingBenchmark, self).setUp()
-        # Make sure we start and end with a clean cache
-        cache_utf8.clear_encoding_cache()
-        self.addCleanup(cache_utf8.clear_encoding_cache)
-
-    def encode_1M(self, revision_id):
-        """Encode the given revision id 1 million times"""
-        # In a real kernel tree there are 7.7M lines of code
-        # so the initial import actually has to encode a revision
-        # id to store annotated lines one time for every line.
-        for i in xrange(1000000):
-            revision_id.encode('utf8')
-
-    def encode_cached_1M(self, revision_id):
-        """Encode the given revision id 1 million times using the cache"""
-        encode = cache_utf8.encode
-        for i in xrange(1000000):
-            encode(revision_id)
-
-    def encode_multi(self, revision_list, count):
-        """Encode each entry in the list count times"""
-        for i in xrange(count):
-            for revision_id in revision_list:
-                revision_id.encode('utf-8')
-
-    def encode_cached_multi(self, revision_list, count):
-        """Encode each entry in the list count times"""
-        encode = cache_utf8.encode
-        for i in xrange(count):
-            for revision_id in revision_list:
-                encode(revision_id)
-
-    def test_encode_1_by_1M_ascii(self):
-        """Test encoding a single revision id 1 million times."""
-        self.time(self.encode_1M, _normal_revision_id)
-
-    def test_encode_1_by_1M_ascii_cached(self):
-        """Test encoding a single revision id 1 million times."""
-        self.time(self.encode_cached_1M, _normal_revision_id)
-
-    def test_encode_1_by_1M_ascii_str(self):
-        # We have places that think they have a unicode revision id
-        # but actually, they have a plain string. So .encode(utf8)
-        # actually has to decode from ascii, and then encode into utf8
-        self.time(self.encode_1M, str(_normal_revision_id))
-
-    def test_encode_1_by_1M_ascii_str_cached(self):
-        self.time(self.encode_cached_1M, str(_normal_revision_id))
-
-    def test_encode_1_by_1M_unicode(self):
-        """Test encoding a single revision id 1 million times."""
-        self.time(self.encode_1M, _unicode_revision_id)
-
-    def test_encode_1_by_1M_unicode_cached(self):
-        """Test encoding a single revision id 1 million times."""
-        self.time(self.encode_cached_1M, _unicode_revision_id)
-
-    def test_encode_1k_by_1k_ascii(self):
-        """Test encoding 5 revisions 100k times"""
-        revisions = [unicode(osutils.rand_chars(60)) for x in xrange(1000)]
-        self.time(self.encode_multi, revisions, 1000)
-
-    def test_encode_1k_by_1k_ascii_cached(self):
-        """Test encoding 5 revisions 100k times"""
-        revisions = [unicode(osutils.rand_chars(60)) for x in xrange(1000)]
-        self.time(self.encode_cached_multi, revisions, 1000)
-
-    def test_encode_1k_by_1k_unicode(self):
-        """Test encoding 5 revisions 100k times"""
-        revisions = [u'\u062c\u0648\u062c\u0648' +
-                     unicode(osutils.rand_chars(60)) for x in xrange(1000)]
-        self.time(self.encode_multi, revisions, 1000)
-
-    def test_encode_1k_by_1k_unicode_cached(self):
-        """Test encoding 5 revisions 100k times"""
-        revisions = [u'\u062c\u0648\u062c\u0648' +
-                     unicode(osutils.rand_chars(60)) for x in xrange(1000)]
-        self.time(self.encode_cached_multi, revisions, 1000)
-
-    def test_encode_500K_by_1_ascii(self):
-        revisions = [unicode("test%07d" % x) for x in xrange(500000)]
-        self.time(self.encode_multi, revisions, 1)
-
-    def test_encode_500K_by_1_ascii_cached(self):
-        revisions = [unicode("test%07d" % x) for x in xrange(500000)]
-        self.time(self.encode_cached_multi, revisions, 1)
-
-    def test_encode_500K_by_1_unicode(self):
-        revisions = [u'\u062c\u0648\u062c\u0648' +
-                     unicode("%07d" % x) for x in xrange(500000)]
-        self.time(self.encode_multi, revisions, 1)
-
-    def test_encode_500K_by_1_unicode_cached(self):
-        revisions = [u'\u062c\u0648\u062c\u0648' +
-                     unicode("%07d" % x) for x in xrange(500000)]
-        self.time(self.encode_cached_multi, revisions, 1)
-
-
-class DecodingBenchmarks(Benchmark):
-    """Benchmark the time to decode strings."""
-
-    def setUp(self):
-        super(DecodingBenchmarks, self).setUp()
-        # Make sure we start and end with a clean cache
-        cache_utf8.clear_encoding_cache()
-        self.addCleanup(cache_utf8.clear_encoding_cache)
-
-    def decode_1M(self, revision_id):
-        for i in xrange(1000000):
-            revision_id.decode('utf8')
-
-    def decode_cached_1M(self, revision_id):
-        decode = cache_utf8.decode
-        for i in xrange(1000000):
-            decode(revision_id)
-
-    def decode_multi(self, revision_list, count):
-        for i in xrange(count):
-            for revision_id in revision_list:
-                revision_id.decode('utf-8')
-
-    def decode_cached_multi(self, revision_list, count):
-        decode = cache_utf8.decode
-        for i in xrange(count):
-            for revision_id in revision_list:
-                decode(revision_id)
-
-    def test_decode_1_by_1M_ascii(self):
-        """Test decoding a single revision id 1 million times."""
-        self.time(self.decode_1M, _normal_revision_id_utf8)
-
-    def test_decode_1_by_1M_ascii_cached(self):
-        """Test decoding a single revision id 1 million times."""
-        self.time(self.decode_cached_1M, _normal_revision_id_utf8)
-
-    def test_decode_1_by_1M_unicode(self):
-        """Test decoding a single revision id 1 million times."""
-        self.time(self.decode_1M, _unicode_revision_id_utf8)
-
-    def test_decode_1_by_1M_unicode_cached(self):
-        """Test decoding a single revision id 1 million times."""
-        self.time(self.decode_cached_1M, _unicode_revision_id_utf8)
-
-    def test_decode_1k_by_1k_ascii(self):
-        """Test decoding 5 revisions 100k times"""
-        revisions = [osutils.rand_chars(60) for x in xrange(1000)]
-        self.time(self.decode_multi, revisions, 1000)
-
-    def test_decode_1k_by_1k_ascii_cached(self):
-        """Test decoding 5 revisions 100k times"""
-        revisions = [osutils.rand_chars(60) for x in xrange(1000)]
-        self.time(self.decode_cached_multi, revisions, 1000)
-
-    def test_decode_1k_by_1k_unicode(self):
-        """Test decoding 5 revisions 100k times"""
-        revisions = [(u'\u062c\u0648\u062c\u0648' +
-                      unicode(osutils.rand_chars(60))).encode('utf8')
-                     for x in xrange(1000)]
-        self.time(self.decode_multi, revisions, 1000)
-
-    def test_decode_1k_by_1k_unicode_cached(self):
-        """Test decoding 5 revisions 100k times"""
-        revisions = [(u'\u062c\u0648\u062c\u0648' +
-                      unicode(osutils.rand_chars(60))).encode('utf8')
-                     for x in xrange(1000)]
-        self.time(self.decode_cached_multi, revisions, 1000)
-
-    def test_decode_500K_by_1_ascii(self):
-        revisions = [("test%07d" % x) for x in xrange(500000)]
-        self.time(self.decode_multi, revisions, 1)
-
-    def test_decode_500K_by_1_ascii_cached(self):
-        revisions = [("test%07d" % x) for x in xrange(500000)]
-        self.time(self.decode_cached_multi, revisions, 1)
-
-    def test_decode_500K_by_1_unicode(self):
-        revisions = [(u'\u062c\u0648\u062c\u0648' +
-                      unicode("%07d" % x)).encode('utf-8')
-                     for x in xrange(500000)]
-        self.time(self.decode_multi, revisions, 1)
-
-    def test_decode_500K_by_1_unicode_cached(self):
-        revisions = [(u'\u062c\u0648\u062c\u0648' +
-                      unicode("%07d" % x)).encode('utf-8')
-                     for x in xrange(500000)]
-        self.time(self.decode_cached_multi, revisions, 1)

=== removed file 'bzrlib/benchmarks/bench_checkout.py'
--- bzrlib/benchmarks/bench_checkout.py	2009-03-23 14:59:43 +0000
+++ bzrlib/benchmarks/bench_checkout.py	1970-01-01 00:00:00 +0000
@@ -1,29 +0,0 @@
-# Copyright (C) 2006 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Tests for bzr tree building (checkout) performance."""
-
-
-from bzrlib.benchmarks import Benchmark
-
-
-class CheckoutBenchmark(Benchmark):
-    """Benchmarks for ``'bzr checkout'`` performance."""
-
-    def test_build_kernel_like_tree(self):
-        """Checkout of a clean kernel sized tree should be (<10secs)."""
-        self.make_kernel_like_committed_tree(link_bzr=True)
-        self.time(self.run_bzr, ['checkout', '--lightweight', '.', 'acheckout'])

=== removed file 'bzrlib/benchmarks/bench_commit.py'
--- bzrlib/benchmarks/bench_commit.py	2009-03-23 14:59:43 +0000
+++ bzrlib/benchmarks/bench_commit.py	1970-01-01 00:00:00 +0000
@@ -1,81 +0,0 @@
-# Copyright (C) 2006 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Benchmarks of bzr commit."""
-
-import os
-
-from bzrlib.benchmarks import Benchmark
-from bzrlib.transport.memory import MemoryServer
-from bzrlib.transport import get_transport
-
-
-class CommitBenchmark(Benchmark):
-    """Benchmarks for ``bzr commit``"""
-
-    def test_commit_kernel_like_tree(self):
-        """Commit of a fresh import of a clean kernel sized tree."""
-        # uncomment this to run the benchmark with the repository in memory
-        # not disk
-        # self.transport_server = MemoryServer
-        # self.make_kernel_like_tree(self.get_url())
-        tree = self.make_kernel_like_added_tree()
-        self.time(self.run_bzr, ['commit', '-m', 'first post'])
-
-    def test_partial_commit_kernel_like_tree(self):
-        """Commit of 1/8th of a fresh import of a clean kernel sized tree."""
-        tree = self.make_kernel_like_added_tree()
-        self.time(self.run_bzr, ['commit', '-m', 'first post', '1'])
-
-    def test_no_op_commit_in_kernel_like_tree(self):
-        """Run commit --unchanged in a kernel sized tree"""
-        tree = self.make_kernel_like_committed_tree()
-        self.time(self.run_bzr, ['commit', '-m', 'no changes', '--unchanged'])
-
-    def test_commit_one_in_kernel_like_tree(self):
-        """Time committing a single change, when not directly specified"""
-        tree = self.make_kernel_like_committed_tree()
-
-        # working-tree is hardlinked, so replace a file and commit the change
-        os.remove('4/4/4/4')
-        open('4/4/4/4', 'wb').write('new contents\n')
-        self.time(self.run_bzr, ['commit', '-m', 'second'])
-
-    def test_partial_commit_one_in_kernel_like_tree(self):
-        """Time committing a single change when it is directly specified"""
-        tree = self.make_kernel_like_committed_tree()
-
-        # working-tree is hardlinked, so replace a file and commit the change
-        os.remove('4/4/4/4')
-        open('4/4/4/4', 'wb').write('new contents\n')
-        self.time(self.run_bzr, ['commit', '-m', 'second', '4/4/4/4'])
-
-    def make_simple_tree(self):
-        """A small, simple tree. No caching needed"""
-        tree = self.make_branch_and_tree('.')
-        self.build_tree(['a', 'b/', 'b/c'])
-        tree.add(['a', 'b', 'b/c'])
-        return tree
-
-    def test_cmd_commit(self):
-        """Test execution of simple commit"""
-        tree = self.make_simple_tree()
-        self.time(self.run_bzr, ['commit', '-m', 'init simple tree'])
-
-    def test_cmd_commit_subprocess(self):
-        """Text startup and execution of a simple commit."""
-        tree = self.make_simple_tree()
-        self.time(self.run_bzr_subprocess, 'commit', '-m', 'init simple tree')

=== removed file 'bzrlib/benchmarks/bench_dirstate.py'
--- bzrlib/benchmarks/bench_dirstate.py	2010-02-17 17:11:16 +0000
+++ bzrlib/benchmarks/bench_dirstate.py	1970-01-01 00:00:00 +0000
@@ -1,426 +0,0 @@
-# Copyright (C) 2007, 2009, 2010 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Benchmarks for bzr DirState performance."""
-
-import os
-
-from bzrlib import (
-    benchmarks,
-    dirstate,
-    generate_ids,
-    osutils,
-    tests,
-    )
-from bzrlib.tests.test__dirstate_helpers import (
-    compiled_dirstate_helpers_feature,
-    )
-
-
-class BenchmarkDirState(benchmarks.Benchmark):
-    """Benchmarks for DirState functions."""
-
-    def build_helper(self, layout):
-        """This is a helper with the common build_??_dirstate funcs.
-
-        :param layout: [(num_dirs, files_per_dir)]
-            The number of directories per level, and the number of files to put
-            in it.
-        :return: A DirState object with the given layout. The blocks will be
-            modified in memory, and the object will be write locked. (Callers
-            must save and unlock the object).
-        """
-        self.build_tree(['dir/'])
-        contents = 'x'*10000
-        self.build_tree_contents([('file', contents)])
-        file_stat = os.lstat('file')
-        dir_stat = os.lstat('dir')
-        file_sha1 = osutils.sha_string(contents)
-
-        state = dirstate.DirState.initialize('state')
-        def create_entries(base, layout):
-            if not layout:
-                return
-            num_dirs, num_files = layout[0]
-            for dnum in xrange(num_dirs):
-                if base:
-                    path = '%s/%02d_directory' % (base, dnum)
-                else:
-                    path = '%02d_directory' % (dnum,)
-                dir_id = generate_ids.gen_file_id(path)
-                state.add(path, dir_id, 'directory', dir_stat, '')
-                for fnum in xrange(num_files):
-                    fname = '%s/%02d_filename' % (path, fnum)
-                    file_id = generate_ids.gen_file_id(fname)
-                    state.add(fname, file_id, 'file', file_stat, file_sha1)
-                create_entries(path, layout[1:])
-        create_entries(None, layout)
-        return state
-
-    def build_10k_dirstate_dirs(self):
-        """Build a DirState file with 10k directories"""
-        state = self.build_helper([(10, 0), (10, 0), (10, 0), (10, 1)])
-        state.save()
-        state.unlock()
-        return state
-
-    def build_20k_dirstate(self):
-        """Build a DirState file with 20k records.
-
-        This approximates a kernel tree, based on the number of directories
-        (1000), and number of files per directory (20) and depth (3).
-        Because DirState doesn't have to have actual disk records, we just add
-        random files.
-        We try to have reasonable filename lengths, as well as a reasonable
-        stat value, etc.
-        """
-        state = self.build_helper([(10, 0), (10, 0), (10, 20)])
-        state.save()
-        state.unlock()
-        return state
-
-    def build_20k_dirstate_with_parents(self, num_parents):
-        """Build a DirState file with 20k records and N parents.
-
-        With 1 parent, this is equivalent to after a simple commit. With 2 it
-        is equivalent to after a merge.
-        """
-        # All files are marked as changed in the same revision, and this occurs
-        # supposedly in the history of the current trees.
-        last_changed_id = generate_ids.gen_revision_id('joe@foo.com')
-        parent_revision_ids = [generate_ids.gen_revision_id('joe@foo.com')
-                               for i in xrange(num_parents)]
-        # Start with a dirstate file with 0 parents
-        state = self.build_helper([(10, 0), (10, 0), (10, 20)])
-        try:
-            # This invasively updates the internals of DirState to be fast,
-            # since we don't have an api other than passing in Revision Tree
-            # objects, but that requires having a real inventory, etc.
-            if num_parents > 0:
-                for entry in state._iter_entries():
-                    minikind, fingerprint, size, is_exec, packed_stat = entry[1][0]
-                    for parent_id in parent_revision_ids:
-                        # Add a parent record for this record
-                        entry[1].append((minikind, fingerprint, size, is_exec,
-                                         last_changed_id))
-            state._parents = parent_revision_ids
-            state._ghosts = []
-            state.save()
-        finally:
-            state.unlock()
-        return state
-
-    def test_add_20k_entries(self):
-        """Time how long it takes to add lots of entries."""
-        state = self.time(self.build_helper, [(10, 0), (10, 0), (10, 20)])
-        state.unlock()
-
-    def test_build_20k_dirstate(self):
-        state = self.time(self.build_20k_dirstate)
-        state.lock_read()
-        try:
-            entries = list(state._iter_entries())
-            self.assertEqual(21111, len(entries))
-        finally:
-            state.unlock()
-
-    def test_build_20k_dirstate_1_parent(self):
-        state = self.time(self.build_20k_dirstate_with_parents, 1)
-        state.lock_read()
-        try:
-            state._validate()
-            entries = list(state._iter_entries())
-            self.assertEqual(21111, len(entries))
-        finally:
-            state.unlock()
-
-    def test_build_20k_dirstate_2_parents(self):
-        state = self.time(self.build_20k_dirstate_with_parents, 2)
-        state.lock_read()
-        try:
-            state._validate()
-            entries = list(state._iter_entries())
-            self.assertEqual(21111, len(entries))
-        finally:
-            state.unlock()
-
-    def test_save_20k_tree_0_parents(self):
-        state = self.build_20k_dirstate()
-        state.lock_read()
-        try:
-            self.assertEqual(dirstate.DirState.NOT_IN_MEMORY,
-                             state._dirblock_state)
-            state._read_dirblocks_if_needed()
-            state._dirblock_state = dirstate.DirState.IN_MEMORY_MODIFIED
-            self.time(state.save)
-        finally:
-            state.unlock()
-
-    def test_save_20k_tree_1_parent(self):
-        state = self.build_20k_dirstate_with_parents(1)
-        state.lock_read()
-        try:
-            self.assertEqual(dirstate.DirState.NOT_IN_MEMORY,
-                             state._dirblock_state)
-            state._read_dirblocks_if_needed()
-            state._dirblock_state = dirstate.DirState.IN_MEMORY_MODIFIED
-            self.time(state.save)
-        finally:
-            state.unlock()
-
-    def test_save_20k_tree_2_parents(self):
-        state = self.build_20k_dirstate_with_parents(2)
-        state.lock_read()
-        try:
-            self.assertEqual(dirstate.DirState.NOT_IN_MEMORY,
-                             state._dirblock_state)
-            state._read_dirblocks_if_needed()
-            state._dirblock_state = dirstate.DirState.IN_MEMORY_MODIFIED
-            self.time(state.save)
-        finally:
-            state.unlock()
-
-    def test__read_dirblocks_20k_tree_0_parents_py(self):
-        from bzrlib._dirstate_helpers_py import _read_dirblocks
-        state = self.build_20k_dirstate()
-        state.lock_read()
-        try:
-            self.assertEqual(dirstate.DirState.NOT_IN_MEMORY,
-                             state._dirblock_state)
-            state._read_header_if_needed()
-            self.time(_read_dirblocks, state)
-        finally:
-            state.unlock()
-
-    def test__read_dirblocks_20k_tree_0_parents_pyx(self):
-        self.requireFeature(compiled_dirstate_helpers_feature)
-        from bzrlib._dirstate_helpers_pyx import _read_dirblocks
-        state = self.build_20k_dirstate()
-        state.lock_read()
-        try:
-            self.assertEqual(dirstate.DirState.NOT_IN_MEMORY,
-                             state._dirblock_state)
-            state._read_header_if_needed()
-            self.time(_read_dirblocks, state)
-        finally:
-            state.unlock()
-
-    def test__read_dirblocks_20k_tree_1_parent_py(self):
-        from bzrlib._dirstate_helpers_py import _read_dirblocks
-        state = self.build_20k_dirstate_with_parents(1)
-        state.lock_read()
-        try:
-            self.assertEqual(dirstate.DirState.NOT_IN_MEMORY,
-                             state._dirblock_state)
-            state._read_header_if_needed()
-            self.time(_read_dirblocks, state)
-        finally:
-            state.unlock()
-
-    def test__read_dirblocks_20k_tree_1_parent_pyx(self):
-        self.requireFeature(compiled_dirstate_helpers_feature)
-        from bzrlib._dirstate_helpers_pyx import _read_dirblocks
-        state = self.build_20k_dirstate_with_parents(1)
-        state.lock_read()
-        try:
-            self.assertEqual(dirstate.DirState.NOT_IN_MEMORY,
-                             state._dirblock_state)
-            state._read_header_if_needed()
-            self.time(_read_dirblocks, state)
-        finally:
-            state.unlock()
-
-    def test__read_dirblocks_20k_tree_2_parents_py(self):
-        from bzrlib._dirstate_helpers_py import _read_dirblocks
-        state = self.build_20k_dirstate_with_parents(2)
-        state.lock_read()
-        try:
-            self.assertEqual(dirstate.DirState.NOT_IN_MEMORY,
-                             state._dirblock_state)
-            state._read_header_if_needed()
-            self.time(_read_dirblocks, state)
-        finally:
-            state.unlock()
-
-    def test__read_dirblocks_20k_tree_2_parents_pyx(self):
-        self.requireFeature(compiled_dirstate_helpers_feature)
-        from bzrlib._dirstate_helpers_pyx import _read_dirblocks
-        state = self.build_20k_dirstate_with_parents(2)
-        state.lock_read()
-        try:
-            self.assertEqual(dirstate.DirState.NOT_IN_MEMORY,
-                             state._dirblock_state)
-            state._read_header_if_needed()
-            self.time(_read_dirblocks, state)
-        finally:
-            state.unlock()
-
-    def do_bisect_list(self, bisect_func):
-        """Call bisect_dirblock for each path."""
-        # We use self._paths and self._blocks because we expect it to be a very
-        # long list. And the interface for 'self.time()' causes the parameters
-        # to be printed when run with --lsprof-timed. Which is *really* ugly
-        # when the list is thousands of entries.
-        blocks = self._blocks
-        return [bisect_func(blocks, path) for path in self._paths]
-
-    def do_bisect_list_cached(self, bisect_func):
-        """Same as do_bisect_list, but cache the split paths"""
-        cache = {}
-        blocks = self._blocks
-        return [bisect_func(blocks, path, cache=cache) for path in self._paths]
-
-    def setup_paths_and_offsets(self, state):
-        """Get a list of paths and expected offsets.
-
-        This will be used to check do_bisect_list*
-        """
-        state._read_dirblocks_if_needed()
-        paths = ['']
-        expected_offsets = [0]
-        for offset, info in enumerate(state._dirblocks):
-            dirname = info[0]
-            # We already handled the empty path
-            if dirname == '':
-                continue
-            # all paths are of the form ##_directory
-            # so search for ##_director, ##_directory
-            paths.extend([dirname[:-1], dirname])
-            expected_offsets.extend([offset, offset])
-        self._paths = paths
-        self._expected_offsets = expected_offsets
-        self._blocks = state._dirblocks
-
-    def checkOffsets(self, offsets):
-        """Make sure offsets matches self._expected_offsets"""
-        # These are really long lists, so it is easier to compare them with
-        # assertEqualDiff. So turn them into strings.
-        expected_str = '\n'.join(str(x) for x in self._expected_offsets)
-        offset_str = '\n'.join(str(x) for x in offsets)
-        self.assertEqualDiff(expected_str, offset_str)
-
-    def test_bisect_dirblock_py(self):
-        from bzrlib._dirstate_helpers_py import bisect_dirblock
-        state = self.build_10k_dirstate_dirs()
-        state.lock_read()
-        try:
-            self.setup_paths_and_offsets(state)
-            offsets = self.time(self.do_bisect_list,
-                                bisect_dirblock)
-            self.checkOffsets(offsets)
-        finally:
-            state.unlock()
-
-    def test_bisect_dirblock_cached_py(self):
-        from bzrlib._dirstate_helpers_py import bisect_dirblock
-        state = self.build_10k_dirstate_dirs()
-        state.lock_read()
-        try:
-            self.setup_paths_and_offsets(state)
-            offsets = self.time(self.do_bisect_list_cached,
-                                bisect_dirblock)
-            self.checkOffsets(offsets)
-        finally:
-            state.unlock()
-
-    def test_bisect_dirblock_pyx(self):
-        self.requireFeature(compiled_dirstate_helpers_feature)
-        from bzrlib._dirstate_helpers_pyx import bisect_dirblock
-        state = self.build_10k_dirstate_dirs()
-        state.lock_read()
-        try:
-            self.setup_paths_and_offsets(state)
-            offsets = self.time(self.do_bisect_list, bisect_dirblock)
-            self.checkOffsets(offsets)
-        finally:
-            state.unlock()
-
-    def create_path_names(self, layout, base=''):
-        """Create a list of paths with auto-generated names.
-
-        :param layout: A list of [(num_dirs, num_files)] tuples. For each
-            level, the given number of directories will be created, each
-            containing that many files.
-            So [(2, 5), (3, 4)] will create 2 top level directories, containing
-            5 files, and each top level directory will contain 3 subdirs with 4
-            files.
-        :param base: The base path to prepend to all entries, most callers will
-            pass ''
-        :return: A list of path names.
-        """
-        if not layout:
-            return []
-
-        paths = []
-        num_dirs, num_files = layout[0]
-        for dnum in xrange(num_dirs):
-            if base:
-                path = '%s/%02d_directory' % (base, dnum)
-            else:
-                path = '%02d_directory' % (dnum,)
-            paths.append(path)
-            for fnum in xrange(num_files):
-                fname = '%s/%02d_filename' % (path, fnum)
-                paths.append(fname)
-            paths.extend(self.create_path_names(layout[1:], base=path))
-        return paths
-
-    def test_create_path_names(self):
-        names = self.create_path_names([(2, 3), (1, 2)])
-        self.assertEqual(['00_directory',
-                          '00_directory/00_filename',
-                          '00_directory/01_filename',
-                          '00_directory/02_filename',
-                          '00_directory/00_directory',
-                          '00_directory/00_directory/00_filename',
-                          '00_directory/00_directory/01_filename',
-                          '01_directory',
-                          '01_directory/00_filename',
-                          '01_directory/01_filename',
-                          '01_directory/02_filename',
-                          '01_directory/00_directory',
-                          '01_directory/00_directory/00_filename',
-                          '01_directory/00_directory/01_filename',
-                         ], names)
-        names = self.time(self.create_path_names, [(10, 2), (10, 2), (10, 20)])
-        # 20 files + 1 directory name, 10 times, plus 2 filenames and 1 dir, 10
-        # times, and another 2 files + 1 dir, 10 times
-        self.assertEqual(21330, 10*(3 + 10*(3 + 10*(1 + 20))))
-        self.assertEqual(21330, len(names))
-
-    def compareAllPaths(self, cmp_func, layout):
-        """Compare N^2 paths.
-
-        Basically, compare every path in the list against every other path.
-        """
-        paths = self.create_path_names(layout)
-        def compare_all():
-            for path1 in paths:
-                for path2 in paths:
-                    cmp_func(path1, path2)
-        self.time(compare_all)
-
-    def test_cmp_by_dirs_py(self):
-        """Benchmark 103041 comparisons."""
-        from bzrlib._dirstate_helpers_py import cmp_by_dirs
-        self.compareAllPaths(cmp_by_dirs,
-                             [(3, 1), (3, 1), (3, 1), (3, 2)])
-
-    def test_cmp_by_dirs_pyrex(self):
-        self.requireFeature(compiled_dirstate_helpers_feature)
-        from bzrlib._dirstate_helpers_pyx import cmp_by_dirs
-        self.compareAllPaths(cmp_by_dirs,
-                             [(3, 1), (3, 1), (3, 1), (3, 2)])

=== removed file 'bzrlib/benchmarks/bench_info.py'
--- bzrlib/benchmarks/bench_info.py	2009-03-23 14:59:43 +0000
+++ bzrlib/benchmarks/bench_info.py	1970-01-01 00:00:00 +0000
@@ -1,37 +0,0 @@
-# Copyright (C) 2006 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Tests for bzr info performance."""
-
-
-from bzrlib.benchmarks import Benchmark
-
-
-class InfoBenchmark(Benchmark):
-    """This is a stub. Use this benchmark with a network transport.
-    Currently "bzr info sftp://..." takes > 4 min"""
-
-    def test_no_ignored_unknown_kernel_like_tree(self):
-        """Info in a kernel sized tree with no ignored or unknowns. """
-        self.make_kernel_like_added_tree()
-        self.time(self.run_bzr, 'info')
-
-    def test_no_changes_known_kernel_like_tree(self):
-        """Info in a kernel sized tree with no ignored, unknowns, or added."""
-        self.make_kernel_like_committed_tree()
-        self.time(self.run_bzr, 'info')
-
-

=== removed file 'bzrlib/benchmarks/bench_inventory.py'
--- bzrlib/benchmarks/bench_inventory.py	2009-03-23 14:59:43 +0000
+++ bzrlib/benchmarks/bench_inventory.py	1970-01-01 00:00:00 +0000
@@ -1,39 +0,0 @@
-# Copyright (C) 2006 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Tests for `bzrlib.inventory.Inventory` performance."""
-
-
-import bzrlib.inventory
-from bzrlib.benchmarks import Benchmark
-
-
-class InvBenchmark(Benchmark):
-    """Benchmarks for building large inventories."""
-
-    def test_make_10824_inv_entries(self):
-        """Making 10824 inv entries should be quick."""
-        entries = []
-        def make_10824_entries():
-            for counter in xrange(10000):
-                bzrlib.inventory.make_entry('file', 'foo',
-                    "a_parent_id")
-            for counter in xrange(824):
-                bzrlib.inventory.make_entry('directory', 'foo',
-                    "a_parent_id")
-        # on roberts machine: this originally took:  533ms/  600ms
-        # fixing slots to be vaguely accurate :      365ms/  419ms
-        self.time(make_10824_entries)

=== removed file 'bzrlib/benchmarks/bench_knit.py'
--- bzrlib/benchmarks/bench_knit.py	2010-01-25 17:48:22 +0000
+++ bzrlib/benchmarks/bench_knit.py	1970-01-01 00:00:00 +0000
@@ -1,93 +0,0 @@
-# Copyright (C) 2007 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Benchmarks for Knit performance"""
-
-import os
-
-from bzrlib import (
-    generate_ids,
-    knit,
-    tests,
-    )
-from bzrlib.benchmarks import Benchmark
-from bzrlib.tests.test_knit import CompiledKnitFeature
-
-
-class BenchKnitIndex(Benchmark):
-    """Benchmark Knit index performance."""
-
-    def create_50k_index(self):
-        """Create an knit index file with 50,000 entries.
-
-        This isn't super realistic, but it *is* big :)
-
-        The file 'test.kndx' will be created.
-        """
-        rev_id = generate_ids.gen_revision_id('long.name@this.example.com')
-        versions = [(rev_id, ('fulltext',), 0, 200, [])]
-        pos = 200
-        alt_parent = None
-        for i in xrange(49999):
-            if alt_parent is not None:
-                parent_ids = [rev_id, alt_parent]
-            else:
-                parent_ids = [rev_id]
-            if i % 8 == 0:
-                # The *next* entry will be a merge
-                alt_parent = rev_id
-            else:
-                alt_parent = None
-            rev_id = generate_ids.gen_revision_id('long.name@this.example.com')
-            versions.append((rev_id, ('line-delta',), pos, 200, parent_ids))
-            pos += 200
-        t = self.get_transport()
-        kndx = knit._KnitIndex(t, 'test.kndx', 'w', create=True,
-                               delay_create=True)
-        kndx.add_versions(versions)
-
-    def setup_load_data_c(self):
-        self.requireFeature(CompiledKnitFeature)
-        from bzrlib._knit_load_data_pyx import _load_data_c
-        self.overrideAttr(knit, '_load_data', _load_data_c)
-
-    def setup_load_data_py(self):
-        from bzrlib._knit_load_data_py import _load_data_py
-        self.overrideAttr(knit, '_load_data', _load_data_py)
-
-    def test_read_50k_index_c(self):
-        self.setup_load_data_c()
-        self.create_50k_index()
-        t = self.get_transport()
-        kndx = self.time(knit._KnitIndex, t, 'test.kndx', 'r')
-
-    def test_read_50k_index_c_again(self):
-        self.setup_load_data_c()
-        self.create_50k_index()
-        t = self.get_transport()
-        kndx = self.time(knit._KnitIndex, t, 'test.kndx', 'r')
-
-    def test_read_50k_index_py(self):
-        self.create_50k_index()
-        self.setup_load_data_py()
-        t = self.get_transport()
-        kndx = self.time(knit._KnitIndex, t, 'test.kndx', 'r')
-
-    def test_read_50k_index_py_again(self):
-        self.create_50k_index()
-        self.setup_load_data_py()
-        t = self.get_transport()
-        kndx = self.time(knit._KnitIndex, t, 'test.kndx', 'r')

=== removed file 'bzrlib/benchmarks/bench_log.py'
--- bzrlib/benchmarks/bench_log.py	2009-03-23 14:59:43 +0000
+++ bzrlib/benchmarks/bench_log.py	1970-01-01 00:00:00 +0000
@@ -1,117 +0,0 @@
-# Copyright (C) 2006 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Tests for tree transform performance"""
-
-import os
-import sys
-
-from bzrlib.benchmarks import Benchmark
-from bzrlib.log import log_formatter, show_log
-from bzrlib.osutils import pathjoin
-from cStringIO import StringIO
-from bzrlib.transform import TreeTransform
-from bzrlib.workingtree import WorkingTree
-
-
-class LinesDone(Exception):
-    """Raised when `LineConsumer` reaches the required number of lines."""
-    pass
-
-
-class LineConsumer(object):
-    """Count lines that are produced.
-
-    When the required number of lines have been reached, raise `LinesDone`.
-    """
-
-    def __init__(self, required_lines):
-        """Create a new consumer.
-
-        :param required_lines: How many lines must be produced.
-        :type required_lines: integer
-        """
-        self.required_lines = required_lines
-
-    def write(self, text):
-        """Write some text to the black hole.
-
-        But count how many lines have been written.
-
-        :param text: A string that would be written.
-        :raise LinesDone: when the required number of lines has been reached.
-        :return: None
-        """
-        self.required_lines -= text.count('\n')
-        if self.required_lines < 0:
-            raise LinesDone()
-
-
-class LogBenchmark(Benchmark):
-    """Benchmarks for ``'bzr log'`` performance."""
-
-    def test_log(self):
-        """Run log in a many-commit tree."""
-        tree = self.make_many_commit_tree(hardlink=True)
-        lf = log_formatter('long', to_file=StringIO())
-        self.time(show_log, tree.branch, lf, direction='reverse')
-
-    def test_merge_log(self):
-        """Run log in a tree with many merges"""
-        tree = self.make_heavily_merged_tree(hardlink=True)
-        lf = log_formatter('short', to_file=StringIO())
-        self.time(show_log, tree.branch, lf, direction='reverse')
-
-    def test_log_screenful(self):
-        """Simulate log --long|less"""
-        self.screenful_tester('long')
-
-    def test_log_screenful_line(self):
-        """Simulate log --line|less"""
-        self.screenful_tester('line')
-
-    def test_log_screenful_short(self):
-        """Simulate log --short|less"""
-        self.screenful_tester('short')
-
-    def screenful_tester(self, formatter):
-        """Run show_log, but stop after 25 lines are generated"""
-        tree = self.make_many_commit_tree(hardlink=True)
-        def log_screenful():
-            lf = log_formatter(formatter, to_file=LineConsumer(25))
-            try:
-                show_log(tree.branch, lf, direction='reverse')
-            except LinesDone:
-                pass
-            else:
-                raise Exception, "LinesDone not raised"
-        self.time(log_screenful)
-
-    def test_cmd_log(self):
-        """Test execution of the log command."""
-        tree = self.make_many_commit_tree(hardlink=True)
-        self.time(self.run_bzr, ['log', '-r', '-4..'])
-
-    def test_cmd_log_subprocess(self):
-        """Text startup and execution of the log command."""
-        tree = self.make_many_commit_tree(hardlink=True)
-        self.time(self.run_bzr_subprocess, 'log', '-r', '-4..')
-
-    def test_log_verbose(self):
-        """'verbose' log -- shows file changes"""
-        tree = self.make_many_commit_tree(hardlink=True)
-        lf = log_formatter('long', to_file=StringIO())
-        self.time(show_log, tree.branch, lf, direction='reverse', verbose=True)

=== removed file 'bzrlib/benchmarks/bench_osutils.py'
--- bzrlib/benchmarks/bench_osutils.py	2009-03-23 14:59:43 +0000
+++ bzrlib/benchmarks/bench_osutils.py	1970-01-01 00:00:00 +0000
@@ -1,37 +0,0 @@
-# Copyright (C) 2006 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Tests for bzr osutils functions performance."""
-
-
-from bzrlib.benchmarks import Benchmark
-import bzrlib.osutils as osutils
-
-
-class WalkDirsBenchmark(Benchmark):
-    """Benchmarks for `bzrlib.osutils.walkdirs`"""
-
-    def test_walkdirs_kernel_like_tree(self):
-        """Walking a kernel sized tree is fast!(150ms)."""
-        self.make_kernel_like_tree(link_working=True)
-        # on roberts machine: this originally took:  157ms/4177ms
-        # plain os.walk takes 213ms on this tree
-        # with the pyrex readdir module:              77ms/5423ms
-        def dowalk():
-            for dirblock in osutils.walkdirs('.'):
-                if dirblock[0][1] == '.bzr':
-                    del dirblock[0]
-        self.time(dowalk)

=== removed file 'bzrlib/benchmarks/bench_pack.py'
--- bzrlib/benchmarks/bench_pack.py	2009-03-23 14:59:43 +0000
+++ bzrlib/benchmarks/bench_pack.py	1970-01-01 00:00:00 +0000
@@ -1,54 +0,0 @@
-# Copyright (C) 2007 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Benchmarks for pack performance"""
-
-import os
-
-from bzrlib import (
-    pack,
-    )
-from bzrlib.benchmarks import Benchmark
-
-
-class BenchPack(Benchmark):
-    """Benchmark pack performance."""
-
-    def test_insert_one_gig_1k_chunks_no_names_disk(self):
-        # test real disk writing of many small chunks.
-        # useful for testing whether buffer sizes are right
-        transport = self.get_transport()
-        stream = transport.open_write_stream('pack.pack')
-        writer = pack.ContainerWriter(stream.write)
-        self.write_1_gig(writer)
-        stream.close()
-
-    def test_insert_one_gig_1k_chunks_no_names_null(self):
-        # write to dev/null so we test the pack processing.
-        transport = self.get_transport()
-        dev_null = open('/dev/null', 'wb')
-        writer = pack.ContainerWriter(dev_null.write)
-        self.write_1_gig(writer)
-        dev_null.close()
-
-    def write_1_gig(self, writer):
-        one_k = "A" * 1024
-        writer.begin()
-        def write_1g():
-            for hunk in xrange(1024 * 1024):
-                writer.add_bytes_record(one_k, [])
-        self.time(write_1g)
-        writer.end()

=== removed file 'bzrlib/benchmarks/bench_rocks.py'
--- bzrlib/benchmarks/bench_rocks.py	2009-03-23 14:59:43 +0000
+++ bzrlib/benchmarks/bench_rocks.py	1970-01-01 00:00:00 +0000
@@ -1,28 +0,0 @@
-# Copyright (C) 2006 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Benchmarks of bzr rocks."""
-
-
-from bzrlib.benchmarks import Benchmark
-
-
-class RocksBenchmark(Benchmark):
-    """Benchmarks for ``'bzr rocks'``"""
-
-    def test_rocks(self):
-        """Test the startup overhead by running a do-nothing command"""
-        self.time(self.run_bzr_subprocess, 'rocks')

=== removed file 'bzrlib/benchmarks/bench_sftp.py'
--- bzrlib/benchmarks/bench_sftp.py	2009-03-23 14:59:43 +0000
+++ bzrlib/benchmarks/bench_sftp.py	1970-01-01 00:00:00 +0000
@@ -1,101 +0,0 @@
-# Copyright (C) 2006 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Tests for bzr performance over SFTP"""
-
-import os
-
-from bzrlib import (
-    bzrdir,
-    )
-from bzrlib.benchmarks import Benchmark
-from bzrlib.tests import test_sftp_transport, TestSkipped
-
-try:
-    import paramiko
-    paramiko_loaded = True
-except ImportError:
-    paramiko_loaded = False
-
-
-class SFTPBenchmark(Benchmark):
-    """Benchmark branch, push and pull across a local sftp connection."""
-
-    def setUp(self):
-        super(SFTPBenchmark, self).setUp()
-        if not paramiko_loaded:
-            raise TestSkipped('you must have paramiko to run this test')
-        test_sftp_transport.set_test_transport_to_sftp(self)
-
-    def test_branch(self):
-        os.mkdir("a")
-        tree, files = self.create_with_commits(100, 100, "a")
-        self.time(bzrdir.BzrDir.open(self.get_url('a')).sprout, "b")
-
-    def create_commit_and_pull(self, num_pull_revisions):
-        os.mkdir("a")
-        tree, files = self.create_with_commits(100, 100, "a")
-        rbzrdir = bzrdir.BzrDir.open(self.get_url('a'))
-        b2 = tree.bzrdir.sprout("b") # branch
-        # change a few files and commit
-        self.commit_some_revisions(tree, files, num_pull_revisions, 20)
-        self.time(b2.open_branch().pull, rbzrdir.open_branch())
-
-    def test_pull_1(self):
-        self.create_commit_and_pull(1)
-
-    def test_pull_10(self):
-        self.create_commit_and_pull(10)
-
-    def test_pull_100(self):
-        self.create_commit_and_pull(100)
-
-    def create_commit_and_push(self, num_push_revisions):
-        os.mkdir("a")
-        tree, files = self.create_with_commits(100, 100, "a")
-        rbzrdir = bzrdir.BzrDir.open(self.get_url('a'))
-        b2 = tree.bzrdir.sprout("b") # branch
-        wtree = b2.open_workingtree()
-        # change a few files and commit
-        self.commit_some_revisions(
-            wtree, ["b/%i" for i in range(100)],
-            num_commits=num_push_revisions,
-            changes_per_commit=20)
-        self.time(rbzrdir.open_branch().pull, wtree.branch)
-
-    def test_initial_push(self):
-        os.mkdir('a')
-        tree, files = self.create_with_commits(100, 100, "a")
-        self.time(tree.bzrdir.clone, self.get_url('b'),
-                  revision_id=tree.last_revision())
-
-    def test_push_1(self):
-        self.create_commit_and_push(1)
-
-    def test_push_10(self):
-        self.create_commit_and_push(10)
-
-    def test_push_100(self):
-        self.create_commit_and_push(100)
-
-
-class SFTPSlowSocketBenchmark(SFTPBenchmark):
-    """Benchmarks of SFTP performance with a 30ms delay per roundtrip."""
-
-    def setUp(self):
-        super(SFTPSlowSocketBenchmark, self).setUp()
-        self.get_server().add_latency = 0.03
-

=== removed file 'bzrlib/benchmarks/bench_startup.py'
--- bzrlib/benchmarks/bench_startup.py	2009-03-23 14:59:43 +0000
+++ bzrlib/benchmarks/bench_startup.py	1970-01-01 00:00:00 +0000
@@ -1,91 +0,0 @@
-# Copyright (C) 2006 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Benchmarks of bzr startup time, for some simple operations."""
-
-
-from bzrlib.benchmarks import Benchmark
-
-
-class StartupBenchmark(Benchmark):
-    """Benchmark startup costs for certain bzr commands."""
-
-    def make_simple_tree(self):
-        """A small, simple tree. No caching needed"""
-        tree = self.make_branch_and_tree('.')
-        self.build_tree(['a', 'b/', 'b/c'])
-        tree.add(['a', 'b', 'b/c'])
-        return tree
-
-    def make_simple_committed_tree(self):
-        tree = self.make_simple_tree()
-        tree.commit('simple commit')
-        return tree
-
-    def test___version(self):
-        """Test the startup overhead of plain bzr --version"""
-        self.time(self.run_bzr_subprocess, '--version')
-
-    def test_branch(self):
-        """Test the time to branch this into other"""
-        tree = self.make_simple_committed_tree()
-        self.time(self.run_bzr_subprocess, 'branch', '.', 'other')
-
-    def test_commit(self):
-        """Test execution of simple commit"""
-        tree = self.make_simple_tree()
-        self.time(self.run_bzr_subprocess, 'commit', '-m', 'init simple tree')
-
-    def test_diff(self):
-        """Test simple diff time"""
-        tree = self.make_simple_committed_tree()
-        self.time(self.run_bzr_subprocess, 'diff')
-
-    def test_help(self):
-        """Test the startup overhead of plain bzr help"""
-        self.time(self.run_bzr_subprocess, 'help')
-
-    def test_help_commands(self):
-        """startup time for bzr help commands, which has to load more"""
-        self.time(self.run_bzr_subprocess, 'help', 'commands')
-
-    def test_log(self):
-        """Test simple log time"""
-        tree = self.make_simple_committed_tree()
-        self.time(self.run_bzr_subprocess, 'log')
-
-    def test_missing(self):
-        """Test simple missing time"""
-        tree = self.make_simple_committed_tree()
-        other = tree.bzrdir.sprout('other')
-        self.time(self.run_bzr_subprocess, 'missing', working_dir='other')
-
-    def test_pull(self):
-        """Test simple pull time"""
-        tree = self.make_simple_committed_tree()
-        other = tree.bzrdir.sprout('other')
-        # There should be nothing to pull, and this should be determined
-        # quickly
-        self.time(self.run_bzr_subprocess, 'pull', working_dir='other')
-
-    def test_rocks(self):
-        """Test the startup overhead by running a do-nothing command"""
-        self.time(self.run_bzr_subprocess, 'rocks')
-
-    def test_status(self):
-        """Test simple status time"""
-        tree = self.make_simple_committed_tree()
-        self.time(self.run_bzr_subprocess, 'status')

=== removed file 'bzrlib/benchmarks/bench_status.py'
--- bzrlib/benchmarks/bench_status.py	2009-03-23 14:59:43 +0000
+++ bzrlib/benchmarks/bench_status.py	1970-01-01 00:00:00 +0000
@@ -1,52 +0,0 @@
-# Copyright (C) 2006, 2007 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Tests for bzr status performance."""
-
-
-from bzrlib.benchmarks import Benchmark
-
-
-class StatusBenchmark(Benchmark):
-    """Benchmarks for ``'bzr status'``."""
-
-    def test_no_ignored_unknown_kernel_like_tree(self):
-        """Status in a kernel sized tree with no ignored or unknowns.
-
-        This should be bearable (<2secs) fast.
-        """
-        self.make_kernel_like_added_tree()
-        # on robertc's machine the first sample of this took 1687ms/15994ms
-        self.time(self.run_bzr, 'status')
-
-    def test_no_changes_known_kernel_like_tree(self):
-        """Status in a kernel sized tree with no ignored, unknowns, or added."""
-        self.make_kernel_like_committed_tree(link_bzr=True)
-        self.time(self.run_bzr, 'status')
-
-    def test_single_file_no_changes_known_kernel_like_tree(self):
-        """Status in a kernel sized tree with no ignored, unknowns, or added
-        of a single file."""
-        self.make_kernel_like_committed_tree()
-        #XXX depends on the filenames generated by make_kernel_like_tree
-        self.time(self.run_bzr, ['status', '7/4/0/16'])
-
-    def test_status_one_added_file_kernel_like_tree(self):
-        """Status of a single added file in our stock large tree."""
-        self.make_kernel_like_tree()
-        self.run_bzr('add')
-        self.run_bzr(['commit', '-m', 'initial import'])
-        self.time(self.run_bzr, ['status', '3/3/3/10'])

=== removed file 'bzrlib/benchmarks/bench_tags.py'
--- bzrlib/benchmarks/bench_tags.py	2009-05-27 08:33:37 +0000
+++ bzrlib/benchmarks/bench_tags.py	1970-01-01 00:00:00 +0000
@@ -1,58 +0,0 @@
-# Copyright (C) 2007 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Test for tags serialization (indirect testing for bencode)"""
-
-
-import time
-
-from bzrlib.benchmarks import Benchmark
-import bzrlib.bencode
-import bzrlib.tag
-
-
-class TagsBencodeBenchmark(Benchmark):
-    """Benchmark for serialization/deserialization of tags"""
-
-    def setUp(self):
-        super(TagsBencodeBenchmark, self).setUp()
-        self.tobj = bzrlib.tag.BasicTags(None)
-        tags = {}
-        revid = 'j.random@example.com-20070812132500-%016d'
-        for i in xrange(100):
-            tags[str(i)] = revid % i
-        self.tags = tags
-        self.bencoded_tags = bzrlib.bencode.bencode(tags)
-
-    def time_N(self, N, kallable, *args, **kwargs):
-        def _func(N, kallable, *args, **kwargs):
-            for i in xrange(N):
-                kallable(*args, **kwargs)
-        self.time(_func, N, kallable, *args, **kwargs)
-
-    def test_serialize_empty_tags(self):
-        # Measure overhead of operation
-        self.time_N(10000, self.tobj._serialize_tag_dict, {})
-
-    def test_deserialize_empty_tags(self):
-        # Measure overhead of operation
-        self.time_N(10000, self.tobj._deserialize_tag_dict, 'de')
-
-    def test_serialize_tags(self):
-        self.time_N(1000, self.tobj._serialize_tag_dict, self.tags)
-
-    def test_deserialize_tags(self):
-        self.time_N(1000, self.tobj._deserialize_tag_dict, self.bencoded_tags)

=== removed file 'bzrlib/benchmarks/bench_transform.py'
--- bzrlib/benchmarks/bench_transform.py	2009-03-23 14:59:43 +0000
+++ bzrlib/benchmarks/bench_transform.py	1970-01-01 00:00:00 +0000
@@ -1,43 +0,0 @@
-# Copyright (C) 2006 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Tests for tree transform performance"""
-
-import os
-
-from bzrlib.benchmarks import Benchmark
-from bzrlib.osutils import pathjoin
-from bzrlib.transform import TreeTransform
-from bzrlib.workingtree import WorkingTree
-
-
-class TransformBenchmark(Benchmark):
-    """Benchmarks for `bzrlib.transform`"""
-
-    def test_canonicalize_path(self):
-        """Canonicalizing paths should be fast."""
-        wt = self.make_kernel_like_tree(link_working=True)
-        paths = []
-        for dirpath, dirnames, filenames in os.walk('.'):
-            paths.extend(pathjoin(dirpath, d) for d in dirnames)
-            paths.extend(pathjoin(dirpath, f) for f in filenames)
-        tt = TreeTransform(wt)
-        self.time(self.canonicalize_paths, tt, paths)
-        tt.finalize()
-
-    def canonicalize_paths(self, tt, paths):
-        for path in paths:
-            tt.canonical_path(path)

=== removed file 'bzrlib/benchmarks/bench_workingtree.py'
--- bzrlib/benchmarks/bench_workingtree.py	2009-03-23 14:59:43 +0000
+++ bzrlib/benchmarks/bench_workingtree.py	1970-01-01 00:00:00 +0000
@@ -1,111 +0,0 @@
-# Copyright (C) 2006 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Tests for bzr working tree performance."""
-
-import os
-
-from bzrlib import ignores
-from bzrlib.benchmarks import Benchmark
-from bzrlib.workingtree import WorkingTree
-
-
-class WorkingTreeBenchmark(Benchmark):
-    """Benchmarks for `bzrlib.workingtree` performance."""
-
-    def test_list_files_kernel_like_tree(self):
-        tree = self.make_kernel_like_added_tree()
-        tree.lock_read()
-        try:
-            self.time(list, tree.list_files())
-        finally:
-            tree.unlock()
-
-    def test_list_files_unknown_kernel_like_tree(self):
-        tree = self.make_kernel_like_tree(link_working=True)
-        tree = WorkingTree.open('.')
-        # Bzr only traverses directories if they are versioned
-        # So add all the directories, but not the files, yielding
-        # lots of unknown files.
-        for root, dirs, files in os.walk('.'):
-            if '.bzr' in dirs:
-                dirs.remove('.bzr')
-            if root == '.':
-                continue
-            tree.add(root)
-        tree.lock_read()
-        try:
-            self.time(list, tree.list_files())
-        finally:
-            tree.unlock()
-
-    def test_is_ignored_single_call(self):
-        """How long does is_ignored take to initialise and check one file."""
-        t = self.make_branch_and_tree('.')
-        self.time(t.is_ignored, "CVS")
-
-    def test_is_ignored_10824_calls(self):
-        """How long does is_ignored take to initialise and check one file."""
-        t = self.make_branch_and_tree('.')
-        def call_is_ignored_10824_not_ignored():
-            for x in xrange(10824):
-                t.is_ignored(str(x))
-        self.time(call_is_ignored_10824_not_ignored)
-
-    def test_is_ignored_10_patterns(self):
-        t = self.make_branch_and_tree('.')
-        ignores.add_runtime_ignores([u'*.%i' % i for i in range(1, 9)])
-        ignores.add_runtime_ignores(['./foo', 'foo/bar'])
-        self.time(t.is_ignored,'bar')
-        ignores._runtime_ignores = set()
-
-    def test_is_ignored_50_patterns(self):
-        t = self.make_branch_and_tree('.')
-        ignores.add_runtime_ignores([u'*.%i' % i for i in range(1, 49)])
-        ignores.add_runtime_ignores(['./foo', 'foo/bar'])
-        self.time(t.is_ignored,'bar')
-        ignores._runtime_ignores = set()
-
-    def test_is_ignored_100_patterns(self):
-        t = self.make_branch_and_tree('.')
-        ignores.add_runtime_ignores([u'*.%i' % i for i in range(1, 99)])
-        ignores.add_runtime_ignores(['./foo', 'foo/bar'])
-        self.time(t.is_ignored,'bar')
-        ignores._runtime_ignores = set()
-
-    def test_is_ignored_1000_patterns(self):
-        t = self.make_branch_and_tree('.')
-        ignores.add_runtime_ignores([u'*.%i' % i for i in range(1, 999)])
-        ignores.add_runtime_ignores(['./foo', 'foo/bar'])
-        self.time(t.is_ignored,'bar')
-        ignores._runtime_ignores = set()
-
-    def test_walkdirs_kernel_like_tree(self):
-        """Walking a kernel sized tree is fast!(150ms)."""
-        self.make_kernel_like_tree()
-        self.run_bzr('add')
-        tree = WorkingTree.open('.')
-        # on roberts machine: this originally took:  157ms/4177ms
-        # plain os.walk takes 213ms on this tree
-        self.time(list, tree.walkdirs())
-
-    def test_walkdirs_kernel_like_tree_unknown(self):
-        """Walking a kernel sized tree is fast!(150ms)."""
-        self.make_kernel_like_tree()
-        tree = WorkingTree.open('.')
-        # on roberts machine: this originally took:  157ms/4177ms
-        # plain os.walk takes 213ms on this tree
-        self.time(list, tree.walkdirs())

=== removed file 'bzrlib/benchmarks/bench_xml.py'
--- bzrlib/benchmarks/bench_xml.py	2009-03-23 14:59:43 +0000
+++ bzrlib/benchmarks/bench_xml.py	1970-01-01 00:00:00 +0000
@@ -1,87 +0,0 @@
-# Copyright (C) 2006 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Tests for bzr xml serialization performance."""
-
-from bzrlib import (
-    cache_utf8,
-    xml5,
-    )
-from bzrlib.benchmarks import Benchmark
-
-
-class BenchXMLSerializer(Benchmark):
-    """Benchmarks for serializing to/from XML."""
-
-    def test_write_to_string_kernel_like_inventory(self):
-        # On jam's machine, ElementTree serializer took: 2161ms/13487ms
-        #                      with Robert's serializer:  631ms/10770ms
-        #                      with Entity escaper:       487ms/11636ms
-        #           caching Entity escaper, empty cache:  448ms/ 9489ms
-        #           caching Entity escaper, full cache:   375ms/ 9489ms
-        #                      passing around function:   406ms/ 8942ms
-        #              cached, passing around function:   328ms/11248ms
-        #                      removing extra function:   354ms/ 8942ms
-        #              cached, removing extra function:   275ms/11248ms
-        #                          no cache, real utf8:   363ms/11697ms
-        #                            cached, real utf8:   272ms/12827ms
-        # Really all we want is a real inventory
-        inv = self.make_kernel_like_inventory()
-
-        xml5._clear_cache()
-        # We want a real tree with lots of file ids and sha strings, etc.
-        self.time(xml5.serializer_v5.write_inventory_to_string, inv)
-
-    def test_write_kernel_like_inventory(self):
-        # Really all we want is a real inventory
-        inv = self.make_kernel_like_inventory()
-
-        xml5._clear_cache()
-        f = open('kernel-like-inventory', 'wb')
-        try:
-            # We want a real tree with lots of file ids and sha strings, etc.
-            self.time(xml5.serializer_v5.write_inventory, inv, f)
-        finally:
-            f.close()
-
-    def test_write_to_string_cached_kernel_like_inventory(self):
-        inv = self.make_kernel_like_inventory()
-
-        xml5._clear_cache()
-        # We want a real tree with lots of file ids and sha strings, etc.
-        xml5.serializer_v5.write_inventory_to_string(inv)
-
-        self.time(xml5.serializer_v5.write_inventory_to_string, inv)
-
-    def test_read_from_string_kernel_like_inventory(self):
-        inv = self.make_kernel_like_inventory()
-        as_str = xml5.serializer_v5.write_inventory_to_string(inv)
-
-        cache_utf8.clear_encoding_cache()
-        read_inv = self.time(xml5.serializer_v5.read_inventory_from_string,
-                             as_str)
-        # TODO: make sure the final inventory is equal as a sanity check
-
-    def test_read_from_string_cached_kernel_like_inventory(self):
-        cache_utf8.clear_encoding_cache()
-        inv = self.make_kernel_like_inventory()
-        as_str = xml5.serializer_v5.write_inventory_to_string(inv)
-
-        xml5.serializer_v5.read_inventory_from_string(as_str)
-
-        read_inv = self.time(xml5.serializer_v5.read_inventory_from_string,
-                             as_str)
-        # TODO: make sure the final inventory is equal as a sanity check

=== removed directory 'bzrlib/benchmarks/tree_creator'
=== removed file 'bzrlib/benchmarks/tree_creator/__init__.py'
--- bzrlib/benchmarks/tree_creator/__init__.py	2009-03-23 14:59:43 +0000
+++ bzrlib/benchmarks/tree_creator/__init__.py	1970-01-01 00:00:00 +0000
@@ -1,175 +0,0 @@
-# Copyright (C) 2006 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Base implementation of TreeCreator classes
-
-These are classes that are used to easily create test trees.
-"""
-
-import os
-import shutil
-
-from bzrlib import (
-    osutils,
-    workingtree,
-    )
-
-
-class TreeCreator(object):
-    """Just a basic class which is used to create various test trees"""
-
-    CACHE_ROOT = None
-
-    def __init__(self, test, tree_name,
-                 link_bzr=False,
-                 link_working=False,
-                 hot_cache=True):
-        """Instantiate a new creator object, supply the id of the tree
-
-        :param test: A TestCaseWithTransport object (most creators need
-            we need the build_tree functionality)
-        """
-
-        self._cache_root = TreeCreator.CACHE_ROOT
-        self._test = test
-        self._tree_name = tree_name
-        self._link_bzr = link_bzr
-        self._link_working = link_working
-        self._hot_cache = hot_cache
-        if not osutils.hardlinks_good():
-            self._link_working = self._link_bzr = False
-
-    def is_caching_enabled(self):
-        """Will we try to cache the tree we create?"""
-        return self._cache_root is not None
-
-    def is_cached(self):
-        """Is this tree already cached?"""
-        cache_dir = self._get_cache_dir()
-        if cache_dir is None:
-            return False
-        return os.path.exists(cache_dir)
-
-    def disable_cache(self):
-        """Do not use the cache"""
-        self._cache_root = None
-
-    def ensure_cached(self):
-        """If caching, make sure the cached copy exists"""
-        cache_dir = self._get_cache_dir()
-        if cache_dir is None:
-            return
-
-        if not self.is_cached():
-            self._create_tree(root=cache_dir, in_cache=True)
-
-    def create(self, root):
-        """Create a new tree at 'root'.
-
-        :return: A WorkingTree object.
-        """
-        cache_dir = self._get_cache_dir()
-        if cache_dir is None:
-            # Not caching
-            return self._create_tree(root, in_cache=False)
-
-        self.ensure_cached()
-
-        return self._clone_cached_tree(root)
-
-    def _get_cache_dir(self):
-        """Get the directory to use for caching this tree
-
-        :return: The path to use for caching. If None, caching is disabled
-        """
-        if self._cache_root is None:
-            return None
-        return osutils.pathjoin(self._cache_root, self._tree_name)
-
-    def _create_tree(self, root, in_cache=False):
-        """Create the desired tree in the given location.
-
-        Children should override this function to provide the actual creation
-        of the desired tree. This will be called by 'create()'. If it is
-        building a tree in the cache, before copying it to the real target,
-        it will pass in_cache=True
-        """
-        raise NotImplemented(self._create_tree)
-
-    def _clone_cached_tree(self, dest):
-        """Copy the contents of the cached dir into the destination
-        Optionally hardlink certain pieces of the tree.
-
-        This is just meant as a helper function for child classes
-
-        :param dest: The destination to copy things to
-        """
-        # We use shutil.copyfile so that we don't copy permissions
-        # because most of our source trees are marked readonly to
-        # prevent modifying in the case of hardlinks
-        handlers = {'file':shutil.copyfile}
-        if osutils.hardlinks_good():
-            if self._link_working:
-                if self._link_bzr:
-                    handlers = {'file':os.link}
-                else:
-                    # Don't hardlink files inside bzr
-                    def file_handler(source, dest):
-                        if '.bzr/' in source:
-                            shutil.copyfile(source, dest)
-                        else:
-                            os.link(source, dest)
-                    handlers = {'file':file_handler}
-            elif self._link_bzr:
-                # Only link files inside .bzr/
-                def file_handler(source, dest):
-                    if '.bzr/' in source:
-                        os.link(source, dest)
-                    else:
-                        shutil.copyfile(source, dest)
-                handlers = {'file':file_handler}
-
-        source = self._get_cache_dir()
-        osutils.copy_tree(source, dest, handlers=handlers)
-        tree = workingtree.WorkingTree.open(dest)
-        if self._hot_cache:
-            tree.lock_write()
-            try:
-                # tree._hashcache.scan() just checks and removes
-                # entries that are out of date
-                # we need to actually store new ones
-                for path, ie in tree.inventory.iter_entries_by_dir():
-                    tree.get_file_sha1(ie.file_id, path)
-            finally:
-                tree.unlock()
-        # If we didn't iterate the tree, the hash cache is technically
-        # invalid, and it would be better to remove it, but there is
-        # no public api for that.
-        return tree
-
-    def _protect_files(self, root):
-        """Chmod all files underneath 'root' to prevent writing
-
-        This is a helper function for child classes.
-
-        :param root: The base directory to modify
-        """
-        for dirinfo, entries in osutils.walkdirs(root):
-            for relpath, name, kind, st, abspath in entries:
-                if kind == 'file':
-                    os.chmod(abspath, 0440)
-
-

=== removed file 'bzrlib/benchmarks/tree_creator/heavily_merged.py'
--- bzrlib/benchmarks/tree_creator/heavily_merged.py	2009-03-23 14:59:43 +0000
+++ bzrlib/benchmarks/tree_creator/heavily_merged.py	1970-01-01 00:00:00 +0000
@@ -1,75 +0,0 @@
-# Copyright (C) 2006 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Tree creator for many commits, but no changes"""
-
-import errno
-import os
-
-from bzrlib import (
-    bzrdir,
-    )
-
-from bzrlib.benchmarks.tree_creator import TreeCreator
-
-
-class HeavilyMergedTreeCreator(TreeCreator):
-    """Create a tree in which almost every commit is a merge.
-
-    No file changes are included.  This produces two trees,
-    one of which is returned.  Except for the first commit, every
-    commit in its revision-history is a merge of another commit in the other
-    tree.
-    Not hardlinking the working tree, because there are no working tree files.
-    """
-
-    def __init__(self, test, link_bzr=True):
-        super(HeavilyMergedTreeCreator, self).__init__(test,
-            tree_name='heavily_merged_tree',
-            link_bzr=link_bzr,
-            link_working=False,
-            hot_cache=True)
-
-    def _create_tree(self, root, in_cache=False):
-        try:
-            os.mkdir(root)
-        except (IOError, OSError), e:
-            if e.errno not in (errno.EEXIST,):
-                raise
-
-        tree = bzrdir.BzrDir.create_standalone_workingtree(root)
-        tree.lock_write()
-        try:
-            tree2 = tree.bzrdir.sprout(root + '/tree2').open_workingtree()
-            tree2.lock_write()
-            try:
-                for i in xrange(250):
-                    revision_id = tree.commit('no-changes commit %d-a' % i)
-                    tree2.branch.fetch(tree.branch, revision_id)
-                    tree2.add_parent_tree_id(revision_id)
-                    revision_id = tree2.commit('no-changes commit %d-b' % i)
-                    tree.branch.fetch(tree2.branch, revision_id)
-                    tree.add_parent_tree_id(revision_id)
-                tree.set_parent_ids(tree.get_parent_ids()[:1])
-            finally:
-                tree2.unlock()
-        finally:
-            tree.unlock()
-        if in_cache:
-            self._protect_files(root+'/.bzr')
-        return tree
-
-

=== removed file 'bzrlib/benchmarks/tree_creator/kernel_like.py'
--- bzrlib/benchmarks/tree_creator/kernel_like.py	2010-03-25 09:39:03 +0000
+++ bzrlib/benchmarks/tree_creator/kernel_like.py	1970-01-01 00:00:00 +0000
@@ -1,225 +0,0 @@
-# Copyright (C) 2006-2010 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Tree creators for kernel-like trees"""
-
-import errno
-import os
-
-from bzrlib import (
-    bzrdir,
-    osutils,
-    workingtree,
-    xml5,
-    )
-
-from bzrlib.benchmarks.tree_creator import TreeCreator
-
-
-class KernelLikeTreeCreator(TreeCreator):
-    """Create a basic tree with ~10k unversioned files"""
-
-    def __init__(self, test, link_working=False, url=None):
-        super(KernelLikeTreeCreator, self).__init__(test,
-            tree_name='kernel_like_tree',
-            link_working=link_working,
-            link_bzr=False)
-
-        self._url = url
-
-    def create(self, root):
-        """Create all the kernel files in the given location.
-
-        This is overloaded for compatibility reasons.
-        """
-        if self._url is not None:
-            b = bzrdir.BzrDir.create_branch_convenience(self._url)
-            d = bzrdir.BzrDir.create(root)
-            bzrlib.branch.BranchReferenceFormat().initialize(d,
-                target_branch=b)
-            tree = d.create_workingtree()
-        else:
-            tree = bzrdir.BzrDir.create_standalone_workingtree(root)
-
-        if not self._link_working or not self.is_caching_enabled():
-            # Turns out that 'shutil.copytree()' is no faster than
-            # just creating them. Probably the python overhead.
-            # Plain _make_kernel_files takes 3-5s
-            # cp -a takes 3s
-            # using hardlinks takes < 1s.
-            self._create_tree(root=root, in_cache=False)
-            return tree
-
-        self.ensure_cached()
-        cache_dir = self._get_cache_dir()
-        osutils.copy_tree(cache_dir, root,
-                          handlers={'file':os.link})
-        return tree
-
-    def _create_tree(self, root, in_cache=False):
-        # a kernel tree has ~10000 and 500 directory, with most files around
-        # 3-4 levels deep.
-        # we simulate this by three levels of dirs named 0-7, givin 512 dirs,
-        # and 20 files each.
-        files = []
-        for outer in range(8):
-            files.append("%s/" % outer)
-            for middle in range(8):
-                files.append("%s/%s/" % (outer, middle))
-                for inner in range(8):
-                    prefix = "%s/%s/%s/" % (outer, middle, inner)
-                    files.append(prefix)
-                    files.extend([prefix + str(foo) for foo in range(20)])
-        cwd = osutils.getcwd()
-        try:
-            os.mkdir(root)
-        except OSError, e:
-            if e.errno not in (errno.EEXIST,):
-                raise
-        os.chdir(root)
-        self._test.build_tree(files)
-        os.chdir(cwd)
-        if in_cache:
-            self._protect_files(root)
-
-
-class KernelLikeAddedTreeCreator(TreeCreator):
-    """Create a tree with ~10k versioned but not committed files"""
-
-    def __init__(self, test, link_working=False, hot_cache=True):
-        super(KernelLikeAddedTreeCreator, self).__init__(test,
-            tree_name='kernel_like_added_tree',
-            link_working=link_working,
-            link_bzr=False,
-            hot_cache=hot_cache)
-
-    def _create_tree(self, root, in_cache=False):
-        """Create a kernel-like tree with the all files added
-
-        :param root: The root directory to create the files
-        :param in_cache: Is this being created in the cache dir?
-        """
-        kernel_creator = KernelLikeTreeCreator(self._test,
-                                               link_working=in_cache)
-        tree = kernel_creator.create(root=root)
-
-        # Add everything to it
-        tree.lock_write()
-        try:
-            tree.smart_add([root], recurse=True, save=True)
-            if in_cache:
-                self._protect_files(root+'/.bzr')
-        finally:
-            tree.unlock()
-        return tree
-
-
-class KernelLikeCommittedTreeCreator(TreeCreator):
-    """Create a tree with ~10K files, and a single commit adding all of them"""
-
-    def __init__(self, test, link_working=False, link_bzr=False,
-                 hot_cache=True):
-        super(KernelLikeCommittedTreeCreator, self).__init__(test,
-            tree_name='kernel_like_committed_tree',
-            link_working=link_working,
-            link_bzr=link_bzr,
-            hot_cache=hot_cache)
-
-    def _create_tree(self, root, in_cache=False):
-        """Create a kernel-like tree with all files committed
-
-        :param root: The root directory to create the files
-        :param in_cache: Is this being created in the cache dir?
-        """
-        kernel_creator = KernelLikeAddedTreeCreator(self._test,
-                                                    link_working=in_cache,
-                                                    hot_cache=(not in_cache))
-        tree = kernel_creator.create(root=root)
-        tree.commit('first post', rev_id='r1')
-
-        if in_cache:
-            self._protect_files(root+'/.bzr')
-        return tree
-
-
-class KernelLikeInventoryCreator(TreeCreator):
-    """Return just the memory representation of a committed kernel-like tree"""
-
-    def __init__(self, test):
-        super(KernelLikeInventoryCreator, self).__init__(test,
-            tree_name='kernel_like_inventory',
-            link_working=True,
-            link_bzr=True,
-            hot_cache=True)
-
-    def ensure_cached(self):
-        """Make sure we have a cached version of the kernel-like inventory"""
-        cache_dir = self._get_cache_dir()
-        if cache_dir is None:
-            return
-
-        if self.is_cached():
-            return
-
-        committed_creator = KernelLikeCommittedTreeCreator(self._test,
-                                                           link_working=True,
-                                                           link_bzr=True,
-                                                           hot_cache=False)
-        committed_creator.ensure_cached()
-        committed_cache_dir = committed_creator._get_cache_dir()
-        committed_tree = workingtree.WorkingTree.open(committed_cache_dir)
-        rev_tree = committed_tree.basis_tree()
-        os.mkdir(cache_dir)
-        f = open(cache_dir+'/inventory', 'wb')
-        try:
-            xml5.serializer_v5.write_inventory(rev_tree.inventory, f)
-        finally:
-            f.close()
-
-    def create(self, root=None):
-        """Create a kernel like inventory
-
-        :param root: Exists to mimic the base class, but this class
-            returns only an in-memory Inventory, so it should always be None.
-        :return: An Inventory object.
-        """
-        cache_dir = self._get_cache_dir()
-        if cache_dir is None:
-            return self._create_and_return()
-
-        self.ensure_cached()
-        return self._open_cached(cache_dir)
-
-    def _create_and_return(self):
-        """Create a kernel-like tree, and return its inventory"""
-        creator = KernelLikeCommittedTreeCreator(self._test,
-                                                 link_working=True,
-                                                 link_bzr=True,
-                                                 hot_cache=False)
-        tree = creator.create('.')
-        basis = tree.basis_tree()
-        basis.lock_read()
-        try:
-            return basis.inventory
-        finally:
-            basis.unlock()
-
-    def _open_cached(self, cache_dir):
-        f = open(cache_dir + '/inventory', 'rb')
-        try:
-            return xml5.serializer_v5.read_inventory(f)
-        finally:
-            f.close()

=== removed file 'bzrlib/benchmarks/tree_creator/many_commit.py'
--- bzrlib/benchmarks/tree_creator/many_commit.py	2009-03-23 14:59:43 +0000
+++ bzrlib/benchmarks/tree_creator/many_commit.py	1970-01-01 00:00:00 +0000
@@ -1,68 +0,0 @@
-# Copyright (C) 2006 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Tree creator for many commits, including file changes."""
-
-from bzrlib import (
-    bzrdir,
-    )
-
-from bzrlib.benchmarks.tree_creator import TreeCreator
-
-
-class ManyCommitTreeCreator(TreeCreator):
-    """Create an tree many files and many commits."""
-
-    def __init__(self, test, link_bzr=False, num_files=10, num_commits=10):
-        tree_name = 'many_files_many_commit_tree_%d_%d' % (
-            num_files, num_commits)
-        super(ManyCommitTreeCreator, self).__init__(test,
-            tree_name=tree_name,
-            link_bzr=link_bzr,
-            link_working=False,
-            hot_cache=True)
-        self.files = ["%s" % (i, ) for i in range(num_files)]
-        self.num_files = num_files
-        self.num_commits = num_commits
-
-    def _create_tree(self, root, in_cache=False):
-        num_files = self.num_files
-        num_commits = self.num_commits
-        files = ["%s/%s" % (root, fn) for fn in self.files]
-        for fn in files:
-            f = open(fn, "wb")
-            try:
-                f.write("some content\n")
-            finally:
-                f.close()
-        tree = bzrdir.BzrDir.create_standalone_workingtree(root)
-        tree.add(self.files)
-        tree.lock_write()
-        try:
-            tree.commit('initial commit')
-            for i in range(num_commits):
-                fn = files[i % len(files)]
-                content = range(i) + [i, i, i, ""]
-                f = open(fn, "wb")
-                try:
-                    f.write("\n".join([str(i) for i in content]))
-                finally:
-                    f.close()
-                tree.commit("changing file %s" % fn)
-        finally:
-            tree.unlock()
-        return tree
-

=== removed file 'bzrlib/benchmarks/tree_creator/simple_many_commit.py'
--- bzrlib/benchmarks/tree_creator/simple_many_commit.py	2009-03-23 14:59:43 +0000
+++ bzrlib/benchmarks/tree_creator/simple_many_commit.py	1970-01-01 00:00:00 +0000
@@ -1,47 +0,0 @@
-# Copyright (C) 2006 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Tree creator for many commits, but no changes"""
-
-from bzrlib import (
-    bzrdir,
-    )
-
-from bzrlib.benchmarks.tree_creator import TreeCreator
-
-
-class SimpleManyCommitTreeCreator(TreeCreator):
-    """Create an empty tree with lots of commits"""
-
-    def __init__(self, test, link_bzr=False):
-        super(SimpleManyCommitTreeCreator, self).__init__(test,
-            tree_name='many_commit_tree',
-            link_bzr=link_bzr,
-            link_working=False,
-            hot_cache=True)
-
-    def _create_tree(self, root, in_cache=False):
-        tree = bzrdir.BzrDir.create_standalone_workingtree(root)
-        tree.lock_write()
-        try:
-            for i in xrange(1000):
-                tree.commit('no-changes commit %d' % i)
-        finally:
-            tree.unlock()
-        if in_cache:
-            self._protect_files(root+'/.bzr')
-
-        return tree

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2010-04-28 09:40:23 +0000
+++ bzrlib/branch.py	2010-09-28 18:51:47 +0000
@@ -25,10 +25,12 @@
         bzrdir,
         cache_utf8,
         config as _mod_config,
+        controldir,
         debug,
         errors,
         lockdir,
         lockable_files,
+        remote,
         repository,
         revision as _mod_revision,
         rio,
@@ -49,7 +51,7 @@
 from bzrlib.decorators import needs_read_lock, needs_write_lock, only_raises
 from bzrlib.hooks import HookPoint, Hooks
 from bzrlib.inter import InterObject
-from bzrlib.lock import _RelockDebugMixin
+from bzrlib.lock import _RelockDebugMixin, LogicalLockResult
 from bzrlib import registry
 from bzrlib.symbol_versioning import (
     deprecated_in,
@@ -63,7 +65,7 @@
 BZR_BRANCH_FORMAT_6 = "Bazaar Branch Format 6 (bzr 0.15)\n"
 
 
-class Branch(bzrdir.ControlComponent):
+class Branch(controldir.ControlComponent):
     """Branch holding a history of revisions.
 
     :ivar base:
@@ -197,6 +199,13 @@
         return self.supports_tags() and self.tags.get_tag_dict()
 
     def get_config(self):
+        """Get a bzrlib.config.BranchConfig for this Branch.
+
+        This can then be used to get and set configuration options for the
+        branch.
+
+        :return: A bzrlib.config.BranchConfig.
+        """
         return BranchConfig(self)
 
     def _get_config(self):
@@ -238,9 +247,13 @@
         if not local and not config.has_explicit_nickname():
             try:
                 master = self.get_master_branch(possible_transports)
+                if master and self.user_url == master.user_url:
+                    raise errors.RecursiveBind(self.user_url)
                 if master is not None:
                     # return the master branch value
                     return master.nick
+            except errors.RecursiveBind, e:
+                raise e
             except errors.BzrError, e:
                 # Silently fall back to local implicit nick if the master is
                 # unavailable
@@ -283,10 +296,20 @@
         new_history.reverse()
         return new_history
 
-    def lock_write(self):
+    def lock_write(self, token=None):
+        """Lock the branch for write operations.
+
+        :param token: A token to permit reacquiring a previously held and
+            preserved lock.
+        :return: A BranchWriteLockResult.
+        """
         raise NotImplementedError(self.lock_write)
 
     def lock_read(self):
+        """Lock the branch for read operations.
+
+        :return: A bzrlib.lock.LogicalLockResult.
+        """
         raise NotImplementedError(self.lock_read)
 
     def unlock(self):
@@ -783,28 +806,56 @@
             if len(old_repository._fallback_repositories) != 1:
                 raise AssertionError("can't cope with fallback repositories "
                     "of %r" % (self.repository,))
-            # unlock it, including unlocking the fallback
+            # Open the new repository object.
+            # Repositories don't offer an interface to remove fallback
+            # repositories today; take the conceptually simpler option and just
+            # reopen it.  We reopen it starting from the URL so that we
+            # get a separate connection for RemoteRepositories and can
+            # stream from one of them to the other.  This does mean doing
+            # separate SSH connection setup, but unstacking is not a
+            # common operation so it's tolerable.
+            new_bzrdir = bzrdir.BzrDir.open(self.bzrdir.root_transport.base)
+            new_repository = new_bzrdir.find_repository()
+            if new_repository._fallback_repositories:
+                raise AssertionError("didn't expect %r to have "
+                    "fallback_repositories"
+                    % (self.repository,))
+            # Replace self.repository with the new repository.
+            # Do our best to transfer the lock state (i.e. lock-tokens and
+            # lock count) of self.repository to the new repository.
+            lock_token = old_repository.lock_write().repository_token
+            self.repository = new_repository
+            if isinstance(self, remote.RemoteBranch):
+                # Remote branches can have a second reference to the old
+                # repository that need to be replaced.
+                if self._real_branch is not None:
+                    self._real_branch.repository = new_repository
+            self.repository.lock_write(token=lock_token)
+            if lock_token is not None:
+                old_repository.leave_lock_in_place()
             old_repository.unlock()
+            if lock_token is not None:
+                # XXX: self.repository.leave_lock_in_place() before this
+                # function will not be preserved.  Fortunately that doesn't
+                # affect the current default format (2a), and would be a
+                # corner-case anyway.
+                #  - Andrew Bennetts, 2010/06/30
+                self.repository.dont_leave_lock_in_place()
+            old_lock_count = 0
+            while True:
+                try:
+                    old_repository.unlock()
+                except errors.LockNotHeld:
+                    break
+                old_lock_count += 1
+            if old_lock_count == 0:
+                raise AssertionError(
+                    'old_repository should have been locked at least once.')
+            for i in range(old_lock_count-1):
+                self.repository.lock_write()
+            # Fetch from the old repository into the new.
             old_repository.lock_read()
             try:
-                # Repositories don't offer an interface to remove fallback
-                # repositories today; take the conceptually simpler option and just
-                # reopen it.  We reopen it starting from the URL so that we
-                # get a separate connection for RemoteRepositories and can
-                # stream from one of them to the other.  This does mean doing
-                # separate SSH connection setup, but unstacking is not a
-                # common operation so it's tolerable.
-                new_bzrdir = bzrdir.BzrDir.open(self.bzrdir.root_transport.base)
-                new_repository = new_bzrdir.find_repository()
-                self.repository = new_repository
-                if self.repository._fallback_repositories:
-                    raise AssertionError("didn't expect %r to have "
-                        "fallback_repositories"
-                        % (self.repository,))
-                # this is not paired with an unlock because it's just restoring
-                # the previous state; the lock's released when set_stacked_on_url
-                # returns
-                self.repository.lock_write()
                 # XXX: If you unstack a branch while it has a working tree
                 # with a pending merge, the pending-merged revisions will no
                 # longer be present.  You can (probably) revert and remerge.
@@ -951,7 +1002,6 @@
                 raise errors.NoSuchRevision(self, stop_revision)
         return other_history[self_len:stop_revision]
 
-    @needs_write_lock
     def update_revisions(self, other, stop_revision=None, overwrite=False,
                          graph=None):
         """Pull in new perfect-fit revisions.
@@ -1006,7 +1056,6 @@
             self._extend_partial_history(distance_from_last)
         return self._partial_revision_history_cache[distance_from_last]
 
-    @needs_write_lock
     def pull(self, source, overwrite=False, stop_revision=None,
              possible_transports=None, *args, **kwargs):
         """Mirror source into this branch.
@@ -1255,24 +1304,14 @@
                 revno = 1
         destination.set_last_revision_info(revno, revision_id)
 
-    @needs_read_lock
     def copy_content_into(self, destination, revision_id=None):
         """Copy the content of self into destination.
 
         revision_id: if not None, the revision history in the new branch will
                      be truncated to end with revision_id.
         """
-        self.update_references(destination)
-        self._synchronize_history(destination, revision_id)
-        try:
-            parent = self.get_parent()
-        except errors.InaccessibleParent, e:
-            mutter('parent was not accessible to copy: %s', e)
-        else:
-            if parent:
-                destination.set_parent(parent)
-        if self._push_should_merge_tags():
-            self.tags.merge_to(destination.tags)
+        return InterBranch.get(self, destination).copy_content_into(
+            revision_id=revision_id)
 
     def update_references(self, target):
         if not getattr(self._format, 'supports_reference_locations', False):
@@ -1346,6 +1385,8 @@
         """
         # XXX: Fix the bzrdir API to allow getting the branch back from the
         # clone call. Or something. 20090224 RBC/spiv.
+        # XXX: Should this perhaps clone colocated branches as well, 
+        # rather than just the default branch? 20100319 JRV
         if revision_id is None:
             revision_id = self.last_revision()
         dir_to = self.bzrdir.clone_on_transport(to_transport,
@@ -1481,7 +1522,7 @@
      * an open routine.
 
     Formats are placed in an dict by their format string for reference
-    during branch opening. Its not required that these be instances, they
+    during branch opening. It's not required that these be instances, they
     can be classes themselves with class methods - it simply depends on
     whether state is needed for a given format or not.
 
@@ -1510,7 +1551,10 @@
         try:
             transport = a_bzrdir.get_branch_transport(None, name=name)
             format_string = transport.get_bytes("format")
-            return klass._formats[format_string]
+            format = klass._formats[format_string]
+            if isinstance(format, MetaDirBranchFormatFactory):
+                return format()
+            return format
         except errors.NoSuchFile:
             raise errors.NotBranchError(path=transport.base, bzrdir=a_bzrdir)
         except KeyError:
@@ -1521,7 +1565,21 @@
         """Return the current default format."""
         return klass._default_format
 
-    def get_reference(self, a_bzrdir):
+    @classmethod
+    def get_formats(klass):
+        """Get all the known formats.
+
+        Warning: This triggers a load of all lazy registered formats: do not
+        use except when that is desireed.
+        """
+        result = []
+        for fmt in klass._formats.values():
+            if isinstance(fmt, MetaDirBranchFormatFactory):
+                fmt = fmt()
+            result.append(fmt)
+        return result
+
+    def get_reference(self, a_bzrdir, name=None):
         """Get the target reference of the branch in a_bzrdir.
 
         format probing must have been completed before calling
@@ -1529,12 +1587,13 @@
         in a_bzrdir is correct.
 
         :param a_bzrdir: The bzrdir to get the branch data from.
+        :param name: Name of the colocated branch to fetch
         :return: None if the branch is not a reference branch.
         """
         return None
 
     @classmethod
-    def set_reference(self, a_bzrdir, to_branch):
+    def set_reference(self, a_bzrdir, name, to_branch):
         """Set the target reference of the branch in a_bzrdir.
 
         format probing must have been completed before calling
@@ -1542,6 +1601,7 @@
         in a_bzrdir is correct.
 
         :param a_bzrdir: The bzrdir to set the branch reference for.
+        :param name: Name of colocated branch to set, None for default
         :param to_branch: branch that the checkout is to reference
         """
         raise NotImplementedError(self.set_reference)
@@ -1661,11 +1721,19 @@
 
     @classmethod
     def register_format(klass, format):
-        """Register a metadir format."""
+        """Register a metadir format.
+        
+        See MetaDirBranchFormatFactory for the ability to register a format
+        without loading the code the format needs until it is actually used.
+        """
         klass._formats[format.get_format_string()] = format
         # Metadir formats have a network name of their format string, and get
-        # registered as class factories.
-        network_format_registry.register(format.get_format_string(), format.__class__)
+        # registered as factories.
+        if isinstance(format, MetaDirBranchFormatFactory):
+            network_format_registry.register(format.get_format_string(), format)
+        else:
+            network_format_registry.register(format.get_format_string(),
+                format.__class__)
 
     @classmethod
     def set_default_format(klass, format):
@@ -1691,6 +1759,34 @@
         return False  # by default
 
 
+class MetaDirBranchFormatFactory(registry._LazyObjectGetter):
+    """A factory for a BranchFormat object, permitting simple lazy registration.
+    
+    While none of the built in BranchFormats are lazy registered yet,
+    bzrlib.tests.test_branch.TestMetaDirBranchFormatFactory demonstrates how to
+    use it, and the bzr-loom plugin uses it as well (see
+    bzrlib.plugins.loom.formats).
+    """
+
+    def __init__(self, format_string, module_name, member_name):
+        """Create a MetaDirBranchFormatFactory.
+
+        :param format_string: The format string the format has.
+        :param module_name: Module to load the format class from.
+        :param member_name: Attribute name within the module for the format class.
+        """
+        registry._LazyObjectGetter.__init__(self, module_name, member_name)
+        self._format_string = format_string
+        
+    def get_format_string(self):
+        """See BranchFormat.get_format_string."""
+        return self._format_string
+
+    def __call__(self):
+        """Used for network_format_registry support."""
+        return self.get_obj()()
+
+
 class BranchHooks(Hooks):
     """A dictionary mapping hook name to a list of callables for branch hooks.
 
@@ -1723,7 +1819,7 @@
             "with a bzrlib.branch.PullResult object and only runs in the "
             "bzr client.", (0, 15), None))
         self.create_hook(HookPoint('pre_commit',
-            "Called after a commit is calculated but before it is is "
+            "Called after a commit is calculated but before it is "
             "completed. pre_commit is called with (local, master, old_revno, "
             "old_revid, future_revno, future_revid, tree_delta, future_tree"
             "). old_revid is NULL_REVISION for the first commit to a branch, "
@@ -1766,7 +1862,7 @@
             "all are called with the url returned from the previous hook."
             "The order is however undefined.", (1, 9), None))
         self.create_hook(HookPoint('automatic_tag_name',
-            "Called to determine an automatic tag name for a revision."
+            "Called to determine an automatic tag name for a revision. "
             "automatic_tag_name is called with (branch, revision_id) and "
             "should return a tag name or None if no tag name could be "
             "determined. The first non-None tag name returned will be used.",
@@ -1863,12 +1959,7 @@
         return self.__dict__ == other.__dict__
 
     def __repr__(self):
-        if self.branch:
-            return "<%s of %s>" % (self.__class__.__name__, self.branch)
-        else:
-            return "<%s of format:%s bzrdir:%s>" % (
-                self.__class__.__name__, self.branch,
-                self.format, self.bzrdir)
+        return "<%s of %s>" % (self.__class__.__name__, self.branch)
 
 
 class SwitchHookParams(object):
@@ -2157,14 +2248,14 @@
         """See BranchFormat.get_format_description()."""
         return "Checkout reference format 1"
 
-    def get_reference(self, a_bzrdir):
+    def get_reference(self, a_bzrdir, name=None):
         """See BranchFormat.get_reference()."""
-        transport = a_bzrdir.get_branch_transport(None)
+        transport = a_bzrdir.get_branch_transport(None, name=name)
         return transport.get_bytes('location')
 
-    def set_reference(self, a_bzrdir, to_branch):
+    def set_reference(self, a_bzrdir, name, to_branch):
         """See BranchFormat.set_reference()."""
-        transport = a_bzrdir.get_branch_transport(None)
+        transport = a_bzrdir.get_branch_transport(None, name=name)
         location = transport.put_bytes('location', to_branch.base)
 
     def initialize(self, a_bzrdir, name=None, target_branch=None):
@@ -2221,7 +2312,7 @@
                 raise AssertionError("wrong format %r found for %r" %
                     (format, self))
         if location is None:
-            location = self.get_reference(a_bzrdir)
+            location = self.get_reference(a_bzrdir, name)
         real_bzrdir = bzrdir.BzrDir.open(
             location, possible_transports=possible_transports)
         result = real_bzrdir.open_branch(name=name, 
@@ -2265,6 +2356,23 @@
     _legacy_formats[0].network_name(), _legacy_formats[0].__class__)
 
 
+class BranchWriteLockResult(LogicalLockResult):
+    """The result of write locking a branch.
+
+    :ivar branch_token: The token obtained from the underlying branch lock, or
+        None.
+    :ivar unlock: A callable which will unlock the lock.
+    """
+
+    def __init__(self, unlock, branch_token):
+        LogicalLockResult.__init__(self, unlock)
+        self.branch_token = branch_token
+
+    def __repr__(self):
+        return "BranchWriteLockResult(%s, %s)" % (self.branch_token,
+            self.unlock)
+
+
 class BzrBranch(Branch, _RelockDebugMixin):
     """A branch stored in the actual filesystem.
 
@@ -2324,6 +2432,12 @@
         return self.control_files.is_locked()
 
     def lock_write(self, token=None):
+        """Lock the branch for write operations.
+
+        :param token: A token to permit reacquiring a previously held and
+            preserved lock.
+        :return: A BranchWriteLockResult.
+        """
         if not self.is_locked():
             self._note_lock('w')
         # All-in-one needs to always unlock/lock.
@@ -2335,13 +2449,18 @@
         else:
             took_lock = False
         try:
-            return self.control_files.lock_write(token=token)
+            return BranchWriteLockResult(self.unlock,
+                self.control_files.lock_write(token=token))
         except:
             if took_lock:
                 self.repository.unlock()
             raise
 
     def lock_read(self):
+        """Lock the branch for read operations.
+
+        :return: A bzrlib.lock.LogicalLockResult.
+        """
         if not self.is_locked():
             self._note_lock('r')
         # All-in-one needs to always unlock/lock.
@@ -2354,6 +2473,7 @@
             took_lock = False
         try:
             self.control_files.lock_read()
+            return LogicalLockResult(self.unlock)
         except:
             if took_lock:
                 self.repository.unlock()
@@ -2983,8 +3103,12 @@
     :ivar tag_conflicts: A list of tag conflicts, see BasicTags.merge_to
     """
 
+    @deprecated_method(deprecated_in((2, 3, 0)))
     def __int__(self):
-        # DEPRECATED: pull used to return the change in revno
+        """Return the relative change in revno.
+
+        :deprecated: Use `new_revno` and `old_revno` instead.
+        """
         return self.new_revno - self.old_revno
 
     def report(self, to_file):
@@ -3015,8 +3139,12 @@
         target, otherwise it will be None.
     """
 
+    @deprecated_method(deprecated_in((2, 3, 0)))
     def __int__(self):
-        # DEPRECATED: push used to return the change in revno
+        """Return the relative change in revno.
+
+        :deprecated: Use `new_revno` and `old_revno` instead.
+        """
         return self.new_revno - self.old_revno
 
     def report(self, to_file):
@@ -3145,11 +3273,17 @@
     _optimisers = []
     """The available optimised InterBranch types."""
 
-    @staticmethod
-    def _get_branch_formats_to_test():
-        """Return a tuple with the Branch formats to use when testing."""
-        raise NotImplementedError(InterBranch._get_branch_formats_to_test)
+    @classmethod
+    def _get_branch_formats_to_test(klass):
+        """Return an iterable of format tuples for testing.
+        
+        :return: An iterable of (from_format, to_format) to use when testing
+            this InterBranch class. Each InterBranch class should define this
+            method itself.
+        """
+        raise NotImplementedError(klass._get_branch_formats_to_test)
 
+    @needs_write_lock
     def pull(self, overwrite=False, stop_revision=None,
              possible_transports=None, local=False):
         """Mirror source into target branch.
@@ -3160,6 +3294,7 @@
         """
         raise NotImplementedError(self.pull)
 
+    @needs_write_lock
     def update_revisions(self, stop_revision=None, overwrite=False,
                          graph=None):
         """Pull in new perfect-fit revisions.
@@ -3173,6 +3308,7 @@
         """
         raise NotImplementedError(self.update_revisions)
 
+    @needs_write_lock
     def push(self, overwrite=False, stop_revision=None,
              _override_hook_source_branch=None):
         """Mirror the source branch into the target branch.
@@ -3181,112 +3317,124 @@
         """
         raise NotImplementedError(self.push)
 
+    @needs_write_lock
+    def copy_content_into(self, revision_id=None):
+        """Copy the content of source into target
+
+        revision_id: if not None, the revision history in the new branch will
+                     be truncated to end with revision_id.
+        """
+        raise NotImplementedError(self.copy_content_into)
+
 
 class GenericInterBranch(InterBranch):
-    """InterBranch implementation that uses public Branch functions.
-    """
-
-    @staticmethod
-    def _get_branch_formats_to_test():
-        return BranchFormat._default_format, BranchFormat._default_format
-
+    """InterBranch implementation that uses public Branch functions."""
+
+    @classmethod
+    def is_compatible(klass, source, target):
+        # GenericBranch uses the public API, so always compatible
+        return True
+
+    @classmethod
+    def _get_branch_formats_to_test(klass):
+        return [(BranchFormat._default_format, BranchFormat._default_format)]
+
+    @classmethod
+    def unwrap_format(klass, format):
+        if isinstance(format, remote.RemoteBranchFormat):
+            format._ensure_real()
+            return format._custom_format
+        return format                                                                                                  
+
+    @needs_write_lock
+    def copy_content_into(self, revision_id=None):
+        """Copy the content of source into target
+
+        revision_id: if not None, the revision history in the new branch will
+                     be truncated to end with revision_id.
+        """
+        self.source.update_references(self.target)
+        self.source._synchronize_history(self.target, revision_id)
+        try:
+            parent = self.source.get_parent()
+        except errors.InaccessibleParent, e:
+            mutter('parent was not accessible to copy: %s', e)
+        else:
+            if parent:
+                self.target.set_parent(parent)
+        if self.source._push_should_merge_tags():
+            self.source.tags.merge_to(self.target.tags)
+
+    @needs_write_lock
     def update_revisions(self, stop_revision=None, overwrite=False,
         graph=None):
         """See InterBranch.update_revisions()."""
-        self.source.lock_read()
-        try:
-            other_revno, other_last_revision = self.source.last_revision_info()
-            stop_revno = None # unknown
-            if stop_revision is None:
-                stop_revision = other_last_revision
-                if _mod_revision.is_null(stop_revision):
-                    # if there are no commits, we're done.
-                    return
-                stop_revno = other_revno
-
-            # what's the current last revision, before we fetch [and change it
-            # possibly]
-            last_rev = _mod_revision.ensure_null(self.target.last_revision())
-            # we fetch here so that we don't process data twice in the common
-            # case of having something to pull, and so that the check for
-            # already merged can operate on the just fetched graph, which will
-            # be cached in memory.
-            self.target.fetch(self.source, stop_revision)
-            # Check to see if one is an ancestor of the other
-            if not overwrite:
-                if graph is None:
-                    graph = self.target.repository.get_graph()
-                if self.target._check_if_descendant_or_diverged(
-                        stop_revision, last_rev, graph, self.source):
-                    # stop_revision is a descendant of last_rev, but we aren't
-                    # overwriting, so we're done.
-                    return
-            if stop_revno is None:
-                if graph is None:
-                    graph = self.target.repository.get_graph()
-                this_revno, this_last_revision = \
-                        self.target.last_revision_info()
-                stop_revno = graph.find_distance_to_null(stop_revision,
-                                [(other_last_revision, other_revno),
-                                 (this_last_revision, this_revno)])
-            self.target.set_last_revision_info(stop_revno, stop_revision)
-        finally:
-            self.source.unlock()
-
+        other_revno, other_last_revision = self.source.last_revision_info()
+        stop_revno = None # unknown
+        if stop_revision is None:
+            stop_revision = other_last_revision
+            if _mod_revision.is_null(stop_revision):
+                # if there are no commits, we're done.
+                return
+            stop_revno = other_revno
+
+        # what's the current last revision, before we fetch [and change it
+        # possibly]
+        last_rev = _mod_revision.ensure_null(self.target.last_revision())
+        # we fetch here so that we don't process data twice in the common
+        # case of having something to pull, and so that the check for
+        # already merged can operate on the just fetched graph, which will
+        # be cached in memory.
+        self.target.fetch(self.source, stop_revision)
+        # Check to see if one is an ancestor of the other
+        if not overwrite:
+            if graph is None:
+                graph = self.target.repository.get_graph()
+            if self.target._check_if_descendant_or_diverged(
+                    stop_revision, last_rev, graph, self.source):
+                # stop_revision is a descendant of last_rev, but we aren't
+                # overwriting, so we're done.
+                return
+        if stop_revno is None:
+            if graph is None:
+                graph = self.target.repository.get_graph()
+            this_revno, this_last_revision = \
+                    self.target.last_revision_info()
+            stop_revno = graph.find_distance_to_null(stop_revision,
+                            [(other_last_revision, other_revno),
+                             (this_last_revision, this_revno)])
+        self.target.set_last_revision_info(stop_revno, stop_revision)
+
+    @needs_write_lock
     def pull(self, overwrite=False, stop_revision=None,
-             possible_transports=None, _hook_master=None, run_hooks=True,
+             possible_transports=None, run_hooks=True,
              _override_hook_target=None, local=False):
-        """See Branch.pull.
+        """Pull from source into self, updating my master if any.
 
-        :param _hook_master: Private parameter - set the branch to
-            be supplied as the master to pull hooks.
         :param run_hooks: Private parameter - if false, this branch
             is being called because it's the master of the primary branch,
             so it should not run its hooks.
-        :param _override_hook_target: Private parameter - set the branch to be
-            supplied as the target_branch to pull hooks.
-        :param local: Only update the local branch, and not the bound branch.
         """
-        # This type of branch can't be bound.
-        if local:
+        bound_location = self.target.get_bound_location()
+        if local and not bound_location:
             raise errors.LocalRequiresBoundBranch()
-        result = PullResult()
-        result.source_branch = self.source
-        if _override_hook_target is None:
-            result.target_branch = self.target
-        else:
-            result.target_branch = _override_hook_target
-        self.source.lock_read()
+        master_branch = None
+        if not local and bound_location and self.source.user_url != bound_location:
+            # not pulling from master, so we need to update master.
+            master_branch = self.target.get_master_branch(possible_transports)
+            master_branch.lock_write()
         try:
-            # We assume that during 'pull' the target repository is closer than
-            # the source one.
-            self.source.update_references(self.target)
-            graph = self.target.repository.get_graph(self.source.repository)
-            # TODO: Branch formats should have a flag that indicates 
-            # that revno's are expensive, and pull() should honor that flag.
-            # -- JRV20090506
-            result.old_revno, result.old_revid = \
-                self.target.last_revision_info()
-            self.target.update_revisions(self.source, stop_revision,
-                overwrite=overwrite, graph=graph)
-            # TODO: The old revid should be specified when merging tags, 
-            # so a tags implementation that versions tags can only 
-            # pull in the most recent changes. -- JRV20090506
-            result.tag_conflicts = self.source.tags.merge_to(self.target.tags,
-                overwrite)
-            result.new_revno, result.new_revid = self.target.last_revision_info()
-            if _hook_master:
-                result.master_branch = _hook_master
-                result.local_branch = result.target_branch
-            else:
-                result.master_branch = result.target_branch
-                result.local_branch = None
-            if run_hooks:
-                for hook in Branch.hooks['post_pull']:
-                    hook(result)
+            if master_branch:
+                # pull from source into master.
+                master_branch.pull(self.source, overwrite, stop_revision,
+                    run_hooks=False)
+            return self._pull(overwrite,
+                stop_revision, _hook_master=master_branch,
+                run_hooks=run_hooks,
+                _override_hook_target=_override_hook_target)
         finally:
-            self.source.unlock()
-        return result
+            if master_branch:
+                master_branch.unlock()
 
     def push(self, overwrite=False, stop_revision=None,
              _override_hook_source_branch=None):
@@ -3332,7 +3480,7 @@
                 # push into the master from the source branch.
                 self.source._basic_push(master_branch, overwrite, stop_revision)
                 # and push into the target branch from the source. Note that we
-                # push from the source branch again, because its considered the
+                # push from the source branch again, because it's considered the
                 # highest bandwidth repository.
                 result = self.source._basic_push(self.target, overwrite,
                     stop_revision)
@@ -3354,48 +3502,63 @@
             _run_hooks()
             return result
 
-    @classmethod
-    def is_compatible(self, source, target):
-        # GenericBranch uses the public API, so always compatible
-        return True
-
-
-class InterToBranch5(GenericInterBranch):
-
-    @staticmethod
-    def _get_branch_formats_to_test():
-        return BranchFormat._default_format, BzrBranchFormat5()
-
-    def pull(self, overwrite=False, stop_revision=None,
-             possible_transports=None, run_hooks=True,
+    def _pull(self, overwrite=False, stop_revision=None,
+             possible_transports=None, _hook_master=None, run_hooks=True,
              _override_hook_target=None, local=False):
-        """Pull from source into self, updating my master if any.
-
+        """See Branch.pull.
+
+        This function is the core worker, used by GenericInterBranch.pull to
+        avoid duplication when pulling source->master and source->local.
+
+        :param _hook_master: Private parameter - set the branch to
+            be supplied as the master to pull hooks.
         :param run_hooks: Private parameter - if false, this branch
             is being called because it's the master of the primary branch,
             so it should not run its hooks.
+        :param _override_hook_target: Private parameter - set the branch to be
+            supplied as the target_branch to pull hooks.
+        :param local: Only update the local branch, and not the bound branch.
         """
-        bound_location = self.target.get_bound_location()
-        if local and not bound_location:
+        # This type of branch can't be bound.
+        if local:
             raise errors.LocalRequiresBoundBranch()
-        master_branch = None
-        if not local and bound_location and self.source.user_url != bound_location:
-            # not pulling from master, so we need to update master.
-            master_branch = self.target.get_master_branch(possible_transports)
-            master_branch.lock_write()
+        result = PullResult()
+        result.source_branch = self.source
+        if _override_hook_target is None:
+            result.target_branch = self.target
+        else:
+            result.target_branch = _override_hook_target
+        self.source.lock_read()
         try:
-            if master_branch:
-                # pull from source into master.
-                master_branch.pull(self.source, overwrite, stop_revision,
-                    run_hooks=False)
-            return super(InterToBranch5, self).pull(overwrite,
-                stop_revision, _hook_master=master_branch,
-                run_hooks=run_hooks,
-                _override_hook_target=_override_hook_target)
+            # We assume that during 'pull' the target repository is closer than
+            # the source one.
+            self.source.update_references(self.target)
+            graph = self.target.repository.get_graph(self.source.repository)
+            # TODO: Branch formats should have a flag that indicates 
+            # that revno's are expensive, and pull() should honor that flag.
+            # -- JRV20090506
+            result.old_revno, result.old_revid = \
+                self.target.last_revision_info()
+            self.target.update_revisions(self.source, stop_revision,
+                overwrite=overwrite, graph=graph)
+            # TODO: The old revid should be specified when merging tags, 
+            # so a tags implementation that versions tags can only 
+            # pull in the most recent changes. -- JRV20090506
+            result.tag_conflicts = self.source.tags.merge_to(self.target.tags,
+                overwrite)
+            result.new_revno, result.new_revid = self.target.last_revision_info()
+            if _hook_master:
+                result.master_branch = _hook_master
+                result.local_branch = result.target_branch
+            else:
+                result.master_branch = result.target_branch
+                result.local_branch = None
+            if run_hooks:
+                for hook in Branch.hooks['post_pull']:
+                    hook(result)
         finally:
-            if master_branch:
-                master_branch.unlock()
+            self.source.unlock()
+        return result
 
 
 InterBranch.register_optimiser(GenericInterBranch)
-InterBranch.register_optimiser(InterToBranch5)

=== modified file 'bzrlib/breakin.py'
--- bzrlib/breakin.py	2010-02-26 03:00:34 +0000
+++ bzrlib/breakin.py	2010-06-17 09:23:19 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007, 2009 Canonical Ltd
+# Copyright (C) 2007, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -78,7 +78,7 @@
 def hook_debugger_to_signal():
     """Add a signal handler so we drop into the debugger.
 
-    On Linux and Mac, this is hooked into SIGQUIT (C-\\) on Windows, this is
+    On Unix, this is hooked into SIGQUIT (C-\\), and on Windows, this is
     hooked into SIGBREAK (C-Pause).
     """
 

=== modified file 'bzrlib/btree_index.py'
--- bzrlib/btree_index.py	2010-03-25 12:32:24 +0000
+++ bzrlib/btree_index.py	2010-08-06 15:38:24 +0000
@@ -33,9 +33,9 @@
     osutils,
     static_tuple,
     trace,
+    transport,
     )
 from bzrlib.index import _OPTION_NODE_REFS, _OPTION_KEY_ELEMENTS, _OPTION_LEN
-from bzrlib.transport import get_transport
 
 
 _BTSIGNATURE = "B+Tree Graph Index 2\n"
@@ -193,7 +193,8 @@
             new_backing_file, size = self._spill_mem_keys_without_combining()
         # Note: The transport here isn't strictly needed, because we will use
         #       direct access to the new_backing._file object
-        new_backing = BTreeGraphIndex(get_transport('.'), '<temp>', size)
+        new_backing = BTreeGraphIndex(transport.get_transport('.'),
+                                      '<temp>', size)
         # GC will clean up the file
         new_backing._file = new_backing_file
         if self._combine_backing_indices:
@@ -601,10 +602,10 @@
         """In memory index's have no known corruption at the moment."""
 
 
-class _LeafNode(object):
+class _LeafNode(dict):
     """A leaf node for a serialised B+Tree index."""
 
-    __slots__ = ('keys', 'min_key', 'max_key')
+    __slots__ = ('min_key', 'max_key', '_keys')
 
     def __init__(self, bytes, key_length, ref_list_length):
         """Parse bytes to create a leaf node object."""
@@ -616,7 +617,20 @@
             self.max_key = key_list[-1][0]
         else:
             self.min_key = self.max_key = None
-        self.keys = dict(key_list)
+        super(_LeafNode, self).__init__(key_list)
+        self._keys = dict(self)
+
+    def all_items(self):
+        """Return a sorted list of (key, (value, refs)) items"""
+        items = self.items()
+        items.sort()
+        return items
+
+    def all_keys(self):
+        """Return a sorted list of all keys."""
+        keys = self.keys()
+        keys.sort()
+        return keys
 
 
 class _InternalNode(object):
@@ -671,6 +685,7 @@
         self._recommended_pages = self._compute_recommended_pages()
         self._root_node = None
         self._base_offset = offset
+        self._leaf_factory = _LeafNode
         # Default max size is 100,000 leave values
         self._leaf_value_cache = None # lru_cache.LRUCache(100*1000)
         if unlimited_cache:
@@ -949,7 +964,7 @@
         """Cache directly from key => value, skipping the btree."""
         if self._leaf_value_cache is not None:
             for node in nodes.itervalues():
-                for key, value in node.keys.iteritems():
+                for key, value in node.all_items():
                     if key in self._leaf_value_cache:
                         # Don't add the rest of the keys, we've seen this node
                         # before.
@@ -979,10 +994,10 @@
         if self._row_offsets[-1] == 1:
             # There is only the root node, and we read that via key_count()
             if self.node_ref_lists:
-                for key, (value, refs) in sorted(self._root_node.keys.items()):
+                for key, (value, refs) in self._root_node.all_items():
                     yield (self, key, value, refs)
             else:
-                for key, (value, refs) in sorted(self._root_node.keys.items()):
+                for key, (value, refs) in self._root_node.all_items():
                     yield (self, key, value)
             return
         start_of_leaves = self._row_offsets[-2]
@@ -998,11 +1013,11 @@
         # for spilling index builds to disk.
         if self.node_ref_lists:
             for _, node in nodes:
-                for key, (value, refs) in sorted(node.keys.items()):
+                for key, (value, refs) in node.all_items():
                     yield (self, key, value, refs)
         else:
             for _, node in nodes:
-                for key, (value, refs) in sorted(node.keys.items()):
+                for key, (value, refs) in node.all_items():
                     yield (self, key, value)
 
     @staticmethod
@@ -1169,8 +1184,8 @@
                 continue
             node = nodes[node_index]
             for next_sub_key in sub_keys:
-                if next_sub_key in node.keys:
-                    value, refs = node.keys[next_sub_key]
+                if next_sub_key in node:
+                    value, refs = node[next_sub_key]
                     if self.node_ref_lists:
                         yield (self, next_sub_key, value, refs)
                     else:
@@ -1244,14 +1259,13 @@
             # sub_keys is all of the keys we are looking for that should exist
             # on this page, if they aren't here, then they won't be found
             node = nodes[node_index]
-            node_keys = node.keys
             parents_to_check = set()
             for next_sub_key in sub_keys:
-                if next_sub_key not in node_keys:
+                if next_sub_key not in node:
                     # This one is just not present in the index at all
                     missing_keys.add(next_sub_key)
                 else:
-                    value, refs = node_keys[next_sub_key]
+                    value, refs = node[next_sub_key]
                     parent_keys = refs[ref_list_num]
                     parent_map[next_sub_key] = parent_keys
                     parents_to_check.update(parent_keys)
@@ -1264,8 +1278,8 @@
             while parents_to_check:
                 next_parents_to_check = set()
                 for key in parents_to_check:
-                    if key in node_keys:
-                        value, refs = node_keys[key]
+                    if key in node:
+                        value, refs = node[key]
                         parent_keys = refs[ref_list_num]
                         parent_map[key] = parent_keys
                         next_parents_to_check.update(parent_keys)
@@ -1545,7 +1559,8 @@
                     continue
             bytes = zlib.decompress(data)
             if bytes.startswith(_LEAF_FLAG):
-                node = _LeafNode(bytes, self._key_length, self.node_ref_lists)
+                node = self._leaf_factory(bytes, self._key_length,
+                                          self.node_ref_lists)
             elif bytes.startswith(_INTERNAL_FLAG):
                 node = _InternalNode(bytes)
             else:
@@ -1570,8 +1585,11 @@
             pass
 
 
+_gcchk_factory = _LeafNode
+
 try:
     from bzrlib import _btree_serializer_pyx as _btree_serializer
+    _gcchk_factory = _btree_serializer._parse_into_chk
 except ImportError, e:
     osutils.failed_to_load_extension(e)
     from bzrlib import _btree_serializer_py as _btree_serializer

=== modified file 'bzrlib/bugtracker.py'
--- bzrlib/bugtracker.py	2010-04-30 11:03:59 +0000
+++ bzrlib/bugtracker.py	2010-09-25 00:42:22 +0000
@@ -93,7 +93,7 @@
 --fixes`` to mark bugs in that tracker as being fixed by that commit. For
 example::
 
-    bugzilla_squid_url = http://www.squid-cache.org/bugs
+    bugzilla_squid_url = http://bugs.squid-cache.org
 
 would allow ``bzr commit --fixes squid:1234`` to mark Squid's bug 1234 as
 fixed.
@@ -127,7 +127,13 @@
 
     bugtracker_cpan_url = http://rt.cpan.org/Public/Bug/Display.html?id={id}
 
-for CPAN's RT bug tracker.
+would allow ``bzr commit --fixes cpan:1234`` to mark bug 1234 in CPAN's
+RT bug tracker as fixed, or::
+
+    bugtracker_hudson_url = http://issues.hudson-ci.org/browse/{id}
+
+would allow ``bzr commit --fixes hudson:HUDSON-1234`` to mark bug HUDSON-1234
+in Hudson's JIRA bug tracker as fixed.
 """
 
 
@@ -228,14 +234,13 @@
     UniqueIntegerBugTracker('gnome', 'http://bugzilla.gnome.org/show_bug.cgi?id='))
 
 
-class URLParametrizedIntegerBugTracker(IntegerBugTracker):
+class URLParametrizedBugTracker(BugTracker):
     """A type of bug tracker that can be found on a variety of different sites,
     and thus needs to have the base URL configured.
 
     Looks for a config setting in the form '<type_name>_<abbreviation>_url'.
-    `type_name` is the name of the type of tracker (e.g. 'bugzilla' or 'trac')
-    and `abbreviation` is a short name for the particular instance (e.g.
-    'squid' or 'apache').
+    `type_name` is the name of the type of tracker and `abbreviation`
+    is a short name for the particular instance.
     """
 
     def get(self, abbreviation, branch):
@@ -256,6 +261,16 @@
         return urlutils.join(self._base_url, self._bug_area) + str(bug_id)
 
 
+class URLParametrizedIntegerBugTracker(IntegerBugTracker, URLParametrizedBugTracker):
+    """A type of bug tracker that can be found on a variety of different sites,
+    and thus needs to have the base URL configured, but only allows integer bug IDs.
+
+    Looks for a config setting in the form '<type_name>_<abbreviation>_url'.
+    `type_name` is the name of the type of tracker (e.g. 'bugzilla' or 'trac')
+    and `abbreviation` is a short name for the particular instance (e.g.
+    'squid' or 'apache').
+    """
+
 tracker_registry.register(
     'trac', URLParametrizedIntegerBugTracker('trac', 'ticket/'))
 
@@ -264,7 +279,7 @@
     URLParametrizedIntegerBugTracker('bugzilla', 'show_bug.cgi?id='))
 
 
-class GenericBugTracker(URLParametrizedIntegerBugTracker):
+class GenericBugTracker(URLParametrizedBugTracker):
     """Generic bug tracker specified by an URL template."""
 
     def __init__(self):

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2010-05-03 09:07:50 +0000
+++ bzrlib/builtins.py	2010-10-07 09:59:40 +0000
@@ -20,8 +20,9 @@
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
-import codecs
 import cStringIO
+import itertools
+import re
 import sys
 import time
 
@@ -33,7 +34,7 @@
     bzrdir,
     directory_service,
     delta,
-    config,
+    config as _mod_config,
     errors,
     globbing,
     hooks,
@@ -75,14 +76,11 @@
 from bzrlib.trace import mutter, note, warning, is_quiet, get_verbosity_level
 
 
+@symbol_versioning.deprecated_function(symbol_versioning.deprecated_in((2, 3, 0)))
 def tree_files(file_list, default_branch=u'.', canonicalize=True,
     apply_view=True):
-    try:
-        return internal_tree_files(file_list, default_branch, canonicalize,
-            apply_view)
-    except errors.FileInWrongBranch, e:
-        raise errors.BzrCommandError("%s is not in the same branch as %s" %
-                                     (e.path, file_list[0]))
+    return internal_tree_files(file_list, default_branch, canonicalize,
+        apply_view)
 
 
 def tree_files_for_add(file_list):
@@ -152,10 +150,13 @@
 
 # XXX: Bad function name; should possibly also be a class method of
 # WorkingTree rather than a function.
+@symbol_versioning.deprecated_function(symbol_versioning.deprecated_in((2, 3, 0)))
 def internal_tree_files(file_list, default_branch=u'.', canonicalize=True,
     apply_view=True):
     """Convert command-line paths to a WorkingTree and relative paths.
 
+    Deprecated: use WorkingTree.open_containing_paths instead.
+
     This is typically used for command-line processors that take one or
     more filenames, and infer the workingtree that contains them.
 
@@ -171,53 +172,10 @@
 
     :return: workingtree, [relative_paths]
     """
-    if file_list is None or len(file_list) == 0:
-        tree = WorkingTree.open_containing(default_branch)[0]
-        if tree.supports_views() and apply_view:
-            view_files = tree.views.lookup_view()
-            if view_files:
-                file_list = view_files
-                view_str = views.view_display_str(view_files)
-                note("Ignoring files outside view. View is %s" % view_str)
-        return tree, file_list
-    tree = WorkingTree.open_containing(osutils.realpath(file_list[0]))[0]
-    return tree, safe_relpath_files(tree, file_list, canonicalize,
-        apply_view=apply_view)
-
-
-def safe_relpath_files(tree, file_list, canonicalize=True, apply_view=True):
-    """Convert file_list into a list of relpaths in tree.
-
-    :param tree: A tree to operate on.
-    :param file_list: A list of user provided paths or None.
-    :param apply_view: if True and a view is set, apply it or check that
-        specified files are within it
-    :return: A list of relative paths.
-    :raises errors.PathNotChild: When a provided path is in a different tree
-        than tree.
-    """
-    if file_list is None:
-        return None
-    if tree.supports_views() and apply_view:
-        view_files = tree.views.lookup_view()
-    else:
-        view_files = []
-    new_list = []
-    # tree.relpath exists as a "thunk" to osutils, but canonical_relpath
-    # doesn't - fix that up here before we enter the loop.
-    if canonicalize:
-        fixer = lambda p: osutils.canonical_relpath(tree.basedir, p)
-    else:
-        fixer = tree.relpath
-    for filename in file_list:
-        try:
-            relpath = fixer(osutils.dereference_path(filename))
-            if  view_files and not osutils.is_inside_any(view_files, relpath):
-                raise errors.FileOutsideView(filename, view_files)
-            new_list.append(relpath)
-        except errors.PathNotChild:
-            raise errors.FileInWrongBranch(tree.branch, filename)
-    return new_list
+    return WorkingTree.open_containing_paths(
+        file_list, default_directory='.',
+        canonicalize=True,
+        apply_view=True)
 
 
 def _get_view_info_for_change_reporter(tree):
@@ -232,6 +190,14 @@
     return view_info
 
 
+def _open_directory_or_containing_tree_or_branch(filename, directory):
+    """Open the tree or branch containing the specified file, unless
+    the --directory option is used to specify a different branch."""
+    if directory is not None:
+        return (None, Branch.open(directory), filename)
+    return bzrdir.BzrDir.open_containing_tree_or_branch(filename)
+
+
 # TODO: Make sure no commands unconditionally use the working directory as a
 # branch.  If a filename argument is used, the first of them should be used to
 # specify the branch.  (Perhaps this can be factored out into some kind of
@@ -315,7 +281,7 @@
             raise errors.BzrCommandError('bzr status --revision takes exactly'
                                          ' one or two revision specifiers')
 
-        tree, relfile_list = tree_files(file_list)
+        tree, relfile_list = WorkingTree.open_containing_paths(file_list)
         # Avoid asking for specific files when that is not needed.
         if relfile_list == ['']:
             relfile_list = None
@@ -340,7 +306,7 @@
 
     hidden = True
     takes_args = ['revision_id?']
-    takes_options = ['revision']
+    takes_options = ['directory', 'revision']
     # cat-revision is more for frontends so should be exact
     encoding = 'strict'
 
@@ -353,14 +319,14 @@
         self.outf.write(revtext.decode('utf-8'))
 
     @display_command
-    def run(self, revision_id=None, revision=None):
+    def run(self, revision_id=None, revision=None, directory=u'.'):
         if revision_id is not None and revision is not None:
             raise errors.BzrCommandError('You can only supply one of'
                                          ' revision_id or --revision')
         if revision_id is None and revision is None:
             raise errors.BzrCommandError('You must supply either'
                                          ' --revision or a revision_id')
-        b = WorkingTree.open_containing(u'.')[0].branch
+        b = WorkingTree.open_containing(directory)[0].branch
 
         revisions = b.repository.revisions
         if revisions is None:
@@ -483,7 +449,7 @@
     takes_options = [
         Option('force',
                help='Remove the working tree even if it has '
-                    'uncommitted changes.'),
+                    'uncommitted or shelved changes.'),
         ]
 
     def run(self, location_list, force=False):
@@ -503,6 +469,8 @@
             if not force:
                 if (working.has_changes()):
                     raise errors.UncommittedChanges(working)
+                if working.get_shelf_manager().last_shelf() is not None:
+                    raise errors.ShelvedChanges(working)
 
             if working.user_url != working.branch.user_url:
                 raise errors.BzrCommandError("You cannot remove the working tree"
@@ -528,10 +496,9 @@
         if tree:
             try:
                 wt = WorkingTree.open_containing(location)[0]
-                wt.lock_read()
+                self.add_cleanup(wt.lock_read().unlock)
             except (errors.NoWorkingTree, errors.NotLocalUrl):
                 raise errors.NoWorkingTree(location)
-            self.add_cleanup(wt.unlock)
             revid = wt.last_revision()
             try:
                 revno_t = wt.branch.revision_id_to_dotted_revno(revid)
@@ -540,8 +507,7 @@
             revno = ".".join(str(n) for n in revno_t)
         else:
             b = Branch.open_containing(location)[0]
-            b.lock_read()
-            self.add_cleanup(b.unlock)
+            self.add_cleanup(b.lock_read().unlock)
             revno = b.revno()
         self.cleanup_now()
         self.outf.write(str(revno) + '\n')
@@ -554,12 +520,9 @@
     takes_args = ['revision_info*']
     takes_options = [
         'revision',
-        Option('directory',
+        custom_help('directory',
             help='Branch to examine, '
-                 'rather than the one containing the working directory.',
-            short_name='d',
-            type=unicode,
-            ),
+                 'rather than the one containing the working directory.'),
         Option('tree', help='Show revno of working tree'),
         ]
 
@@ -570,13 +533,11 @@
         try:
             wt = WorkingTree.open_containing(directory)[0]
             b = wt.branch
-            wt.lock_read()
-            self.add_cleanup(wt.unlock)
+            self.add_cleanup(wt.lock_read().unlock)
         except (errors.NoWorkingTree, errors.NotLocalUrl):
             wt = None
             b = Branch.open_containing(directory)[0]
-            b.lock_read()
-            self.add_cleanup(b.unlock)
+            self.add_cleanup(b.lock_read().unlock)
         revision_ids = []
         if revision is not None:
             revision_ids.extend(rev.as_revision_id(b) for rev in revision)
@@ -681,8 +642,7 @@
                 should_print=(not is_quiet()))
 
         if base_tree:
-            base_tree.lock_read()
-            self.add_cleanup(base_tree.unlock)
+            self.add_cleanup(base_tree.lock_read().unlock)
         tree, file_list = tree_files_for_add(file_list)
         added, ignored = tree.smart_add(file_list, not
             no_recurse, action=action, save=not dry_run)
@@ -759,15 +719,13 @@
             raise errors.BzrCommandError('invalid kind %r specified' % (kind,))
 
         revision = _get_one_revision('inventory', revision)
-        work_tree, file_list = tree_files(file_list)
-        work_tree.lock_read()
-        self.add_cleanup(work_tree.unlock)
+        work_tree, file_list = WorkingTree.open_containing_paths(file_list)
+        self.add_cleanup(work_tree.lock_read().unlock)
         if revision is not None:
             tree = revision.as_tree(work_tree.branch)
 
             extra_trees = [work_tree]
-            tree.lock_read()
-            self.add_cleanup(tree.unlock)
+            self.add_cleanup(tree.lock_read().unlock)
         else:
             tree = work_tree
             extra_trees = []
@@ -832,9 +790,8 @@
             names_list = []
         if len(names_list) < 2:
             raise errors.BzrCommandError("missing file argument")
-        tree, rel_names = tree_files(names_list, canonicalize=False)
-        tree.lock_tree_write()
-        self.add_cleanup(tree.unlock)
+        tree, rel_names = WorkingTree.open_containing_paths(names_list, canonicalize=False)
+        self.add_cleanup(tree.lock_tree_write().unlock)
         self._run(tree, names_list, rel_names, after)
 
     def run_auto(self, names_list, after, dry_run):
@@ -844,9 +801,9 @@
         if after:
             raise errors.BzrCommandError('--after cannot be specified with'
                                          ' --auto.')
-        work_tree, file_list = tree_files(names_list, default_branch='.')
-        work_tree.lock_tree_write()
-        self.add_cleanup(work_tree.unlock)
+        work_tree, file_list = WorkingTree.open_containing_paths(
+            names_list, default_directory='.')
+        self.add_cleanup(work_tree.lock_tree_write().unlock)
         rename_map.RenameMap.guess_renames(work_tree, dry_run)
 
     def _run(self, tree, names_list, rel_names, after):
@@ -960,24 +917,24 @@
     takes_options = ['remember', 'overwrite', 'revision',
         custom_help('verbose',
             help='Show logs of pulled revisions.'),
-        Option('directory',
+        custom_help('directory',
             help='Branch to pull into, '
-                 'rather than the one containing the working directory.',
-            short_name='d',
-            type=unicode,
-            ),
+                 'rather than the one containing the working directory.'),
         Option('local',
             help="Perform a local pull in a bound "
                  "branch.  Local pulls are not applied to "
                  "the master branch."
             ),
+        Option('show-base',
+            help="Show base revision text in conflicts.")
         ]
     takes_args = ['location?']
     encoding_type = 'replace'
 
     def run(self, location=None, remember=False, overwrite=False,
             revision=None, verbose=False,
-            directory=None, local=False):
+            directory=None, local=False,
+            show_base=False):
         # FIXME: too much stuff is in the command class
         revision_id = None
         mergeable = None
@@ -986,13 +943,14 @@
         try:
             tree_to = WorkingTree.open_containing(directory)[0]
             branch_to = tree_to.branch
-            tree_to.lock_write()
-            self.add_cleanup(tree_to.unlock)
+            self.add_cleanup(tree_to.lock_write().unlock)
         except errors.NoWorkingTree:
             tree_to = None
             branch_to = Branch.open_containing(directory)[0]
-            branch_to.lock_write()
-            self.add_cleanup(branch_to.unlock)
+            self.add_cleanup(branch_to.lock_write().unlock)
+
+        if tree_to is None and show_base:
+            raise errors.BzrCommandError("Need working tree for --show-base.")
 
         if local and not branch_to.get_bound_location():
             raise errors.LocalRequiresBoundBranch()
@@ -1029,8 +987,7 @@
         else:
             branch_from = Branch.open(location,
                 possible_transports=possible_transports)
-            branch_from.lock_read()
-            self.add_cleanup(branch_from.unlock)
+            self.add_cleanup(branch_from.lock_read().unlock)
 
             if branch_to.get_parent() is None or remember:
                 branch_to.set_parent(branch_from.base)
@@ -1045,7 +1002,8 @@
                 view_info=view_info)
             result = tree_to.pull(
                 branch_from, overwrite, revision_id, change_reporter,
-                possible_transports=possible_transports, local=local)
+                possible_transports=possible_transports, local=local,
+                show_base=show_base)
         else:
             result = branch_to.pull(
                 branch_from, overwrite, revision_id, local=local)
@@ -1088,12 +1046,9 @@
         Option('create-prefix',
                help='Create the path leading up to the branch '
                     'if it does not already exist.'),
-        Option('directory',
+        custom_help('directory',
             help='Branch to push from, '
-                 'rather than the one containing the working directory.',
-            short_name='d',
-            type=unicode,
-            ),
+                 'rather than the one containing the working directory.'),
         Option('use-existing-dir',
                help='By default push will fail if the target'
                     ' directory exists, but does not already'
@@ -1188,8 +1143,10 @@
 
     _see_also = ['checkout']
     takes_args = ['from_location', 'to_location?']
-    takes_options = ['revision', Option('hardlink',
-        help='Hard-link working tree files where possible.'),
+    takes_options = ['revision',
+        Option('hardlink', help='Hard-link working tree files where possible.'),
+        Option('files-from', type=str,
+               help="Get file contents from this tree."),
         Option('no-tree',
             help="Create a branch without a working-tree."),
         Option('switch',
@@ -1213,14 +1170,21 @@
 
     def run(self, from_location, to_location=None, revision=None,
             hardlink=False, stacked=False, standalone=False, no_tree=False,
-            use_existing_dir=False, switch=False, bind=False):
+            use_existing_dir=False, switch=False, bind=False,
+            files_from=None):
         from bzrlib import switch as _mod_switch
         from bzrlib.tag import _merge_tags_if_possible
         accelerator_tree, br_from = bzrdir.BzrDir.open_tree_or_branch(
             from_location)
+        if not (hardlink or files_from):
+            # accelerator_tree is usually slower because you have to read N
+            # files (no readahead, lots of seeks, etc), but allow the user to
+            # explicitly request it
+            accelerator_tree = None
+        if files_from is not None and files_from != from_location:
+            accelerator_tree = WorkingTree.open(files_from)
         revision = _get_one_revision('branch', revision)
-        br_from.lock_read()
-        self.add_cleanup(br_from.unlock)
+        self.add_cleanup(br_from.lock_read().unlock)
         if revision is not None:
             revision_id = revision.as_revision_id(br_from)
         else:
@@ -1331,8 +1295,13 @@
             to_location = branch_location
         accelerator_tree, source = bzrdir.BzrDir.open_tree_or_branch(
             branch_location)
+        if not (hardlink or files_from):
+            # accelerator_tree is usually slower because you have to read N
+            # files (no readahead, lots of seeks, etc), but allow the user to
+            # explicitly request it
+            accelerator_tree = None
         revision = _get_one_revision('checkout', revision)
-        if files_from is not None:
+        if files_from is not None and files_from != branch_location:
             accelerator_tree = WorkingTree.open(files_from)
         if revision is not None:
             revision_id = revision.as_revision_id(source)
@@ -1366,12 +1335,10 @@
     @display_command
     def run(self, dir=u'.'):
         tree = WorkingTree.open_containing(dir)[0]
-        tree.lock_read()
-        self.add_cleanup(tree.unlock)
+        self.add_cleanup(tree.lock_read().unlock)
         new_inv = tree.inventory
         old_tree = tree.basis_tree()
-        old_tree.lock_read()
-        self.add_cleanup(old_tree.unlock)
+        self.add_cleanup(old_tree.lock_read().unlock)
         old_inv = old_tree.inventory
         renames = []
         iterator = tree.iter_changes(old_tree, include_unchanged=True)
@@ -1396,16 +1363,22 @@
     If you want to discard your local changes, you can just do a
     'bzr revert' instead of 'bzr commit' after the update.
 
+    If you want to restore a file that has been removed locally, use
+    'bzr revert' instead of 'bzr update'.
+
     If the tree's branch is bound to a master branch, it will also update
     the branch from the master.
     """
 
     _see_also = ['pull', 'working-trees', 'status-flags']
     takes_args = ['dir?']
-    takes_options = ['revision']
+    takes_options = ['revision',
+                     Option('show-base',
+                            help="Show base revision text in conflicts."),
+                     ]
     aliases = ['up']
 
-    def run(self, dir='.', revision=None):
+    def run(self, dir='.', revision=None, show_base=None):
         if revision is not None and len(revision) != 1:
             raise errors.BzrCommandError(
                         "bzr update --revision takes exactly one revision")
@@ -1415,11 +1388,11 @@
         master = branch.get_master_branch(
             possible_transports=possible_transports)
         if master is not None:
-            tree.lock_write()
             branch_location = master.base
+            tree.lock_write()
         else:
+            branch_location = tree.branch.base
             tree.lock_tree_write()
-            branch_location = tree.branch.base
         self.add_cleanup(tree.unlock)
         # get rid of the final '/' and be ready for display
         branch_location = urlutils.unescape_for_display(
@@ -1451,7 +1424,8 @@
                 change_reporter,
                 possible_transports=possible_transports,
                 revision=revision_id,
-                old_tip=old_tip)
+                old_tip=old_tip,
+                show_base=show_base)
         except errors.NoSuchRevision, e:
             raise errors.BzrCommandError(
                                   "branch has no revision %s\n"
@@ -1519,10 +1493,11 @@
 class cmd_remove(Command):
     __doc__ = """Remove files or directories.
 
-    This makes bzr stop tracking changes to the specified files. bzr will delete
-    them if they can easily be recovered using revert. If no options or
-    parameters are given bzr will scan for files that are being tracked by bzr
-    but missing in your tree and stop tracking them for you.
+    This makes Bazaar stop tracking changes to the specified files. Bazaar will
+    delete them if they can easily be recovered using revert otherwise they
+    will be backed up (adding an extention of the form .~#~). If no options or
+    parameters are given Bazaar will scan for files that are being tracked by
+    Bazaar but missing in your tree and stop tracking them for you.
     """
     takes_args = ['file*']
     takes_options = ['verbose',
@@ -1530,23 +1505,28 @@
         RegistryOption.from_kwargs('file-deletion-strategy',
             'The file deletion mode to be used.',
             title='Deletion Strategy', value_switches=True, enum_switch=False,
-            safe='Only delete files if they can be'
-                 ' safely recovered (default).',
+            safe='Backup changed files (default).',
             keep='Delete from bzr but leave the working copy.',
+            no_backup='Don\'t backup changed files.',
             force='Delete all the specified files, even if they can not be '
-                'recovered and even if they are non-empty directories.')]
+                'recovered and even if they are non-empty directories. '
+                '(deprecated, use no-backup)')]
     aliases = ['rm', 'del']
     encoding_type = 'replace'
 
     def run(self, file_list, verbose=False, new=False,
         file_deletion_strategy='safe'):
-        tree, file_list = tree_files(file_list)
+        if file_deletion_strategy == 'force':
+            note("(The --force option is deprecated, rather use --no-backup "
+                "in future.)")
+            file_deletion_strategy = 'no-backup'
+
+        tree, file_list = WorkingTree.open_containing_paths(file_list)
 
         if file_list is not None:
             file_list = [f for f in file_list]
 
-        tree.lock_write()
-        self.add_cleanup(tree.unlock)
+        self.add_cleanup(tree.lock_write().unlock)
         # Heuristics should probably all move into tree.remove_smart or
         # some such?
         if new:
@@ -1567,7 +1547,7 @@
             file_deletion_strategy = 'keep'
         tree.remove(file_list, verbose=verbose, to_file=self.outf,
             keep_files=file_deletion_strategy=='keep',
-            force=file_deletion_strategy=='force')
+            force=(file_deletion_strategy=='no-backup'))
 
 
 class cmd_file_id(Command):
@@ -1635,11 +1615,17 @@
 
     _see_also = ['check']
     takes_args = ['branch?']
+    takes_options = [
+        Option('canonicalize-chks',
+               help='Make sure CHKs are in canonical form (repairs '
+                    'bug 522637).',
+               hidden=True),
+        ]
 
-    def run(self, branch="."):
+    def run(self, branch=".", canonicalize_chks=False):
         from bzrlib.reconcile import reconcile
         dir = bzrdir.BzrDir.open(branch)
-        reconcile(dir)
+        reconcile(dir, canonicalize_chks=canonicalize_chks)
 
 
 class cmd_revision_history(Command):
@@ -1921,6 +1907,10 @@
         Same as 'bzr diff' but prefix paths with old/ and new/::
 
             bzr diff --prefix old/:new/
+            
+        Show the differences using a custom diff program with options::
+        
+            bzr diff --using /usr/bin/diff --diff-options -wu
     """
     _see_also = ['status']
     takes_args = ['file*']
@@ -1985,11 +1975,15 @@
          old_branch, new_branch,
          specific_files, extra_trees) = get_trees_and_branches_to_diff_locked(
             file_list, revision, old, new, self.add_cleanup, apply_view=True)
+        # GNU diff on Windows uses ANSI encoding for filenames
+        path_encoding = osutils.get_diff_header_encoding()
         return show_diff_trees(old_tree, new_tree, sys.stdout,
                                specific_files=specific_files,
                                external_diff_options=diff_options,
                                old_label=old_label, new_label=new_label,
-                               extra_trees=extra_trees, using=using,
+                               extra_trees=extra_trees,
+                               path_encoding=path_encoding,
+                               using=using,
                                format_cls=format)
 
 
@@ -2003,16 +1997,14 @@
     # level of effort but possibly much less IO.  (Or possibly not,
     # if the directories are very large...)
     _see_also = ['status', 'ls']
-    takes_options = ['show-ids']
+    takes_options = ['directory', 'show-ids']
 
     @display_command
-    def run(self, show_ids=False):
-        tree = WorkingTree.open_containing(u'.')[0]
-        tree.lock_read()
-        self.add_cleanup(tree.unlock)
+    def run(self, show_ids=False, directory=u'.'):
+        tree = WorkingTree.open_containing(directory)[0]
+        self.add_cleanup(tree.lock_read().unlock)
         old = tree.basis_tree()
-        old.lock_read()
-        self.add_cleanup(old.unlock)
+        self.add_cleanup(old.lock_read().unlock)
         for path, ie in old.inventory.iter_entries():
             if not tree.has_id(ie.file_id):
                 self.outf.write(path)
@@ -2028,15 +2020,11 @@
 
     hidden = True
     _see_also = ['status', 'ls']
-    takes_options = [
-            Option('null',
-                   help='Write an ascii NUL (\\0) separator '
-                   'between files rather than a newline.')
-            ]
+    takes_options = ['directory', 'null']
 
     @display_command
-    def run(self, null=False):
-        tree = WorkingTree.open_containing(u'.')[0]
+    def run(self, null=False, directory=u'.'):
+        tree = WorkingTree.open_containing(directory)[0]
         td = tree.changes_from(tree.basis_tree())
         for path, id, kind, text_modified, meta_modified in td.modified:
             if null:
@@ -2051,20 +2039,14 @@
 
     hidden = True
     _see_also = ['status', 'ls']
-    takes_options = [
-            Option('null',
-                   help='Write an ascii NUL (\\0) separator '
-                   'between files rather than a newline.')
-            ]
+    takes_options = ['directory', 'null']
 
     @display_command
-    def run(self, null=False):
-        wt = WorkingTree.open_containing(u'.')[0]
-        wt.lock_read()
-        self.add_cleanup(wt.unlock)
+    def run(self, null=False, directory=u'.'):
+        wt = WorkingTree.open_containing(directory)[0]
+        self.add_cleanup(wt.lock_read().unlock)
         basis = wt.basis_tree()
-        basis.lock_read()
-        self.add_cleanup(basis.unlock)
+        self.add_cleanup(basis.lock_read().unlock)
         basis_inv = basis.inventory
         inv = wt.inventory
         for file_id in inv:
@@ -2073,7 +2055,7 @@
             if inv.is_root(file_id) and len(basis_inv) == 0:
                 continue
             path = inv.id2path(file_id)
-            if not os.access(osutils.abspath(path), os.F_OK):
+            if not os.access(osutils.pathjoin(wt.basedir, path), os.F_OK):
                 continue
             if null:
                 self.outf.write(path + '\0')
@@ -2279,6 +2261,11 @@
                    help='Show just the specified revision.'
                    ' See also "help revisionspec".'),
             'log-format',
+            RegistryOption('authors',
+                'What names to list as authors - first, all or committer.',
+                title='Authors',
+                lazy_registry=('bzrlib.log', 'author_list_registry'),
+            ),
             Option('levels',
                    short_name='n',
                    help='Number of levels to display - 0 for all, 1 for flat.',
@@ -2319,6 +2306,7 @@
             limit=None,
             show_diff=False,
             include_merges=False,
+            authors=None,
             exclude_common_ancestry=False,
             ):
         from bzrlib.log import (
@@ -2352,8 +2340,7 @@
         if file_list:
             # find the file ids to log and check for directory filtering
             b, file_info_list, rev1, rev2 = _get_info_for_log_files(
-                revision, file_list)
-            self.add_cleanup(b.unlock)
+                revision, file_list, self.add_cleanup)
             for relpath, file_id, kind in file_info_list:
                 if file_id is None:
                     raise errors.BzrCommandError(
@@ -2377,8 +2364,7 @@
                 location = '.'
             dir, relpath = bzrdir.BzrDir.open_containing(location)
             b = dir.open_branch()
-            b.lock_read()
-            self.add_cleanup(b.unlock)
+            self.add_cleanup(b.lock_read().unlock)
             rev1, rev2 = _get_revision_range(revision, b, self.name())
 
         # Decide on the type of delta & diff filtering to use
@@ -2404,7 +2390,8 @@
                         show_timezone=timezone,
                         delta_format=get_verbosity_level(),
                         levels=levels,
-                        show_advice=levels is None)
+                        show_advice=levels is None,
+                        author_list_handler=authors)
 
         # Choose the algorithm for doing the logging. It's annoying
         # having multiple code paths like this but necessary until
@@ -2508,8 +2495,7 @@
         tree, relpath = WorkingTree.open_containing(filename)
         file_id = tree.path2id(relpath)
         b = tree.branch
-        b.lock_read()
-        self.add_cleanup(b.unlock)
+        self.add_cleanup(b.lock_read().unlock)
         touching_revs = log.find_touching_revisions(b, file_id)
         for revno, revision_id, what in touching_revs:
             self.outf.write("%6d %s\n" % (revno, what))
@@ -2528,23 +2514,24 @@
                    help='Recurse into subdirectories.'),
             Option('from-root',
                    help='Print paths relative to the root of the branch.'),
-            Option('unknown', help='Print unknown files.'),
+            Option('unknown', short_name='u',
+                help='Print unknown files.'),
             Option('versioned', help='Print versioned files.',
                    short_name='V'),
-            Option('ignored', help='Print ignored files.'),
-            Option('null',
-                   help='Write an ascii NUL (\\0) separator '
-                   'between files rather than a newline.'),
-            Option('kind',
+            Option('ignored', short_name='i',
+                help='Print ignored files.'),
+            Option('kind', short_name='k',
                    help='List entries of a particular kind: file, directory, symlink.',
                    type=unicode),
+            'null',
             'show-ids',
+            'directory',
             ]
     @display_command
     def run(self, revision=None, verbose=False,
             recursive=False, from_root=False,
             unknown=False, versioned=False, ignored=False,
-            null=False, kind=None, show_ids=False, path=None):
+            null=False, kind=None, show_ids=False, path=None, directory=None):
 
         if kind and kind not in ('file', 'directory', 'symlink'):
             raise errors.BzrCommandError('invalid kind specified')
@@ -2562,8 +2549,8 @@
                 raise errors.BzrCommandError('cannot specify both --from-root'
                                              ' and PATH')
             fs_path = path
-        tree, branch, relpath = bzrdir.BzrDir.open_containing_tree_or_branch(
-            fs_path)
+        tree, branch, relpath = \
+            _open_directory_or_containing_tree_or_branch(fs_path, directory)
 
         # Calculate the prefix to use
         prefix = None
@@ -2584,8 +2571,7 @@
                 view_str = views.view_display_str(view_files)
                 note("Ignoring files outside view. View is %s" % view_str)
 
-        tree.lock_read()
-        self.add_cleanup(tree.unlock)
+        self.add_cleanup(tree.lock_read().unlock)
         for fp, fc, fkind, fid, entry in tree.list_files(include_root=False,
             from_dir=relpath, recursive=recursive):
             # Apply additional masking
@@ -2638,10 +2624,11 @@
 
     hidden = True
     _see_also = ['ls']
+    takes_options = ['directory']
 
     @display_command
-    def run(self):
-        for f in WorkingTree.open_containing(u'.')[0].unknowns():
+    def run(self, directory=u'.'):
+        for f in WorkingTree.open_containing(directory)[0].unknowns():
             self.outf.write(osutils.quotefn(f) + '\n')
 
 
@@ -2712,12 +2699,13 @@
 
     _see_also = ['status', 'ignored', 'patterns']
     takes_args = ['name_pattern*']
-    takes_options = [
+    takes_options = ['directory',
         Option('default-rules',
                help='Display the default ignore rules that bzr uses.')
         ]
 
-    def run(self, name_pattern_list=None, default_rules=None):
+    def run(self, name_pattern_list=None, default_rules=None,
+            directory=u'.'):
         from bzrlib import ignores
         if default_rules is not None:
             # dump the default rules and exit
@@ -2729,23 +2717,30 @@
                 "NAME_PATTERN or --default-rules.")
         name_pattern_list = [globbing.normalize_pattern(p)
                              for p in name_pattern_list]
+        bad_patterns = ''
+        for p in name_pattern_list:
+            if not globbing.Globster.is_pattern_valid(p):
+                bad_patterns += ('\n  %s' % p)
+        if bad_patterns:
+            msg = ('Invalid ignore pattern(s) found. %s' % bad_patterns)
+            ui.ui_factory.show_error(msg)
+            raise errors.InvalidPattern('')
         for name_pattern in name_pattern_list:
             if (name_pattern[0] == '/' or
                 (len(name_pattern) > 1 and name_pattern[1] == ':')):
                 raise errors.BzrCommandError(
                     "NAME_PATTERN should not be an absolute path")
-        tree, relpath = WorkingTree.open_containing(u'.')
+        tree, relpath = WorkingTree.open_containing(directory)
         ignores.tree_ignores_add_patterns(tree, name_pattern_list)
         ignored = globbing.Globster(name_pattern_list)
         matches = []
-        tree.lock_read()
+        self.add_cleanup(tree.lock_read().unlock)
         for entry in tree.list_files():
             id = entry[3]
             if id is not None:
                 filename = entry[0]
                 if ignored.match(filename):
                     matches.append(filename)
-        tree.unlock()
         if len(matches) > 0:
             self.outf.write("Warning: the following files are version controlled and"
                   " match your ignore pattern:\n%s"
@@ -2766,12 +2761,12 @@
 
     encoding_type = 'replace'
     _see_also = ['ignore', 'ls']
+    takes_options = ['directory']
 
     @display_command
-    def run(self):
-        tree = WorkingTree.open_containing(u'.')[0]
-        tree.lock_read()
-        self.add_cleanup(tree.unlock)
+    def run(self, directory=u'.'):
+        tree = WorkingTree.open_containing(directory)[0]
+        self.add_cleanup(tree.lock_read().unlock)
         for path, file_class, kind, file_id, entry in tree.list_files():
             if file_class != 'I':
                 continue
@@ -2788,15 +2783,16 @@
     """
     hidden = True
     takes_args = ['revno']
+    takes_options = ['directory']
 
     @display_command
-    def run(self, revno):
+    def run(self, revno, directory=u'.'):
         try:
             revno = int(revno)
         except ValueError:
             raise errors.BzrCommandError("not a valid revision-number: %r"
                                          % revno)
-        revid = WorkingTree.open_containing(u'.')[0].branch.get_rev_id(revno)
+        revid = WorkingTree.open_containing(directory)[0].branch.get_rev_id(revno)
         self.outf.write("%s\n" % revid)
 
 
@@ -2829,7 +2825,7 @@
       =================       =========================
     """
     takes_args = ['dest', 'branch_or_subdir?']
-    takes_options = [
+    takes_options = ['directory',
         Option('format',
                help="Type of file to export to.",
                type=unicode),
@@ -2844,11 +2840,11 @@
                     'revision in which it was changed.'),
         ]
     def run(self, dest, branch_or_subdir=None, revision=None, format=None,
-        root=None, filters=False, per_file_timestamps=False):
+        root=None, filters=False, per_file_timestamps=False, directory=u'.'):
         from bzrlib.export import export
 
         if branch_or_subdir is None:
-            tree = WorkingTree.open_containing(u'.')[0]
+            tree = WorkingTree.open_containing(directory)[0]
             b = tree.branch
             subdir = None
         else:
@@ -2873,7 +2869,7 @@
     """
 
     _see_also = ['ls']
-    takes_options = [
+    takes_options = ['directory',
         Option('name-from-revision', help='The path name in the old tree.'),
         Option('filters', help='Apply content filters to display the '
                 'convenience form.'),
@@ -2884,14 +2880,13 @@
 
     @display_command
     def run(self, filename, revision=None, name_from_revision=False,
-            filters=False):
+            filters=False, directory=None):
         if revision is not None and len(revision) != 1:
             raise errors.BzrCommandError("bzr cat --revision takes exactly"
                                          " one revision specifier")
         tree, branch, relpath = \
-            bzrdir.BzrDir.open_containing_tree_or_branch(filename)
-        branch.lock_read()
-        self.add_cleanup(branch.unlock)
+            _open_directory_or_containing_tree_or_branch(filename, directory)
+        self.add_cleanup(branch.lock_read().unlock)
         return self._run(tree, branch, relpath, filename, revision,
                          name_from_revision, filters)
 
@@ -2900,8 +2895,7 @@
         if tree is None:
             tree = b.basis_tree()
         rev_tree = _get_one_revision_tree('cat', revision, branch=b)
-        rev_tree.lock_read()
-        self.add_cleanup(rev_tree.unlock)
+        self.add_cleanup(rev_tree.lock_read().unlock)
 
         old_file_id = rev_tree.path2id(relpath)
 
@@ -3128,7 +3122,7 @@
 
         properties = {}
 
-        tree, selected_list = tree_files(selected_list)
+        tree, selected_list = WorkingTree.open_containing_paths(selected_list)
         if selected_list == ['']:
             # workaround - commit of root of tree should be exactly the same
             # as just default commit in that tree, and succeed even though
@@ -3169,8 +3163,11 @@
         def get_message(commit_obj):
             """Callback to get commit message"""
             if file:
-                my_message = codecs.open(
-                    file, 'rt', osutils.get_user_encoding()).read()
+                f = open(file)
+                try:
+                    my_message = f.read().decode(osutils.get_user_encoding())
+                finally:
+                    f.close()
             elif message is not None:
                 my_message = message
             else:
@@ -3205,7 +3202,7 @@
                         reporter=None, verbose=verbose, revprops=properties,
                         authors=author, timestamp=commit_stamp,
                         timezone=offset,
-                        exclude=safe_relpath_files(tree, exclude))
+                        exclude=tree.safe_relpath_files(exclude))
         except PointlessCommit:
             raise errors.BzrCommandError("No changes to commit."
                               " Use --unchanged to commit anyhow.")
@@ -3330,7 +3327,8 @@
 
             bzr whoami "Frank Chu <fchu@example.com>"
     """
-    takes_options = [ Option('email',
+    takes_options = [ 'directory',
+                      Option('email',
                              help='Display email address only.'),
                       Option('branch',
                              help='Set identity for the current branch instead of '
@@ -3340,13 +3338,16 @@
     encoding_type = 'replace'
 
     @display_command
-    def run(self, email=False, branch=False, name=None):
+    def run(self, email=False, branch=False, name=None, directory=None):
         if name is None:
-            # use branch if we're inside one; otherwise global config
-            try:
-                c = Branch.open_containing('.')[0].get_config()
-            except errors.NotBranchError:
-                c = config.GlobalConfig()
+            if directory is None:
+                # use branch if we're inside one; otherwise global config
+                try:
+                    c = Branch.open_containing(u'.')[0].get_config()
+                except errors.NotBranchError:
+                    c = _mod_config.GlobalConfig()
+            else:
+                c = Branch.open(directory).get_config()
             if email:
                 self.outf.write(c.user_email() + '\n')
             else:
@@ -3355,16 +3356,19 @@
 
         # display a warning if an email address isn't included in the given name.
         try:
-            config.extract_email_address(name)
+            _mod_config.extract_email_address(name)
         except errors.NoEmailInUsername, e:
             warning('"%s" does not seem to contain an email address.  '
                     'This is allowed, but not recommended.', name)
 
         # use global config unless --branch given
         if branch:
-            c = Branch.open_containing('.')[0].get_config()
+            if directory is None:
+                c = Branch.open_containing(u'.')[0].get_config()
+            else:
+                c = Branch.open(directory).get_config()
         else:
-            c = config.GlobalConfig()
+            c = _mod_config.GlobalConfig()
         c.set_user_option('email', name)
 
 
@@ -3380,8 +3384,9 @@
 
     _see_also = ['info']
     takes_args = ['nickname?']
-    def run(self, nickname=None):
-        branch = Branch.open_containing(u'.')[0]
+    takes_options = ['directory']
+    def run(self, nickname=None, directory=u'.'):
+        branch = Branch.open_containing(directory)[0]
         if nickname is None:
             self.printme(branch)
         else:
@@ -3436,13 +3441,13 @@
                 'bzr alias --remove expects an alias to remove.')
         # If alias is not found, print something like:
         # unalias: foo: not found
-        c = config.GlobalConfig()
+        c = _mod_config.GlobalConfig()
         c.unset_alias(alias_name)
 
     @display_command
     def print_aliases(self):
         """Print out the defined aliases in a similar format to bash."""
-        aliases = config.GlobalConfig().get_aliases()
+        aliases = _mod_config.GlobalConfig().get_aliases()
         for key, value in sorted(aliases.iteritems()):
             self.outf.write('bzr alias %s="%s"\n' % (key, value))
 
@@ -3458,7 +3463,7 @@
 
     def set_alias(self, alias_name, alias_command):
         """Save the alias in the global config."""
-        c = config.GlobalConfig()
+        c = _mod_config.GlobalConfig()
         c.set_alias(alias_name, alias_command)
 
 
@@ -3499,6 +3504,9 @@
     If you set BZR_TEST_PDB=1 when running selftest, failing tests will drop
     into a pdb postmortem session.
 
+    The --coverage=DIRNAME global option produces a report with covered code
+    indicated.
+
     :Examples:
         Run only tests relating to 'ignore'::
 
@@ -3537,15 +3545,13 @@
                                  'throughout the test suite.',
                             type=get_transport_type),
                      Option('benchmark',
-                            help='Run the benchmarks rather than selftests.'),
+                            help='Run the benchmarks rather than selftests.',
+                            hidden=True),
                      Option('lsprof-timed',
                             help='Generate lsprof output for benchmarked'
                                  ' sections of code.'),
                      Option('lsprof-tests',
                             help='Generate lsprof output for each test.'),
-                     Option('cache-dir', type=str,
-                            help='Cache intermediate benchmark output in this '
-                                 'directory.'),
                      Option('first',
                             help='Run all tests, but run specified tests first.',
                             short_name='f',
@@ -3585,20 +3591,13 @@
 
     def run(self, testspecs_list=None, verbose=False, one=False,
             transport=None, benchmark=None,
-            lsprof_timed=None, cache_dir=None,
+            lsprof_timed=None,
             first=False, list_only=False,
             randomize=None, exclude=None, strict=False,
             load_list=None, debugflag=None, starting_with=None, subunit=False,
             parallel=None, lsprof_tests=False):
-        from bzrlib.tests import selftest
-        import bzrlib.benchmarks as benchmarks
-        from bzrlib.benchmarks import tree_creator
-
-        # Make deprecation warnings visible, unless -Werror is set
-        symbol_versioning.activate_deprecation_warnings(override=False)
-
-        if cache_dir is not None:
-            tree_creator.TreeCreator.CACHE_ROOT = osutils.abspath(cache_dir)
+        from bzrlib import tests
+
         if testspecs_list is not None:
             pattern = '|'.join(testspecs_list)
         else:
@@ -3612,22 +3611,21 @@
             self.additional_selftest_args['runner_class'] = SubUnitBzrRunner
             # On Windows, disable automatic conversion of '\n' to '\r\n' in
             # stdout, which would corrupt the subunit stream. 
-            if sys.platform == "win32" and sys.stdout.fileno() >= 0:
+            # FIXME: This has been fixed in subunit trunk (>0.0.5) so the
+            # following code can be deleted when it's sufficiently deployed
+            # -- vila/mgz 20100514
+            if (sys.platform == "win32"
+                and getattr(sys.stdout, 'fileno', None) is not None):
                 import msvcrt
                 msvcrt.setmode(sys.stdout.fileno(), os.O_BINARY)
         if parallel:
             self.additional_selftest_args.setdefault(
                 'suite_decorators', []).append(parallel)
         if benchmark:
-            test_suite_factory = benchmarks.test_suite
-            # Unless user explicitly asks for quiet, be verbose in benchmarks
-            verbose = not is_quiet()
-            # TODO: should possibly lock the history file...
-            benchfile = open(".perf_history", "at", buffering=1)
-            self.add_cleanup(benchfile.close)
-        else:
-            test_suite_factory = None
-            benchfile = None
+            raise errors.BzrCommandError(
+                "--benchmark is no longer supported from bzr 2.2; "
+                "use bzr-usertest instead")
+        test_suite_factory = None
         selftest_kwargs = {"verbose": verbose,
                           "pattern": pattern,
                           "stop_on_failure": one,
@@ -3635,7 +3633,6 @@
                           "test_suite_factory": test_suite_factory,
                           "lsprof_timed": lsprof_timed,
                           "lsprof_tests": lsprof_tests,
-                          "bench_history": benchfile,
                           "matching_tests_first": first,
                           "list_only": list_only,
                           "random_seed": randomize,
@@ -3646,7 +3643,14 @@
                           "starting_with": starting_with
                           }
         selftest_kwargs.update(self.additional_selftest_args)
-        result = selftest(**selftest_kwargs)
+
+        # Make deprecation warnings visible, unless -Werror is set
+        cleanup = symbol_versioning.activate_deprecation_warnings(
+            override=False)
+        try:
+            result = tests.selftest(**selftest_kwargs)
+        finally:
+            cleanup()
         return int(not result)
 
 
@@ -3690,10 +3694,8 @@
 
         branch1 = Branch.open_containing(branch)[0]
         branch2 = Branch.open_containing(other)[0]
-        branch1.lock_read()
-        self.add_cleanup(branch1.unlock)
-        branch2.lock_read()
-        self.add_cleanup(branch2.unlock)
+        self.add_cleanup(branch1.lock_read().unlock)
+        self.add_cleanup(branch2.lock_read().unlock)
         last1 = ensure_null(branch1.last_revision())
         last2 = ensure_null(branch2.last_revision())
 
@@ -3793,12 +3795,9 @@
                 ' completely merged into the source, pull from the'
                 ' source rather than merging.  When this happens,'
                 ' you do not need to commit the result.'),
-        Option('directory',
+        custom_help('directory',
                help='Branch to merge into, '
-                    'rather than the one containing the working directory.',
-               short_name='d',
-               type=unicode,
-               ),
+                    'rather than the one containing the working directory.'),
         Option('preview', help='Instead of merging, show a diff of the'
                ' merge.'),
         Option('interactive', help='Select changes interactively.',
@@ -3837,8 +3836,7 @@
             unversioned_filter=tree.is_ignored, view_info=view_info)
         pb = ui.ui_factory.nested_progress_bar()
         self.add_cleanup(pb.finished)
-        tree.lock_write()
-        self.add_cleanup(tree.unlock)
+        self.add_cleanup(tree.lock_write().unlock)
         if location is not None:
             try:
                 mergeable = bundle.read_mergeable_from_url(location,
@@ -3905,8 +3903,10 @@
     def _do_preview(self, merger):
         from bzrlib.diff import show_diff_trees
         result_tree = self._get_preview(merger)
+        path_encoding = osutils.get_diff_header_encoding()
         show_diff_trees(merger.this_tree, result_tree, self.outf,
-                        old_label='', new_label='')
+                        old_label='', new_label='',
+                        path_encoding=path_encoding)
 
     def _do_merge(self, merger, change_reporter, allow_pending, verified):
         merger.change_reporter = change_reporter
@@ -4099,9 +4099,8 @@
         from bzrlib.conflicts import restore
         if merge_type is None:
             merge_type = _mod_merge.Merge3Merger
-        tree, file_list = tree_files(file_list)
-        tree.lock_write()
-        self.add_cleanup(tree.unlock)
+        tree, file_list = WorkingTree.open_containing_paths(file_list)
+        self.add_cleanup(tree.lock_write().unlock)
         parents = tree.get_parent_ids()
         if len(parents) != 2:
             raise errors.BzrCommandError("Sorry, remerge only works after normal"
@@ -4216,9 +4215,8 @@
 
     def run(self, revision=None, no_backup=False, file_list=None,
             forget_merges=None):
-        tree, file_list = tree_files(file_list)
-        tree.lock_tree_write()
-        self.add_cleanup(tree.unlock)
+        tree, file_list = WorkingTree.open_containing_paths(file_list)
+        self.add_cleanup(tree.lock_tree_write().unlock)
         if forget_merges:
             tree.set_parent_ids(tree.get_parent_ids()[:1])
         else:
@@ -4313,6 +4311,7 @@
     _see_also = ['merge', 'pull']
     takes_args = ['other_branch?']
     takes_options = [
+        'directory',
         Option('reverse', 'Reverse the order of revisions.'),
         Option('mine-only',
                'Display changes in the local branch only.'),
@@ -4340,7 +4339,8 @@
             theirs_only=False,
             log_format=None, long=False, short=False, line=False,
             show_ids=False, verbose=False, this=False, other=False,
-            include_merges=False, revision=None, my_revision=None):
+            include_merges=False, revision=None, my_revision=None,
+            directory=u'.'):
         from bzrlib.missing import find_unmerged, iter_log_revisions
         def message(s):
             if not is_quiet():
@@ -4359,9 +4359,8 @@
         elif theirs_only:
             restrict = 'remote'
 
-        local_branch = Branch.open_containing(u".")[0]
-        local_branch.lock_read()
-        self.add_cleanup(local_branch.unlock)
+        local_branch = Branch.open_containing(directory)[0]
+        self.add_cleanup(local_branch.lock_read().unlock)
 
         parent = local_branch.get_parent()
         if other_branch is None:
@@ -4378,8 +4377,7 @@
         if remote_branch.base == local_branch.base:
             remote_branch = local_branch
         else:
-            remote_branch.lock_read()
-            self.add_cleanup(remote_branch.unlock)
+            self.add_cleanup(remote_branch.lock_read().unlock)
 
         local_revid_range = _revision_range_to_revid_range(
             _get_revision_range(my_revision, local_branch,
@@ -4440,8 +4438,7 @@
             message("Branches are up to date.\n")
         self.cleanup_now()
         if not status_code and parent is None and other_branch is not None:
-            local_branch.lock_write()
-            self.add_cleanup(local_branch.unlock)
+            self.add_cleanup(local_branch.lock_write().unlock)
             # handle race conditions - a parent might be set while we run.
             if local_branch.get_parent() is None:
                 local_branch.set_parent(remote_branch.base)
@@ -4547,8 +4544,7 @@
             b = Branch.open_containing(branch)[0]
         else:
             b = Branch.open(branch)
-        b.lock_read()
-        self.add_cleanup(b.unlock)
+        self.add_cleanup(b.lock_read().unlock)
         if revision is None:
             rev_id = b.last_revision()
         else:
@@ -4578,24 +4574,22 @@
                      Option('long', help='Show commit date in annotations.'),
                      'revision',
                      'show-ids',
+                     'directory',
                      ]
     encoding_type = 'exact'
 
     @display_command
     def run(self, filename, all=False, long=False, revision=None,
-            show_ids=False):
+            show_ids=False, directory=None):
         from bzrlib.annotate import annotate_file, annotate_file_tree
         wt, branch, relpath = \
-            bzrdir.BzrDir.open_containing_tree_or_branch(filename)
+            _open_directory_or_containing_tree_or_branch(filename, directory)
         if wt is not None:
-            wt.lock_read()
-            self.add_cleanup(wt.unlock)
+            self.add_cleanup(wt.lock_read().unlock)
         else:
-            branch.lock_read()
-            self.add_cleanup(branch.unlock)
+            self.add_cleanup(branch.lock_read().unlock)
         tree = _get_one_revision_tree('annotate', revision, branch=branch)
-        tree.lock_read()
-        self.add_cleanup(tree.unlock)
+        self.add_cleanup(tree.lock_read().unlock)
         if wt is not None:
             file_id = wt.path2id(relpath)
         else:
@@ -4619,16 +4613,15 @@
 
     hidden = True # is this right ?
     takes_args = ['revision_id*']
-    takes_options = ['revision']
+    takes_options = ['directory', 'revision']
 
-    def run(self, revision_id_list=None, revision=None):
+    def run(self, revision_id_list=None, revision=None, directory=u'.'):
         if revision_id_list is not None and revision is not None:
             raise errors.BzrCommandError('You can only supply one of revision_id or --revision')
         if revision_id_list is None and revision is None:
             raise errors.BzrCommandError('You must supply either --revision or a revision_id')
-        b = WorkingTree.open_containing(u'.')[0].branch
-        b.lock_write()
-        self.add_cleanup(b.unlock)
+        b = WorkingTree.open_containing(directory)[0].branch
+        self.add_cleanup(b.lock_write().unlock)
         return self._run(b, revision_id_list, revision)
 
     def _run(self, b, revision_id_list, revision):
@@ -4693,10 +4686,10 @@
 
     _see_also = ['checkouts', 'unbind']
     takes_args = ['location?']
-    takes_options = []
+    takes_options = ['directory']
 
-    def run(self, location=None):
-        b, relpath = Branch.open_containing(u'.')
+    def run(self, location=None, directory=u'.'):
+        b, relpath = Branch.open_containing(directory)
         if location is None:
             try:
                 location = b.get_old_bound_location()
@@ -4729,10 +4722,10 @@
 
     _see_also = ['checkouts', 'bind']
     takes_args = []
-    takes_options = []
+    takes_options = ['directory']
 
-    def run(self):
-        b, relpath = Branch.open_containing(u'.')
+    def run(self, directory=u'.'):
+        b, relpath = Branch.open_containing(directory)
         if not b.unbind():
             raise errors.BzrCommandError('Local branch is not bound')
 
@@ -4784,11 +4777,9 @@
             b = control.open_branch()
 
         if tree is not None:
-            tree.lock_write()
-            self.add_cleanup(tree.unlock)
+            self.add_cleanup(tree.lock_write().unlock)
         else:
-            b.lock_write()
-            self.add_cleanup(b.unlock)
+            self.add_cleanup(b.lock_write().unlock)
         return self._run(b, tree, dry_run, verbose, revision, force, local=local)
 
     def _run(self, b, tree, dry_run, verbose, revision, force, local=False):
@@ -4833,8 +4824,11 @@
             self.outf.write('The above revision(s) will be removed.\n')
 
         if not force:
-            if not ui.ui_factory.get_boolean('Are you sure'):
-                self.outf.write('Canceled')
+            if not ui.ui_factory.confirm_action(
+                    'Uncommit these revisions',
+                    'bzrlib.builtins.uncommit',
+                    {}):
+                self.outf.write('Canceled\n')
                 return 0
 
         mutter('Uncommitting from {%s} to {%s}',
@@ -4846,7 +4840,10 @@
 
 
 class cmd_break_lock(Command):
-    __doc__ = """Break a dead lock on a repository, branch or working directory.
+    __doc__ = """Break a dead lock.
+
+    This command breaks a lock on a repository, branch, working directory or
+    config file.
 
     CAUTION: Locks should only be broken when you are sure that the process
     holding the lock has been stopped.
@@ -4857,17 +4854,33 @@
     :Examples:
         bzr break-lock
         bzr break-lock bzr+ssh://example.com/bzr/foo
+        bzr break-lock --conf ~/.bazaar
     """
+
     takes_args = ['location?']
+    takes_options = [
+        Option('config',
+               help='LOCATION is the directory where the config lock is.'),
+        Option('force',
+            help='Do not ask for confirmation before breaking the lock.'),
+        ]
 
-    def run(self, location=None, show=False):
+    def run(self, location=None, config=False, force=False):
         if location is None:
             location = u'.'
-        control, relpath = bzrdir.BzrDir.open_containing(location)
-        try:
-            control.break_lock()
-        except NotImplementedError:
-            pass
+        if force:
+            ui.ui_factory = ui.ConfirmationUserInterfacePolicy(ui.ui_factory,
+                None,
+                {'bzrlib.lockdir.break': True})
+        if config:
+            conf = _mod_config.LockableConfig(file_name=location)
+            conf.break_lock()
+        else:
+            control, relpath = bzrdir.BzrDir.open_containing(location)
+            try:
+                control.break_lock()
+            except NotImplementedError:
+                pass
 
 
 class cmd_wait_until_signalled(Command):
@@ -4902,9 +4915,8 @@
                     'result in a dynamically allocated port.  The default port '
                     'depends on the protocol.',
                type=str),
-        Option('directory',
-               help='Serve contents of this directory.',
-               type=unicode),
+        custom_help('directory',
+               help='Serve contents of this directory.'),
         Option('allow-writes',
                help='By default the server is a readonly server.  Supplying '
                     '--allow-writes enables write access to the contents of '
@@ -4937,17 +4949,17 @@
 
     def run(self, port=None, inet=False, directory=None, allow_writes=False,
             protocol=None):
-        from bzrlib.transport import get_transport, transport_server_registry
+        from bzrlib import transport
         if directory is None:
             directory = os.getcwd()
         if protocol is None:
-            protocol = transport_server_registry.get()
+            protocol = transport.transport_server_registry.get()
         host, port = self.get_host_and_port(port)
         url = urlutils.local_path_to_url(directory)
         if not allow_writes:
             url = 'readonly+' + url
-        transport = get_transport(url)
-        protocol(transport, host, port, inet)
+        t = transport.get_transport(url)
+        protocol(t, host, port, inet)
 
 
 class cmd_join(Command):
@@ -4959,7 +4971,7 @@
     not part of it.  (Such trees can be produced by "bzr split", but also by
     running "bzr branch" with the target inside a tree.)
 
-    The result is a combined tree, with the subtree no longer an independant
+    The result is a combined tree, with the subtree no longer an independent
     part.  This is marked as a merge of the subtree into the containing tree,
     and all history is preserved.
     """
@@ -5046,6 +5058,7 @@
     _see_also = ['send']
 
     takes_options = [
+        'directory',
         RegistryOption.from_kwargs('patch-type',
             'The type of patch to include in the directive.',
             title='Patch type',
@@ -5064,14 +5077,15 @@
     encoding_type = 'exact'
 
     def run(self, submit_branch=None, public_branch=None, patch_type='bundle',
-            sign=False, revision=None, mail_to=None, message=None):
+            sign=False, revision=None, mail_to=None, message=None,
+            directory=u'.'):
         from bzrlib.revision import ensure_null, NULL_REVISION
         include_patch, include_bundle = {
             'plain': (False, False),
             'diff': (True, False),
             'bundle': (True, True),
             }[patch_type]
-        branch = Branch.open('.')
+        branch = Branch.open(directory)
         stored_submit_branch = branch.get_submit_branch()
         if submit_branch is None:
             submit_branch = stored_submit_branch
@@ -5162,7 +5176,7 @@
     given, in which case it is sent to a file.
 
     Mail is sent using your preferred mail program.  This should be transparent
-    on Windows (it uses MAPI).  On Linux, it requires the xdg-email utility.
+    on Windows (it uses MAPI).  On Unix, it requires the xdg-email utility.
     If the preferred client can't be found (or used), your editor will be used.
 
     To use a specific mail program, set the mail_client configuration option.
@@ -5339,11 +5353,8 @@
         Option('delete',
             help='Delete this tag rather than placing it.',
             ),
-        Option('directory',
-            help='Branch in which to place the tag.',
-            short_name='d',
-            type=unicode,
-            ),
+        custom_help('directory',
+            help='Branch in which to place the tag.'),
         Option('force',
             help='Replace existing tags.',
             ),
@@ -5357,8 +5368,7 @@
             revision=None,
             ):
         branch, relpath = Branch.open_containing(directory)
-        branch.lock_write()
-        self.add_cleanup(branch.unlock)
+        self.add_cleanup(branch.lock_write().unlock)
         if delete:
             if tag_name is None:
                 raise errors.BzrCommandError("No tag specified to delete.")
@@ -5392,14 +5402,13 @@
 
     _see_also = ['tag']
     takes_options = [
-        Option('directory',
-            help='Branch whose tags should be displayed.',
-            short_name='d',
-            type=unicode,
-            ),
+        custom_help('directory',
+            help='Branch whose tags should be displayed.'),
         RegistryOption.from_kwargs('sort',
             'Sort tags by different criteria.', title='Sorting',
-            alpha='Sort tags lexicographically (default).',
+            natural='Sort numeric substrings as numbers:'
+                    ' suitable for version numbers. (default)',
+            alpha='Sort tags lexicographically.',
             time='Sort tags chronologically.',
             ),
         'show-ids',
@@ -5409,7 +5418,7 @@
     @display_command
     def run(self,
             directory='.',
-            sort='alpha',
+            sort='natural',
             show_ids=False,
             revision=None,
             ):
@@ -5419,8 +5428,7 @@
         if not tags:
             return
 
-        branch.lock_read()
-        self.add_cleanup(branch.unlock)
+        self.add_cleanup(branch.lock_read().unlock)
         if revision:
             graph = branch.repository.get_graph()
             rev1, rev2 = _get_revision_range(revision, branch, self.name())
@@ -5428,7 +5436,13 @@
             # only show revisions between revid1 and revid2 (inclusive)
             tags = [(tag, revid) for tag, revid in tags if
                 graph.is_between(revid, revid1, revid2)]
-        if sort == 'alpha':
+        if sort == 'natural':
+            def natural_sort_key(tag):
+                return [f(s) for f,s in 
+                        zip(itertools.cycle((unicode.lower,int)),
+                                            re.split('([0-9]+)', tag[0]))]
+            tags.sort(key=natural_sort_key)
+        elif sort == 'alpha':
             tags.sort()
         elif sort == 'time':
             timestamps = {}
@@ -5573,7 +5587,8 @@
     """
 
     takes_args = ['to_location?']
-    takes_options = [Option('force',
+    takes_options = ['directory',
+                     Option('force',
                         help='Switch even if local commits will be lost.'),
                      'revision',
                      Option('create-branch', short_name='b',
@@ -5582,16 +5597,16 @@
                     ]
 
     def run(self, to_location=None, force=False, create_branch=False,
-            revision=None):
+            revision=None, directory=u'.'):
         from bzrlib import switch
-        tree_location = '.'
+        tree_location = directory
         revision = _get_one_revision('switch', revision)
         control_dir = bzrdir.BzrDir.open_containing(tree_location)[0]
         if to_location is None:
             if revision is None:
                 raise errors.BzrCommandError('You must supply either a'
                                              ' revision or a location')
-            to_location = '.'
+            to_location = tree_location
         try:
             branch = control_dir.open_branch()
             had_explicit_nick = branch.get_config().has_explicit_nickname()
@@ -5732,7 +5747,8 @@
             name=None,
             switch=None,
             ):
-        tree, file_list = tree_files(file_list, apply_view=False)
+        tree, file_list = WorkingTree.open_containing_paths(file_list,
+            apply_view=False)
         current_view, view_dict = tree.views.get_view_info()
         if name is None:
             name = current_view
@@ -5872,6 +5888,7 @@
     takes_args = ['file*']
 
     takes_options = [
+        'directory',
         'revision',
         Option('all', help='Shelve all changes.'),
         'message',
@@ -5886,7 +5903,7 @@
     _see_also = ['unshelve']
 
     def run(self, revision=None, all=False, file_list=None, message=None,
-            writer=None, list=False, destroy=False):
+            writer=None, list=False, destroy=False, directory=u'.'):
         if list:
             return self.run_for_list()
         from bzrlib.shelf_ui import Shelver
@@ -5894,7 +5911,7 @@
             writer = bzrlib.option.diff_writer_registry.get()
         try:
             shelver = Shelver.from_args(writer(sys.stdout), revision, all,
-                file_list, message, destroy=destroy)
+                file_list, message, destroy=destroy, directory=directory)
             try:
                 shelver.run()
             finally:
@@ -5904,8 +5921,7 @@
 
     def run_for_list(self):
         tree = WorkingTree.open_containing('.')[0]
-        tree.lock_read()
-        self.add_cleanup(tree.unlock)
+        self.add_cleanup(tree.lock_read().unlock)
         manager = tree.get_shelf_manager()
         shelves = manager.active_shelves()
         if len(shelves) == 0:
@@ -5929,6 +5945,7 @@
 
     takes_args = ['shelf_id?']
     takes_options = [
+        'directory',
         RegistryOption.from_kwargs(
             'action', help="The action to perform.",
             enum_switch=False, value_switches=True,
@@ -5942,9 +5959,9 @@
     ]
     _see_also = ['shelve']
 
-    def run(self, shelf_id=None, action='apply'):
+    def run(self, shelf_id=None, action='apply', directory=u'.'):
         from bzrlib.shelf_ui import Unshelver
-        unshelver = Unshelver.from_args(shelf_id, action)
+        unshelver = Unshelver.from_args(shelf_id, action, directory=directory)
         try:
             unshelver.run()
         finally:
@@ -5966,7 +5983,8 @@
 
     To check what clean-tree will do, use --dry-run.
     """
-    takes_options = [Option('ignored', help='Delete all ignored files.'),
+    takes_options = ['directory',
+                     Option('ignored', help='Delete all ignored files.'),
                      Option('detritus', help='Delete conflict files, merge'
                             ' backups, and failed selftest dirs.'),
                      Option('unknown',
@@ -5975,14 +5993,14 @@
                             ' deleting them.'),
                      Option('force', help='Do not prompt before deleting.')]
     def run(self, unknown=False, ignored=False, detritus=False, dry_run=False,
-            force=False):
+            force=False, directory=u'.'):
         from bzrlib.clean_tree import clean_tree
         if not (unknown or ignored or detritus):
             unknown = True
         if dry_run:
             force = True
-        clean_tree('.', unknown=unknown, ignored=ignored, detritus=detritus,
-                   dry_run=dry_run, no_prompt=force)
+        clean_tree(directory, unknown=unknown, ignored=ignored,
+                   detritus=detritus, dry_run=dry_run, no_prompt=force)
 
 
 class cmd_reference(Command):

=== modified file 'bzrlib/bundle/__init__.py'
--- bzrlib/bundle/__init__.py	2010-02-17 17:11:16 +0000
+++ bzrlib/bundle/__init__.py	2010-07-15 12:53:44 +0000
@@ -16,7 +16,6 @@
 
 from cStringIO import StringIO
 
-from bzrlib.symbol_versioning import deprecated_function, deprecated_in
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 from bzrlib import (
@@ -33,11 +32,6 @@
 from bzrlib.trace import note
 
 
-@deprecated_function(deprecated_in((1, 12, 0)))
-def read_bundle_from_url(url):
-    return read_mergeable_from_url(url, _do_directive=False)
-
-
 def read_mergeable_from_url(url, _do_directive=True, possible_transports=None):
     """Read mergable object from a given URL.
 

=== modified file 'bzrlib/bundle/bundle_data.py'
--- bzrlib/bundle/bundle_data.py	2009-05-06 05:36:28 +0000
+++ bzrlib/bundle/bundle_data.py	2010-09-28 18:51:47 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -278,14 +278,18 @@
         if rev.revision_id != revision_id:
             raise AssertionError()
         if sha1 != rev.inventory_sha1:
-            open(',,bogus-inv', 'wb').write(s)
+            f = open(',,bogus-inv', 'wb')
+            try:
+                f.write(s)
+            finally:
+                f.close()
             warning('Inventory sha hash mismatch for revision %s. %s'
                     ' != %s' % (revision_id, sha1, rev.inventory_sha1))
 
     def _validate_revision(self, inventory, revision_id):
         """Make sure all revision entries match their checksum."""
 
-        # This is a mapping from each revision id to it's sha hash
+        # This is a mapping from each revision id to its sha hash
         rev_to_sha1 = {}
 
         rev = self.get_revision(revision_id)
@@ -327,7 +331,7 @@
                 try:
                     name, value = info_item.split(':', 1)
                 except ValueError:
-                    raise 'Value %r has no colon' % info_item
+                    raise ValueError('Value %r has no colon' % info_item)
                 if name == 'last-changed':
                     last_changed = value
                 elif name == 'executable':
@@ -711,12 +715,11 @@
                 ie.symlink_target = self.get_symlink_target(file_id)
             ie.revision = revision_id
 
-            if kind in ('directory', 'symlink'):
-                ie.text_size, ie.text_sha1 = None, None
-            else:
+            if kind == 'file':
                 ie.text_size, ie.text_sha1 = self.get_size_and_sha1(file_id)
-            if (ie.text_size is None) and (kind == 'file'):
-                raise BzrError('Got a text_size of None for file_id %r' % file_id)
+                if ie.text_size is None:
+                    raise BzrError(
+                        'Got a text_size of None for file_id %r' % file_id)
             inv.add(ie)
 
         sorted_entries = self.sorted_path_id()

=== modified file 'bzrlib/bundle/serializer/__init__.py'
--- bzrlib/bundle/serializer/__init__.py	2009-03-23 14:59:43 +0000
+++ bzrlib/bundle/serializer/__init__.py	2010-07-15 12:53:00 +0000
@@ -151,17 +151,6 @@
         """
         raise NotImplementedError
 
-    def write(self, source, revision_ids, forced_bases, f):
-        """Write the bundle to the supplied file.
-
-        DEPRECATED: see write_bundle
-        :param source: A source for revision information
-        :param revision_ids: The list of revision ids to serialize
-        :param forced_bases: A dict of revision -> base that overrides default
-        :param f: The file to output to
-        """
-        raise NotImplementedError
-
     def _write_bundle(self, repository, revision_id, base_revision_id, out):
         """Helper function for translating write_bundle to write"""
         forced_bases = {revision_id:base_revision_id}

=== modified file 'bzrlib/bundle/serializer/v4.py'
--- bzrlib/bundle/serializer/v4.py	2009-08-04 14:08:32 +0000
+++ bzrlib/bundle/serializer/v4.py	2010-08-10 20:03:44 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -30,11 +30,49 @@
     serializer,
     trace,
     ui,
+    versionedfile as _mod_versionedfile,
     )
 from bzrlib.bundle import bundle_data, serializer as bundle_serializer
 from bzrlib import bencode
 
 
+class _MPDiffInventoryGenerator(_mod_versionedfile._MPDiffGenerator):
+    """Generate Inventory diffs serialized inventories."""
+
+    def __init__(self, repo, inventory_keys):
+        super(_MPDiffInventoryGenerator, self).__init__(repo.inventories,
+            inventory_keys)
+        self.repo = repo
+        self.sha1s = {}
+
+    def iter_diffs(self):
+        """Compute the diffs one at a time."""
+        # This is instead of compute_diffs() since we guarantee our ordering of
+        # inventories, we don't have to do any buffering
+        self._find_needed_keys()
+        # We actually use a slightly different ordering. We grab all of the
+        # parents first, and then grab the ordered requests.
+        needed_ids = [k[-1] for k in self.present_parents]
+        needed_ids.extend([k[-1] for k in self.ordered_keys])
+        inv_to_str = self.repo._serializer.write_inventory_to_string
+        for inv in self.repo.iter_inventories(needed_ids):
+            revision_id = inv.revision_id
+            key = (revision_id,)
+            if key in self.present_parents:
+                # Not a key we will transmit, which is a shame, since because
+                # of that bundles don't work with stacked branches
+                parent_ids = None
+            else:
+                parent_ids = [k[-1] for k in self.parent_map[key]]
+            as_bytes = inv_to_str(inv)
+            self._process_one_record(key, (as_bytes,))
+            if parent_ids is None:
+                continue
+            diff = self.diffs.pop(key)
+            sha1 = osutils.sha_string(as_bytes)
+            yield revision_id, parent_ids, sha1, diff
+
+
 class BundleWriter(object):
     """Writer for bundle-format files.
 
@@ -348,48 +386,10 @@
         the other side.
         """
         inventory_key_order = [(r,) for r in revision_order]
-        parent_map = self.repository.inventories.get_parent_map(
-                            inventory_key_order)
-        missing_keys = set(inventory_key_order).difference(parent_map)
-        if missing_keys:
-            raise errors.RevisionNotPresent(list(missing_keys)[0],
-                                            self.repository.inventories)
-        inv_to_str = self.repository._serializer.write_inventory_to_string
-        # Make sure that we grab the parent texts first
-        just_parents = set()
-        map(just_parents.update, parent_map.itervalues())
-        just_parents.difference_update(parent_map)
-        # Ignore ghost parents
-        present_parents = self.repository.inventories.get_parent_map(
-                            just_parents)
-        ghost_keys = just_parents.difference(present_parents)
-        needed_inventories = list(present_parents) + inventory_key_order
-        needed_inventories = [k[-1] for k in needed_inventories]
-        all_lines = {}
-        for inv in self.repository.iter_inventories(needed_inventories):
-            revision_id = inv.revision_id
-            key = (revision_id,)
-            as_bytes = inv_to_str(inv)
-            # The sha1 is validated as the xml/textual form, not as the
-            # form-in-the-repository
-            sha1 = osutils.sha_string(as_bytes)
-            as_lines = osutils.split_lines(as_bytes)
-            del as_bytes
-            all_lines[key] = as_lines
-            if key in just_parents:
-                # We don't transmit those entries
-                continue
-            # Create an mpdiff for this text, and add it to the output
-            parent_keys = parent_map[key]
-            # See the comment in VF.make_mpdiffs about how this effects
-            # ordering when there are ghosts present. I think we have a latent
-            # bug
-            parent_lines = [all_lines[p_key] for p_key in parent_keys
-                            if p_key not in ghost_keys]
-            diff = multiparent.MultiParent.from_lines(
-                as_lines, parent_lines)
+        generator = _MPDiffInventoryGenerator(self.repository,
+                                              inventory_key_order)
+        for revision_id, parent_ids, sha1, diff in generator.iter_diffs():
             text = ''.join(diff.to_patch())
-            parent_ids = [k[-1] for k in parent_keys]
             self.bundle.add_multiparent_record(text, sha1, parent_ids,
                                                'inventory', revision_id, None)
 

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2010-04-28 07:03:38 +0000
+++ bzrlib/bzrdir.py	2010-09-10 09:46:15 +0000
@@ -34,12 +34,12 @@
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 from stat import S_ISDIR
-import textwrap
 
 import bzrlib
 from bzrlib import (
     branch,
     config,
+    controldir,
     errors,
     graph,
     lockable_files,
@@ -86,42 +86,9 @@
     registry,
     symbol_versioning,
     )
-    
-    
-class ControlComponent(object):
-    """Abstract base class for control directory components.
-    
-    This provides interfaces that are common across bzrdirs, 
-    repositories, branches, and workingtree control directories.
-    
-    They all expose two urls and transports: the *user* URL is the 
-    one that stops above the control directory (eg .bzr) and that 
-    should normally be used in messages, and the *control* URL is
-    under that in eg .bzr/checkout and is used to read the control
-    files.
-    
-    This can be used as a mixin and is intended to fit with 
-    foreign formats.
-    """
-    
-    @property
-    def control_transport(self):
-        raise NotImplementedError
-   
-    @property
-    def control_url(self):
-        return self.control_transport.base
-    
-    @property
-    def user_transport(self):
-        raise NotImplementedError
-        
-    @property
-    def user_url(self):
-        return self.user_transport.base
-    
-
-class BzrDir(ControlComponent):
+
+
+class BzrDir(controldir.ControlDir):
     """A .bzr control diretory.
 
     BzrDir instances let you create or open any of the things that can be
@@ -158,10 +125,6 @@
                     return
         thing_to_unlock.break_lock()
 
-    def can_convert_format(self):
-        """Return true if this bzrdir is one whose format we can convert from."""
-        return True
-
     def check_conversion_target(self, target_format):
         """Check that a bzrdir as a whole can be converted to a new format."""
         # The only current restriction is that the repository content can be 
@@ -202,25 +165,6 @@
                 format.get_format_description(),
                 basedir)
 
-    def clone(self, url, revision_id=None, force_new_repo=False,
-              preserve_stacking=False):
-        """Clone this bzrdir and its contents to url verbatim.
-
-        :param url: The url create the clone at.  If url's last component does
-            not exist, it will be created.
-        :param revision_id: The tip revision-id to use for any branch or
-            working tree.  If not None, then the clone operation may tune
-            itself to download less data.
-        :param force_new_repo: Do not use a shared repository for the target
-                               even if one is available.
-        :param preserve_stacking: When cloning a stacked branch, stack the
-            new branch on top of the other branch's stacked-on branch.
-        """
-        return self.clone_on_transport(get_transport(url),
-                                       revision_id=revision_id,
-                                       force_new_repo=force_new_repo,
-                                       preserve_stacking=preserve_stacking)
-
     def clone_on_transport(self, transport, revision_id=None,
         force_new_repo=False, preserve_stacking=False, stacked_on=None,
         create_prefix=False, use_existing_dir=True):
@@ -241,12 +185,12 @@
         """
         # Overview: put together a broad description of what we want to end up
         # with; then make as few api calls as possible to do it.
-        
+
         # We may want to create a repo/branch/tree, if we do so what format
         # would we want for each:
         require_stacking = (stacked_on is not None)
         format = self.cloning_metadir(require_stacking)
-        
+
         # Figure out what objects we want:
         try:
             local_repo = self.find_repository()
@@ -329,24 +273,6 @@
         t = get_transport(url)
         t.ensure_base()
 
-    @classmethod
-    def create(cls, base, format=None, possible_transports=None):
-        """Create a new BzrDir at the url 'base'.
-
-        :param format: If supplied, the format of branch to create.  If not
-            supplied, the default is used.
-        :param possible_transports: If supplied, a list of transports that
-            can be reused to share a remote connection.
-        """
-        if cls is not BzrDir:
-            raise AssertionError("BzrDir.create always creates the default"
-                " format, not one of %r" % cls)
-        t = get_transport(base, possible_transports)
-        t.ensure_base()
-        if format is None:
-            format = BzrDirFormat.get_default_format()
-        return format.initialize_on_transport(t)
-
     @staticmethod
     def find_bzrdirs(transport, evaluate=None, list_current=None):
         """Find bzrdirs recursively from current location.
@@ -375,28 +301,19 @@
             recurse = True
             try:
                 bzrdir = BzrDir.open_from_transport(current_transport)
-            except errors.NotBranchError:
+            except (errors.NotBranchError, errors.PermissionDenied):
                 pass
             else:
                 recurse, value = evaluate(bzrdir)
                 yield value
             try:
                 subdirs = list_current(current_transport)
-            except errors.NoSuchFile:
+            except (errors.NoSuchFile, errors.PermissionDenied):
                 continue
             if recurse:
                 for subdir in sorted(subdirs, reverse=True):
                     pending.append(current_transport.clone(subdir))
 
-    def list_branches(self):
-        """Return a sequence of all branches local to this control directory.
-
-        """
-        try:
-            return [self.open_branch()]
-        except errors.NotBranchError:
-            return []
-
     @staticmethod
     def find_branches(transport):
         """Find all branches under a transport.
@@ -425,29 +342,6 @@
                 ret.extend(branches)
         return ret
 
-    def destroy_repository(self):
-        """Destroy the repository in this BzrDir"""
-        raise NotImplementedError(self.destroy_repository)
-
-    def create_branch(self, name=None):
-        """Create a branch in this BzrDir.
-
-        :param name: Name of the colocated branch to create, None for
-            the default branch.
-
-        The bzrdir's format will control what branch format is created.
-        For more control see BranchFormatXX.create(a_bzrdir).
-        """
-        raise NotImplementedError(self.create_branch)
-
-    def destroy_branch(self, name=None):
-        """Destroy a branch in this BzrDir.
-        
-        :param name: Name of the branch to destroy, None for the default 
-            branch.
-        """
-        raise NotImplementedError(self.destroy_branch)
-
     @staticmethod
     def create_branch_and_repo(base, force_new_repo=False, format=None):
         """Create a new BzrDir, Branch and Repository at the url 'base'.
@@ -602,33 +496,22 @@
                                                format=format).bzrdir
         return bzrdir.create_workingtree()
 
-    def create_workingtree(self, revision_id=None, from_branch=None,
-        accelerator_tree=None, hardlink=False):
-        """Create a working tree at this BzrDir.
-
-        :param revision_id: create it as of this revision id.
-        :param from_branch: override bzrdir branch (for lightweight checkouts)
-        :param accelerator_tree: A tree which can be used for retrieving file
-            contents more quickly than the revision tree, i.e. a workingtree.
-            The revision tree will be used for cases where accelerator_tree's
-            content is different.
-        """
-        raise NotImplementedError(self.create_workingtree)
+    def generate_backup_name(self, base):
+        """Generate a non-existing backup file name based on base."""
+        counter = 1
+        name = "%s.~%d~" % (base, counter)
+        while self.root_transport.has(name):
+            counter += 1
+            name = "%s.~%d~" % (base, counter)
+        return name
 
     def backup_bzrdir(self):
         """Backup this bzr control directory.
 
         :return: Tuple with old path name and new path name
         """
-        def name_gen(base='backup.bzr'):
-            counter = 1
-            name = "%s.~%d~" % (base, counter)
-            while self.root_transport.has(name):
-                counter += 1
-                name = "%s.~%d~" % (base, counter)
-            return name
 
-        backup_dir=name_gen()
+        backup_dir=self.generate_backup_name('backup.bzr')
         pb = ui.ui_factory.nested_progress_bar()
         try:
             # FIXME: bug 300001 -- the backup fails if the backup directory
@@ -668,21 +551,6 @@
                 else:
                     pass
 
-    def destroy_workingtree(self):
-        """Destroy the working tree at this BzrDir.
-
-        Formats that do not support this may raise UnsupportedOperation.
-        """
-        raise NotImplementedError(self.destroy_workingtree)
-
-    def destroy_workingtree_metadata(self):
-        """Destroy the control files for the working tree at this BzrDir.
-
-        The contents of working tree files are not affected.
-        Formats that do not support this may raise UnsupportedOperation.
-        """
-        raise NotImplementedError(self.destroy_workingtree_metadata)
-
     def _find_containing(self, evaluate):
         """Find something in a containing control directory.
 
@@ -737,28 +605,6 @@
             raise errors.NoRepositoryPresent(self)
         return found_repo
 
-    def get_branch_reference(self):
-        """Return the referenced URL for the branch in this bzrdir.
-
-        :raises NotBranchError: If there is no Branch.
-        :return: The URL the branch in this bzrdir references if it is a
-            reference branch, or None for regular branches.
-        """
-        return None
-
-    def get_branch_transport(self, branch_format, name=None):
-        """Get the transport for use by branch format in this BzrDir.
-
-        Note that bzr dirs that do not support format strings will raise
-        IncompatibleFormat if the branch format they are given has
-        a format string, and vice versa.
-
-        If branch_format is None, the transport is returned with no
-        checking. If it is not None, then the returned transport is
-        guaranteed to point to an existing directory ready for use.
-        """
-        raise NotImplementedError(self.get_branch_transport)
-
     def _find_creation_modes(self):
         """Determine the appropriate modes for files and directories.
 
@@ -803,32 +649,6 @@
             self._find_creation_modes()
         return self._dir_mode
 
-    def get_repository_transport(self, repository_format):
-        """Get the transport for use by repository format in this BzrDir.
-
-        Note that bzr dirs that do not support format strings will raise
-        IncompatibleFormat if the repository format they are given has
-        a format string, and vice versa.
-
-        If repository_format is None, the transport is returned with no
-        checking. If it is not None, then the returned transport is
-        guaranteed to point to an existing directory ready for use.
-        """
-        raise NotImplementedError(self.get_repository_transport)
-
-    def get_workingtree_transport(self, tree_format):
-        """Get the transport for use by workingtree format in this BzrDir.
-
-        Note that bzr dirs that do not support format strings will raise
-        IncompatibleFormat if the workingtree format they are given has a
-        format string, and vice versa.
-
-        If workingtree_format is None, the transport is returned with no
-        checking. If it is not None, then the returned transport is
-        guaranteed to point to an existing directory ready for use.
-        """
-        raise NotImplementedError(self.get_workingtree_transport)
-
     def get_config(self):
         """Get configuration for this BzrDir."""
         return config.BzrDirConfig(self)
@@ -852,11 +672,11 @@
         self.transport = _transport.clone('.bzr')
         self.root_transport = _transport
         self._mode_check_done = False
-        
+
     @property 
     def user_transport(self):
         return self.root_transport
-        
+
     @property
     def control_transport(self):
         return self.transport
@@ -868,9 +688,7 @@
 
         This is true IF and ONLY IF the filename is part of the namespace reserved
         for bzr control dirs. Currently this is the '.bzr' directory in the root
-        of the root_transport. it is expected that plugins will need to extend
-        this in the future - for instance to make bzr talk with svn working
-        trees.
+        of the root_transport. 
         """
         # this might be better on the BzrDirFormat class because it refers to
         # all the possible bzrdir disk formats.
@@ -880,17 +698,6 @@
         # add new tests for it to the appropriate place.
         return filename == '.bzr' or filename.startswith('.bzr/')
 
-    def needs_format_conversion(self, format=None):
-        """Return true if this bzrdir needs convert_format run on it.
-
-        For instance, if the repository format is out of date but the
-        branch and working tree are not, this should return True.
-
-        :param format: Optional parameter indicating a specific desired
-                       format we plan to arrive at.
-        """
-        raise NotImplementedError(self.needs_format_conversion)
-
     @staticmethod
     def open_unsupported(base):
         """Open a branch which is not supported."""
@@ -919,7 +726,7 @@
         # the redirections.
         base = transport.base
         def find_format(transport):
-            return transport, BzrDirFormat.find_format(
+            return transport, controldir.ControlDirFormat.find_format(
                 transport, _server_formats=_server_formats)
 
         def redirected(transport, e, redirection_notice):
@@ -940,17 +747,6 @@
         BzrDir._check_supported(format, _unsupported)
         return format.open(transport, _found=True)
 
-    def open_branch(self, name=None, unsupported=False,
-                    ignore_fallbacks=False):
-        """Open the branch object at this BzrDir if one is present.
-
-        If unsupported is True, then no longer supported branch formats can
-        still be opened.
-
-        TODO: static convenience version of this?
-        """
-        raise NotImplementedError(self.open_branch)
-
     @staticmethod
     def open_containing(url, possible_transports=None):
         """Open an existing branch which contains url.
@@ -994,22 +790,6 @@
                 raise errors.NotBranchError(path=url)
             a_transport = new_t
 
-    def _get_tree_branch(self):
-        """Return the branch and tree, if any, for this bzrdir.
-
-        Return None for tree if not present or inaccessible.
-        Raise NotBranchError if no branch is present.
-        :return: (tree, branch)
-        """
-        try:
-            tree = self.open_workingtree()
-        except (errors.NoWorkingTree, errors.NotLocalUrl):
-            tree = None
-            branch = self.open_branch()
-        else:
-            branch = tree.branch
-        return tree, branch
-
     @classmethod
     def open_tree_or_branch(klass, location):
         """Return the branch and working tree at a location.
@@ -1061,59 +841,6 @@
                 raise errors.NotBranchError(location)
         return tree, branch, branch.repository, relpath
 
-    def open_repository(self, _unsupported=False):
-        """Open the repository object at this BzrDir if one is present.
-
-        This will not follow the Branch object pointer - it's strictly a direct
-        open facility. Most client code should use open_branch().repository to
-        get at a repository.
-
-        :param _unsupported: a private parameter, not part of the api.
-        TODO: static convenience version of this?
-        """
-        raise NotImplementedError(self.open_repository)
-
-    def open_workingtree(self, _unsupported=False,
-                         recommend_upgrade=True, from_branch=None):
-        """Open the workingtree object at this BzrDir if one is present.
-
-        :param recommend_upgrade: Optional keyword parameter, when True (the
-            default), emit through the ui module a recommendation that the user
-            upgrade the working tree when the workingtree being opened is old
-            (but still fully supported).
-        :param from_branch: override bzrdir branch (for lightweight checkouts)
-        """
-        raise NotImplementedError(self.open_workingtree)
-
-    def has_branch(self, name=None):
-        """Tell if this bzrdir contains a branch.
-
-        Note: if you're going to open the branch, you should just go ahead
-        and try, and not ask permission first.  (This method just opens the
-        branch and discards it, and that's somewhat expensive.)
-        """
-        try:
-            self.open_branch(name)
-            return True
-        except errors.NotBranchError:
-            return False
-
-    def has_workingtree(self):
-        """Tell if this bzrdir contains a working tree.
-
-        This will still raise an exception if the bzrdir has a workingtree that
-        is remote & inaccessible.
-
-        Note: if you're going to open the working tree, you should just go ahead
-        and try, and not ask permission first.  (This method just opens the
-        workingtree and discards it, and that's somewhat expensive.)
-        """
-        try:
-            self.open_workingtree(recommend_upgrade=False)
-            return True
-        except errors.NoWorkingTree:
-            return False
-
     def _cloning_metadir(self):
         """Produce a metadir suitable for cloning with.
 
@@ -1177,191 +904,24 @@
             format.require_stacking()
         return format
 
-    def checkout_metadir(self):
-        return self.cloning_metadir()
-
-    def sprout(self, url, revision_id=None, force_new_repo=False,
-               recurse='down', possible_transports=None,
-               accelerator_tree=None, hardlink=False, stacked=False,
-               source_branch=None, create_tree_if_local=True):
-        """Create a copy of this bzrdir prepared for use as a new line of
-        development.
-
-        If url's last component does not exist, it will be created.
-
-        Attributes related to the identity of the source branch like
-        branch nickname will be cleaned, a working tree is created
-        whether one existed before or not; and a local branch is always
-        created.
-
-        if revision_id is not None, then the clone operation may tune
-            itself to download less data.
-        :param accelerator_tree: A tree which can be used for retrieving file
-            contents more quickly than the revision tree, i.e. a workingtree.
-            The revision tree will be used for cases where accelerator_tree's
-            content is different.
-        :param hardlink: If true, hard-link files from accelerator_tree,
-            where possible.
-        :param stacked: If true, create a stacked branch referring to the
-            location of this control directory.
-        :param create_tree_if_local: If true, a working-tree will be created
-            when working locally.
+    @classmethod
+    def create(cls, base, format=None, possible_transports=None):
+        """Create a new BzrDir at the url 'base'.
+
+        :param format: If supplied, the format of branch to create.  If not
+            supplied, the default is used.
+        :param possible_transports: If supplied, a list of transports that
+            can be reused to share a remote connection.
         """
-        target_transport = get_transport(url, possible_transports)
-        target_transport.ensure_base()
-        cloning_format = self.cloning_metadir(stacked)
-        # Create/update the result branch
-        result = cloning_format.initialize_on_transport(target_transport)
-        # if a stacked branch wasn't requested, we don't create one
-        # even if the origin was stacked
-        stacked_branch_url = None
-        if source_branch is not None:
-            if stacked:
-                stacked_branch_url = self.root_transport.base
-            source_repository = source_branch.repository
-        else:
-            try:
-                source_branch = self.open_branch()
-                source_repository = source_branch.repository
-                if stacked:
-                    stacked_branch_url = self.root_transport.base
-            except errors.NotBranchError:
-                source_branch = None
-                try:
-                    source_repository = self.open_repository()
-                except errors.NoRepositoryPresent:
-                    source_repository = None
-        repository_policy = result.determine_repository_policy(
-            force_new_repo, stacked_branch_url, require_stacking=stacked)
-        result_repo, is_new_repo = repository_policy.acquire_repository()
-        if is_new_repo and revision_id is not None and not stacked:
-            fetch_spec = graph.PendingAncestryResult(
-                [revision_id], source_repository)
-        else:
-            fetch_spec = None
-        if source_repository is not None:
-            # Fetch while stacked to prevent unstacked fetch from
-            # Branch.sprout.
-            if fetch_spec is None:
-                result_repo.fetch(source_repository, revision_id=revision_id)
-            else:
-                result_repo.fetch(source_repository, fetch_spec=fetch_spec)
-
-        if source_branch is None:
-            # this is for sprouting a bzrdir without a branch; is that
-            # actually useful?
-            # Not especially, but it's part of the contract.
-            result_branch = result.create_branch()
-        else:
-            result_branch = source_branch.sprout(result,
-                revision_id=revision_id, repository_policy=repository_policy)
-        mutter("created new branch %r" % (result_branch,))
-
-        # Create/update the result working tree
-        if (create_tree_if_local and
-            isinstance(target_transport, local.LocalTransport) and
-            (result_repo is None or result_repo.make_working_trees())):
-            wt = result.create_workingtree(accelerator_tree=accelerator_tree,
-                hardlink=hardlink)
-            wt.lock_write()
-            try:
-                if wt.path2id('') is None:
-                    try:
-                        wt.set_root_id(self.open_workingtree.get_root_id())
-                    except errors.NoWorkingTree:
-                        pass
-            finally:
-                wt.unlock()
-        else:
-            wt = None
-        if recurse == 'down':
-            if wt is not None:
-                basis = wt.basis_tree()
-                basis.lock_read()
-                subtrees = basis.iter_references()
-            elif result_branch is not None:
-                basis = result_branch.basis_tree()
-                basis.lock_read()
-                subtrees = basis.iter_references()
-            elif source_branch is not None:
-                basis = source_branch.basis_tree()
-                basis.lock_read()
-                subtrees = basis.iter_references()
-            else:
-                subtrees = []
-                basis = None
-            try:
-                for path, file_id in subtrees:
-                    target = urlutils.join(url, urlutils.escape(path))
-                    sublocation = source_branch.reference_parent(file_id, path)
-                    sublocation.bzrdir.sprout(target,
-                        basis.get_reference_revision(file_id, path),
-                        force_new_repo=force_new_repo, recurse=recurse,
-                        stacked=stacked)
-            finally:
-                if basis is not None:
-                    basis.unlock()
-        return result
-
-    def push_branch(self, source, revision_id=None, overwrite=False, 
-        remember=False, create_prefix=False):
-        """Push the source branch into this BzrDir."""
-        br_to = None
-        # If we can open a branch, use its direct repository, otherwise see
-        # if there is a repository without a branch.
-        try:
-            br_to = self.open_branch()
-        except errors.NotBranchError:
-            # Didn't find a branch, can we find a repository?
-            repository_to = self.find_repository()
-        else:
-            # Found a branch, so we must have found a repository
-            repository_to = br_to.repository
-
-        push_result = PushResult()
-        push_result.source_branch = source
-        if br_to is None:
-            # We have a repository but no branch, copy the revisions, and then
-            # create a branch.
-            repository_to.fetch(source.repository, revision_id=revision_id)
-            br_to = source.clone(self, revision_id=revision_id)
-            if source.get_push_location() is None or remember:
-                source.set_push_location(br_to.base)
-            push_result.stacked_on = None
-            push_result.branch_push_result = None
-            push_result.old_revno = None
-            push_result.old_revid = _mod_revision.NULL_REVISION
-            push_result.target_branch = br_to
-            push_result.master_branch = None
-            push_result.workingtree_updated = False
-        else:
-            # We have successfully opened the branch, remember if necessary:
-            if source.get_push_location() is None or remember:
-                source.set_push_location(br_to.base)
-            try:
-                tree_to = self.open_workingtree()
-            except errors.NotLocalUrl:
-                push_result.branch_push_result = source.push(br_to, 
-                    overwrite, stop_revision=revision_id)
-                push_result.workingtree_updated = False
-            except errors.NoWorkingTree:
-                push_result.branch_push_result = source.push(br_to,
-                    overwrite, stop_revision=revision_id)
-                push_result.workingtree_updated = None # Not applicable
-            else:
-                tree_to.lock_write()
-                try:
-                    push_result.branch_push_result = source.push(
-                        tree_to.branch, overwrite, stop_revision=revision_id)
-                    tree_to.update()
-                finally:
-                    tree_to.unlock()
-                push_result.workingtree_updated = True
-            push_result.old_revno = push_result.branch_push_result.old_revno
-            push_result.old_revid = push_result.branch_push_result.old_revid
-            push_result.target_branch = \
-                push_result.branch_push_result.target_branch
-        return push_result
+        if cls is not BzrDir:
+            raise AssertionError("BzrDir.create always creates the"
+                "default format, not one of %r" % cls)
+        t = get_transport(base, possible_transports)
+        t.ensure_base()
+        if format is None:
+            format = controldir.ControlDirFormat.get_default_format()
+        return format.initialize_on_transport(t)
+
 
 
 class BzrDirHooks(hooks.Hooks):
@@ -1437,7 +997,7 @@
     def cloning_metadir(self, require_stacking=False):
         """Produce a metadir suitable for cloning with."""
         if require_stacking:
-            return format_registry.make_bzrdir('1.6')
+            return controldir.format_registry.make_bzrdir('1.6')
         return self._format.__class__()
 
     def clone(self, url, revision_id=None, force_new_repo=False,
@@ -1736,13 +1296,13 @@
     def destroy_workingtree_metadata(self):
         self.transport.delete_tree('checkout')
 
-    def find_branch_format(self):
+    def find_branch_format(self, name=None):
         """Find the branch 'format' for this bzrdir.
 
         This might be a synthetic object for e.g. RemoteBranch and SVN.
         """
         from bzrlib.branch import BranchFormat
-        return BranchFormat.find_format(self)
+        return BranchFormat.find_format(self, name=name)
 
     def _get_mkdir_mode(self):
         """Figure out the mode to use when creating a bzrdir subdir."""
@@ -1750,11 +1310,11 @@
                                      lockable_files.TransportLock)
         return temp_control._dir_mode
 
-    def get_branch_reference(self):
+    def get_branch_reference(self, name=None):
         """See BzrDir.get_branch_reference()."""
         from bzrlib.branch import BranchFormat
-        format = BranchFormat.find_format(self)
-        return format.get_reference(self)
+        format = BranchFormat.find_format(self, name=name)
+        return format.get_reference(self, name=name)
 
     def get_branch_transport(self, branch_format, name=None):
         """See BzrDir.get_branch_transport()."""
@@ -1854,7 +1414,7 @@
     def open_branch(self, name=None, unsupported=False,
                     ignore_fallbacks=False):
         """See BzrDir.open_branch."""
-        format = self.find_branch_format()
+        format = self.find_branch_format(name=name)
         self._check_supported(format, unsupported)
         return format.open(self, name=name,
             _found=True, ignore_fallbacks=ignore_fallbacks)
@@ -1880,13 +1440,66 @@
         return config.TransportConfig(self.transport, 'control.conf')
 
 
-class BzrDirFormat(object):
-    """An encapsulation of the initialization and open routines for a format.
-
-    Formats provide three things:
-     * An initialization routine,
-     * a format string,
-     * an open routine.
+class BzrProber(controldir.Prober):
+    """Prober for formats that use a .bzr/ control directory."""
+
+    _formats = {}
+    """The known .bzr formats."""
+
+    @classmethod
+    def register_bzrdir_format(klass, format):
+        klass._formats[format.get_format_string()] = format
+
+    @classmethod
+    def unregister_bzrdir_format(klass, format):
+        del klass._formats[format.get_format_string()]
+
+    @classmethod
+    def probe_transport(klass, transport):
+        """Return the .bzrdir style format present in a directory."""
+        try:
+            format_string = transport.get_bytes(".bzr/branch-format")
+        except errors.NoSuchFile:
+            raise errors.NotBranchError(path=transport.base)
+        try:
+            return klass._formats[format_string]
+        except KeyError:
+            raise errors.UnknownFormatError(format=format_string, kind='bzrdir')
+
+
+controldir.ControlDirFormat.register_prober(BzrProber)
+
+
+class RemoteBzrProber(controldir.Prober):
+    """Prober for remote servers that provide a Bazaar smart server."""
+
+    @classmethod
+    def probe_transport(klass, transport):
+        """Return a RemoteBzrDirFormat object if it looks possible."""
+        try:
+            medium = transport.get_smart_medium()
+        except (NotImplementedError, AttributeError,
+                errors.TransportNotPossible, errors.NoSmartMedium,
+                errors.SmartProtocolError):
+            # no smart server, so not a branch for this format type.
+            raise errors.NotBranchError(path=transport.base)
+        else:
+            # Decline to open it if the server doesn't support our required
+            # version (3) so that the VFS-based transport will do it.
+            if medium.should_probe():
+                try:
+                    server_version = medium.protocol_version()
+                except errors.SmartProtocolError:
+                    # Apparently there's no usable smart server there, even though
+                    # the medium supports the smart protocol.
+                    raise errors.NotBranchError(path=transport.base)
+                if server_version != '2':
+                    raise errors.NotBranchError(path=transport.base)
+            return RemoteBzrDirFormat()
+
+
+class BzrDirFormat(controldir.ControlDirFormat):
+    """ControlDirFormat base class for .bzr/ directories.
 
     Formats are placed in a dict by their format string for reference
     during bzrdir opening. These should be subclasses of BzrDirFormat
@@ -1895,105 +1508,17 @@
     Once a format is deprecated, just deprecate the initialize and open
     methods on the format class. Do not deprecate the object, as the
     object will be created every system load.
-
-    :cvar colocated_branches: Whether this formats supports colocated branches.
-    """
-
-    _default_format = None
-    """The default format used for new .bzr dirs."""
-
-    _formats = {}
-    """The known formats."""
-
-    _control_formats = []
-    """The registered control formats - .bzr, ....
-
-    This is a list of BzrDirFormat objects.
-    """
-
-    _control_server_formats = []
-    """The registered control server formats, e.g. RemoteBzrDirs.
-
-    This is a list of BzrDirFormat objects.
     """
 
     _lock_file_name = 'branch-lock'
 
-    colocated_branches = False
-    """Whether co-located branches are supported for this control dir format.
-    """
-
     # _lock_class must be set in subclasses to the lock type, typ.
     # TransportLock or LockDir
 
-    @classmethod
-    def find_format(klass, transport, _server_formats=True):
-        """Return the format present at transport."""
-        if _server_formats:
-            formats = klass._control_server_formats + klass._control_formats
-        else:
-            formats = klass._control_formats
-        for format in formats:
-            try:
-                return format.probe_transport(transport)
-            except errors.NotBranchError:
-                # this format does not find a control dir here.
-                pass
-        raise errors.NotBranchError(path=transport.base)
-
-    @classmethod
-    def probe_transport(klass, transport):
-        """Return the .bzrdir style format present in a directory."""
-        try:
-            format_string = transport.get_bytes(".bzr/branch-format")
-        except errors.NoSuchFile:
-            raise errors.NotBranchError(path=transport.base)
-
-        try:
-            return klass._formats[format_string]
-        except KeyError:
-            raise errors.UnknownFormatError(format=format_string, kind='bzrdir')
-
-    @classmethod
-    def get_default_format(klass):
-        """Return the current default format."""
-        return klass._default_format
-
     def get_format_string(self):
         """Return the ASCII format string that identifies this format."""
         raise NotImplementedError(self.get_format_string)
 
-    def get_format_description(self):
-        """Return the short description for this format."""
-        raise NotImplementedError(self.get_format_description)
-
-    def get_converter(self, format=None):
-        """Return the converter to use to convert bzrdirs needing converts.
-
-        This returns a bzrlib.bzrdir.Converter object.
-
-        This should return the best upgrader to step this format towards the
-        current default format. In the case of plugins we can/should provide
-        some means for them to extend the range of returnable converters.
-
-        :param format: Optional format to override the default format of the
-                       library.
-        """
-        raise NotImplementedError(self.get_converter)
-
-    def initialize(self, url, possible_transports=None):
-        """Create a bzr control dir at this url and return an opened copy.
-
-        While not deprecated, this method is very specific and its use will
-        lead to many round trips to setup a working environment. See
-        initialize_on_transport_ex for a [nearly] all-in-one method.
-
-        Subclasses should typically override initialize_on_transport
-        instead of this method.
-        """
-        return self.initialize_on_transport(get_transport(url,
-                                                          possible_transports))
-
     def initialize_on_transport(self, transport):
         """Initialize a new bzrdir in the base directory of a Transport."""
         try:
@@ -2147,55 +1672,13 @@
             control_files.unlock()
         return self.open(transport, _found=True)
 
-    def is_supported(self):
-        """Is this format supported?
-
-        Supported formats must be initializable and openable.
-        Unsupported formats may not support initialization or committing or
-        some other features depending on the reason for not being supported.
-        """
-        return True
-
-    def network_name(self):
-        """A simple byte string uniquely identifying this format for RPC calls.
-
-        Bzr control formats use thir disk format string to identify the format
-        over the wire. Its possible that other control formats have more
-        complex detection requirements, so we permit them to use any unique and
-        immutable string they desire.
-        """
-        raise NotImplementedError(self.network_name)
-
-    def same_model(self, target_format):
-        return (self.repository_format.rich_root_data ==
-            target_format.rich_root_data)
-
-    @classmethod
-    def known_formats(klass):
-        """Return all the known formats.
-
-        Concrete formats should override _known_formats.
-        """
-        # There is double indirection here to make sure that control
-        # formats used by more than one dir format will only be probed
-        # once. This can otherwise be quite expensive for remote connections.
-        result = set()
-        for format in klass._control_formats:
-            result.update(format._known_formats())
-        return result
-
-    @classmethod
-    def _known_formats(klass):
-        """Return the known format instances for this control format."""
-        return set(klass._formats.values())
-
     def open(self, transport, _found=False):
         """Return an instance of this format for the dir transport points at.
 
         _found is a private parameter, do not use it.
         """
         if not _found:
-            found_format = BzrDirFormat.find_format(transport)
+            found_format = controldir.ControlDirFormat.find_format(transport)
             if not isinstance(found_format, self.__class__):
                 raise AssertionError("%s was asked to open %s, but it seems to need "
                         "format %s"
@@ -2215,40 +1698,10 @@
 
     @classmethod
     def register_format(klass, format):
-        klass._formats[format.get_format_string()] = format
+        BzrProber.register_bzrdir_format(format)
         # bzr native formats have a network name of their format string.
-        network_format_registry.register(format.get_format_string(), format.__class__)
-
-    @classmethod
-    def register_control_format(klass, format):
-        """Register a format that does not use '.bzr' for its control dir.
-
-        TODO: This should be pulled up into a 'ControlDirFormat' base class
-        which BzrDirFormat can inherit from, and renamed to register_format
-        there. It has been done without that for now for simplicity of
-        implementation.
-        """
-        klass._control_formats.append(format)
-
-    @classmethod
-    def register_control_server_format(klass, format):
-        """Register a control format for client-server environments.
-
-        These formats will be tried before ones registered with
-        register_control_format.  This gives implementations that decide to the
-        chance to grab it before anything looks at the contents of the format
-        file.
-        """
-        klass._control_server_formats.append(format)
-
-    @classmethod
-    def _set_default_format(klass, format):
-        """Set default format (for testing behavior of defaults only)"""
-        klass._default_format = format
-
-    def __str__(self):
-        # Trim the newline
-        return self.get_format_description().rstrip()
+        controldir.network_format_registry.register(format.get_format_string(), format.__class__)
+        controldir.ControlDirFormat.register_format(format)
 
     def _supply_sub_formats_to(self, other_format):
         """Give other_format the same values for sub formats as this has.
@@ -2264,11 +1717,9 @@
 
     @classmethod
     def unregister_format(klass, format):
-        del klass._formats[format.get_format_string()]
-
-    @classmethod
-    def unregister_control_format(klass, format):
-        klass._control_formats.remove(format)
+        BzrProber.unregister_bzrdir_format(format)
+        controldir.ControlDirFormat.unregister_format(format)
+        controldir.network_format_registry.remove(format.get_format_string())
 
 
 class BzrDirFormat4(BzrDirFormat):
@@ -2677,25 +2128,13 @@
                                   __set_workingtree_format)
 
 
-network_format_registry = registry.FormatRegistry()
-"""Registry of formats indexed by their network name.
-
-The network name for a BzrDirFormat is an identifier that can be used when
-referring to formats with smart server operations. See
-BzrDirFormat.network_name() for more detail.
-"""
-
-
-# Register bzr control format
-BzrDirFormat.register_control_format(BzrDirFormat)
-
 # Register bzr formats
 BzrDirFormat.register_format(BzrDirFormat4())
 BzrDirFormat.register_format(BzrDirFormat5())
 BzrDirFormat.register_format(BzrDirFormat6())
 __default_format = BzrDirMetaFormat1()
 BzrDirFormat.register_format(__default_format)
-BzrDirFormat._default_format = __default_format
+controldir.ControlDirFormat._default_format = __default_format
 
 
 class Converter(object):
@@ -2863,7 +2302,11 @@
             self.revisions[rev_id] = rev
 
     def _load_old_inventory(self, rev_id):
-        old_inv_xml = self.branch.repository.inventory_store.get(rev_id).read()
+        f = self.branch.repository.inventory_store.get(rev_id)
+        try:
+            old_inv_xml = f.read()
+        finally:
+            f.close()
         inv = xml4.serializer_v4.read_inventory_from_string(old_inv_xml)
         inv.revision_id = rev_id
         rev = self.revisions[rev_id]
@@ -2925,7 +2368,6 @@
         previous_entries = dict((head, parent_candiate_entries[head]) for head
             in heads)
         self.snapshot_ie(previous_entries, ie, w, rev_id)
-        del ie.text_id
 
     def get_parent_map(self, revision_ids):
         """See graph.StackedParentsProvider.get_parent_map"""
@@ -2947,8 +2389,11 @@
                 ie.revision = previous_ie.revision
                 return
         if ie.has_text():
-            text = self.branch.repository._text_store.get(ie.text_id)
-            file_lines = text.readlines()
+            f = self.branch.repository._text_store.get(ie.text_id)
+            try:
+                file_lines = f.readlines()
+            finally:
+                f.close()
             w.add_lines(rev_id, previous_revisions, file_lines)
             self.text_count += 1
         else:
@@ -3230,12 +2675,14 @@
 class RemoteBzrDirFormat(BzrDirMetaFormat1):
     """Format representing bzrdirs accessed via a smart server"""
 
+    supports_workingtrees = False
+
     def __init__(self):
         BzrDirMetaFormat1.__init__(self)
         # XXX: It's a bit ugly that the network name is here, because we'd
         # like to believe that format objects are stateless or at least
         # immutable,  However, we do at least avoid mutating the name after
-        # it's returned.  See <https://bugs.edge.launchpad.net/bzr/+bug/504102>
+        # it's returned.  See <https://bugs.launchpad.net/bzr/+bug/504102>
         self._network_name = None
 
     def __repr__(self):
@@ -3244,7 +2691,7 @@
 
     def get_format_description(self):
         if self._network_name:
-            real_format = network_format_registry.get(self._network_name)
+            real_format = controldir.network_format_registry.get(self._network_name)
             return 'Remote: ' + real_format.get_format_description()
         return 'bzr remote bzrdir'
 
@@ -3257,30 +2704,6 @@
         else:
             raise AssertionError("No network name set.")
 
-    @classmethod
-    def probe_transport(klass, transport):
-        """Return a RemoteBzrDirFormat object if it looks possible."""
-        try:
-            medium = transport.get_smart_medium()
-        except (NotImplementedError, AttributeError,
-                errors.TransportNotPossible, errors.NoSmartMedium,
-                errors.SmartProtocolError):
-            # no smart server, so not a branch for this format type.
-            raise errors.NotBranchError(path=transport.base)
-        else:
-            # Decline to open it if the server doesn't support our required
-            # version (3) so that the VFS-based transport will do it.
-            if medium.should_probe():
-                try:
-                    server_version = medium.protocol_version()
-                except errors.SmartProtocolError:
-                    # Apparently there's no usable smart server there, even though
-                    # the medium supports the smart protocol.
-                    raise errors.NotBranchError(path=transport.base)
-                if server_version != '2':
-                    raise errors.NotBranchError(path=transport.base)
-            return klass()
-
     def initialize_on_transport(self, transport):
         try:
             # hand off the request to the smart server
@@ -3485,190 +2908,7 @@
         BzrDirMetaFormat1._set_repository_format) #.im_func)
 
 
-BzrDirFormat.register_control_server_format(RemoteBzrDirFormat)
-
-
-class BzrDirFormatInfo(object):
-
-    def __init__(self, native, deprecated, hidden, experimental):
-        self.deprecated = deprecated
-        self.native = native
-        self.hidden = hidden
-        self.experimental = experimental
-
-
-class BzrDirFormatRegistry(registry.Registry):
-    """Registry of user-selectable BzrDir subformats.
-
-    Differs from BzrDirFormat._control_formats in that it provides sub-formats,
-    e.g. BzrDirMeta1 with weave repository.  Also, it's more user-oriented.
-    """
-
-    def __init__(self):
-        """Create a BzrDirFormatRegistry."""
-        self._aliases = set()
-        self._registration_order = list()
-        super(BzrDirFormatRegistry, self).__init__()
-
-    def aliases(self):
-        """Return a set of the format names which are aliases."""
-        return frozenset(self._aliases)
-
-    def register_metadir(self, key,
-             repository_format, help, native=True, deprecated=False,
-             branch_format=None,
-             tree_format=None,
-             hidden=False,
-             experimental=False,
-             alias=False):
-        """Register a metadir subformat.
-
-        These all use a BzrDirMetaFormat1 bzrdir, but can be parameterized
-        by the Repository/Branch/WorkingTreeformats.
-
-        :param repository_format: The fully-qualified repository format class
-            name as a string.
-        :param branch_format: Fully-qualified branch format class name as
-            a string.
-        :param tree_format: Fully-qualified tree format class name as
-            a string.
-        """
-        # This should be expanded to support setting WorkingTree and Branch
-        # formats, once BzrDirMetaFormat1 supports that.
-        def _load(full_name):
-            mod_name, factory_name = full_name.rsplit('.', 1)
-            try:
-                mod = __import__(mod_name, globals(), locals(),
-                        [factory_name])
-            except ImportError, e:
-                raise ImportError('failed to load %s: %s' % (full_name, e))
-            try:
-                factory = getattr(mod, factory_name)
-            except AttributeError:
-                raise AttributeError('no factory %s in module %r'
-                    % (full_name, mod))
-            return factory()
-
-        def helper():
-            bd = BzrDirMetaFormat1()
-            if branch_format is not None:
-                bd.set_branch_format(_load(branch_format))
-            if tree_format is not None:
-                bd.workingtree_format = _load(tree_format)
-            if repository_format is not None:
-                bd.repository_format = _load(repository_format)
-            return bd
-        self.register(key, helper, help, native, deprecated, hidden,
-            experimental, alias)
-
-    def register(self, key, factory, help, native=True, deprecated=False,
-                 hidden=False, experimental=False, alias=False):
-        """Register a BzrDirFormat factory.
-
-        The factory must be a callable that takes one parameter: the key.
-        It must produce an instance of the BzrDirFormat when called.
-
-        This function mainly exists to prevent the info object from being
-        supplied directly.
-        """
-        registry.Registry.register(self, key, factory, help,
-            BzrDirFormatInfo(native, deprecated, hidden, experimental))
-        if alias:
-            self._aliases.add(key)
-        self._registration_order.append(key)
-
-    def register_lazy(self, key, module_name, member_name, help, native=True,
-        deprecated=False, hidden=False, experimental=False, alias=False):
-        registry.Registry.register_lazy(self, key, module_name, member_name,
-            help, BzrDirFormatInfo(native, deprecated, hidden, experimental))
-        if alias:
-            self._aliases.add(key)
-        self._registration_order.append(key)
-
-    def set_default(self, key):
-        """Set the 'default' key to be a clone of the supplied key.
-
-        This method must be called once and only once.
-        """
-        registry.Registry.register(self, 'default', self.get(key),
-            self.get_help(key), info=self.get_info(key))
-        self._aliases.add('default')
-
-    def set_default_repository(self, key):
-        """Set the FormatRegistry default and Repository default.
-
-        This is a transitional method while Repository.set_default_format
-        is deprecated.
-        """
-        if 'default' in self:
-            self.remove('default')
-        self.set_default(key)
-        format = self.get('default')()
-
-    def make_bzrdir(self, key):
-        return self.get(key)()
-
-    def help_topic(self, topic):
-        output = ""
-        default_realkey = None
-        default_help = self.get_help('default')
-        help_pairs = []
-        for key in self._registration_order:
-            if key == 'default':
-                continue
-            help = self.get_help(key)
-            if help == default_help:
-                default_realkey = key
-            else:
-                help_pairs.append((key, help))
-
-        def wrapped(key, help, info):
-            if info.native:
-                help = '(native) ' + help
-            return ':%s:\n%s\n\n' % (key,
-                textwrap.fill(help, initial_indent='    ',
-                    subsequent_indent='    ',
-                    break_long_words=False))
-        if default_realkey is not None:
-            output += wrapped(default_realkey, '(default) %s' % default_help,
-                              self.get_info('default'))
-        deprecated_pairs = []
-        experimental_pairs = []
-        for key, help in help_pairs:
-            info = self.get_info(key)
-            if info.hidden:
-                continue
-            elif info.deprecated:
-                deprecated_pairs.append((key, help))
-            elif info.experimental:
-                experimental_pairs.append((key, help))
-            else:
-                output += wrapped(key, help, info)
-        output += "\nSee :doc:`formats-help` for more about storage formats."
-        other_output = ""
-        if len(experimental_pairs) > 0:
-            other_output += "Experimental formats are shown below.\n\n"
-            for key, help in experimental_pairs:
-                info = self.get_info(key)
-                other_output += wrapped(key, help, info)
-        else:
-            other_output += \
-                "No experimental formats are available.\n\n"
-        if len(deprecated_pairs) > 0:
-            other_output += "\nDeprecated formats are shown below.\n\n"
-            for key, help in deprecated_pairs:
-                info = self.get_info(key)
-                other_output += wrapped(key, help, info)
-        else:
-            other_output += \
-                "\nNo deprecated formats are available.\n\n"
-        other_output += \
-                "\nSee :doc:`formats-help` for more about storage formats."
-
-        if topic == 'other-formats':
-            return other_output
-        else:
-            return output
+controldir.ControlDirFormat.register_server_prober(RemoteBzrProber)
 
 
 class RepositoryAcquisitionPolicy(object):
@@ -3828,33 +3068,76 @@
         return self._repository, False
 
 
-# Please register new formats after old formats so that formats
-# appear in chronological order and format descriptions can build
-# on previous ones.
-format_registry = BzrDirFormatRegistry()
+def register_metadir(registry, key,
+         repository_format, help, native=True, deprecated=False,
+         branch_format=None,
+         tree_format=None,
+         hidden=False,
+         experimental=False,
+         alias=False):
+    """Register a metadir subformat.
+
+    These all use a BzrDirMetaFormat1 bzrdir, but can be parameterized
+    by the Repository/Branch/WorkingTreeformats.
+
+    :param repository_format: The fully-qualified repository format class
+        name as a string.
+    :param branch_format: Fully-qualified branch format class name as
+        a string.
+    :param tree_format: Fully-qualified tree format class name as
+        a string.
+    """
+    # This should be expanded to support setting WorkingTree and Branch
+    # formats, once BzrDirMetaFormat1 supports that.
+    def _load(full_name):
+        mod_name, factory_name = full_name.rsplit('.', 1)
+        try:
+            mod = __import__(mod_name, globals(), locals(),
+                    [factory_name])
+        except ImportError, e:
+            raise ImportError('failed to load %s: %s' % (full_name, e))
+        try:
+            factory = getattr(mod, factory_name)
+        except AttributeError:
+            raise AttributeError('no factory %s in module %r'
+                % (full_name, mod))
+        return factory()
+
+    def helper():
+        bd = BzrDirMetaFormat1()
+        if branch_format is not None:
+            bd.set_branch_format(_load(branch_format))
+        if tree_format is not None:
+            bd.workingtree_format = _load(tree_format)
+        if repository_format is not None:
+            bd.repository_format = _load(repository_format)
+        return bd
+    registry.register(key, helper, help, native, deprecated, hidden,
+        experimental, alias)
+
 # The pre-0.8 formats have their repository format network name registered in
 # repository.py. MetaDir formats have their repository format network name
 # inferred from their disk format string.
-format_registry.register('weave', BzrDirFormat6,
+controldir.format_registry.register('weave', BzrDirFormat6,
     'Pre-0.8 format.  Slower than knit and does not'
     ' support checkouts or shared repositories.',
     hidden=True,
     deprecated=True)
-format_registry.register_metadir('metaweave',
+register_metadir(controldir.format_registry, 'metaweave',
     'bzrlib.repofmt.weaverepo.RepositoryFormat7',
     'Transitional format in 0.8.  Slower than knit.',
     branch_format='bzrlib.branch.BzrBranchFormat5',
     tree_format='bzrlib.workingtree.WorkingTreeFormat3',
     hidden=True,
     deprecated=True)
-format_registry.register_metadir('knit',
+register_metadir(controldir.format_registry, 'knit',
     'bzrlib.repofmt.knitrepo.RepositoryFormatKnit1',
     'Format using knits.  Recommended for interoperation with bzr <= 0.14.',
     branch_format='bzrlib.branch.BzrBranchFormat5',
     tree_format='bzrlib.workingtree.WorkingTreeFormat3',
     hidden=True,
     deprecated=True)
-format_registry.register_metadir('dirstate',
+register_metadir(controldir.format_registry, 'dirstate',
     'bzrlib.repofmt.knitrepo.RepositoryFormatKnit1',
     help='New in 0.15: Fast local operations. Compatible with bzr 0.8 and '
         'above when accessed over the network.',
@@ -3864,7 +3147,7 @@
     tree_format='bzrlib.workingtree.WorkingTreeFormat4',
     hidden=True,
     deprecated=True)
-format_registry.register_metadir('dirstate-tags',
+register_metadir(controldir.format_registry, 'dirstate-tags',
     'bzrlib.repofmt.knitrepo.RepositoryFormatKnit1',
     help='New in 0.15: Fast local operations and improved scaling for '
         'network operations. Additionally adds support for tags.'
@@ -3873,7 +3156,7 @@
     tree_format='bzrlib.workingtree.WorkingTreeFormat4',
     hidden=True,
     deprecated=True)
-format_registry.register_metadir('rich-root',
+register_metadir(controldir.format_registry, 'rich-root',
     'bzrlib.repofmt.knitrepo.RepositoryFormatKnit4',
     help='New in 1.0.  Better handling of tree roots.  Incompatible with'
         ' bzr < 1.0.',
@@ -3881,7 +3164,7 @@
     tree_format='bzrlib.workingtree.WorkingTreeFormat4',
     hidden=True,
     deprecated=True)
-format_registry.register_metadir('dirstate-with-subtree',
+register_metadir(controldir.format_registry, 'dirstate-with-subtree',
     'bzrlib.repofmt.knitrepo.RepositoryFormatKnit3',
     help='New in 0.15: Fast local operations and improved scaling for '
         'network operations. Additionally adds support for versioning nested '
@@ -3891,7 +3174,7 @@
     experimental=True,
     hidden=True,
     )
-format_registry.register_metadir('pack-0.92',
+register_metadir(controldir.format_registry, 'pack-0.92',
     'bzrlib.repofmt.pack_repo.RepositoryFormatKnitPack1',
     help='New in 0.92: Pack-based format with data compatible with '
         'dirstate-tags format repositories. Interoperates with '
@@ -3900,7 +3183,7 @@
     branch_format='bzrlib.branch.BzrBranchFormat6',
     tree_format='bzrlib.workingtree.WorkingTreeFormat4',
     )
-format_registry.register_metadir('pack-0.92-subtree',
+register_metadir(controldir.format_registry, 'pack-0.92-subtree',
     'bzrlib.repofmt.pack_repo.RepositoryFormatKnitPack3',
     help='New in 0.92: Pack-based format with data compatible with '
         'dirstate-with-subtree format repositories. Interoperates with '
@@ -3911,7 +3194,7 @@
     hidden=True,
     experimental=True,
     )
-format_registry.register_metadir('rich-root-pack',
+register_metadir(controldir.format_registry, 'rich-root-pack',
     'bzrlib.repofmt.pack_repo.RepositoryFormatKnitPack4',
     help='New in 1.0: A variant of pack-0.92 that supports rich-root data '
          '(needed for bzr-svn and bzr-git).',
@@ -3919,7 +3202,7 @@
     tree_format='bzrlib.workingtree.WorkingTreeFormat4',
     hidden=True,
     )
-format_registry.register_metadir('1.6',
+register_metadir(controldir.format_registry, '1.6',
     'bzrlib.repofmt.pack_repo.RepositoryFormatKnitPack5',
     help='A format that allows a branch to indicate that there is another '
          '(stacked) repository that should be used to access data that is '
@@ -3928,7 +3211,7 @@
     tree_format='bzrlib.workingtree.WorkingTreeFormat4',
     hidden=True,
     )
-format_registry.register_metadir('1.6.1-rich-root',
+register_metadir(controldir.format_registry, '1.6.1-rich-root',
     'bzrlib.repofmt.pack_repo.RepositoryFormatKnitPack5RichRoot',
     help='A variant of 1.6 that supports rich-root data '
          '(needed for bzr-svn and bzr-git).',
@@ -3936,7 +3219,7 @@
     tree_format='bzrlib.workingtree.WorkingTreeFormat4',
     hidden=True,
     )
-format_registry.register_metadir('1.9',
+register_metadir(controldir.format_registry, '1.9',
     'bzrlib.repofmt.pack_repo.RepositoryFormatKnitPack6',
     help='A repository format using B+tree indexes. These indexes '
          'are smaller in size, have smarter caching and provide faster '
@@ -3945,7 +3228,7 @@
     tree_format='bzrlib.workingtree.WorkingTreeFormat4',
     hidden=True,
     )
-format_registry.register_metadir('1.9-rich-root',
+register_metadir(controldir.format_registry, '1.9-rich-root',
     'bzrlib.repofmt.pack_repo.RepositoryFormatKnitPack6RichRoot',
     help='A variant of 1.9 that supports rich-root data '
          '(needed for bzr-svn and bzr-git).',
@@ -3953,13 +3236,13 @@
     tree_format='bzrlib.workingtree.WorkingTreeFormat4',
     hidden=True,
     )
-format_registry.register_metadir('1.14',
+register_metadir(controldir.format_registry, '1.14',
     'bzrlib.repofmt.pack_repo.RepositoryFormatKnitPack6',
     help='A working-tree format that supports content filtering.',
     branch_format='bzrlib.branch.BzrBranchFormat7',
     tree_format='bzrlib.workingtree.WorkingTreeFormat5',
     )
-format_registry.register_metadir('1.14-rich-root',
+register_metadir(controldir.format_registry, '1.14-rich-root',
     'bzrlib.repofmt.pack_repo.RepositoryFormatKnitPack6RichRoot',
     help='A variant of 1.14 that supports rich-root data '
          '(needed for bzr-svn and bzr-git).',
@@ -3967,7 +3250,7 @@
     tree_format='bzrlib.workingtree.WorkingTreeFormat5',
     )
 # The following un-numbered 'development' formats should always just be aliases.
-format_registry.register_metadir('development-rich-root',
+register_metadir(controldir.format_registry, 'development-rich-root',
     'bzrlib.repofmt.groupcompress_repo.RepositoryFormatCHK1',
     help='Current development format. Supports rich roots. Can convert data '
         'to and from rich-root-pack (and anything compatible with '
@@ -3981,8 +3264,24 @@
     alias=True,
     hidden=True,
     )
-format_registry.register_metadir('development-subtree',
+register_metadir(controldir.format_registry, 'development5-subtree',
     'bzrlib.repofmt.pack_repo.RepositoryFormatPackDevelopment2Subtree',
+    help='Development format, subtree variant. Can convert data to and '
+        'from pack-0.92-subtree (and anything compatible with '
+        'pack-0.92-subtree) format repositories. Repositories and branches in '
+        'this format can only be read by bzr.dev. Please read '
+        'http://doc.bazaar.canonical.com/latest/developers/development-repo.html '
+        'before use.',
+    branch_format='bzrlib.branch.BzrBranchFormat7',
+    tree_format='bzrlib.workingtree.WorkingTreeFormat6',
+    experimental=True,
+    hidden=True,
+    alias=False,
+    )
+
+
+register_metadir(controldir.format_registry, 'development-subtree',
+    'bzrlib.repofmt.groupcompress_repo.RepositoryFormat2aSubtree',
     help='Current development format, subtree variant. Can convert data to and '
         'from pack-0.92-subtree (and anything compatible with '
         'pack-0.92-subtree) format repositories. Repositories and branches in '
@@ -3999,7 +3298,7 @@
     )
 
 # And the development formats above will have aliased one of the following:
-format_registry.register_metadir('development6-rich-root',
+register_metadir(controldir.format_registry, 'development6-rich-root',
     'bzrlib.repofmt.groupcompress_repo.RepositoryFormatCHK1',
     help='pack-1.9 with 255-way hashed CHK inv, group compress, rich roots '
         'Please read '
@@ -4011,7 +3310,7 @@
     experimental=True,
     )
 
-format_registry.register_metadir('development7-rich-root',
+register_metadir(controldir.format_registry, 'development7-rich-root',
     'bzrlib.repofmt.groupcompress_repo.RepositoryFormatCHK2',
     help='pack-1.9 with 255-way hashed CHK inv, bencode revision, group compress, '
         'rich roots. Please read '
@@ -4023,7 +3322,7 @@
     experimental=True,
     )
 
-format_registry.register_metadir('2a',
+register_metadir(controldir.format_registry, '2a',
     'bzrlib.repofmt.groupcompress_repo.RepositoryFormat2a',
     help='First format for bzr 2.0 series.\n'
         'Uses group-compress storage.\n'
@@ -4032,12 +3331,12 @@
         # 'rich roots. Supported by bzr 1.16 and later.',
     branch_format='bzrlib.branch.BzrBranchFormat7',
     tree_format='bzrlib.workingtree.WorkingTreeFormat6',
-    experimental=True,
+    experimental=False,
     )
 
 # The following format should be an alias for the rich root equivalent 
 # of the default format
-format_registry.register_metadir('default-rich-root',
+register_metadir(controldir.format_registry, 'default-rich-root',
     'bzrlib.repofmt.groupcompress_repo.RepositoryFormat2a',
     branch_format='bzrlib.branch.BzrBranchFormat7',
     tree_format='bzrlib.workingtree.WorkingTreeFormat6',
@@ -4046,4 +3345,9 @@
     help='Same as 2a.')
 
 # The current format that is made on 'bzr init'.
-format_registry.set_default('2a')
+controldir.format_registry.set_default('2a')
+
+# XXX 2010-08-20 JRV: There is still a lot of code relying on
+# bzrlib.bzrdir.format_registry existing. When BzrDir.create/BzrDir.open/etc
+# get changed to ControlDir.create/ControlDir.open/etc this should be removed.
+format_registry = controldir.format_registry

=== modified file 'bzrlib/chk_map.py'
--- bzrlib/chk_map.py	2010-04-21 20:33:10 +0000
+++ bzrlib/chk_map.py	2010-09-28 18:51:47 +0000
@@ -90,8 +90,6 @@
 _INTERESTING_NEW_SIZE = 50
 # If a ChildNode shrinks by more than this amount, we check for a remap
 _INTERESTING_SHRINKAGE_LIMIT = 20
-# If we delete more than this many nodes applying a delta, we check for a remap
-_INTERESTING_DELETES_LIMIT = 5
 
 
 def _search_key_plain(key):
@@ -135,7 +133,7 @@
             into the map; if old_key is not None, then the old mapping
             of old_key is removed.
         """
-        delete_count = 0
+        has_deletes = False
         # Check preconditions first.
         as_st = StaticTuple.from_sequence
         new_items = set([as_st(key) for (old, key, value) in delta
@@ -148,12 +146,11 @@
         for old, new, value in delta:
             if old is not None and old != new:
                 self.unmap(old, check_remap=False)
-                delete_count += 1
+                has_deletes = True
         for old, new, value in delta:
             if new is not None:
                 self.map(new, value)
-        if delete_count > _INTERESTING_DELETES_LIMIT:
-            trace.mutter("checking remap as %d deletions", delete_count)
+        if has_deletes:
             self._check_remap()
         return self._save()
 
@@ -573,7 +570,7 @@
         """Check if nodes can be collapsed."""
         self._ensure_root()
         if type(self._root_node) is InternalNode:
-            self._root_node._check_remap(self._store)
+            self._root_node = self._root_node._check_remap(self._store)
 
     def _save(self):
         """Save the map completely.
@@ -1372,7 +1369,7 @@
         return self._search_prefix
 
     def unmap(self, store, key, check_remap=True):
-        """Remove key from this node and it's children."""
+        """Remove key from this node and its children."""
         if not len(self._items):
             raise AssertionError("can't unmap in an empty InternalNode.")
         children = [node for node, _
@@ -1727,6 +1724,7 @@
 
 try:
     from bzrlib._chk_map_pyx import (
+        _bytes_to_text_key,
         _search_key_16,
         _search_key_255,
         _deserialise_leaf_node,
@@ -1735,6 +1733,7 @@
 except ImportError, e:
     osutils.failed_to_load_extension(e)
     from bzrlib._chk_map_py import (
+        _bytes_to_text_key,
         _search_key_16,
         _search_key_255,
         _deserialise_leaf_node,

=== modified file 'bzrlib/clean_tree.py'
--- bzrlib/clean_tree.py	2010-04-30 11:03:59 +0000
+++ bzrlib/clean_tree.py	2010-08-23 17:54:07 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,9 +15,15 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
+import errno
 import os
 import shutil
 
+from bzrlib import (
+    bzrdir,
+    errors,
+    ui,
+    )
 from bzrlib.osutils import isdir
 from bzrlib.trace import note
 from bzrlib.workingtree import WorkingTree
@@ -51,11 +57,14 @@
     try:
         deletables = list(iter_deletables(tree, unknown=unknown,
             ignored=ignored, detritus=detritus))
+        deletables = _filter_out_nested_bzrdirs(deletables)
         if len(deletables) == 0:
             note('Nothing to delete.')
             return 0
         if not no_prompt:
             for path, subp in deletables:
+                # FIXME using print is very bad idea
+                # clean_tree should accept to_file argument to write the output
                 print subp
             val = raw_input('Are you sure you wish to delete these [y/N]?')
             if val.lower() not in ('y', 'yes'):
@@ -66,18 +75,56 @@
         tree.unlock()
 
 
+def _filter_out_nested_bzrdirs(deletables):
+    result = []
+    for path, subp in deletables:
+        # bzr won't recurse into unknowns/ignored directories by default
+        # so we don't pay a penalty for checking subdirs of path for nested
+        # bzrdir.
+        # That said we won't detect the branch in the subdir of non-branch
+        # directory and therefore delete it. (worth to FIXME?)
+        if isdir(path):
+            try:
+                bzrdir.BzrDir.open(path)
+            except errors.NotBranchError:
+                result.append((path,subp))
+            else:
+                # TODO may be we need to notify user about skipped directories?
+                pass
+        else:
+            result.append((path,subp))
+    return result
+
+
 def delete_items(deletables, dry_run=False):
     """Delete files in the deletables iterable"""
+    def onerror(function, path, excinfo):
+        """Show warning for errors seen by rmtree.
+        """
+        # Handle only permission error while removing files.
+        # Other errors are re-raised.
+        if function is not os.remove or excinfo[1].errno != errno.EACCES:
+            raise
+        ui.ui_factory.show_warning('unable to remove %s' % path)
     has_deleted = False
     for path, subp in deletables:
         if not has_deleted:
             note("deleting paths:")
             has_deleted = True
-        note('  ' + subp)
         if not dry_run:
             if isdir(path):
-                shutil.rmtree(path)
+                shutil.rmtree(path, onerror=onerror)
             else:
-                os.unlink(path)
+                try:
+                    os.unlink(path)
+                    note('  ' + subp)
+                except OSError, e:
+                    # We handle only permission error here
+                    if e.errno != errno.EACCES:
+                        raise e
+                    ui.ui_factory.show_warning(
+                        'unable to remove "%s": %s.' % (path, e.strerror))
+        else:
+            note('  ' + subp)
     if not has_deleted:
         note("No files deleted.")

=== modified file 'bzrlib/cleanup.py'
--- bzrlib/cleanup.py	2010-03-03 22:59:21 +0000
+++ bzrlib/cleanup.py	2010-04-27 07:29:11 +0000
@@ -78,7 +78,28 @@
         _run_cleanup(func, *args, **kwargs)
 
 
-class OperationWithCleanups(object):
+class ObjectWithCleanups(object):
+    """A mixin for objects that hold a cleanup list.
+
+    Subclass or client code can call add_cleanup and then later `cleanup_now`.
+    """
+    def __init__(self):
+        self.cleanups = deque()
+
+    def add_cleanup(self, cleanup_func, *args, **kwargs):
+        """Add a cleanup to run.
+
+        Cleanups may be added at any time.  
+        Cleanups will be executed in LIFO order.
+        """
+        self.cleanups.appendleft((cleanup_func, args, kwargs))
+
+    def cleanup_now(self):
+        _run_cleanups(self.cleanups)
+        self.cleanups.clear()
+
+
+class OperationWithCleanups(ObjectWithCleanups):
     """A way to run some code with a dynamic cleanup list.
 
     This provides a way to add cleanups while the function-with-cleanups is
@@ -102,16 +123,8 @@
     """
 
     def __init__(self, func):
+        super(OperationWithCleanups, self).__init__()
         self.func = func
-        self.cleanups = deque()
-
-    def add_cleanup(self, cleanup_func, *args, **kwargs):
-        """Add a cleanup to run.
-
-        Cleanups may be added at any time before or during the execution of
-        self.func.  Cleanups will be executed in LIFO order.
-        """
-        self.cleanups.appendleft((cleanup_func, args, kwargs))
 
     def run(self, *args, **kwargs):
         return _do_with_cleanups(
@@ -121,10 +134,6 @@
         return _do_with_cleanups(
             self.cleanups, self.func, *args, **kwargs)
 
-    def cleanup_now(self):
-        _run_cleanups(self.cleanups)
-        self.cleanups.clear()
-
 
 def _do_with_cleanups(cleanup_funcs, func, *args, **kwargs):
     """Run `func`, then call all the cleanup_funcs.

=== modified file 'bzrlib/cmd_version_info.py'
--- bzrlib/cmd_version_info.py	2010-04-02 19:12:58 +0000
+++ bzrlib/cmd_version_info.py	2010-09-29 23:53:57 +0000
@@ -116,4 +116,4 @@
                 include_revision_history=include_history,
                 include_file_revisions=include_file_revisions,
                 template=template)
-        builder.generate(ui.ui_factory.make_output_stream())
+        builder.generate(self.outf)

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2010-04-16 07:56:51 +0000
+++ bzrlib/commands.py	2010-08-13 07:56:06 +0000
@@ -222,6 +222,7 @@
     Use of all_command_names() is encouraged rather than builtin_command_names
     and/or plugin_command_names.
     """
+    _register_builtin_commands()
     return builtin_command_registry.keys()
 
 
@@ -397,6 +398,12 @@
             will not mangled.
 
     :cvar hooks: An instance of CommandHooks.
+    :ivar __doc__: The help shown by 'bzr help command' for this command.
+        This is set by assigning explicitly to __doc__ so that -OO can
+        be used::
+
+        class Foo(Command):
+            __doc__ = "My help goes here"
     """
     aliases = []
     takes_args = []
@@ -407,8 +414,6 @@
 
     def __init__(self):
         """Construct an instance of this command."""
-        if self.__doc__ == Command.__doc__:
-            warn("No help message set for %r" % self)
         # List of standard options directly supported
         self.supported_std_options = []
         self._setup_run()
@@ -482,8 +487,8 @@
             message explaining how to obtain full help.
         """
         doc = self.help()
-        if doc is None:
-            raise NotImplementedError("sorry, no detailed help yet for %r" % self.name())
+        if not doc:
+            doc = "No help for this command."
 
         # Extract the summary (purpose) and sections out from the text
         purpose,sections,order = self._get_help_parts(doc)
@@ -1050,6 +1055,8 @@
         elif a == '--coverage':
             opt_coverage_dir = argv[i + 1]
             i += 1
+        elif a == '--profile-imports':
+            pass # already handled in startup script Bug #588277
         elif a.startswith('-D'):
             debug.debug_flags.add(a[2:])
         else:
@@ -1077,15 +1084,9 @@
     if not opt_no_aliases:
         alias_argv = get_alias(argv[0])
         if alias_argv:
-            user_encoding = osutils.get_user_encoding()
-            alias_argv = [a.decode(user_encoding) for a in alias_argv]
             argv[0] = alias_argv.pop(0)
 
     cmd = argv.pop(0)
-    # We want only 'ascii' command names, but the user may have typed
-    # in a Unicode name. In that case, they should just get a
-    # 'command not found' error later.
-
     cmd_obj = get_cmd_object(cmd, plugins_override=not opt_builtin)
     run = cmd_obj.run_argv_aliases
     run_argv = [argv, alias_argv]

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2010-04-30 11:35:43 +0000
+++ bzrlib/commit.py	2010-05-10 11:34:20 +0000
@@ -174,6 +174,43 @@
         self.reporter = reporter
         self.config = config
 
+    @staticmethod
+    def update_revprops(revprops, branch, authors=None, author=None,
+                        local=False, possible_master_transports=None):
+        if revprops is None:
+            revprops = {}
+        if possible_master_transports is None:
+            possible_master_transports = []
+        if not 'branch-nick' in revprops:
+            revprops['branch-nick'] = branch._get_nick(
+                local,
+                possible_master_transports)
+        if authors is not None:
+            if author is not None:
+                raise AssertionError('Specifying both author and authors '
+                        'is not allowed. Specify just authors instead')
+            if 'author' in revprops or 'authors' in revprops:
+                # XXX: maybe we should just accept one of them?
+                raise AssertionError('author property given twice')
+            if authors:
+                for individual in authors:
+                    if '\n' in individual:
+                        raise AssertionError('\\n is not a valid character '
+                                'in an author identity')
+                revprops['authors'] = '\n'.join(authors)
+        if author is not None:
+            symbol_versioning.warn('The parameter author was deprecated'
+                   ' in version 1.13. Use authors instead',
+                   DeprecationWarning)
+            if 'author' in revprops or 'authors' in revprops:
+                # XXX: maybe we should just accept one of them?
+                raise AssertionError('author property given twice')
+            if '\n' in author:
+                raise AssertionError('\\n is not a valid character '
+                        'in an author identity')
+            revprops['authors'] = author
+        return revprops
+
     def commit(self,
                message=None,
                timestamp=None,

=== modified file 'bzrlib/config.py'
--- bzrlib/config.py	2010-04-24 09:08:27 +0000
+++ bzrlib/config.py	2010-08-29 14:32:45 +0000
@@ -65,6 +65,7 @@
 import os
 import sys
 
+from bzrlib.decorators import needs_write_lock
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 import errno
@@ -74,13 +75,16 @@
 
 import bzrlib
 from bzrlib import (
+    atomicfile,
     debug,
     errors,
+    lockdir,
     mail_client,
     osutils,
     registry,
     symbol_versioning,
     trace,
+    transport,
     ui,
     urlutils,
     win32utils,
@@ -257,12 +261,10 @@
 
         Something similar to 'Martin Pool <mbp@sourcefrog.net>'
 
-        $BZR_EMAIL can be set to override this (as well as the
-        deprecated $BZREMAIL), then
+        $BZR_EMAIL can be set to override this, then
         the concrete policy type is checked, and finally
         $EMAIL is examined.
-        If none is found, a reasonable default is (hopefully)
-        created.
+        If no username can be found, errors.NoWhoami exception is raised.
 
         TODO: Check it's reasonably well-formed.
         """
@@ -278,11 +280,14 @@
         if v:
             return v.decode(osutils.get_user_encoding())
 
-        name, email = _auto_user_id()
-        if name:
-            return '%s <%s>' % (name, email)
-        else:
-            return email
+        raise errors.NoWhoami()
+
+    def ensure_username(self):
+        """Raise errors.NoWhoami if username is not set.
+
+        This method relies on the username() function raising the error.
+        """
+        self.username()
 
     def signature_checking(self):
         """What is the current policy for signature checking?."""
@@ -350,24 +355,79 @@
 class IniBasedConfig(Config):
     """A configuration policy that draws from ini files."""
 
-    def __init__(self, get_filename):
+    def __init__(self, get_filename=symbol_versioning.DEPRECATED_PARAMETER,
+                 file_name=None):
+        """Base class for configuration files using an ini-like syntax.
+
+        :param file_name: The configuration file path.
+        """
         super(IniBasedConfig, self).__init__()
-        self._get_filename = get_filename
+        self.file_name = file_name
+        if symbol_versioning.deprecated_passed(get_filename):
+            symbol_versioning.warn(
+                'IniBasedConfig.__init__(get_filename) was deprecated in 2.3.'
+                ' Use file_name instead.',
+                DeprecationWarning,
+                stacklevel=2)
+            if get_filename is not None:
+                self.file_name = get_filename()
+        else:
+            self.file_name = file_name
+        self._content = None
         self._parser = None
 
-    def _get_parser(self, file=None):
+    @classmethod
+    def from_string(cls, str_or_unicode, file_name=None, save=False):
+        """Create a config object from a string.
+
+        :param str_or_unicode: A string representing the file content. This will
+            be utf-8 encoded.
+
+        :param file_name: The configuration file path.
+
+        :param _save: Whether the file should be saved upon creation.
+        """
+        conf = cls(file_name=file_name)
+        conf._create_from_string(str_or_unicode, save)
+        return conf
+
+    def _create_from_string(self, str_or_unicode, save):
+        self._content = StringIO(str_or_unicode.encode('utf-8'))
+        # Some tests use in-memory configs, some other always need the config
+        # file to exist on disk.
+        if save:
+            self._write_config_file()
+
+    def _get_parser(self, file=symbol_versioning.DEPRECATED_PARAMETER):
         if self._parser is not None:
             return self._parser
-        if file is None:
-            input = self._get_filename()
+        if symbol_versioning.deprecated_passed(file):
+            symbol_versioning.warn(
+                'IniBasedConfig._get_parser(file=xxx) was deprecated in 2.3.'
+                ' Use IniBasedConfig(_content=xxx) instead.',
+                DeprecationWarning,
+                stacklevel=2)
+        if self._content is not None:
+            co_input = self._content
+        elif self.file_name is None:
+            raise AssertionError('We have no content to create the config')
         else:
-            input = file
+            co_input = self.file_name
         try:
-            self._parser = ConfigObj(input, encoding='utf-8')
+            self._parser = ConfigObj(co_input, encoding='utf-8')
         except configobj.ConfigObjError, e:
             raise errors.ParseConfigError(e.errors, e.config.filename)
+        # Make sure self.reload() will use the right file name
+        self._parser.filename = self.file_name
         return self._parser
 
+    def reload(self):
+        """Reload the config file from disk."""
+        if self.file_name is None:
+            raise AssertionError('We need a file name to reload the config')
+        if self._parser is not None:
+            self._parser.reload()
+
     def _get_matching_sections(self):
         """Return an ordered list of (section_name, extra_path) pairs.
 
@@ -476,16 +536,105 @@
     def _get_nickname(self):
         return self.get_user_option('nickname')
 
-
-class GlobalConfig(IniBasedConfig):
+    def _write_config_file(self):
+        if self.file_name is None:
+            raise AssertionError('We cannot save, self.file_name is None')
+        conf_dir = os.path.dirname(self.file_name)
+        ensure_config_dir_exists(conf_dir)
+        atomic_file = atomicfile.AtomicFile(self.file_name)
+        self._get_parser().write(atomic_file)
+        atomic_file.commit()
+        atomic_file.close()
+        osutils.copy_ownership_from_path(self.file_name)
+
+
+class LockableConfig(IniBasedConfig):
+    """A configuration needing explicit locking for access.
+
+    If several processes try to write the config file, the accesses need to be
+    serialized.
+
+    Daughter classes should decorate all methods that update a config with the
+    ``@needs_write_lock`` decorator (they call, directly or indirectly, the
+    ``_write_config_file()`` method. These methods (typically ``set_option()``
+    and variants must reload the config file from disk before calling
+    ``_write_config_file()``), this can be achieved by calling the
+    ``self.reload()`` method. Note that the lock scope should cover both the
+    reading and the writing of the config file which is why the decorator can't
+    be applied to ``_write_config_file()`` only.
+
+    This should be enough to implement the following logic:
+    - lock for exclusive write access,
+    - reload the config file from disk,
+    - set the new value
+    - unlock
+
+    This logic guarantees that a writer can update a value without erasing an
+    update made by another writer.
+    """
+
+    lock_name = 'lock'
+
+    def __init__(self, file_name):
+        super(LockableConfig, self).__init__(file_name=file_name)
+        self.dir = osutils.dirname(osutils.safe_unicode(self.file_name))
+        self.transport = transport.get_transport(self.dir)
+        self._lock = lockdir.LockDir(self.transport, 'lock')
+
+    def _create_from_string(self, unicode_bytes, save):
+        super(LockableConfig, self)._create_from_string(unicode_bytes, False)
+        if save:
+            # We need to handle the saving here (as opposed to IniBasedConfig)
+            # to be able to lock
+            self.lock_write()
+            self._write_config_file()
+            self.unlock()
+
+    def lock_write(self, token=None):
+        """Takes a write lock in the directory containing the config file.
+
+        If the directory doesn't exist it is created.
+        """
+        ensure_config_dir_exists(self.dir)
+        return self._lock.lock_write(token)
+
+    def unlock(self):
+        self._lock.unlock()
+
+    def break_lock(self):
+        self._lock.break_lock()
+
+    def _write_config_file(self):
+        if self._lock is None or not self._lock.is_held:
+            # NB: if the following exception is raised it probably means a
+            # missing @needs_write_lock decorator on one of the callers.
+            raise errors.ObjectNotLocked(self)
+        super(LockableConfig, self)._write_config_file()
+
+
+class GlobalConfig(LockableConfig):
     """The configuration that should be used for a specific location."""
 
+    def __init__(self):
+        super(GlobalConfig, self).__init__(file_name=config_filename())
+
+    @classmethod
+    def from_string(cls, str_or_unicode, save=False):
+        """Create a config object from a string.
+
+        :param str_or_unicode: A string representing the file content. This
+            will be utf-8 encoded.
+
+        :param save: Whether the file should be saved upon creation.
+        """
+        conf = cls()
+        conf._create_from_string(str_or_unicode, save)
+        return conf
+
     def get_editor(self):
         return self._get_user_option('editor')
 
-    def __init__(self):
-        super(GlobalConfig, self).__init__(config_filename)
-
+    @needs_write_lock
     def set_user_option(self, option, value):
         """Save option and its value in the configuration."""
         self._set_option(option, value, 'DEFAULT')
@@ -497,12 +646,15 @@
         else:
             return {}
 
+    @needs_write_lock
     def set_alias(self, alias_name, alias_command):
         """Save the alias in the configuration."""
         self._set_option(alias_name, alias_command, 'ALIASES')
 
+    @needs_write_lock
     def unset_alias(self, alias_name):
         """Unset an existing alias."""
+        self.reload()
         aliases = self._get_parser().get('ALIASES')
         if not aliases or alias_name not in aliases:
             raise errors.NoSuchAlias(alias_name)
@@ -510,37 +662,17 @@
         self._write_config_file()
 
     def _set_option(self, option, value, section):
-        # FIXME: RBC 20051029 This should refresh the parser and also take a
-        # file lock on bazaar.conf.
-        conf_dir = os.path.dirname(self._get_filename())
-        ensure_config_dir_exists(conf_dir)
+        self.reload()
         self._get_parser().setdefault(section, {})[option] = value
         self._write_config_file()
 
-    def _write_config_file(self):
-        path = self._get_filename()
-        f = open(path, 'wb')
-        osutils.copy_ownership_from_path(path)
-        self._get_parser().write(f)
-        f.close()
-
-
-class LocationConfig(IniBasedConfig):
+
+class LocationConfig(LockableConfig):
     """A configuration object that gives the policy for a location."""
 
     def __init__(self, location):
-        name_generator = locations_config_filename
-        if (not os.path.exists(name_generator()) and
-                os.path.exists(branches_config_filename())):
-            if sys.platform == 'win32':
-                trace.warning('Please rename %s to %s'
-                              % (branches_config_filename(),
-                                 locations_config_filename()))
-            else:
-                trace.warning('Please rename ~/.bazaar/branches.conf'
-                              ' to ~/.bazaar/locations.conf')
-            name_generator = branches_config_filename
-        super(LocationConfig, self).__init__(name_generator)
+        super(LocationConfig, self).__init__(
+            file_name=locations_config_filename())
         # local file locations are looked up by local path, rather than
         # by file url. This is because the config file is a user
         # file, and we would rather not expose the user to file urls.
@@ -548,6 +680,21 @@
             location = urlutils.local_path_from_url(location)
         self.location = location
 
+    @classmethod
+    def from_string(cls, str_or_unicode, location, save=False):
+        """Create a config object from a string.
+
+        :param str_or_unicode: A string representing the file content. This will
+            be utf-8 encoded.
+
+        :param location: The location url to filter the configuration.
+
+        :param save: Whether the file should be saved upon creation.
+        """
+        conf = cls(location)
+        conf._create_from_string(str_or_unicode, save)
+        return conf
+
     def _get_matching_sections(self):
         """Return an ordered list of section names matching this location."""
         sections = self._get_parser()
@@ -641,6 +788,7 @@
             if policy_key in self._get_parser()[section]:
                 del self._get_parser()[section][policy_key]
 
+    @needs_write_lock
     def set_user_option(self, option, value, store=STORE_LOCATION):
         """Save option and its value in the configuration."""
         if store not in [STORE_LOCATION,
@@ -648,27 +796,34 @@
                          STORE_LOCATION_APPENDPATH]:
             raise ValueError('bad storage policy %r for %r' %
                 (store, option))
-        # FIXME: RBC 20051029 This should refresh the parser and also take a
-        # file lock on locations.conf.
-        conf_dir = os.path.dirname(self._get_filename())
-        ensure_config_dir_exists(conf_dir)
+        self.reload()
         location = self.location
         if location.endswith('/'):
             location = location[:-1]
-        if (not location in self._get_parser() and
-            not location + '/' in self._get_parser()):
-            self._get_parser()[location]={}
-        elif location + '/' in self._get_parser():
+        parser = self._get_parser()
+        if not location in parser and not location + '/' in parser:
+            parser[location] = {}
+        elif location + '/' in parser:
             location = location + '/'
-        self._get_parser()[location][option]=value
+        parser[location][option]=value
         # the allowed values of store match the config policies
         self._set_option_policy(location, option, store)
-        self._get_parser().write(file(self._get_filename(), 'wb'))
+        self._write_config_file()
 
 
 class BranchConfig(Config):
     """A configuration object giving the policy for a branch."""
 
+    def __init__(self, branch):
+        super(BranchConfig, self).__init__()
+        self._location_config = None
+        self._branch_data_config = None
+        self._global_config = None
+        self.branch = branch
+        self.option_sources = (self._get_location_config,
+                               self._get_branch_data_config,
+                               self._get_global_config)
+
     def _get_branch_data_config(self):
         if self._branch_data_config is None:
             self._branch_data_config = TreeConfig(self.branch)
@@ -772,16 +927,6 @@
         """See Config.gpg_signing_command."""
         return self._get_safe_value('_gpg_signing_command')
 
-    def __init__(self, branch):
-        super(BranchConfig, self).__init__()
-        self._location_config = None
-        self._branch_data_config = None
-        self._global_config = None
-        self.branch = branch
-        self.option_sources = (self._get_location_config,
-                               self._get_branch_data_config,
-                               self._get_global_config)
-
     def _post_commit(self):
         """See Config.post_commit."""
         return self._get_safe_value('_post_commit')
@@ -841,7 +986,6 @@
                                   ' or HOME set')
         return osutils.pathjoin(base, 'bazaar', '2.0')
     else:
-        # cygwin, linux, and darwin all have a $HOME directory
         if base is None:
             base = os.path.expanduser("~")
         return osutils.pathjoin(base, ".bazaar")
@@ -852,11 +996,6 @@
     return osutils.pathjoin(config_dir(), 'bazaar.conf')
 
 
-def branches_config_filename():
-    """Return per-user configuration ini file filename."""
-    return osutils.pathjoin(config_dir(), 'branches.conf')
-
-
 def locations_config_filename():
     """Return per-user configuration ini file filename."""
     return osutils.pathjoin(config_dir(), 'locations.conf')
@@ -899,79 +1038,6 @@
         return os.path.expanduser('~/.cache')
 
 
-def _auto_user_id():
-    """Calculate automatic user identification.
-
-    Returns (realname, email).
-
-    Only used when none is set in the environment or the id file.
-
-    This previously used the FQDN as the default domain, but that can
-    be very slow on machines where DNS is broken.  So now we simply
-    use the hostname.
-    """
-    import socket
-
-    if sys.platform == 'win32':
-        name = win32utils.get_user_name_unicode()
-        if name is None:
-            raise errors.BzrError("Cannot autodetect user name.\n"
-                                  "Please, set your name with command like:\n"
-                                  'bzr whoami "Your Name <name@domain.com>"')
-        host = win32utils.get_host_name_unicode()
-        if host is None:
-            host = socket.gethostname()
-        return name, (name + '@' + host)
-
-    try:
-        import pwd
-        uid = os.getuid()
-        try:
-            w = pwd.getpwuid(uid)
-        except KeyError:
-            raise errors.BzrCommandError('Unable to determine your name.  '
-                'Please use "bzr whoami" to set it.')
-
-        # we try utf-8 first, because on many variants (like Linux),
-        # /etc/passwd "should" be in utf-8, and because it's unlikely to give
-        # false positives.  (many users will have their user encoding set to
-        # latin-1, which cannot raise UnicodeError.)
-        try:
-            gecos = w.pw_gecos.decode('utf-8')
-            encoding = 'utf-8'
-        except UnicodeError:
-            try:
-                encoding = osutils.get_user_encoding()
-                gecos = w.pw_gecos.decode(encoding)
-            except UnicodeError:
-                raise errors.BzrCommandError('Unable to determine your name.  '
-                   'Use "bzr whoami" to set it.')
-        try:
-            username = w.pw_name.decode(encoding)
-        except UnicodeError:
-            raise errors.BzrCommandError('Unable to determine your name.  '
-                'Use "bzr whoami" to set it.')
-
-        comma = gecos.find(',')
-        if comma == -1:
-            realname = gecos
-        else:
-            realname = gecos[:comma]
-        if not realname:
-            realname = username
-
-    except ImportError:
-        import getpass
-        try:
-            user_encoding = osutils.get_user_encoding()
-            realname = username = getpass.getuser().decode(user_encoding)
-        except UnicodeDecodeError:
-            raise errors.BzrError("Can't decode username as %s." % \
-                    user_encoding)
-
-    return realname, (username + '@' + socket.gethostname())
-
-
 def parse_username(username):
     """Parse e-mail username and return a (name, address) tuple."""
     match = re.match(r'(.*?)\s*<?([\w+.-]+@[\w+.-]+)>?', username)
@@ -1063,7 +1129,11 @@
         """Save the config file, only tests should use it for now."""
         conf_dir = os.path.dirname(self._filename)
         ensure_config_dir_exists(conf_dir)
-        self._get_config().write(file(self._filename, 'wb'))
+        f = file(self._filename, 'wb')
+        try:
+            self._get_config().write(f)
+        finally:
+            f.close()
 
     def _set_option(self, section_name, option_name, value):
         """Set an authentication configuration option"""
@@ -1517,7 +1587,11 @@
             return StringIO()
 
     def _get_configobj(self):
-        return ConfigObj(self._get_config_file(), encoding='utf-8')
+        f = self._get_config_file()
+        try:
+            return ConfigObj(f, encoding='utf-8')
+        finally:
+            f.close()
 
     def _set_configobj(self, configobj):
         out_file = StringIO()

=== modified file 'bzrlib/conflicts.py'
--- bzrlib/conflicts.py	2010-04-30 11:35:43 +0000
+++ bzrlib/conflicts.py	2010-07-16 15:35:43 +0000
@@ -24,7 +24,6 @@
 import errno
 
 from bzrlib import (
-    builtins,
     cleanup,
     commands,
     errors,
@@ -59,13 +58,14 @@
     Use bzr resolve when you have fixed a problem.
     """
     takes_options = [
+            'directory',
             option.Option('text',
                           help='List paths of files with text conflicts.'),
         ]
     _see_also = ['resolve', 'conflict-types']
 
-    def run(self, text=False):
-        wt = workingtree.WorkingTree.open_containing(u'.')[0]
+    def run(self, text=False, directory=u'.'):
+        wt = workingtree.WorkingTree.open_containing(directory)[0]
         for conflict in wt.conflicts():
             if text:
                 if conflict.typestring != 'text conflict':
@@ -112,20 +112,22 @@
     aliases = ['resolved']
     takes_args = ['file*']
     takes_options = [
+            'directory',
             option.Option('all', help='Resolve all conflicts in this tree.'),
             ResolveActionOption(),
             ]
     _see_also = ['conflicts']
-    def run(self, file_list=None, all=False, action=None):
+    def run(self, file_list=None, all=False, action=None, directory=u'.'):
         if all:
             if file_list:
                 raise errors.BzrCommandError("If --all is specified,"
                                              " no FILE may be provided")
-            tree = workingtree.WorkingTree.open_containing('.')[0]
+            tree = workingtree.WorkingTree.open_containing(directory)[0]
             if action is None:
                 action = 'done'
         else:
-            tree, file_list = builtins.tree_files(file_list)
+            tree, file_list = workingtree.WorkingTree.open_containing_paths(
+                file_list)
             if file_list is None:
                 if action is None:
                     # FIXME: There is a special case here related to the option

=== added file 'bzrlib/controldir.py'
--- bzrlib/controldir.py	1970-01-01 00:00:00 +0000
+++ bzrlib/controldir.py	2010-09-10 09:46:15 +0000
@@ -0,0 +1,977 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""ControlDir is the basic control directory class.
+
+The ControlDir class is the base for the control directory used
+by all bzr and foreign formats. For the ".bzr" implementation,
+see bzrlib.bzrdir.BzrDir.
+
+"""
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+import textwrap
+
+from bzrlib import (
+    errors,
+    graph,
+    registry,
+    revision as _mod_revision,
+    symbol_versioning,
+    urlutils,
+    )
+from bzrlib.push import (
+    PushResult,
+    )
+from bzrlib.trace import (
+    mutter,
+    )
+from bzrlib.transport import (
+    get_transport,
+    local,
+    )
+
+""")
+
+
+class ControlComponent(object):
+    """Abstract base class for control directory components.
+
+    This provides interfaces that are common across controldirs,
+    repositories, branches, and workingtree control directories.
+
+    They all expose two urls and transports: the *user* URL is the
+    one that stops above the control directory (eg .bzr) and that
+    should normally be used in messages, and the *control* URL is
+    under that in eg .bzr/checkout and is used to read the control
+    files.
+
+    This can be used as a mixin and is intended to fit with
+    foreign formats.
+    """
+
+    @property
+    def control_transport(self):
+        raise NotImplementedError
+
+    @property
+    def control_url(self):
+        return self.control_transport.base
+
+    @property
+    def user_transport(self):
+        raise NotImplementedError
+
+    @property
+    def user_url(self):
+        return self.user_transport.base
+
+
+class ControlDir(ControlComponent):
+    """A control directory.
+
+    While this represents a generic control directory, there are a few
+    features that are present in this interface that are currently only
+    supported by one of its implementations, BzrDir.
+
+    These features (bound branches, stacked branches) are currently only
+    supported by Bazaar, but could be supported by other version control
+    systems as well. Implementations are required to raise the appropriate
+    exceptions when an operation is requested that is not supported.
+
+    This also makes life easier for API users who can rely on the
+    implementation always allowing a particular feature to be requested but
+    raising an exception when it is not supported, rather than requiring the
+    API users to check for magic attributes to see what features are supported.
+    """
+
+    def can_convert_format(self):
+        """Return true if this controldir is one whose format we can convert
+        from."""
+        return True
+
+    def list_branches(self):
+        """Return a sequence of all branches local to this control directory.
+
+        """
+        try:
+            return [self.open_branch()]
+        except (errors.NotBranchError, errors.NoRepositoryPresent):
+            return []
+
+    def is_control_filename(self, filename):
+        """True if filename is the name of a path which is reserved for
+        controldirs.
+
+        :param filename: A filename within the root transport of this
+            controldir.
+
+        This is true IF and ONLY IF the filename is part of the namespace reserved
+        for bzr control dirs. Currently this is the '.bzr' directory in the root
+        of the root_transport. it is expected that plugins will need to extend
+        this in the future - for instance to make bzr talk with svn working
+        trees.
+        """
+        raise NotImplementedError(self.is_control_filename)
+
+    def needs_format_conversion(self, format=None):
+        """Return true if this controldir needs convert_format run on it.
+
+        For instance, if the repository format is out of date but the
+        branch and working tree are not, this should return True.
+
+        :param format: Optional parameter indicating a specific desired
+                       format we plan to arrive at.
+        """
+        raise NotImplementedError(self.needs_format_conversion)
+
+    def destroy_repository(self):
+        """Destroy the repository in this ControlDir."""
+        raise NotImplementedError(self.destroy_repository)
+
+    def create_branch(self, name=None):
+        """Create a branch in this ControlDir.
+
+        :param name: Name of the colocated branch to create, None for
+            the default branch.
+
+        The controldirs format will control what branch format is created.
+        For more control see BranchFormatXX.create(a_controldir).
+        """
+        raise NotImplementedError(self.create_branch)
+
+    def destroy_branch(self, name=None):
+        """Destroy a branch in this ControlDir.
+
+        :param name: Name of the branch to destroy, None for the default 
+            branch.
+        """
+        raise NotImplementedError(self.destroy_branch)
+
+    def create_workingtree(self, revision_id=None, from_branch=None,
+        accelerator_tree=None, hardlink=False):
+        """Create a working tree at this ControlDir.
+
+        :param revision_id: create it as of this revision id.
+        :param from_branch: override controldir branch 
+            (for lightweight checkouts)
+        :param accelerator_tree: A tree which can be used for retrieving file
+            contents more quickly than the revision tree, i.e. a workingtree.
+            The revision tree will be used for cases where accelerator_tree's
+            content is different.
+        """
+        raise NotImplementedError(self.create_workingtree)
+
+    def destroy_workingtree(self):
+        """Destroy the working tree at this ControlDir.
+
+        Formats that do not support this may raise UnsupportedOperation.
+        """
+        raise NotImplementedError(self.destroy_workingtree)
+
+    def destroy_workingtree_metadata(self):
+        """Destroy the control files for the working tree at this ControlDir.
+
+        The contents of working tree files are not affected.
+        Formats that do not support this may raise UnsupportedOperation.
+        """
+        raise NotImplementedError(self.destroy_workingtree_metadata)
+
+    def get_branch_reference(self, name=None):
+        """Return the referenced URL for the branch in this controldir.
+
+        :param name: Optional colocated branch name
+        :raises NotBranchError: If there is no Branch.
+        :raises NoColocatedBranchSupport: If a branch name was specified
+            but colocated branches are not supported.
+        :return: The URL the branch in this controldir references if it is a
+            reference branch, or None for regular branches.
+        """
+        if name is not None:
+            raise errors.NoColocatedBranchSupport(self)
+        return None
+
+    def get_branch_transport(self, branch_format, name=None):
+        """Get the transport for use by branch format in this ControlDir.
+
+        Note that bzr dirs that do not support format strings will raise
+        IncompatibleFormat if the branch format they are given has
+        a format string, and vice versa.
+
+        If branch_format is None, the transport is returned with no
+        checking. If it is not None, then the returned transport is
+        guaranteed to point to an existing directory ready for use.
+        """
+        raise NotImplementedError(self.get_branch_transport)
+
+    def get_repository_transport(self, repository_format):
+        """Get the transport for use by repository format in this ControlDir.
+
+        Note that bzr dirs that do not support format strings will raise
+        IncompatibleFormat if the repository format they are given has
+        a format string, and vice versa.
+
+        If repository_format is None, the transport is returned with no
+        checking. If it is not None, then the returned transport is
+        guaranteed to point to an existing directory ready for use.
+        """
+        raise NotImplementedError(self.get_repository_transport)
+
+    def get_workingtree_transport(self, tree_format):
+        """Get the transport for use by workingtree format in this ControlDir.
+
+        Note that bzr dirs that do not support format strings will raise
+        IncompatibleFormat if the workingtree format they are given has a
+        format string, and vice versa.
+
+        If workingtree_format is None, the transport is returned with no
+        checking. If it is not None, then the returned transport is
+        guaranteed to point to an existing directory ready for use.
+        """
+        raise NotImplementedError(self.get_workingtree_transport)
+
+    def open_branch(self, name=None, unsupported=False,
+                    ignore_fallbacks=False):
+        """Open the branch object at this ControlDir if one is present.
+
+        If unsupported is True, then no longer supported branch formats can
+        still be opened.
+
+        TODO: static convenience version of this?
+        """
+        raise NotImplementedError(self.open_branch)
+
+    def open_repository(self, _unsupported=False):
+        """Open the repository object at this ControlDir if one is present.
+
+        This will not follow the Branch object pointer - it's strictly a direct
+        open facility. Most client code should use open_branch().repository to
+        get at a repository.
+
+        :param _unsupported: a private parameter, not part of the api.
+        TODO: static convenience version of this?
+        """
+        raise NotImplementedError(self.open_repository)
+
+    def find_repository(self):
+        """Find the repository that should be used.
+
+        This does not require a branch as we use it to find the repo for
+        new branches as well as to hook existing branches up to their
+        repository.
+        """
+        raise NotImplementedError(self.find_repository)
+
+    def open_workingtree(self, _unsupported=False,
+                         recommend_upgrade=True, from_branch=None):
+        """Open the workingtree object at this ControlDir if one is present.
+
+        :param recommend_upgrade: Optional keyword parameter, when True (the
+            default), emit through the ui module a recommendation that the user
+            upgrade the working tree when the workingtree being opened is old
+            (but still fully supported).
+        :param from_branch: override controldir branch (for lightweight
+            checkouts)
+        """
+        raise NotImplementedError(self.open_workingtree)
+
+    def has_branch(self, name=None):
+        """Tell if this controldir contains a branch.
+
+        Note: if you're going to open the branch, you should just go ahead
+        and try, and not ask permission first.  (This method just opens the
+        branch and discards it, and that's somewhat expensive.)
+        """
+        try:
+            self.open_branch(name)
+            return True
+        except errors.NotBranchError:
+            return False
+
+    def has_workingtree(self):
+        """Tell if this controldir contains a working tree.
+
+        This will still raise an exception if the controldir has a workingtree
+        that is remote & inaccessible.
+
+        Note: if you're going to open the working tree, you should just go ahead
+        and try, and not ask permission first.  (This method just opens the
+        workingtree and discards it, and that's somewhat expensive.)
+        """
+        try:
+            self.open_workingtree(recommend_upgrade=False)
+            return True
+        except errors.NoWorkingTree:
+            return False
+
+    def cloning_metadir(self, require_stacking=False):
+        """Produce a metadir suitable for cloning or sprouting with.
+
+        These operations may produce workingtrees (yes, even though they're
+        "cloning" something that doesn't have a tree), so a viable workingtree
+        format must be selected.
+
+        :require_stacking: If True, non-stackable formats will be upgraded
+            to similar stackable formats.
+        :returns: a ControlDirFormat with all component formats either set
+            appropriately or set to None if that component should not be
+            created.
+        """
+        raise NotImplementedError(self.cloning_metadir)
+
+    def checkout_metadir(self):
+        """Produce a metadir suitable for checkouts of this controldir."""
+        return self.cloning_metadir()
+
+    def sprout(self, url, revision_id=None, force_new_repo=False,
+               recurse='down', possible_transports=None,
+               accelerator_tree=None, hardlink=False, stacked=False,
+               source_branch=None, create_tree_if_local=True):
+        """Create a copy of this controldir prepared for use as a new line of
+        development.
+
+        If url's last component does not exist, it will be created.
+
+        Attributes related to the identity of the source branch like
+        branch nickname will be cleaned, a working tree is created
+        whether one existed before or not; and a local branch is always
+        created.
+
+        if revision_id is not None, then the clone operation may tune
+            itself to download less data.
+        :param accelerator_tree: A tree which can be used for retrieving file
+            contents more quickly than the revision tree, i.e. a workingtree.
+            The revision tree will be used for cases where accelerator_tree's
+            content is different.
+        :param hardlink: If true, hard-link files from accelerator_tree,
+            where possible.
+        :param stacked: If true, create a stacked branch referring to the
+            location of this control directory.
+        :param create_tree_if_local: If true, a working-tree will be created
+            when working locally.
+        """
+        target_transport = get_transport(url, possible_transports)
+        target_transport.ensure_base()
+        cloning_format = self.cloning_metadir(stacked)
+        # Create/update the result branch
+        result = cloning_format.initialize_on_transport(target_transport)
+        # if a stacked branch wasn't requested, we don't create one
+        # even if the origin was stacked
+        stacked_branch_url = None
+        if source_branch is not None:
+            if stacked:
+                stacked_branch_url = self.root_transport.base
+            source_repository = source_branch.repository
+        else:
+            try:
+                source_branch = self.open_branch()
+                source_repository = source_branch.repository
+                if stacked:
+                    stacked_branch_url = self.root_transport.base
+            except errors.NotBranchError:
+                source_branch = None
+                try:
+                    source_repository = self.open_repository()
+                except errors.NoRepositoryPresent:
+                    source_repository = None
+        repository_policy = result.determine_repository_policy(
+            force_new_repo, stacked_branch_url, require_stacking=stacked)
+        result_repo, is_new_repo = repository_policy.acquire_repository()
+        is_stacked = stacked or (len(result_repo._fallback_repositories) != 0)
+        if is_new_repo and revision_id is not None and not is_stacked:
+            fetch_spec = graph.PendingAncestryResult(
+                [revision_id], source_repository)
+        else:
+            fetch_spec = None
+        if source_repository is not None:
+            # Fetch while stacked to prevent unstacked fetch from
+            # Branch.sprout.
+            if fetch_spec is None:
+                result_repo.fetch(source_repository, revision_id=revision_id)
+            else:
+                result_repo.fetch(source_repository, fetch_spec=fetch_spec)
+
+        if source_branch is None:
+            # this is for sprouting a controldir without a branch; is that
+            # actually useful?
+            # Not especially, but it's part of the contract.
+            result_branch = result.create_branch()
+        else:
+            result_branch = source_branch.sprout(result,
+                revision_id=revision_id, repository_policy=repository_policy)
+        mutter("created new branch %r" % (result_branch,))
+
+        # Create/update the result working tree
+        if (create_tree_if_local and
+            isinstance(target_transport, local.LocalTransport) and
+            (result_repo is None or result_repo.make_working_trees())):
+            wt = result.create_workingtree(accelerator_tree=accelerator_tree,
+                hardlink=hardlink)
+            wt.lock_write()
+            try:
+                if wt.path2id('') is None:
+                    try:
+                        wt.set_root_id(self.open_workingtree.get_root_id())
+                    except errors.NoWorkingTree:
+                        pass
+            finally:
+                wt.unlock()
+        else:
+            wt = None
+        if recurse == 'down':
+            if wt is not None:
+                basis = wt.basis_tree()
+                basis.lock_read()
+                subtrees = basis.iter_references()
+            elif result_branch is not None:
+                basis = result_branch.basis_tree()
+                basis.lock_read()
+                subtrees = basis.iter_references()
+            elif source_branch is not None:
+                basis = source_branch.basis_tree()
+                basis.lock_read()
+                subtrees = basis.iter_references()
+            else:
+                subtrees = []
+                basis = None
+            try:
+                for path, file_id in subtrees:
+                    target = urlutils.join(url, urlutils.escape(path))
+                    sublocation = source_branch.reference_parent(file_id, path)
+                    sublocation.bzrdir.sprout(target,
+                        basis.get_reference_revision(file_id, path),
+                        force_new_repo=force_new_repo, recurse=recurse,
+                        stacked=stacked)
+            finally:
+                if basis is not None:
+                    basis.unlock()
+        return result
+
+    def push_branch(self, source, revision_id=None, overwrite=False, 
+        remember=False, create_prefix=False):
+        """Push the source branch into this ControlDir."""
+        br_to = None
+        # If we can open a branch, use its direct repository, otherwise see
+        # if there is a repository without a branch.
+        try:
+            br_to = self.open_branch()
+        except errors.NotBranchError:
+            # Didn't find a branch, can we find a repository?
+            repository_to = self.find_repository()
+        else:
+            # Found a branch, so we must have found a repository
+            repository_to = br_to.repository
+
+        push_result = PushResult()
+        push_result.source_branch = source
+        if br_to is None:
+            # We have a repository but no branch, copy the revisions, and then
+            # create a branch.
+            repository_to.fetch(source.repository, revision_id=revision_id)
+            br_to = source.clone(self, revision_id=revision_id)
+            if source.get_push_location() is None or remember:
+                source.set_push_location(br_to.base)
+            push_result.stacked_on = None
+            push_result.branch_push_result = None
+            push_result.old_revno = None
+            push_result.old_revid = _mod_revision.NULL_REVISION
+            push_result.target_branch = br_to
+            push_result.master_branch = None
+            push_result.workingtree_updated = False
+        else:
+            # We have successfully opened the branch, remember if necessary:
+            if source.get_push_location() is None or remember:
+                source.set_push_location(br_to.base)
+            try:
+                tree_to = self.open_workingtree()
+            except errors.NotLocalUrl:
+                push_result.branch_push_result = source.push(br_to, 
+                    overwrite, stop_revision=revision_id)
+                push_result.workingtree_updated = False
+            except errors.NoWorkingTree:
+                push_result.branch_push_result = source.push(br_to,
+                    overwrite, stop_revision=revision_id)
+                push_result.workingtree_updated = None # Not applicable
+            else:
+                tree_to.lock_write()
+                try:
+                    push_result.branch_push_result = source.push(
+                        tree_to.branch, overwrite, stop_revision=revision_id)
+                    tree_to.update()
+                finally:
+                    tree_to.unlock()
+                push_result.workingtree_updated = True
+            push_result.old_revno = push_result.branch_push_result.old_revno
+            push_result.old_revid = push_result.branch_push_result.old_revid
+            push_result.target_branch = \
+                push_result.branch_push_result.target_branch
+        return push_result
+
+    def _get_tree_branch(self, name=None):
+        """Return the branch and tree, if any, for this bzrdir.
+
+        :param name: Name of colocated branch to open.
+
+        Return None for tree if not present or inaccessible.
+        Raise NotBranchError if no branch is present.
+        :return: (tree, branch)
+        """
+        try:
+            tree = self.open_workingtree()
+        except (errors.NoWorkingTree, errors.NotLocalUrl):
+            tree = None
+            branch = self.open_branch(name=name)
+        else:
+            if name is not None:
+                branch = self.open_branch(name=name)
+            else:
+                branch = tree.branch
+        return tree, branch
+
+    def get_config(self):
+        """Get configuration for this ControlDir."""
+        raise NotImplementedError(self.get_config)
+
+    def check_conversion_target(self, target_format):
+        """Check that a bzrdir as a whole can be converted to a new format."""
+        raise NotImplementedError(self.check_conversion_target)
+
+    def clone(self, url, revision_id=None, force_new_repo=False,
+              preserve_stacking=False):
+        """Clone this bzrdir and its contents to url verbatim.
+
+        :param url: The url create the clone at.  If url's last component does
+            not exist, it will be created.
+        :param revision_id: The tip revision-id to use for any branch or
+            working tree.  If not None, then the clone operation may tune
+            itself to download less data.
+        :param force_new_repo: Do not use a shared repository for the target
+                               even if one is available.
+        :param preserve_stacking: When cloning a stacked branch, stack the
+            new branch on top of the other branch's stacked-on branch.
+        """
+        return self.clone_on_transport(get_transport(url),
+                                       revision_id=revision_id,
+                                       force_new_repo=force_new_repo,
+                                       preserve_stacking=preserve_stacking)
+
+    def clone_on_transport(self, transport, revision_id=None,
+        force_new_repo=False, preserve_stacking=False, stacked_on=None,
+        create_prefix=False, use_existing_dir=True):
+        """Clone this bzrdir and its contents to transport verbatim.
+
+        :param transport: The transport for the location to produce the clone
+            at.  If the target directory does not exist, it will be created.
+        :param revision_id: The tip revision-id to use for any branch or
+            working tree.  If not None, then the clone operation may tune
+            itself to download less data.
+        :param force_new_repo: Do not use a shared repository for the target,
+                               even if one is available.
+        :param preserve_stacking: When cloning a stacked branch, stack the
+            new branch on top of the other branch's stacked-on branch.
+        :param create_prefix: Create any missing directories leading up to
+            to_transport.
+        :param use_existing_dir: Use an existing directory if one exists.
+        """
+        raise NotImplementedError(self.clone_on_transport)
+
+
+class ControlDirFormat(object):
+    """An encapsulation of the initialization and open routines for a format.
+
+    Formats provide three things:
+     * An initialization routine,
+     * a format string,
+     * an open routine.
+
+    Formats are placed in a dict by their format string for reference
+    during controldir opening. These should be subclasses of ControlDirFormat
+    for consistency.
+
+    Once a format is deprecated, just deprecate the initialize and open
+    methods on the format class. Do not deprecate the object, as the
+    object will be created every system load.
+
+    :cvar colocated_branches: Whether this formats supports colocated branches.
+    :cvar supports_workingtrees: This control directory can co-exist with a
+        working tree.
+    """
+
+    _default_format = None
+    """The default format used for new control directories."""
+
+    _formats = []
+    """The registered control formats - .bzr, ....
+
+    This is a list of ControlDirFormat objects.
+    """
+
+    _server_probers = []
+    """The registered server format probers, e.g. RemoteBzrProber.
+
+    This is a list of Prober-derived classes.
+    """
+
+    _probers = []
+    """The registered format probers, e.g. BzrProber.
+
+    This is a list of Prober-derived classes.
+    """
+
+    colocated_branches = False
+    """Whether co-located branches are supported for this control dir format.
+    """
+
+    supports_workingtrees = True
+
+    def get_format_description(self):
+        """Return the short description for this format."""
+        raise NotImplementedError(self.get_format_description)
+
+    def get_converter(self, format=None):
+        """Return the converter to use to convert controldirs needing converts.
+
+        This returns a bzrlib.controldir.Converter object.
+
+        This should return the best upgrader to step this format towards the
+        current default format. In the case of plugins we can/should provide
+        some means for them to extend the range of returnable converters.
+
+        :param format: Optional format to override the default format of the
+                       library.
+        """
+        raise NotImplementedError(self.get_converter)
+
+    def is_supported(self):
+        """Is this format supported?
+
+        Supported formats must be initializable and openable.
+        Unsupported formats may not support initialization or committing or
+        some other features depending on the reason for not being supported.
+        """
+        return True
+
+    def same_model(self, target_format):
+        return (self.repository_format.rich_root_data ==
+            target_format.rich_root_data)
+
+    @classmethod
+    def register_format(klass, format):
+        """Register a format that does not use '.bzr' for its control dir.
+
+        """
+        klass._formats.append(format)
+
+    @classmethod
+    def register_prober(klass, prober):
+        """Register a prober that can look for a control dir.
+
+        """
+        klass._probers.append(prober)
+
+    @classmethod
+    def unregister_prober(klass, prober):
+        """Unregister a prober.
+
+        """
+        klass._probers.remove(prober)
+
+    @classmethod
+    def register_server_prober(klass, prober):
+        """Register a control format prober for client-server environments.
+
+        These probers will be used before ones registered with
+        register_prober.  This gives implementations that decide to the
+        chance to grab it before anything looks at the contents of the format
+        file.
+        """
+        klass._server_probers.append(prober)
+
+    def __str__(self):
+        # Trim the newline
+        return self.get_format_description().rstrip()
+
+    @classmethod
+    def unregister_format(klass, format):
+        klass._formats.remove(format)
+
+    @classmethod
+    def known_formats(klass):
+        """Return all the known formats.
+        """
+        return set(klass._formats)
+
+    @classmethod
+    def find_format(klass, transport, _server_formats=True):
+        """Return the format present at transport."""
+        if _server_formats:
+            _probers = klass._server_probers + klass._probers
+        else:
+            _probers = klass._probers
+        for prober_kls in _probers:
+            prober = prober_kls()
+            try:
+                return prober.probe_transport(transport)
+            except errors.NotBranchError:
+                # this format does not find a control dir here.
+                pass
+        raise errors.NotBranchError(path=transport.base)
+
+    def initialize(self, url, possible_transports=None):
+        """Create a control dir at this url and return an opened copy.
+
+        While not deprecated, this method is very specific and its use will
+        lead to many round trips to setup a working environment. See
+        initialize_on_transport_ex for a [nearly] all-in-one method.
+
+        Subclasses should typically override initialize_on_transport
+        instead of this method.
+        """
+        return self.initialize_on_transport(get_transport(url,
+                                                          possible_transports))
+    def initialize_on_transport(self, transport):
+        """Initialize a new controldir in the base directory of a Transport."""
+        raise NotImplementedError(self.initialize_on_transport)
+
+    def initialize_on_transport_ex(self, transport, use_existing_dir=False,
+        create_prefix=False, force_new_repo=False, stacked_on=None,
+        stack_on_pwd=None, repo_format_name=None, make_working_trees=None,
+        shared_repo=False, vfs_only=False):
+        """Create this format on transport.
+
+        The directory to initialize will be created.
+
+        :param force_new_repo: Do not use a shared repository for the target,
+                               even if one is available.
+        :param create_prefix: Create any missing directories leading up to
+            to_transport.
+        :param use_existing_dir: Use an existing directory if one exists.
+        :param stacked_on: A url to stack any created branch on, None to follow
+            any target stacking policy.
+        :param stack_on_pwd: If stack_on is relative, the location it is
+            relative to.
+        :param repo_format_name: If non-None, a repository will be
+            made-or-found. Should none be found, or if force_new_repo is True
+            the repo_format_name is used to select the format of repository to
+            create.
+        :param make_working_trees: Control the setting of make_working_trees
+            for a new shared repository when one is made. None to use whatever
+            default the format has.
+        :param shared_repo: Control whether made repositories are shared or
+            not.
+        :param vfs_only: If True do not attempt to use a smart server
+        :return: repo, controldir, require_stacking, repository_policy. repo is
+            None if none was created or found, controldir is always valid.
+            require_stacking is the result of examining the stacked_on
+            parameter and any stacking policy found for the target.
+        """
+        raise NotImplementedError(self.initialize_on_transport_ex)
+
+    def network_name(self):
+        """A simple byte string uniquely identifying this format for RPC calls.
+
+        Bzr control formats use this disk format string to identify the format
+        over the wire. Its possible that other control formats have more
+        complex detection requirements, so we permit them to use any unique and
+        immutable string they desire.
+        """
+        raise NotImplementedError(self.network_name)
+
+    def open(self, transport, _found=False):
+        """Return an instance of this format for the dir transport points at.
+        """
+        raise NotImplementedError(self.open)
+
+    @classmethod
+    def _set_default_format(klass, format):
+        """Set default format (for testing behavior of defaults only)"""
+        klass._default_format = format
+
+    @classmethod
+    def get_default_format(klass):
+        """Return the current default format."""
+        return klass._default_format
+
+
+class Prober(object):
+    """Abstract class that can be used to detect a particular kind of 
+    control directory.
+
+    At the moment this just contains a single method to probe a particular 
+    transport, but it may be extended in the future to e.g. avoid 
+    multiple levels of probing for Subversion repositories.
+    """
+
+    def probe_transport(self, transport):
+        """Return the controldir style format present in a directory.
+
+        :raise UnknownFormatError: If a control dir was found but is
+            in an unknown format.
+        :raise NotBranchError: If no control directory was found.
+        :return: A ControlDirFormat instance.
+        """
+        raise NotImplementedError(self.probe_transport)
+
+
+class ControlDirFormatInfo(object):
+
+    def __init__(self, native, deprecated, hidden, experimental):
+        self.deprecated = deprecated
+        self.native = native
+        self.hidden = hidden
+        self.experimental = experimental
+
+
+class ControlDirFormatRegistry(registry.Registry):
+    """Registry of user-selectable ControlDir subformats.
+
+    Differs from ControlDirFormat._formats in that it provides sub-formats,
+    e.g. ControlDirMeta1 with weave repository.  Also, it's more user-oriented.
+    """
+
+    def __init__(self):
+        """Create a ControlDirFormatRegistry."""
+        self._aliases = set()
+        self._registration_order = list()
+        super(ControlDirFormatRegistry, self).__init__()
+
+    def aliases(self):
+        """Return a set of the format names which are aliases."""
+        return frozenset(self._aliases)
+
+    def register(self, key, factory, help, native=True, deprecated=False,
+                 hidden=False, experimental=False, alias=False):
+        """Register a ControlDirFormat factory.
+
+        The factory must be a callable that takes one parameter: the key.
+        It must produce an instance of the ControlDirFormat when called.
+
+        This function mainly exists to prevent the info object from being
+        supplied directly.
+        """
+        registry.Registry.register(self, key, factory, help,
+            ControlDirFormatInfo(native, deprecated, hidden, experimental))
+        if alias:
+            self._aliases.add(key)
+        self._registration_order.append(key)
+
+    def register_lazy(self, key, module_name, member_name, help, native=True,
+        deprecated=False, hidden=False, experimental=False, alias=False):
+        registry.Registry.register_lazy(self, key, module_name, member_name,
+            help, ControlDirFormatInfo(native, deprecated, hidden, experimental))
+        if alias:
+            self._aliases.add(key)
+        self._registration_order.append(key)
+
+    def set_default(self, key):
+        """Set the 'default' key to be a clone of the supplied key.
+
+        This method must be called once and only once.
+        """
+        registry.Registry.register(self, 'default', self.get(key),
+            self.get_help(key), info=self.get_info(key))
+        self._aliases.add('default')
+
+    def set_default_repository(self, key):
+        """Set the FormatRegistry default and Repository default.
+
+        This is a transitional method while Repository.set_default_format
+        is deprecated.
+        """
+        if 'default' in self:
+            self.remove('default')
+        self.set_default(key)
+        format = self.get('default')()
+
+    def make_bzrdir(self, key):
+        return self.get(key)()
+
+    def help_topic(self, topic):
+        output = ""
+        default_realkey = None
+        default_help = self.get_help('default')
+        help_pairs = []
+        for key in self._registration_order:
+            if key == 'default':
+                continue
+            help = self.get_help(key)
+            if help == default_help:
+                default_realkey = key
+            else:
+                help_pairs.append((key, help))
+
+        def wrapped(key, help, info):
+            if info.native:
+                help = '(native) ' + help
+            return ':%s:\n%s\n\n' % (key,
+                textwrap.fill(help, initial_indent='    ',
+                    subsequent_indent='    ',
+                    break_long_words=False))
+        if default_realkey is not None:
+            output += wrapped(default_realkey, '(default) %s' % default_help,
+                              self.get_info('default'))
+        deprecated_pairs = []
+        experimental_pairs = []
+        for key, help in help_pairs:
+            info = self.get_info(key)
+            if info.hidden:
+                continue
+            elif info.deprecated:
+                deprecated_pairs.append((key, help))
+            elif info.experimental:
+                experimental_pairs.append((key, help))
+            else:
+                output += wrapped(key, help, info)
+        output += "\nSee :doc:`formats-help` for more about storage formats."
+        other_output = ""
+        if len(experimental_pairs) > 0:
+            other_output += "Experimental formats are shown below.\n\n"
+            for key, help in experimental_pairs:
+                info = self.get_info(key)
+                other_output += wrapped(key, help, info)
+        else:
+            other_output += \
+                "No experimental formats are available.\n\n"
+        if len(deprecated_pairs) > 0:
+            other_output += "\nDeprecated formats are shown below.\n\n"
+            for key, help in deprecated_pairs:
+                info = self.get_info(key)
+                other_output += wrapped(key, help, info)
+        else:
+            other_output += \
+                "\nNo deprecated formats are available.\n\n"
+        other_output += \
+                "\nSee :doc:`formats-help` for more about storage formats."
+
+        if topic == 'other-formats':
+            return other_output
+        else:
+            return output
+
+
+# Please register new formats after old formats so that formats
+# appear in chronological order and format descriptions can build
+# on previous ones.
+format_registry = ControlDirFormatRegistry()
+
+network_format_registry = registry.FormatRegistry()
+"""Registry of formats indexed by their network name.
+
+The network name for a ControlDirFormat is an identifier that can be used when
+referring to formats with smart server operations. See
+ControlDirFormat.network_name() for more detail.
+"""

=== modified file 'bzrlib/crash.py'
--- bzrlib/crash.py	2010-04-08 04:34:03 +0000
+++ bzrlib/crash.py	2010-09-28 18:51:47 +0000
@@ -20,7 +20,7 @@
 A crash is an exception propagated up almost to the top level of Bazaar.
 
 If we have apport <https://launchpad.net/apport/>, we store a report of the
-crash using apport into it's /var/crash spool directory, from where the user
+crash using apport into its /var/crash spool directory, from where the user
 can either manually send it to Launchpad.  In some cases (at least Ubuntu
 development releases), Apport may pop up a window asking if they want
 to send it.
@@ -173,7 +173,7 @@
     pr['Package'] = 'bzr'
 
     # tell apport to file directly against the bzr package using 
-    # <https://bugs.edge.launchpad.net/bzr/+bug/391015>
+    # <https://bugs.launchpad.net/bzr/+bug/391015>
     #
     # XXX: unfortunately apport may crash later if the crashdb definition
     # file isn't present

=== modified file 'bzrlib/diff-delta.c'
--- bzrlib/diff-delta.c	2009-12-14 15:54:42 +0000
+++ bzrlib/diff-delta.c	2010-08-02 16:35:11 +0000
@@ -853,7 +853,8 @@
     free(index);
 }
 
-unsigned long sizeof_delta_index(struct delta_index *index)
+unsigned long
+sizeof_delta_index(struct delta_index *index)
 {
     if (index)
         return index->memsize;
@@ -872,7 +873,8 @@
              const void *trg_buf, unsigned long trg_size,
              unsigned long *delta_size, unsigned long max_size)
 {
-    unsigned int i, outpos, outsize, moff, msize, val;
+    unsigned int i, outpos, outsize, moff, val;
+    int msize;
     const struct source_info *msource;
     int inscnt;
     const unsigned char *ref_data, *ref_top, *data, *top;
@@ -943,7 +945,7 @@
                  entry++) {
                 const unsigned char *ref;
                 const unsigned char *src;
-                unsigned int ref_size;
+                int ref_size;
                 if (entry->val != val)
                     continue;
                 ref = entry->ptr;
@@ -956,14 +958,14 @@
                  * match more bytes with this location that we have already
                  * matched.
                  */
-                if (ref_size > top - src)
+                if (ref_size > (top - src))
                     ref_size = top - src;
                 if (ref_size <= msize)
                     break;
                 /* See how many bytes actually match at this location. */
                 while (ref_size-- && *src++ == *ref)
                     ref++;
-                if (msize < ref - entry->ptr) {
+                if (msize < (ref - entry->ptr)) {
                     /* this is our best match so far */
                     msize = ref - entry->ptr;
                     msource = entry->src;

=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2010-04-30 11:35:43 +0000
+++ bzrlib/diff.py	2010-08-20 06:49:00 +0000
@@ -99,8 +99,8 @@
     if sequence_matcher is None:
         sequence_matcher = patiencediff.PatienceSequenceMatcher
     ud = patiencediff.unified_diff(oldlines, newlines,
-                      fromfile=old_filename.encode(path_encoding),
-                      tofile=new_filename.encode(path_encoding),
+                      fromfile=old_filename.encode(path_encoding, 'replace'),
+                      tofile=new_filename.encode(path_encoding, 'replace'),
                       sequencematcher=sequence_matcher)
 
     ud = list(ud)
@@ -420,12 +420,9 @@
 
     # Get the specific files (all files is None, no files is [])
     if make_paths_wt_relative and working_tree is not None:
-        try:
-            from bzrlib.builtins import safe_relpath_files
-            other_paths = safe_relpath_files(working_tree, other_paths,
+        other_paths = working_tree.safe_relpath_files(
+            other_paths,
             apply_view=apply_view)
-        except errors.FileInWrongBranch:
-            raise errors.BzrCommandError("Files are in different branches")
     specific_files.extend(other_paths)
     if len(specific_files) == 0:
         specific_files = None
@@ -706,18 +703,18 @@
         """
         def _get_text(tree, file_id, path):
             if file_id is not None:
-                return tree.get_file(file_id, path).readlines()
+                return tree.get_file_lines(file_id, path)
             else:
                 return []
         try:
             from_text = _get_text(self.old_tree, from_file_id, from_path)
             to_text = _get_text(self.new_tree, to_file_id, to_path)
             self.text_differ(from_label, from_text, to_label, to_text,
-                             self.to_file)
+                             self.to_file, path_encoding=self.path_encoding)
         except errors.BinaryFile:
             self.to_file.write(
                   ("Binary files %s and %s differ\n" %
-                  (from_label, to_label)).encode(self.path_encoding))
+                  (from_label, to_label)).encode(self.path_encoding,'replace'))
         return self.CHANGED
 
 
@@ -739,9 +736,12 @@
                      path_encoding)
 
     @classmethod
-    def make_from_diff_tree(klass, command_string):
+    def make_from_diff_tree(klass, command_string, external_diff_options=None):
         def from_diff_tree(diff_tree):
-            return klass.from_string(command_string, diff_tree.old_tree,
+            full_command_string = [command_string]
+            if external_diff_options is not None:
+                full_command_string += ' ' + external_diff_options
+            return klass.from_string(full_command_string, diff_tree.old_tree,
                                      diff_tree.new_tree, diff_tree.to_file)
         return from_diff_tree
 
@@ -915,12 +915,15 @@
         :param using: Commandline to use to invoke an external diff tool
         """
         if using is not None:
-            extra_factories = [DiffFromTool.make_from_diff_tree(using)]
+            extra_factories = [DiffFromTool.make_from_diff_tree(using, external_diff_options)]
         else:
             extra_factories = []
         if external_diff_options:
             opts = external_diff_options.split()
-            def diff_file(olab, olines, nlab, nlines, to_file):
+            def diff_file(olab, olines, nlab, nlines, to_file, path_encoding=None):
+                """:param path_encoding: not used but required
+                        to match the signature of internal_diff.
+                """
                 external_diff(olab, olines, nlab, nlines, to_file, opts)
         else:
             diff_file = internal_diff

=== modified file 'bzrlib/dirstate.py'
--- bzrlib/dirstate.py	2010-01-13 23:06:42 +0000
+++ bzrlib/dirstate.py	2010-09-13 06:36:59 +0000
@@ -220,6 +220,7 @@
     inventory,
     lock,
     osutils,
+    static_tuple,
     trace,
     )
 
@@ -548,7 +549,7 @@
            self._ensure_block(block_index, entry_index, utf8path)
         self._dirblock_state = DirState.IN_MEMORY_MODIFIED
         if self._id_index:
-            self._id_index.setdefault(entry_key[2], set()).add(entry_key)
+            self._add_to_id_index(self._id_index, entry_key)
 
     def _bisect(self, paths):
         """Bisect through the disk structure for specific rows.
@@ -1566,7 +1567,7 @@
             return
         id_index = self._get_id_index()
         for file_id in new_ids:
-            for key in id_index.get(file_id, []):
+            for key in id_index.get(file_id, ()):
                 block_i, entry_i, d_present, f_present = \
                     self._get_block_entry_index(key[0], key[1], tree_index)
                 if not f_present:
@@ -1980,7 +1981,7 @@
                                           ' tree_index, file_id and path')
             return entry
         else:
-            possible_keys = self._get_id_index().get(fileid_utf8, None)
+            possible_keys = self._get_id_index().get(fileid_utf8, ())
             if not possible_keys:
                 return None, None
             for key in possible_keys:
@@ -2143,14 +2144,48 @@
                 yield entry
 
     def _get_id_index(self):
-        """Get an id index of self._dirblocks."""
+        """Get an id index of self._dirblocks.
+        
+        This maps from file_id => [(directory, name, file_id)] entries where
+        that file_id appears in one of the trees.
+        """
         if self._id_index is None:
             id_index = {}
             for key, tree_details in self._iter_entries():
-                id_index.setdefault(key[2], set()).add(key)
+                self._add_to_id_index(id_index, key)
             self._id_index = id_index
         return self._id_index
 
+    def _add_to_id_index(self, id_index, entry_key):
+        """Add this entry to the _id_index mapping."""
+        # This code used to use a set for every entry in the id_index. However,
+        # it is *rare* to have more than one entry. So a set is a large
+        # overkill. And even when we do, we won't ever have more than the
+        # number of parent trees. Which is still a small number (rarely >2). As
+        # such, we use a simple tuple, and do our own uniqueness checks. While
+        # the 'in' check is O(N) since N is nicely bounded it shouldn't ever
+        # cause quadratic failure.
+        # TODO: This should use StaticTuple
+        file_id = entry_key[2]
+        entry_key = static_tuple.StaticTuple.from_sequence(entry_key)
+        if file_id not in id_index:
+            id_index[file_id] = static_tuple.StaticTuple(entry_key,)
+        else:
+            entry_keys = id_index[file_id]
+            if entry_key not in entry_keys:
+                id_index[file_id] = entry_keys + (entry_key,)
+
+    def _remove_from_id_index(self, id_index, entry_key):
+        """Remove this entry from the _id_index mapping.
+
+        It is an programming error to call this when the entry_key is not
+        already present.
+        """
+        file_id = entry_key[2]
+        entry_keys = list(id_index[file_id])
+        entry_keys.remove(entry_key)
+        id_index[file_id] = static_tuple.StaticTuple.from_sequence(entry_keys)
+
     def _get_output_lines(self, lines):
         """Format lines for final output.
 
@@ -2414,7 +2449,9 @@
                 continue
             by_path[entry[0]] = [entry[1][0]] + \
                 [DirState.NULL_PARENT_DETAILS] * parent_count
-            id_index[entry[0][2]] = set([entry[0]])
+            # TODO: Possibly inline this, since we know it isn't present yet
+            #       id_index[entry[0][2]] = (entry[0],)
+            self._add_to_id_index(id_index, entry[0])
 
         # now the parent trees:
         for tree_index, tree in enumerate(parent_trees):
@@ -2442,7 +2479,7 @@
                 new_entry_key = (dirname, basename, file_id)
                 # tree index consistency: All other paths for this id in this tree
                 # index must point to the correct path.
-                for entry_key in id_index.setdefault(file_id, set()):
+                for entry_key in id_index.get(file_id, ()):
                     # TODO:PROFILING: It might be faster to just update
                     # rather than checking if we need to, and then overwrite
                     # the one we are located at.
@@ -2454,7 +2491,8 @@
                         by_path[entry_key][tree_index] = ('r', path_utf8, 0, False, '')
                 # by path consistency: Insert into an existing path record (trivial), or
                 # add a new one with relocation pointers for the other tree indexes.
-                if new_entry_key in id_index[file_id]:
+                entry_keys = id_index.get(file_id, ())
+                if new_entry_key in entry_keys:
                     # there is already an entry where this data belongs, just insert it.
                     by_path[new_entry_key][tree_index] = \
                         self._inv_entry_to_details(entry)
@@ -2465,16 +2503,16 @@
                     new_details = []
                     for lookup_index in xrange(tree_index):
                         # boundary case: this is the first occurence of file_id
-                        # so there are no id_indexs, possibly take this out of
+                        # so there are no id_indexes, possibly take this out of
                         # the loop?
-                        if not len(id_index[file_id]):
+                        if not len(entry_keys):
                             new_details.append(DirState.NULL_PARENT_DETAILS)
                         else:
                             # grab any one entry, use it to find the right path.
                             # TODO: optimise this to reduce memory use in highly
                             # fragmented situations by reusing the relocation
                             # records.
-                            a_key = iter(id_index[file_id]).next()
+                            a_key = iter(entry_keys).next()
                             if by_path[a_key][lookup_index][0] in ('r', 'a'):
                                 # its a pointer or missing statement, use it as is.
                                 new_details.append(by_path[a_key][lookup_index])
@@ -2485,7 +2523,7 @@
                     new_details.append(self._inv_entry_to_details(entry))
                     new_details.extend(new_location_suffix)
                     by_path[new_entry_key] = new_details
-                    id_index[file_id].add(new_entry_key)
+                    self._add_to_id_index(id_index, new_entry_key)
         # --- end generation of full tree mappings
 
         # sort and output all the entries
@@ -2673,7 +2711,7 @@
             block[1].pop(entry_index)
             # if we have an id_index in use, remove this key from it for this id.
             if self._id_index is not None:
-                self._id_index[current_old[0][2]].remove(current_old[0])
+                self._remove_from_id_index(self._id_index, current_old[0])
         # update all remaining keys for this id to record it as absent. The
         # existing details may either be the record we are marking as deleted
         # (if there were other trees with the id present at this path), or may
@@ -2748,7 +2786,7 @@
                     else:
                         break
             # new entry, synthesis cross reference here,
-            existing_keys = id_index.setdefault(key[2], set())
+            existing_keys = id_index.get(key[2], ())
             if not existing_keys:
                 # not currently in the state, simplest case
                 new_entry = key, [new_details] + self._empty_parent_info()
@@ -2785,8 +2823,11 @@
                     # loop.
                     other_entry = other_block[other_entry_index]
                     other_entry[1][0] = ('r', path_utf8, 0, False, '')
-                    self._maybe_remove_row(other_block, other_entry_index,
-                        id_index)
+                    if self._maybe_remove_row(other_block, other_entry_index,
+                                              id_index):
+                        # If the row holding this was removed, we need to
+                        # recompute where this entry goes
+                        entry_index, _ = self._find_entry_index(key, block)
 
                 # This loop:
                 # adds a tuple to the new details for each column
@@ -2794,6 +2835,8 @@
                 #  - or by creating a new pointer to the right row inside that column
                 num_present_parents = self._num_present_parents()
                 if num_present_parents:
+                    # TODO: This re-evaluates the existing_keys set, do we need
+                    #       to do that ourselves?
                     other_key = list(existing_keys)[0]
                 for lookup_index in xrange(1, num_present_parents + 1):
                     # grab any one entry, use it to find the right path.
@@ -2818,7 +2861,7 @@
                         pointer_path = osutils.pathjoin(*other_key[0:2])
                         new_entry[1].append(('r', pointer_path, 0, False, ''))
             block.insert(entry_index, new_entry)
-            existing_keys.add(key)
+            self._add_to_id_index(id_index, key)
         else:
             # Does the new state matter?
             block[entry_index][1][0] = new_details
@@ -2833,7 +2876,12 @@
             # converted to relocated.
             if path_utf8 is None:
                 raise AssertionError('no path')
-            for entry_key in id_index.setdefault(key[2], set()):
+            existing_keys = id_index.get(key[2], ())
+            if key not in existing_keys:
+                raise AssertionError('We found the entry in the blocks, but'
+                    ' the key is not in the id_index.'
+                    ' key: %s, existing_keys: %s' % (key, existing_keys))
+            for entry_key in existing_keys:
                 # TODO:PROFILING: It might be faster to just update
                 # rather than checking if we need to, and then overwrite
                 # the one we are located at.
@@ -2863,6 +2911,7 @@
         """Remove index if it is absent or relocated across the row.
         
         id_index is updated accordingly.
+        :return: True if we removed the row, False otherwise
         """
         present_in_row = False
         entry = block[index]
@@ -2872,7 +2921,9 @@
                 break
         if not present_in_row:
             block.pop(index)
-            id_index[entry[0][2]].remove(entry[0])
+            self._remove_from_id_index(id_index, entry[0])
+            return True
+        return False
 
     def _validate(self):
         """Check that invariants on the dirblock are correct.
@@ -3020,6 +3071,10 @@
                         raise AssertionError(
                             'file_id %r did not match entry key %s'
                             % (file_id, entry_key))
+                if len(entry_keys) != len(set(entry_keys)):
+                    raise AssertionError(
+                        'id_index contained non-unique data for %s'
+                        % (entry_keys,))
 
     def _wipe_state(self):
         """Forget all state information about the dirstate."""

=== modified file 'bzrlib/doc_generate/autodoc_man.py'
--- bzrlib/doc_generate/autodoc_man.py	2010-02-23 07:43:11 +0000
+++ bzrlib/doc_generate/autodoc_man.py	2010-08-28 04:21:40 +0000
@@ -186,7 +186,7 @@
 .B "help"
 .I "command"
 .SH "DESCRIPTION"
-Bazaar (or %(bzrcmd)s) is a project of Canonical to develop an open source
+Bazaar (or %(bzrcmd)s) is a project of Canonical to develop an free
 distributed version control system that is powerful, friendly, and scalable.
 Version control means a system that keeps track of previous revisions
 of software source code or similar information and helps people work on it in teams.
@@ -241,7 +241,7 @@
 .br
 log10 = log --short -r -10..-1
 .SH "SEE ALSO"
-.UR http://www.bazaar.canonical.com/
-.BR http://www.bazaar.canonical.com/
+.UR http://bazaar.canonical.com/
+.BR http://bazaar.canonical.com/
 """
 

=== added directory 'bzrlib/doc_generate/builders'
=== added file 'bzrlib/doc_generate/builders/__init__.py'
--- bzrlib/doc_generate/builders/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/doc_generate/builders/__init__.py	2010-07-05 16:01:33 +0000
@@ -0,0 +1,18 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Placeholder to allow sphinx builders imports."""
+

=== added file 'bzrlib/doc_generate/builders/texinfo.py'
--- bzrlib/doc_generate/builders/texinfo.py	1970-01-01 00:00:00 +0000
+++ bzrlib/doc_generate/builders/texinfo.py	2010-05-06 20:05:01 +0000
@@ -0,0 +1,42 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""A sphinx builder producing texinfo output."""
+
+from sphinx import builders
+from sphinx.builders import text as _text_builder
+
+from bzrlib.doc_generate.writers import texinfo as texinfo_writer
+
+class TexinfoBuilder(_text_builder.TextBuilder):
+
+    name = 'texinfo'
+    format = 'texinfo'
+    out_suffix = '.texi'
+
+    def prepare_writing(self, docnames):
+        self.writer = texinfo_writer.TexinfoWriter(self)
+
+    def get_target_uri(self, docname, typ=None):
+        # FIXME: Revisit when info file generation is defined (the suffix is
+        # left here for clarity but the final version may just get rid of
+        # it). And we probalby will join several files into bigger info files
+        # anyway. -- vila 20100506
+        return docname + '.info'
+
+
+def setup(app):
+    app.add_builder(TexinfoBuilder)

=== renamed file 'bzrlib/doc_generate/sphinx_conf.py' => 'bzrlib/doc_generate/conf.py'
--- bzrlib/doc_generate/sphinx_conf.py	2009-09-08 16:37:05 +0000
+++ bzrlib/doc_generate/conf.py	2010-07-07 11:32:03 +0000
@@ -8,6 +8,9 @@
 
 import sys, os
 
+# FIXME: better move the content of doc/en/conf.py here and cleanup the result
+# -- vila 20100428
+
 # If extensions (or modules to document with autodoc) are in another directory,
 # add these directories to sys.path here. If the directory is relative to the
 # documentation root, use os.path.abspath to make it absolute, like shown here.
@@ -18,7 +21,8 @@
 
 # Add any Sphinx extension module names here, as strings. They can be extensions
 # coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
-extensions = ['sphinx.ext.ifconfig']
+extensions = ['sphinx.ext.ifconfig',
+              'bzrlib.doc_generate.builders.texinfo']
 
 # Add any paths that contain templates here, relative to this directory.
 templates_path = ['_templates']
@@ -34,7 +38,7 @@
 
 # General information about the project.
 project = u'Bazaar'
-copyright = u'2009, Canonical Ltd'
+copyright = u'2009, 2010 Canonical Ltd'
 
 # The version info for the project you're documenting, acts as replacement for
 # |version| and |release|, also used in various other places throughout the

=== added directory 'bzrlib/doc_generate/writers'
=== added file 'bzrlib/doc_generate/writers/__init__.py'
--- bzrlib/doc_generate/writers/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/doc_generate/writers/__init__.py	2010-07-05 16:01:33 +0000
@@ -0,0 +1,18 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Placeholder to allow sphinx writer imports."""
+

=== added file 'bzrlib/doc_generate/writers/texinfo.py'
--- bzrlib/doc_generate/writers/texinfo.py	1970-01-01 00:00:00 +0000
+++ bzrlib/doc_generate/writers/texinfo.py	2010-07-21 08:07:14 +0000
@@ -0,0 +1,520 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""A sphinx/docutil writer producing texinfo output."""
+
+from docutils import (
+    nodes,
+    writers,
+    )
+
+DEBUG = 0
+
+class TexinfoWriter(writers.Writer):
+
+    supported = ('texinfo',)
+    settings_spec = ('No options here.', '', ())
+    settings_defaults = {}
+
+    output = None
+
+    def __init__(self, builder):
+        writers.Writer.__init__(self)
+        self.builder = builder
+
+    def translate(self):
+        visitor = TexinfoTranslator(self.document, self.builder)
+        self.document.walkabout(visitor)
+        self.output = visitor.body
+
+
+class TexinfoTranslator(nodes.NodeVisitor):
+
+    section_names = ['chapter', 'section', 'subsection', 'subsubsection']
+    """texinfo section names differ from the sphinx ones.
+
+    Since this can be confusing, the correspondences are shown below
+    (shpinx -> texinfo):
+    part       -> chapter
+    chapter    -> section
+    section    -> subsection
+    subsection -> subsubsection
+
+    Additionally, sphinx defines subsubsections and paragraphs which are
+    handled as @heading (unnumbered).
+    """
+
+    def __init__(self, document, builder):
+        nodes.NodeVisitor.__init__(self, document)
+        self.builder = builder
+        # toctree uses some nodes for different purposes (namely:
+        # compact_paragraph, bullet_list) that needs to know when they are
+        # processing a toctree.
+        self.in_toctree = False
+        # sections can be embedded and produce different directives depending
+        # on the depth.
+        self.section_level = -1
+        # By default paragraghs are separated by newlines, but there are some
+        # exceptions that set it to '' for some subtrees instead
+        self.paragraph_sep = '\n'
+
+    # The whole document
+
+    def visit_document(self, node):
+        if DEBUG:
+            import sys
+            sys.stdout.write(node.pformat().encode('utf8'))
+        set_item_list_collector(node, 'text')
+
+    def depart_document(self, node):
+        # FIXME: info requires a Top node for each info file, but unless we
+        # chose a global layout to divide the overall documentation into a set
+        # of info files, there is no criteria to decide for a title.
+        top_cmd = '''\
+This file has been converted using a beta rst->texinfo converter. 
+Most of the info links are currently bogus, don't report bugs about them,
+this is currently worked on.
+@node Top
+@top Placeholder
+'''
+        self.body = top_cmd + ''.join(node['text'])
+
+    # Layout
+
+    def visit_section(self, node):
+        self.section_level += 1
+        set_item_list_collector(node, 'text')
+
+    def depart_section(self, node):
+        title = node['title']
+        ids = node.get('ids', [])
+        try:
+            section_name = self.section_names[self.section_level]
+        except IndexError:
+            # Just use @heading, it's not numbered anyway
+            section_name = 'heading'
+        if ids:
+            # There shouldn't be different ids for a section, so until we
+            # encounter bugs, just take the first one.
+            node_cmd = '@node %s\n' % (ids[0],)
+        else:
+            node_cmd = ''
+        section_cmd = '@%s %s\n' % (section_name, title)
+        text = ''.join(node['text'])
+        node.parent.collect_text(node_cmd + section_cmd + text)
+        self.section_level -= 1
+
+    def visit_topic(self, node):
+        pass
+
+    def depart_topic(self, node):
+        pass
+
+    def visit_paragraph(self, node):
+        set_item_list_collector(node, 'text')
+
+    def depart_paragraph(self, node):
+        # End the paragraph with a new line (or '' depending on the parent) and
+        # leave a blank line after it.
+        text = ''.join(node['text']) + self.paragraph_sep * 2
+        node.parent.collect_text(text)
+
+    def visit_compact_paragraph(self, node):
+        set_item_list_collector(node, 'text')
+        if node.has_key('toctree'):
+            self.in_toctree = True
+        elif self.in_toctree:
+            set_item_collector(node, 'reference')
+
+    def depart_compact_paragraph(self, node):
+        # FIXME: Using a different visitor specific to toctree may be a better
+        # design and makes code clearer. -- vila 20100708
+        if node.has_key('toctree'):
+            node.parent.collect_text('@menu\n')
+            node.parent.collect_text(''.join(node['text']))
+            node.parent.collect_text('@end menu\n')
+            self.in_toctree = False
+        elif self.in_toctree:
+            # * FIRST-ENTRY-NAME:(FILENAME)NODENAME.     DESCRIPTION
+            # XXX: the file name should probably be adjusted to the targeted
+            # info file name
+            node_name, file_name, entry_name = node['reference']
+            if not node_name:
+                node_name = entry_name
+            description = '' # We can't specify a description in rest AFAICS
+            # XXX: What if :maxdepth: is not 1 ?
+            text = '* %s: (%s)%s. %s\n' % (entry_name, file_name,
+                                           node_name, description)
+            node.parent.collect_text(text)
+        else:
+            # End the paragraph with a new line (or '' depending on the parent)
+            # and leave a blank line after it.
+            text = ''.join(node['text']) + self.paragraph_sep * 2
+            node.parent.collect_text(text)
+
+    def visit_literal_block(self, node):
+        set_item_collector(node, 'text')
+
+    def depart_literal_block(self, node):
+        text = '@samp{%s}' % ''.join(node['text']) + self.paragraph_sep * 2
+        node.parent.collect_text(text)
+
+    def visit_block_quote(self, node):
+        set_item_list_collector(node, 'text')
+
+    def depart_block_quote(self, node):
+        node.parent.collect_text('@example\n')
+        node.parent.collect_text(''.join(node['text']))
+        node.parent.collect_text('@end example\n\n')
+
+    def depart_warning(self, node):
+        pass
+
+    def visit_warning(self, node):
+        raise nodes.SkipNode # Not implemented yet
+
+    def visit_note(self, node):
+        raise nodes.SkipNode # Not implemented yet
+
+    def depart_note(self, node):
+        pass
+
+    def visit_footnote(self, node):
+        raise nodes.SkipNode # Not implemented yet
+
+    def depart_footnote(self, node):
+        pass
+
+    def visit_comment(self, node):
+        raise nodes.SkipNode # Not implemented yet
+
+    # Attributes
+
+    def visit_title(self, node):
+        set_item_collector(node, 'text')
+
+    def depart_title(self, node):
+        node.parent['title'] = node['text']
+
+    def visit_label(self, node):
+        raise nodes.SkipNode # Not implemented yet
+
+    def visit_substitution_definition(self, node):
+        raise nodes.SkipNode # Not implemented yet
+
+    # Plain text
+
+    def visit_Text(self, node):
+        pass
+
+    def depart_Text(self, node):
+        text = node.astext()
+        if '@' in text:
+            text = text.replace('@', '@@')
+        if '{' in text:
+            text = text.replace('{', '@{')
+        if '}' in text:
+            text = text.replace('}', '@}')
+        node.parent.collect_text(text)
+
+
+    # Styled text
+
+    def visit_emphasis(self, node):
+        set_item_collector(node, 'text')
+
+    def depart_emphasis(self, node):
+        text = '@emph{%s}' % node['text']
+        node.parent.collect_text(text)
+
+    def visit_strong(self, node):
+        set_item_collector(node, 'text')
+
+    def depart_strong(self, node):
+        text = '@strong{%s}' % node['text']
+        node.parent.collect_text(text)
+
+    def visit_literal(self, node):
+        set_item_collector(node, 'text')
+
+    def depart_literal(self, node):
+        text = '@code{%s}' % node['text']
+        node.parent.collect_text(text)
+
+    # Lists
+
+    def _decorate_list(self, item_list, collect, item_fmt='%s',
+                       head=None, foot=None):
+        if head is not None:
+            collect(head)
+        for item in item_list:
+            collect(item_fmt % item)
+        if foot is not None:
+            collect(foot)
+
+    def visit_bullet_list(self, node):
+        set_item_list_collector(node, 'list_item')
+
+    def depart_bullet_list(self, node):
+        l = node['list_item']
+        if self.in_toctree:
+            self._decorate_list(node['list_item'], node.parent.collect_text)
+        else:
+            self._decorate_list(node['list_item'], node.parent.collect_text,
+                                '@item\n%s',
+                                # FIXME: Should respect the 'bullet' attribute
+                                '@itemize @bullet\n', '@end itemize\n')
+
+    def visit_enumerated_list(self, node):
+        set_item_list_collector(node, 'list_item')
+
+    def depart_enumerated_list(self, node):
+        self._decorate_list(node['list_item'], node.parent.collect_text,
+                            '@item\n%s',
+                            '@enumerate\n', '@end enumerate\n')
+
+    def visit_definition_list(self, node):
+        raise nodes.SkipNode # Not implemented yet
+
+    def depart_definition_list(self, node):
+        raise nodes.SkipNode # Not implemented yet
+
+    def visit_definition_list_item(self, node):
+        raise nodes.SkipNode # Not implemented yet
+
+    def depart_definition_list_item(self, node):
+        pass
+
+    def visit_term(self, node):
+        raise nodes.SkipNode # Not implemented yet
+
+    def depart_term(self, node):
+        pass
+
+    def visit_definition(self, node):
+        raise nodes.SkipNode # Not implemented yet
+
+    def depart_definition(self, node):
+        pass
+
+    def visit_field_list(self, node):
+        raise nodes.SkipNode # Not implemented yet
+
+    def depart_field_list(self, node):
+        pass
+
+    def visit_field(self, node):
+        raise nodes.SkipNode # Not implemented yet
+
+    def depart_field(self, node):
+        pass
+
+    def visit_field_name(self, node):
+        raise nodes.SkipNode # Not implemented yet
+
+    def depart_field_name(self, node):
+        pass
+
+    def visit_field_body(self, node):
+        raise nodes.SkipNode # Not implemented yet
+
+    def depart_field_body(self, node):
+        pass
+
+    def visit_list_item(self, node):
+        set_item_list_collector(node, 'text')
+
+    def depart_list_item(self, node):
+        text = ''.join(node['text'])
+        node.parent.collect_list_item(text)
+
+    def visit_option_list(self, node):
+        raise nodes.SkipNode # Not implemented yet
+
+    def depart_option_list(self, node):
+        pass
+
+    def visit_option_list_item(self, node):
+        pass
+
+    def depart_option_list_item(self, node):
+        pass
+
+    def visit_option_group(self, node):
+        pass
+
+    def depart_option_group(self, node):
+        pass
+
+    def visit_option(self, node):
+        pass
+
+    def depart_option(self, node):
+        pass
+
+    def visit_option_string(self, node):
+        pass
+    def depart_option_string(self, node):
+        pass
+
+    def visit_option_argument(self, node):
+        pass
+
+    def depart_option_argument(self, node):
+        pass
+
+    def visit_description(self, node):
+        pass
+    def depart_description(self, node):
+        pass
+
+    # Tables
+    def visit_table(self, node):
+        set_item_collector(node, 'table')
+
+    def depart_table(self, node):
+        node.parent.collect_text(node['table'])
+
+    def visit_tgroup(self, node):
+        set_item_list_collector(node, 'colspec')
+        set_item_collector(node, 'head_entries')
+        set_item_collector(node, 'body_rows')
+
+    def depart_tgroup(self, node):
+        header = []
+        # The '@multitable {xxx}{xxx}' line
+        self._decorate_list(node['colspec'], header.append,
+                            '{%s}', '@multitable ', '\n')
+        # The '@headitem xxx @tab yyy...' line
+        head_entries = node['head_entries']
+        if head_entries is not None:
+            # Not all tables define titles for the columns... rest parser bug ?
+            # FIXME: need a test
+            self._decorate_list(head_entries[1:], header.append,
+                                ' @tab %s',
+                                '@headitem %s' % head_entries[0], '\n')
+        header = ''.join(header)
+        # The '@item xxx\n @tab yyy\n ...' lines
+        body_rows = node['body_rows']
+        rows = []
+        for r in body_rows:
+            self._decorate_list(r[1:], rows.append,
+                                '@tab %s\n', '@item %s\n' % r[0])
+        footer = '@end multitable\n'
+        node.parent.collect_table(header + ''.join(rows) + footer)
+
+    def visit_colspec(self, node):
+        pass
+
+    def depart_colspec(self, node):
+        node.parent.collect_colspec('x' * node['colwidth'])
+
+    def visit_thead(self, node):
+        set_item_collector(node, 'row')
+
+    def depart_thead(self, node):
+        node.parent.collect_head_entries(node['row'])
+
+    def visit_tbody(self, node):
+        set_item_list_collector(node, 'row')
+
+    def depart_tbody(self, node):
+        node.parent.collect_body_rows(node['row'])
+
+    def visit_row(self, node):
+        set_item_list_collector(node, 'entry')
+
+    def depart_row(self, node):
+        node.parent.collect_row(node['entry'])
+
+    def visit_entry(self, node):
+        set_item_list_collector(node, 'text')
+        node['par_sep_orig'] = self.paragraph_sep
+        self.paragraph_sep = ''
+
+    def depart_entry(self, node):
+        node.parent.collect_entry(''.join(node['text']))
+        self.paragraph_sep = node['par_sep_orig']
+
+    # References
+
+    def visit_reference(self, node):
+        for c in node.children:
+            if getattr(c, 'parent', None) is None:
+                # Bug sphinx
+                node.setup_child(c)
+        set_item_collector(node, 'text')
+
+    def depart_reference(self, node):
+        anchorname = node.get('anchorname', None)
+        refuri = node.get('refuri', None)
+        refid = node.get('refid', None)
+        text = ''.join(node['text'])
+        collect = getattr(node.parent, 'collect_reference', None)
+        if collect is not None:
+            if not self.in_toctree:
+                raise AssertionError('collect_reference is specific to toctree')
+            if anchorname is None:
+                anchorname = ''
+            if refuri is None:
+                refuri = ''
+            collect((anchorname, refuri, text))
+        elif refuri is not None:
+            node.parent.collect_text('@uref{%s,%s}' % (refuri, text))
+        elif refid is not None:
+            # Info format requires that a reference is followed by some
+            # punctuation char ('.', ','. ')', etc). Rest is more liberal. To
+            # accommodate, we use pxref inside parenthesis.
+            node.parent.collect_text('%s (@pxref{%s})' % (text, refid))
+
+    def visit_footnote_reference(self, node):
+        raise nodes.SkipNode # Not implemented yet
+
+    def visit_citation_reference(self, node):
+        raise nodes.SkipNode # Not implemented yet
+
+    def visit_title_reference(self, node):
+        raise nodes.SkipNode # Not implemented yet
+
+    def depart_title_reference(self, node):
+        pass
+
+    def visit_target(self, node):
+        raise nodes.SkipNode # Not implemented yet
+
+    def depart_target(self, node):
+        pass
+
+    def visit_image(self, node):
+        raise nodes.SkipNode # Not implemented yet
+
+# Helpers to collect data in parent node
+
+def set_item_collector(node, name):
+    node[name] = None
+    def set_item(item):
+        node[name] = item
+    setattr(node, 'collect_' + name, set_item)
+
+
+def set_item_list_collector(node, name, sep=''):
+    node[name] = []
+    node[name + '_sep'] = sep
+    def append_item(item):
+        node[name].append(item)
+    setattr(node, 'collect_' + name, append_item)
+
+

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2010-04-21 11:27:04 +0000
+++ bzrlib/errors.py	2010-09-28 18:51:47 +0000
@@ -680,7 +680,7 @@
 
     _fmt = 'Path "%(path)s" is not a child of path "%(base)s"%(extra)s'
 
-    internal_error = True
+    internal_error = False
 
     def __init__(self, path, base, extra=None):
         BzrError.__init__(self)
@@ -782,6 +782,8 @@
 
     _fmt = 'File "%(path)s" is not in branch %(branch_base)s.'
 
+    # use PathNotChild instead
+    @symbol_versioning.deprecated_method(symbol_versioning.deprecated_in((2, 3, 0)))
     def __init__(self, branch, path):
         BzrError.__init__(self)
         self.branch = branch
@@ -947,11 +949,8 @@
     # original exception is available as e.original_error
     #
     # New code should prefer to raise specific subclasses
-    def __init__(self, message):
-        # Python 2.5 uses a slot for StandardError.message,
-        # so use a different variable name.  We now work around this in
-        # BzrError.__str__, but this member name is kept for compatability.
-        self.msg = message
+    def __init__(self, msg):
+        self.msg = msg
 
 
 class LockActive(LockError):
@@ -1041,8 +1040,6 @@
 class LockContention(LockError):
 
     _fmt = 'Could not acquire lock "%(lock)s": %(msg)s'
-    # TODO: show full url for lock, combining the transport and relative
-    # bits?
 
     internal_error = False
 
@@ -1075,6 +1072,18 @@
         self.target = target
 
 
+class LockCorrupt(LockError):
+
+    _fmt = ("Lock is apparently held, but corrupted: %(corruption_info)s\n"
+            "Use 'bzr break-lock' to clear it")
+
+    internal_error = False
+
+    def __init__(self, corruption_info, file_data=None):
+        self.corruption_info = corruption_info
+        self.file_data = file_data
+
+
 class LockNotHeld(LockError):
 
     _fmt = "Lock not held: %(lock)s"
@@ -1180,10 +1189,11 @@
 class InvalidRevisionSpec(BzrError):
 
     _fmt = ("Requested revision: '%(spec)s' does not exist in branch:"
-            " %(branch)s%(extra)s")
+            " %(branch_url)s%(extra)s")
 
     def __init__(self, spec, branch, extra=None):
         BzrError.__init__(self, branch=branch, spec=spec)
+        self.branch_url = getattr(branch, 'user_url', str(branch))
         if extra:
             self.extra = '\n' + str(extra)
         else:
@@ -1380,12 +1390,12 @@
 
 class WeaveParentMismatch(WeaveError):
 
-    _fmt = "Parents are mismatched between two revisions. %(message)s"
+    _fmt = "Parents are mismatched between two revisions. %(msg)s"
 
 
 class WeaveInvalidChecksum(WeaveError):
 
-    _fmt = "Text did not match it's checksum: %(message)s"
+    _fmt = "Text did not match its checksum: %(msg)s"
 
 
 class WeaveTextDiffers(WeaveError):
@@ -1439,7 +1449,7 @@
 
 class VersionedFileInvalidChecksum(VersionedFileError):
 
-    _fmt = "Text did not match its checksum: %(message)s"
+    _fmt = "Text did not match its checksum: %(msg)s"
 
 
 class KnitError(InternalBzrError):
@@ -1925,6 +1935,17 @@
     _fmt = "Moving the root directory is not supported at this time"
 
 
+class TransformRenameFailed(BzrError):
+
+    _fmt = "Failed to rename %(from_path)s to %(to_path)s: %(why)s"
+
+    def __init__(self, from_path, to_path, why, errno):
+        self.from_path = from_path
+        self.to_path = to_path
+        self.why = why
+        self.errno = errno
+
+
 class BzrMoveFailedError(BzrError):
 
     _fmt = "Could not move %(from_path)s%(operator)s %(to_path)s%(extra)s"
@@ -1975,6 +1996,8 @@
         "Use --keep to not delete them, or --force to delete them regardless.")
 
     def __init__(self, tree_delta):
+        symbol_versioning.warn(symbol_versioning.deprecated_in((2, 3, 0)) %
+            "BzrRemoveChangedFilesError", DeprecationWarning, stacklevel=2)
         BzrError.__init__(self)
         self.changes_as_text = tree_delta.get_changes_as_text()
         #self.paths_as_string = '\n'.join(changed_files)
@@ -2834,11 +2857,20 @@
         else:
             more = ' ' + more
         import bzrlib.urlutils as urlutils
-        display_url = urlutils.unescape_for_display(
-            tree.user_url, 'ascii')
+        user_url = getattr(tree, "user_url", None)
+        if user_url is None:
+            display_url = str(tree)
+        else:
+            display_url = urlutils.unescape_for_display(user_url, 'ascii')
         BzrError.__init__(self, tree=tree, display_url=display_url, more=more)
 
 
+class ShelvedChanges(UncommittedChanges):
+
+    _fmt = ('Working tree "%(display_url)s" has shelved changes'
+            ' (See bzr shelve --list).%(more)s')
+
+
 class MissingTemplateVariable(BzrError):
 
     _fmt = 'Variable {%(name)s} is not available.'
@@ -3134,3 +3166,27 @@
     def __init__(self, bzrdir):
         self.bzrdir = bzrdir
 
+
+class NoWhoami(BzrError):
+
+    _fmt = ('Unable to determine your name.\n'
+        "Please, set your name with the 'whoami' command.\n"
+        'E.g. bzr whoami "Your Name <name@example.com>"')
+
+
+class InvalidPattern(BzrError):
+
+    _fmt = ('Invalid pattern(s) found. %(msg)s')
+
+    def __init__(self, msg):
+        self.msg = msg
+
+
+class RecursiveBind(BzrError):
+
+    _fmt = ('Branch "%(branch_url)s" appears to be bound to itself. '
+        'Please use `bzr unbind` to fix.')
+
+    def __init__(self, branch_url):
+        self.branch_url = branch_url
+

=== modified file 'bzrlib/fetch.py'
--- bzrlib/fetch.py	2010-04-30 11:35:43 +0000
+++ bzrlib/fetch.py	2010-09-17 04:35:23 +0000
@@ -182,6 +182,9 @@
     This is for use by fetchers and converters.
     """
 
+    # This is a class variable so that the test suite can override it.
+    known_graph_threshold = 100
+
     def __init__(self, source):
         """Constructor.
 
@@ -243,20 +246,13 @@
         # yet, and are unlikely to in non-rich-root environments anyway.
         root_id_order.sort(key=operator.itemgetter(0))
         # Create a record stream containing the roots to create.
-        if len(revs) > 100:
-            # XXX: not covered by tests, should have a flag to always run
-            # this. -- mbp 20100129
-            graph = self.source_repo.get_known_graph_ancestry(revs)
+        if len(revs) > self.known_graph_threshold:
+            graph = self.source.get_known_graph_ancestry(revs)
         new_roots_stream = _new_root_data_stream(
             root_id_order, rev_id_to_root_id, parent_map, self.source, graph)
         return [('texts', new_roots_stream)]
 
 
-def _get_rich_root_heads_graph(source_repo, revision_ids):
-    """Get a Graph object suitable for asking heads() for new rich roots."""
-    return 
-
-
 def _new_root_data_stream(
     root_keys_to_create, rev_id_to_root_id_map, parent_map, repo, graph=None):
     """Generate a texts substream of synthesised root entries.

=== modified file 'bzrlib/filters/eol.py'
--- bzrlib/filters/eol.py	2009-05-07 05:08:46 +0000
+++ bzrlib/filters/eol.py	2010-06-17 09:23:19 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009 Canonical Ltd
+# Copyright (C) 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,8 +25,8 @@
 from bzrlib.errors import BzrError
 
 
-# Real Linux/Unix/OSX newline - \n without \r before it
-_LINUX_NL_RE = re.compile(r'(?<!\r)\n')
+# Real Unix newline - \n without \r before it
+_UNIX_NL_RE = re.compile(r'(?<!\r)\n')
 
 
 def _to_lf_converter(chunks, context=None):
@@ -44,7 +44,7 @@
     if '\x00' in content:
         return [content]
     else:
-        return [_LINUX_NL_RE.sub('\r\n', content)]
+        return [_UNIX_NL_RE.sub('\r\n', content)]
 
 
 # Register the eol content filter.

=== modified file 'bzrlib/generate_ids.py'
--- bzrlib/generate_ids.py	2009-03-23 14:59:43 +0000
+++ bzrlib/generate_ids.py	2010-08-23 17:54:07 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -93,10 +93,10 @@
 def gen_revision_id(username, timestamp=None):
     """Return new revision-id.
 
-    :param username: This is the value returned by config.username(), which is
-        typically a real name, followed by an email address. If found, we will
-        use just the email address portion. Otherwise we flatten the real name,
-        and use that.
+    :param username: The username of the committer, in the format returned by
+        config.username().  This is typically a real name, followed by an
+        email address. If found, we will use just the email address portion.
+        Otherwise we flatten the real name, and use that.
     :return: A new revision id.
     """
     try:

=== modified file 'bzrlib/globbing.py'
--- bzrlib/globbing.py	2010-02-17 17:11:16 +0000
+++ bzrlib/globbing.py	2010-07-21 10:17:11 +0000
@@ -22,8 +22,10 @@
 
 import re
 
+from bzrlib import errors
 from bzrlib.trace import (
-    warning
+    mutter,
+    warning,
     )
 
 
@@ -177,24 +179,41 @@
     so are matched first, then the basename patterns, then the fullpath
     patterns.
     """
+    # We want to _add_patterns in a specific order (as per type_list below)
+    # starting with the shortest and going to the longest.
+    # As some Python version don't support ordered dicts the list below is
+    # used to select inputs for _add_pattern in a specific order.
+    pattern_types = [ "extension", "basename", "fullpath" ]
+
+    pattern_info = {
+        "extension" : {
+            "translator" : _sub_extension,
+            "prefix" : r'(?:.*/)?(?!.*/)(?:.*\.)'
+        },
+        "basename" : {
+            "translator" : _sub_basename,
+            "prefix" : r'(?:.*/)?(?!.*/)'
+        },
+        "fullpath" : {
+            "translator" : _sub_fullpath,
+            "prefix" : r''
+        },
+    }
+
     def __init__(self, patterns):
         self._regex_patterns = []
-        path_patterns = []
-        base_patterns = []
-        ext_patterns = []
+        pattern_lists = {
+            "extension" : [],
+            "basename" : [],
+            "fullpath" : [],
+        }
         for pat in patterns:
             pat = normalize_pattern(pat)
-            if pat.startswith(u'RE:') or u'/' in pat:
-                path_patterns.append(pat)
-            elif pat.startswith(u'*.'):
-                ext_patterns.append(pat)
-            else:
-                base_patterns.append(pat)
-        self._add_patterns(ext_patterns,_sub_extension,
-            prefix=r'(?:.*/)?(?!.*/)(?:.*\.)')
-        self._add_patterns(base_patterns,_sub_basename,
-            prefix=r'(?:.*/)?(?!.*/)')
-        self._add_patterns(path_patterns,_sub_fullpath)
+            pattern_lists[Globster.identify(pat)].append(pat)
+        pi = Globster.pattern_info
+        for t in Globster.pattern_types:
+            self._add_patterns(pattern_lists[t], pi[t]["translator"],
+                pi[t]["prefix"])
 
     def _add_patterns(self, patterns, translator, prefix=''):
         while patterns:
@@ -209,12 +228,60 @@
 
         :return A matching pattern or None if there is no matching pattern.
         """
-        for regex, patterns in self._regex_patterns:
-            match = regex.match(filename)
-            if match:
-                return patterns[match.lastindex -1]
+        try:
+            for regex, patterns in self._regex_patterns:
+                match = regex.match(filename)
+                if match:
+                    return patterns[match.lastindex -1]
+        except errors.InvalidPattern, e:
+            # We can't show the default e.msg to the user as thats for
+            # the combined pattern we sent to regex. Instead we indicate to
+            # the user that an ignore file needs fixing.
+            mutter('Invalid pattern found in regex: %s.', e.msg)
+            e.msg = "File ~/.bazaar/ignore or .bzrignore contains error(s)."
+            bad_patterns = ''
+            for _, patterns in self._regex_patterns:
+                for p in patterns:
+                    if not Globster.is_pattern_valid(p):
+                        bad_patterns += ('\n  %s' % p)
+            e.msg += bad_patterns
+            raise e
         return None
 
+    @staticmethod
+    def identify(pattern):
+        """Returns pattern category.
+
+        :param pattern: normalized pattern.
+        Identify if a pattern is fullpath, basename or extension
+        and returns the appropriate type.
+        """
+        if pattern.startswith(u'RE:') or u'/' in pattern:
+            return "fullpath"
+        elif pattern.startswith(u'*.'):
+            return "extension"
+        else:
+            return "basename"
+
+    @staticmethod
+    def is_pattern_valid(pattern):
+        """Returns True if pattern is valid.
+
+        :param pattern: Normalized pattern.
+        is_pattern_valid() assumes pattern to be normalized.
+        see: globbing.normalize_pattern
+        """
+        result = True
+        translator = Globster.pattern_info[Globster.identify(pattern)]["translator"]
+        tpattern = '(%s)' % translator(pattern)
+        try:
+            re_obj = re.compile(tpattern, re.UNICODE)
+            re_obj.search("") # force compile
+        except errors.InvalidPattern, e:
+            result = False
+        return result
+
+
 class ExceptionGlobster(object):
     """A Globster that supports exception patterns.
     
@@ -262,14 +329,9 @@
         self._regex_patterns = []
         for pat in patterns:
             pat = normalize_pattern(pat)
-            if pat.startswith(u'RE:') or u'/' in pat:
-                self._add_patterns([pat], _sub_fullpath)
-            elif pat.startswith(u'*.'):
-                self._add_patterns([pat], _sub_extension,
-                    prefix=r'(?:.*/)?(?!.*/)(?:.*\.)')
-            else:
-                self._add_patterns([pat], _sub_basename,
-                    prefix=r'(?:.*/)?(?!.*/)')
+            t = Globster.identify(pat)
+            self._add_patterns([pat], Globster.pattern_info[t]["translator"],
+                Globster.pattern_info[t]["prefix"])
 
 
 _slashes = re.compile(r'[\\/]+')

=== modified file 'bzrlib/graph.py'
--- bzrlib/graph.py	2010-04-10 01:22:00 +0000
+++ bzrlib/graph.py	2010-08-15 15:20:14 +0000
@@ -258,6 +258,40 @@
         right = searchers[1].seen
         return (left.difference(right), right.difference(left))
 
+    def find_descendants(self, old_key, new_key):
+        """Find descendants of old_key that are ancestors of new_key."""
+        child_map = self.get_child_map(self._find_descendant_ancestors(
+            old_key, new_key))
+        graph = Graph(DictParentsProvider(child_map))
+        searcher = graph._make_breadth_first_searcher([old_key])
+        list(searcher)
+        return searcher.seen
+
+    def _find_descendant_ancestors(self, old_key, new_key):
+        """Find ancestors of new_key that may be descendants of old_key."""
+        stop = self._make_breadth_first_searcher([old_key])
+        descendants = self._make_breadth_first_searcher([new_key])
+        for revisions in descendants:
+            old_stop = stop.seen.intersection(revisions)
+            descendants.stop_searching_any(old_stop)
+            seen_stop = descendants.find_seen_ancestors(stop.step())
+            descendants.stop_searching_any(seen_stop)
+        return descendants.seen.difference(stop.seen)
+
+    def get_child_map(self, keys):
+        """Get a mapping from parents to children of the specified keys.
+
+        This is simply the inversion of get_parent_map.  Only supplied keys
+        will be discovered as children.
+        :return: a dict of key:child_list for keys.
+        """
+        parent_map = self._parents_provider.get_parent_map(keys)
+        parent_child = {}
+        for child, parents in sorted(parent_map.items()):
+            for parent in parents:
+                parent_child.setdefault(parent, []).append(child)
+        return parent_child
+
     def find_distance_to_null(self, target_revision_id, known_revision_ids):
         """Find the left-hand distance to the NULL_REVISION.
 
@@ -862,6 +896,26 @@
                 stop.add(parent_id)
         return found
 
+    def find_lefthand_merger(self, merged_key, tip_key):
+        """Find the first lefthand ancestor of tip_key that merged merged_key.
+
+        We do this by first finding the descendants of merged_key, then
+        walking through the lefthand ancestry of tip_key until we find a key
+        that doesn't descend from merged_key.  Its child is the key that
+        merged merged_key.
+
+        :return: The first lefthand ancestor of tip_key to merge merged_key.
+            merged_key if it is a lefthand ancestor of tip_key.
+            None if no ancestor of tip_key merged merged_key.
+        """
+        descendants = self.find_descendants(merged_key, tip_key)
+        candidate_iterator = self.iter_lefthand_ancestry(tip_key)
+        last_candidate = None
+        for candidate in candidate_iterator:
+            if candidate not in descendants:
+                return last_candidate
+            last_candidate = candidate
+
     def find_unique_lca(self, left_revision, right_revision,
                         count_steps=False):
         """Find a unique LCA.
@@ -919,6 +973,25 @@
                 yield (ghost, None)
             pending = next_pending
 
+    def iter_lefthand_ancestry(self, start_key, stop_keys=None):
+        if stop_keys is None:
+            stop_keys = ()
+        next_key = start_key
+        def get_parents(key):
+            try:
+                return self._parents_provider.get_parent_map([key])[key]
+            except KeyError:
+                raise errors.RevisionNotPresent(next_key, self)
+        while True:
+            if next_key in stop_keys:
+                return
+            parents = get_parents(next_key)
+            yield next_key
+            if len(parents) == 0:
+                return
+            else:
+                next_key = parents[0]
+
     def iter_topo_order(self, revisions):
         """Iterate through the input revisions in topological order.
 

=== modified file 'bzrlib/groupcompress.py'
--- bzrlib/groupcompress.py	2010-04-22 19:05:14 +0000
+++ bzrlib/groupcompress.py	2010-09-26 20:45:28 +0000
@@ -101,7 +101,7 @@
     def __init__(self):
         # map by key? or just order in file?
         self._compressor_name = None
-        self._z_content = None
+        self._z_content_chunks = None
         self._z_content_decompressor = None
         self._z_content_length = None
         self._content_length = None
@@ -135,26 +135,30 @@
                 self._content = ''.join(self._content_chunks)
                 self._content_chunks = None
         if self._content is None:
-            if self._z_content is None:
+            # We join self._z_content_chunks here, because if we are
+            # decompressing, then it is *very* likely that we have a single
+            # chunk
+            if self._z_content_chunks is None:
                 raise AssertionError('No content to decompress')
-            if self._z_content == '':
+            z_content = ''.join(self._z_content_chunks)
+            if z_content == '':
                 self._content = ''
             elif self._compressor_name == 'lzma':
                 # We don't do partial lzma decomp yet
-                self._content = pylzma.decompress(self._z_content)
+                self._content = pylzma.decompress(z_content)
             elif self._compressor_name == 'zlib':
                 # Start a zlib decompressor
                 if num_bytes * 4 > self._content_length * 3:
                     # If we are requesting more that 3/4ths of the content,
                     # just extract the whole thing in a single pass
                     num_bytes = self._content_length
-                    self._content = zlib.decompress(self._z_content)
+                    self._content = zlib.decompress(z_content)
                 else:
                     self._z_content_decompressor = zlib.decompressobj()
                     # Seed the decompressor with the uncompressed bytes, so
                     # that the rest of the code is simplified
                     self._content = self._z_content_decompressor.decompress(
-                        self._z_content, num_bytes + _ZLIB_DECOMP_WINDOW)
+                        z_content, num_bytes + _ZLIB_DECOMP_WINDOW)
                     if not self._z_content_decompressor.unconsumed_tail:
                         self._z_content_decompressor = None
             else:
@@ -207,7 +211,17 @@
             # XXX: Define some GCCorrupt error ?
             raise AssertionError('Invalid bytes: (%d) != %d + %d' %
                                  (len(bytes), pos, self._z_content_length))
-        self._z_content = bytes[pos:]
+        self._z_content_chunks = (bytes[pos:],)
+
+    @property
+    def _z_content(self):
+        """Return z_content_chunks as a simple string.
+
+        Meant only to be used by the test suite.
+        """
+        if self._z_content_chunks is not None:
+            return ''.join(self._z_content_chunks)
+        return None
 
     @classmethod
     def from_bytes(cls, bytes):
@@ -269,13 +283,13 @@
         self._content_length = length
         self._content_chunks = content_chunks
         self._content = None
-        self._z_content = None
+        self._z_content_chunks = None
 
     def set_content(self, content):
         """Set the content of this block."""
         self._content_length = len(content)
         self._content = content
-        self._z_content = None
+        self._z_content_chunks = None
 
     def _create_z_content_using_lzma(self):
         if self._content_chunks is not None:
@@ -283,39 +297,49 @@
             self._content_chunks = None
         if self._content is None:
             raise AssertionError('Nothing to compress')
-        self._z_content = pylzma.compress(self._content)
-        self._z_content_length = len(self._z_content)
+        z_content = pylzma.compress(self._content)
+        self._z_content_chunks = (z_content,)
+        self._z_content_length = len(z_content)
 
-    def _create_z_content_from_chunks(self):
+    def _create_z_content_from_chunks(self, chunks):
         compressor = zlib.compressobj(zlib.Z_DEFAULT_COMPRESSION)
-        compressed_chunks = map(compressor.compress, self._content_chunks)
+        # Peak in this point is 1 fulltext, 1 compressed text, + zlib overhead
+        # (measured peak is maybe 30MB over the above...)
+        compressed_chunks = map(compressor.compress, chunks)
         compressed_chunks.append(compressor.flush())
-        self._z_content = ''.join(compressed_chunks)
-        self._z_content_length = len(self._z_content)
+        # Ignore empty chunks
+        self._z_content_chunks = [c for c in compressed_chunks if c]
+        self._z_content_length = sum(map(len, self._z_content_chunks))
 
     def _create_z_content(self):
-        if self._z_content is not None:
+        if self._z_content_chunks is not None:
             return
         if _USE_LZMA:
             self._create_z_content_using_lzma()
             return
         if self._content_chunks is not None:
-            self._create_z_content_from_chunks()
-            return
-        self._z_content = zlib.compress(self._content)
-        self._z_content_length = len(self._z_content)
+            chunks = self._content_chunks
+        else:
+            chunks = (self._content,)
+        self._create_z_content_from_chunks(chunks)
 
-    def to_bytes(self):
-        """Encode the information into a byte stream."""
+    def to_chunks(self):
+        """Create the byte stream as a series of 'chunks'"""
         self._create_z_content()
         if _USE_LZMA:
             header = self.GCB_LZ_HEADER
         else:
             header = self.GCB_HEADER
-        chunks = [header,
-                  '%d\n%d\n' % (self._z_content_length, self._content_length),
-                  self._z_content,
+        chunks = ['%s%d\n%d\n'
+                  % (header, self._z_content_length, self._content_length),
                  ]
+        chunks.extend(self._z_content_chunks)
+        total_len = sum(map(len, chunks))
+        return total_len, chunks
+
+    def to_bytes(self):
+        """Encode the information into a byte stream."""
+        total_len, chunks = self.to_chunks()
         return ''.join(chunks)
 
     def _dump(self, include_text=False):
@@ -679,18 +703,21 @@
         z_header_bytes = zlib.compress(header_bytes)
         del header_bytes
         z_header_bytes_len = len(z_header_bytes)
-        block_bytes = self._block.to_bytes()
+        block_bytes_len, block_chunks = self._block.to_chunks()
         lines.append('%d\n%d\n%d\n' % (z_header_bytes_len, header_bytes_len,
-                                       len(block_bytes)))
+                                       block_bytes_len))
         lines.append(z_header_bytes)
-        lines.append(block_bytes)
-        del z_header_bytes, block_bytes
+        lines.extend(block_chunks)
+        del z_header_bytes, block_chunks
+        # TODO: This is a point where we will double the memory consumption. To
+        #       avoid this, we probably have to switch to a 'chunked' api
         return ''.join(lines)
 
     @classmethod
     def from_bytes(cls, bytes):
         # TODO: This does extra string copying, probably better to do it a
-        #       different way
+        #       different way. At a minimum this creates 2 copies of the
+        #       compressed content
         (storage_kind, z_header_len, header_len,
          block_len, rest) = bytes.split('\n', 4)
         del bytes
@@ -854,14 +881,6 @@
 
         After calling this, the compressor should no longer be used
         """
-        # TODO: this causes us to 'bloat' to 2x the size of content in the
-        #       group. This has an impact for 'commit' of large objects.
-        #       One possibility is to use self._content_chunks, and be lazy and
-        #       only fill out self._content as a full string when we actually
-        #       need it. That would at least drop the peak memory consumption
-        #       for 'commit' down to ~1x the size of the largest file, at a
-        #       cost of increased complexity within this code. 2x is still <<
-        #       3x the size of the largest file, so we are doing ok.
         self._block.set_chunked_content(self.chunks, self.endpoint)
         self.chunks = None
         self._delta_index = None
@@ -1630,8 +1649,19 @@
         self._unadded_refs = {}
         keys_to_add = []
         def flush():
-            bytes = self._compressor.flush().to_bytes()
+            bytes_len, chunks = self._compressor.flush().to_chunks()
             self._compressor = GroupCompressor()
+            # Note: At this point we still have 1 copy of the fulltext (in
+            #       record and the var 'bytes'), and this generates 2 copies of
+            #       the compressed text (one for bytes, one in chunks)
+            # TODO: Push 'chunks' down into the _access api, so that we don't
+            #       have to double compressed memory here
+            # TODO: Figure out how to indicate that we would be happy to free
+            #       the fulltext content at this point. Note that sometimes we
+            #       will want it later (streaming CHK pages), but most of the
+            #       time we won't (everything else)
+            bytes = ''.join(chunks)
+            del chunks
             index, start, length = self._access.add_raw_records(
                 [(None, len(bytes))], bytes)[0]
             nodes = []
@@ -1809,6 +1839,54 @@
         return result
 
 
+class _GCBuildDetails(object):
+    """A blob of data about the build details.
+
+    This stores the minimal data, which then allows compatibility with the old
+    api, without taking as much memory.
+    """
+
+    __slots__ = ('_index', '_group_start', '_group_end', '_basis_end',
+                 '_delta_end', '_parents')
+
+    method = 'group'
+    compression_parent = None
+
+    def __init__(self, parents, position_info):
+        self._parents = parents
+        (self._index, self._group_start, self._group_end, self._basis_end,
+         self._delta_end) = position_info
+
+    def __repr__(self):
+        return '%s(%s, %s)' % (self.__class__.__name__,
+            self.index_memo, self._parents)
+
+    @property
+    def index_memo(self):
+        return (self._index, self._group_start, self._group_end,
+                self._basis_end, self._delta_end)
+
+    @property
+    def record_details(self):
+        return static_tuple.StaticTuple(self.method, None)
+
+    def __getitem__(self, offset):
+        """Compatibility thunk to act like a tuple."""
+        if offset == 0:
+            return self.index_memo
+        elif offset == 1:
+            return self.compression_parent # Always None
+        elif offset == 2:
+            return self._parents
+        elif offset == 3:
+            return self.record_details
+        else:
+            raise IndexError('offset out of range')
+            
+    def __len__(self):
+        return 4
+
+
 class _GCGraphIndex(object):
     """Mapper from GroupCompressVersionedFiles needs into GraphIndex storage."""
 
@@ -2009,9 +2087,8 @@
                 parents = None
             else:
                 parents = entry[3][0]
-            method = 'group'
-            result[key] = (self._node_to_position(entry),
-                                  None, parents, (method, None))
+            details = _GCBuildDetails(parents, self._node_to_position(entry))
+            result[key] = details
         return result
 
     def keys(self):
@@ -2033,7 +2110,7 @@
         # each, or about 7MB. Note that it might be even more when you consider
         # how PyInt is allocated in separate slabs. And you can't return a slab
         # to the OS if even 1 int on it is in use. Note though that Python uses
-        # a LIFO when re-using PyInt slots, which probably causes more
+        # a LIFO when re-using PyInt slots, which might cause more
         # fragmentation.
         start = int(bits[0])
         start = self._int_cache.setdefault(start, start)

=== modified file 'bzrlib/hashcache.py'
--- bzrlib/hashcache.py	2009-05-06 05:36:28 +0000
+++ bzrlib/hashcache.py	2010-05-31 21:31:18 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -289,6 +289,9 @@
 
             self._cache[path] = (sha1, fp)
 
+        # GZ 2009-09-20: Should really use a try/finally block to ensure close
+        inf.close()
+
         self.needs_write = False
 
     def _cutoff_time(self):

=== modified file 'bzrlib/help.py'
--- bzrlib/help.py	2010-04-30 11:03:59 +0000
+++ bzrlib/help.py	2010-05-26 07:19:50 +0000
@@ -44,9 +44,9 @@
     try:
         topics = indices.search(topic)
         shadowed_terms = []
-        for index, topic in topics[1:]:
+        for index, topic_obj in topics[1:]:
             shadowed_terms.append('%s%s' % (index.prefix,
-                topic.get_help_topic()))
+                topic_obj.get_help_topic()))
         source = topics[0][1]
         outfile.write(source.get_help_text(shadowed_terms))
     except errors.NoHelpTopic:

=== modified file 'bzrlib/help_topics/__init__.py'
--- bzrlib/help_topics/__init__.py	2010-03-02 10:21:39 +0000
+++ bzrlib/help_topics/__init__.py	2010-10-02 01:58:11 +0000
@@ -279,7 +279,7 @@
 
 _basic_help = \
 """Bazaar %s -- a free distributed version-control tool
-http://www.bazaar.canonical.com/
+http://bazaar.canonical.com/
 
 Basic commands:
   bzr init           makes this directory a versioned branch
@@ -512,7 +512,7 @@
 
   checkout     Create a working tree when a branch does not have one.
   remove-tree  Removes the working tree from a branch when it is safe to do so.
-  update       When a working tree is out of sync with it's associated branch
+  update       When a working tree is out of sync with its associated branch
                this will update the tree to match the branch.
 """
 
@@ -591,29 +591,39 @@
 _env_variables = \
 """Environment Variables
 
-================ =================================================================
-BZRPATH          Path where bzr is to look for shell plugin external commands.
-BZR_EMAIL        E-Mail address of the user. Overrides EMAIL.
-EMAIL            E-Mail address of the user.
-BZR_EDITOR       Editor for editing commit messages. Overrides EDITOR.
-EDITOR           Editor for editing commit messages.
-BZR_PLUGIN_PATH  Paths where bzr should look for plugins.
-BZR_HOME         Directory holding .bazaar config dir. Overrides HOME.
-BZR_HOME (Win32) Directory holding bazaar config dir. Overrides APPDATA and HOME.
-BZR_REMOTE_PATH  Full name of remote 'bzr' command (for bzr+ssh:// URLs).
-BZR_SSH          Path to SSH client, or one of paramiko, openssh, sshcorp, plink.
-BZR_LOG          Location of .bzr.log (use '/dev/null' to suppress log).
-BZR_LOG (Win32)  Location of .bzr.log (use 'NUL' to suppress log).
-BZR_COLUMNS      Override implicit terminal width.
-BZR_CONCURRENCY  Number of processes that can be run concurrently (selftest).
-================ =================================================================
+=================== ===========================================================
+BZRPATH             Path where bzr is to look for shell plugin external
+                    commands.
+BZR_EMAIL           E-Mail address of the user. Overrides EMAIL.
+EMAIL               E-Mail address of the user.
+BZR_EDITOR          Editor for editing commit messages. Overrides EDITOR.
+EDITOR              Editor for editing commit messages.
+BZR_PLUGIN_PATH     Paths where bzr should look for plugins.
+BZR_DISABLE_PLUGINS Plugins that bzr should not load.
+BZR_PLUGINS_AT      Plugins to load from a directory not in BZR_PLUGIN_PATH.
+BZR_HOME            Directory holding .bazaar config dir. Overrides HOME.
+BZR_HOME (Win32)    Directory holding bazaar config dir. Overrides APPDATA and
+                    HOME.
+BZR_REMOTE_PATH     Full name of remote 'bzr' command (for bzr+ssh:// URLs).
+BZR_SSH             Path to SSH client, or one of paramiko, openssh, sshcorp,
+                    plink or lsh.
+BZR_LOG             Location of .bzr.log (use '/dev/null' to suppress log).
+BZR_LOG (Win32)     Location of .bzr.log (use 'NUL' to suppress log).
+BZR_COLUMNS         Override implicit terminal width.
+BZR_CONCURRENCY     Number of processes that can be run concurrently (selftest)
+BZR_PROGRESS_BAR    Override the progress display. Values are 'none', 'dots',
+                    or 'tty'.
+BZR_PDB             Control whether to launch a debugger on error.
+BZR_SIGQUIT_PDB     Control whether SIGQUIT behaves normally or invokes a
+                    breakin debugger.
+=================== ===========================================================
 """
 
 
 _files = \
 r"""Files
 
-:On Linux:   ~/.bazaar/bazaar.conf
+:On Unix:   ~/.bazaar/bazaar.conf
 :On Windows: C:\\Documents and Settings\\username\\Application Data\\bazaar\\2.0\\bazaar.conf
 
 Contains the user's default configuration. The section ``[DEFAULT]`` is
@@ -877,6 +887,8 @@
 def help_as_plain_text(text):
     """Minimal converter of reStructuredText to plain text."""
     import re
+    # Remove the standalone code block marker
+    text = re.sub(r"(?m)^\s*::\n\s*$", "", text)
     lines = text.splitlines()
     result = []
     for line in lines:

=== modified file 'bzrlib/help_topics/en/configuration.txt'
--- bzrlib/help_topics/en/configuration.txt	2010-04-23 07:15:23 +0000
+++ bzrlib/help_topics/en/configuration.txt	2010-07-06 01:26:59 +0000
@@ -59,6 +59,19 @@
 
 Path to the editor Bazaar should use for commit messages, etc.
 
+BZR_LOG
+~~~~~~~
+
+Location of the Bazaar log file. You can check the current location by
+running ``bzr version``.
+
+The log file contains debug information that is useful for diagnosing or
+reporting problems with Bazaar.
+
+Setting this to ``NUL`` on Windows or ``/dev/null`` on other platforms
+will disable logging.
+
+
 BZR_PLUGIN_PATH
 ~~~~~~~~~~~~~~~
 
@@ -77,7 +90,7 @@
 
 As for the ``PATH`` variables, if multiple directories are
 specified in ``BZR_PLUGIN_PATH`` they should be separated by the
-platform specific appropriate character (':' on Unix/Linux/etc,
+platform specific appropriate character (':' on Unix,
 ';' on windows)
 
 By default if ``BZR_PLUGIN_PATH`` is set, it replaces searching
@@ -166,13 +179,22 @@
 The path where Bazaar should look for shell plugin external commands.
 
 
+http_proxy, https_proxy
+~~~~~~~~~~~~~~~~~~~~~~~
+
+Specifies the network proxy for outgoing connections, for example::
+
+  http_proxy=http://proxy.example.com:3128/ 
+  https_proxy=http://proxy.example.com:3128/
+
+
 Configuration files
 -------------------
 
 Location
 ~~~~~~~~
 
-Configuration files are located in ``$HOME/.bazaar`` on Linux/Unix and
+Configuration files are located in ``$HOME/.bazaar`` on Unix and
 ``C:\Documents and Settings\<username>\Application Data\Bazaar\2.0`` on
 Windows. (You can check the location for your system by using
 ``bzr version``.)
@@ -483,6 +505,17 @@
     using deprecated formats.
 
 
+Unicode options
+---------------
+
+output_encoding
+~~~~~~~~~~~~~~~
+
+A Python unicode encoding name for text output from bzr, such as log
+information.  Values include: utf8, cp850, ascii, iso-8859-1.  The default
+is the terminal encoding prefered by the operating system.
+
+
 Branch type specific options
 ----------------------------
 

=== modified file 'bzrlib/help_topics/en/conflict-types.txt'
--- bzrlib/help_topics/en/conflict-types.txt	2010-02-04 13:29:34 +0000
+++ bzrlib/help_topics/en/conflict-types.txt	2010-04-28 13:15:14 +0000
@@ -28,11 +28,11 @@
 
 Whatever the conflict is, resolving it is roughly done in two steps::
 
-- modify the working tree content so that the conflicted item is now in the
-  state you want to keep,
+ - modify the working tree content so that the conflicted item is now in the
+   state you want to keep,
 
-- inform Bazaar that the conflict is now solved and ask to cleanup any
-  remaining generated information (``bzr resolve <item>``).
+ - inform Bazaar that the conflict is now solved and ask to cleanup any
+   remaining generated information (``bzr resolve <item>``).
 
 For most conflict types, there are some obvious ways to modify the working
 tree and put it into the desired state. For some types of conflicts, Bazaar
@@ -126,9 +126,9 @@
 
 ``bzr resolve`` recognizes the following actions:
 
-- ``--action=take-this`` will issue ``bzr mv FILE.THIS FILE``,
-- ``--action=take-other`` will issue ``bzr mv FILE.OTHER FILE``,
-- ``--action=done`` will just mark the conflict as resolved.
+ - ``--action=take-this`` will issue ``bzr mv FILE.THIS FILE``,
+ - ``--action=take-other`` will issue ``bzr mv FILE.OTHER FILE``,
+ - ``--action=done`` will just mark the conflict as resolved.
 
 Any action will also delete the previously generated ``.BASE``, ``.THIS`` and
 ``.OTHER`` files if they are still present in the working tree.
@@ -174,9 +174,9 @@
 
 ``bzr resolve`` recognizes the following actions::
 
-- ``--action=take-this`` will issue ``bzr rm FILE ; bzr mv FILE.moved FILE``,
-- ``--action=take-other`` will issue ``bzr rm FILE.moved``,
-- ``--action=done`` will just mark the conflict as resolved.
+ - ``--action=take-this`` will issue ``bzr rm FILE ; bzr mv FILE.moved FILE``,
+ - ``--action=take-other`` will issue ``bzr rm FILE.moved``,
+ - ``--action=done`` will just mark the conflict as resolved.
 
 Note that you must get rid of FILE.moved before using ``--action=done``.
 
@@ -215,10 +215,11 @@
 
 ``bzr resolve`` recognizes the following actions::
 
-- ``--action=take-this`` will issue ``bzr rm directory`` including the children,
-- ``--action=take-other`` will acknowledge Bazaar choice to keep the children
-  and restoring the directory,
-- ``--action=done`` will just mark the conflict as resolved.
+ - ``--action=take-this`` will issue ``bzr rm directory`` including the
+   children,
+ - ``--action=take-other`` will acknowledge Bazaar choice to keep the children
+   and restoring the directory,
+ - ``--action=done`` will just mark the conflict as resolved.
 
 Bazaar cannot auto-detect when conflicts of this kind have been resolved.
 
@@ -238,10 +239,10 @@
 
 ``bzr resolve`` recognizes the following actions::
 
-- ``--action=take-this`` will acknowledge Bazaar choice to keep the directory,
-- ``--action=take-other`` will issue ``bzr rm directory`` including the 
-  children,
-- ``--action=done`` will just mark the conflict as resolved.
+ - ``--action=take-this`` will acknowledge Bazaar choice to keep the directory,
+ - ``--action=take-other`` will issue ``bzr rm directory`` including the 
+   children,
+ - ``--action=done`` will just mark the conflict as resolved.
 
 Bazaar cannot auto-detect when conflicts of this kind have been resolved.
 
@@ -260,10 +261,10 @@
 
 ``bzr resolve`` recognizes the following actions::
 
-- ``--action=take-this`` will revert Bazaar choice and keep ``PATH1`` by
-  issuing ``bzr mv PATH2 PATH1``,
-- ``--action=take-other`` will acknowledge Bazaar choice of keeping ``PATH2``,
-- ``--action=done`` will just mark the conflict as resolved.
+ - ``--action=take-this`` will revert Bazaar choice and keep ``PATH1`` by
+   issuing ``bzr mv PATH2 PATH1``,
+ - ``--action=take-other`` will acknowledge Bazaar choice of keeping ``PATH2``,
+ - ``--action=done`` will just mark the conflict as resolved.
 
 Bazaar cannot auto-detect when conflicts of this kind have been resolved.
 
@@ -295,11 +296,11 @@
 
 ``bzr resolve`` recognizes the following actions::
 
-- ``--action=take-this`` will acknowledge Bazaar choice of leaving ``white`` 
-  in ``black``,
-- ``--action=take-other`` will revert Bazaar choice and move ``black`` in
-  ``white`` by issuing ``bzr mv black/white white ; bzr mv black white``,
-- ``--action=done`` will just mark the conflict as resolved.
+ - ``--action=take-this`` will acknowledge Bazaar choice of leaving ``white`` 
+   in ``black``,
+ - ``--action=take-other`` will revert Bazaar choice and move ``black`` in
+   ``white`` by issuing ``bzr mv black/white white ; bzr mv black white``,
+ - ``--action=done`` will just mark the conflict as resolved.
 
 Bazaar cannot auto-detect when conflicts of this kind have been resolved.
 
@@ -330,11 +331,11 @@
 
 ``bzr resolve`` recognizes the following actions::
 
-- ``--action=take-this`` will issue ``bzr rm --force foo.new`` and 
-  ``bzr add foo``,
-- ``--action=take-other`` will issue ``bzr rm --force foo`` and 
-  ``bzr mv foo.new foo``,
-- ``--action=done`` will just mark the conflict as resolved.
+ - ``--action=take-this`` will issue ``bzr rm --force foo.new`` and 
+   ``bzr add foo``,
+ - ``--action=take-other`` will issue ``bzr rm --force foo`` and 
+   ``bzr mv foo.new foo``,
+ - ``--action=done`` will just mark the conflict as resolved.
 
 Bazaar cannot auto-detect when conflicts of this kind have been resolved.
 

=== modified file 'bzrlib/help_topics/en/content-filters.txt'
--- bzrlib/help_topics/en/content-filters.txt	2010-01-03 03:33:10 +0000
+++ bzrlib/help_topics/en/content-filters.txt	2010-06-02 05:03:31 +0000
@@ -8,7 +8,7 @@
 format from the copy in your working tree.  This lets you, or your
 co-developers, use Windows development tools that expect CRLF files
 on projects that use other line-ending conventions. Among other things,
-content filters also let Linux developers more easily work on projects
+content filters also let Unix developers more easily work on projects
 using Windows line-ending conventions, keyword expansion/compression,
 and trailing spaces on lines in text files to be implicitly stripped
 when committed.

=== modified file 'bzrlib/help_topics/en/patterns.txt'
--- bzrlib/help_topics/en/patterns.txt	2010-01-02 07:36:36 +0000
+++ bzrlib/help_topics/en/patterns.txt	2010-07-16 12:51:21 +0000
@@ -10,7 +10,7 @@
 slash or is a regular expression, it is compared to the whole path
 from the branch root. Otherwise, it is compared to only the last
 component of the path. To match a file only in the root directory,
-prepend './'. Patterns specifying absolute paths are not allowed.
+prepend ``./``. Patterns specifying absolute paths are not allowed.
 
 Patterns may include globbing wildcards such as::
 
@@ -19,10 +19,20 @@
   /**/ - Matches 0 or more directories in a path
   [a-z] - Matches a single character from within a group of characters
 
-Patterns may also be Python regular expressions. Regular expression
-patterns are identified by a 'RE:' prefix followed by the regular
+Patterns may also be `Python regular expressions`_. Regular expression
+patterns are identified by a ``RE:`` prefix followed by the regular
 expression.  Regular expression patterns may not include named or
 numbered groups.
 
-Ignore patterns may be prefixed with '!', which means that a filename
+Case insensitive ignore patterns can be specified with regular expressions
+by using the ``i`` (for ignore case) flag in the pattern.
+
+For example, a case insensitive match for ``foo`` may be specified as::
+
+  RE:(?i)foo
+
+Ignore patterns may be prefixed with ``!``, which means that a filename
 matched by that pattern will not be ignored.
+
+.. _Python regular expressions: http://docs.python.org/library/re.html
+

=== modified file 'bzrlib/hooks.py'
--- bzrlib/hooks.py	2010-04-30 11:03:59 +0000
+++ bzrlib/hooks.py	2010-08-28 06:13:48 +0000
@@ -54,6 +54,8 @@
     'bzrlib.smart.client', 'SmartClientHooks')
 known_hooks.register_lazy(('bzrlib.smart.server', 'SmartTCPServer.hooks'),
     'bzrlib.smart.server', 'SmartServerHooks')
+known_hooks.register_lazy(('bzrlib.status', 'hooks'),
+    'bzrlib.status', 'StatusHooks')
 known_hooks.register_lazy(
     ('bzrlib.version_info_formats.format_rio', 'RioVersionInfoBuilder.hooks'),
     'bzrlib.version_info_formats.format_rio', 'RioVersionInfoBuilderHooks')
@@ -179,13 +181,13 @@
     """A single hook that clients can register to be called back when it fires.
 
     :ivar name: The name of the hook.
+    :ivar doc: The docs for using the hook.
     :ivar introduced: A version tuple specifying what version the hook was
         introduced in. None indicates an unknown version.
     :ivar deprecated: A version tuple specifying what version the hook was
         deprecated or superseded in. None indicates that the hook is not
         superseded or deprecated. If the hook is superseded then the doc
         should describe the recommended replacement hook to register for.
-    :ivar doc: The docs for using the hook.
     """
 
     def __init__(self, name, doc, introduced, deprecated):
@@ -276,9 +278,7 @@
 
   yyy.hooks.install_named_hook("xxx", ...)
 
-See `Using hooks`_ in the User Guide for examples.
-
-.. _Using hooks: ../user-guide/hooks.html
+See :doc:`Using hooks<../user-guide/hooks>` in the User Guide for examples.
 
 The class that contains each hook is given before the hooks it supplies. For
 instance, BranchHooks as the class is the hooks class for

=== modified file 'bzrlib/ignores.py'
--- bzrlib/ignores.py	2010-04-30 07:54:33 +0000
+++ bzrlib/ignores.py	2010-09-04 15:34:10 +0000
@@ -17,6 +17,8 @@
 """Lists of ignore files, etc."""
 
 import errno
+import os
+from cStringIO import StringIO
 
 import bzrlib
 from bzrlib import (
@@ -39,6 +41,7 @@
     '*~',
     '.#*',
     '[#]*#',
+    '__pycache__',
 ]
 
 
@@ -181,28 +184,44 @@
     The ignore file will be automatically added under version control.
 
     :param tree: Working tree to update the ignore list.
+    :param name_pattern_list: List of ignore patterns.
+    :return: None
     """
+    # read in the existing ignores set
     ifn = tree.abspath(bzrlib.IGNORE_FILENAME)
     if tree.has_filename(ifn):
-        f = open(ifn, 'rt')
+        f = open(ifn, 'rU')
         try:
-            igns = f.read().decode('utf-8')
+            file_contents = f.read()
+            # figure out what kind of line endings are used
+            newline = getattr(f, 'newlines', None)
+            if type(newline) is tuple:
+                newline = newline[0]
+            elif newline is None:
+                newline = os.linesep
         finally:
             f.close()
     else:
-        igns = ""
-
-    # TODO: If the file already uses crlf-style termination, maybe
-    # we should use that for the newly added lines?
-
-    if igns and igns[-1] != '\n':
-        igns += '\n'
-    for name_pattern in name_pattern_list:
-        igns += name_pattern + '\n'
-
+        file_contents = ""
+        newline = os.linesep
+    
+    sio = StringIO(file_contents)
+    try:
+        ignores = parse_ignore_file(sio)
+    finally:
+        sio.close()
+    
+    # write out the updated ignores set
     f = atomicfile.AtomicFile(ifn, 'wb')
     try:
-        f.write(igns.encode('utf-8'))
+        # write the original contents, preserving original line endings
+        f.write(newline.join(file_contents.split('\n')))
+        if len(file_contents) > 0 and not file_contents.endswith('\n'):
+            f.write(newline)
+        for pattern in name_pattern_list:
+            if not pattern in ignores:
+                f.write(pattern.encode('utf-8'))
+                f.write(newline)
         f.commit()
     finally:
         f.close()

=== modified file 'bzrlib/index.py'
--- bzrlib/index.py	2010-04-14 05:06:53 +0000
+++ bzrlib/index.py	2010-05-25 17:27:52 +0000
@@ -462,6 +462,7 @@
         trailers = 0
         pos = stream.tell()
         lines = stream.read().split('\n')
+        # GZ 2009-09-20: Should really use a try/finally block to ensure close
         stream.close()
         del lines[-1]
         _, _, _, trailers = self._parse_lines(lines, pos)

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2010-02-17 17:11:16 +0000
+++ bzrlib/inventory.py	2010-09-14 13:12:20 +0000
@@ -35,13 +35,11 @@
 import re
 import tarfile
 
-import bzrlib
 from bzrlib import (
     chk_map,
     errors,
     generate_ids,
     osutils,
-    symbol_versioning,
     )
 """)
 
@@ -49,7 +47,6 @@
     BzrCheckError,
     BzrError,
     )
-from bzrlib.symbol_versioning import deprecated_in, deprecated_method
 from bzrlib.trace import mutter
 from bzrlib.static_tuple import StaticTuple
 
@@ -131,7 +128,23 @@
     RENAMED = 'renamed'
     MODIFIED_AND_RENAMED = 'modified and renamed'
 
-    __slots__ = []
+    __slots__ = ['file_id', 'revision', 'parent_id', 'name']
+
+    # Attributes that all InventoryEntry instances are expected to have, but
+    # that don't vary for all kinds of entry.  (e.g. symlink_target is only
+    # relevant to InventoryLink, so there's no reason to make every
+    # InventoryFile instance allocate space to hold a value for it.)
+    # Attributes that only vary for files: executable, text_sha1, text_size,
+    # text_id
+    executable = False
+    text_sha1 = None
+    text_size = None
+    text_id = None
+    # Attributes that only vary for symlinks: symlink_target
+    symlink_target = None
+    # Attributes that only vary for tree-references: reference_revision
+    reference_revision = None
+
 
     def detect_changes(self, old_entry):
         """Return a (text_modified, meta_modified) from this to old_entry.
@@ -176,16 +189,6 @@
                     candidates[ie.revision] = ie
         return candidates
 
-    @deprecated_method(deprecated_in((1, 6, 0)))
-    def get_tar_item(self, root, dp, now, tree):
-        """Get a tarfile item and a file stream for its content."""
-        item = tarfile.TarInfo(osutils.pathjoin(root, dp).encode('utf8'))
-        # TODO: would be cool to actually set it to the timestamp of the
-        # revision it was last changed
-        item.mtime = now
-        fileobj = self._put_in_tar(item, tree)
-        return item, fileobj
-
     def has_text(self):
         """Return true if the object this entry represents has textual data.
 
@@ -197,7 +200,7 @@
         """
         return False
 
-    def __init__(self, file_id, name, parent_id, text_id=None):
+    def __init__(self, file_id, name, parent_id):
         """Create an InventoryEntry
 
         The filename must be a single component, relative to the
@@ -214,16 +217,10 @@
         """
         if '/' in name or '\\' in name:
             raise errors.InvalidEntryName(name=name)
-        self.executable = False
+        self.file_id = file_id
         self.revision = None
-        self.text_sha1 = None
-        self.text_size = None
-        self.file_id = file_id
         self.name = name
-        self.text_id = text_id
         self.parent_id = parent_id
-        self.symlink_target = None
-        self.reference_revision = None
 
     def kind_character(self):
         """Return a short kind indicator useful for appending to names."""
@@ -231,29 +228,6 @@
 
     known_kinds = ('file', 'directory', 'symlink')
 
-    def _put_in_tar(self, item, tree):
-        """populate item for stashing in a tar, and return the content stream.
-
-        If no content is available, return None.
-        """
-        raise BzrError("don't know how to export {%s} of kind %r" %
-                       (self.file_id, self.kind))
-
-    @deprecated_method(deprecated_in((1, 6, 0)))
-    def put_on_disk(self, dest, dp, tree):
-        """Create a representation of self on disk in the prefix dest.
-
-        This is a template method - implement _put_on_disk in subclasses.
-        """
-        fullpath = osutils.pathjoin(dest, dp)
-        self._put_on_disk(fullpath, tree)
-        # mutter("  export {%s} kind %s to %s", self.file_id,
-        #         self.kind, fullpath)
-
-    def _put_on_disk(self, fullpath, tree):
-        """Put this entry onto disk at fullpath, from tree tree."""
-        raise BzrError("don't know how to export {%s} of kind %r" % (self.file_id, self.kind))
-
     def sorted_children(self):
         return sorted(self.children.items())
 
@@ -397,47 +371,15 @@
         pass
 
 
-class RootEntry(InventoryEntry):
-
-    __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
-                 'text_id', 'parent_id', 'children', 'executable',
-                 'revision', 'symlink_target', 'reference_revision']
-
-    def _check(self, checker, rev_id):
-        """See InventoryEntry._check"""
-
-    def __init__(self, file_id):
-        self.file_id = file_id
-        self.children = {}
-        self.kind = 'directory'
-        self.parent_id = None
-        self.name = u''
-        self.revision = None
-        symbol_versioning.warn('RootEntry is deprecated as of bzr 0.10.'
-                               '  Please use InventoryDirectory instead.',
-                               DeprecationWarning, stacklevel=2)
-
-    def __eq__(self, other):
-        if not isinstance(other, RootEntry):
-            return NotImplemented
-
-        return (self.file_id == other.file_id) \
-               and (self.children == other.children)
-
-
 class InventoryDirectory(InventoryEntry):
     """A directory in an inventory."""
 
-    __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
-                 'text_id', 'parent_id', 'children', 'executable',
-                 'revision', 'symlink_target', 'reference_revision']
+    __slots__ = ['children']
+
+    kind = 'directory'
 
     def _check(self, checker, rev_id):
         """See InventoryEntry._check"""
-        if (self.text_sha1 is not None or self.text_size is not None or
-            self.text_id is not None):
-            checker._report_items.append('directory {%s} has text in revision {%s}'
-                                % (self.file_id, rev_id))
         # In non rich root repositories we do not expect a file graph for the
         # root.
         if self.name == '' and not checker.rich_roots:
@@ -459,32 +401,25 @@
     def __init__(self, file_id, name, parent_id):
         super(InventoryDirectory, self).__init__(file_id, name, parent_id)
         self.children = {}
-        self.kind = 'directory'
 
     def kind_character(self):
         """See InventoryEntry.kind_character."""
         return '/'
 
-    def _put_in_tar(self, item, tree):
-        """See InventoryEntry._put_in_tar."""
-        item.type = tarfile.DIRTYPE
-        fileobj = None
-        item.name += '/'
-        item.size = 0
-        item.mode = 0755
-        return fileobj
-
-    def _put_on_disk(self, fullpath, tree):
-        """See InventoryEntry._put_on_disk."""
-        os.mkdir(fullpath)
-
 
 class InventoryFile(InventoryEntry):
     """A file in an inventory."""
 
-    __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
-                 'text_id', 'parent_id', 'children', 'executable',
-                 'revision', 'symlink_target', 'reference_revision']
+    __slots__ = ['text_sha1', 'text_size', 'text_id', 'executable']
+
+    kind = 'file'
+
+    def __init__(self, file_id, name, parent_id):
+        super(InventoryFile, self).__init__(file_id, name, parent_id)
+        self.text_sha1 = None
+        self.text_size = None
+        self.text_id = None
+        self.executable = False
 
     def _check(self, checker, tree_revision_id):
         """See InventoryEntry._check"""
@@ -533,31 +468,10 @@
         """See InventoryEntry.has_text."""
         return True
 
-    def __init__(self, file_id, name, parent_id):
-        super(InventoryFile, self).__init__(file_id, name, parent_id)
-        self.kind = 'file'
-
     def kind_character(self):
         """See InventoryEntry.kind_character."""
         return ''
 
-    def _put_in_tar(self, item, tree):
-        """See InventoryEntry._put_in_tar."""
-        item.type = tarfile.REGTYPE
-        fileobj = tree.get_file(self.file_id)
-        item.size = self.text_size
-        if tree.is_executable(self.file_id):
-            item.mode = 0755
-        else:
-            item.mode = 0644
-        return fileobj
-
-    def _put_on_disk(self, fullpath, tree):
-        """See InventoryEntry._put_on_disk."""
-        osutils.pumpfile(tree.get_file(self.file_id), file(fullpath, 'wb'))
-        if tree.is_executable(self.file_id):
-            os.chmod(fullpath, 0755)
-
     def _read_tree_state(self, path, work_tree):
         """See InventoryEntry._read_tree_state."""
         self.text_sha1 = work_tree.get_file_sha1(self.file_id, path=path)
@@ -595,16 +509,16 @@
 class InventoryLink(InventoryEntry):
     """A file in an inventory."""
 
-    __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
-                 'text_id', 'parent_id', 'children', 'executable',
-                 'revision', 'symlink_target', 'reference_revision']
+    __slots__ = ['symlink_target']
+
+    kind = 'symlink'
+
+    def __init__(self, file_id, name, parent_id):
+        super(InventoryLink, self).__init__(file_id, name, parent_id)
+        self.symlink_target = None
 
     def _check(self, checker, tree_revision_id):
         """See InventoryEntry._check"""
-        if self.text_sha1 is not None or self.text_size is not None or self.text_id is not None:
-            checker._report_items.append(
-               'symlink {%s} has text in revision {%s}'
-                    % (self.file_id, tree_revision_id))
         if self.symlink_target is None:
             checker._report_items.append(
                 'symlink {%s} has no target in revision {%s}'
@@ -648,30 +562,10 @@
         differ = DiffSymlink(old_tree, new_tree, output_to)
         return differ.diff_symlink(old_target, new_target)
 
-    def __init__(self, file_id, name, parent_id):
-        super(InventoryLink, self).__init__(file_id, name, parent_id)
-        self.kind = 'symlink'
-
     def kind_character(self):
         """See InventoryEntry.kind_character."""
         return ''
 
-    def _put_in_tar(self, item, tree):
-        """See InventoryEntry._put_in_tar."""
-        item.type = tarfile.SYMTYPE
-        fileobj = None
-        item.size = 0
-        item.mode = 0755
-        item.linkname = self.symlink_target
-        return fileobj
-
-    def _put_on_disk(self, fullpath, tree):
-        """See InventoryEntry._put_on_disk."""
-        try:
-            os.symlink(self.symlink_target, fullpath)
-        except OSError,e:
-            raise BzrError("Failed to create symlink %r -> %r, error: %s" % (fullpath, self.symlink_target, e))
-
     def _read_tree_state(self, path, work_tree):
         """See InventoryEntry._read_tree_state."""
         self.symlink_target = work_tree.get_symlink_target(self.file_id)
@@ -689,6 +583,8 @@
 
 class TreeReference(InventoryEntry):
 
+    __slots__ = ['reference_revision']
+
     kind = 'tree-reference'
 
     def __init__(self, file_id, name, parent_id, revision=None,
@@ -940,7 +836,8 @@
                 if ie.kind == 'directory':
                     descend(ie, child_path)
 
-        descend(self.root, u'')
+        if self.root is not None:
+            descend(self.root, u'')
         return accum
 
     def directories(self):
@@ -1282,9 +1179,6 @@
     def add(self, entry):
         """Add entry to inventory.
 
-        To add  a file to a branch ready to be committed, use Branch.add,
-        which calls this.
-
         :return: entry
         """
         if entry.file_id in self._byid:
@@ -1652,11 +1546,7 @@
             # parent_to_children with at least the tree root.)
             return other
         cache = self._fileid_to_entry_cache
-        try:
-            remaining_children = collections.deque(parent_to_children[self.root_id])
-        except:
-            import pdb; pdb.set_trace()
-            raise
+        remaining_children = collections.deque(parent_to_children[self.root_id])
         while remaining_children:
             file_id = remaining_children.popleft()
             ie = cache[file_id]
@@ -2245,17 +2135,13 @@
 class CHKInventoryDirectory(InventoryDirectory):
     """A directory in an inventory."""
 
-    __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
-                 'text_id', 'parent_id', '_children', 'executable',
-                 'revision', 'symlink_target', 'reference_revision',
-                 '_chk_inventory']
+    __slots__ = ['_children', '_chk_inventory']
 
     def __init__(self, file_id, name, parent_id, chk_inventory):
         # Don't call InventoryDirectory.__init__ - it isn't right for this
         # class.
         InventoryEntry.__init__(self, file_id, name, parent_id)
         self._children = None
-        self.kind = 'directory'
         self._chk_inventory = chk_inventory
 
     @property

=== modified file 'bzrlib/knit.py'
--- bzrlib/knit.py	2010-02-23 07:43:11 +0000
+++ bzrlib/knit.py	2010-06-04 03:09:35 +0000
@@ -68,6 +68,7 @@
     index as _mod_index,
     lru_cache,
     pack,
+    patiencediff,
     progress,
     static_tuple,
     trace,
@@ -79,7 +80,6 @@
 from bzrlib import (
     errors,
     osutils,
-    patiencediff,
     )
 from bzrlib.errors import (
     FileExists,
@@ -3417,10 +3417,6 @@
             raise exc_class, exc_value, exc_traceback
 
 
-# Deprecated, use PatienceSequenceMatcher instead
-KnitSequenceMatcher = patiencediff.PatienceSequenceMatcher
-
-
 def annotate_knit(knit, revision_id):
     """Annotate a knit with no cached annotations.
 

=== modified file 'bzrlib/lazy_import.py'
--- bzrlib/lazy_import.py	2009-03-23 14:59:43 +0000
+++ bzrlib/lazy_import.py	2010-09-25 20:08:01 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -87,6 +87,9 @@
                           " to another variable?",
                 extra=e)
         obj = factory(self, scope, name)
+        if obj is self:
+            raise errors.IllegalUseOfScopeReplacer(name, msg="Object tried"
+                " to replace itself, check it's not using its own scope.")
         if ScopeReplacer._should_proxy:
             object.__setattr__(self, '_real_obj', obj)
         scope[name] = obj

=== modified file 'bzrlib/lazy_regex.py'
--- bzrlib/lazy_regex.py	2009-03-23 14:59:43 +0000
+++ bzrlib/lazy_regex.py	2010-06-30 12:03:30 +0000
@@ -22,6 +22,8 @@
 
 import re
 
+from bzrlib import errors
+
 
 class LazyRegex(object):
     """A proxy around a real regex, which won't be compiled until accessed."""
@@ -58,7 +60,12 @@
 
     def _real_re_compile(self, *args, **kwargs):
         """Thunk over to the original re.compile"""
-        return _real_re_compile(*args, **kwargs)
+        try:
+            return _real_re_compile(*args, **kwargs)
+        except re.error, e:
+            # raise InvalidPattern instead of re.error as this gives a
+            # cleaner message to the user.
+            raise errors.InvalidPattern('"' + args[0] + '" ' +str(e))
 
     def __getattr__(self, attr):
         """Return a member from the proxied regex object.

=== added file 'bzrlib/library_state.py'
--- bzrlib/library_state.py	1970-01-01 00:00:00 +0000
+++ bzrlib/library_state.py	2010-06-26 01:07:16 +0000
@@ -0,0 +1,103 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""The core state needed to make use of bzr is managed here."""
+
+__all__ = [
+    'BzrLibraryState',
+    ]
+
+import sys
+
+import bzrlib
+
+
+class BzrLibraryState(object):
+    """The state about how bzrlib has been configured.
+
+    This is the core state needed to make use of bzr. The current instance is
+    currently always exposed as bzrlib.global_state, but we desired to move
+    to a point where no global state is needed at all.
+    
+    :ivar saved_state: The bzrlib.global_state at the time __enter__ was
+        called.
+    :ivar cleanups: An ObjectWithCleanups which can be used for cleanups that
+        should occur when the use of bzrlib is completed. This is initialised
+        in __enter__ and executed in __exit__.
+    """
+
+    def __init__(self, ui, trace):
+        """Create library start for normal use of bzrlib.
+
+        Most applications that embed bzrlib, including bzr itself, should just
+        call bzrlib.initialize(), but it is possible to use the state class
+        directly. The initialize() function provides sensible defaults for a
+        CLI program, such as a text UI factory.
+
+        More options may be added in future so callers should use named
+        arguments.
+
+        BzrLibraryState implements the Python 2.5 Context Manager protocol
+        PEP343, and can be used with the with statement. Upon __enter__ the
+        global variables in use by bzr are set, and they are cleared on
+        __exit__.
+
+        :param ui: A bzrlib.ui.ui_factory to use.
+        :param trace: A bzrlib.trace.Config context manager to use, perhaps
+            bzrlib.trace.DefaultConfig.
+        """
+        self._ui = ui
+        self._trace = trace
+
+    def __enter__(self):
+        # NB: This function tweaks so much global state it's hard to test it in
+        # isolation within the same interpreter.  It's not reached on normal
+        # in-process run_bzr calls.  If it's broken, we expect that
+        # TestRunBzrSubprocess may fail.
+        import bzrlib
+        if bzrlib.version_info[3] == 'final':
+            from bzrlib.symbol_versioning import suppress_deprecation_warnings
+            warning_cleanup = suppress_deprecation_warnings(override=True)
+        else:
+            warning_cleanup = None
+
+        import bzrlib.cleanup
+        self.cleanups = bzrlib.cleanup.ObjectWithCleanups()
+        if warning_cleanup:
+            self.cleanups.add_cleanup(warning_cleanup)
+        self._trace.__enter__()
+
+        self._orig_ui = bzrlib.ui.ui_factory
+        bzrlib.ui.ui_factory = self._ui
+        self._ui.__enter__()
+
+        self.saved_state = bzrlib.global_state
+        bzrlib.global_state = self
+        return self # This is bound to the 'as' clause in a with statement.
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.cleanups.cleanup_now()
+        import bzrlib.ui
+        bzrlib.trace._flush_stdout_stderr()
+        bzrlib.trace._flush_trace()
+        import bzrlib.osutils
+        bzrlib.osutils.report_extension_load_failures()
+        self._ui.__exit__(None, None, None)
+        self._trace.__exit__(None, None, None)
+        bzrlib.ui.ui_factory = self._orig_ui
+        global global_state
+        global_state = self.saved_state
+        return False # propogate exceptions.

=== modified file 'bzrlib/lock.py'
--- bzrlib/lock.py	2010-04-16 07:56:51 +0000
+++ bzrlib/lock.py	2010-05-11 08:36:16 +0000
@@ -88,6 +88,20 @@
                              self.lock_url, self.details)
 
 
+class LogicalLockResult(object):
+    """The result of a lock_read/lock_write/lock_tree_write call on lockables.
+
+    :ivar unlock: A callable which will unlock the lock.
+    """
+
+    def __init__(self, unlock):
+        self.unlock = unlock
+
+    def __repr__(self):
+        return "LogicalLockResult(%s)" % (self.unlock)
+
+
+
 def cant_unlock_not_held(locked_object):
     """An attempt to unlock failed because the object was not locked.
 

=== modified file 'bzrlib/lockdir.py'
--- bzrlib/lockdir.py	2010-03-03 22:59:21 +0000
+++ bzrlib/lockdir.py	2010-09-29 20:18:37 +0000
@@ -120,6 +120,7 @@
         LockBreakMismatch,
         LockBroken,
         LockContention,
+        LockCorrupt,
         LockFailed,
         LockNotHeld,
         NoSuchFile,
@@ -151,7 +152,7 @@
 # files/dirs created.
 
 
-_DEFAULT_TIMEOUT_SECONDS = 300
+_DEFAULT_TIMEOUT_SECONDS = 30
 _DEFAULT_POLL_SECONDS = 1.0
 
 
@@ -243,7 +244,7 @@
         # have a similar bug allowing someone to think they got the lock
         # when it's already held.
         #
-        # See <https://bugs.edge.launchpad.net/bzr/+bug/498378> for one case.
+        # See <https://bugs.launchpad.net/bzr/+bug/498378> for one case.
         #
         # Strictly the check is unnecessary and a waste of time for most
         # people, but probably worth trapping if something is wrong.
@@ -346,13 +347,25 @@
         This is a UI centric function: it uses the bzrlib.ui.ui_factory to
         prompt for input if a lock is detected and there is any doubt about
         it possibly being still active.
+
+        :returns: LockResult for the broken lock.
         """
         self._check_not_locked()
-        holder_info = self.peek()
+        try:
+            holder_info = self.peek()
+        except LockCorrupt, e:
+            # The lock info is corrupt.
+            if bzrlib.ui.ui_factory.get_boolean("Break (corrupt %r)" % (self,)):
+                self.force_break_corrupt(e.file_data)
+            return
         if holder_info is not None:
             lock_info = '\n'.join(self._format_lock_info(holder_info))
-            if bzrlib.ui.ui_factory.get_boolean("Break %s" % lock_info):
-                self.force_break(holder_info)
+            if bzrlib.ui.ui_factory.confirm_action(
+                "Break %(lock_info)s", 'bzrlib.lockdir.break', 
+                dict(lock_info=lock_info)):
+                result = self.force_break(holder_info)
+                bzrlib.ui.ui_factory.show_message(
+                    "Broke lock %s" % result.lock_url)
 
     def force_break(self, dead_holder_info):
         """Release a lock held by another process.
@@ -369,6 +382,8 @@
         After the lock is broken it will not be held by any process.
         It is possible that another process may sneak in and take the
         lock before the breaking process acquires it.
+
+        :returns: LockResult for the broken lock.
         """
         if not isinstance(dead_holder_info, dict):
             raise ValueError("dead_holder_info: %r" % dead_holder_info)
@@ -394,6 +409,36 @@
                                  current_info.get('nonce'))
         for hook in self.hooks['lock_broken']:
             hook(result)
+        return result
+
+    def force_break_corrupt(self, corrupt_info_lines):
+        """Release a lock that has been corrupted.
+        
+        This is very similar to force_break, it except it doesn't assume that
+        self.peek() can work.
+        
+        :param corrupt_info_lines: the lines of the corrupted info file, used
+            to check that the lock hasn't changed between reading the (corrupt)
+            info file and calling force_break_corrupt.
+        """
+        # XXX: this copes with unparseable info files, but what about missing
+        # info files?  Or missing lock dirs?
+        self._check_not_locked()
+        tmpname = '%s/broken.%s.tmp' % (self.path, rand_chars(20))
+        self.transport.rename(self._held_dir, tmpname)
+        # check that we actually broke the right lock, not someone else;
+        # there's a small race window between checking it and doing the
+        # rename.
+        broken_info_path = tmpname + self.__INFO_NAME
+        f = self.transport.get(broken_info_path)
+        broken_lines = f.readlines()
+        if broken_lines != corrupt_info_lines:
+            raise LockBreakMismatch(self, broken_lines, corrupt_info_lines)
+        self.transport.delete(broken_info_path)
+        self.transport.rmdir(tmpname)
+        result = lock.LockResult(self.transport.abspath(self.path))
+        for hook in self.hooks['lock_broken']:
+            hook(result)
 
     def _check_not_locked(self):
         """If the lock is held by this instance, raise an error."""
@@ -447,9 +492,9 @@
         # XXX: is creating this here inefficient?
         config = bzrlib.config.GlobalConfig()
         try:
-            user = config.user_email()
-        except errors.NoEmailInUsername:
             user = config.username()
+        except errors.NoWhoami:
+            user = osutils.getuser_unicode()
         s = rio.Stanza(hostname=get_host_name(),
                    pid=str(os.getpid()),
                    start_time=str(int(time.time())),
@@ -459,7 +504,13 @@
         return s.to_string()
 
     def _parse_info(self, info_bytes):
-        stanza = rio.read_stanza(osutils.split_lines(info_bytes))
+        lines = osutils.split_lines(info_bytes)
+        try:
+            stanza = rio.read_stanza(lines)
+        except ValueError, e:
+            mutter('Corrupt lock info file: %r', lines)
+            raise LockCorrupt("could not parse lock info file: " + str(e),
+                              lines)
         if stanza is None:
             # see bug 185013; we fairly often end up with the info file being
             # empty after an interruption; we could log a message here but
@@ -539,24 +590,27 @@
                 if deadline_str is None:
                     deadline_str = time.strftime('%H:%M:%S',
                                                  time.localtime(deadline))
+                # As local lock urls are correct we display them.
+                # We avoid displaying remote lock urls.
                 lock_url = self.transport.abspath(self.path)
-                # See <https://bugs.edge.launchpad.net/bzr/+bug/250451>
-                # the URL here is sometimes not one that is useful to the
-                # user, perhaps being wrapped in a lp-%d or chroot decorator,
-                # especially if this error is issued from the server.
-                self._report_function('%s %s\n'
-                    '%s\n' # held by
-                    '%s\n' # locked ... ago
-                    'Will continue to try until %s, unless '
-                    'you press Ctrl-C.\n'
-                    'See "bzr help break-lock" for more.',
-                    start,
-                    formatted_info[0],
-                    formatted_info[1],
-                    formatted_info[2],
-                    deadline_str,
-                    )
-
+                if lock_url.startswith('file://'):
+                    lock_url = lock_url.split('.bzr/')[0]
+                else:
+                    lock_url = ''
+                user, hostname, pid, time_ago = formatted_info
+                msg = ('%s lock %s '        # lock_url
+                    'held by '              # start
+                    '%s\n'                  # user
+                    'at %s '                # hostname
+                    '[process #%s], '       # pid
+                    'acquired %s.')         # time ago
+                msg_args = [start, lock_url, user, hostname, pid, time_ago]
+                if timeout > 0:
+                    msg += ('\nWill continue to try until %s, unless '
+                        'you press Ctrl-C.')
+                    msg_args.append(deadline_str)
+                msg += '\nSee "bzr help break-lock" for more.'
+                self._report_function(msg, *msg_args)
             if (max_attempts is not None) and (attempt_count >= max_attempts):
                 self._trace("exceeded %d attempts")
                 raise LockContention(self)
@@ -564,8 +618,11 @@
                 self._trace("waiting %ss", poll)
                 time.sleep(poll)
             else:
+                # As timeout is always 0 for remote locks
+                # this block is applicable only for local
+                # lock contention
                 self._trace("timeout after waiting %ss", timeout)
-                raise LockContention(self)
+                raise LockContention('(local)', lock_url)
 
     def leave_in_place(self):
         self._locked_via_token = True
@@ -616,17 +673,19 @@
 
     def _format_lock_info(self, info):
         """Turn the contents of peek() into something for the user"""
-        lock_url = self.transport.abspath(self.path)
         start_time = info.get('start_time')
         if start_time is None:
             time_ago = '(unknown)'
         else:
             time_ago = format_delta(time.time() - int(info['start_time']))
+        user = info.get('user', '<unknown>')
+        hostname = info.get('hostname', '<unknown>')
+        pid = info.get('pid', '<unknown>')
         return [
-            'lock %s' % (lock_url,),
-            'held by %s on host %s [process #%s]' %
-                tuple([info.get(x, '<unknown>') for x in ['user', 'hostname', 'pid']]),
-            'locked %s' % (time_ago,),
+            user,
+            hostname,
+            pid,
+            time_ago,
             ]
 
     def validate_token(self, token):

=== modified file 'bzrlib/log.py'
--- bzrlib/log.py	2010-04-14 12:30:17 +0000
+++ bzrlib/log.py	2010-07-16 13:29:07 +0000
@@ -70,6 +70,7 @@
     diff,
     errors,
     foreign,
+    osutils,
     repository as _mod_repository,
     revision as _mod_revision,
     revisionspec,
@@ -85,7 +86,6 @@
     format_date,
     format_date_with_offset_in_original_timezone,
     get_terminal_encoding,
-    re_compile_checked,
     terminal_width,
     )
 from bzrlib.symbol_versioning import (
@@ -432,8 +432,9 @@
         else:
             specific_files = None
         s = StringIO()
+        path_encoding = osutils.get_diff_header_encoding()
         diff.show_diff_trees(tree_1, tree_2, s, specific_files, old_label='',
-            new_label='')
+            new_label='', path_encoding=path_encoding)
         return s.getvalue()
 
     def _create_log_revision_iterator(self):
@@ -522,7 +523,7 @@
     elif not generate_merge_revisions:
         # If we only want to see linear revisions, we can iterate ...
         iter_revs = _generate_flat_revisions(branch, start_rev_id, end_rev_id,
-                                             direction)
+                                             direction, exclude_common_ancestry)
         if direction == 'forward':
             iter_revs = reversed(iter_revs)
     else:
@@ -544,8 +545,11 @@
         return [(rev_id, revno_str, 0)]
 
 
-def _generate_flat_revisions(branch, start_rev_id, end_rev_id, direction):
-    result = _linear_view_revisions(branch, start_rev_id, end_rev_id)
+def _generate_flat_revisions(branch, start_rev_id, end_rev_id, direction,
+                             exclude_common_ancestry=False):
+    result = _linear_view_revisions(
+        branch, start_rev_id, end_rev_id,
+        exclude_common_ancestry=exclude_common_ancestry)
     # If a start limit was given and it's not obviously an
     # ancestor of the end limit, check it before outputting anything
     if direction == 'forward' or (start_rev_id
@@ -572,7 +576,7 @@
     if delayed_graph_generation:
         try:
             for rev_id, revno, depth in  _linear_view_revisions(
-                branch, start_rev_id, end_rev_id):
+                branch, start_rev_id, end_rev_id, exclude_common_ancestry):
                 if _has_merges(branch, rev_id):
                     # The end_rev_id can be nested down somewhere. We need an
                     # explicit ancestry check. There is an ambiguity here as we
@@ -643,14 +647,17 @@
     return True
 
 
-def _linear_view_revisions(branch, start_rev_id, end_rev_id):
+def _linear_view_revisions(branch, start_rev_id, end_rev_id,
+                           exclude_common_ancestry=False):
     """Calculate a sequence of revisions to view, newest to oldest.
 
     :param start_rev_id: the lower revision-id
     :param end_rev_id: the upper revision-id
+    :param exclude_common_ancestry: Whether the start_rev_id should be part of
+        the iterated revisions.
     :return: An iterator of (revision_id, dotted_revno, merge_depth) tuples.
     :raises _StartNotLinearAncestor: if a start_rev_id is specified but
-      is not found walking the left-hand history
+        is not found walking the left-hand history
     """
     br_revno, br_rev_id = branch.last_revision_info()
     repo = branch.repository
@@ -667,7 +674,8 @@
             revno = branch.revision_id_to_dotted_revno(revision_id)
             revno_str = '.'.join(str(n) for n in revno)
             if not found_start and revision_id == start_rev_id:
-                yield revision_id, revno_str, 0
+                if not exclude_common_ancestry:
+                    yield revision_id, revno_str, 0
                 found_start = True
                 break
             else:
@@ -802,8 +810,7 @@
     """
     if search is None:
         return log_rev_iterator
-    searchRE = re_compile_checked(search, re.IGNORECASE,
-            'log message filter')
+    searchRE = re.compile(search, re.IGNORECASE)
     return _filter_message_re(searchRE, log_rev_iterator)
 
 
@@ -1341,7 +1348,7 @@
 
     def __init__(self, to_file, show_ids=False, show_timezone='original',
                  delta_format=None, levels=None, show_advice=False,
-                 to_exact_file=None):
+                 to_exact_file=None, author_list_handler=None):
         """Create a LogFormatter.
 
         :param to_file: the file to output to
@@ -1355,6 +1362,8 @@
           let the log formatter decide.
         :param show_advice: whether to show advice at the end of the
           log or not
+        :param author_list_handler: callable generating a list of
+          authors to display for a given revision
         """
         self.to_file = to_file
         # 'exact' stream used to show diff, it should print content 'as is'
@@ -1375,6 +1384,7 @@
         self.levels = levels
         self._show_advice = show_advice
         self._merge_count = 0
+        self._author_list_handler = author_list_handler
 
     def get_levels(self):
         """Get the number of levels to display or 0 for all."""
@@ -1412,10 +1422,41 @@
         return address
 
     def short_author(self, rev):
-        name, address = config.parse_username(rev.get_apparent_authors()[0])
-        if name:
-            return name
-        return address
+        return self.authors(rev, 'first', short=True, sep=', ')
+
+    def authors(self, rev, who, short=False, sep=None):
+        """Generate list of authors, taking --authors option into account.
+
+        The caller has to specify the name of a author list handler,
+        as provided by the author list registry, using the ``who``
+        argument.  That name only sets a default, though: when the
+        user selected a different author list generation using the
+        ``--authors`` command line switch, as represented by the
+        ``author_list_handler`` constructor argument, that value takes
+        precedence.
+
+        :param rev: The revision for which to generate the list of authors.
+        :param who: Name of the default handler.
+        :param short: Whether to shorten names to either name or address.
+        :param sep: What separator to use for automatic concatenation.
+        """
+        if self._author_list_handler is not None:
+            # The user did specify --authors, which overrides the default
+            author_list_handler = self._author_list_handler
+        else:
+            # The user didn't specify --authors, so we use the caller's default
+            author_list_handler = author_list_registry.get(who)
+        names = author_list_handler(rev)
+        if short:
+            for i in range(len(names)):
+                name, address = config.parse_username(names[i])
+                if name:
+                    names[i] = name
+                else:
+                    names[i] = address
+        if sep is not None:
+            names = sep.join(names)
+        return names
 
     def merge_marker(self, revision):
         """Get the merge marker to include in the output or '' if none."""
@@ -1523,7 +1564,7 @@
         lines.extend(self.custom_properties(revision.rev))
 
         committer = revision.rev.committer
-        authors = revision.rev.get_apparent_authors()
+        authors = self.authors(revision.rev, 'all')
         if authors != [committer]:
             lines.append('author: %s' % (", ".join(authors),))
         lines.append('committer: %s' % (committer,))
@@ -1703,7 +1744,8 @@
                                self.show_timezone,
                                date_fmt='%Y-%m-%d',
                                show_offset=False)
-        committer_str = revision.rev.get_apparent_authors()[0].replace (' <', '  <')
+        committer_str = self.authors(revision.rev, 'first', sep=', ')
+        committer_str = committer_str.replace(' <', '  <')
         to_file.write('%s  %s\n\n' % (date_str,committer_str))
 
         if revision.delta is not None and revision.delta.has_changed():
@@ -1774,6 +1816,34 @@
         raise errors.BzrCommandError("unknown log formatter: %r" % name)
 
 
+def author_list_all(rev):
+    return rev.get_apparent_authors()[:]
+
+
+def author_list_first(rev):
+    lst = rev.get_apparent_authors()
+    try:
+        return [lst[0]]
+    except IndexError:
+        return []
+
+
+def author_list_committer(rev):
+    return [rev.committer]
+
+
+author_list_registry = registry.Registry()
+
+author_list_registry.register('all', author_list_all,
+                              'All authors')
+
+author_list_registry.register('first', author_list_first,
+                              'The first author')
+
+author_list_registry.register('committer', author_list_committer,
+                              'The committer')
+
+
 def show_one_log(revno, rev, delta, verbose, to_file, show_timezone):
     # deprecated; for compatibility
     lf = LongLogFormatter(to_file=to_file, show_timezone=show_timezone)
@@ -1930,7 +2000,7 @@
         lf.log_revision(lr)
 
 
-def _get_info_for_log_files(revisionspec_list, file_list):
+def _get_info_for_log_files(revisionspec_list, file_list, add_cleanup):
     """Find file-ids and kinds given a list of files and a revision range.
 
     We search for files at the end of the range. If not found there,
@@ -1940,14 +2010,16 @@
     :param file_list: the list of paths given on the command line;
       the first of these can be a branch location or a file path,
       the remainder must be file paths
+    :param add_cleanup: When the branch returned is read locked,
+      an unlock call will be queued to the cleanup.
     :return: (branch, info_list, start_rev_info, end_rev_info) where
       info_list is a list of (relative_path, file_id, kind) tuples where
       kind is one of values 'directory', 'file', 'symlink', 'tree-reference'.
       branch will be read-locked.
     """
-    from builtins import _get_revision_range, safe_relpath_files
+    from builtins import _get_revision_range
     tree, b, path = bzrdir.BzrDir.open_containing_tree_or_branch(file_list[0])
-    b.lock_read()
+    add_cleanup(b.lock_read().unlock)
     # XXX: It's damn messy converting a list of paths to relative paths when
     # those paths might be deleted ones, they might be on a case-insensitive
     # filesystem and/or they might be in silly locations (like another branch).
@@ -1957,7 +2029,7 @@
     # case of running log in a nested directory, assuming paths beyond the
     # first one haven't been deleted ...
     if tree:
-        relpaths = [path] + safe_relpath_files(tree, file_list[1:])
+        relpaths = [path] + tree.safe_relpath_files(file_list[1:])
     else:
         relpaths = [path] + file_list[1:]
     info_list = []

=== modified file 'bzrlib/lru_cache.py'
--- bzrlib/lru_cache.py	2009-07-08 15:43:51 +0000
+++ bzrlib/lru_cache.py	2010-07-15 13:05:40 +0000
@@ -17,7 +17,6 @@
 """A simple least-recently-used (LRU) cache."""
 
 from bzrlib import (
-    symbol_versioning,
     trace,
     )
 
@@ -61,14 +60,7 @@
 class LRUCache(object):
     """A class which manages a cache of entries, removing unused ones."""
 
-    def __init__(self, max_cache=100, after_cleanup_count=None,
-                 after_cleanup_size=symbol_versioning.DEPRECATED_PARAMETER):
-        if symbol_versioning.deprecated_passed(after_cleanup_size):
-            symbol_versioning.warn('LRUCache.__init__(after_cleanup_size) was'
-                                   ' deprecated in 1.11. Use'
-                                   ' after_cleanup_count instead.',
-                                   DeprecationWarning)
-            after_cleanup_count = after_cleanup_size
+    def __init__(self, max_cache=100, after_cleanup_count=None):
         self._cache = {}
         # The "HEAD" of the lru linked list
         self._most_recently_used = None

=== modified file 'bzrlib/lsprof.py'
--- bzrlib/lsprof.py	2009-08-24 21:05:09 +0000
+++ bzrlib/lsprof.py	2010-09-19 09:07:42 +0000
@@ -10,6 +10,7 @@
 import threading
 from _lsprof import Profiler, profiler_entry
 
+from bzrlib import errors
 
 __all__ = ['profile', 'Stats']
 
@@ -20,6 +21,9 @@
     raised, pass in a closure that will catch the exceptions and transform them
     appropriately for your driver function.
 
+    Important caveat: only one profile can execute at a time. See BzrProfiler
+    for details.
+
     :return: The functions return value and a stats object.
     """
     profiler = BzrProfiler()
@@ -41,8 +45,21 @@
     To use it, create a BzrProfiler and call start() on it. Some arbitrary
     time later call stop() to stop profiling and retrieve the statistics
     from the code executed in the interim.
+
+    Note that profiling involves a threading.Lock around the actual profiling.
+    This is needed because profiling involves global manipulation of the python
+    interpreter state. As such you cannot perform multiple profiles at once.
+    Trying to do so will lock out the second profiler unless the global 
+    bzrlib.lsprof.BzrProfiler.profiler_block is set to 0. Setting it to 0 will
+    cause profiling to fail rather than blocking.
     """
 
+    profiler_block = 1
+    """Serialise rather than failing to profile concurrent profile requests."""
+
+    profiler_lock = threading.Lock()
+    """Global lock used to serialise profiles."""
+
     def start(self):
         """Start profiling.
         
@@ -51,8 +68,16 @@
         """
         self._g_threadmap = {}
         self.p = Profiler()
-        self.p.enable(subcalls=True)
-        threading.setprofile(self._thread_profile)
+        permitted = self.__class__.profiler_lock.acquire(
+            self.__class__.profiler_block)
+        if not permitted:
+            raise errors.InternalBzrError(msg="Already profiling something")
+        try:
+            self.p.enable(subcalls=True)
+            threading.setprofile(self._thread_profile)
+        except:
+            self.__class__.profiler_lock.release()
+            raise
 
     def stop(self):
         """Stop profiling.
@@ -62,17 +87,20 @@
 
         :return: A bzrlib.lsprof.Stats object.
         """
-        self.p.disable()
-        for pp in self._g_threadmap.values():
-            pp.disable()
-        threading.setprofile(None)
-        p = self.p
-        self.p = None
-        threads = {}
-        for tid, pp in self._g_threadmap.items():
-            threads[tid] = Stats(pp.getstats(), {})
-        self._g_threadmap = None
-        return Stats(p.getstats(), threads)
+        try:
+            self.p.disable()
+            for pp in self._g_threadmap.values():
+                pp.disable()
+            threading.setprofile(None)
+            p = self.p
+            self.p = None
+            threads = {}
+            for tid, pp in self._g_threadmap.items():
+                threads[tid] = Stats(pp.getstats(), {})
+            self._g_threadmap = None
+            return Stats(p.getstats(), threads)
+        finally:
+            self.__class__.profiler_lock.release()
 
     def _thread_profile(self, f, *args, **kwds):
         # we lose the first profile point for a new thread in order to
@@ -84,14 +112,21 @@
 
 
 class Stats(object):
-    """XXX docstring"""
+    """Wrapper around the collected data.
+
+    A Stats instance is created when the profiler finishes. Normal
+    usage is to use save() to write out the data to a file, or pprint()
+    to write human-readable information to the command line.
+    """
 
     def __init__(self, data, threads):
         self.data = data
         self.threads = threads
 
     def sort(self, crit="inlinetime"):
-        """XXX docstring"""
+        """Sort the data by the supplied critera.
+
+        :param crit: the data attribute used as the sort key."""
         if crit not in profiler_entry.__dict__:
             raise ValueError, "Can't sort by %s" % crit
         self.data.sort(lambda b, a: cmp(getattr(a, crit),
@@ -102,7 +137,12 @@
                                               getattr(b, crit)))
 
     def pprint(self, top=None, file=None):
-        """XXX docstring"""
+        """Pretty-print the data as plain text for human consumption.
+
+        :param top: only output the top n entries.
+            The default value of None means output all data.
+        :param file: the output file; if None, output will
+            default to stdout."""
         if file is None:
             file = sys.stdout
         d = self.data

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2010-05-03 04:08:50 +0000
+++ bzrlib/merge.py	2010-08-25 13:02:32 +0000
@@ -16,14 +16,14 @@
 
 import warnings
 
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
 from bzrlib import (
     branch as _mod_branch,
     conflicts as _mod_conflicts,
     debug,
-    decorators,
-    errors,
+    generate_ids,
     graph as _mod_graph,
-    hooks,
     merge3,
     osutils,
     patiencediff,
@@ -34,9 +34,16 @@
     tree as _mod_tree,
     tsort,
     ui,
-    versionedfile
+    versionedfile,
+    workingtree,
     )
 from bzrlib.cleanup import OperationWithCleanups
+""")
+from bzrlib import (
+    decorators,
+    errors,
+    hooks,
+    )
 from bzrlib.symbol_versioning import (
     deprecated_in,
     deprecated_method,
@@ -421,7 +428,6 @@
         return self._cached_trees[revision_id]
 
     def _get_tree(self, treespec, possible_transports=None):
-        from bzrlib import workingtree
         location, revno = treespec
         if revno is None:
             tree = workingtree.WorkingTree.open_containing(location)[0]
@@ -856,6 +862,13 @@
         finally:
             child_pb.finished()
         self.fix_root()
+        self._finish_computing_transform()
+
+    def _finish_computing_transform(self):
+        """Finalize the transform and report the changes.
+
+        This is the second half of _compute_transform.
+        """
         child_pb = ui.ui_factory.nested_progress_bar()
         try:
             fs_conflicts = transform.resolve_conflicts(self.tt, child_pb,
@@ -1071,11 +1084,8 @@
                           ))
         return result
 
-
     def fix_root(self):
-        try:
-            self.tt.final_kind(self.tt.root)
-        except errors.NoSuchFile:
+        if self.tt.final_kind(self.tt.root) is None:
             self.tt.cancel_deletion(self.tt.root)
         if self.tt.final_file_id(self.tt.root) is None:
             self.tt.version_file(self.tt.tree_file_id(self.tt.root),
@@ -1090,10 +1100,9 @@
             # the other tree's root is a non-root in the current tree (as when
             # a previously unrelated branch is merged into another)
             return
-        try:
-            self.tt.final_kind(other_root)
+        if self.tt.final_kind(other_root) is not None:
             other_root_is_present = True
-        except errors.NoSuchFile:
+        else:
             # other_root doesn't have a physical representation. We still need
             # to move any references to the actual root of the tree.
             other_root_is_present = False
@@ -1103,15 +1112,10 @@
         for thing, child in self.other_tree.inventory.root.children.iteritems():
             trans_id = self.tt.trans_id_file_id(child.file_id)
             if not other_root_is_present:
-                # FIXME: Make final_kind returns None instead of raising
-                # NoSuchFile to avoid the ugly construct below -- vila 20100402
-                try:
-                    self.tt.final_kind(trans_id)
+                if self.tt.final_kind(trans_id) is not None:
                     # The item exist in the final tree and has a defined place
                     # to go already.
                     continue
-                except errors.NoSuchFile, e:
-                    pass
             # Move the item into the root
             self.tt.adjust_path(self.tt.final_name(trans_id),
                                 self.tt.root, trans_id)
@@ -1393,10 +1397,7 @@
             self.tt.version_file(file_id, trans_id)
         # The merge has been performed, so the old contents should not be
         # retained.
-        try:
-            self.tt.delete_contents(trans_id)
-        except errors.NoSuchFile:
-            pass
+        self.tt.delete_contents(trans_id)
         return result
 
     def _default_other_winner_merge(self, merge_hook_params):
@@ -1455,7 +1456,7 @@
     def get_lines(self, tree, file_id):
         """Return the lines in a file, or an empty list."""
         if tree.has_id(file_id):
-            return tree.get_file(file_id).readlines()
+            return tree.get_file_lines(file_id)
         else:
             return []
 
@@ -1574,10 +1575,7 @@
         if winner == 'this' and file_status != "modified":
             return
         trans_id = self.tt.trans_id_file_id(file_id)
-        try:
-            if self.tt.final_kind(trans_id) != "file":
-                return
-        except errors.NoSuchFile:
+        if self.tt.final_kind(trans_id) != "file":
             return
         if winner == "this":
             executability = this_executable
@@ -1750,6 +1748,168 @@
             osutils.rmtree(temp_dir)
 
 
+class PathNotInTree(errors.BzrError):
+
+    _fmt = """Merge-into failed because %(tree)s does not contain %(path)s."""
+
+    def __init__(self, path, tree):
+        errors.BzrError.__init__(self, path=path, tree=tree)
+
+
+class MergeIntoMerger(Merger):
+    """Merger that understands other_tree will be merged into a subdir.
+
+    This also changes the Merger api so that it uses real Branch, revision_id,
+    and RevisonTree objects, rather than using revision specs.
+    """
+
+    def __init__(self, this_tree, other_branch, other_tree, target_subdir,
+            source_subpath, other_rev_id=None):
+        """Create a new MergeIntoMerger object.
+
+        source_subpath in other_tree will be effectively copied to
+        target_subdir in this_tree.
+
+        :param this_tree: The tree that we will be merging into.
+        :param other_branch: The Branch we will be merging from.
+        :param other_tree: The RevisionTree object we want to merge.
+        :param target_subdir: The relative path where we want to merge
+            other_tree into this_tree
+        :param source_subpath: The relative path specifying the subtree of
+            other_tree to merge into this_tree.
+        """
+        # It is assumed that we are merging a tree that is not in our current
+        # ancestry, which means we are using the "EmptyTree" as our basis.
+        null_ancestor_tree = this_tree.branch.repository.revision_tree(
+                                _mod_revision.NULL_REVISION)
+        super(MergeIntoMerger, self).__init__(
+            this_branch=this_tree.branch,
+            this_tree=this_tree,
+            other_tree=other_tree,
+            base_tree=null_ancestor_tree,
+            )
+        self._target_subdir = target_subdir
+        self._source_subpath = source_subpath
+        self.other_branch = other_branch
+        if other_rev_id is None:
+            other_rev_id = other_tree.get_revision_id()
+        self.other_rev_id = self.other_basis = other_rev_id
+        self.base_is_ancestor = True
+        self.backup_files = True
+        self.merge_type = Merge3Merger
+        self.show_base = False
+        self.reprocess = False
+        self.interesting_ids = None
+        self.merge_type = _MergeTypeParameterizer(MergeIntoMergeType,
+              target_subdir=self._target_subdir,
+              source_subpath=self._source_subpath)
+        if self._source_subpath != '':
+            # If this isn't a partial merge make sure the revisions will be
+            # present.
+            self._maybe_fetch(self.other_branch, self.this_branch,
+                self.other_basis)
+
+    def set_pending(self):
+        if self._source_subpath != '':
+            return
+        Merger.set_pending(self)
+
+
+class _MergeTypeParameterizer(object):
+    """Wrap a merge-type class to provide extra parameters.
+    
+    This is hack used by MergeIntoMerger to pass some extra parameters to its
+    merge_type.  Merger.do_merge() sets up its own set of parameters to pass to
+    the 'merge_type' member.  It is difficult override do_merge without
+    re-writing the whole thing, so instead we create a wrapper which will pass
+    the extra parameters.
+    """
+
+    def __init__(self, merge_type, **kwargs):
+        self._extra_kwargs = kwargs
+        self._merge_type = merge_type
+
+    def __call__(self, *args, **kwargs):
+        kwargs.update(self._extra_kwargs)
+        return self._merge_type(*args, **kwargs)
+
+    def __getattr__(self, name):
+        return getattr(self._merge_type, name)
+
+
+class MergeIntoMergeType(Merge3Merger):
+    """Merger that incorporates a tree (or part of a tree) into another."""
+
+    def __init__(self, *args, **kwargs):
+        """Initialize the merger object.
+
+        :param args: See Merge3Merger.__init__'s args.
+        :param kwargs: See Merge3Merger.__init__'s keyword args, except for
+            source_subpath and target_subdir.
+        :keyword source_subpath: The relative path specifying the subtree of
+            other_tree to merge into this_tree.
+        :keyword target_subdir: The relative path where we want to merge
+            other_tree into this_tree
+        """
+        # All of the interesting work happens during Merge3Merger.__init__(),
+        # so we have have to hack in to get our extra parameters set.
+        self._source_subpath = kwargs.pop('source_subpath')
+        self._target_subdir = kwargs.pop('target_subdir')
+        super(MergeIntoMergeType, self).__init__(*args, **kwargs)
+
+    def _compute_transform(self):
+        child_pb = ui.ui_factory.nested_progress_bar()
+        try:
+            entries = self._entries_to_incorporate()
+            entries = list(entries)
+            for num, (entry, parent_id) in enumerate(entries):
+                child_pb.update('Preparing file merge', num, len(entries))
+                parent_trans_id = self.tt.trans_id_file_id(parent_id)
+                trans_id = transform.new_by_entry(self.tt, entry,
+                    parent_trans_id, self.other_tree)
+        finally:
+            child_pb.finished()
+        self._finish_computing_transform()
+
+    def _entries_to_incorporate(self):
+        """Yields pairs of (inventory_entry, new_parent)."""
+        other_inv = self.other_tree.inventory
+        subdir_id = other_inv.path2id(self._source_subpath)
+        if subdir_id is None:
+            # XXX: The error would be clearer if it gave the URL of the source
+            # branch, but we don't have a reference to that here.
+            raise PathNotInTree(self._source_subpath, "Source tree")
+        subdir = other_inv[subdir_id]
+        parent_in_target = osutils.dirname(self._target_subdir)
+        target_id = self.this_tree.inventory.path2id(parent_in_target)
+        if target_id is None:
+            raise PathNotInTree(self._target_subdir, "Target tree")
+        name_in_target = osutils.basename(self._target_subdir)
+        merge_into_root = subdir.copy()
+        merge_into_root.name = name_in_target
+        if merge_into_root.file_id in self.this_tree.inventory:
+            # Give the root a new file-id.
+            # This can happen fairly easily if the directory we are
+            # incorporating is the root, and both trees have 'TREE_ROOT' as
+            # their root_id.  Users will expect this to Just Work, so we
+            # change the file-id here.
+            # Non-root file-ids could potentially conflict too.  That's really
+            # an edge case, so we don't do anything special for those.  We let
+            # them cause conflicts.
+            merge_into_root.file_id = generate_ids.gen_file_id(name_in_target)
+        yield (merge_into_root, target_id)
+        if subdir.kind != 'directory':
+            # No children, so we are done.
+            return
+        for ignored_path, entry in other_inv.iter_entries_by_dir(subdir_id):
+            parent_id = entry.parent_id
+            if parent_id == subdir.file_id:
+                # The root's parent ID has changed, so make sure children of
+                # the root refer to the new ID.
+                parent_id = merge_into_root.file_id
+            yield (entry, parent_id)
+
+
 def merge_inner(this_branch, other_tree, base_tree, ignore_zero=False,
                 backup_files=False,
                 merge_type=Merge3Merger,
@@ -1763,10 +1923,11 @@
                 change_reporter=None):
     """Primary interface for merging.
 
-        typical use is probably
-        'merge_inner(branch, branch.get_revision_tree(other_revision),
-                     branch.get_revision_tree(base_revision))'
-        """
+    Typical use is probably::
+
+        merge_inner(branch, branch.get_revision_tree(other_revision),
+                    branch.get_revision_tree(base_revision))
+    """
     if this_tree is None:
         raise errors.BzrError("bzrlib.merge.merge_inner requires a this_tree "
                               "parameter as of bzrlib version 0.8.")

=== modified file 'bzrlib/msgeditor.py'
--- bzrlib/msgeditor.py	2010-04-30 11:35:43 +0000
+++ bzrlib/msgeditor.py	2010-09-13 10:04:19 +0000
@@ -149,8 +149,10 @@
             return None
         edited_content = msg_transport.get_bytes(basename)
         if edited_content == reference_content:
-            if not ui.ui_factory.get_boolean(
-                "Commit message was not edited, use anyway"):
+            if not ui.ui_factory.confirm_action(
+                "Commit message was not edited, use anyway",
+                "bzrlib.msgeditor.unchanged",
+                {}):
                 # Returning "" makes cmd_commit raise 'empty commit message
                 # specified' which is a reasonable error, given the user has
                 # rejected using the unedited template.

=== modified file 'bzrlib/multiparent.py'
--- bzrlib/multiparent.py	2009-06-10 03:56:49 +0000
+++ bzrlib/multiparent.py	2010-08-11 01:31:11 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -76,6 +76,8 @@
 class MultiParent(object):
     """A multi-parent diff"""
 
+    __slots__ = ['hunks']
+
     def __init__(self, hunks=None):
         if hunks is not None:
             self.hunks = hunks
@@ -258,6 +260,8 @@
 class NewText(object):
     """The contents of text that is introduced by this text"""
 
+    __slots__ = ['lines']
+
     def __init__(self, lines):
         self.lines = lines
 
@@ -279,24 +283,30 @@
 class ParentText(object):
     """A reference to text present in a parent text"""
 
+    __slots__ = ['parent', 'parent_pos', 'child_pos', 'num_lines']
+
     def __init__(self, parent, parent_pos, child_pos, num_lines):
         self.parent = parent
         self.parent_pos = parent_pos
         self.child_pos = child_pos
         self.num_lines = num_lines
 
+    def _as_dict(self):
+        return dict(parent=self.parent, parent_pos=self.parent_pos,
+                    child_pos=self.child_pos, num_lines=self.num_lines)
+
     def __repr__(self):
-        return 'ParentText(%(parent)r, %(parent_pos)r, %(child_pos)r,'\
-            ' %(num_lines)r)' % self.__dict__
+        return ('ParentText(%(parent)r, %(parent_pos)r, %(child_pos)r,'
+                ' %(num_lines)r)' % self._as_dict())
 
     def __eq__(self, other):
         if self.__class__ is not other.__class__:
             return False
-        return (self.__dict__ == other.__dict__)
+        return self._as_dict() == other._as_dict()
 
     def to_patch(self):
-        yield 'c %(parent)d %(parent_pos)d %(child_pos)d %(num_lines)d\n'\
-            % self.__dict__
+        yield ('c %(parent)d %(parent_pos)d %(child_pos)d %(num_lines)d\n'
+               % self._as_dict())
 
 
 class BaseVersionedFile(object):

=== modified file 'bzrlib/mutabletree.py'
--- bzrlib/mutabletree.py	2010-04-22 14:18:17 +0000
+++ bzrlib/mutabletree.py	2010-08-17 06:45:33 +0000
@@ -32,6 +32,7 @@
     hooks,
     osutils,
     revisiontree,
+    inventory,
     symbol_versioning,
     trace,
     tree,
@@ -182,39 +183,14 @@
                **kwargs):
         # avoid circular imports
         from bzrlib import commit
-        if revprops is None:
-            revprops = {}
         possible_master_transports=[]
-        if not 'branch-nick' in revprops:
-            revprops['branch-nick'] = self.branch._get_nick(
+        revprops = commit.Commit.update_revprops(
+                revprops,
+                self.branch,
+                kwargs.pop('authors', None),
+                kwargs.pop('author', None),
                 kwargs.get('local', False),
                 possible_master_transports)
-        authors = kwargs.pop('authors', None)
-        author = kwargs.pop('author', None)
-        if authors is not None:
-            if author is not None:
-                raise AssertionError('Specifying both author and authors '
-                        'is not allowed. Specify just authors instead')
-            if 'author' in revprops or 'authors' in revprops:
-                # XXX: maybe we should just accept one of them?
-                raise AssertionError('author property given twice')
-            if authors:
-                for individual in authors:
-                    if '\n' in individual:
-                        raise AssertionError('\\n is not a valid character '
-                                'in an author identity')
-                revprops['authors'] = '\n'.join(authors)
-        if author is not None:
-            symbol_versioning.warn('The parameter author was deprecated'
-                   ' in version 1.13. Use authors instead',
-                   DeprecationWarning)
-            if 'author' in revprops or 'authors' in revprops:
-                # XXX: maybe we should just accept one of them?
-                raise AssertionError('author property given twice')
-            if '\n' in author:
-                raise AssertionError('\\n is not a valid character '
-                        'in an author identity')
-            revprops['authors'] = author
         # args for wt.commit start at message from the Commit.commit method,
         args = (message, ) + args
         for hook in MutableTree.hooks['start_commit']:
@@ -400,6 +376,10 @@
         This is designed more towards DWIM for humans than API clarity.
         For the specific behaviour see the help for cmd_add().
 
+        :param file_list: List of zero or more paths.  *NB: these are 
+            interpreted relative to the process cwd, not relative to the 
+            tree.*  (Add and most other tree methods use tree-relative
+            paths.)
         :param action: A reporter to be called with the inventory, parent_ie,
             path and kind of the path being added. It may return a file_id if
             a specific one should be used.
@@ -436,6 +416,10 @@
             for c in self.conflicts():
                 conflicts_related.update(c.associated_filenames())
 
+        # expand any symlinks in the directory part, while leaving the
+        # filename alone
+        file_list = map(osutils.normalizepath, file_list)
+
         # validate user file paths and convert all paths to tree
         # relative : it's cheaper to make a tree relative path an abspath
         # than to convert an abspath to tree relative, and it's cheaper to
@@ -740,6 +724,17 @@
         file_id or None to generate a new file id
     :returns: None
     """
+    # if the parent exists, but isn't a directory, we have to do the
+    # kind change now -- really the inventory shouldn't pretend to know
+    # the kind of wt files, but it does.
+    if parent_ie.kind != 'directory':
+        # nb: this relies on someone else checking that the path we're using
+        # doesn't contain symlinks.
+        new_parent_ie = inventory.make_entry('directory', parent_ie.name,
+            parent_ie.parent_id, parent_ie.file_id)
+        del inv[parent_ie.file_id]
+        inv.add(new_parent_ie)
+        parent_ie = new_parent_ie
     file_id = file_id_callback(inv, parent_ie, path, kind)
     entry = inv.make_entry(kind, path.base_path, parent_ie.file_id,
         file_id=file_id)

=== modified file 'bzrlib/option.py'
--- bzrlib/option.py	2010-04-20 08:47:38 +0000
+++ bzrlib/option.py	2010-05-23 18:57:38 +0000
@@ -530,6 +530,9 @@
                short_name='m',
                help='Message string.')
 _global_option('no-recurse')
+_global_option('null', short_name='0',
+                 help='Use an ASCII NUL (\\0) separator rather than '
+                      'a newline.')
 _global_option('profile',
                help='Show performance profiling information.')
 _global_option('revision',
@@ -570,6 +573,8 @@
 _global_option('dry-run',
                help="Show what would be done, but don't actually do anything.")
 _global_option('name-from-revision', help='The path name in the old tree.')
+_global_option('directory', short_name='d', type=unicode,
+               help='Branch to operate on, instead of working directory')
 
 diff_writer_registry = _mod_registry.Registry()
 diff_writer_registry.register('plain', lambda x: x, 'Plaintext diff output.')

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2010-05-03 06:25:46 +0000
+++ bzrlib/osutils.py	2010-08-24 12:36:53 +0000
@@ -14,10 +14,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+import errno
 import os
 import re
 import stat
-from stat import S_ISREG, S_ISDIR, S_ISLNK, ST_MODE, ST_SIZE
 import sys
 import time
 import codecs
@@ -25,24 +25,19 @@
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 from datetime import datetime
-import errno
-from ntpath import (abspath as _nt_abspath,
-                    join as _nt_join,
-                    normpath as _nt_normpath,
-                    realpath as _nt_realpath,
-                    splitdrive as _nt_splitdrive,
-                    )
+import getpass
+import ntpath
 import posixpath
+# We need to import both shutil and rmtree as we export the later on posix
+# and need the former on windows
 import shutil
-from shutil import (
-    rmtree,
-    )
+from shutil import rmtree
 import socket
 import subprocess
+# We need to import both tempfile and mkdtemp as we export the later on posix
+# and need the former on windows
 import tempfile
-from tempfile import (
-    mkdtemp,
-    )
+from tempfile import mkdtemp
 import unicodedata
 
 from bzrlib import (
@@ -303,13 +298,13 @@
     running python.exe under cmd.exe return capital C:\\
     running win32 python inside a cygwin shell returns lowercase c:\\
     """
-    drive, path = _nt_splitdrive(path)
+    drive, path = ntpath.splitdrive(path)
     return drive.upper() + path
 
 
 def _win32_abspath(path):
-    # Real _nt_abspath doesn't have a problem with a unicode cwd
-    return _win32_fixdrive(_nt_abspath(unicode(path)).replace('\\', '/'))
+    # Real ntpath.abspath doesn't have a problem with a unicode cwd
+    return _win32_fixdrive(ntpath.abspath(unicode(path)).replace('\\', '/'))
 
 
 def _win98_abspath(path):
@@ -326,30 +321,30 @@
     #   /path       => C:/path
     path = unicode(path)
     # check for absolute path
-    drive = _nt_splitdrive(path)[0]
+    drive = ntpath.splitdrive(path)[0]
     if drive == '' and path[:2] not in('//','\\\\'):
         cwd = os.getcwdu()
         # we cannot simply os.path.join cwd and path
         # because os.path.join('C:','/path') produce '/path'
         # and this is incorrect
         if path[:1] in ('/','\\'):
-            cwd = _nt_splitdrive(cwd)[0]
+            cwd = ntpath.splitdrive(cwd)[0]
             path = path[1:]
         path = cwd + '\\' + path
-    return _win32_fixdrive(_nt_normpath(path).replace('\\', '/'))
+    return _win32_fixdrive(ntpath.normpath(path).replace('\\', '/'))
 
 
 def _win32_realpath(path):
-    # Real _nt_realpath doesn't have a problem with a unicode cwd
-    return _win32_fixdrive(_nt_realpath(unicode(path)).replace('\\', '/'))
+    # Real ntpath.realpath doesn't have a problem with a unicode cwd
+    return _win32_fixdrive(ntpath.realpath(unicode(path)).replace('\\', '/'))
 
 
 def _win32_pathjoin(*args):
-    return _nt_join(*args).replace('\\', '/')
+    return ntpath.join(*args).replace('\\', '/')
 
 
 def _win32_normpath(path):
-    return _win32_fixdrive(_nt_normpath(unicode(path)).replace('\\', '/'))
+    return _win32_fixdrive(ntpath.normpath(unicode(path)).replace('\\', '/'))
 
 
 def _win32_getcwd():
@@ -360,14 +355,6 @@
     return _win32_fixdrive(tempfile.mkdtemp(*args, **kwargs).replace('\\', '/'))
 
 
-def _add_rename_error_details(e, old, new):
-    new_e = OSError(e.errno, "failed to rename %s to %s: %s"
-        % (old, new, e.strerror))
-    new_e.filename = old
-    new_e.to_filename = new
-    return new_e
-
-
 def _win32_rename(old, new):
     """We expect to be able to atomically replace 'new' with old.
 
@@ -375,7 +362,7 @@
     and then deleted.
     """
     try:
-        fancy_rename(old, new, rename_func=_wrapped_rename, unlink_func=os.unlink)
+        fancy_rename(old, new, rename_func=os.rename, unlink_func=os.unlink)
     except OSError, e:
         if e.errno in (errno.EPERM, errno.EACCES, errno.EBUSY, errno.EINVAL):
             # If we try to rename a non-existant file onto cwd, we get
@@ -386,16 +373,6 @@
         raise
 
 
-def _wrapped_rename(old, new):
-    """Rename a file or directory"""
-    try:
-        os.rename(old, new)
-    except (IOError, OSError), e:
-        # this is eventually called by all rename-like functions, so should 
-        # catch all of them
-        raise _add_rename_error_details(e, old, new)
-
-
 def _mac_getcwd():
     return unicodedata.normalize('NFC', os.getcwdu())
 
@@ -406,13 +383,13 @@
 realpath = _posix_realpath
 pathjoin = os.path.join
 normpath = os.path.normpath
-rename = _wrapped_rename # overridden below on win32
 getcwd = os.getcwdu
+rename = os.rename
 dirname = os.path.dirname
 basename = os.path.basename
 split = os.path.split
 splitext = os.path.splitext
-# These were already imported into local scope
+# These were already lazily imported into local scope
 # mkdtemp = tempfile.mkdtemp
 # rmtree = shutil.rmtree
 
@@ -458,7 +435,7 @@
     getcwd = _mac_getcwd
 
 
-def get_terminal_encoding():
+def get_terminal_encoding(trace=False):
     """Find the best encoding for printing to the screen.
 
     This attempts to check both sys.stdout and sys.stdin to see
@@ -470,6 +447,8 @@
 
     On my standard US Windows XP, the preferred encoding is
     cp1252, but the console is cp437
+
+    :param trace: If True trace the selected encoding via mutter().
     """
     from bzrlib.trace import mutter
     output_encoding = getattr(sys.stdout, 'encoding', None)
@@ -477,17 +456,22 @@
         input_encoding = getattr(sys.stdin, 'encoding', None)
         if not input_encoding:
             output_encoding = get_user_encoding()
-            mutter('encoding stdout as osutils.get_user_encoding() %r',
+            if trace:
+                mutter('encoding stdout as osutils.get_user_encoding() %r',
                    output_encoding)
         else:
             output_encoding = input_encoding
-            mutter('encoding stdout as sys.stdin encoding %r', output_encoding)
+            if trace:
+                mutter('encoding stdout as sys.stdin encoding %r',
+                    output_encoding)
     else:
-        mutter('encoding stdout as sys.stdout encoding %r', output_encoding)
+        if trace:
+            mutter('encoding stdout as sys.stdout encoding %r', output_encoding)
     if output_encoding == 'cp0':
         # invalid encoding (cp0 means 'no codepage' on Windows)
         output_encoding = get_user_encoding()
-        mutter('cp0 is invalid encoding.'
+        if trace:
+            mutter('cp0 is invalid encoding.'
                ' encoding stdout as osutils.get_user_encoding() %r',
                output_encoding)
     # check encoding
@@ -519,7 +503,7 @@
 def isdir(f):
     """True if f is an accessible directory."""
     try:
-        return S_ISDIR(os.lstat(f)[ST_MODE])
+        return stat.S_ISDIR(os.lstat(f)[stat.ST_MODE])
     except OSError:
         return False
 
@@ -527,14 +511,14 @@
 def isfile(f):
     """True if f is a regular file."""
     try:
-        return S_ISREG(os.lstat(f)[ST_MODE])
+        return stat.S_ISREG(os.lstat(f)[stat.ST_MODE])
     except OSError:
         return False
 
 def islink(f):
     """True if f is a symlink."""
     try:
-        return S_ISLNK(os.lstat(f)[ST_MODE])
+        return stat.S_ISLNK(os.lstat(f)[stat.ST_MODE])
     except OSError:
         return False
 
@@ -880,7 +864,7 @@
 
 def filesize(f):
     """Return size of given open file."""
-    return os.fstat(f.fileno())[ST_SIZE]
+    return os.fstat(f.fileno())[stat.ST_SIZE]
 
 
 # Define rand_bytes based on platform.
@@ -948,7 +932,7 @@
 
 def parent_directories(filename):
     """Return the list of parent directories, deepest first.
-    
+
     For example, parent_directories("a/b/c") -> ["a/b", "a"].
     """
     parents = []
@@ -978,7 +962,7 @@
     # NB: This docstring is just an example, not a doctest, because doctest
     # currently can't cope with the use of lazy imports in this namespace --
     # mbp 20090729
-    
+
     # This currently doesn't report the failure at the time it occurs, because
     # they tend to happen very early in startup when we can't check config
     # files etc, and also we want to report all failures but not spam the user
@@ -1054,8 +1038,8 @@
 
 
 def delete_any(path):
-    """Delete a file, symlink or directory.  
-    
+    """Delete a file, symlink or directory.
+
     Will delete even if readonly.
     """
     try:
@@ -1250,6 +1234,22 @@
     # but for now, we haven't optimized...
     return [canonical_relpath(base, p) for p in paths]
 
+
+def decode_filename(filename):
+    """Decode the filename using the filesystem encoding
+
+    If it is unicode, it is returned.
+    Otherwise it is decoded from the the filesystem's encoding. If decoding
+    fails, a errors.BadFilenameEncoding exception is raised.
+    """
+    if type(filename) is unicode:
+        return filename
+    try:
+        return filename.decode(_fs_enc)
+    except UnicodeDecodeError:
+        raise errors.BadFilenameEncoding(filename, _fs_enc)
+
+
 def safe_unicode(unicode_or_utf8_string):
     """Coerce unicode_or_utf8_string into unicode.
 
@@ -1338,7 +1338,7 @@
 def normalizes_filenames():
     """Return True if this platform normalizes unicode filenames.
 
-    Mac OSX does, Windows/Linux do not.
+    Only Mac OSX.
     """
     return _platform_normalizes_filenames
 
@@ -1349,7 +1349,7 @@
     On platforms where the system normalizes filenames (Mac OSX),
     you can access a file by any path which will normalize correctly.
     On platforms where the system does not normalize filenames
-    (Windows, Linux), you have to access a file by its exact path.
+    (everything else), you have to access a file by its exact path.
 
     Internally, bzr only supports NFC normalization, since that is
     the standard for XML documents.
@@ -1417,6 +1417,12 @@
 terminal_width() returns None.
 """
 
+# Keep some state so that terminal_width can detect if _terminal_size has
+# returned a different size since the process started.  See docstring and
+# comments of terminal_width for details.
+# _terminal_size_state has 3 possible values: no_data, unchanged, and changed.
+_terminal_size_state = 'no_data'
+_first_terminal_size = None
 
 def terminal_width():
     """Return terminal width.
@@ -1426,20 +1432,34 @@
     The rules are:
     - if BZR_COLUMNS is set, returns its value
     - if there is no controlling terminal, returns None
+    - query the OS, if the queried size has changed since the last query,
+      return its value,
     - if COLUMNS is set, returns its value,
+    - if the OS has a value (even though it's never changed), return its value.
 
     From there, we need to query the OS to get the size of the controlling
     terminal.
 
-    Unices:
+    On Unices we query the OS by:
     - get termios.TIOCGWINSZ
     - if an error occurs or a negative value is obtained, returns None
 
-    Windows:
-    
+    On Windows we query the OS by:
     - win32utils.get_console_size() decides,
     - returns None on error (provided default value)
     """
+    # Note to implementors: if changing the rules for determining the width,
+    # make sure you've considered the behaviour in these cases:
+    #  - M-x shell in emacs, where $COLUMNS is set and TIOCGWINSZ returns 0,0.
+    #  - bzr log | less, in bash, where $COLUMNS not set and TIOCGWINSZ returns
+    #    0,0.
+    #  - (add more interesting cases here, if you find any)
+    # Some programs implement "Use $COLUMNS (if set) until SIGWINCH occurs",
+    # but we don't want to register a signal handler because it is impossible
+    # to do so without risking EINTR errors in Python <= 2.6.5 (see
+    # <http://bugs.python.org/issue8354>).  Instead we check TIOCGWINSZ every
+    # time so we can notice if the reported size has changed, which should have
+    # a similar effect.
 
     # If BZR_COLUMNS is set, take it, user is always right
     try:
@@ -1448,24 +1468,39 @@
         pass
 
     isatty = getattr(sys.stdout, 'isatty', None)
-    if  isatty is None or not isatty():
+    if isatty is None or not isatty():
         # Don't guess, setting BZR_COLUMNS is the recommended way to override.
         return None
 
-    # If COLUMNS is set, take it, the terminal knows better (even inside a
-    # given terminal, the application can decide to set COLUMNS to a lower
-    # value (splitted screen) or a bigger value (scroll bars))
+    # Query the OS
+    width, height = os_size = _terminal_size(None, None)
+    global _first_terminal_size, _terminal_size_state
+    if _terminal_size_state == 'no_data':
+        _first_terminal_size = os_size
+        _terminal_size_state = 'unchanged'
+    elif (_terminal_size_state == 'unchanged' and
+          _first_terminal_size != os_size):
+        _terminal_size_state = 'changed'
+
+    # If the OS claims to know how wide the terminal is, and this value has
+    # ever changed, use that.
+    if _terminal_size_state == 'changed':
+        if width is not None and width > 0:
+            return width
+
+    # If COLUMNS is set, use it.
     try:
         return int(os.environ['COLUMNS'])
     except (KeyError, ValueError):
         pass
 
-    width, height = _terminal_size(None, None)
-    if width <= 0:
-        # Consider invalid values as meaning no width
-        return None
+    # Finally, use an unchanged size from the OS, if we have one.
+    if _terminal_size_state == 'unchanged':
+        if width is not None and width > 0:
+            return width
 
-    return width
+    # The width could not be determined.
+    return None
 
 
 def _win32_terminal_size(width, height):
@@ -1498,31 +1533,6 @@
     _terminal_size = _ioctl_terminal_size
 
 
-def _terminal_size_changed(signum, frame):
-    """Set COLUMNS upon receiving a SIGnal for WINdow size CHange."""
-    width, height = _terminal_size(None, None)
-    if width is not None:
-        os.environ['COLUMNS'] = str(width)
-
-
-_registered_sigwinch = False
-def watch_sigwinch():
-    """Register for SIGWINCH, once and only once.
-
-    Do nothing if the signal module is not available.
-    """
-    global _registered_sigwinch
-    if not _registered_sigwinch:
-        try:
-            import signal
-            if getattr(signal, "SIGWINCH", None) is not None:
-                set_signal_handler(signal.SIGWINCH, _terminal_size_changed)
-        except ImportError:
-            # python doesn't provide signal support, nothing we can do about it
-            pass
-        _registered_sigwinch = True
-
-
 def supports_executable():
     return sys.platform != "win32"
 
@@ -1651,7 +1661,7 @@
         dirblock = []
         append = dirblock.append
         try:
-            names = sorted(_listdir(top))
+            names = sorted(map(decode_filename, _listdir(top)))
         except OSError, e:
             if not _is_error_enotdir(e):
                 raise
@@ -1959,6 +1969,10 @@
     return user_encoding
 
 
+def get_diff_header_encoding():
+    return get_terminal_encoding()
+
+
 def get_host_name():
     """Return the current unicode host name.
 
@@ -2027,14 +2041,14 @@
 
 def send_all(sock, bytes, report_activity=None):
     """Send all bytes on a socket.
- 
+
     Breaks large blocks in smaller chunks to avoid buffering limitations on
     some platforms, and catches EINTR which may be thrown if the send is
     interrupted by a signal.
 
     This is preferred to socket.sendall(), because it avoids portability bugs
     and provides activity reporting.
- 
+
     :param report_activity: Call this as bytes are read, see
         Transport._report_activity
     """
@@ -2051,6 +2065,29 @@
             report_activity(sent, 'write')
 
 
+def connect_socket(address):
+    # Slight variation of the socket.create_connection() function (provided by
+    # python-2.6) that can fail if getaddrinfo returns an empty list. We also
+    # provide it for previous python versions. Also, we don't use the timeout
+    # parameter (provided by the python implementation) so we don't implement
+    # it either).
+    err = socket.error('getaddrinfo returns an empty list')
+    host, port = address
+    for res in socket.getaddrinfo(host, port, 0, socket.SOCK_STREAM):
+        af, socktype, proto, canonname, sa = res
+        sock = None
+        try:
+            sock = socket.socket(af, socktype, proto)
+            sock.connect(sa)
+            return sock
+
+        except socket.error, err:
+            # 'err' is now the most recent error
+            if sock is not None:
+                sock.close()
+    raise err
+
+
 def dereference_path(path):
     """Determine the real path to a file.
 
@@ -2095,9 +2132,11 @@
     base = dirname(bzrlib.__file__)
     if getattr(sys, 'frozen', None):    # bzr.exe
         base = abspath(pathjoin(base, '..', '..'))
-    filename = pathjoin(base, resource_relpath)
-    return open(filename, 'rU').read()
-
+    f = file(pathjoin(base, resource_relpath), "rU")
+    try:
+        return f.read()
+    finally:
+        f.close()
 
 def file_kind_from_stat_mode_thunk(mode):
     global file_kind_from_stat_mode
@@ -2126,7 +2165,7 @@
 
 def until_no_eintr(f, *a, **kw):
     """Run f(*a, **kw), retrying if an EINTR error occurs.
-    
+
     WARNING: you must be certain that it is safe to retry the call repeatedly
     if EINTR does occur.  This is typically only true for low-level operations
     like os.read.  If in any doubt, don't use this.
@@ -2147,6 +2186,7 @@
             raise
 
 
+@deprecated_function(deprecated_in((2, 2, 0)))
 def re_compile_checked(re_string, flags=0, where=""):
     """Return a compiled re, or raise a sensible error.
 
@@ -2162,12 +2202,12 @@
         re_obj = re.compile(re_string, flags)
         re_obj.search("")
         return re_obj
-    except re.error, e:
+    except errors.InvalidPattern, e:
         if where:
             where = ' in ' + where
         # despite the name 'error' is a type
-        raise errors.BzrCommandError('Invalid regular expression%s: %r: %s'
-            % (where, re_string, e))
+        raise errors.BzrCommandError('Invalid regular expression%s: %s'
+            % (where, e.msg))
 
 
 if sys.platform == "win32":
@@ -2263,7 +2303,7 @@
 if sys.platform == 'win32':
     def open_file(filename, mode='r', bufsize=-1):
         """This function is used to override the ``open`` builtin.
-        
+
         But it uses O_NOINHERIT flag so the file handle is not inherited by
         child processes.  Deleting or renaming a closed file opened with this
         function is not blocking child processes.
@@ -2302,3 +2342,15 @@
         return os.fdopen(os.open(filename, flags), mode, bufsize)
 else:
     open_file = open
+
+
+def getuser_unicode():
+    """Return the username as unicode.
+    """
+    try:
+        user_encoding = get_user_encoding()
+        username = getpass.getuser().decode(user_encoding)
+    except UnicodeDecodeError:
+        raise errors.BzrError("Can't decode username as %s." % \
+                user_encoding)
+    return username

=== modified file 'bzrlib/plugin.py'
--- bzrlib/plugin.py	2010-04-06 07:22:31 +0000
+++ bzrlib/plugin.py	2010-06-11 08:02:42 +0000
@@ -81,6 +81,33 @@
     return path.rstrip("\\/")
 
 
+def _get_specific_plugin_paths(paths):
+    """Returns the plugin paths from a string describing the associations.
+
+    :param paths: A string describing the paths associated with the plugins.
+
+    :returns: A list of (plugin name, path) tuples.
+
+    For example, if paths is my_plugin@/test/my-test:her_plugin@/production/her,
+    [('my_plugin', '/test/my-test'), ('her_plugin', '/production/her')] 
+    will be returned.
+
+    Note that ':' in the example above depends on the os.
+    """
+    if not paths:
+        return []
+    specs = []
+    for spec in paths.split(os.pathsep):
+        try:
+            name, path = spec.split('@')
+        except ValueError:
+            raise errors.BzrCommandError(
+                '"%s" is not a valid <plugin_name>@<plugin_path> description '
+                % spec)
+        specs.append((name, path))
+    return specs
+
+
 def set_plugins_path(path=None):
     """Set the path for plugins to be loaded from.
 
@@ -98,10 +125,8 @@
         for name in disabled_plugins.split(os.pathsep):
             PluginImporter.blacklist.add('bzrlib.plugins.' + name)
     # Set up a the specific paths for plugins
-    specific_plugins = os.environ.get('BZR_PLUGINS_AT', None)
-    if specific_plugins is not None:
-        for spec in specific_plugins.split(os.pathsep):
-            plugin_name, plugin_path = spec.split('@')
+    for plugin_name, plugin_path in _get_specific_plugin_paths(os.environ.get(
+            'BZR_PLUGINS_AT', None)):
             PluginImporter.specific_paths[
                 'bzrlib.plugins.%s' % plugin_name] = plugin_path
     return path
@@ -562,7 +587,6 @@
         # We are called only for specific paths
         plugin_path = self.specific_paths[fullname]
         loading_path = None
-        package = False
         if os.path.isdir(plugin_path):
             for suffix, mode, kind in imp.get_suffixes():
                 if kind not in (imp.PY_SOURCE, imp.PY_COMPILED):
@@ -570,8 +594,12 @@
                     continue
                 init_path = osutils.pathjoin(plugin_path, '__init__' + suffix)
                 if os.path.isfile(init_path):
-                    loading_path = init_path
-                    package = True
+                    # We've got a module here and load_module needs specific
+                    # parameters.
+                    loading_path = plugin_path
+                    suffix = ''
+                    mode = ''
+                    kind = imp.PKG_DIRECTORY
                     break
         else:
             for suffix, mode, kind in imp.get_suffixes():
@@ -581,17 +609,18 @@
         if loading_path is None:
             raise ImportError('%s cannot be loaded from %s'
                               % (fullname, plugin_path))
-        f = open(loading_path, mode)
+        if kind is imp.PKG_DIRECTORY:
+            f = None
+        else:
+            f = open(loading_path, mode)
         try:
             mod = imp.load_module(fullname, f, loading_path,
                                   (suffix, mode, kind))
-            if package:
-                # The plugin can contain modules, so be ready
-                mod.__path__ = [plugin_path]
             mod.__package__ = fullname
             return mod
         finally:
-            f.close()
+            if f is not None:
+                f.close()
 
 
 # Install a dedicated importer for plugins requiring special handling

=== added directory 'bzrlib/plugins/bash_completion'
=== added file 'bzrlib/plugins/bash_completion/README.txt'
--- bzrlib/plugins/bash_completion/README.txt	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/bash_completion/README.txt	2010-05-22 11:16:04 +0000
@@ -0,0 +1,143 @@
+.. comment
+
+  Copyright (C) 2010 Canonical Ltd
+
+  This file is part of bzr-bash-completion
+
+  bzr-bash-completion free software: you can redistribute it and/or
+  modify it under the terms of the GNU General Public License as
+  published by the Free Software Foundation, either version 2 of the
+  License, or (at your option) any later version.
+
+  bzr-bash-completion is distributed in the hope that it will be
+  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
+  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+==========================
+bzr bash-completion plugin
+==========================
+
+This plugin generates a shell function which can be used by bash to
+automatically complete the currently typed command when the user
+presses the completion key (usually tab).
+
+It is intended as a bzr plugin, but can be used to some extend as a
+standalone python script as well.
+
+| Copyright (C) 2009, 2010 Canonical Ltd
+
+.. contents::
+
+-------------------------------
+Bundled and standalone versions
+-------------------------------
+
+This plugin has been merged_ into the main source tree of Bazaar.
+Starting with the bzr 2.3 series, a common bzr installation will
+include this plugin.
+
+There is still a standalone version available. It makes the plugin
+available for users of older bzr versions. When using both versions,
+local configuration might determine which version actually gets used,
+and some installations might even overwrite one another, so don't use
+the standalone version if you have the bundled one, unless you know
+what you are doing. Some effort will be made to keep the two versions
+reasonably in sync for some time yet.
+
+This text here documents the bundled version.
+
+.. _merged: http://bazaar.launchpad.net/~bzr-pqm/bzr/bzr.dev/revision/5240
+
+-----
+Using
+-----
+
+Using as a plugin
+-----------------
+
+This is the preferred method of generating the completion function, as
+it will ensure proper bzr initialization.
+
+::
+
+  eval "`bzr bash-completion`"
+
+Lazy initialization
+-------------------
+
+Running the above command automatically from your ``~/.bashrc`` file
+or similar can cause annoying delays in the startup of your shell.
+To avoid this problem, you can delay the generation of the completion
+function until you actually need it.
+
+To do so, source the file ``contrib/bash/bzr`` shipped with the bzr
+source distribution from your ``~/.bashrc`` file
+or add it to your ``~/.bash_completion`` if
+your setup uses such a file. On a system-wide installation, the
+directory ``/usr/share/bash-completion/`` might contain such bash
+completion scripts.
+
+Note that the full completion function is generated only once per
+shell session. If you update your bzr installation or change the set
+of installed plugins, then you might wish to regenerate the completion
+function manually as described above in order for completion to take
+these changes into account.
+
+--------------
+Design concept
+--------------
+
+The plugin is designed to generate a completion function
+containing all the required information about the possible
+completions. This is usually only done once when bash
+initializes. After that, no more invocations of bzr are required. This
+makes the function much faster than a possible implementation talking
+to bzr for each and every completion. On the other hand, this has the
+effect that updates to bzr or its plugins won't show up in the
+completions immediately, but only after the completion function has
+been regenerated.
+
+-------
+License
+-------
+
+As this is built upon a bash completion script originally included in
+the bzr source tree, and as the bzr sources are covered by the GPL 2,
+this plugin here is licensed under these same terms.
+
+If you require a more liberal license, you'll have to contact all
+those who contributed code to this plugin, be it for bash or for
+python.
+
+-------
+History
+-------
+
+The plugin was created by Martin von Gagern in 2009, building on a
+static completion function of very limited scope distributed together
+with bzr.
+
+A version of it was merged into the bzr source tree in May 2010.
+
+----------
+References
+----------
+
+Bazaar homepage
+  | http://bazaar.canonical.com/
+Standalone plugin homepages
+  | https://launchpad.net/bzr-bash-completion
+  | http://pypi.python.org/pypi/bzr-bash-completion
+
+
+
+.. vim: ft=rst
+
+.. emacs
+   Local Variables:
+   mode: rst
+   End:

=== added file 'bzrlib/plugins/bash_completion/__init__.py'
--- bzrlib/plugins/bash_completion/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/bash_completion/__init__.py	2010-05-25 21:43:32 +0000
@@ -0,0 +1,39 @@
+# Copyright (C) 2009, 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+__doc__ = """Generate a shell function for bash command line completion.
+
+This plugin provides a command called bash-completion that generates a
+bash completion function for bzr. See its documentation for details.
+"""
+
+from bzrlib import commands, version_info
+
+
+bzr_plugin_name = 'bash_completion'
+bzr_commands = [ 'bash-completion' ]
+
+commands.plugin_cmds.register_lazy('cmd_bash_completion', [],
+                                   'bzrlib.plugins.bash_completion.bashcomp')
+
+
+def load_tests(basic_tests, module, loader):
+    testmod_names = [
+        'tests',
+        ]
+    basic_tests.addTest(loader.loadTestsFromModuleNames(
+            ["%s.%s" % (__name__, tmn) for tmn in testmod_names]))
+    return basic_tests

=== added file 'bzrlib/plugins/bash_completion/bashcomp.py'
--- bzrlib/plugins/bash_completion/bashcomp.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/bash_completion/bashcomp.py	2010-05-29 08:47:10 +0000
@@ -0,0 +1,482 @@
+#!/usr/bin/env python
+
+# Copyright (C) 2009, 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+from bzrlib import (
+    cmdline,
+    commands,
+    config,
+    help_topics,
+    option,
+    plugin,
+)
+import bzrlib
+import re
+
+
+class BashCodeGen(object):
+    """Generate a bash script for given completion data."""
+
+    def __init__(self, data, function_name='_bzr', debug=False):
+        self.data = data
+        self.function_name = function_name
+        self.debug = debug
+
+    def script(self):
+        return ("""\
+# Programmable completion for the Bazaar-NG bzr command under bash.
+# Known to work with bash 2.05a as well as bash 4.1.2, and probably
+# all versions in between as well.
+
+# Based originally on the svn bash completition script.
+# Customized by Sven Wilhelm/Icecrash.com
+# Adjusted for automatic generation by Martin von Gagern
+
+# Generated using the bash_completion plugin.
+# See https://launchpad.net/bzr-bash-completion for details.
+
+# Commands and options of bzr %(bzr_version)s
+
+shopt -s progcomp
+%(function)s
+complete -F %(function_name)s -o default bzr
+"""     % {
+            "function_name": self.function_name,
+            "function": self.function(),
+            "bzr_version": self.bzr_version(),
+        })
+
+    def function(self):
+        return ("""\
+%(function_name)s ()
+{
+	local cur cmds cmdIdx cmd cmdOpts fixedWords i globalOpts
+	local curOpt optEnums
+	local IFS=$' \\n'
+
+	COMPREPLY=()
+	cur=${COMP_WORDS[COMP_CWORD]}
+
+	cmds='%(cmds)s'
+	globalOpts=( %(global_options)s )
+
+	# do ordinary expansion if we are anywhere after a -- argument
+	for ((i = 1; i < COMP_CWORD; ++i)); do
+		[[ ${COMP_WORDS[i]} == "--" ]] && return 0
+	done
+
+	# find the command; it's the first word not starting in -
+	cmd=
+	for ((cmdIdx = 1; cmdIdx < ${#COMP_WORDS[@]}; ++cmdIdx)); do
+		if [[ ${COMP_WORDS[cmdIdx]} != -* ]]; then
+			cmd=${COMP_WORDS[cmdIdx]}
+			break
+		fi
+	done
+
+	# complete command name if we are not already past the command
+	if [[ $COMP_CWORD -le cmdIdx ]]; then
+		COMPREPLY=( $( compgen -W "$cmds ${globalOpts[*]}" -- $cur ) )
+		return 0
+	fi
+
+	# find the option for which we want to complete a value
+	curOpt=
+	if [[ $cur != -* ]] && [[ $COMP_CWORD -gt 1 ]]; then
+		curOpt=${COMP_WORDS[COMP_CWORD - 1]}
+		if [[ $curOpt == = ]]; then
+			curOpt=${COMP_WORDS[COMP_CWORD - 2]}
+		elif [[ $cur == : ]]; then
+			cur=
+			curOpt="$curOpt:"
+		elif [[ $curOpt == : ]]; then
+			curOpt=${COMP_WORDS[COMP_CWORD - 2]}:
+		fi
+	fi
+%(debug)s
+	cmdOpts=( )
+	optEnums=( )
+	fixedWords=( )
+	case $cmd in
+%(cases)s\
+	*)
+		cmdOpts=(--help -h)
+		;;
+	esac
+
+	IFS=$'\\n'
+	if [[ ${#fixedWords[@]} -eq 0 ]] && [[ ${#optEnums[@]} -eq 0 ]] && [[ $cur != -* ]]; then
+		case $curOpt in
+			tag:|*..tag:)
+				fixedWords=( $(bzr tags 2>/dev/null | sed 's/  *[^ ]*$//; s/ /\\\\\\\\ /g;') )
+				;;
+		esac
+		case $cur in
+			[\\"\\']tag:*)
+				fixedWords=( $(bzr tags 2>/dev/null | sed 's/  *[^ ]*$//; s/^/tag:/') )
+				;;
+			[\\"\\']*..tag:*)
+				fixedWords=( $(bzr tags 2>/dev/null | sed 's/  *[^ ]*$//') )
+				fixedWords=( $(for i in "${fixedWords[@]}"; do echo "${cur%%..tag:*}..tag:${i}"; done) )
+				;;
+		esac
+	elif [[ $cur == = ]] && [[ ${#optEnums[@]} -gt 0 ]]; then
+		# complete directly after "--option=", list all enum values
+		COMPREPLY=( "${optEnums[@]}" )
+		return 0
+	else
+		fixedWords=( "${cmdOpts[@]}"
+		             "${globalOpts[@]}"
+		             "${optEnums[@]}"
+		             "${fixedWords[@]}" )
+	fi
+
+	if [[ ${#fixedWords[@]} -gt 0 ]]; then
+		COMPREPLY=( $( compgen -W "${fixedWords[*]}" -- $cur ) )
+	fi
+
+	return 0
+}
+"""     % {
+            "cmds": self.command_names(),
+            "function_name": self.function_name,
+            "cases": self.command_cases(),
+            "global_options": self.global_options(),
+            "debug": self.debug_output(),
+        })
+        # Help Emacs terminate strings: "
+
+    def command_names(self):
+        return " ".join(self.data.all_command_aliases())
+
+    def debug_output(self):
+        if not self.debug:
+            return ''
+        else:
+            return (r"""
+	# Debugging code enabled using the --debug command line switch.
+	# Will dump some variables to the top portion of the terminal.
+	echo -ne '\e[s\e[H'
+	for (( i=0; i < ${#COMP_WORDS[@]}; ++i)); do
+		echo "\$COMP_WORDS[$i]='${COMP_WORDS[i]}'"$'\e[K'
+	done
+	for i in COMP_CWORD COMP_LINE COMP_POINT COMP_TYPE COMP_KEY cur curOpt; do
+		echo "\$${i}=\"${!i}\""$'\e[K'
+	done
+	echo -ne '---\e[K\e[u'
+""")
+
+    def bzr_version(self):
+        bzr_version = bzrlib.version_string
+        if not self.data.plugins:
+            bzr_version += "."
+        else:
+            bzr_version += " and the following plugins:"
+            for name, plugin in sorted(self.data.plugins.iteritems()):
+                bzr_version += "\n# %s" % plugin
+        return bzr_version
+
+    def global_options(self):
+        return " ".join(sorted(self.data.global_options))
+
+    def command_cases(self):
+        cases = ""
+        for command in self.data.commands:
+            cases += self.command_case(command)
+        return cases
+
+    def command_case(self, command):
+        case = "\t%s)\n" % "|".join(command.aliases)
+        if command.plugin:
+            case += "\t\t# plugin \"%s\"\n" % command.plugin
+        options = []
+        enums = []
+        for option in command.options:
+            for message in option.error_messages:
+                case += "\t\t# %s\n" % message
+            if option.registry_keys:
+                for key in option.registry_keys:
+                    options.append("%s=%s" % (option, key))
+                enums.append("%s) optEnums=( %s ) ;;" %
+                             (option, ' '.join(option.registry_keys)))
+            else:
+                options.append(str(option))
+        case += "\t\tcmdOpts=( %s )\n" % " ".join(options)
+        if command.fixed_words:
+            fixed_words = command.fixed_words
+            if isinstance(fixed_words, list):
+                fixed_words = "( %s )" + ' '.join(fixed_words)
+            case += "\t\tfixedWords=%s\n" % fixed_words
+        if enums:
+            case += "\t\tcase $curOpt in\n\t\t\t"
+            case += "\n\t\t\t".join(enums)
+            case += "\n\t\tesac\n"
+        case += "\t\t;;\n"
+        return case
+
+
+class CompletionData(object):
+
+    def __init__(self):
+        self.plugins = {}
+        self.global_options = set()
+        self.commands = []
+
+    def all_command_aliases(self):
+        for c in self.commands:
+            for a in c.aliases:
+                yield a
+
+
+class CommandData(object):
+
+    def __init__(self, name):
+        self.name = name
+        self.aliases = [name]
+        self.plugin = None
+        self.options = []
+        self.fixed_words = None
+
+
+class PluginData(object):
+
+    def __init__(self, name, version=None):
+        if version is None:
+            try:
+                version = bzrlib.plugin.plugins()[name].__version__
+            except:
+                version = 'unknown'
+        self.name = name
+        self.version = version
+
+    def __str__(self):
+        if self.version == 'unknown':
+            return self.name
+        return '%s %s' % (self.name, self.version)
+
+
+class OptionData(object):
+
+    def __init__(self, name):
+        self.name = name
+        self.registry_keys = None
+        self.error_messages = []
+
+    def __str__(self):
+        return self.name
+
+    def __cmp__(self, other):
+        return cmp(self.name, other.name)
+
+
+class DataCollector(object):
+
+    def __init__(self, no_plugins=False, selected_plugins=None):
+        self.data = CompletionData()
+        self.user_aliases = {}
+        if no_plugins:
+            self.selected_plugins = set()
+        elif selected_plugins is None:
+            self.selected_plugins = None
+        else:
+            self.selected_plugins = set([x.replace('-', '_')
+                                         for x in selected_plugins])
+
+    def collect(self):
+        self.global_options()
+        self.aliases()
+        self.commands()
+        return self.data
+
+    def global_options(self):
+        re_switch = re.compile(r'\n(--[A-Za-z0-9-_]+)(?:, (-\S))?\s')
+        help_text = help_topics.topic_registry.get_detail('global-options')
+        for long, short in re_switch.findall(help_text):
+            self.data.global_options.add(long)
+            if short:
+                self.data.global_options.add(short)
+
+    def aliases(self):
+        for alias, expansion in config.GlobalConfig().get_aliases().iteritems():
+            for token in cmdline.split(expansion):
+                if not token.startswith("-"):
+                    self.user_aliases.setdefault(token, set()).add(alias)
+                    break
+
+    def commands(self):
+        for name in sorted(commands.all_command_names()):
+            self.command(name)
+
+    def command(self, name):
+        cmd = commands.get_cmd_object(name)
+        cmd_data = CommandData(name)
+
+        plugin_name = cmd.plugin_name()
+        if plugin_name is not None:
+            if (self.selected_plugins is not None and
+                plugin not in self.selected_plugins):
+                return None
+            plugin_data = self.data.plugins.get(plugin_name)
+            if plugin_data is None:
+                plugin_data = PluginData(plugin_name)
+                self.data.plugins[plugin_name] = plugin_data
+            cmd_data.plugin = plugin_data
+        self.data.commands.append(cmd_data)
+
+        # Find all aliases to the command; both cmd-defined and user-defined.
+        # We assume a user won't override one command with a different one,
+        # but will choose completely new names or add options to existing
+        # ones while maintaining the actual command name unchanged.
+        cmd_data.aliases.extend(cmd.aliases)
+        cmd_data.aliases.extend(sorted([useralias
+            for cmdalias in cmd_data.aliases
+            if cmdalias in self.user_aliases
+            for useralias in self.user_aliases[cmdalias]
+            if useralias not in cmd_data.aliases]))
+
+        opts = cmd.options()
+        for optname, opt in sorted(opts.iteritems()):
+            cmd_data.options.extend(self.option(opt))
+
+        if 'help' == name or 'help' in cmd.aliases:
+            cmd_data.fixed_words = ('($cmds %s)' %
+                " ".join(sorted(help_topics.topic_registry.keys())))
+
+        return cmd_data
+
+    def option(self, opt):
+        optswitches = {}
+        parser = option.get_optparser({opt.name: opt})
+        parser = self.wrap_parser(optswitches, parser)
+        optswitches.clear()
+        opt.add_option(parser, opt.short_name())
+        if isinstance(opt, option.RegistryOption) and opt.enum_switch:
+            enum_switch = '--%s' % opt.name
+            enum_data = optswitches.get(enum_switch)
+            if enum_data:
+                try:
+                    enum_data.registry_keys = opt.registry.keys()
+                except ImportError, e:
+                    enum_data.error_messages.append(
+                        "ERROR getting registry keys for '--%s': %s"
+                        % (opt.name, str(e).split('\n')[0]))
+        return sorted(optswitches.values())
+
+    def wrap_container(self, optswitches, parser):
+        def tweaked_add_option(*opts, **attrs):
+            for name in opts:
+                optswitches[name] = OptionData(name)
+        parser.add_option = tweaked_add_option
+        return parser
+
+    def wrap_parser(self, optswitches, parser):
+        orig_add_option_group = parser.add_option_group
+        def tweaked_add_option_group(*opts, **attrs):
+            return self.wrap_container(optswitches,
+                orig_add_option_group(*opts, **attrs))
+        parser.add_option_group = tweaked_add_option_group
+        return self.wrap_container(optswitches, parser)
+
+
+def bash_completion_function(out, function_name="_bzr", function_only=False,
+                             debug=False,
+                             no_plugins=False, selected_plugins=None):
+    dc = DataCollector(no_plugins=no_plugins, selected_plugins=selected_plugins)
+    data = dc.collect()
+    cg = BashCodeGen(data, function_name=function_name, debug=debug)
+    if function_only:
+        res = cg.function()
+    else:
+        res = cg.script()
+    out.write(res)
+
+
+class cmd_bash_completion(commands.Command):
+    __doc__ = """Generate a shell function for bash command line completion.
+
+    This command generates a shell function which can be used by bash to
+    automatically complete the currently typed command when the user presses
+    the completion key (usually tab).
+    
+    Commonly used like this:
+        eval "`bzr bash-completion`"
+    """
+
+    takes_options = [
+        option.Option("function-name", short_name="f", type=str, argname="name",
+               help="Name of the generated function (default: _bzr)"),
+        option.Option("function-only", short_name="o", type=None,
+               help="Generate only the shell function, don't enable it"),
+        option.Option("debug", type=None, hidden=True,
+               help="Enable shell code useful for debugging"),
+        option.ListOption("plugin", type=str, argname="name",
+                # param_name="selected_plugins", # doesn't work, bug #387117
+                help="Enable completions for the selected plugin"
+                + " (default: all plugins)"),
+        ]
+
+    def run(self, **kwargs):
+        import sys
+        from bashcomp import bash_completion_function
+        if 'plugin' in kwargs:
+            # work around bug #387117 which prevents us from using param_name
+            if len(kwargs['plugin']) > 0:
+                kwargs['selected_plugins'] = kwargs['plugin']
+            del kwargs['plugin']
+        bash_completion_function(sys.stdout, **kwargs)
+
+
+if __name__ == '__main__':
+
+    import sys
+    import locale
+    import optparse
+
+    def plugin_callback(option, opt, value, parser):
+        values = parser.values.selected_plugins
+        if value == '-':
+            del values[:]
+        else:
+            values.append(value)
+
+    parser = optparse.OptionParser(usage="%prog [-f NAME] [-o]")
+    parser.add_option("--function-name", "-f", metavar="NAME",
+                      help="Name of the generated function (default: _bzr)")
+    parser.add_option("--function-only", "-o", action="store_true",
+                      help="Generate only the shell function, don't enable it")
+    parser.add_option("--debug", action="store_true",
+                      help=optparse.SUPPRESS_HELP)
+    parser.add_option("--no-plugins", action="store_true",
+                      help="Don't load any bzr plugins")
+    parser.add_option("--plugin", metavar="NAME", type="string",
+                      dest="selected_plugins", default=[],
+                      action="callback", callback=plugin_callback,
+                      help="Enable completions for the selected plugin"
+                      + " (default: all plugins)")
+    (opts, args) = parser.parse_args()
+    if args:
+        parser.error("script does not take positional arguments")
+    kwargs = dict()
+    for name, value in opts.__dict__.iteritems():
+        if value is not None:
+            kwargs[name] = value
+
+    locale.setlocale(locale.LC_ALL, '')
+    if not kwargs.get('no_plugins', False):
+        plugin.load_plugins()
+    commands.install_bzr_command_hooks()
+    bash_completion_function(sys.stdout, **kwargs)

=== added directory 'bzrlib/plugins/bash_completion/tests'
=== added file 'bzrlib/plugins/bash_completion/tests/__init__.py'
--- bzrlib/plugins/bash_completion/tests/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/bash_completion/tests/__init__.py	2010-04-29 20:51:13 +0000
@@ -0,0 +1,23 @@
+# Copyright (C) 2010 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+def load_tests(basic_tests, module, loader):
+    testmod_names = [
+        'test_bashcomp',
+        ]
+    basic_tests.addTest(loader.loadTestsFromModuleNames(
+            ["%s.%s" % (__name__, tmn) for tmn in testmod_names]))
+    return basic_tests

=== added file 'bzrlib/plugins/bash_completion/tests/test_bashcomp.py'
--- bzrlib/plugins/bash_completion/tests/test_bashcomp.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/bash_completion/tests/test_bashcomp.py	2010-05-29 10:30:53 +0000
@@ -0,0 +1,326 @@
+# Copyright (C) 2010 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+import bzrlib
+from bzrlib import commands, tests
+from bzrlib.tests import features
+from bzrlib.plugins.bash_completion.bashcomp import *
+
+import os
+import subprocess
+
+
+class BashCompletionMixin(object):
+    """Component for testing execution of a bash completion script."""
+
+    _test_needs_features = [features.bash_feature]
+
+    def complete(self, words, cword=-1):
+        """Perform a bash completion.
+
+        :param words: a list of words representing the current command.
+        :param cword: the current word to complete, defaults to the last one.
+        """
+        if self.script is None:
+            self.script = self.get_script()
+        proc = subprocess.Popen([features.bash_feature.path,
+                                 '--noprofile'],
+                                stdin=subprocess.PIPE,
+                                stdout=subprocess.PIPE,
+                                stderr=subprocess.PIPE)
+        if cword < 0:
+            cword = len(words) + cword
+        input = '%s\n' % self.script
+        input += ('COMP_WORDS=( %s )\n' %
+                  ' '.join(["'"+w.replace("'", "'\\''")+"'" for w in words]))
+        input += 'COMP_CWORD=%d\n' % cword
+        input += '%s\n' % getattr(self, 'script_name', '_bzr')
+        input += 'echo ${#COMPREPLY[*]}\n'
+        input += "IFS=$'\\n'\n"
+        input += 'echo "${COMPREPLY[*]}"\n'
+        (out, err) = proc.communicate(input)
+        if '' != err:
+            raise AssertionError('Unexpected error message:\n%s' % err)
+        self.assertEqual('', err, 'No messages to standard error')
+        #import sys
+        #print >>sys.stdout, '---\n%s\n---\n%s\n---\n' % (input, out)
+        lines = out.split('\n')
+        nlines = int(lines[0])
+        del lines[0]
+        self.assertEqual('', lines[-1], 'Newline at end')
+        del lines[-1]
+        if nlines == 0 and len(lines) == 1 and lines[0] == '':
+            del lines[0]
+        self.assertEqual(nlines, len(lines), 'No newlines in generated words')
+        self.completion_result = set(lines)
+        return self.completion_result
+
+    def assertCompletionEquals(self, *words):
+        self.assertEqual(set(words), self.completion_result)
+
+    def assertCompletionContains(self, *words):
+        missing = set(words) - self.completion_result
+        if missing:
+            raise AssertionError('Completion should contain %r but it has %r'
+                                 % (missing, self.completion_result))
+
+    def assertCompletionOmits(self, *words):
+        surplus = set(words) & self.completion_result
+        if surplus:
+            raise AssertionError('Completion should omit %r but it has %r'
+                                 % (surplus, res, self.completion_result))
+
+    def get_script(self):
+        commands.install_bzr_command_hooks()
+        dc = DataCollector()
+        data = dc.collect()
+        cg = BashCodeGen(data)
+        res = cg.function()
+        return res
+
+
+class TestBashCompletion(tests.TestCase, BashCompletionMixin):
+    """Test bash completions that don't execute bzr."""
+
+    def __init__(self, methodName='testMethod'):
+        super(TestBashCompletion, self).__init__(methodName)
+        self.script = None
+
+    def test_simple_scipt(self):
+        """Ensure that the test harness works as expected"""
+        self.script = """
+_bzr() {
+    COMPREPLY=()
+    # add all words in reverse order, with some markup around them
+    for ((i = ${#COMP_WORDS[@]}; i > 0; --i)); do
+        COMPREPLY+=( "-${COMP_WORDS[i-1]}+" )
+    done
+    # and append the current word
+    COMPREPLY+=( "+${COMP_WORDS[COMP_CWORD]}-" )
+}
+"""
+        self.complete(['foo', '"bar', "'baz"], cword=1)
+        self.assertCompletionEquals("-'baz+", '-"bar+', '-foo+', '+"bar-')
+
+    def test_cmd_ini(self):
+        self.complete(['bzr', 'ini'])
+        self.assertCompletionContains('init', 'init-repo', 'init-repository')
+        self.assertCompletionOmits('commit')
+
+    def test_init_opts(self):
+        self.complete(['bzr', 'init', '-'])
+        self.assertCompletionContains('-h', '--2a', '--format=2a')
+
+    def test_global_opts(self):
+        self.complete(['bzr', '-', 'init'], cword=1)
+        self.assertCompletionContains('--no-plugins', '--builtin')
+
+    def test_commit_dashm(self):
+        self.complete(['bzr', 'commit', '-m'])
+        self.assertCompletionEquals('-m')
+
+    def test_status_negated(self):
+        self.complete(['bzr', 'status', '--n'])
+        self.assertCompletionContains('--no-versioned', '--no-verbose')
+
+    def test_init_format_any(self):
+        self.complete(['bzr', 'init', '--format', '=', 'directory'], cword=3)
+        self.assertCompletionContains('1.9', '2a')
+
+    def test_init_format_2(self):
+        self.complete(['bzr', 'init', '--format', '=', '2', 'directory'],
+                      cword=4)
+        self.assertCompletionContains('2a')
+        self.assertCompletionOmits('1.9')
+
+
+class TestBashCompletionInvoking(tests.TestCaseWithTransport,
+                                 BashCompletionMixin):
+    """Test bash completions that might execute bzr.
+
+    Only the syntax ``$(bzr ...`` is supported so far. The bzr command
+    will be replaced by the bzr instance running this selftest.
+    """
+
+    def __init__(self, methodName='testMethod'):
+        super(TestBashCompletionInvoking, self).__init__(methodName)
+        self.script = None
+
+    def get_script(self):
+        s = super(TestBashCompletionInvoking, self).get_script()
+        return s.replace("$(bzr ", "$('%s' " % self.get_bzr_path())
+
+    def test_revspec_tag_all(self):
+        self.requireFeature(features.sed_feature)
+        wt = self.make_branch_and_tree('.', format='dirstate-tags')
+        wt.branch.tags.set_tag('tag1', 'null:')
+        wt.branch.tags.set_tag('tag2', 'null:')
+        wt.branch.tags.set_tag('3tag', 'null:')
+        self.complete(['bzr', 'log', '-r', 'tag', ':'])
+        self.assertCompletionEquals('tag1', 'tag2', '3tag')
+
+    def test_revspec_tag_prefix(self):
+        self.requireFeature(features.sed_feature)
+        wt = self.make_branch_and_tree('.', format='dirstate-tags')
+        wt.branch.tags.set_tag('tag1', 'null:')
+        wt.branch.tags.set_tag('tag2', 'null:')
+        wt.branch.tags.set_tag('3tag', 'null:')
+        self.complete(['bzr', 'log', '-r', 'tag', ':', 't'])
+        self.assertCompletionEquals('tag1', 'tag2')
+
+    def test_revspec_tag_spaces(self):
+        self.requireFeature(features.sed_feature)
+        wt = self.make_branch_and_tree('.', format='dirstate-tags')
+        wt.branch.tags.set_tag('tag with spaces', 'null:')
+        self.complete(['bzr', 'log', '-r', 'tag', ':', 't'])
+        self.assertCompletionEquals(r'tag\ with\ spaces')
+        self.complete(['bzr', 'log', '-r', '"tag:t'])
+        self.assertCompletionEquals('tag:tag with spaces')
+        self.complete(['bzr', 'log', '-r', "'tag:t"])
+        self.assertCompletionEquals('tag:tag with spaces')
+
+    def test_revspec_tag_endrange(self):
+        self.requireFeature(features.sed_feature)
+        wt = self.make_branch_and_tree('.', format='dirstate-tags')
+        wt.branch.tags.set_tag('tag1', 'null:')
+        wt.branch.tags.set_tag('tag2', 'null:')
+        self.complete(['bzr', 'log', '-r', '3..tag', ':', 't'])
+        self.assertCompletionEquals('tag1', 'tag2')
+        self.complete(['bzr', 'log', '-r', '"3..tag:t'])
+        self.assertCompletionEquals('3..tag:tag1', '3..tag:tag2')
+        self.complete(['bzr', 'log', '-r', "'3..tag:t"])
+        self.assertCompletionEquals('3..tag:tag1', '3..tag:tag2')
+
+
+class TestBashCodeGen(tests.TestCase):
+
+    def test_command_names(self):
+        data = CompletionData()
+        bar = CommandData('bar')
+        bar.aliases.append('baz')
+        data.commands.append(bar)
+        data.commands.append(CommandData('foo'))
+        cg = BashCodeGen(data)
+        self.assertEqual('bar baz foo', cg.command_names())
+
+    def test_debug_output(self):
+        data = CompletionData()
+        self.assertEqual('', BashCodeGen(data, debug=False).debug_output())
+        self.assertTrue(BashCodeGen(data, debug=True).debug_output())
+
+    def test_bzr_version(self):
+        data = CompletionData()
+        cg = BashCodeGen(data)
+        self.assertEqual('%s.' % bzrlib.version_string, cg.bzr_version())
+        data.plugins['foo'] = PluginData('foo', '1.0')
+        data.plugins['bar'] = PluginData('bar', '2.0')
+        cg = BashCodeGen(data)
+        self.assertEqual('''\
+%s and the following plugins:
+# bar 2.0
+# foo 1.0''' % bzrlib.version_string, cg.bzr_version())
+
+    def test_global_options(self):
+        data = CompletionData()
+        data.global_options.add('--foo')
+        data.global_options.add('--bar')
+        cg = BashCodeGen(data)
+        self.assertEqual('--bar --foo', cg.global_options())
+
+    def test_command_cases(self):
+        data = CompletionData()
+        bar = CommandData('bar')
+        bar.aliases.append('baz')
+        bar.options.append(OptionData('--opt'))
+        data.commands.append(bar)
+        data.commands.append(CommandData('foo'))
+        cg = BashCodeGen(data)
+        self.assertEqualDiff('''\
+\tbar|baz)
+\t\tcmdOpts=( --opt )
+\t\t;;
+\tfoo)
+\t\tcmdOpts=(  )
+\t\t;;
+''', cg.command_cases())
+
+    def test_command_case(self):
+        cmd = CommandData('cmd')
+        cmd.plugin = PluginData('plugger', '1.0')
+        bar = OptionData('--bar')
+        bar.registry_keys = ['that', 'this']
+        bar.error_messages.append('Some error message')
+        cmd.options.append(bar)
+        cmd.options.append(OptionData('--foo'))
+        data = CompletionData()
+        data.commands.append(cmd)
+        cg = BashCodeGen(data)
+        self.assertEqualDiff('''\
+\tcmd)
+\t\t# plugin "plugger 1.0"
+\t\t# Some error message
+\t\tcmdOpts=( --bar=that --bar=this --foo )
+\t\tcase $curOpt in
+\t\t\t--bar) optEnums=( that this ) ;;
+\t\tesac
+\t\t;;
+''', cg.command_case(cmd))
+
+
+class TestDataCollector(tests.TestCase):
+
+    def setUp(self):
+        super(TestDataCollector, self).setUp()
+        commands.install_bzr_command_hooks()
+
+    def test_global_options(self):
+        dc = DataCollector()
+        dc.global_options()
+        self.assertSubset(['--no-plugins', '--builtin'],
+                           dc.data.global_options)
+
+    def test_commands(self):
+        dc = DataCollector()
+        dc.commands()
+        self.assertSubset(['init', 'init-repo', 'init-repository'],
+                           dc.data.all_command_aliases())
+
+    def test_commands_from_plugins(self):
+        dc = DataCollector()
+        dc.commands()
+        self.assertSubset(['bash-completion'],
+                           dc.data.all_command_aliases())
+
+    def test_commit_dashm(self):
+        dc = DataCollector()
+        cmd = dc.command('commit')
+        self.assertSubset(['-m'],
+                           [str(o) for o in cmd.options])
+
+    def test_status_negated(self):
+        dc = DataCollector()
+        cmd = dc.command('status')
+        self.assertSubset(['--no-versioned', '--no-verbose'],
+                           [str(o) for o in cmd.options])
+
+    def test_init_format(self):
+        dc = DataCollector()
+        cmd = dc.command('init')
+        for opt in cmd.options:
+            if opt.name == '--format':
+                self.assertSubset(['2a'], opt.registry_keys)
+                return
+        raise AssertionError('Option --format not found')

=== modified file 'bzrlib/plugins/launchpad/__init__.py'
--- bzrlib/plugins/launchpad/__init__.py	2010-04-02 19:12:58 +0000
+++ bzrlib/plugins/launchpad/__init__.py	2010-10-05 15:52:15 +0000
@@ -14,7 +14,26 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-"""Launchpad.net integration plugin for Bazaar."""
+"""Launchpad.net integration plugin for Bazaar.
+
+This plugin provides facilities for working with Bazaar branches that are
+hosted on Launchpad (http://launchpad.net).  It provides a directory service 
+for referring to Launchpad branches using the "lp:" prefix.  For example,
+lp:bzr refers to the Bazaar's main development branch and
+lp:~username/project/branch-name can be used to refer to a specific branch.
+
+This plugin provides a bug tracker so that "bzr commit --fixes lp:1234" will
+record that revision as fixing Launchpad's bug 1234.
+
+The plugin also provides the following commands:
+
+    launchpad-login: Show or set the Launchpad user ID
+    launchpad-open: Open a Launchpad branch page in your web browser
+    lp-propose-merge: Propose merging a branch on Launchpad
+    register-branch: Register a branch with launchpad.net
+    launchpad-mirror: Ask Launchpad to mirror a branch now
+
+"""
 
 # The XMLRPC server address can be overridden by setting the environment
 # variable $BZR_LP_XMLRPC_URL
@@ -310,6 +329,8 @@
                             help='Propose the merge on staging.'),
                      Option('message', short_name='m', type=unicode,
                             help='Commit message.'),
+                     Option('approve',
+                            help='Mark the proposal as approved immediately.'),
                      ListOption('review', short_name='R', type=unicode,
                             help='Requested reviewer and optional type.')]
 
@@ -318,7 +339,7 @@
     aliases = ['lp-submit', 'lp-propose']
 
     def run(self, submit_branch=None, review=None, staging=False,
-            message=None):
+            message=None, approve=False):
         from bzrlib.plugins.launchpad import lp_propose
         tree, branch, relpath = bzrdir.BzrDir.open_containing_tree_or_branch(
             '.')
@@ -338,7 +359,7 @@
         else:
             target = _mod_branch.Branch.open(submit_branch)
         proposer = lp_propose.Proposer(tree, branch, target, message,
-                                       reviews, staging)
+                                       reviews, staging, approve=approve)
         proposer.check_proposal()
         proposer.create_proposal()
 

=== modified file 'bzrlib/plugins/launchpad/account.py'
--- bzrlib/plugins/launchpad/account.py	2009-04-27 16:10:10 +0000
+++ bzrlib/plugins/launchpad/account.py	2010-06-20 11:18:38 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007, 2008 Canonical Ltd
+# Copyright (C) 2007-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,9 +20,12 @@
 than once for each place that needs to take it into account.
 """
 
-from bzrlib import errors, trace
+from bzrlib import (
+    errors,
+    trace,
+    transport,
+    )
 from bzrlib.config import AuthenticationConfig, GlobalConfig
-from bzrlib.transport import get_transport
 
 
 LAUNCHPAD_BASE = 'https://launchpad.net/'
@@ -97,7 +100,7 @@
     uploaded SSH keys.
     """
     if _transport is None:
-        _transport = get_transport(LAUNCHPAD_BASE)
+        _transport = transport.get_transport(LAUNCHPAD_BASE)
 
     try:
         data = _transport.get_bytes('~%s/+sshkeys' % username)

=== modified file 'bzrlib/plugins/launchpad/lp_directory.py'
--- bzrlib/plugins/launchpad/lp_directory.py	2010-03-30 11:05:14 +0000
+++ bzrlib/plugins/launchpad/lp_directory.py	2010-07-26 17:19:33 +0000
@@ -64,7 +64,16 @@
         """Resolve the base URL for this transport."""
         service = LaunchpadService.for_url(url)
         result = urlsplit(url)
-        resolve = _request_factory(result[2].strip('/'))
+        if _lp_login is None:
+            _lp_login = get_lp_login()
+        path = result[2].strip('/')
+        if path.startswith('~/'):
+            if _lp_login is None:
+                raise errors.InvalidURL(path=url,
+                    extra='Cannot resolve "~" to your username.'
+                          ' See "bzr help launchpad-login"')
+            path = '~' + _lp_login + path[1:]
+        resolve = _request_factory(path)
         try:
             result = resolve.submit(service)
         except xmlrpclib.Fault, fault:
@@ -74,8 +83,6 @@
         if 'launchpad' in debug.debug_flags:
             trace.mutter("resolve_lp_path(%r) == %r", url, result)
 
-        if _lp_login is None:
-            _lp_login = get_lp_login()
         _warned_login = False
         for url in result['urls']:
             scheme, netloc, path, query, fragment = urlsplit(url)

=== modified file 'bzrlib/plugins/launchpad/lp_propose.py'
--- bzrlib/plugins/launchpad/lp_propose.py	2010-04-26 13:51:08 +0000
+++ bzrlib/plugins/launchpad/lp_propose.py	2010-08-25 10:20:41 +0000
@@ -55,7 +55,7 @@
     hooks = ProposeMergeHooks()
 
     def __init__(self, tree, source_branch, target_branch, message, reviews,
-                 staging=False):
+                 staging=False, approve=False):
         """Constructor.
 
         :param tree: The working tree for the source branch.
@@ -65,6 +65,10 @@
         :param reviews: A list of tuples of reviewer, review type.
         :param staging: If True, propose the merge against staging instead of
             production.
+        :param approve: If True, mark the new proposal as approved immediately.
+            This is useful when a project permits some things to be approved
+            by the submitter (e.g. merges between release and deployment
+            branches).
         """
         self.tree = tree
         if staging:
@@ -81,6 +85,7 @@
             self.target_branch = lp_api.LaunchpadBranch.from_bzr(
                 self.launchpad, target_branch)
         self.commit_message = message
+        # XXX: this is where bug lp:583638 could be tackled.
         if reviews == []:
             target_reviewer = self.target_branch.lp.reviewer
             if target_reviewer is None:
@@ -90,6 +95,7 @@
             self.reviews = [(self.launchpad.people[reviewer], review_type)
                             for reviewer, review_type in
                             reviews]
+        self.approve = approve
 
     def get_comment(self, prerequisite_branch):
         """Determine the initial comment for the merge proposal."""
@@ -160,6 +166,24 @@
                  'prerequisite_branch': prerequisite_branch})
         return prerequisite_branch
 
+    def call_webservice(self, call, *args, **kwargs):
+        """Make a call to the webservice, wrapping failures.
+        
+        :param call: The call to make.
+        :param *args: *args for the call.
+        :param **kwargs: **kwargs for the call.
+        :return: The result of calling call(*args, *kwargs).
+        """
+        try:
+            return call(*args, **kwargs)
+        except restful_errors.HTTPError, e:
+            error_lines = []
+            for line in e.content.splitlines():
+                if line.startswith('Traceback (most recent call last):'):
+                    break
+                error_lines.append(line)
+            raise Exception(''.join(error_lines))
+
     def create_proposal(self):
         """Perform the submission."""
         prerequisite_branch = self._get_prerequisite_branch()
@@ -175,22 +199,16 @@
             review_types.append(review_type)
             reviewers.append(reviewer.self_link)
         initial_comment = self.get_comment(prerequisite_branch)
-        try:
-            mp = self.source_branch.lp.createMergeProposal(
-                target_branch=self.target_branch.lp,
-                prerequisite_branch=prereq,
-                initial_comment=initial_comment,
-                commit_message=self.commit_message, reviewers=reviewers,
-                review_types=review_types)
-        except restful_errors.HTTPError, e:
-            error_lines = []
-            for line in e.content.splitlines():
-                if line.startswith('Traceback (most recent call last):'):
-                    break
-                error_lines.append(line)
-            raise Exception(''.join(error_lines))
-        else:
-            webbrowser.open(canonical_url(mp))
+        mp = self.call_webservice(
+            self.source_branch.lp.createMergeProposal,
+            target_branch=self.target_branch.lp,
+            prerequisite_branch=prereq,
+            initial_comment=initial_comment,
+            commit_message=self.commit_message, reviewers=reviewers,
+            review_types=review_types)
+        if self.approve:
+            self.call_webservice(mp.setStatus, status='Approved')
+        webbrowser.open(canonical_url(mp))
 
 
 def modified_files(old_tree, new_tree):

=== modified file 'bzrlib/plugins/launchpad/lp_registration.py'
--- bzrlib/plugins/launchpad/lp_registration.py	2010-02-23 07:43:11 +0000
+++ bzrlib/plugins/launchpad/lp_registration.py	2010-05-20 18:23:10 +0000
@@ -98,11 +98,8 @@
     for instance, domain in LAUNCHPAD_DOMAINS.iteritems():
         LAUNCHPAD_INSTANCE[instance] = 'https://xmlrpc.%s/bazaar/' % domain
 
-    # We use edge as the default because:
-    # Beta users get redirected to it
-    # All users can use it
-    # There is a bug in the launchpad side where redirection causes an OOPS.
-    DEFAULT_INSTANCE = 'edge'
+    # Previously 'edge' was used to avoid a launchpad bug with redirection
+    DEFAULT_INSTANCE = 'production'
     DEFAULT_SERVICE_URL = LAUNCHPAD_INSTANCE[DEFAULT_INSTANCE]
 
     transport = None

=== modified file 'bzrlib/plugins/launchpad/test_account.py'
--- bzrlib/plugins/launchpad/test_account.py	2009-03-23 14:59:43 +0000
+++ bzrlib/plugins/launchpad/test_account.py	2010-08-30 07:34:11 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007, 2008 Canonical Ltd
+# Copyright (C) 2007-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,9 +26,7 @@
 class LaunchpadAccountTests(TestCaseInTempDir):
 
     def setup_config(self, text):
-        my_config = config.GlobalConfig()
-        config_file = StringIO(text)
-        my_config._get_parser(config_file)
+        my_config = config.GlobalConfig.from_string(text)
         return my_config
 
     def test_get_lp_login_unconfigured(self):

=== modified file 'bzrlib/plugins/launchpad/test_lp_directory.py'
--- bzrlib/plugins/launchpad/test_lp_directory.py	2010-02-23 07:43:11 +0000
+++ bzrlib/plugins/launchpad/test_lp_directory.py	2010-07-26 17:19:33 +0000
@@ -22,6 +22,7 @@
 from bzrlib import (
     errors,
     tests,
+    transport,
     )
 from bzrlib.branch import Branch
 from bzrlib.directory_service import directories
@@ -29,14 +30,13 @@
     TestCaseInTempDir,
     TestCaseWithMemoryTransport
 )
-from bzrlib.transport import get_transport
 from bzrlib.plugins.launchpad import (
     _register_directory,
     lp_registration,
     )
 from bzrlib.plugins.launchpad.lp_directory import (
     LaunchpadDirectory)
-from bzrlib.plugins.launchpad.account import get_lp_login
+from bzrlib.plugins.launchpad.account import get_lp_login, set_lp_login
 from bzrlib.tests import (
     http_server,
     http_utils,
@@ -91,7 +91,7 @@
         self.assertEquals('http://bazaar.launchpad.net/~apt/apt/devel',
                           directory._resolve('lp:apt', factory))
         # Make sure that resolve went to the production server.
-        self.assertEquals('https://xmlrpc.edge.launchpad.net/bazaar/',
+        self.assertEquals('https://xmlrpc.launchpad.net/bazaar/',
                           factory._service_url)
 
     def test_staging(self):
@@ -199,6 +199,29 @@
         self.assertRaises(errors.InvalidURL,
             directory._resolve, 'lp://ratotehunoahu')
 
+    def test_resolve_tilde_to_user(self):
+        factory = FakeResolveFactory(
+            self, '~username/apt/test', dict(urls=[
+                    'bzr+ssh://bazaar.launchpad.net/~username/apt/test']))
+        directory = LaunchpadDirectory()
+        self.assertEquals(
+            'bzr+ssh://bazaar.launchpad.net/~username/apt/test',
+            directory._resolve('lp:~/apt/test', factory, _lp_login='username'))
+        # Should also happen when the login is just set by config
+        set_lp_login('username')
+        self.assertEquals(
+            'bzr+ssh://bazaar.launchpad.net/~username/apt/test',
+            directory._resolve('lp:~/apt/test', factory))
+
+    def test_tilde_fails_no_login(self):
+        factory = FakeResolveFactory(
+            self, '~username/apt/test', dict(urls=[
+                    'bzr+ssh://bazaar.launchpad.net/~username/apt/test']))
+        self.assertIs(None, get_lp_login())
+        directory = LaunchpadDirectory()
+        e = self.assertRaises(errors.InvalidURL,
+            directory._resolve, 'lp:~/apt/test', factory)
+
 
 class DirectoryOpenBranchTests(TestCaseWithMemoryTransport):
 
@@ -217,8 +240,8 @@
         directories.register('lp:', FooService, 'Map lp URLs to local urls')
         self.addCleanup(_register_directory)
         self.addCleanup(directories.remove, 'lp:')
-        transport = get_transport('lp:///apt')
-        branch = Branch.open_from_transport(transport)
+        t = transport.get_transport('lp:///apt')
+        branch = Branch.open_from_transport(t)
         self.assertEqual(target_branch.base, branch.base)
 
 

=== modified file 'bzrlib/plugins/launchpad/test_lp_open.py'
--- bzrlib/plugins/launchpad/test_lp_open.py	2010-02-17 17:11:16 +0000
+++ bzrlib/plugins/launchpad/test_lp_open.py	2010-05-20 18:23:10 +0000
@@ -54,7 +54,7 @@
         branch.set_public_branch(
             'bzr+ssh://bazaar.launchpad.net/~foo/bar/baz')
         self.assertEqual(
-            ['Opening https://code.edge.launchpad.net/~foo/bar/baz in web '
+            ['Opening https://code.launchpad.net/~foo/bar/baz in web '
              'browser'],
             self.run_open('lp'))
 
@@ -65,7 +65,7 @@
         branch.set_push_location(
             'bzr+ssh://bazaar.launchpad.net/~foo/bar/push')
         self.assertEqual(
-            ['Opening https://code.edge.launchpad.net/~foo/bar/public in web '
+            ['Opening https://code.launchpad.net/~foo/bar/public in web '
              'browser'],
             self.run_open('lp'))
 
@@ -76,7 +76,7 @@
         branch.set_push_location(
             'bzr+ssh://bazaar.launchpad.net/~foo/bar/baz')
         self.assertEqual(
-            ['Opening https://code.edge.launchpad.net/~foo/bar/baz in web '
+            ['Opening https://code.launchpad.net/~foo/bar/baz in web '
              'browser'],
             self.run_open('lp'))
 
@@ -85,7 +85,7 @@
         # location and no push location, then just try to look up the
         # Launchpad page for that URL.
         self.assertEqual(
-            ['Opening https://code.edge.launchpad.net/~foo/bar/baz in web '
+            ['Opening https://code.launchpad.net/~foo/bar/baz in web '
              'browser'],
             self.run_open('bzr+ssh://bazaar.launchpad.net/~foo/bar/baz'))
 
@@ -96,6 +96,6 @@
             'bzr+ssh://bazaar.launchpad.net/~foo/bar/baz')
         self.build_tree(['lp/a/'])
         self.assertEqual(
-            ['Opening https://code.edge.launchpad.net/~foo/bar/baz in web '
+            ['Opening https://code.launchpad.net/~foo/bar/baz in web '
              'browser'],
             self.run_open('.', working_dir='lp/a'))

=== modified file 'bzrlib/plugins/launchpad/test_lp_service.py'
--- bzrlib/plugins/launchpad/test_lp_service.py	2009-03-23 14:59:43 +0000
+++ bzrlib/plugins/launchpad/test_lp_service.py	2010-05-20 18:23:10 +0000
@@ -97,7 +97,7 @@
         web_url = service.get_web_url_from_branch_url(
             'bzr+ssh://bazaar.launchpad.net/~foo/bar/baz')
         self.assertEqual(
-            'https://code.edge.launchpad.net/~foo/bar/baz', web_url)
+            'https://code.launchpad.net/~foo/bar/baz', web_url)
 
     def test_product_bzr_ssh_url(self):
         service = LaunchpadService(lp_instance='production')
@@ -152,7 +152,7 @@
             dict(urls=['http://bazaar.launchpad.net/~foo/bar/baz']))
         web_url = service.get_web_url_from_branch_url('lp:foo', factory)
         self.assertEqual(
-            'https://code.edge.launchpad.net/~foo/bar/baz', web_url)
+            'https://code.launchpad.net/~foo/bar/baz', web_url)
 
     def test_lp_branch_fault(self):
         service = LaunchpadService()

=== modified file 'bzrlib/plugins/launchpad/test_register.py'
--- bzrlib/plugins/launchpad/test_register.py	2009-06-10 03:56:49 +0000
+++ bzrlib/plugins/launchpad/test_register.py	2010-05-20 18:23:10 +0000
@@ -188,7 +188,7 @@
                 'author@launchpad.net',
                 'product')
         rego.submit(service)
-        self.assertEquals(transport.connected_host, 'xmlrpc.edge.launchpad.net')
+        self.assertEquals(transport.connected_host, 'xmlrpc.launchpad.net')
         self.assertEquals(len(transport.sent_params), 6)
         self.assertEquals(transport.sent_params,
                 ('http://test-server.com/bzr/branch',  # branch_url
@@ -205,7 +205,7 @@
         service = LaunchpadService(transport)
         resolve = ResolveLaunchpadPathRequest('bzr')
         resolve.submit(service)
-        self.assertEquals(transport.connected_host, 'xmlrpc.edge.launchpad.net')
+        self.assertEquals(transport.connected_host, 'xmlrpc.launchpad.net')
         self.assertEquals(len(transport.sent_params), 1)
         self.assertEquals(transport.sent_params, ('bzr', ))
         self.assertTrue(transport.got_request)

=== modified file 'bzrlib/progress.py'
--- bzrlib/progress.py	2010-04-30 11:03:59 +0000
+++ bzrlib/progress.py	2010-09-14 04:13:48 +0000
@@ -152,19 +152,6 @@
                 own_fraction = 0.0
             return self._parent_task._overall_completion_fraction(own_fraction)
 
-    @deprecated_method(deprecated_in((2, 1, 0)))
-    def note(self, fmt_string, *args):
-        """Record a note without disrupting the progress bar.
-        
-        Deprecated: use ui_factory.note() instead or bzrlib.trace.  Note that
-        ui_factory.note takes just one string as the argument, not a format
-        string and arguments.
-        """
-        if args:
-            self.ui_factory.note(fmt_string % args)
-        else:
-            self.ui_factory.note(fmt_string)
-
     def clear(self):
         # TODO: deprecate this method; the model object shouldn't be concerned
         # with whether it's shown or not.  Most callers use this because they
@@ -179,54 +166,6 @@
             self.ui_factory.clear_term()
 
 
-# NOTE: This is also deprecated; you should provide a ProgressView instead.
-class _BaseProgressBar(object):
-
-    def __init__(self,
-                 to_file=None,
-                 show_pct=False,
-                 show_spinner=False,
-                 show_eta=False,
-                 show_bar=True,
-                 show_count=True,
-                 to_messages_file=None,
-                 _stack=None):
-        object.__init__(self)
-        if to_file is None:
-            to_file = sys.stderr
-        if to_messages_file is None:
-            to_messages_file = sys.stdout
-        self.to_file = to_file
-        self.to_messages_file = to_messages_file
-        self.last_msg = None
-        self.last_cnt = None
-        self.last_total = None
-        self.show_pct = show_pct
-        self.show_spinner = show_spinner
-        self.show_eta = show_eta
-        self.show_bar = show_bar
-        self.show_count = show_count
-        self._stack = _stack
-        # seed throttler
-        self.MIN_PAUSE = 0.1 # seconds
-        now = time.time()
-        # starting now
-        self.start_time = now
-        # next update should not throttle
-        self.last_update = now - self.MIN_PAUSE - 1
-
-    def finished(self):
-        """Return this bar to its progress stack."""
-        self.clear()
-        self._stack.return_pb(self)
-
-    def note(self, fmt_string, *args, **kwargs):
-        """Record a note without disrupting the progress bar."""
-        self.clear()
-        self.to_messages_file.write(fmt_string % args)
-        self.to_messages_file.write('\n')
-
-
 class DummyProgress(object):
     """Progress-bar standin that does nothing.
 
@@ -248,9 +187,6 @@
     def clear(self):
         pass
 
-    def note(self, fmt_string, *args, **kwargs):
-        """See _BaseProgressBar.note()."""
-
     def child_progress(self, **kwargs):
         return DummyProgress(**kwargs)
 

=== modified file 'bzrlib/python-compat.h'
--- bzrlib/python-compat.h	2009-10-21 20:53:21 +0000
+++ bzrlib/python-compat.h	2010-08-23 21:36:06 +0000
@@ -74,6 +74,9 @@
 
 #ifdef _MSC_VER
 #define  snprintf  _snprintf
+/* gcc (mingw32) has strtoll, while the MSVC compiler uses _strtoi64 */
+#define strtoll _strtoi64
+#define strtoull _strtoui64
 #endif
 
 /* Introduced in Python 2.6 */

=== modified file 'bzrlib/reconcile.py'
--- bzrlib/reconcile.py	2010-04-30 11:35:43 +0000
+++ bzrlib/reconcile.py	2010-08-12 04:19:07 +0000
@@ -36,7 +36,7 @@
 from bzrlib.versionedfile import AdapterFactory, FulltextContentFactory
 
 
-def reconcile(dir, other=None):
+def reconcile(dir, canonicalize_chks=False):
     """Reconcile the data in dir.
 
     Currently this is limited to a inventory 'reweave'.
@@ -46,18 +46,19 @@
     Directly using Reconciler is recommended for library users that
     desire fine grained control or analysis of the found issues.
 
-    :param other: another bzrdir to reconcile against.
+    :param canonicalize_chks: Make sure CHKs are in canonical form.
     """
-    reconciler = Reconciler(dir, other=other)
+    reconciler = Reconciler(dir, canonicalize_chks=canonicalize_chks)
     reconciler.reconcile()
 
 
 class Reconciler(object):
     """Reconcilers are used to reconcile existing data."""
 
-    def __init__(self, dir, other=None):
+    def __init__(self, dir, other=None, canonicalize_chks=False):
         """Create a Reconciler."""
         self.bzrdir = dir
+        self.canonicalize_chks = canonicalize_chks
 
     def reconcile(self):
         """Perform reconciliation.
@@ -98,7 +99,15 @@
         ui.ui_factory.note('Reconciling repository %s' %
             self.repo.user_url)
         self.pb.update("Reconciling repository", 0, 1)
-        repo_reconciler = self.repo.reconcile(thorough=True)
+        if self.canonicalize_chks:
+            try:
+                self.repo.reconcile_canonicalize_chks
+            except AttributeError:
+                raise errors.BzrError(
+                    "%s cannot canonicalize CHKs." % (self.repo,))
+            repo_reconciler = self.repo.reconcile_canonicalize_chks()
+        else:
+            repo_reconciler = self.repo.reconcile(thorough=True)
         self.inconsistent_parents = repo_reconciler.inconsistent_parents
         self.garbage_inventories = repo_reconciler.garbage_inventories
         if repo_reconciler.aborted:
@@ -494,7 +503,13 @@
     #  - lock the names list
     #  - perform a customised pack() that regenerates data as needed
     #  - unlock the names list
-    # https://bugs.edge.launchpad.net/bzr/+bug/154173
+    # https://bugs.launchpad.net/bzr/+bug/154173
+
+    def __init__(self, repo, other=None, thorough=False,
+            canonicalize_chks=False):
+        super(PackReconciler, self).__init__(repo, other=other,
+            thorough=thorough)
+        self.canonicalize_chks = canonicalize_chks
 
     def _reconcile_steps(self):
         """Perform the steps to reconcile this repository."""
@@ -509,8 +524,12 @@
         total_inventories = len(list(
             collection.inventory_index.combined_index.iter_all_entries()))
         if len(all_revisions):
-            new_pack =  self.repo._reconcile_pack(collection, packs,
-                ".reconcile", all_revisions, self.pb)
+            if self.canonicalize_chks:
+                reconcile_meth = self.repo._canonicalize_chks_pack
+            else:
+                reconcile_meth = self.repo._reconcile_pack
+            new_pack = reconcile_meth(collection, packs, ".reconcile",
+                all_revisions, self.pb)
             if new_pack is not None:
                 self._discard_and_save(packs)
         else:

=== added file 'bzrlib/recordcounter.py'
--- bzrlib/recordcounter.py	1970-01-01 00:00:00 +0000
+++ bzrlib/recordcounter.py	2010-06-12 02:58:42 +0000
@@ -0,0 +1,86 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+"""Record counting support for showing progress of revision fetch."""
+
+
+class RecordCounter(object):
+    """Container for maintains estimates of work requires for fetch.
+
+    Instance of this class is used along with a progress bar to provide
+    the user an estimate of the amount of work pending for a fetch (push,
+    pull, branch, checkout) operation.
+    """
+    def __init__(self):
+        self.initialized = False
+        self.current = 0
+        self.key_count = 0
+        self.max = 0
+
+        # Users of RecordCounter instance update progress bar every
+        # _STEP_ records. We choose are reasonably high number to keep
+        # display updates from being too frequent. This is an odd number
+        # to ensure that the last digit of the records fetched in
+        # fetches vs estimate ratio changes periodically.
+        self.STEP = 7
+
+    def is_initialized(self):
+        return self.initialized
+
+    def _estimate_max(self, key_count):
+        """Estimate the maximum amount of 'inserting stream' work.
+
+        This is just an estimate.
+        """
+        # Note: The magic number below is based of empirical data
+        # based on 3 seperate projects. Estimatation can probably
+        # be improved but this should work well for most cases.
+        # The project used for the estimate (with approx. numbers) were:
+        # lp:bzr with records_fetched = 7 * revs_required
+        # lp:emacs with records_fetched = 8 * revs_required
+        # bzr-svn checkout of lp:parrot = 10.63 * revs_required
+        # Hence, 10.3 was chosen as for a realistic progress bar as:
+        # 1. If records fetched is is lower than 10.3x then we simply complete
+        #    with 10.3x. Under promise, over deliver.
+        # 2. In case of remote fetch, when we start the count fetch vs estimate
+        #    display with revs_required/estimate, having a multiplier with a
+        #    decimal point produces a realistic looking _estimate_ number rather
+        #    than using something like 3125/31250 (for 10x)
+        # 3. Based on the above data, the possibility of overshooting this
+        #    factor is minimal, and in case of an overshoot the estimate value
+        #    should not need to be corrected too many times.
+        return int(key_count * 10.3)
+
+    def setup(self, key_count, current=0):
+        """Setup RecordCounter with basic estimate of work pending.
+
+        Setup self.max and self.current to reflect the amount of work
+        pending for a fetch.
+        """
+        self.current = current
+        self.key_count = key_count
+        self.max = self._estimate_max(key_count)
+        self.initialized = True
+
+    def increment(self, count):
+        """Increment self.current by count.
+
+        Apart from incrementing self.current by count, also ensure
+        that self.max > self.current.
+        """
+        self.current += count
+        if self.current > self.max:
+            self.max += self.key_count
+

=== modified file 'bzrlib/remote.py'
--- bzrlib/remote.py	2010-05-03 04:08:50 +0000
+++ bzrlib/remote.py	2010-09-26 02:27:37 +0000
@@ -21,6 +21,7 @@
     branch,
     bzrdir,
     config,
+    controldir,
     debug,
     errors,
     graph,
@@ -33,7 +34,7 @@
     static_tuple,
     symbol_versioning,
 )
-from bzrlib.branch import BranchReferenceFormat
+from bzrlib.branch import BranchReferenceFormat, BranchWriteLockResult
 from bzrlib.bzrdir import BzrDir, RemoteBzrDirFormat
 from bzrlib.decorators import needs_read_lock, needs_write_lock, only_raises
 from bzrlib.errors import (
@@ -43,6 +44,7 @@
 from bzrlib.lockable_files import LockableFiles
 from bzrlib.smart import client, vfs, repository as smart_repo
 from bzrlib.revision import ensure_null, NULL_REVISION
+from bzrlib.repository import RepositoryWriteLockResult
 from bzrlib.trace import mutter, note, warning
 
 
@@ -212,7 +214,7 @@
         if len(branch_info) != 2:
             raise errors.UnexpectedSmartServerResponse(response)
         branch_ref, branch_name = branch_info
-        format = bzrdir.network_format_registry.get(control_name)
+        format = controldir.network_format_registry.get(control_name)
         if repo_name:
             format.repository_format = repository.network_format_registry.get(
                 repo_name)
@@ -269,19 +271,23 @@
         self._real_bzrdir.destroy_branch(name=name)
         self._next_open_branch_result = None
 
-    def create_workingtree(self, revision_id=None, from_branch=None):
+    def create_workingtree(self, revision_id=None, from_branch=None,
+        accelerator_tree=None, hardlink=False):
         raise errors.NotLocalUrl(self.transport.base)
 
-    def find_branch_format(self):
+    def find_branch_format(self, name=None):
         """Find the branch 'format' for this bzrdir.
 
         This might be a synthetic object for e.g. RemoteBranch and SVN.
         """
-        b = self.open_branch()
+        b = self.open_branch(name=name)
         return b._format
 
-    def get_branch_reference(self):
+    def get_branch_reference(self, name=None):
         """See BzrDir.get_branch_reference()."""
+        if name is not None:
+            # XXX JRV20100304: Support opening colocated branches
+            raise errors.NoColocatedBranchSupport(self)
         response = self._get_branch_reference()
         if response[0] == 'ref':
             return response[1]
@@ -318,9 +324,9 @@
             raise errors.UnexpectedSmartServerResponse(response)
         return response
 
-    def _get_tree_branch(self):
+    def _get_tree_branch(self, name=None):
         """See BzrDir._get_tree_branch()."""
-        return None, self.open_branch()
+        return None, self.open_branch(name=name)
 
     def open_branch(self, name=None, unsupported=False,
                     ignore_fallbacks=False):
@@ -644,7 +650,7 @@
 
 
 class RemoteRepository(_RpcHelper, lock._RelockDebugMixin,
-    bzrdir.ControlComponent):
+    controldir.ControlComponent):
     """Repository accessed over rpc.
 
     For the moment most operations are performed using local transport-backed
@@ -895,7 +901,7 @@
     def _has_same_fallbacks(self, other_repo):
         """Returns true if the repositories have the same fallbacks."""
         # XXX: copied from Repository; it should be unified into a base class
-        # <https://bugs.edge.launchpad.net/bzr/+bug/401622>
+        # <https://bugs.launchpad.net/bzr/+bug/401622>
         my_fb = self._fallback_repositories
         other_fb = other_repo._fallback_repositories
         if len(my_fb) != len(other_fb):
@@ -997,6 +1003,10 @@
         pass
 
     def lock_read(self):
+        """Lock the repository for read operations.
+
+        :return: A bzrlib.lock.LogicalLockResult.
+        """
         # wrong eventually - want a local lock cache context
         if not self._lock_mode:
             self._note_lock('r')
@@ -1009,6 +1019,7 @@
                 repo.lock_read()
         else:
             self._lock_count += 1
+        return lock.LogicalLockResult(self.unlock)
 
     def _remote_lock_write(self, token):
         path = self.bzrdir._path_for_remote_call(self._client)
@@ -1054,7 +1065,7 @@
             raise errors.ReadOnlyError(self)
         else:
             self._lock_count += 1
-        return self._lock_token or None
+        return RepositoryWriteLockResult(self.unlock, self._lock_token or None)
 
     def leave_lock_in_place(self):
         if not self._lock_token:
@@ -1306,16 +1317,16 @@
         return self._real_repository.make_working_trees()
 
     def refresh_data(self):
-        """Re-read any data needed to to synchronise with disk.
+        """Re-read any data needed to synchronise with disk.
 
         This method is intended to be called after another repository instance
         (such as one used by a smart server) has inserted data into the
-        repository. It may not be called during a write group, but may be
-        called at any other time.
+        repository. On all repositories this will work outside of write groups.
+        Some repository formats (pack and newer for bzrlib native formats)
+        support refresh_data inside write groups. If called inside a write
+        group on a repository that does not support refreshing in a write group
+        IsInWriteGroupError will be raised.
         """
-        if self.is_in_write_group():
-            raise errors.InternalBzrError(
-                "May not refresh_data while in a write group.")
         if self._real_repository is not None:
             self._real_repository.refresh_data()
 
@@ -1971,7 +1982,8 @@
         if response_tuple[0] != 'ok':
             raise errors.UnexpectedSmartServerResponse(response_tuple)
         byte_stream = response_handler.read_streamed_body()
-        src_format, stream = smart_repo._byte_stream_to_stream(byte_stream)
+        src_format, stream = smart_repo._byte_stream_to_stream(byte_stream,
+            self._record_counter)
         if src_format.network_name() != repo._format.network_name():
             raise AssertionError(
                 "Mismatched RemoteRepository and stream src %r, %r" % (
@@ -2387,6 +2399,10 @@
             self._vfs_set_tags_bytes(bytes)
 
     def lock_read(self):
+        """Lock the branch for read operations.
+
+        :return: A bzrlib.lock.LogicalLockResult.
+        """
         self.repository.lock_read()
         if not self._lock_mode:
             self._note_lock('r')
@@ -2396,18 +2412,26 @@
                 self._real_branch.lock_read()
         else:
             self._lock_count += 1
+        return lock.LogicalLockResult(self.unlock)
 
     def _remote_lock_write(self, token):
         if token is None:
             branch_token = repo_token = ''
         else:
             branch_token = token
-            repo_token = self.repository.lock_write()
+            repo_token = self.repository.lock_write().repository_token
             self.repository.unlock()
         err_context = {'token': token}
-        response = self._call(
-            'Branch.lock_write', self._remote_path(), branch_token,
-            repo_token or '', **err_context)
+        try:
+            response = self._call(
+                'Branch.lock_write', self._remote_path(), branch_token,
+                repo_token or '', **err_context)
+        except errors.LockContention, e:
+            # The LockContention from the server doesn't have any
+            # information about the lock_url. We re-raise LockContention
+            # with valid lock_url.
+            raise errors.LockContention('(remote lock)',
+                self.repository.base.split('.bzr/')[0])
         if response[0] != 'ok':
             raise errors.UnexpectedSmartServerResponse(response)
         ok, branch_token, repo_token = response
@@ -2434,7 +2458,7 @@
             self._lock_mode = 'w'
             self._lock_count = 1
         elif self._lock_mode == 'r':
-            raise errors.ReadOnlyTransaction
+            raise errors.ReadOnlyError(self)
         else:
             if token is not None:
                 # A token was given to lock_write, and we're relocking, so
@@ -2445,7 +2469,7 @@
             self._lock_count += 1
             # Re-lock the repository too.
             self.repository.lock_write(self._repo_lock_token)
-        return self._lock_token or None
+        return BranchWriteLockResult(self.unlock, self._lock_token or None)
 
     def _unlock(self, branch_token, repo_token):
         err_context = {'token': str((branch_token, repo_token))}
@@ -2774,17 +2798,51 @@
         medium = self._branch._client._medium
         if medium._is_remote_before((1, 14)):
             return self._vfs_set_option(value, name, section)
+        if isinstance(value, dict):
+            if medium._is_remote_before((2, 2)):
+                return self._vfs_set_option(value, name, section)
+            return self._set_config_option_dict(value, name, section)
+        else:
+            return self._set_config_option(value, name, section)
+
+    def _set_config_option(self, value, name, section):
         try:
             path = self._branch._remote_path()
             response = self._branch._client.call('Branch.set_config_option',
                 path, self._branch._lock_token, self._branch._repo_lock_token,
                 value.encode('utf8'), name, section or '')
         except errors.UnknownSmartMethod:
+            medium = self._branch._client._medium
             medium._remember_remote_is_before((1, 14))
             return self._vfs_set_option(value, name, section)
         if response != ():
             raise errors.UnexpectedSmartServerResponse(response)
 
+    def _serialize_option_dict(self, option_dict):
+        utf8_dict = {}
+        for key, value in option_dict.items():
+            if isinstance(key, unicode):
+                key = key.encode('utf8')
+            if isinstance(value, unicode):
+                value = value.encode('utf8')
+            utf8_dict[key] = value
+        return bencode.bencode(utf8_dict)
+
+    def _set_config_option_dict(self, value, name, section):
+        try:
+            path = self._branch._remote_path()
+            serialised_dict = self._serialize_option_dict(value)
+            response = self._branch._client.call(
+                'Branch.set_config_option_dict',
+                path, self._branch._lock_token, self._branch._repo_lock_token,
+                serialised_dict, name, section or '')
+        except errors.UnknownSmartMethod:
+            medium = self._branch._client._medium
+            medium._remember_remote_is_before((2, 2))
+            return self._vfs_set_option(value, name, section)
+        if response != ():
+            raise errors.UnexpectedSmartServerResponse(response)
+
     def _real_object(self):
         self._branch._ensure_real()
         return self._branch._real_branch

=== modified file 'bzrlib/repofmt/groupcompress_repo.py'
--- bzrlib/repofmt/groupcompress_repo.py	2010-02-10 02:17:15 +0000
+++ bzrlib/repofmt/groupcompress_repo.py	2010-09-01 11:27:40 +0000
@@ -32,11 +32,13 @@
     revision as _mod_revision,
     trace,
     ui,
+    versionedfile,
     )
 from bzrlib.btree_index import (
     BTreeGraphIndex,
     BTreeBuilder,
     )
+from bzrlib.decorators import needs_write_lock
 from bzrlib.groupcompress import (
     _GCGraphIndex,
     GroupCompressVersionedFiles,
@@ -262,7 +264,6 @@
         remaining_keys = set(keys)
         counter = [0]
         if self._gather_text_refs:
-            bytes_to_info = inventory.CHKInventory._bytes_to_utf8name_key
             self._text_refs = set()
         def _get_referenced_stream(root_keys, parse_leaf_nodes=False):
             cur_keys = root_keys
@@ -289,8 +290,7 @@
                     # Store is None, because we know we have a LeafNode, and we
                     # just want its entries
                     for file_id, bytes in node.iteritems(None):
-                        name_utf8, file_id, revision_id = bytes_to_info(bytes)
-                        self._text_refs.add((file_id, revision_id))
+                        self._text_refs.add(chk_map._bytes_to_text_key(bytes))
                 def next_stream():
                     stream = source_vf.get_record_stream(cur_keys,
                                                          'as-requested', True)
@@ -427,8 +427,11 @@
         self._copy_stream(source_vf, target_vf, inventory_keys,
                           'inventories', self._get_filtered_inv_stream, 2)
 
+    def _get_chk_vfs_for_copy(self):
+        return self._build_vfs('chk', False, False)
+
     def _copy_chk_texts(self):
-        source_vf, target_vf = self._build_vfs('chk', False, False)
+        source_vf, target_vf = self._get_chk_vfs_for_copy()
         # TODO: This is technically spurious... if it is a performance issue,
         #       remove it
         total_keys = source_vf.keys()
@@ -580,6 +583,111 @@
         return new_pack.data_inserted() and self._data_changed
 
 
+class GCCHKCanonicalizingPacker(GCCHKPacker):
+    """A packer that ensures inventories have canonical-form CHK maps.
+    
+    Ideally this would be part of reconcile, but it's very slow and rarely
+    needed.  (It repairs repositories affected by
+    https://bugs.launchpad.net/bzr/+bug/522637).
+    """
+
+    def __init__(self, *args, **kwargs):
+        super(GCCHKCanonicalizingPacker, self).__init__(*args, **kwargs)
+        self._data_changed = False
+    
+    def _exhaust_stream(self, source_vf, keys, message, vf_to_stream, pb_offset):
+        """Create and exhaust a stream, but don't insert it.
+        
+        This is useful to get the side-effects of generating a stream.
+        """
+        self.pb.update('scanning %s' % (message,), pb_offset)
+        child_pb = ui.ui_factory.nested_progress_bar()
+        try:
+            list(vf_to_stream(source_vf, keys, message, child_pb))
+        finally:
+            child_pb.finished()
+
+    def _copy_inventory_texts(self):
+        source_vf, target_vf = self._build_vfs('inventory', True, True)
+        source_chk_vf, target_chk_vf = self._get_chk_vfs_for_copy()
+        inventory_keys = source_vf.keys()
+        # First, copy the existing CHKs on the assumption that most of them
+        # will be correct.  This will save us from having to reinsert (and
+        # recompress) these records later at the cost of perhaps preserving a
+        # few unused CHKs. 
+        # (Iterate but don't insert _get_filtered_inv_stream to populate the
+        # variables needed by GCCHKPacker._copy_chk_texts.)
+        self._exhaust_stream(source_vf, inventory_keys, 'inventories',
+                self._get_filtered_inv_stream, 2)
+        GCCHKPacker._copy_chk_texts(self)
+        # Now copy and fix the inventories, and any regenerated CHKs.
+        def chk_canonicalizing_inv_stream(source_vf, keys, message, pb=None):
+            return self._get_filtered_canonicalizing_inv_stream(
+                source_vf, keys, message, pb, source_chk_vf, target_chk_vf)
+        self._copy_stream(source_vf, target_vf, inventory_keys,
+                          'inventories', chk_canonicalizing_inv_stream, 4)
+
+    def _copy_chk_texts(self):
+        # No-op; in this class this happens during _copy_inventory_texts.
+        pass
+
+    def _get_filtered_canonicalizing_inv_stream(self, source_vf, keys, message,
+            pb=None, source_chk_vf=None, target_chk_vf=None):
+        """Filter the texts of inventories, regenerating CHKs to make sure they
+        are canonical.
+        """
+        total_keys = len(keys)
+        target_chk_vf = versionedfile.NoDupeAddLinesDecorator(target_chk_vf)
+        def _filtered_inv_stream():
+            stream = source_vf.get_record_stream(keys, 'groupcompress', True)
+            search_key_name = None
+            for idx, record in enumerate(stream):
+                # Inventories should always be with revisions; assume success.
+                bytes = record.get_bytes_as('fulltext')
+                chk_inv = inventory.CHKInventory.deserialise(
+                    source_chk_vf, bytes, record.key)
+                if pb is not None:
+                    pb.update('inv', idx, total_keys)
+                chk_inv.id_to_entry._ensure_root()
+                if search_key_name is None:
+                    # Find the name corresponding to the search_key_func
+                    search_key_reg = chk_map.search_key_registry
+                    for search_key_name, func in search_key_reg.iteritems():
+                        if func == chk_inv.id_to_entry._search_key_func:
+                            break
+                canonical_inv = inventory.CHKInventory.from_inventory(
+                    target_chk_vf, chk_inv,
+                    maximum_size=chk_inv.id_to_entry._root_node._maximum_size,
+                    search_key_name=search_key_name)
+                if chk_inv.id_to_entry.key() != canonical_inv.id_to_entry.key():
+                    trace.mutter(
+                        'Non-canonical CHK map for id_to_entry of inv: %s '
+                        '(root is %s, should be %s)' % (chk_inv.revision_id,
+                        chk_inv.id_to_entry.key()[0],
+                        canonical_inv.id_to_entry.key()[0]))
+                    self._data_changed = True
+                p_id_map = chk_inv.parent_id_basename_to_file_id
+                p_id_map._ensure_root()
+                canon_p_id_map = canonical_inv.parent_id_basename_to_file_id
+                if p_id_map.key() != canon_p_id_map.key():
+                    trace.mutter(
+                        'Non-canonical CHK map for parent_id_to_basename of '
+                        'inv: %s (root is %s, should be %s)'
+                        % (chk_inv.revision_id, p_id_map.key()[0],
+                           canon_p_id_map.key()[0]))
+                    self._data_changed = True
+                yield versionedfile.ChunkedContentFactory(record.key,
+                        record.parents, record.sha1,
+                        canonical_inv.to_lines())
+            # We have finished processing all of the inventory records, we
+            # don't need these sets anymore
+        return _filtered_inv_stream()
+
+    def _use_pack(self, new_pack):
+        """Override _use_pack to check for reconcile having changed content."""
+        return new_pack.data_inserted() and self._data_changed
+
+
 class GCRepositoryPackCollection(RepositoryPackCollection):
 
     pack_factory = GCPack
@@ -647,10 +755,10 @@
         chk_diff = chk_map.iter_interesting_nodes(
             chk_bytes_no_fallbacks, root_key_info.interesting_root_keys,
             root_key_info.uninteresting_root_keys)
-        bytes_to_info = inventory.CHKInventory._bytes_to_utf8name_key
         text_keys = set()
         try:
-            for record in _filter_text_keys(chk_diff, text_keys, bytes_to_info):
+            for record in _filter_text_keys(chk_diff, text_keys,
+                                            chk_map._bytes_to_text_key):
                 pass
         except errors.NoSuchRevision, e:
             # XXX: It would be nice if we could give a more precise error here.
@@ -864,6 +972,8 @@
         if basis_inv is None:
             if basis_revision_id == _mod_revision.NULL_REVISION:
                 new_inv = self._create_inv_from_null(delta, new_revision_id)
+                if new_inv.root_id is None:
+                    raise errors.RootMissing()
                 inv_lines = new_inv.to_lines()
                 return self._inventory_add_lines(new_revision_id, parents,
                     inv_lines, check_content=False), new_inv
@@ -999,10 +1109,24 @@
         finally:
             pb.finished()
 
+    @needs_write_lock
+    def reconcile_canonicalize_chks(self):
+        """Reconcile this repository to make sure all CHKs are in canonical
+        form.
+        """
+        from bzrlib.reconcile import PackReconciler
+        reconciler = PackReconciler(self, thorough=True, canonicalize_chks=True)
+        reconciler.reconcile()
+        return reconciler
+
     def _reconcile_pack(self, collection, packs, extension, revs, pb):
         packer = GCCHKReconcilePacker(collection, packs, extension)
         return packer.pack(pb)
 
+    def _canonicalize_chks_pack(self, collection, packs, extension, revs, pb):
+        packer = GCCHKCanonicalizingPacker(collection, packs, extension, revs)
+        return packer.pack(pb)
+
     def _get_source(self, to_format):
         """Return a source for streaming from this repository."""
         if self._format._serializer == to_format._serializer:
@@ -1087,13 +1211,12 @@
                 uninteresting_root_keys.add(inv.id_to_entry.key())
                 uninteresting_pid_root_keys.add(
                     inv.parent_id_basename_to_file_id.key())
-        bytes_to_info = inventory.CHKInventory._bytes_to_utf8name_key
         chk_bytes = self.from_repository.chk_bytes
         def _filter_id_to_entry():
             interesting_nodes = chk_map.iter_interesting_nodes(chk_bytes,
                         self._chk_id_roots, uninteresting_root_keys)
             for record in _filter_text_keys(interesting_nodes, self._text_keys,
-                    bytes_to_info):
+                    chk_map._bytes_to_text_key):
                 if record is not None:
                     yield record
             # Consumed
@@ -1109,13 +1232,29 @@
         yield 'chk_bytes', _get_parent_id_basename_to_file_id_pages()
 
     def get_stream(self, search):
+        def wrap_and_count(pb, rc, stream):
+            """Yield records from stream while showing progress."""
+            count = 0
+            for record in stream:
+                if count == rc.STEP:
+                    rc.increment(count)
+                    pb.update('Estimate', rc.current, rc.max)
+                    count = 0
+                count += 1
+                yield record
+
         revision_ids = search.get_keys()
+        pb = ui.ui_factory.nested_progress_bar()
+        rc = self._record_counter
+        self._record_counter.setup(len(revision_ids))
         for stream_info in self._fetch_revision_texts(revision_ids):
-            yield stream_info
+            yield (stream_info[0],
+                wrap_and_count(pb, rc, stream_info[1]))
         self._revision_keys = [(rev_id,) for rev_id in revision_ids]
         self.from_repository.revisions.clear_cache()
         self.from_repository.signatures.clear_cache()
-        yield self._get_inventory_stream(self._revision_keys)
+        s = self._get_inventory_stream(self._revision_keys)
+        yield (s[0], wrap_and_count(pb, rc, s[1]))
         self.from_repository.inventories.clear_cache()
         # TODO: The keys to exclude might be part of the search recipe
         # For now, exclude all parents that are at the edge of ancestry, for
@@ -1124,10 +1263,13 @@
         parent_keys = from_repo._find_parent_keys_of_revisions(
                         self._revision_keys)
         for stream_info in self._get_filtered_chk_streams(parent_keys):
-            yield stream_info
+            yield (stream_info[0], wrap_and_count(pb, rc, stream_info[1]))
         self.from_repository.chk_bytes.clear_cache()
-        yield self._get_text_stream()
+        s = self._get_text_stream()
+        yield (s[0], wrap_and_count(pb, rc, s[1]))
         self.from_repository.texts.clear_cache()
+        pb.update('Done', rc.max, rc.max)
+        pb.finished()
 
     def get_stream_for_missing_keys(self, missing_keys):
         # missing keys can only occur when we are byte copying and not
@@ -1187,19 +1329,13 @@
     return result
 
 
-def _filter_text_keys(interesting_nodes_iterable, text_keys, bytes_to_info):
+def _filter_text_keys(interesting_nodes_iterable, text_keys, bytes_to_text_key):
     """Iterate the result of iter_interesting_nodes, yielding the records
     and adding to text_keys.
     """
+    text_keys_update = text_keys.update
     for record, items in interesting_nodes_iterable:
-        for name, bytes in items:
-            # Note: we don't care about name_utf8, because groupcompress repos
-            # are always rich-root, so there are no synthesised root records to
-            # ignore.
-            _, file_id, revision_id = bytes_to_info(bytes)
-            file_id = intern(file_id)
-            revision_id = intern(revision_id)
-            text_keys.add(StaticTuple(file_id, revision_id).intern())
+        text_keys_update([bytes_to_text_key(b) for n,b in items])
         yield record
 
 
@@ -1292,3 +1428,28 @@
         """See RepositoryFormat.get_format_description()."""
         return ("Repository format 2a - rich roots, group compression"
             " and chk inventories")
+
+
+class RepositoryFormat2aSubtree(RepositoryFormat2a):
+    """A 2a repository format that supports nested trees.
+
+    """
+
+    def _get_matching_bzrdir(self):
+        return bzrdir.format_registry.make_bzrdir('development-subtree')
+
+    def _ignore_setting_bzrdir(self, format):
+        pass
+
+    _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
+
+    def get_format_string(self):
+        return ('Bazaar development format 8\n')
+
+    def get_format_description(self):
+        """See RepositoryFormat.get_format_description()."""
+        return ("Development repository format 8 - nested trees, "
+                "group compression and chk inventories")
+
+    experimental = True
+    supports_tree_reference = True

=== modified file 'bzrlib/repofmt/knitrepo.py'
--- bzrlib/repofmt/knitrepo.py	2010-04-26 13:51:08 +0000
+++ bzrlib/repofmt/knitrepo.py	2010-08-25 10:20:41 +0000
@@ -38,6 +38,7 @@
 from bzrlib.decorators import needs_read_lock, needs_write_lock
 from bzrlib.repository import (
     CommitBuilder,
+    IsInWriteGroupError,
     MetaDirRepository,
     MetaDirRepositoryFormat,
     RepositoryFormat,
@@ -210,6 +211,8 @@
     def _refresh_data(self):
         if not self.is_locked():
             return
+        if self.is_in_write_group():
+            raise IsInWriteGroupError(self)
         # Create a new transaction to force all knits to see the scope change.
         # This is safe because we're outside a write group.
         self.control_files._finish_transaction()

=== modified file 'bzrlib/repofmt/pack_repo.py'
--- bzrlib/repofmt/pack_repo.py	2010-04-22 17:08:27 +0000
+++ bzrlib/repofmt/pack_repo.py	2010-09-28 18:51:47 +0000
@@ -49,6 +49,7 @@
 """)
 from bzrlib import (
     bzrdir,
+    btree_index,
     errors,
     lockable_files,
     lockdir,
@@ -56,19 +57,17 @@
     )
 
 from bzrlib.decorators import needs_write_lock, only_raises
-from bzrlib.btree_index import (
-    BTreeGraphIndex,
-    BTreeBuilder,
-    )
 from bzrlib.index import (
     GraphIndex,
     InMemoryGraphIndex,
     )
+from bzrlib.lock import LogicalLockResult
 from bzrlib.repofmt.knitrepo import KnitRepository
 from bzrlib.repository import (
     CommitBuilder,
     MetaDirRepositoryFormat,
     RepositoryFormat,
+    RepositoryWriteLockResult,
     RootCommitBuilder,
     StreamSource,
     )
@@ -229,11 +228,13 @@
         unlimited_cache = False
         if index_type == 'chk':
             unlimited_cache = True
-        setattr(self, index_type + '_index',
-            self.index_class(self.index_transport,
-                self.index_name(index_type, self.name),
-                self.index_sizes[self.index_offset(index_type)],
-                unlimited_cache=unlimited_cache))
+        index = self.index_class(self.index_transport,
+                    self.index_name(index_type, self.name),
+                    self.index_sizes[self.index_offset(index_type)],
+                    unlimited_cache=unlimited_cache)
+        if index_type == 'chk':
+            index._leaf_factory = btree_index._gcchk_factory
+        setattr(self, index_type + '_index', index)
 
 
 class ExistingPack(Pack):
@@ -721,7 +722,7 @@
         :return: A Pack object, or None if nothing was copied.
         """
         # open a pack - using the same name as the last temporary file
-        # - which has already been flushed, so its safe.
+        # - which has already been flushed, so it's safe.
         # XXX: - duplicate code warning with start_write_group; fix before
         #      considering 'done'.
         if self._pack_collection._new_pack is not None:
@@ -1291,7 +1292,7 @@
         # reinserted, and if d3 has incorrect parents it will also be
         # reinserted. If we insert d3 first, d2 is present (as it was bulk
         # copied), so we will try to delta, but d2 is not currently able to be
-        # extracted because it's basis d1 is not present. Topologically sorting
+        # extracted because its basis d1 is not present. Topologically sorting
         # addresses this. The following generates a sort for all the texts that
         # are being inserted without having to reference the entire text key
         # space (we only topo sort the revisions, which is smaller).
@@ -1599,9 +1600,9 @@
         pack_operations = [[0, []]]
         # plan out what packs to keep, and what to reorganise
         while len(existing_packs):
-            # take the largest pack, and if its less than the head of the
+            # take the largest pack, and if it's less than the head of the
             # distribution chart we will include its contents in the new pack
-            # for that position. If its larger, we remove its size from the
+            # for that position. If it's larger, we remove its size from the
             # distribution chart
             next_pack_rev_count, next_pack = existing_packs.pop(0)
             if next_pack_rev_count >= pack_distribution[0]:
@@ -1642,7 +1643,7 @@
 
         :return: True if the disk names had not been previously read.
         """
-        # NB: if you see an assertion error here, its probably access against
+        # NB: if you see an assertion error here, it's probably access against
         # an unlocked repo. Naughty.
         if not self.repo.is_locked():
             raise errors.ObjectNotLocked(self.repo)
@@ -1678,7 +1679,7 @@
             txt_index = self._make_index(name, '.tix')
             sig_index = self._make_index(name, '.six')
             if self.chk_index is not None:
-                chk_index = self._make_index(name, '.cix', unlimited_cache=True)
+                chk_index = self._make_index(name, '.cix', is_chk=True)
             else:
                 chk_index = None
             result = ExistingPack(self._pack_transport, name, rev_index,
@@ -1704,7 +1705,7 @@
             sig_index = self._make_index(name, '.six', resume=True)
             if self.chk_index is not None:
                 chk_index = self._make_index(name, '.cix', resume=True,
-                                             unlimited_cache=True)
+                                             is_chk=True)
             else:
                 chk_index = None
             result = self.resumed_pack_factory(name, rev_index, inv_index,
@@ -1740,7 +1741,7 @@
         return self._index_class(self.transport, 'pack-names', None
                 ).iter_all_entries()
 
-    def _make_index(self, name, suffix, resume=False, unlimited_cache=False):
+    def _make_index(self, name, suffix, resume=False, is_chk=False):
         size_offset = self._suffix_offsets[suffix]
         index_name = name + suffix
         if resume:
@@ -1749,8 +1750,11 @@
         else:
             transport = self._index_transport
             index_size = self._names[name][size_offset]
-        return self._index_class(transport, index_name, index_size,
-                                 unlimited_cache=unlimited_cache)
+        index = self._index_class(transport, index_name, index_size,
+                                  unlimited_cache=is_chk)
+        if is_chk and self._index_class is btree_index.BTreeGraphIndex: 
+            index._leaf_factory = btree_index._gcchk_factory
+        return index
 
     def _max_pack_count(self, total_revisions):
         """Return the maximum number of packs to use for total revisions.
@@ -1942,7 +1946,7 @@
                     # disk index because the set values are the same, unless
                     # the only index shows up as deleted by the set difference
                     # - which it may. Until there is a specific test for this,
-                    # assume its broken. RBC 20071017.
+                    # assume it's broken. RBC 20071017.
                     self._remove_pack_from_memory(self.get_pack_by_name(name))
                     self._names[name] = sizes
                     self.get_pack_by_name(name)
@@ -2013,9 +2017,9 @@
         """
         # The ensure_loaded call is to handle the case where the first call
         # made involving the collection was to reload_pack_names, where we 
-        # don't have a view of disk contents. Its a bit of a bandaid, and
-        # causes two reads of pack-names, but its a rare corner case not struck
-        # with regular push/pull etc.
+        # don't have a view of disk contents. It's a bit of a bandaid, and
+        # causes two reads of pack-names, but it's a rare corner case not
+        # struck with regular push/pull etc.
         first_read = self.ensure_loaded()
         if first_read:
             return True
@@ -2340,6 +2344,10 @@
         return self._write_lock_count
 
     def lock_write(self, token=None):
+        """Lock the repository for writes.
+
+        :return: A bzrlib.repository.RepositoryWriteLockResult.
+        """
         locked = self.is_locked()
         if not self._write_lock_count and locked:
             raise errors.ReadOnlyError(self)
@@ -2354,8 +2362,13 @@
                 # Writes don't affect fallback repos
                 repo.lock_read()
             self._refresh_data()
+        return RepositoryWriteLockResult(self.unlock, None)
 
     def lock_read(self):
+        """Lock the repository for reads.
+
+        :return: A bzrlib.lock.LogicalLockResult.
+        """
         locked = self.is_locked()
         if self._write_lock_count:
             self._write_lock_count += 1
@@ -2368,6 +2381,7 @@
             for repo in self._fallback_repositories:
                 repo.lock_read()
             self._refresh_data()
+        return LogicalLockResult(self.unlock)
 
     def leave_lock_in_place(self):
         # not supported - raise an error
@@ -2817,8 +2831,8 @@
     _commit_builder_class = PackCommitBuilder
     supports_external_lookups = True
     # What index classes to use
-    index_builder_class = BTreeBuilder
-    index_class = BTreeGraphIndex
+    index_builder_class = btree_index.BTreeBuilder
+    index_class = btree_index.BTreeGraphIndex
 
     @property
     def _serializer(self):
@@ -2853,8 +2867,8 @@
     supports_tree_reference = False # no subtrees
     supports_external_lookups = True
     # What index classes to use
-    index_builder_class = BTreeBuilder
-    index_class = BTreeGraphIndex
+    index_builder_class = btree_index.BTreeBuilder
+    index_class = btree_index.BTreeGraphIndex
 
     @property
     def _serializer(self):
@@ -2895,8 +2909,8 @@
     supports_tree_reference = True
     supports_external_lookups = True
     # What index classes to use
-    index_builder_class = BTreeBuilder
-    index_class = BTreeGraphIndex
+    index_builder_class = btree_index.BTreeBuilder
+    index_class = btree_index.BTreeGraphIndex
 
     @property
     def _serializer(self):
@@ -2904,7 +2918,7 @@
 
     def _get_matching_bzrdir(self):
         return bzrdir.format_registry.make_bzrdir(
-            'development-subtree')
+            'development5-subtree')
 
     def _ignore_setting_bzrdir(self, format):
         pass

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2010-04-28 07:03:38 +0000
+++ bzrlib/repository.py	2010-09-28 18:51:47 +0000
@@ -25,8 +25,8 @@
     check,
     chk_map,
     config,
+    controldir,
     debug,
-    errors,
     fetch as _mod_fetch,
     fifo_cache,
     generate_ids,
@@ -44,7 +44,6 @@
     symbol_versioning,
     trace,
     tsort,
-    ui,
     versionedfile,
     )
 from bzrlib.bundle import serializer
@@ -53,6 +52,11 @@
 from bzrlib.testament import Testament
 """)
 
+from bzrlib import (
+    errors,
+    registry,
+    ui,
+    )
 from bzrlib.decorators import needs_read_lock, needs_write_lock, only_raises
 from bzrlib.inter import InterObject
 from bzrlib.inventory import (
@@ -61,8 +65,8 @@
     ROOT_ID,
     entry_factory,
     )
-from bzrlib.lock import _RelockDebugMixin
-from bzrlib import registry
+from bzrlib.recordcounter import RecordCounter
+from bzrlib.lock import _RelockDebugMixin, LogicalLockResult
 from bzrlib.trace import (
     log_exception_quietly, note, mutter, mutter_callsite, warning)
 
@@ -71,6 +75,14 @@
 _deprecation_warning_done = False
 
 
+class IsInWriteGroupError(errors.InternalBzrError):
+
+    _fmt = "May not refresh_data of repo %(repo)s while in a write group."
+
+    def __init__(self, repo):
+        errors.InternalBzrError.__init__(self, repo=repo)
+
+
 class CommitBuilder(object):
     """Provides an interface to build up a commit.
 
@@ -231,8 +243,7 @@
 
     def _gen_revision_id(self):
         """Return new revision-id."""
-        return generate_ids.gen_revision_id(self._config.username(),
-                                            self._timestamp)
+        return generate_ids.gen_revision_id(self._committer, self._timestamp)
 
     def _generate_revision_if_needed(self):
         """Create a revision id if None was supplied.
@@ -278,8 +289,8 @@
 
         :param tree: The tree which is being committed.
         """
-        # NB: if there are no parents then this method is not called, so no
-        # need to guard on parents having length.
+        if len(self.parents) == 0:
+            raise errors.RootMissing()
         entry = entry_factory['directory'](tree.path2id(''), '',
             None)
         entry.revision = self._new_revision_id
@@ -423,7 +434,7 @@
             else:
                 # we don't need to commit this, because the caller already
                 # determined that an existing revision of this file is
-                # appropriate. If its not being considered for committing then
+                # appropriate. If it's not being considered for committing then
                 # it and all its parents to the root must be unaltered so
                 # no-change against the basis.
                 if ie.revision == self._new_revision_id:
@@ -745,7 +756,7 @@
                     # after iter_changes examines and decides it has changed,
                     # we will unconditionally record a new version even if some
                     # other process reverts it while commit is running (with
-                    # the revert happening after iter_changes did it's
+                    # the revert happening after iter_changes did its
                     # examination).
                     if change[7][1]:
                         entry.executable = True
@@ -860,11 +871,28 @@
         # versioned roots do not change unless the tree found a change.
 
 
+class RepositoryWriteLockResult(LogicalLockResult):
+    """The result of write locking a repository.
+
+    :ivar repository_token: The token obtained from the underlying lock, or
+        None.
+    :ivar unlock: A callable which will unlock the lock.
+    """
+
+    def __init__(self, unlock, repository_token):
+        LogicalLockResult.__init__(self, unlock)
+        self.repository_token = repository_token
+
+    def __repr__(self):
+        return "RepositoryWriteLockResult(%s, %s)" % (self.repository_token,
+            self.unlock)
+
+
 ######################################################################
 # Repositories
 
 
-class Repository(_RelockDebugMixin, bzrdir.ControlComponent):
+class Repository(_RelockDebugMixin, controldir.ControlComponent):
     """Repository holding history for one or more branches.
 
     The repository holds and retrieves historical information including
@@ -917,7 +945,7 @@
         pointing to .bzr/repository.
     """
 
-    # What class to use for a CommitBuilder. Often its simpler to change this
+    # What class to use for a CommitBuilder. Often it's simpler to change this
     # in a Repository class subclass rather than to override
     # get_commit_builder.
     _commit_builder_class = CommitBuilder
@@ -1018,7 +1046,7 @@
                 " id and insertion revid (%r, %r)"
                 % (inv.revision_id, revision_id))
         if inv.root is None:
-            raise AssertionError()
+            raise errors.RootMissing()
         return self._add_inventory_checked(revision_id, inv, parents)
 
     def _add_inventory_checked(self, revision_id, inv, parents):
@@ -1376,6 +1404,12 @@
         data during reads, and allows a 'write_group' to be obtained. Write
         groups must be used for actual data insertion.
 
+        A token should be passed in if you know that you have locked the object
+        some other way, and need to synchronise this object's state with that
+        fact.
+
+        XXX: this docstring is duplicated in many places, e.g. lockable_files.py
+
         :param token: if this is already locked, then lock_write will fail
             unless the token matches the existing lock.
         :returns: a token if this instance supports tokens, otherwise None.
@@ -1384,15 +1418,10 @@
         :raises MismatchedToken: if the specified token doesn't match the token
             of the existing lock.
         :seealso: start_write_group.
-
-        A token should be passed in if you know that you have locked the object
-        some other way, and need to synchronise this object's state with that
-        fact.
-
-        XXX: this docstring is duplicated in many places, e.g. lockable_files.py
+        :return: A RepositoryWriteLockResult.
         """
         locked = self.is_locked()
-        result = self.control_files.lock_write(token=token)
+        token = self.control_files.lock_write(token=token)
         if not locked:
             self._warn_if_deprecated()
             self._note_lock('w')
@@ -1400,9 +1429,14 @@
                 # Writes don't affect fallback repos
                 repo.lock_read()
             self._refresh_data()
-        return result
+        return RepositoryWriteLockResult(self.unlock, token)
 
     def lock_read(self):
+        """Lock the repository for read operations.
+
+        :return: An object with an unlock method which will release the lock
+            obtained.
+        """
         locked = self.is_locked()
         self.control_files.lock_read()
         if not locked:
@@ -1411,6 +1445,7 @@
             for repo in self._fallback_repositories:
                 repo.lock_read()
             self._refresh_data()
+        return LogicalLockResult(self.unlock)
 
     def get_physical_lock_status(self):
         return self.control_files.get_physical_lock_status()
@@ -1634,16 +1669,16 @@
         return missing_keys
 
     def refresh_data(self):
-        """Re-read any data needed to to synchronise with disk.
+        """Re-read any data needed to synchronise with disk.
 
         This method is intended to be called after another repository instance
         (such as one used by a smart server) has inserted data into the
-        repository. It may not be called during a write group, but may be
-        called at any other time.
+        repository. On all repositories this will work outside of write groups.
+        Some repository formats (pack and newer for bzrlib native formats)
+        support refresh_data inside write groups. If called inside a write
+        group on a repository that does not support refreshing in a write group
+        IsInWriteGroupError will be raised.
         """
-        if self.is_in_write_group():
-            raise errors.InternalBzrError(
-                "May not refresh_data while in a write group.")
         self._refresh_data()
 
     def resume_write_group(self, tokens):
@@ -1688,7 +1723,7 @@
                 "May not fetch while in a write group.")
         # fast path same-url fetch operations
         # TODO: lift out to somewhere common with RemoteRepository
-        # <https://bugs.edge.launchpad.net/bzr/+bug/401646>
+        # <https://bugs.launchpad.net/bzr/+bug/401646>
         if (self.has_same_location(source)
             and fetch_spec is None
             and self._has_same_fallbacks(source)):
@@ -2476,19 +2511,8 @@
             ancestors will be traversed.
         """
         graph = self.get_graph()
-        next_id = revision_id
-        while True:
-            if next_id in (None, _mod_revision.NULL_REVISION):
-                return
-            try:
-                parents = graph.get_parent_map([next_id])[next_id]
-            except KeyError:
-                raise errors.RevisionNotPresent(next_id, self)
-            yield next_id
-            if len(parents) == 0:
-                return
-            else:
-                next_id = parents[0]
+        stop_revisions = (None, _mod_revision.NULL_REVISION)
+        return graph.iter_lefthand_ancestry(revision_id, stop_revisions)
 
     def is_shared(self):
         """Return True if this repository is flagged as a shared repository."""
@@ -2595,7 +2619,7 @@
         types it should be a no-op that just returns.
 
         This stub method does not require a lock, but subclasses should use
-        @needs_write_lock as this is a long running call its reasonable to
+        @needs_write_lock as this is a long running call it's reasonable to
         implicitly lock for the user.
 
         :param hint: If not supplied, the whole repository is packed.
@@ -3354,6 +3378,11 @@
     'bzrlib.repofmt.groupcompress_repo',
     'RepositoryFormat2a',
     )
+format_registry.register_lazy(
+    'Bazaar development format 8\n',
+    'bzrlib.repofmt.groupcompress_repo',
+    'RepositoryFormat2aSubtree',
+    )
 
 
 class InterRepository(InterObject):
@@ -3813,15 +3842,13 @@
                 basis_id, delta, current_revision_id, parents_parents)
             cache[current_revision_id] = parent_tree
 
-    def _fetch_batch(self, revision_ids, basis_id, cache, a_graph=None):
+    def _fetch_batch(self, revision_ids, basis_id, cache):
         """Fetch across a few revisions.
 
         :param revision_ids: The revisions to copy
         :param basis_id: The revision_id of a tree that must be in cache, used
             as a basis for delta when no other base is available
         :param cache: A cache of RevisionTrees that we can use.
-        :param a_graph: A Graph object to determine the heads() of the
-            rich-root data stream.
         :return: The revision_id of the last converted tree. The RevisionTree
             for it will be in cache
         """
@@ -3895,7 +3922,7 @@
         if root_keys_to_create:
             root_stream = _mod_fetch._new_root_data_stream(
                 root_keys_to_create, self._revision_id_to_root_id, parent_map,
-                self.source, graph=a_graph)
+                self.source)
             to_texts.insert_record_stream(root_stream)
         to_texts.insert_record_stream(from_texts.get_record_stream(
             text_keys, self.target._format._fetch_order,
@@ -3958,11 +3985,7 @@
         cache[basis_id] = basis_tree
         del basis_tree # We don't want to hang on to it here
         hints = []
-        if self._converting_to_rich_root and len(revision_ids) > 100:
-            a_graph = _mod_fetch._get_rich_root_heads_graph(self.source,
-                                                            revision_ids)
-        else:
-            a_graph = None
+        a_graph = None
 
         for offset in range(0, len(revision_ids), batch_size):
             self.target.start_write_group()
@@ -3970,8 +3993,7 @@
                 pb.update('Transferring revisions', offset,
                           len(revision_ids))
                 batch = revision_ids[offset:offset+batch_size]
-                basis_id = self._fetch_batch(batch, basis_id, cache,
-                                             a_graph=a_graph)
+                basis_id = self._fetch_batch(batch, basis_id, cache)
             except:
                 self.source._safe_to_return_from_cache = False
                 self.target.abort_write_group()
@@ -4043,7 +4065,7 @@
             basis_id = first_rev.parent_ids[0]
             # only valid as a basis if the target has it
             self.target.get_revision(basis_id)
-            # Try to get a basis tree - if its a ghost it will hit the
+            # Try to get a basis tree - if it's a ghost it will hit the
             # NoSuchRevision case.
             basis_tree = self.source.revision_tree(basis_id)
         except (IndexError, errors.NoSuchRevision):
@@ -4249,7 +4271,8 @@
                 is_resume = False
             try:
                 # locked_insert_stream performs a commit|suspend.
-                return self._locked_insert_stream(stream, src_format, is_resume)
+                return self._locked_insert_stream(stream, src_format,
+                    is_resume)
             except:
                 self.target_repo.abort_write_group(suppress_errors=True)
                 raise
@@ -4302,8 +4325,7 @@
                 # required if the serializers are different only in terms of
                 # the inventory.
                 if src_serializer == to_serializer:
-                    self.target_repo.revisions.insert_record_stream(
-                        substream)
+                    self.target_repo.revisions.insert_record_stream(substream)
                 else:
                     self._extract_and_insert_revisions(substream,
                         src_serializer)
@@ -4417,6 +4439,7 @@
         """Create a StreamSource streaming from from_repository."""
         self.from_repository = from_repository
         self.to_format = to_format
+        self._record_counter = RecordCounter()
 
     def delta_on_metadata(self):
         """Return True if delta's are permitted on metadata streams.

=== modified file 'bzrlib/revision.py'
--- bzrlib/revision.py	2010-02-23 07:43:11 +0000
+++ bzrlib/revision.py	2010-07-15 12:37:32 +0000
@@ -121,21 +121,6 @@
         else:
             return ''
 
-    @symbol_versioning.deprecated_method(symbol_versioning.deprecated_in((1, 13, 0)))
-    def get_apparent_author(self):
-        """Return the apparent author of this revision.
-
-        This method is deprecated in favour of get_apparent_authors.
-
-        If the revision properties contain any author names,
-        return the first. Otherwise return the committer name.
-        """
-        authors = self.get_apparent_authors()
-        if authors:
-            return authors[0]
-        else:
-            return None
-
     def get_apparent_authors(self):
         """Return the apparent authors of this revision.
 

=== modified file 'bzrlib/revisionspec.py'
--- bzrlib/revisionspec.py	2010-02-17 17:11:16 +0000
+++ bzrlib/revisionspec.py	2010-08-18 14:39:37 +0000
@@ -24,12 +24,14 @@
 """)
 
 from bzrlib import (
+    branch as _mod_branch,
     errors,
     osutils,
     registry,
     revision,
     symbol_versioning,
     trace,
+    workingtree,
     )
 
 
@@ -444,7 +446,14 @@
 
 
 
-class RevisionSpec_revid(RevisionSpec):
+class RevisionIDSpec(RevisionSpec):
+
+    def _match_on(self, branch, revs):
+        revision_id = self.as_revision_id(branch)
+        return RevisionInfo.from_revision_id(branch, revision_id, revs)
+
+
+class RevisionSpec_revid(RevisionIDSpec):
     """Selects a revision using the revision id."""
 
     help_txt = """Selects a revision using the revision id.
@@ -459,14 +468,10 @@
 
     prefix = 'revid:'
 
-    def _match_on(self, branch, revs):
+    def _as_revision_id(self, context_branch):
         # self.spec comes straight from parsing the command line arguments,
         # so we expect it to be a Unicode string. Switch it to the internal
         # representation.
-        revision_id = osutils.safe_revision_id(self.spec, warn=False)
-        return RevisionInfo.from_revision_id(branch, revision_id, revs)
-
-    def _as_revision_id(self, context_branch):
         return osutils.safe_revision_id(self.spec, warn=False)
 
 
@@ -813,8 +818,14 @@
         revision_b = other_branch.last_revision()
         if revision_b in (None, revision.NULL_REVISION):
             raise errors.NoCommits(other_branch)
-        # pull in the remote revisions so we can diff
-        branch.fetch(other_branch, revision_b)
+        if branch is None:
+            branch = other_branch
+        else:
+            try:
+                # pull in the remote revisions so we can diff
+                branch.fetch(other_branch, revision_b)
+            except errors.ReadOnlyError:
+                branch = other_branch
         try:
             revno = branch.revision_id_to_revno(revision_b)
         except errors.NoSuchRevision:
@@ -840,6 +851,12 @@
             raise errors.NoCommits(other_branch)
         return other_branch.repository.revision_tree(last_revision)
 
+    def needs_branch(self):
+        return False
+
+    def get_branch(self):
+        return self.spec
+
 
 
 class RevisionSpec_submit(RevisionSpec_ancestor):
@@ -884,6 +901,73 @@
             self._get_submit_location(context_branch))
 
 
+class RevisionSpec_annotate(RevisionIDSpec):
+
+    prefix = 'annotate:'
+
+    help_txt = """Select the revision that last modified the specified line.
+
+    Select the revision that last modified the specified line.  Line is
+    specified as path:number.  Path is a relative path to the file.  Numbers
+    start at 1, and are relative to the current version, not the last-
+    committed version of the file.
+    """
+
+    def _raise_invalid(self, numstring, context_branch):
+        raise errors.InvalidRevisionSpec(self.user_spec, context_branch,
+            'No such line: %s' % numstring)
+
+    def _as_revision_id(self, context_branch):
+        path, numstring = self.spec.rsplit(':', 1)
+        try:
+            index = int(numstring) - 1
+        except ValueError:
+            self._raise_invalid(numstring, context_branch)
+        tree, file_path = workingtree.WorkingTree.open_containing(path)
+        tree.lock_read()
+        try:
+            file_id = tree.path2id(file_path)
+            if file_id is None:
+                raise errors.InvalidRevisionSpec(self.user_spec,
+                    context_branch, "File '%s' is not versioned." %
+                    file_path)
+            revision_ids = [r for (r, l) in tree.annotate_iter(file_id)]
+        finally:
+            tree.unlock()
+        try:
+            revision_id = revision_ids[index]
+        except IndexError:
+            self._raise_invalid(numstring, context_branch)
+        if revision_id == revision.CURRENT_REVISION:
+            raise errors.InvalidRevisionSpec(self.user_spec, context_branch,
+                'Line %s has not been committed.' % numstring)
+        return revision_id
+
+
+class RevisionSpec_mainline(RevisionIDSpec):
+
+    help_txt = """Select mainline revision that merged the specified revision.
+
+    Select the revision that merged the specified revision into mainline.
+    """
+
+    prefix = 'mainline:'
+
+    def _as_revision_id(self, context_branch):
+        revspec = RevisionSpec.from_string(self.spec)
+        if revspec.get_branch() is None:
+            spec_branch = context_branch
+        else:
+            spec_branch = _mod_branch.Branch.open(revspec.get_branch())
+        revision_id = revspec.as_revision_id(spec_branch)
+        graph = context_branch.repository.get_graph()
+        result = graph.find_lefthand_merger(revision_id,
+                                            context_branch.last_revision())
+        if result is None:
+            raise errors.InvalidRevisionSpec(self.user_spec, context_branch)
+        return result
+
+
 # The order in which we want to DWIM a revision spec without any prefix.
 # revno is always tried first and isn't listed here, this is used by
 # RevisionSpec_dwim._match_on
@@ -908,6 +992,8 @@
 _register_revspec(RevisionSpec_ancestor)
 _register_revspec(RevisionSpec_branch)
 _register_revspec(RevisionSpec_submit)
+_register_revspec(RevisionSpec_annotate)
+_register_revspec(RevisionSpec_mainline)
 
 # classes in this list should have a "prefix" attribute, against which
 # string specs are matched

=== modified file 'bzrlib/revisiontree.py'
--- bzrlib/revisiontree.py	2010-04-30 11:03:59 +0000
+++ bzrlib/revisiontree.py	2010-05-06 23:41:35 +0000
@@ -174,8 +174,12 @@
         return set(self._repository.get_ancestry(self._revision_id,
                                                  topo_sorted=False))
 
+    def is_locked(self):
+        return self._repository.is_locked()
+
     def lock_read(self):
         self._repository.lock_read()
+        return self
 
     def __repr__(self):
         return '<%s instance at %x, rev_id=%r>' % (

=== modified file 'bzrlib/shelf.py'
--- bzrlib/shelf.py	2010-04-30 11:03:59 +0000
+++ bzrlib/shelf.py	2010-09-25 20:08:01 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008 Canonical Ltd
+# Copyright (C) 2008, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,6 +18,8 @@
 import errno
 import re
 
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
 from bzrlib import (
     bencode,
     errors,
@@ -26,6 +28,7 @@
     pack,
     transform,
 )
+""")
 
 
 class ShelfCreator(object):

=== modified file 'bzrlib/shelf_ui.py'
--- bzrlib/shelf_ui.py	2010-01-15 05:33:28 +0000
+++ bzrlib/shelf_ui.py	2010-09-20 23:15:01 +0000
@@ -175,7 +175,7 @@
         try:
             target_tree = builtins._get_one_revision_tree('shelf2', revision,
                 tree.branch, tree)
-            files = builtins.safe_relpath_files(tree, file_list)
+            files = tree.safe_relpath_files(file_list)
             return klass(tree, target_tree, diff_writer, all, all, files,
                          message, destroy)
         finally:
@@ -241,7 +241,8 @@
             new_tree = self.work_tree
         old_path = old_tree.id2path(file_id)
         new_path = new_tree.id2path(file_id)
-        text_differ = diff.DiffText(old_tree, new_tree, diff_file)
+        text_differ = diff.DiffText(old_tree, new_tree, diff_file,
+            path_encoding=osutils.get_terminal_encoding())
         patch = text_differ.diff(file_id, old_path, new_path, 'file', 'file')
         diff_file.seek(0)
         return patches.parse_patch(diff_file)
@@ -492,8 +493,10 @@
         tt = tree_merger.make_preview_transform()
         new_tree = tt.get_preview_tree()
         if self.write_diff_to is None:
-            self.write_diff_to = ui.ui_factory.make_output_stream()
-        diff.show_diff_trees(merger.this_tree, new_tree, self.write_diff_to)
+            self.write_diff_to = ui.ui_factory.make_output_stream(encoding_type='exact')
+        path_encoding = osutils.get_diff_header_encoding()
+        diff.show_diff_trees(merger.this_tree, new_tree, self.write_diff_to,
+            path_encoding=path_encoding)
         tt.finalize()
 
     def show_changes(self, merger):

=== modified file 'bzrlib/smart/branch.py'
--- bzrlib/smart/branch.py	2009-10-21 06:05:49 +0000
+++ bzrlib/smart/branch.py	2010-05-13 16:17:54 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +17,10 @@
 """Server-side branch related request implmentations."""
 
 
-from bzrlib import errors
+from bzrlib import (
+    bencode,
+    errors,
+    )
 from bzrlib.bzrdir import BzrDir
 from bzrlib.smart.request import (
     FailedSmartServerResponse,
@@ -194,6 +197,23 @@
         return SuccessfulSmartServerResponse(())
 
 
+class SmartServerBranchRequestSetConfigOptionDict(SmartServerLockedBranchRequest):
+    """Set an option in the branch configuration.
+    
+    New in 2.2.
+    """
+
+    def do_with_locked_branch(self, branch, value_dict, name, section):
+        utf8_dict = bencode.bdecode(value_dict)
+        value_dict = {}
+        for key, value in utf8_dict.items():
+            value_dict[key.decode('utf8')] = value.decode('utf8')
+        if not section:
+            section = None
+        branch._get_config().set_option(value_dict, name, section)
+        return SuccessfulSmartServerResponse(())
+
+
 class SmartServerBranchRequestSetLastRevision(SmartServerSetTipRequest):
 
     def do_tip_change_with_locked_branch(self, branch, new_last_revision_id):
@@ -292,9 +312,11 @@
         if repo_token == '':
             repo_token = None
         try:
-            repo_token = branch.repository.lock_write(token=repo_token)
+            repo_token = branch.repository.lock_write(
+                token=repo_token).repository_token
             try:
-                branch_token = branch.lock_write(token=branch_token)
+                branch_token = branch.lock_write(
+                    token=branch_token).branch_token
             finally:
                 # this leaves the repository with 1 lock
                 branch.repository.unlock()

=== modified file 'bzrlib/smart/bzrdir.py'
--- bzrlib/smart/bzrdir.py	2010-04-23 00:44:15 +0000
+++ bzrlib/smart/bzrdir.py	2010-08-28 11:20:04 +0000
@@ -22,6 +22,9 @@
     BzrDir,
     BzrDirFormat,
     BzrDirMetaFormat1,
+    BzrProber,
+    )
+from bzrlib.controldir import (
     network_format_registry,
     )
 from bzrlib.smart.request import (
@@ -44,10 +47,9 @@
             # clients that don't anticipate errors from this method.
             answer = 'no'
         else:
-            default_format = BzrDirFormat.get_default_format()
-            real_bzrdir = default_format.open(t, _found=True)
+            bzr_prober = BzrProber()
             try:
-                real_bzrdir._format.probe_transport(t)
+                bzr_prober.probe_transport(t)
             except (errors.NotBranchError, errors.UnknownFormatError):
                 answer = 'no'
             else:
@@ -429,7 +431,7 @@
             # It is returned locked, but we need to do the lock to get the lock
             # token.
             repo.unlock()
-            repo_lock_token = repo.lock_write() or ''
+            repo_lock_token = repo.lock_write().repository_token or ''
             if repo_lock_token:
                 repo.leave_lock_in_place()
             repo.unlock()

=== modified file 'bzrlib/smart/medium.py'
--- bzrlib/smart/medium.py	2010-05-03 04:08:50 +0000
+++ bzrlib/smart/medium.py	2010-06-21 08:08:04 +0000
@@ -28,9 +28,9 @@
 import sys
 import urllib
 
+import bzrlib
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
-import atexit
 import socket
 import thread
 import weakref
@@ -494,16 +494,16 @@
 class _DebugCounter(object):
     """An object that counts the HPSS calls made to each client medium.
 
-    When a medium is garbage-collected, or failing that when atexit functions
-    are run, the total number of calls made on that medium are reported via
-    trace.note.
+    When a medium is garbage-collected, or failing that when
+    bzrlib.global_state exits, the total number of calls made on that medium
+    are reported via trace.note.
     """
 
     def __init__(self):
         self.counts = weakref.WeakKeyDictionary()
         client._SmartClient.hooks.install_named_hook(
             'call', self.increment_call_count, 'hpss call counter')
-        atexit.register(self.flush_all)
+        bzrlib.global_state.cleanups.add_cleanup(self.flush_all)
 
     def track(self, medium):
         """Start tracking calls made to a medium.
@@ -715,10 +715,6 @@
     """A client medium using simple pipes.
 
     This client does not manage the pipes: it assumes they will always be open.
-
-    Note that if readable_pipe.read might raise IOError or OSError with errno
-    of EINTR, it must be safe to retry the read.  Plain CPython fileobjects
-    (such as used for sys.stdin) are safe.
     """
 
     def __init__(self, readable_pipe, writeable_pipe, base):
@@ -737,26 +733,40 @@
 
     def _read_bytes(self, count):
         """See SmartClientStreamMedium._read_bytes."""
-        bytes = osutils.until_no_eintr(self._readable_pipe.read, count)
+        bytes_to_read = min(count, _MAX_READ_SIZE)
+        bytes = self._readable_pipe.read(bytes_to_read)
         self._report_activity(len(bytes), 'read')
         return bytes
 
 
+class SSHParams(object):
+    """A set of parameters for starting a remote bzr via SSH."""
+
+    def __init__(self, host, port=None, username=None, password=None,
+            bzr_remote_path='bzr'):
+        self.host = host
+        self.port = port
+        self.username = username
+        self.password = password
+        self.bzr_remote_path = bzr_remote_path
+
+
 class SmartSSHClientMedium(SmartClientStreamMedium):
-    """A client medium using SSH."""
+    """A client medium using SSH.
+    
+    It delegates IO to a SmartClientSocketMedium or
+    SmartClientAlreadyConnectedSocketMedium (depending on platform).
+    """
 
-    def __init__(self, host, port=None, username=None, password=None,
-            base=None, vendor=None, bzr_remote_path=None):
+    def __init__(self, base, ssh_params, vendor=None):
         """Creates a client that will connect on the first use.
 
+        :param ssh_params: A SSHParams instance.
         :param vendor: An optional override for the ssh vendor to use. See
             bzrlib.transport.ssh for details on ssh vendors.
         """
-        self._connected = False
-        self._host = host
-        self._password = password
-        self._port = port
-        self._username = username
+        self._real_medium = None
+        self._ssh_params = ssh_params
         # for the benefit of progress making a short description of this
         # transport
         self._scheme = 'bzr+ssh'
@@ -764,67 +774,70 @@
         # _DebugCounter so we have to store all the values used in our repr
         # method before calling the super init.
         SmartClientStreamMedium.__init__(self, base)
-        self._read_from = None
+        self._vendor = vendor
         self._ssh_connection = None
-        self._vendor = vendor
-        self._write_to = None
-        self._bzr_remote_path = bzr_remote_path
 
     def __repr__(self):
-        if self._port is None:
+        if self._ssh_params.port is None:
             maybe_port = ''
         else:
-            maybe_port = ':%s' % self._port
+            maybe_port = ':%s' % self._ssh_params.port
         return "%s(%s://%s@%s%s/)" % (
             self.__class__.__name__,
             self._scheme,
-            self._username,
-            self._host,
+            self._ssh_params.username,
+            self._ssh_params.host,
             maybe_port)
 
     def _accept_bytes(self, bytes):
         """See SmartClientStreamMedium.accept_bytes."""
         self._ensure_connection()
-        self._write_to.write(bytes)
-        self._report_activity(len(bytes), 'write')
+        self._real_medium.accept_bytes(bytes)
 
     def disconnect(self):
         """See SmartClientMedium.disconnect()."""
-        if not self._connected:
-            return
-        self._read_from.close()
-        self._write_to.close()
-        self._ssh_connection.close()
-        self._connected = False
+        if self._real_medium is not None:
+            self._real_medium.disconnect()
+            self._real_medium = None
+        if self._ssh_connection is not None:
+            self._ssh_connection.close()
+            self._ssh_connection = None
 
     def _ensure_connection(self):
         """Connect this medium if not already connected."""
-        if self._connected:
+        if self._real_medium is not None:
             return
         if self._vendor is None:
             vendor = ssh._get_ssh_vendor()
         else:
             vendor = self._vendor
-        self._ssh_connection = vendor.connect_ssh(self._username,
-                self._password, self._host, self._port,
-                command=[self._bzr_remote_path, 'serve', '--inet',
+        self._ssh_connection = vendor.connect_ssh(self._ssh_params.username,
+                self._ssh_params.password, self._ssh_params.host,
+                self._ssh_params.port,
+                command=[self._ssh_params.bzr_remote_path, 'serve', '--inet',
                          '--directory=/', '--allow-writes'])
-        self._read_from, self._write_to = \
-            self._ssh_connection.get_filelike_channels()
-        self._connected = True
+        io_kind, io_object = self._ssh_connection.get_sock_or_pipes()
+        if io_kind == 'socket':
+            self._real_medium = SmartClientAlreadyConnectedSocketMedium(
+                self.base, io_object)
+        elif io_kind == 'pipes':
+            read_from, write_to = io_object
+            self._real_medium = SmartSimplePipesClientMedium(
+                read_from, write_to, self.base)
+        else:
+            raise AssertionError(
+                "Unexpected io_kind %r from %r"
+                % (io_kind, self._ssh_connection))
 
     def _flush(self):
         """See SmartClientStreamMedium._flush()."""
-        self._write_to.flush()
+        self._real_medium._flush()
 
     def _read_bytes(self, count):
         """See SmartClientStreamMedium.read_bytes."""
-        if not self._connected:
+        if self._real_medium is None:
             raise errors.MediumNotConnected(self)
-        bytes_to_read = min(count, _MAX_READ_SIZE)
-        bytes = self._read_from.read(bytes_to_read)
-        self._report_activity(len(bytes), 'read')
-        return bytes
+        return self._real_medium.read_bytes(count)
 
 
 # Port 4155 is the default port for bzr://, registered with IANA.
@@ -832,22 +845,41 @@
 BZR_DEFAULT_PORT = 4155
 
 
-class SmartTCPClientMedium(SmartClientStreamMedium):
-    """A client medium using TCP."""
+class SmartClientSocketMedium(SmartClientStreamMedium):
+    """A client medium using a socket.
+    
+    This class isn't usable directly.  Use one of its subclasses instead.
+    """
 
-    def __init__(self, host, port, base):
-        """Creates a client that will connect on the first use."""
+    def __init__(self, base):
         SmartClientStreamMedium.__init__(self, base)
+        self._socket = None
         self._connected = False
-        self._host = host
-        self._port = port
-        self._socket = None
 
     def _accept_bytes(self, bytes):
         """See SmartClientMedium.accept_bytes."""
         self._ensure_connection()
         osutils.send_all(self._socket, bytes, self._report_activity)
 
+    def _ensure_connection(self):
+        """Connect this medium if not already connected."""
+        raise NotImplementedError(self._ensure_connection)
+
+    def _flush(self):
+        """See SmartClientStreamMedium._flush().
+
+        For sockets we do no flushing. For TCP sockets we may want to turn off
+        TCP_NODELAY and add a means to do a flush, but that can be done in the
+        future.
+        """
+
+    def _read_bytes(self, count):
+        """See SmartClientMedium.read_bytes."""
+        if not self._connected:
+            raise errors.MediumNotConnected(self)
+        return osutils.read_bytes_from_socket(
+            self._socket, self._report_activity)
+
     def disconnect(self):
         """See SmartClientMedium.disconnect()."""
         if not self._connected:
@@ -856,6 +888,16 @@
         self._socket = None
         self._connected = False
 
+
+class SmartTCPClientMedium(SmartClientSocketMedium):
+    """A client medium that creates a TCP connection."""
+
+    def __init__(self, host, port, base):
+        """Creates a client that will connect on the first use."""
+        SmartClientSocketMedium.__init__(self, base)
+        self._host = host
+        self._port = port
+
     def _ensure_connection(self):
         """Connect this medium if not already connected."""
         if self._connected:
@@ -895,19 +937,22 @@
                     (self._host, port, err_msg))
         self._connected = True
 
-    def _flush(self):
-        """See SmartClientStreamMedium._flush().
-
-        For TCP we do no flushing. We may want to turn off TCP_NODELAY and
-        add a means to do a flush, but that can be done in the future.
-        """
-
-    def _read_bytes(self, count):
-        """See SmartClientMedium.read_bytes."""
-        if not self._connected:
-            raise errors.MediumNotConnected(self)
-        return osutils.read_bytes_from_socket(
-            self._socket, self._report_activity)
+
+class SmartClientAlreadyConnectedSocketMedium(SmartClientSocketMedium):
+    """A client medium for an already connected socket.
+    
+    Note that this class will assume it "owns" the socket, so it will close it
+    when its disconnect method is called.
+    """
+
+    def __init__(self, base, sock):
+        SmartClientSocketMedium.__init__(self, base)
+        self._socket = sock
+        self._connected = True
+
+    def _ensure_connection(self):
+        # Already connected, by definition!  So nothing to do.
+        pass
 
 
 class SmartClientStreamMediumRequest(SmartClientMediumRequest):

=== modified file 'bzrlib/smart/protocol.py'
--- bzrlib/smart/protocol.py	2010-03-26 04:26:55 +0000
+++ bzrlib/smart/protocol.py	2010-06-11 05:57:09 +0000
@@ -1231,6 +1231,7 @@
                     if first_chunk is None:
                         first_chunk = chunk
                     self._write_prefixed_body(chunk)
+                    self.flush()
                     if 'hpssdetail' in debug.debug_flags:
                         # Not worth timing separately, as _write_func is
                         # actually buffered

=== modified file 'bzrlib/smart/repository.py'
--- bzrlib/smart/repository.py	2010-03-25 07:34:15 +0000
+++ bzrlib/smart/repository.py	2010-05-14 13:36:34 +0000
@@ -39,6 +39,7 @@
     SuccessfulSmartServerResponse,
     )
 from bzrlib.repository import _strip_NULL_ghosts, network_format_registry
+from bzrlib.recordcounter import RecordCounter
 from bzrlib import revision as _mod_revision
 from bzrlib.versionedfile import (
     NetworkRecordStream,
@@ -392,7 +393,7 @@
         if token == '':
             token = None
         try:
-            token = repository.lock_write(token=token)
+            token = repository.lock_write(token=token).repository_token
         except errors.LockContention, e:
             return FailedSmartServerResponse(('LockContention',))
         except errors.UnlockableTransport:
@@ -544,12 +545,14 @@
     :ivar first_bytes: The first bytes to give the next NetworkRecordStream.
     """
 
-    def __init__(self, byte_stream):
+    def __init__(self, byte_stream, record_counter):
         """Create a _ByteStreamDecoder."""
         self.stream_decoder = pack.ContainerPushParser()
         self.current_type = None
         self.first_bytes = None
         self.byte_stream = byte_stream
+        self._record_counter = record_counter
+        self.key_count = 0
 
     def iter_stream_decoder(self):
         """Iterate the contents of the pack from stream_decoder."""
@@ -580,13 +583,46 @@
 
     def record_stream(self):
         """Yield substream_type, substream from the byte stream."""
+        def wrap_and_count(pb, rc, substream):
+            """Yield records from stream while showing progress."""
+            counter = 0
+            if rc:
+                if self.current_type != 'revisions' and self.key_count != 0:
+                    # As we know the number of revisions now (in self.key_count)
+                    # we can setup and use record_counter (rc).
+                    if not rc.is_initialized():
+                        rc.setup(self.key_count, self.key_count)
+            for record in substream.read():
+                if rc:
+                    if rc.is_initialized() and counter == rc.STEP:
+                        rc.increment(counter)
+                        pb.update('Estimate', rc.current, rc.max)
+                        counter = 0
+                    if self.current_type == 'revisions':
+                        # Total records is proportional to number of revs
+                        # to fetch. With remote, we used self.key_count to
+                        # track the number of revs. Once we have the revs
+                        # counts in self.key_count, the progress bar changes
+                        # from 'Estimating..' to 'Estimate' above.
+                        self.key_count += 1
+                        if counter == rc.STEP:
+                            pb.update('Estimating..', self.key_count)
+                            counter = 0
+                counter += 1
+                yield record
+
         self.seed_state()
+        pb = ui.ui_factory.nested_progress_bar()
+        rc = self._record_counter
         # Make and consume sub generators, one per substream type:
         while self.first_bytes is not None:
             substream = NetworkRecordStream(self.iter_substream_bytes())
             # after substream is fully consumed, self.current_type is set to
             # the next type, and self.first_bytes is set to the matching bytes.
-            yield self.current_type, substream.read()
+            yield self.current_type, wrap_and_count(pb, rc, substream)
+        if rc:
+            pb.update('Done', rc.max, rc.max)
+        pb.finished()
 
     def seed_state(self):
         """Prepare the _ByteStreamDecoder to decode from the pack stream."""
@@ -597,13 +633,13 @@
         list(self.iter_substream_bytes())
 
 
-def _byte_stream_to_stream(byte_stream):
+def _byte_stream_to_stream(byte_stream, record_counter=None):
     """Convert a byte stream into a format and a stream.
 
     :param byte_stream: A bytes iterator, as output by _stream_to_byte_stream.
     :return: (RepositoryFormat, stream_generator)
     """
-    decoder = _ByteStreamDecoder(byte_stream)
+    decoder = _ByteStreamDecoder(byte_stream, record_counter)
     for bytes in byte_stream:
         decoder.stream_decoder.accept_bytes(bytes)
         for record in decoder.stream_decoder.read_pending_records(max=1):

=== modified file 'bzrlib/smart/request.py'
--- bzrlib/smart/request.py	2010-02-17 17:11:16 +0000
+++ bzrlib/smart/request.py	2010-05-13 16:17:54 +0000
@@ -510,6 +510,8 @@
     'bzrlib.smart.branch', 'SmartServerRequestRevisionHistory')
 request_handlers.register_lazy( 'Branch.set_config_option',
     'bzrlib.smart.branch', 'SmartServerBranchRequestSetConfigOption')
+request_handlers.register_lazy( 'Branch.set_config_option_dict',
+    'bzrlib.smart.branch', 'SmartServerBranchRequestSetConfigOptionDict')
 request_handlers.register_lazy( 'Branch.set_last_revision',
     'bzrlib.smart.branch', 'SmartServerBranchRequestSetLastRevision')
 request_handlers.register_lazy(

=== modified file 'bzrlib/smart/server.py'
--- bzrlib/smart/server.py	2010-03-05 07:27:58 +0000
+++ bzrlib/smart/server.py	2010-07-01 15:25:41 +0000
@@ -18,6 +18,7 @@
 
 import errno
 import os.path
+import select
 import socket
 import sys
 import threading
@@ -51,18 +52,24 @@
     hooks: An instance of SmartServerHooks.
     """
 
-    def __init__(self, backing_transport, host='127.0.0.1', port=0,
-                 root_client_path='/'):
+    def __init__(self, backing_transport, root_client_path='/'):
         """Construct a new server.
 
         To actually start it running, call either start_background_thread or
         serve.
 
         :param backing_transport: The transport to serve.
+        :param root_client_path: The client path that will correspond to root
+            of backing_transport.
+        """
+        self.backing_transport = backing_transport
+        self.root_client_path = root_client_path
+
+    def start_server(self, host, port):
+        """Create the server listening socket.
+
         :param host: Name of the interface to listen on.
         :param port: TCP port to listen on, or 0 to allocate a transient port.
-        :param root_client_path: The client path that will correspond to root
-            of backing_transport.
         """
         # let connections timeout so that we get a chance to terminate
         # Keep a reference to the exceptions we want to catch because the socket
@@ -89,15 +96,10 @@
         self.port = self._sockname[1]
         self._server_socket.listen(1)
         self._server_socket.settimeout(1)
-        self.backing_transport = backing_transport
         self._started = threading.Event()
         self._stopped = threading.Event()
-        self.root_client_path = root_client_path
 
-    def serve(self, thread_name_suffix=''):
-        self._should_terminate = False
-        # for hooks we are letting code know that a server has started (and
-        # later stopped).
+    def _backing_urls(self):
         # There are three interesting urls:
         # The URL the server can be contacted on. (e.g. bzr://host/)
         # The URL that a commit done on the same machine as the server will
@@ -113,15 +115,32 @@
         # The latter two urls are different aliases to the servers url,
         # so we group those in a list - as there might be more aliases
         # in the future.
-        backing_urls = [self.backing_transport.base]
+        urls = [self.backing_transport.base]
         try:
-            backing_urls.append(self.backing_transport.external_url())
+            urls.append(self.backing_transport.external_url())
         except errors.InProcessTransport:
             pass
+        return urls
+
+    def run_server_started_hooks(self, backing_urls=None):
+        if backing_urls is None:
+            backing_urls = self._backing_urls()
         for hook in SmartTCPServer.hooks['server_started']:
             hook(backing_urls, self.get_url())
         for hook in SmartTCPServer.hooks['server_started_ex']:
             hook(backing_urls, self)
+
+    def run_server_stopped_hooks(self, backing_urls=None):
+        if backing_urls is None:
+            backing_urls = self._backing_urls()
+        for hook in SmartTCPServer.hooks['server_stopped']:
+            hook(backing_urls, self.get_url())
+
+    def serve(self, thread_name_suffix=''):
+        self._should_terminate = False
+        # for hooks we are letting code know that a server has started (and
+        # later stopped).
+        self.run_server_started_hooks()
         self._started.set()
         try:
             try:
@@ -155,8 +174,7 @@
             except self._socket_error:
                 # ignore errors on close
                 pass
-            for hook in SmartTCPServer.hooks['server_stopped']:
-                hook(backing_urls, self.get_url())
+            self.run_server_stopped_hooks()
 
     def get_url(self):
         """Return the url of the server"""
@@ -172,8 +190,12 @@
         thread_name = 'smart-server-child' + thread_name_suffix
         connection_thread = threading.Thread(
             None, handler.serve, name=thread_name)
+        # FIXME: This thread is never joined, it should at least be collected
+        # somewhere so that tests that want to check for leaked threads can get
+        # rid of them -- vila 20100531
         connection_thread.setDaemon(True)
         connection_thread.start()
+        return connection_thread
 
     def start_background_thread(self, thread_name_suffix=''):
         self._started.clear()
@@ -324,7 +346,8 @@
                 host = medium.BZR_DEFAULT_INTERFACE
             if port is None:
                 port = medium.BZR_DEFAULT_PORT
-            smart_server = SmartTCPServer(self.transport, host=host, port=port)
+            smart_server = SmartTCPServer(self.transport)
+            smart_server.start_server(host, port)
             trace.note('listening on port: %s' % smart_server.port)
         self.smart_server = smart_server
 

=== modified file 'bzrlib/status.py'
--- bzrlib/status.py	2010-04-30 11:35:43 +0000
+++ bzrlib/status.py	2010-09-17 15:15:34 +0000
@@ -18,6 +18,7 @@
 
 from bzrlib import (
     delta as _mod_delta,
+    hooks as _mod_hooks,
     log,
     osutils,
     tsort,
@@ -150,6 +151,10 @@
         old.lock_read()
         new.lock_read()
         try:
+            for hook in hooks['pre_status']:
+                hook(StatusHookParams(old, new, to_file, versioned,
+                    show_ids, short, verbose))
+
             specific_files, nonexistents \
                 = _filter_nonexistent(specific_files, old, new)
             want_unversioned = not versioned
@@ -210,6 +215,9 @@
                 show_pending_merges(new, to_file, short, verbose=verbose)
             if nonexistents:
                 raise errors.PathsDoNotExist(nonexistents)
+            for hook in hooks['post_status']:
+                hook(StatusHookParams(old, new, to_file, versioned,
+                    show_ids, short, verbose))
         finally:
             old.unlock()
             new.unlock()
@@ -356,3 +364,96 @@
     # their groups individually.  But for consistency of this
     # function's API, it's better to sort both than just 'nonexistent'.
     return sorted(remaining), sorted(nonexistent)
+
+
+class StatusHooks(_mod_hooks.Hooks):
+    """A dictionary mapping hook name to a list of callables for status hooks.
+
+    e.g. ['post_status'] Is the list of items to be called when the
+    status command has finished printing the status.
+    """
+
+    def __init__(self):
+        """Create the default hooks.
+
+        These are all empty initially, because by default nothing should get
+        notified.
+        """
+        _mod_hooks.Hooks.__init__(self)
+        self.create_hook(_mod_hooks.HookPoint('post_status',
+            "Called with argument StatusHookParams after Bazaar has "
+            "displayed the status. StatusHookParams has the attributes "
+            "(old_tree, new_tree, to_file, versioned, show_ids, short, "
+            "verbose). The last four arguments correspond to the command "
+            "line options specified by the user for the status command. "
+            "to_file is the output stream for writing.",
+            (2, 3), None))
+        self.create_hook(_mod_hooks.HookPoint('pre_status',
+            "Called with argument StatusHookParams before Bazaar "
+            "displays the status. StatusHookParams has the attributes "
+            "(old_tree, new_tree, to_file, versioned, show_ids, short, "
+            "verbose). The last four arguments correspond to the command "
+            "line options specified by the user for the status command. "
+            "to_file is the output stream for writing.",
+            (2, 3), None))
+
+
+class StatusHookParams(object):
+    """Object holding parameters passed to post_status hooks.
+
+    :ivar old_tree: Start tree (basis tree) for comparison.
+    :ivar new_tree: Working tree.
+    :ivar to_file: If set, write to this file.
+    :ivar versioned: Show only versioned files.
+    :ivar show_ids: Show internal object ids.
+    :ivar short: Use short status indicators.
+    :ivar verbose: Verbose flag.
+    """
+
+    def __init__(self, old_tree, new_tree, to_file, versioned, show_ids,
+            short, verbose):
+        """Create a group of post_status hook parameters.
+
+        :param old_tree: Start tree (basis tree) for comparison.
+        :param new_tree: Working tree.
+        :param to_file: If set, write to this file.
+        :param versioned: Show only versioned files.
+        :param show_ids: Show internal object ids.
+        :param short: Use short status indicators.
+        :param verbose: Verbose flag.
+        """
+        self.old_tree = old_tree
+        self.new_tree = new_tree
+        self.to_file = to_file
+        self.versioned = versioned
+        self.show_ids = show_ids
+        self.short = short
+        self.verbose = verbose
+
+    def __eq__(self, other):
+        return self.__dict__ == other.__dict__
+
+    def __repr__(self):
+        return "<%s(%s, %s, %s, %s, %s, %s, %s)>" % (self.__class__.__name__,
+            self.old_tree, self.new_tree, self.to_file, self.versioned,
+            self.show_ids, self.short, self.verbose)
+
+
+def _show_shelve_summary(params):
+    """post_status hook to display a summary of shelves.
+
+    :param params: StatusHookParams.
+    """
+    manager = params.new_tree.get_shelf_manager()
+    shelves = manager.active_shelves()
+    if shelves:
+        params.to_file.write('%d shelves exist. '
+            'See "bzr shelve --list" for details.\n' % len(shelves))
+
+
+hooks = StatusHooks()
+
+
+hooks.install_named_hook('post_status', _show_shelve_summary,
+    'bzr status')
+

=== modified file 'bzrlib/store/text.py'
--- bzrlib/store/text.py	2009-03-23 14:59:43 +0000
+++ bzrlib/store/text.py	2010-05-31 21:31:18 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005, 2006, 2008, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -118,7 +118,9 @@
         # so buffer them in a StringIO instead
         if getattr(f, 'tell', None) is not None:
             return gzip.GzipFile(mode='rb', fileobj=f)
-        else:
+        try:
             from cStringIO import StringIO
             sio = StringIO(f.read())
             return gzip.GzipFile(mode='rb', fileobj=sio)
+        finally:
+            f.close()

=== modified file 'bzrlib/strace.py'
--- bzrlib/strace.py	2009-03-23 14:59:43 +0000
+++ bzrlib/strace.py	2010-09-25 20:08:01 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007, 2009, 2010 Canonical Ltd
 #   Authors: Robert Collins <robert.collins@canonical.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -23,6 +23,9 @@
 import subprocess
 import tempfile
 
+from bzrlib import errors
+
+
 # this is currently test-focused, so importing bzrlib.tests is ok. We might
 # want to move feature to its own module though.
 from bzrlib.tests import Feature
@@ -46,14 +49,17 @@
     # capture strace output to a file
     log_file = tempfile.NamedTemporaryFile()
     log_file_fd = log_file.fileno()
+    err_file = tempfile.NamedTemporaryFile()
     pid = os.getpid()
     # start strace
     strace_cmd = ['strace', '-r', '-tt', '-p', str(pid), '-o', log_file.name]
     if follow_children:
         strace_args.append('-f')
+    # need to catch both stdout and stderr to work around
+    # bug 627208
     proc = subprocess.Popen(strace_cmd,
                             stdout=subprocess.PIPE,
-                            stderr=subprocess.STDOUT)
+                            stderr=err_file.fileno())
     # Wait for strace to attach
     attached_notice = proc.stdout.readline()
     # Run the function to strace
@@ -65,18 +71,31 @@
     log_file.seek(0)
     log = log_file.read()
     log_file.close()
-    return result, StraceResult(log)
+    # and stderr
+    err_file.seek(0)
+    err_messages = err_file.read()
+    err_file.close()
+    # and read any errors
+    if err_messages.startswith("attach: ptrace(PTRACE_ATTACH,"):
+        raise StraceError(err_messages=err_messages)
+    return result, StraceResult(log, err_messages)
+
+
+class StraceError(errors.BzrError):
+    
+    _fmt = "strace failed: %(err_messages)s"
 
 
 class StraceResult(object):
     """The result of stracing a function."""
 
-    def __init__(self, raw_log):
+    def __init__(self, raw_log, err_messages):
         """Create a StraceResult.
 
         :param raw_log: The output that strace created.
         """
         self.raw_log = raw_log
+        self.err_messages = err_messages
 
 
 class _StraceFeature(Feature):

=== modified file 'bzrlib/switch.py'
--- bzrlib/switch.py	2010-05-03 04:08:50 +0000
+++ bzrlib/switch.py	2010-05-07 11:50:28 +0000
@@ -78,7 +78,7 @@
     branch_format = control.find_branch_format()
     if branch_format.get_reference(control) is not None:
         # Lightweight checkout: update the branch reference
-        branch_format.set_reference(control, to_branch)
+        branch_format.set_reference(control, None, to_branch)
     else:
         b = control.open_branch()
         bound_branch = b.get_bound_location()

=== modified file 'bzrlib/symbol_versioning.py'
--- bzrlib/symbol_versioning.py	2009-09-15 02:28:34 +0000
+++ bzrlib/symbol_versioning.py	2010-08-28 15:34:24 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007, 2008, 2009 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -29,6 +29,9 @@
            'warn',
            ]
 
+
+import warnings
+# Import the 'warn' symbol so bzrlib can call it even if we redefine it
 from warnings import warn
 
 import bzrlib
@@ -296,7 +299,6 @@
     :param error_only: Only match an 'error' filter
     :return: True if a filter is found, False otherwise
     """
-    import warnings
     for filter in warnings.filters:
         if issubclass(DeprecationWarning, filter[2]):
             # This filter will effect DeprecationWarning
@@ -305,6 +307,19 @@
     return False
 
 
+def _remove_filter_callable(filter):
+    """Build and returns a callable removing filter from the warnings.
+
+    :param filter: The filter to remove (can be None).
+
+    :return: A callable that will remove filter from warnings.filters.
+    """
+    def cleanup():
+        if filter:
+            warnings.filters.remove(filter)
+    return cleanup
+
+
 def suppress_deprecation_warnings(override=True):
     """Call this function to suppress all deprecation warnings.
 
@@ -314,13 +329,17 @@
 
     :param override: If True, always set the ignore, if False, only set the
         ignore if there isn't already a filter.
+
+    :return: A callable to remove the new warnings this added.
     """
-    import warnings
     if not override and _check_for_filter(error_only=False):
         # If there is already a filter effecting suppress_deprecation_warnings,
         # then skip it.
-        return
-    warnings.filterwarnings('ignore', category=DeprecationWarning)
+        filter = None
+    else:
+        warnings.filterwarnings('ignore', category=DeprecationWarning)
+        filter = warnings.filters[0]
+    return _remove_filter_callable(filter)
 
 
 def activate_deprecation_warnings(override=True):
@@ -337,10 +356,14 @@
     :param override: If False, only add a filter if there isn't an error filter
         already. (This slightly differs from suppress_deprecation_warnings, in
         because it always overrides everything but -Werror).
+
+    :return: A callable to remove the new warnings this added.
     """
-    import warnings
     if not override and _check_for_filter(error_only=True):
         # DeprecationWarnings are already turned into errors, don't downgrade
         # them to 'default'.
-        return
-    warnings.filterwarnings('default', category=DeprecationWarning)
+        filter = None
+    else:
+        warnings.filterwarnings('default', category=DeprecationWarning)
+        filter = warnings.filters[0]
+    return _remove_filter_callable(filter)

=== modified file 'bzrlib/tests/EncodingAdapter.py'
--- bzrlib/tests/EncodingAdapter.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/EncodingAdapter.py	2010-05-24 01:05:14 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2009, 2010 Canonical Ltd
 # -*- coding: utf-8 -*-
 #
 # This program is free software; you can redistribute it and/or modify
@@ -19,9 +19,6 @@
 
 from copy import deepcopy
 
-from bzrlib.tests import TestSuite
-
-
 # prefix for micro (1/1000000)
 _mu = u'\xb5'
 

=== modified file 'bzrlib/tests/TestUtil.py'
--- bzrlib/tests/TestUtil.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/TestUtil.py	2010-08-05 18:13:23 +0000
@@ -135,7 +135,13 @@
         >>>         result.addTests([test, test])
         >>>     return result
         """
-        basic_tests = super(TestLoader, self).loadTestsFromModule(module)
+        if sys.version_info < (2, 7):
+            basic_tests = super(TestLoader, self).loadTestsFromModule(module)
+        else:
+            # GZ 2010-07-19: Python 2.7 unittest also uses load_tests but with
+            #                a different and incompatible signature
+            basic_tests = super(TestLoader, self).loadTestsFromModule(module,
+                use_load_tests=False)
         load_tests = getattr(module, "load_tests", None)
         if load_tests is not None:
             return load_tests(basic_tests, module, self)

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2010-05-06 07:48:22 +0000
+++ bzrlib/tests/__init__.py	2010-10-07 12:10:24 +0000
@@ -29,20 +29,21 @@
 
 import atexit
 import codecs
-from copy import copy
+import copy
 from cStringIO import StringIO
 import difflib
 import doctest
 import errno
+import itertools
 import logging
-import math
 import os
-from pprint import pformat
+import platform
+import pprint
 import random
 import re
 import shlex
 import stat
-from subprocess import Popen, PIPE, STDOUT
+import subprocess
 import sys
 import tempfile
 import threading
@@ -70,10 +71,10 @@
     lock as _mod_lock,
     memorytree,
     osutils,
-    progress,
     ui,
     urlutils,
     registry,
+    transport as _mod_transport,
     workingtree,
     )
 import bzrlib.branch
@@ -103,22 +104,15 @@
     )
 import bzrlib.trace
 from bzrlib.transport import (
-    get_transport,
     memory,
     pathfilter,
     )
-import bzrlib.transport
 from bzrlib.trace import mutter, note
 from bzrlib.tests import (
     test_server,
     TestUtil,
+    treeshape,
     )
-from bzrlib.tests.http_server import HttpServer
-from bzrlib.tests.TestUtil import (
-                          TestSuite,
-                          TestLoader,
-                          )
-from bzrlib.tests.treeshape import build_tree_contents
 from bzrlib.ui import NullProgressView
 from bzrlib.ui.text import TextUIFactory
 import bzrlib.version_info_formats.format_custom
@@ -140,8 +134,12 @@
 SUBUNIT_SEEK_SET = 0
 SUBUNIT_SEEK_CUR = 1
 
+# These are intentionally brought into this namespace. That way plugins, etc
+# can just "from bzrlib.tests import TestCase, TestLoader, etc"
+TestSuite = TestUtil.TestSuite
+TestLoader = TestUtil.TestLoader
 
-class ExtendedTestResult(unittest._TextTestResult):
+class ExtendedTestResult(testtools.TextTestResult):
     """Accepts, reports and accumulates the results of running tests.
 
     Compared to the unittest version this class adds support for
@@ -168,7 +166,7 @@
         :param bench_history: Optionally, a writable file object to accumulate
             benchmark results.
         """
-        unittest._TextTestResult.__init__(self, stream, descriptions, verbosity)
+        testtools.TextTestResult.__init__(self, stream)
         if bench_history is not None:
             from bzrlib.version import _get_bzr_source_tree
             src_tree = _get_bzr_source_tree()
@@ -195,17 +193,21 @@
         self.count = 0
         self._overall_start_time = time.time()
         self._strict = strict
+        self._first_thread_leaker_id = None
+        self._tests_leaking_threads_count = 0
 
     def stopTestRun(self):
         run = self.testsRun
         actionTaken = "Ran"
         stopTime = time.time()
         timeTaken = stopTime - self.startTime
-        self.printErrors()
-        self.stream.writeln(self.separator2)
-        self.stream.writeln("%s %d test%s in %.3fs" % (actionTaken,
+        # GZ 2010-07-19: Seems testtools has no printErrors method, and though
+        #                the parent class method is similar have to duplicate
+        self._show_list('ERROR', self.errors)
+        self._show_list('FAIL', self.failures)
+        self.stream.write(self.sep2)
+        self.stream.write("%s %d test%s in %.3fs\n\n" % (actionTaken,
                             run, run != 1 and "s" or "", timeTaken))
-        self.stream.writeln()
         if not self.wasSuccessful():
             self.stream.write("FAILED (")
             failed, errored = map(len, (self.failures, self.errors))
@@ -218,34 +220,34 @@
                 if failed or errored: self.stream.write(", ")
                 self.stream.write("known_failure_count=%d" %
                     self.known_failure_count)
-            self.stream.writeln(")")
+            self.stream.write(")\n")
         else:
             if self.known_failure_count:
-                self.stream.writeln("OK (known_failures=%d)" %
+                self.stream.write("OK (known_failures=%d)\n" %
                     self.known_failure_count)
             else:
-                self.stream.writeln("OK")
+                self.stream.write("OK\n")
         if self.skip_count > 0:
             skipped = self.skip_count
-            self.stream.writeln('%d test%s skipped' %
+            self.stream.write('%d test%s skipped\n' %
                                 (skipped, skipped != 1 and "s" or ""))
         if self.unsupported:
             for feature, count in sorted(self.unsupported.items()):
-                self.stream.writeln("Missing feature '%s' skipped %d tests." %
+                self.stream.write("Missing feature '%s' skipped %d tests.\n" %
                     (feature, count))
         if self._strict:
             ok = self.wasStrictlySuccessful()
         else:
             ok = self.wasSuccessful()
-        if TestCase._first_thread_leaker_id:
+        if self._first_thread_leaker_id:
             self.stream.write(
                 '%s is leaking threads among %d leaking tests.\n' % (
-                TestCase._first_thread_leaker_id,
-                TestCase._leaking_threads_tests))
+                self._first_thread_leaker_id,
+                self._tests_leaking_threads_count))
             # We don't report the main thread as an active one.
             self.stream.write(
                 '%d non-main threads were left active in the end.\n'
-                % (TestCase._active_threads - 1))
+                % (len(self._active_threads) - 1))
 
     def getDescription(self, test):
         return test.id()
@@ -258,7 +260,8 @@
 
     def _elapsedTestTimeString(self):
         """Return a time string for the overall time the current test has taken."""
-        return self._formatTime(time.time() - self._start_time)
+        return self._formatTime(self._delta_to_float(
+            self._now() - self._start_datetime))
 
     def _testTimeString(self, testCase):
         benchmark_time = self._extractBenchmarkTime(testCase)
@@ -275,46 +278,45 @@
 
     def _shortened_test_description(self, test):
         what = test.id()
-        what = re.sub(r'^bzrlib\.(tests|benchmarks)\.', '', what)
+        what = re.sub(r'^bzrlib\.tests\.', '', what)
         return what
 
     def startTest(self, test):
-        unittest.TestResult.startTest(self, test)
+        super(ExtendedTestResult, self).startTest(test)
         if self.count == 0:
             self.startTests()
+        self.count += 1
         self.report_test_start(test)
         test.number = self.count
         self._recordTestStartTime()
+        # Only check for thread leaks if the test case supports cleanups
+        addCleanup = getattr(test, "addCleanup", None)
+        if addCleanup is not None:
+            addCleanup(self._check_leaked_threads, test)
 
     def startTests(self):
-        import platform
-        if getattr(sys, 'frozen', None) is None:
-            bzr_path = osutils.realpath(sys.argv[0])
-        else:
-            bzr_path = sys.executable
-        self.stream.write(
-            'bzr selftest: %s\n' % (bzr_path,))
-        self.stream.write(
-            '   %s\n' % (
-                    bzrlib.__path__[0],))
-        self.stream.write(
-            '   bzr-%s python-%s %s\n' % (
-                    bzrlib.version_string,
-                    bzrlib._format_version_tuple(sys.version_info),
-                    platform.platform(aliased=1),
-                    ))
-        self.stream.write('\n')
+        self.report_tests_starting()
+        self._active_threads = threading.enumerate()
+
+    def _check_leaked_threads(self, test):
+        """See if any threads have leaked since last call
+
+        A sample of live threads is stored in the _active_threads attribute,
+        when this method runs it compares the current live threads and any not
+        in the previous sample are treated as having leaked.
+        """
+        now_active_threads = set(threading.enumerate())
+        threads_leaked = now_active_threads.difference(self._active_threads)
+        if threads_leaked:
+            self._report_thread_leak(test, threads_leaked, now_active_threads)
+            self._tests_leaking_threads_count += 1
+            if self._first_thread_leaker_id is None:
+                self._first_thread_leaker_id = test.id()
+            self._active_threads = now_active_threads
 
     def _recordTestStartTime(self):
         """Record that a test has started."""
-        self._start_time = time.time()
-
-    def _cleanupLogFile(self, test):
-        # We can only do this if we have one of our TestCases, not if
-        # we have a doctest.
-        setKeepLogfile = getattr(test, 'setKeepLogfile', None)
-        if setKeepLogfile is not None:
-            setKeepLogfile()
+        self._start_datetime = self._now()
 
     def addError(self, test, err):
         """Tell result that test finished with an error.
@@ -323,12 +325,11 @@
         fails with an unexpected error.
         """
         self._post_mortem()
-        unittest.TestResult.addError(self, test, err)
+        super(ExtendedTestResult, self).addError(test, err)
         self.error_count += 1
         self.report_error(test, err)
         if self.stop_early:
             self.stop()
-        self._cleanupLogFile(test)
 
     def addFailure(self, test, err):
         """Tell result that test failed.
@@ -337,12 +338,11 @@
         fails because e.g. an assert() method failed.
         """
         self._post_mortem()
-        unittest.TestResult.addFailure(self, test, err)
+        super(ExtendedTestResult, self).addFailure(test, err)
         self.failure_count += 1
         self.report_failure(test, err)
         if self.stop_early:
             self.stop()
-        self._cleanupLogFile(test)
 
     def addSuccess(self, test, details=None):
         """Tell result that test completed successfully.
@@ -356,8 +356,7 @@
                     self._formatTime(benchmark_time),
                     test.id()))
         self.report_success(test)
-        self._cleanupLogFile(test)
-        unittest.TestResult.addSuccess(self, test)
+        super(ExtendedTestResult, self).addSuccess(test)
         test._log_contents = ''
 
     def addExpectedFailure(self, test, err):
@@ -400,8 +399,37 @@
         else:
             raise errors.BzrError("Unknown whence %r" % whence)
 
-    def report_cleaning_up(self):
-        pass
+    def report_tests_starting(self):
+        """Display information before the test run begins"""
+        if getattr(sys, 'frozen', None) is None:
+            bzr_path = osutils.realpath(sys.argv[0])
+        else:
+            bzr_path = sys.executable
+        self.stream.write(
+            'bzr selftest: %s\n' % (bzr_path,))
+        self.stream.write(
+            '   %s\n' % (
+                    bzrlib.__path__[0],))
+        self.stream.write(
+            '   bzr-%s python-%s %s\n' % (
+                    bzrlib.version_string,
+                    bzrlib._format_version_tuple(sys.version_info),
+                    platform.platform(aliased=1),
+                    ))
+        self.stream.write('\n')
+
+    def report_test_start(self, test):
+        """Display information on the test just about to be run"""
+
+    def _report_thread_leak(self, test, leaked_threads, active_threads):
+        """Display information on a test that leaked one or more threads"""
+        # GZ 2010-09-09: A leak summary reported separately from the general
+        #                thread debugging would be nice. Tests under subunit
+        #                need something not using stream, perhaps adding a
+        #                testtools details object would be fitting.
+        if 'threads' in selftest_debug_flags:
+            self.stream.write('%s is leaking, active is now %d\n' %
+                (test.id(), len(active_threads)))
 
     def startTestRun(self):
         self.startTime = time.time()
@@ -444,15 +472,10 @@
         self.pb.finished()
         super(TextTestResult, self).stopTestRun()
 
-    def startTestRun(self):
-        super(TextTestResult, self).startTestRun()
+    def report_tests_starting(self):
+        super(TextTestResult, self).report_tests_starting()
         self.pb.update('[test 0/%d] Starting' % (self.num_tests))
 
-    def printErrors(self):
-        # clear the pb to make room for the error listing
-        self.pb.clear()
-        super(TextTestResult, self).printErrors()
-
     def _progress_prefix_text(self):
         # the longer this text, the less space we have to show the test
         # name...
@@ -480,7 +503,6 @@
         return a
 
     def report_test_start(self, test):
-        self.count += 1
         self.pb.update(
                 self._progress_prefix_text()
                 + ' '
@@ -513,9 +535,6 @@
     def report_unsupported(self, test, feature):
         """test cannot be run because feature is missing."""
 
-    def report_cleaning_up(self):
-        self.pb.update('Cleaning up')
-
 
 class VerboseTestResult(ExtendedTestResult):
     """Produce long output, with one line per test run plus times"""
@@ -528,12 +547,11 @@
             result = a_string
         return result.ljust(final_width)
 
-    def startTestRun(self):
-        super(VerboseTestResult, self).startTestRun()
+    def report_tests_starting(self):
         self.stream.write('running %d tests...\n' % self.num_tests)
+        super(VerboseTestResult, self).report_tests_starting()
 
     def report_test_start(self, test):
-        self.count += 1
         name = self._shortened_test_description(test)
         width = osutils.terminal_width()
         if width is not None:
@@ -551,40 +569,40 @@
         return '%s%s' % (indent, err[1])
 
     def report_error(self, test, err):
-        self.stream.writeln('ERROR %s\n%s'
+        self.stream.write('ERROR %s\n%s\n'
                 % (self._testTimeString(test),
                    self._error_summary(err)))
 
     def report_failure(self, test, err):
-        self.stream.writeln(' FAIL %s\n%s'
+        self.stream.write(' FAIL %s\n%s\n'
                 % (self._testTimeString(test),
                    self._error_summary(err)))
 
     def report_known_failure(self, test, err):
-        self.stream.writeln('XFAIL %s\n%s'
+        self.stream.write('XFAIL %s\n%s\n'
                 % (self._testTimeString(test),
                    self._error_summary(err)))
 
     def report_success(self, test):
-        self.stream.writeln('   OK %s' % self._testTimeString(test))
+        self.stream.write('   OK %s\n' % self._testTimeString(test))
         for bench_called, stats in getattr(test, '_benchcalls', []):
-            self.stream.writeln('LSProf output for %s(%s, %s)' % bench_called)
+            self.stream.write('LSProf output for %s(%s, %s)\n' % bench_called)
             stats.pprint(file=self.stream)
         # flush the stream so that we get smooth output. This verbose mode is
         # used to show the output in PQM.
         self.stream.flush()
 
     def report_skip(self, test, reason):
-        self.stream.writeln(' SKIP %s\n%s'
+        self.stream.write(' SKIP %s\n%s\n'
                 % (self._testTimeString(test), reason))
 
     def report_not_applicable(self, test, reason):
-        self.stream.writeln('  N/A %s\n    %s'
+        self.stream.write('  N/A %s\n    %s\n'
                 % (self._testTimeString(test), reason))
 
     def report_unsupported(self, test, feature):
         """test cannot be run because feature is missing."""
-        self.stream.writeln("NODEP %s\n    The feature '%s' is not available."
+        self.stream.write("NODEP %s\n    The feature '%s' is not available.\n"
                 %(self._testTimeString(test), feature))
 
 
@@ -619,7 +637,7 @@
             encode = codec.encode
         stream = osutils.UnicodeOrBytesToBytesWriter(encode, stream)
         stream.encoding = new_encoding
-        self.stream = unittest._WritelnDecorator(stream)
+        self.stream = stream
         self.descriptions = descriptions
         self.verbosity = verbosity
         self._bench_history = bench_history
@@ -749,7 +767,7 @@
     # XXX: Should probably unify more with CannedInputUIFactory or a
     # particular configuration of TextUIFactory, or otherwise have a clearer
     # idea of how they're supposed to be different.
-    # See https://bugs.edge.launchpad.net/bzr/+bug/408213
+    # See https://bugs.launchpad.net/bzr/+bug/408213
 
     def __init__(self, stdout=None, stderr=None, stdin=None):
         if stdin is not None:
@@ -789,21 +807,17 @@
     routine, and to build and check bzr trees.
 
     In addition to the usual method of overriding tearDown(), this class also
-    allows subclasses to register functions into the _cleanups list, which is
+    allows subclasses to register cleanup functions via addCleanup, which are
     run in order as the object is torn down.  It's less likely this will be
     accidentally overlooked.
     """
 
-    _active_threads = None
-    _leaking_threads_tests = 0
-    _first_thread_leaker_id = None
-    _log_file_name = None
+    _log_file = None
     # record lsprof data when performing benchmark calls.
     _gather_lsprof_in_benchmarks = False
 
     def __init__(self, methodName='testMethod'):
         super(TestCase, self).__init__(methodName)
-        self._cleanups = []
         self._directory_isolation = True
         self.exception_handlers.insert(0,
             (UnavailableFeature, self._do_unsupported_or_skip))
@@ -827,28 +841,27 @@
         self._track_transports()
         self._track_locks()
         self._clear_debug_flags()
-        TestCase._active_threads = threading.activeCount()
-        self.addCleanup(self._check_leaked_threads)
 
     def debug(self):
         # debug a frame up.
         import pdb
         pdb.Pdb().set_trace(sys._getframe().f_back)
 
-    def _check_leaked_threads(self):
-        active = threading.activeCount()
-        leaked_threads = active - TestCase._active_threads
-        TestCase._active_threads = active
-        # If some tests make the number of threads *decrease*, we'll consider
-        # that they are just observing old threads dieing, not agressively kill
-        # random threads. So we don't report these tests as leaking. The risk
-        # is that we have false positives that way (the test see 2 threads
-        # going away but leak one) but it seems less likely than the actual
-        # false positives (the test see threads going away and does not leak).
-        if leaked_threads > 0:
-            TestCase._leaking_threads_tests += 1
-            if TestCase._first_thread_leaker_id is None:
-                TestCase._first_thread_leaker_id = self.id()
+    def discardDetail(self, name):
+        """Extend the addDetail, getDetails api so we can remove a detail.
+
+        eg. bzr always adds the 'log' detail at startup, but we don't want to
+        include it for skipped, xfail, etc tests.
+
+        It is safe to call this for a detail that doesn't exist, in case this
+        gets called multiple times.
+        """
+        # We cheat. details is stored in __details which means we shouldn't
+        # touch it. but getDetails() returns the dict directly, so we can
+        # mutate it.
+        details = self.getDetails()
+        if name in details:
+            del details[name]
 
     def _clear_debug_flags(self):
         """Prevent externally set debug flags affecting tests.
@@ -943,7 +956,7 @@
 
     def permit_dir(self, name):
         """Permit a directory to be used by this test. See permit_url."""
-        name_transport = get_transport(name)
+        name_transport = _mod_transport.get_transport(name)
         self.permit_url(name)
         self.permit_url(name_transport.base)
 
@@ -972,7 +985,7 @@
             try:
                 workingtree.WorkingTree.open(path)
             except (errors.NotBranchError, errors.NoWorkingTree):
-                return
+                raise TestSkipped('Needs a working tree of bzr sources')
         finally:
             self.enable_directory_isolation()
 
@@ -1028,7 +1041,7 @@
         self.addCleanup(transport_server.stop_server)
         # Obtain a real transport because if the server supplies a password, it
         # will be hidden from the base on the client side.
-        t = get_transport(transport_server.get_url())
+        t = _mod_transport.get_transport(transport_server.get_url())
         # Some transport servers effectively chroot the backing transport;
         # others like SFTPServer don't - users of the transport can walk up the
         # transport to read the entire backing transport. This wouldn't matter
@@ -1095,7 +1108,7 @@
             message += '\n'
         raise AssertionError("%snot equal:\na = %s\nb = %s\n"
             % (message,
-               pformat(a), pformat(b)))
+               pprint.pformat(a), pprint.pformat(b)))
 
     assertEquals = assertEqual
 
@@ -1455,10 +1468,8 @@
 
         The file is removed as the test is torn down.
         """
-        fileno, name = tempfile.mkstemp(suffix='.log', prefix='testbzr')
-        self._log_file = os.fdopen(fileno, 'w+')
+        self._log_file = StringIO()
         self._log_memento = bzrlib.trace.push_log_file(self._log_file)
-        self._log_file_name = name
         self.addCleanup(self._finishLogFile)
 
     def _finishLogFile(self):
@@ -1486,14 +1497,6 @@
         """
         debug.debug_flags.discard('strict_locks')
 
-    def addCleanup(self, callable, *args, **kwargs):
-        """Arrange to run a callable when this case is torn down.
-
-        Callables are run in the reverse of the order they are registered,
-        ie last-in first-out.
-        """
-        self._cleanups.append((callable, args, kwargs))
-
     def overrideAttr(self, obj, attr_name, new=_unitialized_attr):
         """Overrides an object attribute restoring it after the test.
 
@@ -1524,7 +1527,7 @@
             'EDITOR': None,
             'BZR_EMAIL': None,
             'BZREMAIL': None, # may still be present in the environment
-            'EMAIL': None,
+            'EMAIL': 'jrandom@example.com', # set EMAIL as bzr does not guess
             'BZR_PROGRESS_BAR': None,
             'BZR_LOG': None,
             'BZR_PLUGIN_PATH': None,
@@ -1583,7 +1586,12 @@
         """This test has failed for some known reason."""
         raise KnownFailure(reason)
 
+    def _suppress_log(self):
+        """Remove the log info from details."""
+        self.discardDetail('log')
+
     def _do_skip(self, result, reason):
+        self._suppress_log()
         addSkip = getattr(result, 'addSkip', None)
         if not callable(addSkip):
             result.addSuccess(result)
@@ -1592,6 +1600,7 @@
 
     @staticmethod
     def _do_known_failure(self, result, e):
+        self._suppress_log()
         err = sys.exc_info()
         addExpectedFailure = getattr(result, 'addExpectedFailure', None)
         if addExpectedFailure is not None:
@@ -1605,6 +1614,7 @@
             reason = 'No reason given'
         else:
             reason = e.args[0]
+        self._suppress_log ()
         addNotApplicable = getattr(result, 'addNotApplicable', None)
         if addNotApplicable is not None:
             result.addNotApplicable(self, reason)
@@ -1612,8 +1622,29 @@
             self._do_skip(result, reason)
 
     @staticmethod
+    def _report_skip(self, result, err):
+        """Override the default _report_skip.
+
+        We want to strip the 'log' detail. If we waint until _do_skip, it has
+        already been formatted into the 'reason' string, and we can't pull it
+        out again.
+        """
+        self._suppress_log()
+        super(TestCase, self)._report_skip(self, result, err)
+
+    @staticmethod
+    def _report_expected_failure(self, result, err):
+        """Strip the log.
+
+        See _report_skip for motivation.
+        """
+        self._suppress_log()
+        super(TestCase, self)._report_expected_failure(self, result, err)
+
+    @staticmethod
     def _do_unsupported_or_skip(self, result, e):
         reason = e.args[0]
+        self._suppress_log()
         addNotSupported = getattr(result, 'addNotSupported', None)
         if addNotSupported is not None:
             result.addNotSupported(self, reason)
@@ -1666,65 +1697,21 @@
                 unicodestr = self._log_contents.decode('utf8', 'replace')
                 self._log_contents = unicodestr.encode('utf8')
             return self._log_contents
-        import bzrlib.trace
-        if bzrlib.trace._trace_file:
-            # flush the log file, to get all content
-            bzrlib.trace._trace_file.flush()
-        if self._log_file_name is not None:
-            logfile = open(self._log_file_name)
-            try:
-                log_contents = logfile.read()
-            finally:
-                logfile.close()
+        if self._log_file is not None:
+            log_contents = self._log_file.getvalue()
             try:
                 log_contents.decode('utf8')
             except UnicodeDecodeError:
                 unicodestr = log_contents.decode('utf8', 'replace')
                 log_contents = unicodestr.encode('utf8')
             if not keep_log_file:
-                close_attempts = 0
-                max_close_attempts = 100
-                first_close_error = None
-                while close_attempts < max_close_attempts:
-                    close_attempts += 1
-                    try:
-                        self._log_file.close()
-                    except IOError, ioe:
-                        if ioe.errno is None:
-                            # No errno implies 'close() called during
-                            # concurrent operation on the same file object', so
-                            # retry.  Probably a thread is trying to write to
-                            # the log file.
-                            if first_close_error is None:
-                                first_close_error = ioe
-                            continue
-                        raise
-                    else:
-                        break
-                if close_attempts > 1:
-                    sys.stderr.write(
-                        'Unable to close log file on first attempt, '
-                        'will retry: %s\n' % (first_close_error,))
-                    if close_attempts == max_close_attempts:
-                        sys.stderr.write(
-                            'Unable to close log file after %d attempts.\n'
-                            % (max_close_attempts,))
                 self._log_file = None
                 # Permit multiple calls to get_log until we clean it up in
                 # finishLogFile
                 self._log_contents = log_contents
-                try:
-                    os.remove(self._log_file_name)
-                except OSError, e:
-                    if sys.platform == 'win32' and e.errno == errno.EACCES:
-                        sys.stderr.write(('Unable to delete log file '
-                                             ' %r\n' % self._log_file_name))
-                    else:
-                        raise
-                self._log_file_name = None
             return log_contents
         else:
-            return "No log file content and no log file name."
+            return "No log file content."
 
     def get_log(self):
         """Get a unicode string containing the log from bzrlib.trace.
@@ -1945,15 +1932,15 @@
             variables. A value of None will unset the env variable.
             The values must be strings. The change will only occur in the
             child, so you don't need to fix the environment after running.
-        :param skip_if_plan_to_signal: raise TestSkipped when true and os.kill
-            is not available.
+        :param skip_if_plan_to_signal: raise TestSkipped when true and system
+            doesn't support signalling subprocesses.
         :param allow_plugins: If False (default) pass --no-plugins to bzr.
 
         :returns: Popen object for the started process.
         """
         if skip_if_plan_to_signal:
-            if not getattr(os, 'kill', None):
-                raise TestSkipped("os.kill not available.")
+            if os.name != "posix":
+                raise TestSkipped("Sending signals not supported")
 
         if env_changes is None:
             env_changes = {}
@@ -1986,7 +1973,9 @@
             if not allow_plugins:
                 command.append('--no-plugins')
             command.extend(process_args)
-            process = self._popen(command, stdin=PIPE, stdout=PIPE, stderr=PIPE)
+            process = self._popen(command, stdin=subprocess.PIPE,
+                                  stdout=subprocess.PIPE,
+                                  stderr=subprocess.PIPE)
         finally:
             restore_environment()
             if cwd is not None:
@@ -2000,7 +1989,7 @@
         Allows tests to override this method to intercept the calls made to
         Popen for introspection.
         """
-        return Popen(*args, **kwargs)
+        return subprocess.Popen(*args, **kwargs)
 
     def get_source_path(self):
         """Return the path of the directory containing bzrlib."""
@@ -2008,7 +1997,7 @@
 
     def get_bzr_path(self):
         """Return the path of the 'bzr' executable for this test suite."""
-        bzr_path = self.get_source_path()+'/bzr'
+        bzr_path = os.path.join(self.get_source_path(), "bzr")
         if not os.path.isfile(bzr_path):
             # We are probably installed. Assume sys.argv is the right file
             bzr_path = sys.argv[0]
@@ -2186,7 +2175,7 @@
 
         :param relpath: a path relative to the base url.
         """
-        t = get_transport(self.get_url(relpath))
+        t = _mod_transport.get_transport(self.get_url(relpath))
         self.assertFalse(t.is_readonly())
         return t
 
@@ -2198,7 +2187,7 @@
 
         :param relpath: a path relative to the base url.
         """
-        t = get_transport(self.get_readonly_url(relpath))
+        t = _mod_transport.get_transport(self.get_readonly_url(relpath))
         self.assertTrue(t.is_readonly())
         return t
 
@@ -2334,7 +2323,7 @@
         propagating. This method ensures than a test did not leaked.
         """
         root = TestCaseWithMemoryTransport.TEST_ROOT
-        self.permit_url(get_transport(root).base)
+        self.permit_url(_mod_transport.get_transport(root).base)
         wt = workingtree.WorkingTree.open(root)
         last_rev = wt.last_revision()
         if last_rev != 'null:':
@@ -2385,7 +2374,7 @@
             # might be a relative or absolute path
             maybe_a_url = self.get_url(relpath)
             segments = maybe_a_url.rsplit('/', 1)
-            t = get_transport(maybe_a_url)
+            t = _mod_transport.get_transport(maybe_a_url)
             if len(segments) > 1 and segments[-1] not in ('', '.'):
                 t.ensure_base()
             if format is None:
@@ -2408,10 +2397,13 @@
         made_control = self.make_bzrdir(relpath, format=format)
         return made_control.create_repository(shared=shared)
 
-    def make_smart_server(self, path):
+    def make_smart_server(self, path, backing_server=None):
+        if backing_server is None:
+            backing_server = self.get_server()
         smart_server = test_server.SmartTCPServer_for_testing()
-        self.start_server(smart_server, self.get_server())
-        remote_transport = get_transport(smart_server.get_url()).clone(path)
+        self.start_server(smart_server, backing_server)
+        remote_transport = _mod_transport.get_transport(smart_server.get_url()
+                                                   ).clone(path)
         return remote_transport
 
     def make_branch_and_memory_tree(self, relpath, format=None):
@@ -2432,6 +2424,15 @@
 
     def setUp(self):
         super(TestCaseWithMemoryTransport, self).setUp()
+        # Ensure that ConnectedTransport doesn't leak sockets
+        def get_transport_with_cleanup(*args, **kwargs):
+            t = orig_get_transport(*args, **kwargs)
+            if isinstance(t, _mod_transport.ConnectedTransport):
+                self.addCleanup(t.disconnect)
+            return t
+
+        orig_get_transport = self.overrideAttr(_mod_transport, 'get_transport',
+                                               get_transport_with_cleanup)
         self._make_test_root()
         self.addCleanup(os.chdir, os.getcwdu())
         self.makeAndChdirToTestDir()
@@ -2482,7 +2483,11 @@
 
     def check_file_contents(self, filename, expect):
         self.log("check contents of file %s" % filename)
-        contents = file(filename, 'r').read()
+        f = file(filename)
+        try:
+            contents = f.read()
+        finally:
+            f.close()
         if contents != expect:
             self.log("expected: %r" % expect)
             self.log("actually: %r" % contents)
@@ -2562,7 +2567,7 @@
                 "a list or a tuple. Got %r instead" % (shape,))
         # It's OK to just create them using forward slashes on windows.
         if transport is None or transport.is_readonly():
-            transport = get_transport(".")
+            transport = _mod_transport.get_transport(".")
         for name in shape:
             self.assertIsInstance(name, basestring)
             if name[-1] == '/':
@@ -2578,8 +2583,7 @@
                 content = "contents of %s%s" % (name.encode('utf-8'), end)
                 transport.put_bytes_non_atomic(urlutils.escape(name), content)
 
-    def build_tree_contents(self, shape):
-        build_tree_contents(shape)
+    build_tree_contents = staticmethod(treeshape.build_tree_contents)
 
     def assertInWorkingTree(self, path, root_path='.', tree=None):
         """Assert whether path or paths are in the WorkingTree"""
@@ -2726,9 +2730,10 @@
     """
 
     def setUp(self):
+        from bzrlib.tests import http_server
         super(ChrootedTestCase, self).setUp()
         if not self.vfs_transport_factory == memory.MemoryServer:
-            self.transport_readonly_server = HttpServer
+            self.transport_readonly_server = http_server.HttpServer
 
 
 def condition_id_re(pattern):
@@ -2737,8 +2742,7 @@
     :param pattern: A regular expression string.
     :return: A callable that returns True if the re matches.
     """
-    filter_re = osutils.re_compile_checked(pattern, 0,
-        'test filter')
+    filter_re = re.compile(pattern, 0)
     def condition(test):
         test_id = test.id()
         return filter_re.search(test_id)
@@ -2996,6 +3000,9 @@
 
 
 def fork_decorator(suite):
+    if getattr(os, "fork", None) is None:
+        raise errors.BzrCommandError("platform does not support fork,"
+            " try --parallel=subprocess instead.")
     concurrency = osutils.local_concurrency()
     if concurrency == 1:
         return suite
@@ -3056,7 +3063,7 @@
     return suite
 
 
-class TestDecorator(TestSuite):
+class TestDecorator(TestUtil.TestSuite):
     """A decorator for TestCase/TestSuite objects.
     
     Usually, subclasses should override __iter__(used when flattening test
@@ -3065,7 +3072,7 @@
     """
 
     def __init__(self, suite):
-        TestSuite.__init__(self)
+        TestUtil.TestSuite.__init__(self)
         self.addTest(suite)
 
     def countTestCases(self):
@@ -3190,15 +3197,16 @@
 
 def partition_tests(suite, count):
     """Partition suite into count lists of tests."""
-    result = []
-    tests = list(iter_suite_tests(suite))
-    tests_per_process = int(math.ceil(float(len(tests)) / count))
-    for block in range(count):
-        low_test = block * tests_per_process
-        high_test = low_test + tests_per_process
-        process_tests = tests[low_test:high_test]
-        result.append(process_tests)
-    return result
+    # This just assigns tests in a round-robin fashion.  On one hand this
+    # splits up blocks of related tests that might run faster if they shared
+    # resources, but on the other it avoids assigning blocks of slow tests to
+    # just one partition.  So the slowest partition shouldn't be much slower
+    # than the fastest.
+    partitions = [list() for i in range(count)]
+    tests = iter_suite_tests(suite)
+    for partition, test in itertools.izip(itertools.cycle(partitions), tests):
+        partition.append(test)
+    return partitions
 
 
 def workaround_zealous_crypto_random():
@@ -3238,7 +3246,7 @@
 
     test_blocks = partition_tests(suite, concurrency)
     for process_tests in test_blocks:
-        process_suite = TestSuite()
+        process_suite = TestUtil.TestSuite()
         process_suite.addTests(process_tests)
         c2pread, c2pwrite = os.pipe()
         pid = os.fork()
@@ -3310,10 +3318,12 @@
                 '--subunit']
             if '--no-plugins' in sys.argv:
                 argv.append('--no-plugins')
-            # stderr=STDOUT would be ideal, but until we prevent noise on
-            # stderr it can interrupt the subunit protocol.
-            process = Popen(argv, stdin=PIPE, stdout=PIPE, stderr=PIPE,
-                bufsize=1)
+            # stderr=subprocess.STDOUT would be ideal, but until we prevent
+            # noise on stderr it can interrupt the subunit protocol.
+            process = subprocess.Popen(argv, stdin=subprocess.PIPE,
+                                      stdout=subprocess.PIPE,
+                                      stderr=subprocess.PIPE,
+                                      bufsize=1)
             test = TestInSubprocess(process, test_list_file_name)
             result.append(test)
         except:
@@ -3368,6 +3378,9 @@
 
     def startTest(self, test):
         self.profiler = bzrlib.lsprof.BzrProfiler()
+        # Prevent deadlocks in tests that use lsprof: those tests will
+        # unavoidably fail.
+        bzrlib.lsprof.BzrProfiler.profiler_block = 0
         self.profiler.start()
         ForwardingResult.startTest(self, test)
 
@@ -3394,6 +3407,8 @@
 #                           rather than failing tests. And no longer raise
 #                           LockContention when fctnl locks are not being used
 #                           with proper exclusion rules.
+#   -Ethreads               Will display thread ident at creation/join time to
+#                           help track thread leaks
 selftest_debug_flags = set()
 
 
@@ -3632,9 +3647,11 @@
         'bzrlib.doc',
         'bzrlib.tests.blackbox',
         'bzrlib.tests.commands',
+        'bzrlib.tests.doc_generate',
         'bzrlib.tests.per_branch',
         'bzrlib.tests.per_bzrdir',
-        'bzrlib.tests.per_bzrdir_colo',
+        'bzrlib.tests.per_controldir',
+        'bzrlib.tests.per_controldir_colo',
         'bzrlib.tests.per_foreign_vcs',
         'bzrlib.tests.per_interrepository',
         'bzrlib.tests.per_intertree',
@@ -3653,6 +3670,7 @@
         'bzrlib.tests.per_workingtree',
         'bzrlib.tests.test__annotator',
         'bzrlib.tests.test__bencode',
+        'bzrlib.tests.test__btree_serializer',
         'bzrlib.tests.test__chk_map',
         'bzrlib.tests.test__dirstate_helpers',
         'bzrlib.tests.test__groupcompress',
@@ -3701,6 +3719,7 @@
         'bzrlib.tests.test_export',
         'bzrlib.tests.test_extract',
         'bzrlib.tests.test_fetch',
+        'bzrlib.tests.test_fixtures',
         'bzrlib.tests.test_fifo_cache',
         'bzrlib.tests.test_filters',
         'bzrlib.tests.test_ftp_transport',
@@ -3727,6 +3746,7 @@
         'bzrlib.tests.test_knit',
         'bzrlib.tests.test_lazy_import',
         'bzrlib.tests.test_lazy_regex',
+        'bzrlib.tests.test_library_state',
         'bzrlib.tests.test_lock',
         'bzrlib.tests.test_lockable_files',
         'bzrlib.tests.test_lockdir',
@@ -3789,6 +3809,7 @@
         'bzrlib.tests.test_switch',
         'bzrlib.tests.test_symbol_versioning',
         'bzrlib.tests.test_tag',
+        'bzrlib.tests.test_test_server',
         'bzrlib.tests.test_testament',
         'bzrlib.tests.test_textfile',
         'bzrlib.tests.test_textmerge',
@@ -3800,6 +3821,7 @@
         'bzrlib.tests.test_transport_log',
         'bzrlib.tests.test_tree',
         'bzrlib.tests.test_treebuilder',
+        'bzrlib.tests.test_treeshape',
         'bzrlib.tests.test_tsort',
         'bzrlib.tests.test_tuned_gzip',
         'bzrlib.tests.test_ui',
@@ -3809,6 +3831,7 @@
         'bzrlib.tests.test_urlutils',
         'bzrlib.tests.test_version',
         'bzrlib.tests.test_version_info',
+        'bzrlib.tests.test_versionedfile',
         'bzrlib.tests.test_weave',
         'bzrlib.tests.test_whitebox',
         'bzrlib.tests.test_win32utils',
@@ -3836,6 +3859,7 @@
         'bzrlib.option',
         'bzrlib.symbol_versioning',
         'bzrlib.tests',
+        'bzrlib.tests.fixtures',
         'bzrlib.timestamp',
         'bzrlib.version_info_formats.format_custom',
         ]
@@ -3982,7 +4006,7 @@
     ...     bzrlib.tests.test_sampler.DemoTest('test_nothing'),
     ...     [('one', dict(param=1)),
     ...      ('two', dict(param=2))],
-    ...     TestSuite())
+    ...     TestUtil.TestSuite())
     >>> tests = list(iter_suite_tests(r))
     >>> len(tests)
     2
@@ -4035,8 +4059,20 @@
     :param new_id: The id to assign to it.
     :return: The new test.
     """
-    new_test = copy(test)
+    new_test = copy.copy(test)
     new_test.id = lambda: new_id
+    # XXX: Workaround <https://bugs.launchpad.net/testtools/+bug/637725>, which
+    # causes cloned tests to share the 'details' dict.  This makes it hard to
+    # read the test output for parameterized tests, because tracebacks will be
+    # associated with irrelevant tests.
+    try:
+        details = new_test._TestCase__details
+    except AttributeError:
+        # must be a different version of testtools than expected.  Do nothing.
+        pass
+    else:
+        # Reset the '__details' dict.
+        new_test._TestCase__details = {}
     return new_test
 
 
@@ -4102,8 +4138,11 @@
         if test_id != None:
             ui.ui_factory.clear_term()
             sys.stderr.write('\nWhile running: %s\n' % (test_id,))
+        # Ugly, but the last thing we want here is fail, so bear with it.
+        printable_e = str(e).decode(osutils.get_user_encoding(), 'replace'
+                                    ).encode('ascii', 'replace')
         sys.stderr.write('Unable to remove testing dir %s\n%s'
-                         % (os.path.basename(dirname), e))
+                         % (os.path.basename(dirname), printable_e))
 
 
 class Feature(object):
@@ -4339,6 +4378,17 @@
 UnicodeFilename = _UnicodeFilename()
 
 
+class _ByteStringNamedFilesystem(Feature):
+    """Is the filesystem based on bytes?"""
+
+    def _probe(self):
+        if os.name == "posix":
+            return True
+        return False
+
+ByteStringNamedFilesystem = _ByteStringNamedFilesystem()
+
+
 class _UTF8Filesystem(Feature):
     """Is the filesystem UTF-8?"""
 
@@ -4454,10 +4504,20 @@
 try:
     from subunit import TestProtocolClient
     from subunit.test_results import AutoTimingTestResultDecorator
+    class SubUnitBzrProtocolClient(TestProtocolClient):
+
+        def addSuccess(self, test, details=None):
+            # The subunit client always includes the details in the subunit
+            # stream, but we don't want to include it in ours.
+            if details is not None and 'log' in details:
+                del details['log']
+            return super(SubUnitBzrProtocolClient, self).addSuccess(
+                test, details)
+
     class SubUnitBzrRunner(TextTestRunner):
         def run(self, test):
             result = AutoTimingTestResultDecorator(
-                TestProtocolClient(self.stream))
+                SubUnitBzrProtocolClient(self.stream))
             test.run(result)
             return result
 except ImportError:

=== modified file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	2010-04-26 13:51:08 +0000
+++ bzrlib/tests/blackbox/__init__.py	2010-07-28 07:05:19 +0000
@@ -23,6 +23,10 @@
 """
 
 
+from bzrlib.symbol_versioning import (
+    deprecated_in,
+    deprecated_method,
+    )
 from bzrlib import tests
 
 
@@ -52,6 +56,7 @@
                      'bzrlib.tests.blackbox.test_commit',
                      'bzrlib.tests.blackbox.test_conflicts',
                      'bzrlib.tests.blackbox.test_debug',
+                     'bzrlib.tests.blackbox.test_deleted',
                      'bzrlib.tests.blackbox.test_diff',
                      'bzrlib.tests.blackbox.test_dump_btree',
                      'bzrlib.tests.blackbox.test_dpush',
@@ -71,6 +76,7 @@
                      'bzrlib.tests.blackbox.test_locale',
                      'bzrlib.tests.blackbox.test_log',
                      'bzrlib.tests.blackbox.test_logformats',
+                     'bzrlib.tests.blackbox.test_lookup_revision',
                      'bzrlib.tests.blackbox.test_ls',
                      'bzrlib.tests.blackbox.test_lsprof',
                      'bzrlib.tests.blackbox.test_merge',
@@ -124,7 +130,9 @@
 
 
 class ExternalBase(tests.TestCaseWithTransport):
+    """Don't use this class anymore, use TestCaseWithTransport or similar"""
 
+    @deprecated_method(deprecated_in((2, 2, 0)))
     def check_output(self, output, *args):
         """Verify that the expected output matches what bzr says.
 

=== modified file 'bzrlib/tests/blackbox/test_add.py'
--- bzrlib/tests/blackbox/test_add.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/blackbox/test_add.py	2010-07-16 13:53:26 +0000
@@ -23,6 +23,9 @@
     osutils,
     tests,
     )
+from bzrlib.tests import (
+    script,
+    )
 
 
 def load_tests(standard_tests, module, loader):
@@ -217,3 +220,14 @@
         os.symlink(osutils.abspath('target'), 'tree/link')
         out = self.run_bzr(['add', 'tree/link'])[0]
         self.assertEquals(out, 'adding link\n')
+
+    def test_add_not_child(self):
+        # https://bugs.launchpad.net/bzr/+bug/98735
+        sr = script.ScriptRunner()
+        self.make_branch_and_tree('tree1')
+        self.make_branch_and_tree('tree2')
+        self.build_tree(['tree1/a', 'tree2/b'])
+        sr.run_script(self, '''
+        $ bzr add tree1/a tree2/b
+        2>bzr: ERROR: Path "...tree2/b" is not a child of path "...tree1"
+        ''')

=== modified file 'bzrlib/tests/blackbox/test_added.py'
--- bzrlib/tests/blackbox/test_added.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_added.py	2010-06-11 07:32:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 # -*- coding: utf-8 -*-
 #
 # This program is free software; you can redistribute it and/or modify
@@ -21,10 +21,10 @@
 import os
 
 from bzrlib.branch import Branch
-from bzrlib.tests.blackbox import ExternalBase
-
-
-class TestAdded(ExternalBase):
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestAdded(TestCaseWithTransport):
 
     def test_added(self):
         """Test that 'added' command reports added files"""
@@ -67,3 +67,11 @@
         # after commit, now no longer listed
         tree.commit(message='add "%s"' % (name))
         check_added('')
+
+    def test_added_directory(self):
+        """Test --directory option"""
+        tree = self.make_branch_and_tree('a')
+        self.build_tree(['a/README'])
+        tree.add('README')
+        out, err = self.run_bzr(['added', '--directory=a'])
+        self.assertEquals('README\n', out)

=== modified file 'bzrlib/tests/blackbox/test_alias.py'
--- bzrlib/tests/blackbox/test_alias.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_alias.py	2010-09-23 07:40:07 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007 Canonical Ltd
+# Copyright (C) 2008, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,10 +17,13 @@
 
 """Tests of the 'bzr alias' command."""
 
-from bzrlib.tests.blackbox import ExternalBase
-
-
-class TestAlias(ExternalBase):
+from bzrlib import (
+    config,
+    tests,
+    )
+
+
+class TestAlias(tests.TestCaseWithTransport):
 
     def test_list_alias_with_none(self):
         """Calling alias with no parameters lists existing aliases."""
@@ -41,6 +44,25 @@
         out, err = self.run_bzr('alias commit')
         self.assertEquals('bzr alias commit="commit --strict"\n', out)
 
+    def test_unicode_alias(self):
+        """Unicode aliases should work (Bug #529930)"""
+        # XXX: strictly speaking, lack of unicode filenames doesn't imply that
+        # unicode command lines aren't available.
+        self.requireFeature(tests.UnicodeFilenameFeature)
+        file_name = u'foo\xb6'
+
+        tree = self.make_branch_and_tree('.')
+        self.build_tree([file_name])
+        tree.add(file_name)
+        tree.commit('added')
+
+        config.GlobalConfig.from_string(
+            u'[ALIASES]\nust=st %s\n' % (file_name,), save=True)
+
+        out, err = self.run_bzr('ust')
+        self.assertEquals(err, '')
+        self.assertEquals(out, '')
+
     def test_alias_listing_alphabetical(self):
         self.run_bzr('alias commit="commit --strict"')
         self.run_bzr('alias ll="log --short"')

=== modified file 'bzrlib/tests/blackbox/test_aliases.py'
--- bzrlib/tests/blackbox/test_aliases.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_aliases.py	2010-08-29 14:32:45 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,13 +18,13 @@
 
 import os
 
+from bzrlib import config
 from bzrlib.branch import Branch
-from bzrlib.tests.blackbox import ExternalBase
+from bzrlib.tests import TestCaseWithTransport
 from bzrlib.trace import mutter
-from bzrlib.config import (ensure_config_dir_exists, config_filename)
-
-
-class TestAliases(ExternalBase):
+
+
+class TestAliases(TestCaseWithTransport):
 
     def test_aliases(self):
 
@@ -35,18 +35,11 @@
             return self.run_bzr(args, **kwargs)[1]
 
 
-        if os.path.isfile(config_filename()):
-            # Something is wrong in environment,
-            # we risk overwriting users config
-            self.assert_(config_filename() + "exists, abort")
-
-        ensure_config_dir_exists()
-        CONFIG=("[ALIASES]\n"
-                "c=cat\n"
-                "c1=cat -r 1\n"
-                "c2=cat -r 1 -r2\n")
-
-        open(config_filename(),'wb').write(CONFIG)
+        conf = config.GlobalConfig.from_string('''[ALIASES]
+c=cat
+c1=cat -r 1
+c2=cat -r 1 -r2
+''', save=True)
 
         str1 = 'foo\n'
         str2 = 'bar\n'

=== modified file 'bzrlib/tests/blackbox/test_annotate.py'
--- bzrlib/tests/blackbox/test_annotate.py	2010-03-24 14:15:01 +0000
+++ bzrlib/tests/blackbox/test_annotate.py	2010-05-02 20:10:25 +0000
@@ -278,3 +278,12 @@
         os.chdir('branch')
         out, err = self.run_bzr('annotate empty')
         self.assertEqual('', out)
+
+    def test_annotate_directory(self):
+        """Test --directory option"""
+        wt = self.make_branch_and_tree('a')
+        self.build_tree_contents([('a/hello.txt', 'my helicopter\n')])
+        wt.add(['hello.txt'])
+        wt.commit('commit', committer='test@user')
+        out, err = self.run_bzr('annotate -d a hello.txt')
+        self.assertEqualDiff('1   test@us | my helicopter\n', out)

=== modified file 'bzrlib/tests/blackbox/test_bound_branches.py'
--- bzrlib/tests/blackbox/test_bound_branches.py	2010-02-04 16:06:36 +0000
+++ bzrlib/tests/blackbox/test_bound_branches.py	2010-05-02 20:10:25 +0000
@@ -422,6 +422,21 @@
         self.check_revno(4)
         self.check_revno(4, '../base')
 
+    def test_bind_directory(self):
+        """Test --directory option"""
+        tree = self.make_branch_and_tree('base')
+        self.build_tree(['base/a', 'base/b'])
+        tree.add('a', 'b')
+        tree.commit(message='init')
+        branch = tree.branch
+        tree.bzrdir.sprout('child')
+        self.run_bzr('bind --directory=child base')
+        d = BzrDir.open('child')
+        self.assertNotEqual(None, d.open_branch().get_master_branch())
+        self.run_bzr('unbind -d child')
+        self.assertEqual(None, d.open_branch().get_master_branch())
+        self.run_bzr('unbind --directory child', retcode=3)
+
 
 class TestBind(script.TestCaseWithTransportAndScript):
 

=== modified file 'bzrlib/tests/blackbox/test_branch.py'
--- bzrlib/tests/blackbox/test_branch.py	2010-04-22 17:08:27 +0000
+++ bzrlib/tests/blackbox/test_branch.py	2010-07-21 20:09:34 +0000
@@ -27,7 +27,7 @@
     revision as _mod_revision,
     )
 from bzrlib.repofmt.knitrepo import RepositoryFormatKnit1
-from bzrlib.tests.blackbox import ExternalBase
+from bzrlib.tests import TestCaseWithTransport
 from bzrlib.tests import (
     KnownFailure,
     HardlinkFeature,
@@ -38,7 +38,7 @@
 from bzrlib.workingtree import WorkingTree
 
 
-class TestBranch(ExternalBase):
+class TestBranch(TestCaseWithTransport):
 
     def example_branch(self, path='.'):
         tree = self.make_branch_and_tree(path)
@@ -174,6 +174,29 @@
         target_stat = os.stat('target/file1')
         self.assertEqual(source_stat, target_stat)
 
+    def test_branch_files_from(self):
+        source = self.make_branch_and_tree('source')
+        self.build_tree(['source/file1'])
+        source.add('file1')
+        source.commit('added file')
+        out, err = self.run_bzr('branch source target --files-from source')
+        self.failUnlessExists('target/file1')
+
+    def test_branch_files_from_hardlink(self):
+        self.requireFeature(HardlinkFeature)
+        source = self.make_branch_and_tree('source')
+        self.build_tree(['source/file1'])
+        source.add('file1')
+        source.commit('added file')
+        source.bzrdir.sprout('second')
+        out, err = self.run_bzr('branch source target --files-from second'
+                                ' --hardlink')
+        source_stat = os.stat('source/file1')
+        second_stat = os.stat('second/file1')
+        target_stat = os.stat('target/file1')
+        self.assertNotEqual(source_stat, target_stat)
+        self.assertEqual(second_stat, target_stat)
+
     def test_branch_standalone(self):
         shared_repo = self.make_repository('repo', shared=True)
         self.example_branch('source')
@@ -248,7 +271,7 @@
         self.assertLength(2, calls)
 
 
-class TestBranchStacked(ExternalBase):
+class TestBranchStacked(TestCaseWithTransport):
     """Tests for branch --stacked"""
 
     def assertRevisionInRepository(self, repo_path, revid):
@@ -376,7 +399,7 @@
             err)
 
 
-class TestSmartServerBranching(ExternalBase):
+class TestSmartServerBranching(TestCaseWithTransport):
 
     def test_branch_from_trivial_branch_to_same_server_branch_acceptance(self):
         self.setup_smart_server_with_call_log()

=== modified file 'bzrlib/tests/blackbox/test_break_lock.py'
--- bzrlib/tests/blackbox/test_break_lock.py	2010-03-25 09:39:03 +0000
+++ bzrlib/tests/blackbox/test_break_lock.py	2010-09-14 07:19:22 +0000
@@ -16,19 +16,21 @@
 
 """Tests for lock-breaking user interface"""
 
-import os
-
-import bzrlib
 from bzrlib import (
+    branch,
+    bzrdir,
+    config,
     errors,
-    lockdir,
-    )
-from bzrlib.branch import Branch
-from bzrlib.bzrdir import BzrDir
-from bzrlib.tests.blackbox import ExternalBase
-
-
-class TestBreakLock(ExternalBase):
+    osutils,
+    tests,
+    )
+from bzrlib.tests.script import (
+    ScriptRunner,
+    run_script,
+    )
+
+
+class TestBreakLock(tests.TestCaseWithTransport):
 
     # General principal for break-lock: All the elements that might be locked
     # by a bzr operation on PATH, are candidates that break-lock may unlock.
@@ -52,14 +54,14 @@
              'repo/',
              'repo/branch/',
              'checkout/'])
-        bzrlib.bzrdir.BzrDir.create('master-repo').create_repository()
-        self.master_branch = bzrlib.bzrdir.BzrDir.create_branch_convenience(
+        bzrdir.BzrDir.create('master-repo').create_repository()
+        self.master_branch = bzrdir.BzrDir.create_branch_convenience(
             'master-repo/master-branch')
-        bzrlib.bzrdir.BzrDir.create('repo').create_repository()
-        local_branch = bzrlib.bzrdir.BzrDir.create_branch_convenience('repo/branch')
+        bzrdir.BzrDir.create('repo').create_repository()
+        local_branch = bzrdir.BzrDir.create_branch_convenience('repo/branch')
         local_branch.bind(self.master_branch)
-        checkoutdir = bzrlib.bzrdir.BzrDir.create('checkout')
-        bzrlib.branch.BranchReferenceFormat().initialize(
+        checkoutdir = bzrdir.BzrDir.create('checkout')
+        branch.BranchReferenceFormat().initialize(
             checkoutdir, target_branch=local_branch)
         self.wt = checkoutdir.create_workingtree()
 
@@ -68,12 +70,22 @@
         # shouldn't fail and should not produce error output
         self.assertEqual('', err)
 
+    def test_break_lock_no_interaction(self):
+        """With --force, the user isn't asked for confirmation"""
+        self.master_branch.lock_write()
+        run_script(self, """
+        $ bzr break-lock --force master-repo/master-branch
+        Broke lock ...master-branch/.bzr/...
+        """)
+        # lock should now be dead
+        self.assertRaises(errors.LockBroken, self.master_branch.unlock)
+
     def test_break_lock_everything_locked(self):
         ### if everything is locked, we should be able to unlock the lot.
         # however, we dont test breaking the working tree because we
         # cannot accurately do so right now: the dirstate lock is held
         # by an os lock, and we need to spawn a separate process to lock it
-        # thne kill -9 it.
+        # then kill -9 it.
         # sketch of test:
         # lock most of the dir:
         self.wt.branch.lock_write()
@@ -82,22 +94,43 @@
         # we need 5 yes's - wt, branch, repo, bound branch, bound repo.
         self.run_bzr('break-lock checkout', stdin="y\ny\ny\ny\n")
         # a new tree instance should be lockable
-        branch = bzrlib.branch.Branch.open('checkout')
-        branch.lock_write()
-        branch.unlock()
+        br = branch.Branch.open('checkout')
+        br.lock_write()
+        br.unlock()
         # and a new instance of the master branch
-        mb = branch.get_master_branch()
+        mb = br.get_master_branch()
         mb.lock_write()
         mb.unlock()
         self.assertRaises(errors.LockBroken, self.wt.unlock)
         self.assertRaises(errors.LockBroken, self.master_branch.unlock)
 
 
-class TestBreakLockOldBranch(ExternalBase):
+class TestBreakLockOldBranch(tests.TestCaseWithTransport):
 
     def test_break_lock_format_5_bzrdir(self):
         # break lock on a format 5 bzrdir should just return
-        self.make_branch_and_tree('foo', format=bzrlib.bzrdir.BzrDirFormat5())
+        self.make_branch_and_tree('foo', format=bzrdir.BzrDirFormat5())
         out, err = self.run_bzr('break-lock foo')
         self.assertEqual('', out)
         self.assertEqual('', err)
+
+class TestConfigBreakLock(tests.TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestConfigBreakLock, self).setUp()
+        self.config_file_name = './my.conf'
+        self.build_tree_contents([(self.config_file_name,
+                                   '[DEFAULT]\none=1\n')])
+        self.config = config.LockableConfig(file_name=self.config_file_name)
+        self.config.lock_write()
+
+    def test_create_pending_lock(self):
+        self.addCleanup(self.config.unlock)
+        self.assertTrue(self.config._lock.is_held)
+
+    def test_break_lock(self):
+        self.run_bzr('break-lock --config %s'
+                     % osutils.dirname(self.config_file_name),
+                     stdin="y\n")
+        self.assertRaises(errors.LockBroken, self.config.unlock)
+

=== modified file 'bzrlib/tests/blackbox/test_cat.py'
--- bzrlib/tests/blackbox/test_cat.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/blackbox/test_cat.py	2010-05-02 18:16:37 +0000
@@ -199,3 +199,22 @@
         self.vfs_transport_factory = memory.MemoryServer
         self.run_bzr_error(['^bzr: ERROR: Not a branch'],
                            ['cat', self.get_url()])
+
+    def test_cat_directory(self):
+        wt = self.make_branch_and_tree('a')
+        self.build_tree(['a/README'])
+        wt.add('README')
+        wt.commit('Making sure there is a basis_tree available')
+
+        out, err = self.run_bzr_subprocess(['cat', '--directory=a', 'README'])
+        self.assertEqual('contents of a/README\n', out)
+
+    def test_cat_remote_directory(self):
+        wt = self.make_branch_and_tree('a')
+        self.build_tree(['a/README'])
+        wt.add('README')
+        wt.commit('Making sure there is a basis_tree available')
+
+        url = self.get_readonly_url() + '/a'
+        out, err = self.run_bzr_subprocess(['cat', '-d', url, 'README'])
+        self.assertEqual('contents of a/README\n', out)

=== modified file 'bzrlib/tests/blackbox/test_cat_revision.py'
--- bzrlib/tests/blackbox/test_cat_revision.py	2010-02-15 20:23:34 +0000
+++ bzrlib/tests/blackbox/test_cat_revision.py	2010-06-23 08:19:28 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2004, 2005, 2007 Canonical Ltd
+# Copyright (C) 2007-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,10 +14,11 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from bzrlib.tests import blackbox
-
-
-class TestCatRevision(blackbox.ExternalBase):
+
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestCatRevision(TestCaseWithTransport):
 
     def test_cat_unicode_revision(self):
         tree = self.make_branch_and_tree('.')
@@ -47,21 +48,23 @@
         finally:
             r.unlock()
 
-        self.check_output(revs[1], 'cat-revision a@r-0-1')
-        self.check_output(revs[2], 'cat-revision a@r-0-2')
-        self.check_output(revs[3], 'cat-revision a@r-0-3')
-
-        self.check_output(revs[1], 'cat-revision -r 1')
-        self.check_output(revs[2], 'cat-revision -r 2')
-        self.check_output(revs[3], 'cat-revision -r 3')
-
-        self.check_output(revs[1], 'cat-revision -r revid:a@r-0-1')
-        self.check_output(revs[2], 'cat-revision -r revid:a@r-0-2')
-        self.check_output(revs[3], 'cat-revision -r revid:a@r-0-3')
+        for i in [1, 2, 3]:
+            self.assertEqual(revs[i],
+                self.run_bzr('cat-revision -r revid:a@r-0-%d' % i)[0])
+            self.assertEqual(revs[i],
+                self.run_bzr('cat-revision a@r-0-%d' % i)[0])
+            self.assertEqual(revs[i],
+                self.run_bzr('cat-revision -r %d' % i)[0])
 
     def test_cat_no_such_revid(self):
         tree = self.make_branch_and_tree('.')
         err = self.run_bzr('cat-revision abcd', retcode=3)[1]
         self.assertContainsRe(err, 'The repository .* contains no revision abcd.')
 
-
+    def test_cat_revision_directory(self):
+        """Test --directory option"""
+        tree = self.make_branch_and_tree('a')
+        tree.commit('This revision', rev_id='abcd')
+        output, errors = self.run_bzr(['cat-revision', '-d', 'a', u'abcd'])
+        self.assertContainsRe(output, 'This revision')
+        self.assertEqual('', errors)

=== modified file 'bzrlib/tests/blackbox/test_check.py'
--- bzrlib/tests/blackbox/test_check.py	2009-08-04 04:36:34 +0000
+++ bzrlib/tests/blackbox/test_check.py	2010-06-23 08:19:28 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007, 2009 Canonical Ltd
+# Copyright (C) 2007-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,10 +17,10 @@
 """Tests for the 'check' CLI command."""
 
 from bzrlib.tests import ChrootedTestCase
-from bzrlib.tests.blackbox import ExternalBase
-
-
-class TestCheck(ExternalBase):
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestCheck(TestCaseWithTransport):
 
     def test_check_no_tree(self):
         self.make_branch('.')

=== modified file 'bzrlib/tests/blackbox/test_checkout.py'
--- bzrlib/tests/blackbox/test_checkout.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/blackbox/test_checkout.py	2010-07-20 10:18:05 +0000
@@ -28,8 +28,8 @@
     errors,
     workingtree,
     )
-from bzrlib.tests.blackbox import (
-    ExternalBase,
+from bzrlib.tests import (
+    TestCaseWithTransport,
     )
 from bzrlib.tests import (
     HardlinkFeature,
@@ -37,7 +37,7 @@
     )
 
 
-class TestCheckout(ExternalBase):
+class TestCheckout(TestCaseWithTransport):
 
     def setUp(self):
         super(TestCheckout, self).setUp()
@@ -64,7 +64,7 @@
                          result.open_branch().bzrdir.root_transport.base)
 
     def test_checkout_dash_r(self):
-        self.run_bzr('checkout -r -2 branch checkout')
+        out, err = self.run_bzr(['checkout', '-r', '-2', 'branch', 'checkout'])
         # the working tree should now be at revision '1' with the content
         # from 1.
         result = bzrdir.BzrDir.open('checkout')
@@ -72,7 +72,8 @@
         self.failIfExists('checkout/added_in_2')
 
     def test_checkout_light_dash_r(self):
-        self.run_bzr('checkout --lightweight -r -2 branch checkout')
+        out, err = self.run_bzr(['checkout','--lightweight', '-r', '-2',
+            'branch', 'checkout'])
         # the working tree should now be at revision '1' with the content
         # from 1.
         result = bzrdir.BzrDir.open('checkout')
@@ -155,9 +156,20 @@
         self.build_tree(['source/file1'])
         source.add('file1')
         source.commit('added file')
-        out, err = self.run_bzr(['checkout', 'source', 'target',
-            '--files-from', 'source',
-            '--hardlink'])
+        out, err = self.run_bzr('checkout source target --hardlink')
         source_stat = os.stat('source/file1')
         target_stat = os.stat('target/file1')
         self.assertEqual(source_stat, target_stat)
+
+    def test_checkout_hardlink_files_from(self):
+        self.requireFeature(HardlinkFeature)
+        source = self.make_branch_and_tree('source')
+        self.build_tree(['source/file1'])
+        source.add('file1')
+        source.commit('added file')
+        source.bzrdir.sprout('second')
+        out, err = self.run_bzr('checkout source target --hardlink'
+                                ' --files-from second')
+        second_stat = os.stat('second/file1')
+        target_stat = os.stat('target/file1')
+        self.assertEqual(second_stat, target_stat)

=== modified file 'bzrlib/tests/blackbox/test_clean_tree.py'
--- bzrlib/tests/blackbox/test_clean_tree.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_clean_tree.py	2010-05-20 02:57:52 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2009 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 
 import os
 
+from bzrlib import ignores
 from bzrlib.tests import TestCaseWithTransport
 
 
@@ -62,3 +63,28 @@
         self.failIfExists('name')
         self.failIfExists('name~')
         self.failIfExists('name.pyc')
+
+    def test_clean_tree_nested_bzrdir(self):
+        # clean-tree should not blindly delete nested bzrdirs (branches)
+        # bug https://bugs.launchpad.net/bzr/+bug/572098
+        # so it will play well with scmproj/bzr-externals plugins.
+        wt1 = self.make_branch_and_tree('.')
+        wt2 = self.make_branch_and_tree('foo')
+        wt3 = self.make_branch_and_tree('bar')
+        ignores.tree_ignores_add_patterns(wt1, ['./foo'])
+        self.run_bzr(['clean-tree', '--unknown', '--force'])
+        self.failUnlessExists('foo')
+        self.failUnlessExists('bar')
+        self.run_bzr(['clean-tree', '--ignored', '--force'])
+        self.failUnlessExists('foo')
+        self.failUnlessExists('bar')
+
+    def test_clean_tree_directory(self):
+        """Test --directory option"""
+        tree = self.make_branch_and_tree('a')
+        self.build_tree(['a/added', 'a/unknown', 'a/ignored'])
+        tree.add('added')
+        self.run_bzr('clean-tree -d a --unknown --ignored --force')
+        self.failIfExists('a/unknown')
+        self.failIfExists('a/ignored')
+        self.failUnlessExists('a/added')

=== modified file 'bzrlib/tests/blackbox/test_commit.py'
--- bzrlib/tests/blackbox/test_commit.py	2010-04-07 22:22:27 +0000
+++ bzrlib/tests/blackbox/test_commit.py	2010-09-23 02:56:19 +0000
@@ -18,9 +18,11 @@
 """Tests for the commit CLI of bzr."""
 
 import os
+import re
 import sys
 
 from bzrlib import (
+    bzrdir,
     osutils,
     ignores,
     msgeditor,
@@ -31,11 +33,12 @@
 from bzrlib.tests import (
     probe_bad_non_ascii,
     TestSkipped,
+    UnicodeFilenameFeature,
     )
-from bzrlib.tests.blackbox import ExternalBase
-
-
-class TestCommit(ExternalBase):
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestCommit(TestCaseWithTransport):
 
     def test_05_empty_commit(self):
         """Commit of tree with no versioned files should fail"""
@@ -107,6 +110,41 @@
                               'modified hello\.txt\n'
                               'Committed revision 2\.\n$')
 
+    def test_unicode_commit_message_is_filename(self):
+        """Unicode commit message same as a filename (Bug #563646).
+        """
+        self.requireFeature(UnicodeFilenameFeature)
+        file_name = u'\N{euro sign}'
+        self.run_bzr(['init'])
+        open(file_name, 'w').write('hello world')
+        self.run_bzr(['add'])
+        out, err = self.run_bzr(['commit', '-m', file_name])
+        reflags = re.MULTILINE|re.DOTALL|re.UNICODE
+        te = osutils.get_terminal_encoding()
+        self.assertContainsRe(err.decode(te),
+            u'The commit message is a file name:',
+            flags=reflags)
+
+        # Run same test with a filename that causes encode
+        # error for the terminal encoding. We do this
+        # by forcing terminal encoding of ascii for
+        # osutils.get_terminal_encoding which is used
+        # by ui.text.show_warning
+        default_get_terminal_enc = osutils.get_terminal_encoding
+        try:
+            osutils.get_terminal_encoding = lambda trace=None: 'ascii'
+            file_name = u'foo\u1234'
+            open(file_name, 'w').write('hello world')
+            self.run_bzr(['add'])
+            out, err = self.run_bzr(['commit', '-m', file_name])
+            reflags = re.MULTILINE|re.DOTALL|re.UNICODE
+            te = osutils.get_terminal_encoding()
+            self.assertContainsRe(err.decode(te, 'replace'),
+                u'The commit message is a file name:',
+                flags=reflags)
+        finally:
+            osutils.get_terminal_encoding = default_get_terminal_enc
+
     def test_warn_about_forgotten_commit_message(self):
         """Test that the lack of -m parameter is caught"""
         wt = self.make_branch_and_tree('.')
@@ -663,7 +701,7 @@
         self.assertContainsRe(err, r'modified test\nCommitted revision 2.')
 
     def test_commit_readonly_checkout(self):
-        # https://bugs.edge.launchpad.net/bzr/+bug/129701
+        # https://bugs.launchpad.net/bzr/+bug/129701
         # "UnlockableTransport error trying to commit in checkout of readonly
         # branch"
         self.make_branch('master')
@@ -710,3 +748,30 @@
         out, err = self.run_bzr_error(["empty commit message"],
             "commit tree/hello.txt", stdin="n\n")
         self.assertEqual(expected, tree.last_revision())
+
+    def test_commit_without_username(self):
+        """Ensure commit error if username is not set.
+        """
+        self.run_bzr(['init', 'foo'])
+        os.chdir('foo')
+        open('foo.txt', 'w').write('hello')
+        self.run_bzr(['add'])
+        osutils.set_or_unset_env('EMAIL', None)
+        osutils.set_or_unset_env('BZR_EMAIL', None)
+        out, err = self.run_bzr(['commit', '-m', 'initial'], 3)
+        self.assertContainsRe(err, 'Unable to determine your name')
+
+    def test_commit_recursive_checkout(self):
+        """Ensure that a commit to a recursive checkout fails cleanly.
+        """
+        self.run_bzr(['init', 'test_branch'])
+        self.run_bzr(['checkout', 'test_branch', 'test_checkout'])
+        os.chdir('test_checkout')
+        self.run_bzr(['bind', '.']) # bind to self
+        open('foo.txt', 'w').write('hello')
+        self.run_bzr(['add'])
+        out, err = self.run_bzr(['commit', '-m', 'addedfoo'], 3)
+        self.assertEqual(out, '')
+        self.assertContainsRe(err,
+            'Branch.*test_checkout.*appears to be bound to itself')
+

=== modified file 'bzrlib/tests/blackbox/test_conflicts.py'
--- bzrlib/tests/blackbox/test_conflicts.py	2010-02-10 17:52:08 +0000
+++ bzrlib/tests/blackbox/test_conflicts.py	2010-05-28 14:15:28 +0000
@@ -70,6 +70,12 @@
         out, err = self.run_bzr('conflicts --text')
         self.assertEqual(['my_other_file', 'myfile'], out.splitlines())
 
+    def test_conflicts_directory(self):
+        """Test --directory option"""
+        out, err = self.run_bzr('conflicts --directory a', working_dir='.')
+        self.assertEqual(3, len(out.splitlines()))
+        self.assertEqual('', err)
+
 
 class TestResolve(TestConflictsBase):
 
@@ -113,3 +119,10 @@
         self.build_tree_contents([('tree/file', 'a\n')])
         note = self.run_bzr('resolve', working_dir='tree')[1]
         self.assertContainsRe(note, 'All conflicts resolved.')
+
+    def test_resolve_all_directory(self):
+        """Test --directory option"""
+        out, err = self.run_bzr('resolve --all -d a', working_dir='.')
+        self.assertEqual('', err)
+        out, err = self.run_bzr('conflicts')
+        self.assertEqual(0, len(out.splitlines()))

=== added file 'bzrlib/tests/blackbox/test_deleted.py'
--- bzrlib/tests/blackbox/test_deleted.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_deleted.py	2010-06-11 07:32:12 +0000
@@ -0,0 +1,37 @@
+# Copyright (C) 2010 Canonical Ltd
+# -*- coding: utf-8 -*-
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+
+"""Black-box tests for 'bzr deleted', which shows newly deleted files."""
+
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestDeleted(TestCaseWithTransport):
+
+    def test_deleted_directory(self):
+        """Test --directory option"""
+        tree = self.make_branch_and_tree('a')
+        self.build_tree(['a/README'])
+        tree.add('README')
+        tree.commit('r1')
+        tree.remove('README')
+        out, err = self.run_bzr(['deleted', '--directory=a'])
+        self.assertEquals('README\n', out)

=== modified file 'bzrlib/tests/blackbox/test_diff.py'
--- bzrlib/tests/blackbox/test_diff.py	2010-04-05 21:56:39 +0000
+++ bzrlib/tests/blackbox/test_diff.py	2010-07-22 13:19:48 +0000
@@ -29,6 +29,9 @@
     DiffTree,
     format_registry as diff_format_registry,
     )
+from bzrlib.tests import (
+    features,
+    )
 
 
 def subst_dates(string):
@@ -321,7 +324,6 @@
         output = self.run_bzr('diff --format=boo', retcode=1)
         self.assertTrue("BOO!" in output[0])
 
-
 class TestCheckoutDiff(TestDiff):
 
     def make_example_branch(self):
@@ -400,6 +402,16 @@
         self.assertEndsWith(out, "\n@@ -0,0 +1 @@\n"
                                  "+baz\n\n")
 
+    def test_external_diff_options_and_using(self):
+        """Test that the options are passed correctly to an external diff process"""
+        self.requireFeature(features.diff_feature)
+        self.make_example_branch()
+        self.build_tree_contents([('hello', 'Foo\n')])
+        out, err = self.run_bzr('diff --diff-options -i --using diff',
+                                    retcode=1)
+        self.assertEquals("=== modified file 'hello'\n", out)
+        self.assertEquals('', err)
+
 
 class TestDiffOutput(DiffBase):
 

=== modified file 'bzrlib/tests/blackbox/test_dpush.py'
--- bzrlib/tests/blackbox/test_dpush.py	2010-04-28 10:30:48 +0000
+++ bzrlib/tests/blackbox/test_dpush.py	2010-06-23 08:19:28 +0000
@@ -29,6 +29,7 @@
     )
 from bzrlib.tests import (
     blackbox,
+    script,
     test_foreign,
     )
 from bzrlib.tests.blackbox import test_push
@@ -58,7 +59,7 @@
     return result
 
 
-class TestDpush(blackbox.ExternalBase):
+class TestDpush(tests.TestCaseWithTransport):
 
     def setUp(self):
         super(TestDpush, self).setUp()
@@ -86,9 +87,11 @@
         self.build_tree(("dc/foo", "blaaaa"))
         dc.open_workingtree().commit('msg')
 
-        output, error = self.run_bzr("dpush -d dc d")
-        self.assertEquals(error, "Pushed up to revision 2.\n")
-        self.check_output("", "status dc")
+        script.run_script(self, """
+            $ bzr dpush -d dc d
+            2>Pushed up to revision 2.
+            $ bzr status dc
+            """)
 
     def test_dpush_new(self):
         b = self.make_dummy_builder('d').get_branch()
@@ -99,9 +102,12 @@
         dc_tree.add("foofile")
         dc_tree.commit("msg")
 
-        self.check_output("", "dpush -d dc d")
-        self.check_output("2\n", "revno dc")
-        self.check_output("", "status dc")
+        script.run_script(self, '''
+            $ bzr dpush -d dc d
+            $ bzr revno dc
+            2
+            $ bzr status dc
+            ''')
 
     def test_dpush_wt_diff(self):
         b = self.make_dummy_builder('d').get_branch()
@@ -113,11 +119,17 @@
         newrevid = dc_tree.commit('msg')
 
         self.build_tree_contents([("dc/foofile", "blaaaal")])
-        self.check_output("", "dpush -d dc d --no-strict")
+        script.run_script(self, '''
+            $ bzr dpush -d dc d --no-strict
+            ''')
         self.assertFileEqual("blaaaal", "dc/foofile")
         # if the dummy vcs wasn't that dummy we could uncomment the line below
         # self.assertFileEqual("blaaaa", "d/foofile")
-        self.check_output('modified:\n  foofile\n', "status dc")
+        script.run_script(self, '''
+            $ bzr status dc
+            modified:
+              foofile
+            ''')
 
     def test_diverged(self):
         builder = self.make_dummy_builder('d')

=== modified file 'bzrlib/tests/blackbox/test_export.py'
--- bzrlib/tests/blackbox/test_export.py	2010-04-19 04:10:53 +0000
+++ bzrlib/tests/blackbox/test_export.py	2010-09-23 02:56:19 +0000
@@ -30,10 +30,10 @@
     export,
     tests,
     )
-from bzrlib.tests.blackbox import ExternalBase
-
-
-class TestExport(ExternalBase):
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestExport(TestCaseWithTransport):
 
     def test_tar_export(self):
         tree = self.make_branch_and_tree('tar')
@@ -66,15 +66,13 @@
         self.assertEqual(['test/a'], sorted(ball.getnames()))
 
     def test_tar_export_unicode(self):
+        self.requireFeature(tests.UnicodeFilenameFeature)
         tree = self.make_branch_and_tree('tar')
         # FIXME: using fname = u'\xe5.txt' below triggers a bug revealed since
         # bzr.dev revno 4216 but more related to OSX/working trees/unicode than
         # export itself --vila 20090406
         fname = u'\N{Euro Sign}.txt'
-        try:
-            self.build_tree(['tar/' + fname])
-        except UnicodeError:
-            raise tests.TestSkipped('Unable to represent path %r' % (fname,))
+        self.build_tree(['tar/' + fname])
         tree.add([fname])
         tree.commit('first')
 
@@ -87,6 +85,7 @@
 
     def test_tar_export_unicode_basedir(self):
         """Test for bug #413406"""
+        self.requireFeature(tests.UnicodeFilenameFeature)
         basedir = u'\N{euro sign}'
         os.mkdir(basedir)
         os.chdir(basedir)
@@ -119,12 +118,10 @@
         self.assertEqual(['test/a'], sorted(zfile.namelist()))
 
     def test_zip_export_unicode(self):
+        self.requireFeature(tests.UnicodeFilenameFeature)
         tree = self.make_branch_and_tree('zip')
         fname = u'\N{Euro Sign}.txt'
-        try:
-            self.build_tree(['zip/' + fname])
-        except UnicodeError:
-            raise tests.TestSkipped('Unable to represent path %r' % (fname,))
+        self.build_tree(['zip/' + fname])
         tree.add([fname])
         tree.commit('first')
 
@@ -313,3 +310,9 @@
         har_st = os.stat('t/har')
         self.assertEquals(315532800, har_st.st_mtime)
 
+    def test_export_directory(self):
+        """Test --directory option"""
+        self.example_branch()
+        self.run_bzr(['export', '--directory=branch', 'latest'])
+        self.assertEqual(['goodbye', 'hello'], sorted(os.listdir('latest')))
+        self.check_file_contents('latest/goodbye', 'baz')

=== modified file 'bzrlib/tests/blackbox/test_filesystem_cicp.py'
--- bzrlib/tests/blackbox/test_filesystem_cicp.py	2010-04-28 07:53:34 +0000
+++ bzrlib/tests/blackbox/test_filesystem_cicp.py	2010-06-23 08:19:28 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2010 Canonical Ltd
+# Copyright (C) 2008, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,13 +19,16 @@
 
 import os
 
-from bzrlib import (osutils,
+from bzrlib import (
+    osutils,
+    tests,
     )
-from bzrlib.tests.blackbox import ExternalBase
 from bzrlib.tests import CaseInsCasePresFilenameFeature, KnownFailure
 from bzrlib.osutils import canonical_relpath, pathjoin
-
-class TestCICPBase(ExternalBase):
+from bzrlib.tests.script import run_script
+
+
+class TestCICPBase(tests.TestCaseWithTransport):
     """Base class for tests on a case-insensitive, case-preserving filesystem.
     """
 
@@ -42,16 +45,6 @@
                                  ])
         return wt
 
-    def check_error_output(self, retcode, output, *args):
-        got = self.run_bzr(retcode=retcode, *args)[1]
-        self.failUnlessEqual(got, output)
-
-    def check_empty_output(self, *args):
-        """Check a bzr command generates no output anywhere and exits with 0"""
-        out, err = self.run_bzr(retcode=0, *args)
-        self.failIf(out)
-        self.failIf(err)
-
 
 class TestAdd(TestCICPBase):
 
@@ -60,28 +53,32 @@
         wt = self.make_branch_and_tree('.')
         # create a file on disk with the mixed-case name
         self.build_tree(['CamelCase'])
-
-        self.check_output('adding CamelCase\n', 'add camelcase')
+        run_script(self, """
+            $ bzr add camelcase
+            adding CamelCase
+            """)
 
     def test_add_subdir(self):
         """test_add_simple but with subdirectories tested too."""
         wt = self.make_branch_and_tree('.')
         # create a file on disk with the mixed-case parent and base name
         self.build_tree(['CamelCaseParent/', 'CamelCaseParent/CamelCase'])
-
-        self.check_output('adding CamelCaseParent\n'
-                          'adding CamelCaseParent/CamelCase\n',
-                          'add camelcaseparent/camelcase')
+        run_script(self, """
+            $ bzr add camelcaseparent/camelcase
+            adding CamelCaseParent
+            adding CamelCaseParent/CamelCase
+            """)
 
     def test_add_implied(self):
         """test add with no args sees the correct names."""
         wt = self.make_branch_and_tree('.')
         # create a file on disk with the mixed-case parent and base name
         self.build_tree(['CamelCaseParent/', 'CamelCaseParent/CamelCase'])
-
-        self.check_output('adding CamelCaseParent\n'
-                          'adding CamelCaseParent/CamelCase\n',
-                          'add')
+        run_script(self, """
+            $ bzr add
+            adding CamelCaseParent
+            adding CamelCaseParent/CamelCase
+            """)
 
     def test_re_add(self):
         """Test than when a file has 'unintentionally' changed case, we can't
@@ -89,10 +86,15 @@
         wt = self.make_branch_and_tree('.')
         # create a file on disk with the mixed-case name
         self.build_tree(['MixedCase'])
-        self.check_output('adding MixedCase\n', 'add MixedCase')
+        run_script(self, """
+            $ bzr add MixedCase
+            adding MixedCase
+            """)
         # 'accidently' rename the file on disk
         osutils.rename('MixedCase', 'mixedcase')
-        self.check_empty_output('add mixedcase')
+        run_script(self, """
+            $ bzr add mixedcase
+            """)
 
     def test_re_add_dir(self):
         # like re-add, but tests when the operation is on a directory.
@@ -101,12 +103,16 @@
         wt = self.make_branch_and_tree('.')
         # create a file on disk with the mixed-case name
         self.build_tree(['MixedCaseParent/', 'MixedCaseParent/MixedCase'])
-        self.check_output('adding MixedCaseParent\n'
-                          'adding MixedCaseParent/MixedCase\n',
-                          'add MixedCaseParent')
+        run_script(self, """
+            $ bzr add MixedCaseParent
+            adding MixedCaseParent
+            adding MixedCaseParent/MixedCase
+            """)
         # 'accidently' rename the directory on disk
         osutils.rename('MixedCaseParent', 'mixedcaseparent')
-        self.check_empty_output('add mixedcaseparent')
+        run_script(self, """
+            $ bzr add mixedcaseparent
+            """)
 
     def test_add_not_found(self):
         """Test add when the input file doesn't exist."""
@@ -114,31 +120,34 @@
         # create a file on disk with the mixed-case name
         self.build_tree(['MixedCaseParent/', 'MixedCaseParent/MixedCase'])
         expected_fname = pathjoin(wt.basedir, "MixedCaseParent", "notfound")
-        expected_msg = "bzr: ERROR: No such file: %r\n" % expected_fname
-        self.check_error_output(3, expected_msg, 'add mixedcaseparent/notfound')
+        run_script(self, """
+            $ bzr add mixedcaseparent/notfound
+            2>bzr: ERROR: No such file: %s
+            """ % (repr(expected_fname),))
 
 
 class TestMove(TestCICPBase):
+
     def test_mv_newname(self):
         wt = self._make_mixed_case_tree()
-        self.run_bzr('add')
-        self.run_bzr('ci -m message')
-
-        self.check_output(
-            'CamelCaseParent/CamelCase => CamelCaseParent/NewCamelCase\n',
-            'mv camelcaseparent/camelcase camelcaseparent/NewCamelCase')
+        run_script(self, """
+            $ bzr add
+            $ bzr ci -m message
+            $ bzr mv camelcaseparent/camelcase camelcaseparent/NewCamelCase
+            CamelCaseParent/CamelCase => CamelCaseParent/NewCamelCase
+            """)
 
     def test_mv_newname_after(self):
         wt = self._make_mixed_case_tree()
-        self.run_bzr('add')
-        self.run_bzr('ci -m message')
-        osutils.rename('CamelCaseParent/CamelCase', 'CamelCaseParent/NewCamelCase')
-
         # In this case we can specify the incorrect case for the destination,
         # as we use --after, so the file-system is sniffed.
-        self.check_output(
-            'CamelCaseParent/CamelCase => CamelCaseParent/NewCamelCase\n',
-            'mv --after camelcaseparent/camelcase camelcaseparent/newcamelcase')
+        run_script(self, """
+            $ bzr add 
+            $ bzr ci -m message
+            $ mv CamelCaseParent/CamelCase CamelCaseParent/NewCamelCase
+            $ bzr mv --after camelcaseparent/camelcase camelcaseparent/newcamelcase
+            CamelCaseParent/CamelCase => CamelCaseParent/NewCamelCase
+            """)
 
     def test_mv_newname_exists(self):
         # test a mv, but when the target already exists with a name that
@@ -146,8 +155,11 @@
         wt = self._make_mixed_case_tree()
         self.run_bzr('add')
         self.run_bzr('ci -m message')
-        ex = 'bzr: ERROR: Could not move CamelCase => lowercase: lowercaseparent/lowercase is already versioned.\n'
-        self.check_error_output(3, ex, 'mv camelcaseparent/camelcase LOWERCASEPARENT/LOWERCASE')
+        run_script(self, """
+            $ bzr mv camelcaseparent/camelcase LOWERCASEPARENT/LOWERCASE
+            2>bzr: ERROR: Could not move CamelCase => lowercase: \
+lowercaseparent/lowercase is already versioned.
+            """)
 
     def test_mv_newname_exists_after(self):
         # test a 'mv --after', but when the target already exists with a name
@@ -160,27 +172,32 @@
         # bzr should report that the filename is already versioned.
         os.unlink('CamelCaseParent/CamelCase')
         osutils.rename('lowercaseparent/lowercase', 'lowercaseparent/LOWERCASE')
-        ex = 'bzr: ERROR: Could not move CamelCase => lowercase: lowercaseparent/lowercase is already versioned.\n'
-        self.check_error_output(3, ex, 'mv --after camelcaseparent/camelcase LOWERCASEPARENT/LOWERCASE')
+        run_script(self, """
+            $ bzr mv --after camelcaseparent/camelcase LOWERCASEPARENT/LOWERCASE
+            2>bzr: ERROR: Could not move CamelCase => lowercase: \
+lowercaseparent/lowercase is already versioned.
+            """)
 
     def test_mv_newname_root(self):
         wt = self._make_mixed_case_tree()
         self.run_bzr('add')
         self.run_bzr('ci -m message')
-
-        self.check_output('CamelCaseParent => NewCamelCaseParent\n',
-                          'mv camelcaseparent NewCamelCaseParent')
+        run_script(self, """
+            $ bzr mv camelcaseparent NewCamelCaseParent
+            CamelCaseParent => NewCamelCaseParent
+            """)
 
     def test_mv_newname_root_after(self):
         wt = self._make_mixed_case_tree()
         self.run_bzr('add')
         self.run_bzr('ci -m message')
-        osutils.rename('CamelCaseParent', 'NewCamelCaseParent')
-
         # In this case we can specify the incorrect case for the destination,
         # as we use --after, so the file-system is sniffed.
-        self.check_output('CamelCaseParent => NewCamelCaseParent\n',
-                          'mv --after camelcaseparent newcamelcaseparent')
+        run_script(self, """
+            $ mv CamelCaseParent NewCamelCaseParent
+            $ bzr mv --after camelcaseparent NewCamelCaseParent
+            CamelCaseParent => NewCamelCaseParent
+            """)
 
     def test_mv_newcase(self):
         wt = self._make_mixed_case_tree()
@@ -189,8 +206,10 @@
 
         # perform a mv to the new case - we expect bzr to accept the new
         # name, as specified, and rename the file on the file-system too.
-        self.check_output('CamelCaseParent/CamelCase => CamelCaseParent/camelCase\n',
-                          'mv camelcaseparent/camelcase camelcaseparent/camelCase')
+        run_script(self, """
+            $ bzr mv camelcaseparent/camelcase camelcaseparent/camelCase
+            CamelCaseParent/CamelCase => CamelCaseParent/camelCase
+            """)
         self.failUnlessEqual(canonical_relpath(wt.basedir, 'camelcaseparent/camelcase'),
                              'CamelCaseParent/camelCase')
 
@@ -202,8 +221,10 @@
         # perform a mv to the new case - we must ensure the file-system has the
         # new case first.
         osutils.rename('CamelCaseParent/CamelCase', 'CamelCaseParent/camelCase')
-        self.check_output('CamelCaseParent/CamelCase => CamelCaseParent/camelCase\n',
-                          'mv --after camelcaseparent/camelcase camelcaseparent/camelCase')
+        run_script(self, """
+            $ bzr mv --after camelcaseparent/camelcase camelcaseparent/camelCase
+            CamelCaseParent/CamelCase => CamelCaseParent/camelCase
+            """)
         # bzr should not have renamed the file to a different case
         self.failUnlessEqual(canonical_relpath(wt.basedir, 'camelcaseparent/camelcase'),
                              'CamelCaseParent/camelCase')
@@ -212,9 +233,11 @@
         wt = self._make_mixed_case_tree()
         self.run_bzr('add')
         self.run_bzr('ci -m message')
-        self.check_output('lowercaseparent/lowercase => CamelCaseParent/lowercase\n'
-                          'lowercaseparent/mixedCase => CamelCaseParent/mixedCase\n',
-                          'mv LOWercaseparent/LOWercase LOWercaseparent/MIXEDCase camelcaseparent')
+        run_script(self, """
+            $ bzr mv LOWercaseparent/LOWercase LOWercaseparent/MIXEDCase camelcaseparent
+            lowercaseparent/lowercase => CamelCaseParent/lowercase
+            lowercaseparent/mixedCase => CamelCaseParent/mixedCase
+            """)
 
 
 class TestMisc(TestCICPBase):
@@ -222,15 +245,14 @@
     def test_status(self):
         wt = self._make_mixed_case_tree()
         self.run_bzr('add')
-
-        self.check_output(
-            """added:
-  CamelCaseParent/
-  CamelCaseParent/CamelCase
-  lowercaseparent/
-  lowercaseparent/lowercase
-""",
-            'status camelcaseparent/camelcase LOWERCASEPARENT/LOWERCASE')
+        run_script(self, """
+            $ bzr status camelcaseparent/camelcase LOWERCASEPARENT/LOWERCASE
+            added:
+              CamelCaseParent/
+              CamelCaseParent/CamelCase
+              lowercaseparent/
+              lowercaseparent/lowercase
+            """)
 
     def test_ci(self):
         wt = self._make_mixed_case_tree()

=== modified file 'bzrlib/tests/blackbox/test_find_merge_base.py'
--- bzrlib/tests/blackbox/test_find_merge_base.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_find_merge_base.py	2010-06-23 08:19:28 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009, 2010 Canonical Ltd
 # -*- coding: utf-8 -*-
 #
 # This program is free software; you can redistribute it and/or modify
@@ -16,10 +16,10 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 import os
 
-from bzrlib.tests.blackbox import ExternalBase
-
-
-class TestFindMergeBase(ExternalBase):
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestFindMergeBase(TestCaseWithTransport):
 
     def test_find_merge_base(self):
         a_tree = self.make_branch_and_tree('a')

=== modified file 'bzrlib/tests/blackbox/test_help.py'
--- bzrlib/tests/blackbox/test_help.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/blackbox/test_help.py	2010-08-29 14:32:45 +0000
@@ -20,11 +20,11 @@
 
 
 import bzrlib
-from bzrlib.tests.blackbox import ExternalBase
-from bzrlib.config import (ensure_config_dir_exists, config_filename)
-
-
-class TestHelp(ExternalBase):
+from bzrlib import config
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestHelp(TestCaseWithTransport):
 
     def test_help_basic(self):
         for cmd in ['--help', 'help', '-h', '-?']:
@@ -161,12 +161,10 @@
     def test_help_with_aliases(self):
         original = self.run_bzr('help cat')[0]
 
-        ensure_config_dir_exists()
-        CONFIG=("[ALIASES]\n"
-        "c=cat\n"
-        "cat=cat\n")
-
-        open(config_filename(),'wb').write(CONFIG)
+        conf = config.GlobalConfig.from_string('''[ALIASES]
+c=cat
+cat=cat
+''', save=True)
 
         expected = original + "'bzr cat' is an alias for 'bzr cat'.\n"
         self.assertEqual(expected, self.run_bzr('help cat')[0])

=== modified file 'bzrlib/tests/blackbox/test_ignore.py'
--- bzrlib/tests/blackbox/test_ignore.py	2010-04-22 12:45:03 +0000
+++ bzrlib/tests/blackbox/test_ignore.py	2010-07-08 12:36:29 +0000
@@ -34,11 +34,11 @@
     pathjoin,
     )
 from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
-from bzrlib.tests.blackbox import ExternalBase
+from bzrlib.tests import TestCaseWithTransport
 from bzrlib.workingtree import WorkingTree
 
 
-class TestCommands(ExternalBase):
+class TestCommands(TestCaseWithTransport):
 
     def test_ignore_absolutes(self):
         """'ignore' with an absolute path returns an error"""
@@ -156,3 +156,25 @@
                          " and match your ignore pattern:\nb\n"\
                          "These files will continue to be version controlled"\
                          " unless you 'bzr remove' them.\n")
+
+    def test_ignore_directory(self):
+        """Test --directory option"""
+        tree = self.make_branch_and_tree('a')
+        self.run_bzr(['ignore', '--directory=a', 'README'])
+        self.check_file_contents('a/.bzrignore', 'README\n')
+
+    def test_ignored_invalid_pattern(self):
+        """Ensure graceful handling for invalid ignore pattern.
+
+        Test case for #300062.
+        Invalid pattern should show clear error message.
+        Invalid pattern should not be added to .bzrignore file.
+        """
+        tree = self.make_branch_and_tree('.')
+        out, err = self.run_bzr(['ignore', 'RE:*.cpp', 'foo', 'RE:['], 3)
+        self.assertEqual(out, '')
+        self.assertContainsRe(err,
+            'Invalid ignore pattern.*RE:\*\.cpp.*RE:\[', re.DOTALL)
+        self.assertNotContainsRe(err, 'foo', re.DOTALL)
+        self.assertFalse(os.path.isfile('.bzrignore'))
+

=== modified file 'bzrlib/tests/blackbox/test_ignored.py'
--- bzrlib/tests/blackbox/test_ignored.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_ignored.py	2010-06-23 08:19:28 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,10 +17,10 @@
 
 """Tests of the 'bzr ignored' command."""
 
-from bzrlib.tests.blackbox import ExternalBase
-
-
-class TestIgnored(ExternalBase):
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestIgnored(TestCaseWithTransport):
 
     def test_ignored_added_file(self):
         """'bzr ignored' should not list versioned files."""
@@ -38,3 +38,11 @@
         out, err = self.run_bzr('ignored')
         self.assertEqual('', out)
         self.assertEqual('', err)
+
+    def test_ignored_directory(self):
+        """Test --directory option"""
+        tree = self.make_branch_and_tree('a')
+        self.build_tree_contents([('a/README', 'contents'),
+                                  ('a/.bzrignore', 'README')])
+        out, err = self.run_bzr(['ignored', '--directory=a'])
+        self.assertStartsWith(out, 'README')

=== modified file 'bzrlib/tests/blackbox/test_init.py'
--- bzrlib/tests/blackbox/test_init.py	2010-04-23 07:15:23 +0000
+++ bzrlib/tests/blackbox/test_init.py	2010-06-11 07:32:12 +0000
@@ -27,15 +27,15 @@
     )
 from bzrlib.bzrdir import BzrDirMetaFormat1
 from bzrlib.tests import TestSkipped
-from bzrlib.tests.blackbox import ExternalBase
+from bzrlib.tests import TestCaseWithTransport
 from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
 from bzrlib.workingtree import WorkingTree
 
 
-class TestInit(ExternalBase):
+class TestInit(TestCaseWithTransport):
 
     def setUp(self):
-        ExternalBase.setUp(self)
+        TestCaseWithTransport.setUp(self)
         self._default_label = '2a'
 
     def test_init_with_format(self):
@@ -202,3 +202,17 @@
         self.assertEqual(True, branch._get_append_revisions_only())
         self.run_bzr_error(['cannot be set to append-revisions-only'],
                            'init --append-revisions-only --knit knit')
+
+    def test_init_without_username(self):
+        """Ensure init works if username is not set.
+        """
+        # bzr makes user specified whoami mandatory for operations
+        # like commit as whoami is recorded. init however is not so final
+        # and uses whoami only in a lock file. Without whoami the login name
+        # is used. This test is to ensure that init passes even when whoami
+        # is not available.
+        osutils.set_or_unset_env('EMAIL', None)
+        osutils.set_or_unset_env('BZR_EMAIL', None)
+        out, err = self.run_bzr(['init', 'foo'])
+        self.assertEqual(err, '')
+        self.assertTrue(os.path.exists('foo'))

=== modified file 'bzrlib/tests/blackbox/test_log.py'
--- bzrlib/tests/blackbox/test_log.py	2010-04-14 12:30:17 +0000
+++ bzrlib/tests/blackbox/test_log.py	2010-07-09 16:16:11 +0000
@@ -158,6 +158,27 @@
         self.make_linear_branch()
         self.assertLogRevnos(['-c1'], ['1'])
 
+    def test_branch_revspec(self):
+        foo = self.make_branch_and_tree('foo')
+        bar = self.make_branch_and_tree('bar')
+        self.build_tree(['foo/foo.txt', 'bar/bar.txt'])
+        foo.add('foo.txt')
+        bar.add('bar.txt')
+        foo.commit(message='foo')
+        bar.commit(message='bar')
+        self.run_bzr('log -r branch:../bar', working_dir='foo')
+        self.assertEqual([bar.branch.get_rev_id(1)],
+                         [r.rev.revision_id
+                          for r in self.get_captured_revisions()])
+
+
+class TestLogExcludeCommonAncestry(TestLogWithLogCatcher):
+
+    def test_exclude_common_ancestry_simple_revnos(self):
+        self.make_linear_branch()
+        self.assertLogRevnos(['-r1..3', '--exclude-common-ancestry'],
+                             ['3', '2'])
+
 
 class TestLogMergedLinearAncestry(TestLogWithLogCatcher):
 
@@ -167,6 +188,18 @@
         # stop calling run_bzr, there is no point) --vila 100118.
         builder = branchbuilder.BranchBuilder(self.get_transport())
         builder.start_series()
+        # 1
+        # | \
+        # 2  1.1.1
+        # | / |
+        # 3  1.1.2
+        # |   |
+        # |  1.1.3
+        # | / |
+        # 4  1.1.4
+        # | /
+        # 5
+
         # mainline
         builder.build_snapshot('1', None, [
             ('add', ('', 'root-id', 'directory', ''))])
@@ -348,16 +381,14 @@
 
     def test_log_bad_message_re(self):
         """Bad --message argument gives a sensible message
-        
+
         See https://bugs.launchpad.net/bzr/+bug/251352
         """
         self.make_minimal_branch()
         out, err = self.run_bzr(['log', '-m', '*'], retcode=3)
-        self.assertEqual("bzr: ERROR: Invalid regular expression"
-            " in log message filter"
-            ": '*'"
-            ": nothing to repeat\n", err)
-        self.assertEqual('', out)
+        self.assertContainsRe(err, "ERROR.*Invalid pattern.*nothing to repeat")
+        self.assertNotContainsRe(err, "Unprintable exception")
+        self.assertEqual(out, '')
 
     def test_log_unsupported_timezone(self):
         self.make_linear_branch()

=== added file 'bzrlib/tests/blackbox/test_lookup_revision.py'
--- bzrlib/tests/blackbox/test_lookup_revision.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_lookup_revision.py	2010-05-02 20:10:25 +0000
@@ -0,0 +1,33 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+
+"""Black-box tests for bzr lookup-revision.
+"""
+
+import os
+
+from bzrlib import tests
+
+class TestLookupRevision(tests.TestCaseWithTransport):
+
+    def test_lookup_revison_directory(self):
+        """Test --directory option"""
+        tree = self.make_branch_and_tree('a')
+        tree.commit('This revision', rev_id='abcd')
+        out, err = self.run_bzr(['lookup-revision', '-d', 'a', '1'])
+        self.assertEqual('abcd\n', out)
+        self.assertEqual('', err)

=== modified file 'bzrlib/tests/blackbox/test_ls.py'
--- bzrlib/tests/blackbox/test_ls.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/blackbox/test_ls.py	2010-05-02 20:10:25 +0000
@@ -18,7 +18,7 @@
 
 import os
 
-from bzrlib import ignores
+from bzrlib import ignores, osutils
 from bzrlib.tests import TestCaseWithTransport
 
 
@@ -235,3 +235,12 @@
                        '%s/a\n'
                        % (self.test_dir, self.test_dir),
                        self.test_dir, recursive=False)
+
+    def test_ls_directory(self):
+        """Test --directory option"""
+        self.wt = self.make_branch_and_tree('dir')
+        self.build_tree(['dir/sub/', 'dir/sub/file'])
+        self.wt.add(['sub', 'sub/file'])
+        self.wt.commit('commit')
+        self.ls_equals('sub/\nsub/file\n', '--directory=dir')
+        self.ls_equals('sub/file\n', '-d dir sub')

=== modified file 'bzrlib/tests/blackbox/test_merge.py'
--- bzrlib/tests/blackbox/test_merge.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/blackbox/test_merge.py	2010-05-20 18:23:17 +0000
@@ -393,7 +393,7 @@
     def test_directive_cherrypick(self):
         source = self.make_branch_and_tree('source')
         source.commit("nothing")
-        # see https://bugs.edge.launchpad.net/bzr/+bug/409688 - trying to
+        # see https://bugs.launchpad.net/bzr/+bug/409688 - trying to
         # cherrypick from one branch into another unrelated branch with a
         # different root id will give shape conflicts.  as a workaround we
         # make sure they share the same root id.

=== modified file 'bzrlib/tests/blackbox/test_merge_directive.py'
--- bzrlib/tests/blackbox/test_merge_directive.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_merge_directive.py	2010-06-17 09:23:19 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -248,3 +248,15 @@
         tree1, tree2 = self.prepare_merge_directive()
         tree1.commit(u'messag\xe9')
         self.run_bzr('merge-directive ../tree2') # no exception raised
+
+    def test_merge_directive_directory(self):
+        """Test --directory option"""
+        import re
+        re_timestamp = re.compile(r'^# timestamp: .*', re.M)
+        self.prepare_merge_directive()
+        md1 = self.run_bzr('merge-directive ../tree2')[0]
+        md1 = re_timestamp.sub('# timestamp: XXX', md1)
+        os.chdir('..')
+        md2 = self.run_bzr('merge-directive --directory tree1 tree2')[0]
+        md2 = re_timestamp.sub('# timestamp: XXX', md2)
+        self.assertEqualDiff(md1.replace('../tree2', 'tree2'), md2)

=== modified file 'bzrlib/tests/blackbox/test_missing.py'
--- bzrlib/tests/blackbox/test_missing.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_missing.py	2010-06-17 09:23:19 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2008 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -200,3 +200,23 @@
                           'Branches are up to date.\n' % location,
                           lines)
         self.assertEquals('', err)
+
+    def test_missing_directory(self):
+        """Test --directory option"""
+
+        # create a source branch
+        a_tree = self.make_branch_and_tree('a')
+        self.build_tree_contents([('a/a', 'initial\n')])
+        a_tree.add('a')
+        a_tree.commit(message='initial')
+
+        # clone and add a differing revision
+        b_tree = a_tree.bzrdir.sprout('b').open_workingtree()
+        self.build_tree_contents([('b/a', 'initial\nmore\n')])
+        b_tree.commit(message='more')
+        
+        out2, err2 = self.run_bzr('missing --directory a b', retcode=1)
+        os.chdir('a')
+        out1, err1 = self.run_bzr('missing ../b', retcode=1)
+        self.assertEqualDiff(out1, out2)
+        self.assertEqualDiff(err1, err2)

=== modified file 'bzrlib/tests/blackbox/test_modified.py'
--- bzrlib/tests/blackbox/test_modified.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_modified.py	2010-06-23 08:19:28 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2008, 2009, 2010 Canonical Ltd
 # -*- coding: utf-8 -*-
 #
 # This program is free software; you can redistribute it and/or modify
@@ -21,9 +21,9 @@
 import os
 
 from bzrlib.branch import Branch
-from bzrlib.tests.blackbox import ExternalBase
+from bzrlib.tests import TestCaseWithTransport
 
-class TestModified(ExternalBase):
+class TestModified(TestCaseWithTransport):
 
     def test_modified(self):
         """Test that 'modified' command reports modified files"""
@@ -70,3 +70,12 @@
         tree.commit(message='modified %s' %(name))
         check_modified('')
 
+    def test_modified_directory(self):
+        """Test --directory option"""
+        tree = self.make_branch_and_tree('a')
+        self.build_tree(['a/README'])
+        tree.add('README')
+        tree.commit('r1')
+        self.build_tree_contents([('a/README', 'changed\n')])
+        out, err = self.run_bzr(['modified', '--directory=a'])
+        self.assertEquals('README\n', out)

=== modified file 'bzrlib/tests/blackbox/test_nick.py'
--- bzrlib/tests/blackbox/test_nick.py	2010-03-01 16:17:57 +0000
+++ bzrlib/tests/blackbox/test_nick.py	2010-06-23 08:19:28 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006  Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,10 +20,10 @@
 
 import bzrlib
 from bzrlib import osutils
-from bzrlib.tests.blackbox import ExternalBase
-
-
-class TestNick(ExternalBase):
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestNick(TestCaseWithTransport):
 
     def test_nick_command(self):
         """bzr nick for viewing, setting nicknames"""
@@ -78,3 +78,12 @@
                          child.branch.get_config().has_explicit_nickname())
         osutils.rmtree('../base')
         self.assertEqual('child', self.run_bzr('nick')[0][:-1])
+
+    def test_nick_directory(self):
+        """Test --directory option"""
+        self.make_branch_and_tree('me.dev')
+        nick = self.run_bzr(['nick', '--directory=me.dev'])[0]
+        self.assertEqual('me.dev\n', nick)
+        self.run_bzr(['nick', '-d', 'me.dev', 'moo'])
+        nick = self.run_bzr(['nick', '--directory', 'me.dev'])[0]
+        self.assertEqual('moo\n', nick)

=== modified file 'bzrlib/tests/blackbox/test_pull.py'
--- bzrlib/tests/blackbox/test_pull.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/blackbox/test_pull.py	2010-09-27 20:22:57 +0000
@@ -29,12 +29,12 @@
 from bzrlib.branch import Branch
 from bzrlib.directory_service import directories
 from bzrlib.osutils import pathjoin
-from bzrlib.tests.blackbox import ExternalBase
+from bzrlib.tests import TestCaseWithTransport
 from bzrlib.uncommit import uncommit
 from bzrlib.workingtree import WorkingTree
 
 
-class TestPull(ExternalBase):
+class TestPull(TestCaseWithTransport):
 
     def example_branch(self, path='.'):
         tree = self.make_branch_and_tree(path)
@@ -453,3 +453,46 @@
         out, err = self.run_bzr(['pull', '-d', 'to', 'from'])
         self.assertContainsRe(err,
             "(?m)Fetching into experimental format")
+
+    def test_pull_show_base(self):
+        """bzr pull supports --show-base
+
+        see https://bugs.launchpad.net/bzr/+bug/202374"""
+        # create two trees with conflicts, setup conflict, check that
+        # conflicted file looks correct
+        a_tree = self.example_branch('a')
+        b_tree = a_tree.bzrdir.sprout('b').open_workingtree()
+
+        f = open(pathjoin('a', 'hello'),'wt')
+        f.write('fee')
+        f.close()
+        a_tree.commit('fee')
+
+        f = open(pathjoin('b', 'hello'),'wt')
+        f.write('fie')
+        f.close()
+
+        out,err=self.run_bzr(['pull','-d','b','a','--show-base'])
+
+        # check for message here
+        self.assertEqual(err,
+                         ' M  hello\nText conflict in hello\n1 conflicts encountered.\n')
+
+        self.assertEqualDiff('<<<<<<< TREE\n'
+                             'fie||||||| BASE-REVISION\n'
+                             'foo=======\n'
+                             'fee>>>>>>> MERGE-SOURCE\n',
+                             open(pathjoin('b', 'hello')).read())
+
+    def test_pull_show_base_working_tree_only(self):
+        """--show-base only allowed if there's a working tree
+
+        see https://bugs.launchpad.net/bzr/+bug/202374"""
+        # create a branch, see that --show-base fails
+        self.make_branch('from')
+        self.make_branch('to')
+        out=self.run_bzr(['pull','-d','to','from','--show-base'],retcode=3)
+        self.assertEqual(out,
+                         ('','bzr: ERROR: Need working tree for --show-base.\n'))
+
+

=== modified file 'bzrlib/tests/blackbox/test_push.py'
--- bzrlib/tests/blackbox/test_push.py	2010-04-28 10:30:48 +0000
+++ bzrlib/tests/blackbox/test_push.py	2010-06-11 07:32:12 +0000
@@ -788,7 +788,7 @@
         self.assertPushSucceeds([])
 
 
-class TestPushForeign(blackbox.ExternalBase):
+class TestPushForeign(tests.TestCaseWithTransport):
 
     def setUp(self):
         super(TestPushForeign, self).setUp()

=== modified file 'bzrlib/tests/blackbox/test_re_sign.py'
--- bzrlib/tests/blackbox/test_re_sign.py	2010-01-25 17:48:22 +0000
+++ bzrlib/tests/blackbox/test_re_sign.py	2010-05-02 20:10:25 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 # -*- coding: utf-8 -*-
 #
 # This program is free software; you can redistribute it and/or modify
@@ -88,3 +88,16 @@
         self.assertEqualSignature(repo, 'A')
         self.assertEqualSignature(repo, 'B')
         self.assertEqualSignature(repo, 'C')
+
+    def test_resign_directory(self):
+        """Test --directory option"""
+        wt = BzrDir.create_standalone_workingtree('a')
+        wt.commit("base A", allow_pointless=True, rev_id='A')
+        wt.commit("base B", allow_pointless=True, rev_id='B')
+        wt.commit("base C", allow_pointless=True, rev_id='C')
+        repo = wt.branch.repository
+        self.monkey_patch_gpg()
+        self.run_bzr('re-sign --directory=a -r revid:A')
+        self.assertEqualSignature(repo, 'A')
+        self.run_bzr('re-sign -d a B')
+        self.assertEqualSignature(repo, 'B')

=== modified file 'bzrlib/tests/blackbox/test_reconfigure.py'
--- bzrlib/tests/blackbox/test_reconfigure.py	2009-07-10 08:18:28 +0000
+++ bzrlib/tests/blackbox/test_reconfigure.py	2010-05-20 18:23:17 +0000
@@ -197,7 +197,7 @@
          * then make the second unstacked, so it has to fill in history from
            the original fallback lying underneath its original content
 
-        See discussion in <https://bugs.edge.launchpad.net/bzr/+bug/391411>
+        See discussion in <https://bugs.launchpad.net/bzr/+bug/391411>
         """
         # there are also per_branch tests that exercise remote operation etc
         tree_1 = self.make_branch_and_tree('b1', format='2a')

=== modified file 'bzrlib/tests/blackbox/test_remerge.py'
--- bzrlib/tests/blackbox/test_remerge.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/blackbox/test_remerge.py	2010-06-11 07:32:12 +0000
@@ -17,11 +17,11 @@
 
 import os
 
-from bzrlib.tests.blackbox import ExternalBase
+from bzrlib.tests import TestCaseWithTransport
 from bzrlib.workingtree import WorkingTree
 
 
-class TestRemerge(ExternalBase):
+class TestRemerge(TestCaseWithTransport):
 
     def make_file(self, name, contents):
         f = open(name, 'wb')

=== modified file 'bzrlib/tests/blackbox/test_remove.py'
--- bzrlib/tests/blackbox/test_remove.py	2009-09-07 08:52:04 +0000
+++ bzrlib/tests/blackbox/test_remove.py	2010-10-02 18:10:06 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,8 +18,12 @@
 import os
 import sys
 
-from bzrlib.tests import SymlinkFeature, TestSkipped
-from bzrlib.tests.blackbox import ExternalBase
+from bzrlib.tests import (
+    script,
+    SymlinkFeature,
+    TestCaseWithTransport,
+    TestSkipped,
+    )
 from bzrlib.workingtree import WorkingTree
 from bzrlib import osutils
 
@@ -31,7 +35,7 @@
 files=(a, b, c, d)
 
 
-class TestRemove(ExternalBase):
+class TestRemove(TestCaseWithTransport):
 
     def _make_tree_and_add(self, paths):
         tree = self.make_branch_and_tree('.')
@@ -61,18 +65,9 @@
         f.write("\nsome other new content!")
         f.close()
 
-    def run_bzr_remove_changed_files(self, error_regexes, files_to_remove,
-                                     working_dir=None):
-        error_regexes.extend(["Can't safely remove modified or unknown files:",
-            'Use --keep to not delete them,'
-            ' or --force to delete them regardless.'
-            ])
-        self.run_bzr_error(error_regexes,
-                           ['remove'] + list(files_to_remove),
-                           working_dir=working_dir)
-        #see if we can force it now
-        self.run_bzr(['remove', '--force'] + list(files_to_remove),
-                     working_dir=working_dir)
+    def run_bzr_remove_changed_files(self, files_to_remove, working_dir=None):
+        self.run_bzr(['remove'] + list(files_to_remove),
+           working_dir=working_dir)
 
     def test_remove_new_no_files_specified(self):
         tree = self.make_branch_and_tree('.')
@@ -177,20 +172,19 @@
     def test_remove_unversioned_files(self):
         self.build_tree(files)
         tree = self.make_branch_and_tree('.')
-        self.run_bzr_remove_changed_files(
-            ['unknown:[.\s]*d/[.\s]*b/c[.\s]*b/[.\s]*a'], files)
+        self.run_bzr_remove_changed_files(files)
 
     def test_remove_changed_files(self):
         tree = self._make_tree_and_add(files)
         self.run_bzr("commit -m 'added files'")
         self.changeFile(a)
         self.changeFile(c)
-        self.run_bzr_remove_changed_files(['modified:[.\s]*a[.\s]*b/c'], files)
+        self.run_bzr_remove_changed_files(files)
 
     def test_remove_changed_ignored_files(self):
         tree = self._make_tree_and_add(['a'])
         self.run_bzr(['ignore', 'a'])
-        self.run_bzr_remove_changed_files(['added:[.\s]*a'], ['a'])
+        self.run_bzr_remove_changed_files(['a'])
 
     def test_remove_changed_files_from_child_dir(self):
         if sys.platform == 'win32':
@@ -200,7 +194,6 @@
         self.changeFile(a)
         self.changeFile(c)
         self.run_bzr_remove_changed_files(
-            ['modified:[.\s]*a[.\s]*b/c'],
             ['../a', 'c', '.', '../d'], working_dir='b')
         self.assertNotInWorkingTree(files)
         self.failIfExists(files)
@@ -211,12 +204,29 @@
         self.run_bzr('remove --keep a', error_regexes=["a is not versioned."])
         self.assertFilesUnversioned(files)
 
+    def test_remove_no_backup_unversioned_files(self):
+        self.build_tree(files)
+        tree = self.make_branch_and_tree('.')
+        script.ScriptRunner().run_script(self, '''
+        $ bzr remove --no-backup a b/ b/c d/
+        2>deleted d
+        2>removed b/c (but kept a copy: b/c.~1~)
+        2>deleted b
+        2>deleted a
+        ''')
+        self.assertFilesDeleted(files)
+
     def test_remove_force_unversioned_files(self):
         self.build_tree(files)
         tree = self.make_branch_and_tree('.')
-        self.run_bzr(['remove', '--force'] + list(files),
-                     error_regexes=["deleted a", "deleted b",
-                                    "deleted b/c", "deleted d"])
+        script.ScriptRunner().run_script(self, '''
+        $ bzr remove --force a b/ b/c d/
+        2>(The --force option is deprecated, rather use --no-backup in future.)
+        2>deleted d
+        2>removed b/c (but kept a copy: b/c.~1~)
+        2>deleted b
+        2>deleted a
+        ''')
         self.assertFilesDeleted(files)
 
     def test_remove_deleted_files(self):

=== modified file 'bzrlib/tests/blackbox/test_remove_tree.py'
--- bzrlib/tests/blackbox/test_remove_tree.py	2009-11-16 05:57:00 +0000
+++ bzrlib/tests/blackbox/test_remove_tree.py	2010-06-23 08:19:28 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,10 +19,11 @@
 
 import os
 
-from bzrlib.tests.blackbox import ExternalBase
-
-
-class TestRemoveTree(ExternalBase):
+from bzrlib import shelf
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestRemoveTree(TestCaseWithTransport):
 
     def setUp(self):
         super(TestRemoveTree, self).setUp()
@@ -155,3 +156,22 @@
         self.run_bzr('remove-tree branch2 --force')
         self.failIfExists('branch2/foo')
         self.failIfExists('branch2/bar')
+
+    def test_remove_tree_shelved_changes(self):
+        # https://bugs.launchpad.net/bzr/+bug/586639
+        tree = self.make_branch_and_tree('.')
+        creator = shelf.ShelfCreator(tree, tree.basis_tree(), [])
+        self.addCleanup(creator.finalize)
+        shelf_id = tree.get_shelf_manager().shelve_changes(creator, 'Foo')
+        output = self.run_bzr_error(["Working tree .* has shelved changes"],
+                                    'remove-tree', retcode=3)
+
+    def test_remove_tree_shelved_changes_force(self):
+        tree = self.make_branch_and_tree('.')
+        creator = shelf.ShelfCreator(tree, tree.basis_tree(), [])
+        self.addCleanup(creator.finalize)
+        shelf_id = tree.get_shelf_manager().shelve_changes(creator, 'Foo')
+        self.run_bzr('remove-tree --force')
+        self.run_bzr('checkout')
+        # Ensure shelf is empty
+        self.assertIs(None, tree.get_shelf_manager().last_shelf())

=== modified file 'bzrlib/tests/blackbox/test_revert.py'
--- bzrlib/tests/blackbox/test_revert.py	2009-08-28 05:00:33 +0000
+++ bzrlib/tests/blackbox/test_revert.py	2010-06-23 08:19:28 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2007 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,12 +19,12 @@
 import os
 
 import bzrlib.osutils
-from bzrlib.tests.blackbox import ExternalBase
+from bzrlib.tests import TestCaseWithTransport
 from bzrlib.trace import mutter
 from bzrlib.workingtree import WorkingTree
 
 
-class TestRevert(ExternalBase):
+class TestRevert(TestCaseWithTransport):
 
     def _prepare_tree(self):
         self.run_bzr('init')

=== modified file 'bzrlib/tests/blackbox/test_revision_info.py'
--- bzrlib/tests/blackbox/test_revision_info.py	2009-06-30 16:59:21 +0000
+++ bzrlib/tests/blackbox/test_revision_info.py	2010-06-23 08:19:28 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2004, 2005, 2009 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,19 +17,19 @@
 import os
 
 from bzrlib.errors import BzrCommandError, NoSuchRevision
-from bzrlib.tests.blackbox import ExternalBase
+from bzrlib.tests import TestCaseWithTransport
 from bzrlib.workingtree import WorkingTree
 
 
-class TestRevisionInfo(ExternalBase):
+class TestRevisionInfo(TestCaseWithTransport):
 
-    def check_error(self, output, *args):
-        """Verify that the expected error matches what bzr says.
+    def check_output(self, output, *args):
+        """Verify that the expected output matches what bzr says.
 
         The output is supplied first, so that you can supply a variable
         number of arguments to bzr.
         """
-        self.assertContainsRe(self.run_bzr(args, retcode=3)[1], output)
+        self.assertEquals(self.run_bzr(*args)[0], output)
 
     def test_revision_info(self):
         """Test that 'bzr revision-info' reports the correct thing."""

=== modified file 'bzrlib/tests/blackbox/test_rmbranch.py'
--- bzrlib/tests/blackbox/test_rmbranch.py	2010-03-22 10:11:00 +0000
+++ bzrlib/tests/blackbox/test_rmbranch.py	2010-06-11 07:32:12 +0000
@@ -20,12 +20,12 @@
 from bzrlib import (
     bzrdir,
     )
-from bzrlib.tests.blackbox import (
-    ExternalBase,
+from bzrlib.tests import (
+    TestCaseWithTransport,
     )
 
 
-class TestRemoveBranch(ExternalBase):
+class TestRemoveBranch(TestCaseWithTransport):
 
     def example_branch(self, path='.'):
         tree = self.make_branch_and_tree(path)

=== modified file 'bzrlib/tests/blackbox/test_selftest.py'
--- bzrlib/tests/blackbox/test_selftest.py	2010-04-18 08:51:37 +0000
+++ bzrlib/tests/blackbox/test_selftest.py	2010-09-03 20:49:17 +0000
@@ -16,8 +16,9 @@
 
 """UI tests for the test framework."""
 
+import os
+
 from bzrlib import (
-    benchmarks,
     tests,
     )
 from bzrlib.tests import (
@@ -46,24 +47,6 @@
             tests.selftest = original_selftest
 
 
-class TestOptionsWritingToDisk(tests.TestCaseInTempDir, SelfTestPatch):
-
-    def test_benchmark_runs_benchmark_tests(self):
-        """selftest --benchmark should change the suite factory."""
-        params = self.get_params_passed_to_core('selftest --benchmark')
-        self.assertEqual(benchmarks.test_suite,
-            params[1]['test_suite_factory'])
-        self.assertNotEqual(None, params[1]['bench_history'])
-        benchfile = open(".perf_history", "rt")
-        try:
-            lines = benchfile.readlines()
-        finally:
-            benchfile.close()
-        # Because we don't run the actual test code no output is made to the
-        # file.
-        self.assertEqual(0, len(lines))
-
-
 class TestOptions(tests.TestCase, SelfTestPatch):
 
     def test_load_list(self):
@@ -167,3 +150,12 @@
     def test_lsprof_tests(self):
         params = self.get_params_passed_to_core('selftest --lsprof-tests')
         self.assertEqual(True, params[1]["lsprof_tests"])
+
+    def test_parallel_fork_unsupported(self):
+        if getattr(os, "fork", None) is not None:
+            self.addCleanup(setattr, os, "fork", os.fork)
+            del os.fork
+        out, err = self.run_bzr(["selftest", "--parallel=fork", "-s", "bt.x"],
+            retcode=3)
+        self.assertIn("platform does not support fork", err)
+        self.assertFalse(out)

=== modified file 'bzrlib/tests/blackbox/test_serve.py'
--- bzrlib/tests/blackbox/test_serve.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/blackbox/test_serve.py	2010-06-20 11:18:38 +0000
@@ -31,6 +31,7 @@
     errors,
     osutils,
     revision as _mod_revision,
+    transport,
     urlutils,
     )
 from bzrlib.branch import Branch
@@ -43,7 +44,7 @@
     TestSkipped,
     )
 from bzrlib.trace import mutter
-from bzrlib.transport import get_transport, remote
+from bzrlib.transport import remote
 
 
 class TestBzrServeBase(TestCaseWithTransport):
@@ -192,8 +193,8 @@
     def test_bzr_serve_port_readonly(self):
         """bzr server should provide a read only filesystem by default."""
         process, url = self.start_server_port()
-        transport = get_transport(url)
-        self.assertRaises(errors.TransportNotPossible, transport.mkdir, 'adir')
+        t = transport.get_transport(url)
+        self.assertRaises(errors.TransportNotPossible, t.mkdir, 'adir')
         self.assertServerFinishesCleanly(process)
 
     def test_bzr_serve_port_readwrite(self):

=== modified file 'bzrlib/tests/blackbox/test_shared_repository.py'
--- bzrlib/tests/blackbox/test_shared_repository.py	2010-03-25 18:02:45 +0000
+++ bzrlib/tests/blackbox/test_shared_repository.py	2010-05-05 14:02:53 +0000
@@ -18,6 +18,7 @@
 
 import os
 
+from bzrlib import osutils
 from bzrlib.bzrdir import BzrDir, BzrDirMetaFormat1
 import bzrlib.errors as errors
 from bzrlib.tests import TestCaseInTempDir
@@ -145,3 +146,17 @@
         self.assertLength(0, calls)
         self.run_bzr("init-repository a")
         self.assertLength(1, calls)
+
+    def test_init_repo_without_username(self):
+        """Ensure init-repo works if username is not set.
+        """
+        # bzr makes user specified whoami mandatory for operations
+        # like commit as whoami is recorded. init-repo however is not so final
+        # and uses whoami only in a lock file. Without whoami the login name
+        # is used. This test is to ensure that init-repo passes even when whoami
+        # is not available.
+        osutils.set_or_unset_env('EMAIL', None)
+        osutils.set_or_unset_env('BZR_EMAIL', None)
+        out, err = self.run_bzr(['init-repo', 'foo'])
+        self.assertEqual(err, '')
+        self.assertTrue(os.path.exists('foo'))

=== modified file 'bzrlib/tests/blackbox/test_shelve.py'
--- bzrlib/tests/blackbox/test_shelve.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/blackbox/test_shelve.py	2010-09-29 12:58:17 +0000
@@ -68,7 +68,7 @@
         self.assertIs(None, tree.get_shelf_manager().last_shelf())
 
     def test_unshelve_keep(self):
-        # https://bugs.edge.launchpad.net/bzr/+bug/492091
+        # https://bugs.launchpad.net/bzr/+bug/492091
         tree = self.make_branch_and_tree('.')
         # shelve apparently unhappy working with a tree with no root yet
         tree.commit('make root')
@@ -87,6 +87,25 @@
 contents of file
 ''')
 
+class TestUnshelvePreview(TestCaseWithTransport):
+    
+    def test_non_ascii(self):
+        """Test that we can show a non-ascii diff that would result from unshelving"""
+        
+        init_content = u'Initial: \u0418\u0437\u043d\u0430\u0447\n'.encode('utf-8')
+        more_content = u'More: \u0415\u0449\u0451\n'.encode('utf-8')
+        next_content = init_content + more_content
+        diff_part = '@@ -1,1 +1,2 @@\n %s+%s' % (init_content, more_content)
+        
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([('a_file', init_content)])
+        tree.add('a_file')
+        tree.commit(message='committed')
+        self.build_tree_contents([('a_file', next_content)])
+        self.run_bzr(['shelve', '--all'])
+        out, err = self.run_bzr(['unshelve', '--preview'], encoding='latin-1')
+        
+        self.assertContainsString(out, diff_part)
 
 
 class TestShelveRelpath(TestCaseWithTransport):
@@ -97,3 +116,18 @@
         tree.add('file')
         os.chdir('tree/dir')
         self.run_bzr('shelve --all ../file')
+
+
+class TestShelveUnshelve(TestCaseWithTransport):
+
+    def test_directory(self):
+        """Test --directory option"""
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([('tree/a', 'initial\n')])
+        tree.add('a')
+        tree.commit(message='committed')
+        self.build_tree_contents([('tree/a', 'initial\nmore\n')])
+        self.run_bzr('shelve -d tree --all')
+        self.assertFileEqual('initial\n', 'tree/a')
+        self.run_bzr('unshelve --directory tree')
+        self.assertFileEqual('initial\nmore\n', 'tree/a')

=== modified file 'bzrlib/tests/blackbox/test_status.py'
--- bzrlib/tests/blackbox/test_status.py	2010-04-07 21:34:13 +0000
+++ bzrlib/tests/blackbox/test_status.py	2010-09-12 05:09:24 +0000
@@ -33,6 +33,7 @@
     conflicts,
     errors,
     osutils,
+    status,
     )
 import bzrlib.branch
 from bzrlib.osutils import pathjoin
@@ -520,6 +521,27 @@
         out, err = self.run_bzr('status branch1 -rbranch:branch2')
         self.assertEqual('', out)
 
+    def test_status_with_shelves(self):
+        """Ensure that _show_shelve_summary handler works.
+        """
+        wt = self.make_branch_and_tree('.')
+        self.build_tree(['hello.c'])
+        wt.add('hello.c')
+        self.run_bzr(['shelve', '--all', '-m', 'foo'])
+        self.build_tree(['bye.c'])
+        wt.add('bye.c')
+        # As TestCase.setUp clears all hooks, we install this default
+        # post_status hook handler for the test.
+        status.hooks.install_named_hook('post_status',
+            status._show_shelve_summary,
+            'bzr status')
+        self.assertStatus([
+                'added:\n',
+                '  bye.c\n',
+                '1 shelves exist. See "bzr shelve --list" for details.\n',
+            ],
+            wt)
+
 
 class CheckoutStatus(BranchStatus):
 
@@ -704,16 +726,6 @@
 
 class TestStatusEncodings(TestCaseWithTransport):
 
-    def setUp(self):
-        TestCaseWithTransport.setUp(self)
-        self.user_encoding = osutils._cached_user_encoding
-        self.stdout = sys.stdout
-
-    def tearDown(self):
-        osutils._cached_user_encoding = self.user_encoding
-        sys.stdout = self.stdout
-        TestCaseWithTransport.tearDown(self)
-
     def make_uncommitted_tree(self):
         """Build a branch with uncommitted unicode named changes in the cwd."""
         working_tree = self.make_branch_and_tree(u'.')
@@ -727,8 +739,7 @@
         return working_tree
 
     def test_stdout_ascii(self):
-        sys.stdout = StringIO()
-        osutils._cached_user_encoding = 'ascii'
+        self.overrideAttr(osutils, '_cached_user_encoding', 'ascii')
         working_tree = self.make_uncommitted_tree()
         stdout, stderr = self.run_bzr("status")
 
@@ -738,8 +749,7 @@
 """)
 
     def test_stdout_latin1(self):
-        sys.stdout = StringIO()
-        osutils._cached_user_encoding = 'latin-1'
+        self.overrideAttr(osutils, '_cached_user_encoding', 'latin-1')
         working_tree = self.make_uncommitted_tree()
         stdout, stderr = self.run_bzr('status')
 

=== modified file 'bzrlib/tests/blackbox/test_switch.py'
--- bzrlib/tests/blackbox/test_switch.py	2010-04-28 07:53:34 +0000
+++ bzrlib/tests/blackbox/test_switch.py	2010-06-20 22:54:30 +0000
@@ -22,11 +22,11 @@
 
 from bzrlib import osutils
 from bzrlib.workingtree import WorkingTree
-from bzrlib.tests.blackbox import ExternalBase
+from bzrlib.tests import TestCaseWithTransport
 from bzrlib.directory_service import directories
 
 
-class TestSwitch(ExternalBase):
+class TestSwitch(TestCaseWithTransport):
 
     def _create_sample_tree(self):
         tree = self.make_branch_and_tree('branch-1')
@@ -252,3 +252,21 @@
         self.assertLength(0, calls)
         out, err = self.run_bzr('switch ../branch2')
         self.assertLength(1, calls)
+
+    def test_switch_lightweight_directory(self):
+        """Test --directory option"""
+
+        # create a source branch
+        a_tree = self.make_branch_and_tree('a')
+        self.build_tree_contents([('a/a', 'initial\n')])
+        a_tree.add('a')
+        a_tree.commit(message='initial')
+
+        # clone and add a differing revision
+        b_tree = a_tree.bzrdir.sprout('b').open_workingtree()
+        self.build_tree_contents([('b/a', 'initial\nmore\n')])
+        b_tree.commit(message='more')
+
+        self.run_bzr('checkout --lightweight a checkout')
+        self.run_bzr('switch --directory checkout b')
+        self.assertFileEqual('initial\nmore\n', 'checkout/a')

=== modified file 'bzrlib/tests/blackbox/test_tags.py'
--- bzrlib/tests/blackbox/test_tags.py	2010-03-24 14:16:15 +0000
+++ bzrlib/tests/blackbox/test_tags.py	2010-10-03 05:08:21 +0000
@@ -132,33 +132,42 @@
 
         b1 = tree1.branch
         # note how the tag for revid-1 sorts after the one for revid-2
-        b1.tags.set_tag(u'tagA\u30d0', 'revid-2')
-        b1.tags.set_tag(u'tagB\u30d0', 'missing') # not present in repository
-        b1.tags.set_tag(u'tagC\u30d0', 'revid-1')
+        b1.tags.set_tag(u'tag1\u30d0', 'revid-2')
+        b1.tags.set_tag(u'tag10\u30d0', 'missing') # not present in repository
+        b1.tags.set_tag(u'tag2\u30d0', 'revid-1')
+
+        # natural order
+        out, err = self.run_bzr('tags -d branch1',
+                                encoding='utf-8')
+        self.assertEquals(err, '')
+        self.assertContainsRe(out, (u'^tag1\u30d0  *2\ntag2\u30d0  *1\n' +
+            u'tag10\u30d0 *\\?\n').encode('utf-8'))
 
         # lexicographical order
-        out, err = self.run_bzr('tags -d branch1', encoding='utf-8')
+        out, err = self.run_bzr('tags --sort=alpha -d branch1',
+                                encoding='utf-8')
         self.assertEquals(err, '')
-        self.assertContainsRe(out, (u'^tagA\u30d0  *2\ntagB\u30d0  *\\?\n' +
-            u'tagC\u30d0 *1\n').encode('utf-8'))
+        self.assertContainsRe(out, (u'^tag10\u30d0  *\\?\ntag1\u30d0  *2\n' +
+            u'tag2\u30d0 *1\n').encode('utf-8'))
 
-        out, err = self.run_bzr('tags --show-ids -d branch1', encoding='utf-8')
+        out, err = self.run_bzr('tags --sort=alpha --show-ids -d branch1',
+                                encoding='utf-8')
         self.assertEquals(err, '')
-        self.assertContainsRe(out, (u'^tagA\u30d0  *revid-2\n' +
-            u'tagB\u30d0  *missing\ntagC\u30d0 *revid-1\n').encode('utf-8'))
+        self.assertContainsRe(out, (u'^tag10\u30d0  *missing\n' +
+            u'tag1\u30d0  *revid-2\ntag2\u30d0 *revid-1\n').encode('utf-8'))
 
         # chronological order
         out, err = self.run_bzr('tags --sort=time -d branch1',
                 encoding='utf-8')
         self.assertEquals(err, '')
-        self.assertContainsRe(out, (u'^tagC\u30d0  *1\ntagA\u30d0  *2\n' +
-            u'tagB\u30d0 *\\?\n').encode('utf-8'))
+        self.assertContainsRe(out, (u'^tag2\u30d0  *1\ntag1\u30d0  *2\n' +
+            u'tag10\u30d0 *\\?\n').encode('utf-8'))
 
         out, err = self.run_bzr('tags --sort=time --show-ids -d branch1',
                 encoding='utf-8')
         self.assertEquals(err, '')
-        self.assertContainsRe(out, (u'^tagC\u30d0  *revid-1\n' +
-            u'tagA\u30d0  *revid-2\ntagB\u30d0 *missing\n').encode('utf-8'))
+        self.assertContainsRe(out, (u'^tag2\u30d0  *revid-1\n' +
+            u'tag1\u30d0  *revid-2\ntag10\u30d0 *missing\n').encode('utf-8'))
 
         # now test dotted revnos
         tree2 = tree1.bzrdir.sprout('branch2').open_workingtree()

=== modified file 'bzrlib/tests/blackbox/test_too_much.py'
--- bzrlib/tests/blackbox/test_too_much.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/blackbox/test_too_much.py	2010-06-11 07:32:12 +0000
@@ -46,11 +46,11 @@
 from bzrlib.errors import BzrCommandError
 from bzrlib.tests.http_utils import TestCaseWithWebserver
 from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
-from bzrlib.tests.blackbox import ExternalBase
+from bzrlib.tests import TestCaseWithTransport
 from bzrlib.workingtree import WorkingTree
 
 
-class TestCommands(ExternalBase):
+class TestCommands(TestCaseWithTransport):
 
     def test_invalid_commands(self):
         self.run_bzr("pants", retcode=3)
@@ -354,7 +354,7 @@
     return L
 
 
-class OldTests(ExternalBase):
+class OldTests(TestCaseWithTransport):
     """old tests moved from ./testbzr."""
 
     def test_bzr(self):

=== modified file 'bzrlib/tests/blackbox/test_uncommit.py'
--- bzrlib/tests/blackbox/test_uncommit.py	2010-03-01 16:18:43 +0000
+++ bzrlib/tests/blackbox/test_uncommit.py	2010-09-15 09:35:42 +0000
@@ -22,7 +22,10 @@
 from bzrlib.bzrdir import BzrDirMetaFormat1
 from bzrlib.errors import BzrError, BoundBranchOutOfDate
 from bzrlib.tests import TestCaseWithTransport
-from bzrlib.tests.script import ScriptRunner
+from bzrlib.tests.script import (
+    run_script,
+    ScriptRunner,
+    )
 
 
 class TestUncommit(TestCaseWithTransport):
@@ -61,6 +64,20 @@
         out, err = self.run_bzr('status')
         self.assertEquals(out, 'modified:\n  a\n')
 
+    def test_uncommit_interactive(self):
+        """Uncommit seeks confirmation, and doesn't proceed without it."""
+        wt = self.create_simple_tree()
+        os.chdir('tree')
+        run_script(self, """    
+        $ bzr uncommit
+        ...
+        The above revision(s) will be removed.
+        2>Uncommit these revisions? [y/n]: 
+        <n
+        Canceled
+        """)
+        self.assertEqual(['a2'], wt.get_parent_ids())
+
     def test_uncommit_no_history(self):
         wt = self.make_branch_and_tree('tree')
         out, err = self.run_bzr('uncommit --force', retcode=1)

=== modified file 'bzrlib/tests/blackbox/test_unknowns.py'
--- bzrlib/tests/blackbox/test_unknowns.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_unknowns.py	2010-06-11 07:32:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007-2010 Canonical Ltd
 # -*- coding: utf-8 -*-
 #
 # This program is free software; you can redistribute it and/or modify
@@ -18,10 +18,10 @@
 
 """Black-box tests for 'bzr unknowns', which shows unknown files."""
 
-from bzrlib.tests.blackbox import ExternalBase
-
-
-class TestUnknowns(ExternalBase):
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestUnknowns(TestCaseWithTransport):
 
     def test_unknowns(self):
         """Test that 'unknown' command reports unknown files"""
@@ -45,3 +45,10 @@
         # after all added, none shown
         tree.add(['b', 'c'])
         self.assertEquals(self.run_bzr('unknowns')[0], '')
+
+    def test_unknowns_directory(self):
+        """Test --directory option"""
+        tree = self.make_branch_and_tree('a')
+        self.build_tree(['a/README'])
+        out, err = self.run_bzr(['unknowns', '--directory=a'])
+        self.assertEquals('README\n', out)

=== modified file 'bzrlib/tests/blackbox/test_update.py'
--- bzrlib/tests/blackbox/test_update.py	2010-04-14 04:48:00 +0000
+++ bzrlib/tests/blackbox/test_update.py	2010-09-27 20:22:57 +0000
@@ -359,6 +359,44 @@
 2>Updated to revision 2 of branch .../master
 ''')
 
+    def test_update_show_base(self):
+        """bzr update support --show-base
+
+        see https://bugs.launchpad.net/bzr/+bug/202374"""
+
+        tree=self.make_branch_and_tree('.')
+
+        f = open('hello','wt')
+        f.write('foo')
+        f.close()
+        tree.add('hello')
+        tree.commit('fie')
+
+        f = open('hello','wt')
+        f.write('fee')
+        f.close()
+        tree.commit('fee')
+
+        #tree.update() gives no such revision, so ...
+        self.run_bzr(['update','-r1'])
+
+        #create conflict
+        f = open('hello','wt')
+        f.write('fie')
+        f.close()
+
+        out, err = self.run_bzr(['update','--show-base'],retcode=1)
+
+        # check for conflict notification
+        self.assertContainsString(err,
+                                  ' M  hello\nText conflict in hello\n1 conflicts encountered.\n')
+        
+        self.assertEqualDiff('<<<<<<< TREE\n'
+                             'fie||||||| BASE-REVISION\n'
+                             'foo=======\n'
+                             'fee>>>>>>> MERGE-SOURCE\n',
+                             open('hello').read())
+
     def test_update_checkout_prevent_double_merge(self):
         """"Launchpad bug 113809 in bzr "update performs two merges"
         https://launchpad.net/bugs/113809"""

=== modified file 'bzrlib/tests/blackbox/test_upgrade.py'
--- bzrlib/tests/blackbox/test_upgrade.py	2010-04-16 07:56:51 +0000
+++ bzrlib/tests/blackbox/test_upgrade.py	2010-08-02 14:09:16 +0000
@@ -20,15 +20,14 @@
 
 from bzrlib import (
     bzrdir,
-    repository,
+    controldir,
+    transport,
     )
 from bzrlib.tests import (
     features,
-    TestCaseInTempDir,
     TestCaseWithTransport,
     )
 from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
-from bzrlib.transport import get_transport
 from bzrlib.repofmt.knitrepo import (
     RepositoryFormatKnit1,
     )
@@ -38,8 +37,8 @@
 
     def setUp(self):
         super(TestWithUpgradableBranches, self).setUp()
-        self.addCleanup(bzrdir.BzrDirFormat._set_default_format,
-                        bzrdir.BzrDirFormat.get_default_format())
+        self.addCleanup(controldir.ControlDirFormat._set_default_format,
+                        controldir.ControlDirFormat.get_default_format())
 
     def make_current_format_branch_and_checkout(self):
         current_tree = self.make_branch_and_tree('current_format_branch',
@@ -79,7 +78,8 @@
         (out, err) = self.run_bzr('upgrade current_format_checkout', retcode=3)
         self.assertEqual("This is a checkout. The branch (%s) needs to be "
                          "upgraded separately.\n"
-                         % get_transport(self.get_url('current_format_branch')).base,
+                         % transport.get_transport(
+                self.get_url('current_format_branch')).base,
                          out)
         self.assertEqualDiff("bzr: ERROR: The branch format Meta "
                              "directory format 1 is already at the most "
@@ -101,9 +101,9 @@
     def test_upgrade_explicit_metaformat(self):
         # users can force an upgrade to metadir format.
         self.make_format_5_branch()
-        url = get_transport(self.get_url('format_5_branch')).base
+        url = transport.get_transport(self.get_url('format_5_branch')).base
         # check --format takes effect
-        bzrdir.BzrDirFormat._set_default_format(bzrdir.BzrDirFormat5())
+        controldir.ControlDirFormat._set_default_format(bzrdir.BzrDirFormat5())
         backup_dir = 'backup.bzr.~1~'
         (out, err) = self.run_bzr(
             ['upgrade', '--format=metaweave', url])
@@ -125,9 +125,9 @@
         # users can force an upgrade to knit format from a metadir weave
         # branch
         self.make_metadir_weave_branch()
-        url = get_transport(self.get_url('metadir_weave_branch')).base
+        url = transport.get_transport(self.get_url('metadir_weave_branch')).base
         # check --format takes effect
-        bzrdir.BzrDirFormat._set_default_format(bzrdir.BzrDirFormat5())
+        controldir.ControlDirFormat._set_default_format(bzrdir.BzrDirFormat5())
         backup_dir = 'backup.bzr.~1~'
         (out, err) = self.run_bzr(
             ['upgrade', '--format=knit', url])
@@ -163,14 +163,14 @@
 
     def test_upgrade_with_existing_backup_dir(self):
         self.make_format_5_branch()
-        transport = get_transport(self.get_url('format_5_branch'))
-        url = transport.base
-        bzrdir.BzrDirFormat._set_default_format(bzrdir.BzrDirFormat5())
+        t = transport.get_transport(self.get_url('format_5_branch'))
+        url = t.base
+        controldir.ControlDirFormat._set_default_format(bzrdir.BzrDirFormat5())
         backup_dir1 = 'backup.bzr.~1~'
         backup_dir2 = 'backup.bzr.~2~'
         # explicitly create backup_dir1. bzr should create the .~2~ directory
         # as backup
-        transport.mkdir(backup_dir1)
+        t.mkdir(backup_dir1)
         (out, err) = self.run_bzr(
             ['upgrade', '--format=metaweave', url])
         self.assertEqualDiff("""starting upgrade of %s
@@ -186,14 +186,14 @@
         self.assertTrue(isinstance(
             bzrdir.BzrDir.open(self.get_url('format_5_branch'))._format,
             bzrdir.BzrDirMetaFormat1))
-        self.assertTrue(transport.has(backup_dir2))
+        self.assertTrue(t.has(backup_dir2))
 
 class SFTPTests(TestCaseWithSFTPServer):
     """Tests for upgrade over sftp."""
 
     def test_upgrade_url(self):
         self.run_bzr('init --format=weave')
-        t = get_transport(self.get_url())
+        t = transport.get_transport(self.get_url())
         url = t.base
         out, err = self.run_bzr(['upgrade', '--format=knit', url])
         backup_dir = 'backup.bzr.~1~'

=== modified file 'bzrlib/tests/blackbox/test_version_info.py'
--- bzrlib/tests/blackbox/test_version_info.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_version_info.py	2010-09-29 12:13:52 +0000
@@ -151,3 +151,19 @@
             '"{revno} {branch_nick} {clean}\n" branch')
         self.assertEqual("2 branch 0\n", out)
         self.assertEqual("", err)
+    
+    def test_non_ascii(self):
+        """Test that we can output non-ascii data"""
+        
+        commit_message = u'Non-ascii message with character not in latin-1: \u1234'
+        
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['a_file'])
+        tree.add('a_file')
+        tree.commit(commit_message)
+        out, err = self.run_bzr(
+            ['version-info', '--include-history'], encoding='latin-1')
+        
+        self.assertContainsString(out, commit_message.encode('utf-8'))
+
+

=== modified file 'bzrlib/tests/blackbox/test_versioning.py'
--- bzrlib/tests/blackbox/test_versioning.py	2010-03-02 15:49:51 +0000
+++ bzrlib/tests/blackbox/test_versioning.py	2010-05-20 18:23:17 +0000
@@ -33,7 +33,7 @@
 
     def test_mkdir_fails_cleanly(self):
         """'mkdir' fails cleanly when no working tree is available.
-        https://bugs.edge.launchpad.net/bzr/+bug/138600
+        https://bugs.launchpad.net/bzr/+bug/138600
         """
         # Since there is a safety working tree above us, we create a bare repo
         # here locally.

=== modified file 'bzrlib/tests/blackbox/test_whoami.py'
--- bzrlib/tests/blackbox/test_whoami.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_whoami.py	2010-06-23 08:19:28 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,11 +20,14 @@
 import os
 
 import bzrlib
-from bzrlib.branch import Branch
-from bzrlib.tests.blackbox import ExternalBase
-
-
-class TestWhoami(ExternalBase):
+from bzrlib import (
+    osutils,
+    config,
+    )
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestWhoami(TestCaseWithTransport):
 
     def test_whoami(self):
         # this should always identify something, if only "john@localhost"
@@ -42,45 +45,29 @@
         b = bzrlib.branch.Branch.open('.')
         b.get_config().set_user_option('email',
                                        'Branch Identity <branch@identi.ty>')
-        bzr_email = os.environ.get('BZR_EMAIL')
-        if bzr_email is not None:
-            del os.environ['BZR_EMAIL']
-        try:
-            whoami = self.run_bzr("whoami")[0]
-            self.assertEquals('Branch Identity <branch@identi.ty>\n', whoami)
-            whoami_email = self.run_bzr("whoami --email")[0]
-            self.assertEquals('branch@identi.ty\n', whoami_email)
+        whoami = self.run_bzr("whoami")[0]
+        self.assertEquals('Branch Identity <branch@identi.ty>\n', whoami)
+        whoami_email = self.run_bzr("whoami --email")[0]
+        self.assertEquals('branch@identi.ty\n', whoami_email)
 
-            # Verify that the environment variable overrides the value
-            # in the file
-            os.environ['BZR_EMAIL'] = 'Different ID <other@environ.ment>'
-            whoami = self.run_bzr("whoami")[0]
-            self.assertEquals('Different ID <other@environ.ment>\n', whoami)
-            whoami_email = self.run_bzr("whoami --email")[0]
-            self.assertEquals('other@environ.ment\n', whoami_email)
-            del os.environ['BZR_EMAIL']
-        finally:
-            if bzr_email is not None:
-                os.environ['BZR_EMAIL'] = bzr_email
+        # Verify that the environment variable overrides the value
+        # in the file
+        os.environ['BZR_EMAIL'] = 'Different ID <other@environ.ment>'
+        whoami = self.run_bzr("whoami")[0]
+        self.assertEquals('Different ID <other@environ.ment>\n', whoami)
+        whoami_email = self.run_bzr("whoami --email")[0]
+        self.assertEquals('other@environ.ment\n', whoami_email)
 
     def test_whoami_utf8(self):
         """verify that an identity can be in utf-8."""
         wt = self.make_branch_and_tree('.')
         self.run_bzr(['whoami', u'Branch Identity \u20ac <branch@identi.ty>'],
                      encoding='utf-8')
-        bzr_email = os.environ.get('BZR_EMAIL')
-        if bzr_email is not None:
-            del os.environ['BZR_EMAIL']
-        try:
-            whoami = self.run_bzr("whoami", encoding='utf-8')[0]
-            self.assertEquals('Branch Identity \xe2\x82\xac ' +
-                              '<branch@identi.ty>\n', whoami)
-            whoami_email = self.run_bzr("whoami --email",
-                                        encoding='utf-8')[0]
-            self.assertEquals('branch@identi.ty\n', whoami_email)
-        finally:
-            if bzr_email is not None:
-                os.environ['BZR_EMAIL'] = bzr_email
+        whoami = self.run_bzr("whoami", encoding='utf-8')[0]
+        self.assertEquals('Branch Identity \xe2\x82\xac ' +
+                          '<branch@identi.ty>\n', whoami)
+        whoami_email = self.run_bzr("whoami --email", encoding='utf-8')[0]
+        self.assertEquals('branch@identi.ty\n', whoami_email)
 
     def test_whoami_ascii(self):
         """
@@ -91,18 +78,10 @@
         b = bzrlib.branch.Branch.open('.')
         b.get_config().set_user_option('email', u'Branch Identity \u20ac ' +
                                        '<branch@identi.ty>')
-        bzr_email = os.environ.get('BZR_EMAIL')
-        if bzr_email is not None:
-            del os.environ['BZR_EMAIL']
-        try:
-            whoami = self.run_bzr("whoami", encoding='ascii')[0]
-            self.assertEquals('Branch Identity ? <branch@identi.ty>\n', whoami)
-            whoami_email = self.run_bzr("whoami --email",
-                                        encoding='ascii')[0]
-            self.assertEquals('branch@identi.ty\n', whoami_email)
-        finally:
-            if bzr_email is not None:
-                os.environ['BZR_EMAIL'] = bzr_email
+        whoami = self.run_bzr("whoami", encoding='ascii')[0]
+        self.assertEquals('Branch Identity ? <branch@identi.ty>\n', whoami)
+        whoami_email = self.run_bzr("whoami --email", encoding='ascii')[0]
+        self.assertEquals('branch@identi.ty\n', whoami_email)
 
     def test_warning(self):
         """verify that a warning is displayed if no email is given."""
@@ -111,3 +90,47 @@
         self.assertEquals('"Branch Identity" does not seem to contain an '
                           'email address.  This is allowed, but not '
                           'recommended.\n', display)
+
+    def test_whoami_not_set(self):
+        """Ensure whoami error if username is not set.
+        """
+        osutils.set_or_unset_env('EMAIL', None)
+        osutils.set_or_unset_env('BZR_EMAIL', None)
+        out, err = self.run_bzr(['whoami'], 3)
+        self.assertContainsRe(err, 'Unable to determine your name')
+
+    def test_whoami_directory(self):
+        """Test --directory option."""
+        wt = self.make_branch_and_tree('subdir')
+        c = wt.branch.get_config()
+        c.set_user_option('email', 'Branch Identity <branch@identi.ty>')
+        out, err = self.run_bzr("whoami --directory subdir")
+        self.assertEquals('Branch Identity <branch@identi.ty>\n', out)
+        self.run_bzr(['whoami', '--directory', 'subdir', '--branch',
+                      'Changed Identity <changed@identi.ty>'])
+        self.assertEquals('Changed Identity <changed@identi.ty>',
+                          c.get_user_option('email'))
+
+    def test_whoami_remote_directory(self):
+        """Test --directory option with a remote directory."""
+        wt = self.make_branch_and_tree('subdir')
+        c = wt.branch.get_config()
+        c.set_user_option('email', 'Branch Identity <branch@identi.ty>')
+        url = self.get_readonly_url() + '/subdir'
+        out, err = self.run_bzr(['whoami', '--directory', url])
+        self.assertEquals('Branch Identity <branch@identi.ty>\n', out)
+        url = self.get_url('subdir')
+        self.run_bzr(['whoami', '--directory', url, '--branch',
+                      'Changed Identity <changed@identi.ty>'])
+        # The identity has been set in the branch config (but not the global
+        # config)
+        self.assertEquals('Changed Identity <changed@identi.ty>',
+                          c.get_user_option('email'))
+        global_conf = config.GlobalConfig()
+        self.assertEquals(None, global_conf.get_user_option('email'))
+
+    def test_whoami_nonbranch_directory(self):
+        """Test --directory mentioning a non-branch directory."""
+        wt = self.build_tree(['subdir/'])
+        out, err = self.run_bzr("whoami --directory subdir", retcode=3)
+        self.assertContainsRe(err, 'ERROR: Not a branch')

=== modified file 'bzrlib/tests/commands/test_checkout.py'
--- bzrlib/tests/commands/test_checkout.py	2010-04-09 03:58:14 +0000
+++ bzrlib/tests/commands/test_checkout.py	2010-05-03 08:33:32 +0000
@@ -14,7 +14,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-
 from bzrlib.builtins import cmd_checkout
 from bzrlib.tests.transport_util import TestCaseWithConnectionHookedTransport
 

=== added directory 'bzrlib/tests/doc_generate'
=== added file 'bzrlib/tests/doc_generate/__init__.py'
--- bzrlib/tests/doc_generate/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/doc_generate/__init__.py	2010-07-21 08:07:14 +0000
@@ -0,0 +1,108 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Documentation generation tests."""
+
+import os
+from bzrlib import tests
+from bzrlib.doc_generate import (
+    # FIXME: doc/en/conf.py should be used here, or rather merged into
+    # bzrlib/doc_generate/conf.py -- vila 20100429
+    conf,
+    )
+from bzrlib.tests import features
+
+
+def load_tests(basic_tests, module, loader):
+    suite = loader.suiteClass()
+    # add the tests for this module
+    suite.addTests(basic_tests)
+
+    testmod_names = [
+        'builders',
+        'writers',
+        ]
+    # add the tests for the sub modules
+    suite.addTests(loader.loadTestsFromModuleNames(
+            ['bzrlib.tests.doc_generate.' + name
+             for name in testmod_names]))
+
+    return suite
+
+
+class TestSphinx(tests.TestCaseInTempDir):
+    """Base class for sphinx tests.
+
+    This is used for both the builder and the writer until a better solution is
+    found to test at a lower level.
+    """
+
+    _test_needs_features = [features.sphinx]
+
+    def make_sphinx(self):
+        out = tests.StringIOWrapper()
+        err = tests.StringIOWrapper()
+        from sphinx import application
+        app = application.Sphinx(
+            '.', confdir=os.path.dirname(conf.__file__), outdir='.',
+            doctreedir='.',
+            buildername='texinfo',
+            confoverrides={},
+            status=out, warning=err,
+            freshenv=True)
+        return app, out, err
+
+    def build(self, app, all_files=True, file_names=None):
+        if file_names is None:
+            file_names = []
+        app.build(all_files, file_names)
+
+    # FIXME: something smells wrong here as we can't process a single file
+    # alone. On top of that, it seems the doc tree must contain an index.txt
+    # file. We may need a texinfo builder ? -- vila 20100505
+
+    def create_content(self, content):
+        """Put content into a single file.
+
+        This is appropriate for simple tests about the content of a single file.
+        """
+        app, out, err = self.make_sphinx()
+        self.build_tree_contents([('index.txt', content),])
+        self.build(app)
+
+    def assertContent(self, expected, header=None, end=None):
+        """Check the content of the file created with creste_content().
+
+        Most texinfo constructs can be tested this way without caring for any
+        boilerplate that texinfo may require at the beginning or the end of the
+        file.
+        """
+        if header is None:
+            # default boilerplate
+            header = '''\
+This file has been converted using a beta rst->texinfo converter. 
+Most of the info links are currently bogus, don't report bugs about them,
+this is currently worked on.
+@node Top
+@top Placeholder
+'''
+        if end is None:
+            # By default we test constructs that are embedded into a paragraph
+            # which always end with two \n (even if the input has none)
+            end = '\n\n'
+        self.assertFileEqual(header + expected + end, 'index.texi')
+
+

=== added directory 'bzrlib/tests/doc_generate/builders'
=== added file 'bzrlib/tests/doc_generate/builders/__init__.py'
--- bzrlib/tests/doc_generate/builders/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/doc_generate/builders/__init__.py	2010-04-29 16:10:59 +0000
@@ -0,0 +1,36 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+
+"""Sphinx builders tests."""
+
+from bzrlib.tests import features
+
+def load_tests(basic_tests, module, loader):
+    suite = loader.suiteClass()
+    # add the tests for this module
+    suite.addTests(basic_tests)
+
+    if features.sphinx.available():
+        testmod_names = [
+            'texinfo',
+            ]
+        # add the tests for the sub modules
+        suite.addTests(loader.loadTestsFromModuleNames(
+                ['bzrlib.tests.doc_generate.builders.test_' + name
+                 for name in testmod_names]))
+
+    return suite

=== added file 'bzrlib/tests/doc_generate/builders/test_texinfo.py'
--- bzrlib/tests/doc_generate/builders/test_texinfo.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/doc_generate/builders/test_texinfo.py	2010-07-06 13:22:32 +0000
@@ -0,0 +1,71 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""sphinx texinfo builder tests."""
+
+from bzrlib import tests
+from bzrlib.doc_generate import (
+    # FIXME: doc/en/conf.py should be used here, or rather merged into
+    # bzrlib/doc_generate/conf.py -- vila 20100429
+    conf,
+    )
+from bzrlib.tests import (
+    doc_generate as test_dg, # Avoid clash with from bzrlib import doc_generate
+    )
+
+
+class TestBuilderDefined(tests.TestCase):
+
+    def test_builder_defined(self):
+        self.assertTrue('bzrlib.doc_generate.builders.texinfo'
+                        in conf.extensions)
+
+class TestBuilderLoaded(test_dg.TestSphinx):
+
+    def test_builder_loaded(self):
+        app, out, err = self.make_sphinx()
+        self.assertTrue('texinfo' in app.builderclasses)
+
+
+class TestFileProduction(test_dg.TestSphinx):
+
+    def test_files_generated(self):
+        self.build_tree_contents(
+            [('index.txt', """
+Table of Contents
+=================
+
+.. toctree::
+   :maxdepth: 1
+
+   content
+"""),
+             ('content.txt', """
+
+bzr 0.0.8
+*********
+
+Improvements
+============
+
+* Adding a file whose parent directory is not versioned will
+  implicitly add the parent, and so on up to the root.
+"""),
+             ])
+        app, out, err = self.make_sphinx()
+        self.build(app)
+        self.failUnlessExists('index.texi')
+        self.failUnlessExists('content.texi')

=== added directory 'bzrlib/tests/doc_generate/writers'
=== added file 'bzrlib/tests/doc_generate/writers/__init__.py'
--- bzrlib/tests/doc_generate/writers/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/doc_generate/writers/__init__.py	2010-07-06 13:22:32 +0000
@@ -0,0 +1,36 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+
+"""Sphinx writers tests."""
+
+from bzrlib.tests import features
+
+def load_tests(basic_tests, module, loader):
+    suite = loader.suiteClass()
+    # add the tests for this module
+    suite.addTests(basic_tests)
+
+    if features.sphinx.available():
+        testmod_names = [
+            'texinfo',
+            ]
+        # add the tests for the sub modules
+        suite.addTests(loader.loadTestsFromModuleNames(
+                ['bzrlib.tests.doc_generate.writers.test_' + name
+                 for name in testmod_names]))
+
+    return suite

=== added file 'bzrlib/tests/doc_generate/writers/test_texinfo.py'
--- bzrlib/tests/doc_generate/writers/test_texinfo.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/doc_generate/writers/test_texinfo.py	2010-07-21 08:07:14 +0000
@@ -0,0 +1,335 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""sphinx texinfo writer tests."""
+
+from bzrlib.tests import (
+    doc_generate as test_dg, # Avoid clash with from bzrlib import doc_generate
+    )
+
+
+class TestTextGeneration(test_dg.TestSphinx):
+
+    def test_special_chars(self):
+        self.create_content("A '@' a '{' and a '}'")
+        self.assertContent("A '@@' a '@{' and a '@}'")
+
+    def test_emphasis(self):
+        self.create_content('*important*')
+        self.assertContent('@emph{important}')
+
+    def test_strong(self):
+        self.create_content('**very important**')
+        self.assertContent('@strong{very important}')
+
+    def test_literal(self):
+        self.create_content('the command is ``foo``')
+        self.assertContent('the command is @code{foo}')
+
+    def test_paragraphs(self):
+        self.create_content('''\
+This is a paragraph.
+
+This is another one.
+''')
+        self.assertContent('''\
+This is a paragraph.
+
+This is another one.''')
+
+    def test_literal_block(self):
+        self.create_content('''\
+Do this::
+
+   bzr xxx
+   bzr yyy
+''')
+        self.assertContent('''\
+Do this:
+
+@samp{bzr xxx
+bzr yyy}
+
+''',
+                           end='')
+
+    def test_block_quote(self):
+        self.create_content('''\
+This is an ordinary paragraph, introducing a block quote.
+
+    "It is my business to know things.  That is my trade."
+
+This is another ordinary paragraph.
+''')
+        self.assertContent('''\
+This is an ordinary paragraph, introducing a block quote.
+
+@example
+"It is my business to know things.  That is my trade."
+
+@end example
+
+This is another ordinary paragraph.
+
+''',
+                           # examples are not followed by an empty line
+                           end='')
+
+
+class TestDocumentAttributesGeneration(test_dg.TestSphinx):
+
+    def test_title(self):
+        self.create_content('''\
+####################
+Bazaar Release Notes
+####################
+''')
+        self.assertContent('''\
+@node bazaar-release-notes
+@chapter Bazaar Release Notes
+''',
+                           end='')
+
+
+class TestListGeneration(test_dg.TestSphinx):
+
+    def test_bullet_list(self):
+        self.create_content('''\
+* This is a bulleted list.
+* It has two items, the second
+  item uses two lines.
+''')
+        self.assertContent('''\
+@itemize @bullet
+@item
+This is a bulleted list.
+
+@item
+It has two items, the second
+item uses two lines.
+
+@end itemize
+''',
+                           end='')
+
+    def test_enumerated_list(self):
+        self.create_content('''\
+#. This is a numbered list.
+#. It has two items, the second
+   item uses two lines.
+''')
+        self.assertContent('''\
+@enumerate
+@item
+This is a numbered list.
+
+@item
+It has two items, the second
+item uses two lines.
+
+@end enumerate
+''',
+                           end='')
+
+
+class TestTableGeneration(test_dg.TestSphinx):
+
+    def test_table(self):
+        self.create_content('''\
+  ===========         ================
+  Prefix              Description
+  ===========         ================
+  first               The first
+  second              The second
+  last                The last
+  ===========         ================
+''')
+        # FIXME: Sphinx bug ? Why are tables enclosed in a block_quote
+        # (translated as an @example).
+        self.assertContent('''\
+@example
+@multitable {xxxxxxxxxxx}{xxxxxxxxxxxxxxxx}
+@headitem Prefix @tab Description
+@item first
+@tab The first
+@item second
+@tab The second
+@item last
+@tab The last
+@end multitable
+@end example''')
+
+
+class TestTocTreeGeneration(test_dg.TestSphinx):
+
+    def test_toctree(self):
+        self.build_tree_contents(
+            [('index.txt', """
+Table of Contents
+=================
+
+.. toctree::
+   :maxdepth: 1
+
+   bzr 0.0.8 <bzr-0.0.8>
+"""),
+             ('bzr-0.0.8.txt', """
+
+bzr 0.0.8
+*********
+
+Improvements
+============
+
+* Adding a file whose parent directory is not versioned will
+  implicitly add the parent, and so on up to the root.
+"""),
+             ])
+        app, out, err = self.make_sphinx()
+        self.build(app)
+        self.assertFileEqual("""\
+This file has been converted using a beta rst->texinfo converter. 
+Most of the info links are currently bogus, don't report bugs about them,
+this is currently worked on.
+@node Top
+@top Placeholder
+@node table-of-contents
+@chapter Table of Contents
+@menu
+* bzr 0.0.8: (bzr-0.0.8.info)bzr 0.0.8. 
+@end menu
+""",
+                             'index.texi')
+        self.assertFileEqual("""\
+This file has been converted using a beta rst->texinfo converter. 
+Most of the info links are currently bogus, don't report bugs about them,
+this is currently worked on.
+@node Top
+@top Placeholder
+@node bzr-0-0-8
+@chapter bzr 0.0.8
+@node improvements
+@section Improvements
+@itemize @bullet
+@item
+Adding a file whose parent directory is not versioned will
+implicitly add the parent, and so on up to the root.
+
+@end itemize
+""",
+                             'bzr-0.0.8.texi')
+
+class TestSections(test_dg.TestSphinx):
+
+    def test_sections(self):
+        self.create_content('''\
+###########
+Chapter one
+###########
+
+Chapter introduction.
+
+***********
+section one
+***********
+
+The first section.
+
+
+subsection one
+==============
+
+The first subsection.
+
+subsection two
+==============
+
+The second subsection.
+
+subsubsection one
+-----------------
+
+Here is sus sub section one.
+
+blob one
+^^^^^^^^
+
+Far tooo deep to get a name
+
+thing one
+"""""""""
+
+No idea how to call that, but sphinx says it's a paragraph.
+''')
+        self.assertContent('''\
+@node chapter-one
+@chapter Chapter one
+Chapter introduction.
+
+@node section-one
+@section section one
+The first section.
+
+@node subsection-one
+@subsection subsection one
+The first subsection.
+
+@node subsection-two
+@subsection subsection two
+The second subsection.
+
+@node subsubsection-one
+@subsubsection subsubsection one
+Here is sus sub section one.
+
+@node blob-one
+@heading blob one
+Far tooo deep to get a name
+
+@node thing-one
+@heading thing one
+No idea how to call that, but sphinx says it's a paragraph.''')
+
+
+class TestReferences(test_dg.TestSphinx):
+
+    def test_external_reference(self):
+        self.create_content('''\
+The `example web site`_ is nice.
+
+.. _example web site: http://www.example.com/
+''')
+        self.assertContent('''\
+The @uref{http://www.example.com/,example web site} is nice.''')
+
+
+    def test_internal_reference(self):
+        self.create_content('''\
+The `example web site`_ contains more examples.
+
+Example web site
+----------------
+
+Here we have a lot of nice examples.
+
+''')
+        self.assertContent('''\
+The example web site (@pxref{example-web-site}) contains more examples.
+
+@node example-web-site
+@chapter Example web site
+Here we have a lot of nice examples.''')
+

=== modified file 'bzrlib/tests/features.py'
--- bzrlib/tests/features.py	2010-03-18 06:25:23 +0000
+++ bzrlib/tests/features.py	2010-09-27 19:31:45 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+"""A collection of commonly used 'Features' which bzrlib uses to skip tests."""
+
 import os
 import stat
 
@@ -21,10 +23,43 @@
 from bzrlib.symbol_versioning import deprecated_in
 
 
+class _NotRunningAsRoot(tests.Feature):
+
+    def _probe(self):
+        try:
+            uid = os.getuid()
+        except AttributeError:
+            # If there is no uid, chances are there is no root either
+            return True
+        return uid != 0
+
+    def feature_name(self):
+        return 'Not running as root'
+
+
+not_running_as_root = _NotRunningAsRoot()
 apport = tests.ModuleAvailableFeature('apport')
 paramiko = tests.ModuleAvailableFeature('paramiko')
 pycurl = tests.ModuleAvailableFeature('pycurl')
+pywintypes = tests.ModuleAvailableFeature('pywintypes')
 subunit = tests.ModuleAvailableFeature('subunit')
+sphinx = tests.ModuleAvailableFeature('sphinx')
+
+
+class _BackslashDirSeparatorFeature(tests.Feature):
+
+    def _probe(self):
+        try:
+            os.lstat(os.getcwd() + '\\')
+        except OSError:
+            return False
+        else:
+            return True
+
+    def feature_name(self):
+        return "Filesystem treats '\\' as a directory separator."
+
+backslashdir_feature = _BackslashDirSeparatorFeature()
 
 
 class _PosixPermissionsFeature(tests.Feature):
@@ -61,3 +96,37 @@
 
 chown_feature = _ChownFeature()
 
+
+class ExecutableFeature(tests.Feature):
+    """Feature testing whether an executable of a given name is on the PATH."""
+
+    def __init__(self, name):
+        super(ExecutableFeature, self).__init__()
+        self.name = name
+        self._path = None
+
+    @property
+    def path(self):
+        # This is a property, so accessing path ensures _probe was called
+        self.available()
+        return self._path
+
+    def _probe(self):
+        path = os.environ.get('PATH')
+        if path is None:
+            return False
+        for d in path.split(os.pathsep):
+            if d:
+                f = os.path.join(d, self.name)
+                if os.access(f, os.X_OK):
+                    self._path = f
+                    return True
+        return False
+
+    def feature_name(self):
+        return '%s executable' % self.name
+
+
+bash_feature = ExecutableFeature('bash')
+sed_feature = ExecutableFeature('sed')
+diff_feature = ExecutableFeature('diff')

=== added file 'bzrlib/tests/fixtures.py'
--- bzrlib/tests/fixtures.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/fixtures.py	2010-06-26 01:07:16 +0000
@@ -0,0 +1,99 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+
+"""Fixtures that can be used within tests.
+
+Fixtures can be created during a test as a way to separate out creation of
+objects to test.  Fixture objects can hold some state so that different 
+objects created during a test instance can be related.  Normally a fixture
+should live only for the duration of a single test, and its tearDown method
+should be passed to `addCleanup` on the test.
+"""
+
+
+import itertools
+
+
+def generate_unicode_names():
+    """Generate a sequence of arbitrary unique unicode names.
+    
+    By default they are not representable in ascii.
+    
+    >>> gen = generate_unicode_names()
+    >>> n1 = gen.next()
+    >>> n2 = gen.next()
+    >>> type(n1)
+    <type 'unicode'>
+    >>> n1 == n2
+    False
+    >>> n1.encode('ascii', 'replace') == n1
+    False
+    """
+    # include a mathematical symbol unlikely to be in 8-bit encodings
+    return (u"\N{SINE WAVE}%d" % x for x in itertools.count())
+
+
+interesting_encodings = [
+    ('iso-8859-1', False),
+    ('ascii', False),
+    ('cp850', False),
+    ('utf-8', True),
+    ('ucs-2', True),
+    ]
+
+
+def generate_unicode_encodings(universal_encoding=None):
+    """Return a generator of unicode encoding names.
+
+    These can be passed to Python encode/decode/etc.
+    
+    :param universal_encoding: True/False/None tristate to say whether the
+        generated encodings either can or cannot encode all unicode 
+        strings.
+
+    >>> n1 = generate_unicode_names().next()
+    >>> enc = generate_unicode_encodings(universal_encoding=True).next()
+    >>> enc2 = generate_unicode_encodings(universal_encoding=False).next()
+    >>> n1.encode(enc).decode(enc) == n1
+    True
+    >>> try:
+    ...   n1.encode(enc2).decode(enc2)
+    ... except UnicodeError:
+    ...   print 'fail'
+    fail
+    """
+    # TODO: check they're supported on this platform?
+    if universal_encoding is not None:
+        e = [n for (n, u) in interesting_encodings if u == universal_encoding]
+    else:
+        e = [n for (n, u) in interesting_encodings]
+    return itertools.cycle(iter(e))
+
+
+class RecordingContextManager(object):
+    """A context manager that records."""
+
+    def __init__(self):
+        self._calls = []
+
+    def __enter__(self):
+        self._calls.append('__enter__')
+        return self # This is bound to the 'as' clause in a with statement.
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self._calls.append('__exit__')
+        return False # propogate exceptions.

=== modified file 'bzrlib/tests/ftp_server/medusa_based.py'
--- bzrlib/tests/ftp_server/medusa_based.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/ftp_server/medusa_based.py	2010-06-30 15:19:36 +0000
@@ -254,6 +254,9 @@
         self._async_thread = threading.Thread(
                 target=FTPTestServer._asyncore_loop_ignore_EBADF,
                 kwargs={'timeout':0.1, 'count':10000})
+        if 'threads' in tests.selftest_debug_flags:
+            sys.stderr.write('Thread started: %s\n'
+                             % (self._async_thread.ident,))
         self._async_thread.setDaemon(True)
         self._async_thread.start()
 
@@ -261,6 +264,9 @@
         self._ftp_server.close()
         asyncore.close_all()
         self._async_thread.join()
+        if 'threads' in tests.selftest_debug_flags:
+            sys.stderr.write('Thread  joined: %s\n'
+                             % (self._async_thread.ident,))
 
     @staticmethod
     def _asyncore_loop_ignore_EBADF(*args, **kwargs):

=== modified file 'bzrlib/tests/ftp_server/pyftpdlib_based.py'
--- bzrlib/tests/ftp_server/pyftpdlib_based.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/ftp_server/pyftpdlib_based.py	2010-06-30 15:19:36 +0000
@@ -28,6 +28,7 @@
 
 from bzrlib import (
     osutils,
+    tests,
     trace,
     )
 from bzrlib.tests import test_server
@@ -183,6 +184,9 @@
         self._ftpd_starting.acquire() # So it can be released by the server
         self._ftpd_thread = threading.Thread(target=self._run_server,)
         self._ftpd_thread.start()
+        if 'threads' in tests.selftest_debug_flags:
+            sys.stderr.write('Thread started: %s\n'
+                             % (self._ftpd_thread.ident,))
         # Wait for the server thread to start (i.e release the lock)
         self._ftpd_starting.acquire()
         self._ftpd_starting.release()
@@ -195,6 +199,9 @@
         self._ftp_server.close()
         self._ftpd_running = False
         self._ftpd_thread.join()
+        if 'threads' in tests.selftest_debug_flags:
+            sys.stderr.write('Thread  joined: %s\n'
+                             % (self._ftpd_thread.ident,))
 
     def _run_server(self):
         """Run the server until stop_server is called, shut it down properly then.

=== modified file 'bzrlib/tests/http_server.py'
--- bzrlib/tests/http_server.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/http_server.py	2010-08-24 13:01:06 +0000
@@ -30,7 +30,11 @@
 import urllib
 import urlparse
 
-from bzrlib import transport
+from bzrlib import (
+    osutils,
+    tests,
+    transport,
+    )
 from bzrlib.tests import test_server
 from bzrlib.transport import local
 
@@ -71,6 +75,14 @@
                 self.headers.get('referer', '-'),
                 self.headers.get('user-agent', '-'))
 
+    def handle(self):
+        SimpleHTTPServer.SimpleHTTPRequestHandler.handle(self)
+        # Some client (pycurl, I'm looking at you) are more picky than others
+        # and require that the socket itself is closed
+        # (SocketServer.StreamRequestHandler only close the two associated
+        # 'makefile' objects)
+        self.connection.close()
+
     def handle_one_request(self):
         """Handle a single HTTP request.
 
@@ -78,7 +90,7 @@
         connection early to avoid polluting the test results.
         """
         try:
-            SimpleHTTPServer.SimpleHTTPRequestHandler.handle_one_request(self)
+            self._handle_one_request()
         except socket.error, e:
             # Any socket error should close the connection, but some errors are
             # due to the client closing early and we don't want to pollute test
@@ -89,6 +101,44 @@
                                      errno.ECONNABORTED, errno.EBADF)):
                 raise
 
+    error_content_type = 'text/plain'
+    error_message_format = '''\
+Error code: %(code)s.
+Message: %(message)s.
+'''
+
+    def send_error(self, code, message=None):
+        """Send and log an error reply.
+
+        We redefine the python-provided version to be able to set a 
+        ``Content-Length`` header as some http/1.1 clients complain otherwise
+        (see bug #568421).
+
+        :param code: The HTTP error code.
+
+        :param message: The explanation of the error code, Defaults to a short
+             entry.
+        """
+
+        if message is None:
+            try:
+                message = self.responses[code][0]
+            except KeyError:
+                message = '???'
+        self.log_error("code %d, message %s", code, message)
+        content = (self.error_message_format %
+                   {'code': code, 'message': message})
+        self.send_response(code, message)
+        self.send_header("Content-Type", self.error_content_type)
+        self.send_header("Content-Length", "%d" % len(content))
+        self.send_header('Connection', 'close')
+        self.end_headers()
+        if self.command != 'HEAD' and code >= 200 and code not in (204, 304):
+            self.wfile.write(content)
+
+    def _handle_one_request(self):
+        SimpleHTTPServer.SimpleHTTPRequestHandler.handle_one_request(self)
+
     _range_regexp = re.compile(r'^(?P<start>\d+)-(?P<end>\d+)$')
     _tail_regexp = re.compile(r'^-(?P<tail>\d+)$')
 
@@ -213,12 +263,12 @@
             # mode may cause newline translations, making the
             # actual size of the content transmitted *less* than
             # the content-length!
-            file = open(path, 'rb')
+            f = open(path, 'rb')
         except IOError:
             self.send_error(404, "File not found")
             return
 
-        file_size = os.fstat(file.fileno())[6]
+        file_size = os.fstat(f.fileno())[6]
         tail, ranges = self.parse_ranges(ranges_header_value)
         # Normalize tail into ranges
         if tail != 0:
@@ -245,7 +295,7 @@
             # RFC2616 14.16 and 14.35 says that when a server
             # encounters unsatisfiable range specifiers, it
             # SHOULD return a 416.
-            file.close()
+            f.close()
             # FIXME: We SHOULD send a Content-Range header too,
             # but the implementation of send_error does not
             # allows that. So far.
@@ -254,10 +304,10 @@
 
         if len(ranges) == 1:
             (start, end) = ranges[0]
-            self.get_single_range(file, file_size, start, end)
+            self.get_single_range(f, file_size, start, end)
         else:
-            self.get_multiple_ranges(file, file_size, ranges)
-        file.close()
+            self.get_multiple_ranges(f, file_size, ranges)
+        f.close()
 
     def translate_path(self, path):
         """Translate a /-separated PATH to the local filename syntax.
@@ -319,52 +369,17 @@
         self.test_case_server = test_case_server
         self._home_dir = test_case_server._home_dir
 
-    def stop_server(self):
-         """Called to clean-up the server.
-
-         Since the server may be (surely is, even) in a blocking listen, we
-         shutdown its socket before closing it.
-         """
-         # Note that is this executed as part of the implicit tear down in the
-         # main thread while the server runs in its own thread. The clean way
-         # to tear down the server is to instruct him to stop accepting
-         # connections and wait for the current connection(s) to end
-         # naturally. To end the connection naturally, the http transports
-         # should close their socket when they do not need to talk to the
-         # server anymore. This happens naturally during the garbage collection
-         # phase of the test transport objetcs (the server clients), so we
-         # don't have to worry about them.  So, for the server, we must tear
-         # down here, from the main thread, when the test have ended.  Note
-         # that since the server is in a blocking operation and since python
-         # use select internally, shutting down the socket is reliable and
-         # relatively clean.
-         try:
-             self.socket.shutdown(socket.SHUT_RDWR)
-         except socket.error, e:
-             # WSAENOTCONN (10057) 'Socket is not connected' is harmless on
-             # windows (occurs before the first connection attempt
-             # vila--20071230)
-
-             # 'Socket is not connected' can also occur on OSX, with a
-             # "regular" ENOTCONN (when something went wrong during test case
-             # setup leading to self.setUp() *not* being called but
-             # self.stop_server() still being called -- vila20081106
-             if not len(e.args) or e.args[0] not in (errno.ENOTCONN, 10057):
-                 raise
-         # Let the server properly close the socket
-         self.server_close()
-
-
-class TestingHTTPServer(SocketServer.TCPServer, TestingHTTPServerMixin):
+
+class TestingHTTPServer(test_server.TestingTCPServer, TestingHTTPServerMixin):
 
     def __init__(self, server_address, request_handler_class,
                  test_case_server):
+        test_server.TestingTCPServer.__init__(self, server_address,
+                                              request_handler_class)
         TestingHTTPServerMixin.__init__(self, test_case_server)
-        SocketServer.TCPServer.__init__(self, server_address,
-                                        request_handler_class)
-
-
-class TestingThreadingHTTPServer(SocketServer.ThreadingTCPServer,
+
+
+class TestingThreadingHTTPServer(test_server.TestingThreadingTCPServer,
                                  TestingHTTPServerMixin):
     """A threading HTTP test server for HTTP 1.1.
 
@@ -372,36 +387,14 @@
     server, we need an independent connection for each of them. We achieve that
     by spawning a new thread for each connection.
     """
-
     def __init__(self, server_address, request_handler_class,
                  test_case_server):
+        test_server.TestingThreadingTCPServer.__init__(self, server_address,
+                                                       request_handler_class)
         TestingHTTPServerMixin.__init__(self, test_case_server)
-        SocketServer.ThreadingTCPServer.__init__(self, server_address,
-                                                 request_handler_class)
-        # Decides how threads will act upon termination of the main
-        # process. This is prophylactic as we should not leave the threads
-        # lying around.
-        self.daemon_threads = True
-
-    def process_request_thread(self, request, client_address):
-        SocketServer.ThreadingTCPServer.process_request_thread(
-            self, request, client_address)
-        # Under some circumstances (as in bug #383920), we need to force the
-        # shutdown as python delays it until gc occur otherwise and the client
-        # may hang.
-        try:
-            # The request process has been completed, the thread is about to
-            # die, let's shutdown the socket if we can.
-            request.shutdown(socket.SHUT_RDWR)
-        except (socket.error, select.error), e:
-            if e[0] in (errno.EBADF, errno.ENOTCONN):
-                # Right, the socket is already down
-                pass
-            else:
-                raise
-
-
-class HttpServer(transport.Server):
+
+
+class HttpServer(test_server.TestingTCPServerInAThread):
     """A test server for http transports.
 
     Subclasses can provide a specific request handler.
@@ -429,82 +422,33 @@
         :param protocol_version: if specified, will override the protocol
             version of the request handler.
         """
-        transport.Server.__init__(self)
-        self.request_handler = request_handler
+        # Depending on the protocol version, we will create the approriate
+        # server
+        if protocol_version is None:
+            # Use the request handler one
+            proto_vers = request_handler.protocol_version
+        else:
+            # Use our own, it will be used to override the request handler
+            # one too.
+            proto_vers = protocol_version
+        # Get the appropriate server class for the required protocol
+        serv_cls = self.http_server_class.get(proto_vers, None)
+        if serv_cls is None:
+            raise httplib.UnknownProtocol(proto_vers)
         self.host = 'localhost'
         self.port = 0
-        self._httpd = None
-        self.protocol_version = protocol_version
+        super(HttpServer, self).__init__((self.host, self.port),
+                                         serv_cls,
+                                         request_handler)
+        self.protocol_version = proto_vers
         # Allows tests to verify number of GET requests issued
         self.GET_request_nb = 0
-
-    def create_httpd(self, serv_cls, rhandler_cls):
-        return serv_cls((self.host, self.port), self.request_handler, self)
-
-    def __repr__(self):
-        return "%s(%s:%s)" % \
-            (self.__class__.__name__, self.host, self.port)
-
-    def _get_httpd(self):
-        if self._httpd is None:
-            rhandler = self.request_handler
-            # Depending on the protocol version, we will create the approriate
-            # server
-            if self.protocol_version is None:
-                # Use the request handler one
-                proto_vers = rhandler.protocol_version
-            else:
-                # Use our own, it will be used to override the request handler
-                # one too.
-                proto_vers = self.protocol_version
-            # Create the appropriate server for the required protocol
-            serv_cls = self.http_server_class.get(proto_vers, None)
-            if serv_cls is None:
-                raise httplib.UnknownProtocol(proto_vers)
-            else:
-                self._httpd = self.create_httpd(serv_cls, rhandler)
-            self.host, self.port = self._httpd.socket.getsockname()
-        return self._httpd
-
-    def _http_start(self):
-        """Server thread main entry point. """
-        self._http_running = False
-        try:
-            try:
-                httpd = self._get_httpd()
-                self._http_base_url = '%s://%s:%s/' % (self._url_protocol,
-                                                       self.host, self.port)
-                self._http_running = True
-            except:
-                # Whatever goes wrong, we save the exception for the main
-                # thread. Note that since we are running in a thread, no signal
-                # can be received, so we don't care about KeyboardInterrupt.
-                self._http_exception = sys.exc_info()
-        finally:
-            # Release the lock or the main thread will block and the whole
-            # process will hang.
-            self._http_starting.release()
-
-        # From now on, exceptions are taken care of by the
-        # SocketServer.BaseServer or the request handler.
-        while self._http_running:
-            try:
-                # Really an HTTP connection but the python framework is generic
-                # and call them requests
-                httpd.handle_request()
-            except socket.timeout:
-                pass
-            except (socket.error, select.error), e:
-                if (e[0] == errno.EBADF
-                    or (sys.platform == 'win32' and e[0] == 10038)):
-                    # Starting with python-2.6, handle_request may raise socket
-                    # or select exceptions when the server is shut down (as we
-                    # do).
-                    # 10038 = WSAENOTSOCK
-                    # http://msdn.microsoft.com/en-us/library/ms740668%28VS.85%29.aspx
-                    pass
-                else:
-                    raise
+        self._http_base_url = None
+        self.logs = []
+
+    def create_server(self):
+        return self.server_class(
+            (self.host, self.port), self.request_handler_class, self)
 
     def _get_remote_url(self, path):
         path_parts = path.split(os.path.sep)
@@ -535,36 +479,15 @@
                 or isinstance(backing_transport_server,
                               test_server.LocalURLServer)):
             raise AssertionError(
-                "HTTPServer currently assumes local transport, got %s" % \
+                "HTTPServer currently assumes local transport, got %s" %
                 backing_transport_server)
         self._home_dir = os.getcwdu()
         self._local_path_parts = self._home_dir.split(os.path.sep)
-        self._http_base_url = None
-
-        # Create the server thread
-        self._http_starting = threading.Lock()
-        self._http_starting.acquire()
-        self._http_thread = threading.Thread(target=self._http_start)
-        self._http_thread.setDaemon(True)
-        self._http_exception = None
-        self._http_thread.start()
-
-        # Wait for the server thread to start (i.e release the lock)
-        self._http_starting.acquire()
-
-        if self._http_exception is not None:
-            # Something went wrong during server start
-            exc_class, exc_value, exc_tb = self._http_exception
-            raise exc_class, exc_value, exc_tb
-        self._http_starting.release()
         self.logs = []
 
-    def stop_server(self):
-        self._httpd.stop_server()
-        self._http_running = False
-        # We don't need to 'self._http_thread.join()' here since the thread is
-        # a daemonic one and will be garbage collected anyway. Joining just
-        # slows us down for no added benefit.
+        super(HttpServer, self).start_server()
+        self._http_base_url = '%s://%s:%s/' % (
+            self._url_protocol, self.host, self.port)
 
     def get_url(self):
         """See bzrlib.transport.Server.get_url."""

=== modified file 'bzrlib/tests/http_utils.py'
--- bzrlib/tests/http_utils.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/http_utils.py	2010-06-23 16:43:16 +0000
@@ -28,13 +28,14 @@
     errors,
     osutils,
     tests,
-    )
-from bzrlib.smart import medium, protocol
+    transport,
+    )
+from bzrlib.smart import (
+    medium,
+    protocol,
+    )
 from bzrlib.tests import http_server
-from bzrlib.transport import (
-    chroot,
-    get_transport,
-    )
+from bzrlib.transport import chroot
 
 
 class HTTPServerWithSmarts(http_server.HttpServer):
@@ -56,11 +57,12 @@
 
     def do_POST(self):
         """Hand the request off to a smart server instance."""
-        backing = get_transport(self.server.test_case_server._home_dir)
+        backing = transport.get_transport(
+            self.server.test_case_server._home_dir)
         chroot_server = chroot.ChrootServer(backing)
         chroot_server.start_server()
         try:
-            t = get_transport(chroot_server.get_url())
+            t = transport.get_transport(chroot_server.get_url())
             self.do_POST_inner(t)
         finally:
             chroot_server.stop_server()
@@ -106,10 +108,23 @@
     one. This will currently fail if the primary transport is not
     backed by regular disk files.
     """
+
+    # These attributes can be overriden or parametrized by daughter clasess if
+    # needed, but must exist so that the create_transport_readonly_server()
+    # method (or any method creating an http(s) server) can propagate it.
+    _protocol_version = None
+    _url_protocol = 'http'
+
     def setUp(self):
         super(TestCaseWithWebserver, self).setUp()
         self.transport_readonly_server = http_server.HttpServer
 
+    def create_transport_readonly_server(self):
+        server = self.transport_readonly_server(
+            protocol_version=self._protocol_version)
+        server._url_protocol = self._url_protocol
+        return server
+
 
 class TestCaseWithTwoWebservers(TestCaseWithWebserver):
     """A support class providing readonly urls on two servers that are http://.
@@ -127,7 +142,10 @@
 
         This is mostly a hook for daughter classes.
         """
-        return self.transport_secondary_server()
+        server = self.transport_secondary_server(
+            protocol_version=self._protocol_version)
+        server._url_protocol = self._url_protocol
+        return server
 
     def get_secondary_server(self):
         """Get the server instance for the secondary transport."""
@@ -136,6 +154,15 @@
             self.start_server(self.__secondary_server)
         return self.__secondary_server
 
+    def get_secondary_url(self, relpath=None):
+        base = self.get_secondary_server().get_url()
+        return self._adjust_url(base, relpath)
+
+    def get_secondary_transport(self, relpath=None):
+        t = transport.get_transport(self.get_secondary_url(relpath))
+        self.assertTrue(t.is_readonly())
+        return t
+
 
 class ProxyServer(http_server.HttpServer):
     """A proxy test server for http transports."""
@@ -215,19 +242,39 @@
    The 'old' server is redirected to the 'new' server.
    """
 
+   def setUp(self):
+       super(TestCaseWithRedirectedWebserver, self).setUp()
+       # The redirections will point to the new server
+       self.new_server = self.get_readonly_server()
+       # The requests to the old server will be redirected to the new server
+       self.old_server = self.get_secondary_server()
+
    def create_transport_secondary_server(self):
        """Create the secondary server redirecting to the primary server"""
        new = self.get_readonly_server()
-       redirecting = HTTPServerRedirecting()
+       redirecting = HTTPServerRedirecting(
+           protocol_version=self._protocol_version)
        redirecting.redirect_to(new.host, new.port)
+       redirecting._url_protocol = self._url_protocol
        return redirecting
 
-   def setUp(self):
-       super(TestCaseWithRedirectedWebserver, self).setUp()
-       # The redirections will point to the new server
-       self.new_server = self.get_readonly_server()
-       # The requests to the old server will be redirected
-       self.old_server = self.get_secondary_server()
+   def get_old_url(self, relpath=None):
+        base = self.old_server.get_url()
+        return self._adjust_url(base, relpath)
+
+   def get_old_transport(self, relpath=None):
+        t = transport.get_transport(self.get_old_url(relpath))
+        self.assertTrue(t.is_readonly())
+        return t
+
+   def get_new_url(self, relpath=None):
+        base = self.new_server.get_url()
+        return self._adjust_url(base, relpath)
+
+   def get_new_transport(self, relpath=None):
+        t = transport.get_transport(self.get_new_url(relpath))
+        self.assertTrue(t.is_readonly())
+        return t
 
 
 class AuthRequestHandler(http_server.TestingHTTPRequestHandler):

=== modified file 'bzrlib/tests/https_server.py'
--- bzrlib/tests/https_server.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/https_server.py	2010-07-01 06:40:14 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 from bzrlib.tests import (
     http_server,
     ssl_certs,
+    test_server,
     )
 
 
@@ -30,17 +31,25 @@
         self.key_file = key_file
         self.cert_file = cert_file
 
-    def get_request (self):
-        """Get the request and client address from the socket.
-
-        This is called in response to a connection issued to the server, we
-        wrap the socket with SSL.
+    def _get_ssl_request (self, sock, addr):
+        """Wrap the socket with SSL"""
+        ssl_sock = ssl.wrap_socket(sock, server_side=True,
+                                   keyfile=self.key_file,
+                                   certfile=self.cert_file,
+                                   do_handshake_on_connect=False)
+        return ssl_sock, addr
+
+    def verify_request(self, request, client_address):
+        """Verify the request.
+
+        Return True if we should proceed with this request, False if we should
+        not even touch a single byte in the socket !
         """
-        sock, addr = self.socket.accept()
-        sslconn = ssl.wrap_socket(sock, server_side=True,
-                                  keyfile=self.key_file,
-                                  certfile=self.cert_file)
-        return sslconn, addr
+        serving = test_server.TestingTCPServerMixin.verify_request(
+            self, request, client_address)
+        if serving:
+            request.do_handshake()
+        return serving
 
 
 class TestingHTTPSServer(TestingHTTPSServerMixin,
@@ -52,6 +61,10 @@
         http_server.TestingHTTPServer.__init__(
             self, server_address, request_handler_class, test_case_server)
 
+    def get_request(self):
+        sock, addr = http_server.TestingHTTPServer.get_request(self)
+        return self._get_ssl_request(sock, addr)
+
 
 class TestingThreadingHTTPSServer(TestingHTTPSServerMixin,
                                   http_server.TestingThreadingHTTPServer):
@@ -62,6 +75,10 @@
         http_server.TestingThreadingHTTPServer.__init__(
             self, server_address, request_handler_class, test_case_server)
 
+    def get_request(self):
+        sock, addr = http_server.TestingThreadingHTTPServer.get_request(self)
+        return self._get_ssl_request(sock, addr)
+
 
 class HTTPSServer(http_server.HttpServer):
 
@@ -73,7 +90,7 @@
                          }
 
     # Provides usable defaults since an https server requires both a
-    # private key and certificate to work.
+    # private key and a certificate to work.
     def __init__(self, request_handler=http_server.TestingHTTPRequestHandler,
                  protocol_version=None,
                  key_file=ssl_certs.build_path('server_without_pass.key'),
@@ -84,9 +101,10 @@
         self.cert_file = cert_file
         self.temp_files = []
 
-    def create_httpd(self, serv_cls, rhandler_cls):
-        return serv_cls((self.host, self.port), self.request_handler,
-                        self, self.key_file, self.cert_file)
+    def create_server(self):
+        return self.server_class(
+            (self.host, self.port), self.request_handler_class, self,
+            self.key_file, self.cert_file)
 
 
 class HTTPSServer_urllib(HTTPSServer):

=== modified file 'bzrlib/tests/per_branch/__init__.py'
--- bzrlib/tests/per_branch/__init__.py	2010-04-01 12:53:53 +0000
+++ bzrlib/tests/per_branch/__init__.py	2010-08-21 16:06:24 +0000
@@ -33,7 +33,7 @@
                            )
 from bzrlib.remote import RemoteBranchFormat, RemoteBzrDirFormat
 from bzrlib.tests import test_server
-from bzrlib.tests.per_bzrdir.test_bzrdir import TestCaseWithBzrDir
+from bzrlib.tests.per_controldir.test_controldir import TestCaseWithControlDir
 from bzrlib.transport import memory
 
 
@@ -60,7 +60,7 @@
     return result
 
 
-class TestCaseWithBranch(TestCaseWithBzrDir):
+class TestCaseWithBranch(TestCaseWithControlDir):
     """This helper will be parameterised in each per_branch test."""
 
     def setUp(self):
@@ -74,7 +74,7 @@
 
     def make_branch(self, relpath, format=None):
         if format is not None:
-            return TestCaseWithBzrDir.make_branch(self, relpath, format)
+            return TestCaseWithControlDir.make_branch(self, relpath, format)
         repo = self.make_repository(relpath)
         # fixme RBC 20060210 this isnt necessarily a fixable thing,
         # Skipped is the wrong exception to raise.
@@ -132,7 +132,7 @@
     # Generate a list of branch formats and their associated bzrdir formats to
     # use.
     combinations = [(format, format._matchingbzrdir) for format in
-         BranchFormat._formats.values() + _legacy_formats]
+         BranchFormat.get_formats() + _legacy_formats]
     scenarios = make_scenarios(
         # None here will cause the default vfs transport server to be used.
         None,
@@ -165,6 +165,7 @@
         'branch',
         'break_lock',
         'check',
+        'config',
         'create_checkout',
         'create_clone',
         'commit',

=== modified file 'bzrlib/tests/per_branch/test_bound_sftp.py'
--- bzrlib/tests/per_branch/test_bound_sftp.py	2010-02-11 09:27:55 +0000
+++ bzrlib/tests/per_branch/test_bound_sftp.py	2010-06-20 11:18:38 +0000
@@ -144,13 +144,13 @@
                 wt_child2.commit, 'child2', rev_id='r@d-2')
 
     def test_unbinding(self):
-        from bzrlib.transport import get_transport
+        from bzrlib import transport
         b_base, wt_child = self.create_branches()
 
         # TestCaseWithSFTPServer only allows you to connect one time
         # to the SFTP server. So we have to create a connection and
         # keep it around, so that it can be reused
-        __unused_t = get_transport(self.get_url('.'))
+        __unused_t = transport.get_transport(self.get_url('.'))
 
         wt_base = b_base.bzrdir.open_workingtree()
         open('base/a', 'wb').write('new base contents\n')

=== modified file 'bzrlib/tests/per_branch/test_branch.py'
--- bzrlib/tests/per_branch/test_branch.py	2010-04-21 04:12:25 +0000
+++ bzrlib/tests/per_branch/test_branch.py	2010-08-29 14:32:45 +0000
@@ -19,6 +19,7 @@
 from bzrlib import (
     branch as _mod_branch,
     bzrdir,
+    config,
     delta as _mod_delta,
     errors,
     gpg,
@@ -617,13 +618,9 @@
         self.assertEqual(None, self.get_branch().get_push_location())
 
     def test_get_push_location_exact(self):
-        from bzrlib.config import (locations_config_filename,
-                                   ensure_config_dir_exists)
-        ensure_config_dir_exists()
-        fn = locations_config_filename()
-        open(fn, 'wt').write(("[%s]\n"
-                                  "push_location=foo\n" %
-                                  self.get_branch().base[:-1]))
+        b = self.get_branch()
+        config.LocationConfig.from_string(
+            '[%s]\npush_location=foo\n' % (b.base,), b.base, save=True)
         self.assertEqual("foo", self.get_branch().get_push_location())
 
     def test_set_push_location(self):
@@ -668,7 +665,8 @@
         this_branch = self.make_branch('this')
         other_branch = self.make_branch('other')
         try:
-            this_branch._format.set_reference(this_branch.bzrdir, other_branch)
+            this_branch._format.set_reference(this_branch.bzrdir, None,
+                other_branch)
         except NotImplementedError:
             # that's ok
             pass

=== modified file 'bzrlib/tests/per_branch/test_break_lock.py'
--- bzrlib/tests/per_branch/test_break_lock.py	2010-02-10 17:52:08 +0000
+++ bzrlib/tests/per_branch/test_break_lock.py	2010-05-06 23:41:35 +0000
@@ -41,7 +41,7 @@
     def test_unlocked_repo_locked(self):
         # break lock on the branch should try on the repository even
         # if the branch isn't locked
-        token = self.branch.repository.lock_write()
+        token = self.branch.repository.lock_write().repository_token
         if token is None:
             self.branch.repository.unlock()
             raise tests.TestNotApplicable(

=== added file 'bzrlib/tests/per_branch/test_config.py'
--- bzrlib/tests/per_branch/test_config.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_branch/test_config.py	2010-05-13 16:31:08 +0000
@@ -0,0 +1,38 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for branch.get_config behaviour."""
+
+from bzrlib import (
+    branch,
+    errors,
+    remote,
+    tests,
+    )
+from bzrlib.tests import per_branch
+
+
+class TestGetConfig(per_branch.TestCaseWithBranch):
+
+    def test_set_user_option_with_dict(self):
+        b = self.make_branch('b')
+        config = b.get_config()
+        value_dict = {
+            'ascii': 'abcd', u'unicode \N{WATCH}': u'foo \N{INTERROBANG}'}
+        config.set_user_option('name', value_dict.copy())
+        self.assertEqual(value_dict, config.get_user_option('name'))
+
+

=== modified file 'bzrlib/tests/per_branch/test_locking.py'
--- bzrlib/tests/per_branch/test_locking.py	2010-02-10 17:52:08 +0000
+++ bzrlib/tests/per_branch/test_locking.py	2010-06-11 07:59:43 +0000
@@ -25,6 +25,7 @@
     lock_helpers,
     per_branch,
     )
+from bzrlib.tests.matchers import *
 
 
 class TestBranchLocking(per_branch.TestCaseWithBranch):
@@ -291,7 +292,7 @@
 
     def test_lock_write_returns_None_refuses_token(self):
         branch = self.make_branch('b')
-        token = branch.lock_write()
+        token = branch.lock_write().branch_token
         try:
             if token is not None:
                 # This test does not apply, because this lockable supports
@@ -304,7 +305,7 @@
 
     def test_reentering_lock_write_raises_on_token_mismatch(self):
         branch = self.make_branch('b')
-        token = branch.lock_write()
+        token = branch.lock_write().branch_token
         try:
             if token is None:
                 # This test does not apply, because this lockable refuses
@@ -321,7 +322,7 @@
 
     def test_lock_write_with_nonmatching_token(self):
         branch = self.make_branch('b')
-        token = branch.lock_write()
+        token = branch.lock_write().branch_token
         try:
             if token is None:
                 # This test does not apply, because this branch refuses
@@ -344,7 +345,7 @@
         """Test that a branch can be locked with a token, if it is already
         locked by that token."""
         branch = self.make_branch('b')
-        token = branch.lock_write()
+        token = branch.lock_write().branch_token
         try:
             if token is None:
                 # This test does not apply, because this branch refuses tokens.
@@ -368,7 +369,7 @@
         # If lock_write did not physically acquire the lock (because it was
         # passed some tokens), then unlock should not physically release it.
         branch = self.make_branch('b')
-        token = branch.lock_write()
+        token = branch.lock_write().branch_token
         try:
             if token is None:
                 # This test does not apply, because this lockable refuses
@@ -390,19 +391,17 @@
         # to lock with a token that is no longer valid (because the original
         # lock was released).
         branch = self.make_branch('b')
-        token = branch.lock_write()
+        token = branch.lock_write().branch_token
         branch.unlock()
         if token is None:
             # This test does not apply, because this lockable refuses
             # tokens.
             return
-
-        self.assertRaises(errors.TokenMismatch,
-                          branch.lock_write, token=token)
+        self.assertRaises(errors.TokenMismatch, branch.lock_write, token=token)
 
     def test_lock_write_reenter_with_token(self):
         branch = self.make_branch('b')
-        token = branch.lock_write()
+        token = branch.lock_write().branch_token
         try:
             if token is None:
                 # This test does not apply, because this lockable refuses
@@ -424,7 +423,7 @@
         branch = self.make_branch('b')
         # Lock the branch, then use leave_lock_in_place so that when we
         # unlock the branch the lock is still held on disk.
-        token = branch.lock_write()
+        token = branch.lock_write().branch_token
         try:
             if token is None:
                 # This test does not apply, because this repository refuses lock
@@ -445,7 +444,7 @@
     def test_dont_leave_lock_in_place(self):
         branch = self.make_branch('b')
         # Create a lock on disk.
-        token = branch.lock_write()
+        token = branch.lock_write().branch_token
         try:
             if token is None:
                 # This test does not apply, because this branch refuses lock
@@ -497,6 +496,10 @@
         branch.lock_read()
         branch.unlock()
 
+    def test_lock_read_returns_unlockable(self):
+        branch = self.make_branch('b')
+        self.assertThat(branch.lock_read, ReturnsUnlockable(branch))
+
     def test_lock_write_locks_repo_too(self):
         if isinstance(self.branch_format, _mod_branch.BzrBranchFormat4):
             # Branch format 4 is combined with the repository, so this test
@@ -523,6 +526,16 @@
         finally:
             branch.unlock()
 
+    def test_lock_write_returns_unlockable(self):
+        branch = self.make_branch('b')
+        self.assertThat(branch.lock_write, ReturnsUnlockable(branch))
+
+    def test_lock_write_raises_in_lock_read(self):
+        branch = self.make_branch('b')
+        branch.lock_read()
+        self.addCleanup(branch.unlock)
+        err = self.assertRaises(errors.ReadOnlyError, branch.lock_write)
+
     def test_lock_and_unlock_leaves_repo_unlocked(self):
         branch = self.make_branch('b')
         branch.lock_write()

=== modified file 'bzrlib/tests/per_branch/test_permissions.py'
--- bzrlib/tests/per_branch/test_permissions.py	2009-09-07 08:46:00 +0000
+++ bzrlib/tests/per_branch/test_permissions.py	2010-06-20 11:18:38 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2008 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -38,7 +38,6 @@
 from bzrlib.remote import RemoteBranchFormat
 from bzrlib.tests.test_permissions import chmod_r, check_mode_r
 from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
-from bzrlib.transport import get_transport
 from bzrlib.workingtree import WorkingTree
 
 

=== modified file 'bzrlib/tests/per_branch/test_pull.py'
--- bzrlib/tests/per_branch/test_pull.py	2010-02-10 17:52:08 +0000
+++ bzrlib/tests/per_branch/test_pull.py	2010-06-14 06:37:11 +0000
@@ -98,17 +98,6 @@
                           master_tree.branch.pull, other.branch, local = True)
         self.assertEqual([rev1], master_tree.branch.revision_history())
 
-    def test_pull_raises_specific_error_on_master_connection_error(self):
-        master_tree = self.make_branch_and_tree('master')
-        checkout = master_tree.branch.create_checkout('checkout')
-        other = master_tree.branch.bzrdir.sprout('other').open_workingtree()
-        # move the branch out of the way on disk to cause a connection
-        # error.
-        os.rename('master', 'master_gone')
-        # try to pull, which should raise a BoundBranchConnectionFailure.
-        self.assertRaises(errors.BoundBranchConnectionFailure,
-                checkout.branch.pull, other.branch)
-
     def test_pull_returns_result(self):
         parent = self.make_branch_and_tree('parent')
         parent.commit('1st post', rev_id='P1')

=== modified file 'bzrlib/tests/per_branch/test_push.py'
--- bzrlib/tests/per_branch/test_push.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/per_branch/test_push.py	2010-09-09 00:32:58 +0000
@@ -30,6 +30,7 @@
     push,
     repository,
     revision,
+    symbol_versioning,
     tests,
     transport,
     )
@@ -61,7 +62,10 @@
         self.assertEqual(result.old_revid, 'M1')
         self.assertEqual(result.new_revid, 'P2')
         # and it can be treated as an integer for compatibility
-        self.assertEqual(int(result), 0)
+        self.assertEqual(self.applyDeprecated(
+            symbol_versioning.deprecated_in((2, 3, 0)),
+            result.__int__),
+            0)
 
     def test_push_merged_indirect(self):
         # it should be possible to do a push from one branch into another
@@ -231,59 +235,6 @@
         trunk.push(remote_branch)
         check.check_dwim(remote_branch.base, False, True, True)
 
-    def test_no_get_parent_map_after_insert_stream(self):
-        # Effort test for bug 331823
-        self.setup_smart_server_with_call_log()
-        # Make a local branch with four revisions.  Four revisions because:
-        # one to push, one there for _walk_to_common_revisions to find, one we
-        # don't want to access, one for luck :)
-        if isinstance(self.branch_format, branch.BranchReferenceFormat):
-            # This test could in principle apply to BranchReferenceFormat, but
-            # make_branch_builder doesn't support it.
-            raise tests.TestSkipped(
-                "BranchBuilder can't make reference branches.")
-        try:
-            builder = self.make_branch_builder('local')
-        except (errors.TransportNotPossible, errors.UninitializableFormat):
-            raise tests.TestNotApplicable('format not directly constructable')
-        builder.start_series()
-        builder.build_snapshot('first', None, [
-            ('add', ('', 'root-id', 'directory', ''))])
-        builder.build_snapshot('second', ['first'], [])
-        builder.build_snapshot('third', ['second'], [])
-        builder.build_snapshot('fourth', ['third'], [])
-        builder.finish_series()
-        local = builder.get_branch()
-        local = branch.Branch.open(self.get_vfs_only_url('local'))
-        # Initial push of three revisions
-        remote_bzrdir = local.bzrdir.sprout(
-            self.get_url('remote'), revision_id='third')
-        remote = remote_bzrdir.open_branch()
-        # Push fourth revision
-        self.reset_smart_call_log()
-        self.disableOptimisticGetParentMap()
-        self.assertFalse(local.is_locked())
-        local.push(remote)
-        hpss_call_names = [item.call.method for item in self.hpss_calls]
-        self.assertTrue('Repository.insert_stream_1.19' in hpss_call_names)
-        insert_stream_idx = hpss_call_names.index(
-            'Repository.insert_stream_1.19')
-        calls_after_insert_stream = hpss_call_names[insert_stream_idx:]
-        # After inserting the stream the client has no reason to query the
-        # remote graph any further.
-        self.assertEqual(
-            ['Repository.insert_stream_1.19', 'Repository.insert_stream_1.19',
-             'get', 'Branch.set_last_revision_info', 'Branch.unlock'],
-            calls_after_insert_stream)
-
-    def disableOptimisticGetParentMap(self):
-        # Tweak some class variables to stop remote get_parent_map calls asking
-        # for or receiving more data than the caller asked for.
-        self.overrideAttr(repository.InterRepository,
-                          '_walk_to_common_revisions_batch_size', 1)
-        self.overrideAttr(_mod_smart_repo.SmartServerRepositoryGetParentMap,
-                          'no_extra_results', True)
-
 
 class TestPushHook(per_branch.TestCaseWithBranch):
 
@@ -382,8 +333,9 @@
 
     def setUp(self):
         # Skip some scenarios that don't apply to these tests.
-        if (self.transport_server is not None and
-            issubclass(self.transport_server, server.SmartTCPServer)):
+        if (self.transport_server is not None
+            and issubclass(self.transport_server,
+                           test_server.SmartTCPServer_for_testing)):
             raise tests.TestNotApplicable(
                 'Does not apply when remote backing branch is also '
                 'a smart branch')

=== modified file 'bzrlib/tests/per_branch/test_stacking.py'
--- bzrlib/tests/per_branch/test_stacking.py	2009-08-20 04:09:58 +0000
+++ bzrlib/tests/per_branch/test_stacking.py	2010-07-01 03:41:20 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008 Canonical Ltd
+# Copyright (C) 2008, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,10 +23,8 @@
     errors,
     )
 from bzrlib.revision import NULL_REVISION
-from bzrlib.smart import server
-from bzrlib.tests import TestNotApplicable, KnownFailure, transport_util
+from bzrlib.tests import TestNotApplicable, transport_util
 from bzrlib.tests.per_branch import TestCaseWithBranch
-from bzrlib.transport import get_transport
 
 
 unstackable_format_errors = (
@@ -207,6 +205,40 @@
         self.assertRaises(errors.NotStacked,
             new_branch.get_stacked_on_url)
 
+    def test_unstack_already_locked(self):
+        """Removing the stacked-on branch with an already write-locked branch
+        works.
+
+        This was bug 551525.
+        """
+        try:
+            stacked_bzrdir = self.make_stacked_bzrdir()
+        except unstackable_format_errors, e:
+            raise TestNotApplicable(e)
+        stacked_branch = stacked_bzrdir.open_branch()
+        stacked_branch.lock_write()
+        stacked_branch.set_stacked_on_url(None)
+        stacked_branch.unlock()
+
+    def test_unstack_already_multiple_locked(self):
+        """Unstacking a branch preserves the lock count (even though it
+        replaces the br.repository object).
+
+        This is a more extreme variation of test_unstack_already_locked.
+        """
+        try:
+            stacked_bzrdir = self.make_stacked_bzrdir()
+        except unstackable_format_errors, e:
+            raise TestNotApplicable(e)
+        stacked_branch = stacked_bzrdir.open_branch()
+        stacked_branch.lock_write()
+        stacked_branch.lock_write()
+        stacked_branch.lock_write()
+        stacked_branch.set_stacked_on_url(None)
+        stacked_branch.unlock()
+        stacked_branch.unlock()
+        stacked_branch.unlock()
+
     def make_stacked_bzrdir(self, in_directory=None):
         """Create a stacked branch and return its bzrdir.
 
@@ -222,7 +254,8 @@
         tree = self.make_branch_and_tree(prefix + 'stacked-on')
         tree.commit('Added foo')
         stacked_bzrdir = tree.branch.bzrdir.sprout(
-            prefix + 'stacked', tree.branch.last_revision(), stacked=True)
+            self.get_url(prefix + 'stacked'), tree.branch.last_revision(),
+            stacked=True)
         return stacked_bzrdir
 
     def test_clone_from_stacked_branch_preserve_stacking(self):
@@ -250,7 +283,8 @@
         except unstackable_format_errors, e:
             raise TestNotApplicable(e)
         stacked_bzrdir.open_branch().set_stacked_on_url('../stacked-on')
-        cloned_bzrdir = stacked_bzrdir.clone('cloned', preserve_stacking=True)
+        cloned_bzrdir = stacked_bzrdir.clone(
+            self.get_url('cloned'), preserve_stacking=True)
         self.assertEqual(
             '../dir/stacked-on',
             cloned_bzrdir.open_branch().get_stacked_on_url())

=== added directory 'bzrlib/tests/per_bzrdir'
=== added file 'bzrlib/tests/per_bzrdir/__init__.py'
--- bzrlib/tests/per_bzrdir/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_bzrdir/__init__.py	2010-09-25 20:08:01 +0000
@@ -0,0 +1,91 @@
+# Copyright (C) 2010 Canonical Ltd
+# Authors: Robert Collins <robert.collins@canonical.com>
+#          Jelmer Vernooij <jelmer.vernooij@canonical.com>
+# -*- coding: utf-8 -*-
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+
+"""BzrDir implementation tests for bzr.
+
+These test the conformance of all the bzrdir variations to the expected API.
+Specific tests for individual formats are in the tests/test_bzrdir.py file
+rather than in tests/per_branch/*.py. Generic control directory tests not
+specific to BzrDir are in tests/per_controldir/*.py.
+"""
+
+from bzrlib.bzrdir import BzrDirFormat
+from bzrlib.controldir import ControlDirFormat
+from bzrlib.tests import (
+    default_transport,
+    multiply_tests,
+    test_server,
+    TestCaseWithTransport,
+    )
+from bzrlib.tests.per_controldir import make_scenarios
+from bzrlib.transport import memory
+
+
+class TestCaseWithBzrDir(TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestCaseWithBzrDir, self).setUp()
+        self.bzrdir = None
+
+    def get_bzrdir(self):
+        if self.bzrdir is None:
+            self.bzrdir = self.make_bzrdir(None)
+        return self.bzrdir
+
+    def make_bzrdir(self, relpath, format=None):
+        if format is None:
+            format = self.bzrdir_format
+        return super(TestCaseWithBzrDir, self).make_bzrdir(
+            relpath, format=format)
+
+
+def load_tests(standard_tests, module, loader):
+    test_per_bzrdir = [
+        'bzrlib.tests.per_bzrdir.test_bzrdir',
+        ]
+    submod_tests = loader.loadTestsFromModuleNames(test_per_bzrdir)
+    formats = [format for format in ControlDirFormat.known_formats()
+               if isinstance(format, BzrDirFormat)]
+    scenarios = make_scenarios(
+        default_transport,
+        None,
+        # None here will cause a readonly decorator to be created
+        # by the TestCaseWithTransport.get_readonly_transport method.
+        None,
+        formats)
+    # This will always add scenarios using the smart server.
+    from bzrlib.remote import RemoteBzrDirFormat
+    # test the remote server behaviour when backed with a MemoryTransport
+    # Once for the current version
+    scenarios.extend(make_scenarios(
+        memory.MemoryServer,
+        test_server.SmartTCPServer_for_testing,
+        test_server.ReadonlySmartTCPServer_for_testing,
+        [(RemoteBzrDirFormat())],
+        name_suffix='-default'))
+    # And once with < 1.6 - the 'v2' protocol.
+    scenarios.extend(make_scenarios(
+        memory.MemoryServer,
+        test_server.SmartTCPServer_for_testing_v2_only,
+        test_server.ReadonlySmartTCPServer_for_testing_v2_only,
+        [(RemoteBzrDirFormat())],
+        name_suffix='-v2'))
+    # add the tests for the sub modules
+    return multiply_tests(submod_tests, scenarios, standard_tests)

=== added file 'bzrlib/tests/per_bzrdir/test_bzrdir.py'
--- bzrlib/tests/per_bzrdir/test_bzrdir.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_bzrdir/test_bzrdir.py	2010-10-05 20:39:07 +0000
@@ -0,0 +1,492 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for bzrdir implementations - tests a bzrdir format."""
+
+import errno
+from stat import S_ISDIR
+
+import bzrlib.branch
+from bzrlib import (
+    errors,
+    revision as _mod_revision,
+    transport,
+    )
+from bzrlib.tests import (
+    TestSkipped,
+    )
+from bzrlib.tests.per_bzrdir import TestCaseWithBzrDir
+from bzrlib.transport.local import (
+    LocalTransport,
+    )
+
+
+class TestBzrDir(TestCaseWithBzrDir):
+
+    # Many of these tests test for disk equality rather than checking
+    # for semantic equivalence. This works well for some tests but
+    # is not good at handling changes in representation or the addition
+    # or removal of control data. It would be nice to for instance:
+    # sprout a new branch, check that the nickname has been reset by hand
+    # and then set the nickname to match the source branch, at which point
+    # a semantic equivalence should pass
+
+    def assertDirectoriesEqual(self, source, target, ignore_list=[]):
+        """Assert that the content of source and target are identical.
+
+        paths in ignore list will be completely ignored.
+
+        We ignore paths that represent data which is allowed to change during
+        a clone or sprout: for instance, inventory.knit contains gzip fragements
+        which have timestamps in them, and as we have read the inventory from
+        the source knit, the already-read data is recompressed rather than
+        reading it again, which leads to changed timestamps. This is ok though,
+        because the inventory.kndx file is not ignored, and the integrity of
+        knit joins is tested by test_knit and test_versionedfile.
+
+        :seealso: Additionally, assertRepositoryHasSameItems provides value
+            rather than representation checking of repositories for
+            equivalence.
+        """
+        files = []
+        directories = ['.']
+        while directories:
+            dir = directories.pop()
+            for path in set(source.list_dir(dir) + target.list_dir(dir)):
+                path = dir + '/' + path
+                if path in ignore_list:
+                    continue
+                try:
+                    stat = source.stat(path)
+                except errors.NoSuchFile:
+                    self.fail('%s not in source' % path)
+                if S_ISDIR(stat.st_mode):
+                    self.assertTrue(S_ISDIR(target.stat(path).st_mode))
+                    directories.append(path)
+                else:
+                    self.assertEqualDiff(source.get(path).read(),
+                                         target.get(path).read(),
+                                         "text for file %r differs:\n" % path)
+
+    def assertRepositoryHasSameItems(self, left_repo, right_repo):
+        """require left_repo and right_repo to contain the same data."""
+        # XXX: TODO: Doesn't work yet, because we need to be able to compare
+        # local repositories to remote ones...  but this is an as-yet unsolved
+        # aspect of format management and the Remote protocols...
+        # self.assertEqual(left_repo._format.__class__,
+        #     right_repo._format.__class__)
+        left_repo.lock_read()
+        try:
+            right_repo.lock_read()
+            try:
+                # revs
+                all_revs = left_repo.all_revision_ids()
+                self.assertEqual(left_repo.all_revision_ids(),
+                    right_repo.all_revision_ids())
+                for rev_id in left_repo.all_revision_ids():
+                    self.assertEqual(left_repo.get_revision(rev_id),
+                        right_repo.get_revision(rev_id))
+                # Assert the revision trees (and thus the inventories) are equal
+                sort_key = lambda rev_tree: rev_tree.get_revision_id()
+                rev_trees_a = sorted(
+                    left_repo.revision_trees(all_revs), key=sort_key)
+                rev_trees_b = sorted(
+                    right_repo.revision_trees(all_revs), key=sort_key)
+                for tree_a, tree_b in zip(rev_trees_a, rev_trees_b):
+                    self.assertEqual([], list(tree_a.iter_changes(tree_b)))
+                # texts
+                text_index = left_repo._generate_text_key_index()
+                self.assertEqual(text_index,
+                    right_repo._generate_text_key_index())
+                desired_files = []
+                for file_id, revision_id in text_index.iterkeys():
+                    desired_files.append(
+                        (file_id, revision_id, (file_id, revision_id)))
+                left_texts = list(left_repo.iter_files_bytes(desired_files))
+                right_texts = list(right_repo.iter_files_bytes(desired_files))
+                left_texts.sort()
+                right_texts.sort()
+                self.assertEqual(left_texts, right_texts)
+                # signatures
+                for rev_id in all_revs:
+                    try:
+                        left_text = left_repo.get_signature_text(rev_id)
+                    except errors.NoSuchRevision:
+                        continue
+                    right_text = right_repo.get_signature_text(rev_id)
+                    self.assertEqual(left_text, right_text)
+            finally:
+                right_repo.unlock()
+        finally:
+            left_repo.unlock()
+
+    def sproutOrSkip(self, from_bzrdir, to_url, revision_id=None,
+                     force_new_repo=False, accelerator_tree=None,
+                     create_tree_if_local=True):
+        """Sprout from_bzrdir into to_url, or raise TestSkipped.
+
+        A simple wrapper for from_bzrdir.sprout that translates NotLocalUrl into
+        TestSkipped.  Returns the newly sprouted bzrdir.
+        """
+        to_transport = transport.get_transport(to_url)
+        if not isinstance(to_transport, LocalTransport):
+            raise TestSkipped('Cannot sprout to remote bzrdirs.')
+        target = from_bzrdir.sprout(to_url, revision_id=revision_id,
+                                    force_new_repo=force_new_repo,
+                                    possible_transports=[to_transport],
+                                    accelerator_tree=accelerator_tree,
+                                    create_tree_if_local=create_tree_if_local)
+        return target
+
+    def skipIfNoWorkingTree(self, a_bzrdir):
+        """Raises TestSkipped if a_bzrdir doesn't have a working tree.
+
+        If the bzrdir does have a workingtree, this is a no-op.
+        """
+        try:
+            a_bzrdir.open_workingtree()
+        except (errors.NotLocalUrl, errors.NoWorkingTree):
+            raise TestSkipped("bzrdir on transport %r has no working tree"
+                              % a_bzrdir.transport)
+
+    def createWorkingTreeOrSkip(self, a_bzrdir):
+        """Create a working tree on a_bzrdir, or raise TestSkipped.
+
+        A simple wrapper for create_workingtree that translates NotLocalUrl into
+        TestSkipped.  Returns the newly created working tree.
+        """
+        try:
+            # This passes in many named options to make sure they're
+            # understood by subclasses: see
+            # <https://bugs.launchpad.net/bzr/+bug/524627>.
+            return a_bzrdir.create_workingtree(
+                revision_id=None,
+                from_branch=None,
+                accelerator_tree=None,
+                hardlink=False)
+        except errors.NotLocalUrl:
+            raise TestSkipped("cannot make working tree with transport %r"
+                              % a_bzrdir.transport)
+
+    def test_clone_bzrdir_repository_under_shared_force_new_repo(self):
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['commit_tree/foo'])
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        dir = self.make_bzrdir('source')
+        repo = dir.create_repository()
+        repo.fetch(tree.branch.repository)
+        self.assertTrue(repo.has_revision('1'))
+        try:
+            self.make_repository('target', shared=True)
+        except errors.IncompatibleFormat:
+            return
+        target = dir.clone(self.get_url('target/child'), force_new_repo=True)
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    ['./.bzr/repository',
+                                     ])
+        self.assertRepositoryHasSameItems(tree.branch.repository, repo)
+
+    def test_clone_bzrdir_branch_and_repo(self):
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['commit_tree/foo'])
+        tree.add('foo')
+        tree.commit('revision 1')
+        source = self.make_branch('source')
+        tree.branch.repository.copy_content_into(source.repository)
+        tree.branch.copy_content_into(source)
+        dir = source.bzrdir
+        target = dir.clone(self.get_url('target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    [
+                                     './.bzr/basis-inventory-cache',
+                                     './.bzr/checkout/stat-cache',
+                                     './.bzr/merge-hashes',
+                                     './.bzr/repository',
+                                     './.bzr/stat-cache',
+                                    ])
+        self.assertRepositoryHasSameItems(
+            tree.branch.repository, target.open_repository())
+
+    def test_clone_on_transport(self):
+        a_dir = self.make_bzrdir('source')
+        target_transport = a_dir.root_transport.clone('..').clone('target')
+        target = a_dir.clone_on_transport(target_transport)
+        self.assertNotEqual(a_dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(a_dir.root_transport, target.root_transport,
+                                    ['./.bzr/merge-hashes'])
+
+    def test_clone_bzrdir_empty(self):
+        dir = self.make_bzrdir('source')
+        target = dir.clone(self.get_url('target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    ['./.bzr/merge-hashes'])
+
+    def test_clone_bzrdir_empty_force_new_ignored(self):
+        # the force_new_repo parameter should have no effect on an empty
+        # bzrdir's clone logic
+        dir = self.make_bzrdir('source')
+        target = dir.clone(self.get_url('target'), force_new_repo=True)
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    ['./.bzr/merge-hashes'])
+
+    def test_clone_bzrdir_repository(self):
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        dir = self.make_bzrdir('source')
+        repo = dir.create_repository()
+        repo.fetch(tree.branch.repository)
+        self.assertTrue(repo.has_revision('1'))
+        target = dir.clone(self.get_url('target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    [
+                                     './.bzr/merge-hashes',
+                                     './.bzr/repository',
+                                     ])
+        self.assertRepositoryHasSameItems(tree.branch.repository,
+            target.open_repository())
+
+    def test_clone_bzrdir_tree_branch_repo(self):
+        tree = self.make_branch_and_tree('source')
+        self.build_tree(['source/foo'])
+        tree.add('foo')
+        tree.commit('revision 1')
+        dir = tree.bzrdir
+        target = dir.clone(self.get_url('target'))
+        self.skipIfNoWorkingTree(target)
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    ['./.bzr/stat-cache',
+                                     './.bzr/checkout/dirstate',
+                                     './.bzr/checkout/stat-cache',
+                                     './.bzr/checkout/merge-hashes',
+                                     './.bzr/merge-hashes',
+                                     './.bzr/repository',
+                                     ])
+        self.assertRepositoryHasSameItems(tree.branch.repository,
+            target.open_repository())
+        target.open_workingtree().revert()
+
+    def test_revert_inventory(self):
+        tree = self.make_branch_and_tree('source')
+        self.build_tree(['source/foo'])
+        tree.add('foo')
+        tree.commit('revision 1')
+        dir = tree.bzrdir
+        target = dir.clone(self.get_url('target'))
+        self.skipIfNoWorkingTree(target)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    ['./.bzr/stat-cache',
+                                     './.bzr/checkout/dirstate',
+                                     './.bzr/checkout/stat-cache',
+                                     './.bzr/checkout/merge-hashes',
+                                     './.bzr/merge-hashes',
+                                     './.bzr/repository',
+                                     ])
+        self.assertRepositoryHasSameItems(tree.branch.repository,
+            target.open_repository())
+
+        target.open_workingtree().revert()
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    ['./.bzr/stat-cache',
+                                     './.bzr/checkout/dirstate',
+                                     './.bzr/checkout/stat-cache',
+                                     './.bzr/checkout/merge-hashes',
+                                     './.bzr/merge-hashes',
+                                     './.bzr/repository',
+                                     ])
+        self.assertRepositoryHasSameItems(tree.branch.repository,
+            target.open_repository())
+
+    def test_clone_bzrdir_tree_branch_reference(self):
+        # a tree with a branch reference (aka a checkout)
+        # should stay a checkout on clone.
+        referenced_branch = self.make_branch('referencced')
+        dir = self.make_bzrdir('source')
+        try:
+            reference = bzrlib.branch.BranchReferenceFormat().initialize(dir,
+                target_branch=referenced_branch)
+        except errors.IncompatibleFormat:
+            # this is ok too, not all formats have to support references.
+            return
+        self.createWorkingTreeOrSkip(dir)
+        target = dir.clone(self.get_url('target'))
+        self.skipIfNoWorkingTree(target)
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    ['./.bzr/stat-cache',
+                                     './.bzr/checkout/stat-cache',
+                                     './.bzr/checkout/merge-hashes',
+                                     './.bzr/merge-hashes',
+                                     './.bzr/repository/inventory.knit',
+                                     ])
+
+    def test_clone_bzrdir_branch_and_repo_into_shared_repo_force_new_repo(self):
+        # by default cloning into a shared repo uses the shared repo.
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['commit_tree/foo'])
+        tree.add('foo')
+        tree.commit('revision 1')
+        source = self.make_branch('source')
+        tree.branch.repository.copy_content_into(source.repository)
+        tree.branch.copy_content_into(source)
+        try:
+            self.make_repository('target', shared=True)
+        except errors.IncompatibleFormat:
+            return
+        dir = source.bzrdir
+        target = dir.clone(self.get_url('target/child'), force_new_repo=True)
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        repo = target.open_repository()
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    ['./.bzr/repository',
+                                     ])
+        self.assertRepositoryHasSameItems(tree.branch.repository, repo)
+
+    def test_clone_bzrdir_branch_reference(self):
+        # cloning should preserve the reference status of the branch in a bzrdir
+        referenced_branch = self.make_branch('referencced')
+        dir = self.make_bzrdir('source')
+        try:
+            reference = bzrlib.branch.BranchReferenceFormat().initialize(dir,
+                target_branch=referenced_branch)
+        except errors.IncompatibleFormat:
+            # this is ok too, not all formats have to support references.
+            return
+        target = dir.clone(self.get_url('target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport)
+
+    def test_sprout_bzrdir_repository(self):
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        dir = self.make_bzrdir('source')
+        repo = dir.create_repository()
+        repo.fetch(tree.branch.repository)
+        self.assertTrue(repo.has_revision('1'))
+        try:
+            self.assertTrue(
+                _mod_revision.is_null(_mod_revision.ensure_null(
+                dir.open_branch().last_revision())))
+        except errors.NotBranchError:
+            pass
+        target = dir.sprout(self.get_url('target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        # testing inventory isn't reasonable for repositories
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    [
+                                     './.bzr/branch',
+                                     './.bzr/checkout',
+                                     './.bzr/inventory',
+                                     './.bzr/parent',
+                                     './.bzr/repository/inventory.knit',
+                                     ])
+        try:
+            local_inventory = dir.transport.local_abspath('inventory')
+        except errors.NotLocalUrl:
+            return
+        try:
+            # If we happen to have a tree, we'll guarantee everything
+            # except for the tree root is the same.
+            inventory_f = file(local_inventory, 'rb')
+            self.addCleanup(inventory_f.close)
+            self.assertContainsRe(inventory_f.read(),
+                                  '<inventory format="5">\n</inventory>\n')
+        except IOError, e:
+            if e.errno != errno.ENOENT:
+                raise
+
+    def test_sprout_bzrdir_branch_and_repo(self):
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['commit_tree/foo'])
+        tree.add('foo')
+        tree.commit('revision 1')
+        source = self.make_branch('source')
+        tree.branch.repository.copy_content_into(source.repository)
+        tree.bzrdir.open_branch().copy_content_into(source)
+        dir = source.bzrdir
+        target = dir.sprout(self.get_url('target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        target_repo = target.open_repository()
+        self.assertRepositoryHasSameItems(source.repository, target_repo)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    [
+                                     './.bzr/basis-inventory-cache',
+                                     './.bzr/branch/branch.conf',
+                                     './.bzr/branch/parent',
+                                     './.bzr/checkout',
+                                     './.bzr/checkout/inventory',
+                                     './.bzr/checkout/stat-cache',
+                                     './.bzr/inventory',
+                                     './.bzr/parent',
+                                     './.bzr/repository',
+                                     './.bzr/stat-cache',
+                                     './foo',
+                                     ])
+
+    def test_sprout_bzrdir_tree_branch_repo(self):
+        tree = self.make_branch_and_tree('source')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1')
+        dir = tree.bzrdir
+        target = self.sproutOrSkip(dir, self.get_url('target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    [
+                                     './.bzr/branch/branch.conf',
+                                     './.bzr/branch/parent',
+                                     './.bzr/checkout/dirstate',
+                                     './.bzr/checkout/stat-cache',
+                                     './.bzr/checkout/inventory',
+                                     './.bzr/inventory',
+                                     './.bzr/parent',
+                                     './.bzr/repository',
+                                     './.bzr/stat-cache',
+                                     ])
+        self.assertRepositoryHasSameItems(
+            tree.branch.repository, target.open_repository())
+
+
+    def test_retire_bzrdir(self):
+        bd = self.make_bzrdir('.')
+        transport = bd.root_transport
+        # must not overwrite existing directories
+        self.build_tree(['.bzr.retired.0/', '.bzr.retired.0/junk',],
+            transport=transport)
+        self.failUnless(transport.has('.bzr'))
+        bd.retire_bzrdir()
+        self.failIf(transport.has('.bzr'))
+        self.failUnless(transport.has('.bzr.retired.1'))
+
+    def test_retire_bzrdir_limited(self):
+        bd = self.make_bzrdir('.')
+        transport = bd.root_transport
+        # must not overwrite existing directories
+        self.build_tree(['.bzr.retired.0/', '.bzr.retired.0/junk',],
+            transport=transport)
+        self.failUnless(transport.has('.bzr'))
+        self.assertRaises((errors.FileExists, errors.DirectoryNotEmpty),
+            bd.retire_bzrdir, limit=0)

=== renamed directory 'bzrlib/tests/per_bzrdir' => 'bzrlib/tests/per_controldir'
=== modified file 'bzrlib/tests/per_controldir/__init__.py'
--- bzrlib/tests/per_bzrdir/__init__.py	2010-02-11 09:21:45 +0000
+++ bzrlib/tests/per_controldir/__init__.py	2010-08-27 17:53:08 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 # Authors: Robert Collins <robert.collins@canonical.com>
 # -*- coding: utf-8 -*-
 #
@@ -17,14 +17,14 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
-"""BzrDir implementation tests for bzr.
+"""ControlDir implementation tests for bzr.
 
-These test the conformance of all the bzrdir variations to the expected API.
+These test the conformance of all the controldir variations to the expected API.
 Specific tests for individual formats are in the tests/test_bzrdir.py file
 rather than in tests/per_branch/*.py.
 """
 
-from bzrlib.bzrdir import BzrDirFormat
+from bzrlib.controldir import ControlDirFormat
 from bzrlib.tests import (
     default_transport,
     multiply_tests,
@@ -56,10 +56,10 @@
     return result
 
 
-class TestCaseWithBzrDir(TestCaseWithTransport):
+class TestCaseWithControlDir(TestCaseWithTransport):
 
     def setUp(self):
-        super(TestCaseWithBzrDir, self).setUp()
+        super(TestCaseWithControlDir, self).setUp()
         self.bzrdir = None
 
     def get_bzrdir(self):
@@ -70,17 +70,17 @@
     def make_bzrdir(self, relpath, format=None):
         if format is None:
             format = self.bzrdir_format
-        return super(TestCaseWithBzrDir, self).make_bzrdir(
+        return super(TestCaseWithControlDir, self).make_bzrdir(
             relpath, format=format)
 
 
 def load_tests(standard_tests, module, loader):
-    test_per_bzrdir = [
-        'bzrlib.tests.per_bzrdir.test_bzrdir',
-        'bzrlib.tests.per_bzrdir.test_push',
+    test_per_controldir = [
+        'bzrlib.tests.per_controldir.test_controldir',
+        'bzrlib.tests.per_controldir.test_push',
         ]
-    submod_tests = loader.loadTestsFromModuleNames(test_per_bzrdir)
-    formats = BzrDirFormat.known_formats()
+    submod_tests = loader.loadTestsFromModuleNames(test_per_controldir)
+    formats = ControlDirFormat.known_formats()
     scenarios = make_scenarios(
         default_transport,
         None,

=== renamed file 'bzrlib/tests/per_bzrdir/test_bzrdir.py' => 'bzrlib/tests/per_controldir/test_controldir.py'
--- bzrlib/tests/per_bzrdir/test_bzrdir.py	2010-04-21 04:02:42 +0000
+++ bzrlib/tests/per_controldir/test_controldir.py	2010-09-11 20:25:26 +0000
@@ -14,154 +14,40 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-"""Tests for bzrdir implementations - tests a bzrdir format."""
+"""Tests for control directory implementations - tests a controldir format."""
 
-from cStringIO import StringIO
-import errno
 from itertools import izip
 import os
-from stat import S_ISDIR
 
 import bzrlib.branch
 from bzrlib import (
     bzrdir,
     check,
+    controldir,
     errors,
     gpg,
-    lockdir,
     osutils,
-    repository,
-    revision as _mod_revision,
-    transactions,
     transport,
     ui,
     urlutils,
     workingtree,
     )
-from bzrlib.branch import Branch, needs_read_lock, needs_write_lock
-from bzrlib.errors import (FileExists,
-                           NoSuchRevision,
-                           NoSuchFile,
-                           UninitializableFormat,
-                           NotBranchError,
-                           )
 import bzrlib.revision
 from bzrlib.tests import (
                           ChrootedTestCase,
-                          TestCase,
-                          TestCaseWithTransport,
                           TestNotApplicable,
                           TestSkipped,
                           )
-from bzrlib.tests.per_bzrdir import TestCaseWithBzrDir
-from bzrlib.trace import mutter
-from bzrlib.transport import get_transport
+from bzrlib.tests.per_controldir import TestCaseWithControlDir
 from bzrlib.transport.local import LocalTransport
 from bzrlib.ui import (
     CannedInputUIFactory,
     )
-from bzrlib.upgrade import upgrade
 from bzrlib.remote import RemoteBzrDir, RemoteRepository
 from bzrlib.repofmt import weaverepo
 
 
-class TestBzrDir(TestCaseWithBzrDir):
-    # Many of these tests test for disk equality rather than checking
-    # for semantic equivalence. This works well for some tests but
-    # is not good at handling changes in representation or the addition
-    # or removal of control data. It would be nice to for instance:
-    # sprout a new branch, check that the nickname has been reset by hand
-    # and then set the nickname to match the source branch, at which point
-    # a semantic equivalence should pass
-
-    def assertDirectoriesEqual(self, source, target, ignore_list=[]):
-        """Assert that the content of source and target are identical.
-
-        paths in ignore list will be completely ignored.
-
-        We ignore paths that represent data which is allowed to change during
-        a clone or sprout: for instance, inventory.knit contains gzip fragements
-        which have timestamps in them, and as we have read the inventory from
-        the source knit, the already-read data is recompressed rather than
-        reading it again, which leads to changed timestamps. This is ok though,
-        because the inventory.kndx file is not ignored, and the integrity of
-        knit joins is tested by test_knit and test_versionedfile.
-
-        :seealso: Additionally, assertRepositoryHasSameItems provides value
-            rather than representation checking of repositories for
-            equivalence.
-        """
-        files = []
-        directories = ['.']
-        while directories:
-            dir = directories.pop()
-            for path in set(source.list_dir(dir) + target.list_dir(dir)):
-                path = dir + '/' + path
-                if path in ignore_list:
-                    continue
-                try:
-                    stat = source.stat(path)
-                except errors.NoSuchFile:
-                    self.fail('%s not in source' % path)
-                if S_ISDIR(stat.st_mode):
-                    self.assertTrue(S_ISDIR(target.stat(path).st_mode))
-                    directories.append(path)
-                else:
-                    self.assertEqualDiff(source.get(path).read(),
-                                         target.get(path).read(),
-                                         "text for file %r differs:\n" % path)
-
-    def assertRepositoryHasSameItems(self, left_repo, right_repo):
-        """require left_repo and right_repo to contain the same data."""
-        # XXX: TODO: Doesn't work yet, because we need to be able to compare
-        # local repositories to remote ones...  but this is an as-yet unsolved
-        # aspect of format management and the Remote protocols...
-        # self.assertEqual(left_repo._format.__class__,
-        #     right_repo._format.__class__)
-        left_repo.lock_read()
-        try:
-            right_repo.lock_read()
-            try:
-                # revs
-                all_revs = left_repo.all_revision_ids()
-                self.assertEqual(left_repo.all_revision_ids(),
-                    right_repo.all_revision_ids())
-                for rev_id in left_repo.all_revision_ids():
-                    self.assertEqual(left_repo.get_revision(rev_id),
-                        right_repo.get_revision(rev_id))
-                # Assert the revision trees (and thus the inventories) are equal
-                sort_key = lambda rev_tree: rev_tree.get_revision_id()
-                rev_trees_a = sorted(
-                    left_repo.revision_trees(all_revs), key=sort_key)
-                rev_trees_b = sorted(
-                    right_repo.revision_trees(all_revs), key=sort_key)
-                for tree_a, tree_b in zip(rev_trees_a, rev_trees_b):
-                    self.assertEqual([], list(tree_a.iter_changes(tree_b)))
-                # texts
-                text_index = left_repo._generate_text_key_index()
-                self.assertEqual(text_index,
-                    right_repo._generate_text_key_index())
-                desired_files = []
-                for file_id, revision_id in text_index.iterkeys():
-                    desired_files.append(
-                        (file_id, revision_id, (file_id, revision_id)))
-                left_texts = list(left_repo.iter_files_bytes(desired_files))
-                right_texts = list(right_repo.iter_files_bytes(desired_files))
-                left_texts.sort()
-                right_texts.sort()
-                self.assertEqual(left_texts, right_texts)
-                # signatures
-                for rev_id in all_revs:
-                    try:
-                        left_text = left_repo.get_signature_text(rev_id)
-                    except NoSuchRevision:
-                        continue
-                    right_text = right_repo.get_signature_text(rev_id)
-                    self.assertEqual(left_text, right_text)
-            finally:
-                right_repo.unlock()
-        finally:
-            left_repo.unlock()
+class TestControlDir(TestCaseWithControlDir):
 
     def skipIfNoWorkingTree(self, a_bzrdir):
         """Raises TestSkipped if a_bzrdir doesn't have a working tree.
@@ -190,7 +76,7 @@
         """
         try:
             return a_bzrdir.create_workingtree()
-        except errors.NotLocalUrl:
+        except (errors.NotLocalUrl, errors.UnsupportedOperation):
             raise TestSkipped("cannot make working tree with transport %r"
                               % a_bzrdir.transport)
 
@@ -202,7 +88,7 @@
         A simple wrapper for from_bzrdir.sprout that translates NotLocalUrl into
         TestSkipped.  Returns the newly sprouted bzrdir.
         """
-        to_transport = get_transport(to_url)
+        to_transport = transport.get_transport(to_url)
         if not isinstance(to_transport, LocalTransport):
             raise TestSkipped('Cannot sprout to remote bzrdirs.')
         target = from_bzrdir.sprout(to_url, revision_id=revision_id,
@@ -218,7 +104,7 @@
         dir.create_branch()
         try:
             wt = dir.create_workingtree(revision_id=bzrlib.revision.NULL_REVISION)
-        except errors.NotLocalUrl:
+        except (errors.NotLocalUrl, errors.UnsupportedOperation):
             raise TestSkipped("cannot make working tree with transport %r"
                               % dir.transport)
         self.assertEqual([], wt.get_parent_ids())
@@ -265,60 +151,17 @@
         bzrdir.open_repository()
 
     def test_open_workingtree_raises_no_working_tree(self):
-        """BzrDir.open_workingtree() should raise NoWorkingTree (rather than
+        """ControlDir.open_workingtree() should raise NoWorkingTree (rather than
         e.g. NotLocalUrl) if there is no working tree.
         """
         dir = self.make_bzrdir('source')
         vfs_dir = bzrdir.BzrDir.open(self.get_vfs_only_url('source'))
         if vfs_dir.has_workingtree():
-            # This BzrDir format doesn't support BzrDirs without working trees,
-            # so this test is irrelevant.
+            # This ControlDir format doesn't support ControlDirs without
+            # working trees, so this test is irrelevant.
             return
         self.assertRaises(errors.NoWorkingTree, dir.open_workingtree)
 
-    def test_clone_on_transport(self):
-        a_dir = self.make_bzrdir('source')
-        target_transport = a_dir.root_transport.clone('..').clone('target')
-        target = a_dir.clone_on_transport(target_transport)
-        self.assertNotEqual(a_dir.transport.base, target.transport.base)
-        self.assertDirectoriesEqual(a_dir.root_transport, target.root_transport,
-                                    ['./.bzr/merge-hashes'])
-
-    def test_clone_bzrdir_empty(self):
-        dir = self.make_bzrdir('source')
-        target = dir.clone(self.get_url('target'))
-        self.assertNotEqual(dir.transport.base, target.transport.base)
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    ['./.bzr/merge-hashes'])
-
-    def test_clone_bzrdir_empty_force_new_ignored(self):
-        # the force_new_repo parameter should have no effect on an empty
-        # bzrdir's clone logic
-        dir = self.make_bzrdir('source')
-        target = dir.clone(self.get_url('target'), force_new_repo=True)
-        self.assertNotEqual(dir.transport.base, target.transport.base)
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    ['./.bzr/merge-hashes'])
-
-    def test_clone_bzrdir_repository(self):
-        tree = self.make_branch_and_tree('commit_tree')
-        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
-        tree.add('foo')
-        tree.commit('revision 1', rev_id='1')
-        dir = self.make_bzrdir('source')
-        repo = dir.create_repository()
-        repo.fetch(tree.branch.repository)
-        self.assertTrue(repo.has_revision('1'))
-        target = dir.clone(self.get_url('target'))
-        self.assertNotEqual(dir.transport.base, target.transport.base)
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    [
-                                     './.bzr/merge-hashes',
-                                     './.bzr/repository',
-                                     ])
-        self.assertRepositoryHasSameItems(tree.branch.repository,
-            target.open_repository())
-
     def test_clone_bzrdir_repository_under_shared(self):
         tree = self.make_branch_and_tree('commit_tree')
         self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
@@ -394,26 +237,6 @@
         self.assertFalse(branch.repository.make_working_trees())
         self.assertTrue(branch.repository.is_shared())
 
-    def test_clone_bzrdir_repository_under_shared_force_new_repo(self):
-        tree = self.make_branch_and_tree('commit_tree')
-        self.build_tree(['commit_tree/foo'])
-        tree.add('foo')
-        tree.commit('revision 1', rev_id='1')
-        dir = self.make_bzrdir('source')
-        repo = dir.create_repository()
-        repo.fetch(tree.branch.repository)
-        self.assertTrue(repo.has_revision('1'))
-        try:
-            self.make_repository('target', shared=True)
-        except errors.IncompatibleFormat:
-            return
-        target = dir.clone(self.get_url('target/child'), force_new_repo=True)
-        self.assertNotEqual(dir.transport.base, target.transport.base)
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    ['./.bzr/repository',
-                                     ])
-        self.assertRepositoryHasSameItems(tree.branch.repository, repo)
-
     def test_clone_bzrdir_repository_revision(self):
         # test for revision limiting, [smoke test, not corner case checks].
         # make a repository with some revisions,
@@ -453,28 +276,6 @@
             tree_repo.get_signature_text(rev1),
             target.repository.get_signature_text(rev1))
 
-    def test_clone_bzrdir_branch_and_repo(self):
-        tree = self.make_branch_and_tree('commit_tree')
-        self.build_tree(['commit_tree/foo'])
-        tree.add('foo')
-        tree.commit('revision 1')
-        source = self.make_branch('source')
-        tree.branch.repository.copy_content_into(source.repository)
-        tree.branch.copy_content_into(source)
-        dir = source.bzrdir
-        target = dir.clone(self.get_url('target'))
-        self.assertNotEqual(dir.transport.base, target.transport.base)
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    [
-                                     './.bzr/basis-inventory-cache',
-                                     './.bzr/checkout/stat-cache',
-                                     './.bzr/merge-hashes',
-                                     './.bzr/repository',
-                                     './.bzr/stat-cache',
-                                    ])
-        self.assertRepositoryHasSameItems(
-            tree.branch.repository, target.open_repository())
-
     def test_clone_bzrdir_branch_and_repo_into_shared_repo(self):
         # by default cloning into a shared repo uses the shared repo.
         tree = self.make_branch_and_tree('commit_tree')
@@ -495,42 +296,6 @@
         self.assertEqual(source.revision_history(),
                          target.open_branch().revision_history())
 
-    def test_clone_bzrdir_branch_and_repo_into_shared_repo_force_new_repo(self):
-        # by default cloning into a shared repo uses the shared repo.
-        tree = self.make_branch_and_tree('commit_tree')
-        self.build_tree(['commit_tree/foo'])
-        tree.add('foo')
-        tree.commit('revision 1')
-        source = self.make_branch('source')
-        tree.branch.repository.copy_content_into(source.repository)
-        tree.branch.copy_content_into(source)
-        try:
-            self.make_repository('target', shared=True)
-        except errors.IncompatibleFormat:
-            return
-        dir = source.bzrdir
-        target = dir.clone(self.get_url('target/child'), force_new_repo=True)
-        self.assertNotEqual(dir.transport.base, target.transport.base)
-        repo = target.open_repository()
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    ['./.bzr/repository',
-                                     ])
-        self.assertRepositoryHasSameItems(tree.branch.repository, repo)
-
-    def test_clone_bzrdir_branch_reference(self):
-        # cloning should preserve the reference status of the branch in a bzrdir
-        referenced_branch = self.make_branch('referencced')
-        dir = self.make_bzrdir('source')
-        try:
-            reference = bzrlib.branch.BranchReferenceFormat().initialize(dir,
-                target_branch=referenced_branch)
-        except errors.IncompatibleFormat:
-            # this is ok too, not all formats have to support references.
-            return
-        target = dir.clone(self.get_url('target'))
-        self.assertNotEqual(dir.transport.base, target.transport.base)
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport)
-
     def test_clone_bzrdir_branch_revision(self):
         # test for revision limiting, [smoke test, not corner case checks].
         # make a branch with some revisions,
@@ -548,27 +313,6 @@
         target = dir.clone(self.get_url('target'), revision_id='1')
         self.assertEqual('1', target.open_branch().last_revision())
 
-    def test_clone_bzrdir_tree_branch_repo(self):
-        tree = self.make_branch_and_tree('source')
-        self.build_tree(['source/foo'])
-        tree.add('foo')
-        tree.commit('revision 1')
-        dir = tree.bzrdir
-        target = dir.clone(self.get_url('target'))
-        self.skipIfNoWorkingTree(target)
-        self.assertNotEqual(dir.transport.base, target.transport.base)
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    ['./.bzr/stat-cache',
-                                     './.bzr/checkout/dirstate',
-                                     './.bzr/checkout/stat-cache',
-                                     './.bzr/checkout/merge-hashes',
-                                     './.bzr/merge-hashes',
-                                     './.bzr/repository',
-                                     ])
-        self.assertRepositoryHasSameItems(tree.branch.repository,
-            target.open_repository())
-        target.open_workingtree().revert()
-
     def test_clone_on_transport_preserves_repo_format(self):
         if self.bzrdir_format == bzrdir.format_registry.make_bzrdir('default'):
             format = 'knit'
@@ -588,60 +332,6 @@
             target_repo = target_repo._real_repository
         self.assertEqual(target_repo._format, source_branch.repository._format)
 
-    def test_revert_inventory(self):
-        tree = self.make_branch_and_tree('source')
-        self.build_tree(['source/foo'])
-        tree.add('foo')
-        tree.commit('revision 1')
-        dir = tree.bzrdir
-        target = dir.clone(self.get_url('target'))
-        self.skipIfNoWorkingTree(target)
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    ['./.bzr/stat-cache',
-                                     './.bzr/checkout/dirstate',
-                                     './.bzr/checkout/stat-cache',
-                                     './.bzr/checkout/merge-hashes',
-                                     './.bzr/merge-hashes',
-                                     './.bzr/repository',
-                                     ])
-        self.assertRepositoryHasSameItems(tree.branch.repository,
-            target.open_repository())
-
-        target.open_workingtree().revert()
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    ['./.bzr/stat-cache',
-                                     './.bzr/checkout/dirstate',
-                                     './.bzr/checkout/stat-cache',
-                                     './.bzr/checkout/merge-hashes',
-                                     './.bzr/merge-hashes',
-                                     './.bzr/repository',
-                                     ])
-        self.assertRepositoryHasSameItems(tree.branch.repository,
-            target.open_repository())
-
-    def test_clone_bzrdir_tree_branch_reference(self):
-        # a tree with a branch reference (aka a checkout)
-        # should stay a checkout on clone.
-        referenced_branch = self.make_branch('referencced')
-        dir = self.make_bzrdir('source')
-        try:
-            reference = bzrlib.branch.BranchReferenceFormat().initialize(dir,
-                target_branch=referenced_branch)
-        except errors.IncompatibleFormat:
-            # this is ok too, not all formats have to support references.
-            return
-        self.createWorkingTreeOrSkip(dir)
-        target = dir.clone(self.get_url('target'))
-        self.skipIfNoWorkingTree(target)
-        self.assertNotEqual(dir.transport.base, target.transport.base)
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    ['./.bzr/stat-cache',
-                                     './.bzr/checkout/stat-cache',
-                                     './.bzr/checkout/merge-hashes',
-                                     './.bzr/merge-hashes',
-                                     './.bzr/repository/inventory.knit',
-                                     ])
-
     def test_clone_bzrdir_tree_revision(self):
         # test for revision limiting, [smoke test, not corner case checks].
         # make a tree with a revision with a last-revision
@@ -733,9 +423,8 @@
         try:
             target.open_workingtree()
         except errors.NoWorkingTree:
-            # bzrdir's that never have working trees are allowed to pass;
-            # whitelist them for now.
-            self.assertIsInstance(target, RemoteBzrDir)
+            # Some bzrdirs can never have working trees.
+            self.assertFalse(target._format.supports_workingtrees)
 
     def test_sprout_bzrdir_empty_under_shared_repo_force_new(self):
         # the force_new_repo parameter should force use of a new repo in an empty
@@ -750,47 +439,6 @@
         target.open_branch()
         self.openWorkingTreeIfLocal(target)
 
-    def test_sprout_bzrdir_repository(self):
-        tree = self.make_branch_and_tree('commit_tree')
-        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
-        tree.add('foo')
-        tree.commit('revision 1', rev_id='1')
-        dir = self.make_bzrdir('source')
-        repo = dir.create_repository()
-        repo.fetch(tree.branch.repository)
-        self.assertTrue(repo.has_revision('1'))
-        try:
-            self.assertTrue(
-                _mod_revision.is_null(_mod_revision.ensure_null(
-                dir.open_branch().last_revision())))
-        except errors.NotBranchError:
-            pass
-        target = dir.sprout(self.get_url('target'))
-        self.assertNotEqual(dir.transport.base, target.transport.base)
-        # testing inventory isn't reasonable for repositories
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    [
-                                     './.bzr/branch',
-                                     './.bzr/checkout',
-                                     './.bzr/inventory',
-                                     './.bzr/parent',
-                                     './.bzr/repository/inventory.knit',
-                                     ])
-        try:
-            local_inventory = dir.transport.local_abspath('inventory')
-        except errors.NotLocalUrl:
-            return
-        try:
-            # If we happen to have a tree, we'll guarantee everything
-            # except for the tree root is the same.
-            inventory_f = file(local_inventory, 'rb')
-            self.addCleanup(inventory_f.close)
-            self.assertContainsRe(inventory_f.read(),
-                                  '<inventory format="5">\n</inventory>\n')
-        except IOError, e:
-            if e.errno != errno.ENOENT:
-                raise
-
     def test_sprout_bzrdir_with_repository_to_shared(self):
         tree = self.make_branch_and_tree('commit_tree')
         self.build_tree(['commit_tree/foo'])
@@ -854,7 +502,7 @@
         self.assertNotEqual(dir.transport.base, shared_repo.bzrdir.transport.base)
         branch = target.open_branch()
         self.assertTrue(branch.repository.has_revision('1'))
-        if not isinstance(branch.bzrdir, RemoteBzrDir):
+        if branch.bzrdir._format.supports_workingtrees:
             self.assertTrue(branch.repository.make_working_trees())
         self.assertFalse(branch.repository.is_shared())
 
@@ -895,34 +543,6 @@
         target = self.sproutOrSkip(dir, self.get_url('target'), revision_id='2')
         raise TestSkipped('revision limiting not strict yet')
 
-    def test_sprout_bzrdir_branch_and_repo(self):
-        tree = self.make_branch_and_tree('commit_tree')
-        self.build_tree(['commit_tree/foo'])
-        tree.add('foo')
-        tree.commit('revision 1')
-        source = self.make_branch('source')
-        tree.branch.repository.copy_content_into(source.repository)
-        tree.bzrdir.open_branch().copy_content_into(source)
-        dir = source.bzrdir
-        target = dir.sprout(self.get_url('target'))
-        self.assertNotEqual(dir.transport.base, target.transport.base)
-        target_repo = target.open_repository()
-        self.assertRepositoryHasSameItems(source.repository, target_repo)
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    [
-                                     './.bzr/basis-inventory-cache',
-                                     './.bzr/branch/branch.conf',
-                                     './.bzr/branch/parent',
-                                     './.bzr/checkout',
-                                     './.bzr/checkout/inventory',
-                                     './.bzr/checkout/stat-cache',
-                                     './.bzr/inventory',
-                                     './.bzr/parent',
-                                     './.bzr/repository',
-                                     './.bzr/stat-cache',
-                                     './foo',
-                                     ])
-
     def test_sprout_bzrdir_branch_and_repo_shared(self):
         # sprouting a branch with a repo into a shared repo uses the shared
         # repo
@@ -1047,29 +667,6 @@
         target = dir.sprout(self.get_url('target'), revision_id='1')
         self.assertEqual('1', target.open_branch().last_revision())
 
-    def test_sprout_bzrdir_tree_branch_repo(self):
-        tree = self.make_branch_and_tree('source')
-        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
-        tree.add('foo')
-        tree.commit('revision 1')
-        dir = tree.bzrdir
-        target = self.sproutOrSkip(dir, self.get_url('target'))
-        self.assertNotEqual(dir.transport.base, target.transport.base)
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    [
-                                     './.bzr/branch/branch.conf',
-                                     './.bzr/branch/parent',
-                                     './.bzr/checkout/dirstate',
-                                     './.bzr/checkout/stat-cache',
-                                     './.bzr/checkout/inventory',
-                                     './.bzr/inventory',
-                                     './.bzr/parent',
-                                     './.bzr/repository',
-                                     './.bzr/stat-cache',
-                                     ])
-        self.assertRepositoryHasSameItems(
-            tree.branch.repository, target.open_repository())
-
     def test_sprout_bzrdir_tree_branch_reference(self):
         # sprouting should create a repository if needed and a sprouted branch.
         # the tree state should not be copied.
@@ -1168,6 +765,29 @@
         self.assertEqual(tree.branch.last_revision(),
                          target.open_branch().last_revision())
 
+    def test_sprout_with_revision_id_uses_default_stack_on(self):
+        # Make a branch with three commits to stack on.
+        builder = self.make_branch_builder('stack-on')
+        builder.start_series()
+        builder.build_commit(message='Rev 1.', rev_id='rev-1')
+        builder.build_commit(message='Rev 2.', rev_id='rev-2')
+        builder.build_commit(message='Rev 3.', rev_id='rev-3')
+        builder.finish_series()
+        stack_on = builder.get_branch()
+        # Make a bzrdir with a default stacking policy to stack on that branch.
+        config = self.make_bzrdir('policy-dir').get_config()
+        try:
+            config.set_default_stack_on(self.get_url('stack-on'))
+        except errors.BzrError:
+            raise TestNotApplicable('Only relevant for stackable formats.')
+        # Sprout the stacked-on branch into the bzrdir.
+        sprouted = stack_on.bzrdir.sprout(
+            self.get_url('policy-dir/sprouted'), revision_id='rev-3')
+        # Not all revisions are copied into the sprouted repository.
+        repo = sprouted.open_repository()
+        self.addCleanup(repo.lock_read().unlock)
+        self.assertEqual(None, repo.get_parent_map(['rev-1']).get('rev-1'))
+
     def test_format_initialize_find_open(self):
         # loopback test to check the current format initializes to itself.
         if not self.bzrdir_format.is_supported():
@@ -1178,24 +798,24 @@
         # for remote formats, there must be no prior assumption about the
         # network name to use - it's possible that this may somehow have got
         # in through an unisolated test though - see
-        # <https://bugs.edge.launchpad.net/bzr/+bug/504102>
+        # <https://bugs.launchpad.net/bzr/+bug/504102>
         self.assertEquals(getattr(self.bzrdir_format,
             '_network_name', None),
             None)
         # supported formats must be able to init and open
-        t = get_transport(self.get_url())
-        readonly_t = get_transport(self.get_readonly_url())
+        t = transport.get_transport(self.get_url())
+        readonly_t = transport.get_transport(self.get_readonly_url())
         made_control = self.bzrdir_format.initialize(t.base)
-        self.failUnless(isinstance(made_control, bzrdir.BzrDir))
+        self.failUnless(isinstance(made_control, controldir.ControlDir))
         self.assertEqual(self.bzrdir_format,
-                         bzrdir.BzrDirFormat.find_format(readonly_t))
+                         controldir.ControlDirFormat.find_format(readonly_t))
         direct_opened_dir = self.bzrdir_format.open(readonly_t)
         opened_dir = bzrdir.BzrDir.open(t.base)
         self.assertEqual(made_control._format,
                          opened_dir._format)
         self.assertEqual(direct_opened_dir._format,
                          opened_dir._format)
-        self.failUnless(isinstance(opened_dir, bzrdir.BzrDir))
+        self.failUnless(isinstance(opened_dir, controldir.ControlDir))
 
     def test_format_initialize_on_transport_ex(self):
         t = self.get_transport('dir')
@@ -1383,18 +1003,19 @@
             network_name = format.network_name()
             self.assertEqual(real_dir._format.network_name(), network_name)
         else:
-            registry = bzrdir.network_format_registry
+            registry = controldir.network_format_registry
             network_name = format.network_name()
             looked_up_format = registry.get(network_name)
-            self.assertEqual(format.__class__, looked_up_format.__class__)
+            self.assertTrue(
+                issubclass(format.__class__, looked_up_format.__class__))
         # The network name must be a byte string.
         self.assertIsInstance(network_name, str)
 
     def test_open_not_bzrdir(self):
         # test the formats specific behaviour for no-content or similar dirs.
-        self.assertRaises(NotBranchError,
+        self.assertRaises(errors.NotBranchError,
                           self.bzrdir_format.open,
-                          get_transport(self.get_readonly_url()))
+                          transport.get_transport(self.get_readonly_url()))
 
     def test_create_branch(self):
         # a bzrdir can construct a branch and repository for itself.
@@ -1403,7 +1024,7 @@
             # because the default open will not open them and
             # they may not be initializable.
             return
-        t = get_transport(self.get_url())
+        t = transport.get_transport(self.get_url())
         made_control = self.bzrdir_format.initialize(t.base)
         made_repo = made_control.create_repository()
         made_branch = made_control.create_branch()
@@ -1416,7 +1037,7 @@
             # because the default open will not open them and
             # they may not be initializable.
             return
-        t = get_transport(self.get_url())
+        t = transport.get_transport(self.get_url())
         made_control = self.bzrdir_format.initialize(t.base)
         made_repo = made_control.create_repository()
         made_branch = made_control.create_branch()
@@ -1431,7 +1052,7 @@
             # because the default open will not open them and
             # they may not be initializable.
             return
-        t = get_transport(self.get_url())
+        t = transport.get_transport(self.get_url())
         made_control = self.bzrdir_format.initialize(t.base)
         made_repo = made_control.create_repository()
         made_branch = made_control.create_branch()
@@ -1452,7 +1073,7 @@
             # because the default open will not open them and
             # they may not be initializable.
             return
-        t = get_transport(self.get_url())
+        t = transport.get_transport(self.get_url())
         made_control = self.bzrdir_format.initialize(t.base)
         made_repo = made_control.create_repository()
         # Check that we have a repository object.
@@ -1467,7 +1088,7 @@
             # because the default open will not open them and
             # they may not be initializable.
             return
-        t = get_transport(self.get_url())
+        t = transport.get_transport(self.get_url())
         made_control = self.bzrdir_format.initialize(t.base)
         try:
             made_repo = made_control.create_repository(shared=True)
@@ -1484,7 +1105,7 @@
             # because the default open will not open them and
             # they may not be initializable.
             return
-        t = get_transport(self.get_url())
+        t = transport.get_transport(self.get_url())
         made_control = self.bzrdir_format.initialize(t.base)
         made_repo = made_control.create_repository(shared=False)
         self.assertFalse(made_repo.is_shared())
@@ -1495,7 +1116,7 @@
             # because the default open will not open them and
             # they may not be initializable.
             return
-        t = get_transport(self.get_url())
+        t = transport.get_transport(self.get_url())
         made_control = self.bzrdir_format.initialize(t.base)
         made_repo = made_control.create_repository()
         opened_repo = made_control.open_repository()
@@ -1531,7 +1152,7 @@
         source.branch.clone(made_control)
         try:
             made_tree = made_control.create_workingtree(revision_id='a')
-        except errors.NotLocalUrl:
+        except (errors.NotLocalUrl, errors.UnsupportedOperation):
             raise TestSkipped("Can't make working tree on transport %r" % t)
         self.assertEqual(['a'], made_tree.get_parent_ids())
 
@@ -1549,7 +1170,7 @@
             made_repo = made_control.create_repository()
             made_branch = made_control.create_branch()
             made_tree = made_control.create_workingtree()
-        except errors.NotLocalUrl:
+        except (errors.NotLocalUrl, errors.UnsupportedOperation):
             raise TestSkipped("Can't initialize %r on transport %r"
                               % (self.bzrdir_format, t))
         opened_tree = made_control.open_workingtree()
@@ -1623,7 +1244,7 @@
     def test_root_transport(self):
         dir = self.make_bzrdir('.')
         self.assertEqual(dir.root_transport.base,
-                         get_transport(self.get_url('.')).base)
+                         transport.get_transport(self.get_url('.')).base)
 
     def test_find_repository_no_repo_under_standalone_branch(self):
         # finding a repo stops at standalone branches even if there is a
@@ -1634,8 +1255,8 @@
             # need a shared repository to test this.
             return
         url = self.get_url('intermediate')
-        get_transport(self.get_url()).mkdir('intermediate')
-        get_transport(self.get_url()).mkdir('intermediate/child')
+        transport.get_transport(self.get_url()).mkdir('intermediate')
+        transport.get_transport(self.get_url()).mkdir('intermediate/child')
         made_control = self.bzrdir_format.initialize(url)
         made_control.create_repository()
         innermost_control = self.bzrdir_format.initialize(
@@ -1659,7 +1280,7 @@
             # need a shared repository to test this.
             return
         url = self.get_url('childbzrdir')
-        get_transport(self.get_url()).mkdir('childbzrdir')
+        transport.get_transport(self.get_url()).mkdir('childbzrdir')
         made_control = self.bzrdir_format.initialize(url)
         try:
             child_repo = made_control.open_repository()
@@ -1693,7 +1314,7 @@
             # need a shared repository to test this.
             return
         url = self.get_url('childrepo')
-        get_transport(self.get_url()).mkdir('childrepo')
+        transport.get_transport(self.get_url()).mkdir('childrepo')
         child_control = self.bzrdir_format.initialize(url)
         child_repo = child_control.create_repository(shared=True)
         opened_control = bzrdir.BzrDir.open(self.get_url('childrepo'))
@@ -1712,8 +1333,8 @@
             # need a shared repository to test this.
             return
         url = self.get_url('intermediate')
-        get_transport(self.get_url()).mkdir('intermediate')
-        get_transport(self.get_url()).mkdir('intermediate/child')
+        transport.get_transport(self.get_url()).mkdir('intermediate')
+        transport.get_transport(self.get_url()).mkdir('intermediate/child')
         made_control = self.bzrdir_format.initialize(url)
         try:
             child_repo = made_control.open_repository()
@@ -1745,7 +1366,7 @@
             self.assertTrue(isinstance(dir._format.get_converter(
                 format=dir._format), bzrdir.Converter))
         dir.needs_format_conversion(
-            bzrdir.BzrDirFormat.get_default_format())
+            controldir.ControlDirFormat.get_default_format())
 
     def test_backup_copies_existing(self):
         tree = self.make_branch_and_tree('test')
@@ -1792,29 +1413,8 @@
         text = dir._format.get_format_description()
         self.failUnless(len(text))
 
-    def test_retire_bzrdir(self):
-        bd = self.make_bzrdir('.')
-        transport = bd.root_transport
-        # must not overwrite existing directories
-        self.build_tree(['.bzr.retired.0/', '.bzr.retired.0/junk',],
-            transport=transport)
-        self.failUnless(transport.has('.bzr'))
-        bd.retire_bzrdir()
-        self.failIf(transport.has('.bzr'))
-        self.failUnless(transport.has('.bzr.retired.1'))
-
-    def test_retire_bzrdir_limited(self):
-        bd = self.make_bzrdir('.')
-        transport = bd.root_transport
-        # must not overwrite existing directories
-        self.build_tree(['.bzr.retired.0/', '.bzr.retired.0/junk',],
-            transport=transport)
-        self.failUnless(transport.has('.bzr'))
-        self.assertRaises((errors.FileExists, errors.DirectoryNotEmpty),
-            bd.retire_bzrdir, limit=0)
-
-
-class TestBreakLock(TestCaseWithBzrDir):
+
+class TestBreakLock(TestCaseWithControlDir):
 
     def test_break_lock_empty(self):
         # break lock on an empty bzrdir should work silently.
@@ -1922,7 +1522,7 @@
         self.assertRaises(errors.LockBroken, tree.unlock)
 
 
-class TestTransportConfig(TestCaseWithBzrDir):
+class TestTransportConfig(TestCaseWithControlDir):
 
     def test_get_config(self):
         my_dir = self.make_bzrdir('.')
@@ -1944,7 +1544,7 @@
         self.assertEqual('http://example.com', config2.get_default_stack_on())
 
 
-class ChrootedBzrDirTests(ChrootedTestCase):
+class ChrootedControlDirTests(ChrootedTestCase):
 
     def test_find_repository_no_repository(self):
         # loopback test to check the current format fails to find a
@@ -1958,7 +1558,7 @@
         # - do the vfs initialisation over the basic vfs transport
         # XXX: TODO this should become a 'bzrdirlocation' api call.
         url = self.get_vfs_only_url('subdir')
-        get_transport(self.get_vfs_only_url()).mkdir('subdir')
+        transport.get_transport(self.get_vfs_only_url()).mkdir('subdir')
         made_control = self.bzrdir_format.initialize(self.get_url('subdir'))
         try:
             repo = made_control.open_repository()
@@ -1972,9 +1572,9 @@
                           made_control.find_repository)
 
 
-class TestBzrDirControlComponent(TestCaseWithBzrDir):
-    """BzrDir implementations adequately implement ControlComponent."""
-    
+class TestControlDirControlComponent(TestCaseWithControlDir):
+    """ControlDir implementations adequately implement ControlComponent."""
+
     def test_urls(self):
         bd = self.make_bzrdir('bd')
         self.assertIsInstance(bd.user_url, str)

=== modified file 'bzrlib/tests/per_controldir/test_push.py'
--- bzrlib/tests/per_bzrdir/test_push.py	2009-07-10 06:45:04 +0000
+++ bzrlib/tests/per_controldir/test_push.py	2010-08-27 17:53:08 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009 Canonical Ltd
+# Copyright (C) 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,15 +16,12 @@
 
 """Tests for bzrdir implementations - push."""
 
-from bzrlib import (
-    errors,
-    )
-from bzrlib.tests.per_bzrdir import (
-    TestCaseWithBzrDir,
-    )
-
-
-class TestPush(TestCaseWithBzrDir):
+from bzrlib.tests.per_controldir import (
+    TestCaseWithControlDir,
+    )
+
+
+class TestPush(TestCaseWithControlDir):
 
     def create_simple_tree(self):
         tree = self.make_branch_and_tree('tree')

=== renamed directory 'bzrlib/tests/per_bzrdir_colo' => 'bzrlib/tests/per_controldir_colo'
=== modified file 'bzrlib/tests/per_controldir_colo/__init__.py'
--- bzrlib/tests/per_bzrdir_colo/__init__.py	2010-04-12 00:34:15 +0000
+++ bzrlib/tests/per_controldir_colo/__init__.py	2010-08-21 16:06:24 +0000
@@ -23,13 +23,13 @@
 run.
 """
 
-from bzrlib.bzrdir import BzrDirFormat
+from bzrlib.controldir import ControlDirFormat
 from bzrlib.tests import (
     default_transport,
     multiply_tests,
     )
-from bzrlib.tests.per_bzrdir import (
-    TestCaseWithBzrDir,
+from bzrlib.tests.per_controldir import (
+    TestCaseWithControlDir,
     make_scenarios,
     )
 
@@ -37,7 +37,7 @@
 def load_tests(standard_tests, module, loader):
     colo_supported_formats = []
     colo_unsupported_formats = []
-    for format in BzrDirFormat.known_formats():
+    for format in ControlDirFormat.known_formats():
         if format.colocated_branches:
             colo_supported_formats.append(format)
         else:
@@ -48,9 +48,9 @@
         colo_unsupported_formats)
     result = loader.suiteClass()
     supported_tests = loader.loadTestsFromModuleNames([
-        'bzrlib.tests.per_bzrdir_colo.test_supported'])
+        'bzrlib.tests.per_controldir_colo.test_supported'])
     unsupported_tests = loader.loadTestsFromModuleNames([
-        'bzrlib.tests.per_bzrdir_colo.test_unsupported'])
+        'bzrlib.tests.per_controldir_colo.test_unsupported'])
     multiply_tests(supported_tests, supported_scenarios, result)
     multiply_tests(unsupported_tests, unsupported_scenarios, result)
     return result

=== modified file 'bzrlib/tests/per_controldir_colo/test_supported.py'
--- bzrlib/tests/per_bzrdir_colo/test_supported.py	2010-04-11 19:45:48 +0000
+++ bzrlib/tests/per_controldir_colo/test_supported.py	2010-08-21 16:06:24 +0000
@@ -25,12 +25,12 @@
     get_transport,
     )
 
-from bzrlib.tests.per_bzrdir_colo import (
-    TestCaseWithBzrDir,
+from bzrlib.tests.per_controldir_colo import (
+    TestCaseWithControlDir,
     )
 
 
-class TestColocatedBranchSupport(TestCaseWithBzrDir):
+class TestColocatedBranchSupport(TestCaseWithControlDir):
 
     def test_destroy_colocated_branch(self):
         branch = self.make_branch('branch')

=== modified file 'bzrlib/tests/per_controldir_colo/test_unsupported.py'
--- bzrlib/tests/per_bzrdir_colo/test_unsupported.py	2010-04-11 19:40:23 +0000
+++ bzrlib/tests/per_controldir_colo/test_unsupported.py	2010-08-21 16:06:24 +0000
@@ -28,22 +28,14 @@
     get_transport,
     )
 
-from bzrlib.tests.per_bzrdir_colo import (
-    TestCaseWithBzrDir,
+from bzrlib.tests.per_controldir_colo import (
+    TestCaseWithControlDir,
     )
 
 
-class TestNoColocatedSupport(TestCaseWithBzrDir):
-
-    def test_destroy_colocated_branch(self):
-        branch = self.make_branch('branch')
-        # Colocated branches should not be supported *or* 
-        # destroy_branch should not be supported at all
-        self.assertRaises(
-            (errors.NoColocatedBranchSupport, errors.UnsupportedOperation),
-            branch.bzrdir.destroy_branch, 'colo')
-
-    def test_create_colo_branch(self):
+class TestNoColocatedSupport(TestCaseWithControlDir):
+
+    def make_bzrdir_with_repo(self):
         # a bzrdir can construct a branch and repository for itself.
         if not self.bzrdir_format.is_supported():
             # unsupported formats are not loopback testable
@@ -53,7 +45,27 @@
         t = get_transport(self.get_url())
         made_control = self.bzrdir_format.initialize(t.base)
         made_repo = made_control.create_repository()
+        return made_control
+
+    def test_destroy_colocated_branch(self):
+        branch = self.make_branch('branch')
+        # Colocated branches should not be supported *or* 
+        # destroy_branch should not be supported at all
+        self.assertRaises(
+            (errors.NoColocatedBranchSupport, errors.UnsupportedOperation),
+            branch.bzrdir.destroy_branch, 'colo')
+
+    def test_create_colo_branch(self):
+        made_control = self.make_bzrdir_with_repo()
         self.assertRaises(errors.NoColocatedBranchSupport, 
             made_control.create_branch, "colo")
 
+    def test_branch_transport(self):
+        made_control = self.make_bzrdir_with_repo()
+        self.assertRaises(errors.NoColocatedBranchSupport,
+            made_control.get_branch_transport, None, "colo")
 
+    def test_get_branch_reference(self):
+        made_control = self.make_bzrdir_with_repo()
+        self.assertRaises(errors.NoColocatedBranchSupport,
+            made_control.get_branch_reference, "colo")

=== modified file 'bzrlib/tests/per_interbranch/__init__.py'
--- bzrlib/tests/per_interbranch/__init__.py	2009-05-07 05:08:46 +0000
+++ bzrlib/tests/per_interbranch/__init__.py	2010-06-20 11:18:38 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009 Canonical Ltd
+# Copyright (C) 2009, 2010 Canonical Ltd
 # -*- coding: utf-8 -*-
 #
 # This program is free software; you can redistribute it and/or modify
@@ -46,7 +46,6 @@
     TestCaseWithTransport,
     multiply_tests,
     )
-from bzrlib.transport import get_transport
 
 
 def make_scenarios(test_list):
@@ -76,11 +75,9 @@
     # test the default InterBranch between format 6 and the current
     # default format.
     for optimiser_class in InterBranch._optimisers:
-        format_from_test, format_to_test = \
-            optimiser_class._get_branch_formats_to_test()
-        if format_to_test is not None:
-            result.append((optimiser_class,
-                           format_from_test, format_to_test))
+        for format_from_test, format_to_test in \
+            optimiser_class._get_branch_formats_to_test():
+            result.append((optimiser_class, format_from_test, format_to_test))
     # if there are specific combinations we want to use, we can add them
     # here.
     return result
@@ -144,8 +141,38 @@
         return newbranch.bzrdir
 
 
+class StubWithFormat(object):
+    """A stub object used to check that convenience methods call Inter's."""
+
+    _format = object()
+
+
+class StubMatchingInter(object):
+    """An inter for tests.
+
+    This is not a subclass of InterBranch so that missing methods are caught
+    and added rather than actually trying to do something.
+    """
+
+    _uses = []
+
+    def __init__(self, source, target):
+        self.source = source
+        self.target = target
+
+    @classmethod
+    def is_compatible(klass, source, target):
+        return StubWithFormat._format in (source._format, target._format)
+
+    def copy_content_into(self, *args, **kwargs):
+        self.__class__._uses.append(
+            (self, 'copy_content_into', args, kwargs))
+
+
 def load_tests(standard_tests, module, loader):
     submod_tests = loader.loadTestsFromModuleNames([
+        'bzrlib.tests.per_interbranch.test_get',
+        'bzrlib.tests.per_interbranch.test_copy_content_into',
         'bzrlib.tests.per_interbranch.test_pull',
         'bzrlib.tests.per_interbranch.test_push',
         'bzrlib.tests.per_interbranch.test_update_revisions',

=== added file 'bzrlib/tests/per_interbranch/test_copy_content_into.py'
--- bzrlib/tests/per_interbranch/test_copy_content_into.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_interbranch/test_copy_content_into.py	2010-06-17 06:30:22 +0000
@@ -0,0 +1,47 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for bzrlib.branch.InterBranch.copy_content_into."""
+
+from bzrlib import branch
+from bzrlib.tests.per_interbranch import (
+    StubMatchingInter,
+    StubWithFormat,
+    TestCaseWithInterBranch,
+    )
+
+
+class TestCopyContentInto(TestCaseWithInterBranch):
+
+    def test_contract_convenience_method(self):
+        self.tree1 = self.make_from_branch_and_tree('tree1')
+        rev1 = self.tree1.commit('one')
+        branch2 = self.make_to_branch('tree2')
+        branch2.repository.fetch(self.tree1.branch.repository)
+        self.tree1.branch.copy_content_into(branch2, revision_id=rev1)
+
+    def test_inter_is_used(self):
+        self.tree1 = self.make_from_branch_and_tree('tree1')
+        self.addCleanup(branch.InterBranch.unregister_optimiser,
+            StubMatchingInter)
+        branch.InterBranch.register_optimiser(StubMatchingInter)
+        del StubMatchingInter._uses[:]
+        self.tree1.branch.copy_content_into(StubWithFormat(), revision_id='54')
+        self.assertLength(1, StubMatchingInter._uses)
+        use = StubMatchingInter._uses[0]
+        self.assertEqual('copy_content_into', use[1])
+        self.assertEqual('54', use[3]['revision_id'])
+        del StubMatchingInter._uses[:]

=== added file 'bzrlib/tests/per_interbranch/test_get.py'
--- bzrlib/tests/per_interbranch/test_get.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_interbranch/test_get.py	2010-06-17 06:30:22 +0000
@@ -0,0 +1,34 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for bzrlib.branch.InterBranch.get."""
+
+from bzrlib import (
+    branch,
+    )
+from bzrlib.tests.per_interbranch import (
+    TestCaseWithInterBranch,
+    )
+
+
+class TestInterBranchGet(TestCaseWithInterBranch):
+
+    def test_gets_right_inter(self):
+        self.tree1 = self.make_from_branch_and_tree('tree1')
+        branch2 = self.make_to_branch('tree2')
+        self.assertIs(branch.InterBranch.get(
+            self.tree1.branch, branch2).__class__,
+            self.interbranch_class)

=== modified file 'bzrlib/tests/per_interbranch/test_pull.py'
--- bzrlib/tests/per_interbranch/test_pull.py	2009-07-10 05:49:34 +0000
+++ bzrlib/tests/per_interbranch/test_pull.py	2010-06-17 09:23:19 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2004, 2005, 2007, 2009 Canonical Ltd
+# Copyright (C) 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -76,13 +76,13 @@
     def test_pull_raises_specific_error_on_master_connection_error(self):
         master_tree = self.make_from_branch_and_tree('master')
         checkout = master_tree.branch.create_checkout('checkout')
-        other = self.sprout_to(master_tree.branch.bzrdir, 'other').open_workingtree()
+        other = self.sprout_to(master_tree.branch.bzrdir, 'other').open_branch()
         # move the branch out of the way on disk to cause a connection
         # error.
         os.rename('master', 'master_gone')
         # try to pull, which should raise a BoundBranchConnectionFailure.
         self.assertRaises(errors.BoundBranchConnectionFailure,
-                checkout.branch.pull, other.branch)
+                checkout.branch.pull, other)
 
     def test_pull_returns_result(self):
         parent = self.make_from_branch_and_tree('parent')

=== modified file 'bzrlib/tests/per_interbranch/test_push.py'
--- bzrlib/tests/per_interbranch/test_push.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/per_interbranch/test_push.py	2010-09-07 23:32:42 +0000
@@ -19,6 +19,11 @@
 from cStringIO import StringIO
 import os
 
+from testtools.matchers import (
+    Equals,
+    MatchesAny,
+    )
+
 from bzrlib import (
     branch,
     builtins,
@@ -28,6 +33,7 @@
     errors,
     push,
     repository,
+    symbol_versioning,
     tests,
     )
 from bzrlib.branch import Branch
@@ -39,7 +45,6 @@
 from bzrlib.tests.per_interbranch import (
     TestCaseWithInterBranch,
     )
-from bzrlib.transport import get_transport
 from bzrlib.tests import test_server
 
 
@@ -64,7 +69,10 @@
         self.assertEqual(result.old_revid, 'M1')
         self.assertEqual(result.new_revid, 'P2')
         # and it can be treated as an integer for compatibility
-        self.assertEqual(int(result), 0)
+        self.assertEqual(self.applyDeprecated(
+            symbol_versioning.deprecated_in((2, 3, 0)),
+            result.__int__),
+            0)
 
     def test_push_merged_indirect(self):
         # it should be possible to do a push from one branch into another
@@ -145,7 +153,7 @@
         except (errors.IncompatibleFormat, errors.UninitializableFormat):
             # This Branch format cannot create shared repositories
             return
-        # This is a little bit trickier because make_branch_and_tree will not
+        # This is a little bit trickier because make_from_branch_and_tree will not
         # re-use a shared repository.
         try:
             a_branch = self.make_from_branch('repo/tree')
@@ -272,10 +280,14 @@
         calls_after_insert_stream = hpss_call_names[insert_stream_idx:]
         # After inserting the stream the client has no reason to query the
         # remote graph any further.
-        self.assertEqual(
-            ['Repository.insert_stream_1.19', 'Repository.insert_stream_1.19',
-             'get', 'Branch.set_last_revision_info', 'Branch.unlock'],
-            calls_after_insert_stream)
+        bzr_core_trace = Equals(
+            ['Repository.insert_stream_1.19', 'Repository.insert_stream_1.19',
+             'get', 'Branch.set_last_revision_info', 'Branch.unlock'])
+        bzr_loom_trace = Equals(
+            ['Repository.insert_stream_1.19', 'Repository.insert_stream_1.19',
+             'get', 'Branch.set_last_revision_info', 'get', 'Branch.unlock'])
+        self.assertThat(calls_after_insert_stream,
+            MatchesAny(bzr_core_trace, bzr_loom_trace))
 
     def disableOptimisticGetParentMap(self):
         # Tweak some class variables to stop remote get_parent_map calls asking

=== modified file 'bzrlib/tests/per_interbranch/test_update_revisions.py'
--- bzrlib/tests/per_interbranch/test_update_revisions.py	2009-05-07 05:08:46 +0000
+++ bzrlib/tests/per_interbranch/test_update_revisions.py	2010-06-17 09:23:19 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009 Canonical Ltd
+# Copyright (C) 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
 
     def setUp(self):
         super(TestUpdateRevisions, self).setUp()
-        self.tree1 = self.make_branch_and_tree('tree1')
+        self.tree1 = self.make_from_branch_and_tree('tree1')
         rev1 = self.tree1.commit('one')
         branch2 = self.make_to_branch('tree2')
         branch2.repository.fetch(self.tree1.branch.repository)

=== modified file 'bzrlib/tests/per_interrepository/__init__.py'
--- bzrlib/tests/per_interrepository/__init__.py	2009-08-14 01:07:44 +0000
+++ bzrlib/tests/per_interrepository/__init__.py	2010-09-17 04:35:23 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 # Authors: Robert Collins <robert.collins@canonical.com>
 # -*- coding: utf-8 -*-
 #
@@ -26,6 +26,7 @@
 """
 
 
+from bzrlib import transport
 from bzrlib.errors import (
     FileExists,
     UninitializableFormat,
@@ -38,8 +39,7 @@
                           default_transport,
                           multiply_tests,
                           )
-from bzrlib.tests.per_bzrdir.test_bzrdir import TestCaseWithBzrDir
-from bzrlib.transport import get_transport
+from bzrlib.tests.per_controldir.test_controldir import TestCaseWithControlDir
 
 
 def make_scenarios(transport_server, transport_readonly_server, formats):
@@ -49,7 +49,7 @@
         (label, repository_format, repository_format_to).
     """
     result = []
-    for label, repository_format, repository_format_to in formats:
+    for label, repository_format, repository_format_to, extra_setup in formats:
         id = '%s,%s,%s' % (label, repository_format.__class__.__name__,
                            repository_format_to.__class__.__name__)
         scenario = (id,
@@ -57,6 +57,7 @@
              "transport_readonly_server": transport_readonly_server,
              "repository_format": repository_format,
              "repository_format_to": repository_format_to,
+             "extra_setup": extra_setup,
              })
         result.append(scenario)
     return result
@@ -71,8 +72,8 @@
         weaverepo,
         )
     result = []
-    def add_combo(label, from_format, to_format):
-        result.append((label, from_format, to_format))
+    def add_combo(label, from_format, to_format, extra_setup=None):
+        result.append((label, from_format, to_format, extra_setup))
     # test the default InterRepository between format 6 and the current
     # default format.
     # XXX: robertc 20060220 reinstate this when there are two supported
@@ -89,6 +90,9 @@
     # XXX: although we attach InterRepository class names to these scenarios,
     # there's nothing asserting that these labels correspond to what is
     # actually used.
+    def force_known_graph(testcase):
+        from bzrlib.fetch import Inter1and2Helper
+        testcase.overrideAttr(Inter1and2Helper, 'known_graph_threshold', -1)
     add_combo('InterRepository',
               weaverepo.RepositoryFormat5(),
               knitrepo.RepositoryFormatKnit3())
@@ -113,6 +117,11 @@
     add_combo('InterDifferingSerializer',
               pack_repo.RepositoryFormatKnitPack1(),
               pack_repo.RepositoryFormatKnitPack6RichRoot())
+    add_combo('InterDifferingSerializer+get_known_graph_ancestry',
+              pack_repo.RepositoryFormatKnitPack1(),
+              pack_repo.RepositoryFormatKnitPack6RichRoot(),
+              force_known_graph,
+              )
     add_combo('InterDifferingSerializer',
               pack_repo.RepositoryFormatKnitPack6RichRoot(),
               groupcompress_repo.RepositoryFormat2a())
@@ -128,10 +137,12 @@
     return result
 
 
-class TestCaseWithInterRepository(TestCaseWithBzrDir):
+class TestCaseWithInterRepository(TestCaseWithControlDir):
 
     def setUp(self):
         super(TestCaseWithInterRepository, self).setUp()
+        if self.extra_setup:
+            self.extra_setup(self)
 
     def make_branch(self, relpath, format=None):
         repo = self.make_repository(relpath, format=format)
@@ -143,7 +154,7 @@
             segments = url.split('/')
             if segments and segments[-1] not in ('', '.'):
                 parent = '/'.join(segments[:-1])
-                t = get_transport(parent)
+                t = transport.get_transport(parent)
                 try:
                     t.mkdir(segments[-1])
                 except FileExists:

=== modified file 'bzrlib/tests/per_intertree/__init__.py'
--- bzrlib/tests/per_intertree/__init__.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/per_intertree/__init__.py	2010-06-20 11:18:38 +0000
@@ -28,7 +28,6 @@
     revisiontree,
     tests,
     )
-from bzrlib.transport import get_transport
 from bzrlib.transform import TransformPreview
 from bzrlib.tests import (
     default_transport,

=== modified file 'bzrlib/tests/per_lock/test_lock.py'
--- bzrlib/tests/per_lock/test_lock.py	2009-07-07 09:00:59 +0000
+++ bzrlib/tests/per_lock/test_lock.py	2010-09-29 22:03:03 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
     osutils,
     )
 
-from bzrlib.tests import UnicodeFilenameFeature
+from bzrlib.tests import (features, UnicodeFilenameFeature)
 from bzrlib.tests.per_lock import TestCaseWithLock
 
 
@@ -63,6 +63,7 @@
 
         But we shouldn't be able to take a write lock.
         """
+        self.requireFeature(features.not_running_as_root)
         osutils.make_readonly('a-file')
         # Make sure the file is read-only (on all platforms)
         self.assertRaises(IOError, open, 'a-file', 'rb+')

=== modified file 'bzrlib/tests/per_pack_repository.py'
--- bzrlib/tests/per_pack_repository.py	2010-03-18 05:24:02 +0000
+++ bzrlib/tests/per_pack_repository.py	2010-05-06 23:41:35 +0000
@@ -720,6 +720,16 @@
         self.assertEqual(self.format_supports_external_lookups,
             repo._format.supports_external_lookups)
 
+    def _lock_write(self, write_lockable):
+        """Lock write_lockable, add a cleanup and return the result.
+        
+        :param write_lockable: An object with a lock_write method.
+        :return: The result of write_lockable.lock_write().
+        """
+        result = write_lockable.lock_write()
+        self.addCleanup(result.unlock)
+        return result
+
     def test_abort_write_group_does_not_raise_when_suppressed(self):
         """Similar to per_repository.test_write_group's test of the same name.
 
@@ -727,8 +737,7 @@
         """
         self.vfs_transport_factory = memory.MemoryServer
         repo = self.make_repository('repo', format=self.get_format())
-        token = repo.lock_write()
-        self.addCleanup(repo.unlock)
+        token = self._lock_write(repo).repository_token
         repo.start_write_group()
         # Damage the repository on the filesystem
         self.get_transport('').rename('repo', 'foo')
@@ -744,8 +753,7 @@
     def test_abort_write_group_does_raise_when_not_suppressed(self):
         self.vfs_transport_factory = memory.MemoryServer
         repo = self.make_repository('repo', format=self.get_format())
-        token = repo.lock_write()
-        self.addCleanup(repo.unlock)
+        token = self._lock_write(repo).repository_token
         repo.start_write_group()
         # Damage the repository on the filesystem
         self.get_transport('').rename('repo', 'foo')
@@ -757,8 +765,7 @@
     def test_suspend_write_group(self):
         self.vfs_transport_factory = memory.MemoryServer
         repo = self.make_repository('repo', format=self.get_format())
-        token = repo.lock_write()
-        self.addCleanup(repo.unlock)
+        token = self._lock_write(repo).repository_token
         repo.start_write_group()
         repo.texts.add_lines(('file-id', 'revid'), (), ['lines'])
         wg_tokens = repo.suspend_write_group()
@@ -779,8 +786,7 @@
         repo = self.make_repository('repo', format=self.get_format())
         if repo.chk_bytes is None:
             raise TestNotApplicable('no chk_bytes for this repository')
-        token = repo.lock_write()
-        self.addCleanup(repo.unlock)
+        token = self._lock_write(repo).repository_token
         repo.start_write_group()
         text = 'a bit of text\n'
         key = ('sha1:' + osutils.sha_string(text),)
@@ -801,8 +807,7 @@
         # Create a repo, start a write group, insert some data, suspend.
         self.vfs_transport_factory = memory.MemoryServer
         repo = self.make_repository('repo', format=self.get_format())
-        token = repo.lock_write()
-        self.addCleanup(repo.unlock)
+        token = self._lock_write(repo).repository_token
         repo.start_write_group()
         text_key = ('file-id', 'revid')
         repo.texts.add_lines(text_key, (), ['lines'])
@@ -822,8 +827,7 @@
     def test_commit_resumed_write_group(self):
         self.vfs_transport_factory = memory.MemoryServer
         repo = self.make_repository('repo', format=self.get_format())
-        token = repo.lock_write()
-        self.addCleanup(repo.unlock)
+        token = self._lock_write(repo).repository_token
         repo.start_write_group()
         text_key = ('file-id', 'revid')
         repo.texts.add_lines(text_key, (), ['lines'])
@@ -851,16 +855,14 @@
         self.vfs_transport_factory = memory.MemoryServer
         # Make a repository with a suspended write group
         repo = self.make_repository('repo', format=self.get_format())
-        token = repo.lock_write()
-        self.addCleanup(repo.unlock)
+        token = self._lock_write(repo).repository_token
         repo.start_write_group()
         text_key = ('file-id', 'revid')
         repo.texts.add_lines(text_key, (), ['lines'])
         wg_tokens = repo.suspend_write_group()
         # Make a new repository
         new_repo = self.make_repository('new_repo', format=self.get_format())
-        token = new_repo.lock_write()
-        self.addCleanup(new_repo.unlock)
+        token = self._lock_write(new_repo).repository_token
         hacked_wg_token = (
             '../../../../repo/.bzr/repository/upload/' + wg_tokens[0])
         self.assertRaises(

=== modified file 'bzrlib/tests/per_repository/__init__.py'
--- bzrlib/tests/per_repository/__init__.py	2010-02-11 09:21:45 +0000
+++ bzrlib/tests/per_repository/__init__.py	2010-08-27 17:53:08 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007, 2008, 2009 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 # Authors: Robert Collins <robert.collins@canonical.com>
 #          and others
 #
@@ -38,7 +38,7 @@
     multiply_tests,
     test_server,
     )
-from bzrlib.tests.per_bzrdir.test_bzrdir import TestCaseWithBzrDir
+from bzrlib.tests.per_controldir.test_controldir import TestCaseWithControlDir
 from bzrlib.transport import memory
 
 
@@ -97,7 +97,7 @@
     return format_scenarios
 
 
-class TestCaseWithRepository(TestCaseWithBzrDir):
+class TestCaseWithRepository(TestCaseWithControlDir):
 
     def make_repository(self, relpath, shared=False, format=None):
         if format is None:

=== modified file 'bzrlib/tests/per_repository/test_commit_builder.py'
--- bzrlib/tests/per_repository/test_commit_builder.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/per_repository/test_commit_builder.py	2010-08-10 19:17:49 +0000
@@ -193,6 +193,23 @@
         self.assertEqual(revision_id,
             tree.branch.repository.get_inventory(revision_id).revision_id)
 
+    def test_commit_without_root_errors(self):
+        tree = self.make_branch_and_tree(".")
+        tree.lock_write()
+        try:
+            builder = tree.branch.get_commit_builder([])
+            def do_commit():
+                try:
+                    list(builder.record_iter_changes(
+                        tree, tree.last_revision(), []))
+                    builder.finish_inventory()
+                except:
+                    builder.abort()
+                    raise
+            self.assertRaises(errors.RootMissing, do_commit)
+        finally:
+            tree.unlock()
+
     def test_commit_without_root_or_record_iter_changes_errors(self):
         tree = self.make_branch_and_tree(".")
         tree.lock_write()
@@ -1286,3 +1303,27 @@
         self.addCleanup(repo_local.unlock)
         self.assertRaises(errors.BzrError, repo_local.get_commit_builder,
             branch, [], branch.get_config())
+
+    def test_committer_no_username(self):
+        # Ensure that when no username is available but a committer is
+        # supplied, commit works.
+        del os.environ['EMAIL']
+        tree = self.make_branch_and_tree(".")
+        tree.lock_write()
+        try:
+            # Make sure no username is available.
+            self.assertRaises(errors.NoWhoami, tree.branch.get_commit_builder,
+                              [])
+            builder = tree.branch.get_commit_builder(
+                [], committer='me@example.com')
+            try:
+                list(builder.record_iter_changes(tree, tree.last_revision(),
+                    tree.iter_changes(tree.basis_tree())))
+                builder.finish_inventory()
+            except:
+                builder.abort()
+                raise
+            repo = tree.branch.repository
+            repo.commit_write_group()
+        finally:
+            tree.unlock()

=== modified file 'bzrlib/tests/per_repository/test_fetch.py'
--- bzrlib/tests/per_repository/test_fetch.py	2009-08-05 02:12:22 +0000
+++ bzrlib/tests/per_repository/test_fetch.py	2010-06-20 11:18:38 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,7 +28,6 @@
 from bzrlib.inventory import ROOT_ID
 from bzrlib.tests import TestSkipped
 from bzrlib.tests.per_repository import TestCaseWithRepository
-from bzrlib.transport import get_transport
 
 
 class TestFetchSameRepository(TestCaseWithRepository):

=== modified file 'bzrlib/tests/per_repository/test_has_same_location.py'
--- bzrlib/tests/per_repository/test_has_same_location.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/per_repository/test_has_same_location.py	2010-06-20 11:18:38 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007, 2008 Canonical Ltd
+# Copyright (C) 2007-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,12 +16,14 @@
 
 """Tests for implementations of Repository.has_same_location."""
 
-from bzrlib import bzrdir
+from bzrlib import (
+    bzrdir,
+    transport,
+    )
 from bzrlib.tests import (
     TestNotApplicable,
     )
 from bzrlib.tests.per_repository import TestCaseWithRepository
-from bzrlib.transport import get_transport
 
 
 class TestHasSameLocation(TestCaseWithRepository):
@@ -112,7 +114,8 @@
         if repo._format == other_repo._format:
             # We're testing the default format!  So we have to use a non-default
             # format for other_repo.
-            get_transport(self.get_vfs_only_url()).delete_tree('other')
+            transport.get_transport(self.get_vfs_only_url()
+                                    ).delete_tree('other')
             other_repo = self.make_repository('other', format='metaweave')
         # Make sure the other_repo is not a RemoteRepository.
         other_bzrdir = bzrdir.BzrDir.open(self.get_vfs_only_url('other'))

=== modified file 'bzrlib/tests/per_repository/test_reconcile.py'
--- bzrlib/tests/per_repository/test_reconcile.py	2009-09-08 11:32:53 +0000
+++ bzrlib/tests/per_repository/test_reconcile.py	2010-06-20 11:18:38 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,10 @@
 
 
 import bzrlib
-import bzrlib.errors as errors
+from bzrlib import (
+    errors,
+    transport,
+    )
 from bzrlib.inventory import Inventory
 from bzrlib.reconcile import reconcile, Reconciler
 from bzrlib.repofmt.knitrepo import RepositoryFormatKnit
@@ -30,7 +33,6 @@
 from bzrlib.tests.per_repository import (
     TestCaseWithRepository,
     )
-from bzrlib.transport import get_transport
 from bzrlib.uncommit import uncommit
 
 
@@ -59,7 +61,7 @@
     def setUp(self):
         super(TestsNeedingReweave, self).setUp()
 
-        t = get_transport(self.get_url())
+        t = transport.get_transport(self.get_url())
         # an empty inventory with no revision for testing with.
         repo = self.make_repository('inventory_without_revision')
         repo.lock_write()
@@ -310,7 +312,7 @@
         self.reduceLockdirTimeout()
         super(TestReconcileWithIncorrectRevisionCache, self).setUp()
 
-        t = get_transport(self.get_url())
+        t = transport.get_transport(self.get_url())
         # we need a revision with two parents in the wrong order
         # which should trigger reinsertion.
         # and another with the first one correct but the other two not
@@ -379,7 +381,7 @@
 
     def test_reconcile_wrong_order(self):
         # a wrong order in primary parents is optionally correctable
-        t = get_transport(self.get_url()).clone('wrong-first-parent')
+        t = transport.get_transport(self.get_url()).clone('wrong-first-parent')
         d = bzrlib.bzrdir.BzrDir.open_from_transport(t)
         repo = d.open_repository()
         repo.lock_read()
@@ -408,7 +410,8 @@
 
     def test_reconcile_wrong_order_secondary_inventory(self):
         # a wrong order in the parents for inventories is ignored.
-        t = get_transport(self.get_url()).clone('reversed-secondary-parents')
+        t = transport.get_transport(self.get_url()
+                                    ).clone('reversed-secondary-parents')
         d = bzrlib.bzrdir.BzrDir.open_from_transport(t)
         repo = d.open_repository()
         self.checkUnreconciled(d, repo.reconcile())

=== modified file 'bzrlib/tests/per_repository/test_refresh_data.py'
--- bzrlib/tests/per_repository/test_refresh_data.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/per_repository/test_refresh_data.py	2010-05-11 14:14:55 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009 Canonical Ltd
+# Copyright (C) 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 
 from bzrlib import (
     errors,
-    remote,
+    repository,
     )
 from bzrlib.tests import TestSkipped
 from bzrlib.tests.per_repository import TestCaseWithRepository
@@ -45,22 +45,34 @@
         self.addCleanup(repo.unlock)
         repo.refresh_data()
 
-    def test_refresh_data_in_write_group_errors(self):
+    def test_refresh_data_in_write_group(self):
+        # refresh_data may either succeed or raise IsInWriteGroupError during a
+        # write group.
         repo = self.make_repository('.')
         repo.lock_write()
         self.addCleanup(repo.unlock)
         repo.start_write_group()
         self.addCleanup(repo.abort_write_group)
-        # No flow control anticipated, BzrError is enough
-        self.assertRaises(errors.BzrError, repo.refresh_data)
+        try:
+            repo.refresh_data()
+        except repository.IsInWriteGroupError:
+            # This is ok.
+            pass
+        else:
+            # This is ok too.
+            pass
 
-    def test_refresh_data_after_fetch_new_data_visible(self):
-        source = self.make_branch_and_tree('source')
-        revid = source.commit('foo')
-        repo = self.make_repository('target')
-        token = repo.lock_write()
-        self.addCleanup(repo.unlock)
+    def fetch_new_revision_into_concurrent_instance(self, repo, token):
+        """Create a new revision (revid 'new-rev') and fetch it into a
+        concurrent instance of repo.
+        """
+        source = self.make_branch_and_memory_tree('source')
+        source.lock_write()
+        self.addCleanup(source.unlock)
+        source.add([''], ['root-id'])
+        revid = source.commit('foo', rev_id='new-rev')
         # Force data reading on weaves/knits
+        repo.all_revision_ids()
         repo.revisions.keys()
         repo.inventories.keys()
         # server repo is the instance a smart server might hold for this
@@ -75,5 +87,34 @@
             server_repo.fetch(source.branch.repository, revid)
         finally:
             server_repo.unlock()
+
+    def test_refresh_data_after_fetch_new_data_visible(self):
+        repo = self.make_repository('target')
+        token = repo.lock_write().repository_token
+        self.addCleanup(repo.unlock)
+        self.fetch_new_revision_into_concurrent_instance(repo, token)
         repo.refresh_data()
-        self.assertNotEqual({}, repo.get_graph().get_parent_map([revid]))
+        self.assertNotEqual({}, repo.get_graph().get_parent_map(['new-rev']))
+
+    def test_refresh_data_after_fetch_new_data_visible_in_write_group(self):
+        tree = self.make_branch_and_memory_tree('target')
+        tree.lock_write()
+        self.addCleanup(tree.unlock)
+        tree.add([''], ['root-id'])
+        tree.commit('foo', rev_id='commit-in-target')
+        repo = tree.branch.repository
+        token = repo.lock_write().repository_token
+        self.addCleanup(repo.unlock)
+        repo.start_write_group()
+        self.addCleanup(repo.abort_write_group)
+        self.fetch_new_revision_into_concurrent_instance(repo, token)
+        # Call refresh_data.  It either fails with IsInWriteGroupError, or it
+        # succeeds and the new revisions are visible.
+        try:
+            repo.refresh_data()
+        except repository.IsInWriteGroupError:
+            pass
+        else:
+            self.assertEqual(
+                ['commit-in-target', 'new-rev'],
+                sorted(repo.all_revision_ids()))

=== modified file 'bzrlib/tests/per_repository/test_repository.py'
--- bzrlib/tests/per_repository/test_repository.py	2010-04-21 04:53:33 +0000
+++ bzrlib/tests/per_repository/test_repository.py	2010-05-06 23:41:35 +0000
@@ -47,6 +47,7 @@
     per_repository,
     test_server,
     )
+from bzrlib.tests.matchers import *
 
 
 class TestRepositoryMakeBranchAndTree(per_repository.TestCaseWithRepository):
@@ -1087,7 +1088,7 @@
         repo = self.make_repository('r')
         # Lock the repository, then use leave_lock_in_place so that when we
         # unlock the repository the lock is still held on disk.
-        token = repo.lock_write()
+        token = repo.lock_write().repository_token
         try:
             if token is None:
                 # This test does not apply, because this repository refuses lock
@@ -1107,7 +1108,7 @@
     def test_dont_leave_lock_in_place(self):
         repo = self.make_repository('r')
         # Create a lock on disk.
-        token = repo.lock_write()
+        token = repo.lock_write().repository_token
         try:
             if token is None:
                 # This test does not apply, because this repository refuses lock
@@ -1141,6 +1142,14 @@
         repo.lock_read()
         repo.unlock()
 
+    def test_lock_read_returns_unlockable(self):
+        repo = self.make_repository('r')
+        self.assertThat(repo.lock_read, ReturnsUnlockable(repo))
+
+    def test_lock_write_returns_unlockable(self):
+        repo = self.make_repository('r')
+        self.assertThat(repo.lock_write, ReturnsUnlockable(repo))
+
 
 class TestCaseWithComplexRepository(per_repository.TestCaseWithRepository):
 

=== modified file 'bzrlib/tests/per_transport.py'
--- bzrlib/tests/per_transport.py	2010-03-05 05:30:19 +0000
+++ bzrlib/tests/per_transport.py	2010-08-24 13:03:18 +0000
@@ -251,7 +251,6 @@
 
     def test_get_bytes_unknown_file(self):
         t = self.get_transport()
-
         self.assertRaises(NoSuchFile, t.get_bytes, 'c')
 
     def test_get_with_open_write_stream_sees_all_content(self):
@@ -1122,7 +1121,8 @@
             self.failUnless(t.has(link_name))
 
             st = t.stat(link_name)
-            self.failUnless(S_ISLNK(st.st_mode))
+            self.failUnless(S_ISLNK(st.st_mode),
+                "expected symlink, got mode %o" % st.st_mode)
         except TransportNotPossible:
             raise TestSkipped("Transport %s does not support symlinks." %
                               self._server.__class__)
@@ -1265,7 +1265,7 @@
         self.assertIs(t._get_connection(), c._get_connection())
 
         # Temporary failure, we need to create a new dummy connection
-        new_connection = object()
+        new_connection = None
         t._set_connection(new_connection)
         # Check that both transports use the same connection
         self.assertIs(new_connection, t._get_connection())
@@ -1765,3 +1765,15 @@
         # also raise a special error
         self.assertListRaises((errors.ShortReadvError, errors.InvalidRange),
                               transport.readv, 'a', [(12,2)])
+
+    def test_stat_symlink(self):
+        # if a transport points directly to a symlink (and supports symlinks
+        # at all) you can tell this.  helps with bug 32669.
+        t = self.get_transport()
+        try:
+            t.symlink('target', 'link')
+        except TransportNotPossible:
+            raise TestSkipped("symlinks not supported")
+        t2 = t.clone('link')
+        st = t2.stat('')
+        self.assertTrue(stat.S_ISLNK(st.st_mode))

=== modified file 'bzrlib/tests/per_tree/__init__.py'
--- bzrlib/tests/per_tree/__init__.py	2010-02-10 17:52:08 +0000
+++ bzrlib/tests/per_tree/__init__.py	2010-08-21 16:06:24 +0000
@@ -27,22 +27,18 @@
 
 from bzrlib import (
     errors,
-    osutils,
-    progress,
     tests,
     transform,
     )
-from bzrlib.tests.per_bzrdir.test_bzrdir import TestCaseWithBzrDir
+from bzrlib.tests.per_controldir.test_controldir import TestCaseWithControlDir
 from bzrlib.tests.per_workingtree import (
     make_scenarios as wt_make_scenarios,
     make_scenario as wt_make_scenario,
     )
-from bzrlib.revision import NULL_REVISION
 from bzrlib.revisiontree import RevisionTree
 from bzrlib.transform import TransformPreview
 from bzrlib.workingtree import (
     WorkingTreeFormat,
-    WorkingTreeFormat3,
     _legacy_formats,
     )
 from bzrlib.workingtree_4 import (
@@ -98,7 +94,7 @@
         self.assertIsInstance(tree, RevisionTree)
 
 
-class TestCaseWithTree(TestCaseWithBzrDir):
+class TestCaseWithTree(TestCaseWithControlDir):
 
     def make_branch_and_tree(self, relpath):
         made_control = self.make_bzrdir(relpath, format=
@@ -386,6 +382,7 @@
         'inv',
         'iter_search_rules',
         'list_files',
+        'locking',
         'path_content_summary',
         'revision_tree',
         'test_trees',

=== modified file 'bzrlib/tests/per_tree/test_inv.py'
--- bzrlib/tests/per_tree/test_inv.py	2010-02-16 16:08:40 +0000
+++ bzrlib/tests/per_tree/test_inv.py	2010-05-20 18:23:17 +0000
@@ -170,7 +170,7 @@
                          work_tree.get_canonical_inventory_path('Dir/None'))
 
     def test_canonical_tree_name_mismatch(self):
-        # see <https://bugs.edge.launchpad.net/bzr/+bug/368931>
+        # see <https://bugs.launchpad.net/bzr/+bug/368931>
         # some of the trees we want to use can only exist on a disk, not in
         # memory - therefore we can only test this if the filesystem is
         # case-sensitive.

=== added file 'bzrlib/tests/per_tree/test_locking.py'
--- bzrlib/tests/per_tree/test_locking.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_tree/test_locking.py	2010-05-06 23:41:35 +0000
@@ -0,0 +1,28 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Test that all trees support Tree.lock_read()"""
+
+from bzrlib.tests.matchers import *
+from bzrlib.tests.per_tree import TestCaseWithTree
+
+
+class TestLocking(TestCaseWithTree):
+
+    def test_lock_read(self):
+        work_tree = self.make_branch_and_tree('wt')
+        tree = self.workingtree_to_test_tree(work_tree)
+        self.assertThat(tree.lock_read, ReturnsUnlockable(tree))

=== modified file 'bzrlib/tests/per_uifactory/__init__.py'
--- bzrlib/tests/per_uifactory/__init__.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/per_uifactory/__init__.py	2010-09-09 07:49:00 +0000
@@ -62,6 +62,18 @@
         self.factory.be_quiet(False)
         self.assertEquals(False, self.factory.is_quiet())
 
+    def test_confirm_action(self):
+        # confirm_action should be answered by every ui factory; even
+        # noninteractive ones should have a reasonable default
+        self._load_responses([True])
+        result = self.factory.confirm_action(
+            'Break a lock?',
+            'bzr.lock.break.confirm',
+            {})
+        # will be true either because we read it from the input or because
+        # that's the default
+        self.assertEquals(result, True)
+
     def test_note(self):
         self.factory.note("a note to the user")
         self._check_note("a note to the user")
@@ -150,6 +162,11 @@
         # Without a TTY, we shouldn't display anything
         self.assertEqual('', self.stderr.getvalue())
 
+    def _load_responses(self, responses):
+        self.factory.stdin.seek(0)
+        self.factory.stdin.writelines([(r and "y\n" or "n\n") for r in responses])
+        self.factory.stdin.seek(0)
+
 
 class TestTTYTextUIFactory(TestTextUIFactory):
 
@@ -222,6 +239,9 @@
     def _check_log_transport_activity_display_no_bytes(self):
         pass
 
+    def _load_responses(self, responses):
+        pass
+
 
 class TestCannedInputUIFactory(tests.TestCase, UIFactoryTestMixin):
     # discards output, reads input from variables
@@ -250,3 +270,6 @@
 
     def _check_log_transport_activity_display_no_bytes(self):
         pass
+
+    def _load_responses(self, responses):
+        self.factory.responses.extend(responses)

=== modified file 'bzrlib/tests/per_versionedfile.py'
--- bzrlib/tests/per_versionedfile.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/per_versionedfile.py	2010-06-20 11:18:38 +0000
@@ -31,6 +31,7 @@
     knit as _mod_knit,
     osutils,
     progress,
+    transport,
     ui,
     )
 from bzrlib.errors import (
@@ -56,7 +57,6 @@
     )
 from bzrlib.tests.http_utils import TestCaseWithWebserver
 from bzrlib.trace import mutter
-from bzrlib.transport import get_transport
 from bzrlib.transport.memory import MemoryTransport
 from bzrlib.tsort import topo_sort
 from bzrlib.tuned_gzip import GzipFile
@@ -849,10 +849,10 @@
         self.assertEquals(('references_ghost', 'line_c\n'), origins[2])
 
     def test_readonly_mode(self):
-        transport = get_transport(self.get_url('.'))
+        t = transport.get_transport(self.get_url('.'))
         factory = self.get_factory()
-        vf = factory('id', transport, 0777, create=True, access_mode='w')
-        vf = factory('id', transport, access_mode='r')
+        vf = factory('id', t, 0777, create=True, access_mode='w')
+        vf = factory('id', t, access_mode='r')
         self.assertRaises(errors.ReadOnlyError, vf.add_lines, 'base', [], [])
         self.assertRaises(errors.ReadOnlyError,
                           vf.add_lines_with_ghosts,
@@ -880,12 +880,14 @@
 class TestWeave(TestCaseWithMemoryTransport, VersionedFileTestMixIn):
 
     def get_file(self, name='foo'):
-        return WeaveFile(name, get_transport(self.get_url('.')), create=True,
-            get_scope=self.get_transaction)
+        return WeaveFile(name, transport.get_transport(self.get_url('.')),
+                         create=True,
+                         get_scope=self.get_transaction)
 
     def get_file_corrupted_text(self):
-        w = WeaveFile('foo', get_transport(self.get_url('.')), create=True,
-            get_scope=self.get_transaction)
+        w = WeaveFile('foo', transport.get_transport(self.get_url('.')),
+                      create=True,
+                      get_scope=self.get_transaction)
         w.add_lines('v1', [], ['hello\n'])
         w.add_lines('v2', ['v1'], ['hello\n', 'there\n'])
 
@@ -919,14 +921,15 @@
         return w
 
     def reopen_file(self, name='foo', create=False):
-        return WeaveFile(name, get_transport(self.get_url('.')), create=create,
-            get_scope=self.get_transaction)
+        return WeaveFile(name, transport.get_transport(self.get_url('.')),
+                         create=create,
+                         get_scope=self.get_transaction)
 
     def test_no_implicit_create(self):
         self.assertRaises(errors.NoSuchFile,
                           WeaveFile,
                           'foo',
-                          get_transport(self.get_url('.')),
+                          transport.get_transport(self.get_url('.')),
                           get_scope=self.get_transaction)
 
     def get_factory(self):
@@ -999,13 +1002,20 @@
         # we should be able to read from http with a versioned file.
         vf = self.get_file()
         # try an empty file access
-        readonly_vf = self.get_factory()('foo', get_transport(self.get_readonly_url('.')))
+        readonly_vf = self.get_factory()('foo', transport.get_transport(
+                self.get_readonly_url('.')))
         self.assertEqual([], readonly_vf.versions())
+
+    def test_readonly_http_works_with_feeling(self):
+        # we should be able to read from http with a versioned file.
+        vf = self.get_file()
         # now with feeling.
         vf.add_lines('1', [], ['a\n'])
         vf.add_lines('2', ['1'], ['b\n', 'a\n'])
-        readonly_vf = self.get_factory()('foo', get_transport(self.get_readonly_url('.')))
+        readonly_vf = self.get_factory()('foo', transport.get_transport(
+                self.get_readonly_url('.')))
         self.assertEqual(['1', '2'], vf.versions())
+        self.assertEqual(['1', '2'], readonly_vf.versions())
         for version in readonly_vf.versions():
             readonly_vf.get_lines(version)
 
@@ -1013,8 +1023,9 @@
 class TestWeaveHTTP(TestCaseWithWebserver, TestReadonlyHttpMixin):
 
     def get_file(self):
-        return WeaveFile('foo', get_transport(self.get_url('.')), create=True,
-            get_scope=self.get_transaction)
+        return WeaveFile('foo', transport.get_transport(self.get_url('.')),
+                         create=True,
+                         get_scope=self.get_transaction)
 
     def get_factory(self):
         return WeaveFile
@@ -1264,7 +1275,8 @@
 class TestWeaveMerge(TestCaseWithMemoryTransport, MergeCasesMixin):
 
     def get_file(self, name='foo'):
-        return WeaveFile(name, get_transport(self.get_url('.')), create=True)
+        return WeaveFile(name, transport.get_transport(self.get_url('.')),
+                         create=True)
 
     def log_contents(self, w):
         self.log('weave is:')

=== modified file 'bzrlib/tests/per_workingtree/__init__.py'
--- bzrlib/tests/per_workingtree/__init__.py	2010-04-01 12:53:53 +0000
+++ bzrlib/tests/per_workingtree/__init__.py	2010-08-21 16:06:24 +0000
@@ -27,7 +27,7 @@
     tests,
     workingtree,
     )
-from bzrlib.tests import per_bzrdir
+from bzrlib.tests import per_controldir
 
 
 def make_scenarios(transport_server, transport_readonly_server, formats):
@@ -50,7 +50,7 @@
         }
 
 
-class TestCaseWithWorkingTree(per_bzrdir.TestCaseWithBzrDir):
+class TestCaseWithWorkingTree(per_controldir.TestCaseWithControlDir):
 
     def make_branch_and_tree(self, relpath, format=None):
         made_control = self.make_bzrdir(relpath, format=format)
@@ -101,6 +101,7 @@
         'revision_tree',
         'set_root_id',
         'smart_add',
+        'symlinks',
         'uncommit',
         'unversion',
         'views',

=== modified file 'bzrlib/tests/per_workingtree/test_commit.py'
--- bzrlib/tests/per_workingtree/test_commit.py	2010-04-01 12:53:53 +0000
+++ bzrlib/tests/per_workingtree/test_commit.py	2010-09-14 08:14:22 +0000
@@ -38,55 +38,7 @@
 from bzrlib.trace import mutter
 from bzrlib.workingtree import (TreeEntry, TreeDirectory, TreeFile, TreeLink,
                                 WorkingTree)
-
-
-class CapturingUIFactory(ui.UIFactory):
-    """A UI Factory for testing - capture the updates made through it."""
-
-    def __init__(self):
-        super(CapturingUIFactory, self).__init__()
-        self._calls = []
-        self.depth = 0
-
-    def clear(self):
-        """See progress.ProgressTask.clear()."""
-
-    def clear_term(self):
-        """See progress.ProgressTask.clear_term()."""
-
-    def finished(self):
-        """See progress.ProgressTask.finished()."""
-        self.depth -= 1
-
-    def note(self, fmt_string, *args, **kwargs):
-        """See progress.ProgressTask.note()."""
-
-    def progress_bar(self):
-        return self
-
-    def nested_progress_bar(self):
-        self.depth += 1
-        return self
-
-    def update(self, message, count=None, total=None):
-        """See progress.ProgressTask.update()."""
-        if self.depth == 1:
-            self._calls.append(("update", count, total, message))
-
-
-class TestCapturingUI(TestCase):
-
-    def test_nested_ignore_depth_beyond_one(self):
-        # we only want to capture the first level out progress, not
-        # want sub-components might do. So we have nested bars ignored.
-        factory = CapturingUIFactory()
-        pb1 = factory.nested_progress_bar()
-        pb1.update('foo', 0, 1)
-        pb2 = factory.nested_progress_bar()
-        pb2.update('foo', 0, 1)
-        pb2.finished()
-        pb1.finished()
-        self.assertEqual([("update", 0, 1, 'foo')], factory._calls)
+from bzrlib.tests.testui import ProgressRecordingUIFactory
 
 
 class TestCommit(TestCaseWithWorkingTree):
@@ -507,7 +459,7 @@
 
     def setUp(self):
         super(TestCommitProgress, self).setUp()
-        ui.ui_factory = CapturingUIFactory()
+        ui.ui_factory = ProgressRecordingUIFactory()
 
     def test_commit_progress_steps(self):
         # during commit we one progress update for every entry in the
@@ -526,7 +478,7 @@
         f.close()
         # set a progress bar that captures the calls so we can see what is
         # emitted
-        factory = CapturingUIFactory()
+        factory = ProgressRecordingUIFactory()
         ui.ui_factory = factory
         # TODO RBC 20060421 it would be nice to merge the reporter output
         # into the factory for this test - just make the test ui factory
@@ -547,7 +499,7 @@
         tree = self.make_branch_and_tree('.')
         # set a progress bar that captures the calls so we can see what is
         # emitted
-        factory = CapturingUIFactory()
+        factory = ProgressRecordingUIFactory()
         ui.ui_factory = factory
         def a_hook(_, _2, _3, _4, _5, _6):
             pass
@@ -570,7 +522,7 @@
         tree = self.make_branch_and_tree('.')
         # set a progress bar that captures the calls so we can see what is
         # emitted
-        factory = CapturingUIFactory()
+        factory = ProgressRecordingUIFactory()
         ui.ui_factory = factory
         def a_hook(_, _2, _3, _4, _5, _6, _7, _8):
             pass

=== modified file 'bzrlib/tests/per_workingtree/test_content_filters.py'
--- bzrlib/tests/per_workingtree/test_content_filters.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/per_workingtree/test_content_filters.py	2010-05-20 18:23:17 +0000
@@ -223,7 +223,7 @@
 
     def test_path_content_summary(self):
         """path_content_summary should always talk about the canonical form."""
-        # see https://bugs.edge.launchpad.net/bzr/+bug/415508
+        # see https://bugs.launchpad.net/bzr/+bug/415508
         #
         # set up a tree where the canonical form has a string added to the
         # end

=== modified file 'bzrlib/tests/per_workingtree/test_is_ignored.py'
--- bzrlib/tests/per_workingtree/test_is_ignored.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/per_workingtree/test_is_ignored.py	2010-07-08 13:00:19 +0000
@@ -22,6 +22,16 @@
 
 class TestIsIgnored(TestCaseWithWorkingTree):
 
+    def _set_user_ignore_content(self, ignores):
+        """Create user ignore file and set its content to ignores."""
+        config.ensure_config_dir_exists()
+        user_ignore_file = config.user_ignore_config_filename()
+        f = open(user_ignore_file, 'wb')
+        try:
+            f.write(ignores)
+        finally:
+            f.close()
+
     def test_is_ignored(self):
         tree = self.make_branch_and_tree('.')
         # this will break if a tree changes the ignored format. That is fine
@@ -46,6 +56,11 @@
                            '#comment\n'
                            ' xx \n' # whitespace
             )])
+        # We set user ignore file to contain '' to avoid patterns from
+        # user ignore being used instead of bzrignore. For .e.g. If we
+        # don't do this 'foo.~1~' will match '*~' default user ignore
+        # pattern instead of '*.~*' from bzr ignore as we expect below.
+        self._set_user_ignore_content('')
         # is_ignored returns the matching ignore regex when a path is ignored.
         # we check some expected matches for each rule, and one or more
         # relevant not-matches that look plausible as cases for bugs.
@@ -119,19 +134,16 @@
 
         config.ensure_config_dir_exists()
         user_ignore_file = config.user_ignore_config_filename()
-        f = open(user_ignore_file, 'wb')
-        try:
-            f.write('*.py[co]\n'
-                    './.shelf\n'
-                    '# comment line\n'
-                    '\n' #Blank line
-                    '\r\n' #Blank dos line
-                    ' * \n' #Trailing and suffix spaces
-                    'crlf\r\n' # dos style line
-                    '*\xc3\xa5*\n' # u'\xe5'.encode('utf8')
-                    )
-        finally:
-            f.close()
+        self._set_user_ignore_content(
+            '*.py[co]\n'
+            './.shelf\n'
+            '# comment line\n'
+            '\n' #Blank line
+            '\r\n' #Blank dos line
+            ' * \n' #Trailing and suffix spaces
+            'crlf\r\n' # dos style line
+            '*\xc3\xa5*\n' # u'\xe5'.encode('utf8')
+            )
 
         # Rooted
         self.assertEqual('./.shelf', tree.is_ignored('.shelf'))

=== modified file 'bzrlib/tests/per_workingtree/test_locking.py'
--- bzrlib/tests/per_workingtree/test_locking.py	2009-08-25 23:38:10 +0000
+++ bzrlib/tests/per_workingtree/test_locking.py	2010-05-11 14:14:55 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2008, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
     lockdir,
     )
 from bzrlib.tests import TestSkipped
+from bzrlib.tests.matchers import *
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
 
 
@@ -44,6 +45,10 @@
         self.assertFalse(wt.is_locked())
         self.assertFalse(wt.branch.is_locked())
 
+    def test_lock_read_returns_unlocker(self):
+        wt = self.make_branch_and_tree('.')
+        self.assertThat(wt.lock_read, ReturnsUnlockable(wt))
+
     def test_trivial_lock_write_unlock(self):
         """Locking for write and unlocking should work trivially."""
         wt = self.make_branch_and_tree('.')
@@ -59,6 +64,10 @@
         self.assertFalse(wt.is_locked())
         self.assertFalse(wt.branch.is_locked())
 
+    def test_lock_write_returns_unlocker(self):
+        wt = self.make_branch_and_tree('.')
+        self.assertThat(wt.lock_write, ReturnsUnlockable(wt))
+
     def test_trivial_lock_tree_write_unlock(self):
         """Locking for tree write is ok when the branch is not locked."""
         wt = self.make_branch_and_tree('.')
@@ -74,6 +83,10 @@
         self.assertFalse(wt.is_locked())
         self.assertFalse(wt.branch.is_locked())
 
+    def test_lock_tree_write_returns_unlocker(self):
+        wt = self.make_branch_and_tree('.')
+        self.assertThat(wt.lock_tree_write, ReturnsUnlockable(wt))
+
     def test_trivial_lock_tree_write_branch_read_locked(self):
         """It is ok to lock_tree_write when the branch is read locked."""
         wt = self.make_branch_and_tree('.')

=== modified file 'bzrlib/tests/per_workingtree/test_move.py'
--- bzrlib/tests/per_workingtree/test_move.py	2009-07-10 07:14:02 +0000
+++ bzrlib/tests/per_workingtree/test_move.py	2010-07-15 13:33:30 +0000
@@ -79,43 +79,6 @@
             tree.move(['a1'], 'sub1', after=False))
         tree._validate()
 
-    def test_move_deprecated_wrong_call(self):
-        """tree.move has the deprecated parameter 'to_name'.
-        It has been replaced by 'to_dir' for consistency.
-        Test the new API using wrong parameter
-        """
-        self.build_tree(['a1', 'sub1/'])
-        tree = self.make_branch_and_tree('.')
-        tree.add(['a1', 'sub1'])
-        tree.commit('initial commit')
-        self.assertRaises(TypeError, tree.move, ['a1'],
-                          to_this_parameter_does_not_exist='sub1',
-                          after=False)
-        tree._validate()
-
-    def test_move_deprecated_call(self):
-        """tree.move has the deprecated parameter 'to_name'.
-        It has been replaced by 'to_dir' for consistency.
-        Test the new API using deprecated parameter
-        """
-        self.build_tree(['a1', 'sub1/'])
-        tree = self.make_branch_and_tree('.')
-        tree.add(['a1', 'sub1'])
-        tree.commit('initial commit')
-
-        try:
-            self.callDeprecated(['The parameter to_name was deprecated'
-                                 ' in version 0.13. Use to_dir instead'],
-                                tree.move, ['a1'], to_name='sub1',
-                                after=False)
-        except TypeError:
-            # WorkingTreeFormat4 and later don't have to maintain api
-            # compatibility since it was deprecated before they were introduced.
-            if not isinstance(self.workingtree_format,
-                DirStateWorkingTreeFormat):
-                raise
-        tree._validate()
-
     def test_move_target_not_dir(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a'])

=== modified file 'bzrlib/tests/per_workingtree/test_pull.py'
--- bzrlib/tests/per_workingtree/test_pull.py	2010-01-11 23:02:32 +0000
+++ bzrlib/tests/per_workingtree/test_pull.py	2010-09-06 10:08:36 +0000
@@ -16,17 +16,11 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 from cStringIO import StringIO
-import os
-
-from bzrlib import errors
-from bzrlib.errors import NotBranchError, NotVersionedError
-from bzrlib.osutils import basename
-from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
-from bzrlib.trace import mutter
-from bzrlib.transport import get_transport
-
-
-class TestPull(TestCaseWithWorkingTree):
+
+from bzrlib.tests import per_workingtree
+
+
+class TestPull(per_workingtree.TestCaseWithWorkingTree):
 
     def get_pullable_trees(self):
         self.build_tree(['from/', 'from/file', 'to/'])

=== modified file 'bzrlib/tests/per_workingtree/test_remove.py'
--- bzrlib/tests/per_workingtree/test_remove.py	2009-07-10 07:14:02 +0000
+++ bzrlib/tests/per_workingtree/test_remove.py	2010-08-20 09:39:20 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007, 2008 Canonical Ltd
+# Copyright (C) 2007-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 
     files = ['a', 'b/', 'b/c', 'd/']
     rfiles = ['b/c', 'b', 'a', 'd']
+    backup_files = ['a.~1~', 'b.~1~/', 'b.~1~/c.~1~', 'd.~1~/']
 
     def get_tree(self, files):
         tree = self.make_branch_and_tree('.')
@@ -72,28 +73,23 @@
         tree._validate()
 
     def test_remove_added_files(self):
-        """Removal of newly added files must fail."""
+        """Removal of newly added files must back them up."""
         tree = self.get_tree(TestRemove.files)
         tree.add(TestRemove.files)
         self.assertInWorkingTree(TestRemove.files)
-        err = self.assertRaises(errors.BzrRemoveChangedFilesError, tree.remove,
-            TestRemove.files, keep_files=False)
-        self.assertContainsRe(err.changes_as_text,
-            '(?s)added:.*a.*b/.*b/c.*d/')
-        self.assertInWorkingTree(TestRemove.files)
-        self.failUnlessExists(TestRemove.files)
+        tree.remove(TestRemove.files, keep_files=False)
+        self.assertNotInWorkingTree(TestRemove.files)
+        self.failUnlessExists(TestRemove.backup_files)
         tree._validate()
 
     def test_remove_changed_file(self):
-        """Removal of a changed files must fail."""
+        """Removal of changed files must back it up."""
         tree = self.get_committed_tree(['a'])
         self.build_tree_contents([('a', "some other new content!")])
         self.assertInWorkingTree('a')
-        err = self.assertRaises(errors.BzrRemoveChangedFilesError, tree.remove,
-            'a', keep_files=False)
-        self.assertContainsRe(err.changes_as_text, '(?s)modified:.*a')
-        self.assertInWorkingTree('a')
-        self.failUnlessExists('a')
+        tree.remove('a', keep_files=False)
+        self.assertNotInWorkingTree(TestRemove.files)
+        self.failUnlessExists('a.~1~')
         tree._validate()
 
     def test_remove_deleted_files(self):
@@ -122,7 +118,7 @@
         tree._validate()
 
     def test_remove_renamed_changed_files(self):
-        """Check that files are not removed if they are renamed and changed."""
+        """Check that files that are renamed and changed are backed up."""
         tree = self.get_committed_tree(TestRemove.files)
 
         for f in TestRemove.rfiles:
@@ -133,12 +129,10 @@
         self.assertInWorkingTree(rfilesx)
         self.failUnlessExists(rfilesx)
 
-        err = self.assertRaises(errors.BzrRemoveChangedFilesError, tree.remove,
-            rfilesx, keep_files=False)
-        self.assertContainsRe(err.changes_as_text,
-            '(?s)modified:.*ax.*bx/cx')
-        self.assertInWorkingTree(rfilesx)
-        self.failUnlessExists(rfilesx)
+        tree.remove(rfilesx, keep_files=False)
+        self.assertNotInWorkingTree(rfilesx)
+        self.failUnlessExists(['bx.~1~/cx.~1~', 'bx.~1~', 'ax.~1~'])
+        self.failIfExists('dx.~1~') # unchanged file
         tree._validate()
 
     def test_force_remove_changed_files(self):
@@ -149,15 +143,15 @@
 
         tree.remove(TestRemove.files, keep_files=False, force=True)
         self.assertRemovedAndDeleted(TestRemove.files)
+        self.failIfExists(['a.~1~', 'b.~1~/', 'b.~1~/c', 'd.~1~/'])
         tree._validate()
 
     def test_remove_unknown_files(self):
-        """Try to delete unknown files."""
+        """Unknown files shuld be backed up"""
         tree = self.get_tree(TestRemove.files)
-        err = self.assertRaises(errors.BzrRemoveChangedFilesError, tree.remove,
-            TestRemove.files, keep_files=False)
-        self.assertContainsRe(err.changes_as_text,
-            '(?s)unknown:.*d/.*b/c.*b/.*a.*')
+        tree.remove(TestRemove.files, keep_files=False)
+        self.assertRemovedAndDeleted(TestRemove.files)
+        self.failUnlessExists(TestRemove.backup_files)
         tree._validate()
 
     def test_remove_nonexisting_files(self):
@@ -202,45 +196,40 @@
         tree._validate()
 
     def test_remove_changed_ignored_files(self):
-        """Changed ignored files should not be deleted."""
+        """Changed ignored files should be backed up."""
         files = ['an_ignored_file']
         tree = self.get_tree(files)
         tree.add(files)
         ignores.add_runtime_ignores(["*ignored*"])
         self.assertInWorkingTree(files)
         self.assertNotEquals(None, tree.is_ignored(files[0]))
-        err = self.assertRaises(errors.BzrRemoveChangedFilesError, tree.remove,
-            files, keep_files=False)
-        self.assertContainsRe(err.changes_as_text,
-            '(?s)added:.*' + files[0])
-        self.assertInWorkingTree(files)
+
+        tree.remove(files, keep_files=False)
+        self.assertNotInWorkingTree(files)
+        self.failUnlessExists('an_ignored_file.~1~')
         tree._validate()
 
     def test_dont_remove_directory_with_unknowns(self):
-        """Directories with unknowns should not be deleted."""
+        """Directories with unknowns should be backed up."""
         directories = ['a/', 'b/', 'c/', 'c/c/']
         tree = self.get_committed_tree(directories)
 
         self.build_tree(['a/unknown_file'])
-        err = self.assertRaises(errors.BzrRemoveChangedFilesError, tree.remove,
-            'a', keep_files=False)
-        self.assertContainsRe(err.changes_as_text,
-            '(?s)unknown:.*a/unknown_file')
+        tree.remove('a', keep_files=False)
+        self.failUnlessExists('a.~1~/unknown_file')
 
         self.build_tree(['b/unknown_directory'])
-        err = self.assertRaises(errors.BzrRemoveChangedFilesError, tree.remove,
-            'b', keep_files=False)
-        self.assertContainsRe(err.changes_as_text,
-            '(?s)unknown:.*b/unknown_directory')
+        tree.remove('b', keep_files=False)
+        self.failUnlessExists('b.~1~/unknown_directory')
 
         self.build_tree(['c/c/unknown_file'])
-        err = self.assertRaises(errors.BzrRemoveChangedFilesError, tree.remove,
-            'c/c', keep_files=False)
-        self.assertContainsRe(err.changes_as_text,
-            '(?s)unknown:.*c/c/unknown_file')
-
-        self.assertInWorkingTree(directories)
-        self.failUnlessExists(directories)
+        tree.remove('c/c', keep_files=False)
+        self.failUnlessExists('c/c.~1~/unknown_file')
+
+        tree.remove('c', keep_files=False)
+        self.failUnlessExists('c.~1~/')
+
+        self.assertNotInWorkingTree(directories)
         tree._validate()
 
     def test_force_remove_directory_with_unknowns(self):
@@ -262,22 +251,40 @@
         tree._validate()
 
     def test_remove_directory_with_changed_file(self):
-        """Refuse to delete directories with changed files."""
-        files = ['b/', 'b/c']
-        tree = self.get_committed_tree(files)
-        self.build_tree_contents([('b/c', "some other new content!")])
-
-        err = self.assertRaises(errors.BzrRemoveChangedFilesError, tree.remove,
-            'b', keep_files=False)
-        self.assertContainsRe(err.changes_as_text, '(?s)modified:.*b/c')
-        self.assertInWorkingTree(files)
-        self.failUnlessExists(files)
+        """Backup directories with changed files."""
+        files = ['b/', 'b/c']
+        tree = self.get_committed_tree(files)
+        self.build_tree_contents([('b/c', "some other new content!")])
+
+        tree.remove('b', keep_files=False)
+        self.failUnlessExists('b.~1~/c.~1~')
+        self.assertNotInWorkingTree(files)
+
+    def test_remove_force_directory_with_changed_file(self):
+        """Delete directories with changed files when forced."""
+        files = ['b/', 'b/c']
+        tree = self.get_committed_tree(files)
+        self.build_tree_contents([('b/c', "some other new content!")])
 
         # see if we can force it now..
         tree.remove('b', keep_files=False, force=True)
         self.assertRemovedAndDeleted(files)
         tree._validate()
 
+    def test_remove_directory_with_changed_emigrated_file(self):
+        # As per bug #129880
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([('somedir/',), ('somedir/file', 'contents')])
+        tree.add(['somedir', 'somedir/file'])
+        tree.commit(message="first")
+        self.build_tree_contents([('somedir/file', 'changed')])
+        tree.rename_one('somedir/file', 'moved-file')
+        tree.remove('somedir', keep_files=False)
+        self.assertNotInWorkingTree('somedir')
+        self.failIfExists('somedir')
+        self.assertInWorkingTree('moved-file')
+        self.failUnlessExists('moved-file')
+
     def test_remove_directory_with_renames(self):
         """Delete directory with renames in or out."""
 

=== added file 'bzrlib/tests/per_workingtree/test_symlinks.py'
--- bzrlib/tests/per_workingtree/test_symlinks.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_workingtree/test_symlinks.py	2010-08-17 06:45:33 +0000
@@ -0,0 +1,175 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Test symlink support.
+"""
+
+import os
+
+from bzrlib import (
+    builtins,
+    osutils,
+    tests,
+    workingtree,
+    )
+from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
+
+
+class TestSmartAddTree(TestCaseWithWorkingTree):
+
+    # See eg <https://bugs.launchpad.net/bzr/+bug/192859>
+
+    _test_needs_features = [tests.SymlinkFeature]
+
+    def test_smart_add_symlink(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([
+            ('tree/link@', 'target'),
+            ])
+        tree.smart_add(['tree/link'])
+        self.assertIsNot(None, tree.path2id('link'))
+        self.assertIs(None, tree.path2id('target'))
+        self.assertEqual('symlink',
+            tree.kind(tree.path2id('link')))
+
+    def test_smart_add_symlink_pointing_outside(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([
+            ('tree/link@', '../../../../target'),
+            ])
+        tree.smart_add(['tree/link'])
+        self.assertIsNot(None, tree.path2id('link'))
+        self.assertIs(None, tree.path2id('target'))
+        self.assertEqual('symlink',
+            tree.kind(tree.path2id('link')))
+
+    def test_add_file_under_symlink(self):
+        # similar to 
+        # https://bugs.launchpad.net/bzr/+bug/192859/comments/3
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([
+            ('tree/link@', 'dir'),
+            ('tree/dir/',),
+            ('tree/dir/file', 'content'),
+            ])
+        self.assertEquals(
+            tree.smart_add(['tree/link/file']),
+            ([u'dir', u'dir/file'], {}))
+        # should add the actual parent directory, not the apparent parent
+        # (which is actually a symlink)
+        self.assertTrue(tree.path2id('dir/file'))
+        self.assertTrue(tree.path2id('dir'))
+        self.assertIs(None, tree.path2id('link'))
+        self.assertIs(None, tree.path2id('link/file'))
+
+
+class TestKindChanges(TestCaseWithWorkingTree):
+
+    _test_needs_features = [tests.SymlinkFeature]
+
+    def test_symlink_changes_to_dir(self):
+        # <https://bugs.launchpad.net/bzr/+bug/192859>:
+        # we had some past problems with the workingtree remembering for too
+        # long what kind of object was at a particular name; we really
+        # shouldn't do that.  Operating on the dirstate through passing
+        # inventory deltas rather than mutating the inventory largely avoids
+        # that.
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([
+            ('tree/a@', 'target')])
+        tree.smart_add(['tree/a'])
+        tree.commit('add symlink')
+        os.unlink('tree/a')
+        self.build_tree_contents([
+            ('tree/a/',),
+            ('tree/a/f', 'content'),
+            ])
+        tree.smart_add(['tree/a/f'])
+        tree.commit('change to dir')
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        self.assertEquals([], list(tree.iter_changes(tree.basis_tree())))
+        self.assertEquals(
+            ['a', 'a/f'], sorted(info[0] for info in tree.list_files()))
+
+    def test_dir_changes_to_symlink(self):
+        # <https://bugs.launchpad.net/bzr/+bug/192859>:
+        # we had some past problems with the workingtree remembering for too
+        # long what kind of object was at a particular name; we really
+        # shouldn't do that.  Operating on the dirstate through passing
+        # inventory deltas rather than mutating the inventory largely avoids
+        # that.
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([
+            ('tree/a/',),
+            ('tree/a/file', 'content'),
+            ])
+        tree.smart_add(['tree/a'])
+        tree.commit('add dir')
+        osutils.rmtree('tree/a')
+        self.build_tree_contents([
+            ('tree/a@', 'target'),
+            ])
+        tree.commit('change to symlink')
+
+
+class TestOpenTree(TestCaseWithWorkingTree):
+
+    _test_needs_features = [tests.SymlinkFeature]
+
+    def test_open_containing_through_symlink(self):
+        self.make_test_tree()
+        self.check_open_containing('link/content', 'tree', 'content')
+        self.check_open_containing('link/sublink', 'tree', 'sublink')
+        # this next one is a bit debatable, but arguably it's better that
+        # open_containing is only concerned with opening the tree 
+        # and then you can deal with symlinks along the way if you want
+        self.check_open_containing('link/sublink/subcontent', 'tree',
+            'sublink/subcontent')
+
+    def check_open_containing(self, to_open, expected_tree_name,
+        expected_relpath):
+        wt, relpath = workingtree.WorkingTree.open_containing(to_open)
+        self.assertEquals(relpath, expected_relpath)
+        self.assertEndsWith(wt.basedir, expected_tree_name)
+
+    def test_tree_files(self):
+        # not strictly a WorkingTree method, but it should be
+        # probably the root cause for
+        # <https://bugs.launchpad.net/bzr/+bug/128562>
+        self.make_test_tree()
+        self.check_tree_files(['tree/outerlink'],
+            'tree', ['outerlink'])
+        self.check_tree_files(['link/outerlink'],
+            'tree', ['outerlink'])
+        self.check_tree_files(['link/sublink/subcontent'],
+            'tree', ['subdir/subcontent'])
+
+    def check_tree_files(self, to_open, expected_tree, expect_paths):
+        tree, relpaths = workingtree.WorkingTree.open_containing_paths(to_open)
+        self.assertEndsWith(tree.basedir, expected_tree)
+        self.assertEquals(expect_paths, relpaths)
+
+    def make_test_tree(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([
+            ('link@', 'tree'),
+            ('tree/outerlink@', '/not/there'),
+            ('tree/content', 'hello'),
+            ('tree/sublink@', 'subdir'),
+            ('tree/subdir/',),
+            ('tree/subdir/subcontent', 'subcontent stuff')
+            ])

=== modified file 'bzrlib/tests/per_workingtree/test_unversion.py'
--- bzrlib/tests/per_workingtree/test_unversion.py	2009-07-15 04:33:14 +0000
+++ bzrlib/tests/per_workingtree/test_unversion.py	2010-10-04 22:10:49 +0000
@@ -42,7 +42,7 @@
         # unversioning children of reparented children of unversioned
         # paths when relocation entries are present and the relocation
         # points later into the dirstate.
-        tree = self.make_branch_and_tree(['.'])
+        tree = self.make_branch_and_tree('.')
         self.build_tree(['del/', 'del/sub/', 'del/sub/b'])
         tree.add(['del', 'del/sub', 'del/sub/b'], ['del', 'sub', 'b'])
         tree.commit('setup')

=== modified file 'bzrlib/tests/per_workingtree/test_workingtree.py'
--- bzrlib/tests/per_workingtree/test_workingtree.py	2010-04-21 09:41:26 +0000
+++ bzrlib/tests/per_workingtree/test_workingtree.py	2010-05-20 18:23:17 +0000
@@ -615,7 +615,7 @@
         rev1 = wt.commit('first master commit')
         self.build_tree_contents([('wt/a', 'new content')])
         rev2 = wt.commit('second master commit')
-        # https://bugs.edge.launchpad.net/bzr/+bug/45719/comments/20
+        # https://bugs.launchpad.net/bzr/+bug/45719/comments/20
         # when adding 'update -r' we should make sure all wt formats support
         # it
         conflicts = wt.update(revision=rev1)

=== modified file 'bzrlib/tests/script.py'
--- bzrlib/tests/script.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/script.py	2010-09-13 08:14:44 +0000
@@ -24,6 +24,7 @@
 import glob
 import os
 import shlex
+import textwrap
 from cStringIO import StringIO
 
 from bzrlib import (
@@ -55,6 +56,10 @@
     Input lines start with '<'.
     Output lines start with nothing.
     Error lines start with '2>'.
+
+    :return: A sequence of ([args], input, output, errors), where the args are
+        split in to words, and the input, output, and errors are just strings,
+        typically containing newlines.
     """
 
     commands = []
@@ -73,18 +78,27 @@
     cmd_line = 1
     lineno = 0
     input, output, error = None, None, None
-    for line in text.split('\n'):
+    text = textwrap.dedent(text)
+    lines = text.split('\n')
+    # to make use of triple-quoted strings easier, we ignore a blank line
+    # right at the start and right at the end; the rest are meaningful
+    if lines and lines[0] == '':
+        del lines[0]
+    if lines and lines[-1] == '':
+        del lines[-1]
+    for line in lines:
         lineno += 1
         # Keep a copy for error reporting
         orig = line
         comment =  line.find('#')
         if comment >= 0:
             # Delete comments
+            # NB: this syntax means comments are allowed inside output, which
+            # may be confusing...
             line = line[0:comment]
             line = line.rstrip()
-        if line == '':
-            # Ignore empty lines
-            continue
+            if line == '':
+                continue
         if line.startswith('$'):
             # Time to output the current command
             add_command(cmd_cur, input, output, error)
@@ -230,7 +244,15 @@
             # 'expected' parameter. So we just fallback to our good old
             # assertEqualDiff since we know there *are* differences and the
             # output should be decently readable.
-            test_case.assertEqualDiff(expected, actual)
+            #
+            # As a special case, we allow output that's missing a final
+            # newline to match an expected string that does have one, so that
+            # we can match a prompt printed on one line, then input given on
+            # the next line.
+            if expected == actual + '\n':
+                pass
+            else:
+                test_case.assertEqualDiff(expected, actual)
 
     def _pre_process_args(self, args):
         new_args = []
@@ -475,3 +497,7 @@
     def run_command(self, cmd, input, output, error):
         return self.script_runner.run_command(self, cmd, input, output, error)
 
+
+def run_script(test_case, script_string):
+    """Run the given script within a testcase"""
+    return ScriptRunner().run_script(test_case, script_string)

=== modified file 'bzrlib/tests/stub_sftp.py'
--- bzrlib/tests/stub_sftp.py	2010-03-11 04:33:41 +0000
+++ bzrlib/tests/stub_sftp.py	2010-09-02 19:19:54 +0000
@@ -23,6 +23,7 @@
 import paramiko
 import select
 import socket
+import SocketServer
 import sys
 import threading
 import time
@@ -38,24 +39,24 @@
 from bzrlib.tests import test_server
 
 
-class StubServer (paramiko.ServerInterface):
+class StubServer(paramiko.ServerInterface):
 
-    def __init__(self, test_case):
+    def __init__(self, test_case_server):
         paramiko.ServerInterface.__init__(self)
-        self._test_case = test_case
+        self.log = test_case_server.log
 
     def check_auth_password(self, username, password):
         # all are allowed
-        self._test_case.log('sftpserver - authorizing: %s' % (username,))
+        self.log('sftpserver - authorizing: %s' % (username,))
         return paramiko.AUTH_SUCCESSFUL
 
     def check_channel_request(self, kind, chanid):
-        self._test_case.log(
-            'sftpserver - channel request: %s, %s' % (kind, chanid))
+        self.log('sftpserver - channel request: %s, %s' % (kind, chanid))
         return paramiko.OPEN_SUCCEEDED
 
 
-class StubSFTPHandle (paramiko.SFTPHandle):
+class StubSFTPHandle(paramiko.SFTPHandle):
+
     def stat(self):
         try:
             return paramiko.SFTPAttributes.from_stat(
@@ -73,7 +74,7 @@
             return paramiko.SFTPServer.convert_errno(e.errno)
 
 
-class StubSFTPServer (paramiko.SFTPServerInterface):
+class StubSFTPServer(paramiko.SFTPServerInterface):
 
     def __init__(self, server, root, home=None):
         paramiko.SFTPServerInterface.__init__(self, server)
@@ -90,7 +91,7 @@
             self.home = home[len(self.root):]
         if self.home.startswith('/'):
             self.home = self.home[1:]
-        server._test_case.log('sftpserver - new connection')
+        server.log('sftpserver - new connection')
 
     def _realpath(self, path):
         # paths returned from self.canonicalize() always start with
@@ -135,7 +136,7 @@
         try:
             out = [ ]
             # TODO: win32 incorrectly lists paths with non-ascii if path is not
-            # unicode. However on Linux the server should only deal with
+            # unicode. However on unix the server should only deal with
             # bytestreams and posix.listdir does the right thing
             if sys.platform == 'win32':
                 flist = [f.encode('utf8') for f in os.listdir(path)]
@@ -241,6 +242,7 @@
     # removed: chattr, symlink, readlink
     # (nothing in bzr's sftp transport uses those)
 
+
 # ------------- server test implementation --------------
 
 STUB_SERVER_KEY = """
@@ -262,51 +264,6 @@
 """
 
 
-class SocketListener(threading.Thread):
-
-    def __init__(self, callback):
-        threading.Thread.__init__(self)
-        self._callback = callback
-        self._socket = socket.socket()
-        self._socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-        self._socket.bind(('localhost', 0))
-        self._socket.listen(1)
-        self.host, self.port = self._socket.getsockname()[:2]
-        self._stop_event = threading.Event()
-
-    def stop(self):
-        # called from outside this thread
-        self._stop_event.set()
-        # use a timeout here, because if the test fails, the server thread may
-        # never notice the stop_event.
-        self.join(5.0)
-        self._socket.close()
-
-    def run(self):
-        while True:
-            readable, writable_unused, exception_unused = \
-                select.select([self._socket], [], [], 0.1)
-            if self._stop_event.isSet():
-                return
-            if len(readable) == 0:
-                continue
-            try:
-                s, addr_unused = self._socket.accept()
-                # because the loopback socket is inline, and transports are
-                # never explicitly closed, best to launch a new thread.
-                threading.Thread(target=self._callback, args=(s,)).start()
-            except socket.error, x:
-                sys.excepthook(*sys.exc_info())
-                trace.warning('Socket error during accept() '
-                              'within unit test server thread: %r' % x)
-            except Exception, x:
-                # probably a failed test; unit test thread will log the
-                # failure/error
-                sys.excepthook(*sys.exc_info())
-                trace.warning(
-                    'Exception from within unit test server thread: %r' % x)
-
-
 class SocketDelay(object):
     """A socket decorator to make TCP appear slower.
 
@@ -382,55 +339,139 @@
         return bytes_sent
 
 
-class SFTPServer(test_server.TestServer):
+class TestingSFTPConnectionHandler(SocketServer.BaseRequestHandler):
+
+    def setup(self):
+        self.wrap_for_latency()
+        tcs = self.server.test_case_server
+        ptrans = paramiko.Transport(self.request)
+        self.paramiko_transport = ptrans
+        # Set it to a channel under 'bzr' so that we get debug info
+        ptrans.set_log_channel('bzr.paramiko.transport')
+        ptrans.add_server_key(tcs.get_host_key())
+        ptrans.set_subsystem_handler('sftp', paramiko.SFTPServer,
+                                     StubSFTPServer, root=tcs._root,
+                                     home=tcs._server_homedir)
+        server = tcs._server_interface(tcs)
+        # This blocks until the key exchange has been done
+        ptrans.start_server(None, server)
+
+    def finish(self):
+        # Wait for the conversation to finish, when the paramiko.Transport
+        # thread finishes
+        # TODO: Consider timing out after XX seconds rather than hanging.
+        #       Also we could check paramiko_transport.active and possibly
+        #       paramiko_transport.getException().
+        self.paramiko_transport.join()
+
+    def wrap_for_latency(self):
+        tcs = self.server.test_case_server
+        if tcs.add_latency:
+            # Give the socket (which the request really is) a latency adding
+            # decorator.
+            self.request = SocketDelay(self.request, tcs.add_latency)
+
+
+class TestingSFTPWithoutSSHConnectionHandler(TestingSFTPConnectionHandler):
+
+    def setup(self):
+        self.wrap_for_latency()
+        # Re-import these as locals, so that they're still accessible during
+        # interpreter shutdown (when all module globals get set to None, leading
+        # to confusing errors like "'NoneType' object has no attribute 'error'".
+        class FakeChannel(object):
+            def get_transport(self):
+                return self
+            def get_log_channel(self):
+                return 'bzr.paramiko'
+            def get_name(self):
+                return '1'
+            def get_hexdump(self):
+                return False
+            def close(self):
+                pass
+
+        tcs = self.server.test_case_server
+        sftp_server = paramiko.SFTPServer(
+            FakeChannel(), 'sftp', StubServer(tcs), StubSFTPServer,
+            root=tcs._root, home=tcs._server_homedir)
+        self.sftp_server = sftp_server
+        sys_stderr = sys.stderr # Used in error reporting during shutdown
+        try:
+            sftp_server.start_subsystem(
+                'sftp', None, ssh.SocketAsChannelAdapter(self.request))
+        except socket.error, e:
+            if (len(e.args) > 0) and (e.args[0] == errno.EPIPE):
+                # it's okay for the client to disconnect abruptly
+                # (bug in paramiko 1.6: it should absorb this exception)
+                pass
+            else:
+                raise
+        except Exception, e:
+            # This typically seems to happen during interpreter shutdown, so
+            # most of the useful ways to report this error won't work.
+            # Writing the exception type, and then the text of the exception,
+            # seems to be the best we can do.
+            # FIXME: All interpreter shutdown errors should have been related
+            # to daemon threads, cleanup needed -- vila 20100623
+            sys_stderr.write('\nEXCEPTION %r: ' % (e.__class__,))
+            sys_stderr.write('%s\n\n' % (e,))
+
+    def finish(self):
+        self.sftp_server.finish_subsystem()
+
+
+class TestingSFTPServer(test_server.TestingThreadingTCPServer):
+
+    def __init__(self, server_address, request_handler_class, test_case_server):
+        test_server.TestingThreadingTCPServer.__init__(
+            self, server_address, request_handler_class)
+        self.test_case_server = test_case_server
+
+
+class SFTPServer(test_server.TestingTCPServerInAThread):
     """Common code for SFTP server facilities."""
 
     def __init__(self, server_interface=StubServer):
+        self.host = '127.0.0.1'
+        self.port = 0
+        super(SFTPServer, self).__init__((self.host, self.port),
+                                         TestingSFTPServer,
+                                         TestingSFTPConnectionHandler)
         self._original_vendor = None
-        self._homedir = None
-        self._server_homedir = None
-        self._listener = None
-        self._root = None
         self._vendor = ssh.ParamikoVendor()
         self._server_interface = server_interface
-        # sftp server logs
+        self._host_key = None
         self.logs = []
         self.add_latency = 0
+        self._homedir = None
+        self._server_homedir = None
+        self._root = None
 
     def _get_sftp_url(self, path):
         """Calculate an sftp url to this server for path."""
-        return 'sftp://foo:bar@%s:%d/%s' % (self._listener.host,
-                                            self._listener.port, path)
+        return "sftp://foo:bar@%s:%s/%s" % (self.host, self.port, path)
 
     def log(self, message):
         """StubServer uses this to log when a new server is created."""
         self.logs.append(message)
 
-    def _run_server_entry(self, sock):
-        """Entry point for all implementations of _run_server.
-
-        If self.add_latency is > 0.000001 then sock is given a latency adding
-        decorator.
-        """
-        if self.add_latency > 0.000001:
-            sock = SocketDelay(sock, self.add_latency)
-        return self._run_server(sock)
-
-    def _run_server(self, s):
-        ssh_server = paramiko.Transport(s)
-        key_file = osutils.pathjoin(self._homedir, 'test_rsa.key')
-        f = open(key_file, 'w')
-        f.write(STUB_SERVER_KEY)
-        f.close()
-        host_key = paramiko.RSAKey.from_private_key_file(key_file)
-        ssh_server.add_server_key(host_key)
-        server = self._server_interface(self)
-        ssh_server.set_subsystem_handler('sftp', paramiko.SFTPServer,
-                                         StubSFTPServer, root=self._root,
-                                         home=self._server_homedir)
-        event = threading.Event()
-        ssh_server.start_server(event, server)
-        event.wait(5.0)
+    def create_server(self):
+        server = self.server_class((self.host, self.port),
+                                   self.request_handler_class,
+                                   self)
+        return server
+
+    def get_host_key(self):
+        if self._host_key is None:
+            key_file = osutils.pathjoin(self._homedir, 'test_rsa.key')
+            f = open(key_file, 'w')
+            try:
+                f.write(STUB_SERVER_KEY)
+            finally:
+                f.close()
+            self._host_key = paramiko.RSAKey.from_private_key_file(key_file)
+        return self._host_key
 
     def start_server(self, backing_server=None):
         # XXX: TODO: make sftpserver back onto backing_server rather than local
@@ -442,30 +483,30 @@
                 'the local current working directory.' % (backing_server,))
         self._original_vendor = ssh._ssh_vendor_manager._cached_ssh_vendor
         ssh._ssh_vendor_manager._cached_ssh_vendor = self._vendor
-        # FIXME: the following block should certainly just be self._homedir =
-        # osutils.getcwd() but that fails badly on Unix -- vila 20100224
         if sys.platform == 'win32':
             # Win32 needs to use the UNICODE api
             self._homedir = os.getcwdu()
+            # Normalize the path or it will be wrongly escaped
+            self._homedir = osutils.normpath(self._homedir)
         else:
-            # But Linux SFTP servers should just deal in bytestreams
+            # But unix SFTP servers should just deal in bytestreams
             self._homedir = os.getcwd()
         if self._server_homedir is None:
             self._server_homedir = self._homedir
         self._root = '/'
         if sys.platform == 'win32':
             self._root = ''
-        self._listener = SocketListener(self._run_server_entry)
-        self._listener.setDaemon(True)
-        self._listener.start()
+        super(SFTPServer, self).start_server()
 
     def stop_server(self):
-        self._listener.stop()
-        ssh._ssh_vendor_manager._cached_ssh_vendor = self._original_vendor
+        try:
+            super(SFTPServer, self).stop_server()
+        finally:
+            ssh._ssh_vendor_manager._cached_ssh_vendor = self._original_vendor
 
     def get_bogus_url(self):
         """See bzrlib.transport.Server.get_bogus_url."""
-        # this is chosen to try to prevent trouble with proxies, wierd dns, etc
+        # this is chosen to try to prevent trouble with proxies, weird dns, etc
         # we bind a random socket, so that we get a guaranteed unused port
         # we just never listen on that port
         s = socket.socket()
@@ -491,45 +532,10 @@
     def __init__(self):
         super(SFTPServerWithoutSSH, self).__init__()
         self._vendor = ssh.LoopbackVendor()
-
-    def _run_server(self, sock):
-        # Re-import these as locals, so that they're still accessible during
-        # interpreter shutdown (when all module globals get set to None, leading
-        # to confusing errors like "'NoneType' object has no attribute 'error'".
-        class FakeChannel(object):
-            def get_transport(self):
-                return self
-            def get_log_channel(self):
-                return 'paramiko'
-            def get_name(self):
-                return '1'
-            def get_hexdump(self):
-                return False
-            def close(self):
-                pass
-
-        server = paramiko.SFTPServer(
-            FakeChannel(), 'sftp', StubServer(self), StubSFTPServer,
-            root=self._root, home=self._server_homedir)
-        try:
-            server.start_subsystem(
-                'sftp', None, ssh.SocketAsChannelAdapter(sock))
-        except socket.error, e:
-            if (len(e.args) > 0) and (e.args[0] == errno.EPIPE):
-                # it's okay for the client to disconnect abruptly
-                # (bug in paramiko 1.6: it should absorb this exception)
-                pass
-            else:
-                raise
-        except Exception, e:
-            # This typically seems to happen during interpreter shutdown, so
-            # most of the useful ways to report this error are won't work.
-            # Writing the exception type, and then the text of the exception,
-            # seems to be the best we can do.
-            import sys
-            sys.stderr.write('\nEXCEPTION %r: ' % (e.__class__,))
-            sys.stderr.write('%s\n\n' % (e,))
-        server.finish_subsystem()
+        self.request_handler_class = TestingSFTPWithoutSSHConnectionHandler
+
+    def get_host_key():
+        return None
 
 
 class SFTPAbsoluteServer(SFTPServerWithoutSSH):
@@ -558,7 +564,9 @@
     It does this by serving from a deeply-nested directory that doesn't exist.
     """
 
-    def start_server(self, backing_server=None):
-        self._server_homedir = '/dev/noone/runs/tests/here'
-        super(SFTPSiblingAbsoluteServer, self).start_server(backing_server)
+    def create_server(self):
+        # FIXME: Can't we do that in a cleaner way ? -- vila 20100623
+        server = super(SFTPSiblingAbsoluteServer, self).create_server()
+        server._server_homedir = '/dev/noone/runs/tests/here'
+        return server
 

=== modified file 'bzrlib/tests/test__bencode.py'
--- bzrlib/tests/test__bencode.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test__bencode.py	2010-09-17 04:32:50 +0000
@@ -16,6 +16,8 @@
 
 """Tests for bencode structured encoding"""
 
+import sys
+
 from bzrlib import tests
 
 def load_tests(standard_tests, module, loader):
@@ -105,7 +107,14 @@
                             'd8:spam.mp3d6:author5:Alice6:lengthi100000eee')
 
     def test_dict_deepnested(self):
-        self._run_check_error(RuntimeError, ("d0:" * 10000) + 'i1e' + ("e" * 10000))
+        # The recursion here provokes CPython into emitting a warning on
+        # stderr, "maximum recursion depth exceeded in __subclasscheck__", due
+        # to running out of stack space while evaluating "except (...):" in
+        # _bencode_py.  This is harmless, so we temporarily override stderr to
+        # avoid distracting noise in the test output.
+        self.overrideAttr(sys, 'stderr', self._log_file)
+        self._run_check_error(
+            RuntimeError, ("d0:" * 10000) + 'i1e' + ("e" * 10000))
 
     def test_malformed_dict(self):
         self._run_check_error(ValueError, 'd')

=== added file 'bzrlib/tests/test__btree_serializer.py'
--- bzrlib/tests/test__btree_serializer.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test__btree_serializer.py	2010-08-23 17:51:17 +0000
@@ -0,0 +1,305 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+#
+
+"""Direct tests of the btree serializer extension"""
+
+import binascii
+import bisect
+
+from bzrlib import tests
+
+from bzrlib.tests.test_btree_index import compiled_btreeparser_feature
+
+
+class TestBtreeSerializer(tests.TestCase):
+
+    _test_needs_features = [compiled_btreeparser_feature]
+
+    def setUp(self):
+        super(TestBtreeSerializer, self).setUp()
+        self.module = compiled_btreeparser_feature.module
+
+
+class TestHexAndUnhex(TestBtreeSerializer):
+
+    def assertHexlify(self, as_binary):
+        self.assertEqual(binascii.hexlify(as_binary),
+                         self.module._py_hexlify(as_binary))
+
+    def assertUnhexlify(self, as_hex):
+        ba_unhex = binascii.unhexlify(as_hex)
+        mod_unhex = self.module._py_unhexlify(as_hex)
+        if ba_unhex != mod_unhex:
+            if mod_unhex is None:
+                mod_hex = '<None>'
+            else:
+                mod_hex = binascii.hexlify(mod_unhex)
+            self.fail('_py_unhexlify returned a different answer'
+                      ' from binascii:\n    %s\n != %s'
+                      % (binascii.hexlify(ba_unhex), mod_hex))
+
+    def assertFailUnhexlify(self, as_hex):
+        # Invalid hex content
+        self.assertIs(None, self.module._py_unhexlify(as_hex))
+
+    def test_to_hex(self):
+        raw_bytes = ''.join(map(chr, range(256)))
+        for i in range(0, 240, 20):
+            self.assertHexlify(raw_bytes[i:i+20])
+        self.assertHexlify(raw_bytes[240:]+raw_bytes[0:4])
+
+    def test_from_hex(self):
+        self.assertUnhexlify('0123456789abcdef0123456789abcdef01234567')
+        self.assertUnhexlify('123456789abcdef0123456789abcdef012345678')
+        self.assertUnhexlify('0123456789ABCDEF0123456789ABCDEF01234567')
+        self.assertUnhexlify('123456789ABCDEF0123456789ABCDEF012345678')
+        hex_chars = binascii.hexlify(''.join(map(chr, range(256))))
+        for i in range(0, 480, 40):
+            self.assertUnhexlify(hex_chars[i:i+40])
+        self.assertUnhexlify(hex_chars[480:]+hex_chars[0:8])
+
+    def test_from_invalid_hex(self):
+        self.assertFailUnhexlify('123456789012345678901234567890123456789X')
+        self.assertFailUnhexlify('12345678901234567890123456789012345678X9')
+
+
+_hex_form = '123456789012345678901234567890abcdefabcd'
+
+class Test_KeyToSha1(TestBtreeSerializer):
+
+    def assertKeyToSha1(self, expected, key):
+        if expected is None:
+            expected_bin = None
+        else:
+            expected_bin = binascii.unhexlify(expected)
+        actual_sha1 = self.module._py_key_to_sha1(key)
+        if expected_bin != actual_sha1:
+            actual_hex_sha1 = None
+            if actual_sha1 is not None:
+                actual_hex_sha1 = binascii.hexlify(actual_sha1)
+            self.fail('_key_to_sha1 returned:\n    %s\n != %s'
+                      % (actual_sha1, expected))
+
+    def test_simple(self):
+        self.assertKeyToSha1(_hex_form, ('sha1:' + _hex_form,))
+
+    def test_invalid_not_tuple(self):
+        self.assertKeyToSha1(None, _hex_form)
+        self.assertKeyToSha1(None, 'sha1:' + _hex_form)
+
+    def test_invalid_empty(self):
+        self.assertKeyToSha1(None, ())
+
+    def test_invalid_not_string(self):
+        self.assertKeyToSha1(None, (None,))
+        self.assertKeyToSha1(None, (list(_hex_form),))
+
+    def test_invalid_not_sha1(self):
+        self.assertKeyToSha1(None, (_hex_form,))
+        self.assertKeyToSha1(None, ('sha2:' + _hex_form,))
+
+    def test_invalid_not_hex(self):
+        self.assertKeyToSha1(None,
+            ('sha1:abcdefghijklmnopqrstuvwxyz12345678901234',))
+
+
+class Test_Sha1ToKey(TestBtreeSerializer):
+
+    def assertSha1ToKey(self, hex_sha1):
+        bin_sha1 = binascii.unhexlify(hex_sha1)
+        key = self.module._py_sha1_to_key(bin_sha1)
+        self.assertEqual(('sha1:' + hex_sha1,), key)
+
+    def test_simple(self):
+        self.assertSha1ToKey(_hex_form)
+
+
+_one_key_content = """type=leaf
+sha1:123456789012345678901234567890abcdefabcd\x00\x001 2 3 4
+"""
+
+_large_offsets = """type=leaf
+sha1:123456789012345678901234567890abcdefabcd\x00\x0012345678901 1234567890 0 1
+sha1:abcd123456789012345678901234567890abcdef\x00\x002147483648 2147483647 0 1
+sha1:abcdefabcd123456789012345678901234567890\x00\x004294967296 4294967295 4294967294 1
+"""
+
+_multi_key_content = """type=leaf
+sha1:c80c881d4a26984ddce795f6f71817c9cf4480e7\x00\x000 0 0 0
+sha1:c86f7e437faa5a7fce15d1ddcb9eaeaea377667b\x00\x001 1 1 1
+sha1:c8e240de74fb1ed08fa08d38063f6a6a91462a81\x00\x002 2 2 2
+sha1:cda39a3ee5e6b4b0d3255bfef95601890afd8070\x00\x003 3 3 3
+sha1:cdf51e37c269aa94d38f93e537bf6e2020b21406\x00\x004 4 4 4
+sha1:ce0c9035898dd52fc65c41454cec9c4d2611bfb3\x00\x005 5 5 5
+sha1:ce93b4e3c464ffd51732fbd6ded717e9efda28aa\x00\x006 6 6 6
+sha1:cf7a9e24777ec23212c54d7a350bc5bea5477fdb\x00\x007 7 7 7
+"""
+
+_multi_key_same_offset = """type=leaf
+sha1:080c881d4a26984ddce795f6f71817c9cf4480e7\x00\x000 0 0 0
+sha1:c86f7e437faa5a7fce15d1ddcb9eaeaea377667b\x00\x001 1 1 1
+sha1:cd0c9035898dd52fc65c41454cec9c4d2611bfb3\x00\x002 2 2 2
+sha1:cda39a3ee5e6b4b0d3255bfef95601890afd8070\x00\x003 3 3 3
+sha1:cde240de74fb1ed08fa08d38063f6a6a91462a81\x00\x004 4 4 4
+sha1:cdf51e37c269aa94d38f93e537bf6e2020b21406\x00\x005 5 5 5
+sha1:ce7a9e24777ec23212c54d7a350bc5bea5477fdb\x00\x006 6 6 6
+sha1:ce93b4e3c464ffd51732fbd6ded717e9efda28aa\x00\x007 7 7 7
+"""
+
+_common_32_bits = """type=leaf
+sha1:123456784a26984ddce795f6f71817c9cf4480e7\x00\x000 0 0 0
+sha1:1234567874fb1ed08fa08d38063f6a6a91462a81\x00\x001 1 1 1
+sha1:12345678777ec23212c54d7a350bc5bea5477fdb\x00\x002 2 2 2
+sha1:123456787faa5a7fce15d1ddcb9eaeaea377667b\x00\x003 3 3 3
+sha1:12345678898dd52fc65c41454cec9c4d2611bfb3\x00\x004 4 4 4
+sha1:12345678c269aa94d38f93e537bf6e2020b21406\x00\x005 5 5 5
+sha1:12345678c464ffd51732fbd6ded717e9efda28aa\x00\x006 6 6 6
+sha1:12345678e5e6b4b0d3255bfef95601890afd8070\x00\x007 7 7 7
+"""
+
+
+class TestGCCKHSHA1LeafNode(TestBtreeSerializer):
+
+    def assertInvalid(self, bytes):
+        """Ensure that we get a proper error when trying to parse invalid bytes.
+
+        (mostly this is testing that bad input doesn't cause us to segfault)
+        """
+        self.assertRaises((ValueError, TypeError), 
+                          self.module._parse_into_chk, bytes, 1, 0)
+
+    def test_non_str(self):
+        self.assertInvalid(u'type=leaf\n')
+
+    def test_not_leaf(self):
+        self.assertInvalid('type=internal\n')
+
+    def test_empty_leaf(self):
+        leaf = self.module._parse_into_chk('type=leaf\n', 1, 0)
+        self.assertEqual(0, len(leaf))
+        self.assertEqual([], leaf.all_items())
+        self.assertEqual([], leaf.all_keys())
+        # It should allow any key to be queried
+        self.assertFalse(('key',) in leaf)
+
+    def test_one_key_leaf(self):
+        leaf = self.module._parse_into_chk(_one_key_content, 1, 0)
+        self.assertEqual(1, len(leaf))
+        sha_key = ('sha1:' + _hex_form,)
+        self.assertEqual([sha_key], leaf.all_keys())
+        self.assertEqual([(sha_key, ('1 2 3 4', ()))], leaf.all_items())
+        self.assertTrue(sha_key in leaf)
+
+    def test_large_offsets(self):
+        leaf = self.module._parse_into_chk(_large_offsets, 1, 0)
+        self.assertEqual(['12345678901 1234567890 0 1',
+                          '2147483648 2147483647 0 1',
+                          '4294967296 4294967295 4294967294 1',
+                         ], [x[1][0] for x in leaf.all_items()])
+
+    def test_many_key_leaf(self):
+        leaf = self.module._parse_into_chk(_multi_key_content, 1, 0)
+        self.assertEqual(8, len(leaf))
+        all_keys = leaf.all_keys()
+        self.assertEqual(8, len(leaf.all_keys()))
+        for idx, key in enumerate(all_keys):
+            self.assertEqual(str(idx), leaf[key][0].split()[0])
+
+    def test_common_shift(self):
+        # The keys were deliberately chosen so that the first 5 bits all
+        # overlapped, it also happens that a later bit overlaps
+        # Note that by 'overlap' we mean that given bit is either on in all
+        # keys, or off in all keys
+        leaf = self.module._parse_into_chk(_multi_key_content, 1, 0)
+        self.assertEqual(19, leaf.common_shift)
+        # The interesting byte for each key is
+        # (defined as the 8-bits that come after the common prefix)
+        lst = [1, 13, 28, 180, 190, 193, 210, 239]
+        offsets = leaf._get_offsets()
+        self.assertEqual([bisect.bisect_left(lst, x) for x in range(0, 257)],
+                         offsets)
+        for idx, val in enumerate(lst):
+            self.assertEqual(idx, offsets[val])
+        for idx, key in enumerate(leaf.all_keys()):
+            self.assertEqual(str(idx), leaf[key][0].split()[0])
+
+    def test_multi_key_same_offset(self):
+        # there is no common prefix, though there are some common bits
+        leaf = self.module._parse_into_chk(_multi_key_same_offset, 1, 0)
+        self.assertEqual(24, leaf.common_shift)
+        offsets = leaf._get_offsets()
+        # The interesting byte is just the first 8-bits of the key
+        lst = [8, 200, 205, 205, 205, 205, 206, 206]
+        self.assertEqual([bisect.bisect_left(lst, x) for x in range(0, 257)],
+                         offsets)
+        for val in lst:
+            self.assertEqual(lst.index(val), offsets[val])
+        for idx, key in enumerate(leaf.all_keys()):
+            self.assertEqual(str(idx), leaf[key][0].split()[0])
+
+    def test_all_common_prefix(self):
+        # The first 32 bits of all hashes are the same. This is going to be
+        # pretty much impossible, but I don't want to fail because of this
+        leaf = self.module._parse_into_chk(_common_32_bits, 1, 0)
+        self.assertEqual(0, leaf.common_shift)
+        lst = [0x78] * 8
+        offsets = leaf._get_offsets()
+        self.assertEqual([bisect.bisect_left(lst, x) for x in range(0, 257)],
+                         offsets)
+        for val in lst:
+            self.assertEqual(lst.index(val), offsets[val])
+        for idx, key in enumerate(leaf.all_keys()):
+            self.assertEqual(str(idx), leaf[key][0].split()[0])
+
+    def test_many_entries(self):
+        # Again, this is almost impossible, but we should still work
+        # It would be hard to fit more that 120 entries in a 4k page, much less
+        # more than 256 of them. but hey, weird stuff happens sometimes
+        lines = ['type=leaf\n']
+        for i in range(500):
+            key_str = 'sha1:%04x%s' % (i, _hex_form[:36])
+            key = (key_str,)
+            lines.append('%s\0\0%d %d %d %d\n' % (key_str, i, i, i, i))
+        bytes = ''.join(lines)
+        leaf = self.module._parse_into_chk(bytes, 1, 0)
+        self.assertEqual(24-7, leaf.common_shift)
+        offsets = leaf._get_offsets()
+        # This is the interesting bits for each entry
+        lst = [x // 2 for x in range(500)]
+        expected_offsets = [x * 2 for x in range(128)] + [255]*129
+        self.assertEqual(expected_offsets, offsets)
+        # We truncate because offsets is an unsigned char. So the bisection
+        # will just say 'greater than the last one' for all the rest
+        lst = lst[:255]
+        self.assertEqual([bisect.bisect_left(lst, x) for x in range(0, 257)],
+                         offsets)
+        for val in lst:
+            self.assertEqual(lst.index(val), offsets[val])
+        for idx, key in enumerate(leaf.all_keys()):
+            self.assertEqual(str(idx), leaf[key][0].split()[0])
+
+    def test__sizeof__(self):
+        # We can't use the exact numbers because of platform variations, etc.
+        # But what we really care about is that it does get bigger with more
+        # content.
+        leaf0 = self.module._parse_into_chk('type=leaf\n', 1, 0)
+        leaf1 = self.module._parse_into_chk(_one_key_content, 1, 0)
+        leafN = self.module._parse_into_chk(_multi_key_content, 1, 0)
+        sizeof_1 = leaf1.__sizeof__() - leaf0.__sizeof__()
+        self.assertTrue(sizeof_1 > 0)
+        sizeof_N = leafN.__sizeof__() - leaf0.__sizeof__()
+        self.assertEqual(sizeof_1 * len(leafN), sizeof_N)

=== modified file 'bzrlib/tests/test__chk_map.py'
--- bzrlib/tests/test__chk_map.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test__chk_map.py	2010-05-11 10:45:26 +0000
@@ -18,6 +18,7 @@
 
 from bzrlib import (
     chk_map,
+    inventory,
     tests,
     )
 from bzrlib.static_tuple import StaticTuple
@@ -236,3 +237,44 @@
         self.assertEqual(("sha1:1234",), node.key())
         self.assertEqual('pref\x00fo', node._search_prefix)
         self.assertEqual({'pref\x00fo\x00': ('sha1:abcd',)}, node._items)
+
+
+class Test_BytesToTextKey(tests.TestCase):
+
+    def assertBytesToTextKey(self, key, bytes):
+        self.assertEqual(key,
+                         self.module._bytes_to_text_key(bytes))
+
+    def assertBytesToTextKeyRaises(self, bytes):
+        # These are invalid bytes, and we want to make sure the code under test
+        # raises an exception rather than segfaults, etc. We don't particularly
+        # care what exception.
+        self.assertRaises(Exception, self.module._bytes_to_text_key, bytes)
+
+    def test_file(self):
+        self.assertBytesToTextKey(('file-id', 'revision-id'),
+                 'file: file-id\nparent-id\nname\nrevision-id\n'
+                 'da39a3ee5e6b4b0d3255bfef95601890afd80709\n100\nN')
+
+    def test_invalid_no_kind(self):
+        self.assertBytesToTextKeyRaises(
+                 'file  file-id\nparent-id\nname\nrevision-id\n'
+                 'da39a3ee5e6b4b0d3255bfef95601890afd80709\n100\nN')
+
+    def test_invalid_no_space(self):
+        self.assertBytesToTextKeyRaises(
+                 'file:file-id\nparent-id\nname\nrevision-id\n'
+                 'da39a3ee5e6b4b0d3255bfef95601890afd80709\n100\nN')
+
+    def test_invalid_too_short_file_id(self):
+        self.assertBytesToTextKeyRaises('file:file-id')
+
+    def test_invalid_too_short_parent_id(self):
+        self.assertBytesToTextKeyRaises('file:file-id\nparent-id')
+
+    def test_invalid_too_short_name(self):
+        self.assertBytesToTextKeyRaises('file:file-id\nparent-id\nname')
+
+    def test_dir(self):
+        self.assertBytesToTextKey(('dir-id', 'revision-id'),
+                 'dir: dir-id\nparent-id\nname\nrevision-id')

=== modified file 'bzrlib/tests/test__dirstate_helpers.py'
--- bzrlib/tests/test__dirstate_helpers.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/test__dirstate_helpers.py	2010-06-02 04:50:35 +0000
@@ -737,7 +737,7 @@
 
     def test_trailing_garbage(self):
         tree, state, expected = self.create_basic_dirstate()
-        # On Linux, we can write extra data as long as we haven't read yet, but
+        # On Unix, we can write extra data as long as we haven't read yet, but
         # on Win32, if you've opened the file with FILE_SHARE_READ, trying to
         # open it in append mode will fail.
         state.unlock()

=== modified file 'bzrlib/tests/test__simple_set.py'
--- bzrlib/tests/test__simple_set.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test__simple_set.py	2010-08-02 19:20:30 +0000
@@ -379,3 +379,13 @@
         # And even removing an item still causes it to fail
         obj.discard(k2)
         self.assertRaises(RuntimeError, iterator.next)
+
+    def test__sizeof__(self):
+        # SimpleSet needs a custom sizeof implementation, because it allocates
+        # memory that Python cannot directly see (_table).
+        # Too much variability in platform sizes for us to give a fixed size
+        # here. However without a custom implementation, __sizeof__ would give
+        # us only the size of the object, and not its table. We know the table
+        # is at least 4bytes*1024entries in size.
+        obj = self.module.SimpleSet()
+        self.assertTrue(obj.__sizeof__() > 4096)

=== modified file 'bzrlib/tests/test__walkdirs_win32.py'
--- bzrlib/tests/test__walkdirs_win32.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test__walkdirs_win32.py	2010-05-16 12:21:31 +0000
@@ -74,7 +74,7 @@
         self.assertReadDir([('foo', 'foo', 'file', u'./foo')],
             '', u'.')
 
-    def test_dir(self):
+    def test_directory(self):
         self.build_tree(['bar/'])
         self.assertReadDir([('bar', 'bar', 'directory', u'./bar')],
             '', u'.')

=== modified file 'bzrlib/tests/test_branch.py'
--- bzrlib/tests/test_branch.py	2010-04-23 07:15:23 +0000
+++ bzrlib/tests/test_branch.py	2010-09-09 00:32:58 +0000
@@ -29,6 +29,7 @@
     bzrdir,
     config,
     errors,
+    symbol_versioning,
     tests,
     trace,
     transport,
@@ -86,18 +87,7 @@
         self.assertIsDirectory('.bzr/branch/lock/held', t)
 
     def test_set_push_location(self):
-        from bzrlib.config import (locations_config_filename,
-                                   ensure_config_dir_exists)
-        ensure_config_dir_exists()
-        fn = locations_config_filename()
-        # write correct newlines to locations.conf
-        # by default ConfigObj uses native line-endings for new files
-        # but uses already existing line-endings if file is not empty
-        f = open(fn, 'wb')
-        try:
-            f.write('# comment\n')
-        finally:
-            f.close()
+        conf = config.LocationConfig.from_string('# comment\n', '.', save=True)
 
         branch = self.make_branch('.', format='knit')
         branch.set_push_location('foo')
@@ -106,7 +96,7 @@
                              "[%s]\n"
                              "push_location = foo\n"
                              "push_location:policy = norecurse\n" % local_path,
-                             fn)
+                             config.locations_config_filename())
 
     # TODO RBC 20051029 test getting a push location from a branch in a
     # recursive section - that is, it appends the branch name.
@@ -136,6 +126,27 @@
         return "opened branch."
 
 
+# Demonstrating how lazy loading is often implemented:
+# A constant string is created.
+SampleSupportedBranchFormatString = "Sample supported branch format."
+
+# And the format class can then reference the constant to avoid skew.
+class SampleSupportedBranchFormat(_mod_branch.BranchFormat):
+    """A sample supported format."""
+
+    def get_format_string(self):
+        """See BzrBranchFormat.get_format_string()."""
+        return SampleSupportedBranchFormatString
+
+    def initialize(self, a_bzrdir, name=None):
+        t = a_bzrdir.get_branch_transport(self, name=name)
+        t.put_bytes('format', self.get_format_string())
+        return 'A branch'
+
+    def open(self, transport, name=None, _found=False, ignore_fallbacks=False):
+        return "opened supported branch."
+
+
 class TestBzrBranchFormat(tests.TestCaseWithTransport):
     """Tests for the BzrBranchFormat facility."""
 
@@ -152,6 +163,17 @@
             self.failUnless(isinstance(found_format, format.__class__))
         check_format(_mod_branch.BzrBranchFormat5(), "bar")
 
+    def test_find_format_factory(self):
+        dir = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
+        SampleSupportedBranchFormat().initialize(dir)
+        factory = _mod_branch.MetaDirBranchFormatFactory(
+            SampleSupportedBranchFormatString,
+            "bzrlib.tests.test_branch", "SampleSupportedBranchFormat")
+        _mod_branch.BranchFormat.register_format(factory)
+        self.addCleanup(_mod_branch.BranchFormat.unregister_format, factory)
+        b = _mod_branch.Branch.open(self.get_url())
+        self.assertEqual(b, "opened supported branch.")
+
     def test_find_format_not_branch(self):
         dir = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
         self.assertRaises(errors.NotBranchError,
@@ -186,6 +208,34 @@
         self.make_branch_and_tree('bar')
 
 
+#Used by TestMetaDirBranchFormatFactory 
+FakeLazyFormat = None
+
+
+class TestMetaDirBranchFormatFactory(tests.TestCase):
+
+    def test_get_format_string_does_not_load(self):
+        """Formats have a static format string."""
+        factory = _mod_branch.MetaDirBranchFormatFactory("yo", None, None)
+        self.assertEqual("yo", factory.get_format_string())
+
+    def test_call_loads(self):
+        # __call__ is used by the network_format_registry interface to get a
+        # Format.
+        global FakeLazyFormat
+        del FakeLazyFormat
+        factory = _mod_branch.MetaDirBranchFormatFactory(None,
+            "bzrlib.tests.test_branch", "FakeLazyFormat")
+        self.assertRaises(AttributeError, factory)
+
+    def test_call_returns_call_of_referenced_object(self):
+        global FakeLazyFormat
+        FakeLazyFormat = lambda:'called'
+        factory = _mod_branch.MetaDirBranchFormatFactory(None,
+            "bzrlib.tests.test_branch", "FakeLazyFormat")
+        self.assertEqual('called', factory())
+
+
 class TestBranch67(object):
     """Common tests for both branch 6 and 7 which are mostly the same."""
 
@@ -492,6 +542,15 @@
         self.assertTrue(hasattr(params, 'bzrdir'))
         self.assertTrue(hasattr(params, 'branch'))
 
+    def test_post_branch_init_hook_repr(self):
+        param_reprs = []
+        _mod_branch.Branch.hooks.install_named_hook('post_branch_init',
+            lambda params: param_reprs.append(repr(params)), None)
+        branch = self.make_branch('a')
+        self.assertLength(1, param_reprs)
+        param_repr = param_reprs[0]
+        self.assertStartsWith(param_repr, '<BranchInitHookParams of ')
+
     def test_post_switch_hook(self):
         from bzrlib import switch
         calls = []
@@ -564,8 +623,10 @@
         # this usage of results is not recommended for new code (because it
         # doesn't describe very well what happened), but for api stability
         # it's still supported
-        a = "%d revisions pulled" % r
-        self.assertEqual(a, "10 revisions pulled")
+        self.assertEqual(self.applyDeprecated(
+            symbol_versioning.deprecated_in((2, 3, 0)),
+            r.__int__),
+            10)
 
     def test_report_changed(self):
         r = _mod_branch.PullResult()

=== modified file 'bzrlib/tests/test_btree_index.py'
--- bzrlib/tests/test_btree_index.py	2010-03-25 17:09:53 +0000
+++ bzrlib/tests/test_btree_index.py	2010-08-03 20:56:39 +0000
@@ -27,6 +27,7 @@
     lru_cache,
     osutils,
     tests,
+    transport,
     )
 from bzrlib.tests import (
     TestCaseWithTransport,
@@ -34,7 +35,6 @@
     multiply_tests,
     split_suite_by_condition,
     )
-from bzrlib.transport import get_transport
 
 
 def load_tests(standard_tests, module, loader):
@@ -216,12 +216,12 @@
         leaf1_bytes = zlib.decompress(leaf1)
         sorted_node_keys = sorted(node[0] for node in nodes)
         node = btree_index._LeafNode(leaf1_bytes, 1, 0)
-        self.assertEqual(231, len(node.keys))
-        self.assertEqual(sorted_node_keys[:231], sorted(node.keys))
+        self.assertEqual(231, len(node))
+        self.assertEqual(sorted_node_keys[:231], node.all_keys())
         leaf2_bytes = zlib.decompress(leaf2)
         node = btree_index._LeafNode(leaf2_bytes, 1, 0)
-        self.assertEqual(400 - 231, len(node.keys))
-        self.assertEqual(sorted_node_keys[231:], sorted(node.keys))
+        self.assertEqual(400 - 231, len(node))
+        self.assertEqual(sorted_node_keys[231:], node.all_keys())
 
     def test_last_page_rounded_1_layer(self):
         builder = btree_index.BTreeBuilder(key_elements=1, reference_lists=0)
@@ -241,9 +241,9 @@
         leaf2 = content[74:]
         leaf2_bytes = zlib.decompress(leaf2)
         node = btree_index._LeafNode(leaf2_bytes, 1, 0)
-        self.assertEqual(10, len(node.keys))
+        self.assertEqual(10, len(node))
         sorted_node_keys = sorted(node[0] for node in nodes)
-        self.assertEqual(sorted_node_keys, sorted(node.keys))
+        self.assertEqual(sorted_node_keys, node.all_keys())
 
     def test_last_page_not_rounded_2_layer(self):
         builder = btree_index.BTreeBuilder(key_elements=1, reference_lists=0)
@@ -263,9 +263,9 @@
         leaf2 = content[8192:]
         leaf2_bytes = zlib.decompress(leaf2)
         node = btree_index._LeafNode(leaf2_bytes, 1, 0)
-        self.assertEqual(400 - 231, len(node.keys))
+        self.assertEqual(400 - 231, len(node))
         sorted_node_keys = sorted(node[0] for node in nodes)
-        self.assertEqual(sorted_node_keys[231:], sorted(node.keys))
+        self.assertEqual(sorted_node_keys[231:], node.all_keys())
 
     def test_three_level_tree_details(self):
         # The left most pointer in the second internal node in a row should
@@ -280,10 +280,10 @@
 
         for node in nodes:
             builder.add_node(*node)
-        transport = get_transport('trace+' + self.get_url(''))
-        size = transport.put_file('index', self.time(builder.finish))
+        t = transport.get_transport('trace+' + self.get_url(''))
+        size = t.put_file('index', self.time(builder.finish))
         del builder
-        index = btree_index.BTreeGraphIndex(transport, 'index', size)
+        index = btree_index.BTreeGraphIndex(t, 'index', size)
         # Seed the metadata, we're using internal calls now.
         index.key_count()
         self.assertEqual(3, len(index._row_lengths),
@@ -302,7 +302,7 @@
         # in the second node it points at
         pos = index._row_offsets[2] + internal_node2.offset + 1
         leaf = index._get_leaf_nodes([pos])[pos]
-        self.assertTrue(internal_node2.keys[0] in leaf.keys)
+        self.assertTrue(internal_node2.keys[0] in leaf)
 
     def test_2_leaves_2_2(self):
         builder = btree_index.BTreeBuilder(key_elements=2, reference_lists=2)
@@ -409,9 +409,9 @@
         self.assertEqual(None, builder._backing_indices[2])
         self.assertEqual(16, builder._backing_indices[3].key_count())
         # Now finish, and check we got a correctly ordered tree
-        transport = self.get_transport('')
-        size = transport.put_file('index', builder.finish())
-        index = btree_index.BTreeGraphIndex(transport, 'index', size)
+        t = self.get_transport('')
+        size = t.put_file('index', builder.finish())
+        index = btree_index.BTreeGraphIndex(t, 'index', size)
         nodes = list(index.iter_all_entries())
         self.assertEqual(sorted(nodes), nodes)
         self.assertEqual(16, len(nodes))
@@ -607,7 +607,7 @@
         for key, value, references in nodes:
             builder.add_node(key, value, references)
         stream = builder.finish()
-        trans = get_transport('trace+' + self.get_url())
+        trans = transport.get_transport('trace+' + self.get_url())
         size = trans.put_file('index', stream)
         return btree_index.BTreeGraphIndex(trans, 'index', size)
 
@@ -648,57 +648,57 @@
         self.assertEqual(0, len(index._leaf_node_cache))
 
     def test_trivial_constructor(self):
-        transport = get_transport('trace+' + self.get_url(''))
-        index = btree_index.BTreeGraphIndex(transport, 'index', None)
+        t = transport.get_transport('trace+' + self.get_url(''))
+        index = btree_index.BTreeGraphIndex(t, 'index', None)
         # Checks the page size at load, but that isn't logged yet.
-        self.assertEqual([], transport._activity)
+        self.assertEqual([], t._activity)
 
     def test_with_size_constructor(self):
-        transport = get_transport('trace+' + self.get_url(''))
-        index = btree_index.BTreeGraphIndex(transport, 'index', 1)
+        t = transport.get_transport('trace+' + self.get_url(''))
+        index = btree_index.BTreeGraphIndex(t, 'index', 1)
         # Checks the page size at load, but that isn't logged yet.
-        self.assertEqual([], transport._activity)
+        self.assertEqual([], t._activity)
 
     def test_empty_key_count_no_size(self):
         builder = btree_index.BTreeBuilder(key_elements=1, reference_lists=0)
-        transport = get_transport('trace+' + self.get_url(''))
-        transport.put_file('index', builder.finish())
-        index = btree_index.BTreeGraphIndex(transport, 'index', None)
-        del transport._activity[:]
-        self.assertEqual([], transport._activity)
+        t = transport.get_transport('trace+' + self.get_url(''))
+        t.put_file('index', builder.finish())
+        index = btree_index.BTreeGraphIndex(t, 'index', None)
+        del t._activity[:]
+        self.assertEqual([], t._activity)
         self.assertEqual(0, index.key_count())
         # The entire index should have been requested (as we generally have the
         # size available, and doing many small readvs is inappropriate).
         # We can't tell how much was actually read here, but - check the code.
-        self.assertEqual([('get', 'index')], transport._activity)
+        self.assertEqual([('get', 'index')], t._activity)
 
     def test_empty_key_count(self):
         builder = btree_index.BTreeBuilder(key_elements=1, reference_lists=0)
-        transport = get_transport('trace+' + self.get_url(''))
-        size = transport.put_file('index', builder.finish())
+        t = transport.get_transport('trace+' + self.get_url(''))
+        size = t.put_file('index', builder.finish())
         self.assertEqual(72, size)
-        index = btree_index.BTreeGraphIndex(transport, 'index', size)
-        del transport._activity[:]
-        self.assertEqual([], transport._activity)
+        index = btree_index.BTreeGraphIndex(t, 'index', size)
+        del t._activity[:]
+        self.assertEqual([], t._activity)
         self.assertEqual(0, index.key_count())
         # The entire index should have been read, as 4K > size
         self.assertEqual([('readv', 'index', [(0, 72)], False, None)],
-            transport._activity)
+                         t._activity)
 
     def test_non_empty_key_count_2_2(self):
         builder = btree_index.BTreeBuilder(key_elements=2, reference_lists=2)
         nodes = self.make_nodes(35, 2, 2)
         for node in nodes:
             builder.add_node(*node)
-        transport = get_transport('trace+' + self.get_url(''))
-        size = transport.put_file('index', builder.finish())
-        index = btree_index.BTreeGraphIndex(transport, 'index', size)
-        del transport._activity[:]
-        self.assertEqual([], transport._activity)
+        t = transport.get_transport('trace+' + self.get_url(''))
+        size = t.put_file('index', builder.finish())
+        index = btree_index.BTreeGraphIndex(t, 'index', size)
+        del t._activity[:]
+        self.assertEqual([], t._activity)
         self.assertEqual(70, index.key_count())
         # The entire index should have been read, as it is one page long.
         self.assertEqual([('readv', 'index', [(0, size)], False, None)],
-            transport._activity)
+            t._activity)
         self.assertEqual(1173, size)
 
     def test_with_offset_no_size(self):
@@ -722,13 +722,13 @@
 
     def test__read_nodes_no_size_one_page_reads_once(self):
         self.make_index(nodes=[(('key',), 'value', ())])
-        trans = get_transport('trace+' + self.get_url())
+        trans = transport.get_transport('trace+' + self.get_url())
         index = btree_index.BTreeGraphIndex(trans, 'index', None)
         del trans._activity[:]
         nodes = dict(index._read_nodes([0]))
         self.assertEqual([0], nodes.keys())
         node = nodes[0]
-        self.assertEqual([('key',)], node.keys.keys())
+        self.assertEqual([('key',)], node.all_keys())
         self.assertEqual([('get', 'index')], trans._activity)
 
     def test__read_nodes_no_size_multiple_pages(self):
@@ -736,7 +736,7 @@
         index.key_count()
         num_pages = index._row_offsets[-1]
         # Reopen with a traced transport and no size
-        trans = get_transport('trace+' + self.get_url())
+        trans = transport.get_transport('trace+' + self.get_url())
         index = btree_index.BTreeGraphIndex(trans, 'index', None)
         del trans._activity[:]
         nodes = dict(index._read_nodes([0]))
@@ -747,31 +747,31 @@
         nodes = self.make_nodes(160, 2, 2)
         for node in nodes:
             builder.add_node(*node)
-        transport = get_transport('trace+' + self.get_url(''))
-        size = transport.put_file('index', builder.finish())
+        t = transport.get_transport('trace+' + self.get_url(''))
+        size = t.put_file('index', builder.finish())
         self.assertEqual(17692, size)
-        index = btree_index.BTreeGraphIndex(transport, 'index', size)
-        del transport._activity[:]
-        self.assertEqual([], transport._activity)
+        index = btree_index.BTreeGraphIndex(t, 'index', size)
+        del t._activity[:]
+        self.assertEqual([], t._activity)
         self.assertEqual(320, index.key_count())
         # The entire index should not have been read.
         self.assertEqual([('readv', 'index', [(0, 4096)], False, None)],
-            transport._activity)
+                         t._activity)
 
     def test_validate_one_page(self):
         builder = btree_index.BTreeBuilder(key_elements=2, reference_lists=2)
         nodes = self.make_nodes(45, 2, 2)
         for node in nodes:
             builder.add_node(*node)
-        transport = get_transport('trace+' + self.get_url(''))
-        size = transport.put_file('index', builder.finish())
-        index = btree_index.BTreeGraphIndex(transport, 'index', size)
-        del transport._activity[:]
-        self.assertEqual([], transport._activity)
+        t = transport.get_transport('trace+' + self.get_url(''))
+        size = t.put_file('index', builder.finish())
+        index = btree_index.BTreeGraphIndex(t, 'index', size)
+        del t._activity[:]
+        self.assertEqual([], t._activity)
         index.validate()
         # The entire index should have been read linearly.
         self.assertEqual([('readv', 'index', [(0, size)], False, None)],
-            transport._activity)
+                         t._activity)
         self.assertEqual(1488, size)
 
     def test_validate_two_pages(self):
@@ -779,64 +779,65 @@
         nodes = self.make_nodes(80, 2, 2)
         for node in nodes:
             builder.add_node(*node)
-        transport = get_transport('trace+' + self.get_url(''))
-        size = transport.put_file('index', builder.finish())
+        t = transport.get_transport('trace+' + self.get_url(''))
+        size = t.put_file('index', builder.finish())
         # Root page, 2 leaf pages
         self.assertEqual(9339, size)
-        index = btree_index.BTreeGraphIndex(transport, 'index', size)
-        del transport._activity[:]
-        self.assertEqual([], transport._activity)
+        index = btree_index.BTreeGraphIndex(t, 'index', size)
+        del t._activity[:]
+        self.assertEqual([], t._activity)
         index.validate()
         # The entire index should have been read linearly.
-        self.assertEqual([('readv', 'index', [(0, 4096)], False, None),
-            ('readv', 'index', [(4096, 4096), (8192, 1147)], False, None)],
-            transport._activity)
+        self.assertEqual(
+            [('readv', 'index', [(0, 4096)], False, None),
+             ('readv', 'index', [(4096, 4096), (8192, 1147)], False, None)],
+            t._activity)
         # XXX: TODO: write some badly-ordered nodes, and some pointers-to-wrong
         # node and make validate find them.
 
     def test_eq_ne(self):
         # two indices are equal when constructed with the same parameters:
-        transport1 = get_transport('trace+' + self.get_url(''))
-        transport2 = get_transport(self.get_url(''))
-        self.assertTrue(
-            btree_index.BTreeGraphIndex(transport1, 'index', None) ==
-            btree_index.BTreeGraphIndex(transport1, 'index', None))
-        self.assertTrue(
-            btree_index.BTreeGraphIndex(transport1, 'index', 20) ==
-            btree_index.BTreeGraphIndex(transport1, 'index', 20))
-        self.assertFalse(
-            btree_index.BTreeGraphIndex(transport1, 'index', 20) ==
-            btree_index.BTreeGraphIndex(transport2, 'index', 20))
-        self.assertFalse(
-            btree_index.BTreeGraphIndex(transport1, 'inde1', 20) ==
-            btree_index.BTreeGraphIndex(transport1, 'inde2', 20))
-        self.assertFalse(
-            btree_index.BTreeGraphIndex(transport1, 'index', 10) ==
-            btree_index.BTreeGraphIndex(transport1, 'index', 20))
-        self.assertFalse(
-            btree_index.BTreeGraphIndex(transport1, 'index', None) !=
-            btree_index.BTreeGraphIndex(transport1, 'index', None))
-        self.assertFalse(
-            btree_index.BTreeGraphIndex(transport1, 'index', 20) !=
-            btree_index.BTreeGraphIndex(transport1, 'index', 20))
-        self.assertTrue(
-            btree_index.BTreeGraphIndex(transport1, 'index', 20) !=
-            btree_index.BTreeGraphIndex(transport2, 'index', 20))
-        self.assertTrue(
-            btree_index.BTreeGraphIndex(transport1, 'inde1', 20) !=
-            btree_index.BTreeGraphIndex(transport1, 'inde2', 20))
-        self.assertTrue(
-            btree_index.BTreeGraphIndex(transport1, 'index', 10) !=
-            btree_index.BTreeGraphIndex(transport1, 'index', 20))
+        t1 = transport.get_transport('trace+' + self.get_url(''))
+        t2 = transport.get_transport(self.get_url(''))
+        self.assertTrue(
+            btree_index.BTreeGraphIndex(t1, 'index', None) ==
+            btree_index.BTreeGraphIndex(t1, 'index', None))
+        self.assertTrue(
+            btree_index.BTreeGraphIndex(t1, 'index', 20) ==
+            btree_index.BTreeGraphIndex(t1, 'index', 20))
+        self.assertFalse(
+            btree_index.BTreeGraphIndex(t1, 'index', 20) ==
+            btree_index.BTreeGraphIndex(t2, 'index', 20))
+        self.assertFalse(
+            btree_index.BTreeGraphIndex(t1, 'inde1', 20) ==
+            btree_index.BTreeGraphIndex(t1, 'inde2', 20))
+        self.assertFalse(
+            btree_index.BTreeGraphIndex(t1, 'index', 10) ==
+            btree_index.BTreeGraphIndex(t1, 'index', 20))
+        self.assertFalse(
+            btree_index.BTreeGraphIndex(t1, 'index', None) !=
+            btree_index.BTreeGraphIndex(t1, 'index', None))
+        self.assertFalse(
+            btree_index.BTreeGraphIndex(t1, 'index', 20) !=
+            btree_index.BTreeGraphIndex(t1, 'index', 20))
+        self.assertTrue(
+            btree_index.BTreeGraphIndex(t1, 'index', 20) !=
+            btree_index.BTreeGraphIndex(t2, 'index', 20))
+        self.assertTrue(
+            btree_index.BTreeGraphIndex(t1, 'inde1', 20) !=
+            btree_index.BTreeGraphIndex(t1, 'inde2', 20))
+        self.assertTrue(
+            btree_index.BTreeGraphIndex(t1, 'index', 10) !=
+            btree_index.BTreeGraphIndex(t1, 'index', 20))
 
     def test_iter_all_only_root_no_size(self):
         self.make_index(nodes=[(('key',), 'value', ())])
-        trans = get_transport('trace+' + self.get_url(''))
-        index = btree_index.BTreeGraphIndex(trans, 'index', None)
-        del trans._activity[:]
+        t = transport.get_transport('trace+' + self.get_url(''))
+        index = btree_index.BTreeGraphIndex(t, 'index', None)
+        del t._activity[:]
         self.assertEqual([(('key',), 'value')],
                          [x[1:] for x in index.iter_all_entries()])
-        self.assertEqual([('get', 'index')], trans._activity)
+        self.assertEqual([('get', 'index')], t._activity)
 
     def test_iter_all_entries_reads(self):
         # iterating all entries reads the header, then does a linear
@@ -848,15 +849,15 @@
         nodes = self.make_nodes(10000, 2, 2)
         for node in nodes:
             builder.add_node(*node)
-        transport = get_transport('trace+' + self.get_url(''))
-        size = transport.put_file('index', builder.finish())
+        t = transport.get_transport('trace+' + self.get_url(''))
+        size = t.put_file('index', builder.finish())
         self.assertEqual(1303220, size, 'number of expected bytes in the'
                                         ' output changed')
         page_size = btree_index._PAGE_SIZE
         del builder
-        index = btree_index.BTreeGraphIndex(transport, 'index', size)
-        del transport._activity[:]
-        self.assertEqual([], transport._activity)
+        index = btree_index.BTreeGraphIndex(t, 'index', size)
+        del t._activity[:]
+        self.assertEqual([], t._activity)
         found_nodes = self.time(list, index.iter_all_entries())
         bare_nodes = []
         for node in found_nodes:
@@ -883,7 +884,7 @@
         readv_request[-1] = (readv_request[-1][0], 1303220 % page_size)
         expected = [('readv', 'index', [(0, page_size)], False, None),
              ('readv',  'index', readv_request, False, None)]
-        if expected != transport._activity:
+        if expected != t._activity:
             self.assertEqualDiff(pprint.pformat(expected),
                                  pprint.pformat(transport._activity))
 
@@ -903,12 +904,12 @@
         nodes = self.make_nodes(160, 2, 2)
         for node in nodes:
             builder.add_node(*node)
-        transport = get_transport('trace+' + self.get_url(''))
-        size = transport.put_file('index', builder.finish())
+        t = transport.get_transport('trace+' + self.get_url(''))
+        size = t.put_file('index', builder.finish())
         del builder
-        index = btree_index.BTreeGraphIndex(transport, 'index', size)
-        del transport._activity[:]
-        self.assertEqual([], transport._activity)
+        index = btree_index.BTreeGraphIndex(t, 'index', size)
+        del t._activity[:]
+        self.assertEqual([], t._activity)
         # search for one key
         found_nodes = list(index.iter_entries([nodes[30][0]]))
         bare_nodes = []
@@ -922,7 +923,7 @@
         # Should have read the root node, then one leaf page:
         self.assertEqual([('readv', 'index', [(0, 4096)], False, None),
              ('readv',  'index', [(8192, 4096), ], False, None)],
-            transport._activity)
+            t._activity)
 
     def test_iter_key_prefix_1_element_key_None(self):
         index = self.make_index()
@@ -1111,7 +1112,7 @@
         min_l2_key = l2.min_key
         max_l1_key = l1.max_key
         self.assertTrue(max_l1_key < min_l2_key)
-        parents_min_l2_key = l2.keys[min_l2_key][1][0]
+        parents_min_l2_key = l2[min_l2_key][1][0]
         self.assertEqual((l1.max_key,), parents_min_l2_key)
         # Now, whatever key we select that would fall on the second page,
         # should give us all the parents until the page break
@@ -1130,7 +1131,7 @@
         parent_map = {}
         search_keys = index._find_ancestors([max_l1_key], 0, parent_map,
                                             missing_keys)
-        self.assertEqual(sorted(l1.keys), sorted(parent_map))
+        self.assertEqual(l1.all_keys(), sorted(parent_map))
         self.assertEqual(set(), missing_keys)
         self.assertEqual(set(), search_keys)
 
@@ -1152,7 +1153,7 @@
         for node in nodes:
             builder.add_node(*node)
         stream = builder.finish()
-        trans = get_transport(self.get_url())
+        trans = transport.get_transport(self.get_url())
         size = trans.put_file('index', stream)
         index = btree_index.BTreeGraphIndex(trans, 'index', size)
         self.assertEqual(500, index.key_count())
@@ -1204,7 +1205,7 @@
             ("2222222222222222222222222222222222222222",): ("value:2", ()),
             ("3333333333333333333333333333333333333333",): ("value:3", ()),
             ("4444444444444444444444444444444444444444",): ("value:4", ()),
-            }, node.keys)
+            }, dict(node.all_items()))
 
     def test_LeafNode_2_2(self):
         node_bytes = ("type=leaf\n"
@@ -1224,7 +1225,7 @@
             ('11', '33'): ('value:3',
                 ((('11', 'ref22'),), (('11', 'ref22'), ('11', 'ref22')))),
             ('11', '44'): ('value:4', ((), (('11', 'ref00'),)))
-            }, node.keys)
+            }, dict(node.all_items()))
 
     def test_InternalNode_1(self):
         node_bytes = ("type=internal\n"
@@ -1265,7 +1266,7 @@
             ('11', '33'): ('value:3',
                 ((('11', 'ref22'),), (('11', 'ref22'), ('11', 'ref22')))),
             ('11', '44'): ('value:4', ((), (('11', 'ref00'),)))
-            }, node.keys)
+            }, dict(node.all_items()))
 
     def assertFlattened(self, expected, key, value, refs):
         flat_key, flat_line = self.parse_btree._flatten_node(
@@ -1349,8 +1350,8 @@
         This doesn't actually create anything on disk, it just primes a
         BTreeGraphIndex with the recommended information.
         """
-        index = btree_index.BTreeGraphIndex(get_transport('memory:///'),
-                                            'test-index', size=size)
+        index = btree_index.BTreeGraphIndex(
+            transport.get_transport('memory:///'), 'test-index', size=size)
         if recommended_pages is not None:
             index._recommended_pages = recommended_pages
         return index

=== modified file 'bzrlib/tests/test_bugtracker.py'
--- bzrlib/tests/test_bugtracker.py	2010-01-25 15:55:48 +0000
+++ bzrlib/tests/test_bugtracker.py	2010-09-25 20:08:01 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -117,6 +117,14 @@
         self.assertEqual('http://bugs.com/1234/view.html',
                          tracker.get_bug_url('1234'))
 
+    def test_generic_registered_non_integer(self):
+        branch = self.make_branch('some_branch')
+        config = branch.get_config()
+        config.set_user_option('bugtracker_foo_url', 'http://bugs.com/{id}/view.html')
+        tracker = bugtracker.tracker_registry.get_tracker('foo', branch)
+        self.assertEqual('http://bugs.com/ABC-1234/view.html',
+                         tracker.get_bug_url('ABC-1234'))
+
     def test_generic_incorrect_url(self):
         branch = self.make_branch('some_branch')
         config = branch.get_config()
@@ -173,15 +181,13 @@
         self.assertRaises(
             errors.MalformedBugIdentifier, tracker.check_bug_id, 'red')
 
-
-class TestURLParametrizedIntegerBugTracker(TestCaseWithMemoryTransport):
-    """Tests for TracTracker."""
+class TestURLParametrizedBugTracker(TestCaseWithMemoryTransport):
+    """Tests for URLParametrizedBugTracker."""
 
     def setUp(self):
         TestCaseWithMemoryTransport.setUp(self)
         self.url = 'http://twistedmatrix.com/trac'
-        self.tracker = bugtracker.URLParametrizedIntegerBugTracker('some',
-                                                                   'ticket/')
+        self.tracker = bugtracker.URLParametrizedBugTracker('some', 'ticket/')
 
     def test_get_with_unsupported_tag(self):
         """If asked for an unrecognized or unconfigured tag, return None."""
@@ -203,6 +209,22 @@
             urlutils.join(self.url, 'ticket/') + '1234',
             tracker.get_bug_url('1234'))
 
+    def test_get_bug_url_for_integer_id(self):
+        self.tracker.check_bug_id('1234')
+
+    def test_get_bug_url_for_non_integer_id(self):
+        self.tracker.check_bug_id('ABC-1234')
+
+
+class TestURLParametrizedIntegerBugTracker(TestCaseWithMemoryTransport):
+    """Tests for URLParametrizedIntegerBugTracker."""
+
+    def setUp(self):
+        TestCaseWithMemoryTransport.setUp(self)
+        self.url = 'http://twistedmatrix.com/trac'
+        self.tracker = bugtracker.URLParametrizedIntegerBugTracker('some',
+                                                                   'ticket/')
+
     def test_get_bug_url_for_bad_bug(self):
         """When given a bug identifier that is invalid for Trac, get_bug_url
         should raise an error.

=== modified file 'bzrlib/tests/test_bundle.py'
--- bzrlib/tests/test_bundle.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/test_bundle.py	2010-08-05 04:21:03 +0000
@@ -114,12 +114,12 @@
             ie = InventoryDirectory(file_id, name, parent_id)
         elif kind == 'file':
             ie = InventoryFile(file_id, name, parent_id)
+            ie.text_sha1 = text_sha_1
+            ie.text_size = text_size
         elif kind == 'symlink':
             ie = InventoryLink(file_id, name, parent_id)
         else:
             raise errors.BzrError('unknown kind %r' % kind)
-        ie.text_sha1 = text_sha_1
-        ie.text_size = text_size
         return ie
 
     def add_dir(self, file_id, path):

=== modified file 'bzrlib/tests/test_bzrdir.py'
--- bzrlib/tests/test_bzrdir.py	2010-03-25 14:22:41 +0000
+++ bzrlib/tests/test_bzrdir.py	2010-08-20 19:07:17 +0000
@@ -24,7 +24,9 @@
 import sys
 
 from bzrlib import (
+    branch,
     bzrdir,
+    controldir,
     errors,
     help_topics,
     repository,
@@ -54,6 +56,7 @@
 from bzrlib.transport import (
     get_transport,
     memory,
+    pathfilter,
     )
 from bzrlib.transport.http._urllib import HttpTransport_urllib
 from bzrlib.transport.nosmart import NoSmartTransportDecorator
@@ -67,37 +70,37 @@
         old_format = bzrdir.BzrDirFormat.get_default_format()
         # default is BzrDirFormat6
         self.failUnless(isinstance(old_format, bzrdir.BzrDirMetaFormat1))
-        bzrdir.BzrDirFormat._set_default_format(SampleBzrDirFormat())
+        controldir.ControlDirFormat._set_default_format(SampleBzrDirFormat())
         # creating a bzr dir should now create an instrumented dir.
         try:
             result = bzrdir.BzrDir.create('memory:///')
             self.failUnless(isinstance(result, SampleBzrDir))
         finally:
-            bzrdir.BzrDirFormat._set_default_format(old_format)
+            controldir.ControlDirFormat._set_default_format(old_format)
         self.assertEqual(old_format, bzrdir.BzrDirFormat.get_default_format())
 
 
 class TestFormatRegistry(TestCase):
 
     def make_format_registry(self):
-        my_format_registry = bzrdir.BzrDirFormatRegistry()
+        my_format_registry = controldir.ControlDirFormatRegistry()
         my_format_registry.register('weave', bzrdir.BzrDirFormat6,
             'Pre-0.8 format.  Slower and does not support checkouts or shared'
             ' repositories', deprecated=True)
         my_format_registry.register_lazy('lazy', 'bzrlib.bzrdir',
             'BzrDirFormat6', 'Format registered lazily', deprecated=True)
-        my_format_registry.register_metadir('knit',
+        bzrdir.register_metadir(my_format_registry, 'knit',
             'bzrlib.repofmt.knitrepo.RepositoryFormatKnit1',
             'Format using knits',
             )
         my_format_registry.set_default('knit')
-        my_format_registry.register_metadir(
+        bzrdir.register_metadir(my_format_registry,
             'branch6',
             'bzrlib.repofmt.knitrepo.RepositoryFormatKnit3',
             'Experimental successor to knit.  Use at your own risk.',
             branch_format='bzrlib.branch.BzrBranchFormat6',
             experimental=True)
-        my_format_registry.register_metadir(
+        bzrdir.register_metadir(my_format_registry,
             'hidden format',
             'bzrlib.repofmt.knitrepo.RepositoryFormatKnit3',
             'Experimental successor to knit.  Use at your own risk.',
@@ -172,7 +175,7 @@
             bzrdir.format_registry.set_default_repository(old_default)
 
     def test_aliases(self):
-        a_registry = bzrdir.BzrDirFormatRegistry()
+        a_registry = controldir.ControlDirFormatRegistry()
         a_registry.register('weave', bzrdir.BzrDirFormat6,
             'Pre-0.8 format.  Slower and does not support checkouts or shared'
             ' repositories', deprecated=True)
@@ -807,6 +810,38 @@
         self.assertEqualBzrdirs([baz, foo, bar],
                                 bzrdir.BzrDir.find_bzrdirs(transport))
 
+    def make_fake_permission_denied_transport(self, transport, paths):
+        """Create a transport that raises PermissionDenied for some paths."""
+        def filter(path):
+            if path in paths:
+                raise errors.PermissionDenied(path)
+            return path
+        path_filter_server = pathfilter.PathFilteringServer(transport, filter)
+        path_filter_server.start_server()
+        self.addCleanup(path_filter_server.stop_server)
+        path_filter_transport = pathfilter.PathFilteringTransport(
+            path_filter_server, '.')
+        return (path_filter_server, path_filter_transport)
+
+    def assertBranchUrlsEndWith(self, expect_url_suffix, actual_bzrdirs):
+        """Check that each branch url ends with the given suffix."""
+        for actual_bzrdir in actual_bzrdirs:
+            self.assertEndsWith(actual_bzrdir.user_url, expect_url_suffix)
+
+    def test_find_bzrdirs_permission_denied(self):
+        foo, bar, baz = self.make_foo_bar_baz()
+        transport = get_transport(self.get_url())
+        path_filter_server, path_filter_transport = \
+            self.make_fake_permission_denied_transport(transport, ['foo'])
+        # local transport
+        self.assertBranchUrlsEndWith('/baz/',
+            bzrdir.BzrDir.find_bzrdirs(path_filter_transport))
+        # smart server
+        smart_transport = self.make_smart_server('.',
+            backing_server=path_filter_server)
+        self.assertBranchUrlsEndWith('/baz/',
+            bzrdir.BzrDir.find_bzrdirs(smart_transport))
+
     def test_find_bzrdirs_list_current(self):
         def list_current(transport):
             return [s for s in transport.list_dir('') if s != 'baz']
@@ -817,7 +852,6 @@
                                 bzrdir.BzrDir.find_bzrdirs(transport,
                                     list_current=list_current))
 
-
     def test_find_bzrdirs_evaluate(self):
         def evaluate(bzrdir):
             try:
@@ -856,6 +890,21 @@
         self.assertEqual(bar.root_transport.base, branches[1].base)
 
 
+class TestMissingRepoBranchesSkipped(TestCaseWithMemoryTransport):
+
+    def test_find_bzrdirs_missing_repo(self):
+        transport = get_transport(self.get_url())
+        arepo = self.make_repository('arepo', shared=True)
+        abranch_url = arepo.user_url + '/abranch'
+        abranch = bzrdir.BzrDir.create(abranch_url).create_branch()
+        transport.delete_tree('arepo/.bzr')
+        self.assertRaises(errors.NoRepositoryPresent,
+            branch.Branch.open, abranch_url)
+        self.make_branch('baz')
+        for actual_bzrdir in bzrdir.BzrDir.find_branches(transport):
+            self.assertEndsWith(actual_bzrdir.user_url, '/baz/')
+
+
 class TestMeta1DirFormat(TestCaseWithTransport):
     """Tests specific to the meta1 dir format."""
 
@@ -1010,7 +1059,9 @@
     def _known_formats(self):
         return set([NotBzrDirFormat()])
 
-    @classmethod
+
+class NotBzrDirProber(controldir.Prober):
+
     def probe_transport(self, transport):
         """Our format is present if the transport ends in '.not/'."""
         if transport.has('.not'):
@@ -1030,16 +1081,17 @@
         dir = format.initialize(self.get_url())
         self.assertIsInstance(dir, NotBzrDir)
         # now probe for it.
-        bzrlib.bzrdir.BzrDirFormat.register_control_format(format)
+        controldir.ControlDirFormat.register_prober(NotBzrDirProber)
         try:
             found = bzrlib.bzrdir.BzrDirFormat.find_format(
                 get_transport(self.get_url()))
             self.assertIsInstance(found, NotBzrDirFormat)
         finally:
-            bzrlib.bzrdir.BzrDirFormat.unregister_control_format(format)
+            controldir.ControlDirFormat.unregister_prober(NotBzrDirProber)
 
     def test_included_in_known_formats(self):
-        bzrlib.bzrdir.BzrDirFormat.register_control_format(NotBzrDirFormat)
+        not_format = NotBzrDirFormat()
+        bzrlib.controldir.ControlDirFormat.register_format(not_format)
         try:
             formats = bzrlib.bzrdir.BzrDirFormat.known_formats()
             for format in formats:
@@ -1047,7 +1099,7 @@
                     return
             self.fail("No NotBzrDirFormat in %s" % formats)
         finally:
-            bzrlib.bzrdir.BzrDirFormat.unregister_control_format(NotBzrDirFormat)
+            bzrlib.controldir.ControlDirFormat.unregister_format(not_format)
 
 
 class NonLocalTests(TestCaseWithTransport):
@@ -1111,9 +1163,11 @@
     """
 
     def create_transport_readonly_server(self):
+        # We don't set the http protocol version, relying on the default
         return http_utils.HTTPServerRedirecting()
 
     def create_transport_secondary_server(self):
+        # We don't set the http protocol version, relying on the default
         return http_utils.HTTPServerRedirecting()
 
     def setUp(self):
@@ -1351,3 +1405,29 @@
         self.assertIsInstance(params, RepoInitHookParams)
         self.assertTrue(hasattr(params, 'bzrdir'))
         self.assertTrue(hasattr(params, 'repository'))
+
+    def test_post_repo_init_hook_repr(self):
+        param_reprs = []
+        bzrdir.BzrDir.hooks.install_named_hook('post_repo_init',
+            lambda params: param_reprs.append(repr(params)), None)
+        self.make_repository('foo')
+        self.assertLength(1, param_reprs)
+        param_repr = param_reprs[0]
+        self.assertStartsWith(param_repr, '<RepoInitHookParams for ')
+
+
+class TestGenerateBackupName(TestCaseWithMemoryTransport):
+
+    def setUp(self):
+        super(TestGenerateBackupName, self).setUp()
+        self._transport = get_transport(self.get_url())
+        bzrdir.BzrDir.create(self.get_url(),
+            possible_transports=[self._transport])
+        self._bzrdir = bzrdir.BzrDir.open_from_transport(self._transport)
+
+    def test_new(self):
+        self.assertEqual("a.~1~", self._bzrdir.generate_backup_name("a"))
+
+    def test_exiting(self):
+        self._transport.put_bytes("a.~1~", "some content")
+        self.assertEqual("a.~2~", self._bzrdir.generate_backup_name("a"))

=== modified file 'bzrlib/tests/test_chk_map.py'
--- bzrlib/tests/test_chk_map.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/test_chk_map.py	2010-07-21 09:58:42 +0000
@@ -467,6 +467,25 @@
         # updated key.
         self.assertEqual(new_root, chkmap._root_node._key)
 
+    def test_apply_delete_to_internal_node(self):
+        # applying a delta should be convert an internal root node to a leaf
+        # node if the delta shrinks the map enough.
+        store = self.get_chk_bytes()
+        chkmap = CHKMap(store, None)
+        # Add three items: 2 small enough to fit in one node, and one huge to
+        # force multiple nodes.
+        chkmap._root_node.set_maximum_size(100)
+        chkmap.map(('small',), 'value')
+        chkmap.map(('little',), 'value')
+        chkmap.map(('very-big',), 'x' * 100)
+        # (Check that we have constructed the scenario we want to test)
+        self.assertIsInstance(chkmap._root_node, InternalNode)
+        # Delete the huge item so that the map fits in one node again.
+        delta = [(('very-big',), None, None)]
+        chkmap.apply_delta(delta)
+        self.assertCanonicalForm(chkmap)
+        self.assertIsInstance(chkmap._root_node, LeafNode)
+
     def test_apply_new_keys_must_be_new(self):
         # applying a delta (None, "a", "b") to a map with 'a' in it generates
         # an error.

=== modified file 'bzrlib/tests/test_clean_tree.py'
--- bzrlib/tests/test_clean_tree.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_clean_tree.py	2010-08-23 17:54:07 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2009 Canonical Ltd
+# Copyright (C) 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,9 +15,14 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
+import errno
 import os
+import shutil
 from StringIO import StringIO
+import sys
+import types
 
+from bzrlib import tests, ui
 from bzrlib.bzrdir import (
     BzrDir,
     )
@@ -76,3 +81,60 @@
             self.assertEqual([], dels)
         finally:
             tree.unlock()
+
+    def test_delete_items_warnings(self):
+        """Ensure delete_items issues warnings on EACCES. (bug #430785)
+        """
+        def _dummy_unlink(path):
+            """unlink() files other than files named '0foo'.
+            """
+            if path.endswith('0foo'):
+                # Simulate 'permission denied' error.
+                # This should show up as a warning for the
+                # user.
+                e = OSError()
+                e.errno = errno.EACCES
+                raise e
+
+        def _dummy_rmtree(path, ignore_errors=False, onerror=None):
+            """Call user supplied error handler onerror.
+            """
+            self.assertTrue(isinstance(onerror, types.FunctionType))
+            # Indicate failure in removing 'path' if path is subdir0
+            # We later check to ensure that this is indicated
+            # to the user as a warning. We raise OSError to construct
+            # proper excinfo that needs to be passed to onerror
+            try:
+                raise OSError
+            except OSError, e:
+                e.errno = errno.EACCES
+                excinfo = sys.exc_info()
+                function = os.remove
+                if 'subdir0' not in path:
+                    # onerror should show warning only for os.remove
+                    # error. For any other failures the error should
+                    # be shown to the user.
+                    function = os.listdir
+                onerror(function=function,
+                    path=path, excinfo=excinfo)
+
+        self.overrideAttr(os, 'unlink', _dummy_unlink)
+        self.overrideAttr(shutil, 'rmtree', _dummy_rmtree)
+        stdout = tests.StringIOWrapper()
+        stderr = tests.StringIOWrapper()
+        ui.ui_factory = tests.TestUIFactory(stdout=stdout, stderr=stderr)
+
+        BzrDir.create_standalone_workingtree('.')
+        self.build_tree(['0foo', '1bar', '2baz', 'subdir0/'])
+        clean_tree('.', unknown=True, no_prompt=True)
+        self.assertContainsRe(stderr.getvalue(),
+            'bzr: warning: unable to remove.*0foo')
+        self.assertContainsRe(stderr.getvalue(),
+            'bzr: warning: unable to remove.*subdir0')
+
+        # Ensure that error other than EACCES during os.remove are
+        # not turned into warnings.
+        self.build_tree(['subdir1/'])
+        self.assertRaises(OSError, clean_tree, '.',
+            unknown=True, no_prompt=True)
+

=== modified file 'bzrlib/tests/test_cleanup.py'
--- bzrlib/tests/test_cleanup.py	2010-03-03 22:59:21 +0000
+++ bzrlib/tests/test_cleanup.py	2010-04-27 07:29:11 +0000
@@ -20,6 +20,7 @@
 from bzrlib.cleanup import (
     _do_with_cleanups,
     _run_cleanup,
+    ObjectWithCleanups,
     OperationWithCleanups,
     )
 from bzrlib.tests import TestCase
@@ -276,3 +277,17 @@
             [('func called', 'foo'), 'cleanup 1', 'cleanup 2', 'cleanup 3',
             'cleanup 4'], call_log)
 
+
+class SampleWithCleanups(ObjectWithCleanups):
+
+    pass
+
+
+class TestObjectWithCleanups(TestCase):
+
+    def test_object_with_cleanups(self):
+        a = []
+        s = SampleWithCleanups()
+        s.add_cleanup(a.append, 42)
+        s.cleanup_now()
+        self.assertEqual(a, [42])

=== modified file 'bzrlib/tests/test_cmdline.py'
--- bzrlib/tests/test_cmdline.py	2010-03-01 09:02:18 +0000
+++ bzrlib/tests/test_cmdline.py	2010-05-20 02:57:52 +0000
@@ -17,7 +17,9 @@
 
 from bzrlib import (
     cmdline,
-    tests)
+    tests,
+    )
+from bzrlib.tests.features import backslashdir_feature
 
 class TestSplitter(tests.TestCase):
 
@@ -91,3 +93,24 @@
             u'"x x" "y y"')
         self.assertAsTokens([(True, u'x x'), (True, u'y y')],
             u'"x x" \'y y\'', single_quotes_allowed=True)
+
+    def test_n_backslashes_handling(self):
+        # https://bugs.launchpad.net/bzr/+bug/528944
+        # actually we care about the doubled backslashes when they're
+        # represents UNC paths.
+        # But in fact there is too much weird corner cases
+        # (see https://bugs.launchpad.net/tortoisebzr/+bug/569050)
+        # so to reproduce every bit of windows command-line handling
+        # could be not worth of efforts?
+        self.requireFeature(backslashdir_feature)
+        self.assertAsTokens([(True, r'\\host\path')], r'"\\host\path"')
+        self.assertAsTokens([(False, r'\\host\path')], r'\\host\path')
+        # handling of " after the 2n and 2n+1 backslashes
+        # inside and outside the quoted string
+        self.assertAsTokens([(True, r'\\'), (False, r'*.py')], r'"\\\\" *.py')
+        self.assertAsTokens([(True, r'\\" *.py')], r'"\\\\\" *.py"')
+        self.assertAsTokens([(True, r'\\ *.py')], r'\\\\" *.py"')
+        self.assertAsTokens([(False, r'\\"'), (False, r'*.py')],
+                            r'\\\\\" *.py')
+        self.assertAsTokens([(True, u'\\\\')], u'"\\\\')
+

=== modified file 'bzrlib/tests/test_commands.py'
--- bzrlib/tests/test_commands.py	2010-04-22 18:36:13 +0000
+++ bzrlib/tests/test_commands.py	2010-08-28 16:56:36 +0000
@@ -16,6 +16,7 @@
 
 from cStringIO import StringIO
 import errno
+import inspect
 import sys
 
 from bzrlib import (
@@ -33,6 +34,17 @@
 
 class TestCommands(tests.TestCase):
 
+    def test_all_commands_have_help(self):
+        commands._register_builtin_commands()
+        commands_without_help = set()
+        base_doc = inspect.getdoc(commands.Command)
+        for cmd_name in commands.all_command_names():
+            cmd = commands.get_cmd_object(cmd_name)
+            cmd_help = cmd.help()
+            if not cmd_help or cmd_help == base_doc:
+                commands_without_help.append(cmd_name)
+        self.assertLength(0, commands_without_help)
+
     def test_display_command(self):
         """EPIPE message is selectively suppressed"""
         def pipe_thrower():
@@ -83,9 +95,7 @@
 class TestGetAlias(tests.TestCase):
 
     def _get_config(self, config_text):
-        my_config = config.GlobalConfig()
-        config_file = StringIO(config_text.encode('utf-8'))
-        my_config._parser = my_config._get_parser(file=config_file)
+        my_config = config.GlobalConfig.from_string(config_text)
         return my_config
 
     def test_simple(self):
@@ -334,6 +344,7 @@
         self.assertEqual(['called'], hook_calls)
         self.assertSubset(['foo', 'bar'], cmds)
 
+
 class TestDeprecations(tests.TestCase):
 
     def test_shlex_split_unicode_deprecation(self):

=== modified file 'bzrlib/tests/test_config.py'
--- bzrlib/tests/test_config.py	2010-04-23 07:15:23 +0000
+++ bzrlib/tests/test_config.py	2010-09-28 16:28:45 +0000
@@ -19,6 +19,7 @@
 from cStringIO import StringIO
 import os
 import sys
+import threading
 
 #import bzrlib specific imports here
 from bzrlib import (
@@ -35,9 +36,34 @@
     trace,
     transport,
     )
+from bzrlib.tests import features
 from bzrlib.util.configobj import configobj
 
 
+def lockable_config_scenarios():
+    return [
+        ('global',
+         {'config_class': config.GlobalConfig,
+          'config_args': [],
+          'config_section': 'DEFAULT'}),
+        ('locations',
+         {'config_class': config.LocationConfig,
+          'config_args': ['.'],
+          'config_section': '.'}),]
+
+
+def load_tests(standard_tests, module, loader):
+    suite = loader.suiteClass()
+
+    lc_tests, remaining_tests = tests.split_suite_by_condition(
+        standard_tests, tests.condition_isinstance((
+                TestLockableConfig,
+                )))
+    tests.multiply_tests(lc_tests, lockable_config_scenarios(), suite)
+    suite.addTest(remaining_tests)
+    return suite
+
+
 sample_long_alias="log -r-15..-1 --line"
 sample_config_text = u"""
 [DEFAULT]
@@ -129,6 +155,9 @@
         self._calls.append(('keys',))
         return []
 
+    def reload(self):
+        self._calls.append(('reload',))
+
     def write(self, arg):
         self._calls.append(('write',))
 
@@ -350,10 +379,6 @@
         self.assertEqual(config.config_filename(),
                          self.bzr_home + '/bazaar.conf')
 
-    def test_branches_config_filename(self):
-        self.assertEqual(config.branches_config_filename(),
-                         self.bzr_home + '/branches.conf')
-
     def test_locations_config_filename(self):
         self.assertEqual(config.locations_config_filename(),
                          self.bzr_home + '/locations.conf')
@@ -367,32 +392,213 @@
             '/home/bogus/.cache')
 
 
-class TestIniConfig(tests.TestCase):
+class TestIniConfig(tests.TestCaseInTempDir):
 
     def make_config_parser(self, s):
-        conf = config.IniBasedConfig(None)
-        parser = conf._get_parser(file=StringIO(s.encode('utf-8')))
-        return conf, parser
+        conf = config.IniBasedConfig.from_string(s)
+        return conf, conf._get_parser()
 
 
 class TestIniConfigBuilding(TestIniConfig):
 
     def test_contructs(self):
-        my_config = config.IniBasedConfig("nothing")
+        my_config = config.IniBasedConfig()
 
     def test_from_fp(self):
-        config_file = StringIO(sample_config_text.encode('utf-8'))
-        my_config = config.IniBasedConfig(None)
-        self.failUnless(
-            isinstance(my_config._get_parser(file=config_file),
-                        configobj.ConfigObj))
+        my_config = config.IniBasedConfig.from_string(sample_config_text)
+        self.assertIsInstance(my_config._get_parser(), configobj.ConfigObj)
 
     def test_cached(self):
-        config_file = StringIO(sample_config_text.encode('utf-8'))
-        my_config = config.IniBasedConfig(None)
-        parser = my_config._get_parser(file=config_file)
+        my_config = config.IniBasedConfig.from_string(sample_config_text)
+        parser = my_config._get_parser()
         self.failUnless(my_config._get_parser() is parser)
 
+    def _dummy_chown(self, path, uid, gid):
+        self.path, self.uid, self.gid = path, uid, gid
+
+    def test_ini_config_ownership(self):
+        """Ensure that chown is happening during _write_config_file"""
+        self.requireFeature(features.chown_feature)
+        self.overrideAttr(os, 'chown', self._dummy_chown)
+        self.path = self.uid = self.gid = None
+        conf = config.IniBasedConfig(file_name='./foo.conf')
+        conf._write_config_file()
+        self.assertEquals(self.path, './foo.conf')
+        self.assertTrue(isinstance(self.uid, int))
+        self.assertTrue(isinstance(self.gid, int))
+
+    def test_get_filename_parameter_is_deprecated_(self):
+        conf = self.callDeprecated([
+            'IniBasedConfig.__init__(get_filename) was deprecated in 2.3.'
+            ' Use file_name instead.'],
+            config.IniBasedConfig, lambda: 'ini.conf')
+        self.assertEqual('ini.conf', conf.file_name)
+
+    def test_get_parser_file_parameter_is_deprecated_(self):
+        config_file = StringIO(sample_config_text.encode('utf-8'))
+        conf = config.IniBasedConfig.from_string(sample_config_text)
+        conf = self.callDeprecated([
+            'IniBasedConfig._get_parser(file=xxx) was deprecated in 2.3.'
+            ' Use IniBasedConfig(_content=xxx) instead.'],
+            conf._get_parser, file=config_file)
+
+class TestIniConfigSaving(tests.TestCaseInTempDir):
+
+    def test_cant_save_without_a_file_name(self):
+        conf = config.IniBasedConfig()
+        self.assertRaises(AssertionError, conf._write_config_file)
+
+    def test_saved_with_content(self):
+        content = 'foo = bar\n'
+        conf = config.IniBasedConfig.from_string(
+            content, file_name='./test.conf', save=True)
+        self.assertFileEqual(content, 'test.conf')
+
+
+class TestIniBaseConfigOnDisk(tests.TestCaseInTempDir):
+
+    def test_cannot_reload_without_name(self):
+        conf = config.IniBasedConfig.from_string(sample_config_text)
+        self.assertRaises(AssertionError, conf.reload)
+
+    def test_reload_see_new_value(self):
+        c1 = config.IniBasedConfig.from_string('editor=vim\n',
+                                               file_name='./test/conf')
+        c1._write_config_file()
+        c2 = config.IniBasedConfig.from_string('editor=emacs\n',
+                                               file_name='./test/conf')
+        c2._write_config_file()
+        self.assertEqual('vim', c1.get_user_option('editor'))
+        self.assertEqual('emacs', c2.get_user_option('editor'))
+        # Make sure we get the Right value
+        c1.reload()
+        self.assertEqual('emacs', c1.get_user_option('editor'))
+
+
+class TestLockableConfig(tests.TestCaseInTempDir):
+
+    # Set by load_tests
+    config_class = None
+    config_args = None
+    config_section = None
+
+    def setUp(self):
+        super(TestLockableConfig, self).setUp()
+        self._content = '[%s]\none=1\ntwo=2\n' % (self.config_section,)
+        self.config = self.create_config(self._content)
+
+    def get_existing_config(self):
+        return self.config_class(*self.config_args)
+
+    def create_config(self, content):
+        kwargs = dict(save=True)
+        c = self.config_class.from_string(content, *self.config_args, **kwargs)
+        return c
+
+    def test_simple_read_access(self):
+        self.assertEquals('1', self.config.get_user_option('one'))
+
+    def test_simple_write_access(self):
+        self.config.set_user_option('one', 'one')
+        self.assertEquals('one', self.config.get_user_option('one'))
+
+    def test_listen_to_the_last_speaker(self):
+        c1 = self.config
+        c2 = self.get_existing_config()
+        c1.set_user_option('one', 'ONE')
+        c2.set_user_option('two', 'TWO')
+        self.assertEquals('ONE', c1.get_user_option('one'))
+        self.assertEquals('TWO', c2.get_user_option('two'))
+        # The second update respect the first one
+        self.assertEquals('ONE', c2.get_user_option('one'))
+
+    def test_last_speaker_wins(self):
+        # If the same config is not shared, the same variable modified twice
+        # can only see a single result.
+        c1 = self.config
+        c2 = self.get_existing_config()
+        c1.set_user_option('one', 'c1')
+        c2.set_user_option('one', 'c2')
+        self.assertEquals('c2', c2._get_user_option('one'))
+        # The first modification is still available until another refresh
+        # occur
+        self.assertEquals('c1', c1._get_user_option('one'))
+        c1.set_user_option('two', 'done')
+        self.assertEquals('c2', c1._get_user_option('one'))
+
+    def test_writes_are_serialized(self):
+        c1 = self.config
+        c2 = self.get_existing_config()
+
+        # We spawn a thread that will pause *during* the write
+        before_writing = threading.Event()
+        after_writing = threading.Event()
+        writing_done = threading.Event()
+        c1_orig = c1._write_config_file
+        def c1_write_config_file():
+            before_writing.set()
+            c1_orig()
+            # The lock is held we wait for the main thread to decide when to
+            # continue
+            after_writing.wait()
+        c1._write_config_file = c1_write_config_file
+        def c1_set_option():
+            c1.set_user_option('one', 'c1')
+            writing_done.set()
+        t1 = threading.Thread(target=c1_set_option)
+        # Collect the thread after the test
+        self.addCleanup(t1.join)
+        # Be ready to unblock the thread if the test goes wrong
+        self.addCleanup(after_writing.set)
+        t1.start()
+        before_writing.wait()
+        self.assertTrue(c1._lock.is_held)
+        self.assertRaises(errors.LockContention,
+                          c2.set_user_option, 'one', 'c2')
+        self.assertEquals('c1', c1.get_user_option('one'))
+        # Let the lock be released
+        after_writing.set()
+        writing_done.wait()
+        c2.set_user_option('one', 'c2')
+        self.assertEquals('c2', c2.get_user_option('one'))
+
+    def test_read_while_writing(self):
+       c1 = self.config
+       # We spawn a thread that will pause *during* the write
+       ready_to_write = threading.Event()
+       do_writing = threading.Event()
+       writing_done = threading.Event()
+       c1_orig = c1._write_config_file
+       def c1_write_config_file():
+           ready_to_write.set()
+           # The lock is held we wait for the main thread to decide when to
+           # continue
+           do_writing.wait()
+           c1_orig()
+           writing_done.set()
+       c1._write_config_file = c1_write_config_file
+       def c1_set_option():
+           c1.set_user_option('one', 'c1')
+       t1 = threading.Thread(target=c1_set_option)
+       # Collect the thread after the test
+       self.addCleanup(t1.join)
+       # Be ready to unblock the thread if the test goes wrong
+       self.addCleanup(do_writing.set)
+       t1.start()
+       # Ensure the thread is ready to write
+       ready_to_write.wait()
+       self.assertTrue(c1._lock.is_held)
+       self.assertEquals('c1', c1.get_user_option('one'))
+       # If we read during the write, we get the old value
+       c2 = self.get_existing_config()
+       self.assertEquals('1', c2.get_user_option('one'))
+       # Let the writing occur and ensure it occurred
+       do_writing.set()
+       writing_done.wait()
+       # Now we get the updated value
+       c3 = self.get_existing_config()
+       self.assertEquals('c1', c3.get_user_option('one'))
+
 
 class TestGetUserOptionAs(TestIniConfig):
 
@@ -505,11 +711,10 @@
         branch = self.make_branch('branch')
         self.assertEqual('branch', branch.nick)
 
-        locations = config.locations_config_filename()
-        config.ensure_config_dir_exists()
         local_url = urlutils.local_path_to_url('branch')
-        open(locations, 'wb').write('[%s]\nnickname = foobar'
-                                    % (local_url,))
+        conf = config.LocationConfig.from_string(
+            '[%s]\nnickname = foobar' % (local_url,),
+            local_url, save=True)
         self.assertEqual('foobar', branch.nick)
 
     def test_config_local_path(self):
@@ -517,20 +722,19 @@
         branch = self.make_branch('branch')
         self.assertEqual('branch', branch.nick)
 
-        locations = config.locations_config_filename()
-        config.ensure_config_dir_exists()
-        open(locations, 'wb').write('[%s/branch]\nnickname = barry'
-                                    % (osutils.getcwd().encode('utf8'),))
+        local_path = osutils.getcwd().encode('utf8')
+        conf = config.LocationConfig.from_string(
+            '[%s/branch]\nnickname = barry' % (local_path,),
+            'branch',  save=True)
         self.assertEqual('barry', branch.nick)
 
     def test_config_creates_local(self):
         """Creating a new entry in config uses a local path."""
         branch = self.make_branch('branch', format='knit')
         branch.set_push_location('http://foobar')
-        locations = config.locations_config_filename()
         local_path = osutils.getcwd().encode('utf8')
         # Surprisingly ConfigObj doesn't create a trailing newline
-        self.check_file_contents(locations,
+        self.check_file_contents(config.locations_config_filename(),
                                  '[%s/branch]\n'
                                  'push_location = http://foobar\n'
                                  'push_location:policy = norecurse\n'
@@ -541,10 +745,10 @@
         self.assertEqual('!repo', b.get_config().get_nickname())
 
     def test_warn_if_masked(self):
-        _warning = trace.warning
         warnings = []
         def warning(*args):
             warnings.append(args[0] % args[1:])
+        self.overrideAttr(trace, 'warning', warning)
 
         def set_option(store, warn_masked=True):
             warnings[:] = []
@@ -556,53 +760,41 @@
             else:
                 self.assertEqual(1, len(warnings))
                 self.assertEqual(warning, warnings[0])
-        trace.warning = warning
-        try:
-            branch = self.make_branch('.')
-            conf = branch.get_config()
-            set_option(config.STORE_GLOBAL)
-            assertWarning(None)
-            set_option(config.STORE_BRANCH)
-            assertWarning(None)
-            set_option(config.STORE_GLOBAL)
-            assertWarning('Value "4" is masked by "3" from branch.conf')
-            set_option(config.STORE_GLOBAL, warn_masked=False)
-            assertWarning(None)
-            set_option(config.STORE_LOCATION)
-            assertWarning(None)
-            set_option(config.STORE_BRANCH)
-            assertWarning('Value "3" is masked by "0" from locations.conf')
-            set_option(config.STORE_BRANCH, warn_masked=False)
-            assertWarning(None)
-        finally:
-            trace.warning = _warning
-
-
-class TestGlobalConfigItems(tests.TestCase):
+        branch = self.make_branch('.')
+        conf = branch.get_config()
+        set_option(config.STORE_GLOBAL)
+        assertWarning(None)
+        set_option(config.STORE_BRANCH)
+        assertWarning(None)
+        set_option(config.STORE_GLOBAL)
+        assertWarning('Value "4" is masked by "3" from branch.conf')
+        set_option(config.STORE_GLOBAL, warn_masked=False)
+        assertWarning(None)
+        set_option(config.STORE_LOCATION)
+        assertWarning(None)
+        set_option(config.STORE_BRANCH)
+        assertWarning('Value "3" is masked by "0" from locations.conf')
+        set_option(config.STORE_BRANCH, warn_masked=False)
+        assertWarning(None)
+
+
+class TestGlobalConfigItems(tests.TestCaseInTempDir):
 
     def test_user_id(self):
-        config_file = StringIO(sample_config_text.encode('utf-8'))
-        my_config = config.GlobalConfig()
-        my_config._parser = my_config._get_parser(file=config_file)
+        my_config = config.GlobalConfig.from_string(sample_config_text)
         self.assertEqual(u"Erik B\u00e5gfors <erik@bagfors.nu>",
                          my_config._get_user_id())
 
     def test_absent_user_id(self):
-        config_file = StringIO("")
         my_config = config.GlobalConfig()
-        my_config._parser = my_config._get_parser(file=config_file)
         self.assertEqual(None, my_config._get_user_id())
 
     def test_configured_editor(self):
-        config_file = StringIO(sample_config_text.encode('utf-8'))
-        my_config = config.GlobalConfig()
-        my_config._parser = my_config._get_parser(file=config_file)
+        my_config = config.GlobalConfig.from_string(sample_config_text)
         self.assertEqual("vim", my_config.get_editor())
 
     def test_signatures_always(self):
-        config_file = StringIO(sample_always_signatures)
-        my_config = config.GlobalConfig()
-        my_config._parser = my_config._get_parser(file=config_file)
+        my_config = config.GlobalConfig.from_string(sample_always_signatures)
         self.assertEqual(config.CHECK_NEVER,
                          my_config.signature_checking())
         self.assertEqual(config.SIGN_ALWAYS,
@@ -610,9 +802,7 @@
         self.assertEqual(True, my_config.signature_needed())
 
     def test_signatures_if_possible(self):
-        config_file = StringIO(sample_maybe_signatures)
-        my_config = config.GlobalConfig()
-        my_config._parser = my_config._get_parser(file=config_file)
+        my_config = config.GlobalConfig.from_string(sample_maybe_signatures)
         self.assertEqual(config.CHECK_NEVER,
                          my_config.signature_checking())
         self.assertEqual(config.SIGN_WHEN_REQUIRED,
@@ -620,9 +810,7 @@
         self.assertEqual(False, my_config.signature_needed())
 
     def test_signatures_ignore(self):
-        config_file = StringIO(sample_ignore_signatures)
-        my_config = config.GlobalConfig()
-        my_config._parser = my_config._get_parser(file=config_file)
+        my_config = config.GlobalConfig.from_string(sample_ignore_signatures)
         self.assertEqual(config.CHECK_ALWAYS,
                          my_config.signature_checking())
         self.assertEqual(config.SIGN_NEVER,
@@ -630,9 +818,7 @@
         self.assertEqual(False, my_config.signature_needed())
 
     def _get_sample_config(self):
-        config_file = StringIO(sample_config_text.encode('utf-8'))
-        my_config = config.GlobalConfig()
-        my_config._parser = my_config._get_parser(file=config_file)
+        my_config = config.GlobalConfig.from_string(sample_config_text)
         return my_config
 
     def test_gpg_signing_command(self):
@@ -641,9 +827,7 @@
         self.assertEqual(False, my_config.signature_needed())
 
     def _get_empty_config(self):
-        config_file = StringIO("")
         my_config = config.GlobalConfig()
-        my_config._parser = my_config._get_parser(file=config_file)
         return my_config
 
     def test_gpg_signing_command_unset(self):
@@ -744,18 +928,6 @@
         self.assertEqual(parser._calls,
                          [('__init__', config.locations_config_filename(),
                            'utf-8')])
-        config.ensure_config_dir_exists()
-        #os.mkdir(config.config_dir())
-        f = file(config.branches_config_filename(), 'wb')
-        f.write('')
-        f.close()
-        oldparserclass = config.ConfigObj
-        config.ConfigObj = InstrumentedConfigObj
-        try:
-            my_config = config.LocationConfig('http://www.example.com')
-            parser = my_config._get_parser()
-        finally:
-            config.ConfigObj = oldparserclass
 
     def test_get_global_config(self):
         my_config = config.BranchConfig(FakeBranch('http://example.com'))
@@ -990,42 +1162,30 @@
                          self.my_config.post_commit())
 
     def get_branch_config(self, location, global_config=None):
+        my_branch = FakeBranch(location)
         if global_config is None:
-            global_file = StringIO(sample_config_text.encode('utf-8'))
-        else:
-            global_file = StringIO(global_config.encode('utf-8'))
-        branches_file = StringIO(sample_branches_text.encode('utf-8'))
-        self.my_config = config.BranchConfig(FakeBranch(location))
-        # Force location config to use specified file
-        self.my_location_config = self.my_config._get_location_config()
-        self.my_location_config._get_parser(branches_file)
-        # Force global config to use specified file
-        self.my_config._get_global_config()._get_parser(global_file)
+            global_config = sample_config_text
+
+        my_global_config = config.GlobalConfig.from_string(global_config,
+                                                           save=True)
+        my_location_config = config.LocationConfig.from_string(
+            sample_branches_text, my_branch.base, save=True)
+        my_config = config.BranchConfig(my_branch)
+        self.my_config = my_config
+        self.my_location_config = my_config._get_location_config()
 
     def test_set_user_setting_sets_and_saves(self):
         self.get_branch_config('/a/c')
         record = InstrumentedConfigObj("foo")
         self.my_location_config._parser = record
 
-        real_mkdir = os.mkdir
-        self.created = False
-        def checked_mkdir(path, mode=0777):
-            self.log('making directory: %s', path)
-            real_mkdir(path, mode)
-            self.created = True
-
-        os.mkdir = checked_mkdir
-        try:
-            self.callDeprecated(['The recurse option is deprecated as of '
-                                 '0.14.  The section "/a/c" has been '
-                                 'converted to use policies.'],
-                                self.my_config.set_user_option,
-                                'foo', 'bar', store=config.STORE_LOCATION)
-        finally:
-            os.mkdir = real_mkdir
-
-        self.failUnless(self.created, 'Failed to create ~/.bazaar')
-        self.assertEqual([('__contains__', '/a/c'),
+        self.callDeprecated(['The recurse option is deprecated as of '
+                             '0.14.  The section "/a/c" has been '
+                             'converted to use policies.'],
+                            self.my_config.set_user_option,
+                            'foo', 'bar', store=config.STORE_LOCATION)
+        self.assertEqual([('reload',),
+                          ('__contains__', '/a/c'),
                           ('__contains__', '/a/c/'),
                           ('__setitem__', '/a/c', {}),
                           ('__getitem__', '/a/c'),
@@ -1074,19 +1234,18 @@
 option = exact
 """
 
-
 class TestBranchConfigItems(tests.TestCaseInTempDir):
 
     def get_branch_config(self, global_config=None, location=None,
                           location_config=None, branch_data_config=None):
-        my_config = config.BranchConfig(FakeBranch(location))
+        my_branch = FakeBranch(location)
         if global_config is not None:
-            global_file = StringIO(global_config.encode('utf-8'))
-            my_config._get_global_config()._get_parser(global_file)
-        self.my_location_config = my_config._get_location_config()
+            my_global_config = config.GlobalConfig.from_string(global_config,
+                                                               save=True)
         if location_config is not None:
-            location_file = StringIO(location_config.encode('utf-8'))
-            self.my_location_config._get_parser(location_file)
+            my_location_config = config.LocationConfig.from_string(
+                location_config, my_branch.base, save=True)
+        my_config = config.BranchConfig(my_branch)
         if branch_data_config is not None:
             my_config.branch.control_files.files['branch.conf'] = \
                 branch_data_config
@@ -1106,7 +1265,7 @@
                          my_config.username())
 
     def test_not_set_in_branch(self):
-        my_config = self.get_branch_config(sample_config_text)
+        my_config = self.get_branch_config(global_config=sample_config_text)
         self.assertEqual(u"Erik B\u00e5gfors <erik@bagfors.nu>",
                          my_config._get_user_id())
         my_config.branch.control_files.files['email'] = "John"
@@ -1136,29 +1295,25 @@
 
     def test_gpg_signing_command(self):
         my_config = self.get_branch_config(
+            global_config=sample_config_text,
             # branch data cannot set gpg_signing_command
             branch_data_config="gpg_signing_command=pgp")
-        config_file = StringIO(sample_config_text.encode('utf-8'))
-        my_config._get_global_config()._get_parser(config_file)
         self.assertEqual('gnome-gpg', my_config.gpg_signing_command())
 
     def test_get_user_option_global(self):
-        branch = FakeBranch()
-        my_config = config.BranchConfig(branch)
-        config_file = StringIO(sample_config_text.encode('utf-8'))
-        (my_config._get_global_config()._get_parser(config_file))
+        my_config = self.get_branch_config(global_config=sample_config_text)
         self.assertEqual('something',
                          my_config.get_user_option('user_global_option'))
 
     def test_post_commit_default(self):
-        branch = FakeBranch()
-        my_config = self.get_branch_config(sample_config_text, '/a/c',
-                                           sample_branches_text)
+        my_config = self.get_branch_config(global_config=sample_config_text,
+                                      location='/a/c',
+                                      location_config=sample_branches_text)
         self.assertEqual(my_config.branch.base, '/a/c')
         self.assertEqual('bzrlib.tests.test_config.post_commit',
                          my_config.post_commit())
         my_config.set_user_option('post_commit', 'rmtree_root')
-        # post-commit is ignored when bresent in branch data
+        # post-commit is ignored when present in branch data
         self.assertEqual('bzrlib.tests.test_config.post_commit',
                          my_config.post_commit())
         my_config.set_user_option('post_commit', 'rmtree_root',
@@ -1166,19 +1321,23 @@
         self.assertEqual('rmtree_root', my_config.post_commit())
 
     def test_config_precedence(self):
+        # FIXME: eager test, luckily no persitent config file makes it fail
+        # -- vila 20100716
         my_config = self.get_branch_config(global_config=precedence_global)
         self.assertEqual(my_config.get_user_option('option'), 'global')
         my_config = self.get_branch_config(global_config=precedence_global,
-                                      branch_data_config=precedence_branch)
+                                           branch_data_config=precedence_branch)
         self.assertEqual(my_config.get_user_option('option'), 'branch')
-        my_config = self.get_branch_config(global_config=precedence_global,
-                                      branch_data_config=precedence_branch,
-                                      location_config=precedence_location)
+        my_config = self.get_branch_config(
+            global_config=precedence_global,
+            branch_data_config=precedence_branch,
+            location_config=precedence_location)
         self.assertEqual(my_config.get_user_option('option'), 'recurse')
-        my_config = self.get_branch_config(global_config=precedence_global,
-                                      branch_data_config=precedence_branch,
-                                      location_config=precedence_location,
-                                      location='http://example.com/specific')
+        my_config = self.get_branch_config(
+            global_config=precedence_global,
+            branch_data_config=precedence_branch,
+            location_config=precedence_location,
+            location='http://example.com/specific')
         self.assertEqual(my_config.get_user_option('option'), 'exact')
 
     def test_get_mail_client(self):

=== modified file 'bzrlib/tests/test_debug.py'
--- bzrlib/tests/test_debug.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_debug.py	2010-08-29 14:32:45 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009 Canonical Ltd
+# Copyright (C) 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,47 +17,23 @@
 """Tests for bzrlib.debug"""
 
 
-import os
-
-
-from bzrlib import debug
-from bzrlib.config import config_filename, ensure_config_dir_exists
-from bzrlib.transport import get_transport
-from bzrlib.tests import TestCaseInTempDir
-
-
-class TestDebugFlags(TestCaseInTempDir):
+from bzrlib import (
+    config,
+    debug,
+    tests,
+    )
+
+
+class TestDebugFlags(tests.TestCaseInTempDir):
 
     def test_set_debug_flags_from_config(self):
         # test both combinations because configobject automatically splits up
         # comma-separated lists
-        if os.path.isfile(config_filename()):
-            # Something is wrong in environment,
-            # we risk overwriting users config
-            self.assert_(config_filename() + "exists, abort")
-
-        self.try_debug_flags(
-            """debug_flags = hpss, error\n""",
-            set(['hpss', 'error']))
-
-        self.try_debug_flags(
-            """debug_flags = hpss\n""",
-            set(['hpss']))
-
-    def try_debug_flags(self, conf_bytes, expected_flags):
-        ensure_config_dir_exists()
-        f = open(config_filename(), 'wb')
-        try:
-            f.write(conf_bytes)
-        finally:
-            f.close()
-        saved_debug = set(debug.debug_flags)
-        debug.debug_flags.clear()
-        try:
-            debug.set_debug_flags_from_config()
-            self.assertEqual(expected_flags,
-                debug.debug_flags)
-        finally:
-            # restore without rebinding the variable
-            debug.debug_flags.clear()
-            debug.debug_flags.update(saved_debug)
+        self.try_debug_flags(['hpss', 'error'], 'debug_flags = hpss, error\n')
+        self.try_debug_flags(['hpss'], 'debug_flags = hpss\n')
+
+    def try_debug_flags(self, expected_flags, conf_bytes):
+        conf = config.GlobalConfig.from_string(conf_bytes, save=True)
+        self.overrideAttr(debug, 'debug_flags', set())
+        debug.set_debug_flags_from_config()
+        self.assertEqual(set(expected_flags), debug.debug_flags)

=== modified file 'bzrlib/tests/test_diff.py'
--- bzrlib/tests/test_diff.py	2010-04-20 14:04:54 +0000
+++ bzrlib/tests/test_diff.py	2010-05-26 15:58:08 +0000
@@ -33,7 +33,8 @@
     transform,
     )
 from bzrlib.symbol_versioning import deprecated_in
-from bzrlib.tests import test_win32utils
+from bzrlib.tests import features
+from bzrlib.tests.blackbox.test_diff import subst_dates
 
 
 class _AttribFeature(tests.Feature):
@@ -521,7 +522,6 @@
         self.assertNotContainsRe(d, r"file 'e'")
         self.assertNotContainsRe(d, r"file 'f'")
 
-
     def test_binary_unicode_filenames(self):
         """Test that contents of files are *not* encoded in UTF-8 when there
         is a binary file in the diff.
@@ -580,6 +580,49 @@
         self.assertContainsRe(d, "=== modified file 'mod_%s'"%autf8)
         self.assertContainsRe(d, "=== removed file 'del_%s'"%autf8)
 
+    def test_unicode_filename_path_encoding(self):
+        """Test for bug #382699: unicode filenames on Windows should be shown
+        in user encoding.
+        """
+        self.requireFeature(tests.UnicodeFilenameFeature)
+        # The word 'test' in Russian
+        _russian_test = u'\u0422\u0435\u0441\u0442'
+        directory = _russian_test + u'/'
+        test_txt = _russian_test + u'.txt'
+        u1234 = u'\u1234.txt'
+
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([
+            (test_txt, 'foo\n'),
+            (u1234, 'foo\n'),
+            (directory, None),
+            ])
+        tree.add([test_txt, u1234, directory])
+
+        sio = StringIO()
+        diff.show_diff_trees(tree.basis_tree(), tree, sio,
+            path_encoding='cp1251')
+
+        output = subst_dates(sio.getvalue())
+        shouldbe = ('''\
+=== added directory '%(directory)s'
+=== added file '%(test_txt)s'
+--- a/%(test_txt)s\tYYYY-MM-DD HH:MM:SS +ZZZZ
++++ b/%(test_txt)s\tYYYY-MM-DD HH:MM:SS +ZZZZ
+@@ -0,0 +1,1 @@
++foo
+
+=== added file '?.txt'
+--- a/?.txt\tYYYY-MM-DD HH:MM:SS +ZZZZ
++++ b/?.txt\tYYYY-MM-DD HH:MM:SS +ZZZZ
+@@ -0,0 +1,1 @@
++foo
+
+''' % {'directory': _russian_test.encode('cp1251'),
+       'test_txt': test_txt.encode('cp1251'),
+      })
+        self.assertEqualDiff(output, shouldbe)
+
 
 class DiffWasIs(diff.DiffPath):
 
@@ -1298,7 +1341,7 @@
                          diff_obj._get_command('old-path', 'new-path'))
 
     def test_from_string_path_with_backslashes(self):
-        self.requireFeature(test_win32utils.BackslashDirSeparatorFeature)
+        self.requireFeature(features.backslashdir_feature)
         tool = 'C:\\Tools\\Diff.exe'
         diff_obj = diff.DiffFromTool.from_string(tool, None, None, None)
         self.addCleanup(diff_obj.finish)

=== modified file 'bzrlib/tests/test_directory_service.py'
--- bzrlib/tests/test_directory_service.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/test_directory_service.py	2010-06-20 11:18:38 +0000
@@ -18,17 +18,17 @@
 
 from bzrlib import (
     errors,
+    transport,
     urlutils,
     )
 from bzrlib.directory_service import DirectoryServiceRegistry, directories
 from bzrlib.tests import TestCase, TestCaseWithTransport
-from bzrlib.transport import get_transport
 
 
 class FooService(object):
     """A directory service that maps the name to a FILE url"""
 
-    # eg 'file:///foo' on Linux, or 'file:///C:/foo' on Windows
+    # eg 'file:///foo' on Unix, or 'file:///C:/foo' on Windows
     base = urlutils.local_path_to_url('/foo')
 
     def look_up(self, name, url):
@@ -56,7 +56,7 @@
         directories.register('foo:', FooService, 'Map foo URLs to http urls')
         self.addCleanup(directories.remove, 'foo:')
         self.assertEqual(FooService.base + 'bar/',
-                         get_transport('foo:bar').base)
+                         transport.get_transport('foo:bar').base)
 
 
 class TestAliasDirectory(TestCaseWithTransport):

=== modified file 'bzrlib/tests/test_dirstate.py'
--- bzrlib/tests/test_dirstate.py	2010-01-25 17:48:22 +0000
+++ bzrlib/tests/test_dirstate.py	2010-09-13 06:36:59 +0000
@@ -730,6 +730,22 @@
 
 class TestDirStateManipulations(TestCaseWithDirState):
 
+    def test_update_minimal_updates_id_index(self):
+        state = self.create_dirstate_with_root_and_subdir()
+        self.addCleanup(state.unlock)
+        id_index = state._get_id_index()
+        self.assertEqual(['a-root-value', 'subdir-id'], sorted(id_index))
+        state.add('file-name', 'file-id', 'file', None, '')
+        self.assertEqual(['a-root-value', 'file-id', 'subdir-id'],
+                         sorted(id_index))
+        state.update_minimal(('', 'new-name', 'file-id'), 'f',
+                             path_utf8='new-name')
+        self.assertEqual(['a-root-value', 'file-id', 'subdir-id'],
+                         sorted(id_index))
+        self.assertEqual([('', 'new-name', 'file-id')],
+                         sorted(id_index['file-id']))
+        state._validate()
+
     def test_set_state_from_inventory_no_content_no_parents(self):
         # setting the current inventory is a slow but important api to support.
         tree1 = self.make_branch_and_memory_tree('tree1')

=== modified file 'bzrlib/tests/test_errors.py'
--- bzrlib/tests/test_errors.py	2010-04-22 18:36:13 +0000
+++ bzrlib/tests/test_errors.py	2010-09-13 02:32:44 +0000
@@ -16,6 +16,8 @@
 
 """Tests for the formatting and construction of errors."""
 
+import inspect
+import re
 import socket
 import sys
 
@@ -26,11 +28,36 @@
     symbol_versioning,
     urlutils,
     )
-from bzrlib.tests import TestCase, TestCaseWithTransport
+from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
 
 
 class TestErrors(TestCaseWithTransport):
 
+    def test_no_arg_named_message(self):
+        """Ensure the __init__ and _fmt in errors do not have "message" arg.
+
+        This test fails if __init__ or _fmt in errors has an argument
+        named "message" as this can cause errors in some Python versions.
+        Python 2.5 uses a slot for StandardError.message.
+        See bug #603461
+        """
+        fmt_pattern = re.compile("%\(message\)[sir]")
+        subclasses_present = getattr(errors.BzrError, '__subclasses__', None)
+        if not subclasses_present:
+            raise TestSkipped('__subclasses__ attribute required for classes. '
+                'Requires Python 2.5 or later.')
+        for c in errors.BzrError.__subclasses__():
+            init = getattr(c, '__init__', None)
+            fmt = getattr(c, '_fmt', None)
+            if init:
+                args = inspect.getargspec(init)[0]
+                self.assertFalse('message' in args,
+                    ('Argument name "message" not allowed for '
+                    '"errors.%s.__init__"' % c.__name__))
+            if fmt and fmt_pattern.search(fmt):
+                self.assertFalse(True, ('"message" not allowed in '
+                    '"errors.%s._fmt"' % c.__name__))
+
     def test_bad_filename_encoding(self):
         error = errors.BadFilenameEncoding('bad/filen\xe5me', 'UTF-8')
         self.assertEqualDiff(
@@ -132,6 +159,13 @@
             "cannot be broken.",
             str(error))
 
+    def test_lock_corrupt(self):
+        error = errors.LockCorrupt("corruption info")
+        self.assertEqualDiff("Lock is apparently held, but corrupted: "
+            "corruption info\n"
+            "Use 'bzr break-lock' to clear it",
+            str(error))
+
     def test_knit_data_stream_incompatible(self):
         error = errors.KnitDataStreamIncompatible(
             'stream format', 'target format')
@@ -655,6 +689,17 @@
         str(err)
         self.assertEqual(['open_repository'], fake_bzrdir.calls)
 
+    def test_invalid_pattern(self):
+        error = errors.InvalidPattern('Bad pattern msg.')
+        self.assertEqualDiff("Invalid pattern(s) found. Bad pattern msg.",
+            str(error))
+
+    def test_recursive_bind(self):
+        error = errors.RecursiveBind('foo_bar_branch')
+        msg = ('Branch "foo_bar_branch" appears to be bound to itself. '
+            'Please use `bzr unbind` to fix.')
+        self.assertEqualDiff(msg, str(error))
+
 
 class PassThroughError(errors.BzrError):
 
@@ -703,7 +748,7 @@
             str(e), 'Unprintable exception ErrorWithBadFormat')
 
     def test_cannot_bind_address(self):
-        # see <https://bugs.edge.launchpad.net/bzr/+bug/286871>
+        # see <https://bugs.launchpad.net/bzr/+bug/286871>
         e = errors.CannotBindAddress('example.com', 22,
             socket.error(13, 'Permission denied'))
         self.assertContainsRe(str(e),
@@ -713,3 +758,9 @@
         e = errors.FileTimestampUnavailable("/path/foo")
         self.assertEquals("The filestamp for /path/foo is not available.",
             str(e))
+            
+    def test_transform_rename_failed(self):
+        e = errors.TransformRenameFailed(u"from", u"to", "readonly file", 2)
+        self.assertEquals(
+            u"Failed to rename from to to: readonly file",
+            str(e))

=== added file 'bzrlib/tests/test_fixtures.py'
--- bzrlib/tests/test_fixtures.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_fixtures.py	2010-06-21 22:29:38 +0000
@@ -0,0 +1,28 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for test fixtures"""
+
+import codecs
+
+from bzrlib import (
+    tests,
+    )
+from bzrlib.tests import (
+    fixtures,
+    )
+
+

=== modified file 'bzrlib/tests/test_foreign.py'
--- bzrlib/tests/test_foreign.py	2010-04-23 00:44:15 +0000
+++ bzrlib/tests/test_foreign.py	2010-08-02 18:36:31 +0000
@@ -21,6 +21,7 @@
 from bzrlib import (
     branch,
     bzrdir,
+    controldir,
     errors,
     foreign,
     lockable_files,
@@ -205,13 +206,6 @@
     def get_branch_format(self):
         return DummyForeignVcsBranchFormat()
 
-    @classmethod
-    def probe_transport(klass, transport):
-        """Return the .bzrdir style format present in a directory."""
-        if not transport.has('.dummy'):
-            raise errors.NotBranchError(path=transport.base)
-        return klass()
-
     def initialize_on_transport(self, transport):
         """Initialize a new bzrdir in the base directory of a Transport."""
         # Since we don't have a .bzr directory, inherit the
@@ -266,9 +260,12 @@
 
 
 def register_dummy_foreign_for_test(testcase):
-    bzrdir.BzrDirFormat.register_control_format(DummyForeignVcsDirFormat)
-    testcase.addCleanup(bzrdir.BzrDirFormat.unregister_control_format,
+    controldir.ControlDirFormat.register_format(DummyForeignVcsDirFormat)
+    testcase.addCleanup(controldir.ControlDirFormat.unregister_format,
                         DummyForeignVcsDirFormat)
+    controldir.ControlDirFormat.register_prober(DummyForeignProber)
+    testcase.addCleanup(controldir.ControlDirFormat.unregister_prober,
+        DummyForeignProber)
     # We need to register the optimiser to make the dummy appears really
     # different from a regular bzr repository.
     branch.InterBranch.register_optimiser(InterToDummyVcsBranch)
@@ -276,6 +273,16 @@
                         InterToDummyVcsBranch)
 
 
+class DummyForeignProber(controldir.Prober):
+
+    @classmethod
+    def probe_transport(klass, transport):
+        """Return the .bzrdir style format present in a directory."""
+        if not transport.has('.dummy'):
+            raise errors.NotBranchError(path=transport.base)
+        return DummyForeignVcsDirFormat()
+
+
 class ForeignVcsRegistryTests(tests.TestCase):
     """Tests for the ForeignVcsRegistry class."""
 

=== modified file 'bzrlib/tests/test_globbing.py'
--- bzrlib/tests/test_globbing.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test_globbing.py	2010-08-07 16:42:09 +0000
@@ -15,6 +15,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+import re
+
+from bzrlib import errors
 from bzrlib.globbing import (
     Globster,
     ExceptionGlobster,
@@ -52,14 +55,16 @@
     def test_char_group_digit(self):
         self.assertMatchBasenameAndFullpath([
             # The definition of digit this uses includes arabic digits from
-            # non-latin scripts (arabic, indic, etc.) and subscript/superscript
-            # digits, but neither roman numerals nor vulgar fractions.
+            # non-latin scripts (arabic, indic, etc.) but neither roman
+            # numerals nor vulgar fractions. Some characters such as
+            # subscript/superscript digits may or may not match depending on
+            # the Python version used, see: <http://bugs.python.org/issue6561>
             (u'[[:digit:]]',
-             [u'0', u'5', u'\u0663', u'\u06f9', u'\u0f21', u'\xb9'],
+             [u'0', u'5', u'\u0663', u'\u06f9', u'\u0f21'],
              [u'T', u'q', u' ', u'\u8336', u'.']),
             (u'[^[:digit:]]',
              [u'T', u'q', u' ', u'\u8336', u'.'],
-             [u'0', u'5', u'\u0663', u'\u06f9', u'\u0f21', u'\xb9']),
+             [u'0', u'5', u'\u0663', u'\u06f9', u'\u0f21']),
             ])
 
     def test_char_group_space(self):
@@ -308,6 +313,15 @@
             self.assertEqual(patterns[x],globster.match(filename))
         self.assertEqual(None,globster.match('foobar.300'))
 
+    def test_bad_pattern(self):
+        """Ensure that globster handles bad patterns cleanly."""
+        patterns = [u'RE:[', u'/home/foo', u'RE:*.cpp']
+        g = Globster(patterns)
+        e = self.assertRaises(errors.InvalidPattern, g.match, 'filename')
+        self.assertContainsRe(e.msg,
+            "File.*ignore.*contains error.*RE:\[.*RE:\*\.cpp", flags=re.DOTALL)
+
+
 class TestExceptionGlobster(TestCase):
 
     def test_exclusion_patterns(self):

=== modified file 'bzrlib/tests/test_graph.py'
--- bzrlib/tests/test_graph.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test_graph.py	2010-08-15 15:20:14 +0000
@@ -1424,6 +1424,61 @@
         self.assertMergeOrder(['rev3', 'rev1'], graph, 'rev4', ['rev1', 'rev3'])
 
 
+class TestFindDescendants(TestGraphBase):
+
+    def test_find_descendants_rev1_rev3(self):
+        graph = self.make_graph(ancestry_1)
+        descendants = graph.find_descendants('rev1', 'rev3')
+        self.assertEqual(set(['rev1', 'rev2a', 'rev3']), descendants)
+
+    def test_find_descendants_rev1_rev4(self):
+        graph = self.make_graph(ancestry_1)
+        descendants = graph.find_descendants('rev1', 'rev4')
+        self.assertEqual(set(['rev1', 'rev2a', 'rev2b', 'rev3', 'rev4']),
+                         descendants)
+
+    def test_find_descendants_rev2a_rev4(self):
+        graph = self.make_graph(ancestry_1)
+        descendants = graph.find_descendants('rev2a', 'rev4')
+        self.assertEqual(set(['rev2a', 'rev3', 'rev4']), descendants)
+
+class TestFindLefthandMerger(TestGraphBase):
+
+    def check_merger(self, result, ancestry, merged, tip):
+        graph = self.make_graph(ancestry)
+        self.assertEqual(result, graph.find_lefthand_merger(merged, tip))
+
+    def test_find_lefthand_merger_rev2b(self):
+        self.check_merger('rev4', ancestry_1, 'rev2b', 'rev4')
+
+    def test_find_lefthand_merger_rev2a(self):
+        self.check_merger('rev2a', ancestry_1, 'rev2a', 'rev4')
+
+    def test_find_lefthand_merger_rev4(self):
+        self.check_merger(None, ancestry_1, 'rev4', 'rev2a')
+
+    def test_find_lefthand_merger_f(self):
+        self.check_merger('i', complex_shortcut, 'f', 'm')
+
+    def test_find_lefthand_merger_g(self):
+        self.check_merger('i', complex_shortcut, 'g', 'm')
+
+    def test_find_lefthand_merger_h(self):
+        self.check_merger('n', complex_shortcut, 'h', 'n')
+
+
+class TestGetChildMap(TestGraphBase):
+
+    def test_get_child_map(self):
+        graph = self.make_graph(ancestry_1)
+        child_map = graph.get_child_map(['rev4', 'rev3', 'rev2a', 'rev2b'])
+        self.assertEqual({'rev1': ['rev2a', 'rev2b'],
+                          'rev2a': ['rev3'],
+                          'rev2b': ['rev4'],
+                          'rev3': ['rev4']},
+                          child_map)
+
+
 class TestCachingParentsProvider(tests.TestCase):
     """These tests run with:
 

=== modified file 'bzrlib/tests/test_groupcompress.py'
--- bzrlib/tests/test_groupcompress.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test_groupcompress.py	2010-09-21 19:30:33 +0000
@@ -347,6 +347,30 @@
         self.assertEqual(z_content, block._z_content)
         self.assertEqual(content, block._content)
 
+    def test_to_chunks(self):
+        content_chunks = ['this is some content\n',
+                          'this content will be compressed\n']
+        content_len = sum(map(len, content_chunks))
+        content = ''.join(content_chunks)
+        gcb = groupcompress.GroupCompressBlock()
+        gcb.set_chunked_content(content_chunks, content_len)
+        total_len, block_chunks = gcb.to_chunks()
+        block_bytes = ''.join(block_chunks)
+        self.assertEqual(gcb._z_content_length, len(gcb._z_content))
+        self.assertEqual(total_len, len(block_bytes))
+        self.assertEqual(gcb._content_length, content_len)
+        expected_header =('gcb1z\n' # group compress block v1 zlib
+                          '%d\n' # Length of compressed content
+                          '%d\n' # Length of uncompressed content
+                         ) % (gcb._z_content_length, gcb._content_length)
+        # The first chunk should be the header chunk. It is small, fixed size,
+        # and there is no compelling reason to split it up
+        self.assertEqual(expected_header, block_chunks[0])
+        self.assertStartsWith(block_bytes, expected_header)
+        remaining_bytes = block_bytes[len(expected_header):]
+        raw_bytes = zlib.decompress(remaining_bytes)
+        self.assertEqual(content, raw_bytes)
+
     def test_to_bytes(self):
         content = ('this is some content\n'
                    'this content will be compressed\n')
@@ -389,7 +413,7 @@
         z_content = zlib.compress(content)
         self.assertEqual(57182, len(z_content))
         block = groupcompress.GroupCompressBlock()
-        block._z_content = z_content
+        block._z_content_chunks = (z_content,)
         block._z_content_length = len(z_content)
         block._compressor_name = 'zlib'
         block._content_length = 158634
@@ -434,7 +458,7 @@
         z_content = zlib.compress(content)
         self.assertEqual(57182, len(z_content))
         block = groupcompress.GroupCompressBlock()
-        block._z_content = z_content
+        block._z_content_chunks = (z_content,)
         block._z_content_length = len(z_content)
         block._compressor_name = 'zlib'
         block._content_length = 158634
@@ -1066,3 +1090,25 @@
         # consumption
         self.add_key_to_manager(('key4',), locations, block, manager)
         self.assertTrue(manager.check_is_well_utilized())
+
+
+class Test_GCBuildDetails(tests.TestCase):
+
+    def test_acts_like_tuple(self):
+        # _GCBuildDetails inlines some of the data that used to be spread out
+        # across a bunch of tuples
+        bd = groupcompress._GCBuildDetails((('parent1',), ('parent2',)),
+            ('INDEX', 10, 20, 0, 5))
+        self.assertEqual(4, len(bd))
+        self.assertEqual(('INDEX', 10, 20, 0, 5), bd[0])
+        self.assertEqual(None, bd[1]) # Compression Parent is always None
+        self.assertEqual((('parent1',), ('parent2',)), bd[2])
+        self.assertEqual(('group', None), bd[3]) # Record details
+
+    def test__repr__(self):
+        bd = groupcompress._GCBuildDetails((('parent1',), ('parent2',)),
+            ('INDEX', 10, 20, 0, 5))
+        self.assertEqual("_GCBuildDetails(('INDEX', 10, 20, 0, 5),"
+                         " (('parent1',), ('parent2',)))",
+                         repr(bd))
+                    

=== modified file 'bzrlib/tests/test_help.py'
--- bzrlib/tests/test_help.py	2010-04-23 08:51:52 +0000
+++ bzrlib/tests/test_help.py	2010-05-24 05:50:53 +0000
@@ -112,6 +112,12 @@
                 Example 2::
 
                     cmd arg2
+
+                A code block follows.
+
+                ::
+
+                    bzr Demo something
             """
         cmd = cmd_Demo()
         helptext = cmd.get_help_text()
@@ -134,6 +140,10 @@
             '    Example 2:\n'
             '\n'
             '        cmd arg2\n'
+            '\n'
+            '    A code block follows.\n'
+            '\n'
+            '        bzr Demo something\n'
             '\n')
         helptext = cmd.get_help_text(plain=False)
         self.assertEquals(helptext,
@@ -154,6 +164,12 @@
             '    Example 2::\n'
             '\n'
             '        cmd arg2\n'
+            '\n'
+            '    A code block follows.\n'
+            '\n'
+            '    ::\n'
+            '\n'
+            '        bzr Demo something\n'
             '\n')
 
     def test_concise_help_text(self):

=== modified file 'bzrlib/tests/test_http.py'
--- bzrlib/tests/test_http.py	2010-03-18 23:11:15 +0000
+++ bzrlib/tests/test_http.py	2010-08-24 13:03:18 +0000
@@ -44,13 +44,11 @@
     ui,
     urlutils,
     )
-from bzrlib.symbol_versioning import (
-    deprecated_in,
-    )
 from bzrlib.tests import (
     features,
     http_server,
     http_utils,
+    test_server,
     )
 from bzrlib.transport import (
     http,
@@ -80,15 +78,27 @@
     transport_scenarios = [
         ('urllib', dict(_transport=_urllib.HttpTransport_urllib,
                         _server=http_server.HttpServer_urllib,
-                        _qualified_prefix='http+urllib',)),
+                        _url_protocol='http+urllib',)),
         ]
     if features.pycurl.available():
         transport_scenarios.append(
             ('pycurl', dict(_transport=PyCurlTransport,
                             _server=http_server.HttpServer_PyCurl,
-                            _qualified_prefix='http+pycurl',)))
+                            _url_protocol='http+pycurl',)))
     tests.multiply_tests(t_tests, transport_scenarios, result)
 
+    protocol_scenarios = [
+            ('HTTP/1.0',  dict(_protocol_version='HTTP/1.0')),
+            ('HTTP/1.1',  dict(_protocol_version='HTTP/1.1')),
+            ]
+
+    # some tests are parametrized by the protocol version only
+    p_tests, remaining_tests = tests.split_suite_by_condition(
+        remaining_tests, tests.condition_isinstance((
+                TestAuthOnRedirected,
+                )))
+    tests.multiply_tests(p_tests, protocol_scenarios, result)
+
     # each implementation tested with each HTTP version
     tp_tests, remaining_tests = tests.split_suite_by_condition(
         remaining_tests, tests.condition_isinstance((
@@ -103,10 +113,6 @@
                 TestRanges,
                 TestSpecificRequestHandler,
                 )))
-    protocol_scenarios = [
-            ('HTTP/1.0',  dict(_protocol_version='HTTP/1.0')),
-            ('HTTP/1.1',  dict(_protocol_version='HTTP/1.1')),
-            ]
     tp_scenarios = tests.multiply_scenarios(transport_scenarios,
                                             protocol_scenarios)
     tests.multiply_tests(tp_tests, tp_scenarios, result)
@@ -223,25 +229,22 @@
         self._sock.bind(('127.0.0.1', 0))
         self.host, self.port = self._sock.getsockname()
         self._ready = threading.Event()
-        self._thread = threading.Thread(target=self._accept_read_and_reply)
-        self._thread.setDaemon(True)
+        self._thread = test_server.ThreadWithException(
+            event=self._ready, target=self._accept_read_and_reply)
         self._thread.start()
-        self._ready.wait(5)
+        if 'threads' in tests.selftest_debug_flags:
+            sys.stderr.write('Thread started: %s\n' % (self._thread.ident,))
+        self._ready.wait()
 
     def _accept_read_and_reply(self):
         self._sock.listen(1)
         self._ready.set()
-        self._sock.settimeout(5)
-        try:
-            conn, address = self._sock.accept()
-            # On win32, the accepted connection will be non-blocking to start
-            # with because we're using settimeout.
-            conn.setblocking(True)
+        conn, address = self._sock.accept()
+        if self._expect_body_tail is not None:
             while not self.received_bytes.endswith(self._expect_body_tail):
                 self.received_bytes += conn.recv(4096)
             conn.sendall('HTTP/1.1 200 OK\r\n')
-        except socket.timeout:
-            # Make sure the client isn't stuck waiting for us to e.g. accept.
+        try:
             self._sock.close()
         except socket.error:
             # The client may have already closed the socket.
@@ -249,12 +252,18 @@
 
     def stop_server(self):
         try:
-            self._sock.close()
+            # Issue a fake connection to wake up the server and allow it to
+            # finish quickly
+            fake_conn = osutils.connect_socket((self.host, self.port))
+            fake_conn.close()
         except socket.error:
             # We might have already closed it.  We don't care.
             pass
         self.host = None
         self.port = None
+        self._thread.join()
+        if 'threads' in tests.selftest_debug_flags:
+            sys.stderr.write('Thread  joined: %s\n' % (self._thread.ident,))
 
 
 class TestAuthHeader(tests.TestCase):
@@ -304,29 +313,20 @@
 
             protocol_version = 'HTTP/0.1'
 
-        server = http_server.HttpServer(BogusRequestHandler)
-        try:
-            self.assertRaises(httplib.UnknownProtocol, server.start_server)
-        except:
-            server.stop_server()
-            self.fail('HTTP Server creation did not raise UnknownProtocol')
+        self.assertRaises(httplib.UnknownProtocol,
+                          http_server.HttpServer, BogusRequestHandler)
 
     def test_force_invalid_protocol(self):
-        server = http_server.HttpServer(protocol_version='HTTP/0.1')
-        try:
-            self.assertRaises(httplib.UnknownProtocol, server.start_server)
-        except:
-            server.stop_server()
-            self.fail('HTTP Server creation did not raise UnknownProtocol')
+        self.assertRaises(httplib.UnknownProtocol,
+                          http_server.HttpServer, protocol_version='HTTP/0.1')
 
     def test_server_start_and_stop(self):
         server = http_server.HttpServer()
+        self.addCleanup(server.stop_server)
         server.start_server()
-        try:
-            self.assertTrue(server._http_running)
-        finally:
-            server.stop_server()
-        self.assertFalse(server._http_running)
+        self.assertTrue(server.server is not None)
+        self.assertTrue(server.server.serving is not None)
+        self.assertTrue(server.server.serving)
 
     def test_create_http_server_one_zero(self):
         class RequestHandlerOneZero(http_server.TestingHTTPRequestHandler):
@@ -335,7 +335,7 @@
 
         server = http_server.HttpServer(RequestHandlerOneZero)
         self.start_server(server)
-        self.assertIsInstance(server._httpd, http_server.TestingHTTPServer)
+        self.assertIsInstance(server.server, http_server.TestingHTTPServer)
 
     def test_create_http_server_one_one(self):
         class RequestHandlerOneOne(http_server.TestingHTTPRequestHandler):
@@ -344,7 +344,7 @@
 
         server = http_server.HttpServer(RequestHandlerOneOne)
         self.start_server(server)
-        self.assertIsInstance(server._httpd,
+        self.assertIsInstance(server.server,
                               http_server.TestingThreadingHTTPServer)
 
     def test_create_http_server_force_one_one(self):
@@ -355,7 +355,7 @@
         server = http_server.HttpServer(RequestHandlerOneZero,
                                         protocol_version='HTTP/1.1')
         self.start_server(server)
-        self.assertIsInstance(server._httpd,
+        self.assertIsInstance(server.server,
                               http_server.TestingThreadingHTTPServer)
 
     def test_create_http_server_force_one_zero(self):
@@ -366,7 +366,7 @@
         server = http_server.HttpServer(RequestHandlerOneOne,
                                         protocol_version='HTTP/1.0')
         self.start_server(server)
-        self.assertIsInstance(server._httpd,
+        self.assertIsInstance(server.server,
                               http_server.TestingHTTPServer)
 
 
@@ -431,7 +431,7 @@
         server.start_server()
         try:
             url = server.get_url()
-            self.assertTrue(url.startswith('%s://' % self._qualified_prefix))
+            self.assertTrue(url.startswith('%s://' % self._url_protocol))
         finally:
             server.stop_server()
 
@@ -482,7 +482,7 @@
 
     def test_http_has(self):
         server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         self.assertEqual(t.has('foo/bar'), True)
         self.assertEqual(len(server.logs), 1)
         self.assertContainsRe(server.logs[0],
@@ -490,14 +490,14 @@
 
     def test_http_has_not_found(self):
         server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         self.assertEqual(t.has('not-found'), False)
         self.assertContainsRe(server.logs[1],
             r'"HEAD /not-found HTTP/1.." 404 - "-" "bzr/')
 
     def test_http_get(self):
         server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         fp = t.get('foo/bar')
         self.assertEqualDiff(
             fp.read(),
@@ -526,7 +526,7 @@
     """Test registrations of various http implementations"""
 
     def test_http_registered(self):
-        t = transport.get_transport('%s://foo.com/' % self._qualified_prefix)
+        t = transport.get_transport('%s://foo.com/' % self._url_protocol)
         self.assertIsInstance(t, transport.Transport)
         self.assertIsInstance(t, self._transport)
 
@@ -535,10 +535,11 @@
 
     def test_post_body_is_received(self):
         server = RecordingServer(expect_body_tail='end-of-body',
-            scheme=self._qualified_prefix)
+                                 scheme=self._url_protocol)
         self.start_server(server)
         url = server.get_url()
-        http_transport = self._transport(url)
+        # FIXME: needs a cleanup -- vila 20100611
+        http_transport = transport.get_transport(url)
         code, response = http_transport._post('abc def end-of-body')
         self.assertTrue(
             server.received_bytes.startswith('POST /.bzr/smart HTTP/1.'))
@@ -588,8 +589,10 @@
     _req_handler_class = http_server.TestingHTTPRequestHandler
 
     def create_transport_readonly_server(self):
-        return http_server.HttpServer(self._req_handler_class,
-                                      protocol_version=self._protocol_version)
+        server = http_server.HttpServer(self._req_handler_class,
+                                        protocol_version=self._protocol_version)
+        server._url_protocol = self._url_protocol
+        return server
 
     def _testing_pycurl(self):
         # TODO: This is duplicated for lots of the classes in this file
@@ -600,7 +603,7 @@
 class WallRequestHandler(http_server.TestingHTTPRequestHandler):
     """Whatever request comes in, close the connection"""
 
-    def handle_one_request(self):
+    def _handle_one_request(self):
         """Handle a single HTTP request, by abruptly closing the connection"""
         self.close_connection = 1
 
@@ -611,8 +614,7 @@
     _req_handler_class = WallRequestHandler
 
     def test_http_has(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         # Unfortunately httplib (see HTTPResponse._read_status
         # for details) make no distinction between a closed
         # socket and badly formatted status line, so we can't
@@ -624,8 +626,7 @@
                           t.has, 'foo/bar')
 
     def test_http_get(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         self.assertRaises((errors.ConnectionError, errors.ConnectionReset,
                            errors.InvalidHttpResponse),
                           t.get, 'foo/bar')
@@ -648,13 +649,11 @@
     _req_handler_class = BadStatusRequestHandler
 
     def test_http_has(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         self.assertRaises(errors.InvalidHttpResponse, t.has, 'foo/bar')
 
     def test_http_get(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         self.assertRaises(errors.InvalidHttpResponse, t.get, 'foo/bar')
 
 
@@ -665,6 +664,10 @@
         """Fakes handling a single HTTP request, returns a bad status"""
         ignored = http_server.TestingHTTPRequestHandler.parse_request(self)
         self.wfile.write("Invalid status line\r\n")
+        # If we don't close the connection pycurl will hang. Since this is a
+        # stress test we don't *have* to respect the protocol, but we don't
+        # have to sabotage it too much either.
+        self.close_connection = True
         return False
 
 
@@ -676,18 +679,6 @@
 
     _req_handler_class = InvalidStatusRequestHandler
 
-    def test_http_has(self):
-        if self._testing_pycurl() and self._protocol_version == 'HTTP/1.1':
-            raise tests.KnownFailure(
-                'pycurl hangs if the server send back garbage')
-        super(TestInvalidStatusServer, self).test_http_has()
-
-    def test_http_get(self):
-        if self._testing_pycurl() and self._protocol_version == 'HTTP/1.1':
-            raise tests.KnownFailure(
-                'pycurl hangs if the server send back garbage')
-        super(TestInvalidStatusServer, self).test_http_get()
-
 
 class BadProtocolRequestHandler(http_server.TestingHTTPRequestHandler):
     """Whatever request comes in, returns a bad protocol version"""
@@ -715,13 +706,11 @@
         super(TestBadProtocolServer, self).setUp()
 
     def test_http_has(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         self.assertRaises(errors.InvalidHttpResponse, t.has, 'foo/bar')
 
     def test_http_get(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         self.assertRaises(errors.InvalidHttpResponse, t.get, 'foo/bar')
 
 
@@ -741,13 +730,11 @@
     _req_handler_class = ForbiddenRequestHandler
 
     def test_http_has(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         self.assertRaises(errors.TransportError, t.has, 'foo/bar')
 
     def test_http_get(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         self.assertRaises(errors.TransportError, t.get, 'foo/bar')
 
 
@@ -792,8 +779,7 @@
         self.build_tree_contents([('a', '0123456789')],)
 
     def test_readv(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         l = list(t.readv('a', ((0, 1), (1, 1), (3, 2), (9, 1))))
         self.assertEqual(l[0], (0, '0'))
         self.assertEqual(l[1], (1, '1'))
@@ -801,8 +787,7 @@
         self.assertEqual(l[3], (9, '9'))
 
     def test_readv_out_of_order(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         l = list(t.readv('a', ((1, 1), (9, 1), (0, 1), (3, 2))))
         self.assertEqual(l[0], (1, '1'))
         self.assertEqual(l[1], (9, '9'))
@@ -810,8 +795,7 @@
         self.assertEqual(l[3], (3, '34'))
 
     def test_readv_invalid_ranges(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
 
         # This is intentionally reading off the end of the file
         # since we are sure that it cannot get there
@@ -825,7 +809,7 @@
 
     def test_readv_multiple_get_requests(self):
         server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         # force transport to issue multiple requests
         t._max_readv_combine = 1
         t._max_get_ranges = 1
@@ -839,7 +823,7 @@
 
     def test_readv_get_max_size(self):
         server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         # force transport to issue multiple requests by limiting the number of
         # bytes by request. Note that this apply to coalesced offsets only, a
         # single range will keep its size even if bigger than the limit.
@@ -854,7 +838,7 @@
 
     def test_complete_readv_leave_pipe_clean(self):
         server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         # force transport to issue multiple requests
         t._get_max_size = 2
         l = list(t.readv('a', ((0, 1), (1, 1), (2, 4), (6, 4))))
@@ -865,7 +849,7 @@
 
     def test_incomplete_readv_leave_pipe_clean(self):
         server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         # force transport to issue multiple requests
         t._get_max_size = 2
         # Don't collapse readv results into a list so that we leave unread
@@ -1021,7 +1005,7 @@
 
     def test_readv_with_short_reads(self):
         server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         # Force separate ranges for each offset
         t._bytes_to_read_before_seek = 0
         ireadv = iter(t.readv('a', ((0, 1), (2, 1), (4, 2), (9, 1))))
@@ -1078,9 +1062,6 @@
         return LimitedRangeHTTPServer(range_limit=self.range_limit,
                                       protocol_version=self._protocol_version)
 
-    def get_transport(self):
-        return self._transport(self.get_readonly_server().get_url())
-
     def setUp(self):
         http_utils.TestCaseWithWebserver.setUp(self)
         # We need to manipulate ranges that correspond to real chunks in the
@@ -1090,14 +1071,14 @@
         self.build_tree_contents([('a', content)],)
 
     def test_few_ranges(self):
-        t = self.get_transport()
+        t = self.get_readonly_transport()
         l = list(t.readv('a', ((0, 4), (1024, 4), )))
         self.assertEqual(l[0], (0, '0000'))
         self.assertEqual(l[1], (1024, '0001'))
         self.assertEqual(1, self.get_readonly_server().GET_request_nb)
 
     def test_more_ranges(self):
-        t = self.get_transport()
+        t = self.get_readonly_transport()
         l = list(t.readv('a', ((0, 4), (1024, 4), (4096, 4), (8192, 4))))
         self.assertEqual(l[0], (0, '0000'))
         self.assertEqual(l[1], (1024, '0001'))
@@ -1117,10 +1098,7 @@
     def setUp(self):
         tests.TestCase.setUp(self)
         self._old_env = {}
-
-    def tearDown(self):
-        self._restore_env()
-        tests.TestCase.tearDown(self)
+        self.addCleanup(self._restore_env)
 
     def _install_env(self, env):
         for name, value in env.iteritems():
@@ -1157,24 +1135,24 @@
     """
 
     # FIXME: We don't have an https server available, so we don't
-    # test https connections.
+    # test https connections. --vila toolongago
 
     def setUp(self):
         super(TestProxyHttpServer, self).setUp()
+        self.transport_secondary_server = http_utils.ProxyServer
         self.build_tree_contents([('foo', 'contents of foo\n'),
                                   ('foo-proxied', 'proxied contents of foo\n')])
         # Let's setup some attributes for tests
-        self.server = self.get_readonly_server()
-        self.proxy_address = '%s:%d' % (self.server.host, self.server.port)
+        server = self.get_readonly_server()
+        self.server_host_port = '%s:%d' % (server.host, server.port)
         if self._testing_pycurl():
             # Oh my ! pycurl does not check for the port as part of
             # no_proxy :-( So we just test the host part
-            self.no_proxy_host = self.server.host
+            self.no_proxy_host = server.host
         else:
-            self.no_proxy_host = self.proxy_address
+            self.no_proxy_host = self.server_host_port
         # The secondary server is the proxy
-        self.proxy = self.get_secondary_server()
-        self.proxy_url = self.proxy.get_url()
+        self.proxy_url = self.get_secondary_url()
         self._old_env = {}
 
     def _testing_pycurl(self):
@@ -1182,12 +1160,6 @@
         return (features.pycurl.available()
                 and self._transport == PyCurlTransport)
 
-    def create_transport_secondary_server(self):
-        """Creates an http server that will serve files with
-        '-proxied' appended to their names.
-        """
-        return http_utils.ProxyServer(protocol_version=self._protocol_version)
-
     def _install_env(self, env):
         for name, value in env.iteritems():
             self._old_env[name] = osutils.set_or_unset_env(name, value)
@@ -1198,8 +1170,7 @@
 
     def proxied_in_env(self, env):
         self._install_env(env)
-        url = self.server.get_url()
-        t = self._transport(url)
+        t = self.get_readonly_transport()
         try:
             self.assertEqual('proxied contents of foo\n', t.get('foo').read())
         finally:
@@ -1207,8 +1178,7 @@
 
     def not_proxied_in_env(self, env):
         self._install_env(env)
-        url = self.server.get_url()
-        t = self._transport(url)
+        t = self.get_readonly_transport()
         try:
             self.assertEqual('contents of foo\n', t.get('foo').read())
         finally:
@@ -1256,11 +1226,11 @@
             # pycurl *ignores* invalid proxy env variables. If that ever change
             # in the future, this test will fail indicating that pycurl do not
             # ignore anymore such variables.
-            self.not_proxied_in_env({'http_proxy': self.proxy_address})
+            self.not_proxied_in_env({'http_proxy': self.server_host_port})
         else:
             self.assertRaises(errors.InvalidURL,
                               self.proxied_in_env,
-                              {'http_proxy': self.proxy_address})
+                              {'http_proxy': self.server_host_port})
 
 
 class TestRanges(http_utils.TestCaseWithWebserver):
@@ -1269,24 +1239,24 @@
     def setUp(self):
         http_utils.TestCaseWithWebserver.setUp(self)
         self.build_tree_contents([('a', '0123456789')],)
-        server = self.get_readonly_server()
-        self.transport = self._transport(server.get_url())
 
     def create_transport_readonly_server(self):
         return http_server.HttpServer(protocol_version=self._protocol_version)
 
     def _file_contents(self, relpath, ranges):
+        t = self.get_readonly_transport()
         offsets = [ (start, end - start + 1) for start, end in ranges]
-        coalesce = self.transport._coalesce_offsets
+        coalesce = t._coalesce_offsets
         coalesced = list(coalesce(offsets, limit=0, fudge_factor=0))
-        code, data = self.transport._get(relpath, coalesced)
+        code, data = t._get(relpath, coalesced)
         self.assertTrue(code in (200, 206),'_get returns: %d' % code)
         for start, end in ranges:
             data.seek(start)
             yield data.read(end - start + 1)
 
     def _file_tail(self, relpath, tail_amount):
-        code, data = self.transport._get(relpath, [], tail_amount)
+        t = self.get_readonly_transport()
+        code, data = t._get(relpath, [], tail_amount)
         self.assertTrue(code in (200, 206),'_get returns: %d' % code)
         data.seek(-tail_amount, 2)
         return data.read(tail_amount)
@@ -1311,36 +1281,17 @@
 class TestHTTPRedirections(http_utils.TestCaseWithRedirectedWebserver):
     """Test redirection between http servers."""
 
-    def create_transport_secondary_server(self):
-        """Create the secondary server redirecting to the primary server"""
-        new = self.get_readonly_server()
-
-        redirecting = http_utils.HTTPServerRedirecting(
-            protocol_version=self._protocol_version)
-        redirecting.redirect_to(new.host, new.port)
-        return redirecting
-
     def setUp(self):
         super(TestHTTPRedirections, self).setUp()
         self.build_tree_contents([('a', '0123456789'),
                                   ('bundle',
                                   '# Bazaar revision bundle v0.9\n#\n')
                                   ],)
-        # The requests to the old server will be redirected to the new server
-        self.old_transport = self._transport(self.old_server.get_url())
 
     def test_redirected(self):
-        self.assertRaises(errors.RedirectRequested, self.old_transport.get, 'a')
-        t = self._transport(self.new_server.get_url())
-        self.assertEqual('0123456789', t.get('a').read())
-
-    def test_read_redirected_bundle_from_url(self):
-        from bzrlib.bundle import read_bundle_from_url
-        url = self.old_transport.abspath('bundle')
-        bundle = self.applyDeprecated(deprecated_in((1, 12, 0)),
-                read_bundle_from_url, url)
-        # If read_bundle_from_url was successful we get an empty bundle
-        self.assertEqual([], bundle.revisions)
+        self.assertRaises(errors.RedirectRequested,
+                          self.get_old_transport().get, 'a')
+        self.assertEqual('0123456789', self.get_new_transport().get('a').read())
 
 
 class RedirectedRequest(_urllib2_wrappers.Request):
@@ -1363,6 +1314,27 @@
     test.overrideAttr(_urllib2_wrappers, 'Request', RedirectedRequest)
 
 
+def cleanup_http_redirection_connections(test):
+    # Some sockets are opened but never seen by _urllib, so we trap them at
+    # the _urllib2_wrappers level to be able to clean them up.
+    def socket_disconnect(sock):
+        try:
+            sock.shutdown(socket.SHUT_RDWR)
+            sock.close()
+        except socket.error:
+            pass
+    def connect(connection):
+        test.http_connect_orig(connection)
+        test.addCleanup(socket_disconnect, connection.sock)
+    test.http_connect_orig = test.overrideAttr(
+        _urllib2_wrappers.HTTPConnection, 'connect', connect)
+    def connect(connection):
+        test.https_connect_orig(connection)
+        test.addCleanup(socket_disconnect, connection.sock)
+    test.https_connect_orig = test.overrideAttr(
+        _urllib2_wrappers.HTTPSConnection, 'connect', connect)
+
+
 class TestHTTPSilentRedirections(http_utils.TestCaseWithRedirectedWebserver):
     """Test redirections.
 
@@ -1381,9 +1353,10 @@
         if (features.pycurl.available()
             and self._transport == PyCurlTransport):
             raise tests.TestNotApplicable(
-                "pycurl doesn't redirect silently annymore")
+                "pycurl doesn't redirect silently anymore")
         super(TestHTTPSilentRedirections, self).setUp()
         install_redirected_request(self)
+        cleanup_http_redirection_connections(self)
         self.build_tree_contents([('a','a'),
                                   ('1/',),
                                   ('1/a', 'redirected once'),
@@ -1397,27 +1370,18 @@
                                   ('5/a', 'redirected 5 times'),
                                   ],)
 
-        self.old_transport = self._transport(self.old_server.get_url())
-
-    def create_transport_secondary_server(self):
-        """Create the secondary server, redirections are defined in the tests"""
-        return http_utils.HTTPServerRedirecting(
-            protocol_version=self._protocol_version)
-
     def test_one_redirection(self):
-        t = self.old_transport
-
-        req = RedirectedRequest('GET', t.abspath('a'))
+        t = self.get_old_transport()
+        req = RedirectedRequest('GET', t._remote_path('a'))
         new_prefix = 'http://%s:%s' % (self.new_server.host,
                                        self.new_server.port)
         self.old_server.redirections = \
             [('(.*)', r'%s/1\1' % (new_prefix), 301),]
-        self.assertEqual('redirected once',t._perform(req).read())
+        self.assertEqual('redirected once', t._perform(req).read())
 
     def test_five_redirections(self):
-        t = self.old_transport
-
-        req = RedirectedRequest('GET', t.abspath('a'))
+        t = self.get_old_transport()
+        req = RedirectedRequest('GET', t._remote_path('a'))
         old_prefix = 'http://%s:%s' % (self.old_server.host,
                                        self.old_server.port)
         new_prefix = 'http://%s:%s' % (self.new_server.host,
@@ -1429,7 +1393,7 @@
             ('/4(.*)', r'%s/5\1' % (new_prefix), 301),
             ('(/[^/]+)', r'%s/1\1' % (old_prefix), 301),
             ]
-        self.assertEqual('redirected 5 times',t._perform(req).read())
+        self.assertEqual('redirected 5 times', t._perform(req).read())
 
 
 class TestDoCatchRedirections(http_utils.TestCaseWithRedirectedWebserver):
@@ -1438,14 +1402,15 @@
     def setUp(self):
         super(TestDoCatchRedirections, self).setUp()
         self.build_tree_contents([('a', '0123456789'),],)
-
-        self.old_transport = self._transport(self.old_server.get_url())
-
-    def get_a(self, transport):
-        return transport.get('a')
+        cleanup_http_redirection_connections(self)
+
+        self.old_transport = self.get_old_transport()
+
+    def get_a(self, t):
+        return t.get('a')
 
     def test_no_redirection(self):
-        t = self._transport(self.new_server.get_url())
+        t = self.get_new_transport()
 
         # We use None for redirected so that we fail if redirected
         self.assertEqual('0123456789',
@@ -1455,10 +1420,10 @@
     def test_one_redirection(self):
         self.redirections = 0
 
-        def redirected(transport, exception, redirection_notice):
+        def redirected(t, exception, redirection_notice):
             self.redirections += 1
-            dir, file = urlutils.split(exception.target)
-            return self._transport(dir)
+            redirected_t = t._redirected_to(exception.source, exception.target)
+            return redirected_t
 
         self.assertEqual('0123456789',
                          transport.do_catching_redirections(
@@ -1494,7 +1459,9 @@
                                   ('b', 'contents of b\n'),])
 
     def create_transport_readonly_server(self):
-        return self._auth_server(protocol_version=self._protocol_version)
+        server = self._auth_server(protocol_version=self._protocol_version)
+        server._url_protocol = self._url_protocol
+        return server
 
     def _testing_pycurl(self):
         # TODO: This is duplicated for lots of the classes in this file
@@ -1513,7 +1480,8 @@
         return url
 
     def get_user_transport(self, user, password):
-        return self._transport(self.get_user_url(user, password))
+        t = transport.get_transport(self.get_user_url(user, password))
+        return t
 
     def test_no_user(self):
         self.server.add_user('joe', 'foo')
@@ -1704,7 +1672,7 @@
 
     def get_user_transport(self, user, password):
         self._install_env({'all_proxy': self.get_user_url(user, password)})
-        return self._transport(self.server.get_url())
+        return TestAuth.get_user_transport(self, user, password)
 
     def _install_env(self, env):
         for name, value in env.iteritems():
@@ -1737,6 +1705,7 @@
         self.readfile = StringIO(socket_read_content)
         self.writefile = StringIO()
         self.writefile.close = lambda: None
+        self.close = lambda: None
 
     def makefile(self, mode='r', bufsize=None):
         if 'r' in mode:
@@ -1752,16 +1721,19 @@
         # We use the VFS layer as part of HTTP tunnelling tests.
         self._captureVar('BZR_NO_SMART_VFS', None)
         self.transport_readonly_server = http_utils.HTTPServerWithSmarts
+        self.http_server = self.get_readonly_server()
 
     def create_transport_readonly_server(self):
-        return http_utils.HTTPServerWithSmarts(
+        server = http_utils.HTTPServerWithSmarts(
             protocol_version=self._protocol_version)
+        server._url_protocol = self._url_protocol
+        return server
 
     def test_open_bzrdir(self):
         branch = self.make_branch('relpath')
-        http_server = self.get_readonly_server()
-        url = http_server.get_url() + 'relpath'
+        url = self.http_server.get_url() + 'relpath'
         bd = bzrdir.BzrDir.open(url)
+        self.addCleanup(bd.transport.disconnect)
         self.assertIsInstance(bd, _mod_remote.RemoteBzrDir)
 
     def test_bulk_data(self):
@@ -1769,8 +1741,7 @@
         # The 'readv' command in the smart protocol both sends and receives
         # bulk data, so we use that.
         self.build_tree(['data-file'])
-        http_server = self.get_readonly_server()
-        http_transport = self._transport(http_server.get_url())
+        http_transport = transport.get_transport(self.http_server.get_url())
         medium = http_transport.get_smart_medium()
         # Since we provide the medium, the url below will be mostly ignored
         # during the test, as long as the path is '/'.
@@ -1784,15 +1755,14 @@
         post_body = 'hello\n'
         expected_reply_body = 'ok\x012\n'
 
-        http_server = self.get_readonly_server()
-        http_transport = self._transport(http_server.get_url())
+        http_transport = transport.get_transport(self.http_server.get_url())
         medium = http_transport.get_smart_medium()
         response = medium.send_http_smart_request(post_body)
         reply_body = response.read()
         self.assertEqual(expected_reply_body, reply_body)
 
     def test_smart_http_server_post_request_handler(self):
-        httpd = self.get_readonly_server()._get_httpd()
+        httpd = self.http_server.server
 
         socket = SampleSocket(
             'POST /.bzr/smart %s \r\n' % self._protocol_version
@@ -1830,14 +1800,14 @@
 
     def test_probe_smart_server(self):
         """Test error handling against server refusing smart requests."""
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         # No need to build a valid smart request here, the server will not even
         # try to interpret it.
         self.assertRaises(errors.SmartProtocolError,
                           t.get_smart_medium().send_http_smart_request,
                           'whatever')
 
+
 class Test_redirected_to(tests.TestCase):
 
     def test_redirected_to_subdir(self):
@@ -1896,7 +1866,7 @@
     line.
     """
 
-    def handle_one_request(self):
+    def _handle_one_request(self):
         tcs = self.server.test_case_server
         requestline = self.rfile.readline()
         headers = self.MessageClass(self.rfile, 0)
@@ -1966,16 +1936,13 @@
         # We override at class level because constructors may propagate the
         # bound method and render instance overriding ineffective (an
         # alternative would be to define a specific ui factory instead...)
-        self.orig_report_activity = self._transport._report_activity
-        self._transport._report_activity = report_activity
-
-    def tearDown(self):
-        self._transport._report_activity = self.orig_report_activity
-        self.server.stop_server()
-        tests.TestCase.tearDown(self)
+        self.overrideAttr(self._transport, '_report_activity', report_activity)
+        self.addCleanup(self.server.stop_server)
 
     def get_transport(self):
-        return self._transport(self.server.get_url())
+        t = self._transport(self.server.get_url())
+        # FIXME: Needs cleanup -- vila 20100611
+        return t
 
     def assertActivitiesMatch(self):
         self.assertEqual(self.server.bytes_read,
@@ -2086,7 +2053,7 @@
 '''
         t = self.get_transport()
         # We must send a single line of body bytes, see
-        # PredefinedRequestHandler.handle_one_request
+        # PredefinedRequestHandler._handle_one_request
         code, f = t._post('abc def end-of-body\n')
         self.assertEqual('lalala whatever as long as itsssss\n', f.read())
         self.assertActivitiesMatch()
@@ -2095,51 +2062,22 @@
 class TestActivity(tests.TestCase, TestActivityMixin):
 
     def setUp(self):
-        tests.TestCase.setUp(self)
-        self.server = self._activity_server(self._protocol_version)
-        self.server.start_server()
-        self.activities = {}
-        def report_activity(t, bytes, direction):
-            count = self.activities.get(direction, 0)
-            count += bytes
-            self.activities[direction] = count
-
-        # We override at class level because constructors may propagate the
-        # bound method and render instance overriding ineffective (an
-        # alternative would be to define a specific ui factory instead...)
-        self.orig_report_activity = self._transport._report_activity
-        self._transport._report_activity = report_activity
-
-    def tearDown(self):
-        self._transport._report_activity = self.orig_report_activity
-        self.server.stop_server()
-        tests.TestCase.tearDown(self)
+        TestActivityMixin.setUp(self)
 
 
 class TestNoReportActivity(tests.TestCase, TestActivityMixin):
 
+    # Unlike TestActivity, we are really testing ReportingFileSocket and
+    # ReportingSocket, so we don't need all the parametrization. Since
+    # ReportingFileSocket and ReportingSocket are wrappers, it's easier to
+    # test them through their use by the transport than directly (that's a
+    # bit less clean but far more simpler and effective).
+    _activity_server = ActivityHTTPServer
+    _protocol_version = 'HTTP/1.1'
+
     def setUp(self):
-        tests.TestCase.setUp(self)
-        # Unlike TestActivity, we are really testing ReportingFileSocket and
-        # ReportingSocket, so we don't need all the parametrization. Since
-        # ReportingFileSocket and ReportingSocket are wrappers, it's easier to
-        # test them through their use by the transport than directly (that's a
-        # bit less clean but far more simpler and effective).
-        self.server = ActivityHTTPServer('HTTP/1.1')
-        self._transport=_urllib.HttpTransport_urllib
-
-        self.server.start_server()
-
-        # We override at class level because constructors may propagate the
-        # bound method and render instance overriding ineffective (an
-        # alternative would be to define a specific ui factory instead...)
-        self.orig_report_activity = self._transport._report_activity
-        self._transport._report_activity = None
-
-    def tearDown(self):
-        self._transport._report_activity = self.orig_report_activity
-        self.server.stop_server()
-        tests.TestCase.tearDown(self)
+        self._transport =_urllib.HttpTransport_urllib
+        TestActivityMixin.setUp(self)
 
     def assertActivitiesMatch(self):
         # Nothing to check here
@@ -2155,17 +2093,6 @@
     _auth_server = http_utils.HTTPBasicAuthServer
     _transport = _urllib.HttpTransport_urllib
 
-    def create_transport_readonly_server(self):
-        return self._auth_server()
-
-    def create_transport_secondary_server(self):
-        """Create the secondary server redirecting to the primary server"""
-        new = self.get_readonly_server()
-
-        redirecting = http_utils.HTTPServerRedirecting()
-        redirecting.redirect_to(new.host, new.port)
-        return redirecting
-
     def setUp(self):
         super(TestAuthOnRedirected, self).setUp()
         self.build_tree_contents([('a','a'),
@@ -2176,19 +2103,26 @@
                                        self.new_server.port)
         self.old_server.redirections = [
             ('(.*)', r'%s/1\1' % (new_prefix), 301),]
-        self.old_transport = self._transport(self.old_server.get_url())
+        self.old_transport = self.get_old_transport()
         self.new_server.add_user('joe', 'foo')
-
-    def get_a(self, transport):
-        return transport.get('a')
+        cleanup_http_redirection_connections(self)
+
+    def create_transport_readonly_server(self):
+        server = self._auth_server(protocol_version=self._protocol_version)
+        server._url_protocol = self._url_protocol
+        return server
+
+    def get_a(self, t):
+        return t.get('a')
 
     def test_auth_on_redirected_via_do_catching_redirections(self):
         self.redirections = 0
 
-        def redirected(transport, exception, redirection_notice):
+        def redirected(t, exception, redirection_notice):
             self.redirections += 1
-            dir, file = urlutils.split(exception.target)
-            return self._transport(dir)
+            redirected_t = t._redirected_to(exception.source, exception.target)
+            self.addCleanup(redirected_t.disconnect)
+            return redirected_t
 
         stdout = tests.StringIOWrapper()
         stderr = tests.StringIOWrapper()
@@ -2215,7 +2149,7 @@
                                        self.new_server.port)
         self.old_server.redirections = [
             ('(.*)', r'%s/1\1' % (new_prefix), 301),]
-        self.assertEqual('redirected once',t._perform(req).read())
+        self.assertEqual('redirected once', t._perform(req).read())
         # stdin should be empty
         self.assertEqual('', ui.ui_factory.stdin.readline())
         # stdout should be empty, stderr will contains the prompts

=== modified file 'bzrlib/tests/test_ignores.py'
--- bzrlib/tests/test_ignores.py	2009-12-26 21:31:56 +0000
+++ bzrlib/tests/test_ignores.py	2010-05-03 04:51:58 +0000
@@ -165,27 +165,55 @@
 
 
 class TestTreeIgnores(TestCaseWithTransport):
+    
+    def assertPatternsEquals(self, patterns):
+        contents = open(".bzrignore", 'rU').read().strip().split('\n')
+        self.assertEquals(sorted(patterns), sorted(contents))
 
     def test_new_file(self):
         tree = self.make_branch_and_tree(".")
         ignores.tree_ignores_add_patterns(tree, ["myentry"])
         self.assertTrue(tree.has_filename(".bzrignore"))
-        self.assertEquals("myentry\n",
-                          open(".bzrignore", 'r').read())
+        self.assertPatternsEquals(["myentry"])
 
     def test_add_to_existing(self):
         tree = self.make_branch_and_tree(".")
         self.build_tree_contents([('.bzrignore', "myentry1\n")])
         tree.add([".bzrignore"])
         ignores.tree_ignores_add_patterns(tree, ["myentry2", "foo"])
-        self.assertEquals("myentry1\nmyentry2\nfoo\n",
-                          open(".bzrignore", 'r').read())
+        self.assertPatternsEquals(["myentry1", "myentry2", "foo"])
 
     def test_adds_ending_newline(self):
         tree = self.make_branch_and_tree(".")
         self.build_tree_contents([('.bzrignore', "myentry1")])
         tree.add([".bzrignore"])
         ignores.tree_ignores_add_patterns(tree, ["myentry2"])
-        self.assertEquals("myentry1\nmyentry2\n",
-                          open(".bzrignore", 'r').read())
-
+        self.assertPatternsEquals(["myentry1", "myentry2"])
+        text = open(".bzrignore", 'r').read()
+        self.assertTrue(text.endswith('\r\n') or
+                        text.endswith('\n') or
+                        text.endswith('\r'))
+
+    def test_does_not_add_dupe(self):
+        tree = self.make_branch_and_tree(".")
+        self.build_tree_contents([('.bzrignore', "myentry\n")])
+        tree.add([".bzrignore"])
+        ignores.tree_ignores_add_patterns(tree, ["myentry"])
+        self.assertPatternsEquals(["myentry"])
+
+    def test_non_ascii(self):
+        tree = self.make_branch_and_tree(".")
+        self.build_tree_contents([('.bzrignore',
+                                   u"myentry\u1234\n".encode('utf-8'))])
+        tree.add([".bzrignore"])
+        ignores.tree_ignores_add_patterns(tree, [u"myentry\u5678"])
+        self.assertPatternsEquals([u"myentry\u1234".encode('utf-8'),
+                                   u"myentry\u5678".encode('utf-8')])
+
+    def test_crlf(self):
+        tree = self.make_branch_and_tree(".")
+        self.build_tree_contents([('.bzrignore', "myentry1\r\n")])
+        tree.add([".bzrignore"])
+        ignores.tree_ignores_add_patterns(tree, ["myentry2", "foo"])
+        self.assertEquals(open('.bzrignore', 'rb').read(), 'myentry1\r\nmyentry2\r\nfoo\r\n')
+        self.assertPatternsEquals(["myentry1", "myentry2", "foo"])

=== modified file 'bzrlib/tests/test_import_tariff.py'
--- bzrlib/tests/test_import_tariff.py	2010-04-01 04:38:26 +0000
+++ bzrlib/tests/test_import_tariff.py	2010-06-17 16:54:26 +0000
@@ -55,13 +55,15 @@
         # explicitly do want to test against things installed there, therefore
         # we pass it through.
         env_changes = dict(PYTHONVERBOSE='1')
-        for name in ['BZR_HOME', 'BZR_PLUGIN_PATH', 'HOME',]:
+        for name in ['BZR_HOME', 'BZR_PLUGIN_PATH',
+                     'BZR_DISABLE_PLUGINS', 'BZR_PLUGINS_AT',
+                     'HOME',]:
             env_changes[name] = self._old_env.get(name)
         out, err = self.run_bzr_subprocess(args,
             allow_plugins=(not are_plugins_disabled()),
             env_changes=env_changes)
 
-        self.addDetail('subprocess_stderr', 
+        self.addDetail('subprocess_stderr',
             content.Content(content.ContentType("text", "plain"),
                 lambda:[err]))
 
@@ -71,7 +73,7 @@
                 bad_modules.append(module_name)
 
         if bad_modules:
-            self.fail("command %r loaded forbidden modules %r" 
+            self.fail("command %r loaded forbidden modules %r"
                 % (args, bad_modules))
         return out, err
 
@@ -93,13 +95,17 @@
             'bzrlib.bundle.commands',
             'bzrlib.cmd_version_info',
             'bzrlib.foreign',
+            'bzrlib.merge3',
+            'bzrlib.patiencediff',
             'bzrlib.remote',
             'bzrlib.sign_my_commits',
             'bzrlib.smart',
+            'bzrlib.transform',
+            'kerberos',
             'smtplib',
             'tarfile',
             ])
         # TODO: similar test for repository-only operations, checking we avoid
         # loading wt-specific stuff
         #
-        # See https://bugs.edge.launchpad.net/bzr/+bug/553017
+        # See https://bugs.launchpad.net/bzr/+bug/553017

=== modified file 'bzrlib/tests/test_index.py'
--- bzrlib/tests/test_index.py	2010-04-08 07:01:10 +0000
+++ bzrlib/tests/test_index.py	2010-06-20 11:18:38 +0000
@@ -16,16 +16,18 @@
 
 """Tests for indices."""
 
-from bzrlib import errors
-from bzrlib.index import *
-from bzrlib.tests import TestCaseWithMemoryTransport
-from bzrlib.transport import get_transport
-
-
-class TestGraphIndexBuilder(TestCaseWithMemoryTransport):
+from bzrlib import (
+    errors,
+    index,
+    tests,
+    transport,
+    )
+
+
+class TestGraphIndexBuilder(tests.TestCaseWithMemoryTransport):
 
     def test_build_index_empty(self):
-        builder = GraphIndexBuilder()
+        builder = index.GraphIndexBuilder()
         stream = builder.finish()
         contents = stream.read()
         self.assertEqual(
@@ -33,7 +35,7 @@
             contents)
 
     def test_build_index_empty_two_element_keys(self):
-        builder = GraphIndexBuilder(key_elements=2)
+        builder = index.GraphIndexBuilder(key_elements=2)
         stream = builder.finish()
         contents = stream.read()
         self.assertEqual(
@@ -41,7 +43,7 @@
             contents)
 
     def test_build_index_one_reference_list_empty(self):
-        builder = GraphIndexBuilder(reference_lists=1)
+        builder = index.GraphIndexBuilder(reference_lists=1)
         stream = builder.finish()
         contents = stream.read()
         self.assertEqual(
@@ -49,7 +51,7 @@
             contents)
 
     def test_build_index_two_reference_list_empty(self):
-        builder = GraphIndexBuilder(reference_lists=2)
+        builder = index.GraphIndexBuilder(reference_lists=2)
         stream = builder.finish()
         contents = stream.read()
         self.assertEqual(
@@ -57,7 +59,7 @@
             contents)
 
     def test_build_index_one_node_no_refs(self):
-        builder = GraphIndexBuilder()
+        builder = index.GraphIndexBuilder()
         builder.add_node(('akey', ), 'data')
         stream = builder.finish()
         contents = stream.read()
@@ -66,7 +68,7 @@
             "akey\x00\x00\x00data\n\n", contents)
 
     def test_build_index_one_node_no_refs_accepts_empty_reflist(self):
-        builder = GraphIndexBuilder()
+        builder = index.GraphIndexBuilder()
         builder.add_node(('akey', ), 'data', ())
         stream = builder.finish()
         contents = stream.read()
@@ -78,7 +80,7 @@
         # multipart keys are separated by \x00 - because they are fixed length,
         # not variable this does not cause any issues, and seems clearer to the
         # author.
-        builder = GraphIndexBuilder(key_elements=2)
+        builder = index.GraphIndexBuilder(key_elements=2)
         builder.add_node(('akey', 'secondpart'), 'data')
         stream = builder.finish()
         contents = stream.read()
@@ -87,7 +89,7 @@
             "akey\x00secondpart\x00\x00\x00data\n\n", contents)
 
     def test_add_node_empty_value(self):
-        builder = GraphIndexBuilder()
+        builder = index.GraphIndexBuilder()
         builder.add_node(('akey', ), '')
         stream = builder.finish()
         contents = stream.read()
@@ -97,7 +99,7 @@
 
     def test_build_index_nodes_sorted(self):
         # the highest sorted node comes first.
-        builder = GraphIndexBuilder()
+        builder = index.GraphIndexBuilder()
         # use three to have a good chance of glitching dictionary hash
         # lookups etc. Insert in randomish order that is not correct
         # and not the reverse of the correct order.
@@ -115,7 +117,7 @@
 
     def test_build_index_2_element_key_nodes_sorted(self):
         # multiple element keys are sorted first-key, second-key.
-        builder = GraphIndexBuilder(key_elements=2)
+        builder = index.GraphIndexBuilder(key_elements=2)
         # use three values of each key element, to have a good chance of
         # glitching dictionary hash lookups etc. Insert in randomish order that
         # is not correct and not the reverse of the correct order.
@@ -144,7 +146,7 @@
             "\n", contents)
 
     def test_build_index_reference_lists_are_included_one(self):
-        builder = GraphIndexBuilder(reference_lists=1)
+        builder = index.GraphIndexBuilder(reference_lists=1)
         builder.add_node(('key', ), 'data', ([], ))
         stream = builder.finish()
         contents = stream.read()
@@ -154,7 +156,7 @@
             "\n", contents)
 
     def test_build_index_reference_lists_with_2_element_keys(self):
-        builder = GraphIndexBuilder(reference_lists=1, key_elements=2)
+        builder = index.GraphIndexBuilder(reference_lists=1, key_elements=2)
         builder.add_node(('key', 'key2'), 'data', ([], ))
         stream = builder.finish()
         contents = stream.read()
@@ -164,7 +166,7 @@
             "\n", contents)
 
     def test_build_index_reference_lists_are_included_two(self):
-        builder = GraphIndexBuilder(reference_lists=2)
+        builder = index.GraphIndexBuilder(reference_lists=2)
         builder.add_node(('key', ), 'data', ([], []))
         stream = builder.finish()
         contents = stream.read()
@@ -174,12 +176,12 @@
             "\n", contents)
 
     def test_clear_cache(self):
-        builder = GraphIndexBuilder(reference_lists=2)
+        builder = index.GraphIndexBuilder(reference_lists=2)
         # This is a no-op, but the api should exist
         builder.clear_cache()
 
     def test_node_references_are_byte_offsets(self):
-        builder = GraphIndexBuilder(reference_lists=1)
+        builder = index.GraphIndexBuilder(reference_lists=1)
         builder.add_node(('reference', ), 'data', ([], ))
         builder.add_node(('key', ), 'data', ([('reference', )], ))
         stream = builder.finish()
@@ -191,10 +193,11 @@
             "\n", contents)
 
     def test_node_references_are_cr_delimited(self):
-        builder = GraphIndexBuilder(reference_lists=1)
+        builder = index.GraphIndexBuilder(reference_lists=1)
         builder.add_node(('reference', ), 'data', ([], ))
         builder.add_node(('reference2', ), 'data', ([], ))
-        builder.add_node(('key', ), 'data', ([('reference', ), ('reference2', )], ))
+        builder.add_node(('key', ), 'data',
+                         ([('reference', ), ('reference2', )], ))
         stream = builder.finish()
         contents = stream.read()
         self.assertEqual(
@@ -205,9 +208,10 @@
             "\n", contents)
 
     def test_multiple_reference_lists_are_tab_delimited(self):
-        builder = GraphIndexBuilder(reference_lists=2)
+        builder = index.GraphIndexBuilder(reference_lists=2)
         builder.add_node(('keference', ), 'data', ([], []))
-        builder.add_node(('rey', ), 'data', ([('keference', )], [('keference', )]))
+        builder.add_node(('rey', ), 'data',
+                         ([('keference', )], [('keference', )]))
         stream = builder.finish()
         contents = stream.read()
         self.assertEqual(
@@ -217,8 +221,9 @@
             "\n", contents)
 
     def test_add_node_referencing_missing_key_makes_absent(self):
-        builder = GraphIndexBuilder(reference_lists=1)
-        builder.add_node(('rey', ), 'data', ([('beference', ), ('aeference2', )], ))
+        builder = index.GraphIndexBuilder(reference_lists=1)
+        builder.add_node(('rey', ), 'data',
+                         ([('beference', ), ('aeference2', )], ))
         stream = builder.finish()
         contents = stream.read()
         self.assertEqual(
@@ -230,7 +235,7 @@
 
     def test_node_references_three_digits(self):
         # test the node digit expands as needed.
-        builder = GraphIndexBuilder(reference_lists=1)
+        builder = index.GraphIndexBuilder(reference_lists=1)
         references = [(str(val), ) for val in reversed(range(9))]
         builder.add_node(('2-key', ), '', (references, ))
         stream = builder.finish()
@@ -252,7 +257,7 @@
     def test_absent_has_no_reference_overhead(self):
         # the offsets after an absent record should be correct when there are
         # >1 reference lists.
-        builder = GraphIndexBuilder(reference_lists=2)
+        builder = index.GraphIndexBuilder(reference_lists=2)
         builder.add_node(('parent', ), '', ([('aail', ), ('zther', )], []))
         stream = builder.finish()
         contents = stream.read()
@@ -264,7 +269,7 @@
             "\n", contents)
 
     def test_add_node_bad_key(self):
-        builder = GraphIndexBuilder()
+        builder = index.GraphIndexBuilder()
         for bad_char in '\t\n\x0b\x0c\r\x00 ':
             self.assertRaises(errors.BadIndexKey, builder.add_node,
                 ('a%skey' % bad_char, ), 'data')
@@ -279,30 +284,30 @@
         self.assertRaises(errors.BadIndexKey, builder.add_node,
                 ('primary', 'secondary'), 'data')
         # secondary key elements get checked too:
-        builder = GraphIndexBuilder(key_elements=2)
+        builder = index.GraphIndexBuilder(key_elements=2)
         for bad_char in '\t\n\x0b\x0c\r\x00 ':
             self.assertRaises(errors.BadIndexKey, builder.add_node,
                 ('prefix', 'a%skey' % bad_char), 'data')
 
     def test_add_node_bad_data(self):
-        builder = GraphIndexBuilder()
+        builder = index.GraphIndexBuilder()
         self.assertRaises(errors.BadIndexValue, builder.add_node, ('akey', ),
             'data\naa')
         self.assertRaises(errors.BadIndexValue, builder.add_node, ('akey', ),
             'data\x00aa')
 
     def test_add_node_bad_mismatched_ref_lists_length(self):
-        builder = GraphIndexBuilder()
+        builder = index.GraphIndexBuilder()
         self.assertRaises(errors.BadIndexValue, builder.add_node, ('akey', ),
             'data aa', ([], ))
-        builder = GraphIndexBuilder(reference_lists=1)
+        builder = index.GraphIndexBuilder(reference_lists=1)
         self.assertRaises(errors.BadIndexValue, builder.add_node, ('akey', ),
             'data aa')
         self.assertRaises(errors.BadIndexValue, builder.add_node, ('akey', ),
             'data aa', (), )
         self.assertRaises(errors.BadIndexValue, builder.add_node, ('akey', ),
             'data aa', ([], []))
-        builder = GraphIndexBuilder(reference_lists=2)
+        builder = index.GraphIndexBuilder(reference_lists=2)
         self.assertRaises(errors.BadIndexValue, builder.add_node, ('akey', ),
             'data aa')
         self.assertRaises(errors.BadIndexValue, builder.add_node, ('akey', ),
@@ -312,7 +317,7 @@
 
     def test_add_node_bad_key_in_reference_lists(self):
         # first list, first key - trivial
-        builder = GraphIndexBuilder(reference_lists=1)
+        builder = index.GraphIndexBuilder(reference_lists=1)
         self.assertRaises(errors.BadIndexKey, builder.add_node, ('akey', ),
             'data aa', ([('a key', )], ))
         # references keys must be tuples too
@@ -329,41 +334,41 @@
             'data aa', ([('agoodkey', ), ('that is a bad key', )], ))
         # and if there is more than one list it should be getting checked
         # too
-        builder = GraphIndexBuilder(reference_lists=2)
+        builder = index.GraphIndexBuilder(reference_lists=2)
         self.assertRaises(errors.BadIndexKey, builder.add_node, ('akey', ),
             'data aa', ([], ['a bad key']))
 
     def test_add_duplicate_key(self):
-        builder = GraphIndexBuilder()
+        builder = index.GraphIndexBuilder()
         builder.add_node(('key', ), 'data')
-        self.assertRaises(errors.BadIndexDuplicateKey, builder.add_node, ('key', ),
-            'data')
+        self.assertRaises(errors.BadIndexDuplicateKey,
+                          builder.add_node, ('key', ), 'data')
 
     def test_add_duplicate_key_2_elements(self):
-        builder = GraphIndexBuilder(key_elements=2)
+        builder = index.GraphIndexBuilder(key_elements=2)
         builder.add_node(('key', 'key'), 'data')
         self.assertRaises(errors.BadIndexDuplicateKey, builder.add_node,
             ('key', 'key'), 'data')
 
     def test_add_key_after_referencing_key(self):
-        builder = GraphIndexBuilder(reference_lists=1)
+        builder = index.GraphIndexBuilder(reference_lists=1)
         builder.add_node(('key', ), 'data', ([('reference', )], ))
         builder.add_node(('reference', ), 'data', ([],))
 
     def test_add_key_after_referencing_key_2_elements(self):
-        builder = GraphIndexBuilder(reference_lists=1, key_elements=2)
+        builder = index.GraphIndexBuilder(reference_lists=1, key_elements=2)
         builder.add_node(('k', 'ey'), 'data', ([('reference', 'tokey')], ))
         builder.add_node(('reference', 'tokey'), 'data', ([],))
 
     def test_set_optimize(self):
-        builder = GraphIndexBuilder(reference_lists=1, key_elements=2)
+        builder = index.GraphIndexBuilder(reference_lists=1, key_elements=2)
         builder.set_optimize(for_size=True)
         self.assertTrue(builder._optimize_for_size)
         builder.set_optimize(for_size=False)
         self.assertFalse(builder._optimize_for_size)
 
 
-class TestGraphIndex(TestCaseWithMemoryTransport):
+class TestGraphIndex(tests.TestCaseWithMemoryTransport):
 
     def make_key(self, number):
         return (str(number) + 'X'*100,)
@@ -380,24 +385,24 @@
         return nodes
 
     def make_index(self, ref_lists=0, key_elements=1, nodes=[]):
-        builder = GraphIndexBuilder(ref_lists, key_elements=key_elements)
+        builder = index.GraphIndexBuilder(ref_lists, key_elements=key_elements)
         for key, value, references in nodes:
             builder.add_node(key, value, references)
         stream = builder.finish()
-        trans = get_transport('trace+' + self.get_url())
+        trans = transport.get_transport('trace+' + self.get_url())
         size = trans.put_file('index', stream)
-        return GraphIndex(trans, 'index', size)
+        return index.GraphIndex(trans, 'index', size)
 
     def make_index_with_offset(self, ref_lists=0, key_elements=1, nodes=[],
                                offset=0):
-        builder = GraphIndexBuilder(ref_lists, key_elements=key_elements)
+        builder = index.GraphIndexBuilder(ref_lists, key_elements=key_elements)
         for key, value, references in nodes:
             builder.add_node(key, value, references)
         content = builder.finish().read()
         size = len(content)
         trans = self.get_transport()
         trans.put_bytes('index', (' '*offset) + content)
-        return GraphIndex(trans, 'index', size, offset=offset)
+        return index.GraphIndex(trans, 'index', size, offset=offset)
 
     def test_clear_cache(self):
         index = self.make_index()
@@ -408,18 +413,18 @@
     def test_open_bad_index_no_error(self):
         trans = self.get_transport()
         trans.put_bytes('name', "not an index\n")
-        index = GraphIndex(trans, 'name', 13)
+        idx = index.GraphIndex(trans, 'name', 13)
 
     def test_with_offset(self):
         nodes = self.make_nodes(200)
-        index = self.make_index_with_offset(offset=1234567, nodes=nodes)
-        self.assertEqual(200, index.key_count())
+        idx = self.make_index_with_offset(offset=1234567, nodes=nodes)
+        self.assertEqual(200, idx.key_count())
 
     def test_buffer_all_with_offset(self):
         nodes = self.make_nodes(200)
-        index = self.make_index_with_offset(offset=1234567, nodes=nodes)
-        index._buffer_all()
-        self.assertEqual(200, index.key_count())
+        idx = self.make_index_with_offset(offset=1234567, nodes=nodes)
+        idx._buffer_all()
+        self.assertEqual(200, idx.key_count())
 
     def test_side_effect_buffering_with_offset(self):
         nodes = self.make_nodes(20)
@@ -830,9 +835,9 @@
         self.assertEqual([], list(index.iter_entries([('a', )])))
 
     def test_iter_missing_entry_empty_no_size(self):
-        index = self.make_index()
-        index = GraphIndex(index._transport, 'index', None)
-        self.assertEqual([], list(index.iter_entries([('a', )])))
+        idx = self.make_index()
+        idx = index.GraphIndex(idx._transport, 'index', None)
+        self.assertEqual([], list(idx.iter_entries([('a', )])))
 
     def test_iter_key_prefix_1_element_key_None(self):
         index = self.make_index()
@@ -928,17 +933,17 @@
     def test_validate_bad_index_errors(self):
         trans = self.get_transport()
         trans.put_bytes('name', "not an index\n")
-        index = GraphIndex(trans, 'name', 13)
-        self.assertRaises(errors.BadIndexFormatSignature, index.validate)
+        idx = index.GraphIndex(trans, 'name', 13)
+        self.assertRaises(errors.BadIndexFormatSignature, idx.validate)
 
     def test_validate_bad_node_refs(self):
-        index = self.make_index(2)
+        idx = self.make_index(2)
         trans = self.get_transport()
         content = trans.get_bytes('index')
         # change the options line to end with a rather than a parseable number
         new_content = content[:-2] + 'a\n\n'
         trans.put_bytes('index', new_content)
-        self.assertRaises(errors.BadIndexOptions, index.validate)
+        self.assertRaises(errors.BadIndexOptions, idx.validate)
 
     def test_validate_missing_end_line_empty(self):
         index = self.make_index(2)
@@ -1049,25 +1054,25 @@
         self.assertEqual(set(), search_keys)
 
     def test_supports_unlimited_cache(self):
-        builder = GraphIndexBuilder(0, key_elements=1)
+        builder = index.GraphIndexBuilder(0, key_elements=1)
         stream = builder.finish()
-        trans = get_transport(self.get_url())
+        trans = transport.get_transport(self.get_url())
         size = trans.put_file('index', stream)
         # It doesn't matter what unlimited_cache does here, just that it can be
         # passed
-        index = GraphIndex(trans, 'index', size, unlimited_cache=True)
-
-
-class TestCombinedGraphIndex(TestCaseWithMemoryTransport):
+        idx = index.GraphIndex(trans, 'index', size, unlimited_cache=True)
+
+
+class TestCombinedGraphIndex(tests.TestCaseWithMemoryTransport):
 
     def make_index(self, name, ref_lists=0, key_elements=1, nodes=[]):
-        builder = GraphIndexBuilder(ref_lists, key_elements=key_elements)
+        builder = index.GraphIndexBuilder(ref_lists, key_elements=key_elements)
         for key, value, references in nodes:
             builder.add_node(key, value, references)
         stream = builder.finish()
         trans = self.get_transport()
         size = trans.put_file(name, stream)
-        return GraphIndex(trans, name, size)
+        return index.GraphIndex(trans, name, size)
 
     def make_combined_index_with_missing(self, missing=['1', '2']):
         """Create a CombinedGraphIndex which will have missing indexes.
@@ -1079,9 +1084,9 @@
         :param missing: The underlying indexes to delete
         :return: (CombinedGraphIndex, reload_counter)
         """
-        index1 = self.make_index('1', nodes=[(('1',), '', ())])
-        index2 = self.make_index('2', nodes=[(('2',), '', ())])
-        index3 = self.make_index('3', nodes=[
+        idx1 = self.make_index('1', nodes=[(('1',), '', ())])
+        idx2 = self.make_index('2', nodes=[(('2',), '', ())])
+        idx3 = self.make_index('3', nodes=[
             (('1',), '', ()),
             (('2',), '', ())])
 
@@ -1089,29 +1094,30 @@
         reload_counter = [0, 0, 0]
         def reload():
             reload_counter[0] += 1
-            new_indices = [index3]
-            if index._indices == new_indices:
+            new_indices = [idx3]
+            if idx._indices == new_indices:
                 reload_counter[2] += 1
                 return False
             reload_counter[1] += 1
-            index._indices[:] = new_indices
+            idx._indices[:] = new_indices
             return True
-        index = CombinedGraphIndex([index1, index2], reload_func=reload)
+        idx = index.CombinedGraphIndex([idx1, idx2], reload_func=reload)
         trans = self.get_transport()
         for fname in missing:
             trans.delete(fname)
-        return index, reload_counter
+        return idx, reload_counter
 
     def test_open_missing_index_no_error(self):
         trans = self.get_transport()
-        index1 = GraphIndex(trans, 'missing', 100)
-        index = CombinedGraphIndex([index1])
+        idx1 = index.GraphIndex(trans, 'missing', 100)
+        idx = index.CombinedGraphIndex([idx1])
 
     def test_add_index(self):
-        index = CombinedGraphIndex([])
-        index1 = self.make_index('name', 0, nodes=[(('key', ), '', ())])
-        index.insert_index(0, index1)
-        self.assertEqual([(index1, ('key', ), '')], list(index.iter_all_entries()))
+        idx = index.CombinedGraphIndex([])
+        idx1 = self.make_index('name', 0, nodes=[(('key', ), '', ())])
+        idx.insert_index(0, idx1)
+        self.assertEqual([(idx1, ('key', ), '')],
+                         list(idx.iter_all_entries()))
 
     def test_clear_cache(self):
         log = []
@@ -1128,165 +1134,166 @@
                 log.append(self._index)
                 return self._index.clear_cache()
 
-        index = CombinedGraphIndex([])
-        index1 = self.make_index('name', 0, nodes=[(('key', ), '', ())])
-        index.insert_index(0, ClearCacheProxy(index1))
-        index2 = self.make_index('name', 0, nodes=[(('key', ), '', ())])
-        index.insert_index(1, ClearCacheProxy(index2))
+        idx = index.CombinedGraphIndex([])
+        idx1 = self.make_index('name', 0, nodes=[(('key', ), '', ())])
+        idx.insert_index(0, ClearCacheProxy(idx1))
+        idx2 = self.make_index('name', 0, nodes=[(('key', ), '', ())])
+        idx.insert_index(1, ClearCacheProxy(idx2))
         # CombinedGraphIndex should call 'clear_cache()' on all children
-        index.clear_cache()
-        self.assertEqual(sorted([index1, index2]), sorted(log))
+        idx.clear_cache()
+        self.assertEqual(sorted([idx1, idx2]), sorted(log))
 
     def test_iter_all_entries_empty(self):
-        index = CombinedGraphIndex([])
-        self.assertEqual([], list(index.iter_all_entries()))
+        idx = index.CombinedGraphIndex([])
+        self.assertEqual([], list(idx.iter_all_entries()))
 
     def test_iter_all_entries_children_empty(self):
-        index1 = self.make_index('name')
-        index = CombinedGraphIndex([index1])
-        self.assertEqual([], list(index.iter_all_entries()))
+        idx1 = self.make_index('name')
+        idx = index.CombinedGraphIndex([idx1])
+        self.assertEqual([], list(idx.iter_all_entries()))
 
     def test_iter_all_entries_simple(self):
-        index1 = self.make_index('name', nodes=[(('name', ), 'data', ())])
-        index = CombinedGraphIndex([index1])
-        self.assertEqual([(index1, ('name', ), 'data')],
-            list(index.iter_all_entries()))
+        idx1 = self.make_index('name', nodes=[(('name', ), 'data', ())])
+        idx = index.CombinedGraphIndex([idx1])
+        self.assertEqual([(idx1, ('name', ), 'data')],
+            list(idx.iter_all_entries()))
 
     def test_iter_all_entries_two_indices(self):
-        index1 = self.make_index('name1', nodes=[(('name', ), 'data', ())])
-        index2 = self.make_index('name2', nodes=[(('2', ), '', ())])
-        index = CombinedGraphIndex([index1, index2])
-        self.assertEqual([(index1, ('name', ), 'data'),
-            (index2, ('2', ), '')],
-            list(index.iter_all_entries()))
+        idx1 = self.make_index('name1', nodes=[(('name', ), 'data', ())])
+        idx2 = self.make_index('name2', nodes=[(('2', ), '', ())])
+        idx = index.CombinedGraphIndex([idx1, idx2])
+        self.assertEqual([(idx1, ('name', ), 'data'),
+                          (idx2, ('2', ), '')],
+                         list(idx.iter_all_entries()))
 
     def test_iter_entries_two_indices_dup_key(self):
-        index1 = self.make_index('name1', nodes=[(('name', ), 'data', ())])
-        index2 = self.make_index('name2', nodes=[(('name', ), 'data', ())])
-        index = CombinedGraphIndex([index1, index2])
-        self.assertEqual([(index1, ('name', ), 'data')],
-            list(index.iter_entries([('name', )])))
+        idx1 = self.make_index('name1', nodes=[(('name', ), 'data', ())])
+        idx2 = self.make_index('name2', nodes=[(('name', ), 'data', ())])
+        idx = index.CombinedGraphIndex([idx1, idx2])
+        self.assertEqual([(idx1, ('name', ), 'data')],
+                         list(idx.iter_entries([('name', )])))
 
     def test_iter_all_entries_two_indices_dup_key(self):
-        index1 = self.make_index('name1', nodes=[(('name', ), 'data', ())])
-        index2 = self.make_index('name2', nodes=[(('name', ), 'data', ())])
-        index = CombinedGraphIndex([index1, index2])
-        self.assertEqual([(index1, ('name', ), 'data')],
-            list(index.iter_all_entries()))
+        idx1 = self.make_index('name1', nodes=[(('name', ), 'data', ())])
+        idx2 = self.make_index('name2', nodes=[(('name', ), 'data', ())])
+        idx = index.CombinedGraphIndex([idx1, idx2])
+        self.assertEqual([(idx1, ('name', ), 'data')],
+                         list(idx.iter_all_entries()))
 
     def test_iter_key_prefix_2_key_element_refs(self):
-        index1 = self.make_index('1', 1, key_elements=2, nodes=[
-            (('name', 'fin1'), 'data', ([('ref', 'erence')], ))])
-        index2 = self.make_index('2', 1, key_elements=2, nodes=[
-            (('name', 'fin2'), 'beta', ([], )),
-            (('ref', 'erence'), 'refdata', ([], ))])
-        index = CombinedGraphIndex([index1, index2])
-        self.assertEqual(set([(index1, ('name', 'fin1'), 'data', ((('ref', 'erence'),),)),
-            (index2, ('ref', 'erence'), 'refdata', ((), ))]),
-            set(index.iter_entries_prefix([('name', 'fin1'), ('ref', 'erence')])))
-        self.assertEqual(set([(index1, ('name', 'fin1'), 'data', ((('ref', 'erence'),),)),
-            (index2, ('name', 'fin2'), 'beta', ((), ))]),
-            set(index.iter_entries_prefix([('name', None)])))
+        idx1 = self.make_index('1', 1, key_elements=2, nodes=[
+                (('name', 'fin1'), 'data', ([('ref', 'erence')], ))])
+        idx2 = self.make_index('2', 1, key_elements=2, nodes=[
+                (('name', 'fin2'), 'beta', ([], )),
+                (('ref', 'erence'), 'refdata', ([], ))])
+        idx = index.CombinedGraphIndex([idx1, idx2])
+        self.assertEqual(set([(idx1, ('name', 'fin1'), 'data',
+                               ((('ref', 'erence'),),)),
+                              (idx2, ('ref', 'erence'), 'refdata', ((), ))]),
+                         set(idx.iter_entries_prefix([('name', 'fin1'),
+                                                        ('ref', 'erence')])))
+        self.assertEqual(set([(idx1, ('name', 'fin1'), 'data',
+                               ((('ref', 'erence'),),)),
+                              (idx2, ('name', 'fin2'), 'beta', ((), ))]),
+                         set(idx.iter_entries_prefix([('name', None)])))
 
     def test_iter_nothing_empty(self):
-        index = CombinedGraphIndex([])
-        self.assertEqual([], list(index.iter_entries([])))
+        idx = index.CombinedGraphIndex([])
+        self.assertEqual([], list(idx.iter_entries([])))
 
     def test_iter_nothing_children_empty(self):
-        index1 = self.make_index('name')
-        index = CombinedGraphIndex([index1])
-        self.assertEqual([], list(index.iter_entries([])))
+        idx1 = self.make_index('name')
+        idx = index.CombinedGraphIndex([idx1])
+        self.assertEqual([], list(idx.iter_entries([])))
 
     def test_iter_all_keys(self):
-        index1 = self.make_index('1', 1, nodes=[
-            (('name', ), 'data', ([('ref', )], ))])
-        index2 = self.make_index('2', 1, nodes=[
-            (('ref', ), 'refdata', ((), ))])
-        index = CombinedGraphIndex([index1, index2])
-        self.assertEqual(set([(index1, ('name', ), 'data', ((('ref', ), ), )),
-            (index2, ('ref', ), 'refdata', ((), ))]),
-            set(index.iter_entries([('name', ), ('ref', )])))
+        idx1 = self.make_index('1', 1, nodes=[(('name', ), 'data',
+                                               ([('ref', )], ))])
+        idx2 = self.make_index('2', 1, nodes=[(('ref', ), 'refdata', ((), ))])
+        idx = index.CombinedGraphIndex([idx1, idx2])
+        self.assertEqual(set([(idx1, ('name', ), 'data', ((('ref', ), ), )),
+                              (idx2, ('ref', ), 'refdata', ((), ))]),
+                         set(idx.iter_entries([('name', ), ('ref', )])))
 
     def test_iter_all_keys_dup_entry(self):
-        index1 = self.make_index('1', 1, nodes=[
-            (('name', ), 'data', ([('ref', )], )),
-            (('ref', ), 'refdata', ([], ))])
-        index2 = self.make_index('2', 1, nodes=[
-            (('ref', ), 'refdata', ([], ))])
-        index = CombinedGraphIndex([index1, index2])
-        self.assertEqual(set([(index1, ('name', ), 'data', ((('ref',),),)),
-            (index1, ('ref', ), 'refdata', ((), ))]),
-            set(index.iter_entries([('name', ), ('ref', )])))
+        idx1 = self.make_index('1', 1, nodes=[(('name', ), 'data',
+                                                 ([('ref', )], )),
+                                                (('ref', ), 'refdata', ([], ))])
+        idx2 = self.make_index('2', 1, nodes=[(('ref', ), 'refdata', ([], ))])
+        idx = index.CombinedGraphIndex([idx1, idx2])
+        self.assertEqual(set([(idx1, ('name', ), 'data', ((('ref',),),)),
+                              (idx1, ('ref', ), 'refdata', ((), ))]),
+                         set(idx.iter_entries([('name', ), ('ref', )])))
 
     def test_iter_missing_entry_empty(self):
-        index = CombinedGraphIndex([])
-        self.assertEqual([], list(index.iter_entries([('a', )])))
+        idx = index.CombinedGraphIndex([])
+        self.assertEqual([], list(idx.iter_entries([('a', )])))
 
     def test_iter_missing_entry_one_index(self):
-        index1 = self.make_index('1')
-        index = CombinedGraphIndex([index1])
-        self.assertEqual([], list(index.iter_entries([('a', )])))
+        idx1 = self.make_index('1')
+        idx = index.CombinedGraphIndex([idx1])
+        self.assertEqual([], list(idx.iter_entries([('a', )])))
 
     def test_iter_missing_entry_two_index(self):
-        index1 = self.make_index('1')
-        index2 = self.make_index('2')
-        index = CombinedGraphIndex([index1, index2])
-        self.assertEqual([], list(index.iter_entries([('a', )])))
+        idx1 = self.make_index('1')
+        idx2 = self.make_index('2')
+        idx = index.CombinedGraphIndex([idx1, idx2])
+        self.assertEqual([], list(idx.iter_entries([('a', )])))
 
     def test_iter_entry_present_one_index_only(self):
-        index1 = self.make_index('1', nodes=[(('key', ), '', ())])
-        index2 = self.make_index('2', nodes=[])
-        index = CombinedGraphIndex([index1, index2])
-        self.assertEqual([(index1, ('key', ), '')],
-            list(index.iter_entries([('key', )])))
+        idx1 = self.make_index('1', nodes=[(('key', ), '', ())])
+        idx2 = self.make_index('2', nodes=[])
+        idx = index.CombinedGraphIndex([idx1, idx2])
+        self.assertEqual([(idx1, ('key', ), '')],
+                         list(idx.iter_entries([('key', )])))
         # and in the other direction
-        index = CombinedGraphIndex([index2, index1])
-        self.assertEqual([(index1, ('key', ), '')],
-            list(index.iter_entries([('key', )])))
+        idx = index.CombinedGraphIndex([idx2, idx1])
+        self.assertEqual([(idx1, ('key', ), '')],
+                         list(idx.iter_entries([('key', )])))
 
     def test_key_count_empty(self):
-        index1 = self.make_index('1', nodes=[])
-        index2 = self.make_index('2', nodes=[])
-        index = CombinedGraphIndex([index1, index2])
-        self.assertEqual(0, index.key_count())
+        idx1 = self.make_index('1', nodes=[])
+        idx2 = self.make_index('2', nodes=[])
+        idx = index.CombinedGraphIndex([idx1, idx2])
+        self.assertEqual(0, idx.key_count())
 
     def test_key_count_sums_index_keys(self):
-        index1 = self.make_index('1', nodes=[
+        idx1 = self.make_index('1', nodes=[
             (('1',), '', ()),
             (('2',), '', ())])
-        index2 = self.make_index('2', nodes=[(('1',), '', ())])
-        index = CombinedGraphIndex([index1, index2])
-        self.assertEqual(3, index.key_count())
+        idx2 = self.make_index('2', nodes=[(('1',), '', ())])
+        idx = index.CombinedGraphIndex([idx1, idx2])
+        self.assertEqual(3, idx.key_count())
 
     def test_validate_bad_child_index_errors(self):
         trans = self.get_transport()
         trans.put_bytes('name', "not an index\n")
-        index1 = GraphIndex(trans, 'name', 13)
-        index = CombinedGraphIndex([index1])
-        self.assertRaises(errors.BadIndexFormatSignature, index.validate)
+        idx1 = index.GraphIndex(trans, 'name', 13)
+        idx = index.CombinedGraphIndex([idx1])
+        self.assertRaises(errors.BadIndexFormatSignature, idx.validate)
 
     def test_validate_empty(self):
-        index = CombinedGraphIndex([])
-        index.validate()
+        idx = index.CombinedGraphIndex([])
+        idx.validate()
 
     def test_key_count_reloads(self):
-        index, reload_counter = self.make_combined_index_with_missing()
-        self.assertEqual(2, index.key_count())
+        idx, reload_counter = self.make_combined_index_with_missing()
+        self.assertEqual(2, idx.key_count())
         self.assertEqual([1, 1, 0], reload_counter)
 
     def test_key_count_no_reload(self):
-        index, reload_counter = self.make_combined_index_with_missing()
-        index._reload_func = None
+        idx, reload_counter = self.make_combined_index_with_missing()
+        idx._reload_func = None
         # Without a _reload_func we just raise the exception
-        self.assertRaises(errors.NoSuchFile, index.key_count)
+        self.assertRaises(errors.NoSuchFile, idx.key_count)
 
     def test_key_count_reloads_and_fails(self):
         # We have deleted all underlying indexes, so we will try to reload, but
         # still fail. This is mostly to test we don't get stuck in an infinite
         # loop trying to reload
-        index, reload_counter = self.make_combined_index_with_missing(
-                                    ['1', '2', '3'])
-        self.assertRaises(errors.NoSuchFile, index.key_count)
+        idx, reload_counter = self.make_combined_index_with_missing(
+            ['1', '2', '3'])
+        self.assertRaises(errors.NoSuchFile, idx.key_count)
         self.assertEqual([2, 1, 1], reload_counter)
 
     def test_iter_entries_reloads(self):
@@ -1392,27 +1399,27 @@
         return self.make_index('index-%s' % name, 0, nodes=nodes)
 
     def test_reorder_after_iter_entries(self):
-        # Four indices: [key1] in index1, [key2,key3] in index2, [] in index3,
-        # [key4] in index4.
-        index = CombinedGraphIndex([])
-        index.insert_index(0, self.make_index_with_simple_nodes('1'), '1')
-        index.insert_index(1, self.make_index_with_simple_nodes('2'), '2')
-        index.insert_index(2, self.make_index_with_simple_nodes('3'), '3')
-        index.insert_index(3, self.make_index_with_simple_nodes('4'), '4')
-        index1, index2, index3, index4 = index._indices
-        # Query a key from index4 and index2.
-        self.assertLength(2, list(index.iter_entries(
+        # Four indices: [key1] in idx1, [key2,key3] in idx2, [] in idx3,
+        # [key4] in idx4.
+        idx = index.CombinedGraphIndex([])
+        idx.insert_index(0, self.make_index_with_simple_nodes('1'), '1')
+        idx.insert_index(1, self.make_index_with_simple_nodes('2'), '2')
+        idx.insert_index(2, self.make_index_with_simple_nodes('3'), '3')
+        idx.insert_index(3, self.make_index_with_simple_nodes('4'), '4')
+        idx1, idx2, idx3, idx4 = idx._indices
+        # Query a key from idx4 and idx2.
+        self.assertLength(2, list(idx.iter_entries(
             [('index-4-key-1',), ('index-2-key-1',)])))
-        # Now index2 and index4 should be moved to the front (and index1 should
-        # still be before index3).
-        self.assertEqual([index2, index4, index1, index3], index._indices)
-        self.assertEqual(['2', '4', '1', '3'], index._index_names)
+        # Now idx2 and idx4 should be moved to the front (and idx1 should
+        # still be before idx3).
+        self.assertEqual([idx2, idx4, idx1, idx3], idx._indices)
+        self.assertEqual(['2', '4', '1', '3'], idx._index_names)
 
     def test_reorder_propagates_to_siblings(self):
         # Two CombinedGraphIndex objects, with the same number of indicies with
         # matching names.
-        cgi1 = CombinedGraphIndex([])
-        cgi2 = CombinedGraphIndex([])
+        cgi1 = index.CombinedGraphIndex([])
+        cgi2 = index.CombinedGraphIndex([])
         cgi1.insert_index(0, self.make_index_with_simple_nodes('1-1'), 'one')
         cgi1.insert_index(1, self.make_index_with_simple_nodes('1-2'), 'two')
         cgi2.insert_index(0, self.make_index_with_simple_nodes('2-1'), 'one')
@@ -1425,23 +1432,23 @@
         self.assertEqual(['two', 'one'], cgi2._index_names)
 
     def test_validate_reloads(self):
-        index, reload_counter = self.make_combined_index_with_missing()
-        index.validate()
+        idx, reload_counter = self.make_combined_index_with_missing()
+        idx.validate()
         self.assertEqual([1, 1, 0], reload_counter)
 
     def test_validate_reloads_midway(self):
-        index, reload_counter = self.make_combined_index_with_missing(['2'])
-        index.validate()
+        idx, reload_counter = self.make_combined_index_with_missing(['2'])
+        idx.validate()
 
     def test_validate_no_reload(self):
-        index, reload_counter = self.make_combined_index_with_missing()
-        index._reload_func = None
-        self.assertRaises(errors.NoSuchFile, index.validate)
+        idx, reload_counter = self.make_combined_index_with_missing()
+        idx._reload_func = None
+        self.assertRaises(errors.NoSuchFile, idx.validate)
 
     def test_validate_reloads_and_fails(self):
-        index, reload_counter = self.make_combined_index_with_missing(
-                                    ['1', '2', '3'])
-        self.assertRaises(errors.NoSuchFile, index.validate)
+        idx, reload_counter = self.make_combined_index_with_missing(
+            ['1', '2', '3'])
+        self.assertRaises(errors.NoSuchFile, idx.validate)
 
     def test_find_ancestors_across_indexes(self):
         key1 = ('key-1',)
@@ -1456,7 +1463,7 @@
             (key3, 'value', ([key2],)),
             (key4, 'value', ([key3],)),
             ])
-        c_index = CombinedGraphIndex([index1, index2])
+        c_index = index.CombinedGraphIndex([index1, index2])
         parent_map, missing_keys = c_index.find_ancestry([key1], 0)
         self.assertEqual({key1: ()}, parent_map)
         self.assertEqual(set(), missing_keys)
@@ -1479,7 +1486,7 @@
         index2 = self.make_index('34', ref_lists=1, nodes=[
             (key3, 'value', ([key2],)),
             ])
-        c_index = CombinedGraphIndex([index1, index2])
+        c_index = index.CombinedGraphIndex([index1, index2])
         # Searching for a key which is actually not present at all should
         # eventually converge
         parent_map, missing_keys = c_index.find_ancestry([key4], 0)
@@ -1487,7 +1494,7 @@
         self.assertEqual(set([key4]), missing_keys)
 
     def test_find_ancestors_no_indexes(self):
-        c_index = CombinedGraphIndex([])
+        c_index = index.CombinedGraphIndex([])
         key1 = ('key-1',)
         parent_map, missing_keys = c_index.find_ancestry([key1], 0)
         self.assertEqual({}, parent_map)
@@ -1505,7 +1512,7 @@
         index2 = self.make_index('34', ref_lists=1, nodes=[
             (key4, 'value', ([key2, key3],)),
             ])
-        c_index = CombinedGraphIndex([index1, index2])
+        c_index = index.CombinedGraphIndex([index1, index2])
         # Searching for a key which is actually not present at all should
         # eventually converge
         parent_map, missing_keys = c_index.find_ancestry([key4], 0)
@@ -1514,20 +1521,20 @@
         self.assertEqual(set([key3]), missing_keys)
 
     def test__find_ancestors_empty_index(self):
-        index = self.make_index('test', ref_lists=1, key_elements=1, nodes=[])
+        idx = self.make_index('test', ref_lists=1, key_elements=1, nodes=[])
         parent_map = {}
         missing_keys = set()
-        search_keys = index._find_ancestors([('one',), ('two',)], 0, parent_map,
-                                            missing_keys)
+        search_keys = idx._find_ancestors([('one',), ('two',)], 0, parent_map,
+                                          missing_keys)
         self.assertEqual(set(), search_keys)
         self.assertEqual({}, parent_map)
         self.assertEqual(set([('one',), ('two',)]), missing_keys)
 
 
-class TestInMemoryGraphIndex(TestCaseWithMemoryTransport):
+class TestInMemoryGraphIndex(tests.TestCaseWithMemoryTransport):
 
     def make_index(self, ref_lists=0, key_elements=1, nodes=[]):
-        result = InMemoryGraphIndex(ref_lists, key_elements=key_elements)
+        result = index.InMemoryGraphIndex(ref_lists, key_elements=key_elements)
         result.add_nodes(nodes)
         return result
 
@@ -1654,23 +1661,26 @@
         index.validate()
 
 
-class TestGraphIndexPrefixAdapter(TestCaseWithMemoryTransport):
+class TestGraphIndexPrefixAdapter(tests.TestCaseWithMemoryTransport):
 
-    def make_index(self, ref_lists=1, key_elements=2, nodes=[], add_callback=False):
-        result = InMemoryGraphIndex(ref_lists, key_elements=key_elements)
+    def make_index(self, ref_lists=1, key_elements=2, nodes=[],
+                   add_callback=False):
+        result = index.InMemoryGraphIndex(ref_lists, key_elements=key_elements)
         result.add_nodes(nodes)
         if add_callback:
             add_nodes_callback = result.add_nodes
         else:
             add_nodes_callback = None
-        adapter = GraphIndexPrefixAdapter(result, ('prefix', ), key_elements - 1,
+        adapter = index.GraphIndexPrefixAdapter(
+            result, ('prefix', ), key_elements - 1,
             add_nodes_callback=add_nodes_callback)
         return result, adapter
 
     def test_add_node(self):
         index, adapter = self.make_index(add_callback=True)
         adapter.add_node(('key',), 'value', ((('ref',),),))
-        self.assertEqual(set([(index, ('prefix', 'key'), 'value', ((('prefix', 'ref'),),))]),
+        self.assertEqual(set([(index, ('prefix', 'key'), 'value',
+                               ((('prefix', 'ref'),),))]),
             set(index.iter_all_entries()))
 
     def test_add_nodes(self):
@@ -1686,12 +1696,13 @@
             set(index.iter_all_entries()))
 
     def test_construct(self):
-        index = InMemoryGraphIndex()
-        adapter = GraphIndexPrefixAdapter(index, ('prefix', ), 1)
+        idx = index.InMemoryGraphIndex()
+        adapter = index.GraphIndexPrefixAdapter(idx, ('prefix', ), 1)
 
     def test_construct_with_callback(self):
-        index = InMemoryGraphIndex()
-        adapter = GraphIndexPrefixAdapter(index, ('prefix', ), 1, index.add_nodes)
+        idx = index.InMemoryGraphIndex()
+        adapter = index.GraphIndexPrefixAdapter(idx, ('prefix', ), 1,
+                                                idx.add_nodes)
 
     def test_iter_all_entries_cross_prefix_map_errors(self):
         index, adapter = self.make_index(nodes=[

=== modified file 'bzrlib/tests/test_inv.py'
--- bzrlib/tests/test_inv.py	2009-09-24 19:26:45 +0000
+++ bzrlib/tests/test_inv.py	2010-09-15 21:06:23 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007, 2008, 2009 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,6 @@
 from bzrlib import (
     chk_map,
     groupcompress,
-    bzrdir,
     errors,
     inventory,
     osutils,
@@ -590,6 +589,11 @@
         self.assertFalse(inv.is_root('TREE_ROOT'))
         self.assertFalse(inv.is_root('booga'))
 
+    def test_entries_for_empty_inventory(self):
+        """Test that entries() will not fail for an empty inventory"""
+        inv = Inventory(root_id=None)
+        self.assertEqual([], inv.entries())
+
 
 class TestInventoryEntry(TestCase):
 
@@ -607,12 +611,7 @@
 
     def test_dir_detect_changes(self):
         left = inventory.InventoryDirectory('123', 'hello.c', ROOT_ID)
-        left.text_sha1 = 123
-        left.executable = True
-        left.symlink_target='foo'
         right = inventory.InventoryDirectory('123', 'hello.c', ROOT_ID)
-        right.text_sha1 = 321
-        right.symlink_target='bar'
         self.assertEqual((False, False), left.detect_changes(right))
         self.assertEqual((False, False), right.detect_changes(left))
 
@@ -632,11 +631,8 @@
 
     def test_symlink_detect_changes(self):
         left = inventory.InventoryLink('123', 'hello.c', ROOT_ID)
-        left.text_sha1 = 123
-        left.executable = True
         left.symlink_target='foo'
         right = inventory.InventoryLink('123', 'hello.c', ROOT_ID)
-        right.text_sha1 = 321
         right.symlink_target='foo'
         self.assertEqual((False, False), left.detect_changes(right))
         self.assertEqual((False, False), right.detect_changes(left))

=== modified file 'bzrlib/tests/test_inventory_delta.py'
--- bzrlib/tests/test_inventory_delta.py	2009-08-13 00:20:29 +0000
+++ bzrlib/tests/test_inventory_delta.py	2010-08-23 17:54:07 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009 Canonical Ltd
+# Copyright (C) 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -387,9 +387,10 @@
         root = new_inv.make_entry('directory', '', None, 'my-rich-root-id')
         root.revision = 'changed'
         new_inv.add(root)
-        non_root = new_inv.make_entry('directory', 'foo', root.file_id, 'id')
+        class StrangeInventoryEntry(inventory.InventoryEntry):
+            kind = 'strange'
+        non_root = StrangeInventoryEntry('id', 'foo', root.file_id)
         non_root.revision = 'changed'
-        non_root.kind = 'strangelove'
         new_inv.add(non_root)
         delta = new_inv._make_delta(old_inv)
         serializer = inventory_delta.InventoryDeltaSerializer(
@@ -398,7 +399,7 @@
         # This test aims to prove that it errors more than how it errors.
         err = self.assertRaises(KeyError,
             serializer.delta_to_lines, NULL_REVISION, 'entry-version', delta)
-        self.assertEqual(('strangelove',), err.args)
+        self.assertEqual(('strange',), err.args)
 
     def test_tree_reference_disabled(self):
         old_inv = Inventory(None)

=== modified file 'bzrlib/tests/test_knit.py'
--- bzrlib/tests/test_knit.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/test_knit.py	2010-06-20 11:18:38 +0000
@@ -17,30 +17,26 @@
 """Tests for Knit data structure"""
 
 from cStringIO import StringIO
-import difflib
-import gzip
 import sys
 
 from bzrlib import (
     errors,
-    generate_ids,
     knit,
     multiparent,
     osutils,
     pack,
     tests,
+    transport,
+    tuned_gzip,
     )
 from bzrlib.errors import (
-    RevisionAlreadyPresent,
     KnitHeaderError,
-    RevisionNotPresent,
     NoSuchFile,
     )
 from bzrlib.index import *
 from bzrlib.knit import (
     AnnotatedKnitContent,
     KnitContent,
-    KnitSequenceMatcher,
     KnitVersionedFiles,
     PlainKnitContent,
     _VFContentMapGenerator,
@@ -50,18 +46,14 @@
     _KnitKeyAccess,
     make_file_factory,
     )
+from bzrlib.patiencediff import PatienceSequenceMatcher
 from bzrlib.repofmt import pack_repo
 from bzrlib.tests import (
-    Feature,
-    KnownFailure,
     TestCase,
     TestCaseWithMemoryTransport,
     TestCaseWithTransport,
     TestNotApplicable,
     )
-from bzrlib.transport import get_transport
-from bzrlib.transport.memory import MemoryTransport
-from bzrlib.tuned_gzip import GzipFile
 from bzrlib.versionedfile import (
     AbsentContentFactory,
     ConstantMapper,
@@ -106,8 +98,8 @@
         line_delta = source_content.line_delta(target_content)
         delta_blocks = list(KnitContent.get_line_delta_blocks(line_delta,
             source_lines, target_lines))
-        matcher = KnitSequenceMatcher(None, source_lines, target_lines)
-        matcher_blocks = list(list(matcher.get_matching_blocks()))
+        matcher = PatienceSequenceMatcher(None, source_lines, target_lines)
+        matcher_blocks = list(matcher.get_matching_blocks())
         self.assertEqual(matcher_blocks, delta_blocks)
 
     def test_get_line_delta_blocks(self):
@@ -700,7 +692,7 @@
 
     def create_gz_content(self, text):
         sio = StringIO()
-        gz_file = gzip.GzipFile(mode='wb', fileobj=sio)
+        gz_file = tuned_gzip.GzipFile(mode='wb', fileobj=sio)
         gz_file.write(text)
         gz_file.close()
         return sio.getvalue()
@@ -1579,13 +1571,13 @@
         # could leave an empty .kndx file, which bzr would later claim was a
         # corrupted file since the header was not present. In reality, the file
         # just wasn't created, so it should be ignored.
-        t = get_transport('.')
+        t = transport.get_transport('.')
         t.put_bytes('test.kndx', '')
 
         knit = self.make_test_knit()
 
     def test_knit_index_checks_header(self):
-        t = get_transport('.')
+        t = transport.get_transport('.')
         t.put_bytes('test.kndx', '# not really a knit header\n\n')
         k = self.make_test_knit()
         self.assertRaises(KnitHeaderError, k.keys)

=== modified file 'bzrlib/tests/test_lazy_import.py'
--- bzrlib/tests/test_lazy_import.py	2010-01-25 15:55:48 +0000
+++ bzrlib/tests/test_lazy_import.py	2010-09-25 20:08:01 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -127,13 +127,9 @@
         else:
             self.fail('test_obj1 was not supposed to exist yet')
 
-        orig_globals = set(globals().keys())
-
         InstrumentedReplacer(scope=globals(), name='test_obj1',
                              factory=factory)
 
-        new_globals = set(globals().keys())
-
         # We can't use isinstance() because that uses test_obj1.__class__
         # and that goes through __getattribute__ which would activate
         # the replacement
@@ -168,13 +164,9 @@
         else:
             self.fail('test_obj6 was not supposed to exist yet')
 
-        orig_globals = set(globals().keys())
-
         lazy_import.ScopeReplacer(scope=globals(), name='test_obj6',
                                   factory=factory)
 
-        new_globals = set(globals().keys())
-
         # We can't use isinstance() because that uses test_obj6.__class__
         # and that goes through __getattribute__ which would activate
         # the replacement
@@ -440,6 +432,36 @@
                           ('foo', 4),
                          ], actions)
 
+    def test_replacing_from_own_scope_fails(self):
+        """If a ScopeReplacer tries to replace itself a nice error is given"""
+        actions = []
+        InstrumentedReplacer.use_actions(actions)
+        TestClass.use_actions(actions)
+
+        def factory(replacer, scope, name):
+            actions.append('factory')
+            # return the name in given scope, which is currently the replacer
+            return scope[name]
+
+        try:
+            test_obj7
+        except NameError:
+            # test_obj7 shouldn't exist yet
+            pass
+        else:
+            self.fail('test_obj7 was not supposed to exist yet')
+
+        InstrumentedReplacer(scope=globals(), name='test_obj7',
+                             factory=factory)
+
+        self.assertEqual(InstrumentedReplacer,
+                         object.__getattribute__(test_obj7, '__class__'))
+        e = self.assertRaises(errors.IllegalUseOfScopeReplacer, test_obj7)
+        self.assertIn("replace itself", e.msg)
+        self.assertEqual([('__call__', (), {}),
+                          '_replace',
+                          'factory'], actions)
+
 
 class ImportReplacerHelper(TestCaseInTempDir):
     """Test the ability to have a lazily imported module or object"""

=== modified file 'bzrlib/tests/test_lazy_regex.py'
--- bzrlib/tests/test_lazy_regex.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_lazy_regex.py	2010-07-09 16:16:11 +0000
@@ -18,6 +18,7 @@
 
 import re
 
+from bzrlib import errors
 from bzrlib import (
     lazy_regex,
     tests,
@@ -63,6 +64,14 @@
                           ('_real_re_compile', ('foo',), {}),
                          ], actions)
 
+    def test_bad_pattern(self):
+        """Ensure lazy regex handles bad patterns cleanly."""
+        p = lazy_regex.lazy_compile('RE:[')
+        # As p.match is lazy, we make it into a lambda so its handled
+        # by assertRaises correctly.
+        e = self.assertRaises(errors.InvalidPattern, lambda: p.match('foo'))
+        self.assertEqual(e.msg, '"RE:[" unexpected end of regular expression')
+
 
 class TestLazyCompile(tests.TestCase):
 

=== added file 'bzrlib/tests/test_library_state.py'
--- bzrlib/tests/test_library_state.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_library_state.py	2010-06-26 01:07:16 +0000
@@ -0,0 +1,52 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for BzrLibraryState."""
+
+import bzrlib
+from bzrlib import (
+    library_state,
+    tests,
+    ui as _mod_ui
+    )
+from bzrlib.tests import fixtures
+
+
+# TODO: once sufficiently cleaned up this should be able to be TestCase.
+class TestLibraryState(tests.TestCaseWithTransport):
+
+    def test_ui_is_used(self):
+        ui = _mod_ui.SilentUIFactory()
+        state = library_state.BzrLibraryState(
+            ui=ui, trace=fixtures.RecordingContextManager())
+        orig_ui = _mod_ui.ui_factory
+        state.__enter__()
+        try:
+            self.assertEqual(ui, _mod_ui.ui_factory)
+        finally:
+            state.__exit__(None, None, None)
+            self.assertEqual(orig_ui, _mod_ui.ui_factory)
+
+    def test_trace_context(self):
+        tracer = fixtures.RecordingContextManager()
+        ui = _mod_ui.SilentUIFactory()
+        state = library_state.BzrLibraryState(ui=ui, trace=tracer)
+        state.__enter__()
+        try:
+            self.assertEqual(['__enter__'], tracer._calls)
+        finally:
+            state.__exit__(None, None, None)
+            self.assertEqual(['__enter__', '__exit__'], tracer._calls)

=== modified file 'bzrlib/tests/test_lockable_files.py'
--- bzrlib/tests/test_lockable_files.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test_lockable_files.py	2010-06-20 11:18:38 +0000
@@ -21,6 +21,7 @@
     errors,
     lockdir,
     osutils,
+    transport,
     )
 from bzrlib.errors import BzrBadParameterNotString, NoSuchFile, ReadOnlyError
 from bzrlib.lockable_files import LockableFiles, TransportLock
@@ -37,7 +38,6 @@
                                  ReadOnlyTransaction,
                                  WriteTransaction,
                                  )
-from bzrlib.transport import get_transport
 
 
 # these tests are applied in each parameterized suite for LockableFiles
@@ -279,9 +279,9 @@
 
     def setUp(self):
         TestCaseInTempDir.setUp(self)
-        transport = get_transport('.')
-        transport.mkdir('.bzr')
-        self.sub_transport = transport.clone('.bzr')
+        t = transport.get_transport('.')
+        t.mkdir('.bzr')
+        self.sub_transport = t.clone('.bzr')
         self.lockable = self.get_lockable()
         self.lockable.create_lock()
 
@@ -304,7 +304,7 @@
 
     def setUp(self):
         TestCaseInTempDir.setUp(self)
-        self.transport = get_transport('.')
+        self.transport = transport.get_transport('.')
         self.lockable = self.get_lockable()
         # the lock creation here sets mode - test_permissions on branch
         # tests that implicitly, but it might be a good idea to factor
@@ -347,7 +347,7 @@
         # in test_remote and test_smart as usual.
         b = self.make_branch('foo')
         self.addCleanup(b.bzrdir.transport.disconnect)
-        self.transport = get_transport('.')
+        self.transport = transport.get_transport('.')
         self.lockable = self.get_lockable()
 
     def get_lockable(self):

=== modified file 'bzrlib/tests/test_lockdir.py'
--- bzrlib/tests/test_lockdir.py	2010-03-03 22:59:21 +0000
+++ bzrlib/tests/test_lockdir.py	2010-09-27 07:24:54 +0000
@@ -39,7 +39,7 @@
     LockNotHeld,
     )
 from bzrlib.lockdir import LockDir
-from bzrlib.tests import TestCaseWithTransport
+from bzrlib.tests import (features, TestCaseWithTransport)
 from bzrlib.trace import note
 
 # These tests sometimes use threads to test the behaviour of lock files with
@@ -191,22 +191,21 @@
                     "took %f seconds to detect lock contention" % (after - before))
         finally:
             lf1.unlock()
-        lock_base = lf2.transport.abspath(lf2.path)
         self.assertEqual(1, len(self._logged_reports))
-        lock_url = lf2.transport.abspath(lf2.path)
-        self.assertEqual('%s %s\n'
-                         '%s\n%s\n'
-                         'Will continue to try until %s, unless '
-                         'you press Ctrl-C.\n'
-                         'See "bzr help break-lock" for more.',
-                         self._logged_reports[0][0])
-        args = self._logged_reports[0][1]
-        self.assertEqual('Unable to obtain', args[0])
-        self.assertEqual('lock %s' % (lock_base,), args[1])
-        self.assertStartsWith(args[2], 'held by ')
-        self.assertStartsWith(args[3], 'locked ')
-        self.assertEndsWith(args[3], ' ago')
-        self.assertContainsRe(args[4], r'\d\d:\d\d:\d\d')
+        self.assertEqual(self._logged_reports[0][0],
+            '%s lock %s held by %s\n'
+            'at %s [process #%s], acquired %s.\n'
+            'Will continue to try until %s, unless '
+            'you press Ctrl-C.\n'
+            'See "bzr help break-lock" for more.')
+        start, lock_url, user, hostname, pid, time_ago, deadline_str = \
+            self._logged_reports[0][1]
+        self.assertEqual(start, u'Unable to obtain')
+        self.assertEqual(user, u'jrandom@example.com')
+        # skip hostname
+        self.assertContainsRe(pid, r'\d+')
+        self.assertContainsRe(time_ago, r'.* ago')
+        self.assertContainsRe(deadline_str, r'\d{2}:\d{2}:\d{2}')
 
     def test_31_lock_wait_easy(self):
         """Succeed when waiting on a lock with no contention.
@@ -565,6 +564,62 @@
         finally:
             bzrlib.ui.ui_factory = orig_factory
 
+    def test_break_lock_corrupt_info(self):
+        """break_lock works even if the info file is corrupt (and tells the UI
+        that it is corrupt).
+        """
+        ld = self.get_lock()
+        ld2 = self.get_lock()
+        ld.create()
+        ld.lock_write()
+        ld.transport.put_bytes_non_atomic('test_lock/held/info', '\0')
+        class LoggingUIFactory(bzrlib.ui.SilentUIFactory):
+            def __init__(self):
+                self.prompts = []
+            def get_boolean(self, prompt):
+                self.prompts.append(('boolean', prompt))
+                return True
+        ui = LoggingUIFactory()
+        orig_factory = bzrlib.ui.ui_factory
+        bzrlib.ui.ui_factory = ui
+        try:
+            ld2.break_lock()
+            self.assertLength(1, ui.prompts)
+            self.assertEqual('boolean', ui.prompts[0][0])
+            self.assertStartsWith(ui.prompts[0][1], 'Break (corrupt LockDir')
+            self.assertRaises(LockBroken, ld.unlock)
+        finally:
+            bzrlib.ui.ui_factory = orig_factory
+
+    def test_break_lock_missing_info(self):
+        """break_lock works even if the info file is missing (and tells the UI
+        that it is corrupt).
+        """
+        ld = self.get_lock()
+        ld2 = self.get_lock()
+        ld.create()
+        ld.lock_write()
+        ld.transport.delete('test_lock/held/info')
+        class LoggingUIFactory(bzrlib.ui.SilentUIFactory):
+            def __init__(self):
+                self.prompts = []
+            def get_boolean(self, prompt):
+                self.prompts.append(('boolean', prompt))
+                return True
+        ui = LoggingUIFactory()
+        orig_factory = bzrlib.ui.ui_factory
+        bzrlib.ui.ui_factory = ui
+        try:
+            ld2.break_lock()
+            self.assertRaises(LockBroken, ld.unlock)
+            self.assertLength(0, ui.prompts)
+        finally:
+            bzrlib.ui.ui_factory = orig_factory
+        # Suppress warnings due to ld not being unlocked
+        # XXX: if lock_broken hook was invoked in this case, this hack would
+        # not be necessary.  - Andrew Bennetts, 2010-09-06.
+        del self._lock_actions[:]
+
     def test_create_missing_base_directory(self):
         """If LockDir.path doesn't exist, it can be created
 
@@ -597,11 +652,10 @@
             info_list = ld1._format_lock_info(ld1.peek())
         finally:
             ld1.unlock()
-        self.assertEqual('lock %s' % (ld1.transport.abspath(ld1.path),),
-                         info_list[0])
-        self.assertContainsRe(info_list[1],
-                              r'^held by .* on host .* \[process #\d*\]$')
-        self.assertContainsRe(info_list[2], r'locked \d+ seconds? ago$')
+        self.assertEqual(info_list[0], u'jrandom@example.com')
+        # info_list[1] is hostname. we skip this.
+        self.assertContainsRe(info_list[2], '^\d+$') # pid
+        self.assertContainsRe(info_list[3], r'^\d+ seconds? ago$') # time_ago
 
     def test_lock_without_email(self):
         global_config = config.GlobalConfig()
@@ -613,6 +667,7 @@
         ld1.unlock()
 
     def test_lock_permission(self):
+        self.requireFeature(features.not_running_as_root)
         if not osutils.supports_posix_readonly():
             raise tests.TestSkipped('Cannot induce a permission failure')
         ld1 = self.get_lock()
@@ -669,7 +724,7 @@
     def test_no_lockdir_info(self):
         """We can cope with empty info files."""
         # This seems like a fairly common failure case - see
-        # <https://bugs.edge.launchpad.net/bzr/+bug/185103> and all its dupes.
+        # <https://bugs.launchpad.net/bzr/+bug/185103> and all its dupes.
         # Processes are often interrupted after opening the file
         # before the actual contents are committed.
         t = self.get_transport()
@@ -680,11 +735,54 @@
         info = lf.peek()
         formatted_info = lf._format_lock_info(info)
         self.assertEquals(
-            ['lock %s' % t.abspath('test_lock'),
-             'held by <unknown> on host <unknown> [process #<unknown>]',
-             'locked (unknown)'],
+            ['<unknown>', '<unknown>', '<unknown>', '(unknown)'],
             formatted_info)
 
+    def test_corrupt_lockdir_info(self):
+        """We can cope with corrupt (and thus unparseable) info files."""
+        # This seems like a fairly common failure case too - see
+        # <https://bugs.edge.launchpad.net/bzr/+bug/619872> for instance.
+        # In particular some systems tend to fill recently created files with
+        # nul bytes after recovering from a system crash.
+        t = self.get_transport()
+        t.mkdir('test_lock')
+        t.mkdir('test_lock/held')
+        t.put_bytes('test_lock/held/info', '\0')
+        lf = LockDir(t, 'test_lock')
+        self.assertRaises(errors.LockCorrupt, lf.peek)
+        # Currently attempt_lock gives LockContention, but LockCorrupt would be
+        # a reasonable result too.
+        self.assertRaises(
+            (errors.LockCorrupt, errors.LockContention), lf.attempt_lock)
+        self.assertRaises(errors.LockCorrupt, lf.validate_token, 'fake token')
+
+    def test_missing_lockdir_info(self):
+        """We can cope with absent info files."""
+        t = self.get_transport()
+        t.mkdir('test_lock')
+        t.mkdir('test_lock/held')
+        lf = LockDir(t, 'test_lock')
+        # In this case we expect the 'not held' result from peek, because peek
+        # cannot be expected to notice that there is a 'held' directory with no
+        # 'info' file.
+        self.assertEqual(None, lf.peek())
+        # And lock/unlock may work or give LockContention (but not any other
+        # error).
+        try:
+            lf.attempt_lock()
+        except LockContention:
+            # LockContention is ok, and expected on Windows
+            pass
+        else:
+            # no error is ok, and expected on POSIX (because POSIX allows
+            # os.rename over an empty directory).
+            lf.unlock()
+        # Currently raises TokenMismatch, but LockCorrupt would be reasonable
+        # too.
+        self.assertRaises(
+            (errors.TokenMismatch, errors.LockCorrupt),
+            lf.validate_token, 'fake token')
+
 
 class TestLockDirHooks(TestCaseWithTransport):
 

=== modified file 'bzrlib/tests/test_log.py'
--- bzrlib/tests/test_log.py	2010-04-28 07:10:36 +0000
+++ bzrlib/tests/test_log.py	2010-06-08 09:50:27 +0000
@@ -1543,6 +1543,151 @@
     def test_bugs_handler_present(self):
         self.properties_handler_registry.get('bugs_properties_handler')
 
+
+class TestLogForAuthors(TestCaseForLogFormatter):
+
+    def setUp(self):
+        TestCaseForLogFormatter.setUp(self)
+        self.wt = self.make_standard_commit('nicky',
+            authors=['John Doe <jdoe@example.com>',
+                     'Jane Rey <jrey@example.com>'])
+
+    def assertFormatterResult(self, formatter, who, result):
+        formatter_kwargs = dict()
+        if who is not None:
+            author_list_handler = log.author_list_registry.get(who)
+            formatter_kwargs['author_list_handler'] = author_list_handler
+        TestCaseForLogFormatter.assertFormatterResult(self, result,
+            self.wt.branch, formatter, formatter_kwargs=formatter_kwargs)
+
+    def test_line_default(self):
+        self.assertFormatterResult(log.LineLogFormatter, None, """\
+1: John Doe 2005-11-22 add a
+""")
+
+    def test_line_committer(self):
+        self.assertFormatterResult(log.LineLogFormatter, 'committer', """\
+1: Lorem Ipsum 2005-11-22 add a
+""")
+
+    def test_line_first(self):
+        self.assertFormatterResult(log.LineLogFormatter, 'first', """\
+1: John Doe 2005-11-22 add a
+""")
+
+    def test_line_all(self):
+        self.assertFormatterResult(log.LineLogFormatter, 'all', """\
+1: John Doe, Jane Rey 2005-11-22 add a
+""")
+
+
+    def test_short_default(self):
+        self.assertFormatterResult(log.ShortLogFormatter, None, """\
+    1 John Doe\t2005-11-22
+      add a
+
+""")
+
+    def test_short_committer(self):
+        self.assertFormatterResult(log.ShortLogFormatter, 'committer', """\
+    1 Lorem Ipsum\t2005-11-22
+      add a
+
+""")
+
+    def test_short_first(self):
+        self.assertFormatterResult(log.ShortLogFormatter, 'first', """\
+    1 John Doe\t2005-11-22
+      add a
+
+""")
+
+    def test_short_all(self):
+        self.assertFormatterResult(log.ShortLogFormatter, 'all', """\
+    1 John Doe, Jane Rey\t2005-11-22
+      add a
+
+""")
+
+    def test_long_default(self):
+        self.assertFormatterResult(log.LongLogFormatter, None, """\
+------------------------------------------------------------
+revno: 1
+author: John Doe <jdoe@example.com>, Jane Rey <jrey@example.com>
+committer: Lorem Ipsum <test@example.com>
+branch nick: nicky
+timestamp: Tue 2005-11-22 00:00:00 +0000
+message:
+  add a
+""")
+
+    def test_long_committer(self):
+        self.assertFormatterResult(log.LongLogFormatter, 'committer', """\
+------------------------------------------------------------
+revno: 1
+committer: Lorem Ipsum <test@example.com>
+branch nick: nicky
+timestamp: Tue 2005-11-22 00:00:00 +0000
+message:
+  add a
+""")
+
+    def test_long_first(self):
+        self.assertFormatterResult(log.LongLogFormatter, 'first', """\
+------------------------------------------------------------
+revno: 1
+author: John Doe <jdoe@example.com>
+committer: Lorem Ipsum <test@example.com>
+branch nick: nicky
+timestamp: Tue 2005-11-22 00:00:00 +0000
+message:
+  add a
+""")
+
+    def test_long_all(self):
+        self.assertFormatterResult(log.LongLogFormatter, 'all', """\
+------------------------------------------------------------
+revno: 1
+author: John Doe <jdoe@example.com>, Jane Rey <jrey@example.com>
+committer: Lorem Ipsum <test@example.com>
+branch nick: nicky
+timestamp: Tue 2005-11-22 00:00:00 +0000
+message:
+  add a
+""")
+
+    def test_gnu_changelog_default(self):
+        self.assertFormatterResult(log.GnuChangelogLogFormatter, None, """\
+2005-11-22  John Doe  <jdoe@example.com>
+
+\tadd a
+
+""")
+
+    def test_gnu_changelog_committer(self):
+        self.assertFormatterResult(log.GnuChangelogLogFormatter, 'committer', """\
+2005-11-22  Lorem Ipsum  <test@example.com>
+
+\tadd a
+
+""")
+
+    def test_gnu_changelog_first(self):
+        self.assertFormatterResult(log.GnuChangelogLogFormatter, 'first', """\
+2005-11-22  John Doe  <jdoe@example.com>
+
+\tadd a
+
+""")
+
+    def test_gnu_changelog_all(self):
+        self.assertFormatterResult(log.GnuChangelogLogFormatter, 'all', """\
+2005-11-22  John Doe  <jdoe@example.com>, Jane Rey  <jrey@example.com>
+
+\tadd a
+
+""")
+
 class TestLogExcludeAncestry(tests.TestCaseWithTransport):
 
     def make_branch_with_alternate_ancestries(self, relpath='.'):
@@ -1579,13 +1724,13 @@
         return br
 
     def assertLogRevnos(self, expected_revnos, b, start, end,
-                        exclude_common_ancestry):
+                        exclude_common_ancestry, generate_merge_revisions=True):
         # FIXME: the layering in log makes it hard to test intermediate levels,
         # I wish adding filters with their parameters were easier...
         # -- vila 20100413
         iter_revs = log._calc_view_revisions(
             b, start, end, direction='reverse',
-            generate_merge_revisions=True,
+            generate_merge_revisions=generate_merge_revisions,
             exclude_common_ancestry=exclude_common_ancestry)
         self.assertEqual(expected_revnos,
                          [revid for revid, revno, depth in iter_revs])
@@ -1593,11 +1738,19 @@
     def test_merge_sorted_exclude_ancestry(self):
         b = self.make_branch_with_alternate_ancestries()
         self.assertLogRevnos(['3', '1.1.2', '1.2.1', '1.1.1', '2', '1'],
-                             b, '1', '3', False)
+                             b, '1', '3', exclude_common_ancestry=False)
         # '2' is part of the '3' ancestry but not part of '1.1.1' ancestry so
         # it should be mentioned even if merge_sort order will make it appear
         # after 1.1.1
         self.assertLogRevnos(['3', '1.1.2', '1.2.1', '2'],
-                             b, '1.1.1', '3', True)
+                             b, '1.1.1', '3', exclude_common_ancestry=True)
 
+    def test_merge_sorted_simple_revnos_exclude_ancestry(self):
+        b = self.make_branch_with_alternate_ancestries()
+        self.assertLogRevnos(['3', '2'],
+                             b, '1', '3', exclude_common_ancestry=True,
+                             generate_merge_revisions=False)
+        self.assertLogRevnos(['3', '1.1.2', '1.2.1', '1.1.1', '2'],
+                             b, '1', '3', exclude_common_ancestry=True,
+                             generate_merge_revisions=True)
 

=== modified file 'bzrlib/tests/test_lru_cache.py'
--- bzrlib/tests/test_lru_cache.py	2009-07-08 14:28:04 +0000
+++ bzrlib/tests/test_lru_cache.py	2010-07-15 13:05:40 +0000
@@ -282,13 +282,6 @@
         cache[6] = 7
         self.assertEqual([2, 3, 4, 5, 6], sorted(cache.keys()))
 
-    def test_after_cleanup_size_deprecated(self):
-        obj = self.callDeprecated([
-            'LRUCache.__init__(after_cleanup_size) was deprecated in 1.11.'
-            ' Use after_cleanup_count instead.'],
-            lru_cache.LRUCache, 50, after_cleanup_size=25)
-        self.assertEqual(obj._after_cleanup_count, 25)
-
     def test_resize_smaller(self):
         cache = lru_cache.LRUCache(max_cache=5, after_cleanup_count=4)
         cache[1] = 2

=== modified file 'bzrlib/tests/test_lsprof.py'
--- bzrlib/tests/test_lsprof.py	2009-08-24 21:05:09 +0000
+++ bzrlib/tests/test_lsprof.py	2010-07-05 08:10:03 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2007, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,9 +19,10 @@
 
 import cPickle
 import os
+import threading
 
 import bzrlib
-from bzrlib import tests
+from bzrlib import errors, tests
 
 
 class _LSProfFeature(tests.Feature):
@@ -111,3 +112,40 @@
         lines = [str(data) for data in stats.data]
         lines = [line for line in lines if 'a_function' in line]
         self.assertLength(1, lines)
+
+    def test_block_0(self):
+        # When profiler_block is 0, reentrant profile requests fail.
+        self.overrideAttr(bzrlib.lsprof.BzrProfiler, 'profiler_block', 0)
+        inner_calls = []
+        def inner():
+            profiler = bzrlib.lsprof.BzrProfiler()
+            self.assertRaises(errors.BzrError, profiler.start)
+            inner_calls.append(True)
+        bzrlib.lsprof.profile(inner)
+        self.assertLength(1, inner_calls)
+
+    def test_block_1(self):
+        # When profiler_block is 1, concurrent profiles serialise.
+        # This is tested by manually acquiring the profiler lock, then
+        # starting a thread that tries to profile, and releasing the lock. 
+        # We know due to test_block_0 that two profiles at once hit the lock,
+        # so while this isn't perfect (we'd want a callback on the lock being
+        # entered to allow lockstep evaluation of the actions), its good enough
+        # to be confident regressions would be caught. Alternatively, if this
+        # is flakey, a fake Lock object can be used to trace the calls made.
+        calls = []
+        def profiled():
+            calls.append('profiled')
+        def do_profile():
+            bzrlib.lsprof.profile(profiled)
+            calls.append('after_profiled')
+        thread = threading.Thread(target=do_profile)
+        bzrlib.lsprof.BzrProfiler.profiler_lock.acquire()
+        try:
+            try:
+                thread.start()
+            finally:
+                bzrlib.lsprof.BzrProfiler.profiler_lock.release()
+        finally:
+            thread.join()
+        self.assertLength(2, calls)

=== modified file 'bzrlib/tests/test_merge.py'
--- bzrlib/tests/test_merge.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/test_merge.py	2010-07-29 04:07:27 +0000
@@ -18,13 +18,16 @@
 from StringIO import StringIO
 
 from bzrlib import (
+    branch as _mod_branch,
+    cleanup,
     conflicts,
     errors,
+    inventory,
     knit,
     memorytree,
     merge as _mod_merge,
     option,
-    progress,
+    revision as _mod_revision,
     tests,
     transform,
     versionedfile,
@@ -32,7 +35,7 @@
 from bzrlib.conflicts import ConflictList, TextConflict
 from bzrlib.errors import UnrelatedBranches, NoCommits
 from bzrlib.merge import transform_tree, merge_inner, _PlanMerge
-from bzrlib.osutils import pathjoin, file_kind
+from bzrlib.osutils import basename, pathjoin, file_kind
 from bzrlib.tests import (
     TestCaseWithMemoryTransport,
     TestCaseWithTransport,
@@ -2917,3 +2920,240 @@
         conflicts = builder.merge()
         # The hook should not call the merge_text() method
         self.assertEqual([], self.calls)
+
+
+class TestMergeIntoBase(tests.TestCaseWithTransport):
+
+    def setup_simple_branch(self, relpath, shape=None, root_id=None):
+        """One commit, containing tree specified by optional shape.
+        
+        Default is empty tree (just root entry).
+        """
+        if root_id is None:
+            root_id = '%s-root-id' % (relpath,)
+        wt = self.make_branch_and_tree(relpath)
+        wt.set_root_id(root_id)
+        if shape is not None:
+            adjusted_shape = [relpath + '/' + elem for elem in shape]
+            self.build_tree(adjusted_shape)
+            ids = ['%s-%s-id' % (relpath, basename(elem.rstrip('/')))
+                   for elem in shape]
+            wt.add(shape, ids=ids)
+        rev_id = 'r1-%s' % (relpath,)
+        wt.commit("Initial commit of %s" % (relpath,), rev_id=rev_id)
+        self.assertEqual(root_id, wt.path2id(''))
+        return wt
+
+    def setup_two_branches(self, custom_root_ids=True):
+        """Setup 2 branches, one will be a library, the other a project."""
+        if custom_root_ids:
+            root_id = None
+        else:
+            root_id = inventory.ROOT_ID
+        project_wt = self.setup_simple_branch(
+            'project', ['README', 'dir/', 'dir/file.c'],
+            root_id)
+        lib_wt = self.setup_simple_branch(
+            'lib1', ['README', 'Makefile', 'foo.c'], root_id)
+
+        return project_wt, lib_wt
+
+    def do_merge_into(self, location, merge_as):
+        """Helper for using MergeIntoMerger.
+        
+        :param location: location of directory to merge from, either the
+            location of a branch or of a path inside a branch.
+        :param merge_as: the path in a tree to add the new directory as.
+        :returns: the conflicts from 'do_merge'.
+        """
+        operation = cleanup.OperationWithCleanups(self._merge_into)
+        return operation.run(location, merge_as)
+
+    def _merge_into(self, op, location, merge_as):
+        # Open and lock the various tree and branch objects
+        wt, subdir_relpath = WorkingTree.open_containing(merge_as)
+        op.add_cleanup(wt.lock_write().unlock)
+        branch_to_merge, subdir_to_merge = _mod_branch.Branch.open_containing(
+            location)
+        op.add_cleanup(branch_to_merge.lock_read().unlock)
+        other_tree = branch_to_merge.basis_tree()
+        op.add_cleanup(other_tree.lock_read().unlock)
+        # Perform the merge
+        merger = _mod_merge.MergeIntoMerger(this_tree=wt, other_tree=other_tree,
+            other_branch=branch_to_merge, target_subdir=subdir_relpath,
+            source_subpath=subdir_to_merge)
+        merger.set_base_revision(_mod_revision.NULL_REVISION, branch_to_merge)
+        conflicts = merger.do_merge()
+        merger.set_pending()
+        return conflicts
+
+    def assertTreeEntriesEqual(self, expected_entries, tree):
+        """Assert that 'tree' contains the expected inventory entries.
+
+        :param expected_entries: sequence of (path, file-id) pairs.
+        """
+        files = [(path, ie.file_id) for path, ie in tree.iter_entries_by_dir()]
+        self.assertEqual(expected_entries, files)
+
+
+class TestMergeInto(TestMergeIntoBase):
+
+    def test_newdir_with_unique_roots(self):
+        """Merge a branch with a unique root into a new directory."""
+        project_wt, lib_wt = self.setup_two_branches()
+        self.do_merge_into('lib1', 'project/lib1')
+        project_wt.lock_read()
+        self.addCleanup(project_wt.unlock)
+        # The r1-lib1 revision should be merged into this one
+        self.assertEqual(['r1-project', 'r1-lib1'], project_wt.get_parent_ids())
+        self.assertTreeEntriesEqual(
+            [('', 'project-root-id'),
+             ('README', 'project-README-id'),
+             ('dir', 'project-dir-id'),
+             ('lib1', 'lib1-root-id'),
+             ('dir/file.c', 'project-file.c-id'),
+             ('lib1/Makefile', 'lib1-Makefile-id'),
+             ('lib1/README', 'lib1-README-id'),
+             ('lib1/foo.c', 'lib1-foo.c-id'),
+            ], project_wt)
+
+    def test_subdir(self):
+        """Merge a branch into a subdirectory of an existing directory."""
+        project_wt, lib_wt = self.setup_two_branches()
+        self.do_merge_into('lib1', 'project/dir/lib1')
+        project_wt.lock_read()
+        self.addCleanup(project_wt.unlock)
+        # The r1-lib1 revision should be merged into this one
+        self.assertEqual(['r1-project', 'r1-lib1'], project_wt.get_parent_ids())
+        self.assertTreeEntriesEqual(
+            [('', 'project-root-id'),
+             ('README', 'project-README-id'),
+             ('dir', 'project-dir-id'),
+             ('dir/file.c', 'project-file.c-id'),
+             ('dir/lib1', 'lib1-root-id'),
+             ('dir/lib1/Makefile', 'lib1-Makefile-id'),
+             ('dir/lib1/README', 'lib1-README-id'),
+             ('dir/lib1/foo.c', 'lib1-foo.c-id'),
+            ], project_wt)
+
+    def test_newdir_with_repeat_roots(self):
+        """If the file-id of the dir to be merged already exists a new ID will
+        be allocated to let the merge happen.
+        """
+        project_wt, lib_wt = self.setup_two_branches(custom_root_ids=False)
+        root_id = project_wt.path2id('')
+        self.do_merge_into('lib1', 'project/lib1')
+        project_wt.lock_read()
+        self.addCleanup(project_wt.unlock)
+        # The r1-lib1 revision should be merged into this one
+        self.assertEqual(['r1-project', 'r1-lib1'], project_wt.get_parent_ids())
+        new_lib1_id = project_wt.path2id('lib1')
+        self.assertNotEqual(None, new_lib1_id)
+        self.assertTreeEntriesEqual(
+            [('', root_id),
+             ('README', 'project-README-id'),
+             ('dir', 'project-dir-id'),
+             ('lib1', new_lib1_id),
+             ('dir/file.c', 'project-file.c-id'),
+             ('lib1/Makefile', 'lib1-Makefile-id'),
+             ('lib1/README', 'lib1-README-id'),
+             ('lib1/foo.c', 'lib1-foo.c-id'),
+            ], project_wt)
+
+    def test_name_conflict(self):
+        """When the target directory name already exists a conflict is
+        generated and the original directory is renamed to foo.moved.
+        """
+        dest_wt = self.setup_simple_branch('dest', ['dir/', 'dir/file.txt'])
+        src_wt = self.setup_simple_branch('src', ['README'])
+        conflicts = self.do_merge_into('src', 'dest/dir')
+        self.assertEqual(1, conflicts)
+        dest_wt.lock_read()
+        self.addCleanup(dest_wt.unlock)
+        # The r1-lib1 revision should be merged into this one
+        self.assertEqual(['r1-dest', 'r1-src'], dest_wt.get_parent_ids())
+        self.assertTreeEntriesEqual(
+            [('', 'dest-root-id'),
+             ('dir', 'src-root-id'),
+             ('dir.moved', 'dest-dir-id'),
+             ('dir/README', 'src-README-id'),
+             ('dir.moved/file.txt', 'dest-file.txt-id'),
+            ], dest_wt)
+
+    def test_file_id_conflict(self):
+        """A conflict is generated if the merge-into adds a file (or other
+        inventory entry) with a file-id that already exists in the target tree.
+        """
+        dest_wt = self.setup_simple_branch('dest', ['file.txt'])
+        # Make a second tree with a file-id that will clash with file.txt in
+        # dest.
+        src_wt = self.make_branch_and_tree('src')
+        self.build_tree(['src/README'])
+        src_wt.add(['README'], ids=['dest-file.txt-id'])
+        src_wt.commit("Rev 1 of src.", rev_id='r1-src')
+        conflicts = self.do_merge_into('src', 'dest/dir')
+        # This is an edge case that shouldn't happen to users very often.  So
+        # we don't care really about the exact presentation of the conflict,
+        # just that there is one.
+        self.assertEqual(1, conflicts)
+
+    def test_only_subdir(self):
+        """When the location points to just part of a tree, merge just that
+        subtree.
+        """
+        dest_wt = self.setup_simple_branch('dest')
+        src_wt = self.setup_simple_branch(
+            'src', ['hello.txt', 'dir/', 'dir/foo.c'])
+        conflicts = self.do_merge_into('src/dir', 'dest/dir')
+        dest_wt.lock_read()
+        self.addCleanup(dest_wt.unlock)
+        # The r1-lib1 revision should NOT be merged into this one (this is a
+        # partial merge).
+        self.assertEqual(['r1-dest'], dest_wt.get_parent_ids())
+        self.assertTreeEntriesEqual(
+            [('', 'dest-root-id'),
+             ('dir', 'src-dir-id'),
+             ('dir/foo.c', 'src-foo.c-id'),
+            ], dest_wt)
+
+    def test_only_file(self):
+        """An edge case: merge just one file, not a whole dir."""
+        dest_wt = self.setup_simple_branch('dest')
+        two_file_wt = self.setup_simple_branch(
+            'two-file', ['file1.txt', 'file2.txt'])
+        conflicts = self.do_merge_into('two-file/file1.txt', 'dest/file1.txt')
+        dest_wt.lock_read()
+        self.addCleanup(dest_wt.unlock)
+        # The r1-lib1 revision should NOT be merged into this one
+        self.assertEqual(['r1-dest'], dest_wt.get_parent_ids())
+        self.assertTreeEntriesEqual(
+            [('', 'dest-root-id'), ('file1.txt', 'two-file-file1.txt-id')],
+            dest_wt)
+
+    def test_no_such_source_path(self):
+        """PathNotInTree is raised if the specified path in the source tree
+        does not exist.
+        """
+        dest_wt = self.setup_simple_branch('dest')
+        two_file_wt = self.setup_simple_branch('src', ['dir/'])
+        self.assertRaises(_mod_merge.PathNotInTree, self.do_merge_into,
+            'src/no-such-dir', 'dest/foo')
+        dest_wt.lock_read()
+        self.addCleanup(dest_wt.unlock)
+        # The dest tree is unmodified.
+        self.assertEqual(['r1-dest'], dest_wt.get_parent_ids())
+        self.assertTreeEntriesEqual([('', 'dest-root-id')], dest_wt)
+
+    def test_no_such_target_path(self):
+        """PathNotInTree is also raised if the specified path in the target
+        tree does not exist.
+        """
+        dest_wt = self.setup_simple_branch('dest')
+        two_file_wt = self.setup_simple_branch('src', ['file.txt'])
+        self.assertRaises(_mod_merge.PathNotInTree, self.do_merge_into,
+            'src', 'dest/no-such-dir/foo')
+        dest_wt.lock_read()
+        self.addCleanup(dest_wt.unlock)
+        # The dest tree is unmodified.
+        self.assertEqual(['r1-dest'], dest_wt.get_parent_ids())
+        self.assertTreeEntriesEqual([('', 'dest-root-id')], dest_wt)

=== modified file 'bzrlib/tests/test_msgeditor.py'
--- bzrlib/tests/test_msgeditor.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/test_msgeditor.py	2010-08-29 14:32:45 +0000
@@ -22,6 +22,7 @@
 
 from bzrlib import (
     commit,
+    config,
     errors,
     msgeditor,
     osutils,
@@ -236,46 +237,25 @@
         self.assertRaises((IOError, OSError), msgeditor.edit_commit_message, '')
 
     def test__get_editor(self):
-        # Test that _get_editor can return a decent list of items
-        bzr_editor = os.environ.get('BZR_EDITOR')
-        visual = os.environ.get('VISUAL')
-        editor = os.environ.get('EDITOR')
-        try:
-            os.environ['BZR_EDITOR'] = 'bzr_editor'
-            os.environ['VISUAL'] = 'visual'
-            os.environ['EDITOR'] = 'editor'
-
-            ensure_config_dir_exists()
-            f = open(config_filename(), 'wb')
-            f.write('editor = config_editor\n')
-            f.close()
-
-            editors = list(msgeditor._get_editor())
-            editors = [editor for (editor, cfg_src) in editors]
-
-            self.assertEqual(['bzr_editor', 'config_editor', 'visual',
-                              'editor'], editors[:4])
-
-            if sys.platform == 'win32':
-                self.assertEqual(['wordpad.exe', 'notepad.exe'], editors[4:])
-            else:
-                self.assertEqual(['/usr/bin/editor', 'vi', 'pico', 'nano',
-                                  'joe'], editors[4:])
-
-        finally:
-            # Restore the environment
-            if bzr_editor is None:
-                del os.environ['BZR_EDITOR']
-            else:
-                os.environ['BZR_EDITOR'] = bzr_editor
-            if visual is None:
-                del os.environ['VISUAL']
-            else:
-                os.environ['VISUAL'] = visual
-            if editor is None:
-                del os.environ['EDITOR']
-            else:
-                os.environ['EDITOR'] = editor
+        os.environ['BZR_EDITOR'] = 'bzr_editor'
+        os.environ['VISUAL'] = 'visual'
+        os.environ['EDITOR'] = 'editor'
+
+        conf = config.GlobalConfig.from_string('editor = config_editor\n',
+                                               save=True)
+
+        editors = list(msgeditor._get_editor())
+        editors = [editor for (editor, cfg_src) in editors]
+
+        self.assertEqual(['bzr_editor', 'config_editor', 'visual', 'editor'],
+                         editors[:4])
+
+        if sys.platform == 'win32':
+            self.assertEqual(['wordpad.exe', 'notepad.exe'], editors[4:])
+        else:
+            self.assertEqual(['/usr/bin/editor', 'vi', 'pico', 'nano', 'joe'],
+                             editors[4:])
+
 
     def test__run_editor_EACCES(self):
         """If running a configured editor raises EACESS, the user is warned."""

=== modified file 'bzrlib/tests/test_options.py'
--- bzrlib/tests/test_options.py	2010-04-20 08:47:38 +0000
+++ bzrlib/tests/test_options.py	2010-08-27 17:53:08 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 from bzrlib import (
     bzrdir,
     commands,
+    controldir,
     errors,
     option,
     )
@@ -107,10 +108,10 @@
         self.assertFalse(option.Option('foo', hidden=False).is_hidden('foo'))
 
     def test_registry_conversion(self):
-        registry = bzrdir.BzrDirFormatRegistry()
-        registry.register_metadir('one', 'RepositoryFormat7', 'one help')
-        registry.register_metadir('two', 'RepositoryFormatKnit1', 'two help')
-        registry.register_metadir('hidden', 'RepositoryFormatKnit1',
+        registry = controldir.ControlDirFormatRegistry()
+        bzrdir.register_metadir(registry, 'one', 'RepositoryFormat7', 'one help')
+        bzrdir.register_metadir(registry, 'two', 'RepositoryFormatKnit1', 'two help')
+        bzrdir.register_metadir(registry, 'hidden', 'RepositoryFormatKnit1',
             'two help', hidden=True)
         registry.set_default('one')
         options = [option.RegistryOption('format', '', registry, str)]
@@ -177,13 +178,13 @@
         self.assertEqual('test option', my_option.help)
 
     def test_help(self):
-        registry = bzrdir.BzrDirFormatRegistry()
-        registry.register_metadir('one', 'RepositoryFormat7', 'one help')
-        registry.register_metadir('two',
+        registry = controldir.ControlDirFormatRegistry()
+        bzrdir.register_metadir(registry, 'one', 'RepositoryFormat7', 'one help')
+        bzrdir.register_metadir(registry, 'two',
             'bzrlib.repofmt.knitrepo.RepositoryFormatKnit1',
             'two help',
             )
-        registry.register_metadir('hidden', 'RepositoryFormat7', 'hidden help',
+        bzrdir.register_metadir(registry, 'hidden', 'RepositoryFormat7', 'hidden help',
             hidden=True)
         registry.set_default('one')
         options = [option.RegistryOption('format', 'format help', registry,
@@ -205,9 +206,9 @@
         opt = option.Option('hello', help='fg', type=int, argname='gar')
         self.assertEqual(list(opt.iter_switches()),
                          [('hello', None, 'GAR', 'fg')])
-        registry = bzrdir.BzrDirFormatRegistry()
-        registry.register_metadir('one', 'RepositoryFormat7', 'one help')
-        registry.register_metadir('two',
+        registry = controldir.ControlDirFormatRegistry()
+        bzrdir.register_metadir(registry, 'one', 'RepositoryFormat7', 'one help')
+        bzrdir.register_metadir(registry, 'two',
                 'bzrlib.repofmt.knitrepo.RepositoryFormatKnit1',
                 'two help',
                 )
@@ -386,10 +387,10 @@
                     + '\n'.join(msgs))
 
     def test_is_hidden(self):
-        registry = bzrdir.BzrDirFormatRegistry()
-        registry.register_metadir('hidden', 'HiddenFormat',
+        registry = controldir.ControlDirFormatRegistry()
+        bzrdir.register_metadir(registry, 'hidden', 'HiddenFormat',
             'hidden help text', hidden=True)
-        registry.register_metadir('visible', 'VisibleFormat',
+        bzrdir.register_metadir(registry, 'visible', 'VisibleFormat',
             'visible help text', hidden=False)
         format = option.RegistryOption('format', '', registry, str)
         self.assertTrue(format.is_hidden('hidden'))

=== modified file 'bzrlib/tests/test_osutils.py'
--- bzrlib/tests/test_osutils.py	2010-04-27 07:52:08 +0000
+++ bzrlib/tests/test_osutils.py	2010-09-27 07:24:54 +0000
@@ -21,13 +21,14 @@
 import os
 import re
 import socket
-import stat
 import sys
 import time
 
 from bzrlib import (
     errors,
+    lazy_regex,
     osutils,
+    symbol_versioning,
     tests,
     trace,
     win32utils,
@@ -184,13 +185,6 @@
         shape = sorted(os.listdir('.'))
         self.assertEquals(['A', 'B'], shape)
 
-    def test_rename_error(self):
-        # We wrap os.rename to make it give an error including the filenames
-        # https://bugs.launchpad.net/bzr/+bug/491763
-        err = self.assertRaises(OSError, osutils.rename,
-            'nonexistent', 'target')
-        self.assertContainsString(str(err), 'nonexistent')
-
 
 class TestRandChars(tests.TestCase):
 
@@ -868,7 +862,7 @@
         self.assertEqual('//HOST/path', osutils._win98_abspath('//HOST/path'))
         # relative path
         cwd = osutils.getcwd().rstrip('/')
-        drive = osutils._nt_splitdrive(cwd)[0]
+        drive = osutils.ntpath.splitdrive(cwd)[0]
         self.assertEqual(cwd+'/path', osutils._win98_abspath('path'))
         self.assertEqual(drive+'/path', osutils._win98_abspath('/path'))
         # unicode path
@@ -1071,12 +1065,13 @@
         self.assertExpectedBlocks(expected_dirblocks[1:], result)
 
     def test_walkdirs_os_error(self):
-        # <https://bugs.edge.launchpad.net/bzr/+bug/338653>
+        # <https://bugs.launchpad.net/bzr/+bug/338653>
         # Pyrex readdir didn't raise useful messages if it had an error
         # reading the directory
         if sys.platform == 'win32':
             raise tests.TestNotApplicable(
                 "readdir IOError not tested on win32")
+        self.requireFeature(features.not_running_as_root)
         os.mkdir("test-unreadable")
         os.chmod("test-unreadable", 0000)
         # must chmod it back so that it can be removed
@@ -1090,6 +1085,40 @@
         # Ensure the message contains the file name
         self.assertContainsRe(str(e), "\./test-unreadable")
 
+
+    def test_walkdirs_encoding_error(self):
+        # <https://bugs.launchpad.net/bzr/+bug/488519>
+        # walkdirs didn't raise a useful message when the filenames
+        # are not using the filesystem's encoding
+
+        # require a bytestring based filesystem
+        self.requireFeature(tests.ByteStringNamedFilesystem)
+
+        tree = [
+            '.bzr',
+            '0file',
+            '1dir/',
+            '1dir/0file',
+            '1dir/1dir/',
+            '1file'
+            ]
+
+        self.build_tree(tree)
+
+        # rename the 1file to a latin-1 filename
+        os.rename("./1file", "\xe8file")
+
+        self._save_platform_info()
+        win32utils.winver = None # Avoid the win32 detection code
+        osutils._fs_enc = 'UTF-8'
+
+        # this should raise on error
+        def attempt():
+            for dirdetail, dirblock in osutils.walkdirs('.'):
+                pass
+
+        self.assertRaises(errors.BadFilenameEncoding, attempt)
+
     def test__walkdirs_utf8(self):
         tree = [
             '.bzr',
@@ -1678,19 +1707,27 @@
 
 class TestReCompile(tests.TestCase):
 
+    def _deprecated_re_compile_checked(self, *args, **kwargs):
+        return self.applyDeprecated(symbol_versioning.deprecated_in((2, 2, 0)),
+            osutils.re_compile_checked, *args, **kwargs)
+
     def test_re_compile_checked(self):
-        r = osutils.re_compile_checked(r'A*', re.IGNORECASE)
+        r = self._deprecated_re_compile_checked(r'A*', re.IGNORECASE)
         self.assertTrue(r.match('aaaa'))
         self.assertTrue(r.match('aAaA'))
 
     def test_re_compile_checked_error(self):
         # like https://bugs.launchpad.net/bzr/+bug/251352
+
+        # Due to possible test isolation error, re.compile is not lazy at
+        # this point. We re-install lazy compile.
+        lazy_regex.install_lazy_compile()
         err = self.assertRaises(
             errors.BzrCommandError,
-            osutils.re_compile_checked, '*', re.IGNORECASE, 'test case')
+            self._deprecated_re_compile_checked, '*', re.IGNORECASE, 'test case')
         self.assertEqual(
-            "Invalid regular expression in test case: '*': "
-            "nothing to repeat",
+            'Invalid regular expression in test case: '
+            '"*" nothing to repeat',
             str(err))
 
 
@@ -1917,6 +1954,18 @@
 
 class TestTerminalWidth(tests.TestCase):
 
+    def setUp(self):
+        tests.TestCase.setUp(self)
+        self._orig_terminal_size_state = osutils._terminal_size_state
+        self._orig_first_terminal_size = osutils._first_terminal_size
+        self.addCleanup(self.restore_osutils_globals)
+        osutils._terminal_size_state = 'no_data'
+        osutils._first_terminal_size = None
+
+    def restore_osutils_globals(self):
+        osutils._terminal_size_state = self._orig_terminal_size_state
+        osutils._first_terminal_size = self._orig_first_terminal_size
+
     def replace_stdout(self, new):
         self.overrideAttr(sys, 'stdout', new)
 
@@ -2018,3 +2067,21 @@
         self.assertEquals(self.path, 'test_file')
         self.assertEquals(self.uid, s.st_uid)
         self.assertEquals(self.gid, s.st_gid)
+
+class TestGetuserUnicode(tests.TestCase):
+
+    def test_ascii_user(self):
+        osutils.set_or_unset_env('LOGNAME', 'jrandom')
+        self.assertEqual(u'jrandom', osutils.getuser_unicode())
+
+    def test_unicode_user(self):
+        ue = osutils.get_user_encoding()
+        uni_val, env_val = tests.probe_unicode_in_user_encoding()
+        if uni_val is None:
+            raise tests.TestSkipped(
+                'Cannot find a unicode character that works in encoding %s'
+                % (osutils.get_user_encoding(),))
+        uni_username = u'jrandom' + uni_val
+        encoded_username = uni_username.encode(ue)
+        osutils.set_or_unset_env('LOGNAME', encoded_username)
+        self.assertEqual(uni_username, osutils.getuser_unicode())

=== modified file 'bzrlib/tests/test_osutils_encodings.py'
--- bzrlib/tests/test_osutils_encodings.py	2010-01-25 17:48:22 +0000
+++ bzrlib/tests/test_osutils_encodings.py	2010-06-29 16:21:13 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -114,6 +114,26 @@
         # and in the worst case, use osutils.get_user_encoding()
         self.assertEqual('user_encoding', osutils.get_terminal_encoding())
 
+    def test_get_terminal_encoding_silent(self):
+        self.make_wrapped_streams('stdout_encoding',
+                                  'stderr_encoding',
+                                  'stdin_encoding')
+        # Calling get_terminal_encoding should not mutter when silent=True is
+        # passed.
+        log = self.get_log()
+        osutils.get_terminal_encoding()
+        self.assertEqual(log, self.get_log())
+
+    def test_get_terminal_encoding_trace(self):
+        self.make_wrapped_streams('stdout_encoding',
+                                  'stderr_encoding',
+                                  'stdin_encoding')
+        # Calling get_terminal_encoding should not mutter when silent=True is
+        # passed.
+        log = self.get_log()
+        osutils.get_terminal_encoding(trace=True)
+        self.assertNotEqual(log, self.get_log())
+
     def test_terminal_cp0(self):
         # test cp0 encoding (Windows returns cp0 when there is no encoding)
         self.make_wrapped_streams('cp0',

=== modified file 'bzrlib/tests/test_patches.py'
--- bzrlib/tests/test_patches.py	2010-02-25 06:17:27 +0000
+++ bzrlib/tests/test_patches.py	2010-05-20 18:23:17 +0000
@@ -55,8 +55,8 @@
             datafile.close()
 
     def test_parse_patches_leading_noise(self):
-        # https://bugs.edge.launchpad.net/bzr/+bug/502076
-        # https://code.edge.launchpad.net/~toshio/bzr/allow-dirty-patches/+merge/18854
+        # https://bugs.launchpad.net/bzr/+bug/502076
+        # https://code.launchpad.net/~toshio/bzr/allow-dirty-patches/+merge/18854
         lines = ["diff -pruN commands.py",
             "--- orig/commands.py",
             "+++ mod/dommands.py"]

=== modified file 'bzrlib/tests/test_permissions.py'
--- bzrlib/tests/test_permissions.py	2009-09-07 08:46:00 +0000
+++ bzrlib/tests/test_permissions.py	2010-06-20 11:18:38 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 # -*- coding: utf-8 -*-
 #
 # This program is free software; you can redistribute it and/or modify
@@ -36,11 +36,11 @@
 from cStringIO import StringIO
 import urllib
 
+from bzrlib import transport
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
 from bzrlib.tests import TestCaseWithTransport, TestSkipped
 from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
-from bzrlib.transport import get_transport
 from bzrlib.workingtree import WorkingTree
 
 
@@ -65,7 +65,7 @@
     :param dir_mode: The mode for all directories
     :param include_base: If false, only check the subdirectories
     """
-    t = get_transport(".")
+    t = transport.get_transport(".")
     if include_base:
         test.assertTransportMode(t, base, dir_mode)
     for root, dirs, files in os.walk(base):
@@ -180,7 +180,7 @@
 
         # bodge around for stubsftpserver not letting use connect
         # more than once
-        _t = get_transport(self.get_url())
+        _t = transport.get_transport(self.get_url())
 
         os.mkdir('local')
         t_local = self.make_branch_and_tree('local')
@@ -257,7 +257,7 @@
         original_umask = os.umask(umask)
 
         try:
-            t = get_transport(self.get_url())
+            t = transport.get_transport(self.get_url())
             # Direct access should be masked by umask
             t._sftp_open_exclusive('a', mode=0666).write('foo\n')
             self.assertTransportMode(t, 'a', 0666 &~umask)

=== modified file 'bzrlib/tests/test_plugins.py'
--- bzrlib/tests/test_plugins.py	2010-04-20 22:38:03 +0000
+++ bzrlib/tests/test_plugins.py	2010-06-11 08:02:42 +0000
@@ -27,6 +27,7 @@
 
 import bzrlib
 from bzrlib import (
+    errors,
     osutils,
     plugin,
     plugins,
@@ -79,6 +80,16 @@
         if getattr(bzrlib.plugins, name, None) is not None:
             delattr(bzrlib.plugins, name)
 
+    def _unregister_plugin_submodule(self, plugin_name, submodule_name):
+        """Remove the submodule from sys.modules and the bzrlib namespace."""
+        py_name = 'bzrlib.plugins.%s.%s' % (plugin_name, submodule_name)
+        if py_name in sys.modules:
+            del sys.modules[py_name]
+        plugin = getattr(bzrlib.plugins, plugin_name, None)
+        if plugin is not None:
+            if getattr(plugin, submodule_name, None) is not None:
+                delattr(plugin, submodule_name)
+
     def assertPluginUnknown(self, name):
         self.failIf(getattr(bzrlib.plugins, name, None) is not None)
         self.failIf('bzrlib.plugins.%s' % name in sys.modules)
@@ -798,14 +809,35 @@
         self.assertLength(0, self.warnings)
 
 
+class TestLoadPluginAtSyntax(tests.TestCase):
+
+    def _get_paths(self, paths):
+        return plugin._get_specific_plugin_paths(paths)
+
+    def test_empty(self):
+        self.assertEquals([], self._get_paths(None))
+        self.assertEquals([], self._get_paths(''))
+
+    def test_one_path(self):
+        self.assertEquals([('b', 'man')], self._get_paths('b@man'))
+
+    def test_bogus_path(self):
+        # We need a '@'
+        self.assertRaises(errors.BzrCommandError, self._get_paths, 'batman')
+        # Too much '@' isn't good either
+        self.assertRaises(errors.BzrCommandError, self._get_paths,
+                          'batman@mobile@cave')
+        # An empty description probably indicates a problem
+        self.assertRaises(errors.BzrCommandError, self._get_paths,
+                          os.pathsep.join(['batman@cave', '', 'robin@mobile']))
+
+
 class TestLoadPluginAt(tests.TestCaseInTempDir, TestPluginMixin):
 
     def setUp(self):
         super(TestLoadPluginAt, self).setUp()
         # Make sure we don't pollute the plugins namespace
         self.overrideAttr(plugins, '__path__')
-        # Be paranoid in case a test fail
-        self.addCleanup(self._unregister_plugin, 'test_foo')
         # Reset the flag that protect against double loading
         self.overrideAttr(plugin, '_loaded', False)
         # Create the same plugin in two directories
@@ -813,6 +845,8 @@
         # The "normal" directory, we use 'standard' instead of 'plugins' to
         # avoid depending on the precise naming.
         self.create_plugin_package('test_foo', dir='standard/test_foo')
+        # All the tests will load the 'test_foo' plugin from various locations
+        self.addCleanup(self._unregister_plugin, 'test_foo')
 
     def assertTestFooLoadedFrom(self, path):
         self.assertPluginKnown('test_foo')
@@ -842,8 +876,8 @@
         osutils.set_or_unset_env('BZR_PLUGINS_AT', 'test_foo@non-standard-dir')
         plugin.load_plugins(['standard'])
         self.assertTestFooLoadedFrom('non-standard-dir')
-        self.assertEqual('non-standard-dir/__init__.py',
-                         bzrlib.plugins.test_foo.__file__)
+        self.assertIsSameRealPath('non-standard-dir/__init__.py',
+                                  bzrlib.plugins.test_foo.__file__)
 
         # Try importing again now that the source has been compiled
         self._unregister_plugin('test_foo')
@@ -854,20 +888,38 @@
             suffix = 'pyc'
         else:
             suffix = 'pyo'
-        self.assertEqual('non-standard-dir/__init__.%s' % suffix,
-                         bzrlib.plugins.test_foo.__file__)
+        self.assertIsSameRealPath('non-standard-dir/__init__.%s' % suffix,
+                                  bzrlib.plugins.test_foo.__file__)
 
     def test_submodule_loading(self):
         # We create an additional directory under the one for test_foo
         self.create_plugin_package('test_bar', dir='non-standard-dir/test_bar')
+        self.addCleanup(self._unregister_plugin_submodule,
+                        'test_foo', 'test_bar')
         osutils.set_or_unset_env('BZR_PLUGINS_AT', 'test_foo@non-standard-dir')
         plugin.set_plugins_path(['standard'])
         import bzrlib.plugins.test_foo
         self.assertEqual('bzrlib.plugins.test_foo',
                          bzrlib.plugins.test_foo.__package__)
         import bzrlib.plugins.test_foo.test_bar
-        self.assertEqual('non-standard-dir/test_bar/__init__.py',
-                         bzrlib.plugins.test_foo.test_bar.__file__)
+        self.assertIsSameRealPath('non-standard-dir/test_bar/__init__.py',
+                                  bzrlib.plugins.test_foo.test_bar.__file__)
+
+    def test_relative_submodule_loading(self):
+        self.create_plugin_package('test_foo', dir='another-dir', source='''
+import test_bar
+''')
+        # We create an additional directory under the one for test_foo
+        self.create_plugin_package('test_bar', dir='another-dir/test_bar')
+        self.addCleanup(self._unregister_plugin_submodule,
+                        'test_foo', 'test_bar')
+        osutils.set_or_unset_env('BZR_PLUGINS_AT', 'test_foo@another-dir')
+        plugin.set_plugins_path(['standard'])
+        import bzrlib.plugins.test_foo
+        self.assertEqual('bzrlib.plugins.test_foo',
+                         bzrlib.plugins.test_foo.__package__)
+        self.assertIsSameRealPath('another-dir/test_bar/__init__.py',
+                                  bzrlib.plugins.test_foo.test_bar.__file__)
 
     def test_loading_from___init__only(self):
         # We rename the existing __init__.py file to ensure that we don't load

=== modified file 'bzrlib/tests/test_progress.py'
--- bzrlib/tests/test_progress.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test_progress.py	2010-08-05 08:18:06 +0000
@@ -58,7 +58,7 @@
     def make_view(self):
         out = StringIO()
         view = TextProgressView(out)
-        view._width = 80
+        view._avail_width = lambda: 79
         return out, view
     
     def make_task(self, parent_task, view, msg, curr, total):
@@ -70,6 +70,21 @@
         task.total_cnt = total
         return task
 
+    def test_clear(self):
+        # <https://bugs.launchpad.net/bzr/+bug/611127> clear must actually
+        # send spaces to clear the line
+        out, view = self.make_view()
+        task = self.make_task(None, view, 'reticulating splines', 5, 20)
+        view.show_progress(task)
+        self.assertEqual(
+'\r/ reticulating splines 5/20                                                    \r'
+            , out.getvalue())
+        view.clear()
+        self.assertEqual(
+'\r/ reticulating splines 5/20                                                    \r'
+            + '\r' + 79 * ' ' + '\r',
+            out.getvalue())
+
     def test_render_progress_no_bar(self):
         """The default view now has a spinner but no bar."""
         out, view = self.make_view()
@@ -100,13 +115,13 @@
         # so we're in the first half of the main task, and half way through
         # that
         self.assertEqual(
-r'[####-               ] reticulating splines:stage2 1/2'
+'[####-               ] reticulating splines:stage2 1/2                         '
             , view._render_line())
         # if the nested task is complete, then we're all the way through the
         # first half of the overall work
         task2.update('stage2', 2, 2)
         self.assertEqual(
-r'[#########\          ] reticulating splines:stage2 2/2'
+'[#########\          ] reticulating splines:stage2 2/2                         '
             , view._render_line())
 
     def test_render_progress_sub_nested(self):
@@ -123,5 +138,38 @@
         # progress indication, just a label; and the bottom one is half done,
         # so the overall fraction is 1/4
         self.assertEqual(
-            r'[####|               ] a:b:c 1/2'
+'[####|               ] a:b:c 1/2                                               '
             , view._render_line())
+
+    def test_render_truncated(self):
+        # when the bar is too long for the terminal, we prefer not to truncate
+        # the counters because they might be interesting, and because
+        # truncating the numbers might be misleading
+        out, view = self.make_view()
+        task_a = ProgressTask(None, progress_view=view)
+        task_a.update('start_' + 'a' * 200 + '_end', 2000, 5000)
+        line = view._render_line()
+        self.assertEqual(
+'- start_aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa.. 2000/5000',
+           line) 
+        self.assertEqual(len(line), 79)
+
+
+    def test_render_with_activity(self):
+        # if the progress view has activity, it's shown before the spinner
+        out, view = self.make_view()
+        task_a = ProgressTask(None, progress_view=view)
+        view._last_transport_msg = '   123kB   100kB/s '
+        line = view._render_line()
+        self.assertEqual(
+'   123kB   100kB/s /                                                           ',
+           line) 
+        self.assertEqual(len(line), 79)
+
+        task_a.update('start_' + 'a' * 200 + '_end', 2000, 5000)
+        view._last_transport_msg = '   123kB   100kB/s '
+        line = view._render_line()
+        self.assertEqual(
+'   123kB   100kB/s \\ start_aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa.. 2000/5000',
+           line) 
+        self.assertEqual(len(line), 79)

=== modified file 'bzrlib/tests/test_read_bundle.py'
--- bzrlib/tests/test_read_bundle.py	2009-07-10 07:14:02 +0000
+++ bzrlib/tests/test_read_bundle.py	2010-07-15 12:53:44 +0000
@@ -35,7 +35,8 @@
 def load_tests(standard_tests, module, loader):
     """Multiply tests for tranport implementations."""
     transport_tests, remaining_tests = tests.split_suite_by_condition(
-        standard_tests, tests.condition_isinstance(TestReadBundleFromURL))
+        standard_tests,
+        tests.condition_isinstance(TestReadMergeableBundleFromURL))
     return tests.multiply_tests(transport_tests, transport_test_permutations(),
         remaining_tests)
 
@@ -60,32 +61,11 @@
     return out, wt
 
 
-class TestDeprecations(tests.TestCaseInTempDir):
-
-    def create_test_bundle(self):
-        out, wt = create_bundle_file(self)
-        f = open('test_bundle', 'wb')
-        try:
-            f.write(out.getvalue())
-        finally:
-            f.close()
-        return wt
-
-    def test_read_bundle_from_url_deprecated(self):
-        wt = self.create_test_bundle()
-        t = bzrlib.transport.get_transport(self.test_dir)
-        url = t.abspath('test_bundle')
-        self.callDeprecated([deprecated_in((1, 12, 0))
-                             % 'bzrlib.bundle.read_bundle_from_url'],
-                            bzrlib.bundle.read_bundle_from_url,
-                            url)
-
-
-class TestReadBundleFromURL(TestTransportImplementation):
+class TestReadMergeableBundleFromURL(TestTransportImplementation):
     """Test that read_bundle works properly across multiple transports"""
 
     def setUp(self):
-        super(TestReadBundleFromURL, self).setUp()
+        super(TestReadMergeableBundleFromURL, self).setUp()
         self.bundle_name = 'test_bundle'
         # read_mergeable_from_url will invoke get_transport which may *not*
         # respect self._transport (i.e. returns a transport that is different

=== modified file 'bzrlib/tests/test_reconcile.py'
--- bzrlib/tests/test_reconcile.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_reconcile.py	2010-06-20 11:18:38 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2008, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,15 +17,18 @@
 """Tests for reconiliation behaviour that is repository independent."""
 
 
-from bzrlib import bzrdir, errors, tests
+from bzrlib import (
+    bzrdir,
+    errors,
+    tests,
+    )
 from bzrlib.reconcile import reconcile, Reconciler
 from bzrlib.revision import Revision
-from bzrlib.tests.per_repository import TestCaseWithRepository
-from bzrlib.transport import get_transport
+from bzrlib.tests import per_repository
 from bzrlib.workingtree import WorkingTree
 
 
-class TestWorksWithSharedRepositories(TestCaseWithRepository):
+class TestWorksWithSharedRepositories(per_repository.TestCaseWithRepository):
 
     def test_reweave_empty(self):
         # we want a repo capable format

=== modified file 'bzrlib/tests/test_remote.py'
--- bzrlib/tests/test_remote.py	2010-05-03 04:08:50 +0000
+++ bzrlib/tests/test_remote.py	2010-08-02 18:36:31 +0000
@@ -30,25 +30,28 @@
     branch,
     bzrdir,
     config,
+    controldir,
     errors,
     graph,
     inventory,
     inventory_delta,
-    pack,
     remote,
     repository,
     tests,
+    transport,
     treebuilder,
-    urlutils,
     versionedfile,
     )
 from bzrlib.branch import Branch
-from bzrlib.bzrdir import BzrDir, BzrDirFormat
+from bzrlib.bzrdir import (
+    BzrDir,
+    BzrDirFormat,
+    RemoteBzrProber,
+    )
 from bzrlib.remote import (
     RemoteBranch,
     RemoteBranchFormat,
     RemoteBzrDir,
-    RemoteBzrDirFormat,
     RemoteRepository,
     RemoteRepositoryFormat,
     )
@@ -63,7 +66,6 @@
     multiply_tests,
     test_server,
     )
-from bzrlib.transport import get_transport
 from bzrlib.transport.memory import MemoryTransport
 from bzrlib.transport.remote import (
     RemoteTransport,
@@ -89,10 +91,7 @@
         self.transport = self.get_transport()
         # make a branch that can be opened over the smart transport
         self.local_wt = BzrDir.create_standalone_workingtree('.')
-
-    def tearDown(self):
-        self.transport.disconnect()
-        tests.TestCaseWithTransport.tearDown(self)
+        self.addCleanup(self.transport.disconnect)
 
     def test_create_remote_bzrdir(self):
         b = remote.RemoteBzrDir(self.transport, remote.RemoteBzrDirFormat())
@@ -122,8 +121,8 @@
     def test_find_correct_format(self):
         """Should open a RemoteBzrDir over a RemoteTransport"""
         fmt = BzrDirFormat.find_format(self.transport)
-        self.assertTrue(RemoteBzrDirFormat
-                        in BzrDirFormat._control_server_formats)
+        self.assertTrue(bzrdir.RemoteBzrProber
+                        in controldir.ControlDirFormat._server_probers)
         self.assertIsInstance(fmt, remote.RemoteBzrDirFormat)
 
     def test_open_detected_smart_format(self):
@@ -359,8 +358,8 @@
         a given client_base and transport_base.
         """
         client_medium = medium.SmartClientMedium(client_base)
-        transport = get_transport(transport_base)
-        result = client_medium.remote_path_from_transport(transport)
+        t = transport.get_transport(transport_base)
+        result = client_medium.remote_path_from_transport(t)
         self.assertEqual(expected, result)
 
     def test_remote_path_from_transport(self):
@@ -377,7 +376,7 @@
         a given transport_base and relpath of that transport.  (Note that
         HttpTransportBase is a subclass of SmartClientMedium)
         """
-        base_transport = get_transport(transport_base)
+        base_transport = transport.get_transport(transport_base)
         client_medium = base_transport.get_smart_medium()
         cloned_transport = base_transport.clone(relpath)
         result = client_medium.remote_path_from_transport(cloned_transport)
@@ -609,7 +608,7 @@
         # _get_tree_branch is a form of open_branch, but it should only ask for
         # branch opening, not any other network requests.
         calls = []
-        def open_branch():
+        def open_branch(name=None):
             calls.append("Called")
             return "a-branch"
         transport = MemoryTransport()
@@ -686,7 +685,7 @@
         old.
         """
         self.assertRaises(errors.NotBranchError,
-            RemoteBzrDirFormat.probe_transport, OldServerTransport())
+            RemoteBzrProber.probe_transport, OldServerTransport())
 
 
 class TestBzrDirCreateBranch(TestRemote):
@@ -1618,7 +1617,7 @@
     def test_get_multi_line_branch_conf(self):
         # Make sure that multiple-line branch.conf files are supported
         #
-        # https://bugs.edge.launchpad.net/bzr/+bug/354075
+        # https://bugs.launchpad.net/bzr/+bug/354075
         client = FakeClient()
         client.add_expected_call(
             'Branch.get_stacked_on_url', ('memory:///',),
@@ -1652,6 +1651,32 @@
         branch.unlock()
         self.assertFinished(client)
 
+    def test_set_option_with_dict(self):
+        client = FakeClient()
+        client.add_expected_call(
+            'Branch.get_stacked_on_url', ('memory:///',),
+            'error', ('NotStacked',),)
+        client.add_expected_call(
+            'Branch.lock_write', ('memory:///', '', ''),
+            'success', ('ok', 'branch token', 'repo token'))
+        encoded_dict_value = 'd5:ascii1:a11:unicode \xe2\x8c\x9a3:\xe2\x80\xbde'
+        client.add_expected_call(
+            'Branch.set_config_option_dict', ('memory:///', 'branch token',
+            'repo token', encoded_dict_value, 'foo', ''),
+            'success', ())
+        client.add_expected_call(
+            'Branch.unlock', ('memory:///', 'branch token', 'repo token'),
+            'success', ('ok',))
+        transport = MemoryTransport()
+        branch = self.make_remote_branch(transport, client)
+        branch.lock_write()
+        config = branch._get_config()
+        config.set_option(
+            {'ascii': 'a', u'unicode \N{WATCH}': u'\N{INTERROBANG}'},
+            'foo')
+        branch.unlock()
+        self.assertFinished(client)
+
     def test_backwards_compat_set_option(self):
         self.setup_smart_server_with_call_log()
         branch = self.make_branch('.')
@@ -1664,6 +1689,20 @@
         self.assertLength(10, self.hpss_calls)
         self.assertEqual('value', branch._get_config().get_option('name'))
 
+    def test_backwards_compat_set_option_with_dict(self):
+        self.setup_smart_server_with_call_log()
+        branch = self.make_branch('.')
+        verb = 'Branch.set_config_option_dict'
+        self.disable_verb(verb)
+        branch.lock_write()
+        self.addCleanup(branch.unlock)
+        self.reset_smart_call_log()
+        config = branch._get_config()
+        value_dict = {'ascii': 'a', u'unicode \N{WATCH}': u'\N{INTERROBANG}'}
+        config.set_option(value_dict, 'name')
+        self.assertLength(10, self.hpss_calls)
+        self.assertEqual(value_dict, branch._get_config().get_option('name'))
+
 
 class TestBranchLockWrite(RemoteBranchTestCase):
 
@@ -2257,6 +2296,7 @@
         self.setup_smart_server_with_call_log()
         tree = self.make_branch_and_memory_tree('.')
         tree.lock_write()
+        tree.add('')
         rev1 = tree.commit('First')
         rev2 = tree.commit('Second')
         tree.unlock()
@@ -2301,11 +2341,11 @@
         transport_path = 'quack'
         repo, client = self.setup_fake_client_and_repository(transport_path)
         client.add_success_response('ok', 'a token')
-        result = repo.lock_write()
+        token = repo.lock_write().repository_token
         self.assertEqual(
             [('call', 'Repository.lock_write', ('quack/', ''))],
             client._calls)
-        self.assertEqual('a token', result)
+        self.assertEqual('a token', token)
 
     def test_lock_write_already_locked(self):
         transport_path = 'quack'

=== modified file 'bzrlib/tests/test_repository.py'
--- bzrlib/tests/test_repository.py	2010-03-31 16:29:25 +0000
+++ bzrlib/tests/test_repository.py	2010-08-05 19:25:52 +0000
@@ -31,6 +31,7 @@
                            UnsupportedFormatError,
                            )
 from bzrlib import (
+    btree_index,
     graph,
     tests,
     )
@@ -681,6 +682,21 @@
 
 class Test2a(tests.TestCaseWithMemoryTransport):
 
+    def test_chk_bytes_uses_custom_btree_parser(self):
+        mt = self.make_branch_and_memory_tree('test', format='2a')
+        mt.lock_write()
+        self.addCleanup(mt.unlock)
+        mt.add([''], ['root-id'])
+        mt.commit('first')
+        index = mt.branch.repository.chk_bytes._index._graph_index._indices[0]
+        self.assertEqual(btree_index._gcchk_factory, index._leaf_factory)
+        # It should also work if we re-open the repo
+        repo = mt.branch.repository.bzrdir.open_repository()
+        repo.lock_read()
+        self.addCleanup(repo.unlock)
+        index = repo.chk_bytes._index._graph_index._indices[0]
+        self.assertEqual(btree_index._gcchk_factory, index._leaf_factory)
+
     def test_fetch_combines_groups(self):
         builder = self.make_branch_builder('source', format='2a')
         builder.start_series()

=== modified file 'bzrlib/tests/test_revision.py'
--- bzrlib/tests/test_revision.py	2009-06-03 10:24:28 +0000
+++ bzrlib/tests/test_revision.py	2010-07-15 12:37:32 +0000
@@ -211,31 +211,6 @@
         r.message = None
         self.assertEqual('', r.get_summary())
 
-    def test_get_apparent_author(self):
-        r = revision.Revision('1')
-        r.committer = 'A'
-        author = self.applyDeprecated(
-                symbol_versioning.deprecated_in((1, 13, 0)),
-                r.get_apparent_author)
-        self.assertEqual('A', author)
-        r.properties['author'] = 'B'
-        author = self.applyDeprecated(
-                symbol_versioning.deprecated_in((1, 13, 0)),
-                r.get_apparent_author)
-        self.assertEqual('B', author)
-        r.properties['authors'] = 'C\nD'
-        author = self.applyDeprecated(
-                symbol_versioning.deprecated_in((1, 13, 0)),
-                r.get_apparent_author)
-        self.assertEqual('C', author)
-
-    def test_get_apparent_author_none(self):
-        r = revision.Revision('1')
-        author = self.applyDeprecated(
-                symbol_versioning.deprecated_in((1, 13, 0)),
-                r.get_apparent_author)
-        self.assertEqual(None, author)
-
     def test_get_apparent_authors(self):
         r = revision.Revision('1')
         r.committer = 'A'

=== modified file 'bzrlib/tests/test_revisionspec.py'
--- bzrlib/tests/test_revisionspec.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test_revisionspec.py	2010-09-24 02:19:53 +0000
@@ -652,3 +652,79 @@
     def test_as_revision_id(self):
         self.tree.branch.set_submit_branch('tree2')
         self.assertAsRevisionId('alt_r2', 'branch:tree2')
+
+
+class TestRevisionSpec_mainline(TestRevisionSpec):
+
+    def test_as_revision_id(self):
+        self.assertAsRevisionId('r1', 'mainline:1')
+        self.assertAsRevisionId('r2', 'mainline:1.1.1')
+        self.assertAsRevisionId('r2', 'mainline:revid:alt_r2')
+        spec = RevisionSpec.from_string('mainline:revid:alt_r22')
+        e = self.assertRaises(errors.InvalidRevisionSpec,
+                              spec.as_revision_id, self.tree.branch)
+        self.assertContainsRe(str(e),
+            "Requested revision: 'mainline:revid:alt_r22' does not exist in"
+            " branch: ")
+
+    def test_in_history(self):
+        self.assertInHistoryIs(2, 'r2', 'mainline:revid:alt_r2')
+
+
+class TestRevisionSpec_annotate(TestRevisionSpec):
+
+    def setUp(self):
+        TestRevisionSpec.setUp(self)
+        self.tree = self.make_branch_and_tree('annotate-tree')
+        self.build_tree_contents([('annotate-tree/file1', '1\n')])
+        self.tree.add('file1')
+        self.tree.commit('r1', rev_id='r1')
+        self.build_tree_contents([('annotate-tree/file1', '2\n1\n')])
+        self.tree.commit('r2', rev_id='r2')
+        self.build_tree_contents([('annotate-tree/file1', '2\n1\n3\n')])
+
+    def test_as_revision_id_r1(self):
+        self.assertAsRevisionId('r1', 'annotate:annotate-tree/file1:2')
+
+    def test_as_revision_id_r2(self):
+        self.assertAsRevisionId('r2', 'annotate:annotate-tree/file1:1')
+
+    def test_as_revision_id_uncommitted(self):
+        spec = RevisionSpec.from_string('annotate:annotate-tree/file1:3')
+        e = self.assertRaises(errors.InvalidRevisionSpec,
+                              spec.as_revision_id, self.tree.branch)
+        self.assertContainsRe(str(e),
+            r"Requested revision: \'annotate:annotate-tree/file1:3\' does not"
+            " exist in branch: .*\nLine 3 has not been committed.")
+
+    def test_non_existent_line(self):
+        spec = RevisionSpec.from_string('annotate:annotate-tree/file1:4')
+        e = self.assertRaises(errors.InvalidRevisionSpec,
+                              spec.as_revision_id, self.tree.branch)
+        self.assertContainsRe(str(e),
+            r"Requested revision: \'annotate:annotate-tree/file1:4\' does not"
+            " exist in branch: .*\nNo such line: 4")
+
+    def test_invalid_line(self):
+        spec = RevisionSpec.from_string('annotate:annotate-tree/file1:q')
+        e = self.assertRaises(errors.InvalidRevisionSpec,
+                              spec.as_revision_id, self.tree.branch)
+        self.assertContainsRe(str(e),
+            r"Requested revision: \'annotate:annotate-tree/file1:q\' does not"
+            " exist in branch: .*\nNo such line: q")
+
+    def test_no_such_file(self):
+        spec = RevisionSpec.from_string('annotate:annotate-tree/file2:1')
+        e = self.assertRaises(errors.InvalidRevisionSpec,
+                              spec.as_revision_id, self.tree.branch)
+        self.assertContainsRe(str(e),
+            r"Requested revision: \'annotate:annotate-tree/file2:1\' does not"
+            " exist in branch: .*\nFile 'file2' is not versioned")
+
+    def test_no_such_file_with_colon(self):
+        spec = RevisionSpec.from_string('annotate:annotate-tree/fi:le2:1')
+        e = self.assertRaises(errors.InvalidRevisionSpec,
+                              spec.as_revision_id, self.tree.branch)
+        self.assertContainsRe(str(e),
+            r"Requested revision: \'annotate:annotate-tree/fi:le2:1\' does not"
+            " exist in branch: .*\nFile 'fi:le2' is not versioned")

=== modified file 'bzrlib/tests/test_rio.py'
--- bzrlib/tests/test_rio.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_rio.py	2010-09-25 20:08:01 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -188,6 +188,14 @@
         self.assertEqual(s, None)
         self.assertTrue(s is None)
 
+    def test_read_nul_byte(self):
+        """File consisting of a nul byte causes an error."""
+        self.assertRaises(ValueError, read_stanza, ['\0'])
+
+    def test_read_nul_bytes(self):
+        """File consisting of many nul bytes causes an error."""
+        self.assertRaises(ValueError, read_stanza, ['\0' * 100])
+
     def test_read_iter(self):
         """Read several stanzas from file"""
         tmpf = TemporaryFile()

=== modified file 'bzrlib/tests/test_script.py'
--- bzrlib/tests/test_script.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/test_script.py	2010-09-13 09:00:03 +0000
@@ -16,8 +16,10 @@
 
 
 from bzrlib import (
+    commands,
     osutils,
     tests,
+    ui,
     )
 from bzrlib.tests import script
 
@@ -27,8 +29,33 @@
     def test_comment_is_ignored(self):
         self.assertEquals([], script._script_to_commands('#comment\n'))
 
-    def test_empty_line_is_ignored(self):
-        self.assertEquals([], script._script_to_commands('\n'))
+    def test_comment_multiple_lines(self):
+        self.assertEquals([
+            (['bar'], None, None, None),
+            ],
+            script._script_to_commands("""
+            # this comment is ignored
+            # so is this
+            # no we run bar
+            $ bar
+            """))
+
+    def test_trim_blank_lines(self):
+        """Blank lines are respected, but trimmed at the start and end.
+
+        Python triple-quoted syntax is going to give stubby/empty blank lines 
+        right at the start and the end.  These are cut off so that callers don't 
+        need special syntax to avoid them.
+
+        However we do want to be able to match commands that emit blank lines.
+        """
+        self.assertEquals([
+            (['bar'], None, '\n', None),
+            ],
+            script._script_to_commands("""
+            $bar
+
+            """))
 
     def test_simple_command(self):
         self.assertEquals([(['cd', 'trunk'], None, None, None)],
@@ -51,6 +78,18 @@
             [(['cat', '>file'], 'content\n', None, None)],
             script._script_to_commands('$ cat >file\n<content\n'))
 
+    def test_indented(self):
+        # scripts are commonly given indented within the test source code, and
+        # common indentation is stripped off
+        story = """
+            $ bzr add
+            adding file
+            adding file2
+            """
+        self.assertEquals([(['bzr', 'add'], None,
+                            'adding file\nadding file2\n', None)],
+                          script._script_to_commands(story))
+
     def test_command_with_output(self):
         story = """
 $ bzr add
@@ -365,6 +404,14 @@
         self.assertEquals(None, err)
         self.assertFileEqual('hello\nhappy\n', 'file')
 
+    def test_empty_line_in_output_is_respected(self):
+        self.run_script("""
+            $ echo
+
+            $ echo bar
+            bar
+            """)
+
 
 class TestRm(script.TestCaseWithTransportAndScript):
 
@@ -447,3 +494,39 @@
         self.failIfExists('file')
         self.failUnlessExists('dir/file')
 
+
+class cmd_test_confirm(commands.Command):
+
+    def run(self):
+        if ui.ui_factory.get_boolean(
+            'Really do it',
+            # 'bzrlib.tests.test_script.confirm',
+            # {}
+            ):
+            self.outf.write('Do it!\n')
+        else:
+            print 'ok, no'
+
+
+class TestUserInteraction(script.TestCaseWithMemoryTransportAndScript):
+
+    def test_confirm_action(self):
+        """You can write tests that demonstrate user confirmation.
+        
+        Specifically, ScriptRunner does't care if the output line for the prompt
+        isn't terminated by a newline from the program; it's implicitly terminated 
+        by the input.
+        """
+        commands.builtin_command_registry.register(cmd_test_confirm)
+        self.addCleanup(commands.builtin_command_registry.remove, 'test-confirm')
+        self.run_script("""
+            $ bzr test-confirm
+            2>Really do it? [y/n]: 
+            <yes
+            Do it!
+            $ bzr test-confirm
+            2>Really do it? [y/n]: 
+            <no
+            ok, no
+            """)
+

=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2010-04-22 18:36:13 +0000
+++ bzrlib/tests/test_selftest.py	2010-10-07 12:10:24 +0000
@@ -21,11 +21,13 @@
 import os
 import signal
 import sys
+import threading
 import time
 import unittest
 import warnings
 
 from testtools import MultiTestResult
+from testtools.content import Content
 from testtools.content_type import ContentType
 from testtools.matchers import (
     DocTestMatches,
@@ -42,7 +44,6 @@
     lockdir,
     memorytree,
     osutils,
-    progress,
     remote,
     repository,
     symbol_versioning,
@@ -67,7 +68,7 @@
     test_sftp_transport,
     TestUtil,
     )
-from bzrlib.trace import note
+from bzrlib.trace import note, mutter
 from bzrlib.transport import memory
 from bzrlib.version import _get_bzr_source_tree
 
@@ -122,6 +123,19 @@
         self.failUnlessExists(filename)
 
 
+class TestClassesAvailable(tests.TestCase):
+    """As a convenience we expose Test* classes from bzrlib.tests"""
+
+    def test_test_case(self):
+        from bzrlib.tests import TestCase
+
+    def test_test_loader(self):
+        from bzrlib.tests import TestLoader
+
+    def test_test_suite(self):
+        from bzrlib.tests import TestSuite
+
+
 class TestTransportScenarios(tests.TestCase):
     """A group of tests that test the transport implementation adaption core.
 
@@ -208,7 +222,7 @@
     def test_scenarios(self):
         # check that constructor parameters are passed through to the adapted
         # test.
-        from bzrlib.tests.per_bzrdir import make_scenarios
+        from bzrlib.tests.per_controldir import make_scenarios
         vfs_factory = "v"
         server1 = "a"
         server2 = "b"
@@ -312,19 +326,21 @@
         from bzrlib.tests.per_interrepository import make_scenarios
         server1 = "a"
         server2 = "b"
-        formats = [("C0", "C1", "C2"), ("D0", "D1", "D2")]
+        formats = [("C0", "C1", "C2", "C3"), ("D0", "D1", "D2", "D3")]
         scenarios = make_scenarios(server1, server2, formats)
         self.assertEqual([
             ('C0,str,str',
              {'repository_format': 'C1',
               'repository_format_to': 'C2',
               'transport_readonly_server': 'b',
-              'transport_server': 'a'}),
+              'transport_server': 'a',
+              'extra_setup': 'C3'}),
             ('D0,str,str',
              {'repository_format': 'D1',
               'repository_format_to': 'D2',
               'transport_readonly_server': 'b',
-              'transport_server': 'a'})],
+              'transport_server': 'a',
+              'extra_setup': 'D3'})],
             scenarios)
 
 
@@ -609,19 +625,19 @@
                 l.attempt_lock()
         test = TestDanglingLock('test_function')
         result = test.run()
+        total_failures = result.errors + result.failures
         if self._lock_check_thorough:
-            self.assertEqual(1, len(result.errors))
+            self.assertEqual(1, len(total_failures))
         else:
             # When _lock_check_thorough is disabled, then we don't trigger a
             # failure
-            self.assertEqual(0, len(result.errors))
+            self.assertEqual(0, len(total_failures))
 
 
 class TestTestCaseWithTransport(tests.TestCaseWithTransport):
     """Tests for the convenience functions TestCaseWithTransport introduces."""
 
     def test_get_readonly_url_none(self):
-        from bzrlib.transport import get_transport
         from bzrlib.transport.readonly import ReadonlyTransportDecorator
         self.vfs_transport_factory = memory.MemoryServer
         self.transport_readonly_server = None
@@ -629,15 +645,14 @@
         # for the server
         url = self.get_readonly_url()
         url2 = self.get_readonly_url('foo/bar')
-        t = get_transport(url)
-        t2 = get_transport(url2)
+        t = transport.get_transport(url)
+        t2 = transport.get_transport(url2)
         self.failUnless(isinstance(t, ReadonlyTransportDecorator))
         self.failUnless(isinstance(t2, ReadonlyTransportDecorator))
         self.assertEqual(t2.base[:-1], t.abspath('foo/bar'))
 
     def test_get_readonly_url_http(self):
         from bzrlib.tests.http_server import HttpServer
-        from bzrlib.transport import get_transport
         from bzrlib.transport.http import HttpTransportBase
         self.transport_server = test_server.LocalURLServer
         self.transport_readonly_server = HttpServer
@@ -645,8 +660,8 @@
         url = self.get_readonly_url()
         url2 = self.get_readonly_url('foo/bar')
         # the transport returned may be any HttpTransportBase subclass
-        t = get_transport(url)
-        t2 = get_transport(url2)
+        t = transport.get_transport(url)
+        t2 = transport.get_transport(url2)
         self.failUnless(isinstance(t, HttpTransportBase))
         self.failUnless(isinstance(t2, HttpTransportBase))
         self.assertEqual(t2.base[:-1], t.abspath('foo/bar'))
@@ -690,8 +705,7 @@
 class TestChrootedTest(tests.ChrootedTestCase):
 
     def test_root_is_root(self):
-        from bzrlib.transport import get_transport
-        t = get_transport(self.get_readonly_url())
+        t = transport.get_transport(self.get_readonly_url())
         url = t.base
         self.assertEqual(url, t.clone('..').base)
 
@@ -803,7 +817,7 @@
         self.requireFeature(test_lsprof.LSProfFeature)
         result_stream = StringIO()
         result = bzrlib.tests.VerboseTestResult(
-            unittest._WritelnDecorator(result_stream),
+            result_stream,
             descriptions=0,
             verbosity=2,
             )
@@ -835,12 +849,26 @@
         self.assertContainsRe(output,
             r"LSProf output for <type 'unicode'>\(\('world',\), {'errors': 'replace'}\)\n")
 
+    def test_uses_time_from_testtools(self):
+        """Test case timings in verbose results should use testtools times"""
+        import datetime
+        class TimeAddedVerboseTestResult(tests.VerboseTestResult):
+            def startTest(self, test):
+                self.time(datetime.datetime.utcfromtimestamp(1.145))
+                super(TimeAddedVerboseTestResult, self).startTest(test)
+            def addSuccess(self, test):
+                self.time(datetime.datetime.utcfromtimestamp(51.147))
+                super(TimeAddedVerboseTestResult, self).addSuccess(test)
+            def report_tests_starting(self): pass
+        sio = StringIO()
+        self.get_passing_test().run(TimeAddedVerboseTestResult(sio, 0, 2))
+        self.assertEndsWith(sio.getvalue(), "OK    50002ms\n")
+
     def test_known_failure(self):
         """A KnownFailure being raised should trigger several result actions."""
         class InstrumentedTestResult(tests.ExtendedTestResult):
             def stopTestRun(self): pass
-            def startTests(self): pass
-            def report_test_start(self, test): pass
+            def report_tests_starting(self): pass
             def report_known_failure(self, test, err=None, details=None):
                 self._call = test, 'known failure'
         result = InstrumentedTestResult(None, None, None, None)
@@ -864,7 +892,7 @@
         # verbose test output formatting
         result_stream = StringIO()
         result = bzrlib.tests.VerboseTestResult(
-            unittest._WritelnDecorator(result_stream),
+            result_stream,
             descriptions=0,
             verbosity=2,
             )
@@ -880,6 +908,9 @@
         output = result_stream.getvalue()[prefix:]
         lines = output.splitlines()
         self.assertContainsRe(lines[0], r'XFAIL *\d+ms$')
+        if sys.version_info > (2, 7):
+            self.expectFailure("_ExpectedFailure on 2.7 loses the message",
+                self.assertNotEqual, lines[1], '    ')
         self.assertEqual(lines[1], '    foo')
         self.assertEqual(2, len(lines))
 
@@ -893,8 +924,7 @@
         """Test the behaviour of invoking addNotSupported."""
         class InstrumentedTestResult(tests.ExtendedTestResult):
             def stopTestRun(self): pass
-            def startTests(self): pass
-            def report_test_start(self, test): pass
+            def report_tests_starting(self): pass
             def report_unsupported(self, test, feature):
                 self._call = test, feature
         result = InstrumentedTestResult(None, None, None, None)
@@ -919,7 +949,7 @@
         # verbose test output formatting
         result_stream = StringIO()
         result = bzrlib.tests.VerboseTestResult(
-            unittest._WritelnDecorator(result_stream),
+            result_stream,
             descriptions=0,
             verbosity=2,
             )
@@ -939,8 +969,7 @@
         """An UnavailableFeature being raised should invoke addNotSupported."""
         class InstrumentedTestResult(tests.ExtendedTestResult):
             def stopTestRun(self): pass
-            def startTests(self): pass
-            def report_test_start(self, test): pass
+            def report_tests_starting(self): pass
             def addNotSupported(self, test, feature):
                 self._call = test, feature
         result = InstrumentedTestResult(None, None, None, None)
@@ -988,7 +1017,6 @@
         class InstrumentedTestResult(tests.ExtendedTestResult):
             calls = 0
             def startTests(self): self.calls += 1
-            def report_test_start(self, test): pass
         result = InstrumentedTestResult(None, None, None, None)
         def test_function():
             pass
@@ -996,6 +1024,19 @@
         test.run(result)
         self.assertEquals(1, result.calls)
 
+    def test_startTests_only_once(self):
+        """With multiple tests startTests should still only be called once"""
+        class InstrumentedTestResult(tests.ExtendedTestResult):
+            calls = 0
+            def startTests(self): self.calls += 1
+        result = InstrumentedTestResult(None, None, None, None)
+        suite = unittest.TestSuite([
+            unittest.FunctionTestCase(lambda: None),
+            unittest.FunctionTestCase(lambda: None)])
+        suite.run(result)
+        self.assertEquals(1, result.calls)
+        self.assertEquals(2, result.count)
+
 
 class TestUnicodeFilenameFeature(tests.TestCase):
 
@@ -1022,14 +1063,11 @@
         because of our use of global state.
         """
         old_root = tests.TestCaseInTempDir.TEST_ROOT
-        old_leak = tests.TestCase._first_thread_leaker_id
         try:
             tests.TestCaseInTempDir.TEST_ROOT = None
-            tests.TestCase._first_thread_leaker_id = None
             return testrunner.run(test)
         finally:
             tests.TestCaseInTempDir.TEST_ROOT = old_root
-            tests.TestCase._first_thread_leaker_id = old_leak
 
     def test_known_failure_failed_run(self):
         # run a test that generates a known failure which should be printed in
@@ -1213,6 +1251,18 @@
         self.assertContainsRe(output_string, "--date [0-9.]+")
         self.assertLength(1, self._get_source_tree_calls)
 
+    def test_verbose_test_count(self):
+        """A verbose test run reports the right test count at the start"""
+        suite = TestUtil.TestSuite([
+            unittest.FunctionTestCase(lambda:None),
+            unittest.FunctionTestCase(lambda:None)])
+        self.assertEqual(suite.countTestCases(), 2)
+        stream = StringIO()
+        runner = tests.TextTestRunner(stream=stream, verbosity=2)
+        # Need to use the CountingDecorator as that's what sets num_tests
+        result = self.run_test_runner(runner, tests.CountingDecorator(suite))
+        self.assertStartsWith(stream.getvalue(), "running 2 tests")
+
     def test_startTestRun(self):
         """run should call result.startTestRun()"""
         calls = []
@@ -1421,7 +1471,7 @@
         sample_test = TestTestCase("method_that_times_a_bit_twice")
         output_stream = StringIO()
         result = bzrlib.tests.VerboseTestResult(
-            unittest._WritelnDecorator(output_stream),
+            output_stream,
             descriptions=0,
             verbosity=2)
         sample_test.run(result)
@@ -1655,6 +1705,142 @@
         self.assertEqual('original', obj.test_attr)
 
 
+class _MissingFeature(tests.Feature):
+    def _probe(self):
+        return False
+missing_feature = _MissingFeature()
+
+
+def _get_test(name):
+    """Get an instance of a specific example test.
+
+    We protect this in a function so that they don't auto-run in the test
+    suite.
+    """
+
+    class ExampleTests(tests.TestCase):
+
+        def test_fail(self):
+            mutter('this was a failing test')
+            self.fail('this test will fail')
+
+        def test_error(self):
+            mutter('this test errored')
+            raise RuntimeError('gotcha')
+
+        def test_missing_feature(self):
+            mutter('missing the feature')
+            self.requireFeature(missing_feature)
+
+        def test_skip(self):
+            mutter('this test will be skipped')
+            raise tests.TestSkipped('reason')
+
+        def test_success(self):
+            mutter('this test succeeds')
+
+        def test_xfail(self):
+            mutter('test with expected failure')
+            self.knownFailure('this_fails')
+
+        def test_unexpected_success(self):
+            mutter('test with unexpected success')
+            self.expectFailure('should_fail', lambda: None)
+
+    return ExampleTests(name)
+
+
+class TestTestCaseLogDetails(tests.TestCase):
+
+    def _run_test(self, test_name):
+        test = _get_test(test_name)
+        result = testtools.TestResult()
+        test.run(result)
+        return result
+
+    def test_fail_has_log(self):
+        result = self._run_test('test_fail')
+        self.assertEqual(1, len(result.failures))
+        result_content = result.failures[0][1]
+        self.assertContainsRe(result_content, 'Text attachment: log')
+        self.assertContainsRe(result_content, 'this was a failing test')
+
+    def test_error_has_log(self):
+        result = self._run_test('test_error')
+        self.assertEqual(1, len(result.errors))
+        result_content = result.errors[0][1]
+        self.assertContainsRe(result_content, 'Text attachment: log')
+        self.assertContainsRe(result_content, 'this test errored')
+
+    def test_skip_has_no_log(self):
+        result = self._run_test('test_skip')
+        self.assertEqual(['reason'], result.skip_reasons.keys())
+        skips = result.skip_reasons['reason']
+        self.assertEqual(1, len(skips))
+        test = skips[0]
+        self.assertFalse('log' in test.getDetails())
+
+    def test_missing_feature_has_no_log(self):
+        # testtools doesn't know about addNotSupported, so it just gets
+        # considered as a skip
+        result = self._run_test('test_missing_feature')
+        self.assertEqual([missing_feature], result.skip_reasons.keys())
+        skips = result.skip_reasons[missing_feature]
+        self.assertEqual(1, len(skips))
+        test = skips[0]
+        self.assertFalse('log' in test.getDetails())
+
+    def test_xfail_has_no_log(self):
+        result = self._run_test('test_xfail')
+        self.assertEqual(1, len(result.expectedFailures))
+        result_content = result.expectedFailures[0][1]
+        self.assertNotContainsRe(result_content, 'Text attachment: log')
+        self.assertNotContainsRe(result_content, 'test with expected failure')
+
+    def test_unexpected_success_has_log(self):
+        result = self._run_test('test_unexpected_success')
+        self.assertEqual(1, len(result.unexpectedSuccesses))
+        # Inconsistency, unexpectedSuccesses is a list of tests,
+        # expectedFailures is a list of reasons?
+        test = result.unexpectedSuccesses[0]
+        details = test.getDetails()
+        self.assertTrue('log' in details)
+
+
+class TestTestCloning(tests.TestCase):
+    """Tests that test cloning of TestCases (as used by multiply_tests)."""
+
+    def test_cloned_testcase_does_not_share_details(self):
+        """A TestCase cloned with clone_test does not share mutable attributes
+        such as details or cleanups.
+        """
+        class Test(tests.TestCase):
+            def test_foo(self):
+                self.addDetail('foo', Content('text/plain', lambda: 'foo'))
+        orig_test = Test('test_foo')
+        cloned_test = tests.clone_test(orig_test, orig_test.id() + '(cloned)')
+        orig_test.run(unittest.TestResult())
+        self.assertEqual('foo', orig_test.getDetails()['foo'].iter_bytes())
+        self.assertEqual(None, cloned_test.getDetails().get('foo'))
+
+    def test_double_apply_scenario_preserves_first_scenario(self):
+        """Applying two levels of scenarios to a test preserves the attributes
+        added by both scenarios.
+        """
+        class Test(tests.TestCase):
+            def test_foo(self):
+                pass
+        test = Test('test_foo')
+        scenarios_x = [('x=1', {'x': 1}), ('x=2', {'x': 2})]
+        scenarios_y = [('y=1', {'y': 1}), ('y=2', {'y': 2})]
+        suite = tests.multiply_tests(test, scenarios_x, unittest.TestSuite())
+        suite = tests.multiply_tests(suite, scenarios_y, unittest.TestSuite())
+        all_tests = list(tests.iter_suite_tests(suite))
+        self.assertLength(4, all_tests)
+        all_xys = sorted((t.x, t.y) for t in all_tests)
+        self.assertEqual([(1, 1), (1, 2), (2, 1), (2, 2)], all_xys)
+
+
 # NB: Don't delete this; it's not actually from 0.11!
 @deprecated_function(deprecated_in((0, 11, 0)))
 def sample_deprecated_function():
@@ -1814,7 +2000,7 @@
                 tree.branch.repository.bzrdir.root_transport)
 
 
-class SelfTestHelper:
+class SelfTestHelper(object):
 
     def run_selftest(self, **kwargs):
         """Run selftest returning its output."""
@@ -1971,6 +2157,83 @@
             load_list='missing file name', list_only=True)
 
 
+class TestSubunitLogDetails(tests.TestCase, SelfTestHelper):
+
+    _test_needs_features = [features.subunit]
+
+    def run_subunit_stream(self, test_name):
+        from subunit import ProtocolTestCase
+        def factory():
+            return TestUtil.TestSuite([_get_test(test_name)])
+        stream = self.run_selftest(runner_class=tests.SubUnitBzrRunner,
+            test_suite_factory=factory)
+        test = ProtocolTestCase(stream)
+        result = testtools.TestResult()
+        test.run(result)
+        content = stream.getvalue()
+        return content, result
+
+    def test_fail_has_log(self):
+        content, result = self.run_subunit_stream('test_fail')
+        self.assertEqual(1, len(result.failures))
+        self.assertContainsRe(content, '(?m)^log$')
+        self.assertContainsRe(content, 'this test will fail')
+
+    def test_error_has_log(self):
+        content, result = self.run_subunit_stream('test_error')
+        self.assertContainsRe(content, '(?m)^log$')
+        self.assertContainsRe(content, 'this test errored')
+
+    def test_skip_has_no_log(self):
+        content, result = self.run_subunit_stream('test_skip')
+        self.assertNotContainsRe(content, '(?m)^log$')
+        self.assertNotContainsRe(content, 'this test will be skipped')
+        self.assertEqual(['reason'], result.skip_reasons.keys())
+        skips = result.skip_reasons['reason']
+        self.assertEqual(1, len(skips))
+        test = skips[0]
+        # RemotedTestCase doesn't preserve the "details"
+        ## self.assertFalse('log' in test.getDetails())
+
+    def test_missing_feature_has_no_log(self):
+        content, result = self.run_subunit_stream('test_missing_feature')
+        self.assertNotContainsRe(content, '(?m)^log$')
+        self.assertNotContainsRe(content, 'missing the feature')
+        self.assertEqual(['_MissingFeature\n'], result.skip_reasons.keys())
+        skips = result.skip_reasons['_MissingFeature\n']
+        self.assertEqual(1, len(skips))
+        test = skips[0]
+        # RemotedTestCase doesn't preserve the "details"
+        ## self.assertFalse('log' in test.getDetails())
+
+    def test_xfail_has_no_log(self):
+        content, result = self.run_subunit_stream('test_xfail')
+        self.assertNotContainsRe(content, '(?m)^log$')
+        self.assertNotContainsRe(content, 'test with expected failure')
+        self.assertEqual(1, len(result.expectedFailures))
+        result_content = result.expectedFailures[0][1]
+        self.assertNotContainsRe(result_content, 'Text attachment: log')
+        self.assertNotContainsRe(result_content, 'test with expected failure')
+
+    def test_unexpected_success_has_log(self):
+        content, result = self.run_subunit_stream('test_unexpected_success')
+        self.assertContainsRe(content, '(?m)^log$')
+        self.assertContainsRe(content, 'test with unexpected success')
+        self.expectFailure('subunit treats "unexpectedSuccess"'
+                           ' as a plain success',
+            self.assertEqual, 1, len(result.unexpectedSuccesses))
+        self.assertEqual(1, len(result.unexpectedSuccesses))
+        test = result.unexpectedSuccesses[0]
+        # RemotedTestCase doesn't preserve the "details"
+        ## self.assertTrue('log' in test.getDetails())
+
+    def test_success_has_no_log(self):
+        content, result = self.run_subunit_stream('test_success')
+        self.assertEqual(1, result.testsRun)
+        self.assertNotContainsRe(content, '(?m)^log$')
+        self.assertNotContainsRe(content, 'this test succeeds')
+
+
 class TestRunBzr(tests.TestCase):
 
     out = ''
@@ -2339,6 +2602,11 @@
             os.chdir = orig_chdir
         self.assertEqual(['foo', 'current'], chdirs)
 
+    def test_get_bzr_path_with_cwd_bzrlib(self):
+        self.get_source_path = lambda: ""
+        self.overrideAttr(os.path, "isfile", lambda path: True)
+        self.assertEqual(self.get_bzr_path(), "bzr")
+
 
 class TestActuallyStartBzrSubprocess(tests.TestCaseWithTransport):
     """Tests that really need to do things with an external bzr."""
@@ -2960,6 +3228,94 @@
         self.assertEquals('bzrlib.plugins', tpr.resolve_alias('bp'))
 
 
+class TestThreadLeakDetection(tests.TestCase):
+    """Ensure when tests leak threads we detect and report it"""
+
+    class LeakRecordingResult(tests.ExtendedTestResult):
+        def __init__(self):
+            tests.ExtendedTestResult.__init__(self, StringIO(), 0, 1)
+            self.leaks = []
+        def _report_thread_leak(self, test, leaks, alive):
+            self.leaks.append((test, leaks))
+
+    def test_testcase_without_addCleanups(self):
+        """Check old TestCase instances don't break with leak detection"""
+        class Test(unittest.TestCase):
+            def runTest(self):
+                pass
+            addCleanup = None # for when on Python 2.7 with native addCleanup
+        result = self.LeakRecordingResult()
+        test = Test()
+        self.assertIs(getattr(test, "addCleanup", None), None)
+        result.startTestRun()
+        test.run(result)
+        result.stopTestRun()
+        self.assertEqual(result._tests_leaking_threads_count, 0)
+        self.assertEqual(result.leaks, [])
+        
+    def test_thread_leak(self):
+        """Ensure a thread that outlives the running of a test is reported
+
+        Uses a thread that blocks on an event, and is started by the inner
+        test case. As the thread outlives the inner case's run, it should be
+        detected as a leak, but the event is then set so that the thread can
+        be safely joined in cleanup so it's not leaked for real.
+        """
+        event = threading.Event()
+        thread = threading.Thread(name="Leaker", target=event.wait)
+        class Test(tests.TestCase):
+            def test_leak(self):
+                thread.start()
+        result = self.LeakRecordingResult()
+        test = Test("test_leak")
+        self.addCleanup(thread.join)
+        self.addCleanup(event.set)
+        result.startTestRun()
+        test.run(result)
+        result.stopTestRun()
+        self.assertEqual(result._tests_leaking_threads_count, 1)
+        self.assertEqual(result._first_thread_leaker_id, test.id())
+        self.assertEqual(result.leaks, [(test, set([thread]))])
+        self.assertContainsString(result.stream.getvalue(), "leaking threads")
+
+    def test_multiple_leaks(self):
+        """Check multiple leaks are blamed on the test cases at fault
+
+        Same concept as the previous test, but has one inner test method that
+        leaks two threads, and one that doesn't leak at all.
+        """
+        event = threading.Event()
+        thread_a = threading.Thread(name="LeakerA", target=event.wait)
+        thread_b = threading.Thread(name="LeakerB", target=event.wait)
+        thread_c = threading.Thread(name="LeakerC", target=event.wait)
+        class Test(tests.TestCase):
+            def test_first_leak(self):
+                thread_b.start()
+            def test_second_no_leak(self):
+                pass
+            def test_third_leak(self):
+                thread_c.start()
+                thread_a.start()
+        result = self.LeakRecordingResult()
+        first_test = Test("test_first_leak")
+        third_test = Test("test_third_leak")
+        self.addCleanup(thread_a.join)
+        self.addCleanup(thread_b.join)
+        self.addCleanup(thread_c.join)
+        self.addCleanup(event.set)
+        result.startTestRun()
+        unittest.TestSuite(
+            [first_test, Test("test_second_no_leak"), third_test]
+            ).run(result)
+        result.stopTestRun()
+        self.assertEqual(result._tests_leaking_threads_count, 2)
+        self.assertEqual(result._first_thread_leaker_id, first_test.id())
+        self.assertEqual(result.leaks, [
+            (first_test, set([thread_b])),
+            (third_test, set([thread_a, thread_c]))])
+        self.assertContainsString(result.stream.getvalue(), "leaking threads")
+
+
 class TestRunSuite(tests.TestCase):
 
     def test_runner_class(self):

=== modified file 'bzrlib/tests/test_server.py'
--- bzrlib/tests/test_server.py	2010-02-11 09:21:45 +0000
+++ bzrlib/tests/test_server.py	2010-08-31 08:24:17 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007, 2008, 2010 Canonical Ltd
+# Copyright (C) 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,7 +14,16 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+import errno
+import socket
+import SocketServer
+import select
+import sys
+import threading
+
+
 from bzrlib import (
+    osutils,
     transport,
     urlutils,
     )
@@ -22,7 +31,18 @@
     chroot,
     pathfilter,
     )
-from bzrlib.smart import server
+from bzrlib.smart import (
+    medium,
+    server,
+    )
+
+
+def debug_threads():
+    # FIXME: There is a dependency loop between bzrlib.tests and
+    # bzrlib.tests.test_server that needs to be fixed. In the mean time
+    # defining this function is enough for our needs. -- vila 20100611
+    from bzrlib import tests
+    return 'threads' in tests.selftest_debug_flags
 
 
 class TestServer(transport.Server):
@@ -223,23 +243,495 @@
         raise NotImplementedError
 
 
-class SmartTCPServer_for_testing(server.SmartTCPServer):
+class ThreadWithException(threading.Thread):
+    """A catching exception thread.
+
+    If an exception occurs during the thread execution, it's caught and
+    re-raised when the thread is joined().
+    """
+
+    def __init__(self, *args, **kwargs):
+        # There are cases where the calling thread must wait, yet, if an
+        # exception occurs, the event should be set so the caller is not
+        # blocked. The main example is a calling thread that want to wait for
+        # the called thread to be in a given state before continuing.
+        try:
+            event = kwargs.pop('event')
+        except KeyError:
+            # If the caller didn't pass a specific event, create our own
+            event = threading.Event()
+        super(ThreadWithException, self).__init__(*args, **kwargs)
+        self.set_ready_event(event)
+        self.exception = None
+        self.ignored_exceptions = None # see set_ignored_exceptions
+
+    # compatibility thunk for python-2.4 and python-2.5...
+    if sys.version_info < (2, 6):
+        name = property(threading.Thread.getName, threading.Thread.setName)
+
+    def set_ready_event(self, event):
+        """Set the ``ready`` event used to synchronize exception catching.
+
+        When the thread uses an event to synchronize itself with another thread
+        (setting it when the other thread can wake up from a ``wait`` call),
+        the event must be set after catching an exception or the other thread
+        will hang.
+
+        Some threads require multiple events and should set the relevant one
+        when appropriate.
+        """
+        self.ready = event
+
+    def set_ignored_exceptions(self, ignored):
+        """Declare which exceptions will be ignored.
+
+        :param ignored: Can be either:
+           - None: all exceptions will be raised,
+           - an exception class: the instances of this class will be ignored,
+           - a tuple of exception classes: the instances of any class of the
+             list will be ignored,
+           - a callable: that will be passed the exception object
+             and should return True if the exception should be ignored
+        """
+        if ignored is None:
+            self.ignored_exceptions = None
+        elif isinstance(ignored, (Exception, tuple)):
+            self.ignored_exceptions = lambda e: isinstance(e, ignored)
+        else:
+            self.ignored_exceptions = ignored
+
+    def run(self):
+        """Overrides Thread.run to capture any exception."""
+        self.ready.clear()
+        try:
+            try:
+                super(ThreadWithException, self).run()
+            except:
+                self.exception = sys.exc_info()
+        finally:
+            # Make sure the calling thread is released
+            self.ready.set()
+
+
+    def join(self, timeout=5):
+        """Overrides Thread.join to raise any exception caught.
+
+
+        Calling join(timeout=0) will raise the caught exception or return None
+        if the thread is still alive.
+
+        The default timeout is set to 5 and should expire only when a thread
+        serving a client connection is hung.
+        """
+        super(ThreadWithException, self).join(timeout)
+        if self.exception is not None:
+            exc_class, exc_value, exc_tb = self.exception
+            self.exception = None # The exception should be raised only once
+            if (self.ignored_exceptions is None
+                or not self.ignored_exceptions(exc_value)):
+                # Raise non ignored exceptions
+                raise exc_class, exc_value, exc_tb
+        if timeout and self.isAlive():
+            # The timeout expired without joining the thread, the thread is
+            # therefore stucked and that's a failure as far as the test is
+            # concerned. We used to hang here.
+
+            # FIXME: we need to kill the thread, but as far as the test is
+            # concerned, raising an assertion is too strong. On most of the
+            # platforms, this doesn't occur, so just mentioning the problem is
+            # enough for now -- vila 2010824
+            sys.stderr.write('thread %s hung\n' % (self.name,))
+            #raise AssertionError('thread %s hung' % (self.name,))
+
+    def pending_exception(self):
+        """Raise the caught exception.
+
+        This does nothing if no exception occurred.
+        """
+        self.join(timeout=0)
+
+
+class TestingTCPServerMixin:
+    """Mixin to support running SocketServer.TCPServer in a thread.
+
+    Tests are connecting from the main thread, the server has to be run in a
+    separate thread.
+    """
+
+    def __init__(self):
+        self.started = threading.Event()
+        self.serving = None
+        self.stopped = threading.Event()
+        # We collect the resources used by the clients so we can release them
+        # when shutting down
+        self.clients = []
+        self.ignored_exceptions = None
+
+    def server_bind(self):
+        self.socket.bind(self.server_address)
+        self.server_address = self.socket.getsockname()
+
+    def serve(self):
+        self.serving = True
+        self.stopped.clear()
+        # We are listening and ready to accept connections
+        self.started.set()
+        try:
+            while self.serving:
+                # Really a connection but the python framework is generic and
+                # call them requests
+                self.handle_request()
+            # Let's close the listening socket
+            self.server_close()
+        finally:
+            self.stopped.set()
+
+    def handle_request(self):
+        """Handle one request.
+
+        The python version swallows some socket exceptions and we don't use
+        timeout, so we override it to better control the server behavior.
+        """
+        request, client_address = self.get_request()
+        if self.verify_request(request, client_address):
+            try:
+                self.process_request(request, client_address)
+            except:
+                self.handle_error(request, client_address)
+                self.close_request(request)
+
+    def get_request(self):
+        return self.socket.accept()
+
+    def verify_request(self, request, client_address):
+        """Verify the request.
+
+        Return True if we should proceed with this request, False if we should
+        not even touch a single byte in the socket ! This is useful when we
+        stop the server with a dummy last connection.
+        """
+        return self.serving
+
+    def handle_error(self, request, client_address):
+        # Stop serving and re-raise the last exception seen
+        self.serving = False
+        # The following can be used for debugging purposes, it will display the
+        # exception and the traceback just when it occurs instead of waiting
+        # for the thread to be joined.
+
+        # SocketServer.BaseServer.handle_error(self, request, client_address)
+        raise
+
+    def ignored_exceptions_during_shutdown(self, e):
+        if sys.platform == 'win32':
+            accepted_errnos = [errno.EBADF,
+                               errno.EPIPE,
+                               errno.WSAEBADF,
+                               errno.WSAECONNRESET,
+                               errno.WSAENOTCONN,
+                               errno.WSAESHUTDOWN,
+                               ]
+        else:
+            accepted_errnos = [errno.EBADF,
+                               errno.ECONNRESET,
+                               errno.ENOTCONN,
+                               errno.EPIPE,
+                               ]
+        if isinstance(e, socket.error) and e[0] in accepted_errnos:
+            return True
+        return False
+
+    # The following methods are called by the main thread
+
+    def stop_client_connections(self):
+        while self.clients:
+            c = self.clients.pop()
+            self.shutdown_client(c)
+
+    def shutdown_socket(self, sock):
+        """Properly shutdown a socket.
+
+        This should be called only when no other thread is trying to use the
+        socket.
+        """
+        try:
+            sock.shutdown(socket.SHUT_RDWR)
+            sock.close()
+        except Exception, e:
+            if self.ignored_exceptions(e):
+                pass
+            else:
+                raise
+
+    # The following methods are called by the main thread
+
+    def set_ignored_exceptions(self, thread, ignored_exceptions):
+        self.ignored_exceptions = ignored_exceptions
+        thread.set_ignored_exceptions(self.ignored_exceptions)
+
+    def _pending_exception(self, thread):
+        """Raise server uncaught exception.
+
+        Daughter classes can override this if they use daughter threads.
+        """
+        thread.pending_exception()
+
+
+class TestingTCPServer(TestingTCPServerMixin, SocketServer.TCPServer):
+
+    def __init__(self, server_address, request_handler_class):
+        TestingTCPServerMixin.__init__(self)
+        SocketServer.TCPServer.__init__(self, server_address,
+                                        request_handler_class)
+
+    def get_request(self):
+        """Get the request and client address from the socket."""
+        sock, addr = TestingTCPServerMixin.get_request(self)
+        self.clients.append((sock, addr))
+        return sock, addr
+
+    # The following methods are called by the main thread
+
+    def shutdown_client(self, client):
+        sock, addr = client
+        self.shutdown_socket(sock)
+
+
+class TestingThreadingTCPServer(TestingTCPServerMixin,
+                                SocketServer.ThreadingTCPServer):
+
+    def __init__(self, server_address, request_handler_class):
+        TestingTCPServerMixin.__init__(self)
+        SocketServer.ThreadingTCPServer.__init__(self, server_address,
+                                                 request_handler_class)
+
+    def get_request (self):
+        """Get the request and client address from the socket."""
+        sock, addr = TestingTCPServerMixin.get_request(self)
+        # The thread is not create yet, it will be updated in process_request
+        self.clients.append((sock, addr, None))
+        return sock, addr
+
+    def process_request_thread(self, started, stopped, request, client_address):
+        started.set()
+        SocketServer.ThreadingTCPServer.process_request_thread(
+            self, request, client_address)
+        self.close_request(request)
+        stopped.set()
+
+    def process_request(self, request, client_address):
+        """Start a new thread to process the request."""
+        started = threading.Event()
+        stopped = threading.Event()
+        t = ThreadWithException(
+            event=stopped,
+            name='%s -> %s' % (client_address, self.server_address),
+            target = self.process_request_thread,
+            args = (started, stopped, request, client_address))
+        # Update the client description
+        self.clients.pop()
+        self.clients.append((request, client_address, t))
+        # Propagate the exception handler since we must use the same one for
+        # connections running in their own threads than TestingTCPServer.
+        t.set_ignored_exceptions(self.ignored_exceptions)
+        t.start()
+        started.wait()
+        if debug_threads():
+            sys.stderr.write('Client thread %s started\n' % (t.name,))
+        # If an exception occured during the thread start, it will get raised.
+        t.pending_exception()
+
+    # The following methods are called by the main thread
+
+    def shutdown_client(self, client):
+        sock, addr, connection_thread = client
+        self.shutdown_socket(sock)
+        if connection_thread is not None:
+            # The thread has been created only if the request is processed but
+            # after the connection is inited. This could happen during server
+            # shutdown. If an exception occurred in the thread it will be
+            # re-raised
+            if debug_threads():
+                sys.stderr.write('Client thread %s will be joined\n'
+                                 % (connection_thread.name,))
+            connection_thread.join()
+
+    def set_ignored_exceptions(self, thread, ignored_exceptions):
+        TestingTCPServerMixin.set_ignored_exceptions(self, thread,
+                                                     ignored_exceptions)
+        for sock, addr, connection_thread in self.clients:
+            if connection_thread is not None:
+                connection_thread.set_ignored_exceptions(
+                    self.ignored_exceptions)
+
+    def _pending_exception(self, thread):
+        for sock, addr, connection_thread in self.clients:
+            if connection_thread is not None:
+                connection_thread.pending_exception()
+        TestingTCPServerMixin._pending_exception(self, thread)
+
+
+class TestingTCPServerInAThread(transport.Server):
+    """A server in a thread that re-raise thread exceptions."""
+
+    def __init__(self, server_address, server_class, request_handler_class):
+        self.server_class = server_class
+        self.request_handler_class = request_handler_class
+        self.host, self.port = server_address
+        self.server = None
+        self._server_thread = None
+
+    def __repr__(self):
+        return "%s(%s:%s)" % (self.__class__.__name__, self.host, self.port)
+
+    def create_server(self):
+        return self.server_class((self.host, self.port),
+                                 self.request_handler_class)
+
+    def start_server(self):
+        self.server = self.create_server()
+        self._server_thread = ThreadWithException(
+            event=self.server.started,
+            target=self.run_server)
+        self._server_thread.start()
+        # Wait for the server thread to start (i.e release the lock)
+        self.server.started.wait()
+        # Get the real address, especially the port
+        self.host, self.port = self.server.server_address
+        self._server_thread.name = self.server.server_address
+        if debug_threads():
+            sys.stderr.write('Server thread %s started\n'
+                             % (self._server_thread.name,))
+        # If an exception occured during the server start, it will get raised,
+        # otherwise, the server is blocked on its accept() call.
+        self._server_thread.pending_exception()
+        # From now on, we'll use a different event to ensure the server can set
+        # its exception
+        self._server_thread.set_ready_event(self.server.stopped)
+
+    def run_server(self):
+        self.server.serve()
+
+    def stop_server(self):
+        if self.server is None:
+            return
+        try:
+            # The server has been started successfully, shut it down now.  As
+            # soon as we stop serving, no more connection are accepted except
+            # one to get out of the blocking listen.
+            self.set_ignored_exceptions(
+                self.server.ignored_exceptions_during_shutdown)
+            self.server.serving = False
+            if debug_threads():
+                sys.stderr.write('Server thread %s will be joined\n'
+                                 % (self._server_thread.name,))
+            # The server is listening for a last connection, let's give it:
+            last_conn = None
+            try:
+                last_conn = osutils.connect_socket((self.host, self.port))
+            except socket.error, e:
+                # But ignore connection errors as the point is to unblock the
+                # server thread, it may happen that it's not blocked or even
+                # not started.
+                pass
+            # We start shutting down the client while the server itself is
+            # shutting down.
+            self.server.stop_client_connections()
+            # Now we wait for the thread running self.server.serve() to finish
+            self.server.stopped.wait()
+            if last_conn is not None:
+                # Close the last connection without trying to use it. The
+                # server will not process a single byte on that socket to avoid
+                # complications (SSL starts with a handshake for example).
+                last_conn.close()
+            # Check for any exception that could have occurred in the server
+            # thread
+            try:
+                self._server_thread.join()
+            except Exception, e:
+                if self.server.ignored_exceptions(e):
+                    pass
+                else:
+                    raise
+        finally:
+            # Make sure we can be called twice safely, note that this means
+            # that we will raise a single exception even if several occurred in
+            # the various threads involved.
+            self.server = None
+
+    def set_ignored_exceptions(self, ignored_exceptions):
+        """Install an exception handler for the server."""
+        self.server.set_ignored_exceptions(self._server_thread,
+                                           ignored_exceptions)
+
+    def pending_exception(self):
+        """Raise uncaught exception in the server."""
+        self.server._pending_exception(self._server_thread)
+
+
+class TestingSmartConnectionHandler(SocketServer.BaseRequestHandler,
+                                    medium.SmartServerSocketStreamMedium):
+
+    def __init__(self, request, client_address, server):
+        medium.SmartServerSocketStreamMedium.__init__(
+            self, request, server.backing_transport,
+            server.root_client_path)
+        request.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
+        SocketServer.BaseRequestHandler.__init__(self, request, client_address,
+                                                 server)
+
+    def handle(self):
+        while not self.finished:
+            server_protocol = self._build_protocol()
+            self._serve_one_request(server_protocol)
+
+
+class TestingSmartServer(TestingThreadingTCPServer, server.SmartTCPServer):
+
+    def __init__(self, server_address, request_handler_class,
+                 backing_transport, root_client_path):
+        TestingThreadingTCPServer.__init__(self, server_address,
+                                           request_handler_class)
+        server.SmartTCPServer.__init__(self, backing_transport,
+                                       root_client_path)
+    def serve(self):
+        # FIXME: No test are exercising the hooks for the test server
+        # -- vila 20100618
+        self.run_server_started_hooks()
+        try:
+            TestingThreadingTCPServer.serve(self)
+        finally:
+            self.run_server_stopped_hooks()
+
+    def get_url(self):
+        """Return the url of the server"""
+        return "bzr://%s:%d/" % self.server_address
+
+
+class SmartTCPServer_for_testing(TestingTCPServerInAThread):
     """Server suitable for use by transport tests.
 
     This server is backed by the process's cwd.
     """
-
     def __init__(self, thread_name_suffix=''):
-        super(SmartTCPServer_for_testing, self).__init__(None)
         self.client_path_extra = None
         self.thread_name_suffix = thread_name_suffix
-
-    def get_backing_transport(self, backing_transport_server):
-        """Get a backing transport from a server we are decorating."""
-        return transport.get_transport(backing_transport_server.get_url())
+        self.host = '127.0.0.1'
+        self.port = 0
+        super(SmartTCPServer_for_testing, self).__init__(
+                (self.host, self.port),
+                TestingSmartServer,
+                TestingSmartConnectionHandler)
+
+    def create_server(self):
+        return self.server_class((self.host, self.port),
+                                 self.request_handler_class,
+                                 self.backing_transport,
+                                 self.root_client_path)
+
 
     def start_server(self, backing_transport_server=None,
-              client_path_extra='/extra/'):
+                     client_path_extra='/extra/'):
         """Set up server for testing.
 
         :param backing_transport_server: backing server to use.  If not
@@ -254,6 +746,7 @@
         """
         if not client_path_extra.startswith('/'):
             raise ValueError(client_path_extra)
+        self.root_client_path = self.client_path_extra = client_path_extra
         from bzrlib.transport.chroot import ChrootServer
         if backing_transport_server is None:
             backing_transport_server = LocalURLServer()
@@ -262,15 +755,20 @@
         self.chroot_server.start_server()
         self.backing_transport = transport.get_transport(
             self.chroot_server.get_url())
-        self.root_client_path = self.client_path_extra = client_path_extra
-        self.start_background_thread(self.thread_name_suffix)
+        super(SmartTCPServer_for_testing, self).start_server()
 
     def stop_server(self):
-        self.stop_background_thread()
-        self.chroot_server.stop_server()
+        try:
+            super(SmartTCPServer_for_testing, self).stop_server()
+        finally:
+            self.chroot_server.stop_server()
+
+    def get_backing_transport(self, backing_transport_server):
+        """Get a backing transport from a server we are decorating."""
+        return transport.get_transport(backing_transport_server.get_url())
 
     def get_url(self):
-        url = super(SmartTCPServer_for_testing, self).get_url()
+        url = self.server.get_url()
         return url[:-1] + self.client_path_extra
 
     def get_bogus_url(self):

=== modified file 'bzrlib/tests/test_setup.py'
--- bzrlib/tests/test_setup.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_setup.py	2010-09-25 20:08:01 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2008, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -74,9 +74,11 @@
         self.log('args: %r', args)
         p = subprocess.Popen(args,
                              cwd=self.source_dir,
-                             stdout=self._log_file,
-                             stderr=self._log_file,
+                             stdout=subprocess.PIPE,
+                             stderr=subprocess.PIPE,
                              )
-        s = p.communicate()
+        stdout, stderr = p.communicate()
+        self.log('stdout: %r', stdout)
+        self.log('stderr: %r', stderr)
         self.assertEqual(0, p.returncode,
                          'invocation of %r failed' % args)

=== modified file 'bzrlib/tests/test_sftp_transport.py'
--- bzrlib/tests/test_sftp_transport.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/test_sftp_transport.py	2010-06-23 08:17:20 +0000
@@ -41,7 +41,6 @@
     TestSkipped,
     )
 from bzrlib.tests.http_server import HttpServer
-from bzrlib.transport import get_transport
 import bzrlib.transport.http
 
 if features.paramiko.available():
@@ -190,7 +189,7 @@
         server = stub_sftp.SFTPSiblingAbsoluteServer()
         server.start_server()
         try:
-            transport = get_transport(server.get_url())
+            transport = _mod_transport.get_transport(server.get_url())
             self.assertFalse(transport.abspath('/').endswith('/~/'))
             self.assertTrue(transport.abspath('/').endswith('/'))
             del transport
@@ -313,7 +312,7 @@
         """Test that a real connection attempt raises the right error"""
         from bzrlib.transport import ssh
         self.set_vendor(ssh.ParamikoVendor())
-        t = bzrlib.transport.get_transport(self.bogus_url)
+        t = _mod_transport.get_transport(self.bogus_url)
         self.assertRaises(errors.ConnectionError, t.get, 'foobar')
 
     def test_bad_connection_ssh(self):
@@ -324,7 +323,7 @@
         # However, 'ssh' will create stipple on the output, so instead
         # I'm using run_bzr_subprocess, and parsing the output
         # try:
-        #     t = bzrlib.transport.get_transport(self.bogus_url)
+        #     t = _mod_transport.get_transport(self.bogus_url)
         # except errors.ConnectionError:
         #     # Correct error
         #     pass
@@ -505,13 +504,13 @@
     """Test that AuthenticationConfig can supply default usernames."""
 
     def get_transport_for_connection(self, set_config):
-        port = self.get_server()._listener.port
+        port = self.get_server().port
         if set_config:
             conf = config.AuthenticationConfig()
             conf._get_config().update(
                 {'sftptest': {'scheme': 'ssh', 'port': port, 'user': 'bar'}})
             conf._save()
-        t = get_transport('sftp://localhost:%d' % port)
+        t = _mod_transport.get_transport('sftp://localhost:%d' % port)
         # force a connection to be performed.
         t.has('foo')
         return t

=== modified file 'bzrlib/tests/test_smart.py'
--- bzrlib/tests/test_smart.py	2010-03-02 22:25:58 +0000
+++ bzrlib/tests/test_smart.py	2010-05-13 16:17:54 +0000
@@ -767,8 +767,8 @@
 class TestLockedBranch(tests.TestCaseWithMemoryTransport):
 
     def get_lock_tokens(self, branch):
-        branch_token = branch.lock_write()
-        repo_token = branch.repository.lock_write()
+        branch_token = branch.lock_write().branch_token
+        repo_token = branch.repository.lock_write().repository_token
         branch.repository.unlock()
         return branch_token, repo_token
 
@@ -802,6 +802,44 @@
         branch.unlock()
 
 
+class TestSmartServerBranchRequestSetConfigOptionDict(TestLockedBranch):
+
+    def setUp(self):
+        TestLockedBranch.setUp(self)
+        # A dict with non-ascii keys and values to exercise unicode
+        # roundtripping.
+        self.encoded_value_dict = (
+            'd5:ascii1:a11:unicode \xe2\x8c\x9a3:\xe2\x80\xbde')
+        self.value_dict = {
+            'ascii': 'a', u'unicode \N{WATCH}': u'\N{INTERROBANG}'}
+
+    def test_value_name(self):
+        branch = self.make_branch('.')
+        request = smart_branch.SmartServerBranchRequestSetConfigOptionDict(
+            branch.bzrdir.root_transport)
+        branch_token, repo_token = self.get_lock_tokens(branch)
+        config = branch._get_config()
+        result = request.execute('', branch_token, repo_token,
+            self.encoded_value_dict, 'foo', '')
+        self.assertEqual(smart_req.SuccessfulSmartServerResponse(()), result)
+        self.assertEqual(self.value_dict, config.get_option('foo'))
+        # Cleanup
+        branch.unlock()
+
+    def test_value_name_section(self):
+        branch = self.make_branch('.')
+        request = smart_branch.SmartServerBranchRequestSetConfigOptionDict(
+            branch.bzrdir.root_transport)
+        branch_token, repo_token = self.get_lock_tokens(branch)
+        config = branch._get_config()
+        result = request.execute('', branch_token, repo_token,
+            self.encoded_value_dict, 'foo', 'gam')
+        self.assertEqual(smart_req.SuccessfulSmartServerResponse(()), result)
+        self.assertEqual(self.value_dict, config.get_option('foo', 'gam'))
+        # Cleanup
+        branch.unlock()
+
+
 class TestSmartServerBranchRequestSetTagsBytes(TestLockedBranch):
     # Only called when the branch format and tags match [yay factory
     # methods] so only need to test straight forward cases.
@@ -1073,7 +1111,7 @@
             response)
 
 
-class TestSmartServerBranchRequestSetParent(tests.TestCaseWithMemoryTransport):
+class TestSmartServerBranchRequestSetParent(TestLockedBranch):
 
     def test_set_parent_none(self):
         branch = self.make_branch('base', format="1.9")
@@ -1082,12 +1120,10 @@
         branch.unlock()
         request = smart_branch.SmartServerBranchRequestSetParentLocation(
             self.get_transport())
-        branch_token = branch.lock_write()
-        repo_token = branch.repository.lock_write()
+        branch_token, repo_token = self.get_lock_tokens(branch)
         try:
             response = request.execute('base', branch_token, repo_token, '')
         finally:
-            branch.repository.unlock()
             branch.unlock()
         self.assertEqual(smart_req.SuccessfulSmartServerResponse(()), response)
         self.assertEqual(None, branch.get_parent())
@@ -1096,13 +1132,11 @@
         branch = self.make_branch('base', format="1.9")
         request = smart_branch.SmartServerBranchRequestSetParentLocation(
             self.get_transport())
-        branch_token = branch.lock_write()
-        repo_token = branch.repository.lock_write()
+        branch_token, repo_token = self.get_lock_tokens(branch)
         try:
             response = request.execute('base', branch_token, repo_token,
             'http://bar/')
         finally:
-            branch.repository.unlock()
             branch.unlock()
         self.assertEqual(smart_req.SuccessfulSmartServerResponse(()), response)
         self.assertEqual('http://bar/', branch.get_parent())
@@ -1137,7 +1171,7 @@
             response)
 
 
-class TestSmartServerBranchRequestLockWrite(tests.TestCaseWithMemoryTransport):
+class TestSmartServerBranchRequestLockWrite(TestLockedBranch):
 
     def setUp(self):
         tests.TestCaseWithMemoryTransport.setUp(self)
@@ -1165,7 +1199,7 @@
         backing = self.get_transport()
         request = smart_branch.SmartServerBranchRequestLockWrite(backing)
         branch = self.make_branch('.')
-        branch_token = branch.lock_write()
+        branch_token = branch.lock_write().branch_token
         branch.leave_lock_in_place()
         branch.unlock()
         response = request.execute('')
@@ -1180,9 +1214,7 @@
         backing = self.get_transport()
         request = smart_branch.SmartServerBranchRequestLockWrite(backing)
         branch = self.make_branch('.', format='knit')
-        branch_token = branch.lock_write()
-        repo_token = branch.repository.lock_write()
-        branch.repository.unlock()
+        branch_token, repo_token = self.get_lock_tokens(branch)
         branch.leave_lock_in_place()
         branch.repository.leave_lock_in_place()
         branch.unlock()
@@ -1203,9 +1235,7 @@
         backing = self.get_transport()
         request = smart_branch.SmartServerBranchRequestLockWrite(backing)
         branch = self.make_branch('.', format='knit')
-        branch_token = branch.lock_write()
-        repo_token = branch.repository.lock_write()
-        branch.repository.unlock()
+        branch_token, repo_token = self.get_lock_tokens(branch)
         branch.leave_lock_in_place()
         branch.repository.leave_lock_in_place()
         branch.unlock()
@@ -1226,7 +1256,7 @@
         request = smart_branch.SmartServerBranchRequestLockWrite(backing)
         branch = self.make_branch('.', format='knit')
         repo = branch.repository
-        repo_token = repo.lock_write()
+        repo_token = repo.lock_write().repository_token
         repo.leave_lock_in_place()
         repo.unlock()
         response = request.execute('')
@@ -1249,7 +1279,7 @@
         self.assertEqual('LockFailed', error_name)
 
 
-class TestSmartServerBranchRequestUnlock(tests.TestCaseWithMemoryTransport):
+class TestSmartServerBranchRequestUnlock(TestLockedBranch):
 
     def setUp(self):
         tests.TestCaseWithMemoryTransport.setUp(self)
@@ -1259,9 +1289,7 @@
         request = smart_branch.SmartServerBranchRequestUnlock(backing)
         branch = self.make_branch('.', format='knit')
         # Lock the branch
-        branch_token = branch.lock_write()
-        repo_token = branch.repository.lock_write()
-        branch.repository.unlock()
+        branch_token, repo_token = self.get_lock_tokens(branch)
         # Unlock the branch (and repo) object, leaving the physical locks
         # in place.
         branch.leave_lock_in_place()
@@ -1291,13 +1319,12 @@
         request = smart_branch.SmartServerBranchRequestUnlock(backing)
         branch = self.make_branch('.', format='knit')
         # Lock the repository.
-        repo_token = branch.repository.lock_write()
+        repo_token = branch.repository.lock_write().repository_token
         branch.repository.leave_lock_in_place()
         branch.repository.unlock()
         # Issue branch lock_write request on the unlocked branch (with locked
         # repo).
-        response = request.execute(
-            '', 'branch token', repo_token)
+        response = request.execute('', 'branch token', repo_token)
         self.assertEqual(
             smart_req.SmartServerResponse(('TokenMismatch',)), response)
         # Cleanup
@@ -1610,7 +1637,7 @@
         backing = self.get_transport()
         request = smart_repo.SmartServerRepositoryLockWrite(backing)
         repository = self.make_repository('.', format='knit')
-        repo_token = repository.lock_write()
+        repo_token = repository.lock_write().repository_token
         repository.leave_lock_in_place()
         repository.unlock()
         response = request.execute('')
@@ -1658,7 +1685,7 @@
         request = smart_repo.SmartServerRepositoryInsertStreamLocked(
             backing)
         repository = self.make_repository('.', format='knit')
-        lock_token = repository.lock_write()
+        lock_token = repository.lock_write().repository_token
         response = request.execute('', '', lock_token)
         self.assertEqual(None, response)
         response = request.do_chunk(self.make_empty_byte_stream(repository))
@@ -1672,7 +1699,7 @@
         request = smart_repo.SmartServerRepositoryInsertStreamLocked(
             backing)
         repository = self.make_repository('.', format='knit')
-        lock_token = repository.lock_write()
+        lock_token = repository.lock_write().repository_token
         self.assertRaises(
             errors.TokenMismatch, request.execute, '', '', 'wrong-token')
         repository.unlock()
@@ -1687,7 +1714,7 @@
         backing = self.get_transport()
         request = smart_repo.SmartServerRepositoryUnlock(backing)
         repository = self.make_repository('.', format='knit')
-        token = repository.lock_write()
+        token = repository.lock_write().repository_token
         repository.leave_lock_in_place()
         repository.unlock()
         response = request.execute('', token)

=== modified file 'bzrlib/tests/test_smart_request.py'
--- bzrlib/tests/test_smart_request.py	2009-07-27 02:11:25 +0000
+++ bzrlib/tests/test_smart_request.py	2010-07-13 19:02:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009 Canonical Ltd
+# Copyright (C) 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,11 +18,13 @@
 
 import threading
 
-from bzrlib import errors
+from bzrlib import (
+    errors,
+    transport,
+    )
 from bzrlib.bzrdir import BzrDir
 from bzrlib.smart import request
 from bzrlib.tests import TestCase, TestCaseWithMemoryTransport
-from bzrlib.transport import get_transport
 
 
 class NoBodyRequest(request.SmartServerRequest):
@@ -34,7 +36,7 @@
 
 class DoErrorRequest(request.SmartServerRequest):
     """A request that raises an error from self.do()."""
-    
+
     def do(self):
         raise errors.NoSuchFile('xyzzy')
 
@@ -172,7 +174,7 @@
 
 
 class TestRequestJail(TestCaseWithMemoryTransport):
-    
+
     def test_jail(self):
         transport = self.get_transport('blah')
         req = request.SmartServerRequest(transport)
@@ -185,9 +187,11 @@
 
 class TestJailHook(TestCaseWithMemoryTransport):
 
-    def tearDown(self):
-        request.jail_info.transports = None
-        TestCaseWithMemoryTransport.tearDown(self)
+    def setUp(self):
+        super(TestJailHook, self).setUp()
+        def clear_jail_info():
+            request.jail_info.transports = None
+        self.addCleanup(clear_jail_info)
 
     def test_jail_hook(self):
         request.jail_info.transports = None
@@ -203,8 +207,8 @@
         # A parent is not allowed
         self.assertRaises(errors.JailBreak, _pre_open_hook, t.clone('..'))
         # A completely unrelated transport is not allowed
-        self.assertRaises(
-            errors.JailBreak, _pre_open_hook, get_transport('http://host/'))
+        self.assertRaises(errors.JailBreak, _pre_open_hook,
+                          transport.get_transport('http://host/'))
 
     def test_open_bzrdir_in_non_main_thread(self):
         """Opening a bzrdir in a non-main thread should work ok.

=== modified file 'bzrlib/tests/test_smart_transport.py'
--- bzrlib/tests/test_smart_transport.py	2010-03-18 23:11:15 +0000
+++ bzrlib/tests/test_smart_transport.py	2010-07-01 15:25:41 +0000
@@ -40,12 +40,16 @@
         server,
         vfs,
 )
-from bzrlib.tests import test_smart
+from bzrlib.tests import (
+    test_smart,
+    test_server,
+    )
 from bzrlib.transport import (
         http,
         local,
         memory,
         remote,
+        ssh,
         )
 
 
@@ -63,7 +67,7 @@
         return StringIOSSHConnection(self)
 
 
-class StringIOSSHConnection(object):
+class StringIOSSHConnection(ssh.SSHConnection):
     """A SSH connection that uses StringIO to buffer writes and answer reads."""
 
     def __init__(self, vendor):
@@ -71,9 +75,11 @@
 
     def close(self):
         self.vendor.calls.append(('close', ))
+        self.vendor.read_from.close()
+        self.vendor.write_to.close()
 
-    def get_filelike_channels(self):
-        return self.vendor.read_from, self.vendor.write_to
+    def get_sock_or_pipes(self):
+        return 'pipes', (self.vendor.read_from, self.vendor.write_to)
 
 
 class _InvalidHostnameFeature(tests.Feature):
@@ -243,9 +249,9 @@
         unopened_port = sock.getsockname()[1]
         # having vendor be invalid means that if it tries to connect via the
         # vendor it will blow up.
-        client_medium = medium.SmartSSHClientMedium('127.0.0.1', unopened_port,
-            username=None, password=None, base='base', vendor="not a vendor",
-            bzr_remote_path='bzr')
+        ssh_params = medium.SSHParams('127.0.0.1', unopened_port, None, None)
+        client_medium = medium.SmartSSHClientMedium(
+            'base', ssh_params, "not a vendor")
         sock.close()
 
     def test_ssh_client_connects_on_first_use(self):
@@ -253,9 +259,9 @@
         # it bytes.
         output = StringIO()
         vendor = StringIOSSHVendor(StringIO(), output)
-        client_medium = medium.SmartSSHClientMedium(
-            'a hostname', 'a port', 'a username', 'a password', 'base', vendor,
-            'bzr')
+        ssh_params = medium.SSHParams(
+            'a hostname', 'a port', 'a username', 'a password', 'bzr')
+        client_medium = medium.SmartSSHClientMedium('base', ssh_params, vendor)
         client_medium._accept_bytes('abc')
         self.assertEqual('abc', output.getvalue())
         self.assertEqual([('connect_ssh', 'a username', 'a password',
@@ -268,8 +274,10 @@
         # it bytes.
         output = StringIO()
         vendor = StringIOSSHVendor(StringIO(), output)
-        client_medium = medium.SmartSSHClientMedium('a hostname', 'a port',
-            'a username', 'a password', 'base', vendor, bzr_remote_path='fugly')
+        ssh_params = medium.SSHParams(
+            'a hostname', 'a port', 'a username', 'a password',
+            bzr_remote_path='fugly')
+        client_medium = medium.SmartSSHClientMedium('base', ssh_params, vendor)
         client_medium._accept_bytes('abc')
         self.assertEqual('abc', output.getvalue())
         self.assertEqual([('connect_ssh', 'a username', 'a password',
@@ -284,7 +292,7 @@
         output = StringIO()
         vendor = StringIOSSHVendor(input, output)
         client_medium = medium.SmartSSHClientMedium(
-            'a hostname', base='base', vendor=vendor, bzr_remote_path='bzr')
+            'base', medium.SSHParams('a hostname'), vendor)
         client_medium._accept_bytes('abc')
         client_medium.disconnect()
         self.assertTrue(input.closed)
@@ -305,7 +313,7 @@
         output = StringIO()
         vendor = StringIOSSHVendor(input, output)
         client_medium = medium.SmartSSHClientMedium(
-            'a hostname', base='base', vendor=vendor, bzr_remote_path='bzr')
+            'base', medium.SSHParams('a hostname'), vendor)
         client_medium._accept_bytes('abc')
         client_medium.disconnect()
         # the disconnect has closed output, so we need a new output for the
@@ -334,14 +342,14 @@
         # Doing a disconnect on a new (and thus unconnected) SSH medium
         # does not fail.  It's ok to disconnect an unconnected medium.
         client_medium = medium.SmartSSHClientMedium(
-            None, base='base', bzr_remote_path='bzr')
+            'base', medium.SSHParams(None))
         client_medium.disconnect()
 
     def test_ssh_client_raises_on_read_when_not_connected(self):
         # Doing a read on a new (and thus unconnected) SSH medium raises
         # MediumNotConnected.
         client_medium = medium.SmartSSHClientMedium(
-            None, base='base', bzr_remote_path='bzr')
+            'base', medium.SSHParams(None))
         self.assertRaises(errors.MediumNotConnected, client_medium.read_bytes,
                           0)
         self.assertRaises(errors.MediumNotConnected, client_medium.read_bytes,
@@ -359,7 +367,7 @@
         output.flush = logging_flush
         vendor = StringIOSSHVendor(input, output)
         client_medium = medium.SmartSSHClientMedium(
-            'a hostname', base='base', vendor=vendor, bzr_remote_path='bzr')
+            'base', medium.SSHParams('a hostname'), vendor=vendor)
         # this call is here to ensure we only flush once, not on every
         # _accept_bytes call.
         client_medium._accept_bytes('abc')
@@ -967,24 +975,27 @@
             base = 'a_url'
             def external_url(self):
                 return self.base
-            def get_bytes(self, path):
+            def get(self, path):
                 raise Exception("some random exception from inside server")
-        smart_server = server.SmartTCPServer(backing_transport=FlakyTransport())
-        smart_server.start_background_thread('-' + self.id())
-        try:
-            transport = remote.RemoteTCPTransport(smart_server.get_url())
-            err = self.assertRaises(errors.UnknownErrorFromSmartServer,
-                transport.get, 'something')
-            self.assertContainsRe(str(err), 'some random exception')
-            transport.disconnect()
-        finally:
-            smart_server.stop_background_thread()
+
+        class FlakyServer(test_server.SmartTCPServer_for_testing):
+            def get_backing_transport(self, backing_transport_server):
+                return FlakyTransport()
+
+        smart_server = FlakyServer()
+        smart_server.start_server()
+        self.addCleanup(smart_server.stop_server)
+        t = remote.RemoteTCPTransport(smart_server.get_url())
+        self.addCleanup(t.disconnect)
+        err = self.assertRaises(errors.UnknownErrorFromSmartServer,
+                                t.get, 'something')
+        self.assertContainsRe(str(err), 'some random exception')
 
 
 class SmartTCPTests(tests.TestCase):
     """Tests for connection/end to end behaviour using the TCP server.
 
-    All of these tests are run with a server running on another thread serving
+    All of these tests are run with a server running in another thread serving
     a MemoryTransport, and a connection to it already open.
 
     the server is obtained by calling self.start_server(readonly=False).
@@ -998,7 +1009,7 @@
         # NB: Tests using this fall into two categories: tests of the server,
         # tests wanting a server. The latter should be updated to use
         # self.vfs_transport_factory etc.
-        if not backing_transport:
+        if backing_transport is None:
             mem_server = memory.MemoryServer()
             mem_server.start_server()
             self.addCleanup(mem_server.stop_server)
@@ -1012,41 +1023,45 @@
             self.backing_transport = transport.get_transport(
                 "readonly+" + self.backing_transport.abspath('.'))
         self.server = server.SmartTCPServer(self.backing_transport)
+        self.server.start_server('127.0.0.1', 0)
         self.server.start_background_thread('-' + self.id())
         self.transport = remote.RemoteTCPTransport(self.server.get_url())
-        self.addCleanup(self.tearDownServer)
+        self.addCleanup(self.stop_server)
         self.permit_url(self.server.get_url())
 
-    def tearDownServer(self):
+    def stop_server(self):
+        """Disconnect the client and stop the server.
+
+        This must be re-entrant as some tests will call it explicitly in
+        addition to the normal cleanup.
+        """
         if getattr(self, 'transport', None):
             self.transport.disconnect()
             del self.transport
         if getattr(self, 'server', None):
             self.server.stop_background_thread()
-            # XXX: why not .stop_server() -- mbp 20100106
             del self.server
 
 
 class TestServerSocketUsage(SmartTCPTests):
 
-    def test_server_setup_teardown(self):
-        """It should be safe to teardown the server with no requests."""
+    def test_server_start_stop(self):
+        """It should be safe to stop the server with no requests."""
         self.start_server()
-        server = self.server
-        transport = remote.RemoteTCPTransport(self.server.get_url())
-        self.tearDownServer()
-        self.assertRaises(errors.ConnectionError, transport.has, '.')
+        t = remote.RemoteTCPTransport(self.server.get_url())
+        self.stop_server()
+        self.assertRaises(errors.ConnectionError, t.has, '.')
 
     def test_server_closes_listening_sock_on_shutdown_after_request(self):
         """The server should close its listening socket when it's stopped."""
         self.start_server()
-        server = self.server
+        server_url = self.server.get_url()
         self.transport.has('.')
-        self.tearDownServer()
+        self.stop_server()
         # if the listening socket has closed, we should get a BADFD error
         # when connecting, rather than a hang.
-        transport = remote.RemoteTCPTransport(server.get_url())
-        self.assertRaises(errors.ConnectionError, transport.has, '.')
+        t = remote.RemoteTCPTransport(server_url)
+        self.assertRaises(errors.ConnectionError, t.has, '.')
 
 
 class WritableEndToEndTests(SmartTCPTests):
@@ -1187,7 +1202,7 @@
         self.transport.has('.')
         self.assertEqual([], self.hook_calls)
         # clean up the server
-        self.tearDownServer()
+        self.stop_server()
         # now it should have fired.
         self.assertEqual(result, self.hook_calls)
 
@@ -1206,7 +1221,7 @@
         self.transport.has('.')
         self.assertEqual([], self.hook_calls)
         # clean up the server
-        self.tearDownServer()
+        self.stop_server()
         # now it should have fired.
         self.assertEqual(result, self.hook_calls)
 
@@ -2859,9 +2874,10 @@
         self.responder = protocol.ProtocolThreeResponder(self.writes.append)
 
     def assertWriteCount(self, expected_count):
+        # self.writes can be quite large; don't show the whole thing
         self.assertEqual(
             expected_count, len(self.writes),
-            "Too many writes: %r" % (self.writes,))
+            "Too many writes: %d, expected %d" % (len(self.writes), expected_count))
 
     def test_send_error_writes_just_once(self):
         """An error response is written to the medium all at once."""
@@ -2890,22 +2906,9 @@
         response = _mod_request.SuccessfulSmartServerResponse(
             ('arg', 'arg'), body_stream=['chunk1', 'chunk2'])
         self.responder.send_response(response)
-        # We will write just once, despite the multiple chunks, due to
-        # buffering.
-        self.assertWriteCount(1)
-
-    def test_send_response_with_body_stream_flushes_buffers_sometimes(self):
-        """When there are many bytes (>1MB), multiple writes will occur rather
-        than buffering indefinitely.
-        """
-        # Construct a response with stream with ~1.5MB in it. This should
-        # trigger 2 writes, but not 3
-        onekib = '12345678' * 128
-        body_stream = [onekib] * (1024 + 512)
-        response = _mod_request.SuccessfulSmartServerResponse(
-            ('arg', 'arg'), body_stream=body_stream)
-        self.responder.send_response(response)
-        self.assertWriteCount(2)
+        # Per the discussion in bug 590638 we flush once after the header and
+        # then once after each chunk
+        self.assertWriteCount(3)
 
 
 class TestSmartClientUnicode(tests.TestCase):

=== modified file 'bzrlib/tests/test_smtp_connection.py'
--- bzrlib/tests/test_smtp_connection.py	2009-07-19 01:36:56 +0000
+++ bzrlib/tests/test_smtp_connection.py	2010-08-30 07:34:11 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2007, 2009 Canonical Ltd
+# Copyright (C) 2007, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -91,9 +91,7 @@
 class TestSMTPConnection(tests.TestCaseInTempDir):
 
     def get_connection(self, text, smtp_factory=None):
-        my_config = config.GlobalConfig()
-        config_file = StringIO(text)
-        my_config._get_parser(config_file)
+        my_config = config.GlobalConfig.from_string(text)
         return smtp_connection.SMTPConnection(my_config,
                                               _smtp_factory=smtp_factory)
 
@@ -160,17 +158,18 @@
 
     def test_authenticate_with_byte_strings(self):
         user = 'joe'
-        password = 'h\xC3\xACspass'
+        unicode_pass = u'h\xECspass'
+        utf8_pass = unicode_pass.encode('utf-8')
         factory = WideOpenSMTPFactory()
         conn = self.get_connection(
-            '[DEFAULT]\nsmtp_username=%s\nsmtp_password=%s\n'
-            % (user, password), smtp_factory=factory)
-        self.assertEqual(u'h\xECspass', conn._smtp_password)
+            u'[DEFAULT]\nsmtp_username=%s\nsmtp_password=%s\n'
+            % (user, unicode_pass), smtp_factory=factory)
+        self.assertEqual(unicode_pass, conn._smtp_password)
         conn._connect()
         self.assertEqual([('connect', 'localhost'),
                           ('ehlo',),
                           ('has_extn', 'starttls'),
-                          ('login', user, password)], factory._calls)
+                          ('login', user, utf8_pass)], factory._calls)
         smtp_username, smtp_password = factory._calls[-1][1:]
         self.assertIsInstance(smtp_username, str)
         self.assertIsInstance(smtp_password, str)

=== modified file 'bzrlib/tests/test_source.py'
--- bzrlib/tests/test_source.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test_source.py	2010-08-24 19:21:32 +0000
@@ -40,11 +40,17 @@
 
 # Files which are listed here will be skipped when testing for Copyright (or
 # GPL) statements.
-COPYRIGHT_EXCEPTIONS = ['bzrlib/lsprof.py', 'bzrlib/_bencode_py.py',
-    'bzrlib/doc_generate/sphinx_conf.py']
+COPYRIGHT_EXCEPTIONS = [
+    'bzrlib/_bencode_py.py',
+    'bzrlib/doc_generate/conf.py',
+    'bzrlib/lsprof.py',
+    ]
 
-LICENSE_EXCEPTIONS = ['bzrlib/lsprof.py', 'bzrlib/_bencode_py.py',
-    'bzrlib/doc_generate/sphinx_conf.py']
+LICENSE_EXCEPTIONS = [
+    'bzrlib/_bencode_py.py',
+    'bzrlib/doc_generate/conf.py',
+    'bzrlib/lsprof.py',
+    ]
 # Technically, 'bzrlib/lsprof.py' should be 'bzrlib/util/lsprof.py',
 # (we do not check bzrlib/util/, since that is code bundled from elsewhere)
 # but for compatibility with previous releases, we don't want to move it.
@@ -382,8 +388,10 @@
         """
         both_exc_and_no_exc = []
         missing_except = []
-        class_re = re.compile(r'^(cdef\s+)?(public\s+)?(api\s+)?class (\w+).*:',
-                              re.MULTILINE)
+        class_re = re.compile(r'^(cdef\s+)?(public\s+)?'
+                              r'(api\s+)?class (\w+).*:', re.MULTILINE)
+        extern_class_re = re.compile(r'## extern cdef class (\w+)',
+                                     re.MULTILINE)
         except_re = re.compile(r'cdef\s+' # start with cdef
                                r'([\w *]*?)\s*' # this is the return signature
                                r'(\w+)\s*\(' # the function name
@@ -394,6 +402,7 @@
         for fname, text in self.get_source_file_contents(
                 extensions=('.pyx',)):
             known_classes = set([m[-1] for m in class_re.findall(text)])
+            known_classes.update(extern_class_re.findall(text))
             cdefs = except_re.findall(text)
             for sig, func, exc_clause, no_exc_comment in cdefs:
                 if sig.startswith('api '):

=== modified file 'bzrlib/tests/test_ssh_transport.py'
--- bzrlib/tests/test_ssh_transport.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test_ssh_transport.py	2010-10-07 12:45:51 +0000
@@ -20,6 +20,7 @@
     OpenSSHSubprocessVendor,
     PLinkSubprocessVendor,
     SSHCorpSubprocessVendor,
+    LSHSubprocessVendor,
     SSHVendorManager,
     )
 
@@ -72,6 +73,12 @@
         manager.set_ssh_version_string("SSH Secure Shell")
         self.assertIsInstance(manager.get_vendor({}), SSHCorpSubprocessVendor)
 
+    def test_get_vendor_by_inspection_lsh(self):
+        manager = TestSSHVendorManager()
+        self.assertRaises(SSHVendorNotFound, manager.get_vendor, {})
+        manager.set_ssh_version_string("lsh")
+        self.assertIsInstance(manager.get_vendor({}), LSHSubprocessVendor)
+
     def test_get_vendor_by_inspection_plink(self):
         manager = TestSSHVendorManager()
         self.assertRaises(SSHVendorNotFound, manager.get_vendor, {})
@@ -160,7 +167,7 @@
             vendor._get_vendor_specific_argv(
                 "user", "host", 100, command=["bzr"]),
             ["ssh", "-oForwardX11=no", "-oForwardAgent=no",
-                "-oClearAllForwardings=yes", "-oProtocol=2",
+                "-oClearAllForwardings=yes",
                 "-oNoHostAuthenticationForLocalhost=yes",
                 "-p", "100",
                 "-l", "user",
@@ -173,7 +180,7 @@
             vendor._get_vendor_specific_argv(
                 "user", "host", 100, subsystem="sftp"),
             ["ssh", "-oForwardX11=no", "-oForwardAgent=no",
-                "-oClearAllForwardings=yes", "-oProtocol=2",
+                "-oClearAllForwardings=yes",
                 "-oNoHostAuthenticationForLocalhost=yes",
                 "-p", "100",
                 "-l", "user",
@@ -202,6 +209,28 @@
                 "-s", "sftp", "host"]
             )
 
+    def test_lsh_command_arguments(self):
+        vendor = LSHSubprocessVendor()
+        self.assertEqual(
+            vendor._get_vendor_specific_argv(
+                "user", "host", 100, command=["bzr"]),
+            ["lsh",
+                "-p", "100",
+                "-l", "user",
+                "host", "bzr"]
+            )
+
+    def test_lsh_subsystem_arguments(self):
+        vendor = LSHSubprocessVendor()
+        self.assertEqual(
+            vendor._get_vendor_specific_argv(
+                "user", "host", 100, subsystem="sftp"),
+            ["lsh",
+                "-p", "100",
+                "-l", "user",
+                "--subsystem", "sftp", "host"]
+            )
+
     def test_plink_command_arguments(self):
         vendor = PLinkSubprocessVendor()
         self.assertEqual(

=== modified file 'bzrlib/tests/test_status.py'
--- bzrlib/tests/test_status.py	2009-10-06 14:40:37 +0000
+++ bzrlib/tests/test_status.py	2010-09-25 20:08:01 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +17,10 @@
 
 from StringIO import StringIO
 
-from bzrlib import config
+from bzrlib import (
+    config,
+    status as _mod_status,
+    )
 from bzrlib.revisionspec import RevisionSpec
 from bzrlib.status import show_pending_merges, show_tree_status
 from bzrlib.tests import TestCaseWithTransport
@@ -130,3 +133,67 @@
                      revision=[RevisionSpec.from_string("revid:%s" % r1_id),
                                RevisionSpec.from_string("revid:%s" % r2_id)])
         # return does not matter as long as it did not raise.
+
+
+class TestHooks(TestCaseWithTransport):
+
+    def test_constructor(self):
+        """Check that creating a StatusHooks instance has the right defaults.
+        """
+        hooks = _mod_status.StatusHooks()
+        self.assertTrue("post_status" in hooks, "post_status not in %s" % hooks)
+        self.assertTrue("pre_status" in hooks, "pre_status not in %s" % hooks)
+
+    def test_installed_hooks_are_StatusHooks(self):
+        """The installed hooks object should be a StatusHooks.
+        """
+        # the installed hooks are saved in self._preserved_hooks.
+        self.assertIsInstance(self._preserved_hooks[_mod_status][1],
+            _mod_status.StatusHooks)
+
+    def test_post_status_hook(self):
+        """Ensure that post_status hook is invoked with the right args.
+        """
+        calls = []
+        _mod_status.hooks.install_named_hook('post_status', calls.append, None)
+        self.assertLength(0, calls)
+        tree = self.make_branch_and_tree('.')
+        r1_id = tree.commit('one', allow_pointless=True)
+        r2_id = tree.commit('two', allow_pointless=True)
+        r2_tree = tree.branch.repository.revision_tree(r2_id)
+        output = StringIO()
+        show_tree_status(tree, to_file=output,
+            revision=[RevisionSpec.from_string("revid:%s" % r1_id),
+                RevisionSpec.from_string("revid:%s" % r2_id)])
+        self.assertLength(1, calls)
+        params = calls[0]
+        self.assertIsInstance(params, _mod_status.StatusHookParams)
+        attrs = ['old_tree', 'new_tree', 'to_file', 'versioned',
+            'show_ids', 'short', 'verbose']
+        for a in attrs:
+            self.assertTrue(hasattr(params, a),
+                'Attribute "%s" not found in StatusHookParam' % a)
+
+    def test_pre_status_hook(self):
+        """Ensure that pre_status hook is invoked with the right args.
+        """
+        calls = []
+        _mod_status.hooks.install_named_hook('pre_status', calls.append, None)
+        self.assertLength(0, calls)
+        tree = self.make_branch_and_tree('.')
+        r1_id = tree.commit('one', allow_pointless=True)
+        r2_id = tree.commit('two', allow_pointless=True)
+        r2_tree = tree.branch.repository.revision_tree(r2_id)
+        output = StringIO()
+        show_tree_status(tree, to_file=output,
+            revision=[RevisionSpec.from_string("revid:%s" % r1_id),
+                RevisionSpec.from_string("revid:%s" % r2_id)])
+        self.assertLength(1, calls)
+        params = calls[0]
+        self.assertIsInstance(params, _mod_status.StatusHookParams)
+        attrs = ['old_tree', 'new_tree', 'to_file', 'versioned',
+            'show_ids', 'short', 'verbose']
+        for a in attrs:
+            self.assertTrue(hasattr(params, a),
+                'Attribute "%s" not found in StatusHookParam' % a)
+

=== modified file 'bzrlib/tests/test_strace.py'
--- bzrlib/tests/test_strace.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_strace.py	2010-09-25 20:08:01 +0000
@@ -1,5 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
-#   Authors: Robert Collins <robert.collins@canonical.com>
+# Copyright (C) 2007-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,34 +21,26 @@
 import threading
 
 from bzrlib import (
+    strace,
     tests,
     )
 from bzrlib.strace import StraceFeature, strace_detailed, StraceResult
 
 
-class TestStraceFeature(tests.TestCaseWithTransport):
-
-    def test_strace_detection(self):
-        """Strace is available if its runnable."""
-        try:
-            proc = subprocess.Popen(['strace'],
-                stderr=subprocess.PIPE,
-                stdout=subprocess.PIPE)
-            proc.communicate()
-            found_strace = True
-        except OSError, e:
-            if e.errno == errno.ENOENT:
-                # strace is not installed
-                found_strace = False
-            else:
-                raise
-        self.assertEqual(found_strace, StraceFeature.available())
-
-
 class TestStrace(tests.TestCaseWithTransport):
 
     _test_needs_features = [StraceFeature]
 
+    def setUp(self):
+        # NB: see http://pad.lv/626679 and
+        # <https://code.launchpad.net/~mbp/bzr/626679-strace/+merge/34157>:
+        # testing strace by connecting to ourselves has repeatedly caused
+        # hangs in running the test suite; these are fixable given enough
+        # determination but given that strace is not used by any other tests
+        # at the moment and that it's only test-support code, we just leave it
+        # untested -- mbp 20100901
+        raise tests.TestSkipped("strace selftests are broken and disabled")
+
     def _check_threads(self):
         # For bug #226769, it was decided that the strace tests should not be
         # run when more than one thread is active. A lot of tests are currently
@@ -61,14 +52,26 @@
             raise tests.KnownFailure(
                 '%d active threads, bug #103133 needs to be fixed.' % active)
 
+    def strace_detailed_or_skip(self, *args, **kwargs):
+        """Run strace, but cope if it's not allowed"""
+        try:
+            return strace_detailed(*args, **kwargs)
+        except strace.StraceError, e:
+            if e.err_messages.startswith(
+                    "attach: ptrace(PTRACE_ATTACH, ...): Operation not permitted"):
+                raise tests.TestSkipped("ptrace not permitted")
+            else:
+                raise
+
     def test_strace_callable_is_called(self):
         self._check_threads()
 
         output = []
         def function(positional, *args, **kwargs):
             output.append((positional, args, kwargs))
-        strace_detailed(function, ["a", "b"], {"c": "c"},
-                        follow_children=False)
+        self.strace_detailed_or_skip(
+            function, ["a", "b"], {"c": "c"},
+            follow_children=False)
         self.assertEqual([("a", ("b",), {"c":"c"})], output)
 
     def test_strace_callable_result(self):
@@ -76,7 +79,7 @@
 
         def function():
             return "foo"
-        result, strace_result = strace_detailed(function,[], {},
+        result, strace_result = self.strace_detailed_or_skip(function,[], {},
                                                 follow_children=False)
         self.assertEqual("foo", result)
         self.assertIsInstance(strace_result, StraceResult)
@@ -87,6 +90,6 @@
 
         def function():
             self.build_tree(['myfile'])
-        unused, result = strace_detailed(function, [], {},
+        unused, result = self.strace_detailed_or_skip(function, [], {},
                                          follow_children=False)
         self.assertContainsRe(result.raw_log, 'myfile')

=== modified file 'bzrlib/tests/test_symbol_versioning.py'
--- bzrlib/tests/test_symbol_versioning.py	2010-04-23 08:51:52 +0000
+++ bzrlib/tests/test_symbol_versioning.py	2010-06-25 06:11:21 +0000
@@ -241,6 +241,11 @@
         symbol_versioning.suppress_deprecation_warnings()
         self.assertFirstWarning('ignore', DeprecationWarning)
 
+    def test_set_restore_filters(self):
+        original_filters = warnings.filters[:]
+        symbol_versioning.suppress_deprecation_warnings()()
+        self.assertEqual(original_filters, warnings.filters)
+
     def test_suppress_deprecation_with_warning_filter(self):
         """don't suppress if we already have a filter"""
         warnings.filterwarnings('error', category=Warning)

=== added file 'bzrlib/tests/test_test_server.py'
--- bzrlib/tests/test_test_server.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_test_server.py	2010-06-30 16:24:01 +0000
@@ -0,0 +1,263 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+import errno
+import socket
+import SocketServer
+import threading
+
+from bzrlib import (
+    osutils,
+    tests,
+    )
+from bzrlib.tests import test_server
+
+
+def load_tests(basic_tests, module, loader):
+    suite = loader.suiteClass()
+    server_tests, remaining_tests = tests.split_suite_by_condition(
+        basic_tests, tests.condition_isinstance(TestTCPServerInAThread))
+    server_scenarios = [ (name, {'server_class': getattr(test_server, name)})
+                         for name in
+                         ('TestingTCPServer', 'TestingThreadingTCPServer')]
+    tests.multiply_tests(server_tests, server_scenarios, suite)
+    suite.addTest(remaining_tests)
+    return suite
+
+
+class TestThreadWithException(tests.TestCase):
+
+    def test_start_and_join_smoke_test(self):
+        def do_nothing():
+            pass
+
+        tt = test_server.ThreadWithException(target=do_nothing)
+        tt.start()
+        tt.join()
+
+    def test_exception_is_re_raised(self):
+        class MyException(Exception):
+            pass
+
+        def raise_my_exception():
+            raise MyException()
+
+        tt = test_server.ThreadWithException(target=raise_my_exception)
+        tt.start()
+        self.assertRaises(MyException, tt.join)
+
+    def test_join_when_no_exception(self):
+        resume = threading.Event()
+        class MyException(Exception):
+            pass
+
+        def raise_my_exception():
+            # Wait for the test to tell us to resume
+            resume.wait()
+            # Now we can raise
+            raise MyException()
+
+        tt = test_server.ThreadWithException(target=raise_my_exception)
+        tt.start()
+        tt.join(timeout=0)
+        self.assertIs(None, tt.exception)
+        resume.set()
+        self.assertRaises(MyException, tt.join)
+
+
+class TCPClient(object):
+
+    def __init__(self):
+        self.sock = None
+
+    def connect(self, addr):
+        if self.sock is not None:
+            raise AssertionError('Already connected to %r'
+                                 % (self.sock.getsockname(),))
+        self.sock = osutils.connect_socket(addr)
+
+    def disconnect(self):
+        if self.sock is not None:
+            try:
+                self.sock.shutdown(socket.SHUT_RDWR)
+                self.sock.close()
+            except socket.error, e:
+                if e[0] in (errno.EBADF, errno.ENOTCONN):
+                    # Right, the socket is already down
+                    pass
+                else:
+                    raise
+            self.sock = None
+
+    def write(self, s):
+        return self.sock.sendall(s)
+
+    def read(self, bufsize=4096):
+        return self.sock.recv(bufsize)
+
+
+class TCPConnectionHandler(SocketServer.StreamRequestHandler):
+
+    def handle(self):
+        self.done = False
+        self.handle_connection()
+        while not self.done:
+            self.handle_connection()
+
+    def handle_connection(self):
+        req = self.rfile.readline()
+        if not req:
+            self.done = True
+        elif req == 'ping\n':
+            self.wfile.write('pong\n')
+        else:
+            raise ValueError('[%s] not understood' % req)
+
+
+class TestTCPServerInAThread(tests.TestCase):
+
+    # Set by load_tests()
+    server_class = None
+
+    def get_server(self, server_class=None, connection_handler_class=None):
+        if server_class is not None:
+            self.server_class = server_class
+        if connection_handler_class is None:
+            connection_handler_class = TCPConnectionHandler
+        server =  test_server.TestingTCPServerInAThread(
+            ('localhost', 0), self.server_class, connection_handler_class)
+        server.start_server()
+        self.addCleanup(server.stop_server)
+        return server
+
+    def get_client(self):
+        client = TCPClient()
+        self.addCleanup(client.disconnect)
+        return client
+
+    def get_server_connection(self, server, conn_rank):
+        return server.server.clients[conn_rank]
+
+    def assertClientAddr(self, client, server, conn_rank):
+        conn = self.get_server_connection(server, conn_rank)
+        self.assertEquals(client.sock.getsockname(), conn[1])
+
+    def test_start_stop(self):
+        server = self.get_server()
+        client = self.get_client()
+        server.stop_server()
+        # since the server doesn't accept connections anymore attempting to
+        # connect should fail
+        client = self.get_client()
+        self.assertRaises(socket.error,
+                          client.connect, (server.host, server.port))
+
+    def test_client_talks_server_respond(self):
+        server = self.get_server()
+        client = self.get_client()
+        client.connect((server.host, server.port))
+        self.assertIs(None, client.write('ping\n'))
+        resp = client.read()
+        self.assertClientAddr(client, server, 0)
+        self.assertEquals('pong\n', resp)
+
+    def test_server_fails_to_start(self):
+        class CantStart(Exception):
+            pass
+
+        class CantStartServer(test_server.TestingTCPServer):
+
+            def server_bind(self):
+                raise CantStart()
+
+        # The exception is raised in the main thread
+        self.assertRaises(CantStart,
+                          self.get_server, server_class=CantStartServer)
+
+    def test_server_fails_while_serving_or_stopping(self):
+        class CantConnect(Exception):
+            pass
+
+        class FailingConnectionHandler(TCPConnectionHandler):
+
+            def handle(self):
+                raise CantConnect()
+
+        server = self.get_server(
+            connection_handler_class=FailingConnectionHandler)
+        # The server won't fail until a client connect
+        client = self.get_client()
+        client.connect((server.host, server.port))
+        try:
+            # Now we must force the server to answer by sending the request and
+            # waiting for some answer. But since we don't control when the
+            # server thread will be given cycles, we don't control either
+            # whether our reads or writes may hang.
+            client.sock.settimeout(0.1)
+            client.write('ping\n')
+            client.read()
+        except socket.error:
+            pass
+        # Now the server has raised the exception in its own thread
+        self.assertRaises(CantConnect, server.stop_server)
+
+    def test_server_crash_while_responding(self):
+        sync = threading.Event()
+        sync.clear()
+        class FailToRespond(Exception):
+            pass
+
+        class FailingDuringResponseHandler(TCPConnectionHandler):
+
+            def handle_connection(self):
+                req = self.rfile.readline()
+                threading.currentThread().set_ready_event(sync)
+                raise FailToRespond()
+
+        server = self.get_server(
+            connection_handler_class=FailingDuringResponseHandler)
+        client = self.get_client()
+        client.connect((server.host, server.port))
+        client.write('ping\n')
+        sync.wait()
+        self.assertRaises(FailToRespond, server.pending_exception)
+
+    def test_exception_swallowed_while_serving(self):
+        sync = threading.Event()
+        sync.clear()
+        class CantServe(Exception):
+            pass
+
+        class FailingWhileServingConnectionHandler(TCPConnectionHandler):
+
+            def handle(self):
+                # We want to sync with the thread that is serving the
+                # connection.
+                threading.currentThread().set_ready_event(sync)
+                raise CantServe()
+
+        server = self.get_server(
+            connection_handler_class=FailingWhileServingConnectionHandler)
+        # Install the exception swallower
+        server.set_ignored_exceptions(CantServe)
+        client = self.get_client()
+        # Connect to the server so the exception is raised there
+        client.connect((server.host, server.port))
+        # Wait for the exception to propagate.
+        sync.wait()
+        # The connection wasn't served properly but the exception should have
+        # been swallowed.
+        server.pending_exception()

=== modified file 'bzrlib/tests/test_trace.py'
--- bzrlib/tests/test_trace.py	2010-02-25 13:15:08 +0000
+++ bzrlib/tests/test_trace.py	2010-06-26 01:07:16 +0000
@@ -29,7 +29,7 @@
     errors,
     trace,
     )
-from bzrlib.tests import TestCaseInTempDir, TestCase
+from bzrlib.tests import features, TestCaseInTempDir, TestCase
 from bzrlib.trace import (
     mutter, mutter_callsite, report_exception,
     set_verbosity_level, get_verbosity_level, is_quiet, is_verbose, be_quiet,
@@ -104,6 +104,19 @@
         self.assertContainsRe(msg,
             r'^bzr: ERROR: \[Errno .*\] No such file.*nosuchfile')
 
+    def test_format_pywintypes_error(self):
+        self.requireFeature(features.pywintypes)
+        import pywintypes, win32file
+        try:
+            win32file.RemoveDirectory('nosuchfile22222')
+        except pywintypes.error:
+            pass
+        msg = _format_exception()
+        # GZ 2010-05-03: Formatting for pywintypes.error is basic, a 3-tuple
+        #                with errno, function name, and locale error message
+        self.assertContainsRe(msg,
+            r"^bzr: ERROR: \(2, 'RemoveDirectory[AW]?', .*\)")
+
     def test_format_unicode_error(self):
         try:
             raise errors.BzrCommandError(u'argument foo\xb5 does not exist')
@@ -320,3 +333,21 @@
         _rollover_trace_maybe(temp_log_name)
         # should have been rolled over
         self.assertFalse(os.access(temp_log_name, os.R_OK))
+
+
+class TestTraceConfiguration(TestCaseInTempDir):
+
+    def test_default_config(self):
+        config = trace.DefaultConfig()
+        self.overrideAttr(trace, "_bzr_log_filename", None)
+        trace._bzr_log_filename = None
+        expected_filename = trace._get_bzr_log_filename()
+        self.assertEqual(None, trace._bzr_log_filename)
+        config.__enter__()
+        try:
+            # Should have entered and setup a default filename.
+            self.assertEqual(expected_filename, trace._bzr_log_filename)
+        finally:
+            config.__exit__(None, None, None)
+            # Should have exited and cleaned up.
+            self.assertEqual(None, trace._bzr_log_filename)

=== modified file 'bzrlib/tests/test_transform.py'
--- bzrlib/tests/test_transform.py	2010-03-12 10:52:01 +0000
+++ bzrlib/tests/test_transform.py	2010-08-25 10:20:41 +0000
@@ -25,23 +25,37 @@
     filters,
     generate_ids,
     osutils,
-    progress,
     revision as _mod_revision,
     rules,
     tests,
     urlutils,
     )
 from bzrlib.bzrdir import BzrDir
-from bzrlib.conflicts import (DuplicateEntry, DuplicateID, MissingParent,
-                              UnversionedParent, ParentLoop, DeletingParent,
-                              NonDirectoryParent)
+from bzrlib.conflicts import (
+    DeletingParent,
+    DuplicateEntry,
+    DuplicateID,
+    MissingParent,
+    NonDirectoryParent,
+    ParentLoop,
+    UnversionedParent,
+)
 from bzrlib.diff import show_diff_trees
-from bzrlib.errors import (DuplicateKey, MalformedTransform, NoSuchFile,
-                           ReusingTransform, CantMoveRoot,
-                           PathsNotVersionedError, ExistingLimbo,
-                           ExistingPendingDeletion, ImmortalLimbo,
-                           ImmortalPendingDeletion, LockError)
-from bzrlib.osutils import file_kind, pathjoin
+from bzrlib.errors import (
+    DuplicateKey,
+    ExistingLimbo,
+    ExistingPendingDeletion,
+    ImmortalLimbo,
+    ImmortalPendingDeletion,
+    LockError,
+    MalformedTransform,
+    NoSuchFile,
+    ReusingTransform,
+)
+from bzrlib.osutils import (
+    file_kind,
+    pathjoin,
+)
 from bzrlib.merge import Merge3Merger, Merger
 from bzrlib.tests import (
     HardlinkFeature,
@@ -49,12 +63,20 @@
     TestCase,
     TestCaseInTempDir,
     TestSkipped,
-    )
-from bzrlib.transform import (TreeTransform, ROOT_PARENT, FinalPaths,
-                              resolve_conflicts, cook_conflicts,
-                              build_tree, get_backup_name,
-                              _FileMover, resolve_checkout,
-                              TransformPreview, create_from_tree)
+)
+from bzrlib.transform import (
+    build_tree,
+    create_from_tree,
+    cook_conflicts,
+    _FileMover,
+    FinalPaths,
+    get_backup_name,
+    resolve_conflicts,
+    resolve_checkout,
+    ROOT_PARENT,
+    TransformPreview,
+    TreeTransform,
+)
 
 
 class TestTreeTransform(tests.TestCaseWithTransport):
@@ -101,12 +123,12 @@
         imaginary_id = transform.trans_id_tree_path('imaginary')
         imaginary_id2 = transform.trans_id_tree_path('imaginary/')
         self.assertEqual(imaginary_id, imaginary_id2)
-        self.assertEqual(transform.get_tree_parent(imaginary_id), root)
-        self.assertEqual(transform.final_kind(root), 'directory')
-        self.assertEqual(transform.final_file_id(root), self.wt.get_root_id())
+        self.assertEqual(root, transform.get_tree_parent(imaginary_id))
+        self.assertEqual('directory', transform.final_kind(root))
+        self.assertEqual(self.wt.get_root_id(), transform.final_file_id(root))
         trans_id = transform.create_path('name', root)
         self.assertIs(transform.final_file_id(trans_id), None)
-        self.assertRaises(NoSuchFile, transform.final_kind, trans_id)
+        self.assertIs(None, transform.final_kind(trans_id))
         transform.create_file('contents', trans_id)
         transform.set_executability(True, trans_id)
         transform.version_file('my_pretties', trans_id)
@@ -830,6 +852,38 @@
         rename.set_executability(True, myfile)
         rename.apply()
 
+    def test_rename_fails(self):
+        # see https://bugs.launchpad.net/bzr/+bug/491763
+        create, root_id = self.get_transform()
+        first_dir = create.new_directory('first-dir', root_id, 'first-id')
+        myfile = create.new_file('myfile', root_id, 'myfile-text',
+                                 'myfile-id')
+        create.apply()
+        if os.name == "posix" and sys.platform != "cygwin":
+            # posix filesystems fail on renaming if the readonly bit is set
+            osutils.make_readonly(self.wt.abspath('first-dir'))
+        elif os.name == "nt":
+            # windows filesystems fail on renaming open files
+            self.addCleanup(file(self.wt.abspath('myfile')).close)
+        else:
+            self.skip("Don't know how to force a permissions error on rename")
+        # now transform to rename
+        rename_transform, root_id = self.get_transform()
+        file_trans_id = rename_transform.trans_id_file_id('myfile-id')
+        dir_id = rename_transform.trans_id_file_id('first-id')
+        rename_transform.adjust_path('newname', dir_id, file_trans_id)
+        e = self.assertRaises(errors.TransformRenameFailed,
+            rename_transform.apply)
+        # On nix looks like: 
+        # "Failed to rename .../work/.bzr/checkout/limbo/new-1
+        # to .../first-dir/newname: [Errno 13] Permission denied"
+        # On windows looks like:
+        # "Failed to rename .../work/myfile to 
+        # .../work/.bzr/checkout/limbo/new-1: [Errno 13] Permission denied"
+        # The strerror will vary per OS and language so it's not checked here
+        self.assertContainsRe(str(e),
+            "Failed to rename .*(first-dir.newname:|myfile)")
+
     def test_set_executability_order(self):
         """Ensure that executability behaves the same, no matter what order.
 
@@ -2091,6 +2145,38 @@
         self.assertRaises(errors.MalformedTransform, tt.commit, branch,
                           'message')
 
+    def test_commit_rich_revision_data(self):
+        branch, tt = self.get_branch_and_transform()
+        rev_id = tt.commit(branch, 'message', timestamp=1, timezone=43201,
+                           committer='me <me@example.com>',
+                           revprops={'foo': 'bar'}, revision_id='revid-1',
+                           authors=['Author1 <author1@example.com>',
+                              'Author2 <author2@example.com>',
+                               ])
+        self.assertEqual('revid-1', rev_id)
+        revision = branch.repository.get_revision(rev_id)
+        self.assertEqual(1, revision.timestamp)
+        self.assertEqual(43201, revision.timezone)
+        self.assertEqual('me <me@example.com>', revision.committer)
+        self.assertEqual(['Author1 <author1@example.com>',
+                          'Author2 <author2@example.com>'],
+                         revision.get_apparent_authors())
+        del revision.properties['authors']
+        self.assertEqual({'foo': 'bar',
+                          'branch-nick': 'tree'},
+                         revision.properties)
+
+    def test_no_explicit_revprops(self):
+        branch, tt = self.get_branch_and_transform()
+        rev_id = tt.commit(branch, 'message', authors=[
+            'Author1 <author1@example.com>',
+            'Author2 <author2@example.com>', ])
+        revision = branch.repository.get_revision(rev_id)
+        self.assertEqual(['Author1 <author1@example.com>',
+                          'Author2 <author2@example.com>'],
+                         revision.get_apparent_authors())
+        self.assertEqual('tree', revision.properties['branch-nick'])
+
 
 class MockTransform(object):
 

=== modified file 'bzrlib/tests/test_transport.py'
--- bzrlib/tests/test_transport.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/test_transport.py	2010-09-01 18:43:32 +0000
@@ -955,8 +955,12 @@
         ssh_server = stub_sftp.SFTPFullAbsoluteServer(StubSSHServer)
         # We *don't* want to override the default SSH vendor: the detected one
         # is the one to use.
+
+        # FIXME: I don't understand the above comment, SFTPFullAbsoluteServer
+        # inherits from SFTPServer which forces the SSH vendor to
+        # ssh.ParamikoVendor(). So it's forced, not detected. --vila 20100623
         self.start_server(ssh_server)
-        port = ssh_server._listener.port
+        port = ssh_server.port
 
         if sys.platform == 'win32':
             bzr_remote_path = sys.executable + ' ' + self.get_bzr_path()

=== modified file 'bzrlib/tests/test_transport_log.py'
--- bzrlib/tests/test_transport_log.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test_transport_log.py	2010-06-20 11:18:38 +0000
@@ -20,10 +20,9 @@
 
 import types
 
-
+from bzrlib import transport
 from bzrlib.tests import TestCaseWithMemoryTransport
 from bzrlib.trace import mutter
-from bzrlib.transport import get_transport
 from bzrlib.transport.log import TransportLogDecorator
 
 
@@ -31,7 +30,8 @@
 
     def test_log_transport(self):
         base_transport = self.get_transport('')
-        logging_transport = get_transport('log+' + base_transport.base)
+        logging_transport = transport.get_transport(
+            'log+' + base_transport.base)
 
         # operations such as mkdir are logged
         mutter('where are you?')

=== added file 'bzrlib/tests/test_treeshape.py'
--- bzrlib/tests/test_treeshape.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_treeshape.py	2010-07-15 13:55:46 +0000
@@ -0,0 +1,41 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+
+import os
+
+
+from bzrlib import tests
+
+
+class TestTreeShape(tests.TestCaseWithTransport):
+
+    def test_build_tree(self):
+        """Test tree-building test helper"""
+        self.build_tree_contents([
+            ('foo', 'new contents'),
+            ('.bzr/',),
+            ('.bzr/README', 'hello'),
+            ])
+        self.failUnlessExists('foo')
+        self.failUnlessExists('.bzr/README')
+        self.assertFileEqual('hello', '.bzr/README')
+
+    def test_build_tree_symlink(self):
+        self.requireFeature(tests.SymlinkFeature)
+        self.build_tree_contents([('link@', 'target')])
+        self.assertEqual('target',
+            os.readlink('link'))

=== modified file 'bzrlib/tests/test_tuned_gzip.py'
--- bzrlib/tests/test_tuned_gzip.py	2009-06-02 19:56:24 +0000
+++ bzrlib/tests/test_tuned_gzip.py	2010-09-25 20:08:01 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -86,6 +86,15 @@
         # and it should be new member time in the stream.
         self.failUnless(myfile._new_member)
 
+    def test_negative_crc(self):
+        """Content with a negative crc should not break when written"""
+        sio = StringIO()
+        gfile = tuned_gzip.GzipFile(mode="w", fileobj=sio)
+        gfile.write("\xFF")
+        gfile.close()
+        self.assertEqual(gfile.crc & 0xFFFFFFFFL, 0xFF000000L)
+        self.assertEqual(sio.getvalue()[-8:-4], "\x00\x00\x00\xFF")
+
 
 class TestToGzip(TestCase):
 

=== modified file 'bzrlib/tests/test_ui.py'
--- bzrlib/tests/test_ui.py	2010-04-26 13:51:08 +0000
+++ bzrlib/tests/test_ui.py	2010-09-29 20:18:37 +0000
@@ -18,12 +18,14 @@
 """
 
 import os
-import re
 import time
 
 from StringIO import StringIO
 
+from testtools.matchers import *
+
 from bzrlib import (
+    config,
     errors,
     remote,
     repository,
@@ -33,16 +35,47 @@
 from bzrlib.symbol_versioning import (
     deprecated_in,
     )
-from bzrlib.tests import test_progress
+from bzrlib.tests import (
+    fixtures,
+    test_progress,
+    )
 from bzrlib.ui import text as _mod_ui_text
+from bzrlib.tests.testui import (
+    ProgressRecordingUIFactory,
+    )
+
+
+class TestUIConfiguration(tests.TestCaseWithTransport):
+
+    def test_output_encoding_configuration(self):
+        enc = fixtures.generate_unicode_encodings().next()
+        config.GlobalConfig().set_user_option('output_encoding',
+            enc)
+        ui = tests.TestUIFactory(stdin=None,
+            stdout=tests.StringIOWrapper(),
+            stderr=tests.StringIOWrapper())
+        output = ui.make_output_stream()
+        self.assertEquals(output.encoding, enc)
 
 
 class TestTextUIFactory(tests.TestCase):
 
+    def make_test_ui_factory(self, stdin_contents):
+        ui = tests.TestUIFactory(stdin=stdin_contents,
+                                 stdout=tests.StringIOWrapper(),
+                                 stderr=tests.StringIOWrapper())
+        return ui
+
+    def test_text_factory_confirm(self):
+        # turns into reading a regular boolean
+        ui = self.make_test_ui_factory('n\n')
+        self.assertEquals(ui.confirm_action('Should %(thing)s pass?',
+            'bzrlib.tests.test_ui.confirmation',
+            {'thing': 'this'},),
+            False)
+
     def test_text_factory_ascii_password(self):
-        ui = tests.TestUIFactory(stdin='secret\n',
-                                 stdout=tests.StringIOWrapper(),
-                                 stderr=tests.StringIOWrapper())
+        ui = self.make_test_ui_factory('secret\n')
         pb = ui.nested_progress_bar()
         try:
             self.assertEqual('secret',
@@ -63,9 +96,7 @@
         We can't predict what encoding users will have for stdin, so we force
         it to utf8 to test that we transport the password correctly.
         """
-        ui = tests.TestUIFactory(stdin=u'baz\u1234'.encode('utf8'),
-                                 stdout=tests.StringIOWrapper(),
-                                 stderr=tests.StringIOWrapper())
+        ui = self.make_test_ui_factory(u'baz\u1234'.encode('utf8'))
         ui.stderr.encoding = ui.stdout.encoding = ui.stdin.encoding = 'utf8'
         pb = ui.nested_progress_bar()
         try:
@@ -83,51 +114,6 @@
         finally:
             pb.finished()
 
-    def test_progress_note(self):
-        stderr = tests.StringIOWrapper()
-        stdout = tests.StringIOWrapper()
-        ui_factory = _mod_ui_text.TextUIFactory(stdin=tests.StringIOWrapper(''),
-                                                stderr=stderr,
-                                                stdout=stdout)
-        pb = ui_factory.nested_progress_bar()
-        try:
-            result = self.applyDeprecated(deprecated_in((2, 1, 0)),
-                pb.note,
-                't')
-            self.assertEqual(None, result)
-            self.assertEqual("t\n", stdout.getvalue())
-            # Since there was no update() call, there should be no clear() call
-            self.failIf(re.search(r'^\r {10,}\r$',
-                                  stderr.getvalue()) is not None,
-                        'We cleared the stderr without anything to put there')
-        finally:
-            pb.finished()
-
-    def test_progress_note_clears(self):
-        stderr = test_progress._TTYStringIO()
-        stdout = test_progress._TTYStringIO()
-        # so that we get a TextProgressBar
-        os.environ['TERM'] = 'xterm'
-        ui_factory = _mod_ui_text.TextUIFactory(
-            stdin=tests.StringIOWrapper(''),
-            stdout=stdout, stderr=stderr)
-        self.assertIsInstance(ui_factory._progress_view,
-                              _mod_ui_text.TextProgressView)
-        pb = ui_factory.nested_progress_bar()
-        try:
-            # Create a progress update that isn't throttled
-            pb.update('x', 1, 1)
-            result = self.applyDeprecated(deprecated_in((2, 1, 0)),
-                pb.note, 't')
-            self.assertEqual(None, result)
-            self.assertEqual("t\n", stdout.getvalue())
-            # the exact contents will depend on the terminal width and we don't
-            # care about that right now - but you're probably running it on at
-            # least a 10-character wide terminal :)
-            self.assertContainsRe(stderr.getvalue(), r'\r {10,}\r$')
-        finally:
-            pb.finished()
-
     def test_text_ui_get_boolean(self):
         stdin = tests.StringIOWrapper("y\n" # True
                                       "n\n" # False
@@ -176,6 +162,7 @@
         factory = _mod_ui_text.TextUIFactory(
             stdin=tests.StringIOWrapper("yada\ny\n"),
             stdout=out, stderr=out)
+        factory._avail_width = lambda: 79
         pb = factory.nested_progress_bar()
         pb.show_bar = False
         pb.show_spinner = False
@@ -187,9 +174,9 @@
                                                factory.get_boolean,
                                                "what do you want"))
         output = out.getvalue()
-        self.assertContainsRe(factory.stdout.getvalue(),
-            "foo *\r\r  *\r*")
-        self.assertContainsRe(factory.stdout.getvalue(),
+        self.assertContainsRe(output,
+            "| foo *\r\r  *\r*")
+        self.assertContainsRe(output,
             r"what do you want\? \[y/n\]: what do you want\? \[y/n\]: ")
         # stdin should have been totally consumed
         self.assertEqual('', factory.stdin.readline())
@@ -385,7 +372,7 @@
 
     def test_test_ui_factory_progress(self):
         # there's no output; we just want to make sure this doesn't crash -
-        # see https://bugs.edge.launchpad.net/bzr/+bug/408201
+        # see https://bugs.launchpad.net/bzr/+bug/408201
         ui = tests.TestUIFactory()
         pb = ui.nested_progress_bar()
         pb.update('hello')
@@ -459,3 +446,52 @@
         self.assertIsNone('0', av)
         self.assertIsNone('on', av)
         self.assertIsNone('off', av)
+
+
+class TestConfirmationUserInterfacePolicy(tests.TestCase):
+
+    def test_confirm_action_default(self):
+        base_ui = _mod_ui.NoninteractiveUIFactory()
+        for answer in [True, False]:
+            self.assertEquals(
+                _mod_ui.ConfirmationUserInterfacePolicy(base_ui, answer, {})
+                .confirm_action("Do something?",
+                    "bzrlib.tests.do_something", {}),
+                answer)
+
+    def test_confirm_action_specific(self):
+        base_ui = _mod_ui.NoninteractiveUIFactory()
+        for default_answer in [True, False]:
+            for specific_answer in [True, False]:
+                for conf_id in ['given_id', 'other_id']:
+                    wrapper = _mod_ui.ConfirmationUserInterfacePolicy(
+                        base_ui, default_answer, dict(given_id=specific_answer))
+                    result = wrapper.confirm_action("Do something?", conf_id, {})
+                    if conf_id == 'given_id':
+                        self.assertEquals(result, specific_answer)
+                    else:
+                        self.assertEquals(result, default_answer)
+
+    def test_repr(self):
+        base_ui = _mod_ui.NoninteractiveUIFactory()
+        wrapper = _mod_ui.ConfirmationUserInterfacePolicy(
+            base_ui, True, dict(a=2))
+        self.assertThat(repr(wrapper),
+            Equals("ConfirmationUserInterfacePolicy("
+                "NoninteractiveUIFactory(), True, {'a': 2})"))
+
+
+class TestProgressRecordingUI(tests.TestCase):
+    """Test test-oriented UIFactory that records progress updates"""
+
+    def test_nested_ignore_depth_beyond_one(self):
+        # we only want to capture the first level out progress, not
+        # want sub-components might do. So we have nested bars ignored.
+        factory = ProgressRecordingUIFactory()
+        pb1 = factory.nested_progress_bar()
+        pb1.update('foo', 0, 1)
+        pb2 = factory.nested_progress_bar()
+        pb2.update('foo', 0, 1)
+        pb2.finished()
+        pb1.finished()
+        self.assertEqual([("update", 0, 1, 'foo')], factory._calls)

=== modified file 'bzrlib/tests/test_upgrade.py'
--- bzrlib/tests/test_upgrade.py	2010-02-15 10:46:49 +0000
+++ bzrlib/tests/test_upgrade.py	2010-07-29 11:17:57 +0000
@@ -31,24 +31,18 @@
     bzrdir,
     progress,
     repository,
+    transport,
     workingtree,
     workingtree_4,
     )
 import bzrlib.branch
 from bzrlib.branch import Branch
 from bzrlib.tests import TestCaseWithTransport
-from bzrlib.transport import get_transport
 from bzrlib.upgrade import upgrade
 
 
 class TestUpgrade(TestCaseWithTransport):
 
-    def test_build_tree(self):
-        """Test tree-building test helper"""
-        self.build_tree_contents(_upgrade1_template)
-        self.failUnlessExists('foo')
-        self.failUnlessExists('.bzr/README')
-
     def test_upgrade_simple(self):
         """Upgrade simple v0.0.4 format to latest format"""
         eq = self.assertEquals
@@ -80,27 +74,27 @@
             rt.unlock()
         # check a backup was made:
         backup_dir = 'backup.bzr.~1~'
-        transport = get_transport(b.base)
-        transport.stat(backup_dir)
-        transport.stat(backup_dir + '/README')
-        transport.stat(backup_dir + '/branch-format')
-        transport.stat(backup_dir + '/revision-history')
-        transport.stat(backup_dir + '/merged-patches')
-        transport.stat(backup_dir + '/pending-merged-patches')
-        transport.stat(backup_dir + '/pending-merges')
-        transport.stat(backup_dir + '/branch-name')
-        transport.stat(backup_dir + '/branch-lock')
-        transport.stat(backup_dir + '/inventory')
-        transport.stat(backup_dir + '/stat-cache')
-        transport.stat(backup_dir + '/text-store')
-        transport.stat(backup_dir + '/text-store/foo-20051004035611-1591048e9dc7c2d4.gz')
-        transport.stat(backup_dir + '/text-store/foo-20051004035756-4081373d897c3453.gz')
-        transport.stat(backup_dir + '/inventory-store/')
-        transport.stat(backup_dir + '/inventory-store/mbp@sourcefrog.net-20051004035611-176b16534b086b3c.gz')
-        transport.stat(backup_dir + '/inventory-store/mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd.gz')
-        transport.stat(backup_dir + '/revision-store/')
-        transport.stat(backup_dir + '/revision-store/mbp@sourcefrog.net-20051004035611-176b16534b086b3c.gz')
-        transport.stat(backup_dir + '/revision-store/mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd.gz')
+        t = transport.get_transport(b.base)
+        t.stat(backup_dir)
+        t.stat(backup_dir + '/README')
+        t.stat(backup_dir + '/branch-format')
+        t.stat(backup_dir + '/revision-history')
+        t.stat(backup_dir + '/merged-patches')
+        t.stat(backup_dir + '/pending-merged-patches')
+        t.stat(backup_dir + '/pending-merges')
+        t.stat(backup_dir + '/branch-name')
+        t.stat(backup_dir + '/branch-lock')
+        t.stat(backup_dir + '/inventory')
+        t.stat(backup_dir + '/stat-cache')
+        t.stat(backup_dir + '/text-store')
+        t.stat(backup_dir + '/text-store/foo-20051004035611-1591048e9dc7c2d4.gz')
+        t.stat(backup_dir + '/text-store/foo-20051004035756-4081373d897c3453.gz')
+        t.stat(backup_dir + '/inventory-store/')
+        t.stat(backup_dir + '/inventory-store/mbp@sourcefrog.net-20051004035611-176b16534b086b3c.gz')
+        t.stat(backup_dir + '/inventory-store/mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd.gz')
+        t.stat(backup_dir + '/revision-store/')
+        t.stat(backup_dir + '/revision-store/mbp@sourcefrog.net-20051004035611-176b16534b086b3c.gz')
+        t.stat(backup_dir + '/revision-store/mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd.gz')
 
     def test_upgrade_with_ghosts(self):
         """Upgrade v0.0.4 tree containing ghost references.
@@ -153,12 +147,12 @@
         # such a branch to metadir must not setup a working tree.
         self.build_tree_contents(_upgrade1_template)
         upgrade('.', bzrdir.BzrDirFormat6())
-        transport = get_transport('.')
-        transport.delete_multi(['.bzr/pending-merges', '.bzr/inventory'])
-        self.assertFalse(transport.has('.bzr/stat-cache'))
+        t = transport.get_transport('.')
+        t.delete_multi(['.bzr/pending-merges', '.bzr/inventory'])
+        self.assertFalse(t.has('.bzr/stat-cache'))
         # XXX: upgrade fails if a backup.bzr is already present
         # -- David Allouche 2006-08-11
-        transport.delete_tree('backup.bzr.~1~')
+        t.delete_tree('backup.bzr.~1~')
         # At this point, we have a format6 branch without checkout files.
         upgrade('.', bzrdir.BzrDirMetaFormat1())
         # The upgrade should not have set up a working tree.

=== modified file 'bzrlib/tests/test_urlutils.py'
--- bzrlib/tests/test_urlutils.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test_urlutils.py	2010-06-15 03:08:31 +0000
@@ -156,7 +156,7 @@
         # Weird stuff
         # Can't have slashes or colons in the scheme
         test_one('/path/to/://foo', None)
-        test_one('path:path://foo', None)
+        test_one('scheme:stuff://foo', ('scheme', 'stuff://foo'))
         # Must have more than one character for scheme
         test_one('C://foo', None)
         test_one('ab://foo', ('ab', 'foo'))
@@ -195,6 +195,20 @@
             dirname('path/to/foo/', exclude_trailing_slash=False))
         self.assertEqual('path/..', dirname('path/../foo'))
         self.assertEqual('../path', dirname('../path/foo'))
+    
+    def test_is_url(self):
+        self.assertTrue(urlutils.is_url('http://foo/bar'))
+        self.assertTrue(urlutils.is_url('bzr+ssh://foo/bar'))
+        self.assertTrue(urlutils.is_url('lp:foo/bar'))
+        self.assertTrue(urlutils.is_url('file:///foo/bar'))
+        self.assertFalse(urlutils.is_url(''))
+        self.assertFalse(urlutils.is_url('foo'))
+        self.assertFalse(urlutils.is_url('foo/bar'))
+        self.assertFalse(urlutils.is_url('/foo'))
+        self.assertFalse(urlutils.is_url('/foo/bar'))
+        self.assertFalse(urlutils.is_url('C:/'))
+        self.assertFalse(urlutils.is_url('C:/foo'))
+        self.assertFalse(urlutils.is_url('C:/foo/bar'))
 
     def test_join(self):
         def test(expected, *args):
@@ -210,6 +224,8 @@
         test('http://foo/bar/baz', 'http://foo', 'bar/baz')
         test('http://foo/baz', 'http://foo', 'bar/../baz')
         test('http://foo/baz', 'http://foo/bar/', '../baz')
+        test('lp:foo/bar', 'lp:foo', 'bar')
+        test('lp:foo/bar/baz', 'lp:foo', 'bar/baz')
 
         # Absolute paths
         test('http://foo', 'http://foo') # abs url with nothing is preserved.
@@ -219,6 +235,9 @@
         test('http://bar/', 'http://foo', 'http://bar/')
         test('http://bar/a', 'http://foo', 'http://bar/a')
         test('http://bar/a/', 'http://foo', 'http://bar/a/')
+        test('lp:bar', 'http://foo', 'lp:bar')
+        test('lp:bar', 'lp:foo', 'lp:bar')
+        test('file:///stuff', 'lp:foo', 'file:///stuff')
 
         # From a base path
         test('file:///foo', 'file:///', 'foo')
@@ -270,6 +289,51 @@
         self.assertRaises(InvalidURLJoin, urlutils.joinpath, '/', '..')
         self.assertRaises(InvalidURLJoin, urlutils.joinpath, '/', '/..')
 
+    def test_join_segment_parameters_raw(self):
+        join_segment_parameters_raw = urlutils.join_segment_parameters_raw
+        self.assertEquals("/somedir/path", 
+            join_segment_parameters_raw("/somedir/path"))
+        self.assertEquals("/somedir/path,rawdata", 
+            join_segment_parameters_raw("/somedir/path", "rawdata"))
+        self.assertRaises(InvalidURLJoin,
+            join_segment_parameters_raw, "/somedir/path",
+                "rawdata1,rawdata2,rawdata3")
+        self.assertEquals("/somedir/path,bla,bar",
+            join_segment_parameters_raw("/somedir/path", "bla", "bar"))
+        self.assertEquals("/somedir,exist=some/path,bla,bar",
+            join_segment_parameters_raw("/somedir,exist=some/path",
+                "bla", "bar"))
+        self.assertRaises(TypeError, join_segment_parameters_raw, 
+            "/somepath", 42)
+
+    def test_join_segment_parameters(self):
+        join_segment_parameters = urlutils.join_segment_parameters
+        self.assertEquals("/somedir/path", 
+            join_segment_parameters("/somedir/path", {}))
+        self.assertEquals("/somedir/path,key1=val1", 
+            join_segment_parameters("/somedir/path", {"key1": "val1"}))
+        self.assertRaises(InvalidURLJoin,
+            join_segment_parameters, "/somedir/path",
+            {"branch": "brr,brr,brr"})
+        self.assertRaises(InvalidURLJoin,
+            join_segment_parameters, "/somedir/path", {"key1=val1": "val2"})
+        self.assertEquals("/somedir/path,key1=val1,key2=val2",
+            join_segment_parameters("/somedir/path", {
+                "key1": "val1", "key2": "val2"}))
+        self.assertEquals("/somedir/path,key1=val1,key2=val2",
+            join_segment_parameters("/somedir/path,key1=val1", {
+                "key2": "val2"}))
+        self.assertEquals("/somedir/path,key1=val2",
+            join_segment_parameters("/somedir/path,key1=val1", {
+                "key1": "val2"}))
+        self.assertEquals("/somedir,exist=some/path,key1=val1",
+            join_segment_parameters("/somedir,exist=some/path",
+                {"key1": "val1"}))
+        self.assertEquals("/,key1=val1,key2=val2",
+            join_segment_parameters("/,key1=val1", {"key2": "val2"}))
+        self.assertRaises(TypeError,
+            join_segment_parameters, "/,key1=val1", {"foo": 42})
+
     def test_function_type(self):
         if sys.platform == 'win32':
             self.assertEqual(urlutils._win32_local_path_to_url, urlutils.local_path_to_url)
@@ -412,6 +476,53 @@
         self.assertEqual(('path/..', 'foo'), split('path/../foo'))
         self.assertEqual(('../path', 'foo'), split('../path/foo'))
 
+    def test_split_segment_parameters_raw(self):
+        split_segment_parameters_raw = urlutils.split_segment_parameters_raw
+        self.assertEquals(("/some/path", []),
+            split_segment_parameters_raw("/some/path"))
+        self.assertEquals(("/some/path", ["tip"]),
+            split_segment_parameters_raw("/some/path,tip"))
+        self.assertEquals(("/some,dir/path", ["tip"]),
+            split_segment_parameters_raw("/some,dir/path,tip"))
+        self.assertEquals(("/somedir/path", ["heads%2Ftip"]),
+            split_segment_parameters_raw("/somedir/path,heads%2Ftip"))
+        self.assertEquals(("/somedir/path", ["heads%2Ftip", "bar"]),
+            split_segment_parameters_raw("/somedir/path,heads%2Ftip,bar"))
+        self.assertEquals(("/", ["key1=val1"]),
+            split_segment_parameters_raw(",key1=val1"))
+        self.assertEquals(("foo/", ["key1=val1"]),
+            split_segment_parameters_raw("foo/,key1=val1"))
+        self.assertEquals(("foo/base,la=bla/other/elements", []),
+            split_segment_parameters_raw("foo/base,la=bla/other/elements"))
+        self.assertEquals(("foo/base,la=bla/other/elements", ["a=b"]),
+            split_segment_parameters_raw("foo/base,la=bla/other/elements,a=b"))
+
+    def test_split_segment_parameters(self):
+        split_segment_parameters = urlutils.split_segment_parameters
+        self.assertEquals(("/some/path", {}),
+            split_segment_parameters("/some/path"))
+        self.assertEquals(("/some/path", {"branch": "tip"}),
+            split_segment_parameters("/some/path,branch=tip"))
+        self.assertEquals(("/some,dir/path", {"branch": "tip"}),
+            split_segment_parameters("/some,dir/path,branch=tip"))
+        self.assertEquals(("/somedir/path", {"ref": "heads%2Ftip"}),
+            split_segment_parameters("/somedir/path,ref=heads%2Ftip"))
+        self.assertEquals(("/somedir/path",
+            {"ref": "heads%2Ftip", "key1": "val1"}),
+            split_segment_parameters(
+                "/somedir/path,ref=heads%2Ftip,key1=val1"))
+        self.assertEquals(("/somedir/path", {"ref": "heads%2F=tip"}),
+            split_segment_parameters("/somedir/path,ref=heads%2F=tip"))
+        self.assertEquals(("/", {"key1": "val1"}),
+            split_segment_parameters(",key1=val1"))
+        self.assertEquals(("foo/", {"key1": "val1"}),
+            split_segment_parameters("foo/,key1=val1"))
+        self.assertEquals(("foo/base,key1=val1/other/elements", {}),
+            split_segment_parameters("foo/base,key1=val1/other/elements"))
+        self.assertEquals(("foo/base,key1=val1/other/elements",
+            {"key2": "val2"}), split_segment_parameters(
+                "foo/base,key1=val1/other/elements,key2=val2"))
+
     def test_win32_strip_local_trailing_slash(self):
         strip = urlutils._win32_strip_local_trailing_slash
         self.assertEqual('file://', strip('file://'))

=== added file 'bzrlib/tests/test_versionedfile.py'
--- bzrlib/tests/test_versionedfile.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_versionedfile.py	2010-08-10 20:03:44 +0000
@@ -0,0 +1,139 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for VersionedFile classes"""
+
+from bzrlib import (
+    errors,
+    groupcompress,
+    multiparent,
+    tests,
+    versionedfile,
+    )
+
+
+class Test_MPDiffGenerator(tests.TestCaseWithMemoryTransport):
+    # Should this be a per vf test?
+
+    def make_vf(self):
+        t = self.get_transport('')
+        factory = groupcompress.make_pack_factory(True, True, 1)
+        return factory(t)
+
+    def make_three_vf(self):
+        vf = self.make_vf()
+        vf.add_lines(('one',), (), ['first\n'])
+        vf.add_lines(('two',), [('one',)], ['first\n', 'second\n'])
+        vf.add_lines(('three',), [('one',), ('two',)],
+                    ['first\n', 'second\n', 'third\n'])
+        return vf
+
+    def test_finds_parents(self):
+        vf = self.make_three_vf()
+        gen = versionedfile._MPDiffGenerator(vf, [('three',)])
+        needed_keys, refcount = gen._find_needed_keys()
+        self.assertEqual(sorted([('one',), ('two',), ('three',)]),
+                         sorted(needed_keys))
+        self.assertEqual({('one',): 1, ('two',): 1}, refcount)
+
+    def test_ignores_ghost_parents(self):
+        # If a parent is a ghost, it is just ignored
+        vf = self.make_vf()
+        vf.add_lines(('two',), [('one',)], ['first\n', 'second\n'])
+        gen = versionedfile._MPDiffGenerator(vf, [('two',)])
+        needed_keys, refcount = gen._find_needed_keys()
+        self.assertEqual(sorted([('two',)]), sorted(needed_keys))
+        # It is returned, but we don't really care as we won't extract it
+        self.assertEqual({('one',): 1}, refcount)
+        self.assertEqual([('one',)], sorted(gen.ghost_parents))
+        self.assertEqual([], sorted(gen.present_parents))
+
+    def test_raises_on_ghost_keys(self):
+        # If the requested key is a ghost, then we have a problem
+        vf = self.make_vf()
+        gen = versionedfile._MPDiffGenerator(vf, [('one',)])
+        self.assertRaises(errors.RevisionNotPresent,
+                          gen._find_needed_keys)
+
+    def test_refcount_multiple_children(self):
+        vf = self.make_three_vf()
+        gen = versionedfile._MPDiffGenerator(vf, [('two',), ('three',)])
+        needed_keys, refcount = gen._find_needed_keys()
+        self.assertEqual(sorted([('one',), ('two',), ('three',)]),
+                         sorted(needed_keys))
+        self.assertEqual({('one',): 2, ('two',): 1}, refcount)
+        self.assertEqual([('one',)], sorted(gen.present_parents))
+
+    def test_process_contents(self):
+        vf = self.make_three_vf()
+        gen = versionedfile._MPDiffGenerator(vf, [('two',), ('three',)])
+        gen._find_needed_keys()
+        self.assertEqual({('two',): (('one',),),
+                          ('three',): (('one',), ('two',))},
+                         gen.parent_map)
+        self.assertEqual({('one',): 2, ('two',): 1}, gen.refcounts)
+        self.assertEqual(sorted([('one',), ('two',), ('three',)]),
+                         sorted(gen.needed_keys))
+        stream = vf.get_record_stream(gen.needed_keys, 'topological', True)
+        record = stream.next()
+        self.assertEqual(('one',), record.key)
+        # one is not needed in the output, but it is needed by children. As
+        # such, it should end up in the various caches
+        gen._process_one_record(record.key, record.get_bytes_as('chunked'))
+        # The chunks should be cached, the refcount untouched
+        self.assertEqual([('one',)], gen.chunks.keys())
+        self.assertEqual({('one',): 2, ('two',): 1}, gen.refcounts)
+        self.assertEqual([], gen.diffs.keys())
+        # Next we get 'two', which is something we output, but also needed for
+        # three
+        record = stream.next()
+        self.assertEqual(('two',), record.key)
+        gen._process_one_record(record.key, record.get_bytes_as('chunked'))
+        # Both are now cached, and the diff for two has been extracted, and
+        # one's refcount has been updated. two has been removed from the
+        # parent_map
+        self.assertEqual(sorted([('one',), ('two',)]),
+                         sorted(gen.chunks.keys()))
+        self.assertEqual({('one',): 1, ('two',): 1}, gen.refcounts)
+        self.assertEqual([('two',)], gen.diffs.keys())
+        self.assertEqual({('three',): (('one',), ('two',))},
+                         gen.parent_map)
+        # Finally 'three', which allows us to remove all parents from the
+        # caches
+        record = stream.next()
+        self.assertEqual(('three',), record.key)
+        gen._process_one_record(record.key, record.get_bytes_as('chunked'))
+        # Both are now cached, and the diff for two has been extracted, and
+        # one's refcount has been updated
+        self.assertEqual([], gen.chunks.keys())
+        self.assertEqual({}, gen.refcounts)
+        self.assertEqual(sorted([('two',), ('three',)]),
+                         sorted(gen.diffs.keys()))
+
+    def test_compute_diffs(self):
+        vf = self.make_three_vf()
+        # The content is in the order requested, even if it isn't topological
+        gen = versionedfile._MPDiffGenerator(vf, [('two',), ('three',),
+                                                  ('one',)])
+        diffs = gen.compute_diffs()
+        expected_diffs = [
+            multiparent.MultiParent([multiparent.ParentText(0, 0, 0, 1),
+                                     multiparent.NewText(['second\n'])]),
+            multiparent.MultiParent([multiparent.ParentText(1, 0, 0, 2),
+                                     multiparent.NewText(['third\n'])]),
+            multiparent.MultiParent([multiparent.NewText(['first\n'])]),
+            ]
+        self.assertEqual(expected_diffs, diffs)

=== modified file 'bzrlib/tests/test_win32utils.py'
--- bzrlib/tests/test_win32utils.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/test_win32utils.py	2010-06-22 23:58:34 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+"""Tests for win32utils."""
+
 import os
 import sys
 
@@ -29,25 +31,10 @@
     TestSkipped,
     UnicodeFilenameFeature,
     )
+from bzrlib.tests.features import backslashdir_feature
 from bzrlib.win32utils import glob_expand, get_app_path
 
 
-class _BackslashDirSeparatorFeature(tests.Feature):
-
-    def _probe(self):
-        try:
-            os.lstat(os.getcwd() + '\\')
-        except OSError:
-            return False
-        else:
-            return True
-
-    def feature_name(self):
-        return "Filesystem treats '\\' as a directory separator."
-
-BackslashDirSeparatorFeature = _BackslashDirSeparatorFeature()
-
-
 class _RequiredModuleFeature(Feature):
 
     def __init__(self, mod_name):
@@ -67,6 +54,7 @@
 Win32RegistryFeature = _RequiredModuleFeature('_winreg')
 CtypesFeature = _RequiredModuleFeature('ctypes')
 Win32comShellFeature = _RequiredModuleFeature('win32com.shell')
+Win32ApiFeature = _RequiredModuleFeature('win32api') 
 
 
 # Tests
@@ -121,7 +109,7 @@
             ])
 
     def test_backslash_globbing(self):
-        self.requireFeature(BackslashDirSeparatorFeature)
+        self.requireFeature(backslashdir_feature)
         self.build_ascii_tree()
         self._run_testset([
             [[u'd\\'], [u'd/']],
@@ -164,7 +152,7 @@
             ])
 
     def test_unicode_backslashes(self):
-        self.requireFeature(BackslashDirSeparatorFeature)
+        self.requireFeature(backslashdir_feature)
         self.build_unicode_tree()
         self._run_testset([
             # no wildcards
@@ -202,6 +190,7 @@
         # typical windows users should have wordpad in the system
         # but there is problem: its path has the format REG_EXPAND_SZ
         # so naive attempt to get the path is not working
+        self.requireFeature(Win32ApiFeature)
         for a in ('wordpad', 'wordpad.exe'):
             p = get_app_path(a)
             d, b = os.path.split(p)
@@ -287,14 +276,15 @@
         win32utils.set_file_attr_hidden(path)
 
 
-
-
 class Test_CommandLineToArgv(tests.TestCaseInTempDir):
 
-    def assertCommandLine(self, expected, line, single_quotes_allowed=False):
+    def assertCommandLine(self, expected, line, argv=None,
+            single_quotes_allowed=False):
         # Strictly speaking we should respect parameter order versus glob
         # expansions, but it's not really worth the effort here
-        argv = win32utils._command_line_to_argv(line,
+        if argv is None:
+            argv = [line]
+        argv = win32utils._command_line_to_argv(line, argv,
                 single_quotes_allowed=single_quotes_allowed)
         self.assertEqual(expected, sorted(argv))
 
@@ -328,9 +318,12 @@
 
     def test_single_quote_support(self):
         self.assertCommandLine(["add", "let's-do-it.txt"],
-            "add let's-do-it.txt")
-        self.assertCommandLine(["add", "lets do it.txt"],
-            "add 'lets do it.txt'", single_quotes_allowed=True)
+            "add let's-do-it.txt",
+            ["add", "let's-do-it.txt"])
+        self.expectFailure("Using single quotes breaks trimming from argv",
+            self.assertCommandLine, ["add", "lets do it.txt"],
+            "add 'lets do it.txt'", ["add", "'lets", "do", "it.txt'"],
+            single_quotes_allowed=True)
 
     def test_case_insensitive_globs(self):
         self.requireFeature(tests.CaseInsCasePresFilenameFeature)
@@ -338,6 +331,14 @@
         self.assertCommandLine([u'A/b.c'], 'A/B*')
 
     def test_backslashes(self):
-        self.requireFeature(BackslashDirSeparatorFeature)
+        self.requireFeature(backslashdir_feature)
         self.build_tree(['a/', 'a/b.c', 'a/c.c', 'a/c.h'])
         self.assertCommandLine([u'a/b.c'], 'a\\b*')
+
+    def test_with_pdb(self):
+        """Check stripping Python arguments before bzr script per lp:587868"""
+        self.assertCommandLine([u"rocks"], "-m pdb rocks", ["rocks"])
+        self.build_tree(['d/', 'd/f1', 'd/f2'])
+        self.assertCommandLine([u"rm", u"x*"], "-m pdb rm x*", ["rm", u"x*"])
+        self.assertCommandLine([u"add", u"d/f1", u"d/f2"], "-m pdb add d/*",
+            ["add", u"d/*"])

=== modified file 'bzrlib/tests/test_workingtree.py'
--- bzrlib/tests/test_workingtree.py	2010-04-23 08:51:52 +0000
+++ bzrlib/tests/test_workingtree.py	2010-06-20 11:18:38 +0000
@@ -22,6 +22,7 @@
     bzrdir,
     conflicts,
     errors,
+    transport,
     workingtree,
     )
 from bzrlib.branch import Branch
@@ -29,7 +30,6 @@
 from bzrlib.lockdir import LockDir
 from bzrlib.mutabletree import needs_tree_write_lock
 from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
-from bzrlib.transport import get_transport
 from bzrlib.workingtree import (
     TreeEntry,
     TreeDirectory,
@@ -138,7 +138,7 @@
             dir.create_repository()
             dir.create_branch()
             format.initialize(dir)
-            t = get_transport(url)
+            t = transport.get_transport(url)
             found_format = workingtree.WorkingTreeFormat.find_format(dir)
             self.failUnless(isinstance(found_format, format.__class__))
         check_format(workingtree.WorkingTreeFormat3(), "bar")

=== modified file 'bzrlib/tests/test_workingtree_4.py'
--- bzrlib/tests/test_workingtree_4.py	2010-02-04 16:06:36 +0000
+++ bzrlib/tests/test_workingtree_4.py	2010-07-21 09:58:42 +0000
@@ -757,3 +757,23 @@
             ('', [(('', 'dir', 'dir-id'), ['d', 'd'])]),
             ('dir', [(('dir', 'file', 'file-id'), ['a', 'f'])]),
         ],  self.get_simple_dirblocks(state))
+
+
+class TestInventoryCoherency(TestCaseWithTransport):
+
+    def test_inventory_is_synced_when_unversioning_a_dir(self):
+        """Unversioning the root of a subtree unversions the entire subtree."""
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['a/', 'a/b', 'c/'])
+        tree.add(['a', 'a/b', 'c'], ['a-id', 'b-id', 'c-id'])
+        # within a lock unversion should take effect
+        tree.lock_write()
+        self.addCleanup(tree.unlock)
+        # Force access to the in memory inventory to trigger bug #494221: try
+        # maintaining the in-memory inventory
+        inv = tree.inventory
+        self.assertTrue(inv.has_id('a-id'))
+        self.assertTrue(inv.has_id('b-id'))
+        tree.unversion(['a-id', 'b-id'])
+        self.assertFalse(inv.has_id('a-id'))
+        self.assertFalse(inv.has_id('b-id'))

=== added file 'bzrlib/tests/testui.py'
--- bzrlib/tests/testui.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/testui.py	2010-09-25 20:08:01 +0000
@@ -0,0 +1,46 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""UI implementations for use in testing.
+"""
+
+
+from bzrlib import (
+    progress,
+    ui,
+    )
+
+
+class ProgressRecordingUIFactory(ui.UIFactory, progress.DummyProgress):
+    """Captures progress updates made through it.
+    
+    This is overloaded as both the UIFactory and the progress model."""
+
+    def __init__(self):
+        super(ProgressRecordingUIFactory, self).__init__()
+        self._calls = []
+        self.depth = 0
+
+    def nested_progress_bar(self):
+        self.depth += 1
+        return self
+
+    def finished(self):
+        self.depth -= 1
+
+    def update(self, message, count=None, total=None):
+        if self.depth == 1:
+            self._calls.append(("update", count, total, message))

=== modified file 'bzrlib/tests/treeshape.py'
--- bzrlib/tests/treeshape.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/treeshape.py	2010-07-29 11:17:57 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -41,13 +41,17 @@
 
     The template is built relative to the Python process's current
     working directory.
+
+    ('foo/',) will build a directory.
+    ('foo', 'bar') will write 'bar' to 'foo'
+    ('foo@', 'linktarget') will raise an error
     """
     for tt in template:
         name = tt[0]
         if name[-1] == '/':
             os.mkdir(name)
         elif name[-1] == '@':
-            raise NotImplementedError('symlinks not handled yet')
+            os.symlink(tt[1], tt[0][:-1])
         else:
             f = file(name, 'wb')
             try:

=== modified file 'bzrlib/textmerge.py'
--- bzrlib/textmerge.py	2009-03-23 14:59:43 +0000
+++ bzrlib/textmerge.py	2010-06-17 09:23:19 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2006, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,10 @@
 #         Aaron Bentley <aaron.bentley@utoronto.ca>
 
 
-import bzrlib.patiencediff
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+from bzrlib import patiencediff
+""")
 
 
 class TextMerge(object):
@@ -131,7 +134,8 @@
         """Return structured merge info.
         See TextMerge docstring.
         """
-        sm = bzrlib.patiencediff.PatienceSequenceMatcher(None, self.lines_a, self.lines_b)
+        sm = patiencediff.PatienceSequenceMatcher(
+            None, self.lines_a, self.lines_b)
         pos_a = 0
         pos_b = 0
         for ai, bi, l in sm.get_matching_blocks():

=== modified file 'bzrlib/trace.py'
--- bzrlib/trace.py	2010-03-30 14:44:30 +0000
+++ bzrlib/trace.py	2010-06-28 02:41:22 +0000
@@ -111,6 +111,12 @@
 
 
 def note(*args, **kwargs):
+    """Output a note to the user.
+
+    Takes the same parameters as logging.info.
+
+    :return: None
+    """
     # FIXME note always emits utf-8, regardless of the terminal encoding
     #
     # FIXME: clearing the ui and then going through the abstract logging
@@ -300,26 +306,27 @@
     logging.getLogger("bzr").
 
     Output can be redirected away by calling _push_log_file.
+
+    :return: A memento from push_log_file for restoring the log state.
     """
-    # Do this before we open the log file, so we prevent
-    # get_terminal_encoding() from mutter()ing multiple times
-    term_encoding = osutils.get_terminal_encoding()
     start_time = osutils.format_local_date(_bzr_log_start_time,
                                            timezone='local')
     # create encoded wrapper around stderr
     bzr_log_file = _open_bzr_log()
     if bzr_log_file is not None:
         bzr_log_file.write(start_time.encode('utf-8') + '\n')
-    push_log_file(bzr_log_file,
+    memento = push_log_file(bzr_log_file,
         r'[%(process)5d] %(asctime)s.%(msecs)03d %(levelname)s: %(message)s',
         r'%Y-%m-%d %H:%M:%S')
     # after hooking output into bzr_log, we also need to attach a stderr
     # handler, writing only at level info and with encoding
+    term_encoding = osutils.get_terminal_encoding()
     writer_factory = codecs.getwriter(term_encoding)
     encoded_stderr = writer_factory(sys.stderr, errors='replace')
     stderr_handler = logging.StreamHandler(encoded_stderr)
     stderr_handler.setLevel(logging.INFO)
     logging.getLogger('bzr').addHandler(stderr_handler)
+    return memento
 
 
 def push_log_file(to_file, log_format=None, date_format=None):
@@ -361,18 +368,20 @@
 def pop_log_file((magic, old_handlers, new_handler, old_trace_file, new_trace_file)):
     """Undo changes to logging/tracing done by _push_log_file.
 
-    This flushes, but does not close the trace file.
+    This flushes, but does not close the trace file (so that anything that was
+    in it is output.
 
     Takes the memento returned from _push_log_file."""
     global _trace_file
     _trace_file = old_trace_file
     bzr_logger = logging.getLogger('bzr')
     bzr_logger.removeHandler(new_handler)
-    # must be closed, otherwise logging will try to close it atexit, and the
+    # must be closed, otherwise logging will try to close it at exit, and the
     # file will likely already be closed underneath.
     new_handler.close()
     bzr_logger.handlers = old_handlers
-    new_trace_file.flush()
+    if new_trace_file is not None:
+        new_trace_file.flush()
 
 
 def log_exception_quietly():
@@ -492,7 +501,9 @@
     elif not getattr(exc_object, 'internal_error', True):
         report_user_error(exc_info, err_file)
         return errors.EXIT_ERROR
-    elif isinstance(exc_object, (OSError, IOError)):
+    elif isinstance(exc_object, (OSError, IOError)) or (
+        # GZ 2010-05-20: Like (exc_type is pywintypes.error) but avoid import
+        exc_type.__name__ == "error" and exc_type.__module__ == "pywintypes"):
         # Might be nice to catch all of these and show them as something more
         # specific, but there are too many cases at the moment.
         report_user_error(exc_info, err_file)
@@ -532,7 +543,7 @@
 
 
 def _flush_stdout_stderr():
-    # installed into an atexit hook by bzrlib.initialize()
+    # called from the bzrlib library finalizer returned by bzrlib.initialize()
     try:
         sys.stdout.flush()
         sys.stderr.flush()
@@ -545,7 +556,40 @@
 
 
 def _flush_trace():
-    # run from atexit hook
+    # called from the bzrlib library finalizer returned by bzrlib.initialize()
     global _trace_file
     if _trace_file:
         _trace_file.flush()
+
+
+class Config(object):
+    """Configuration of message tracing in bzrlib.
+
+    This implements the context manager protocol and should manage any global
+    variables still used. The default config used is DefaultConfig, but
+    embedded uses of bzrlib may wish to use a custom manager.
+    """
+
+    def __enter__(self):
+        return self # This is bound to the 'as' clause in a with statement.
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        return False # propogate exceptions.
+
+
+class DefaultConfig(Config):
+    """A default configuration for tracing of messages in bzrlib.
+
+    This implements the context manager protocol.
+    """
+
+    def __enter__(self):
+        self._original_filename = _bzr_log_filename
+        self._original_state = enable_default_logging()
+        return self # This is bound to the 'as' clause in a with statement.
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        pop_log_file(self._original_state)
+        global _bzr_log_filename
+        _bzr_log_filename = self._original_filename
+        return False # propogate exceptions.

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2010-04-30 11:35:43 +0000
+++ bzrlib/transform.py	2010-08-27 07:54:21 +0000
@@ -25,6 +25,7 @@
     annotate,
     bencode,
     bzrdir,
+    commit,
     delta,
     errors,
     inventory,
@@ -314,10 +315,9 @@
 
     def delete_contents(self, trans_id):
         """Schedule the contents of a path entry for deletion"""
-        # Ensure that the object exists in the WorkingTree, this will raise an
-        # exception if there is a problem
-        self.tree_kind(trans_id)
-        self._removed_contents.add(trans_id)
+        kind = self.tree_kind(trans_id)
+        if kind is not None:
+            self._removed_contents.add(trans_id)
 
     def cancel_deletion(self, trans_id):
         """Cancel a scheduled deletion"""
@@ -388,22 +388,22 @@
         changed_kind = set(self._removed_contents)
         changed_kind.intersection_update(self._new_contents)
         changed_kind.difference_update(new_ids)
-        changed_kind = (t for t in changed_kind if self.tree_kind(t) !=
-                        self.final_kind(t))
+        changed_kind = (t for t in changed_kind
+                        if self.tree_kind(t) != self.final_kind(t))
         new_ids.update(changed_kind)
         return sorted(FinalPaths(self).get_paths(new_ids))
 
     def final_kind(self, trans_id):
         """Determine the final file kind, after any changes applied.
 
-        Raises NoSuchFile if the file does not exist/has no contents.
-        (It is conceivable that a path would be created without the
-        corresponding contents insertion command)
+        :return: None if the file does not exist/has no contents.  (It is
+            conceivable that a path would be created without the corresponding
+            contents insertion command)
         """
         if trans_id in self._new_contents:
             return self._new_contents[trans_id]
         elif trans_id in self._removed_contents:
-            raise NoSuchFile(None)
+            return None
         else:
             return self.tree_kind(trans_id)
 
@@ -595,9 +595,8 @@
         """
         conflicts = []
         for trans_id in self._new_id.iterkeys():
-            try:
-                kind = self.final_kind(trans_id)
-            except NoSuchFile:
+            kind = self.final_kind(trans_id)
+            if kind is None:
                 conflicts.append(('versioning no contents', trans_id))
                 continue
             if not InventoryEntry.versionable_kind(kind):
@@ -617,11 +616,7 @@
             if self.final_file_id(trans_id) is None:
                 conflicts.append(('unversioned executability', trans_id))
             else:
-                try:
-                    non_file = self.final_kind(trans_id) != "file"
-                except NoSuchFile:
-                    non_file = True
-                if non_file is True:
+                if self.final_kind(trans_id) != "file":
                     conflicts.append(('non-file executability', trans_id))
         return conflicts
 
@@ -629,9 +624,7 @@
         """Check for overwrites (not permitted on Win32)"""
         conflicts = []
         for trans_id in self._new_contents:
-            try:
-                self.tree_kind(trans_id)
-            except NoSuchFile:
+            if self.tree_kind(trans_id) is None:
                 continue
             if trans_id not in self._removed_contents:
                 conflicts.append(('overwrite', trans_id,
@@ -651,10 +644,7 @@
             last_name = None
             last_trans_id = None
             for name, trans_id in name_ids:
-                try:
-                    kind = self.final_kind(trans_id)
-                except NoSuchFile:
-                    kind = None
+                kind = self.final_kind(trans_id)
                 file_id = self.final_file_id(trans_id)
                 if kind is None and file_id is None:
                     continue
@@ -686,15 +676,7 @@
                 continue
             if not self._any_contents(children):
                 continue
-            for child in children:
-                try:
-                    self.final_kind(child)
-                except NoSuchFile:
-                    continue
-            try:
-                kind = self.final_kind(parent_id)
-            except NoSuchFile:
-                kind = None
+            kind = self.final_kind(parent_id)
             if kind is None:
                 conflicts.append(('missing parent', parent_id))
             elif kind != "directory":
@@ -704,11 +686,8 @@
     def _any_contents(self, trans_ids):
         """Return true if any of the trans_ids, will have contents."""
         for trans_id in trans_ids:
-            try:
-                kind = self.final_kind(trans_id)
-            except NoSuchFile:
-                continue
-            return True
+            if self.final_kind(trans_id) is not None:
+                return True
         return False
 
     def _set_executability(self, path, trans_id):
@@ -844,10 +823,7 @@
         Return a (name, parent, kind, executable) tuple
         """
         to_name = self.final_name(to_trans_id)
-        try:
-            to_kind = self.final_kind(to_trans_id)
-        except NoSuchFile:
-            to_kind = None
+        to_kind = self.final_kind(to_trans_id)
         to_parent = self.final_file_id(self.final_parent(to_trans_id))
         if to_trans_id in self._new_executability:
             to_executable = self._new_executability[to_trans_id]
@@ -927,12 +903,27 @@
         """
         return _PreviewTree(self)
 
-    def commit(self, branch, message, merge_parents=None, strict=False):
+    def commit(self, branch, message, merge_parents=None, strict=False,
+               timestamp=None, timezone=None, committer=None, authors=None,
+               revprops=None, revision_id=None):
         """Commit the result of this TreeTransform to a branch.
 
         :param branch: The branch to commit to.
         :param message: The message to attach to the commit.
-        :param merge_parents: Additional parents specified by pending merges.
+        :param merge_parents: Additional parent revision-ids specified by
+            pending merges.
+        :param strict: If True, abort the commit if there are unversioned
+            files.
+        :param timestamp: if not None, seconds-since-epoch for the time and
+            date.  (May be a float.)
+        :param timezone: Optional timezone for timestamp, as an offset in
+            seconds.
+        :param committer: Optional committer in email-id format.
+            (e.g. "J Random Hacker <jrandom@example.com>")
+        :param authors: Optional list of authors in email-id format.
+        :param revprops: Optional dictionary of revision properties.
+        :param revision_id: Optional revision id.  (Specifying a revision-id
+            may reduce performance for some non-native formats.)
         :return: The revision_id of the revision committed.
         """
         self._check_malformed()
@@ -955,7 +946,13 @@
         if self._tree.get_revision_id() != last_rev_id:
             raise ValueError('TreeTransform not based on branch basis: %s' %
                              self._tree.get_revision_id())
-        builder = branch.get_commit_builder(parent_ids)
+        revprops = commit.Commit.update_revprops(revprops, branch, authors)
+        builder = branch.get_commit_builder(parent_ids,
+                                            timestamp=timestamp,
+                                            timezone=timezone,
+                                            committer=committer,
+                                            revprops=revprops,
+                                            revision_id=revision_id)
         preview = self.get_preview_tree()
         list(builder.record_iter_changes(preview, last_rev_id,
                                          self.iter_changes()))
@@ -1160,7 +1157,7 @@
             if trans_id not in self._new_contents:
                 continue
             new_path = self._limbo_name(trans_id)
-            osutils.rename(old_path, new_path)
+            os.rename(old_path, new_path)
             for descendant in self._limbo_descendants(trans_id):
                 desc_path = self._limbo_files[descendant]
                 desc_path = new_path + desc_path[len(old_path):]
@@ -1397,18 +1394,15 @@
     def tree_kind(self, trans_id):
         """Determine the file kind in the working tree.
 
-        Raises NoSuchFile if the file does not exist
+        :returns: The file kind or None if the file does not exist
         """
         path = self._tree_id_paths.get(trans_id)
         if path is None:
-            raise NoSuchFile(None)
+            return None
         try:
             return file_kind(self._tree.abspath(path))
-        except OSError, e:
-            if e.errno != errno.ENOENT:
-                raise
-            else:
-                raise NoSuchFile(path)
+        except errors.NoSuchFile:
+            return None
 
     def _set_mode(self, trans_id, mode_id, typefunc):
         """Set the mode of new file contents.
@@ -1583,9 +1577,8 @@
                 if file_id is None:
                     continue
                 needs_entry = False
-                try:
-                    kind = self.final_kind(trans_id)
-                except NoSuchFile:
+                kind = self.final_kind(trans_id)
+                if kind is None:
                     kind = self._tree.stored_kind(file_id)
                 parent_trans_id = self.final_parent(trans_id)
                 parent_file_id = new_path_file_ids.get(parent_trans_id)
@@ -1635,7 +1628,7 @@
                       or trans_id in self._new_parent):
                     try:
                         mover.rename(full_path, self._limbo_name(trans_id))
-                    except OSError, e:
+                    except errors.TransformRenameFailed, e:
                         if e.errno != errno.ENOENT:
                             raise
                     else:
@@ -1666,7 +1659,7 @@
                 if trans_id in self._needs_rename:
                     try:
                         mover.rename(self._limbo_name(trans_id), full_path)
-                    except OSError, e:
+                    except errors.TransformRenameFailed, e:
                         # We may be renaming a dangling inventory id
                         if e.errno != errno.ENOENT:
                             raise
@@ -1703,9 +1696,12 @@
     def tree_kind(self, trans_id):
         path = self._tree_id_paths.get(trans_id)
         if path is None:
-            raise NoSuchFile(None)
+            return None
         file_id = self._tree.path2id(path)
-        return self._tree.kind(file_id)
+        try:
+            return self._tree.kind(file_id)
+        except errors.NoSuchFile:
+            return None
 
     def _set_mode(self, trans_id, mode_id, typefunc):
         """Set the mode of new file contents.
@@ -1770,7 +1766,7 @@
         parent_keys = [(file_id, self._file_revision(t, file_id)) for t in
                        self._iter_parent_trees()]
         vf.add_lines((file_id, tree_revision), parent_keys,
-                     self.get_file(file_id).readlines())
+                     self.get_file_lines(file_id))
         repo = self._get_repository()
         base_vf = repo.texts
         if base_vf not in vf.fallback_versionedfiles:
@@ -1798,9 +1794,12 @@
             executable = self.is_executable(file_id, path)
         return kind, executable, None
 
+    def is_locked(self):
+        return False
+
     def lock_read(self):
         # Perhaps in theory, this should lock the TreeTransform?
-        pass
+        return self
 
     def unlock(self):
         pass
@@ -1904,9 +1903,8 @@
             if (specific_file_ids is not None
                 and file_id not in specific_file_ids):
                 continue
-            try:
-                kind = self._transform.final_kind(trans_id)
-            except NoSuchFile:
+            kind = self._transform.final_kind(trans_id)
+            if kind is None:
                 kind = self._transform._tree.stored_kind(file_id)
             new_entry = inventory.make_entry(
                 kind,
@@ -2144,10 +2142,10 @@
                 path_from_root = self._final_paths.get_path(child_id)
                 basename = self._transform.final_name(child_id)
                 file_id = self._transform.final_file_id(child_id)
-                try:
-                    kind = self._transform.final_kind(child_id)
+                kind  = self._transform.final_kind(child_id)
+                if kind is not None:
                     versioned_kind = kind
-                except NoSuchFile:
+                else:
                     kind = 'unknown'
                     versioned_kind = self._transform._tree.stored_kind(file_id)
                 if versioned_kind == 'directory':
@@ -2266,9 +2264,6 @@
     for num, _unused in enumerate(wt.all_file_ids()):
         if num > 0:  # more than just a root
             raise errors.WorkingTreeAlreadyPopulated(base=wt.basedir)
-    existing_files = set()
-    for dir, files in wt.walkdirs():
-        existing_files.update(f[0] for f in files)
     file_trans_id = {}
     top_pb = bzrlib.ui.ui_factory.nested_progress_bar()
     pp = ProgressPhase("Build phase", 2, top_pb)
@@ -2298,6 +2293,15 @@
                 precomputed_delta = []
             else:
                 precomputed_delta = None
+            # Check if tree inventory has content. If so, we populate
+            # existing_files with the directory content. If there are no
+            # entries we skip populating existing_files as its not used.
+            # This improves performance and unncessary work on large
+            # directory trees. (#501307)
+            if total > 0:
+                existing_files = set()
+                for dir, files in wt.walkdirs():
+                    existing_files.update(f[0] for f in files)
             for num, (tree_path, entry) in \
                 enumerate(tree.inventory.iter_entries_by_dir()):
                 pb.update("Building tree", num - len(deferred_contents), total)
@@ -2435,8 +2439,12 @@
     if entry.kind == "directory":
         return True
     if entry.kind == "file":
-        if tree.get_file(file_id).read() == file(target_path, 'rb').read():
-            return True
+        f = file(target_path, 'rb')
+        try:
+            if tree.get_file_text(file_id) == f.read():
+                return True
+        finally:
+            f.close()
     elif entry.kind == "symlink":
         if tree.get_symlink_target(file_id) == os.readlink(target_path):
             return True
@@ -2494,22 +2502,6 @@
         raise errors.BadFileKindError(name, kind)
 
 
-@deprecated_function(deprecated_in((1, 9, 0)))
-def create_by_entry(tt, entry, tree, trans_id, lines=None, mode_id=None):
-    """Create new file contents according to an inventory entry.
-
-    DEPRECATED.  Use create_from_tree instead.
-    """
-    if entry.kind == "file":
-        if lines is None:
-            lines = tree.get_file(entry.file_id).readlines()
-        tt.create_file(lines, trans_id, mode_id=mode_id)
-    elif entry.kind == "symlink":
-        tt.create_symlink(tree.get_symlink_target(entry.file_id), trans_id)
-    elif entry.kind == "directory":
-        tt.create_directory(trans_id)
-
-
 def create_from_tree(tt, trans_id, tree, file_id, bytes=None,
     filter_tree_path=None):
     """Create new file contents according to tree contents.
@@ -2900,11 +2892,13 @@
     def rename(self, from_, to):
         """Rename a file from one path to another."""
         try:
-            osutils.rename(from_, to)
+            os.rename(from_, to)
         except OSError, e:
             if e.errno in (errno.EEXIST, errno.ENOTEMPTY):
                 raise errors.FileExists(to, str(e))
-            raise
+            # normal OSError doesn't include filenames so it's hard to see where
+            # the problem is, see https://bugs.launchpad.net/bzr/+bug/491763
+            raise errors.TransformRenameFailed(from_, to, str(e), e.errno)
         self.past_renames.append((from_, to))
 
     def pre_delete(self, from_, to):
@@ -2920,7 +2914,10 @@
     def rollback(self):
         """Reverse all renames that have been performed"""
         for from_, to in reversed(self.past_renames):
-            osutils.rename(to, from_)
+            try:
+                os.rename(to, from_)
+            except OSError, e:
+                raise errors.TransformRenameFailed(to, from_, str(e), e.errno)                
         # after rollback, don't reuse _FileMover
         past_renames = None
         pending_deletions = None

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2010-04-16 07:56:51 +0000
+++ bzrlib/transport/__init__.py	2010-06-18 15:38:20 +0000
@@ -47,8 +47,6 @@
 """)
 
 from bzrlib.symbol_versioning import (
-        deprecated_method,
-        deprecated_function,
         DEPRECATED_PARAMETER,
         )
 from bzrlib.trace import (
@@ -539,6 +537,9 @@
 
         This function will only be defined for Transports which have a
         physical local filesystem representation.
+
+        :raises errors.NotLocalUrl: When no local path representation is
+            available.
         """
         raise errors.NotLocalUrl(self.abspath(relpath))
 
@@ -1285,6 +1286,12 @@
         # should be asked to ConnectedTransport only.
         return None
 
+    def disconnect(self):
+        # This is really needed for ConnectedTransport only, but it's easier to
+        # have Transport do nothing than testing that the disconnect should be
+        # asked to ConnectedTransport only.
+        pass
+
     def _redirected_to(self, source, target):
         """Returns a transport suitable to re-issue a redirected request.
 
@@ -1549,9 +1556,13 @@
             transport = self.__class__(other_base, _from_transport=self)
         return transport
 
-
-# We try to recognize an url lazily (ignoring user, password, etc)
-_urlRE = re.compile(r'^(?P<proto>[^:/\\]+)://(?P<rest>.*)$')
+    def disconnect(self):
+        """Disconnect the transport.
+
+        If and when required the transport willl reconnect automatically.
+        """
+        raise NotImplementedError(self.disconnect)
+
 
 def get_transport(base, possible_transports=None):
     """Open a transport to access a URL or directory.
@@ -1571,8 +1582,7 @@
     base = directories.dereference(base)
 
     def convert_path_to_url(base, error_str):
-        m = _urlRE.match(base)
-        if m:
+        if urlutils.is_url(base):
             # This looks like a URL, but we weren't able to
             # instantiate it as such raise an appropriate error
             # FIXME: we have a 'error_str' unused and we use last_err below
@@ -1739,33 +1749,28 @@
 register_lazy_transport('ftp://', 'bzrlib.transport.ftp', 'FtpTransport')
 register_transport_proto('aftp://', help="Access using active FTP.")
 register_lazy_transport('aftp://', 'bzrlib.transport.ftp', 'FtpTransport')
-
-try:
-    import kerberos
-    kerberos_available = True
-except ImportError:
-    kerberos_available = False
-
-if kerberos_available:
-    # Default to trying GSSAPI authentication (if the kerberos module is
-    # available)
-    register_transport_proto('ftp+gssapi://', register_netloc=True)
-    register_lazy_transport('ftp+gssapi://', 'bzrlib.transport.ftp._gssapi',
-                            'GSSAPIFtpTransport')
-    register_transport_proto('aftp+gssapi://', register_netloc=True)
-    register_lazy_transport('aftp+gssapi://', 'bzrlib.transport.ftp._gssapi',
-                            'GSSAPIFtpTransport')
-    register_transport_proto('ftp+nogssapi://', register_netloc=True)
-    register_transport_proto('aftp+nogssapi://', register_netloc=True)
-
-    register_lazy_transport('ftp://', 'bzrlib.transport.ftp._gssapi',
-                            'GSSAPIFtpTransport')
-    register_lazy_transport('aftp://', 'bzrlib.transport.ftp._gssapi',
-                            'GSSAPIFtpTransport')
-    register_lazy_transport('ftp+nogssapi://', 'bzrlib.transport.ftp',
-                            'FtpTransport')
-    register_lazy_transport('aftp+nogssapi://', 'bzrlib.transport.ftp',
-                            'FtpTransport')
+register_transport_proto('gio+', help="Access using any GIO supported protocols.")
+register_lazy_transport('gio+', 'bzrlib.transport.gio_transport', 'GioTransport')
+
+
+# Default to trying GSSAPI authentication (if the kerberos module is
+# available)
+register_transport_proto('ftp+gssapi://', register_netloc=True)
+register_transport_proto('aftp+gssapi://', register_netloc=True)
+register_transport_proto('ftp+nogssapi://', register_netloc=True)
+register_transport_proto('aftp+nogssapi://', register_netloc=True)
+register_lazy_transport('ftp+gssapi://', 'bzrlib.transport.ftp._gssapi',
+                        'GSSAPIFtpTransport')
+register_lazy_transport('aftp+gssapi://', 'bzrlib.transport.ftp._gssapi',
+                        'GSSAPIFtpTransport')
+register_lazy_transport('ftp://', 'bzrlib.transport.ftp._gssapi',
+                        'GSSAPIFtpTransport')
+register_lazy_transport('aftp://', 'bzrlib.transport.ftp._gssapi',
+                        'GSSAPIFtpTransport')
+register_lazy_transport('ftp+nogssapi://', 'bzrlib.transport.ftp',
+                        'FtpTransport')
+register_lazy_transport('aftp+nogssapi://', 'bzrlib.transport.ftp',
+                        'FtpTransport')
 
 register_transport_proto('memory://')
 register_lazy_transport('memory://', 'bzrlib.transport.memory',

=== modified file 'bzrlib/transport/decorator.py'
--- bzrlib/transport/decorator.py	2010-02-23 07:43:11 +0000
+++ bzrlib/transport/decorator.py	2010-06-20 11:18:38 +0000
@@ -20,10 +20,10 @@
 stub functions to allow other decorators to be written easily.
 """
 
-from bzrlib.transport import get_transport, Transport, Server
-
-
-class TransportDecorator(Transport):
+from bzrlib import transport
+
+
+class TransportDecorator(transport.Transport):
     """A no-change decorator for Transports.
 
     Subclasses of this are new transports that are based on an
@@ -50,7 +50,7 @@
                              (url, prefix))
         not_decorated_url = url[len(prefix):]
         if _decorated is None:
-            self._decorated = get_transport(not_decorated_url)
+            self._decorated = transport.get_transport(not_decorated_url)
         else:
             self._decorated = _decorated
         super(TransportDecorator, self).__init__(prefix + self._decorated.base)

=== modified file 'bzrlib/transport/ftp/__init__.py'
--- bzrlib/transport/ftp/__init__.py	2010-02-27 01:37:54 +0000
+++ bzrlib/transport/ftp/__init__.py	2010-08-30 07:42:12 +0000
@@ -40,6 +40,12 @@
     osutils,
     urlutils,
     )
+from bzrlib.symbol_versioning import (
+    DEPRECATED_PARAMETER,
+    deprecated_in,
+    deprecated_passed,
+    warn,
+    )
 from bzrlib.trace import mutter, warning
 from bzrlib.transport import (
     AppendBasedFileStream,
@@ -164,6 +170,11 @@
         connection, credentials = self._create_connection(credentials)
         self._set_connection(connection, credentials)
 
+    def disconnect(self):
+        connection = self._get_connection()
+        if connection is not None:
+            connection.close()
+
     def _translate_ftp_error(self, err, path, extra=None,
                               unknown_exc=FtpPathError):
         """Try to translate an ftplib exception to a bzrlib exception.
@@ -191,6 +202,9 @@
             or 'file/directory not found' in s # filezilla server
             # Microsoft FTP-Service RNFR reply if file not found
             or (s.startswith('550 ') and 'unable to rename to' in extra)
+            # if containing directory doesn't exist, suggested by
+            # <https://bugs.edge.launchpad.net/bzr/+bug/224373>
+            or (s.startswith('550 ') and "can't find folder" in s)
             ):
             raise errors.NoSuchFile(path, extra=extra)
         elif ('file exists' in s):
@@ -232,7 +246,7 @@
             mutter("FTP has not: %s: %s", abspath, e)
             return False
 
-    def get(self, relpath, decode=False, retries=0):
+    def get(self, relpath, decode=DEPRECATED_PARAMETER, retries=0):
         """Get the file at the given relative path.
 
         :param relpath: The relative path to the file
@@ -242,7 +256,10 @@
         We're meant to return a file-like object which bzr will
         then read from. For now we do this via the magic of StringIO
         """
-        # TODO: decode should be deprecated
+        if deprecated_passed(decode):
+            warn(deprecated_in((2,3,0)) %
+                 '"decode" parameter to FtpTransport.get()',
+                 DeprecationWarning, stacklevel=2)
         try:
             mutter("FTP get: %s", self._remote_path(relpath))
             f = self._get_FTP()
@@ -314,8 +331,9 @@
                     return len(bytes)
                 else:
                     return fp.counted_bytes
-            except (ftplib.error_temp,EOFError), e:
-                warning("Failure during ftp PUT. Deleting temporary file.")
+            except (ftplib.error_temp, EOFError), e:
+                warning("Failure during ftp PUT of %s: %s. Deleting temporary file."
+                    % (tmp_abspath, e, ))
                 try:
                     f.delete(tmp_abspath)
                 except:
@@ -328,8 +346,9 @@
                                        unknown_exc=errors.NoSuchFile)
         except ftplib.error_temp, e:
             if retries > _number_of_retries:
-                raise errors.TransportError("FTP temporary error during PUT %s. Aborting."
-                                     % self.abspath(relpath), orig_error=e)
+                raise errors.TransportError(
+                    "FTP temporary error during PUT %s: %s. Aborting."
+                    % (self.abspath(relpath), e), orig_error=e)
             else:
                 warning("FTP temporary error: %s. Retrying.", str(e))
                 self._reconnect()
@@ -350,7 +369,17 @@
         try:
             mutter("FTP mkd: %s", abspath)
             f = self._get_FTP()
-            f.mkd(abspath)
+            try:
+                f.mkd(abspath)
+            except ftplib.error_reply, e:
+                # <https://bugs.launchpad.net/bzr/+bug/224373> Microsoft FTP
+                # server returns "250 Directory created." which is kind of
+                # reasonable, 250 meaning "requested file action OK", but not what
+                # Python's ftplib expects.
+                if e[0][:3] == '250':
+                    pass
+                else:
+                    raise
             self._setmode(relpath, mode)
         except ftplib.error_perm, e:
             self._translate_ftp_error(e, abspath,

=== modified file 'bzrlib/transport/ftp/_gssapi.py'
--- bzrlib/transport/ftp/_gssapi.py	2009-10-06 08:24:14 +0000
+++ bzrlib/transport/ftp/_gssapi.py	2010-05-31 21:31:18 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007, 2008 Canonical Ltd
+# Copyright (C) 2008, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,15 +19,13 @@
 See RFC2228 for details.
 """
 
-import base64, ftplib, getpass, socket
+import base64, ftplib
 
 from bzrlib import (
-    config,
     errors,
     )
 from bzrlib.trace import mutter
 from bzrlib.transport.ftp import FtpTransport
-from bzrlib.transport import register_transport_proto, register_transport
 
 try:
     import kerberos

=== added file 'bzrlib/transport/gio_transport.py'
--- bzrlib/transport/gio_transport.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transport/gio_transport.py	2010-08-30 07:42:12 +0000
@@ -0,0 +1,600 @@
+# Copyright (C) 2010 Canonical Ltd.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+#
+# Author: Mattias Eriksson
+
+"""Implementation of Transport over gio.
+
+Written by Mattias Eriksson <snaggen@acc.umu.se> based on the ftp transport.
+
+It provides the gio+XXX:// protocols where XXX is any of the protocols
+supported by gio.
+"""
+from cStringIO import StringIO
+import getpass
+import os
+import random
+import socket
+import stat
+import urllib
+import time
+import sys
+import getpass
+import urlparse
+
+from bzrlib import (
+    config,
+    errors,
+    osutils,
+    urlutils,
+    debug,
+    ui,
+    )
+from bzrlib.symbol_versioning import (
+    DEPRECATED_PARAMETER,
+    deprecated_in,
+    deprecated_passed,
+    warn,
+    )
+from bzrlib.trace import mutter, warning
+from bzrlib.transport import (
+    FileStream,
+    ConnectedTransport,
+    _file_streams,
+    Server,
+    )
+
+from bzrlib.tests.test_server import TestServer
+
+try:
+    import glib
+except ImportError, e:
+    raise errors.DependencyNotPresent('glib', e)
+try:
+    import gio
+except ImportError, e:
+    raise errors.DependencyNotPresent('gio', e)
+
+
+class GioLocalURLServer(TestServer):
+    """A pretend server for local transports, using file:// urls.
+
+    Of course no actual server is required to access the local filesystem, so
+    this just exists to tell the test code how to get to it.
+    """
+
+    def start_server(self):
+        pass
+
+    def get_url(self):
+        """See Transport.Server.get_url."""
+        return "gio+" + urlutils.local_path_to_url('')
+
+
+class GioFileStream(FileStream):
+    """A file stream object returned by open_write_stream.
+
+    This version uses GIO to perform writes.
+    """
+
+    def __init__(self, transport, relpath):
+        FileStream.__init__(self, transport, relpath)
+        self.gio_file = transport._get_GIO(relpath)
+        self.stream = self.gio_file.create()
+
+    def _close(self):
+        self.stream.close()
+
+    def write(self, bytes):
+        try:
+            #Using pump_string_file seems to make things crash
+            osutils.pumpfile(StringIO(bytes), self.stream)
+        except gio.Error, e:
+            #self.transport._translate_gio_error(e,self.relpath)
+            raise errors.BzrError(str(e))
+
+
+class GioStatResult(object):
+
+    def __init__(self, f):
+        info = f.query_info('standard::size,standard::type')
+        self.st_size = info.get_size()
+        type = info.get_file_type()
+        if (type == gio.FILE_TYPE_REGULAR):
+            self.st_mode = stat.S_IFREG
+        elif type == gio.FILE_TYPE_DIRECTORY:
+            self.st_mode = stat.S_IFDIR
+
+
+class GioTransport(ConnectedTransport):
+    """This is the transport agent for gio+XXX:// access."""
+
+    def __init__(self, base, _from_transport=None):
+        """Initialize the GIO transport and make sure the url is correct."""
+
+        if not base.startswith('gio+'):
+            raise ValueError(base)
+
+        (scheme, netloc, path, params, query, fragment) = \
+                urlparse.urlparse(base[len('gio+'):], allow_fragments=False)
+        if '@' in netloc:
+            user, netloc = netloc.rsplit('@', 1)
+        #Seems it is not possible to list supported backends for GIO
+        #so a hardcoded list it is then.
+        gio_backends = ['dav', 'file', 'ftp', 'obex', 'sftp', 'ssh', 'smb']
+        if scheme not in gio_backends:
+            raise errors.InvalidURL(base,
+                    extra="GIO support is only available for " + \
+                    ', '.join(gio_backends))
+
+        #Remove the username and password from the url we send to GIO
+        #by rebuilding the url again.
+        u = (scheme, netloc, path, '', '', '')
+        self.url = urlparse.urlunparse(u)
+
+        # And finally initialize super
+        super(GioTransport, self).__init__(base,
+            _from_transport=_from_transport)
+
+    def _relpath_to_url(self, relpath):
+        full_url = urlutils.join(self.url, relpath)
+        if isinstance(full_url, unicode):
+            raise errors.InvalidURL(full_url)
+        return full_url
+
+    def _get_GIO(self, relpath):
+        """Return the ftplib.GIO instance for this object."""
+        # Ensures that a connection is established
+        connection = self._get_connection()
+        if connection is None:
+            # First connection ever
+            connection, credentials = self._create_connection()
+            self._set_connection(connection, credentials)
+        fileurl = self._relpath_to_url(relpath)
+        file = gio.File(fileurl)
+        return file
+
+    def _auth_cb(self, op, message, default_user, default_domain, flags):
+        #really use bzrlib.auth get_password for this
+        #or possibly better gnome-keyring?
+        auth = config.AuthenticationConfig()
+        (scheme, urluser, urlpassword, host, port, urlpath) = \
+           urlutils.parse_url(self.url)
+        user = None
+        if (flags & gio.ASK_PASSWORD_NEED_USERNAME and
+                flags & gio.ASK_PASSWORD_NEED_DOMAIN):
+            prompt = scheme.upper() + ' %(host)s DOMAIN\username'
+            user_and_domain = auth.get_user(scheme, host,
+                    port=port, ask=True, prompt=prompt)
+            (domain, user) = user_and_domain.split('\\', 1)
+            op.set_username(user)
+            op.set_domain(domain)
+        elif flags & gio.ASK_PASSWORD_NEED_USERNAME:
+            user = auth.get_user(scheme, host,
+                    port=port, ask=True)
+            op.set_username(user)
+        elif flags & gio.ASK_PASSWORD_NEED_DOMAIN:
+            #Don't know how common this case is, but anyway
+            #a DOMAIN and a username prompt should be the
+            #same so I will missuse the ui_factory get_username
+            #a little bit here.
+            prompt = scheme.upper() + ' %(host)s DOMAIN'
+            domain = ui.ui_factory.get_username(prompt=prompt)
+            op.set_domain(domain)
+
+        if flags & gio.ASK_PASSWORD_NEED_PASSWORD:
+            if user is None:
+                user = op.get_username()
+            password = auth.get_password(scheme, host,
+                    user, port=port)
+            op.set_password(password)
+        op.reply(gio.MOUNT_OPERATION_HANDLED)
+
+    def _mount_done_cb(self, obj, res):
+        try:
+            obj.mount_enclosing_volume_finish(res)
+            self.loop.quit()
+        except gio.Error, e:
+            self.loop.quit()
+            raise errors.BzrError("Failed to mount the given location: " + str(e));
+
+    def _create_connection(self, credentials=None):
+        if credentials is None:
+            user, password = self._user, self._password
+        else:
+            user, password = credentials
+
+        try:
+            connection = gio.File(self.url)
+            mount = None
+            try:
+                mount = connection.find_enclosing_mount()
+            except gio.Error, e:
+                if (e.code == gio.ERROR_NOT_MOUNTED):
+                    self.loop = glib.MainLoop()
+                    ui.ui_factory.show_message('Mounting %s using GIO' % \
+                            self.url)
+                    op = gio.MountOperation()
+                    if user:
+                        op.set_username(user)
+                    if password:
+                        op.set_password(password)
+                    op.connect('ask-password', self._auth_cb)
+                    m = connection.mount_enclosing_volume(op,
+                            self._mount_done_cb)
+                    self.loop.run()
+        except gio.Error, e:
+            raise errors.TransportError(msg="Error setting up connection:"
+                                        " %s" % str(e), orig_error=e)
+        return connection, (user, password)
+
+    def disconnect(self):
+        # FIXME: Nothing seems to be necessary here, which sounds a bit strange
+        # -- vila 20100601
+        pass
+
+    def _reconnect(self):
+        # FIXME: This doesn't seem to be used -- vila 20100601
+        """Create a new connection with the previously used credentials"""
+        credentials = self._get_credentials()
+        connection, credentials = self._create_connection(credentials)
+        self._set_connection(connection, credentials)
+
+    def _remote_path(self, relpath):
+        relative = urlutils.unescape(relpath).encode('utf-8')
+        remote_path = self._combine_paths(self._path, relative)
+        return remote_path
+
+    def has(self, relpath):
+        """Does the target location exist?"""
+        try:
+            if 'gio' in debug.debug_flags:
+                mutter('GIO has check: %s' % relpath)
+            f = self._get_GIO(relpath)
+            st = GioStatResult(f)
+            if stat.S_ISREG(st.st_mode) or stat.S_ISDIR(st.st_mode):
+                return True
+            return False
+        except gio.Error, e:
+            if e.code == gio.ERROR_NOT_FOUND:
+                return False
+            else:
+                self._translate_gio_error(e, relpath)
+
+    def get(self, relpath, decode=DEPRECATED_PARAMETER, retries=0):
+        """Get the file at the given relative path.
+
+        :param relpath: The relative path to the file
+        :param retries: Number of retries after temporary failures so far
+                        for this operation.
+
+        We're meant to return a file-like object which bzr will
+        then read from. For now we do this via the magic of StringIO
+        """
+        if deprecated_passed(decode):
+            warn(deprecated_in((2,3,0)) %
+                 '"decode" parameter to GioTransport.get()',
+                 DeprecationWarning, stacklevel=2)
+        try:
+            if 'gio' in debug.debug_flags:
+                mutter("GIO get: %s" % relpath)
+            f = self._get_GIO(relpath)
+            fin = f.read()
+            buf = fin.read()
+            fin.close()
+            ret = StringIO(buf)
+            return ret
+        except gio.Error, e:
+            #If we get a not mounted here it might mean
+            #that a bad path has been entered (or that mount failed)
+            if (e.code == gio.ERROR_NOT_MOUNTED):
+                raise errors.PathError(relpath,
+                  extra='Failed to get file, make sure the path is correct. ' \
+                  + str(e))
+            else:
+                self._translate_gio_error(e, relpath)
+
+    def put_file(self, relpath, fp, mode=None):
+        """Copy the file-like object into the location.
+
+        :param relpath: Location to put the contents, relative to base.
+        :param fp:       File-like or string object.
+        """
+        if 'gio' in debug.debug_flags:
+            mutter("GIO put_file %s" % relpath)
+        tmppath = '%s.tmp.%.9f.%d.%d' % (relpath, time.time(),
+                    os.getpid(), random.randint(0, 0x7FFFFFFF))
+        f = None
+        fout = None
+        try:
+            closed = True
+            try:
+                f = self._get_GIO(tmppath)
+                fout = f.create()
+                closed = False
+                length = self._pump(fp, fout)
+                fout.close()
+                closed = True
+                self.stat(tmppath)
+                dest = self._get_GIO(relpath)
+                f.move(dest, flags=gio.FILE_COPY_OVERWRITE)
+                f = None
+                if mode is not None:
+                    self._setmode(relpath, mode)
+                return length
+            except gio.Error, e:
+                self._translate_gio_error(e, relpath)
+        finally:
+            if not closed and fout is not None:
+                fout.close()
+            if f is not None and f.query_exists():
+                f.delete()
+
+    def mkdir(self, relpath, mode=None):
+        """Create a directory at the given path."""
+        try:
+            if 'gio' in debug.debug_flags:
+                mutter("GIO mkdir: %s" % relpath)
+            f = self._get_GIO(relpath)
+            f.make_directory()
+            self._setmode(relpath, mode)
+        except gio.Error, e:
+            self._translate_gio_error(e, relpath)
+
+    def open_write_stream(self, relpath, mode=None):
+        """See Transport.open_write_stream."""
+        if 'gio' in debug.debug_flags:
+            mutter("GIO open_write_stream %s" % relpath)
+        if mode is not None:
+            self._setmode(relpath, mode)
+        result = GioFileStream(self, relpath)
+        _file_streams[self.abspath(relpath)] = result
+        return result
+
+    def recommended_page_size(self):
+        """See Transport.recommended_page_size().
+
+        For FTP we suggest a large page size to reduce the overhead
+        introduced by latency.
+        """
+        if 'gio' in debug.debug_flags:
+            mutter("GIO recommended_page")
+        return 64 * 1024
+
+    def rmdir(self, relpath):
+        """Delete the directory at rel_path"""
+        try:
+            if 'gio' in debug.debug_flags:
+                mutter("GIO rmdir %s" % relpath)
+            st = self.stat(relpath)
+            if stat.S_ISDIR(st.st_mode):
+                f = self._get_GIO(relpath)
+                f.delete()
+            else:
+                raise errors.NotADirectory(relpath)
+        except gio.Error, e:
+            self._translate_gio_error(e, relpath)
+        except errors.NotADirectory, e:
+            #just pass it forward
+            raise e
+        except Exception, e:
+            mutter('failed to rmdir %s: %s' % (relpath, e))
+            raise errors.PathError(relpath)
+
+    def append_file(self, relpath, file, mode=None):
+        """Append the text in the file-like object into the final
+        location.
+        """
+        #GIO append_to seems not to append but to truncate
+        #Work around this.
+        if 'gio' in debug.debug_flags:
+            mutter("GIO append_file: %s" % relpath)
+        tmppath = '%s.tmp.%.9f.%d.%d' % (relpath, time.time(),
+                    os.getpid(), random.randint(0, 0x7FFFFFFF))
+        try:
+            result = 0
+            fo = self._get_GIO(tmppath)
+            fi = self._get_GIO(relpath)
+            fout = fo.create()
+            try:
+                info = GioStatResult(fi)
+                result = info.st_size
+                fin = fi.read()
+                self._pump(fin, fout)
+                fin.close()
+            #This separate except is to catch and ignore the
+            #gio.ERROR_NOT_FOUND for the already existing file.
+            #It is valid to open a non-existing file for append.
+            #This is caused by the broken gio append_to...
+            except gio.Error, e:
+                if e.code != gio.ERROR_NOT_FOUND:
+                    self._translate_gio_error(e, relpath)
+            length = self._pump(file, fout)
+            fout.close()
+            info = GioStatResult(fo)
+            if info.st_size != result + length:
+                raise errors.BzrError("Failed to append size after " \
+                      "(%d) is not original (%d) + written (%d) total (%d)" % \
+                      (info.st_size, result, length, result + length))
+            fo.move(fi, flags=gio.FILE_COPY_OVERWRITE)
+            return result
+        except gio.Error, e:
+            self._translate_gio_error(e, relpath)
+
+    def _setmode(self, relpath, mode):
+        """Set permissions on a path.
+
+        Only set permissions on Unix systems
+        """
+        if 'gio' in debug.debug_flags:
+            mutter("GIO _setmode %s" % relpath)
+        if mode:
+            try:
+                f = self._get_GIO(relpath)
+                f.set_attribute_uint32(gio.FILE_ATTRIBUTE_UNIX_MODE, mode)
+            except gio.Error, e:
+                if e.code == gio.ERROR_NOT_SUPPORTED:
+                    # Command probably not available on this server
+                    mutter("GIO Could not set permissions to %s on %s. %s",
+                        oct(mode), self._remote_path(relpath), str(e))
+                else:
+                    self._translate_gio_error(e, relpath)
+
+    def rename(self, rel_from, rel_to):
+        """Rename without special overwriting"""
+        try:
+            if 'gio' in debug.debug_flags:
+                mutter("GIO move (rename): %s => %s", rel_from, rel_to)
+            f = self._get_GIO(rel_from)
+            t = self._get_GIO(rel_to)
+            f.move(t)
+        except gio.Error, e:
+            self._translate_gio_error(e, rel_from)
+
+    def move(self, rel_from, rel_to):
+        """Move the item at rel_from to the location at rel_to"""
+        try:
+            if 'gio' in debug.debug_flags:
+                mutter("GIO move: %s => %s", rel_from, rel_to)
+            f = self._get_GIO(rel_from)
+            t = self._get_GIO(rel_to)
+            f.move(t, flags=gio.FILE_COPY_OVERWRITE)
+        except gio.Error, e:
+            self._translate_gio_error(e, relfrom)
+
+    def delete(self, relpath):
+        """Delete the item at relpath"""
+        try:
+            if 'gio' in debug.debug_flags:
+                mutter("GIO delete: %s", relpath)
+            f = self._get_GIO(relpath)
+            f.delete()
+        except gio.Error, e:
+            self._translate_gio_error(e, relpath)
+
+    def external_url(self):
+        """See bzrlib.transport.Transport.external_url."""
+        if 'gio' in debug.debug_flags:
+            mutter("GIO external_url", self.base)
+        # GIO external url
+        return self.base
+
+    def listable(self):
+        """See Transport.listable."""
+        if 'gio' in debug.debug_flags:
+            mutter("GIO listable")
+        return True
+
+    def list_dir(self, relpath):
+        """See Transport.list_dir."""
+        if 'gio' in debug.debug_flags:
+            mutter("GIO list_dir")
+        try:
+            entries = []
+            f = self._get_GIO(relpath)
+            children = f.enumerate_children(gio.FILE_ATTRIBUTE_STANDARD_NAME)
+            for child in children:
+                entries.append(urlutils.escape(child.get_name()))
+            return entries
+        except gio.Error, e:
+            self._translate_gio_error(e, relpath)
+
+    def iter_files_recursive(self):
+        """See Transport.iter_files_recursive.
+
+        This is cargo-culted from the SFTP transport"""
+        if 'gio' in debug.debug_flags:
+            mutter("GIO iter_files_recursive")
+        queue = list(self.list_dir("."))
+        while queue:
+            relpath = queue.pop(0)
+            st = self.stat(relpath)
+            if stat.S_ISDIR(st.st_mode):
+                for i, basename in enumerate(self.list_dir(relpath)):
+                    queue.insert(i, relpath + "/" + basename)
+            else:
+                yield relpath
+
+    def stat(self, relpath):
+        """Return the stat information for a file."""
+        try:
+            if 'gio' in debug.debug_flags:
+                mutter("GIO stat: %s", relpath)
+            f = self._get_GIO(relpath)
+            return GioStatResult(f)
+        except gio.Error, e:
+            self._translate_gio_error(e, relpath, extra='error w/ stat')
+
+    def lock_read(self, relpath):
+        """Lock the given file for shared (read) access.
+        :return: A lock object, which should be passed to Transport.unlock()
+        """
+        if 'gio' in debug.debug_flags:
+            mutter("GIO lock_read", relpath)
+
+        class BogusLock(object):
+            # The old RemoteBranch ignore lock for reading, so we will
+            # continue that tradition and return a bogus lock object.
+
+            def __init__(self, path):
+                self.path = path
+
+            def unlock(self):
+                pass
+
+        return BogusLock(relpath)
+
+    def lock_write(self, relpath):
+        """Lock the given file for exclusive (write) access.
+        WARNING: many transports do not support this, so trying avoid using it
+
+        :return: A lock object, whichshould be passed to Transport.unlock()
+        """
+        if 'gio' in debug.debug_flags:
+            mutter("GIO lock_write", relpath)
+        return self.lock_read(relpath)
+
+    def _translate_gio_error(self, err, path, extra=None):
+        if 'gio' in debug.debug_flags:
+            mutter("GIO Error: %s %s" % (str(err), path))
+        if extra is None:
+            extra = str(err)
+        if err.code == gio.ERROR_NOT_FOUND:
+            raise errors.NoSuchFile(path, extra=extra)
+        elif err.code == gio.ERROR_EXISTS:
+            raise errors.FileExists(path, extra=extra)
+        elif err.code == gio.ERROR_NOT_DIRECTORY:
+            raise errors.NotADirectory(path, extra=extra)
+        elif err.code == gio.ERROR_NOT_EMPTY:
+            raise errors.DirectoryNotEmpty(path, extra=extra)
+        elif err.code == gio.ERROR_BUSY:
+            raise errors.ResourceBusy(path, extra=extra)
+        elif err.code == gio.ERROR_PERMISSION_DENIED:
+            raise errors.PermissionDenied(path, extra=extra)
+        elif err.code == gio.ERROR_HOST_NOT_FOUND:
+            raise errors.PathError(path, extra=extra)
+        elif err.code == gio.ERROR_IS_DIRECTORY:
+            raise errors.PathError(path, extra=extra)
+        else:
+            mutter('unable to understand error for path: %s: %s', path, err)
+            raise errors.PathError(path,
+                    extra="Unhandled gio error: " + str(err))
+
+
+def get_test_permutations():
+    """Return the permutations to be used in testing."""
+    from bzrlib.tests import test_server
+    return [(GioTransport, GioLocalURLServer)]

=== modified file 'bzrlib/transport/http/__init__.py'
--- bzrlib/transport/http/__init__.py	2010-02-17 17:11:16 +0000
+++ bzrlib/transport/http/__init__.py	2010-06-01 13:01:20 +0000
@@ -629,6 +629,11 @@
         """
         pass
 
+    def disconnect(self):
+        """See SmartClientMedium.disconnect()."""
+        t = self._http_transport_ref()
+        t.disconnect()
+
 
 # TODO: May be better located in smart/medium.py with the other
 # SmartMediumRequest classes

=== modified file 'bzrlib/transport/http/_pycurl.py'
--- bzrlib/transport/http/_pycurl.py	2009-08-19 16:33:39 +0000
+++ bzrlib/transport/http/_pycurl.py	2010-06-01 13:01:20 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,16 +31,12 @@
 # from _curl_perform.  Not done because we may deprecate pycurl in the
 # future -- vila 20070212
 
-import os
 from cStringIO import StringIO
 import httplib
-import sys
 
 from bzrlib import (
     debug,
     errors,
-    trace,
-    __version__ as bzrlib_version,
     )
 import bzrlib
 from bzrlib.trace import mutter
@@ -132,6 +128,11 @@
             self._set_connection(connection, auth)
         return connection
 
+    def disconnect(self):
+        connection = self._get_connection()
+        if connection is not None:
+            connection.close()
+
     def has(self, relpath):
         """See Transport.has()"""
         # We set NO BODY=0 in _get_full, so it should be safe

=== modified file 'bzrlib/transport/http/_urllib.py'
--- bzrlib/transport/http/_urllib.py	2010-02-17 17:11:16 +0000
+++ bzrlib/transport/http/_urllib.py	2010-06-01 13:01:20 +0000
@@ -99,6 +99,11 @@
 
         return response
 
+    def disconnect(self):
+        connection = self._get_connection()
+        if connection is not None:
+            connection.close()
+
     def _get(self, relpath, offsets, tail_amount=0):
         """See HttpTransport._get"""
         abspath = self._remote_path(relpath)

=== modified file 'bzrlib/transport/http/_urllib2_wrappers.py'
--- bzrlib/transport/http/_urllib2_wrappers.py	2010-02-17 17:11:16 +0000
+++ bzrlib/transport/http/_urllib2_wrappers.py	2010-09-15 02:42:19 +0000
@@ -75,6 +75,26 @@
     )
 
 
+class addinfourl(urllib2.addinfourl):
+    '''Replacement addinfourl class compatible with python-2.7's xmlrpclib
+
+    In python-2.7, xmlrpclib expects that the response object that it receives
+    has a getheader method.  httplib.HTTPResponse provides this but
+    urllib2.addinfourl does not.  Add the necessary functions here, ported to
+    use the internal data structures of addinfourl.
+    '''
+
+    def getheader(self, name, default=None):
+        if self.headers is None:
+            raise httplib.ResponseNotReady()
+        return self.headers.getheader(name, default)
+
+    def getheaders(self):
+        if self.headers is None:
+            raise httplib.ResponseNotReady()
+        return self.headers.items()
+
+
 class _ReportingFileSocket(object):
 
     def __init__(self, filesock, report_activity=None):
@@ -378,6 +398,11 @@
             port = conn_class.default_port
         self.proxied_host = '%s:%s' % (host, port)
         urllib2.Request.set_proxy(self, proxy, type)
+        # When urllib2 makes a https request with our wrapper code and a proxy,
+        # it sets Host to the https proxy, not the host we want to talk to.
+        # I'm fairly sure this is our fault, but what is the cause is an open
+        # question. -- Robert Collins May 8 2010.
+        self.add_unredirected_header('Host', self.proxied_host)
 
 
 class _ConnectRequest(Request):
@@ -651,7 +676,7 @@
             r = response
             r.recv = r.read
             fp = socket._fileobject(r, bufsize=65536)
-            resp = urllib2.addinfourl(fp, r.msg, req.get_full_url())
+            resp = addinfourl(fp, r.msg, req.get_full_url())
             resp.code = r.status
             resp.msg = r.reason
             resp.version = r.version
@@ -711,7 +736,7 @@
             connect = _ConnectRequest(request)
             response = self.parent.open(connect)
             if response.code != 200:
-                raise ConnectionError("Can't connect to %s via proxy %s" % (
+                raise errors.ConnectionError("Can't connect to %s via proxy %s" % (
                         connect.proxied_host, self.host))
             # Housekeeping
             connection.cleanup_pipe()
@@ -868,21 +893,19 @@
                 print 'Will unbind %s_open for %r' % (type, proxy)
             delattr(self, '%s_open' % type)
 
+        def bind_scheme_request(proxy, scheme):
+            if proxy is None:
+                return
+            scheme_request = scheme + '_request'
+            if self._debuglevel >= 3:
+                print 'Will bind %s for %r' % (scheme_request, proxy)
+            setattr(self, scheme_request,
+                lambda request: self.set_proxy(request, scheme))
         # We are interested only by the http[s] proxies
         http_proxy = self.get_proxy_env_var('http')
+        bind_scheme_request(http_proxy, 'http')
         https_proxy = self.get_proxy_env_var('https')
-
-        if http_proxy is not None:
-            if self._debuglevel >= 3:
-                print 'Will bind http_request for %r' % http_proxy
-            setattr(self, 'http_request',
-                    lambda request: self.set_proxy(request, 'http'))
-
-        if https_proxy is not None:
-            if self._debuglevel >= 3:
-                print 'Will bind http_request for %r' % https_proxy
-            setattr(self, 'https_request',
-                    lambda request: self.set_proxy(request, 'https'))
+        bind_scheme_request(https_proxy, 'https')
 
     def get_proxy_env_var(self, name, default_to='all'):
         """Get a proxy env var.

=== modified file 'bzrlib/transport/local.py'
--- bzrlib/transport/local.py	2010-04-28 07:53:34 +0000
+++ bzrlib/transport/local.py	2010-07-19 20:09:53 +0000
@@ -99,7 +99,10 @@
          - relative_reference is url escaped.
         """
         if relative_reference in ('.', ''):
-            return self._local_base
+            # _local_base normally has a trailing slash; strip it so that stat
+            # on a transport pointing to a symlink reads the link not the
+            # referent but be careful of / and c:\
+            return osutils.split(self._local_base)[0]
         return self._local_base + urlutils.unescape(relative_reference)
 
     def abspath(self, relpath):
@@ -403,14 +406,11 @@
         try:
             # *don't* call bzrlib.osutils.rename, because we want to
             # detect conflicting names on rename, and osutils.rename tries to
-            # mask cross-platform differences there; however we do update the
-            # exception to include the filenames
+            # mask cross-platform differences there
             os.rename(path_from, path_to)
         except (IOError, OSError),e:
             # TODO: What about path_to?
-            self._translate_error(
-                osutils._add_rename_error_details(e, path_from, path_to),
-                path_from)
+            self._translate_error(e, path_from)
 
     def move(self, rel_from, rel_to):
         """Move the item at rel_from to the location at rel_to"""

=== modified file 'bzrlib/transport/remote.py'
--- bzrlib/transport/remote.py	2010-02-09 20:28:26 +0000
+++ bzrlib/transport/remote.py	2010-06-18 15:38:20 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -435,7 +435,9 @@
         remote._translate_error(err, path=relpath)
 
     def disconnect(self):
-        self.get_smart_medium().disconnect()
+        m = self.get_smart_medium()
+        if m is not None:
+            m.disconnect()
 
     def stat(self, relpath):
         resp = self._call2('stat', self._remote_path(relpath))
@@ -514,9 +516,9 @@
         if user is None:
             auth = config.AuthenticationConfig()
             user = auth.get_user('ssh', self._host, self._port)
-        client_medium = medium.SmartSSHClientMedium(self._host, self._port,
-            user, self._password, self.base,
-            bzr_remote_path=bzr_remote_path)
+        ssh_params = medium.SSHParams(self._host, self._port, user,
+            self._password, bzr_remote_path)
+        client_medium = medium.SmartSSHClientMedium(self.base, ssh_params)
         return client_medium, (user, self._password)
 
 

=== modified file 'bzrlib/transport/sftp.py'
--- bzrlib/transport/sftp.py	2010-03-04 02:43:41 +0000
+++ bzrlib/transport/sftp.py	2010-08-24 13:03:18 +0000
@@ -389,6 +389,11 @@
                                          self._host, self._port)
         return connection, (user, password)
 
+    def disconnect(self):
+        connection = self._get_connection()
+        if connection is not None:
+            connection.close()
+
     def _get_sftp(self):
         """Ensures that a connection is established"""
         connection = self._get_connection()
@@ -715,6 +720,8 @@
             if (e.args[0].startswith('Directory not empty: ')
                 or getattr(e, 'errno', None) == errno.ENOTEMPTY):
                 raise errors.DirectoryNotEmpty(path, str(e))
+            if e.args == ('Operation unsupported',):
+                raise errors.TransportNotPossible()
             mutter('Raising exception with args %s', e.args)
         if getattr(e, 'errno', None) is not None:
             mutter('Raising exception with errno %s', e.errno)

=== modified file 'bzrlib/transport/ssh.py'
--- bzrlib/transport/ssh.py	2010-03-02 06:41:36 +0000
+++ bzrlib/transport/ssh.py	2010-10-07 12:45:51 +0000
@@ -126,12 +126,15 @@
         elif 'SSH Secure Shell' in version:
             trace.mutter('ssh implementation is SSH Corp.')
             vendor = SSHCorpSubprocessVendor()
+        elif 'lsh' in version:
+            trace.mutter('ssh implementation is GNU lsh.')
+            vendor = LSHSubprocessVendor()
         # As plink user prompts are not handled currently, don't auto-detect
         # it by inspection below, but keep this vendor detection for if a path
         # is given in BZR_SSH. See https://bugs.launchpad.net/bugs/414743
         elif 'plink' in version and progname == 'plink':
             # Checking if "plink" was the executed argument as Windows
-            # sometimes reports 'ssh -V' incorrectly with 'plink' in it's
+            # sometimes reports 'ssh -V' incorrectly with 'plink' in its
             # version.  See https://bugs.launchpad.net/bzr/+bug/107155
             trace.mutter("ssh implementation is Putty's plink.")
             vendor = PLinkSubprocessVendor()
@@ -239,8 +242,7 @@
     def connect_ssh(self, username, password, host, port, command):
         """Make an SSH connection.
 
-        :returns: something with a `close` method, and a `get_filelike_channels`
-            method that returns a pair of (read, write) filelike objects.
+        :returns: an SSHConnection.
         """
         raise NotImplementedError(self.connect_ssh)
 
@@ -269,17 +271,6 @@
 register_ssh_vendor('loopback', LoopbackVendor())
 
 
-class _ParamikoSSHConnection(object):
-    def __init__(self, channel):
-        self.channel = channel
-
-    def get_filelike_channels(self):
-        return self.channel.makefile('rb'), self.channel.makefile('wb')
-
-    def close(self):
-        return self.channel.close()
-
-
 class ParamikoVendor(SSHVendor):
     """Vendor that uses paramiko."""
 
@@ -348,26 +339,37 @@
             self._raise_connection_error(host, port=port, orig_error=e,
                                          msg='Unable to invoke remote bzr')
 
+_ssh_connection_errors = (EOFError, OSError, IOError, socket.error)
 if paramiko is not None:
     vendor = ParamikoVendor()
     register_ssh_vendor('paramiko', vendor)
     register_ssh_vendor('none', vendor)
     register_default_ssh_vendor(vendor)
-    _sftp_connection_errors = (EOFError, paramiko.SSHException)
+    _ssh_connection_errors += (paramiko.SSHException,)
     del vendor
-else:
-    _sftp_connection_errors = (EOFError,)
 
 
 class SubprocessVendor(SSHVendor):
     """Abstract base class for vendors that use pipes to a subprocess."""
 
     def _connect(self, argv):
-        proc = subprocess.Popen(argv,
-                                stdin=subprocess.PIPE,
-                                stdout=subprocess.PIPE,
+        # Attempt to make a socketpair to use as stdin/stdout for the SSH
+        # subprocess.  We prefer sockets to pipes because they support
+        # non-blocking short reads, allowing us to optimistically read 64k (or
+        # whatever) chunks.
+        try:
+            my_sock, subproc_sock = socket.socketpair()
+        except (AttributeError, socket.error):
+            # This platform doesn't support socketpair(), so just use ordinary
+            # pipes instead.
+            stdin = stdout = subprocess.PIPE
+            sock = None
+        else:
+            stdin = stdout = subproc_sock
+            sock = my_sock
+        proc = subprocess.Popen(argv, stdin=stdin, stdout=stdout,
                                 **os_specific_subprocess_params())
-        return SSHSubprocess(proc)
+        return SSHSubprocessConnection(proc, sock=sock)
 
     def connect_sftp(self, username, password, host, port):
         try:
@@ -375,14 +377,7 @@
                                                   subsystem='sftp')
             sock = self._connect(argv)
             return SFTPClient(SocketAsChannelAdapter(sock))
-        except _sftp_connection_errors, e:
-            self._raise_connection_error(host, port=port, orig_error=e)
-        except (OSError, IOError), e:
-            # If the machine is fast enough, ssh can actually exit
-            # before we try and send it the sftp request, which
-            # raises a Broken Pipe
-            if e.errno not in (errno.EPIPE,):
-                raise
+        except _ssh_connection_errors, e:
             self._raise_connection_error(host, port=port, orig_error=e)
 
     def connect_ssh(self, username, password, host, port, command):
@@ -390,14 +385,7 @@
             argv = self._get_vendor_specific_argv(username, host, port,
                                                   command=command)
             return self._connect(argv)
-        except (EOFError), e:
-            self._raise_connection_error(host, port=port, orig_error=e)
-        except (OSError, IOError), e:
-            # If the machine is fast enough, ssh can actually exit
-            # before we try and send it the sftp request, which
-            # raises a Broken Pipe
-            if e.errno not in (errno.EPIPE,):
-                raise
+        except _ssh_connection_errors, e:
             self._raise_connection_error(host, port=port, orig_error=e)
 
     def _get_vendor_specific_argv(self, username, host, port, subsystem=None,
@@ -418,7 +406,7 @@
                                   command=None):
         args = [self.executable_path,
                 '-oForwardX11=no', '-oForwardAgent=no',
-                '-oClearAllForwardings=yes', '-oProtocol=2',
+                '-oClearAllForwardings=yes',
                 '-oNoHostAuthenticationForLocalhost=yes']
         if port is not None:
             args.extend(['-p', str(port)])
@@ -454,6 +442,27 @@
 register_ssh_vendor('sshcorp', SSHCorpSubprocessVendor())
 
 
+class LSHSubprocessVendor(SubprocessVendor):
+    """SSH vendor that uses the 'lsh' executable from GNU"""
+
+    executable_path = 'lsh'
+
+    def _get_vendor_specific_argv(self, username, host, port, subsystem=None,
+                                  command=None):
+        args = [self.executable_path]
+        if port is not None:
+            args.extend(['-p', str(port)])
+        if username is not None:
+            args.extend(['-l', username])
+        if subsystem is not None:
+            args.extend(['--subsystem', subsystem, host])
+        else:
+            args.extend([host] + command)
+        return args
+
+register_ssh_vendor('lsh', LSHSubprocessVendor())
+
+
 class PLinkSubprocessVendor(SubprocessVendor):
     """SSH vendor that uses the 'plink' executable from Putty."""
 
@@ -645,18 +654,64 @@
 _subproc_weakrefs = set()
 
 def _close_ssh_proc(proc):
-    for func in [proc.stdin.close, proc.stdout.close, proc.wait]:
-        try:
-            func()
+    """Carefully close stdin/stdout and reap the SSH process.
+
+    If the pipes are already closed and/or the process has already been
+    wait()ed on, that's ok, and no error is raised.  The goal is to do our best
+    to clean up (whether or not a clean up was already tried).
+    """
+    dotted_names = ['stdin.close', 'stdout.close', 'wait']
+    for dotted_name in dotted_names:
+        attrs = dotted_name.split('.')
+        try:
+            obj = proc
+            for attr in attrs:
+                obj = getattr(obj, attr)
+        except AttributeError:
+            # It's ok for proc.stdin or proc.stdout to be None.
+            continue
+        try:
+            obj()
         except OSError:
-            pass
-
-
-class SSHSubprocess(object):
-    """A socket-like object that talks to an ssh subprocess via pipes."""
-
-    def __init__(self, proc):
+            # It's ok for the pipe to already be closed, or the process to
+            # already be finished.
+            continue
+
+
+class SSHConnection(object):
+    """Abstract base class for SSH connections."""
+
+    def get_sock_or_pipes(self):
+        """Returns a (kind, io_object) pair.
+
+        If kind == 'socket', then io_object is a socket.
+
+        If kind == 'pipes', then io_object is a pair of file-like objects
+        (read_from, write_to).
+        """
+        raise NotImplementedError(self.get_sock_or_pipes)
+
+    def close(self):
+        raise NotImplementedError(self.close)
+
+
+class SSHSubprocessConnection(SSHConnection):
+    """A connection to an ssh subprocess via pipes or a socket.
+
+    This class is also socket-like enough to be used with
+    SocketAsChannelAdapter (it has 'send' and 'recv' methods).
+    """
+
+    def __init__(self, proc, sock=None):
+        """Constructor.
+
+        :param proc: a subprocess.Popen
+        :param sock: if proc.stdin/out is a socket from a socketpair, then sock
+            should bzrlib's half of that socketpair.  If not passed, proc's
+            stdin/out is assumed to be ordinary pipes.
+        """
         self.proc = proc
+        self._sock = sock
         # Add a weakref to proc that will attempt to do the same as self.close
         # to avoid leaving processes lingering indefinitely.
         def terminate(ref):
@@ -665,14 +720,37 @@
         _subproc_weakrefs.add(weakref.ref(self, terminate))
 
     def send(self, data):
-        return os.write(self.proc.stdin.fileno(), data)
+        if self._sock is not None:
+            return self._sock.send(data)
+        else:
+            return os.write(self.proc.stdin.fileno(), data)
 
     def recv(self, count):
-        return os.read(self.proc.stdout.fileno(), count)
+        if self._sock is not None:
+            return self._sock.recv(count)
+        else:
+            return os.read(self.proc.stdout.fileno(), count)
 
     def close(self):
         _close_ssh_proc(self.proc)
 
-    def get_filelike_channels(self):
-        return (self.proc.stdout, self.proc.stdin)
+    def get_sock_or_pipes(self):
+        if self._sock is not None:
+            return 'socket', self._sock
+        else:
+            return 'pipes', (self.proc.stdout, self.proc.stdin)
+
+
+class _ParamikoSSHConnection(SSHConnection):
+    """An SSH connection via paramiko."""
+
+    def __init__(self, channel):
+        self.channel = channel
+
+    def get_sock_or_pipes(self):
+        return ('socket', self.channel)
+
+    def close(self):
+        return self.channel.close()
+
 

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2010-04-30 11:03:59 +0000
+++ bzrlib/tree.py	2010-08-31 07:12:18 +0000
@@ -520,7 +520,7 @@
             parent_keys = [(file_id, self._file_revision(t, file_id)) for t in
                 self._iter_parent_trees()]
             vf.add_lines((file_id, last_revision), parent_keys,
-                         self.get_file(file_id).readlines())
+                         self.get_file_lines(file_id))
             repo = self.branch.repository
             base_vf = repo.texts
         else:
@@ -582,6 +582,10 @@
             yield child.file_id
 
     def lock_read(self):
+        """Lock this tree for multiple read only operations.
+        
+        :return: A bzrlib.lock.LogicalLockResult.
+        """
         pass
 
     def revision_tree(self, revision_id):
@@ -1126,7 +1130,7 @@
             if file_id in to_paths:
                 # already returned
                 continue
-            if file_id not in self.target.all_file_ids():
+            if not self.target.has_id(file_id):
                 # common case - paths we have not emitted are not present in
                 # target.
                 to_path = None

=== modified file 'bzrlib/treebuilder.py'
--- bzrlib/treebuilder.py	2009-03-23 14:59:43 +0000
+++ bzrlib/treebuilder.py	2010-06-17 07:11:55 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2008, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 
 """TreeBuilder helper class.
 
-TreeBuilders are used to build trees of various shapres or properties. This
+TreeBuilders are used to build trees of various shapes or properties. This
 can be extremely useful in testing for instance.
 """
 

=== modified file 'bzrlib/tuned_gzip.py'
--- bzrlib/tuned_gzip.py	2009-06-02 19:56:24 +0000
+++ bzrlib/tuned_gzip.py	2010-09-25 20:08:01 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 # Written by Robert Collins <robert.collins@canonical.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -395,4 +395,12 @@
         # (4 seconds to 1 seconds for the sample upgrades I was testing).
         self.write(''.join(lines))
 
+    if sys.version_info > (2, 7):
+        # As of Python 2.7 the crc32 must be positive when close is called
+        def close(self):
+            if self.fileobj is None:
+                return
+            if self.mode == gzip.WRITE:
+                self.crc &= 0xFFFFFFFFL
+            gzip.GzipFile.close(self)
 

=== modified file 'bzrlib/ui/__init__.py'
--- bzrlib/ui/__init__.py	2010-03-25 07:34:15 +0000
+++ bzrlib/ui/__init__.py	2010-09-14 06:46:18 +0000
@@ -100,12 +100,51 @@
     return val
 
 
+class ConfirmationUserInterfacePolicy(object):
+    """Wrapper for a UIFactory that allows or denies all confirmed actions."""
+
+    def __init__(self, wrapped_ui, default_answer, specific_answers):
+        """Generate a proxy UI that does no confirmations.
+
+        :param wrapped_ui: Underlying UIFactory.
+        :param default_answer: Bool for whether requests for
+            confirmation from the user should be noninteractively accepted or
+            denied.
+        :param specific_answers: Map from confirmation_id to bool answer.
+        """
+        self.wrapped_ui = wrapped_ui
+        self.default_answer = default_answer
+        self.specific_answers = specific_answers
+
+    def __getattr__(self, name):
+        return getattr(self.wrapped_ui, name)
+
+    def __repr__(self):
+        return '%s(%r, %r, %r)' % (
+            self.__class__.__name__,
+            self.wrapped_ui,
+            self.default_answer, 
+            self.specific_answers)
+
+    def confirm_action(self, prompt, confirmation_id, prompt_kwargs):
+        if confirmation_id in self.specific_answers:
+            return self.specific_answers[confirmation_id]
+        elif self.default_answer is not None:
+            return self.default_answer
+        else:
+            return self.wrapped_ui.confirm_action(
+                prompt, confirmation_id, prompt_kwargs)
+
+
 class UIFactory(object):
     """UI abstraction.
 
     This tells the library how to display things to the user.  Through this
     layer different applications can choose the style of UI.
 
+    UI Factories are also context managers, for some syntactic sugar some users
+    need.
+
     :ivar suppressed_warnings: Identifiers for user warnings that should 
         no be emitted.
     """
@@ -123,6 +162,23 @@
         self.suppressed_warnings = set()
         self._quiet = False
 
+    def __enter__(self):
+        """Context manager entry support.
+
+        Override in a concrete factory class if initialisation before use is
+        needed.
+        """
+        return self # This is bound to the 'as' clause in a with statement.
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        """Context manager exit support.
+
+        Override in a concrete factory class if more cleanup than a simple
+        self.clear_term() is needed when the UIFactory is finished with.
+        """
+        self.clear_term()
+        return False # propogate exceptions.
+
     def be_quiet(self, state):
         """Tell the UI to be more quiet, or not.
 
@@ -131,6 +187,24 @@
         """
         self._quiet = state
 
+    def confirm_action(self, prompt, confirmation_id, prompt_kwargs):
+        """Seek user confirmation for an action.
+
+        If the UI is noninteractive, or the user does not want to be asked
+        about this action, True is returned, indicating bzr should just
+        proceed.
+
+        The confirmation id allows the user to configure certain actions to
+        always be confirmed or always denied, and for UIs to specialize the
+        display of particular confirmations.
+
+        :param prompt: Suggested text to display to the user.
+        :param prompt_kwargs: A dictionary of arguments that can be
+            string-interpolated into the prompt.
+        :param confirmation_id: Unique string identifier for the confirmation.
+        """
+        return self.get_boolean(prompt % prompt_kwargs)
+
     def get_password(self, prompt='', **kwargs):
         """Prompt the user for a password.
 
@@ -158,8 +232,9 @@
         version of stdout, but in a GUI it might be appropriate to send it to a 
         window displaying the text.
      
-        :param encoding: Unicode encoding for output; default is the 
-            terminal encoding, which may be different from the user encoding.
+        :param encoding: Unicode encoding for output; if not specified 
+            uses the configured 'output_encoding' if any; otherwise the 
+            terminal encoding. 
             (See get_terminal_encoding.)
 
         :param encoding_type: How to handle encoding errors:
@@ -167,7 +242,11 @@
         """
         # XXX: is the caller supposed to close the resulting object?
         if encoding is None:
-            encoding = osutils.get_terminal_encoding()
+            from bzrlib import config
+            encoding = config.GlobalConfig().get_user_option(
+                'output_encoding')
+        if encoding is None:
+            encoding = osutils.get_terminal_encoding(trace=True)
         if encoding_type is None:
             encoding_type = 'replace'
         out_stream = self._make_output_stream_explicit(encoding, encoding_type)
@@ -352,8 +431,17 @@
                 "without an upgrade path.\n" % (inter.target._format,))
 
 
-
-class SilentUIFactory(UIFactory):
+class NoninteractiveUIFactory(UIFactory):
+    """Base class for UIs with no user."""
+
+    def confirm_action(self, prompt, confirmation_id, prompt_kwargs):
+        return True
+
+    def __repr__(self):
+        return '%s()' % (self.__class__.__name__, )
+
+
+class SilentUIFactory(NoninteractiveUIFactory):
     """A UI Factory which never prints anything.
 
     This is the default UI, if another one is never registered by a program
@@ -394,6 +482,9 @@
     def __repr__(self):
         return "%s(%r)" % (self.__class__.__name__, self.responses)
 
+    def confirm_action(self, prompt, confirmation_id, args):
+        return self.get_boolean(prompt % args)
+
     def get_boolean(self, prompt):
         return self.responses.pop(0)
 

=== modified file 'bzrlib/ui/text.py'
--- bzrlib/ui/text.py	2010-03-25 07:34:15 +0000
+++ bzrlib/ui/text.py	2010-08-05 08:18:06 +0000
@@ -37,8 +37,6 @@
 
 """)
 
-from bzrlib.osutils import watch_sigwinch
-
 from bzrlib.ui import (
     UIFactory,
     NullProgressView,
@@ -62,8 +60,6 @@
         self.stderr = stderr
         # paints progress, network activity, etc
         self._progress_view = self.make_progress_view()
-        # hook up the signals to watch for terminal size changes
-        watch_sigwinch()
 
     def be_quiet(self, state):
         if state and not self._quiet:
@@ -157,7 +153,7 @@
         """Construct and return a new ProgressView subclass for this UI.
         """
         # with --quiet, never any progress view
-        # <https://bugs.edge.launchpad.net/bzr/+bug/320035>.  Otherwise if the
+        # <https://bugs.launchpad.net/bzr/+bug/320035>.  Otherwise if the
         # user specifically requests either text or no progress bars, always
         # do that.  otherwise, guess based on $TERM and tty presence.
         if self.is_quiet():
@@ -233,6 +229,9 @@
 
     def show_warning(self, msg):
         self.clear_term()
+        if isinstance(msg, unicode):
+            te = osutils.get_terminal_encoding()
+            msg = msg.encode(te, 'replace')
         self.stderr.write("bzr: warning: %s\n" % msg)
 
     def _progress_updated(self, task):
@@ -301,12 +300,18 @@
         # correspond reliably to overall command progress
         self.enable_bar = False
 
+    def _avail_width(self):
+        # we need one extra space for terminals that wrap on last char
+        w = osutils.terminal_width() 
+        if w is None:
+            return None
+        else:
+            return w - 1
+
     def _show_line(self, s):
         # sys.stderr.write("progress %r\n" % s)
-        width = osutils.terminal_width()
+        width = self._avail_width()
         if width is not None:
-            # we need one extra space for terminals that wrap on last char
-            width = width - 1
             s = '%-*.*s' % (width, width, s)
         self._term_file.write('\r' + s + '\r')
 
@@ -349,6 +354,10 @@
             return ''
 
     def _format_task(self, task):
+        """Format task-specific parts of progress bar.
+
+        :returns: (text_part, counter_part) both unicode strings.
+        """
         if not task.show_count:
             s = ''
         elif task.current_cnt is not None and task.total_cnt is not None:
@@ -364,21 +373,39 @@
             t = t._parent_task
             if t.msg:
                 m = t.msg + ':' + m
-        return m + s
+        return m, s
 
     def _render_line(self):
         bar_string = self._render_bar()
         if self._last_task:
-            task_msg = self._format_task(self._last_task)
+            task_part, counter_part = self._format_task(self._last_task)
         else:
-            task_msg = ''
+            task_part = counter_part = ''
         if self._last_task and not self._last_task.show_transport_activity:
             trans = ''
         else:
             trans = self._last_transport_msg
-            if trans:
-                trans += ' | '
-        return (bar_string + trans + task_msg)
+        # the bar separates the transport activity from the message, so even
+        # if there's no bar or spinner, we must show something if both those
+        # fields are present
+        if (task_part or trans) and not bar_string:
+            bar_string = '| '
+        # preferentially truncate the task message if we don't have enough
+        # space
+        avail_width = self._avail_width()
+        if avail_width is not None:
+            # if terminal avail_width is unknown, don't truncate
+            current_len = len(bar_string) + len(trans) + len(task_part) + len(counter_part)
+            gap = current_len - avail_width
+            if gap > 0:
+                task_part = task_part[:-gap-2] + '..'
+        s = trans + bar_string + task_part + counter_part
+        if avail_width is not None:
+            if len(s) < avail_width:
+                s = s.ljust(avail_width)
+            elif len(s) > avail_width:
+                s = s[:avail_width]
+        return s
 
     def _repaint(self):
         s = self._render_line()
@@ -440,7 +467,7 @@
             rate = (self._bytes_since_update
                     / (now - self._transport_update_time))
             # using base-10 units (see HACKING.txt).
-            msg = ("%6dkB %5dkB/s" %
+            msg = ("%6dkB %5dkB/s " %
                     (self._total_byte_count / 1000, int(rate) / 1000,))
             self._transport_update_time = now
             self._last_repaint = now

=== modified file 'bzrlib/urlutils.py'
--- bzrlib/urlutils.py	2010-02-17 17:11:16 +0000
+++ bzrlib/urlutils.py	2010-06-15 04:32:49 +0000
@@ -101,7 +101,12 @@
     first_path_slash = path.find('/')
     if first_path_slash == -1:
         return len(scheme), None
-    return len(scheme), first_path_slash+len(scheme)+3
+    return len(scheme), first_path_slash+m.start('path')
+
+
+def is_url(url):
+    """Tests whether a URL is in actual fact a URL."""
+    return _url_scheme_re.match(url) is not None
 
 
 def join(base, *args):
@@ -118,67 +123,26 @@
     """
     if not args:
         return base
-    match = _url_scheme_re.match(base)
-    scheme = None
-    if match:
-        scheme = match.group('scheme')
-        path = match.group('path').split('/')
-        if path[-1:] == ['']:
-            # Strip off a trailing slash
-            # This helps both when we are at the root, and when
-            # 'base' has an extra slash at the end
-            path = path[:-1]
-    else:
-        path = base.split('/')
-
-    if scheme is not None and len(path) >= 1:
-        host = path[:1]
-        # the path should be represented as an abs path.
-        # we know this must be absolute because of the presence of a URL scheme.
-        remove_root = True
-        path = [''] + path[1:]
-    else:
-        # create an empty host, but dont alter the path - this might be a
-        # relative url fragment.
-        host = []
-        remove_root = False
-
+    scheme_end, path_start = _find_scheme_and_separator(base)
+    if scheme_end is None and path_start is None:
+        path_start = 0
+    elif path_start is None:
+        path_start = len(base)
+    path = base[path_start:]
     for arg in args:
-        match = _url_scheme_re.match(arg)
-        if match:
-            # Absolute URL
-            scheme = match.group('scheme')
-            # this skips .. normalisation, making http://host/../../..
-            # be rather strange.
-            path = match.group('path').split('/')
-            # set the host and path according to new absolute URL, discarding
-            # any previous values.
-            # XXX: duplicates mess from earlier in this function.  This URL
-            # manipulation code needs some cleaning up.
-            if scheme is not None and len(path) >= 1:
-                host = path[:1]
-                path = path[1:]
-                # url scheme implies absolute path.
-                path = [''] + path
-            else:
-                # no url scheme we take the path as is.
-                host = []
+        arg_scheme_end, arg_path_start = _find_scheme_and_separator(arg)
+        if arg_scheme_end is None and arg_path_start is None:
+            arg_path_start = 0
+        elif arg_path_start is None:
+            arg_path_start = len(arg)
+        if arg_scheme_end is not None:
+            base = arg
+            path = arg[arg_path_start:]
+            scheme_end = arg_scheme_end
+            path_start = arg_path_start
         else:
-            path = '/'.join(path)
             path = joinpath(path, arg)
-            path = path.split('/')
-    if remove_root and path[0:1] == ['']:
-        del path[0]
-    if host:
-        # Remove the leading slash from the path, so long as it isn't also the
-        # trailing slash, which we want to keep if present.
-        if path and path[0] == '' and len(path) > 1:
-            del path[0]
-        path = host + path
-
-    if scheme is None:
-        return '/'.join(path)
-    return scheme + '://' + '/'.join(path)
+    return base[:path_start] + path
 
 
 def joinpath(base, *args):
@@ -278,7 +242,7 @@
     # on non-win32 platform
     # FIXME: It turns out that on nt, ntpath.abspath uses nt._getfullpathname
     #       which actually strips trailing space characters.
-    #       The worst part is that under linux ntpath.abspath has different
+    #       The worst part is that on linux ntpath.abspath has different
     #       semantics, since 'nt' is not an available module.
     if path == '/':
         return 'file:///'
@@ -303,7 +267,7 @@
     MIN_ABS_FILEURL_LENGTH = WIN32_MIN_ABS_FILEURL_LENGTH
 
 
-_url_scheme_re = re.compile(r'^(?P<scheme>[^:/]{2,})://(?P<path>.*)$')
+_url_scheme_re = re.compile(r'^(?P<scheme>[^:/]{2,}):(//)?(?P<path>.*)$')
 _url_hex_escapes_re = re.compile(r'(%[0-9a-fA-F]{2})')
 
 
@@ -339,18 +303,18 @@
     :param url: Either a hybrid URL or a local path
     :return: A normalized URL which only includes 7-bit ASCII characters.
     """
-    m = _url_scheme_re.match(url)
-    if not m:
+    scheme_end, path_start = _find_scheme_and_separator(url)
+    if scheme_end is None:
         return local_path_to_url(url)
-    scheme = m.group('scheme')
-    path = m.group('path')
+    prefix = url[:path_start]
+    path = url[path_start:]
     if not isinstance(url, unicode):
         for c in url:
             if c not in _url_safe_characters:
                 raise errors.InvalidURL(url, 'URLs can only contain specific'
                                             ' safe characters (not %r)' % c)
         path = _url_hex_escapes_re.sub(_unescape_safe_chars, path)
-        return str(scheme + '://' + ''.join(path))
+        return str(prefix + ''.join(path))
 
     # We have a unicode (hybrid) url
     path_chars = list(path)
@@ -362,7 +326,7 @@
                 ['%%%02X' % ord(c) for c in path_chars[i].encode('utf-8')])
     path = ''.join(path_chars)
     path = _url_hex_escapes_re.sub(_unescape_safe_chars, path)
-    return str(scheme + '://' + path)
+    return str(prefix + path)
 
 
 def relative_url(base, other):
@@ -469,6 +433,78 @@
     return url_base + head, tail
 
 
+def split_segment_parameters_raw(url):
+    """Split the subsegment of the last segment of a URL.
+
+    :param url: A relative or absolute URL
+    :return: (url, subsegments)
+    """
+    (parent_url, child_dir) = split(url)
+    subsegments = child_dir.split(",")
+    if len(subsegments) == 1:
+        return (url, [])
+    return (join(parent_url, subsegments[0]), subsegments[1:])
+
+
+def split_segment_parameters(url):
+    """Split the segment parameters of the last segment of a URL.
+
+    :param url: A relative or absolute URL
+    :return: (url, segment_parameters)
+    """
+    (base_url, subsegments) = split_segment_parameters_raw(url)
+    parameters = {}
+    for subsegment in subsegments:
+        (key, value) = subsegment.split("=", 1)
+        parameters[key] = value
+    return (base_url, parameters)
+
+
+def join_segment_parameters_raw(base, *subsegments):
+    """Create a new URL by adding subsegments to an existing one. 
+
+    This adds the specified subsegments to the last path in the specified
+    base URL. The subsegments should be bytestrings.
+
+    :note: You probably want to use join_segment_parameters instead.
+    """
+    if not subsegments:
+        return base
+    for subsegment in subsegments:
+        if type(subsegment) is not str:
+            raise TypeError("Subsegment %r is not a bytestring" % subsegment)
+        if "," in subsegment:
+            raise errors.InvalidURLJoin(", exists in subsegments",
+                                        base, subsegments)
+    return ",".join((base,) + subsegments)
+
+
+def join_segment_parameters(url, parameters):
+    """Create a new URL by adding segment parameters to an existing one.
+
+    The parameters of the last segment in the URL will be updated; if a
+    parameter with the same key already exists it will be overwritten.
+
+    :param url: A URL, as string
+    :param parameters: Dictionary of parameters, keys and values as bytestrings
+    """
+    (base, existing_parameters) = split_segment_parameters(url)
+    new_parameters = {}
+    new_parameters.update(existing_parameters)
+    for key, value in parameters.iteritems():
+        if type(key) is not str:
+            raise TypeError("parameter key %r is not a bytestring" % key)
+        if type(value) is not str:
+            raise TypeError("parameter value %r for %s is not a bytestring" %
+                (key, value))
+        if "=" in key:
+            raise errors.InvalidURLJoin("= exists in parameter key", url,
+                parameters)
+        new_parameters[key] = value
+    return join_segment_parameters_raw(base, 
+        *["%s=%s" % item for item in sorted(new_parameters.items())])
+
+
 def _win32_strip_local_trailing_slash(url):
     """Strip slashes after the drive letter"""
     if len(url) > WIN32_MIN_ABS_FILEURL_LENGTH:

=== modified file 'bzrlib/util/_bencode_py.py'
--- bzrlib/util/_bencode_py.py	2009-12-01 19:44:12 +0000
+++ bzrlib/util/_bencode_py.py	2010-05-03 20:57:39 +0000
@@ -90,8 +90,9 @@
             raise TypeError
         try:
             r, l = self.decode_func[x[0]](x, 0)
-        except (IndexError, KeyError):
-            raise ValueError
+        except (IndexError, KeyError, OverflowError), e:
+            import sys
+            raise ValueError, ValueError(str(e)), sys.exc_info()[2]
         if l != len(x):
             raise ValueError
         return r

=== modified file 'bzrlib/version_info_formats/__init__.py'
--- bzrlib/version_info_formats/__init__.py	2009-03-24 01:53:42 +0000
+++ bzrlib/version_info_formats/__init__.py	2010-07-15 13:41:31 +0000
@@ -20,9 +20,6 @@
 
 from bzrlib.osutils import local_time_offset, format_date
 from bzrlib import registry
-from bzrlib.symbol_versioning import (
-    deprecated_function,
-    )
 
 
 def create_date_str(timestamp=None, offset=None):

=== modified file 'bzrlib/versionedfile.py'
--- bzrlib/versionedfile.py	2010-04-30 11:03:59 +0000
+++ bzrlib/versionedfile.py	2010-08-28 15:34:24 +0000
@@ -46,7 +46,6 @@
 from bzrlib.transport.memory import MemoryTransport
 """)
 from bzrlib.registry import Registry
-from bzrlib.symbol_versioning import *
 from bzrlib.textmerge import TextMerge
 from bzrlib import bencode
 
@@ -206,6 +205,138 @@
             yield record
 
 
+class _MPDiffGenerator(object):
+    """Pull out the functionality for generating mp_diffs."""
+
+    def __init__(self, vf, keys):
+        self.vf = vf
+        # This is the order the keys were requested in
+        self.ordered_keys = tuple(keys)
+        # keys + their parents, what we need to compute the diffs
+        self.needed_keys = ()
+        # Map from key: mp_diff
+        self.diffs = {}
+        # Map from key: parents_needed (may have ghosts)
+        self.parent_map = {}
+        # Parents that aren't present
+        self.ghost_parents = ()
+        # Map from parent_key => number of children for this text
+        self.refcounts = {}
+        # Content chunks that are cached while we still need them
+        self.chunks = {}
+
+    def _find_needed_keys(self):
+        """Find the set of keys we need to request.
+
+        This includes all the original keys passed in, and the non-ghost
+        parents of those keys.
+
+        :return: (needed_keys, refcounts)
+            needed_keys is the set of all texts we need to extract
+            refcounts is a dict of {key: num_children} letting us know when we
+                no longer need to cache a given parent text
+        """
+        # All the keys and their parents
+        needed_keys = set(self.ordered_keys)
+        parent_map = self.vf.get_parent_map(needed_keys)
+        self.parent_map = parent_map
+        # TODO: Should we be using a different construct here? I think this
+        #       uses difference_update internally, and we expect the result to
+        #       be tiny
+        missing_keys = needed_keys.difference(parent_map)
+        if missing_keys:
+            raise errors.RevisionNotPresent(list(missing_keys)[0], self.vf)
+        # Parents that might be missing. They are allowed to be ghosts, but we
+        # should check for them
+        refcounts = {}
+        setdefault = refcounts.setdefault
+        just_parents = set()
+        for child_key, parent_keys in parent_map.iteritems():
+            if not parent_keys:
+                # parent_keys may be None if a given VersionedFile claims to
+                # not support graph operations.
+                continue
+            just_parents.update(parent_keys)
+            needed_keys.update(parent_keys)
+            for p in parent_keys:
+                refcounts[p] = setdefault(p, 0) + 1
+        just_parents.difference_update(parent_map)
+        # Remove any parents that are actually ghosts from the needed set
+        self.present_parents = set(self.vf.get_parent_map(just_parents))
+        self.ghost_parents = just_parents.difference(self.present_parents)
+        needed_keys.difference_update(self.ghost_parents)
+        self.needed_keys = needed_keys
+        self.refcounts = refcounts
+        return needed_keys, refcounts
+
+    def _compute_diff(self, key, parent_lines, lines):
+        """Compute a single mp_diff, and store it in self._diffs"""
+        if len(parent_lines) > 0:
+            # XXX: _extract_blocks is not usefully defined anywhere...
+            #      It was meant to extract the left-parent diff without
+            #      having to recompute it for Knit content (pack-0.92,
+            #      etc). That seems to have regressed somewhere
+            left_parent_blocks = self.vf._extract_blocks(key,
+                parent_lines[0], lines)
+        else:
+            left_parent_blocks = None
+        diff = multiparent.MultiParent.from_lines(lines,
+                    parent_lines, left_parent_blocks)
+        self.diffs[key] = diff
+
+    def _process_one_record(self, key, this_chunks):
+        parent_keys = None
+        if key in self.parent_map:
+            # This record should be ready to diff, since we requested
+            # content in 'topological' order
+            parent_keys = self.parent_map.pop(key)
+            # If a VersionedFile claims 'no-graph' support, then it may return
+            # None for any parent request, so we replace it with an empty tuple
+            if parent_keys is None:
+                parent_keys = ()
+            parent_lines = []
+            for p in parent_keys:
+                # Alternatively we could check p not in self.needed_keys, but
+                # ghost_parents should be tiny versus huge
+                if p in self.ghost_parents:
+                    continue
+                refcount = self.refcounts[p]
+                if refcount == 1: # Last child reference
+                    self.refcounts.pop(p)
+                    parent_chunks = self.chunks.pop(p)
+                else:
+                    self.refcounts[p] = refcount - 1
+                    parent_chunks = self.chunks[p]
+                p_lines = osutils.chunks_to_lines(parent_chunks)
+                # TODO: Should we cache the line form? We did the
+                #       computation to get it, but storing it this way will
+                #       be less memory efficient...
+                parent_lines.append(p_lines)
+                del p_lines
+            lines = osutils.chunks_to_lines(this_chunks)
+            # Since we needed the lines, we'll go ahead and cache them this way
+            this_chunks = lines
+            self._compute_diff(key, parent_lines, lines)
+            del lines
+        # Is this content required for any more children?
+        if key in self.refcounts:
+            self.chunks[key] = this_chunks
+
+    def _extract_diffs(self):
+        needed_keys, refcounts = self._find_needed_keys()
+        for record in self.vf.get_record_stream(needed_keys,
+                                                'topological', True):
+            if record.storage_kind == 'absent':
+                raise errors.RevisionNotPresent(record.key, self.vf)
+            self._process_one_record(record.key,
+                                     record.get_bytes_as('chunked'))
+        
+    def compute_diffs(self):
+        self._extract_diffs()
+        dpop = self.diffs.pop
+        return [dpop(k) for k in self.ordered_keys]
+
+
 class VersionedFile(object):
     """Versioned text file storage.
 
@@ -348,6 +479,10 @@
 
     def make_mpdiffs(self, version_ids):
         """Create multiparent diffs for specified versions."""
+        # XXX: Can't use _MPDiffGenerator just yet. This is because version_ids
+        #      is a list of strings, not keys. And while self.get_record_stream
+        #      is supported, it takes *keys*, while self.get_parent_map() takes
+        #      strings... *sigh*
         knit_versions = set()
         knit_versions.update(version_ids)
         parent_map = self.get_parent_map(version_ids)
@@ -1047,43 +1182,11 @@
 
     def make_mpdiffs(self, keys):
         """Create multiparent diffs for specified keys."""
-        keys_order = tuple(keys)
-        keys = frozenset(keys)
-        knit_keys = set(keys)
-        parent_map = self.get_parent_map(keys)
-        for parent_keys in parent_map.itervalues():
-            if parent_keys:
-                knit_keys.update(parent_keys)
-        missing_keys = keys - set(parent_map)
-        if missing_keys:
-            raise errors.RevisionNotPresent(list(missing_keys)[0], self)
-        # We need to filter out ghosts, because we can't diff against them.
-        maybe_ghosts = knit_keys - keys
-        ghosts = maybe_ghosts - set(self.get_parent_map(maybe_ghosts))
-        knit_keys.difference_update(ghosts)
-        lines = {}
-        chunks_to_lines = osutils.chunks_to_lines
-        for record in self.get_record_stream(knit_keys, 'topological', True):
-            lines[record.key] = chunks_to_lines(record.get_bytes_as('chunked'))
-            # line_block_dict = {}
-            # for parent, blocks in record.extract_line_blocks():
-            #   line_blocks[parent] = blocks
-            # line_blocks[record.key] = line_block_dict
-        diffs = []
-        for key in keys_order:
-            target = lines[key]
-            parents = parent_map[key] or []
-            # Note that filtering knit_keys can lead to a parent difference
-            # between the creation and the application of the mpdiff.
-            parent_lines = [lines[p] for p in parents if p in knit_keys]
-            if len(parent_lines) > 0:
-                left_parent_blocks = self._extract_blocks(key, parent_lines[0],
-                    target)
-            else:
-                left_parent_blocks = None
-            diffs.append(multiparent.MultiParent.from_lines(target,
-                parent_lines, left_parent_blocks))
-        return diffs
+        generator = _MPDiffGenerator(self, keys)
+        return generator.compute_diffs()
+
+    def get_annotator(self):
+        return annotate.Annotator(self)
 
     missing_keys = index._missing_keys_from_parent_map
 
@@ -1159,9 +1262,6 @@
             result.append((prefix + (origin,), line))
         return result
 
-    def get_annotator(self):
-        return annotate.Annotator(self)
-
     def check(self, progress_bar=None, keys=None):
         """See VersionedFiles.check()."""
         # XXX: This is over-enthusiastic but as we only thunk for Weaves today
@@ -1624,6 +1724,46 @@
                 yield (l, key)
 
 
+class NoDupeAddLinesDecorator(object):
+    """Decorator for a VersionedFiles that skips doing an add_lines if the key
+    is already present.
+    """
+
+    def __init__(self, store):
+        self._store = store
+
+    def add_lines(self, key, parents, lines, parent_texts=None,
+            left_matching_blocks=None, nostore_sha=None, random_id=False,
+            check_content=True):
+        """See VersionedFiles.add_lines.
+        
+        This implementation may return None as the third element of the return
+        value when the original store wouldn't.
+        """
+        if nostore_sha:
+            raise NotImplementedError(
+                "NoDupeAddLinesDecorator.add_lines does not implement the "
+                "nostore_sha behaviour.")
+        if key[-1] is None:
+            sha1 = osutils.sha_strings(lines)
+            key = ("sha1:" + sha1,)
+        else:
+            sha1 = None
+        if key in self._store.get_parent_map([key]):
+            # This key has already been inserted, so don't do it again.
+            if sha1 is None:
+                sha1 = osutils.sha_strings(lines)
+            return sha1, sum(map(len, lines)), None
+        return self._store.add_lines(key, parents, lines,
+                parent_texts=parent_texts,
+                left_matching_blocks=left_matching_blocks,
+                nostore_sha=nostore_sha, random_id=random_id,
+                check_content=check_content)
+
+    def __getattr__(self, name):
+        return getattr(self._store, name)
+
+
 def network_bytes_to_kind_and_offset(network_bytes):
     """Strip of a record kind from the front of network_bytes.
 

=== modified file 'bzrlib/weavefile.py'
--- bzrlib/weavefile.py	2009-03-23 14:59:43 +0000
+++ bzrlib/weavefile.py	2010-05-31 21:31:18 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -118,7 +118,10 @@
 
     from weave import WeaveFormatError
 
-    lines = iter(f.readlines())
+    try:
+        lines = iter(f.readlines())
+    finally:
+        f.close()
 
     try:
         l = lines.next()

=== modified file 'bzrlib/win32utils.py'
--- bzrlib/win32utils.py	2010-04-30 11:03:59 +0000
+++ bzrlib/win32utils.py	2010-06-22 00:32:37 +0000
@@ -522,7 +522,7 @@
             trace.mutter('Unable to set hidden attribute on %r: %s', path, e)
 
 
-def _command_line_to_argv(command_line, single_quotes_allowed=False):
+def _command_line_to_argv(command_line, argv, single_quotes_allowed=False):
     """Convert a Unicode command line into a list of argv arguments.
 
     It performs wildcard expansion to make wildcards act closer to how they
@@ -535,12 +535,27 @@
                                   default.
     :return: A list of unicode strings.
     """
+    # First, spit the command line
     s = cmdline.Splitter(command_line, single_quotes_allowed=single_quotes_allowed)
-    # Now that we've split the content, expand globs if necessary
+    
+    # Bug #587868 Now make sure that the length of s agrees with sys.argv 
+    # we do this by simply counting the number of arguments in each. The counts should 
+    # agree no matter what encoding sys.argv is in (AFAIK) 
+    # len(arguments) < len(sys.argv) should be an impossibility since python gets 
+    # args from the very same PEB as does GetCommandLineW
+    arguments = list(s)
+    
+    # Now shorten the command line we get from GetCommandLineW to match sys.argv
+    if len(arguments) < len(argv):
+        raise AssertionError("Split command line can't be shorter than argv")
+    arguments = arguments[len(arguments) - len(argv):]
+    
+    # Carry on to process globs (metachars) in the command line
+    # expand globs if necessary
     # TODO: Use 'globbing' instead of 'glob.glob', this gives us stuff like
     #       '**/' style globs
     args = []
-    for is_quoted, arg in s:
+    for is_quoted, arg in arguments:
         if is_quoted or not glob.has_magic(arg):
             args.append(arg)
         else:
@@ -557,17 +572,7 @@
         if command_line is None:
             raise ctypes.WinError()
         # Skip the first argument, since we only care about parameters
-        argv = _command_line_to_argv(command_line)[1:]
-        if getattr(sys, 'frozen', None) is None:
-            # Invoked via 'python.exe' which takes the form:
-            #   python.exe [PYTHON_OPTIONS] C:\Path\bzr [BZR_OPTIONS]
-            # we need to get only BZR_OPTIONS part,
-            # We already removed 'python.exe' so we remove everything up to and
-            # including the first non-option ('-') argument.
-            for idx in xrange(len(argv)):
-                if argv[idx][:1] != '-':
-                    break
-            argv = argv[idx+1:]
+        argv = _command_line_to_argv(command_line, sys.argv)[1:]
         return argv
 else:
     get_unicode_argv = None

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2010-04-30 11:35:43 +0000
+++ bzrlib/workingtree.py	2010-09-24 16:16:13 +0000
@@ -49,6 +49,7 @@
     branch,
     bzrdir,
     conflicts as _mod_conflicts,
+    controldir,
     errors,
     generate_ids,
     globbing,
@@ -61,13 +62,12 @@
     revisiontree,
     trace,
     transform,
+    transport,
     ui,
     views,
     xml5,
     xml7,
     )
-import bzrlib.branch
-from bzrlib.transport import get_transport
 from bzrlib.workingtree_4 import (
     WorkingTreeFormat4,
     WorkingTreeFormat5,
@@ -77,6 +77,7 @@
 
 from bzrlib import symbol_versioning
 from bzrlib.decorators import needs_read_lock, needs_write_lock
+from bzrlib.lock import LogicalLockResult
 from bzrlib.lockable_files import LockableFiles
 from bzrlib.lockdir import LockDir
 import bzrlib.mutabletree
@@ -168,7 +169,7 @@
 
 
 class WorkingTree(bzrlib.mutabletree.MutableTree,
-    bzrdir.ControlComponent):
+    controldir.ControlComponent):
     """Working copy tree.
 
     The inventory is held in the `Branch` working-inventory, and the
@@ -176,6 +177,9 @@
 
     It is possible for a `WorkingTree` to have a filename which is
     not listed in the Inventory and vice versa.
+
+    :ivar basedir: The root of the tree on disk. This is a unicode path object
+        (as opposed to a URL).
     """
 
     # override this to set the strategy for storing views
@@ -346,10 +350,67 @@
         if path is None:
             path = osutils.getcwd()
         control, relpath = bzrdir.BzrDir.open_containing(path)
-
         return control.open_workingtree(), relpath
 
     @staticmethod
+    def open_containing_paths(file_list, default_directory='.',
+        canonicalize=True, apply_view=True):
+        """Open the WorkingTree that contains a set of paths.
+
+        Fail if the paths given are not all in a single tree.
+
+        This is used for the many command-line interfaces that take a list of
+        any number of files and that require they all be in the same tree.
+        """
+        # recommended replacement for builtins.internal_tree_files
+        if file_list is None or len(file_list) == 0:
+            tree = WorkingTree.open_containing(default_directory)[0]
+            # XXX: doesn't really belong here, and seems to have the strange
+            # side effect of making it return a bunch of files, not the whole
+            # tree -- mbp 20100716
+            if tree.supports_views() and apply_view:
+                view_files = tree.views.lookup_view()
+                if view_files:
+                    file_list = view_files
+                    view_str = views.view_display_str(view_files)
+                    note("Ignoring files outside view. View is %s" % view_str)
+            return tree, file_list
+        tree = WorkingTree.open_containing(file_list[0])[0]
+        return tree, tree.safe_relpath_files(file_list, canonicalize,
+            apply_view=apply_view)
+
+    def safe_relpath_files(self, file_list, canonicalize=True, apply_view=True):
+        """Convert file_list into a list of relpaths in tree.
+
+        :param self: A tree to operate on.
+        :param file_list: A list of user provided paths or None.
+        :param apply_view: if True and a view is set, apply it or check that
+            specified files are within it
+        :return: A list of relative paths.
+        :raises errors.PathNotChild: When a provided path is in a different self
+            than self.
+        """
+        if file_list is None:
+            return None
+        if self.supports_views() and apply_view:
+            view_files = self.views.lookup_view()
+        else:
+            view_files = []
+        new_list = []
+        # self.relpath exists as a "thunk" to osutils, but canonical_relpath
+        # doesn't - fix that up here before we enter the loop.
+        if canonicalize:
+            fixer = lambda p: osutils.canonical_relpath(self.basedir, p)
+        else:
+            fixer = self.relpath
+        for filename in file_list:
+            relpath = fixer(osutils.dereference_path(filename))
+            if view_files and not osutils.is_inside_any(view_files, relpath):
+                raise errors.FileOutsideView(filename, view_files)
+            new_list.append(relpath)
+        return new_list
+
+    @staticmethod
     def open_downlevel(path=None):
         """Open an unsupported working tree.
 
@@ -368,10 +429,10 @@
                 return True, None
             else:
                 return True, tree
-        transport = get_transport(location)
-        iterator = bzrdir.BzrDir.find_bzrdirs(transport, evaluate=evaluate,
+        t = transport.get_transport(location)
+        iterator = bzrdir.BzrDir.find_bzrdirs(t, evaluate=evaluate,
                                               list_current=list_current)
-        return [t for t in iterator if t is not None]
+        return [tr for tr in iterator if tr is not None]
 
     # should be deprecated - this is slow and in any case treating them as a
     # container is (we now know) bad style -- mbp 20070302
@@ -462,7 +523,11 @@
         return (file_obj, stat_value)
 
     def get_file_text(self, file_id, path=None, filtered=True):
-        return self.get_file(file_id, path=path, filtered=filtered).read()
+        my_file = self.get_file(file_id, path=path, filtered=filtered)
+        try:
+            return my_file.read()
+        finally:
+            my_file.close()
 
     def get_file_byname(self, filename, filtered=True):
         path = self.abspath(filename)
@@ -522,7 +587,7 @@
 
         # Now we have the parents of this content
         annotator = self.branch.repository.texts.get_annotator()
-        text = self.get_file(file_id).read()
+        text = self.get_file_text(file_id)
         this_key =(file_id, default_revision)
         annotator.add_special_text(this_key, file_parent_keys, text)
         annotations = [(key[-1], line)
@@ -1202,13 +1267,18 @@
                 # absolute path
                 fap = from_dir_abspath + '/' + f
 
-                f_ie = inv.get_child(from_dir_id, f)
+                dir_ie = inv[from_dir_id]
+                if dir_ie.kind == 'directory':
+                    f_ie = dir_ie.children.get(f)
+                else:
+                    f_ie = None
                 if f_ie:
                     c = 'V'
                 elif self.is_ignored(fp[1:]):
                     c = 'I'
                 else:
-                    # we may not have found this file, because of a unicode issue
+                    # we may not have found this file, because of a unicode
+                    # issue, or because the directory was actually a symlink.
                     f_norm, can_access = osutils.normalized_filename(f)
                     if f == f_norm or not can_access:
                         # No change, so treat this file normally
@@ -1257,7 +1327,7 @@
                 stack.pop()
 
     @needs_tree_write_lock
-    def move(self, from_paths, to_dir=None, after=False, **kwargs):
+    def move(self, from_paths, to_dir=None, after=False):
         """Rename files.
 
         to_dir must exist in the inventory.
@@ -1297,14 +1367,7 @@
 
         # check for deprecated use of signature
         if to_dir is None:
-            to_dir = kwargs.get('to_name', None)
-            if to_dir is None:
-                raise TypeError('You must supply a target directory')
-            else:
-                symbol_versioning.warn('The parameter to_name was deprecated'
-                                       ' in version 0.13. Use to_dir instead',
-                                       DeprecationWarning)
-
+            raise TypeError('You must supply a target directory')
         # check destination directory
         if isinstance(from_paths, basestring):
             raise ValueError()
@@ -1600,7 +1663,8 @@
 
     @needs_write_lock
     def pull(self, source, overwrite=False, stop_revision=None,
-             change_reporter=None, possible_transports=None, local=False):
+             change_reporter=None, possible_transports=None, local=False,
+             show_base=False):
         source.lock_read()
         try:
             old_revision_info = self.branch.last_revision_info()
@@ -1620,7 +1684,8 @@
                                 basis_tree,
                                 this_tree=self,
                                 pb=None,
-                                change_reporter=change_reporter)
+                                change_reporter=change_reporter,
+                                show_base=show_base)
                     basis_root_id = basis_tree.get_root_id()
                     new_root_id = new_basis_tree.get_root_id()
                     if basis_root_id != new_root_id:
@@ -1798,34 +1863,48 @@
             raise errors.ObjectNotLocked(self)
 
     def lock_read(self):
-        """See Branch.lock_read, and WorkingTree.unlock."""
+        """Lock the tree for reading.
+
+        This also locks the branch, and can be unlocked via self.unlock().
+
+        :return: A bzrlib.lock.LogicalLockResult.
+        """
         if not self.is_locked():
             self._reset_data()
         self.branch.lock_read()
         try:
-            return self._control_files.lock_read()
+            self._control_files.lock_read()
+            return LogicalLockResult(self.unlock)
         except:
             self.branch.unlock()
             raise
 
     def lock_tree_write(self):
-        """See MutableTree.lock_tree_write, and WorkingTree.unlock."""
+        """See MutableTree.lock_tree_write, and WorkingTree.unlock.
+
+        :return: A bzrlib.lock.LogicalLockResult.
+        """
         if not self.is_locked():
             self._reset_data()
         self.branch.lock_read()
         try:
-            return self._control_files.lock_write()
+            self._control_files.lock_write()
+            return LogicalLockResult(self.unlock)
         except:
             self.branch.unlock()
             raise
 
     def lock_write(self):
-        """See MutableTree.lock_write, and WorkingTree.unlock."""
+        """See MutableTree.lock_write, and WorkingTree.unlock.
+
+        :return: A bzrlib.lock.LogicalLockResult.
+        """
         if not self.is_locked():
             self._reset_data()
         self.branch.lock_write()
         try:
-            return self._control_files.lock_write()
+            self._control_files.lock_write()
+            return LogicalLockResult(self.unlock)
         except:
             self.branch.unlock()
             raise
@@ -1948,35 +2027,36 @@
 
         inv_delta = []
 
-        new_files=set()
+        all_files = set() # specified and nested files 
         unknown_nested_files=set()
         if to_file is None:
             to_file = sys.stdout
 
+        files_to_backup = []
+
         def recurse_directory_to_add_files(directory):
             # Recurse directory and add all files
             # so we can check if they have changed.
-            for parent_info, file_infos in\
-                self.walkdirs(directory):
+            for parent_info, file_infos in self.walkdirs(directory):
                 for relpath, basename, kind, lstat, fileid, kind in file_infos:
                     # Is it versioned or ignored?
-                    if self.path2id(relpath) or self.is_ignored(relpath):
+                    if self.path2id(relpath):
                         # Add nested content for deletion.
-                        new_files.add(relpath)
+                        all_files.add(relpath)
                     else:
-                        # Files which are not versioned and not ignored
+                        # Files which are not versioned
                         # should be treated as unknown.
-                        unknown_nested_files.add((relpath, None, kind))
+                        files_to_backup.append(relpath)
 
         for filename in files:
             # Get file name into canonical form.
             abspath = self.abspath(filename)
             filename = self.relpath(abspath)
             if len(filename) > 0:
-                new_files.add(filename)
+                all_files.add(filename)
                 recurse_directory_to_add_files(filename)
 
-        files = list(new_files)
+        files = list(all_files)
 
         if len(files) == 0:
             return # nothing to do
@@ -1986,32 +2066,23 @@
 
         # Bail out if we are going to delete files we shouldn't
         if not keep_files and not force:
-            has_changed_files = len(unknown_nested_files) > 0
-            if not has_changed_files:
-                for (file_id, path, content_change, versioned, parent_id, name,
-                     kind, executable) in self.iter_changes(self.basis_tree(),
-                         include_unchanged=True, require_versioned=False,
-                         want_unversioned=True, specific_files=files):
-                    if versioned == (False, False):
-                        # The record is unknown ...
-                        if not self.is_ignored(path[1]):
-                            # ... but not ignored
-                            has_changed_files = True
-                            break
-                    elif content_change and (kind[1] is not None):
-                        # Versioned and changed, but not deleted
-                        has_changed_files = True
-                        break
+            for (file_id, path, content_change, versioned, parent_id, name,
+                 kind, executable) in self.iter_changes(self.basis_tree(),
+                     include_unchanged=True, require_versioned=False,
+                     want_unversioned=True, specific_files=files):
+                if versioned[0] == False:
+                    # The record is unknown or newly added
+                    files_to_backup.append(path[1])
+                elif (content_change and (kind[1] is not None) and
+                        osutils.is_inside_any(files, path[1])):
+                    # Versioned and changed, but not deleted, and still
+                    # in one of the dirs to be deleted.
+                    files_to_backup.append(path[1])
 
-            if has_changed_files:
-                # Make delta show ALL applicable changes in error message.
-                tree_delta = self.changes_from(self.basis_tree(),
-                    require_versioned=False, want_unversioned=True,
-                    specific_files=files)
-                for unknown_file in unknown_nested_files:
-                    if unknown_file not in tree_delta.unversioned:
-                        tree_delta.unversioned.extend((unknown_file,))
-                raise errors.BzrRemoveChangedFilesError(tree_delta)
+        def backup(file_to_backup):
+            backup_name = self.bzrdir.generate_backup_name(file_to_backup)
+            osutils.rename(abs_path, self.abspath(backup_name))
+            return "removed %s (but kept a copy: %s)" % (file_to_backup, backup_name)
 
         # Build inv_delta and delete files where applicable,
         # do this before any modifications to inventory.
@@ -2041,12 +2112,15 @@
                         len(os.listdir(abs_path)) > 0):
                         if force:
                             osutils.rmtree(abs_path)
+                            message = "deleted %s" % (f,)
                         else:
-                            message = "%s is not an empty directory "\
-                                "and won't be deleted." % (f,)
+                            message = backup(f)
                     else:
-                        osutils.delete_any(abs_path)
-                        message = "deleted %s" % (f,)
+                        if f in files_to_backup:
+                            message = backup(f)
+                        else:
+                            osutils.delete_any(abs_path)
+                            message = "deleted %s" % (f,)
                 elif message is not None:
                     # Only care if we haven't done anything yet.
                     message = "%s does not exist." % (f,)
@@ -2189,7 +2263,7 @@
     _marker = object()
 
     def update(self, change_reporter=None, possible_transports=None,
-               revision=None, old_tip=_marker):
+               revision=None, old_tip=_marker, show_base=False):
         """Update a working tree along its branch.
 
         This will update the branch if its bound too, which means we have
@@ -2232,12 +2306,13 @@
             else:
                 if old_tip is self._marker:
                     old_tip = None
-            return self._update_tree(old_tip, change_reporter, revision)
+            return self._update_tree(old_tip, change_reporter, revision, show_base)
         finally:
             self.unlock()
 
     @needs_tree_write_lock
-    def _update_tree(self, old_tip=None, change_reporter=None, revision=None):
+    def _update_tree(self, old_tip=None, change_reporter=None, revision=None,
+                     show_base=False):
         """Update a tree to the master branch.
 
         :param old_tip: if supplied, the previous tip revision the branch,
@@ -2270,7 +2345,8 @@
             other_tree = self.branch.repository.revision_tree(old_tip)
             nb_conflicts = merge.merge_inner(self.branch, other_tree,
                                              base_tree, this_tree=self,
-                                             change_reporter=change_reporter)
+                                             change_reporter=change_reporter,
+                                             show_base=show_base)
             if nb_conflicts:
                 self.add_parent_tree((old_tip, other_tree))
                 trace.note('Rerun update after fixing the conflicts.')
@@ -2300,7 +2376,8 @@
 
             nb_conflicts = merge.merge_inner(self.branch, to_tree, base_tree,
                                              this_tree=self,
-                                             change_reporter=change_reporter)
+                                             change_reporter=change_reporter,
+                                             show_base=show_base)
             self.set_last_revision(revision)
             # TODO - dedup parents list with things merged by pull ?
             # reuse the tree we've updated to to set the basis:
@@ -2636,10 +2713,14 @@
 
         In Format2 WorkingTrees we have a single lock for the branch and tree
         so lock_tree_write() degrades to lock_write().
+
+        :return: An object with an unlock method which will release the lock
+            obtained.
         """
         self.branch.lock_write()
         try:
-            return self._control_files.lock_write()
+            self._control_files.lock_write()
+            return self
         except:
             self.branch.unlock()
             raise

=== modified file 'bzrlib/workingtree_4.py'
--- bzrlib/workingtree_4.py	2010-04-30 11:03:59 +0000
+++ bzrlib/workingtree_4.py	2010-08-03 13:31:06 +0000
@@ -53,6 +53,7 @@
 from bzrlib.decorators import needs_read_lock, needs_write_lock
 from bzrlib.filters import filtered_input_file, internal_size_sha_file_byname
 from bzrlib.inventory import Inventory, ROOT_ID, entry_factory
+from bzrlib.lock import LogicalLockResult
 from bzrlib.mutabletree import needs_tree_write_lock
 from bzrlib.osutils import (
     file_kind,
@@ -567,7 +568,10 @@
             return _mod_revision.NULL_REVISION
 
     def lock_read(self):
-        """See Branch.lock_read, and WorkingTree.unlock."""
+        """See Branch.lock_read, and WorkingTree.unlock.
+
+        :return: A bzrlib.lock.LogicalLockResult.
+        """
         self.branch.lock_read()
         try:
             self._control_files.lock_read()
@@ -586,6 +590,7 @@
         except:
             self.branch.unlock()
             raise
+        return LogicalLockResult(self.unlock)
 
     def _lock_self_write(self):
         """This should be called after the branch is locked."""
@@ -606,16 +611,23 @@
         except:
             self.branch.unlock()
             raise
+        return LogicalLockResult(self.unlock)
 
     def lock_tree_write(self):
-        """See MutableTree.lock_tree_write, and WorkingTree.unlock."""
+        """See MutableTree.lock_tree_write, and WorkingTree.unlock.
+
+        :return: A bzrlib.lock.LogicalLockResult.
+        """
         self.branch.lock_read()
-        self._lock_self_write()
+        return self._lock_self_write()
 
     def lock_write(self):
-        """See MutableTree.lock_write, and WorkingTree.unlock."""
+        """See MutableTree.lock_write, and WorkingTree.unlock.
+
+        :return: A bzrlib.lock.LogicalLockResult.
+        """
         self.branch.lock_write()
-        self._lock_self_write()
+        return self._lock_self_write()
 
     @needs_tree_write_lock
     def move(self, from_paths, to_dir, after=False):
@@ -1235,7 +1247,8 @@
         # have to change the legacy inventory too.
         if self._inventory is not None:
             for file_id in file_ids:
-                self._inventory.remove_recursive_id(file_id)
+                if self._inventory.has_id(file_id):
+                    self._inventory.remove_recursive_id(file_id)
 
     @needs_tree_write_lock
     def rename_one(self, from_rel, to_rel, after=False):
@@ -1317,7 +1330,7 @@
     def _file_content_summary(self, path, stat_result):
         # This is to support the somewhat obsolete path_content_summary method
         # with content filtering: see
-        # <https://bugs.edge.launchpad.net/bzr/+bug/415508>.
+        # <https://bugs.launchpad.net/bzr/+bug/415508>.
         #
         # If the dirstate cache is up to date and knows the hash and size,
         # return that.
@@ -1725,8 +1738,6 @@
                 elif kind == 'directory':
                     parent_ies[(dirname + '/' + name).strip('/')] = inv_entry
                 elif kind == 'symlink':
-                    inv_entry.executable = False
-                    inv_entry.text_size = None
                     inv_entry.symlink_target = utf8_decode(fingerprint)[0]
                 elif kind == 'tree-reference':
                     inv_entry.reference_revision = fingerprint or None
@@ -1859,6 +1870,9 @@
             return None
         return ie.executable
 
+    def is_locked(self):
+        return self._locked
+
     def list_files(self, include_root=False, from_dir=None, recursive=True):
         # We use a standard implementation, because DirStateRevisionTree is
         # dealing with one of the parents of the current state
@@ -1877,13 +1891,17 @@
             yield path, 'V', entry.kind, entry.file_id, entry
 
     def lock_read(self):
-        """Lock the tree for a set of operations."""
+        """Lock the tree for a set of operations.
+
+        :return: A bzrlib.lock.LogicalLockResult.
+        """
         if not self._locked:
             self._repository.lock_read()
             if self._dirstate._lock_token is None:
                 self._dirstate.lock_read()
                 self._dirstate_locked = True
         self._locked += 1
+        return LogicalLockResult(self.unlock)
 
     def _must_be_locked(self):
         if not self._locked:

=== modified file 'bzrlib/xml_serializer.py'
--- bzrlib/xml_serializer.py	2010-04-30 11:03:59 +0000
+++ bzrlib/xml_serializer.py	2010-09-17 04:35:23 +0000
@@ -22,6 +22,8 @@
 # importing this module is fairly slow because it has to load several
 # ElementTree bits
 
+import re
+
 from bzrlib.serializer import Serializer
 from bzrlib.trace import mutter
 
@@ -31,6 +33,8 @@
         from xml.etree.cElementTree import (ElementTree, SubElement, Element,
             XMLTreeBuilder, fromstring, tostring)
         import xml.etree as elementtree
+        # Also import ElementTree module so monkey-patching below always works
+        import xml.etree.ElementTree
     except ImportError:
         from cElementTree import (ElementTree, SubElement, Element,
                                   XMLTreeBuilder, fromstring, tostring)
@@ -83,8 +87,11 @@
 
     def read_inventory(self, f, revision_id=None):
         try:
-            return self._unpack_inventory(self._read_element(f),
-                revision_id=None)
+            try:
+                return self._unpack_inventory(self._read_element(f),
+                    revision_id=None)
+            finally:
+                f.close()
         except ParseError, e:
             raise errors.UnexpectedInventoryFormat(e)
 
@@ -108,73 +115,6 @@
         return ElementTree().parse(f)
 
 
-# performance tuning for elementree's serialiser. This should be
-# sent upstream - RBC 20060523.
-# the functions here are patched into elementtree at runtime.
-import re
-escape_re = re.compile("[&'\"<>]")
-escape_map = {
-    "&":'&amp;',
-    "'":"&apos;", # FIXME: overkill
-    "\"":"&quot;",
-    "<":"&lt;",
-    ">":"&gt;",
-    }
-def _escape_replace(match, map=escape_map):
-    return map[match.group()]
-
-def _escape_attrib(text, encoding=None, replace=None):
-    # escape attribute value
-    try:
-        if encoding:
-            try:
-                text = elementtree.ElementTree._encode(text, encoding)
-            except UnicodeError:
-                return elementtree.ElementTree._encode_entity(text)
-        if replace is None:
-            return escape_re.sub(_escape_replace, text)
-        else:
-            text = replace(text, "&", "&amp;")
-            text = replace(text, "'", "&apos;") # FIXME: overkill
-            text = replace(text, "\"", "&quot;")
-            text = replace(text, "<", "&lt;")
-            text = replace(text, ">", "&gt;")
-            return text
-    except (TypeError, AttributeError):
-        elementtree.ElementTree._raise_serialization_error(text)
-
-elementtree.ElementTree._escape_attrib = _escape_attrib
-
-escape_cdata_re = re.compile("[&<>]")
-escape_cdata_map = {
-    "&":'&amp;',
-    "<":"&lt;",
-    ">":"&gt;",
-    }
-def _escape_cdata_replace(match, map=escape_cdata_map):
-    return map[match.group()]
-
-def _escape_cdata(text, encoding=None, replace=None):
-    # escape character data
-    try:
-        if encoding:
-            try:
-                text = elementtree.ElementTree._encode(text, encoding)
-            except UnicodeError:
-                return elementtree.ElementTree._encode_entity(text)
-        if replace is None:
-            return escape_cdata_re.sub(_escape_cdata_replace, text)
-        else:
-            text = replace(text, "&", "&amp;")
-            text = replace(text, "<", "&lt;")
-            text = replace(text, ">", "&gt;")
-            return text
-    except (TypeError, AttributeError):
-        elementtree.ElementTree._raise_serialization_error(text)
-
-elementtree.ElementTree._escape_cdata = _escape_cdata
-
-
 def escape_invalid_chars(message):
     """Escape the XML-invalid characters in a commit message.
 

=== added file 'contrib/bash/bzr'
--- contrib/bash/bzr	1970-01-01 00:00:00 +0000
+++ contrib/bash/bzr	2010-04-22 07:01:22 +0000
@@ -0,0 +1,40 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+# Programmable completion for the Bazaar-NG bzr command under bash.
+# Source this file (or add it to your ~/.bash_completion or ~/.bashrc
+# file, depending on your system configuration, and start a new shell)
+# and bash's completion mechanism will know all about bzr's options!
+#
+# This completion function assumes you have the bzr-bash-completion
+# plugin installed as a bzr plugin. It will generate the full
+# completion function at first invocation, thus avoiding long delays
+# for every shell you start.
+
+shopt -s progcomp
+_bzr_lazy ()
+{
+	unset _bzr
+	eval "$(bzr bash-completion)"
+	if [[ $(type -t _bzr) == function ]]; then
+		unset _bzr_lazy
+		_bzr
+		return $?
+	else
+		return 1
+	fi
+}
+complete -F _bzr_lazy -o default bzr

=== removed file 'contrib/bash/bzr'
--- contrib/bash/bzr	2005-09-19 06:05:19 +0000
+++ contrib/bash/bzr	1970-01-01 00:00:00 +0000
@@ -1,104 +0,0 @@
-# Programmable completion for the Bazaar-NG bzr command under bash. Source
-# this file (or on some systems add it to ~/.bash_completion and start a new
-# shell) and bash's completion mechanism will know all about bzr's options!
-
-# Known to work with bash 2.05a with programmable completion and extended
-# pattern matching enabled (use 'shopt -s extglob progcomp' to enable
-# these if they are not already enabled).
-
-# Based originally on the svn bash completition script.
-# Customized by Sven Wilhelm/Icecrash.com
-
-_bzr ()
-{
-	local cur cmds cmdOpts opt helpCmds optBase i
-
-	COMPREPLY=()
-	cur=${COMP_WORDS[COMP_CWORD]}
-
-	cmds='status diff commit ci checkin move remove log info check ignored'
-
-	if [[ $COMP_CWORD -eq 1 ]] ; then
-		COMPREPLY=( $( compgen -W "$cmds" -- $cur ) )
-		return 0
-	fi
-
-	# if not typing an option, or if the previous option required a
-	# parameter, then fallback on ordinary filename expansion
-	helpCmds='help|--help|h|\?'
-	if [[ ${COMP_WORDS[1]} != @($helpCmds) ]] && \
-	   [[ "$cur" != -* ]] ; then
-		return 0
-	fi
-
-	cmdOpts=
-	case ${COMP_WORDS[1]} in
-	status)
-		cmdOpts="--all --show-ids"
-		;;
-	diff)
-		cmdOpts="-r --revision --diff-options"
-		;;
-	commit|ci|checkin)
-		cmdOpts="-r --message -F --file -v --verbose"
-		;;
-	move)
-		cmdOpts=""
-		;;
-	remove)
-		cmdOpts="-v --verbose"
-		;;
-	log)
-		cmdOpts="--forward --timezone -v --verbose --show-ids -r --revision"
-		;;
-	info)
-		cmdOpts=""
-		;;
-	ignored)
-		cmdOpts=""
-		;;
-	check)
-		cmdOpts=""
-		;;
-	help|h|\?)
-		cmdOpts="$cmds $qOpts"
-		;;
-	*)
-		;;
-	esac
-
-	cmdOpts="$cmdOpts --help -h"
-
-	# take out options already given
-	for (( i=2; i<=$COMP_CWORD-1; ++i )) ; do
-		opt=${COMP_WORDS[$i]}
-
-		case $opt in
-		--*)    optBase=${opt/=*/} ;;
-		-*)     optBase=${opt:0:2} ;;
-		esac
-
-		cmdOpts=" $cmdOpts "
-		cmdOpts=${cmdOpts/ ${optBase} / }
-
-		# take out alternatives
-		case $optBase in
-		-v)              cmdOpts=${cmdOpts/ --verbose / } ;;
-		--verbose)       cmdOpts=${cmdOpts/ -v / } ;;
-		-h)              cmdOpts=${cmdOpts/ --help / } ;;
-		--help)          cmdOpts=${cmdOpts/ -h / } ;;
-		-r)              cmdOpts=${cmdOpts/ --revision / } ;;
-		--revision)      cmdOpts=${cmdOpts/ -r / } ;;
-		esac
-
-		# skip next option if this one requires a parameter
-		if [[ $opt == @($optsParam) ]] ; then
-			((++i))
-		fi
-	done
-
-	COMPREPLY=( $( compgen -W "$cmdOpts" -- $cur ) )
-
-	return 0
-}
-complete -F _bzr -o default bzr

=== removed file 'contrib/bash/bzr.simple'
--- contrib/bash/bzr.simple	2007-06-06 19:44:39 +0000
+++ contrib/bash/bzr.simple	1970-01-01 00:00:00 +0000
@@ -1,28 +0,0 @@
-# -*- shell-script -*-
-
-# experimental bzr bash completion
-
-# author: Martin Pool
-
-_bzr_commands() 
-{
-     bzr help commands | sed -r 's/^([-[:alnum:]]*).*/\1/' | grep '^[[:alnum:]]' 
-}
-
-_bzr() 
-{
-    cur=${COMP_WORDS[COMP_CWORD]}
-    prev=${COMP_WORDS[COMP_CWORD-1]}
-    if [ $COMP_CWORD -eq 1 ]; then
-        COMPREPLY=( $( compgen -W "$(_bzr_commands)" $cur ) )
-    elif [ $COMP_CWORD -eq 2 ]; then
-        case "$prev" in 
-        help)
-            COMPREPLY=( $( compgen -W "$(_bzr_commands) commands" $cur ) )
-            ;;
-        esac
-    fi 
-}
-
-complete -F _bzr -o default bzr
-

=== added file 'contrib/zsh/README'
--- contrib/zsh/README	1970-01-01 00:00:00 +0000
+++ contrib/zsh/README	2010-05-25 11:34:35 +0000
@@ -0,0 +1,19 @@
+This directory used to contain a shell completion function for zsh.
+That script has been unmaintained since 2005, though.  Zsh itself does
+ship a _bzr completion function, and has done so for quite some time
+now.  So we suggest you use the version installed with your zsh
+instead of the version that used to be located in this directory.
+
+Note that the shell-complete builtin of bzr does provide completion
+information in a style suitable for zsh.  There are some completion
+scripts for zsh in the wild that make use of this feature, although
+none of these scripts is officially supported by the bzr developers.
+
+Also note that recent versions of bzr (starting with the 2.3 series)
+do include a bash_completion plugin. It should be possible to improve
+this in such a way that it generates code for zsh instead of bash.  So
+if you want better zsh completion, this might be the place to start.
+
+References:
+https://bugs.launchpad.net/bzr/+bug/560030
+http://zsh.git.sourceforge.net/git/gitweb.cgi?p=zsh/zsh;a=history;f=Completion/Unix/Command/_bzr

=== removed file 'contrib/zsh/_bzr'
--- contrib/zsh/_bzr	2005-11-18 08:10:07 +0000
+++ contrib/zsh/_bzr	1970-01-01 00:00:00 +0000
@@ -1,316 +0,0 @@
-#compdef bzr
-
-# Rudimentary zsh completion support for bzr.
-
-# -S means there are no options after a -- and that argument is ignored
-
-# To use this you must arrange for it to be in a directory that is on
-# your $fpath, and also for compinit to be run.  I don't understand
-# how to get zsh to reload this file when it changes, other than by
-# starting a new zsh.
-
-local _bzr_subcommands expl curcontext="$curcontext" state line
-local bzrRoot helpArgs parentFile
-typeset -A opt_args
-
-helpArgs=( '(--help)-h[show help message]' '(-h)--help[show help message]' )
-bzrRoot=$(bzr root 2> /dev/null)
-parentFile=$bzrRoot/.bzr/parent
-
-if [[ $service == "bzr" ]]; then
-    _arguments -C -A "-*" \
-    '*::command:->subcmd' && return 0
-
-    if (( CURRENT == 1 )); then
-      _bzr_subcommands=(${(f)"$(_call_program bzr bzr shell-complete)"})
-      _describe -t subcommand 'subcommand' _bzr_subcommands
-      return
-    fi
-
-    service="$words[1]"
-    curcontext="${curcontext%:*}=$service:"
-fi
-
-_unknownFiles() {
-    local fileList;
-    fileList=(${(ps:\0:)"$(bzr ls --null --unknown)"})
-    compadd -af fileList
-    return 0
-}
-
-_unknownRoot() {
-    local fileList;
-    fileList=(${(ps:\0:)"$(bzr ls --null --from-root --unknown)"})
-    compadd -af fileList
-    return 0
-}
-
-_versionedFiles() {
-    local fileList;
-    fileList=(${(ps:\0:)"$(bzr ls --null --versioned)"})
-    compadd -af fileList
-    return 0
-}
-
-case $service in
-    (add)
-       _arguments $helpArgs \
-       '--no-recurse[do not recurse into subdirectories]' \
-       '(-q)--quiet' \
-       '(--quiet)-q' \
-       '*:unknown files:_unknownFiles'
-    ;;
-
-    (annotate|praise|blame)
-        _arguments $helpArgs '*:files:_versionedFiles'
-    ;;
-
-    (clone|branch|get)
-        if (( CURRENT == 2 )); then
-            _arguments $helpArgs \
-            '(-r)--revision:rev:' \
-            '(--revision)-r:rev:' \
-            '--basis[specify basis branch]:basis:' \
-            '*:FROM_LOCATION:_files -/'
-        elif (( CURRENT == 3 )); then
-            _arguments $helpArgs \
-            '(-r)--revision:rev:' \
-            '(--revision)-r:rev:' \
-            '--basis[specify basis branch]:basis:' \
-            '*:TO_LOCATION:_files -/'
-        fi
-    ;;
-
-    (rename|mv)
-        if (( CURRENT == 2 )); then
-            _arguments $helpArgs '*:old name:_versionedFiles'
-        else
-            _arguments $helpArgs '*:new name:'
-        fi
-    ;;
-
-    (cat)
-        _arguments $helpArgs \
-        '(-r)--revision:rev:' \
-        '(--revision)-r:rev:' \
-        '*:file:_versionedFiles'
-    ;;
-
-    (root)
-        _arguments $helpArgs '*:file:_files'
-    ;;
-
-    (log)
-        _arguments $helpArgs \
-        '(-r)--revision[revision or range]:rev or rev range:' \
-        '(--revision)-r[revision or range]:rev or rev range:' \
-        '(--verbose)-v[show revision manifest]' \
-        '(-v)--verbose[show revision manifest]' \
-        '(--short --long)-l[use long format (default)]' \
-        '(--short -l)--long[use long format (default)]' \
-        '(-l --long)--short[use short format]' \
-        '(--message)-m[specify regexp]:regexp:' \
-        '(-m)--message[specify regexp]:regexp:' \
-        '--show-ids[show file IDs]' \
-        '--forward[reverse direction of revisions]' \
-        '--timezoe[specify timezone for dates]:timezone:' \
-        '*:file:_versionedFiles'
-    ;;
-
-    (resolve)
-        _arguments $helpArgs '--all' '*:file:_versionedFiles'
-    ;;
-
-    (status|st|stat)
-        _arguments $helpArgs \
-        '--all[include unchanged versioned files]' \
-        '--show-ids[show file IDs]' \
-        '*:file:_versionedFiles'
-    ;;
-
-    (check)
-        _arguments $helpArgs \
-        '(--verbose)-v' \
-        '(-v)--verbose' \
-        '*:DIR:_files -/'
-    ;;
-
-    (mkdir|upgrade|renames)
-        _arguments $helpArgs '*:DIR:_files -/'
-    ;;
-
-    (remove|rm)
-        _arguments $helpArgs \
-        '(--verbose)-v' \
-        '(-v)--verbose' \
-        '*:file:_versionedFiles'
-    ;;
-
-    (pull)
-        _arguments $helpArgs \
-        '--remember[Remember the specified location as a default]' \
-        '--overwrite[Ignore differences, overwrite unconditionally]' \
-        '*:local repository:_files -/'
-        [[ -r $parentFile ]] && compadd -X "Completing parents" $(cat $parentFile)
-    ;;
-
-    (missing|miss|mis)
-        _arguments $helpArgs \
-        '(--verbose -q --quiet)-v' \
-        '(-v -q --quiet)--verbose' \
-        '(--verbose -v --quiet)-q' \
-        '(-v -q --verbose)--quiet' \
-        '*:local repository:_files -/'
-        [[ -r $parentFile ]] && compadd -X "Completing parents" $(cat $parentFile)
-    ;;
-
-    (commit|checkin|ci)
-        _arguments $helpArgs \
-        '(--message)-m[commit message]:message text:' \
-        '(-m)--message[commit message]:message text:' \
-        '(--file)-F[commit message from file]:message file:' \
-        '(-F)--file[commit message from file]:message file:' \
-        '--unchanged[include unchanged files]' \
-        '(--verbose)-v' \
-        '(-v)--verbose' \
-        '*:modified files:_versionedFiles'
-    ;;
-
-    (conflicts|added|deleted|modified|unknowns|directories|ignored)
-        _arguments $helpArgs
-    ;;
-
-    (revno|init|version)
-        _arguments $helpArgs
-    ;;
-
-    (whoami)
-        _arguments $helpArgs '--email[only show e-mail address]' 
-    ;;
-
-    (inventory)
-        _arguments $helpArgs \
-        '(-r)--revision[show inventory of a revision]:revision:' \
-        '(--revision)-r[show inventory of a revision]:revision:' \
-        '--show-ids[show file IDs]'
-    ;;
-
-    (diff|dif|di)
-        _arguments $helpArgs \
-        '(-r)--revision:revision:' \
-        '(--revision)-r:revision:' \
-        '--diff-options[options to pass to gdiff]:diff options:' \
-        '*:files:_files'
-    ;;
-
-    (export)
-        _arguments $helpArgs \
-        '(-r)--revision:revision:' \
-        '(--revision)-r:revision:' \
-        '--format=[format of exported file]:format:(dir tar tgz tbz2)' \
-        '--root[root directory of patch]:_files -/' \
-        '*:destination:_files'
-    ;;
-
-    (ignore)
-        _arguments $helpArgs '*:NAME_PATTERN:_unknownRoot'
-    ;;
-
-    (info)
-        _arguments $helpArgs '*:branch:_files -/'
-    ;;
-
-    (testament)
-        _arguments $helpArgs \
-        '(-r)--revision:revision:' \
-        '(--revision)-r:revision:' \
-        '(--long)-l' \
-        '(-l)--long' \
-        '*:branch:_files -/'
-    ;;
-
-    (revert|merge-revert)
-        _arguments $helpArgs \
-        '(-r)--revision:revision:' \
-        '(--revision)-r:revision:' \
-        '--no-backup[skip generation of backup~ files]' \
-        '*:file:_versionedFiles'
-    ;;
-
-    (merge)
-        _arguments $helpArgs \
-        '(-r)--revision:revision:' \
-        '(--revision)-r:revision:' \
-        '--reprocess[Reprocess to reduce spurious conflicts]' \
-        '--show-base[Show base revision text in conflicts]' \
-        '--force[ignore uncommitted changes]' \
-        '--merge-type:merge type:(diff3 merge3 weave)' \
-        '*:local repository:_files -/'
-        [[ -r $parentFile ]] && compadd -X "Completing parents" $(cat $parentFile)
-    ;;
-
-    (ls)
-        _arguments $helpArgs \
-        '(--verbose)-v[display more information]' \
-        '(-v)--verbose[display more information]' \
-        '(-r)--revision:revision:' \
-        '(--revision)-r:revision:' \
-        '--from-root[Print all paths from the root of the branch]' \
-        '--non-recursive[Do not recurse into subdirectories]' \
-        '--null[Null separate the files]' \
-        '--ignored[Print ignored files]' \
-        '--unknown[Print unknown files]' \
-        '--versioned[Print versioned files]'
-    ;;
-
-    (move)
-        if (( CURRENT == 2 )); then
-            _arguments $helpArgs '*:files:_versionedFiles'
-        else
-            _arguments '*:destination dir:_files -/'
-        fi
-    ;;
-
-    (help)
-        _bzr_subcommands=(${(f)"$(_call_program bzr bzr shell-complete)"})
-        _arguments $helpArgs \
-        '(--long)-l' \
-        '(-l)--long' \
-        '*:subcmds:->cmds'
-        _describe -t subcommand 'subcommand' _bzr_subcommands
-    ;;
-
-    # Plugins
-
-    (visualize|visualise|viz|vis)
-        _arguments $helpArgs \
-            '(-r)--revision[starting revision]:rev:' \
-            '(--revision)-r[starting revision]:rev:'
-    ;;
-
-    (gannotate|gblame|gpraise)
-        _arguments $helpArgs \
-            '--all[show annotations on all lines]' \
-            '--plain[do not hightlight annotation lines]' \
-            '*:files:_versionedFiles'
-    ;;
-
-    (push)
-        _arguments $helpArgs \
-        '--overwrite[Ignore differences, overwrite unconditionally]' \
-        '*:local repository:_files -/'
-        [[ -r $parentFile ]] && compadd -X "Completing parents" $(cat $parentFile)
-    ;;
-
-    (clean-tree)
-        _arguments $helpArgs \
-        '--dry-run[show files to delete instead of deleting them]' \
-        '--ignored[delete all ignored files]' \
-        '--detritus[delete conflict files, merge backups, failed self-tests, *~, *.tmp, etc]'
-    ;;
-
-
-    (*)
-        _message "unknown bzr command completion: $service"
-    ;;
-esac

=== modified file 'doc/developers/HACKING.txt'
--- doc/developers/HACKING.txt	2010-04-16 12:59:03 +0000
+++ doc/developers/HACKING.txt	2010-09-03 09:14:12 +0000
@@ -12,8 +12,7 @@
 document, send a merge request or new text to the mailing list.
 
 The latest developer documentation can be found online at
-http://doc.bazaar-vcs.org/developers/.
-
+http://doc.bazaar.canonical.com/developers/.
 
 Getting Started
 ###############
@@ -29,14 +28,12 @@
 To answer these questions and more, take a moment to explore the
 overall Bazaar Platform. Here are some links to browse:
 
-* The Plugins page on the Wiki - http://bazaar-vcs.org/BzrPlugins
+* The Plugins page on the Wiki - http://wiki.bazaar.canonical.com/BzrPlugins
 
 * The Bazaar product family on Launchpad - https://launchpad.net/bazaar
 
 * Bug Tracker for the core product - https://bugs.launchpad.net/bzr/
 
-* Blueprint Tracker for the core product - https://blueprints.launchpad.net/bzr/
-
 If nothing else, perhaps you'll find inspiration in how other developers
 have solved their challenges.
 
@@ -55,7 +52,7 @@
 
 There is a very active community around Bazaar. Mostly we meet on IRC
 (#bzr on irc.freenode.net) and on the mailing list. To join the Bazaar
-community, see http://bazaar-vcs.org/BzrSupport.
+community, see http://wiki.bazaar.canonical.com/BzrSupport.
 
 If you are planning to make a change, it's a very good idea to mention it
 on the IRC channel and/or on the mailing list. There are many advantages
@@ -76,7 +73,7 @@
 Bazaar Development in a Nutshell
 ================================
 
-.. was from bazaar-vcs.org/BzrGivingBack
+.. was from http://wiki.bazaar.canonical.com/BzrGivingBack
 
 One of the fun things about working on a version control system like Bazaar is
 that the users have a high level of proficiency in contributing back into
@@ -92,7 +89,7 @@
 
  $ bzr init-repo ~/bzr
  $ cd ~/bzr
- $ bzr branch http://bazaar-vcs.org/bzr/bzr.dev/ bzr.dev
+ $ bzr branch lp:bzr bzr.dev
 
 Now make your own branch::
 
@@ -128,6 +125,9 @@
 "Propose for merging into another branch".  Select "~bzr/bzr/trunk" to hand
 your changes off to the Bazaar developers for review and merging.
 
+Alternatively, after pushing you can use the ``lp-propose`` command to 
+create the merge proposal.
+
 Using a meaningful name for your branch will help you and the reviewer(s)
 better track the submission. Use a very succint description of your submission
 and prefix it with bug number if needed (lp:~mbp/bzr/484558-merge-directory
@@ -135,6 +135,30 @@
 (lp:~jameinel/bzr/export-file-511987).
 
 
+Review cover letters
+--------------------
+
+Please put a "cover letter" on your merge request explaining:
+
+* the reason **why** you're making this change
+
+* **how** this change achieves this purpose
+
+* anything else you may have fixed in passing
+
+* anything significant that you thought of doing, such as a more
+  extensive fix or a different approach, but didn't or couldn't do now
+
+A good cover letter makes reviewers' lives easier because they can decide
+from the letter whether they agree with the purpose and approach, and then
+assess whether the patch actually does what the cover letter says.
+Explaining any "drive-by fixes" or roads not taken may also avoid queries
+from the reviewer.  All in all this should give faster and better reviews.
+Sometimes writing the cover letter helps the submitter realize something
+else they need to do.  The size of the cover letter should be proportional
+to the size and complexity of the patch.
+
+
 Why make a local copy of bzr.dev?
 ---------------------------------
 
@@ -177,13 +201,11 @@
 
 * Launchpad - https://launchpad.net/
 
-* Bazaar - http://bazaar-vcs.org/
-
-* Bundle Buggy - http://bundlebuggy.aaronbentley.com/
+* Bazaar - http://bazaar.canonical.com/
 
 * Patch Queue Manager - https://launchpad.net/pqm/
 
-For further information, see http://bazaar-vcs.org/BzrDevelopment.
+For further information, see <http://wiki.bazaar.canonical.com/BzrDevelopment>.
 
 
 
@@ -192,7 +214,7 @@
 ================================================
 
 Bazaar supports many ways of organising your work. See
-http://bazaar-vcs.org/SharedRepositoryLayouts for a summary of the
+http://wiki.bazaar.canonical.com/SharedRepositoryLayouts for a summary of the
 popular alternatives.
 
 Of course, the best choice for you will depend on numerous factors:
@@ -202,7 +224,7 @@
 * create a local copy of the main development branch (bzr.dev) by using
   this command::
 
-    bzr branch http://bazaar-vcs.org/bzr/bzr.dev/ bzr.dev
+    bzr branch lp:bzr bzr.dev
 
 * keep your copy of bzr.dev pristine (by not developing in it) and keep
   it up to date (by using bzr pull)
@@ -220,7 +242,7 @@
 Navigating the Code Base
 ========================
 
-.. Was at <http://bazaar-vcs.org/NewDeveloperIntroduction>
+.. Was at <http://wiki.bazaar.canonical.com/NewDeveloperIntroduction>
 
 Some of the key files in this directory are:
 
@@ -269,597 +291,7 @@
 <http://starship.python.net/crew/mwh/bzrlibapi/>.
 
 See also the `Bazaar Architectural Overview
-<http://doc.bazaar-vcs.org/developers/overview.html>`_.
-
-
-The Code Review Process
-#######################
-
-All code changes coming in to Bazaar are reviewed by someone else.
-Normally changes by core contributors are reviewed by one other core
-developer, and changes from other people are reviewed by two core
-developers.  Use intelligent discretion if the patch is trivial.
-
-Good reviews do take time. They also regularly require a solid
-understanding of the overall code base. In practice, this means a small
-number of people often have a large review burden - with knowledge comes
-responsibility. No one likes their merge requests sitting in a queue going
-nowhere, so reviewing sooner rather than later is strongly encouraged.
-
-
-
-
-
-Review cover letters
-====================
-
-Please put a "cover letter" on your merge request explaining:
-
-* the reason **why** you're making this change
-
-* **how** this change achieves this purpose
-
-* anything else you may have fixed in passing
-
-* anything significant that you thought of doing, such as a more
-  extensive fix or a different approach, but didn't or couldn't do now
-
-A good cover letter makes reviewers' lives easier because they can decide
-from the letter whether they agree with the purpose and approach, and then
-assess whether the patch actually does what the cover letter says.
-Explaining any "drive-by fixes" or roads not taken may also avoid queries
-from the reviewer.  All in all this should give faster and better reviews.
-Sometimes writing the cover letter helps the submitter realize something
-else they need to do.  The size of the cover letter should be proportional
-to the size and complexity of the patch.
-
-
-Reviewing proposed changes
-==========================
-
-Anyone is welcome to review code, and reply to the thread with their
-opinion or comments.
-
-The simplest way to review a proposed change is to just read the patch on
-the list or in Bundle Buggy.  For more complex changes it may be useful
-to make a new working tree or branch from trunk, and merge the proposed
-change into it, so you can experiment with the code or look at a wider
-context.
-
-There are three main requirements for code to get in:
-
-* Doesn't reduce test coverage: if it adds new methods or commands,
-  there should be tests for them.  There is a good test framework
-  and plenty of examples to crib from, but if you are having trouble
-  working out how to test something feel free to post a draft patch
-  and ask for help.
-
-* Doesn't reduce design clarity, such as by entangling objects
-  we're trying to separate.  This is mostly something the more
-  experienced reviewers need to help check.
-
-* Improves bugs, features, speed, or code simplicity.
-
-Code that goes in should not degrade any of these aspects.  Patches are
-welcome that only cleanup the code without changing the external
-behaviour.  The core developers take care to keep the code quality high
-and understandable while recognising that perfect is sometimes the enemy
-of good.
-
-It is easy for reviews to make people notice other things which should be
-fixed but those things should not hold up the original fix being accepted.
-New things can easily be recorded in the Bug Tracker instead.
-
-It's normally much easier to review several smaller patches than one large
-one.  You might want to use ``bzr-loom`` to maintain threads of related
-work, or submit a preparatory patch that will make your "real" change
-easier.
-
-
-Checklist for reviewers
-=======================
-
-* Do you understand what the code's doing and why?
-
-* Will it perform reasonably for large inputs, both in memory size and
-  run time?  Are there some scenarios where performance should be
-  measured?
-
-* Is it tested, and are the tests at the right level?  Are there both
-  blackbox (command-line level) and API-oriented tests?
-
-* If this change will be visible to end users or API users, is it
-  appropriately documented in NEWS?
-
-* Does it meet the coding standards below?
-
-* If it changes the user-visible behaviour, does it update the help
-  strings and user documentation?
-
-* If it adds a new major concept or standard practice, does it update the
-  developer documentation?
-
-* (your ideas here...)
-
-
-Reviews on Launchpad
-====================
-
-From May 2009 on, we prefer people to propose code reviews through
-Launchpad.
-
- * <https://launchpad.net/+tour/code-review>
-
- * <https://help.launchpad.net/Code/Review>
-
-Anyone can propose or comment on a merge proposal just by creating a
-Launchpad account.
-
-There are two ways to create a new merge proposal: through the web
-interface or by email.
-
-
-Proposing a merge through the web
----------------------------------
-
-To create the proposal through the web, first push your branch to Launchpad.
-For example, a branch dealing with documentation belonging to the Launchpad
-User mbp could be pushed as ::
-
-  bzr push lp:~mbp/bzr/doc
-
-Then go to the branch's web page, which in this case would be
-<https://code.launchpad.net/~mbp/bzr/doc>.  You can simplify this step by just
-running ::
-
-  bzr lp-open
-
-You can then click "Propose for merging into another branch", and enter your
-cover letter (see above) into the web form.  Typically you'll want to merge
-into ``~bzr/bzr/trunk`` which will be the default; you might also want to
-nominate merging into a release branch for a bug fix.  There is the option to
-specify a specific reviewer or type of review, and you shouldn't normally
-change those.
-
-Submitting the form takes you to the new page about the merge proposal
-containing the diff of the changes, comments by interested people, and
-controls to comment or vote on the change.
-
-Proposing a merge by mail
--------------------------
-
-To propose a merge by mail, send a bundle to ``merge@code.launchpad.net``.
-
-You can generate a merge request like this::
-
-  bzr send -o bug-1234.diff
-
-``bzr send`` can also send mail directly if you prefer; see the help.
-
-Reviewing changes
------------------
-
-From <https://code.launchpad.net/bzr/+activereviews> you can see all
-currently active reviews, and choose one to comment on.  This page also
-shows proposals that are now approved and should be merged by someone with
-PQM access.
-
-
-Reviews through Bundle Buggy
-============================
-
-The Bundle Buggy tool used up to May 2009 is still available as a review
-mechanism.
-
-Sending patches for review
---------------------------
-
-If you'd like to propose a change, please post to the
-bazaar@lists.canonical.com list with a bundle, patch, or link to a
-branch. Put ``[PATCH]`` or ``[MERGE]`` in the subject so Bundle Buggy
-can pick it out, and explain the change in the email message text.
-Remember to update the NEWS file as part of your change if it makes any
-changes visible to users or plugin developers. Please include a diff
-against mainline if you're giving a link to a branch.
-
-You can generate a merge request like this::
-
-  bzr send -o bug-1234.patch
-
-A ``.patch`` extension is recommended instead of .bundle as many mail clients
-will send the latter as a binary file.
-
-``bzr send`` can also send mail directly if you prefer; see the help.
-
-Please do **NOT** put [PATCH] or [MERGE] in the subject line if you don't
-want it to be merged. If you want comments from developers rather than
-to be merged, you can put ``[RFC]`` in the subject line.
-
-If this change addresses a bug, please put the bug number in the subject
-line too, in the form ``[#1]`` so that Bundle Buggy can recognize it.
-
-If the change is intended for a particular release mark that in the
-subject too, e.g. ``[1.6]``.
-Anyone can "vote" on the mailing list by expressing an opinion. Core
-developers can also vote using Bundle Buggy. Here are the voting codes and
-their explanations.
-
-:approve:  Reviewer wants this submission merged.
-:tweak:    Reviewer wants this submission merged with small changes. (No
-  re-review required.)
-:abstain:  Reviewer does not intend to vote on this patch.
-:resubmit: Please make changes and resubmit for review.
-:reject:   Reviewer doesn't want this kind of change merged.
-:comment:  Not really a vote. Reviewer just wants to comment, for now.
-
-If a change gets two approvals from core reviewers, and no rejections,
-then it's OK to come in.  Any of the core developers can bring it into the
-bzr.dev trunk and backport it to maintenance branches if required.  The
-Release Manager will merge the change into the branch for a pending
-release, if any. As a guideline, core developers usually merge their own
-changes and volunteer to merge other contributions if they were the second
-reviewer to agree to a change.
-
-To track the progress of proposed changes, use Bundle Buggy. See
-http://bundlebuggy.aaronbentley.com/help for a link to all the
-outstanding merge requests together with an explanation of the columns.
-Bundle Buggy will also mail you a link to track just your change.
-
-Coding Style Guidelines
-#######################
-
-hasattr and getattr
-===================
-
-``hasattr`` should not be used because it swallows exceptions including
-``KeyboardInterrupt``.  Instead, say something like ::
-
-  if getattr(thing, 'name', None) is None
-
-
-Code layout
-===========
-
-Please write PEP-8__ compliant code.
-
-__ http://www.python.org/peps/pep-0008.html
-
-One often-missed requirement is that the first line of docstrings
-should be a self-contained one-sentence summary.
-
-We use 4 space indents for blocks, and never use tab characters.  (In vim,
-``set expandtab``.)
-
-Trailing white space should be avoided, but is allowed.
-You should however not make lots of unrelated white space changes.
-
-Unix style newlines (LF) are used.
-
-Each file must have a newline at the end of it.
-
-Lines should be no more than 79 characters if at all possible.
-Lines that continue a long statement may be indented in either of
-two ways:
-
-within the parenthesis or other character that opens the block, e.g.::
-
-    my_long_method(arg1,
-                   arg2,
-                   arg3)
-
-or indented by four spaces::
-
-    my_long_method(arg1,
-        arg2,
-        arg3)
-
-The first is considered clearer by some people; however it can be a bit
-harder to maintain (e.g. when the method name changes), and it does not
-work well if the relevant parenthesis is already far to the right.  Avoid
-this::
-
-     self.legbone.kneebone.shinbone.toebone.shake_it(one,
-                                                     two,
-                                                     three)
-
-but rather ::
-
-     self.legbone.kneebone.shinbone.toebone.shake_it(one,
-         two,
-         three)
-
-or ::
-
-     self.legbone.kneebone.shinbone.toebone.shake_it(
-         one, two, three)
-
-For long lists, we like to add a trailing comma and put the closing
-character on the following line.  This makes it easier to add new items in
-future::
-
-    from bzrlib.goo import (
-        jam,
-        jelly,
-        marmalade,
-        )
-
-There should be spaces between function parameters, but not between the
-keyword name and the value::
-
-    call(1, 3, cheese=quark)
-
-In emacs::
-
-    ;(defface my-invalid-face
-    ;  '((t (:background "Red" :underline t)))
-    ;  "Face used to highlight invalid constructs or other uglyties"
-    ;  )
-
-    (defun my-python-mode-hook ()
-     ;; setup preferred indentation style.
-     (setq fill-column 79)
-     (setq indent-tabs-mode nil) ; no tabs, never, I will not repeat
-    ;  (font-lock-add-keywords 'python-mode
-    ;                         '(("^\\s *\t" . 'my-invalid-face) ; Leading tabs
-    ;                            ("[ \t]+$" . 'my-invalid-face)  ; Trailing spaces
-    ;                            ("^[ \t]+$" . 'my-invalid-face)); Spaces only
-    ;                          )
-     )
-
-    (add-hook 'python-mode-hook 'my-python-mode-hook)
-
-The lines beginning with ';' are comments. They can be activated
-if one want to have a strong notice of some tab/space usage
-violations.
-
-
-Module Imports
-==============
-
-* Imports should be done at the top-level of the file, unless there is
-  a strong reason to have them lazily loaded when a particular
-  function runs.  Import statements have a cost, so try to make sure
-  they don't run inside hot functions.
-
-* Module names should always be given fully-qualified,
-  i.e. ``bzrlib.hashcache`` not just ``hashcache``.
-
-
-Naming
-======
-
-Functions, methods or members that are relatively private are given
-a leading underscore prefix.  Names without a leading underscore are
-public not just across modules but to programmers using bzrlib as an
-API.
-
-We prefer class names to be concatenated capital words (``TestCase``)
-and variables, methods and functions to be lowercase words joined by
-underscores (``revision_id``, ``get_revision``).
-
-For the purposes of naming some names are treated as single compound
-words: "filename", "revno".
-
-Consider naming classes as nouns and functions/methods as verbs.
-
-Try to avoid using abbreviations in names, because there can be
-inconsistency if other people use the full name.
-
-
-Standard Names
-==============
-
-``revision_id`` not ``rev_id`` or ``revid``
-
-Functions that transform one thing to another should be named ``x_to_y``
-(not ``x2y`` as occurs in some old code.)
-
-
-Destructors
-===========
-
-Python destructors (``__del__``) work differently to those of other
-languages.  In particular, bear in mind that destructors may be called
-immediately when the object apparently becomes unreferenced, or at some
-later time, or possibly never at all.  Therefore we have restrictions on
-what can be done inside them.
-
- 0. If you think you need to use a ``__del__`` method ask another
-    developer for alternatives.  If you do need to use one, explain
-    why in a comment.
-
- 1. Never rely on a ``__del__`` method running.  If there is code that
-    must run, do it from a ``finally`` block instead.
-
- 2. Never ``import`` from inside a ``__del__`` method, or you may crash the
-    interpreter!!
-
- 3. In some places we raise a warning from the destructor if the object
-    has not been cleaned up or closed.  This is considered OK: the warning
-    may not catch every case but it's still useful sometimes.
-
-
-Cleanup methods
-===============
-
-Often when something has failed later code, including cleanups invoked
-from ``finally`` blocks, will fail too.  These secondary failures are
-generally uninteresting compared to the original exception.  So use the
-``only_raises`` decorator (from ``bzrlib.decorators``) for methods that
-are typically called in ``finally`` blocks, such as ``unlock`` methods.
-For example, ``@only_raises(LockNotHeld, LockBroken)``.  All errors that
-are unlikely to be a knock-on failure from a previous failure should be
-allowed.
-
-
-Factories
-=========
-
-In some places we have variables which point to callables that construct
-new instances.  That is to say, they can be used a lot like class objects,
-but they shouldn't be *named* like classes::
-
-> I think that things named FooBar should create instances of FooBar when
-> called. Its plain confusing for them to do otherwise. When we have
-> something that is going to be used as a class - that is, checked for via
-> isinstance or other such idioms, them I would call it foo_class, so that
-> it is clear that a callable is not sufficient. If it is only used as a
-> factory, then yes, foo_factory is what I would use.
-
-
-Registries
-==========
-
-Several places in Bazaar use (or will use) a registry, which is a
-mapping from names to objects or classes.  The registry allows for
-loading in registered code only when it's needed, and keeping
-associated information such as a help string or description.
-
-
-InterObject and multiple dispatch
-=================================
-
-The ``InterObject`` provides for two-way `multiple dispatch`__: matching
-up for example a source and destination repository to find the right way
-to transfer data between them.
-
-.. __: http://en.wikipedia.org/wiki/Multiple_dispatch
-
-There is a subclass ``InterObject`` classes for each type of object that is
-dispatched this way, e.g. ``InterRepository``.  Calling ``.get()`` on this
-class will return an ``InterObject`` instance providing the best match for
-those parameters, and this instance then has methods for operations
-between the objects.
-
-::
-
-  inter = InterRepository.get(source_repo, target_repo)
-  inter.fetch(revision_id)
-
-``InterRepository`` also acts as a registry-like object for its
-subclasses, and they can be added through ``.register_optimizer``.  The
-right one to run is selected by asking each class, in reverse order of
-registration, whether it ``.is_compatible`` with the relevant objects.
-
-Lazy Imports
-============
-
-To make startup time faster, we use the ``bzrlib.lazy_import`` module to
-delay importing modules until they are actually used. ``lazy_import`` uses
-the same syntax as regular python imports. So to import a few modules in a
-lazy fashion do::
-
-  from bzrlib.lazy_import import lazy_import
-  lazy_import(globals(), """
-  import os
-  import subprocess
-  import sys
-  import time
-
-  from bzrlib import (
-     errors,
-     transport,
-     revision as _mod_revision,
-     )
-  import bzrlib.transport
-  import bzrlib.xml5
-  """)
-
-At this point, all of these exist as a ``ImportReplacer`` object, ready to
-be imported once a member is accessed. Also, when importing a module into
-the local namespace, which is likely to clash with variable names, it is
-recommended to prefix it as ``_mod_<module>``. This makes it clearer that
-the variable is a module, and these object should be hidden anyway, since
-they shouldn't be imported into other namespaces.
-
-While it is possible for ``lazy_import()`` to import members of a module
-when using the ``from module import member`` syntax, it is recommended to
-only use that syntax to load sub modules ``from module import submodule``.
-This is because variables and classes can frequently be used without
-needing a sub-member for example::
-
-  lazy_import(globals(), """
-  from module import MyClass
-  """)
-
-  def test(x):
-      return isinstance(x, MyClass)
-
-This will incorrectly fail, because ``MyClass`` is a ``ImportReplacer``
-object, rather than the real class.
-
-It also is incorrect to assign ``ImportReplacer`` objects to other variables.
-Because the replacer only knows about the original name, it is unable to
-replace other variables. The ``ImportReplacer`` class will raise an
-``IllegalUseOfScopeReplacer`` exception if it can figure out that this
-happened. But it requires accessing a member more than once from the new
-variable, so some bugs are not detected right away.
-
-
-The Null revision
-=================
-
-The null revision is the ancestor of all revisions.  Its revno is 0, its
-revision-id is ``null:``, and its tree is the empty tree.  When referring
-to the null revision, please use ``bzrlib.revision.NULL_REVISION``.  Old
-code sometimes uses ``None`` for the null revision, but this practice is
-being phased out.
-
-
-Object string representations
-=============================
-
-Python prints objects using their ``__repr__`` method when they are
-written to logs, exception tracebacks, or the debugger.  We want
-objects to have useful representations to help in determining what went
-wrong.
-
-If you add a new class you should generally add a ``__repr__`` method
-unless there is an adequate method in a parent class.  There should be a
-test for the repr.
-
-Representations should typically look like Python constructor syntax, but
-they don't need to include every value in the object and they don't need
-to be able to actually execute.  They're to be read by humans, not
-machines.  Don't hardcode the classname in the format, so that we get the
-correct value if the method is inherited by a subclass.  If you're
-printing attributes of the object, including strings, you should normally
-use ``%r`` syntax (to call their repr in turn).
-
-Try to avoid the representation becoming more than one or two lines long.
-(But balance this against including useful information, and simplicity of
-implementation.)
-
-Because repr methods are often called when something has already gone
-wrong, they should be written somewhat more defensively than most code.
-The object may be half-initialized or in some other way in an illegal
-state.  The repr method shouldn't raise an exception, or it may hide the
-(probably more useful) underlying exception.
-
-Example::
-
-    def __repr__(self):
-        return '%s(%r)' % (self.__class__.__name__,
-                           self._transport)
-
-
-Exception handling
-==================
-
-A bare ``except`` statement will catch all exceptions, including ones that
-really should terminate the program such as ``MemoryError`` and
-``KeyboardInterrupt``.  They should rarely be used unless the exception is
-later re-raised.  Even then, think about whether catching just
-``Exception`` (which excludes system errors in Python2.5 and later) would
-be better.
-
-
-Test coverage
-=============
-
-All code should be exercised by the test suite.  See the `Bazaar Testing
-Guide <http://doc.bazaar-vcs.org/developers/testing.html>`_ for detailed
-information about writing tests.
+<http://doc.bazaar.canonical.com/developers/overview.html>`_.
 
 
 Core Topics
@@ -927,235 +359,6 @@
 can't fix.
 
 
-Getting Input
-=============
-
-Processing Command Lines
-------------------------
-
-bzrlib has a standard framework for parsing command lines and calling
-processing routines associated with various commands. See builtins.py
-for numerous examples.
-
-
-Standard Parameter Types
-------------------------
-
-There are some common requirements in the library: some parameters need to be
-unicode safe, some need byte strings, and so on. At the moment we have
-only codified one specific pattern: Parameters that need to be unicode
-should be checked via ``bzrlib.osutils.safe_unicode``. This will coerce the
-input into unicode in a consistent fashion, allowing trivial strings to be
-used for programmer convenience, but not performing unpredictably in the
-presence of different locales.
-
-
-Writing Output
-==============
-
-(The strategy described here is what we want to get to, but it's not
-consistently followed in the code at the moment.)
-
-bzrlib is intended to be a generically reusable library.  It shouldn't
-write messages to stdout or stderr, because some programs that use it
-might want to display that information through a GUI or some other
-mechanism.
-
-We can distinguish two types of output from the library:
-
- 1. Structured data representing the progress or result of an
-    operation.  For example, for a commit command this will be a list
-    of the modified files and the finally committed revision number
-    and id.
-
-    These should be exposed either through the return code or by calls
-    to a callback parameter.
-
-    A special case of this is progress indicators for long-lived
-    operations, where the caller should pass a ProgressBar object.
-
- 2. Unstructured log/debug messages, mostly for the benefit of the
-    developers or users trying to debug problems.  This should always
-    be sent through ``bzrlib.trace`` and Python ``logging``, so that
-    it can be redirected by the client.
-
-The distinction between the two is a bit subjective, but in general if
-there is any chance that a library would want to see something as
-structured data, we should make it so.
-
-The policy about how output is presented in the text-mode client
-should be only in the command-line tool.
-
-
-Progress and Activity Indications
----------------------------------
-
-bzrlib has a way for code to display to the user that stuff is happening
-during a long operation.  There are two particular types: *activity* which
-means that IO is happening on a Transport, and *progress* which means that
-higher-level application work is occurring.  Both are drawn together by
-the `ui_factory`.
-
-Transport objects are responsible for calling `report_transport_activity`
-when they do IO.
-
-Progress uses a model/view pattern: application code acts on a
-`ProgressTask` object, which notifies the UI when it needs to be
-displayed.  Progress tasks form a stack.  To create a new progress task on
-top of the stack, call `bzrlib.ui.ui_factory.nested_progress_bar()`, then
-call `update()` on the returned ProgressTask.  It can be updated with just
-a text description, with a numeric count, or with a numeric count and
-expected total count.  If an expected total count is provided the view
-can show the progress moving along towards the expected total.
-
-The user should call `finish` on the `ProgressTask` when the logical
-operation has finished, so it can be removed from the stack.
-
-Progress tasks have a complex relationship with generators: it's a very
-good place to use them, but because python2.4 does not allow ``finally``
-blocks in generators it's hard to clean them up properly.  In this case
-it's probably better to have the code calling the generator allocate a
-progress task for its use and then call `finalize` when it's done, which
-will close it if it was not already closed.  The generator should also
-finish the progress task when it exits, because it may otherwise be a long
-time until the finally block runs.
-
-
-Message guidelines
-------------------
-
-When filenames or similar variables are presented inline within a message,
-they should be enclosed in double quotes (ascii 0x22, not chiral unicode
-quotes)::
-
-  bzr: ERROR: No such file "asdf"
-
-When we print just a list of filenames there should not be any quoting:
-see `bug 544297`_.
-
-.. _bug 544297: https://bugs.launchpad.net/bugs/544297
-
-https://wiki.ubuntu.com/UnitsPolicy provides a good explanation about
-which unit should be used when. Roughly speaking, IEC standard applies
-for base-2 units and SI standard applies for base-10 units:
-
-* for network bandwidth and disk sizes, use base-10 (Mbits/s, kB/s, GB)
-
-* for RAM sizes, use base-2 (GiB, TiB)
-
-
-
-Displaying help
-===============
-
-Bazaar has online help for various topics through ``bzr help COMMAND`` or
-equivalently ``bzr command -h``.  We also have help on command options,
-and on other help topics.  (See ``help_topics.py``.)
-
-As for python docstrings, the first paragraph should be a single-sentence
-synopsis of the command. These are user-visible and should be prefixed with
-``__doc__ =`` so help works under ``python -OO`` with docstrings stripped.
-
-The help for options should be one or more proper sentences, starting with
-a capital letter and finishing with a full stop (period).
-
-All help messages and documentation should have two spaces between
-sentences.
-
-
-Handling Errors and Exceptions
-==============================
-
-Commands should return non-zero when they encounter circumstances that
-the user should really pay attention to - which includes trivial shell
-pipelines.
-
-Recommended values are:
-
-    0. OK.
-    1. Conflicts in merge-like operations, or changes are present in
-       diff-like operations.
-    2. Unrepresentable diff changes (i.e. binary files that we cannot show
-       a diff of).
-    3. An error or exception has occurred.
-    4. An internal error occurred (one that shows a traceback.)
-
-Errors are handled through Python exceptions. Exceptions should be defined
-inside bzrlib.errors, so that we can see the whole tree at a glance.
-
-We broadly classify errors as either being either internal or not,
-depending on whether ``internal_error`` is set or not.  If we think it's our
-fault, we show a backtrace, an invitation to report the bug, and possibly
-other details.  This is the default for errors that aren't specifically
-recognized as being caused by a user error.  Otherwise we show a briefer
-message, unless -Derror was given.
-
-Many errors originate as "environmental errors" which are raised by Python
-or builtin libraries -- for example IOError.  These are treated as being
-our fault, unless they're caught in a particular tight scope where we know
-that they indicate a user errors.  For example if the repository format
-is not found, the user probably gave the wrong path or URL.  But if one of
-the files inside the repository is not found, then it's our fault --
-either there's a bug in bzr, or something complicated has gone wrong in
-the environment that means one internal file was deleted.
-
-Many errors are defined in ``bzrlib/errors.py`` but it's OK for new errors
-to be added near the place where they are used.
-
-Exceptions are formatted for the user by conversion to a string
-(eventually calling their ``__str__`` method.)  As a convenience the
-``._fmt`` member can be used as a template which will be mapped to the
-error's instance dict.
-
-New exception classes should be defined when callers might want to catch
-that exception specifically, or when it needs a substantially different
-format string.
-
-#. If it is something that a caller can recover from, a custom exception
-   is reasonable.
-
-#. If it is a data consistency issue, using a builtin like
-   ``ValueError``/``TypeError`` is reasonable.
-
-#. If it is a programmer error (using an api incorrectly)
-   ``AssertionError`` is reasonable.
-
-#. Otherwise, use ``BzrError`` or ``InternalBzrError``.
-
-Exception strings should start with a capital letter and should not have a
-final fullstop.  If long, they may contain newlines to break the text.
-
-
-Assertions
-==========
-
-Do not use the Python ``assert`` statement, either in tests or elsewhere.
-A source test checks that it is not used.  It is ok to explicitly raise
-AssertionError.
-
-Rationale:
-
- * It makes the behaviour vary depending on whether bzr is run with -O
-   or not, therefore giving a chance for bugs that occur in one case or
-   the other, several of which have already occurred: assertions with
-   side effects, code which can't continue unless the assertion passes,
-   cases where we should give the user a proper message rather than an
-   assertion failure.
- * It's not that much shorter than an explicit if/raise.
- * It tends to lead to fuzzy thinking about whether the check is
-   actually needed or not, and whether it's an internal error or not
- * It tends to cause look-before-you-leap patterns.
- * It's unsafe if the check is needed to protect the integrity of the
-   user's data.
- * It tends to give poor messages since the developer can get by with
-   no explanatory text at all.
- * We can't rely on people always running with -O in normal use, so we
-   can't use it for tests that are actually expensive.
- * Expensive checks that help developers are better turned on from the
-   test suite or a -D flag.
- * If used instead of ``self.assert*()`` in tests it makes them falsely pass with -O.
-
-
 Documenting Changes
 ===================
 
@@ -1378,17 +581,6 @@
 valid characters are generated where possible.
 
 
-Portability Tips
-================
-
-The ``bzrlib.osutils`` module has many useful helper functions, including
-some more portable variants of functions in the standard library.
-
-In particular, don't use ``shutil.rmtree`` unless it's acceptable for it
-to fail on Windows if some files are readonly or still open elsewhere.
-Use ``bzrlib.osutils.rmtree`` instead.
-
-
 C Extension Modules
 ===================
 
@@ -1431,8 +623,7 @@
 Making Installers for OS Windows
 ================================
 To build a win32 installer, see the instructions on the wiki page:
-http://bazaar-vcs.org/BzrWin32Installer
-
+http://wiki.bazaar.canonical.com/BzrWin32Installer
 
 Core Developer Tasks
 ####################
@@ -1451,7 +642,7 @@
 * reviewing changes
 * reviewing blueprints
 * planning releases
-* managing releases (see `Releasing Bazaar <http://doc.bazaar-vcs.org/developers/releasing.html>`_)
+* managing releases (see `Releasing Bazaar <http://doc.bazaar.canonical.com/developers/releasing.html>`_)
 
 .. note::
   Removing barriers to community participation is a key reason for adopting
@@ -1478,7 +669,7 @@
 energy by emailing the **bazaar-commits** list implicitly. To do this,
 install and configure the Email plugin. One way to do this is add these
 configuration settings to your central configuration file (e.g.
-``~/.bazaar/bazaar.conf`` on Linux)::
+``~/.bazaar/bazaar.conf``)::
 
   [DEFAULT]
   email = Joe Smith <joe.smith@internode.on.net>
@@ -1494,196 +685,11 @@
 how to set it up and configure it.
 
 
-Submitting Changes
-==================
-
-An Overview of PQM
-------------------
-
-Of the many workflows supported by Bazaar, the one adopted for Bazaar
-development itself is known as "Decentralized with automatic gatekeeper".
-To repeat the explanation of this given on
-http://bazaar-vcs.org/Workflows:
-
-.. pull-quote::
-  In this workflow, each developer has their own branch or
-  branches, plus read-only access to the mainline. A software gatekeeper
-  (e.g. PQM) has commit rights to the main branch. When a developer wants
-  their work merged, they request the gatekeeper to merge it. The gatekeeper
-  does a merge, a compile, and runs the test suite. If the code passes, it
-  is merged into the mainline.
-
-In a nutshell, here's the overall submission process:
-
-#. get your work ready (including review except for trivial changes)
-#. push to a public location
-#. ask PQM to merge from that location
-
-.. note::
-  At present, PQM always takes the changes to merge from a branch
-  at a URL that can be read by it. For Bazaar, that means a public,
-  typically http, URL.
-
-As a result, the following things are needed to use PQM for submissions:
-
-#. A publicly available web server
-#. Your OpenPGP key registered with PQM (contact RobertCollins for this)
-#. The PQM plugin installed and configured (not strictly required but
-   highly recommended).
-
-
-Selecting a Public Branch Location
-----------------------------------
-
-If you don't have your own web server running, branches can always be
-pushed to Launchpad. Here's the process for doing that:
-
-Depending on your location throughout the world and the size of your
-repository though, it is often quicker to use an alternative public
-location to Launchpad, particularly if you can set up your own repo and
-push into that. By using an existing repo, push only needs to send the
-changes, instead of the complete repository every time. Note that it is
-easy to register branches in other locations with Launchpad so no benefits
-are lost by going this way.
-
-.. note::
-  For Canonical staff, http://people.ubuntu.com/~<user>/ is one
-  suggestion for public http branches. Contact your manager for information
-  on accessing this system if required.
-
-It should also be noted that best practice in this area is subject to
-change as things evolve. For example, once the Bazaar smart server on
-Launchpad supports server-side branching, the performance situation will
-be very different to what it is now (Jun 2007).
-
-
-Configuring the PQM Plug-In
----------------------------
-
-While not strictly required, the PQM plugin automates a few things and
-reduces the chance of error. Before looking at the plugin, it helps to
-understand  a little more how PQM operates. Basically, PQM requires an
-email indicating what you want it to do. The email typically looks like
-this::
-
-  star-merge source-branch target-branch
-
-For example::
-
-  star-merge http://bzr.arbash-meinel.com/branches/bzr/jam-integration http://bazaar-vcs.org/bzr/bzr.dev
-
-Note that the command needs to be on one line. The subject of the email
-will be used for the commit message. The email also needs to be ``gpg``
-signed with a key that PQM accepts.
-
-The advantages of using the PQM plugin are:
-
-#. You can use the config policies to make it easy to set up public
-   branches, so you don't have to ever type the full paths you want to merge
-   from or into.
-
-#. It checks to make sure the public branch last revision matches the
-   local last revision so you are submitting what you think you are.
-
-#. It uses the same public_branch and smtp sending settings as bzr-email,
-   so if you have one set up, you have the other mostly set up.
-
-#. Thunderbird refuses to not wrap lines, and request lines are usually
-   pretty long (you have 2 long URLs in there).
-
-Here are sample configuration settings for the PQM plugin. Here are the
-lines in bazaar.conf::
-
-  [DEFAULT]
-  email = Joe Smith <joe.smith@internode.on.net>
-  smtp_server=mail.internode.on.net:25
-
-And here are the lines in ``locations.conf`` (or ``branch.conf`` for
-dirstate-tags branches)::
-
-  [/home/joe/bzr/my-integration]
-  push_location = sftp://joe-smith@bazaar.launchpad.net/%7Ejoe-smith/bzr/my-integration/
-  push_location:policy = norecurse
-  public_branch = http://bazaar.launchpad.net/~joe-smith/bzr/my-integration/
-  public_branch:policy = appendpath
-  pqm_email = Bazaar PQM <pqm@bazaar-vcs.org>
-  pqm_branch = http://bazaar-vcs.org/bzr/bzr.dev
-
-Note that the push settings will be added by the first ``push`` on
-a branch. Indeed the preferred way to generate the lines above is to use
-``push`` with an argument, then copy-and-paste the other lines into
-the relevant file.
-
-
-Submitting a Change
--------------------
-
-Here is one possible recipe once the above environment is set up:
-
-#. pull bzr.dev => my-integration
-#. merge patch => my-integration
-#. fix up any final merge conflicts (NEWS being the big killer here).
-#. commit
-#. push
-#. pqm-submit
-
-.. note::
-  The ``push`` step is not required if ``my-integration`` is a checkout of
-  a public branch.
-
-  Because of defaults, you can type a single message into commit and
-  pqm-commit will reuse that.
-
-
-Tracking Change Acceptance
---------------------------
-
-The web interface to PQM is https://pqm.bazaar-vcs.org/. After submitting
-a change, you can visit this URL to confirm it was received and placed in
-PQM's queue.
-
-When PQM completes processing a change, an email is sent to you with the
-results.
-
-
-Reviewing Blueprints
-====================
-
-Blueprint Tracking Using Launchpad
-----------------------------------
-
-New features typically require a fair amount of discussion, design and
-debate. For Bazaar, that information is often captured in a so-called
-"blueprint" on our Wiki. Overall tracking of blueprints and their status
-is done using Launchpad's relevant tracker,
-https://blueprints.launchpad.net/bzr/. Once a blueprint for ready for
-review, please announce it on the mailing list.
-
-Alternatively, send an email beginning with [RFC] with the proposal to the
-list. In some cases, you may wish to attach proposed code  or a proposed
-developer document if that best communicates the idea. Debate can then
-proceed using the normal merge review processes.
-
-
-Recording Blueprint Review Feedback
------------------------------------
-
-Unlike its Bug Tracker, Launchpad's Blueprint Tracker doesn't currently
-(Jun 2007) support a chronological list of comment responses. Review
-feedback can either be recorded on the Wiki hosting the blueprints or by
-using Launchpad's whiteboard feature.
-
 
 Planning Releases
 =================
 
 
-Using Releases and Milestones in Launchpad
-------------------------------------------
-
-TODO ... (Exact policies still under discussion)
-
-
 Bug Triage
 ----------
 

=== modified file 'doc/developers/bug-handling.txt'
--- doc/developers/bug-handling.txt	2010-02-18 04:17:22 +0000
+++ doc/developers/bug-handling.txt	2010-05-31 19:33:05 +0000
@@ -303,6 +303,9 @@
 performance
     bugs about performance problems.
 
+regression
+    bugs which represent an aspect of bzr becoming accidentally less good than it was.
+
 test
     needs changes to the test framework
 

=== modified file 'doc/developers/case-insensitive-file-systems.txt'
--- doc/developers/case-insensitive-file-systems.txt	2009-12-02 20:34:07 +0000
+++ doc/developers/case-insensitive-file-systems.txt	2010-08-13 19:08:57 +0000
@@ -9,13 +9,13 @@
 For example, the FAT32 file-system is most commonly found on Windows operating
 systems, and has the characteristics usually associated with a Windows
 file-system.  However, USB devices means FAT32 file-systems are often used
-with Linux, so the current operating system doesn't necessarily reflect the
+with GNU/Linux systems, so the current operating system doesn't necessarily reflect the
 capabilities of the file-system.
 
 Bazaar supports 3 kinds of file-systems, each to different degrees.
 
 * Case-sensitive file-systems: This is the file-system generally used on
-  Linux - 2 files can differ only by case, and the exact case must be used
+  GNU/Linux: 2 files can differ only by case, and the exact case must be used
   when opening a file.
 
 * Case-insensitive, case-preserving (cicp) file-systems: This is the
@@ -32,7 +32,7 @@
 relevant to a modern Bazaar.
 
 For more details, including use cases, please see
-http://bazaar-vcs.org/CasePreservingWorkingTreeUseCases
+http://wiki.bazaar.canonical.com/CasePreservingWorkingTreeUseCases
 
 Handling these file-systems
 ---------------------------

=== added file 'doc/developers/code-review.txt'
--- doc/developers/code-review.txt	1970-01-01 00:00:00 +0000
+++ doc/developers/code-review.txt	2010-09-24 08:42:02 +0000
@@ -0,0 +1,104 @@
+Reviewing proposed changes to Bazaar
+####################################
+
+All non-trivial code changes coming in to Bazaar are reviewed by someone else.
+
+Anyone is welcome to review any patch.  You don't need to have a full 
+understanding of the codebase to find problems in the code, the documentation,
+or the concept of the patch.
+
+Normally changes by core contributors are reviewed by one other core
+developer, and changes from other people are reviewed by two core
+developers.  Use intelligent discretion about whether the patch is trivial.
+
+No one likes their merge requests sitting in a queue going nowhere: this 
+is pure waste.  We prioritize reviewing existing proposals.  
+Canonical dedicates some staff time to providing prompt helpful reviews.  
+(See <http://wiki.bazaar.canonical.com/PatchPilot/>.)
+
+From late 2009 on, we do all our code reviews through Launchpad's 
+merge proposal interface.
+
+
+Reviewing proposed changes
+==========================
+
+There are three main requirements for code to get in:
+
+* Doesn't reduce test coverage: if it adds new methods or commands,
+  there should be tests for them.  There is a good test framework
+  and plenty of examples to crib from, but if you are having trouble
+  working out how to test something feel free to post a draft patch
+  and ask for help.
+
+* Doesn't reduce design clarity, such as by entangling objects
+  we're trying to separate.  This is mostly something the more
+  experienced reviewers need to help check.
+
+* Improves bugs, features, speed, or code simplicity.
+
+Code that goes in should not degrade any of these aspects.  Patches are
+welcome that only cleanup the code without changing the external
+behaviour.  The core developers take care to keep the code quality high
+and understandable while recognising that perfect is sometimes the enemy
+of good.
+
+It is easy for reviews to make people notice other things which should be
+fixed but those things should not hold up the original fix being accepted.
+New things can easily be recorded in the bug tracker instead.
+
+It's normally much easier to review several smaller patches than one large
+one.  You might want to use ``bzr-loom`` to maintain threads of related
+work, or submit a preparatory patch that will make your "real" change
+easier.
+
+
+Checklist for reviewers
+=======================
+
+* Do you understand what the code's doing and why?
+
+* Will it perform reasonably for large inputs, both in memory size and
+  run time?  Are there some scenarios where performance should be
+  measured?
+
+* Is it tested, and are the tests at the right level?  Are there both
+  blackbox (command-line level) and API-oriented tests?
+
+* If this change will be visible to end users or API users, is it
+  appropriately documented in NEWS and/or in whats-new ?
+
+* Does it meet the `coding standards <code-style.html>`_?
+
+* If it changes the user-visible behaviour, does it update the help
+  strings and user documentation?
+
+* If it adds a new major concept or standard practice, does it update the
+  developer documentation?
+
+* (your ideas here...)
+
+
+Reviews on Launchpad
+====================
+
+Anyone can propose or comment on a merge proposal just by creating a
+Launchpad account.
+
+From <https://code.launchpad.net/bzr/+activereviews> you can see all
+currently active reviews, and choose one to comment on.  This page also
+shows proposals that are now approved and should be merged by someone with
+PQM access.
+
+<https://help.launchpad.net/Code/Review> explains the various merge proposal
+states.  Note that we don't use state *Approved* until the patch is completely
+ready to merge.
+
+
+Landing approved changes
+========================
+
+Once a merge proposal is approved and finished, it's sent to PQM (the patch
+queue manager) which will automatically test and integrate it.  The recommended
+way to start this off is by running the ``feed-pqm`` script from 
+<https://launchpad.net/hydrazine/>.

=== added file 'doc/developers/code-style.txt'
--- doc/developers/code-style.txt	1970-01-01 00:00:00 +0000
+++ doc/developers/code-style.txt	2010-09-24 08:42:02 +0000
@@ -0,0 +1,489 @@
+***********************
+Bazaar Code Style Guide
+***********************
+
+Code layout
+===========
+
+Please write PEP-8__ compliant code.
+
+__ http://www.python.org/peps/pep-0008.html
+
+One often-missed requirement is that the first line of docstrings
+should be a self-contained one-sentence summary.
+
+We use 4 space indents for blocks, and never use tab characters.  (In vim,
+``set expandtab``.)
+
+Trailing white space should be avoided, but is allowed.
+You should however not make lots of unrelated white space changes.
+
+Unix style newlines (LF) are used.
+
+Each file must have a newline at the end of it.
+
+Lines should be no more than 79 characters if at all possible.
+Lines that continue a long statement may be indented in either of
+two ways:
+
+within the parenthesis or other character that opens the block, e.g.::
+
+    my_long_method(arg1,
+                   arg2,
+                   arg3)
+
+or indented by four spaces::
+
+    my_long_method(arg1,
+        arg2,
+        arg3)
+
+The first is considered clearer by some people; however it can be a bit
+harder to maintain (e.g. when the method name changes), and it does not
+work well if the relevant parenthesis is already far to the right.  Avoid
+this::
+
+     self.legbone.kneebone.shinbone.toebone.shake_it(one,
+                                                     two,
+                                                     three)
+
+but rather ::
+
+     self.legbone.kneebone.shinbone.toebone.shake_it(one,
+         two,
+         three)
+
+or ::
+
+     self.legbone.kneebone.shinbone.toebone.shake_it(
+         one, two, three)
+
+For long lists, we like to add a trailing comma and put the closing
+character on the following line.  This makes it easier to add new items in
+future::
+
+    from bzrlib.goo import (
+        jam,
+        jelly,
+        marmalade,
+        )
+
+There should be spaces between function parameters, but not between the
+keyword name and the value::
+
+    call(1, 3, cheese=quark)
+
+
+Python versions
+===============
+
+Bazaar supports Python from 2.4 through 2.6, and in the future we want to
+support Python 2.7 and 3.0.  Avoid using language features added in 2.5,
+2.6 or 2.7, or features deprecated in Python 3.0.  (You can check v3
+compatibility using the ``-3`` option of Python2.6.)
+
+Specifically:
+
+* Don't use the ``with`` statement.
+
+* Don't ``from . import``.
+
+* Don't use ``try/except/finally``, which is not supported in Python2.4,
+  use separate nested ``try/except`` and ``try/finally`` blocks.
+
+
+hasattr and getattr
+===================
+
+``hasattr`` should not be used because it swallows exceptions including
+``KeyboardInterrupt``.  Instead, say something like ::
+
+  if getattr(thing, 'name', None) is None
+
+
+kwargs
+======
+
+``**kwargs`` in the prototype of a function should be used sparingly.
+It can be good on higher-order functions that decorate other functions,
+such as ``addCleanup`` or ``assertRaises``, or on functions that take only
+(or almost only) kwargs, where any kwargs can be passed.  
+
+Otherwise, be careful: if the parameters to a function are a bit complex
+and might vary over time (e.g.  the ``commit`` API) then we prefer to pass an
+object rather than a bag of positional and/or keyword args.  If you have
+an arbitrary set of keys and values that are different with each use (e.g.
+string interpolation inputs) then again that should not be mixed in with
+the regular positional/keyword args, it seems like a different category of
+thing.
+
+
+Imitating standard objects
+==========================
+
+Don't provide methods that imitate built-in classes (eg ``__in__``,
+``__call__``, ``__int__``, ``__getitem__``) unless the class you're
+implementing really does act like the builtin class, in semantics and
+performance.
+
+For example, old code lets you say ``file_id in inv`` but we no longer
+consider this good style.  Instead, say more explicitly
+``inv.has_id(file_id)``.
+
+``__repr__``, ``__cmp__``, ``__str__`` are usually fine.
+
+
+Module Imports
+==============
+
+* Imports should be done at the top-level of the file, unless there is
+  a strong reason to have them lazily loaded when a particular
+  function runs.  Import statements have a cost, so try to make sure
+  they don't run inside hot functions.
+
+* Module names should always be given fully-qualified,
+  i.e. ``bzrlib.hashcache`` not just ``hashcache``.
+
+
+Naming
+======
+
+Functions, methods or members that are relatively private are given
+a leading underscore prefix.  Names without a leading underscore are
+public not just across modules but to programmers using bzrlib as an
+API.
+
+We prefer class names to be concatenated capital words (``TestCase``)
+and variables, methods and functions to be lowercase words joined by
+underscores (``revision_id``, ``get_revision``).
+
+For the purposes of naming some names are treated as single compound
+words: "filename", "revno".
+
+Consider naming classes as nouns and functions/methods as verbs.
+
+Try to avoid using abbreviations in names, because there can be
+inconsistency if other people use the full name.
+
+
+Standard Names
+==============
+
+``revision_id`` not ``rev_id`` or ``revid``
+
+Functions that transform one thing to another should be named ``x_to_y``
+(not ``x2y`` as occurs in some old code.)
+
+
+Destructors
+===========
+
+Python destructors (``__del__``) work differently to those of other
+languages.  In particular, bear in mind that destructors may be called
+immediately when the object apparently becomes unreferenced, or at some
+later time, or possibly never at all.  Therefore we have restrictions on
+what can be done inside them.
+
+0. If you think you need to use a ``__del__`` method ask another
+   developer for alternatives.  If you do need to use one, explain
+   why in a comment.
+
+1. Never rely on a ``__del__`` method running.  If there is code that
+   must run, do it from a ``finally`` block instead.
+
+2. Never ``import`` from inside a ``__del__`` method, or you may crash the
+   interpreter!!
+
+3. In some places we raise a warning from the destructor if the object
+   has not been cleaned up or closed.  This is considered OK: the warning
+   may not catch every case but it's still useful sometimes.
+
+
+Cleanup methods
+===============
+
+Often when something has failed later code will fail too, including
+cleanups invoked from ``finally`` blocks.  These secondary failures are
+generally uninteresting compared to the original exception.  ``bzrlib``
+has some facilities you can use to mitigate this.
+
+* In ``Command`` subclasses, prefer the ``add_cleanup`` method to using
+  ``try``/``finally`` blocks.  E.g. to acquire a lock and ensure it will
+  always be released when the command is done::
+
+    self.add_cleanup(branch.lock_read().unlock)
+
+  This also avoids heavily indented code. It also makes it easier to notice
+  mismatched lock/unlock pairs (and other kinds of resource
+  acquire/release) because there isn't a large block of code separating
+  them.
+
+* Use the ``only_raises`` decorator (from ``bzrlib.decorators``) when
+  defining methods that are typically called in ``finally`` blocks, such
+  as ``unlock`` methods.  For example, ``@only_raises(LockNotHeld,
+  LockBroken)``.  All errors that are unlikely to be a knock-on failure
+  from a previous failure should be allowed.
+
+* Consider using the ``OperationWithCleanups`` helper from
+  ``bzrlib.cleanup`` anywhere else you have a ``finally`` block that
+  might fail.
+
+
+Factories
+=========
+
+In some places we have variables which point to callables that construct
+new instances.  That is to say, they can be used a lot like class objects,
+but they shouldn't be *named* like classes.  Things called ``FooBar`` should
+create an instance of ``FooBar``.  A factory method that might create a
+``FooBar`` or might make something else should be called ``foo_factory``.
+
+
+Registries
+==========
+
+Several places in Bazaar use (or will use) a registry, which is a
+mapping from names to objects or classes.  The registry allows for
+loading in registered code only when it's needed, and keeping
+associated information such as a help string or description.
+
+
+InterObject and multiple dispatch
+=================================
+
+The ``InterObject`` provides for two-way `multiple dispatch`__: matching
+up for example a source and destination repository to find the right way
+to transfer data between them.
+
+.. __: http://en.wikipedia.org/wiki/Multiple_dispatch
+
+There is a subclass ``InterObject`` classes for each type of object that is
+dispatched this way, e.g. ``InterRepository``.  Calling ``.get()`` on this
+class will return an ``InterObject`` instance providing the best match for
+those parameters, and this instance then has methods for operations
+between the objects.
+
+::
+
+  inter = InterRepository.get(source_repo, target_repo)
+  inter.fetch(revision_id)
+
+``InterRepository`` also acts as a registry-like object for its
+subclasses, and they can be added through ``.register_optimizer``.  The
+right one to run is selected by asking each class, in reverse order of
+registration, whether it ``.is_compatible`` with the relevant objects.
+
+Lazy Imports
+============
+
+To make startup time faster, we use the ``bzrlib.lazy_import`` module to
+delay importing modules until they are actually used. ``lazy_import`` uses
+the same syntax as regular python imports. So to import a few modules in a
+lazy fashion do::
+
+  from bzrlib.lazy_import import lazy_import
+  lazy_import(globals(), """
+  import os
+  import subprocess
+  import sys
+  import time
+
+  from bzrlib import (
+     errors,
+     transport,
+     revision as _mod_revision,
+     )
+  import bzrlib.transport
+  import bzrlib.xml5
+  """)
+
+At this point, all of these exist as a ``ImportReplacer`` object, ready to
+be imported once a member is accessed. Also, when importing a module into
+the local namespace, which is likely to clash with variable names, it is
+recommended to prefix it as ``_mod_<module>``. This makes it clearer that
+the variable is a module, and these object should be hidden anyway, since
+they shouldn't be imported into other namespaces.
+
+While it is possible for ``lazy_import()`` to import members of a module
+when using the ``from module import member`` syntax, it is recommended to
+only use that syntax to load sub modules ``from module import submodule``.
+This is because variables and classes can frequently be used without
+needing a sub-member for example::
+
+  lazy_import(globals(), """
+  from module import MyClass
+  """)
+
+  def test(x):
+      return isinstance(x, MyClass)
+
+This will incorrectly fail, because ``MyClass`` is a ``ImportReplacer``
+object, rather than the real class.
+
+It also is incorrect to assign ``ImportReplacer`` objects to other variables.
+Because the replacer only knows about the original name, it is unable to
+replace other variables. The ``ImportReplacer`` class will raise an
+``IllegalUseOfScopeReplacer`` exception if it can figure out that this
+happened. But it requires accessing a member more than once from the new
+variable, so some bugs are not detected right away.
+
+
+The Null revision
+=================
+
+The null revision is the ancestor of all revisions.  Its revno is 0, its
+revision-id is ``null:``, and its tree is the empty tree.  When referring
+to the null revision, please use ``bzrlib.revision.NULL_REVISION``.  Old
+code sometimes uses ``None`` for the null revision, but this practice is
+being phased out.
+
+
+Object string representations
+=============================
+
+Python prints objects using their ``__repr__`` method when they are
+written to logs, exception tracebacks, or the debugger.  We want
+objects to have useful representations to help in determining what went
+wrong.
+
+If you add a new class you should generally add a ``__repr__`` method
+unless there is an adequate method in a parent class.  There should be a
+test for the repr.
+
+Representations should typically look like Python constructor syntax, but
+they don't need to include every value in the object and they don't need
+to be able to actually execute.  They're to be read by humans, not
+machines.  Don't hardcode the classname in the format, so that we get the
+correct value if the method is inherited by a subclass.  If you're
+printing attributes of the object, including strings, you should normally
+use ``%r`` syntax (to call their repr in turn).
+
+Try to avoid the representation becoming more than one or two lines long.
+(But balance this against including useful information, and simplicity of
+implementation.)
+
+Because repr methods are often called when something has already gone
+wrong, they should be written somewhat more defensively than most code.
+The object may be half-initialized or in some other way in an illegal
+state.  The repr method shouldn't raise an exception, or it may hide the
+(probably more useful) underlying exception.
+
+Example::
+
+    def __repr__(self):
+        return '%s(%r)' % (self.__class__.__name__,
+                           self._transport)
+
+
+Exception handling
+==================
+
+A bare ``except`` statement will catch all exceptions, including ones that
+really should terminate the program such as ``MemoryError`` and
+``KeyboardInterrupt``.  They should rarely be used unless the exception is
+later re-raised.  Even then, think about whether catching just
+``Exception`` (which excludes system errors in Python2.5 and later) would
+be better.
+
+
+Test coverage
+=============
+
+All code should be exercised by the test suite.  See the `Bazaar Testing
+Guide <http://doc.bazaar.canonical.com/developers/testing.html>`_ for detailed
+information about writing tests.
+
+
+Assertions
+==========
+
+Do not use the Python ``assert`` statement, either in tests or elsewhere.
+A source test checks that it is not used.  It is ok to explicitly raise
+AssertionError.
+
+Rationale:
+
+* It makes the behaviour vary depending on whether bzr is run with -O
+  or not, therefore giving a chance for bugs that occur in one case or
+  the other, several of which have already occurred: assertions with
+  side effects, code which can't continue unless the assertion passes,
+  cases where we should give the user a proper message rather than an
+  assertion failure.
+* It's not that much shorter than an explicit if/raise.
+* It tends to lead to fuzzy thinking about whether the check is
+  actually needed or not, and whether it's an internal error or not
+* It tends to cause look-before-you-leap patterns.
+* It's unsafe if the check is needed to protect the integrity of the
+  user's data.
+* It tends to give poor messages since the developer can get by with
+  no explanatory text at all.
+* We can't rely on people always running with -O in normal use, so we
+  can't use it for tests that are actually expensive.
+* Expensive checks that help developers are better turned on from the
+  test suite or a -D flag.
+* If used instead of ``self.assert*()`` in tests it makes them falsely
+  pass with -O.
+
+emacs setup
+===========
+
+In emacs::
+
+    ;(defface my-invalid-face
+    ;  '((t (:background "Red" :underline t)))
+    ;  "Face used to highlight invalid constructs or other uglyties"
+    ;  )
+
+    (defun my-python-mode-hook ()
+     ;; setup preferred indentation style.
+     (setq fill-column 79)
+     (setq indent-tabs-mode nil) ; no tabs, never, I will not repeat
+    ;  (font-lock-add-keywords 'python-mode
+    ;                         '(("^\\s *\t" . 'my-invalid-face) ; Leading tabs
+    ;                            ("[ \t]+$" . 'my-invalid-face)  ; Trailing spaces
+    ;                            ("^[ \t]+$" . 'my-invalid-face)); Spaces only
+    ;                          )
+     )
+
+    (add-hook 'python-mode-hook 'my-python-mode-hook)
+
+The lines beginning with ';' are comments. They can be activated
+if one want to have a strong notice of some tab/space usage
+violations.
+
+Portability Tips
+================
+
+The ``bzrlib.osutils`` module has many useful helper functions, including
+some more portable variants of functions in the standard library.
+
+In particular, don't use ``shutil.rmtree`` unless it's acceptable for it
+to fail on Windows if some files are readonly or still open elsewhere.
+Use ``bzrlib.osutils.rmtree`` instead.
+
+Using the ``open(..).read(..)`` or ``open(..).write(..)`` style chaining
+of methods for reading or writing file content relies on garbage collection
+to close the file which may keep the file open for an undefined period of
+time. This may break some follow up operations like rename on Windows.
+Use ``try/finally`` to explictly close the file. E.g.::
+
+    f = open('foo.txt', 'w')
+    try:
+        f.write(s)
+    finally:
+        f.close()
+
+
+Terminology
+===========
+
+Bazaar is a GNU project and uses standard GNU terminology, especially:
+
+ * Use the word "Linux" to refer to the Linux kernel, not as a synechoche
+   for the entire operating system.  (See `bug 528253
+   <https://bugs.launchpad.net/bzr/+bug/528253>`_).
+
+ * Don't say "open source" when you mean "free software".
+
+..
+   vim: ft=rst tw=74 ai

=== modified file 'doc/developers/conf.py'
--- doc/developers/conf.py	2009-09-09 15:51:18 +0000
+++ doc/developers/conf.py	2010-07-07 15:03:14 +0000
@@ -13,7 +13,7 @@
 sys.path = [os.path.abspath('../..')] + sys.path
 
 # Most of the configuration for Bazaar docs is defined here ...
-from bzrlib.doc_generate.sphinx_conf import *
+from bzrlib.doc_generate.conf import *
 
 
 ## Configuration specific to this site ##

=== modified file 'doc/developers/content-filtering.txt'
--- doc/developers/content-filtering.txt	2009-12-02 20:34:07 +0000
+++ doc/developers/content-filtering.txt	2010-08-13 19:08:57 +0000
@@ -136,9 +136,9 @@
 See also
 ********
 
-* http://bazaar-vcs.org/LineEndings
+* http://wiki.bazaar.canonical.com/LineEndings
 
-* http://bazaar-vcs.org/LineEndings/Roadmap
+* http://wiki.bazaar.canonical.com/LineEndings/Roadmap
 
 * `Developer Documentation <index.html>`_
 

=== modified file 'doc/developers/contribution-quickstart.txt'
--- doc/developers/contribution-quickstart.txt	2010-02-23 02:32:52 +0000
+++ doc/developers/contribution-quickstart.txt	2010-10-01 03:19:52 +0000
@@ -65,6 +65,16 @@
 with your local and Launchpad usernames inserted.
 
 
+
+Publishing your changes
+-----------------------
+
+After you've locally committed your changes, the configuration above
+should be enough that you can push them to Launchpad with a simple ::
+
+    $ bzr push
+
+
 Writing tests
 -------------
 
@@ -103,7 +113,7 @@
 
 
 Then propose a merge into bzr; for bzr 2.2 and later you can use the ``bzr
-propose-merge`` command.  In the comment for your merge proposal please
+lp-propose-merge`` command.  In the comment for your merge proposal please
 explain what you're trying to do and why.  For `example
 <https://code.launchpad.net/~ian-clatworthy/bzr/whats-new-in-2.1/+merge/19677>`_:
 

=== modified file 'doc/developers/development-repo.txt'
--- doc/developers/development-repo.txt	2009-12-02 20:34:07 +0000
+++ doc/developers/development-repo.txt	2010-08-13 19:08:57 +0000
@@ -171,7 +171,7 @@
 
 If you need any help or encounter any problems, please contact the developers
 via the usual ways, i.e. chat to us on IRC or send a message to our mailing
-list. See http://bazaar-vcs.org/BzrSupport for contact details.
+list. See http://wiki.bazaar.canonical.com/BzrSupport for contact details.
 
 
 Technical notes

=== modified file 'doc/developers/index-plain.txt'
--- doc/developers/index-plain.txt	2010-02-23 04:12:26 +0000
+++ doc/developers/index-plain.txt	2010-08-17 06:45:33 +0000
@@ -15,7 +15,7 @@
   (external link)
   |--| automatically generated API reference information
 
-* `Integrating with Bazaar <http://bazaar-vcs.org/Integrating_with_Bazaar>`_
+* `Integrating with Bazaar <http://wiki.bazaar.canonical.com/Integrating_with_Bazaar>`_
   (wiki) |--| a guide for writing Python programs that work with Bazaar.
 
 * `Revision Properties <revision-properties.html>`_ |--| An application
@@ -24,6 +24,10 @@
 
 * `Testing <testing.html>`_ |--| Guide to writing tests for Bazaar.
 
+* `Code Review <code-review.html>`_.
+
+* `Bazaar Code Style Guide <code-style.html>`_.
+
 * `Writing plugins <http://doc.bazaar.canonical.com/plugins/en/plugin-development.html>`_
   |--| specific advice on writing Bazaar plugins. (web link)
 
@@ -38,7 +42,7 @@
 
 * `Managing the Bazaar PPA <ppa.html>`_ |--| Packaging Bazaar for Ubuntu.
 
-* `Giving back <http://bazaar-vcs.org/BzrGivingBack>`_ (wiki) |--| How to get
+* `Giving back <http://wiki.bazaar.canonical.com/BzrGivingBack>`_ (wiki) |--| How to get
   your changes to Bazaar integrated into a release.
 
 * `Profiling notes <profiling.html>`_ |--| Instructions on how to profile
@@ -50,6 +54,12 @@
 * `Tracking Bugs in Bazaar <bug-handling.html>`_ |--| How we use the bug
   tracker.
 
+Architecture overviews
+======================
+
+* `Transports <transports.html>`_ |--| Transport virtual filesystem
+  abstraction.
+
 Plans
 =====
 

=== modified file 'doc/developers/index.txt'
--- doc/developers/index.txt	2010-02-23 04:12:26 +0000
+++ doc/developers/index.txt	2010-09-03 09:14:12 +0000
@@ -23,10 +23,21 @@
    bug-handling
    HACKING
    testing
+   code-review
+   code-style
 
 * `Contributing to Bazaar Documentation <http://wiki.bazaar.canonical.com/ContributingToTheDocs>`_ (wiki)
 
 
+Architecture overviews
+======================
+
+.. toctree::
+   :maxdepth: 1
+
+   transports
+   ui
+
 Releasing and Packaging
 =======================
 

=== modified file 'doc/developers/integration.txt'
--- doc/developers/integration.txt	2009-09-15 06:07:11 +0000
+++ doc/developers/integration.txt	2010-05-16 09:29:44 +0000
@@ -2,8 +2,35 @@
 Integrating with Bazaar
 =======================
 
-This page should hopefully become a quick guide to integrating other
-(Python-based) software with Bazaar.
+This document provides some general observations on integrating with
+Bazaar and some recipes for typical tasks.  It is intended to be useful to
+someone developing either a plugin or some other piece of software that
+integrates with bzr.  If you want to know about a topic that's not covered
+here, just ask us.
+
+
+
+
+Starting with bzrlib
+====================
+
+Before doing anything else with bzrlib, you should run `bzrlib.initialize`
+which sets up some global state.  
+
+
+Running bzr commands
+====================
+
+To run command-line commands in-process::
+
+  from bzrlib.commands import get_command
+  
+  cmd = get_command('version')
+  cmd.run([])
+  
+This will send output through the current UIFactory; you can redirect this
+elsewhere through the parameters to `bzrlib.initialize`.
+
 
 Manipulating the Working Tree
 =============================
@@ -21,7 +48,8 @@
 to see which methods are available.
 
 Compare trees
-===============
+-------------
+
 There are two methods for comparing trees: ``changes_from`` and
 ``iter_changes``.  ``iter_changes`` is more regular and precise, but it is
 somewhat harder to work with.  See the API documentation for more details.
@@ -54,7 +82,7 @@
 
 
 Adding Files
-============
+------------
 
 If you want to add files the same way ``bzr add`` does, you can use
 MutableTree.smart_add.  By default, this is recursive. Paths can either be
@@ -70,7 +98,7 @@
 
 
 Removing Files
-==============
+--------------
 
 You can remove multiple files at once.  The file paths need to be relative
 to the workingtree::
@@ -85,7 +113,7 @@
 
 
 Renaming a File
-===============
+---------------
 
 You can rename one file to a different name using WorkingTree.rename_one.
 You just provide the old and new names, eg::
@@ -94,7 +122,7 @@
 
 
 Moving Files
-============
+------------
 
 You can move multiple files from one directory into another using
 WorkingTree.move::
@@ -107,7 +135,7 @@
 
 
 Committing Changes
-==================
+------------------
 
 To commit _all_ the changes to our working tree we can just call the
 WorkingTree's commit method, giving it a commit message, eg::
@@ -187,7 +215,7 @@
 
 
 Branching from an existing branch
-=================================
+---------------------------------
 
 To branch you create a branch object representing the branch you are
 branching from, and supply a path/url to the new branch location.
@@ -205,7 +233,7 @@
 
 
 Pushing and pulling branches
-============================
+----------------------------
 
 To push a branch you need to open the source and destination branches, then
 just call push with the other branch as a parameter::

=== modified file 'doc/developers/overview.txt'
--- doc/developers/overview.txt	2009-12-02 20:34:07 +0000
+++ doc/developers/overview.txt	2010-08-24 14:45:18 +0000
@@ -4,36 +4,52 @@
 
 This document describes the key classes and concepts within Bazaar.  It is
 intended to be useful to people working on the Bazaar codebase, or to
-people writing plugins.
+people writing plugins.  People writing plugins may also like to read the 
+guide to `Integrating with Bazaar <integration.html>`_ for some specific
+recipes.
 
 If you have any questions, or if something seems to be incorrect, unclear
 or missing, please talk to us in ``irc://irc.freenode.net/#bzr``, or write
-to the Bazaar mailing list.  To propose a correction or addition to this
-document, send a merge request or new text to the mailing list.
-
-The current version of this document is available in the file
-``doc/developers/overview.txt`` in the source tree, and available online
-within the developer documentation, <http://doc.bazaar-vcs.org/developers/>.
-
-
-Essential Domain Classes
-########################
-
-The core domain objects within the bazaar model are:
-
-* Transport
-
-* Branch
-
-* Repository
-
-* WorkingTree
-
-Transports are explained below. See http://bazaar-vcs.org/Classes/
-for an introduction to the other key classes.
+to the Bazaar mailing list.  
+
+
+Using bzrlib
+############
+
+Within bzr
+==========
+
+When using bzrlib within the ``bzr`` program (for instance as a bzr
+plugin), bzrlib's global state is already available for use.
+
+From outside bzr
+================
+
+To use bzrlib outside of ``bzr`` some global state needs to be setup.
+bzrlib needs ways to handle user input, passwords, a place to emit
+progress bars, logging setup appropriately for your program. The easiest
+way to set all this up in the same fashion ``bzr`` does is to call
+``bzrlib.initialize``. This returns a context manager within which bzrlib
+functions will work correctly. See the pydoc for ``bzrlib.initialize`` for
+more information. In Python 2.4 the ``with`` keyword is not supported and
+so you need to use the context manager manually::
+
+  # This sets up your ~/.bzr.log, ui factory and so on and so forth. It is
+  # not safe to use as a doctest.
+  library_state = bzrlib.initialize()
+  library_state.__enter__()
+  try:
+      pass
+      # do stuff here
+  finally:
+      library_state.__exit__(None, None, None)
+
+
+Core classes
+############
 
 Transport
-#########
+=========
 
 The ``Transport`` layer handles access to local or remote directories.
 Each Transport object acts as a logical connection to a particular
@@ -46,7 +62,7 @@
 Python file I/O mechanisms.
 
 Filenames vs URLs
-=================
+-----------------
 
 Transports work in terms of URLs.  Take note that URLs are by definition
 only ASCII - the decision of how to encode a Unicode string into a URL
@@ -83,14 +99,67 @@
 is also in the form of URL components.
 
 
+WorkingTree
+===========
+
+A workingtree is a special type of Tree that's associated with a working
+directory on disk, where the user can directly modify the files. 
+
+Responsibilities:
+
+ * Maintaining a WorkingTree on disk at a file path.
+ * Maintaining the basis inventory (the inventory of the last commit done)
+ * Maintaining the working inventory.
+ * Maintaining the pending merges list.
+ * Maintaining the stat cache.
+ * Maintaining the last revision the working tree was updated to.
+ * Knows where its Branch is located.
+
+Dependencies:
+
+ * a Branch
+ * an MutableInventory
+ * local access to the working tree
+
+Branch
+======
+
+A Branch is a key user concept - its a single line of history that one or
+more people have been committing to. 
+
+A Branch is responsible for:
+
+ * Holding user preferences that are set in a Branch.
+ * Holding the 'tip': the last revision to be committed to this Branch. (And the revno of that revision.)
+ * Knowing how to open the Repository that holds its history.
+ * Allowing write locks to be taken out to prevent concurrent alterations to the branch.
+
+Depends on:
+ * URL access to its base directory.
+ * A Transport to access its files.
+ * A Repository to hold its history.
+
 Repository
-##########
+==========
 
 Repositories store committed history: file texts, revisions, inventories,
-and graph relationships between them.
+and graph relationships between them.  A repository holds a bag of
+revision data that can be pointed to by various branches:
+
+ * Maintains storage of various history data at a URL:
+
+   * Revisions (Must have a matching inventory)
+   * Digital Signatures
+   * Inventories for each Revision. (Must have all the file texts available).
+   * File texts
+
+ * Synchronizes concurrent access to the repository by different
+   processes.  (Most repository implementations use a physical 
+   mutex only for a short period, and effectively support multiple readers
+   and writers.)
 
 Stacked Repositories
-====================
+--------------------
 
 A repository can be configured to refer to a list of "fallback"
 repositories.  If a particular revision is not present in the original

=== modified file 'doc/developers/packrepo.txt'
--- doc/developers/packrepo.txt	2007-12-05 19:55:07 +0000
+++ doc/developers/packrepo.txt	2010-08-13 19:08:57 +0000
@@ -138,7 +138,7 @@
 
 If you need any help or encounter any problems, please contact the developers
 via the usual ways, i.e. chat to us on IRC or send a message to our mailing
-list. See http://bazaar-vcs.org/BzrSupport for contact details.
+list. See http://wiki.bazaar.canonical.com/BzrSupport for contact details.
 
 
 Technical notes

=== modified file 'doc/developers/planned-change-integration.txt'
--- doc/developers/planned-change-integration.txt	2009-12-02 20:34:07 +0000
+++ doc/developers/planned-change-integration.txt	2010-06-02 05:03:31 +0000
@@ -73,7 +73,7 @@
  * Working tree disk ordering: Knowing the expected order for disk operations
    may influence the needed use case specific APIs, so having a solid
    understanding of what is optimal - and why - and whether it is pessimal on
-   non linux platforms is rather important.
+   non-Linux-kernel platforms is rather important.
 
  * Be able to version files greater than memory in size: This cannot be
    achieved until all parts of the library which deal with user files are able

=== modified file 'doc/developers/planned-performance-changes.txt'
--- doc/developers/planned-performance-changes.txt	2009-12-02 20:34:07 +0000
+++ doc/developers/planned-performance-changes.txt	2010-08-13 19:08:57 +0000
@@ -120,13 +120,6 @@
    between two historical trees much more efficiently than the current
    repositories.
 
- * Delta storage optimisation. We plan to change the delta storage logic to use
-   a binary delta like xdelta rather than using line based deltas from python.
-   These binary deltas could be done along ancestry ordering, or other
-   arbitrary patterns chosen for their intended use. Line based deltas will
-   still be created for cached annotations. This is still under some discussion.
-   http://bazaar-vcs.org/PerformanceRoadmap/Xdelta
-
  * Greatest distance from origin cache. This is a possible change to introduce,
    but it may be unnecessary - listed here for completeness till it has been
    established as [un]needed.

=== modified file 'doc/developers/ppa.txt'
--- doc/developers/ppa.txt	2009-12-02 20:34:07 +0000
+++ doc/developers/ppa.txt	2010-09-20 12:05:38 +0000
@@ -7,43 +7,68 @@
 Background
 ----------
 
-We build Ubuntu ``.deb`` packages for Bazaar as an important part of the release
-process.  These packages are hosted in a few `Personal Package Archives (PPA)`__ on
-Launchpad.
+We build Ubuntu ``.deb`` packages for Bazaar as an important part of the
+release process.  These packages are hosted in a few `Personal Package
+Archives (PPA)`__ on Launchpad.
 
   __ https://help.launchpad.net/PPAQuickStart
 
-As of June 2008, there are three PPAs:
-
-<https://launchpad.net/~bzr/+archive>
-    Final released versions.
+As of September 2010, there are four PPAs:
+
+<https://launchpad.net/~bzr/+archive/ppa>
+    Final released versions and updates.
+    Most users who want updates to bzr should add this.
+
+<https://launchpad.net/~bzr/+archive/proposed>
+    Proposed uploads to move into ~bzr, awaiting testing.
 
 <https://launchpad.net/~bzr-beta-ppa/+archive>
-    Releases and release candidates.
+    Beta releases.
 
 <https://launchpad.net/~bzr-nightly-ppa/+archive>
     Automatic nightly builds from trunk.
 
-We build packages for every supported Ubuntu release
-<https://wiki.ubuntu.com/Releases>.  Packages need no longer be updated
-when the release passes end-of-life because all users should
-have upgraded by then.  (As of May 2008, Edgy Eft is no longer supported.)
-
-We build a distinct package for each distrorelease.  As of bzr 1.5, Dapper
-uses ``python-support`` and later distributions use ``python-central``.
+We build a distinct package for each distrorelease.  
 If you upload a release-specific version, you should add a suffix to the
 package version, e.g. ``bzr.1.3-1~bazaar1~dapper1``.
 
+Dapper uses the ``python-support`` framework and later distributions use
+``python-central``.  This has little effect on everyday packaging but does
+mean that some of the control files are quite different.
+
 Every package is first uploaded into the beta ppa.  For final release
 versions it is also copied to the main PPA.
 
 The packaging information is kept in branches of bzr on Launchpad, named
 like
-<https://code.launchpad.net/~bzr/bzr/packaging-hardy>.
+<https://code.launchpad.net/~bzr/ubuntu/hardy/bzr/bzr-ppa>.
 or
-<lp:~bzr/bzr/packaging-hardy>.  These branches are intended to be used
+<lp:~bzr/ubuntu/hardy/bzr/bzr-ppa>.  These branches are intended to be used
 with the ``bzr-builddeb`` plugin.
 
+**You should almost always upload to the beta ppa first** and then either 
+upload again or copy the packages into the release ppa.  That reduces the 
+risk of breaking the main archive from which people get bzr updates.
+
+
+Supported releases
+------------------
+
+We build packages for every supported Ubuntu release
+<https://wiki.ubuntu.com/Releases>.  Packages need no longer be updated
+when the release passes end-of-life because all users should
+have upgraded by then.  
+
+As of August 2010, the following releases are supported:
+
+* Maverick 
+* Lucid LTS
+* Karmic
+* Jaunty (support ends October 2010)
+* Hardy LTS
+* Dapper LTS (supported but no longer updated for new releases)
+
+
 Preconditions
 -------------
 
@@ -52,26 +77,15 @@
 
 * You must have a GPG key registered to your Launchpad account.
 
-* Configure ``dput`` to upload to our PPA with this section in your
-  ``~/.dput.cf``::
-
-    [bzr-beta-ppa]
-    fqdn = ppa.launchpad.net
-    method = ftp
-    incoming = ~bzr-beta-ppa/ubuntu
-    login = anonymous
-    allow_unsigned_uploads = 0
-
-    [bzr-ppa]
-    fqdn = ppa.launchpad.net
-    method = ftp
-    incoming = ~bzr/ubuntu
-    login = anonymous
-    allow_unsigned_uploads = 0
-
-  You may also want to add these lines to prevent inadvertently attempting
-  to upload into Ubuntu or Debian, which will give a somewhat unclear
-  error::
+On reasonably recent versions of Ubuntu you no longer need special dput
+configuration, because you can just say ::
+
+  dput ppa:bzr/proposed <source.changes
+  
+
+However, you may still want to add these lines to ``~/.dput.cf`` prevent 
+inadvertently attempting to upload into Ubuntu or Debian, which will
+give a somewhat unclear error::
 
     [DEFAULT]
     default_host_main = notspecified
@@ -100,6 +114,39 @@
 Packaging Bazaar
 ----------------
 
+Overview of packaging with builddeb
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+* First update the oldest supported branch, using ``bzr merge-upstream``.
+
+* Run ``bzr builddeb -S -- -sa`` to build a source package, then put 
+  that into the ppa.  
+  
+  (``-S`` says to make a source-only upload, which is
+  required for Launchpad's builders.  ``--sa`` says to include the
+  ``.orig.tgz`` even if this doesn't seem to be the first upload for an
+  upstream release: this is often needed when rebuilding something that's
+  previously been uploaded to Debian or Ubuntu or into a different PPA.)
+
+* Now merge across that change into each supported branch with a 
+  simple ``bzr merge``.
+  
+Locally testing using pbuilder
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+It may be useful to locally test builds inside pbuilder.  You may want to 
+use the script from <http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=255165> 
+to wrap it, and to give it sensible defaults for your local machine.
+
+Update all packages in proposed before copping the main ppa
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+If one updates bzr, and there are plugins that are not compatible with the
+new version of bzr, this can cause pain for users using the ppa. In order to
+avoid this, we first get all packages up to date in the proposed ppa, and then
+copy them to the main ppa.
+
+
 Short form
 ~~~~~~~~~~
 
@@ -111,10 +158,11 @@
   export PACKAGE="bzr"
   export UBUNTU_RELEASES="dapper hardy intrepid jaunty karmic"
   ~/dev/bzr/bzr.dev/tools/packaging/update-packaging-branches.sh
+  * Optionaly merge debian unstable. 
   ~/dev/bzr/bzr.dev/tools/packaging/update-changelogs.sh
   ~/dev/bzr/bzr.dev/tools/packaging/update-control.sh 1.16 1.17 1.18
   ~/dev/bzr/bzr.dev/tools/packaging/build-packages.sh
-  dput bzr-beta-ppa ${PACKAGE}_$VERSION*.changes
+  dput ppa:bzr/proposed ${PACKAGE}_$VERSION*.changes
 
 Rinse and repeat for all the plugins by changing VERSION and PACKAGE.
 
@@ -156,12 +204,16 @@
 
 #. Checkout (or update) the packaging branch for each supported release::
 
-      bzr co lp:~bzr/bzr/packaging-hardy
+      bzr co lp:~bzr/ubuntu/hardy/bzr/bzr-ppa
 
    There is a script available to help::
 
       tools/packaging/update-packaging-branches.sh
 
+#. Optionaly, merge the Debian unstable branch into each of the packaging
+   branches. You can find the Debian unstable branch here:
+   http://bzr.debian.org/pkg-bazaar/
+
 #. The ``bzr-builddeb`` step will download the original tarball if you do
    not already have it, putting it into a ``tarballs`` directory.
 
@@ -217,7 +269,7 @@
 
 #. Build the source packages::
 
-      cd packaging-$DISTRO; bzr builddeb -S
+      cd bzr-$DISTRO; bzr builddeb -S
 
    This will create a ``.changes`` file.  If you didn't configure builddeb
    to automatically sign them, you can use ::
@@ -231,16 +283,9 @@
 
       tools/packaging/build-packages.sh
 
-#. Upload into the beta PPA for each release::
-
-     dput bzr-beta-ppa bzr*1.17-1*.changes
-
-#. For final release versions, also copy it into the ``~bzr`` PPA::
-
-     dput bzr-ppa ../bzr_1.17-1\~bazaar1\~hardy1\_source.changes
-
-   Alternatively, you can use Launchpad's "copy" feature to copy the
-   packages between repositories.
+#. Upload into the PPA for each release::
+
+     dput dput ppa:bzr/proposed bzr*1.17-1*.changes
 
 #. You should soon get an "upload accepted" mail from Launchpad, which
    means that your package is waiting to be built.  You can then track its
@@ -282,6 +327,29 @@
 summary of current package versions with::
 
   apt-cache madison bzr
+  
+  
+Testing the contents of the PPA
+-------------------------------
+
+A somewhat crude but useful way to test the contents of the PPA is to
+install the relevant packages into an schroot::
+
+ schroot -c hardy-test -u root -- \
+   apt-get install -o 'APT::Install-Suggests="true"' \
+   -o 'APT::Install-Recommends="true"' \
+   bzr
+
+This should make sure everything can be installed; it won't guarantee that 
+  
+
+Packaging dependencies
+----------------------
+
+Some of our updates to bzr in previous releases require backports of our
+dependencies.  Specific branches holding these backports:
+
+ * ``lp:~bzr/ubuntu/dapper/configobj/dapper-backport``
 
 
 ..

=== modified file 'doc/developers/releasing.txt'
--- doc/developers/releasing.txt	2010-03-17 02:58:08 +0000
+++ doc/developers/releasing.txt	2010-09-24 09:56:50 +0000
@@ -1,12 +1,11 @@
 Releasing Bazaar
-================
+################
 
 This document describes the processes for making and announcing a Bazaar
 release, and managing the release process.  This is just one phase of the
-`overall development cycle <http://doc.bazaar-vcs.org/developers/cycle.html>`_,
-but it's the most complex part.
-This document gives a checklist you can follow from start to end in one
-go.
+`overall development cycle <http://doc.bazaar.canonical.com/developers/cycle.html>`_,
+but it's the most complex part.  This document gives a checklist you can
+follow from start to end in one go.
 
 If you're helping the Release Manager (RM) for one reason or another, you
 may notice that he didn't follow that document scrupulously. He may have
@@ -19,46 +18,37 @@
 
 
 Preconditions
--------------
+=============
 
 #. Download the pqm plugin and install it into your ``~/.bazaar/plugins``::
 
      bzr branch lp:bzr-pqm ~/.bazaar/plugins/pqm
 
 
-Starting a cycle
-----------------
+At the start of a release cycle
+===============================
 
 To start a new release cycle:
 
-#. Create a new series at <https://launchpad.net/bzr/+addseries>. There is one
-   series for every *x.y* release.
-
-#. Go to the series web page at <https://launchpad.net/bzr/x.y>
-
-#. Create a new release at
-   <https://launchpad.net/bzr/x.y/+addrelease> and add
-   information about this release. We will not use it yet, but it
+#. If this is the first release for a given *x.y* then create a new
+   series at <https://launchpad.net/bzr/+addseries>. There is one series
+   for every *x.y* release.
+
+#. If you made a new series, create a new pqm-controlled branch for this
+   release series, by asking a Canonical sysadmin.  This branch means that
+   from the first release beta or candidate onwards, general development
+   continues on the trunk, and only specifically-targeted fixes go into
+   the release branch.
+
+#. If you made a new series, add milestones at
+   <https://edge.launchpad.net/bzr/x.y/+addmilestone> to that series for
+   the beta release, release candidate and the final release, and their
+   expected dates.
+
+#. Create a new milestone <https://edge.launchpad.net/bzr/x.y/+addmilestone>
+   and add information about this release.  We will not use it yet, but it
    will be available for targeting or nominating bugs.
 
-#. We create a new pqm-controlled branch for this release series, by
-   asking a Canonical sysadmin.
-   This branch means that from the first release beta or candidate onwards,
-   general development continues on the trunk, and only
-   specifically-targeted fixes go into the release branch.
-
-#. Add milestones at <https://edge.launchpad.net/bzr/x.y/+addmilestone> to
-   that series for the beta release, release candidate and the final release,
-   and their expected dates.
-
-#. Update the version number in the ``bzr`` script, and the
-   ``bzrlib/__init__.py`` file. Make sure there is always a corresponding
-   milestone when you change that version number.
-
-#. Add a new section at the top of ``NEWS`` about the new release,
-   including its version number and the headings from
-   ``NEWS-template.txt``.
-
 #. Send mail to the list with the key dates, who will be the release
    manager, and the main themes or targeted bugs.  Ask people to nominate
    objectives, or point out any high-risk things that are best done early,
@@ -71,7 +61,7 @@
      bzr branch trunk prepare-1.14
 
 #. Configure pqm-submit for this branch, with a section like this (where
-   x.y is the version to release).
+   x.y is the version to release). **Or use hydrazine for easy use**
    ``~/.bazaar/locations.conf``::
 
 	[/home/mbp/bzr/prepare-x.y]
@@ -82,9 +72,34 @@
 	submit_to = bazaar@lists.canonical.com
 	smtp_server = mail.example.com:25
 
-    Please see <http://doc.bazaar-vcs.org/developers/HACKING.html#an-overview-of-pqm>
+    Please see <http://doc.bazaar.canonical.com/developers/HACKING.html#an-overview-of-pqm>
     for more details on PQM
 
+#. Update the version number in the ``bzr`` script, and the
+   ``bzrlib/__init__.py`` file::
+   
+       version_info = (x, y, z, 'dev', 0)
+   
+#. Add a new section at the top of ``NEWS`` about the new release,
+   including its version number and the headings from
+   ``NEWS-template.txt``.
+
+#. Update the "What's New" documents in ``doc/en/whats-new``.
+
+#. Commit this and send it to PQM.
+
+
+Doing a particular release
+==========================
+
+Update the source code
+----------------------
+
+#. Check that there is a milestone for the release you're doing. If there
+   is no milestone it indicates a process problem - make the milestone but
+   also mail the list to raise this issue in our process. Milestones are
+   found at <https://launchpad.net/bzr/+milestone/x.y.z>.
+
 #. In the release branch, update  ``version_info`` in ``./bzrlib/__init__.py``.
    Make sure the corresponding milestone exists.
    Double check that ./bzr ``_script_version`` matches ``version_info``. Check
@@ -92,40 +107,39 @@
 
    For beta releases use::
 
+       version_info = (2, 1, 0, 'beta', SERIAL)
+
+   For instance 2.1b1::
+
        version_info = (2, 1, 0, 'beta', 1)
 
-
    For release candidates use::
 
-       version_info = (2, 0, 1, 'candidate', 1)
-
-
-Starting the release phase
---------------------------
-
-#. Create a new milestone at <https://launchpad.net/bzr/x.y/+addmilestone>
-   for the beta release or release candidate if you haven't already.
-
-#. Add the date and release number to ``./NEWS``
-
-   Depending on whether you're doing a beta or a bugfix release, you'll
-   have to create a NEWS section for your release in the right
-   place. Most of the time, the new section is at the top of the file
-   (look what have been done for the various 2.0x and 2.1.0bx releases).
-   The rule is to keep the sections sorted by date. You'll need to be
-   cautious when merging back to trunk to respect that.
+       version_info = (2, 0, 1, 'candidate', SERIAL)
+
+   For stable releases use::
+
+       version_info = (2, 1, 2, 'final', 0)
+
+#. Update the ``./NEWS`` section for this release.
+
+   Fill out the date and a description of the release under the existing
+   header. If there isn't one, follow the above for using the NEWS
+   template.
+
+   See *2.1.1* or similar for an example of what this looks like.
+
+#. Add a summary of the release into the "What's New" document.
 
 #. To check that all bugs mentioned in ``./NEWS`` are actually marked as
    closed in Launchpad, you can run ``tools/check-newsbugs.py``::
 
      ./tools/check-newsbugs.py NEWS
 
-   (But note there can be some false positives, and this script may be
+   (But note there will be many false positives, and this script may be
    flaky <https://bugs.edge.launchpad.net/bzr/+bug/354985>.  Don't let
    this slow you down too much.)
 
-#. Summarize into one or two paragraphs what's new in this release.
-
 #. Commit these changes to the release branch, using a command like::
 
      bzr commit -m "Release 1.14."
@@ -170,7 +184,7 @@
 
      bzr tag bzr-1.14
 
-#. Push those changes to a bzr reposistory that is public and accessible on
+#. Push those changes to a bzr repository that is public and accessible on
    the Internet. PQM will pull from this repository when it attempts to merge
    your changes. Then submit those changes to PQM for merge into the
    appropriate release branch::
@@ -178,6 +192,11 @@
      bzr push
      bzr pqm-submit -m "(mbp) prepare 1.14"
 
+   Or with hydrazine::
+
+     bzr lp-propose -m "Release 1.14" --approve lp:bzr/1.14
+     feed-pqm bzr
+
 #. When PQM succeeds, pull down the master release branch.
 
 
@@ -199,15 +218,24 @@
    disable the faulty plugins one by one until you get no more
    failures.
 
+   Remember that PQM has just tested everything too, this step is
+   particularly testing that the pyrex extensions, which are updated
+   by your local pyrex version when you run make dist, are in good
+   shape.
+
 
 Publishing the source tarball
 -----------------------------
 
 #. Go to the relevant milestone page in Launchpad.
 
-#. Within that release, upload the source tarball and the GPG
-   signature.  Or, if you prefer, use the
-   ``tools/packaging/lp-upload-release`` script to do this.
+#. Create a release of the milestone, and upload the source tarball and
+   the GPG signature.  Or, if you prefer, use the
+   ``tools/packaging/lp-upload-release`` script to do this. Note that
+   this changes what the download widget on the Launchpad bzr home
+   page shows, so don't stop the release process yet, or platform binary
+   installers won't be made and the download list will stay very small!
+   <https://bugs.edge.launchpad.net/launchpad/+bug/586445>
 
 
 Announcing the source freeze
@@ -219,6 +247,15 @@
    release.
 
 
+Kick off the next cycle
+-----------------------
+
+#. To let developers work on the next release, do
+   `At the start of a release cycle` now.
+
+#. Pause for a few days.
+
+
 Publishing the release
 ----------------------
 
@@ -227,23 +264,14 @@
 we have a releasable product.  The next step is to make it generally
 available to the world.
 
-go to the release
-
-#. Within that release, upload the source tarball and zipfile and the GPG
-   signature.  Or, if you prefer, use the
-   ``tools/packaging/lp-upload-release`` script to do this.
-
-#. Link from http://bazaar-vcs.org/SourceDownloads to the tarball and
-   signature.
-
-#. Announce on the `Bazaar website <http://bazaar-vcs.org/>`_.
+#. Go to the release web page at <https://launchpad.net/bzr/x.y/x.y.z>
+
+#. Announce on the `Bazaar website <http://bazaar.canonical.com/>`_.
    This page is edited via the lp:bzr-website branch. (Changes
    pushed to this branch are refreshed by a cron job on escudero.)
 
-#. Announce on the `Bazaar wiki <http://bazaar-vcs.org/Welcome>`_.
-
 #. Check that the documentation for this release is available in
-   <http://doc.bazaar-vcs.org>.  It should be automatically build when the
+   <http://doc.bazaar.canonical.com>.  It should be automatically build when the
    branch is created, by a cron script ``update-bzr-docs`` on
    ``escudero``. As of today (2009-08-27) ``igc`` manually updates the
    pretty version of it.
@@ -270,7 +298,7 @@
 
    The announce mail will look something like this::
 
-      Subject: bzr x.yy released!
+      Subject: bzr x.y.z released!
 
       <<Summary paragraph from news>>
 
@@ -282,7 +310,7 @@
       feedback.
 
       Bazaar is now available for download from
-      http://bazaar-vcs.org/Download as a source tarball; packages
+      https://launchpad.net/bzr/2.x/2.x/ as a source tarball; packages
       for various systems will be available soon.
 
       <<NEWS section from this release back to the last major release>>
@@ -318,6 +346,9 @@
 Merging the released code back to trunk
 ---------------------------------------
 
+The rule is to keep ``NEWS`` sections sorted by date. You'll need to
+review the merge and make sure that that is respected.
+
 Merge the release branch back into the trunk.  Check that changes in NEWS
 were merged into the right sections.  If it's not already done, advance
 the version number in ``bzr`` and ``bzrlib/__init__.py``.  Submit this
@@ -327,7 +358,7 @@
 created the corresponding milestone to ensure the continuity in bug
 targeting or nominating. Depending on the change, you may even have to
 create a new series (if your change the major or minor release number), in
-that case go to `Starting a cycle` and follow the instructions from there.
+that case go to `At the start of a release cycle` and follow the instructions from there.
 
 You should also merge (not pull) the release branch into
 ``lp:~bzr/bzr/current``, so that branch contains the current released code
@@ -341,12 +372,61 @@
 candidate, you're not finished yet. Another beta or candidate or
 hopefully a final release is still to come.
 
-The process is the same as for the first release. Goto `Starting the
-release phase`_ and follow the instructions again. Some details change
+The process is the same as for the first release. Goto `Doing a
+particular release`_ and follow the instructions again. Some details change
 between beta, candidate and final releases, but they should be
 documented. If the instructions aren't clear enough, please fix them.
 
 
+Getting the release into Ubuntu
+-------------------------------
+
+(Feel free to propose or add new sections here about what we should do to
+get bzr into other places.)
+
+For the currently-under-development release of Ubuntu, no special action
+is needed: the release should be picked by Debian and synced from there into
+Ubuntu.
+
+Releases off stable bzr branches should go in to the ``-updates`` of the
+Ubuntu release that originally contained that branch.  (Ubuntu Lucid had
+bzr 2.2.0, so should get every 2.2.x update.)  This means going through
+the `SRU (Stable Release Updates)
+<https://wiki.ubuntu.com/StableReleaseUpdates>`__ process.   
+
+As of September 2010, bzr has applied to the technical board to be added
+to the `MicroReleaseExceptions
+<https://wiki.ubuntu.com/StableReleaseUpdates/MicroReleaseExceptions>`__
+category so that whole bugfix releases can more easily be approved.
+
+**After making a bzr stable-release release, nominate the most serious bug
+for the appropriate Ubuntu release and subscribe the `ubuntu-sru` team.**
+
+This requires a couple of tricks (please reconsider and tweak as things
+evolves from one release to the other):
+
+ * create a distro task with the ``Also affects distribution`` button and
+   select ``bzr (Ubuntu)``.
+
+ * change the *URL* to point to ``ubuntu/+source/bzr`` instead of ``bzr``
+   (this is needed if you create the distro task but not if it exists
+   already). You should now be able to click the ``Nominate for release``
+   button and select the right Ubuntu release. As of September 2010, this
+   means:
+
+  * ``maverick`` for the 2.2 series,
+  * ``lucid`` for the 2.1 series,
+  * ``karmic`` for the 2.0 series.
+
+ * Subscribe the ``~ubuntu-sru`` team to the bug.
+
+ * Add a comment targeted to ``~ubuntu-sru`` explaining the expectations
+   (we are targeting running the test suite during the build which, as of
+   September 2010, fails for known reasons that are currently addressed).
+   Search for bugs tagged with ``sru`` for examples and don't forget to tag
+   the bug you selected.
+
+
 See also
 --------
 

=== modified file 'doc/developers/testing.txt'
--- doc/developers/testing.txt	2010-04-16 07:56:51 +0000
+++ doc/developers/testing.txt	2010-09-23 09:54:51 +0000
@@ -6,7 +6,7 @@
 The Importance of Testing
 =========================
 
-Reliability is a critical success factor for any Version Control System.
+Reliability is a critical success factor for any version control system.
 We want Bazaar to be highly reliable across multiple platforms while
 evolving over time to meet the needs of its community.
 
@@ -183,6 +183,51 @@
 
 .. _testrepository: https://launchpad.net/testrepository
 
+
+Babune continuous integration
+-----------------------------
+
+We have a Hudson continuous-integration system that automatically runs 
+tests across various platforms.  In the future we plan to add more 
+combinations including testing plugins.  See 
+<http://babune.ladeuil.net:24842/>.  (Babune = Bazaar Buildbot Network.)
+
+
+Running tests in parallel
+-------------------------
+
+Bazaar can use subunit to spawn multiple test processes.  There is
+slightly more chance you will hit ordering or timing-dependent bugs but
+it's much faster::
+
+  $ ./bzr selftest --parallel=fork
+
+Note that you will need the Subunit library
+<https://launchpad.net/subunit/> to use this, which is in
+``python-subunit`` on Ubuntu.
+
+
+Running tests from a ramdisk
+----------------------------
+
+The tests create and delete a lot of temporary files.  In some cases you
+can make the test suite run much faster by running it on a ramdisk.  For
+example::
+
+  $ sudo mkdir /ram
+  $ sudo mount -t tmpfs none /ram
+  $ TMPDIR=/ram ./bzr selftest ...
+
+You could also change ``/tmp`` in ``/etc/fstab`` to have type ``tmpfs``,
+if you don't mind possibly losing other files in there when the machine
+restarts.  Add this line (if there is none for ``/tmp`` already)::
+
+  none           /tmp            tmpfs  defaults        0       0
+
+With a 6-core machine and ``--parallel=fork`` using a tmpfs doubles the
+test execution speed.
+
+
 Writing Tests
 =============
 
@@ -385,21 +430,34 @@
 The actual use of ScriptRunner within a TestCase looks something like
 this::
 
-        def test_unshelve_keep(self):
-                # some setup here
-                sr = ScriptRunner()
-                sr.run_script(self, '''
-        $ bzr add file
-        $ bzr shelve --all -m Foo
-        $ bzr shelve --list
-        1: Foo
-        $ bzr unshelve --keep
-        $ bzr shelve --list
-        1: Foo
-        $ cat file
-        contents of file
-        ''')
-
+    from bzrlib.tests import script
+
+    def test_unshelve_keep(self):
+        # some setup here
+        script.run_script(self, '''
+            $ bzr add file
+            $ bzr shelve --all -m Foo
+            $ bzr shelve --list
+            1: Foo
+            $ bzr unshelve --keep
+            $ bzr shelve --list
+            1: Foo
+            $ cat file
+            contents of file
+            ''')
+
+You can also test commands that read user interaction::
+
+    def test_confirm_action(self):
+        """You can write tests that demonstrate user confirmation"""
+        commands.builtin_command_registry.register(cmd_test_confirm)
+        self.addCleanup(commands.builtin_command_registry.remove, 'test-confirm')
+        self.run_script("""
+            $ bzr test-confirm
+            2>Really do it? [y/n]: 
+            <yes
+            yes
+            """)
 
 Import tariff tests
 -------------------
@@ -420,7 +478,7 @@
   dependencies and only needed for particular cases.  If they're loaded in
   other cases then bzr may break for people who don't have those modules.
   
-`test_import_tarrif` allows us to check that removal of imports doesn't
+`test_import_tariff` allows us to check that removal of imports doesn't
 regress.
 
 This is done by running the command in a subprocess with
@@ -653,6 +711,22 @@
         _test_needs_features = [features.apport]
 
 
+Testing deprecated code
+-----------------------
+
+When code is deprecated, it is still supported for some length of time,
+usually until the next major version. The ``applyDeprecated`` helper
+wraps calls to deprecated code to verify that it is correctly issuing the
+deprecation warning, and also prevents the warnings from being printed
+during test runs.
+
+Typically patches that apply the ``@deprecated_function`` decorator should
+update the accompanying tests to use the ``applyDeprecated`` wrapper.
+
+``applyDeprecated`` is defined in ``bzrlib.tests.TestCase``. See the API
+docs for more details.
+
+
 Testing exceptions and errors
 -----------------------------
 
@@ -772,12 +846,15 @@
 
 TestCase
     A base TestCase that extends the Python standard library's
-    TestCase in several ways.  It adds more assertion methods (e.g.
-    ``assertContainsRe``), ``addCleanup``, and other features (see its API
-    docs for details).  It also has a ``setUp`` that makes sure that
-    global state like registered hooks and loggers won't interfere with
-    your test.  All tests should use this base class (whether directly or
-    via a subclass).
+    TestCase in several ways.  TestCase is build on
+    ``testtools.TestCase``, which gives it support for more assertion
+    methods (e.g.  ``assertContainsRe``), ``addCleanup``, and other
+    features (see its API docs for details).  It also has a ``setUp`` that
+    makes sure that global state like registered hooks and loggers won't
+    interfere with your test.  All tests should use this base class
+    (whether directly or via a subclass).  Note that we are trying not to
+    add more assertions at this point, and instead to build up a library
+    of ``bzrlib.tests.matchers``.
 
 TestCaseWithMemoryTransport
     Extends TestCase and adds methods like ``get_transport``,
@@ -856,7 +933,64 @@
 Please see bzrlib.treebuilder for more details.
 
 
+Temporarily changing state
+~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+If your test needs to temporarily mutate some global state, and you need
+it restored at the end, you can say for example::
+
+    self.overrideAttr(osutils, '_cached_user_encoding', 'latin-1')
+
+Cleaning up
+~~~~~~~~~~~
+
+Our base ``TestCase`` class provides an ``addCleanup`` method, which
+should be used instead of ``tearDown``.  All the cleanups are run when the
+test finishes, regardless of whether it passes or fails.  If one cleanup
+fails, later cleanups are still run.
+
+(The same facility is available outside of tests through
+``bzrlib.cleanup``.)
+
+
+Manual testing
+==============
+
+Generally we prefer automated testing but sometimes a manual test is the
+right thing, especially for performance tests that want to measure elapsed
+time rather than effort.
+
+Simulating slow networks
+------------------------
+
+To get realistically slow network performance for manually measuring
+performance, we can simulate 500ms latency (thus 1000ms round trips)::
+
+  $ sudo tc qdisc add dev lo root netem delay 500ms
+
+Normal system behaviour is restored with ::
+
+  $ sudo tc qdisc del dev lo root
+
+A more precise version that only filters traffic to port 4155 is::
+
+    tc qdisc add dev lo root handle 1: prio
+    tc qdisc add dev lo parent 1:3 handle 30: netem delay 500ms 
+    tc qdisc add dev lo parent 30:1 handle 40: prio
+    tc filter add dev lo protocol ip parent 1:0 prio 3 u32 match ip dport 4155 0xffff flowid 1:3 handle 800::800
+    tc filter add dev lo protocol ip parent 1:0 prio 3 u32 match ip sport 4155 0xffff flowid 1:3 handle 800::801
+
+and to remove this::
+
+    tc filter del dev lo protocol ip parent 1: pref 3 u32
+    tc qdisc del dev lo root handle 1:
+
+You can use similar code to add additional delay to a real network
+interface, perhaps only when talking to a particular server or pointing at
+a VM.  For more information see <http://lartc.org/>.
+
+
 .. |--| unicode:: U+2014
 
 ..
-   vim: ft=rst tw=74 ai
+   vim: ft=rst tw=74 ai et sw=4

=== added file 'doc/developers/transports.txt'
--- doc/developers/transports.txt	1970-01-01 00:00:00 +0000
+++ doc/developers/transports.txt	2010-07-16 12:40:07 +0000
@@ -0,0 +1,108 @@
+####################################
+Developer guide to bzrlib transports
+####################################
+
+This guide describes the `Transport` classes that Bazaar uses for most
+local and remote file access.  (Working tree files are the major
+exception (`bug 606249 <https://bugs.launchpad.net/bzr/+bug/606249>`).
+
+
+Handling symlinks
+#################
+
+A symlink creates an alias where files or directories can be accessed by a
+different name.  Symlinks are useful but raise a few annoying cases for
+bzr.
+
+It's important to have tests for symlinks but these tests can't run on
+Windows, so you need eg ::
+
+    _test_needs_features = [tests.SymlinkFeature]
+
+or ::
+
+    self.requireFeature(tests.SymlinkFeature)
+
+Bazaar versions symlinks as objects in their own right, whose content is
+the path they point to.  bzr doesn't care whether a versioned
+symlink is absolute or relative; or whether it points inside or outside
+the working tree; or whether its referent exists or not.  In Unix the
+target of a symlink is a byte string; bzr treats this as a Unicode string
+in the filesystem encoding (`osutils._fs_enc`).
+
+So when we say ``bzr add symlink``, this should always add the symlink to
+its containing working tree, and never dereference the symlink.
+
+However, ``bzr add symlink/file`` shouldn't add ``file`` as a child of
+``symlink``.  (Symlinks don't have files underneath them: they may point to
+a directory which contains children, but if the symlink was pointed
+somewhere else those children would be unaffected.)  This could either add
+the file in its containing working tree, or fail outright.
+
+One interesting case for this is ::
+
+    bzr add ~/dev/bug123/a.c
+
+where ``~/dev`` is actually a symlink to ``/srv/dev/joe/``.  In this case
+clearly the user does want us to follow the symlink to open the tree.
+
+As of bzr2.2, when we open a `WorkingTree`, we typically immediately
+compute its real path and store that as ``.basedir``, but `BzrDir` stores
+its apparent path.  (This may not be the best thing.)
+
+
+Useful functions
+----------------
+
+`bzrlib.osutils.dereference_path` does the commonly useful operation of
+resolving the directory part of a path, but leaving the filename
+untouched.  In other words ::
+
+  ln -s x a
+  ln -s y x/b
+  dereference_path('a/b') => 'x/b'
+
+
+Relative paths beyond symlinks
+------------------------------
+
+Another interesting case is when a control directory contains a relative
+path, perhaps from a branch to its master or from a working tree to its
+branch.  If it contains ``../`` parts as it typically will, these may have
+different effects depending on whether they're looked up relative to the
+real path or the apparent path given by the user.  It may be that some
+users expect different behaviours at different times.  
+
+Resolving the path relative to the real directory makes it somewhat more
+consistent with what you would see by in a shell entering that directory
+and then opening the given name.  It may also make things more consistent
+when there are multiple links to the same bzrdir.  However it may cause
+problems when using a transport that hides symlinks.
+
+We could possibly handle this by doing less path arithmetic and asking the
+OS or server to open the path including ``..`` and other relative
+elements, but that might cause other problems.  HTTP servers may do their
+own path arithmetic before passing it to the OS.
+
+
+Transports that hide symlinks
+-----------------------------
+
+On local, sftp and bzr+ssh transports, we can directly see symlinks as
+symlinks.  Over http (and ftp?) they're expanded by the server and we
+cannot detect them.  This can cause problems when bzr follows relative
+paths because typically we will join the paths, and we may do this
+inconsistently with how the server, which can see the symlinks, would do.
+
+
+Symlinks and ChrootTransports
+-----------------------------
+
+bzr has an internal concept of a `ChrootTransport` that locks access into
+a particular directory.  Symlinks should not break out of a chroot jail
+which implies they should be expanded and checked within bzrlib.
+(At least as long as the transport lets us see the symlink; otherwise it
+may not be possible.) 
+
+
+ .. vim: ft=rst sw=4

=== added file 'doc/developers/ui.txt'
--- doc/developers/ui.txt	1970-01-01 00:00:00 +0000
+++ doc/developers/ui.txt	2010-09-14 06:46:18 +0000
@@ -0,0 +1,235 @@
+=========================
+Interacting with the user
+=========================
+
+Getting Input
+=============
+
+Processing Command Lines
+------------------------
+
+bzrlib has a standard framework for parsing command lines and calling
+processing routines associated with various commands. See builtins.py
+for numerous examples.
+
+
+Standard Parameter Types
+------------------------
+
+There are some common requirements in the library: some parameters need to be
+unicode safe, some need byte strings, and so on. At the moment we have
+only codified one specific pattern: Parameters that need to be unicode
+should be checked via ``bzrlib.osutils.safe_unicode``. This will coerce the
+input into unicode in a consistent fashion, allowing trivial strings to be
+used for programmer convenience, but not performing unpredictably in the
+presence of different locales.
+
+Confirmation
+============
+
+There are some operations, such as uncommitting, or breaking a lock, where
+bzr may want to get confirmation from the user before proceeding.
+However in some circumstances bzr should just go ahead without asking: if
+it's being used from a noninteractive program, or if the user's asked to
+never be asked for this particular confirmation or for any confirmations
+at all.
+
+We provide a special UIFactory method `confirm_action` to do this.  It
+takes a `confirmation_id` parameter that acts as a symbolic name for the
+type of confirmation, so the user can configure them off.  (This is not
+implemented at present.)  GUIs can have a "don't ask me again" option
+keyed by the confirmation id.
+
+Confirmation ids look like Python paths to the logical code that should
+use them.  (Because code may move or the check may for implementation
+reasons be done elsewhere, they need not perfectly correspond to the place
+they're used, and they should stay stable even when the code moves.)
+
+``bzrlib.builtins.uncommit``
+    Before the ``uncommit`` command actually changes the branch.
+
+``bzrlib.lockdir.break``
+    Before breaking a lock.
+
+``bzrlib.msgeditor.unchanged``
+    Proceed even though the user made no changes to the template message.
+
+Interactive confirmations can be overridden by using a
+`ConfirmationUserInterfacePolicy` decorator as the default
+ui_factory.
+
+
+Writing Output
+==============
+
+(The strategy described here is what we want to get to, but it's not
+consistently followed in the code at the moment.)
+
+bzrlib is intended to be a generically reusable library.  It shouldn't
+write messages to stdout or stderr, because some programs that use it
+might want to display that information through a GUI or some other
+mechanism.
+
+We can distinguish two types of output from the library:
+
+ 1. Structured data representing the progress or result of an
+    operation.  For example, for a commit command this will be a list
+    of the modified files and the finally committed revision number
+    and id.
+
+    These should be exposed either through the return code or by calls
+    to a callback parameter.
+
+    A special case of this is progress indicators for long-lived
+    operations, where the caller should pass a ProgressBar object.
+
+ 2. Unstructured log/debug messages, mostly for the benefit of the
+    developers or users trying to debug problems.  This should always
+    be sent through ``bzrlib.trace`` and Python ``logging``, so that
+    it can be redirected by the client.
+
+The distinction between the two is a bit subjective, but in general if
+there is any chance that a library would want to see something as
+structured data, we should make it so.
+
+The policy about how output is presented in the text-mode client
+should be only in the command-line tool.
+
+
+Progress and Activity Indications
+---------------------------------
+
+bzrlib has a way for code to display to the user that stuff is happening
+during a long operation.  There are two particular types: *activity* which
+means that IO is happening on a Transport, and *progress* which means that
+higher-level application work is occurring.  Both are drawn together by
+the `ui_factory`.
+
+Transport objects are responsible for calling `report_transport_activity`
+when they do IO.
+
+Progress uses a model/view pattern: application code acts on a
+`ProgressTask` object, which notifies the UI when it needs to be
+displayed.  Progress tasks form a stack.  To create a new progress task on
+top of the stack, call `bzrlib.ui.ui_factory.nested_progress_bar()`, then
+call `update()` on the returned ProgressTask.  It can be updated with just
+a text description, with a numeric count, or with a numeric count and
+expected total count.  If an expected total count is provided the view
+can show the progress moving along towards the expected total.
+
+The user should call `finish` on the `ProgressTask` when the logical
+operation has finished, so it can be removed from the stack.
+
+Progress tasks have a complex relationship with generators: it's a very
+good place to use them, but because python2.4 does not allow ``finally``
+blocks in generators it's hard to clean them up properly.  In this case
+it's probably better to have the code calling the generator allocate a
+progress task for its use and then call `finalize` when it's done, which
+will close it if it was not already closed.  The generator should also
+finish the progress task when it exits, because it may otherwise be a long
+time until the finally block runs.
+
+
+Message guidelines
+------------------
+
+When filenames or similar variables are presented inline within a message,
+they should be enclosed in double quotes (ascii 0x22, not chiral unicode
+quotes)::
+
+  bzr: ERROR: No such file "asdf"
+
+When we print just a list of filenames there should not be any quoting:
+see `bug 544297`_.
+
+.. _bug 544297: https://bugs.launchpad.net/bugs/544297
+
+https://wiki.ubuntu.com/UnitsPolicy provides a good explanation about
+which unit should be used when. Roughly speaking, IEC standard applies
+for base-2 units and SI standard applies for base-10 units:
+
+* for network bandwidth and disk sizes, use base-10 (Mbits/s, kB/s, GB)
+
+* for RAM sizes, use base-2 (GiB, TiB)
+
+
+
+Displaying help
+===============
+
+Bazaar has online help for various topics through ``bzr help COMMAND`` or
+equivalently ``bzr command -h``.  We also have help on command options,
+and on other help topics.  (See ``help_topics.py``.)
+
+As for python docstrings, the first paragraph should be a single-sentence
+synopsis of the command. These are user-visible and should be prefixed with
+``__doc__ =`` so help works under ``python -OO`` with docstrings stripped.
+
+The help for options should be one or more proper sentences, starting with
+a capital letter and finishing with a full stop (period).
+
+All help messages and documentation should have two spaces between
+sentences.
+
+
+Handling Errors and Exceptions
+==============================
+
+Commands should return non-zero when they encounter circumstances that
+the user should really pay attention to - which includes trivial shell
+pipelines.
+
+Recommended values are:
+
+    0. OK.
+    1. Conflicts in merge-like operations, or changes are present in
+       diff-like operations.
+    2. Unrepresentable diff changes (i.e. binary files that we cannot show
+       a diff of).
+    3. An error or exception has occurred.
+    4. An internal error occurred (one that shows a traceback.)
+
+Errors are handled through Python exceptions. Exceptions should be defined
+inside bzrlib.errors, so that we can see the whole tree at a glance.
+
+We broadly classify errors as either being either internal or not,
+depending on whether ``internal_error`` is set or not.  If we think it's our
+fault, we show a backtrace, an invitation to report the bug, and possibly
+other details.  This is the default for errors that aren't specifically
+recognized as being caused by a user error.  Otherwise we show a briefer
+message, unless -Derror was given.
+
+Many errors originate as "environmental errors" which are raised by Python
+or builtin libraries -- for example IOError.  These are treated as being
+our fault, unless they're caught in a particular tight scope where we know
+that they indicate a user errors.  For example if the repository format
+is not found, the user probably gave the wrong path or URL.  But if one of
+the files inside the repository is not found, then it's our fault --
+either there's a bug in bzr, or something complicated has gone wrong in
+the environment that means one internal file was deleted.
+
+Many errors are defined in ``bzrlib/errors.py`` but it's OK for new errors
+to be added near the place where they are used.
+
+Exceptions are formatted for the user by conversion to a string
+(eventually calling their ``__str__`` method.)  As a convenience the
+``._fmt`` member can be used as a template which will be mapped to the
+error's instance dict.
+
+New exception classes should be defined when callers might want to catch
+that exception specifically, or when it needs a substantially different
+format string.
+
+#. If it is something that a caller can recover from, a custom exception
+   is reasonable.
+
+#. If it is a data consistency issue, using a builtin like
+   ``ValueError``/``TypeError`` is reasonable.
+
+#. If it is a programmer error (using an api incorrectly)
+   ``AssertionError`` is reasonable.
+
+#. Otherwise, use ``BzrError`` or ``InternalBzrError``.
+
+Exception strings should start with a capital letter and should not have a
+final fullstop.  If long, they may contain newlines to break the text.

=== modified file 'doc/en/Makefile'
--- doc/en/Makefile	2010-03-25 10:09:32 +0000
+++ doc/en/Makefile	2010-07-05 12:40:28 +0000
@@ -112,3 +112,7 @@
 	$(SPHINXBUILD) -b doctest $(ALLSPHINXOPTS) _build/doctest
 	@echo "Testing of doctests in the sources finished, look at the " \
 	      "results in _build/doctest/output.txt."
+
+texinfo:
+	$(SPHINXBUILD) -b texinfo $(ALLSPHINXOPTS) _build/texinfo
+

=== modified file 'doc/en/_templates/index.html'
--- doc/en/_templates/index.html	2010-03-04 04:15:18 +0000
+++ doc/en/_templates/index.html	2010-09-24 02:19:28 +0000
@@ -7,7 +7,7 @@
 
   <table class="contentstable" align="center" style="margin-left: 30px"><tr>
     <td width="50%">
-        <p class="biglink"><a class="biglink" href="{{ pathto("whats-new/whats-new-in-2.2") }}">What's New in Bazaar 2.2?</a><br/>
+        <p class="biglink"><a class="biglink" href="{{ pathto("whats-new/whats-new-in-2.3") }}">What's New in Bazaar 2.3?</a><br/>
         <span class="linkdescr">what's new in this release</span>
       </p>
         <p class="biglink"><a class="biglink" href="{{ pathto("user-guide/index") }}">User Guide</a><br/>
@@ -45,7 +45,7 @@
         <p class="biglink"><a class="biglink" href="https://answers.launchpad.net/bzr/+faqs">FAQ</a><br/>
       <span class="linkdescr">frequently asked questions</span>
       </p>
-      <p class="biglink"><a class="biglink" href="http://bazaar.canonical.com/BzrGlossary/">Glossary</a><br/>
+      <p class="biglink"><a class="biglink" href="http://wiki.bazaar.canonical.com/BzrGlossary/">Glossary</a><br/>
       <span class="linkdescr">help with terminology</span>
       </p>
     </td>

=== modified file 'doc/en/admin-guide/code-browsing.txt'
--- doc/en/admin-guide/code-browsing.txt	2009-12-11 00:00:04 +0000
+++ doc/en/admin-guide/code-browsing.txt	2010-08-13 19:08:57 +0000
@@ -47,8 +47,8 @@
    On Ubuntu, `sudo apt-get install python-flup`
    or use `easy_install flup`
 
-Although directions for installing these on Ubuntu Linux are given, most other
-Linux distributions should package these dependencies, making installation
+Although directions for installing these on Ubuntu are given, most other
+GNU/Linux distributions should package these dependencies, making installation
 easy.  For Windows and Mac OS X, they should all be ``easy_install``-able or at
 worst installable from the Python sources.
 
@@ -101,17 +101,17 @@
 This would allow the trunk branch of ProjectX to be browsed at
 ``http://www.example.com/loggerhead/projectx/trunk``.
 
-Loggerhead comes with a script allowing it to run as a service on init.d based
-Linux systems.  Contributions to do a similar thing on Windows servers would
-be welcomed at http://launchpad.net/loggerhead.
+Loggerhead comes with a script allowing it to run as a service on
+``init.d`` based Unix systems.  Contributions to do a similar thing on
+Windows servers would be welcomed at http://launchpad.net/loggerhead.
   
 
 Other web interfaces
 --------------------
 
 There are a number of other web interfaces available for Bazaar branches (see
-the list at http://bazaar-vcs.org/WebInterfaces) and we will just mention a
-couple of them here for their advantages in particular situations.
+the list at http://wiki.bazaar.canonical.com/WebInterfaces) and we will just
+mention a couple of them here for their advantages in particular situations.
 
 trac+bzr (http://launchpad.net/trac-bzr)
   Trac is a popular web app that integrates a browser for branches, an issue

=== modified file 'doc/en/admin-guide/hooks-plugins.txt'
--- doc/en/admin-guide/hooks-plugins.txt	2010-03-01 05:15:11 +0000
+++ doc/en/admin-guide/hooks-plugins.txt	2010-08-13 19:08:57 +0000
@@ -59,8 +59,8 @@
 mail should be sent.  If it isn't set, email is sent via ``/usr/bin/mail``.
 It can also be configured to communicate directly with an SMTP server.
 For more details on configuring this plugin, see
-http://doc.bazaar-vcs.org/plugins/en/email-plugin.html.  As examples, consider
-the following two possible configurations.  A minimal one (uses
+http://doc.bazaar.canonical.com/plugins/en/email-plugin.html.  As examples,
+consider the following two possible configurations.  A minimal one (uses
 ``/usr/bin/mail``)
 
 ::
@@ -166,8 +166,8 @@
 ``branch.atom`` files.
 
 In addition, there are other tools that are not plugins for creating news
-feeds from Bazaar branches.  See http://bazaar-vcs.org/FeedGenerators for more
-on those tools.
+feeds from Bazaar branches.  See
+http://wiki.bazaar.canonical.com/FeedGenerators for more on those tools.
 
 Mirroring
 ---------

=== modified file 'doc/en/admin-guide/introduction.txt'
--- doc/en/admin-guide/introduction.txt	2009-12-07 21:51:01 +0000
+++ doc/en/admin-guide/introduction.txt	2010-06-02 05:03:31 +0000
@@ -27,7 +27,7 @@
 What you need to run a Bazaar server
 ------------------------------------
 
-Where possible, we will discuss both Unix (including Linux) and Windows server
+Where possible, we will discuss both Unix (including GNU/Linux) and Windows server
 environments.  For the purposes of this document, we will consider Mac OS X as
 a type of Unix.
 
@@ -44,7 +44,7 @@
 critical components of the code.  Pure Python alternatives exist for all of
 these components, but they may be considerably slower.  To compile these
 extensions, you need a C compiler and the relevant header files from the
-Python package.  On Linux, these may be in a separate package.  Other
+Python package.  On GNU/Linux, these may be in a separate package.  Other
 operating systems should have the required headers installed by default.
 
 If you are installing a development version of Bazaar, rather than a released

=== modified file 'doc/en/admin-guide/migration.txt'
--- doc/en/admin-guide/migration.txt	2009-12-07 18:29:00 +0000
+++ doc/en/admin-guide/migration.txt	2010-08-13 19:08:57 +0000
@@ -3,7 +3,7 @@
 
 Migrating between version control systems can be a complicated process, and
 Bazaar has extensive documentation on the process at 
-http://doc.bazaar-vcs.org/migration/en and we won't attempt to repeat that
+http://doc.bazaar.canonical.com/migration/en and we won't attempt to repeat that
 here.  We will try to give a few motivating examples for conversion from
 Mercurial and Subversion.
 
@@ -47,7 +47,7 @@
 Bazaar's `svn`_ plugin provides tools for interaction with Subversion
 projects.  In fact, Bazaar can be used transparently with projects stored in
 Subversion, but that is beyond the scope of this document.  (See
-http://doc.bazaar-vcs.org/en/migration/foreign/bzr-on-svn-projects.html for
+http://doc.bazaar.canonical.com/en/migration/foreign/bzr-on-svn-projects.html for
 more on that subject.)  What is relevant here is the ``svn-import`` command
 provided by that plugin.  This can import an entire subversion repository
 including tags and branches, particularly if they are stored in Subversion's

=== modified file 'doc/en/admin-guide/other-setups.txt'
--- doc/en/admin-guide/other-setups.txt	2009-12-07 21:51:01 +0000
+++ doc/en/admin-guide/other-setups.txt	2010-09-23 10:56:44 +0000
@@ -6,8 +6,8 @@
 
 Bazaar can also serve branches over protocols that know nothing about Bazaar's
 specific needs.  These are called "dumb servers" to distinguish them from
-Bazaar's native protocol.  Currently HTTP, FTP, SFTP and HTTP+WebDAV can be
-used to read branches remotely.  FTP, SFTP and HTTP+WebDAV can be used for
+Bazaar's native protocol.  Currently HTTP, HTTPS, FTP, SFTP and HTTP+WebDAV can
+be used to read branches remotely.  FTP, SFTP and HTTP+WebDAV can be used for
 writing as well.  To use any of these protocols, it is just necessary to
 provide access to the server's filesystem under ``/srv/bzr``.
 
@@ -33,14 +33,14 @@
 Smart server over HTTP(S)
 -------------------------
 
-Bazaar can use its native protocol with HTTP requests.  Since HTTP is a network
-protocol that is available on many networks, this can be a good option where
-SSH access is not possible.  Another benefit of this setup is that all of the
-authentication and access control methods available to the HTTP server (basic,
-LDAP, ActiveDirectory, etc.) are then available to control access to Bazaar
-branches.  More information about setting up this type of access using Apache
-and FastCGI or mod_python or WSGI is in the `smart server section of the User's
-Guide <../user-guide/http_smart_server.html>`_.
+Bazaar can use its native protocol with HTTP or HTTPS requests.  Since HTTP is
+a network protocol that is available on many networks, this can be a good
+option where SSH access is not possible.  Another benefit of this setup is that
+all of the authentication and access control methods available to the HTTP
+server (basic, LDAP, ActiveDirectory, etc.) are then available to control
+access to Bazaar branches.  More information about setting up this type of
+access using Apache and FastCGI or mod_python or WSGI is in the `smart server
+section of the User's Guide <../user-guide/http_smart_server.html>`_.
 
 Direct Smart Server Access
 --------------------------

=== modified file 'doc/en/admin-guide/simple-setups.txt'
--- doc/en/admin-guide/simple-setups.txt	2009-12-07 21:51:01 +0000
+++ doc/en/admin-guide/simple-setups.txt	2010-09-06 20:47:57 +0000
@@ -3,7 +3,7 @@
 
 Consider the following simple scenario where we will be serving Bazaar branches
 that live on a single server.  Those branches are in the subdirectories of
-``/srv/bzr`` (or ``C:\\bzr``) and they will all be related to a single project
+``/srv/bzr`` (or ``C:\bzr``) and they will all be related to a single project
 called "ProjectX".  ProjectX will have a trunk branch and at least one feature
 branch.  As we get further, we will consider other scenarios, but this will be
 a sufficiently motivating example.

=== modified file 'doc/en/admin-guide/upgrade.txt'
--- doc/en/admin-guide/upgrade.txt	2009-12-18 10:09:49 +0000
+++ doc/en/admin-guide/upgrade.txt	2010-08-13 19:08:57 +0000
@@ -28,9 +28,9 @@
 
 Upgrading the Bazaar software is as simple as re-installing the Python package
 using either the latest binary package for Windows or Mac OS X, the binary
-package provided by your Linux distribution, or installing from the source
-release.  See http://bazaar-vcs.org/Downloads for the latest releases for all
-supported platforms.
+package provided by your GNU/Linux distribution, or installing from the source
+release.  See http://wiki.bazaar.canonical.com/Downloads for the latest
+releases for all supported platforms.
 
 Bazaar's later versions support all of the earlier disk formats (back to the
 very first one), so there is no need to upgrade the branches on the disk when

=== modified file 'doc/en/conf.py'
--- doc/en/conf.py	2010-03-03 04:18:28 +0000
+++ doc/en/conf.py	2010-07-05 12:40:28 +0000
@@ -13,8 +13,7 @@
 sys.path = [os.path.abspath('../..')] + sys.path
 
 # Most of the configuration for Bazaar docs is defined here ...
-from bzrlib.doc_generate.sphinx_conf import *
-
+from bzrlib.doc_generate.conf import *
 
 ## Configuration specific to this site ##
 
@@ -54,7 +53,7 @@
   # Manuals
   ('user-guide/index', 'bzr-%s-user-guide.tex' % (bzr_locale,),
     bzr_title(u'Bazaar User Guide'), bzr_team, 'manual'),
-  ('user-reference/bzr_man', 'bzr-%s-user-reference.tex' % (bzr_locale,),
+  ('user-reference/index', 'bzr-%s-user-reference.tex' % (bzr_locale,),
     bzr_title(u'Bazaar User Reference'), bzr_team, 'manual'),
   ('release-notes/NEWS', 'bzr-%s-release-notes.tex' % (bzr_locale,),
     bzr_title(u'Bazaar Release Notes'), bzr_team, 'manual'),

=== modified file 'doc/en/index.txt'
--- doc/en/index.txt	2010-03-04 04:15:18 +0000
+++ doc/en/index.txt	2010-08-18 07:25:22 +0000
@@ -10,7 +10,7 @@
 .. toctree::
    :maxdepth: 1
 
-   whats-new/whats-new-in-2.2
+   whats-new/whats-new-in-2.3
    user-guide/index
    tutorials/index
    quick-reference/index

=== modified file 'doc/en/mini-tutorial/index.txt'
--- doc/en/mini-tutorial/index.txt	2009-12-03 00:44:16 +0000
+++ doc/en/mini-tutorial/index.txt	2010-08-17 06:45:33 +0000
@@ -29,10 +29,10 @@
 For other platforms and to install from source code, see the Download_
 and Installation_ pages.
 
-.. _installation instructions for Windows: http://bazaar-vcs.org/WindowsDownloads
-.. _installation instructions for Mac OS X: http://bazaar-vcs.org/MacOSXBundle
-.. _Download: http://bazaar-vcs.org/Download
-.. _Installation: http://bazaar-vcs.org/InstallationFaq
+.. _installation instructions for Windows: http://wiki.bazaar.canonical.com/WindowsDownloads
+.. _installation instructions for Mac OS X: http://wiki.bazaar.canonical.com/MacOSXBundle
+.. _Download: http://wiki.bazaar.canonical.com/Download
+.. _Installation: http://wiki.bazaar.canonical.com/InstallationFaq
 
 
 Introducing yourself
@@ -64,7 +64,7 @@
  $ mkdir subdirectory
  $ touch test1.txt test2.txt test3.txt subdirectory/test4.txt
 
-**Note for Windows users:** use Windows Explorer to create your
+**Note for Windows users**: use Windows Explorer to create your
 directories, then right-click in those directories and select
 ``New file`` to create your files.
 
@@ -75,7 +75,7 @@
 If it looks like nothing happened, don't worry. Bazaar has created a
 branch_ where it will store your files and their revision histories.
 
-.. _branch: http://bazaar-vcs.org/Branch
+.. _branch: http://wiki.bazaar.canonical.com/Branch
 
 The next step is to tell Bazaar which files you want to track. Running
 ``bzr add`` will recursively add everything in the project::
@@ -164,8 +164,8 @@
 
  $ bzr branch http://www.example.com/myproject
 
-**Note:** to use sftp, you may need to install ``paramiko`` and
-``pyCrypto``. See http://bazaar-vcs.org/InstallationFaq for details.
+**Note**: to use sftp, you may need to install ``paramiko`` and
+``pyCrypto``. See http://wiki.bazaar.canonical.com/InstallationFaq for details.
 
 
 Publishing your branch with Launchpad
@@ -186,7 +186,7 @@
 
 .. [#] Use of the ``lp:`` URL scheme requires bzr 0.92 or later.
 
-**Note:** ``+junk`` means that this branch isn't associated with any particular
+**Note**: ``+junk`` means that this branch isn't associated with any particular
 project in Launchpad.
 
 Now, anyone can create their own copy of your branch by typing::

=== modified file 'doc/en/tutorials/centralized_workflow.txt'
--- doc/en/tutorials/centralized_workflow.txt	2009-12-03 23:21:16 +0000
+++ doc/en/tutorials/centralized_workflow.txt	2010-08-13 19:08:57 +0000
@@ -21,7 +21,7 @@
 in sync.  This workflow is also applicable to a single developer working
 on several different machines.
 
-.. _Bazaar: http://bazaar-vcs.org
+.. _Bazaar: http://bazaar.canonical.com
 
 
 Initial Setup
@@ -100,7 +100,7 @@
 document. See `Tracking Upstream`_ for some possibilities (section
 "Converting and keeping history").
 
-.. _Tracking Upstream: http://bazaar-vcs.org/TrackingUpstream
+.. _Tracking Upstream: http://wiki.bazaar.canonical.com/TrackingUpstream
 
 ..
    XXX: We really need a different document for discussing conversion of a
@@ -231,7 +231,7 @@
 
    % bzr cbranch dev my-feature-branch
 
-.. _bzrtools: http://bazaar-vcs.org/BzrTools
+.. _bzrtools: http://wiki.bazaar.canonical.com/BzrTools
 
 
 Merging changes back

=== modified file 'doc/en/tutorials/tutorial.txt'
--- doc/en/tutorials/tutorial.txt	2009-12-04 06:21:36 +0000
+++ doc/en/tutorials/tutorial.txt	2010-08-13 19:08:57 +0000
@@ -211,8 +211,8 @@
 giving the name of the directory containing the branch.  bzr also supports
 accessing branches over http and sftp, for example::
 
-    % bzr log http://bazaar-vcs.org/bzr/bzr.dev/
-    % bzr log sftp://bazaar-vcs.org/bzr/bzr.dev/
+    % bzr log http://bazaar.launchpad.net/~bzr-pqm/bzr/bzr.dev/
+    % bzr log sftp://bazaar.launchpad.net/~bzr-pqm/bzr/bzr.dev/
 
 By installing bzr plugins you can also access branches using the rsync
 protocol.
@@ -535,7 +535,7 @@
 the existing branch.  Because this new copy is potentially a new branch,
 the command is called **branch**::
 
-    % bzr branch http://bazaar-vcs.org/bzr/bzr.dev
+    % bzr branch lp:bzr bzr.dev
     % cd bzr.dev
 
 This copies down the complete history of this branch, so we can do all

=== modified file 'doc/en/tutorials/using_bazaar_with_launchpad.txt'
--- doc/en/tutorials/using_bazaar_with_launchpad.txt	2009-12-02 20:34:07 +0000
+++ doc/en/tutorials/using_bazaar_with_launchpad.txt	2010-08-13 19:08:57 +0000
@@ -23,7 +23,7 @@
 the software: the team, users, training partners, support partners,
 third-party developers and so on.
 
-Great communities are well understood in the open source world.
+Great communities are well understood in the free software community.
 Their applicability extends well beyond that though: most
 successful commercial software vendors are well skilled at
 building and managing the communities that grow up around
@@ -92,9 +92,9 @@
 ------------------------------------------
 
 As well as sponsoring `Ubuntu <http://www.ubuntu.com>`_ and
-`Bazaar <http://bazaar-vcs.org>`_ development, Canonical
-provides Launchpad, https://launchpad.net, as a free service
-for the open source community. Launchpad is one of the most
+`Bazaar <http://bazaar.canonical.com>`_ development, Canonical
+provides Launchpad, <https://launchpad.net/>, as a free service
+for the community. Launchpad is one of the most
 exciting CDEs around for several notable reasons:
 
 * it models relationships between many of things tracked, e.g.
@@ -179,10 +179,10 @@
 Accessing code in Launchpad using Bazaar
 ========================================
 
-Getting the code for an open source project
--------------------------------------------
+Getting the code for a project
+------------------------------
 
-As Launchpad keeps track of thousands of open source projects
+As Launchpad keeps track of thousands of projects
 and their latest code whether it be managed by Bazaar, CVS or Subversion,
 Bazaar users can grab that code as easily as this::
 
@@ -447,8 +447,7 @@
 further integrated, please contact us on the Bazaar mailing list,
 bazaar@lists.canonical.com.
 
-While designed as a free service to support open source projects,
+While designed as a free service to support free software projects,
 Canonical may make Launchpad available to commercial software developers
 depending on their requirements. We would be happy to hear from you
-if you think Launchpad would be useful for managing your community,
-open source or otherwise.
+if you think Launchpad would be useful for managing your community.

=== modified file 'doc/en/upgrade-guide/overview.txt'
--- doc/en/upgrade-guide/overview.txt	2009-07-13 06:58:49 +0000
+++ doc/en/upgrade-guide/overview.txt	2010-08-13 19:08:57 +0000
@@ -36,7 +36,7 @@
 to Bazaar 1.y. In either case, a brief outline of the steps is given
 below.
 
-To upgrade Bazaar on Linux:
+To upgrade Bazaar on Ubuntu:
 
 1. Ensure your package manager is configured with the required
    software sources, e.g. the official release PPA for Ubuntu:
@@ -61,7 +61,7 @@
 2. Upgrade to the latest version using **sudo port upgrade bzr**
 
 For further information on installing and upgrading, see
-http://bazaar-vcs.org/Download.
+http://wiki.bazaar.canonical.com/Download.
 
 
 Upgrading required plugins
@@ -72,10 +72,10 @@
 bzr-svn, are more tightly associated with Bazaar's APIs so these
 typically need to be upgraded in lockstep with the core software.
 
-For Windows and OS X users, bzrtools and bzr-svn are typically
-included in the installer so no special steps are required to upgrade
-these. For Linux and UNIX users, bztrools, bzr-svn and many other
-popular plugins can be installed and upgraded using your
+For Windows and OS X users, bzrtools and bzr-svn are typically included in
+the installer so no special steps are required to upgrade these. For
+Ubuntu and other GNU/Linux or Unix systems users, bztrools, bzr-svn and
+many other popular plugins can be installed and upgraded using your
 platform's package manager, e.g. Synaptic on Ubuntu.
 
 

=== modified file 'doc/en/user-guide/branching_a_project.txt'
--- doc/en/user-guide/branching_a_project.txt	2010-03-22 00:29:50 +0000
+++ doc/en/user-guide/branching_a_project.txt	2010-10-01 23:09:27 +0000
@@ -8,7 +8,7 @@
 agree on a transfer technology.
 You may decide to make the top level directory of your branch
 a network share, an approach familiar to Windows users.
-Linux and OS X users might prefer access to be
+Unix users might prefer access to be
 via SFTP, a secure protocol built-in to most SSH servers.
 Bazaar is *very* flexible in this regard with support for
 lots of protocols some of which are given below.
@@ -20,14 +20,15 @@
   \sftp://            Access using SFTP (most SSH servers provide SFTP).
   \bzr://             Fast access using the Bazaar smart server.
   \ftp://             Access using passive FTP.
-  \http://            Read-only access to branches exported by a web server.
+  \http://            Access to branches exported by a web server.
+  \https://           Encrypted access to branches exported by a web server.
   ===========         ======================================================
 
 As indicated above, branches are identified using URLs with the
 prefix indicating the transfer technology. If no prefix is given,
 normal filenames are assumed. For a complete list of supported
 protocols, see the ``urlspec`` online help topic or the
-`URL Identifiers <../user-reference/bzr_man.html#url-identifiers>`_
+`URL Identifiers <../user-reference/urlspec-help.html>`_
 section of the Bazaar User Reference.
 
 URLs are normally resolved relative to the root directory of the server,
@@ -41,6 +42,11 @@
 tilde like so: ``sftp://example.com/~/public_html`` should map to
 ``public_html`` within your home directory.
 
+.. note:: Access over HTTP or HTTPS is read-only by default.
+   See `Pushing over the http smart server
+   <http_smart_server.html#pushing-over-the-http-smart-server>`_ for
+   details on configuring read-write access.
+
 A reminder about shared repositories
 ------------------------------------
 

=== modified file 'doc/en/user-guide/bzrtools_plugin.txt'
--- doc/en/user-guide/bzrtools_plugin.txt	2009-03-12 06:04:54 +0000
+++ doc/en/user-guide/bzrtools_plugin.txt	2010-08-13 19:08:57 +0000
@@ -6,7 +6,7 @@
 
 BzrTools is a collection of useful enhancements to Bazaar.
 For installation instructions, see the BzrTools home page:
-http://bazaar-vcs.org/BzrTools.
+http://wiki.bazaar.canonical.com/BzrTools.
 Here is a sample of the frequently used commands it provides.
 
 

=== modified file 'doc/en/user-guide/configuring_bazaar.txt'
--- doc/en/user-guide/configuring_bazaar.txt	2009-12-02 20:34:07 +0000
+++ doc/en/user-guide/configuring_bazaar.txt	2010-09-27 19:31:45 +0000
@@ -16,10 +16,32 @@
 
 If ``whoami`` is used without an argument, the current value is displayed.
 
+Using a network proxy
+---------------------
+
+If your network requires that you use an http proxy for outbound
+connections, you must set the ``http_proxy`` variable.  If the proxy is
+also required for https connections, you need to set ``https_proxy`` too.
+If you need these and don't have them set, you may find that connections
+to Launchpad or other external servers fail or time out.
+
+On Unix you typically want to set these in ``/etc/environment`` or
+``~/.bash_profile`` and on Windows in the user profile.
+
+::
+
+  http_proxy=http://proxy.example.com:3128/
+  https_proxy=http://proxy.example.com:3128/
+
+The ``no_proxy`` variable can be set to a comma-separated list of hosts
+which shouldn't be reached by the proxy.  (See
+<http://docs.python.org/library/urllib.html> for more details.)
+
+
 Configuration files
 -------------------
 
-Configuration files are located in ``$HOME/.bazaar`` on Linux/Unix and
+Configuration files are located in ``$HOME/.bazaar`` on Unix and
 ``C:\Documents and Settings\<username>\Application Data\Bazaar\2.0`` on
 Windows. There are three primary configuration files in this location:
 
@@ -44,7 +66,7 @@
   email = Your Name <email@example.com>
 
 For further details on the syntax and configuration settings supported, see
-`Configuration Settings <../user-reference/bzr_man.html#configuration-settings>`_
+`Configuration Settings <../user-reference/configuration-help.html>`_
 in the Bazaar User Reference.
 
 
@@ -56,5 +78,5 @@
 ``BZR_HOME/rules``.
 
 For further information on how rules are searched and the detailed syntax of
-the relevant files, see `Rules <../user-reference/bzr_man.html#rules>`_
+the relevant files, see `Rules <../user-reference/rules-help.html>`_
 in the Bazaar User Reference.

=== modified file 'doc/en/user-guide/hooks.txt'
--- doc/en/user-guide/hooks.txt	2010-04-22 07:35:06 +0000
+++ doc/en/user-guide/hooks.txt	2010-08-17 03:08:33 +0000
@@ -81,7 +81,7 @@
   
   def merge_xml_files_hook(merger):
       """Hook to merge *.xml files"""
-      return MergeXMLFiles(merger)
+      return AlwaysConflictXMLMerger(merger)
   
   class AlwaysConflictXMLMerger(PerFileMerger):
   

=== modified file 'doc/en/user-guide/http_smart_server.txt'
--- doc/en/user-guide/http_smart_server.txt	2010-02-26 18:47:48 +0000
+++ doc/en/user-guide/http_smart_server.txt	2010-10-01 23:09:27 +0000
@@ -5,9 +5,7 @@
 using Apache 2.0 and FastCGI or mod_python or mod_wsgi.
 
 For more information on the smart server, and other ways to configure it
-see the main `smart server documentation`_.
-
-.. _smart server documentation: #running-a-smart-server
+see the main `smart server documentation <server.html>`_.
 
 Example
 -------

=== modified file 'doc/en/user-guide/installing_bazaar.txt'
--- doc/en/user-guide/installing_bazaar.txt	2009-09-09 15:08:17 +0000
+++ doc/en/user-guide/installing_bazaar.txt	2010-08-13 19:08:57 +0000
@@ -1,12 +1,12 @@
 Installing Bazaar
 =================
 
-Linux
------
+GNU/Linux
+---------
 
-Bazaar packages are available for most popular Linux distributions
-including Ubuntu/Debian, Red Hat and Gentoo.
-See http://bazaar-vcs.org/Download for the latest instructions.
+Bazaar packages are available for most popular GNU/Linux distributions
+including Ubuntu, Debian, Red Hat and Gentoo.
+See http://wiki.bazaar.canonical.com/Download for the latest instructions.
 
 Windows
 -------
@@ -14,7 +14,7 @@
 For Windows users, an installer is available that includes
 the core Bazaar package together with necessary pre-requisites
 and some useful plug-ins.
-See http://bazaar-vcs.org/Download for the latest instructions.
+See http://wiki.bazaar.canonical.com/Download for the latest instructions.
 
 Note: If you are running Cygwin on Windows, a Bazaar for Cygwin package
 is available and ought to be used instead of the Windows version.
@@ -24,7 +24,7 @@
 
 Beyond Linux and Windows, Bazaar packages are available for a large
 range of other operating systems include Mac OS X, FreeBSD and Solaris.
-See http://bazaar-vcs.org/Download for the latest instructions.
+See http://wiki.bazaar.canonical.com/Download for the latest instructions.
 
 
 Installing from scratch
@@ -36,7 +36,7 @@
   1. If it is not installed already, install Python 2.4 or later.
 
   2. Download the ``bazaar-xxx.tar.gz`` file (where xxx is the version
-     number) from http://bazaar-vcs.org/Download or from Launchpad
+     number) from http://wiki.bazaar.canonical.com/Download or from Launchpad
      (https://launchpad.net/~bzr/).
 
   3. Unpack the archive using tar, WinZip or equivalent.

=== modified file 'doc/en/user-guide/introducing_bazaar.txt'
--- doc/en/user-guide/introducing_bazaar.txt	2010-03-11 04:33:41 +0000
+++ doc/en/user-guide/introducing_bazaar.txt	2010-08-13 19:08:57 +0000
@@ -79,7 +79,7 @@
  * trunk quality and stability can be kept higher, making everyone's
    job less stressful
 
- * in open source communities:
+ * in open communities:
 
    * easier for non-core developers to create and maintain changes
 
@@ -89,7 +89,7 @@
  * in companies, easier to work with distributed and outsourced teams.
 
 For a detailed look at the advantages of distributed VCS tools over
-centralized VCS tools, see http://bazaar-vcs.org/BzrWhy.
+centralized VCS tools, see http://wiki.bazaar.canonical.com/BzrWhy.
 
 
 Key features of Bazaar
@@ -98,12 +98,12 @@
 While Bazaar is not the only distributed VCS tool around, it does have some
 notable features that make it an excellent choice for many teams and
 communities. A summary of these and comparisons with other VCS tools
-can be found on the Bazaar Wiki, http://bazaar-vcs.org.
+can be found on the Bazaar Wiki, http://wiki.bazaar.canonical.com.
 
 Of the many features, one in particular is worth highlighting:
 Bazaar is completely free software written in Python. As a result,
 it is easy to contribute improvements. If you wish to get involved,
-please see http://bazaar-vcs.org/BzrSupport.
+please see http://wiki.bazaar.canonical.com/BzrSupport.
 
 
 Learning more

=== modified file 'doc/en/user-guide/plugins.txt'
--- doc/en/user-guide/plugins.txt	2009-12-02 20:34:07 +0000
+++ doc/en/user-guide/plugins.txt	2010-09-27 19:31:45 +0000
@@ -28,20 +28,20 @@
 Where to find plugins
 ---------------------
 
-We keep our list of plugins on the http://bazaar-vcs.org/BzrPlugins page.
+We keep our list of plugins on the http://wiki.bazaar.canonical.com/BzrPlugins page.
 
 How to install a plugin
 -----------------------
 
 Installing a plugin is very easy! If not already created, create a
 ``plugins`` directory under your Bazaar configuration directory,
-``~/.bazaar/`` on Linux and
+``~/.bazaar/`` on Unix and
 ``C:\Documents and Settings\<username>\Application Data\Bazaar\2.0\``
 on Windows. Within this directory (referred to as $BZR_HOME below),
 each plugin is placed in its own subdirectory.
 
 Plugins work particularly well with Bazaar branches. For example, to
-install the bzrtools plugins for your main user account on Linux,
+install the bzrtools plugins for your main user account on GNU/Linux,
 one can perform the following::
 
     bzr branch http://panoramicfeedback.com/opensource/bzr/bzrtools
@@ -59,8 +59,8 @@
 If you have the necessary permissions, plugins can also be installed on a
 system-wide basis.  One can additionally override the personal plugins
 location by setting the environment variable ``BZR_PLUGIN_PATH`` (see `User
-Reference <../user-reference/bzr_man.html#bzr-plugin-path>`_ for a detailed
-explanation).
+Reference <../user-reference/configuration-help.html#bzr-plugin-path>`_ 
+for a detailed explanation).
 
 Listing the installed plugins
 -----------------------------

=== modified file 'doc/en/user-guide/sending_changes.txt'
--- doc/en/user-guide/sending_changes.txt	2009-12-02 20:34:07 +0000
+++ doc/en/user-guide/sending_changes.txt	2010-09-27 19:31:45 +0000
@@ -42,7 +42,7 @@
 ``send`` without options will create a merge directive, fire up your email
 tool and attach it, ready for you to add the explanatory text bit.
 (See the online help for ``send`` and
-`Configuration Settings <../user-reference/bzr_man.html#configuration-settings>`_
+`Configuration Settings <../user-reference/configuration-help.html>`_
 in the User Reference for further details on how to configure this.)
 
 Most projects like people to add some explanation to the mail along with

=== modified file 'doc/en/user-guide/server.txt'
--- doc/en/user-guide/server.txt	2009-12-02 20:34:07 +0000
+++ doc/en/user-guide/server.txt	2010-08-13 19:08:57 +0000
@@ -13,7 +13,7 @@
 over any of these protocols, Bazaar will allow you to read it remotely.  Just
 enter the URL to the branch in the Bazaar command you are running.::
 
-    bzr log http://bazaar-vcs.org/bzr/bzr.dev
+    bzr log http://bazaar.launchpad.net/~bzr-pqm/bzr/bzr.dev
 
 Bazaar supports writing over FTP, SFTP and (via a plugin) over HTTP-WebDAV.
 

=== modified file 'doc/en/user-guide/setting_up_email.txt'
--- doc/en/user-guide/setting_up_email.txt	2009-12-02 20:34:07 +0000
+++ doc/en/user-guide/setting_up_email.txt	2010-09-27 19:31:45 +0000
@@ -62,7 +62,7 @@
 --------------------------------------------
 
 To use the default ini file, create or edit the ``bazaar.conf`` file (in
-``~/.bazaar/`` on Linux and in ``%APPDATA%\bazaar\2.0\`` in Windows)
+``~/.bazaar/`` on Unix and in ``%APPDATA%\bazaar\2.0\`` in Windows)
 and set an email address as shown below.  Please note that the word DEFAULT
 is case sensitive, and must be in upper-case.
 ::
@@ -74,7 +74,7 @@
 For more information on the ini file format, see `Configuration Settings`_ in
 the Bazaar User Reference.
 
-.. _Configuration Settings: ../user-reference/bzr_man.html#configuration-settings
+.. _Configuration Settings: ../user-reference/configuration-help.html
 
 Setting email on a per-branch basis
 -----------------------------------

=== modified file 'doc/en/user-guide/shared_repository_layouts.txt'
--- doc/en/user-guide/shared_repository_layouts.txt	2009-12-02 20:34:07 +0000
+++ doc/en/user-guide/shared_repository_layouts.txt	2010-08-13 19:08:57 +0000
@@ -121,7 +121,7 @@
     Letting you ``bzr checkout`` one project, and have it grab all the necessary
     sub-projects.
 
-.. _NestedTreeSupport: http://bazaar-vcs.org/NestedTrees
+.. _NestedTreeSupport: http://wiki.bazaar.canonical.com/NestedTrees
 
 
 Nested Style (``project/branch/sub-branch/``)

=== modified file 'doc/en/user-guide/specifying_revisions.txt'
--- doc/en/user-guide/specifying_revisions.txt	2009-12-02 23:09:40 +0000
+++ doc/en/user-guide/specifying_revisions.txt	2010-09-27 19:31:45 +0000
@@ -65,7 +65,7 @@
 For complete details, see `Revision Identifiers`_ in the
 Bazaar User Reference.
 
-.. _Revision Identifiers: ../user-reference/bzr_man.html#revision-identifiers
+.. _Revision Identifiers: ../user-reference/revisionspec-help.html
 
 Numbers
 ~~~~~~~

=== modified file 'doc/en/user-guide/svn_plugin.txt'
--- doc/en/user-guide/svn_plugin.txt	2009-09-25 10:22:44 +0000
+++ doc/en/user-guide/svn_plugin.txt	2010-08-13 19:08:57 +0000
@@ -20,7 +20,7 @@
 who are unable to do so yet for timing or non-technical reasons.
 
 For installation instructions, see the bzr-svn home page:
-http://bazaar-vcs.org/BzrForeignBranches/Subversion.
+http://wiki.bazaar.canonical.com/BzrForeignBranches/Subversion.
 
 
 A simple example
@@ -110,5 +110,5 @@
  * Bazaar doesn't support tracking of file copies.
 
 See the bzr-svn web page,
-http://bazaar-vcs.org/BzrForeignBranches/Subversion,
+http://wiki.bazaar.canonical.com/BzrForeignBranches/Subversion,
 for the current list of constraints.

=== modified file 'doc/en/user-guide/version_info.txt'
--- doc/en/user-guide/version_info.txt	2009-12-02 20:34:07 +0000
+++ doc/en/user-guide/version_info.txt	2010-09-27 19:31:45 +0000
@@ -25,7 +25,7 @@
   branch-nick: bzr.dev
 
 You can easily filter that output using operating system tools or
-scripts. For example (on Linux/Unix)::
+scripts. For example::
 
   $ bzr version-info | grep ^date
   date: 2007-12-11 17:51:18 +0000
@@ -87,7 +87,7 @@
 variables that can be used in templates, see `Version Info`_ in the
 Bazaar User Reference.
 
-.. _Version Info: ../user-reference/bzr_man.html#version-info
+.. _Version Info: ../user-reference/version-info-help.html
 
 Predefined formats for dumping version information in specific languages
 are currently in development. Please contact us on the mailing list about

=== modified file 'doc/en/user-guide/web_browsing.txt'
--- doc/en/user-guide/web_browsing.txt	2009-12-02 20:34:07 +0000
+++ doc/en/user-guide/web_browsing.txt	2010-08-13 19:08:57 +0000
@@ -9,7 +9,7 @@
 can be found at https://launchpad.net/loggerhead.
 
 A list of alternative web viewers including download links can be found on
-http://bazaar-vcs.org/WebInterface.
+http://wiki.bazaar.canonical.com/WebInterface.
 
 Note: If your project is hosted or mirrored on Launchpad,
 Loggerhead code browsing is provided as part of the service.

=== modified file 'doc/en/user-guide/writing_a_plugin.txt'
--- doc/en/user-guide/writing_a_plugin.txt	2010-04-21 07:52:31 +0000
+++ doc/en/user-guide/writing_a_plugin.txt	2010-09-27 19:31:45 +0000
@@ -42,8 +42,8 @@
 
 Bzr will scan ``~/.bazaar/plugins``  and ``bzrlib/plugins`` for plugins
 by default.  You can override this with  ``BZR_PLUGIN_PATH``
-(see `User Reference <../user-reference/bzr_man.html#bzr-plugin-path>`_
-for details).
+(see `User Reference 
+<../user-reference/configuration-help.html#bzr-plugin-path>`_ for details).
 
 Plugins may be either modules or packages.  If your plugin is a single
 file, you can structure it as a module.  If it has multiple files, or if

=== modified file 'doc/en/user-reference/readme.txt'
--- doc/en/user-reference/readme.txt	2007-12-11 13:40:18 +0000
+++ doc/en/user-reference/readme.txt	2010-08-13 19:08:57 +0000
@@ -5,4 +5,4 @@
 registries, e.g. the set of available file formats. If you're
 not sure where to find the source content you're looking for,
 please contact the developers on the mailing list or IRC.
-See http://bazaar-vcs.org/BzrSupport for contact details.
+See http://wiki.bazaar.canonical.com/BzrSupport for contact details.

=== modified file 'doc/en/whats-new/whats-new-in-2.2.txt'
--- doc/en/whats-new/whats-new-in-2.2.txt	2010-03-04 04:15:18 +0000
+++ doc/en/whats-new/whats-new-in-2.2.txt	2010-08-20 03:53:57 +0000
@@ -1,62 +1,285 @@
+*************************
 What's New in Bazaar 2.2?
-=========================
-
-This document outlines the major improvements in Bazaar 2.2
-vs Bazaar 2.1. As well as summarizing improvements made to
-the core product, it highlights enhancements within the broader
-Bazaar world of potential interest to those upgrading.
-
-.. warning::
-
-   Bazaar 2.2.0 is currently in development. While less stable than
-   Bazaar 2.1.x, we aim to keep quality at a high level:
-
-   * Monthly beta releases should be suitable for early adopters of new
-     features.
-
-   * Nightly builds are suitable for users interested in assisting
-     with testing.
-
-   If it doubt, please use the latest 2.1 release instead.
-
+*************************
+
+Bazaar 2.2.0, released on the 6th of August 2010, marks the start of
+another long-term-stable series. From here, we will only make bugfix
+releases on the 2.2 series (2.2.1, etc), while 2.3 will become our new
+development series. The 2.0 and 2.1 series will also continue to get
+bugfixes. (Currently 2.0 is planned to be supported for another 6 months.)
+
+The main changes in 2.2 are: **better local and network performance**,
+**reduced memory usage**, and several user-interface improvements.
+
+Users are encouraged to upgrade from the other stable series.  This
+document outlines the improvements in Bazaar 2.2 vs Bazaar 2.1. As well as
+summarizing improvements made to the core product, it highlights
+enhancements within the broader Bazaar world of potential interest to
+those upgrading.
+
+Bazaar 2.2.0 includes all the fixes from 2.1.2 and 2.0.6.
+
+Over 120 bugs have been fixed in total.  See the
+:doc:`../release-notes/index` for a full list.
+
+Bazaar 2.2.0 is fully compatible both locally and on the network with 2.0
+and 2.1, and can read and write repositories generated by all previous
+versions.
+
+
+Behaviour changes
+*****************
+
+There are some compatibility changes in this release.  
+
+* For commandline users we no longer guess user identity for ``bzr
+  commit``: users must specify their identity using ``bzr whoami`` (you
+  don't need to specify your identity for readonly operations).
+  This avoids problems where the previous guessed default caused commits
+  be recorded as coming from, for example ``<sam@localhost>``.
 
 Improved conflict handling
---------------------------
+**************************
 
 Tree-shape conflicts can be resolved by providing ``--take-this`` and
 ``--take-other`` to the ``bzr resolve`` command. Just marking the conflict
 as resolved is still accessible via the ``--done`` default action.
 
+Local performance
+*****************
+
+* ``bzr init`` does not recursively scan directory contents anymore
+  leading to faster init for directories with existing content.
+  (Martin [gz], Parth Malwankar, #501307)
+
+* Less code is loaded at startup, so there's less overhead on running all
+  bzr commands.
+  (Andrew Bennetts, Martin Pool)
+
+* Reduce peak memory by one copy of compressed text.
+  (John Arbash Meinel, #566940)
+
+* Avoid repeated locking of local objects in ``diff``, ``missing``, and
+  ``pull``, so those options are faster.
+  (Andrew Bennetts)
+
+Network performance
+*******************
+
+* Bazaar now reads data from SSH connections more efficiently on platforms
+  that provide the ``socketpair`` function, and when using paramiko.
+  (Andrew Bennetts, #590637)
+
+* Index lookups in pack repositories search recently hit pack files
+  first.  In repositories with many pack files this can greatly reduce the
+  number of files accessed, the number of bytes read, and the number of
+  read calls.  An incremental pull via plain HTTP takes half the time and
+  bytes for a moderately large repository.  (Andrew Bennetts)
+
+* Index lookups only re-order the indexes when the hit files aren't
+  already first. Reduces the cost of reordering
+  (John Arbash Meinel, #562429)
+
+
+Command improvements
+********************
+
+* Added ``bzr remove-branch`` command that can remove a local or remote 
+  branch. (Jelmer Vernooij, #276295)
+
+* ``bzr export`` now takes an optional argument ``--per-file-timestamps``
+  to set file mtimes to the last timestamp of the last revision in which
+  they were changed rather than the current time. (Jelmer Vernooij)
+
+* Tag names can now be determined automatically by ``automatic_tag_name`` 
+  hooks on ``Branch`` if they are not specified on the command line.
+  (Jelmer Vernooij)
+
+* Tree-shape conflicts can be resolved by providing ``--take-this`` and
+  ``--take-other`` to the ``bzr resolve`` command. Just marking the conflict
+  as resolved is still accessible via the ``--done`` default action.
+  (Vincent Ladeuil)
+
+* The ``--directory`` option is supported for a number of additional
+  commands: added, annotate, bind, cat, cat-revision, clean-tree,
+  conflicts, deleted, export, ignore, ignored, lookup-revision, ls,
+  merge-directive, missing, modified, nick, re-sign, resolve, shelve,
+  switch, unbind, unknowns, unshelve, whoami.  
+  (Martin von Gagern, #527878)
+
+* ``bzr commit`` accepts ``-p`` (for "patch") as a shorter name for
+  ``--show-diff``.
+  (Parth Malwankar, #571467)
+  
+* ``bzr ignore`` now supports a ``--default-rules`` option that displays
+  the default ignore rules used by bzr. The flag ``--old-default-rules``
+  is no longer supported by ``ignore``.
+  (Parth Malwankar, #538703)
+
+* ``bzr pack`` now supports a ``--clean-obsolete-packs`` option that
+  can save disk space by deleting obsolete pack files created during the
+  pack operation.
+  (Parth Malwankar, #304320)
+
+* New command line option ``--authors`` to ``bzr log`` allows users to
+  select which of the apparent authors and committer should be
+  included in the log. Defaults depend on format. (Martin von Gagern, #513322)
+
+* The bash_completion plugin from the bzr-bash-completion project has
+  been merged into the tree. It provides a bash-completion command and
+  replaces the outdated ``contrib/bash/bzr`` script with a version
+  using the plugin. (Martin von Gagern, #560030)
+
+* A new transport based on GIO (the gnome i/o library) provides access to
+  samba shares, webdav using gio+smb and gio+dav. It is also possible to
+  use gio for some already existing transport methods as gio+file,
+  gio+sftp, gio+ftp. 
+  (Mattias Eriksson)
+
+
+Controlling plugins
+*******************
+
+* Plugins can be disabled by defining ``BZR_DISABLE_PLUGINS`` as
+  a list of plugin names separated by ':' (';' on windows).
+  (Vincent Ladeuil, #411413)
+
+* Plugins can be loaded from arbitrary locations by defining
+  ``BZR_PLUGINS_AT`` as a list of ``name@path`` separated by ':' (';' on
+  Microsoft
+  Windows). This takes precedence over ``BZR_PLUGIN_PATH`` for the
+  specified plugins, and is expected to be most useful for plugin
+  developers.
+  (Vincent Ladeuil, #82693)
+
+
+Apport crash reporting
+**********************
+
+* If the Apport crash-reporting tool is available, bzr crashes are now
+  stored into the ``/var/crash`` apport spool directory, and the user is
+  invited to report them to the developers from there, either
+  automatically or by running ``apport-bug``.  No information is sent
+  without specific permission from the user.  (Martin Pool, #515052)
+
 
 Improved Launchpad integration
-------------------------------
-
-Merges can be proposed on Launchpad with the new **lp-propose-merge**
-command.
-
-
-Notable plugin enhancements
----------------------------
-
-The grep plugin ...
+******************************
+
+* Merges can be proposed on Launchpad with the new ``lp-propose-merge``
+  command.
 
 
 Better documentation
---------------------
-
-Numerous improvements have been made to the developer documentation.
-
-
-Enhanced GUI clients
---------------------
-
-(Details coming soon.)
+********************
+
+* ``bzr help patterns`` now explains case insensitive patterns and
+  points to Python regular expression documentation.
+  (Parth Malwankar, #594386)
+
+* Numerous improvements have been made to the developer documentation.
+
+  
+Changes to plugins
+******************
+
+
+bzr grep
+========
+
+The `grep plugin <https://launchpad.net/bzr-grep>`_  has developed well
+during the bzr 2.2 cycle.  bzr grep can search the versioned files in the
+working tree, or in one or a series of revisions, or it can search through
+only the changes in a revision range.
+
+qbzr
+====
+
+`qbzr <https://launchpad.net/qbzr>`_, a cross-platform graphical interface
+to Bazaar, gained many features and fixes in its 0.19 release, including:
+
+* qannotate has new look and feel; with new features: find text and goto
+  to line.
+
+* Improved performance of qlog, and treewidget-based dialogs (qcommit,
+  qadd, qrevert etc.)
+
+* qpush, qmerge, etc.: when there are uncommitted changes in the working
+  tree, user has the option to commit, or revert.
+
+* qcommit: user can update bound branch/checkout if it is not up to date.
+
+* Better support of Mac OS X: dialog windows no more start in background.
+
+* qlog: Context menu actions for tag and revert will now show a branch
+  menu if more than one branch is open.
+
+* qlog: more context menu actions for update, cherry-pick, and reverse
+  cherry-pick.
+
+* Language of GUI can be set in DEFAULT section of bazaar.conf 
+  as ``language = code``. Language codes are the same 
+  as for ``LANG`` environment variable. 
+  Environment variable ``LANGUAGE`` still preferred over settings 
+  in bazaar.conf.
+
+
+Platform-specific changes
+*************************
+
+Microsoft Windows
+=================
+
+* There's a new py2exe windows program ``bzrw.exe``, which allows for starting a Bazaar GUI with out have a console open in the background.  (Gary van der Merwe, #433781`) 
+
+* The all-in-one Windows installer will now be built with docstrings stripped
+  from the library zip, reducing the size and slightly improving cold startup
+  time. Bundled plugins are unchanged for the moment, but if adding other new
+  plugins to an all-in-one installation, ensure they are compiled and
+  installed with -O1 or help may not work. (Martin [gz])
+
+* Parsing of command lines, for example in ``diff --using``, no longer
+  treats backslash as an escape character on Windows.   (Gordon Tyler,
+  #392248)
+
+
+API changes
+***********
+
+* BzrError subclasses no longer support the name "message" to be used
+  as an argument for __init__ or in _fmt format specification as this
+  breaks in some Python versions. errors.LockError.__init__ argument
+  is now named "msg" instead of earlier "message".
+  (Parth Malwankar, #603461)
+
+* The old ``bzr selftest --benchmark`` option has been removed.
+  <https://launchpad.net/bzr-usertest> is an actively-maintained
+  macrobenchmark suite.
+  (Martin Pool)
+
+* bzrlib library users now need to call ``__enter__`` and ``__exit__`` on
+  the result of ``bzrlib.initialize``. This change was made when fixing
+  the bad habit recent bzr versions have had of leaving progress bars 
+  behind on the screen. That required calling another function before
+  exiting the program, and it made sense to provide a full context
+  manager at the same time. (Robert Collins)
+
+* The ``bzr`` front end now requires a ``bzrlib.ui.ui_factory`` which is a
+  context manager in the Python 2.5 and above sense. The bzrlib base class
+  is such a manager, but third party UI factories which do not derive from
+  ``bzrlib.ui.UIFactory`` will be incompatible with the command line front
+  end.
+
+* URLs like ``foo:bar/baz`` are now always parsed as a URL with scheme "foo"
+  and path "bar/baz", even if bzr does not recognize "foo" as a known URL
+  scheme.  Previously these URLs would be treated as local paths.
+  (Gordon Tyler)
 
 
 Further information
--------------------
+*******************
 
-For more detailed information on the changes made, be sure to check
+For more detailed information on the changes made, see the
 the :doc:`../release-notes/index` for:
 
 * the interim bzr `milestones <https://launchpad.net/bzr/2.2>`_
@@ -66,5 +289,5 @@
 
 * :doc:`whats-new-in-2.1`
 
-Enjoy,
-The Bazaar Development Team
+
+.. vim: ft=rst

=== added file 'doc/en/whats-new/whats-new-in-2.3.txt'
--- doc/en/whats-new/whats-new-in-2.3.txt	1970-01-01 00:00:00 +0000
+++ doc/en/whats-new/whats-new-in-2.3.txt	2010-10-06 04:14:52 +0000
@@ -0,0 +1,116 @@
+*************************
+What's New in Bazaar 2.3?
+*************************
+
+Bazaar 2.3 is still under development, and will be released in February
+2011.  This document accumulates a high level summary of what's changed.
+See the
+:doc:`../release-notes/index` for a full list.
+
+Users are encouraged to upgrade from the other stable series.  This
+document outlines the improvements in Bazaar 2.3 vs Bazaar 2.2. As well as
+summarizing improvements made to the core product, it highlights
+enhancements within the broader Bazaar world of potential interest to
+those upgrading.
+
+Bazaar 2.3.0 is fully compatible both locally and on the network with 2.0
+2.1, and 2.2, and can read and write repositories generated by all
+previous versions.
+
+Changed Behaviour
+*****************
+
+* The default ``ignore`` file created by Bazaar will contain ``__pycache__``,
+  which is the name of the directory that will be used by Python to store
+  bytecode files.
+  (Andrea Corbellini, #626687)
+
+* The default sort order for the ``bzr tags`` command now uses a natural sort
+  where numeric substrings are sorted numerically.  The previous default was
+  "asciibetical" where tags were sorted by the characters they contained.  To
+  get the old behavior, one can use ``bzr tags --sort=alpha``.
+  (Neil Martinsen-Burrell, #640760)
+
+Launchpad integration
+*********************
+
+* The ``lp:`` prefix will now use your known username (from
+  ``bzr launchpad-login``) to expand ``~`` to your username.  For example:
+  ``bzr launchpad-login user && bzr push lp:~/project/branch`` will now
+  push to ``lp:~user/project/branch``.  (John Arbash Meinel)
+
+Performance improvements
+************************
+
+* ``bzr revert`` and ``bzr status`` are up to 15% faster on large trees
+  with many changes by not repeatedly building a list of all file-ids.
+  (Andrew Bennetts)
+
+* ``bzr send`` uses less memory.
+  (John Arbash Meinel, #614576)
+
+* Inventory entries now consume less memory (on 32-bit Ubuntu file entries
+  have dropped from 68 bytes to 40, and directory entries from 120 bytes
+  to 48).  This affects most operations, and depending on the size of the
+  tree may substantially improve the speed of operations like ``bzr
+  commit``.  (Andrew Bennetts)
+
+* Lower memory consumption when reading many chk index pages. Helpful for
+  things like ``bzr co`` or ``bzr ls -R`` on large trees.
+  (John Arbash Meinel)
+
+* When building new working trees, default to reading from the repository
+  rather than the source tree unless explicitly requested. (via
+  ``--files-from`` and ``--hardlink`` for ``bzr branch`` and
+  ``bzr checkout``. Generally, 2a format repositories extract
+  content faster than seeking and reading content from another tree,
+  especially in cold-cache situations. (John Arbash Meinel, #607298)
+
+New revision specifiers
+***********************
+
+* The ``mainline`` revision specifier has been added.  It takes another revision
+  spec as its input, and selects the revision which merged that revision into
+  the mainline.
+  
+  For example, ``bzr log -vp -r mainline:1.2.3`` will show the log of the
+  revision that merged revision 1.2.3 into mainline, along with its status
+  output and diff.  (Aaron Bentley)
+
+* The ``annotate`` revision specifier has been added.  It takes a path and a
+  line as its input (in the form ``path:line``), and selects the revision which
+  introduced that line of that file.
+
+  For example: ``bzr log -vp -r annotate:bzrlib/transform.py:500`` will select
+  the revision that introduced line 500 of transform.py, and display its log,
+  status output and diff.
+
+  It can be combined with ``mainline`` to select the revision that landed this
+  line into trunk, like so: 
+  ``bzr log -vp -r mainline:annotate:bzrlib/transform.py:500``
+  (Aaron Bentley)
+
+Documentation
+*************
+
+* A beta version of the documentation is now available in GNU TexInfo
+  format, used by emacs and the standalone ``info`` reader.
+  (Vincent Ladeuil, #219334)
+
+
+Further information
+*******************
+
+For more detailed information on the changes made, see the
+the :doc:`../release-notes/index` for:
+
+* the interim bzr `milestones <https://launchpad.net/bzr/2.3>`_
+* the plugins you use.
+
+For a summary of changes made in earlier releases, see:
+
+* :doc:`whats-new-in-2.1`
+* :doc:`whats-new-in-2.2`
+
+
+.. vim: ft=rst

=== modified file 'doc/es/conf.py'
--- doc/es/conf.py	2009-09-09 15:51:18 +0000
+++ doc/es/conf.py	2010-07-07 15:03:14 +0000
@@ -13,7 +13,7 @@
 sys.path = [os.path.abspath('../..')] + sys.path
 
 # Most of the configuration for Bazaar docs is defined here ...
-from bzrlib.doc_generate.sphinx_conf import *
+from bzrlib.doc_generate.conf import *
 
 
 ## Configuration specific to this site ##

=== modified file 'doc/es/index.txt'
--- doc/es/index.txt	2009-12-02 20:34:07 +0000
+++ doc/es/index.txt	2010-08-13 19:08:57 +0000
@@ -3,7 +3,7 @@
 =================================
 
 Las ultimas versiones de estos documentos se encuentran disponibles
-en la pagina de  Bazaar, http://doc.bazaar-vcs.org/en/.
+en la pagina de  Bazaar, http://doc.bazaar.canonical.com/en/.
 
 Documentacion Principal
 =======================
@@ -21,20 +21,20 @@
 
 Mudandose a Bazaar (enlances web):
 
-* `Guias de Migracion <http://doc.bazaar-vcs.org/migration/en/>`_
+* `Guias de Migracion <http://doc.bazaar.canonical.com/migration/en/>`_
   - para equipos que mudan el historial de otros SCV
 
-* `Guias de Plugins <http://doc.bazaar-vcs.org/plugins/en/>`_
+* `Guias de Plugins <http://doc.bazaar.canonical.com/plugins/en/>`_
 
 Otros Documentos (enlances web):
 
-* `Glosario <http://bazaar-vcs.org/BzrGlossary>`_
+* `Glosario <http://wiki.bazaar.canonical.com/BzrGlossary>`_
 
 * `Preguntas Frecuentes <https://answers.launchpad.net/bzr>`_
 
-* `Referencia del API bzrlib <http://bazaar-vcs.org/BzrLib>`_
+* `Referencia del API bzrlib <http://wiki.bazaar.canonical.com/BzrLib>`_
 
 Especificaciones tecnicas, planes a futuro y varias
 notas tecnicas varias en Ingles:
 
-* `Catalogo de Documentos para Desarrolladores <http://doc.bazaar-vcs.org/latest/developers/>`_
+* `Catalogo de Documentos para Desarrolladores <http://doc.bazaar.canonical.com/latest/developers/>`_

=== modified file 'doc/es/mini-tutorial/index.txt'
--- doc/es/mini-tutorial/index.txt	2009-12-02 20:34:07 +0000
+++ doc/es/mini-tutorial/index.txt	2010-08-13 19:08:57 +0000
@@ -30,10 +30,10 @@
 Para otras plataformas y para instalar desde el código fuente, vea las
 páginas de Descarga_ e Instalación_.
 
-.. _instrucciones de instalación para Windows: http://bazaar-vcs.org/WindowsDownloads
-.. _instrucciones de instalación para Mac OS X: http://bazaar-vcs.org/MacOSXBundle
-.. _Descarga: http://bazaar-vcs.org/Download
-.. _Instalación: http://bazaar-vcs.org/InstallationFaq
+.. _instrucciones de instalación para Windows: http://wiki.bazaar.canonical.com/WindowsDownloads
+.. _instrucciones de instalación para Mac OS X: http://wiki.bazaar.canonical.com/MacOSXBundle
+.. _Descarga: http://wiki.bazaar.canonical.com/Download
+.. _Instalación: http://wiki.bazaar.canonical.com/InstallationFaq
 
 
 Preséntese
@@ -77,7 +77,7 @@
 Si parece que no ha ocurrido nada no se preocupe. Bazaar ha creado un
 branch_ dónde guardará sus archivos y su histórico de revisiones.
 
-.. _branch: http://bazaar-vcs.org/Branch
+.. _branch: http://wiki.bazaar.canonical.com/Branch
 
 El siguiente paso es decirle a Bazaar a que archivos desea seguirles la pista.
 Ejecutando ``bzr add`` agregará recursivamente todos los elementos dentro del
@@ -167,7 +167,7 @@
  $ bzr branch http://www.example.com/miproyecto
 
 **Nota:** para utilizar sftp deberá instalar ``paramiko`` y
-``pyCrypto``. Vea http://bazaar-vcs.org/InstallationFaq para más información.
+``pyCrypto``. Vea http://wiki.bazaar.canonical.com/InstallationFaq para más información.
 
 
 Publicando su branch con Launchpad

=== modified file 'doc/index.es.txt'
--- doc/index.es.txt	2010-02-23 03:06:44 +0000
+++ doc/index.es.txt	2010-08-13 19:08:57 +0000
@@ -26,19 +26,19 @@
 
 Mudandose a Bazaar (enlances web):
 
-* `Guias de Mudanzas <http://bazaar-vcs.org/BzrSwitching>`_
+* `Guias de Mudanzas <http://wiki.bazaar.canonical.com/BzrSwitching>`_
   - para los usuarios que se mudan de otros SCV
 
-* `Guias de Migracion <http://bazaar-vcs.org/BzrMigration>`_
+* `Guias de Migracion <http://wiki.bazaar.canonical.com/BzrMigration>`_
   - para equipos que mudan el historial de otros SCV
 
 Otros Documentos (enlances web):
 
-* `Glosario <http://bazaar-vcs.org/BzrGlossary>`_
-
-* `Preguntas Frecuentes <http://bazaar-vcs.org/FAQ>`_
-
-* `Referencia del API bzrlib <http://bazaar-vcs.org/BzrLib>`_
+* `Glosario <http://wiki.bazaar.canonical.com/BzrGlossary>`_
+
+* `Preguntas Frecuentes <http://wiki.bazaar.canonical.com/FAQ>`_
+
+* `Referencia del API bzrlib <http://wiki.bazaar.canonical.com/BzrLib>`_
 
 Especificaciones tecnicas, planes a futuro y varias
 notas tecnicas varias en Ingles:

=== modified file 'doc/index.ja.txt'
--- doc/index.ja.txt	2010-02-23 03:06:44 +0000
+++ doc/index.ja.txt	2010-08-17 06:45:33 +0000
@@ -3,7 +3,7 @@
 ===========================
 
 これらのドキュメントの最新版はBazaarのドキュメントのサイト、 http://doc.bazaar.canonical.com/ から入手可能で、
-詳しい情報は http://bazaar-vcs.org/Documentation のページからリンクされています。
+詳しい情報は http://wiki.bazaar.canonical.com/Documentation のページからリンクされています。
 
 
 コアドキュメント
@@ -42,15 +42,15 @@
 ウェブリンク
 =============
 
-* `切り替えガイド <http://bazaar-vcs.org/BzrSwitching>`_
+* `切り替えガイド <http://wiki.bazaar.canonical.com/BzrSwitching>`_
   |--| 別のVCSツールから移ってきたユーザー用
 
-* `移行ガイド <http://bazaar-vcs.org/BzrMigration>`_
+* `移行ガイド <http://wiki.bazaar.canonical.com/BzrMigration>`_
   |--| 別のVCSツールから履歴を移行するチーム用
 
-* `用語 <http://bazaar-vcs.org/BzrGlossary>`_
+* `用語 <http://wiki.bazaar.canonical.com/BzrGlossary>`_
 
-* `よく聞かれる質問 <http://bazaar-vcs.org/FAQ>`_
+* `よく聞かれる質問 <http://wiki.bazaar.canonical.com/FAQ>`_
 
 
 TortoiseBzrをインストールするには？

=== modified file 'doc/index.ru.txt'
--- doc/index.ru.txt	2010-02-23 03:06:44 +0000
+++ doc/index.ru.txt	2010-08-17 06:45:33 +0000
@@ -4,7 +4,7 @@
 
 Последняя версия этих документов доступа со страницы документации
 на сайте Bazaar, <http://doc.bazaar.canonical.com/>. Еще больше информации
-можно получить по адресу <http://bazaar-vcs.org/Documentation>.
+можно получить по адресу <http://wiki.bazaar.canonical.com/Documentation>.
 
 Основная документация
 =====================
@@ -38,18 +38,18 @@
 Ссылки в сети
 =============
 
-* `Руководства по переходу <http://bazaar-vcs.org/BzrSwitching>`_
+* `Руководства по переходу <http://wiki.bazaar.canonical.com/BzrSwitching>`_
   |--| для пользователей переходящих с других систем контроля версий
 
-* `Руководство по миграции <http://bazaar-vcs.org/BzrMigration>`_
+* `Руководство по миграции <http://wiki.bazaar.canonical.com/BzrMigration>`_
   |--| для команд переносящих историю с других систем контроля версий
 
-* `Словарь терминов <http://bazaar-vcs.org/BzrGlossary>`_ (англ.),
+* `Словарь терминов <http://wiki.bazaar.canonical.com/BzrGlossary>`_ (англ.),
   см. также `русскую версию`__
 
 __  http://groups.google.com/group/ru_bzr/web/%D0%B3%D0%BB%D0%BE%D1%81%D1%81%D0%B0%D1%80%D0%B8%D0%B9
 
-* `Часто задаваемые вопросы <http://bazaar-vcs.org/FAQ>`_ (англ.)
+* `Часто задаваемые вопросы <http://wiki.bazaar.canonical.com/FAQ>`_ (англ.)
 
 Другие языки
 ============

=== modified file 'doc/index.txt'
--- doc/index.txt	2010-02-23 03:06:44 +0000
+++ doc/index.txt	2010-08-17 06:45:33 +0000
@@ -4,7 +4,7 @@
 
 The latest version of these documents are available from Bazaar's
 documentation site, <http://doc.bazaar.canonical.com/>, and more documentation
-may be linked from <http://bazaar-vcs.org/Documentation>.
+may be linked from <http://wiki.bazaar.canonical.com/Documentation>.
 
 Core Documentation
 ==================
@@ -43,15 +43,15 @@
 Web links
 =========
 
-* `Switching Guides <http://bazaar-vcs.org/BzrSwitching>`_
+* `Switching Guides <http://wiki.bazaar.canonical.com/BzrSwitching>`_
   |--| for users moving from another VCS tool
 
-* `Migration Guide <http://bazaar-vcs.org/BzrMigration>`_
+* `Migration Guide <http://wiki.bazaar.canonical.com/BzrMigration>`_
   |--| for teams migrating history from another VCS tool
 
-* `Glossary <http://bazaar-vcs.org/BzrGlossary>`_
+* `Glossary <http://wiki.bazaar.canonical.com/BzrGlossary>`_
 
-* `Frequently Asked Questions <http://bazaar-vcs.org/FAQ>`_
+* `Frequently Asked Questions <http://wiki.bazaar.canonical.com/FAQ>`_
 
 
 Other Languages

=== modified file 'doc/ja/conf.py'
--- doc/ja/conf.py	2009-11-18 06:18:14 +0000
+++ doc/ja/conf.py	2010-07-07 15:03:14 +0000
@@ -13,7 +13,7 @@
 sys.path = [os.path.abspath('../..')] + sys.path
 
 # Most of the configuration for Bazaar docs is defined here ...
-from bzrlib.doc_generate.sphinx_conf import *
+from bzrlib.doc_generate.conf import *
 
 
 ## Configuration specific to this site ##

=== modified file 'doc/ja/index.txt'
--- doc/ja/index.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/index.txt	2010-08-17 06:45:33 +0000
@@ -2,8 +2,8 @@
 Bazaarのメインドキュメント
 ===========================
 
-これらのドキュメントの最新版はBazaarのドキュメントのサイト、 http://doc.bazaar-vcs.org/ から入手可能で、
-詳しい情報は http://bazaar-vcs.org/Documentation のページからリンクされています。
+これらのドキュメントの最新版はBazaarのドキュメントのサイト、 http://doc.bazaar.canonical.com/ から入手可能で、
+詳しい情報は http://wiki.bazaar.canonical.com/Documentation のページからリンクされています。
 
 
 目次
@@ -20,15 +20,15 @@
 ウェブリンク
 =============
 
-* `切り替えガイド <http://bazaar-vcs.org/BzrSwitching>`_
+* `切り替えガイド <http://wiki.bazaar.canonical.com/BzrSwitching>`_
   |--| 別のVCSツールから移ってきたユーザー用
 
-* `移行ガイド <http://bazaar-vcs.org/BzrMigration>`_
+* `移行ガイド <http://wiki.bazaar.canonical.com/BzrMigration>`_
   |--| 別のVCSツールから履歴を移行するチーム用
 
-* `用語 <http://bazaar-vcs.org/BzrGlossary>`_
+* `用語 <http://wiki.bazaar.canonical.com/BzrGlossary>`_
 
-* `よく聞かれる質問 <http://bazaar-vcs.org/FAQ>`_
+* `よく聞かれる質問 <http://wiki.bazaar.canonical.com/FAQ>`_
 
 
 TortoiseBzrをインストールするには？

=== modified file 'doc/ja/mini-tutorial/index.txt'
--- doc/ja/mini-tutorial/index.txt	2009-10-29 16:52:18 +0000
+++ doc/ja/mini-tutorial/index.txt	2010-08-13 19:08:57 +0000
@@ -29,10 +29,10 @@
 別のプラットフォームとソースコードからインストールする方法に関しては、 ダウンロード_
 と インストール方法_ のページを参照してください。
 
-.. _Windowsのためのインストールの手引き: http://bazaar-vcs.org/WindowsDownloads
-.. _Mac OS Xのためのインストールの手引き: http://bazaar-vcs.org/MacOSXBundle
-.. _ダウンロード: http://bazaar-vcs.org/Download
-.. _インストール方法: http://bazaar-vcs.org/InstallationFaq
+.. _Windowsのためのインストールの手引き: http://wiki.bazaar.canonical.com/WindowsDownloads
+.. _Mac OS Xのためのインストールの手引き: http://wiki.bazaar.canonical.com/MacOSXBundle
+.. _ダウンロード: http://wiki.bazaar.canonical.com/Download
+.. _インストール方法: http://wiki.bazaar.canonical.com/InstallationFaq
 
 
 まずは自己紹介
@@ -74,7 +74,7 @@
 何も起きていないように見えても心配しないでください。
 Bazaarはファイルとリビジョンの履歴を保存する branch_ を作りました。
 
-.. _branch: http://bazaar-vcs.org/Branch
+.. _branch: http://wiki.bazaar.canonical.com/Branch
 
 次のステップはBazaarに管理して欲しいファイルを教えることです。
 ``bzr add`` を実行するとすべてのディレクトリとファイルがプロジェクトに\
@@ -166,7 +166,7 @@
  $ bzr branch http://www.example.com/myproject
 
 **注:** sftpを使うためには、 ``paramiko`` と ``pyCrypto`` をインストールする必要があります。
-詳細は http://bazaar-vcs.org/InstallationFaq を参照してください。
+詳細は http://wiki.bazaar.canonical.com/InstallationFaq を参照してください。
 
 
 Launchpadでブランチを公開する

=== modified file 'doc/ja/tutorials/centralized_workflow.txt'
--- doc/ja/tutorials/centralized_workflow.txt	2010-02-17 07:15:12 +0000
+++ doc/ja/tutorials/centralized_workflow.txt	2010-08-13 19:08:57 +0000
@@ -18,7 +18,7 @@
 お互いの作業の同期をとることになります。 
 また、このワークフローは、一人の開発者が複数のマシーン上で作業をする場合にも適用できます。
 
-.. _Bazaar: http://bazaar-vcs.org
+.. _Bazaar: http://bazaar.canonical.com
 
 
 初期セットアップ
@@ -86,7 +86,7 @@
 しかしながら、それについてはここでは説明しません。詳しくは、 `Tracking Upstream`_ の\
 "Converting and keeping history"セクションを見てください。
 
-.. _Tracking Upstream: http://bazaar-vcs.org/TrackingUpstream
+.. _Tracking Upstream: http://wiki.bazaar.canonical.com/TrackingUpstream
 
 .. 
    XXX: プロジェクトの変換について話をするための別の文書が必要なのですが、\
@@ -203,7 +203,7 @@
 
    % bzr cbranch dev my-feature-branch
 
-.. _bzrtools: http://bazaar-vcs.org/BzrTools
+.. _bzrtools: http://wiki.bazaar.canonical.com/BzrTools
 
 
 変更内容をマージする

=== modified file 'doc/ja/tutorials/tutorial.txt'
--- doc/ja/tutorials/tutorial.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/tutorials/tutorial.txt	2010-08-13 19:08:57 +0000
@@ -239,8 +239,8 @@
 ブランチを格納しているディレクトリ名で指定できます。 bzr は http や ftp\
 経由でブランチにアクセスすることもできます。例::
 
-    % bzr log http://bazaar-vcs.org/bzr/bzr.dev/
-    % bzr log sftp://bazaar-vcs.org/bzr/bzr.dev/
+    % bzr log http://bazaar.launchpad.net/~bzr-pqm/bzr/bzr.dev/
+    % bzr log sftp://bazaar.launchpad.net/~bzr-pqm/bzr/bzr.dev/
 
 プラグインをインストールすれば、 rsync プロトコルを使ってブランチにアクセス\
 することもできます。
@@ -655,7 +655,7 @@
 この場合、既存のブランチのコピーを取得する必要があります。
 このコピーは新しいブランチになるので、このコマンドは **branch** という名前になっています::
 
-    % bzr branch http://bazaar-vcs.org/bzr/bzr.dev
+    % bzr branch lp:bzr bzr.dev
     % cd bzr.dev
 
 .. This copies down the complete history of this branch, so we can do all

=== modified file 'doc/ja/tutorials/using_bazaar_with_launchpad.txt'
--- doc/ja/tutorials/using_bazaar_with_launchpad.txt	2009-12-02 20:34:07 +0000
+++ doc/ja/tutorials/using_bazaar_with_launchpad.txt	2010-08-13 19:08:57 +0000
@@ -176,7 +176,7 @@
   提供します。
 
 .. As well as sponsoring `Ubuntu <http://www.ubuntu.com>`_ and
-   `Bazaar <http://bazaar-vcs.org>`_ development, Canonical
+   `Bazaar <http://bazaar.canonical.com>`_ development, Canonical
    provides Launchpad, https://launchpad.net, as a free service
    for the open source community. Launchpad is one of the most
    exciting CDEs around for several notable reasons:

=== modified file 'doc/ja/upgrade-guide/overview.txt'
--- doc/ja/upgrade-guide/overview.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/upgrade-guide/overview.txt	2010-08-13 19:08:57 +0000
@@ -52,7 +52,7 @@
 
 2. 最新のバージョンに更新する。 **sudo port upgrade bzr**
 
-インストールや移行に関する詳しい情報については http://bazaar-vcs.org/Download
+インストールや移行に関する詳しい情報については http://wiki.bazaar.canonical.com/Download
 をごらんください。	
 
 必要なプラグインの移行

=== modified file 'doc/ja/user-guide/bzrtools_plugin.txt'
--- doc/ja/user-guide/bzrtools_plugin.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/bzrtools_plugin.txt	2010-08-13 19:08:57 +0000
@@ -6,7 +6,7 @@
 
 BzrToolsは便利なBazaar強化機能のコレクションです。
 インストールの手引きに関しては、BzrToolsのホームページを参照してください:
-http://bazaar-vcs.org/BzrTools.
+http://wiki.bazaar.canonical.com/BzrTools.
 よく使われるコマンドのサンプルは下記のとおりです。
 
 

=== modified file 'doc/ja/user-guide/installing_bazaar.txt'
--- doc/ja/user-guide/installing_bazaar.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/installing_bazaar.txt	2010-08-13 19:08:57 +0000
@@ -6,14 +6,14 @@
 
 BazaarのパッケージはUbuntu/Debian、Red HatとGentooを含む人気のある
 大抵のLinuxディストリビューションで利用できます。
-最新の手引きは http://bazaar-vcs.org/Download を参照してください。
+最新の手引きは http://wiki.bazaar.canonical.com/Download を参照してください。
 
 Windows
 -------
 
 Windowsユーザーは、Bazaarのコアパッケージと一緒に必要な前提要件と
 いくつかの便利なプラグインを含むインストーラが利用できます。
-最新の手引きは http://bazaar-vcs.org/Download を参照してください。
+最新の手引きは http://wiki.bazaar.canonical.com/Download を参照してください。
 
 注: Windows上でCygwinを動かしている場合、Cygwin用のBazaarパッケージを\
 Windows版の代わりに使わなければなりません。
@@ -23,7 +23,7 @@
 
 BazaarのパッケージはLinuxとWindowsだけでなく、Mac OS X, FreeBSD, Solaris\
 を含む広い範囲のオペレーティングシステムで利用可能です。
-最新の手引きに関しては http://bazaar-vcs.org/Download を参照してください。
+最新の手引きに関しては http://wiki.bazaar.canonical.com/Download を参照してください。
 
 
 ゼロからインストールする
@@ -34,7 +34,7 @@
 
   1. まだインストールしていなければ、バージョン2.4以降のPythonをインストールします。
 
-  2. http://bazaar-vcs.org/Download もしくはLaunchpad (https://launchpad.net/~bzr/)から
+  2. http://wiki.bazaar.canonical.com/Download もしくはLaunchpad (https://launchpad.net/~bzr/)から
      ``bazaar-xxx.tar.gz`` ファイル(xxxはバージョン番号)をダウンロードします。
 
   3. tar、WinZipもしくは同等のコマンドを用いてアーカイブを解凍します。
@@ -80,7 +80,7 @@
 
   2. 次のように開発バージョンのコピーを入手します::
 
-       bzr branch http://bazaar-vcs.org/bzr/bzr.dev
+       bzr branch lp:bzr
 
   3. 作成されたディレクトリ(bzr.dev)をPATHに登録します
 

=== modified file 'doc/ja/user-guide/introducing_bazaar.txt'
--- doc/ja/user-guide/introducing_bazaar.txt	2009-12-02 20:34:07 +0000
+++ doc/ja/user-guide/introducing_bazaar.txt	2010-08-13 19:08:57 +0000
@@ -71,7 +71,7 @@
 
  * 会社において、分散されたチームや外部のチームと連携するのが簡単になります
 
-分散型VCSツールの集中型より優れた点の詳細な一覧に関しては、 http://bazaar-vcs.org/BzrWhy を参照してください。
+分散型VCSツールの集中型より優れた点の詳細な一覧に関しては、 http://wiki.bazaar.canonical.com/BzrWhy を参照してください。
 
 
 Bazaarの主要な機能
@@ -79,12 +79,12 @@
 
 Bazaarは現存する唯一のVCSツールではありませんが、多くのチームとコミュニティにとって\
 よい選択肢になる優れた機能をいくつか持ちます。
-別のVCSツールとの比較の要約はBazaarのWiki、 http://bazaar-vcs.org/ で見つかります。
+別のVCSツールとの比較の要約はBazaarのWiki、 http://wiki.bazaar.canonical.com/ で見つかります。
 
 多くの機能の中で、とりわけ強調するものがあります:
 BazaarはPythonで書かれた完全にフリーなソフトウェアです。
 結果として、改善に貢献することは簡単です。
-手を貸して頂けるのであれば、 http://bazaar-vcs.org/BzrSupport を参照して頂くようお願いします。
+手を貸して頂けるのであれば、 http://wiki.bazaar.canonical.com/BzrSupport を参照して頂くようお願いします。
 
 
 さらに学ぶ

=== modified file 'doc/ja/user-guide/plugins.txt'
--- doc/ja/user-guide/plugins.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/plugins.txt	2010-08-13 19:08:57 +0000
@@ -26,7 +26,7 @@
 プラグインが見つかる場所
 -------------------------
 
-http://bazaar-vcs.org/BzrPlugins ページでプラグインのリストが見つかります。
+http://wiki.bazaar.canonical.com/BzrPlugins ページでプラグインのリストが見つかります。
 
 プラグインをインストールする方法
 ---------------------------------

=== modified file 'doc/ja/user-guide/server.txt'
--- doc/ja/user-guide/server.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/server.txt	2010-08-13 19:08:57 +0000
@@ -15,7 +15,7 @@
 Bazaarはリモートからの読み込みを許可します。
 実行しているBazaarコマンドの中でブランチへのURLを入力するだけです。::
 
-    bzr log http://bazaar-vcs.org/bzr/bzr.dev
+    bzr log http://bazaar.launchpad.net/~bzr-pqm/bzr/bzr.dev
 
 BazaarはFTP、SFTPと(プラグインを通した)HTTP-WebDAVを通した書き込みをサポートします。
 

=== modified file 'doc/ja/user-guide/shared_repository_layouts.txt'
--- doc/ja/user-guide/shared_repository_layouts.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/shared_repository_layouts.txt	2010-08-13 19:08:57 +0000
@@ -121,7 +121,7 @@
     メタプロジェクトはリポジトリのレイアウトにかかわらず複数のプロジェクトを集約します。
     1つのプロジェクトを ``bzr checkout`` すれば、必要なサブプロジェクトがすべて手に入ります。
 
-.. _NestedTreeSupport: http://bazaar-vcs.org/NestedTrees
+.. _NestedTreeSupport: http://wiki.bazaar.canonical.com/NestedTrees
 
 
 入れ子形式 (``project/branch/sub-branch/``)

=== modified file 'doc/ja/user-guide/svn_plugin.txt'
--- doc/ja/user-guide/svn_plugin.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/svn_plugin.txt	2010-08-13 19:08:57 +0000
@@ -18,7 +18,7 @@
 利用からまだ採用していないチームのための共通の暫定ステップです
 
 インストールの手引きに関しては、bzr-svnのホームページをご覧ください:
-http://bazaar-vcs.org/BzrForeignBranches/Subversion.
+http://wiki.bazaar.canonical.com/BzrForeignBranches/Subversion.
 
 
 シンプルな例
@@ -109,4 +109,4 @@
  * Bazaarはファイルのコピーのトラッキングをサポートしません
 
 現在の制約の一覧に関しては、bzr-svnのウェブページ、\
-http://bazaar-vcs.org/BzrForeignBranches/Subversion を参照してください。
+http://wiki.bazaar.canonical.com/BzrForeignBranches/Subversion を参照してください。

=== modified file 'doc/ja/user-guide/web_browsing.txt'
--- doc/ja/user-guide/web_browsing.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/web_browsing.txt	2010-08-13 19:08:57 +0000
@@ -9,7 +9,7 @@
 Loggerhead のホームページは https://launchpad.net/loggerhead にあります。
 
 ダウンロードリンクを含めてこれらのパッケージの最新情報に関しては
-http://bazaar-vcs.org/WebInterface を参照してください。
+http://wiki.bazaar.canonical.com/WebInterface を参照してください。
 
 注: プロジェクトがホストされているもしくはLaunchpadでミラーリングされている場合、
 Loggerheadのコードブラウジング機能はサービスの一部として提供されます。

=== modified file 'doc/ja/user-reference/index.txt'
--- doc/ja/user-reference/index.txt	2009-12-02 20:34:07 +0000
+++ doc/ja/user-reference/index.txt	2010-09-25 00:42:22 +0000
@@ -40,8 +40,8 @@
 
 次のウェブサイトはBazaarに関する詳細な情報を提供します:
 
-:ホームページ:                     http://www.bazaar-vcs.org/
-:公式ドキュメント:                 http://doc.bazaar-vcs.org/
+:ホームページ:                     http://bazaar.canonical.com/
+:公式ドキュメント:                 http://doc.bazaar.canonical.com/
 :Launchpad:                     https://launchpad.net/bzr/
 
 概念
@@ -614,7 +614,7 @@
 存在するのであれば、Bugzillaのバグトラッカーの位置は <tracker_abbreviation> によって参照されます。
 そのバグトラッカーのバグをそのコミットで修正されたものとしてマークするためにはこのオプションは ``bzr commit --fixes`` と一緒に使用できます::
 
-    bugzilla_squid_url = http://www.squid-cache.org/bugs
+    bugzilla_squid_url = http://bugs.squid-cache.org
 
 上記の例はSquidのバグ 1234が修正されたものとしてマークするために ``bzr commit --fixes squid:1234`` を許可します。
 
@@ -838,7 +838,7 @@
     create_signatures = always
     check_signatures  = always
 
-    [http://bazaar-vcs.org/bzr/*]
+    [http://bazaar.launchpad.net/*/bzr/*]
     check_signatures  = require
 
 認証用の設定ファイル、authentication.conf
@@ -1223,7 +1223,7 @@
     dirstate-tagsフォーマットリポジトリと互換性のあるデータを持つパックベースのフォーマット。
     0.92以前のbzrリポジトリと相互運用できますが0.92以前のbzrでは読めません。
     以前はknitpack-experimentalと呼ばれていました。詳細な情報に関しては、
-    http://doc.bazaar-vcs.org/latest/developers/packrepo.html を参照。
+    http://doc.bazaar.canonical.com/latest/developers/packrepo.html を参照。
 
 :1.6:
     (ネイティブ) スタックをサポートするディレクトリに基づいたブランチとパック。
@@ -1596,7 +1596,7 @@
     (ネイティブ) 現在の開発フォーマット。
     pack-0.92 (とpack-0.92と互換性のある)フォーマットリポジトリにデータを変換できます。
     このフォーマットのリポジトリとブランチはbzr.devによってのみ読み込みできます。
-    使用する前に http://doc.bazaar-vcs.org/latest/developers/development-repo.html
+    使用する前に http://doc.bazaar.canonical.com/latest/developers/development-repo.html
     をご覧下さるようお願いします。
 
 
@@ -1605,7 +1605,7 @@
     pack-0.92-subtree (pack-0.92-subtreeと互換性のあるものなら何でも)フォーマットリポジトリ
     から/へのデータの変換ができる。このフォーマットのリポジトリとブランチは
     bzr.devによってのみ読み込みできます。
-    使用する前に http://doc.bazaar-vcs.org/latest/developers/development-repo.html
+    使用する前に http://doc.bazaar.canonical.com/latest/developers/development-repo.html
     をご覧下さるようお願いします。
 
 
@@ -2500,17 +2500,17 @@
                         0.92以前のbzrリポジトリと相互運用しますが
                         bzr < 0.92では読むことができません。
                         以前はknitpack-experimentalと呼ばれていました。
-                        詳細な情報は http://doc.bazaar-vcs.org/latest/developers/packrepo.html を参照。
+                        詳細な情報は http://doc.bazaar.canonical.com/latest/developers/packrepo.html を参照。
     --development       現在の開発フォーマット。データをpack-0.92 (とpack-0.92と互換性のある)
                         フォーマットリポジトリに変換できる。
                         このフォーマットのリポジトリとブランチはbzr.devによってのみ読み込みできます。
-                        使用する前に http://doc.bazaar-vcs.org/latest/developers/development-repo.html を参照して頂くようお願いします。
+                        使用する前に http://doc.bazaar.canonical.com/latest/developers/development-repo.html を参照して頂くようお願いします。
     --development-subtree
                         現在の開発フォーマットで、subtreeバリアント。
                         データをpack-0.92-subtree(とpack-0.92-subtreeと互換性のある)
                         フォーマットリポジトリに変換できる。
                         このフォーマットのリポジトリとブランチはbzr.devでのみ読み込みできる。
-                        使用する前に http://doc.bazaar-vcs.org/latest/developers/development-
+                        使用する前に http://doc.bazaar.canonical.com/latest/developers/development-
                         repo.html をご覧いただくようお願いします。
     --dirstate          0.15の新しいフォーマット: 速いローカルオペレーション。
                         ネットワークを通したアクセスのときbzr 0.8とそれ以降と互換性がある。
@@ -2523,8 +2523,7 @@
                         互換性のあるデータを持つパックベースのフォーマット。
                         0.92以前のbzrリポジトリと相互運用できるがbzr < 0.92.によって読み込みできない。
                         以前はknitpack-experimentalと呼ばれていた。
-                        詳細な情報に関しては、 http://doc
-                        .bazaar-vcs.org/latest/developers/packrepo.html を参照。
+                        詳細な情報に関しては、 http://doc.bazaar.canonical.com/latest/developers/packrepo.html を参照。
     --rich-root         1.0の新しいフォーマット。ツリーrootのベターな扱い。
                         bzr < 1.0と互換性がない。
     --rich-root-pack    1.0の新しいフォーマット: rich-rootデータをサポートする
@@ -2580,17 +2579,17 @@
                         0.92以前のbzrリポジトリと相互運用しますが
                         bzr < 0.92では読むことができません。
                         以前はknitpack-experimentalと呼ばれていました。
-                        詳細な情報は http://doc.bazaar-vcs.org/latest/developers/packrepo.html を参照。
+                        詳細な情報は http://doc.bazaar.canonical.com/latest/developers/packrepo.html を参照。
     --development       現在の開発フォーマット。データをpack-0.92 (とpack-0.92と互換性のある)
                         フォーマットリポジトリに変換できる。
                         このフォーマットのリポジトリとブランチはbzr.devによってのみ読み込みできます。
-                        使用する前に http://doc.bazaar-vcs.org/latest/developers/development-repo.html を参照して頂くようお願いします。
+                        使用する前に http://doc.bazaar.canonical.com/latest/developers/development-repo.html を参照して頂くようお願いします。
     --development-subtree
                         現在の開発フォーマットで、subtreeバリアント。
                         データをpack-0.92-subtree(とpack-0.92-subtreeと互換性のある)
                         フォーマットリポジトリに変換できる。
                         このフォーマットのリポジトリとブランチはbzr.devでのみ読み込みできる。
-                        使用する前に http://doc.bazaar-vcs.org/latest/developers/development-
+                        使用する前に http://doc.bazaar.canonical.com/latest/developers/development-
                         repo.html をご覧いただくようお願いします。
     --dirstate          0.15の新しいフォーマット: 速いローカルオペレーション。
                         ネットワークを通したアクセスのときbzr 0.8とそれ以降と互換性がある。
@@ -2603,8 +2602,7 @@
                         互換性のあるデータを持つパックベースのフォーマット。
                         0.92以前のbzrリポジトリと相互運用できるがbzr < 0.92.によって読み込みできない。
                         以前はknitpack-experimentalと呼ばれていた。
-                        詳細な情報に関しては、 http://doc
-                        .bazaar-vcs.org/latest/developers/packrepo.html を参照。
+                        詳細な情報に関しては、 http://doc.bazaar.canonical.com/latest/developers/packrepo.html を参照。
     --rich-root         1.0の新しいフォーマット。ツリーrootのベターな扱い。
                         bzr < 1.0と互換性がない。
     --rich-root-pack    1.0の新しいフォーマット: rich-rootデータをサポートする
@@ -2924,7 +2922,7 @@
   プラグインはさまざまなことを行うことができます。
 
   プラグインを見つけてインストール方法を含めた詳細な情報に関しては、
-  Bazaarのウェブサイト、http://bazaar-vcs.org を参照してください。
+  Bazaarのウェブサイト、http://bazaar.canonical.com を参照してください。
   プログラミング言語のPyhonを利用して
   新しいコマンドを作成する方法に関する手引きもあります。
 
@@ -3731,17 +3729,17 @@
                         0.92以前のbzrリポジトリと相互運用しますが
                         bzr < 0.92では読むことができません。
                         以前はknitpack-experimentalと呼ばれていました。
-                        詳細な情報は http://doc.bazaar-vcs.org/latest/developers/packrepo.html を参照。
+                        詳細な情報は http://doc.bazaar.canonical.com/latest/developers/packrepo.html を参照。
     --development       現在の開発フォーマット。データをpack-0.92 (とpack-0.92と互換性のある)
                         フォーマットリポジトリに変換できる。
                         このフォーマットのリポジトリとブランチはbzr.devによってのみ読み込みできます。
-                        使用する前に http://doc.bazaar-vcs.org/latest/developers/development-repo.html を参照して頂くようお願いします。
+                        使用する前に http://doc.bazaar.canonical.com/latest/developers/development-repo.html を参照して頂くようお願いします。
     --development-subtree
                         現在の開発フォーマットで、subtreeバリアント。
                         データをpack-0.92-subtree(とpack-0.92-subtreeと互換性のある)
                         フォーマットリポジトリに変換できる。
                         このフォーマットのリポジトリとブランチはbzr.devでのみ読み込みできる。
-                        使用する前に http://doc.bazaar-vcs.org/latest/developers/development-
+                        使用する前に http://doc.bazaar.canonical.com/latest/developers/development-
                         repo.html をご覧いただくようお願いします。
     --dirstate          0.15の新しいフォーマット: 速いローカルオペレーション。
                         ネットワークを通したアクセスのときbzr 0.8とそれ以降と互換性がある。
@@ -3754,8 +3752,7 @@
                         互換性のあるデータを持つパックベースのフォーマット。
                         0.92以前のbzrリポジトリと相互運用できるがbzr < 0.92.によって読み込みできない。
                         以前はknitpack-experimentalと呼ばれていた。
-                        詳細な情報に関しては、 http://doc
-                        .bazaar-vcs.org/latest/developers/packrepo.html を参照。
+                        詳細な情報に関しては、 http://doc.bazaar.canonical.com/latest/developers/packrepo.html を参照。
     --rich-root         1.0の新しいフォーマット。ツリーrootのベターな扱い。
                         bzr < 1.0と互換性がない。
     --rich-root-pack    1.0の新しいフォーマット: rich-rootデータをサポートする

=== modified file 'doc/ru/conf.py'
--- doc/ru/conf.py	2009-09-09 15:51:18 +0000
+++ doc/ru/conf.py	2010-07-07 15:03:14 +0000
@@ -13,7 +13,7 @@
 sys.path = [os.path.abspath('../..')] + sys.path
 
 # Most of the configuration for Bazaar docs is defined here ...
-from bzrlib.doc_generate.sphinx_conf import *
+from bzrlib.doc_generate.conf import *
 
 
 ## Configuration specific to this site ##

=== modified file 'doc/ru/index.txt'
--- doc/ru/index.txt	2009-12-02 20:34:07 +0000
+++ doc/ru/index.txt	2010-08-17 06:45:33 +0000
@@ -3,7 +3,7 @@
 =================================
 
 Последняя версия этих документов доступа со страницы документации
-на сайте Bazaar, <http://doc.bazaar-vcs.org/ru/>.
+на сайте Bazaar, <http://doc.bazaar.canonical.com/ru/>.
 
 Основная документация
 =====================
@@ -22,10 +22,10 @@
 Ссылки в сети
 =============
 
-* `Руководство по миграции <http://doc.bazaar-vcs.org/migration/en/>`_
+* `Руководство по миграции <http://doc.bazaar.canonical.com/migration/en/>`_
   |--| для команд переносящих историю с других систем контроля версий
 
-* `Словарь терминов <http://bazaar-vcs.org/BzrGlossary>`_ (англ.),
+* `Словарь терминов <http://wiki.bazaar.canonical.com/BzrGlossary>`_ (англ.),
   см. также `русскую версию`__
 
 __  http://groups.google.com/group/ru_bzr/web/%D0%B3%D0%BB%D0%BE%D1%81%D1%81%D0%B0%D1%80%D0%B8%D0%B9

=== modified file 'doc/ru/mini-tutorial/index.txt'
--- doc/ru/mini-tutorial/index.txt	2009-12-02 20:34:07 +0000
+++ doc/ru/mini-tutorial/index.txt	2010-08-17 06:45:33 +0000
@@ -31,10 +31,10 @@
 Для других платформ и для установки из исходных кодов, обратитесь к страницам
 Загрузка_ и Установка_.
 
-.. _инструкции по установке для Windows: http://bazaar-vcs.org/WindowsDownloads
-.. _инструкции по установке для Mac OS X: http://bazaar-vcs.org/MacOSXBundle
-.. _Загрузка: http://bazaar-vcs.org/Download
-.. _Установка: http://bazaar-vcs.org/InstallationFaq
+.. _инструкции по установке для Windows: http://wiki.bazaar.canonical.com/WindowsDownloads
+.. _инструкции по установке для Mac OS X: http://wiki.bazaar.canonical.com/MacOSXBundle
+.. _Загрузка: http://wiki.bazaar.canonical.com/Download
+.. _Установка: http://wiki.bazaar.canonical.com/InstallationFaq
 
 Представьтесь
 =============
@@ -78,7 +78,7 @@
 Если всё выглядит так, как будто ничего не случилось |--| не волнуйтесь. Bazaar
 создал ветку_, в которой он будет хранить рабочие файлы и историю их изменений.
 
-.. _ветку: http://bazaar-vcs.org/Branch
+.. _ветку: http://wiki.bazaar.canonical.com/Branch
 
 Следующий шаг |--| сказать Bazaar какие файлы вы хотите контролировать. Команда
 ``bzr add`` рекурсивно добавит все файлы в проект::
@@ -172,7 +172,7 @@
 
 **Замечание:** чтобы использовать SFTP, может понадобиться установить
 ``paramiko`` и ``pyCrypto``. За подробностями обращайтесь к
-http://bazaar-vcs.org/InstallationFaq.
+http://wiki.bazaar.canonical.com/InstallationFaq.
 
 
 Публикация ветки на Launchpad

=== modified file 'doc/ru/tutorials/centralized_workflow.txt'
--- doc/ru/tutorials/centralized_workflow.txt	2010-02-17 07:15:12 +0000
+++ doc/ru/tutorials/centralized_workflow.txt	2010-08-13 19:08:57 +0000
@@ -24,7 +24,7 @@
 работу. Такой режим так же подходит для разработчика-одиночки,
 работающего на нескольких различных машинах.
 
-.. _Bazaar: http://bazaar-vcs.org
+.. _Bazaar: http://bazaar.canonical.com
 
 .. contents::
         Содержание
@@ -107,7 +107,7 @@
 `Отслеживание изменений на основной ветке`_ для некоторых способов (секция
 "Конвертирование и сохранение истории").
 
-.. _Отслеживание изменений на основной ветке: http://bazaar-vcs.org/TrackingUpstream
+.. _Отслеживание изменений на основной ветке: http://wiki.bazaar.canonical.com/TrackingUpstream
 
 ..
    TODO: На самом деле нам нужен другой документ для описания
@@ -238,7 +238,7 @@
 
    % bzr cbranch dev my-feature-branch
 
-.. _bzrtools: http://bazaar-vcs.org/BzrTools
+.. _bzrtools: http://wiki.bazaar.canonical.com/BzrTools
 
 
 Объединение изменений обратно

=== modified file 'doc/ru/tutorials/tutorial.txt'
--- doc/ru/tutorials/tutorial.txt	2009-12-02 20:34:07 +0000
+++ doc/ru/tutorials/tutorial.txt	2010-08-13 19:08:57 +0000
@@ -204,8 +204,8 @@
 содержащего ветку. bzr также поддерживает доступ к веткам через http и sftp,
 например::
 
-    % bzr log http://bazaar-vcs.org/bzr/bzr.dev/
-    % bzr log sftp://bazaar-vcs.org/bzr/bzr.dev/
+    % bzr log http://bazaar.launchpad.net/~bzr-pqm/bzr/bzr.dev/
+    % bzr log sftp://bazaar.launchpad.net/~bzr-pqm/bzr/bzr.dev/
 
 Установив для bzr плагины можно также осуществлять доступ к веткам с
 использованием rsync.
@@ -482,7 +482,7 @@
 готовой ветки. Так как эта копия может быть потенциальной новой веткой эта
 команда называется **branch**::
 
-    % bzr branch http://bazaar-vcs.org/bzr/bzr.dev
+    % bzr branch lp:bzr bzr.dev
     % cd bzr.dev
 
 Эта команда копирует полную историю ветки и после этого вы можете делать все

=== modified file 'doc/ru/tutorials/using_bazaar_with_launchpad.txt'
--- doc/ru/tutorials/using_bazaar_with_launchpad.txt	2009-12-02 20:34:07 +0000
+++ doc/ru/tutorials/using_bazaar_with_launchpad.txt	2010-08-17 06:45:33 +0000
@@ -93,7 +93,7 @@
 -------------------------------------------
 
 Кроме спонсорства разработки `Ubuntu <http://www.ubuntu.com>`_ и `Bazaar
-<http://bazaar-vcs.org>`_, Canonical так же предоставляет Launchpad,
+<http://bazaar.canonical.com>`_, Canonical так же предоставляет Launchpad,
 https://launchpad.net, как бесплатный сервис для сообществ с открытым исходным
 кодом. Launchpad является одной из самых интересных сред совместной разработки
 по следующим причинам:

=== modified file 'doc/ru/user-guide/introducing_bazaar.txt'
--- doc/ru/user-guide/introducing_bazaar.txt	2009-12-02 20:34:07 +0000
+++ doc/ru/user-guide/introducing_bazaar.txt	2010-08-13 19:08:57 +0000
@@ -94,7 +94,7 @@
  * для компаний - упрощение работы с распределенными и внешними командами.
 
 Для более детального взгляда на преимущества распределенных систем контроля
-версий по сравнению с централизованными смотрите http://bazaar-vcs.org/BzrWhy.
+версий по сравнению с централизованными смотрите http://wiki.bazaar.canonical.com/BzrWhy.
 
 
 Ключевые особенности Bazaar
@@ -104,12 +104,12 @@
 некоторые значимые преимущества, которые делают ее прекрасным выбором для
 многих команд и сообществ. Описание этих особенностей и сравнение с другими
 системами контроля версий может быть найдено на Wiki Bazaar -
-http://bazaar-vcs.org.
+http://wiki.bazaar.canonical.com.
 
 Из большинства особенностей, одна требует особого упоминания: Bazaar - это
 полностью свободное ПО написанное на языке Python. Это упрощает сотрудничество
 для внесения улучшений. Если вы хотите помочь, обратите внимание на
-http://bazaar-vcs.org/BzrSupport.
+http://wiki.bazaar.canonical.com/BzrSupport.
 
 
 Продолжаем изучение

=== modified file 'setup.py'
--- setup.py	2010-04-08 04:34:03 +0000
+++ setup.py	2010-08-02 19:56:52 +0000
@@ -9,6 +9,7 @@
 import os
 import os.path
 import sys
+import copy
 
 if sys.version_info < (2, 4):
     sys.stderr.write("[ERROR] Not a supported Python version. Need 2.4+\n")
@@ -125,9 +126,10 @@
                 f = file(batch_path, "w")
                 f.write(batch_str)
                 f.close()
-                print "Created:", batch_path
-            except Exception, e:
-                print "ERROR: Unable to create %s: %s" % (batch_path, e)
+                print("Created: %s" % batch_path)
+            except Exception:
+                e = sys.exc_info()[1]
+                print("ERROR: Unable to create %s: %s" % (batch_path, e))
 
     def _quoted_path(self, path):
         if ' ' in path:
@@ -171,18 +173,18 @@
         from Pyrex.Distutils import build_ext
         from Pyrex.Compiler.Version import version as pyrex_version
     except ImportError:
-        print "No Pyrex, trying Cython..."
+        print("No Pyrex, trying Cython...")
         from Cython.Distutils import build_ext
         from Cython.Compiler.Version import version as pyrex_version
 except ImportError:
     have_pyrex = False
     # try to build the extension from the prior generated source.
-    print
-    print ("The python package 'Pyrex' is not available."
-           " If the .c files are available,")
-    print ("they will be built,"
-           " but modifying the .pyx files will not rebuild them.")
-    print
+    print("")
+    print("The python package 'Pyrex' is not available."
+          " If the .c files are available,")
+    print("they will be built,"
+          " but modifying the .pyx files will not rebuild them.")
+    print("")
     from distutils.command.build_ext import build_ext
 else:
     have_pyrex = True
@@ -204,7 +206,8 @@
     def run(self):
         try:
             build_ext.run(self)
-        except DistutilsPlatformError, e:
+        except DistutilsPlatformError:
+            e = sys.exc_info()[1]
             if not self.allow_python_fallback:
                 log.warn('\n  Cannot build extensions.\n'
                          '  Use "build_ext --allow-python-fallback" to use'
@@ -281,7 +284,6 @@
     add_pyrex_extension('bzrlib._dirstate_helpers_pyx',
                         libraries=['Ws2_32'])
     add_pyrex_extension('bzrlib._walkdirs_win32')
-    z_lib = 'zdll'
 else:
     if have_pyrex and pyrex_version_info[:3] == (0,9,4):
         # Pyrex 0.9.4.1 fails to compile this extension correctly
@@ -290,25 +292,24 @@
         # which is NULL safe with PY_DECREF which is not.)
         # <https://bugs.edge.launchpad.net/bzr/+bug/449372>
         # <https://bugs.edge.launchpad.net/bzr/+bug/276868>
-        print 'Cannot build extension "bzrlib._dirstate_helpers_pyx" using'
-        print 'your version of pyrex "%s". Please upgrade your pyrex' % (
-            pyrex_version,)
-        print 'install. For now, the non-compiled (python) version will'
-        print 'be used instead.'
+        print('Cannot build extension "bzrlib._dirstate_helpers_pyx" using')
+        print('your version of pyrex "%s". Please upgrade your pyrex' % (
+            pyrex_version,))
+        print('install. For now, the non-compiled (python) version will')
+        print('be used instead.')
     else:
         add_pyrex_extension('bzrlib._dirstate_helpers_pyx')
     add_pyrex_extension('bzrlib._readdir_pyx')
-    z_lib = 'z'
-add_pyrex_extension('bzrlib._chk_map_pyx', libraries=[z_lib])
+add_pyrex_extension('bzrlib._chk_map_pyx')
 ext_modules.append(Extension('bzrlib._patiencediff_c',
                              ['bzrlib/_patiencediff_c.c']))
 if have_pyrex and pyrex_version_info < (0, 9, 6, 3):
-    print
-    print 'Your Pyrex/Cython version %s is too old to build the simple_set' % (
-        pyrex_version)
-    print 'and static_tuple extensions.'
-    print 'Please upgrade to at least Pyrex 0.9.6.3'
-    print
+    print("")
+    print('Your Pyrex/Cython version %s is too old to build the simple_set' % (
+        pyrex_version))
+    print('and static_tuple extensions.')
+    print('Please upgrade to at least Pyrex 0.9.6.3')
+    print("")
     # TODO: Should this be a fatal error?
 else:
     # We only need 0.9.6.3 to build _simple_set_pyx, but static_tuple depends
@@ -320,10 +321,10 @@
 
 
 if unavailable_files:
-    print 'C extension(s) not found:'
-    print '   %s' % ('\n  '.join(unavailable_files),)
-    print 'The python versions will be used instead.'
-    print
+    print('C extension(s) not found:')
+    print('   %s' % ('\n  '.join(unavailable_files),))
+    print('The python versions will be used instead.')
+    print("")
 
 
 def get_tbzr_py2exe_info(includes, excludes, packages, console_targets,
@@ -413,19 +414,12 @@
 
 def get_qbzr_py2exe_info(includes, excludes, packages, data_files):
     # PyQt4 itself still escapes the plugin detection code for some reason...
-    packages.append('PyQt4')
-    excludes.append('PyQt4.elementtree.ElementTree')
-    excludes.append('PyQt4.uic.port_v3')
+    includes.append('PyQt4.QtCore')
+    includes.append('PyQt4.QtGui')
     includes.append('sip') # extension module required for Qt.
     packages.append('pygments') # colorizer for qbzr
     packages.append('docutils') # html formatting
     includes.append('win32event')  # for qsubprocess stuff
-    # but we can avoid many Qt4 Dlls.
-    dll_excludes.extend(
-        """QtAssistantClient4.dll QtCLucene4.dll QtDesigner4.dll
-        QtHelp4.dll QtNetwork4.dll QtOpenGL4.dll QtScript4.dll
-        QtSql4.dll QtTest4.dll QtWebKit4.dll QtXml4.dll
-        qscintilla2.dll""".split())
     # the qt binaries might not be on PATH...
     # They seem to install to a place like C:\Python25\PyQt4\*
     # Which is not the same as C:\Python25\Lib\site-packages\PyQt4
@@ -534,17 +528,23 @@
             install_data.run(self)
 
             py2exe = self.distribution.get_command_obj('py2exe', False)
-            optimize = py2exe.optimize
+            # GZ 2010-04-19: Setup has py2exe.optimize as 2, but give plugins
+            #                time before living with docstring stripping
+            optimize = 1
             compile_names = [f for f in self.outfiles if f.endswith('.py')]
+            # Round mtime to nearest even second so that installing on a FAT
+            # filesystem bytecode internal and script timestamps will match
+            for f in compile_names:
+                mtime = os.stat(f).st_mtime
+                remainder = mtime % 2
+                if remainder:
+                    mtime -= remainder
+                    os.utime(f, (mtime, mtime))
             byte_compile(compile_names,
                          optimize=optimize,
                          force=self.force, prefix=self.install_dir,
                          dry_run=self.dry_run)
-            if optimize:
-                suffix = 'o'
-            else:
-                suffix = 'c'
-            self.outfiles.extend([f + suffix for f in compile_names])
+            self.outfiles.extend([f + 'o' for f in compile_names])
     # end of class install_data_with_bytecompile
 
     target = py2exe.build_exe.Target(script = "bzr",
@@ -558,6 +558,8 @@
                                      company_name = "Canonical Ltd.",
                                      comments = META_INFO['description'],
                                     )
+    gui_target = copy.copy(target)
+    gui_target.dest_base = "bzrw"
 
     packages = BZRLIB['packages']
     packages.remove('bzrlib')
@@ -648,7 +650,7 @@
     console_targets = [target,
                        'tools/win32/bzr_postinstall.py',
                        ]
-    gui_targets = []
+    gui_targets = [gui_target]
     data_files = topics_files + plugins_files
 
     if 'qbzr' in plugins:
@@ -681,8 +683,8 @@
         # print this warning to stderr as output is redirected, so it is seen
         # at build time.  Also to stdout so it appears in the log
         for f in (sys.stderr, sys.stdout):
-            print >> f, \
-                "Skipping TBZR binaries - please set TBZR to a directory to enable"
+            f.write("Skipping TBZR binaries - "
+                "please set TBZR to a directory to enable\n")
 
     # MSWSOCK.dll is a system-specific library, which py2exe accidentally pulls
     # in on Vista.
@@ -692,17 +694,30 @@
                                "excludes": excludes,
                                "dll_excludes": dll_excludes,
                                "dist_dir": "win32_bzr.exe",
-                               "optimize": 1,
+                               "optimize": 2,
+                               "custom_boot_script":
+                                        "tools/win32/py2exe_boot_common.py",
                               },
                    }
 
-    setup(options=options_list,
-          console=console_targets,
-          windows=gui_targets,
-          zipfile='lib/library.zip',
-          data_files=data_files,
-          cmdclass={'install_data': install_data_with_bytecompile},
-          )
+    # We want the libaray.zip to have optimize = 2, but the exe to have
+    # optimize = 1, so that .py files that get compilied at run time
+    # (e.g. user installed plugins) dont have their doc strings removed.
+    class py2exe_no_oo_exe(py2exe.build_exe.py2exe):
+        def build_executable(self, *args, **kwargs):
+            self.optimize = 1
+            py2exe.build_exe.py2exe.build_executable(self, *args, **kwargs)
+            self.optimize = 2
+
+    if __name__ == '__main__':
+        setup(options=options_list,
+              console=console_targets,
+              windows=gui_targets,
+              zipfile='lib/library.zip',
+              data_files=data_files,
+              cmdclass={'install_data': install_data_with_bytecompile,
+                        'py2exe': py2exe_no_oo_exe},
+              )
 
 else:
     # ad-hoc for easy_install
@@ -736,4 +751,5 @@
     ARGS.update(BZRLIB)
     ARGS.update(PKG_DATA)
 
-    setup(**ARGS)
+    if __name__ == '__main__':
+        setup(**ARGS)

=== modified file 'tools/check-newsbugs.py'
--- tools/check-newsbugs.py	2009-01-30 16:55:33 +0000
+++ tools/check-newsbugs.py	2010-09-20 12:05:38 +0000
@@ -4,10 +4,17 @@
 
 import getopt, re, sys
 try:
-    from launchpadbugs import connector
-except ImportError:
-    print "Please install launchpadbugs from lp:python-launchpad-bugs"
-    sys.exit(1)
+    from launchpadlib.launchpad import Launchpad
+    from lazr.restfulclient import errors
+except ImportError:
+    print "Please install launchpadlib from lp:launchpadlib"
+    sys.exit(1)
+try:
+    import hydrazine
+except ImportError:
+    print "Please install hydrazine from lp:launchpadlib"
+    sys.exit(1)
+
 
 options, args = getopt.gnu_getopt(sys.argv, "l", ["launchpad"])
 options = dict(options)
@@ -22,13 +29,13 @@
 
 def report_notmarked(bug, task, section):
     print 
-    print "Bug %d was mentioned in NEWS but is not marked fix released:" % (bug.bugnumber, )
+    print "Bug %d was mentioned in NEWS but is not marked fix released:" % (bug.id, )
     print "Launchpad title: %s" % bug.title
     print "NEWS summary: "
     print section
     if "--launchpad" in options or "-l" in options:
-        print "  bug %d" % bug.bugnumber
-        print "  affects bzr"
+        print "  bug %d" % bug.id
+        print "  affects %s" % task.bug_target_name
         print "  status fixreleased"
 
 
@@ -60,16 +67,43 @@
     finally:
         f.close()
 
-open_bug = connector.ConnectBug("TEXT")
-
+
+def print_bug_url(bugno):
+    print '<URL:http://pad.lv/%s>' % (bugno,)
+
+launchpad = hydrazine.create_session()
 bugnos = read_news_bugnos(args[1])
 for bugno, section in bugnos:
-    bug = open_bug(url="https://bugs.launchpad.net/bzr/+bug/%d" % bugno)
+    try:
+        bug = launchpad.bugs[bugno]
+    except errors.HTTPError, e:
+        if e.response.status == 401:
+            print_bug_url(bugno)
+            # Private, we can't access the bug content
+            print '%s is private and cannot be accessed' % (bugno,)
+            continue
+        raise
+     
     found_bzr = False
-    for task in bug.infotable:
-        if task.affects == "bzr":
+    fix_released = False
+    for task in bug.bug_tasks:
+        parts = task.bug_target_name.split('/')
+        if len(parts) == 1:
+            project = parts[0]
+            distribution = None
+        else:
+            project = parts[0]
+            distribution = parts[1]
+        if project == "bzr":
             found_bzr = True
-            if task.status != "Fix Released":
-                report_notmarked(bug, task, section)
+            if not fix_released and task.status == "Fix Released":
+                # We could check that the NEWS section and task_status are in
+                # sync, but that would be overkill. (case at hand: bug #416732)
+                fix_released = True
+
     if not found_bzr:
+        print_bug_url(bugno)
         print "Bug %d was mentioned in NEWS but is not marked as affecting bzr" % bugno
+    elif not fix_released:
+        print_bug_url(bugno)
+        report_notmarked(bug, task, section)

=== modified file 'tools/generate_release_notes.py'
--- tools/generate_release_notes.py	2009-09-03 05:09:22 +0000
+++ tools/generate_release_notes.py	2010-05-05 17:04:30 +0000
@@ -20,8 +20,6 @@
 import sys
 from optparse import OptionParser
 
-sys.path.insert(0, os.path.dirname(os.path.dirname(__file__)))
-
 
 def split_into_topics(lines, out_file, out_dir):
     """Split a large NEWS file into topics, one per release.
@@ -32,7 +30,7 @@
     """
     topic_file = None
     for index, line in enumerate(lines):
-        maybe_new_topic = line[:4] in ['bzr ', 'bzr-0',]
+        maybe_new_topic = line[:4] in ('bzr ', 'bzr-',)
         if maybe_new_topic and lines[index + 1].startswith('####'):
             release = line.strip()
             if topic_file is None:
@@ -45,13 +43,22 @@
         elif topic_file:
             topic_file.write(line)
         else:
+            # FIXME: the 'content' directive is used for rst2html (and
+            # conflicts with the 'toctree' we insert), we should get rid of
+            # that once we fully switch to sphinx -- vila 20100505
+            if (line.startswith('.. contents::')
+                or line.startswith('   :depth:')):
+                    continue
             # Still in the header - dump content straight to output
             out_file.write(line)
+    if topic_file is not None:
+        # Close the last topic_file opened
+        topic_file.close()
 
 
 def open_topic_file(out_file, out_dir, release):
     topic_name = release.replace(' ', '-')
-    out_file.write("   %s\n" % (topic_name,))
+    out_file.write("   %s <%s>\n" % (release, topic_name,))
     topic_path = os.path.join(out_dir, "%s.txt" % (topic_name,))
     result = open(topic_path, 'w')
     result.write("%s\n" % (release,))

=== modified file 'tools/packaging/update-changelogs.sh'
--- tools/packaging/update-changelogs.sh	2009-06-03 10:26:06 +0000
+++ tools/packaging/update-changelogs.sh	2010-08-19 08:07:42 +0000
@@ -27,7 +27,7 @@
     (
         echo "Updating changelog for $DISTRO"
         cd "$PACKAGE-$DISTRO" &&
-            dch -v $PPAVERSION -D $DISTRO -c changelog "$MSG." &&
+            dch -v $PPAVERSION -D $DISTRO "$MSG." &&
             bzr commit -m "$MSG: $PPAVERSION"
     )
 done

=== modified file 'tools/packaging/update-packaging-branches.sh'
--- tools/packaging/update-packaging-branches.sh	2009-06-03 10:20:19 +0000
+++ tools/packaging/update-packaging-branches.sh	2010-08-27 00:33:07 +0000
@@ -18,10 +18,7 @@
             cd ..
         fi
     else
-        SRC="lp:~bzr/$PACKAGE/packaging-$DISTRO"
-        if [ "$PACKAGE" = "bzr-svn" ] ; then
-            SRC="lp:~bzr/$PACKAGE/beta-ppa-$DISTRO"
-        fi
+        SRC="lp:~bzr/ubuntu/$DISTRO/$PACKAGE/bzr-ppa"
         echo "Checking out $SRC"
         bzr co $SRC $PACKAGE-$DISTRO
     fi

=== added file 'tools/win32/py2exe_boot_common.py'
--- tools/win32/py2exe_boot_common.py	1970-01-01 00:00:00 +0000
+++ tools/win32/py2exe_boot_common.py	2010-08-02 16:12:01 +0000
@@ -0,0 +1,35 @@
+# Common py2exe boot script - executed for all target types.
+
+# In the standard py2exe boot script, it setup stderr so that anything written
+# to it will be written to exe.log, and a message dialog is shown.
+# For Bazaar, we log most things to .bzr.log, and there are many things that
+# write to stderr, that are not errors, and so we don't want the py2exe dialog
+# message, So also blackhole stderr.
+
+import sys
+if sys.frozen == "windows_exe":
+
+    class Blackhole(object):
+        softspace = 0
+        def write(self, text):
+            pass
+        def flush(self):
+            pass
+    sys.stdout = Blackhole()
+    sys.stderr = Blackhole()
+    del Blackhole
+del sys
+
+# Disable linecache.getline() which is called by
+# traceback.extract_stack() when an exception occurs to try and read
+# the filenames embedded in the packaged python code.  This is really
+# annoying on windows when the d: or e: on our build box refers to
+# someone elses removable or network drive so the getline() call
+# causes it to ask them to insert a disk in that drive.
+import linecache
+def fake_getline(filename, lineno, module_globals=None):
+    return ''
+linecache.orig_getline = linecache.getline
+linecache.getline = fake_getline
+
+del linecache, fake_getline

